/*!
*******************************************************************************
**
** \file    abst_common.c
**
** \brief   abstraction library source file (common system service)
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*******************************************************************************
*/

#include <stdio.h>
#include <string.h>

#ifdef _LINUX_FAPI
#include <sys/ioctl.h>
#include <termios.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/time.h>
#endif

#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "abst_common_priv.h"

#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#endif

//*****************************************************************************
//*****************************************************************************
//** Local Defines
//*****************************************************************************
//*****************************************************************************

#define FAPEX_SYS_OK 0
#define FAPEX_SYS_NG 1

#define FAPEX_SYS_RTOS_OK 1
#define FAPEX_SYS_RTOS_NG 0

/*
********************************************************************************
********************************************************************************
**
** API functions related to SELECT
**
********************************************************************************
********************************************************************************
*/
#ifndef _LINUX_FAPI
#define FAPEX_SLCSTAT_ARRAY_SIZE   (((2*FAPEX_DESCRIPTOR_NUM)+(8*sizeof(FAPEX_SYS_SLCTSTAT_T))-1) \
                                    / (8*sizeof(FAPEX_SYS_SLCTSTAT_T)))
#define FAPEX_SELECT_SLEEP_TIME    1  /* [msec] */
#endif

/*!
*******************************************************************************
**
** \brief  Get select status memory size.
**
** \param  mode  read/write/exception flag<br>
**               The value of mode is the bitwise-inclusive OR of values from the following list.<br>
**               - #FAPEX_SELECT_MODE_READ<br>
**                 read select mode
**               - #FAPEX_SELECT_MODE_WRITE<br>
**                 write select mode
**               - #FAPEX_SELECT_MODE_EXPT<br>
**                 exception select mode
**
** \retval >0   size of the memory for select status
**
** \sa FAPEX_SYS_SelectInit
*******************************************************************************/
int32_t FAPEX_SYS_SelectGetSize(uint32_t mode)
{
    int32_t size, a_size;

    if(mode & ~(FAPEX_SELECT_MODE_READ | FAPEX_SELECT_MODE_WRITE | FAPEX_SELECT_MODE_EXPT)) {
        return 0;
    }

#ifndef _LINUX_FAPI
    a_size = FAPEX_SLCSTAT_ARRAY_SIZE * sizeof(FAPEX_SYS_SLCTSTAT_T);
#else /* !_LINUX_FAPI */
    a_size = sizeof(FAPEX_SYS_SLCTSTAT_T);
#endif /* !_LINUX_FAPI */
    a_size <<= 1;
    size = 0;
    if(mode & FAPEX_SELECT_MODE_READ) {
        size += a_size;
    }
    if(mode & FAPEX_SELECT_MODE_WRITE) {
        size += a_size;
    }
    if(mode & FAPEX_SELECT_MODE_EXPT) {
        size += a_size;
    }
    return size;
}

/*!
*******************************************************************************
**
** \brief  Initialize select.
**
** \param  *psel   The pointer to a select structure
** \param  *pbase  The pointer to a select status context<br>
**                 This pbase is top of memory area that size should be large than
**                 returned size of FAPEX_SYS_SelectGetSize().
** \param  mode    read/write/exception flag<br>
**                 The value of mode is the bitwise-inclusive OR of values from the following list.<br>
**                 - #FAPEX_SELECT_MODE_READ<br>
**                   read select mode
**                 - #FAPEX_SELECT_MODE_WRITE<br>
**                   write select mode
**                 - #FAPEX_SELECT_MODE_EXPT<br>
**                   exception select mode
**
** \retval 0   process OK
**
** \note
** If this 'mode' is different from 'mode' used in FAPEX_SYS_SelectGetSize(),
** the operation of *psel is not guaranteed.
**
** \sa FAPEX_SYS_SelectGetSize
*******************************************************************************/
int32_t FAPEX_SYS_SelectInit(FAPEX_SYS_SelectParamStrT *psel, void* pbase, uint32_t mode)
{
    FAPEX_SYS_SLCTSTAT_T *ptr;
#ifndef _LINUX_FAPI
    int32_t a_size;
#endif /* !_LINUX_FAPI */

    ptr = (FAPEX_SYS_SLCTSTAT_T*)pbase;

    psel->rstat      = NULL;
    psel->rstat_org  = NULL;
    psel->wstat      = NULL;
    psel->wstat_org  = NULL;
    psel->exstat     = NULL;
    psel->exstat_org = NULL;

#ifndef _LINUX_FAPI
    a_size = FAPEX_SLCSTAT_ARRAY_SIZE;

    if(mode & FAPEX_SELECT_MODE_READ) {
        psel->rstat = ptr;
        ptr += a_size;
        psel->rstat_org = ptr;
        ptr += a_size;
        /* todo: abstract of memset (user space) */
        memset(psel->rstat_org, 0 , FAPEX_SLCSTAT_ARRAY_SIZE * sizeof(FAPEX_SYS_SLCTSTAT_T));
    }
    if(mode & FAPEX_SELECT_MODE_WRITE) {
        psel->wstat = ptr;
        ptr += a_size;
        psel->wstat_org = ptr;
        ptr += a_size;
        /* todo: abstract of memset (user space) */
        memset(psel->wstat_org, 0, FAPEX_SLCSTAT_ARRAY_SIZE * sizeof(FAPEX_SYS_SLCTSTAT_T));
    }
    if(mode & FAPEX_SELECT_MODE_EXPT) {
        psel->exstat = ptr;
        ptr += a_size;
        psel->exstat_org = ptr;
        ptr += a_size;
        /* todo: abstract of memset (user space) */
        memset(psel->exstat_org, 0, FAPEX_SLCSTAT_ARRAY_SIZE * sizeof(FAPEX_SYS_SLCTSTAT_T));
    }
#else /* !_LINUX_FAPI */

    if(mode & FAPEX_SELECT_MODE_READ) {
        psel->rstat = ptr;
        ptr ++;
        psel->rstat_org = ptr;
        ptr ++;
        FD_ZERO(psel->rstat_org);
    }
    if(mode & FAPEX_SELECT_MODE_WRITE) {
        psel->wstat = ptr;
        ptr ++;
        psel->wstat_org = ptr;
        ptr ++;
        FD_ZERO(psel->wstat_org);
    }
    if(mode & FAPEX_SELECT_MODE_EXPT) {
        psel->exstat = ptr;
        ptr ++;
        psel->exstat_org = ptr;
        ptr ++;
        FD_ZERO(psel->exstat_org);
    }
#endif /* !_LINUX_FAPI */

    return 0;
}

/*!
*******************************************************************************
**
** \brief  Set select status.
**
** \param  *psel   The pointer to a select structure
** \param  id      The identifier for file descriptor.
** \param  mode    read/write/exception flag<br>
**                 The value of mode is the bitwise-inclusive OR of values from the following list.<br>
**                 - #FAPEX_SELECT_MODE_READ<br>
**                   read select mode
**                 - #FAPEX_SELECT_MODE_WRITE<br>
**                   write select mode
**                 - #FAPEX_SELECT_MODE_EXPT<br>
**                   exception select mode
**
** \retval 0   process OK
** \retval -1  process fails
**
** \note
** If *psel initialized in FAPEX_SYS_SelectInit() is not used, this operation is not guaranteed.
**
** \sa FAPEX_SYS_SelectInit
** \sa FAPEX_SYS_SelectClearID
*******************************************************************************/
int32_t FAPEX_SYS_SelectSetID(FAPEX_SYS_SelectParamStrT *psel, int32_t id, uint32_t mode)
{
#ifndef _LINUX_FAPI
    int32_t  group, sub_id;
    uint32_t set_flag;

    if(id < 0) {
        return -1;
    }
    group    = id >> 4;
    sub_id   = (id & 0x0000000f) << 1;
    set_flag = 1 << sub_id;

    if(mode & FAPEX_SELECT_MODE_READ){
        if(psel->rstat_org == NULL){
            return -1;
        }
        psel->rstat_org[group] |= set_flag;
    }
    if(mode & FAPEX_SELECT_MODE_WRITE){
        if(psel->wstat_org == NULL){
            return -1;
        }
        psel->wstat_org[group] |= set_flag;
    }
    if(mode & FAPEX_SELECT_MODE_EXPT){
        if(psel->exstat_org == NULL){
            return -1;
        }
        psel->exstat_org[group] |= set_flag;
    }
#else /* !_LINUX_FAPI */
    if(id < 0) {
        return -1;
    }
    if(mode & FAPEX_SELECT_MODE_READ){
        if(psel->rstat_org == NULL){
            return -1;
        }
        FD_SET(id, psel->rstat_org);
    }
    if(mode & FAPEX_SELECT_MODE_WRITE){
        if(psel->wstat_org == NULL){
            return -1;
        }
        FD_SET(id, psel->wstat_org);
    }
    if(mode & FAPEX_SELECT_MODE_EXPT){
        if(psel->exstat_org == NULL){
            return -1;
        }
        FD_SET(id, psel->exstat_org);
    }
#endif /* !_LINUX_FAPI */
    return 0;
}

/*!
*******************************************************************************
**
** \brief  Clear select status.
**
** \param  *psel   The pointer to a select structure
** \param  id      The identifier for file descriptor.
** \param  mode    read/write/exception flag<br>
**                 The value of mode is the bitwise-inclusive OR of values from the following list.<br>
**                 - #FAPEX_SELECT_MODE_READ<br>
**                   read select mode
**                 - #FAPEX_SELECT_MODE_WRITE<br>
**                   write select mode
**                 - #FAPEX_SELECT_MODE_EXPT<br>
**                   exception select mode
**
** \retval 0   process OK
** \retval -1  process fails
**
** \note
** If *psel initialized in FAPEX_SYS_SelectInit() is not used, this operation is not guaranteed.
**
** \sa FAPEX_SYS_SelectInit
** \sa FAPEX_SYS_SelectSetID
*******************************************************************************/
int32_t FAPEX_SYS_SelectClearID(FAPEX_SYS_SelectParamStrT *psel, int32_t id, uint32_t mode)
{
#ifndef _LINUX_FAPI
    int32_t  group, sub_id;
    uint32_t clr_flag;

    if(id < 0) {
        return -1;
    }
    group    = id >> 4;
    sub_id   = (id & 0x0000000f) << 1;
    clr_flag = ~(1 << sub_id);

    if(mode & FAPEX_SELECT_MODE_READ){
        if(psel->rstat_org == NULL){
            return -1;
        }
        psel->rstat_org[group] &= clr_flag;
    }
    if(mode & FAPEX_SELECT_MODE_WRITE){
        if(psel->wstat_org == NULL){
            return -1;
        }
        psel->wstat_org[group] &= clr_flag;
    }
    if(mode & FAPEX_SELECT_MODE_EXPT){
        if(psel->exstat_org == NULL){
            return -1;
        }
        psel->exstat_org[group] &= clr_flag;
    }
#else /* !_LINUX_FAPI */
    if(id < 0) {
        return -1;
    }
    if(mode & FAPEX_SELECT_MODE_READ){
        if(psel->rstat_org == NULL){
            return -1;
        }
        FD_CLR(id, psel->rstat_org);
    }
    if(mode & FAPEX_SELECT_MODE_WRITE){
        if(psel->wstat_org == NULL){
            return -1;
        }
        FD_CLR(id, psel->wstat_org);
    }
    if(mode & FAPEX_SELECT_MODE_EXPT){
        if(psel->exstat_org == NULL){
            return -1;
        }
        FD_CLR(id, psel->exstat_org);
    }
#endif /* !_LINUX_FAPI */
    return 0;
}

/*!
*******************************************************************************
**
** \brief  set parameter to a select structure.
**
** \param  *psel     The pointer to a select structure
** \param  maxid     The maximum value of watched identifiers
** \param  *pTimeout The pointer to a time structure (with seconds and microseconds)
**
** \retval 0   process OK
**
** \note
** If *psel initialized in FAPEX_SYS_SelectInit() is not used, this operation is not guaranteed.
**
** \sa FAPEX_SYS_SelectInit
*******************************************************************************/
int32_t FAPEX_SYS_SelectSetParam(FAPEX_SYS_SelectParamStrT *psel, int32_t maxid,
                                 FAPEX_SYS_TIMEVAL_T *pTimeout)
{
    psel->maxid = maxid;
    psel->tout  = pTimeout;

    return 0;
}

/*!
*******************************************************************************
**
** \brief  Call select.
** This function allows a program to monitor multiple file descriptors, waiting
** until one or more of the file descriptors become "ready" for some class of I/O
** operation.
**
** \param  *psel   The pointer to a select structure
**
** \retval >0  process OK, its value is the total number of bits set in the bit masks.
** \retval 0   process timeout
** \retval -1  process fails, its value is set to errno.
**
** \note
** If *psel initialized in FAPEX_SYS_SelectInit() is not used, this operation is not guaranteed.
**
** \sa FAPEX_SYS_SelectInit
** \sa FAPEX_SYS_SelectSetID
** \sa FAPEX_SYS_SelectClearID
** \sa FAPEX_SYS_SelectSetParam
*******************************************************************************/
int32_t FAPEX_SYS_SelectCall(FAPEX_SYS_SelectParamStrT *psel)
{
#ifndef _LINUX_FAPI
    int32_t  ret, count, max_count, sel_count;
    int32_t  max_group, group;
    uint32_t sub_mask;
    int32_t  read, write, exception, group_status;
    uint32_t rest_time, dur_time, sub_time;  // [msec]

     /* todo: abstract of memcpy (user space) */
    if(psel->rstat_org!=NULL){
        memcpy(psel->rstat,  psel->rstat_org,  FAPEX_SLCSTAT_ARRAY_SIZE * sizeof(FAPEX_SYS_SLCTSTAT_T));
        memset(psel->rstat_org, 0 , FAPEX_SLCSTAT_ARRAY_SIZE * sizeof(FAPEX_SYS_SLCTSTAT_T));
    }
    if(psel->wstat_org!=NULL){
        memcpy(psel->wstat,  psel->wstat_org,  FAPEX_SLCSTAT_ARRAY_SIZE * sizeof(FAPEX_SYS_SLCTSTAT_T));
        memset(psel->wstat_org, 0 , FAPEX_SLCSTAT_ARRAY_SIZE * sizeof(FAPEX_SYS_SLCTSTAT_T));
    }
    if(psel->exstat_org!=NULL){
        memcpy(psel->exstat, psel->exstat_org, FAPEX_SLCSTAT_ARRAY_SIZE * sizeof(FAPEX_SYS_SLCTSTAT_T));
        memset(psel->exstat_org, 0 , FAPEX_SLCSTAT_ARRAY_SIZE * sizeof(FAPEX_SYS_SLCTSTAT_T));
    }

    /* setup time parameter */
    if(psel->tout != NULL) {
        rest_time = (psel->tout->tv_sec != 0) ? (psel->tout->tv_sec * 1000) : (psel->tout->tv_usec / 1000);
        dur_time  = FAPEX_SELECT_SLEEP_TIME;
        sub_time  = dur_time;
    }
    else {
        rest_time = dur_time = FAPEX_SELECT_SLEEP_TIME;
        sub_time  = 0;
    }

    sel_count = 0;
    while(rest_time > 0) {
        max_count = psel->maxid + 1;
        max_group = (max_count + 15) >> 4;
        for(group=0; group<max_group; group++) {
            read      = (psel->rstat  != NULL)? psel->rstat [group] : 0;
            write     = (psel->wstat  != NULL)? psel->wstat [group] : 0;
            exception = (psel->exstat != NULL)? psel->exstat[group] : 0;

            group_status = read | write | exception;
            if(group_status == 0) {
                continue;
            }
            sub_mask = 0x00000003;

            for(count=0; count<16; count++) {
                if(group_status & sub_mask) {
                    ret = FAPEXSAL_Poll(((group << 4) + count), NULL);

                    if(ret > 0) {
                        if(read & sub_mask) {
                            if( ret & (POLLIN | POLLRDNORM | POLLPRI) ){
                                psel->rstat[group] |= sub_mask;
                            }
                        }
                        if(write & sub_mask){ 
                            if( ret & (POLLOUT | POLLWRNORM) ){
                                psel->wstat[group] |= sub_mask;
                            }
                        }
                        if(exception & sub_mask) {
                            if( ret & (POLLHUP | POLLERR | POLLMSG) ) {
                                psel->exstat[group] |= sub_mask;
                            }
                        }
                        sel_count ++;
                        if(ret & POLLPRI) {
                            return sel_count;
                        }
                    }
                    else if(ret < 0) {
                        errno = -ret;
                        return -1;
                    }
                    group_status &= ~sub_mask;
                }
                if(group_status == 0) {
                    break;
                }
                sub_mask <<= 2;
            }
        }
        if(sel_count > 0) {
            return sel_count;
        }
        RTOS_Sleep(dur_time);

        if(rest_time >= sub_time) {
            rest_time -= sub_time;
        }
        else {
            sub_time = rest_time;
        }
    }
    /* timeout */
    return 0;

#else /* !_LINUX_FAPI */

    /* todo: abstract of memcpy (user space) */
    if(psel->rstat_org!=NULL){
        memcpy(psel->rstat, psel->rstat_org, sizeof(FAPEX_SYS_SLCTSTAT_T));
    }
    if(psel->wstat_org!=NULL){
        memcpy(psel->wstat, psel->wstat_org, sizeof(FAPEX_SYS_SLCTSTAT_T));
    }
    if(psel->exstat_org!=NULL){
        memcpy(psel->exstat, psel->exstat_org, sizeof(FAPEX_SYS_SLCTSTAT_T));
    }
    return select(psel->maxid + 1, psel->rstat, psel->wstat, psel->exstat, psel->tout);

#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  Check select.
** This function checks whether data of this id is readable/writeable/exceptional.
**
** \param  *psel   The pointer to a select structure
** \param  id      The identifier for file descriptor.
** \param  mode    read/write/exception flag<br>
**                 The value of mode is the bitwise-inclusive OR of values from the following list.<br>
**                 - #FAPEX_SELECT_MODE_READ<br>
**                   read select mode
**                 - #FAPEX_SELECT_MODE_WRITE<br>
**                   write select mode
**                 - #FAPEX_SELECT_MODE_EXPT<br>
**                   exception select mode
**
** \retval >0  process OK
** \retval -1  process fails.
**
** \note
** If *psel initialized in FAPEX_SYS_SelectInit() is not used, this operation is not guaranteed.
**
** \sa FAPEX_SYS_SelectInit
*******************************************************************************/
int32_t FAPEX_SYS_SelectCheck(FAPEX_SYS_SelectParamStrT *psel, int32_t id, uint32_t mode)
{
    int32_t ret = 0;
#ifndef _LINUX_FAPI
    int32_t group, mask;

    if(id < 0) {
        return -1;
    }
    group  = id >> 4;
    mask   = 1 << (((id & 0x0000000f) << 1) + 1);
    if(mode & FAPEX_SELECT_MODE_READ) {
        if(psel->rstat[group] & mask){
            ret = 1;
        }
    }
    if(mode & FAPEX_SELECT_MODE_WRITE) {
        if(psel->wstat[group] & mask){
            ret = 1;
        }
    }
    if(mode & FAPEX_SELECT_MODE_EXPT) {
        if(psel->exstat[group] & mask){
            ret = 1;
        }
    }
#else /* !_LINUX_FAPI */
    if(mode & FAPEX_SELECT_MODE_READ) {
        ret |= FD_ISSET(id, psel->rstat);
    }
    if(mode & FAPEX_SELECT_MODE_WRITE) {
        ret |= FD_ISSET(id, psel->wstat);
    }
    if(mode & FAPEX_SELECT_MODE_EXPT) {
        ret |= FD_ISSET(id, psel->exstat);
    }
#endif /* !_LINUX_FAPI */

    return ret;
}

/*
********************************************************************************
********************************************************************************
**
** API functions related to SEMAPHORE
**
********************************************************************************
********************************************************************************
*/

#ifdef _LINUX_FAPI
/*!
*******************************************************************************
** \brief Create a semaphore.
**
** \param value The initial value of the semaphore.
**
** \retval >0  Handle to the new semaphore (sucessful)
** \retval 0   semaphore could not be created.
**
** \sa FAPEX_SYS_DestroySemaphore
** \sa FAPEX_SYS_GetSemaphore
** \sa FAPEX_SYS_SetSemaphore
*******************************************************************************/
FAPEX_SYS_SemaphoreT FAPEX_SYS_CreateSemaphore(uint32_t value)
{
    FAPEX_SYS_SemaphoreT sem;
    sem = (FAPEX_SYS_SemaphoreT)FAPI_SYS_MALLOC(sizeof(sem_t));
    if(sem == NULL) {
        return 0;
    }
    sem_init(sem, 0, value);
    return(sem);
}
#endif /* _LINUX_FAPI */

/*!
*******************************************************************************
** \brief Destroy a semaphore.
**
** \param sem The semaphore to destroy. It must come from a previous call
**            to FAPEX_SYS_CreateSemaphore.
**
** \retval 0   sucessful
** \retval !0  semaphore could not be destroyed.
**
** \sa FAPEX_SYS_CreateSemaphore
** \sa FAPEX_SYS_GetSemaphore
** \sa FAPEX_SYS_SetSemaphore
*******************************************************************************/
int32_t FAPEX_SYS_DestroySemaphore(FAPEX_SYS_SemaphoreT sem)
{
#ifndef _LINUX_FAPI
    uint32_t  ret;

    ret = RTOS_DestroySemaphore(sem);
    if( ret == FAPEX_SYS_RTOS_OK ) {
        return FAPEX_SYS_OK;
    }
    else {
        return FAPEX_SYS_NG;
    }
#else
    sem_destroy(sem);
    FAPI_SYS_FREE(sem);

    return 0;
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
** \brief Get a semaphore.
**
** \param sem     The semaphore to get. It must come from a previous call to
**                FAPEX_SYS_CreateSemaphore.
** \param timeout timeout value
**                - #FAPEX_SYS_SUSPEND<br>
**                  The current thread should wait until the semaphore is free
**                  (blocking mode).
**                - #FAPEX_SYS_NO_SUSPEND<br>
**                  The current thread should not be blocked (non-blocking mode).
**                - other<br>
**                  The value represents the time in milliseconds to (at most)
**                  wait for the semaphore (timeout mode).
**
** \retval 0   sucessful
** \retval !0  unsuccessful
**
** \sa FAPEX_SYS_CreateSemaphore
** \sa FAPEX_SYS_DestroySemaphore
** \sa FAPEX_SYS_SetSemaphore
*******************************************************************************/
int32_t FAPEX_SYS_GetSemaphore(FAPEX_SYS_SemaphoreT sem, int32_t timeout)
{
#ifndef _LINUX_FAPI
    uint32_t  ret;

    ret = RTOS_GetSemaphore( sem, timeout );
    if( ret == FAPEX_SYS_RTOS_OK ) {
        return FAPEX_SYS_OK;
    }
    else {
        return FAPEX_SYS_NG;
    }
#else
    int32_t  ret;

    if(timeout == FAPI_SYS_SUSPEND) {
        ret = sem_wait(sem);
    }
    else if(timeout == FAPI_SYS_NO_SUSPEND) {
        ret = sem_trywait(sem);
    }
    else {
        struct timespec  tout;

        tout.tv_nsec = (timeout % 1000) * 1000;
        tout.tv_sec  = timeout / 1000;
        ret = sem_timedwait(sem, &tout);
    }
    return ret;
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
** \brief Set a semaphore.
**
** \param sem     The semaphore to set. It must come from a previous call to
**                FAPEX_SYS_CreateSemaphore.
** \param timeout timeout value
**                - #FAPEX_SYS_SUSPEND<br>
**                  The current thread should wait until the semaphore is free
**                  (blocking mode).
**                - #FAPEX_SYS_NO_SUSPEND<br>
**                  The current thread should not be blocked (non-blocking mode).
**                - other<br>
**                  The value represents the time in milliseconds to (at most)
**                  wait for the semaphore (timeout mode).
**
** \retval 0   sucessful
** \retval !0  unsuccessful
**
** \sa FAPEX_SYS_CreateSemaphore
** \sa FAPEX_SYS_DestroySemaphore
** \sa FAPEX_SYS_GetSemaphore
*******************************************************************************/
int32_t FAPEX_SYS_SetSemaphore(FAPEX_SYS_SemaphoreT sem, int32_t timeout)
{
#ifndef _LINUX_FAPI
    uint32_t  ret;

    ret = RTOS_SetSemaphore( sem, timeout );
    if( ret == FAPEX_SYS_RTOS_OK ) {
        return FAPEX_SYS_OK;
    }
    else {
        return FAPEX_SYS_NG;
    }
#else
    int32_t  ret;

    ret = sem_post(sem);
    return ret;
#endif /* !_LINUX_FAPI */
}

/*
********************************************************************************
********************************************************************************
**
** API functions related to THREAD
**
********************************************************************************
********************************************************************************
*/

/* debug */
//#define FAPEX_SYS_THREAD_DEBUG

#ifdef FAPEX_SYS_THREAD_DEBUG
#define FAPEX_SYS_THREAD_DMSG(fmt,...)    printf(fmt,##__VA_ARGS__)
#else
#define FAPEX_SYS_THREAD_DMSG(fmt,...)
#endif

#ifdef _LINUX_FAPI

#define FAPEX_SYS_TC_ST_NO_INIT 0
#define FAPEX_SYS_TC_ST_INIT    1

static fapex_sys_thread_controlT fapex_sys_TC = { FAPEX_SYS_TC_ST_NO_INIT, };
static fapex_sys_thread_listT *fapex_sys_thread_list[FAPEX_SYS_THREAD_LIST_MAX];
int32_t fapex_sys_thread_reg_num = 0;
int32_t fapex_sys_thread_reg_max = 0;


/*!
*******************************************************************************
**
*******************************************************************************/
void fapex_sys_TC_Init()
{
    if ( fapex_sys_TC.inited != FAPEX_SYS_TC_ST_INIT )
    {
        fapex_sys_TC.inited = FAPEX_SYS_TC_ST_INIT;
        pthread_mutex_init( &(fapex_sys_TC.mutexT), NULL );
    }
}

void fapex_sys_TC_Lock()
{
    fapex_sys_TC_Init();
    pthread_mutex_lock( &(fapex_sys_TC.mutexT) );
}

void fapex_sys_TC_UnLock()
{
     pthread_mutex_unlock( &(fapex_sys_TC.mutexT) );
}
/*!
*******************************************************************************
**
*******************************************************************************/
static int32_t fapex_sys_thread_list_reg( fapex_sys_thread_listT *thread_addr )
{
    int32_t ret;
    int32_t i;
    
    if ( fapex_sys_thread_reg_num >= FAPEX_SYS_THREAD_LIST_MAX )
    {
        ret = FAPEX_SYS_NG; /* nothing list area */
    }
    else if ( fapex_sys_thread_reg_num == fapex_sys_thread_reg_max )
    {
        fapex_sys_thread_list[fapex_sys_thread_reg_max] = thread_addr;
        thread_addr->no = fapex_sys_thread_reg_max;
        fapex_sys_thread_reg_num++;
        fapex_sys_thread_reg_max++;
        ret = FAPEX_SYS_OK;
    }
    else
    {
        ret = FAPEX_SYS_NG;
        for(i=0;i<FAPEX_SYS_THREAD_LIST_MAX;i++)
        {
            if ( fapex_sys_thread_list[i] == NULL )
            {
                fapex_sys_thread_list[i] = thread_addr;
                thread_addr->no = i;
                fapex_sys_thread_reg_num++;
                ret = FAPEX_SYS_OK;
                break;
            }
        }
    }
    return ret;
}
static int32_t fapex_sys_thread_list_del( fapex_sys_thread_listT *thread_addr )
{
    int32_t ret;
    
    if ( fapex_sys_thread_reg_num == 0 )
    {
        ret = FAPEX_SYS_NG;
    }
    else if ( thread_addr->no >= fapex_sys_thread_reg_max )
    {
        ret = FAPEX_SYS_NG;
    }
    else
    {
        fapex_sys_thread_list[thread_addr->no] = NULL;
        if ( thread_addr->no == fapex_sys_thread_reg_max - 1)
        {
            fapex_sys_thread_reg_max--;
        }
        fapex_sys_thread_reg_num--;
        ret = FAPEX_SYS_OK;
    }
    return ret;

}

static FAPEX_SYS_ThreadT fapex_sys_thread_search( pthread_t thread_obj )
{
    FAPEX_SYS_ThreadT work = NULL;
    int32_t i;
    
    for(i=0;i<fapex_sys_thread_reg_max;i++)
    {
        if ( fapex_sys_thread_list[i] != NULL )
        {
            if ( fapex_sys_thread_list[i]->thread_obj == thread_obj )
            {
                work = fapex_sys_thread_list[i];
                break;
            }
        }
    }
    return work;
}

/*!
*******************************************************************************
** \brief Create a new thread.
**
** This function creates a new thread. It searches an unused element
** within the internal thread table and initializes it with the given values.
**
** \param stackBuffer a pointer to the begin (low address) of a memory buffer
**                    to be used as thread stack. A NULL pointer forces the
**                    internal dynamical allocate of memory using the size
**                    argument stackBytes.
** \param stackSize   the number of 4 kbyte elements in the given stack memory.
**                    This value must not be '0'.
** \param priority    the priority value for the thread to create, this value
**                    will be used only by the priority-control scheduler, the
**                    round-robin scheduler silently ignores this value.
** \param function    a pointer to the real thread function, this function
**                    must accept an optional void* argument and must not
**                    return a value.
** \param optArg      an optional function argument passed to the real thread
**                    function during the initial wakeup.
** \param optData     an optional data pointer to be stored in the thread
**                    structure.
** \param optName     an optional string containing a thread name (max.15 chars)
**
** \return
** - A pointer of eventType FAPEX_SYS_ThreadT which will be required by other
**   kernel thread functions
** - NULL in case of error, please use FAPEX_SYS_GetLastError() to read the
**   last error code.
*******************************************************************************/
FAPEX_SYS_ThreadT FAPEX_SYS_CreateThread(
        uint32_t* stackBuffer, uint32_t stackSize, uint32_t priority,
        FAPEX_SYS_ThreadFunctionT function, void* optArg, void* optData,
        const char_t* optName )
{
    int32_t ret;
    fapex_sys_thread_listT *work_thread;
    
    fapex_sys_TC_Lock();
    
    work_thread = FAPI_SYS_MALLOC( sizeof( fapex_sys_thread_listT ) );
    if ( work_thread == NULL )
    {
        fapex_sys_TC_UnLock();
        FAPEX_SYS_THREAD_DMSG("TD_REPO: ERROR cannot allocate memory\n" );
        return NULL;
    }

    stackSize *= 4 * 1024;

    work_thread->stackBuffer = stackBuffer; /* not support */
    work_thread->stackSize = (stackSize > PTHREAD_STACK_MIN) ? stackSize : PTHREAD_STACK_MIN;
    work_thread->priority  = priority;      /* not support */
    work_thread->function  = function;
    work_thread->optArg    = optArg;
    work_thread->optData   = optData;       /* not support */
    work_thread->optStatus = FAPEX_SYS_THREAD_STATE_INIT;
    work_thread->wait_cnt  = 0;
    memset( work_thread->optName, 0, sizeof(work_thread->optName)-1 );
    if ( optName != NULL )
    {
        strncpy( work_thread->optName, optName, sizeof(work_thread->optName)-2 );
    }
    pthread_mutex_init( &work_thread->thread_mutex, NULL );
    pthread_cond_init ( &work_thread->thread_cond,  NULL );

    ret = fapex_sys_thread_list_reg( work_thread );
    if ( ret == FAPEX_SYS_NG )
    {
        FAPEX_SYS_THREAD_DMSG("TD_REPO: ERROR fapex_sys_thread_list_reg %d\n", ret );
        goto FAPEX_SYS_CreateThread_error;
    }
    
    ret = pthread_attr_init( &work_thread->thread_attr );
    if ( ret != 0 )
    {
        FAPEX_SYS_THREAD_DMSG("TD_REPO: ERROR pthread_attr_init %d\n", ret );
        goto FAPEX_SYS_CreateThread_error;
    }
    
#if 0
    if(stackSize != 0) {
        ret = pthread_attr_setstacksize( &work_thread->thread_attr, work_thread->stackSize );
        if ( ret != 0 )
        {
            pthread_attr_destroy( &work_thread->thread_attr );
            FAPEX_SYS_THREAD_DMSG("TD_REPO: ERROR pthread_attr_setstacksize %d\n", ret );
            goto FAPEX_SYS_CreateThread_error;
        }
    }
#endif
    
    ret = pthread_create( &work_thread->thread_obj, &work_thread->thread_attr,
                          (void *)work_thread->function, work_thread->optArg );
    if ( ret != 0 )
    {
        pthread_attr_destroy( &work_thread->thread_attr );
        FAPEX_SYS_THREAD_DMSG("TD_REPO: ERROR pthread_create %d\n", ret );
        goto FAPEX_SYS_CreateThread_error;
    }
    
    FAPEX_SYS_THREAD_DMSG("TD_REPO:%s() thread create OK\n",work_thread->optName );

    fapex_sys_TC_UnLock();

    return (FAPEX_SYS_ThreadT)work_thread;

FAPEX_SYS_CreateThread_error:
    pthread_mutex_destroy( &work_thread->thread_mutex );
    pthread_cond_destroy ( &work_thread->thread_cond );

    FAPI_SYS_FREE( work_thread );

    FAPEX_SYS_THREAD_DMSG("FAPEX_SYS_CreateThread() error\n" );

    fapex_sys_TC_UnLock();

    return NULL;
}
#endif /* _LINUX_FAPI */

/*!
*******************************************************************************
** \brief Destroy the current thread
**
** This function destroys the given thread and performs a context switch
** to activate the next waiting thread.
**
** \param threadHandle The thread handle to destroy.
**
** \retval 0   sucessful
** \retval !0  unsuccessful
**
** \note
** in case of a '0' return value you should check the real error code
** by calling FAPEX_SYS_GetLastError().
*******************************************************************************/
uint32_t FAPEX_SYS_DestroyThread( FAPEX_SYS_ThreadT threadHandle )
{
#ifndef _LINUX_FAPI
    uint32_t ret;

    ret = RTOS_DestroyThread( threadHandle );
    if( ret == FAPEX_SYS_RTOS_OK ) {
        return FAPEX_SYS_OK;
    }
    else {
        return FAPEX_SYS_NG;
    }
#else
    int32_t ret;
    fapex_sys_thread_listT *work_thread = threadHandle;

    if ( work_thread == NULL )
    {
        FAPEX_SYS_THREAD_DMSG("TD_REPO: ERROR work_thread=%x\n", work_thread );
        return FAPEX_SYS_NG;
    }

    FAPEX_SYS_THREAD_DMSG("TD_REPO:%s() Destroy(wait count %d) obj=%x\n",
                          work_thread->optName, work_thread->wait_cnt,
                          work_thread->thread_obj );

    fapex_sys_TC_Lock();

    ret = fapex_sys_thread_list_del( work_thread );
    if ( ret == FAPEX_SYS_OK )
    {
        ret = pthread_cancel( work_thread->thread_obj );
        if ( ret != 0 )
        {
            FAPEX_SYS_THREAD_DMSG("FAPEX_SYS_DestroyThread() error %d\n", ret );
            fapex_sys_TC_UnLock();
            return FAPEX_SYS_NG;
        }

//      FAPI_SYS_SLEEP( 100 );

        pthread_attr_destroy( &work_thread->thread_attr );

        pthread_mutex_destroy( &work_thread->thread_mutex );
        pthread_cond_destroy ( &work_thread->thread_cond );

        FAPI_SYS_FREE( work_thread );
    }
    else
    {
        FAPEX_SYS_THREAD_DMSG("TD_REPO:%s() Destroy ERROR not create \n",
                              work_thread->optName );
    }
    fapex_sys_TC_UnLock();

    return FAPEX_SYS_OK;

#endif /* !_LINUX_FAPI */
}

#ifdef _LINUX_FAPI
/*!
*******************************************************************************
** \brief Get the thread handle of the current thread
**
** This function returns the thread identifier of the current thread which
** will be required as argument to other kernel thread functions.
**
** \return
** - The thread handle of the current thread.
** - The thread handle of the IDLE thread is case of problems
*******************************************************************************/
FAPEX_SYS_ThreadT FAPEX_SYS_GetThread( void )
{
    pthread_t   work_thread_obj;
    
    work_thread_obj = pthread_self();
    
    return (FAPEX_SYS_ThreadT)fapex_sys_thread_search( work_thread_obj );
}

/*!
*******************************************************************************
** \brief Get current state of the given thread handle
**
** \param threadHandle The thread handle to access.
*******************************************************************************/
uint32_t FAPEX_SYS_GetThreadState( FAPEX_SYS_ThreadT threadHandle )
{
    return ((fapex_sys_thread_listT *)threadHandle)->optStatus;
}

/*!
*******************************************************************************
** \brief Get the thread name of the given thread handle
**
** This function returns the optional thread name from the given
** thread handle.
**
** \param threadHandle The thread handle to access.
**
** \return
** - '0' if the given thread identifier did not address a valid thread
** - any other positiv value specifies the pointer to the data element.
*******************************************************************************/
const char_t* FAPEX_SYS_GetThreadName( FAPEX_SYS_ThreadT threadHandle )
{
    if(threadHandle == NULL) {
        return NULL;
    }
    return (char_t *)((fapex_sys_thread_listT *)threadHandle)->optName;
}
#endif /* _LINUX_FAPI */

/*!
********************************************************************************
** \brief Suspends the given thread.
**
** This function suspends the given thread. A suspended thread can be
** re-activated only by the function FAMOS_ThreadWakeup(). After setting
** the thread mode to SUSPENDED to scheduler will be called to execute the
** next waiting thread.
**
** \param threadHandle The thread handle to access.
**
** \retval 0   sucessful
** \retval !0  unsuccessful
********************************************************************************
*/
uint32_t FAPEX_SYS_SuspendThread( FAPEX_SYS_ThreadT threadHandle )
{
#ifndef _LINUX_FAPI
    uint32_t  ret;

    ret = RTOS_SuspendThread( threadHandle );
    if( ret == FAPEX_SYS_RTOS_OK ) {
        return FAPEX_SYS_OK;
    }
    else {
        return FAPEX_SYS_NG;
    }
#else
    fapex_sys_thread_listT *work_thread = threadHandle;
    int32_t  ret;

    if( work_thread == NULL )
    {
        FAPEX_SYS_THREAD_DMSG("FAPEX_SYS_SuspendThread() error\n");
        return FAPEX_SYS_NG;
    }

    ret = pthread_mutex_lock( &work_thread->thread_mutex );
    if(ret == 0) {
        ret = pthread_cond_wait( &work_thread->thread_cond, &work_thread->thread_mutex);
        if(ret != 0) {
            FAPEX_SYS_THREAD_DMSG("pthread_cond_wait() error %d\n", ret);
        }
        ret = pthread_mutex_unlock( &work_thread->thread_mutex );
    }

    if(ret == 0)  {
        return FAPEX_SYS_OK;
    }
    else {
        FAPEX_SYS_THREAD_DMSG("FAPEX_SYS_SuspendThread() error %d\n", ret);
        return FAPEX_SYS_NG;
    }
#endif /* !_LINUX_FAPI */
}

/*!
********************************************************************************
** \brief Wake up a suspended thread.
**
** This function wakes up a previously suspended thread. Only threads which
** were suspended before using the function FAMOS_ThreadSuspend() can be
** re-activated.
**
** \param threadHandle The thread handle to access.
**
** \retval 0   sucessful
** \retval !0  unsuccessful
********************************************************************************
*/
uint32_t FAPEX_SYS_WakeupThread( FAPEX_SYS_ThreadT threadHandle )
{
#ifndef _LINUX_FAPI
    uint32_t  ret;

    ret = RTOS_WakeupThread( threadHandle );
    if( ret == FAPEX_SYS_RTOS_OK ) {
        return FAPEX_SYS_OK;
    }
    else {
        return FAPEX_SYS_NG;
    }
#else
    fapex_sys_thread_listT *work_thread = threadHandle;
    int32_t  ret;

    if( work_thread == NULL )
    {
        FAPEX_SYS_THREAD_DMSG("FAPEX_SYS_WakeupThread() error\n");
        return FAPEX_SYS_NG;
    }

    ret = pthread_mutex_lock( &work_thread->thread_mutex );
    if(ret == 0) {
        ret = pthread_cond_signal( &work_thread->thread_cond );
        if(ret != 0) {
            FAPEX_SYS_THREAD_DMSG("pthread_cond_signal() error %d\n", ret);
        }
        ret = pthread_mutex_unlock( &work_thread->thread_mutex );
    }

    if(ret == 0)  {
        return FAPEX_SYS_OK;
    }
    else {
        FAPEX_SYS_THREAD_DMSG("FAPEX_SYS_WakeupThread() error %d\n", ret);
        return FAPEX_SYS_NG;
    }
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
** \brief wait for termination of another thread
**
** this function suspends execution of the calling thread until the target
** thread terminates, unless the target thread has already terminated.
**
** \param threadHandle The thread handle to wait.
**
** \retval 0   sucessful
** \retval !0  unsuccessful
*******************************************************************************/
int32_t FAPEX_SYS_WaitEndThread( FAPEX_SYS_ThreadT threadHandle )
{
#ifndef _LINUX_FAPI
    while( RTOS_GetThreadState( threadHandle ) == RTOS_THREAD_ACTIVE ) {
        FAPI_SYS_SLEEP( 1000 );
    }
    return FAPEX_SYS_OK;
#else
    fapex_sys_thread_listT *work_thread = threadHandle;
    int32_t  ret;

    if( work_thread == NULL )
    {
        return FAPEX_SYS_NG;
    }
    ret = pthread_join( work_thread->thread_obj, NULL );
    if ( ret != 0 )
    {
        FAPEX_SYS_THREAD_DMSG("FAPEX_SYS_WaitEndThread() error %d\n", ret );
        return FAPEX_SYS_NG;
    }

    fapex_sys_TC_Lock();

    ret = fapex_sys_thread_list_del( work_thread );
    if ( ret == FAPEX_SYS_OK )
    {
        pthread_attr_destroy( &work_thread->thread_attr );

        pthread_mutex_destroy( &work_thread->thread_mutex );
        pthread_cond_destroy ( &work_thread->thread_cond );

        FAPI_SYS_FREE( work_thread );
    }
    fapex_sys_TC_UnLock();

    return FAPEX_SYS_OK;
#endif
}

/*!
*******************************************************************************
**
*******************************************************************************/
void FAPEX_SYS_WaitEndAllThread()
{
#ifdef _LINUX_FAPI
    for(;;)
    {
        if ( fapex_sys_thread_reg_num == 0 ) break;
        
        FAPI_SYS_SLEEP(10);
    }
    FAPI_SYS_SLEEP( 100 );
#endif
}

/*
********************************************************************************
********************************************************************************
**
** API functions related to KERNEL
**
********************************************************************************
********************************************************************************
*/
#ifdef _LINUX_FAPI
/*!
*******************************************************************************
**
*******************************************************************************/
void FAPEX_SYS_InitServices( void )
{
   /* not support */
}

/*!
*******************************************************************************
**
*******************************************************************************/
void FAPEX_SYS_ExitApplication( int32_t status )
{
    /* to do zombi thread */
    exit( status );
}

/*!
*******************************************************************************
**
*******************************************************************************/
const char_t* FAPEX_SYS_GetIdentity( void )
{
    /* not support (out version of system call abstruct) */
    
    return (char_t *)0;
}

/*!
*******************************************************************************
**
*******************************************************************************/
void FAPEX_SYS_StartMultitasking( void )
{
    /* not support */
}
#endif /* _LINUX_FAPI */

/*
********************************************************************************
********************************************************************************
**
** API functions related to ERROR CHECKING
**
********************************************************************************
********************************************************************************
*/
#ifdef _LINUX_FAPI
/*!
*******************************************************************************
**
*******************************************************************************/
void FAPEX_SYS_StartPrintThread( void )
{
    /* not support */
}

/*!
*******************************************************************************
**
*******************************************************************************/
void FAPEX_SYS_StopPrintThread( void )
{
    /* not support */
}

/*!
*******************************************************************************
** \brief Retrieve the last error code of the current thread
**
** This function reads the last error code from the current thread handle.
**
** \return  the last error code value
*******************************************************************************
*/
int32_t FAPEX_SYS_GetLastError( void )
{
    fapex_sys_thread_listT* act_thread = 0;

    act_thread = (fapex_sys_thread_listT*)FAPEX_SYS_GetThread();
    if ( act_thread == NULL )
    {
        return 99; /* to do .... thread not found */
    }
    else
    {
        return( act_thread->lastError );
    }
}
#endif /* _LINUX_FAPI */

/*
********************************************************************************
********************************************************************************
**
** API functions related to CRITICAL SECTION
**
********************************************************************************
********************************************************************************
*/
#ifdef _LINUX_FAPI
static uint32_t        fapex_sys_criticalsection_flag = 0; /* 0:no init, 1:init */
static pthread_mutex_t fapex_sys_criticalsection_mutexT;

/*!
*******************************************************************************
**
*******************************************************************************/
uint32_t FAPEX_SYS_CriticalSection_Init( void )
{
    if ( fapex_sys_criticalsection_flag == 0 )
    {
        fapex_sys_criticalsection_flag = 1;
        pthread_mutex_init( &fapex_sys_criticalsection_mutexT, NULL );
    }
    return FAPEX_SYS_OK;
}

/*!
*******************************************************************************
**
*******************************************************************************/
uint32_t FAPEX_SYS_EnterCriticalSection( void )
{
    FAPEX_SYS_CriticalSection_Init();
    
    pthread_mutex_lock( &fapex_sys_criticalsection_mutexT );
    
    return FAPEX_SYS_OK;
}

/*!
*******************************************************************************
**
*******************************************************************************/
void FAPEX_SYS_LeaveCriticalSection( uint32_t cpuStatus )
{
    pthread_mutex_unlock( &fapex_sys_criticalsection_mutexT );
}
#endif /* _LINUX_FAPI */

/*
********************************************************************************
********************************************************************************
**
** API functions related to MAILBOX
**
********************************************************************************
********************************************************************************
*/
#ifdef _LINUX_FAPI
/* need check... */
//#define FAPEX_SYS_MAILBOX_DEBUG

#ifdef FAPEX_SYS_MAILBOX_DEBUG
#define FAPEX_SYS_MAILBOX_DMSG(fmt,...)    printf(fmt,##__VA_ARGS__)
#else
#define FAPEX_SYS_MAILBOX_DMSG(fmt,...)
#endif

#define FAPEX_SYS_MBOX_USE   1
#define FAPEX_SYS_MBOX_NOUSE 2

#define FAPEX_SYS_MAILBOX_TYPE_POINTER 'P'
#define FAPEX_SYS_MAILBOX_TYPE_BUFFER  'B'

#define FAPEX_SYS_MAILBOX_DESC_NOWAIT  1
#define FAPEX_SYS_MAILBOX_TIMEOUT      2
#define FAPEX_SYS_MAILBOX_NO_DATA      3
#define FAPEX_SYS_MAILBOX_ERROR        4


#define FAPEX_SYS_TIME_WAIT_UNIT      1

typedef struct _FAPEX_SYS_MBOX
{
    int32_t   mailbox_type;
    uint32_t  mailbox_byte;
    uint32_t  use_flag;
    int32_t   mid;
    int32_t   actCnt;
    pthread_mutex_t mutexT; // mutex variable
    pthread_cond_t  condT;  // condition variable
    int32_t   lastError;
    uint32_t  mailbox;
    uint32_t  mailbox1;
    uint32_t  mailbox2;
    uint32_t  mailbox3;
} FAPEX_SYS_MBOX;


#define FAPEX_SYS_MBOX_LIST_MAX  256
FAPEX_SYS_MBOX *fapex_sys_mbox_list[FAPEX_SYS_MBOX_LIST_MAX];
uint32_t fapex_sys_mbox_freenum = 0;
uint32_t fapex_sys_mbox_maxno   = 0;

/*!
*******************************************************************************
**
*******************************************************************************/
static int32_t fapex_sys_set_mbox_list( FAPEX_SYS_MBOX *mbox_addr )
{
    int32_t i;
    int32_t flg = 0;
    int32_t ret = FAPEX_SYS_NG;
    FAPEX_SYS_MBOX *work_mbox_addr = (FAPEX_SYS_MBOX *)mbox_addr;

    if ( fapex_sys_mbox_freenum == fapex_sys_mbox_maxno )
    {

        fapex_sys_mbox_list[fapex_sys_mbox_maxno] = work_mbox_addr;
        work_mbox_addr->mid = fapex_sys_mbox_maxno;
        fapex_sys_mbox_maxno++;
        fapex_sys_mbox_freenum++;
        ret = FAPEX_SYS_OK;

    }
    else
    {

        for(i=0;i<FAPEX_SYS_MBOX_LIST_MAX;i++)
        {
            
            if ( fapex_sys_mbox_list[i] == 0 )
            {
                flg = 1;
                break;
            }
            if ( fapex_sys_mbox_list[i]->use_flag == FAPEX_SYS_MBOX_NOUSE )
            {
                flg = 1;
                break;
            }
        }
        if ( flg == 1 )
        {
                fapex_sys_mbox_freenum++;
                fapex_sys_mbox_list[i] = work_mbox_addr;
                work_mbox_addr->mid = i;
                ret = FAPEX_SYS_OK;
        }
    }

    return ret;
}

/*!
*******************************************************************************
**
*******************************************************************************/
static int32_t fapex_sys_unset_mbox_list( FAPEX_SYS_MBOX *mbox_addr )
{
    int32_t mid;
    int32_t ret = FAPEX_SYS_NG;;
    
    FAPEX_SYS_MBOX  *work_mbox_addr = (FAPEX_SYS_MBOX *)mbox_addr;
    FAPEX_SYS_MBOX  *temp_mbox_addr;
    
    mid = work_mbox_addr->mid;
    temp_mbox_addr = fapex_sys_mbox_list[mid];
    if ( temp_mbox_addr == NULL )
    {
        ret = FAPEX_SYS_NG; /* already unset mailbox */
    }
    else if ( mid == temp_mbox_addr->mid)
    {
        if ( fapex_sys_mbox_freenum == fapex_sys_mbox_maxno  )
        {
            fapex_sys_mbox_maxno--;
        }
        fapex_sys_mbox_freenum--;
        fapex_sys_mbox_list[mid] = NULL;
        ret = FAPEX_SYS_OK;
    }
    else
    {
        ret = FAPEX_SYS_NG; /* system abnormal error */
    }
    
    return ret;
}

/*!
*******************************************************************************
**
*******************************************************************************/
FAPEX_SYS_MailboxT FAPEX_SYS_CreateMailbox( uint32_t messageBytes )
{
    FAPEX_SYS_MBOX *work_mailbox;
    
    work_mailbox = (FAPEX_SYS_MBOX *)FAPI_SYS_MALLOC( sizeof(FAPEX_SYS_MBOX) );
    if ( work_mailbox == NULL )
    {
        return NULL;
    }
    if ( messageBytes == FAPEX_SYS_MSG_IS_POINTER )
    {
        work_mailbox->mailbox_type = FAPEX_SYS_MAILBOX_TYPE_POINTER;
        work_mailbox->mailbox_byte = messageBytes;
    }
    else
    {
        if ( messageBytes > 16 )
        {
            FAPI_SYS_FREE( work_mailbox );
            return NULL;
        }
        work_mailbox->mailbox_type = FAPEX_SYS_MAILBOX_TYPE_BUFFER;
        work_mailbox->mailbox_byte = messageBytes;
    }
    
    fapex_sys_set_mbox_list( work_mailbox );
    
    work_mailbox->actCnt = 0;
    work_mailbox->lastError = 0;

    pthread_mutex_init( &work_mailbox->mutexT, NULL );
    pthread_cond_init( &work_mailbox->condT, NULL );

    FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_CreateMailbox() ID:%03d(%x) create datatype:%c byte:%d \n",
                           work_mailbox->mid, work_mailbox, work_mailbox->mailbox_type,
                           work_mailbox->mailbox_byte );
    return work_mailbox;
}
#endif /* _LINUX_FAPI */

/*!
*******************************************************************************
**
*******************************************************************************/
uint32_t FAPEX_SYS_DestroyMailbox( FAPEX_SYS_MailboxT mailboxHandle )
{
#ifndef _LINUX_FAPI
    uint32_t  ret;

    ret = RTOS_DestroyMailbox( mailboxHandle );
    if( ret == 1 ) {
        return FAPEX_SYS_OK;
    }
    else {
        return FAPEX_SYS_NG;
    }
#else
    FAPEX_SYS_MBOX *work_mailbox = (FAPEX_SYS_MBOX *)mailboxHandle;

    work_mailbox->lastError = 0;

    FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_DestroyMailbox() ID:%03d(%x) destroy \n",
                           work_mailbox->mid, work_mailbox );

    fapex_sys_unset_mbox_list( work_mailbox );
    
    
    FAPI_SYS_FREE( work_mailbox );
    
    return FAPEX_SYS_OK;
#endif /* !_LINUX_FAPI */
}


#ifdef _LINUX_FAPI

#define FAPEX_SYS_TIME_TRUE  1
#define FAPEX_SYS_TIME_FALSE 2

typedef struct timeval FAPEX_SYS_TIME_T;
/*!
*******************************************************************************
**
*******************************************************************************/
static int32_t fapex_sys_time_checkstart( FAPEX_SYS_TIME_T *start_tval )
{
    int32_t ret_work;
    int32_t ret;
    ret_work = gettimeofday( start_tval, NULL );
    if ( ret_work == 0 ) ret = FAPEX_SYS_OK;
    else ret = FAPEX_SYS_NG;

    return ret;
}

/*!
*******************************************************************************
**
*******************************************************************************/
static int32_t fapex_sys_time_check( FAPEX_SYS_TIME_T *start_tval, uint32_t check_time_msec )
{
    FAPEX_SYS_TIME_T *now_tval;
    int32_t ret_work;
    int32_t ret;
    ret_work = gettimeofday( now_tval, NULL );
    if ( ret_work == 0 ) ret = FAPEX_SYS_OK;
    else
    {
        ret = FAPEX_SYS_NG;
        return ret;
    }
    if ( now_tval->tv_usec > start_tval->tv_usec + check_time_msec * 1000 )
    {
        return FAPEX_SYS_TIME_TRUE;
    }
    else
    {
        return FAPEX_SYS_TIME_FALSE;
    }
    
}
#endif /* _LINUX_FAPI */

/*!
*******************************************************************************
**
*******************************************************************************/
uint32_t FAPEX_SYS_SetMailbox( FAPEX_SYS_MailboxT mailboxHandle,
                               void* messagePtr, uint32_t msecsTimeout )
{
#ifndef _LINUX_FAPI
    uint32_t    ret;
    
    ret = RTOS_SetMailbox( mailboxHandle, messagePtr, msecsTimeout );
    if( ret == 1 ) {
        return FAPEX_SYS_OK;
    }
    else {
        return FAPEX_SYS_NG;
    }
#else
    uint32_t ret = FAPEX_SYS_OK;
    FAPEX_SYS_TIME_T starttime;
    
    FAPEX_SYS_MBOX *work_mailbox = (FAPEX_SYS_MBOX*)mailboxHandle;

    work_mailbox->lastError = 0;
    
#ifdef FAPEX_SYS_MAILBOX_DEBUG
    {
        FAPEX_SYS_ThreadT now_th = FAPEX_SYS_GetThread();
        FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_SetMailbox() ID:%03d(%x) call from '",
                               work_mailbox->mid, work_mailbox );
        if ( now_th != NULL ) {
            FAPEX_SYS_MAILBOX_DMSG("%s", FAPEX_SYS_GetThreadName( FAPEX_SYS_GetThread() ) );
        }
        else {
            FAPEX_SYS_MAILBOX_DMSG("NULL");
        }
        FAPEX_SYS_MAILBOX_DMSG("' thread \n");
    }
#endif
    
    pthread_mutex_lock( &work_mailbox->mutexT );

    fapex_sys_time_checkstart( &starttime );
    for(;;)
    {
        
        
        if ( work_mailbox->actCnt == 0 )
        {

            
            if ( work_mailbox->mailbox_type == FAPEX_SYS_MAILBOX_TYPE_POINTER )
            {
                work_mailbox->mailbox = (uint32_t)messagePtr;
                work_mailbox->actCnt  = 1;
            }
            else if ( work_mailbox->mailbox_type == FAPEX_SYS_MAILBOX_TYPE_BUFFER )
            {
                memcpy( &(work_mailbox->mailbox), messagePtr, work_mailbox->mailbox_byte );
                work_mailbox->actCnt = 1;
            }
            else
            {
                ret = FAPEX_SYS_NG;
                work_mailbox->lastError = FAPEX_SYS_MAILBOX_ERROR;
                break;
            }
            FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_SetMailbox() ID:%03d(%x) signal start DATA(%c):"
                                   "%08x:%08x:%08x:%08x\n",
                                   work_mailbox->mid, work_mailbox, work_mailbox->mailbox_type,
                                   work_mailbox->mailbox, work_mailbox->mailbox1,
                                   work_mailbox->mailbox2, work_mailbox->mailbox3 );

            pthread_cond_signal( &work_mailbox->condT );

            FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_SetMailbox() ID:%03d(%x) signal end\n",
                                   work_mailbox->mid, work_mailbox );

            /* to do desc check run ?.....*/
            ret = FAPEX_SYS_OK;
            break;
        }
        else if ( msecsTimeout == FAPEX_SYS_NO_SUSPEND )
        {
            ret = FAPEX_SYS_NG;
            work_mailbox->lastError = FAPEX_SYS_MAILBOX_DESC_NOWAIT;
            break;
        }
        else
        {
            pthread_mutex_unlock( &work_mailbox->mutexT );
            if ( FAPEX_SYS_TIME_TRUE == fapex_sys_time_check( &starttime, msecsTimeout ) )
            {
                ret = FAPEX_SYS_NG;
                work_mailbox->lastError = FAPEX_SYS_MAILBOX_TIMEOUT;
                break;
            }
            else
            {
                FAPI_SYS_SLEEP( FAPEX_SYS_TIME_WAIT_UNIT );
            }
            pthread_mutex_lock( &work_mailbox->mutexT );
        }
    }
    pthread_mutex_unlock( &work_mailbox->mutexT );

    FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_SetMailbox() ID:%03d(%x) ret=%d (err)code=%d msecsTimeout=%d \n",
                           work_mailbox->mid, work_mailbox, ret, work_mailbox->lastError, msecsTimeout );
    return ret;
#endif /* !_LINUX_FAPI */
}

#ifdef _LINUX_FAPI
/*
    tspec = tval + msec
*/
void fapex_sys_time_add( struct timespec *tspec, struct timeval tval, uint32_t msec )
{
    struct timeval  tval_work;
    
    tval_work.tv_sec  = 0;
    tval_work.tv_usec = 0;
    tspec->tv_sec     = 0;
    tspec->tv_nsec    = 0;
    
    /* msec --> tval_work */
    if ( msec >= 1000 )
    {
        tval_work.tv_sec  = msec / 1000;
        tval_work.tv_usec = (msec - tval_work.tv_sec*1000)*1000;
    }
    else
    {
        tval_work.tv_usec = msec * 1000;
    }

    /* tspec_work = tval + tval_work */
    tval_work.tv_sec  += tval.tv_sec;
    tval_work.tv_usec += tval.tv_usec;
    
    if ( tval_work.tv_usec >= 1000000 )
    {
        tspec->tv_sec  = tval_work.tv_usec / 1000000;
        tspec->tv_nsec = (tval_work.tv_usec - tspec->tv_sec * 1000000) * 1000;
    }
    else
    {
        tspec->tv_nsec = tval_work.tv_usec * 1000;
    }
    tspec->tv_sec += tval_work.tv_sec;
}
#endif /* _LINUX_FAPI */

/*!
*******************************************************************************
**
*******************************************************************************/
uint32_t FAPEX_SYS_GetMailbox( FAPEX_SYS_MailboxT mailboxHandle,
                               void* resultBuffer, uint32_t msecsTimeout )
{
#ifndef _LINUX_FAPI
    uint32_t    ret;

    ret = RTOS_GetMailbox( mailboxHandle, resultBuffer, msecsTimeout );
    if( ret == 1 ) {
        return FAPEX_SYS_OK;
    }
    else {
        return FAPEX_SYS_NG;
    }
#else
    FAPEX_SYS_MBOX   *work_mailbox = (FAPEX_SYS_MBOX*)mailboxHandle;
    FAPEX_SYS_TIME_T  time_val;
    void*             message_ptr  = 0;
    struct timespec   time_out;
    int32_t           ret;
    fapex_sys_thread_listT *cur_thread;
    
    work_mailbox->lastError = 0;
    
    cur_thread = FAPEX_SYS_GetThread();


    FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_GetMailbox() ID:%03d(%x) call from '",
                           work_mailbox->mid, work_mailbox );
    if ( cur_thread != NULL ) {
        FAPEX_SYS_MAILBOX_DMSG("%s", FAPEX_SYS_GetThreadName( cur_thread ) );
    }
    else {
        FAPEX_SYS_MAILBOX_DMSG("NULL");
    }
    FAPEX_SYS_MAILBOX_DMSG("' thread \n");


    if( work_mailbox->mailbox_type == FAPEX_SYS_MAILBOX_TYPE_POINTER )
    {
        *((void**)resultBuffer) = 0;
    }
    else
    {
        memset( resultBuffer, 0, work_mailbox->mailbox_byte );
    }

    
    pthread_mutex_lock( &work_mailbox->mutexT );
    
    for(;;)
    {
        if ( work_mailbox->actCnt == 1 )
        {
            if ( work_mailbox->mailbox_type == FAPEX_SYS_MAILBOX_TYPE_POINTER )
            {
                *((void**)resultBuffer) = (void*)(work_mailbox->mailbox);
            }
            else
            {
                message_ptr = &(work_mailbox->mailbox);
                memcpy( resultBuffer, message_ptr, work_mailbox->mailbox_byte );
            }
            work_mailbox->actCnt = 0;
#ifdef FAPEX_SYS_MAILBOX_DEBUG
            {
                char wait_mode[10];
                
                if ( msecsTimeout == FAPEX_SYS_SUSPEND ) strcpy( wait_mode, "WAIT" );
                else strcpy( wait_mode, "TWAIT" );
                FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_GetMailbox() ID:%03d(%x) %s end DATA(%c):"
                                       "%08x:%08x:%08x:%08x\n",
                                       work_mailbox->mid, work_mailbox, wait_mode, work_mailbox->mailbox_type,
                                       work_mailbox->mailbox, work_mailbox->mailbox1, work_mailbox->mailbox2,
                                       work_mailbox->mailbox3 );
            }
#endif
            break;
        }
        else if ( msecsTimeout == FAPEX_SYS_NO_SUSPEND )
        {
            work_mailbox->lastError = FAPEX_SYS_MAILBOX_NO_DATA;
            break;
        }
        else
        {
//          pthread_mutex_unlock( &work_mailbox->mutexT );
            
            if ( msecsTimeout == FAPEX_SYS_SUSPEND )
            {
                cur_thread->optStatus = FAPEX_SYS_THREAD_STATE_WAIT;
                cur_thread->wait_cnt++;

                FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_GetMailbox() ID:%03d(%x) WAIT start (th:%x)\n",
                                       work_mailbox->mid, work_mailbox, cur_thread  );
                FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_GetMailbox() ID:%03d(%x) WAIT optName='%s'\n",
                                       work_mailbox->mid, work_mailbox, cur_thread->optName );
                FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_GetMailbox() ID:%03d(%x) WAIT condT=%x,mutexT=%x\n",
                                       work_mailbox->mid, work_mailbox, work_mailbox->condT,
                                       work_mailbox->mutexT );

                ret = pthread_cond_wait( &work_mailbox->condT, &work_mailbox->mutexT );

                FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_GetMailbox() ID:%03d(%x) WAIT end ret=%d[%s] \n",
                                       work_mailbox->mid, work_mailbox, ret, strerror(ret) );
                if ( ret != 0 )
                {
                    work_mailbox->lastError = FAPEX_SYS_MAILBOX_ERROR;
                    break;
                }
            }
            else
            {
                fapex_sys_time_checkstart( &time_val );
                fapex_sys_time_add( &time_out, time_val, msecsTimeout );
                cur_thread->optStatus = FAPEX_SYS_THREAD_STATE_TIMEWAIT;
                cur_thread->wait_cnt++;

                FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_GetMailbox() ID:%03d(%x) "
                                       "TWAIT start (th:%x) wtime(%d/%d) ntime(%d/%d)\n",
                                       work_mailbox->mid, work_mailbox, cur_thread,
                                       time_out.tv_sec, time_out.tv_nsec,
                                       time_val.tv_sec, time_val.tv_usec );
                FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_GetMailbox() ID:%03d(%x) TWAIT optName='%s'\n",
                                       work_mailbox->mid, work_mailbox, cur_thread->optName );
                FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_GetMailbox() ID:%03d(%x) TWAIT condT=%x,mutexT=%x\n",
                                       work_mailbox->mid, work_mailbox, work_mailbox->condT,
                                       work_mailbox->mutexT );

                ret = pthread_cond_timedwait( &work_mailbox->condT, &work_mailbox->mutexT, &time_out );


                FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_GetMailbox() ID:%03d(%x) TWAIT end ret=%d[%s] \n",
                                       work_mailbox->mid, work_mailbox, ret, strerror(ret) );

                if ( ret == ETIMEDOUT)
                {
                    work_mailbox->lastError = FAPEX_SYS_MAILBOX_TIMEOUT;
                 break;
                }
                else if ( ret == EINTR )
                {
                    FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_GetMailbox() ID:%03d(%x) WAIT NONE (th:%x)\n",
                                           work_mailbox->mid, work_mailbox, cur_thread );
                }
                else
                {
                    FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_GetMailbox() ID:%03d(%x) TWAIT end syserror \n",
                                           work_mailbox->mid, (uint32_t)work_mailbox );
                    work_mailbox->lastError = FAPEX_SYS_MAILBOX_ERROR;
                    break;
                }
            }
//          pthread_mutex_lock( &work_mailbox->mutexT );
        }
    }
#ifdef FAPEX_SYS_MAILBOX_DEBUG
    {
        char wait_mode[10];
        
        if( msecsTimeout == FAPEX_SYS_SUSPEND ) {
            strcpy( wait_mode, "WAIT" );
        }
        else {
            strcpy( wait_mode, "TWAIT" );
        }
        FAPEX_SYS_MAILBOX_DMSG("FAPEX_SYS_GetMailbox() ID:%03d(%x) %s (err)code=%d msecsTimeout=%d (th:%x)\n",
                               work_mailbox->mid, work_mailbox, wait_mode, work_mailbox->lastError,
                               msecsTimeout, cur_thread );
    }
#endif

    cur_thread->optStatus = FAPEX_SYS_THREAD_STATE_ACT;

    pthread_mutex_unlock( &work_mailbox->mutexT );
    
    return FAPEX_SYS_OK;
#endif /* !_LINUX_FAPI */
}

/*
********************************************************************************
********************************************************************************
**
** API functions related to MAILQUEUE
**
********************************************************************************
********************************************************************************
*/
#ifdef _LINUX_FAPI

#define FAPEX_SYS_PRIORITY_ADJUST_CEILING  1
#define FAPEX_SYS_PRIORITY_ADJUST_RESET    0

#define FAPEX_SYS_SET_NULL_VALUE =0
typedef fapex_sys_thread_listT FAPEX_SYS_ThreadInfoT;
#define FAPEX_SYS_EVENT_NAME_SIZE           8                      // size for event name buffers
#define FAPEX_SYS_EVENT_TABLE_SIZE          256                    //!< number of event table entries
#define FAPEX_SYS_MAX_MAILQUEUE_ENTRIES     4096                   // max. number of mailqueue entries
#define FAPEX_SYS_EVENT_TYPE_UNDEFINED      0x00000000UL           // Enent eventType is undefined
#define FAPEX_SYS_THREAD_STATE_EVENT_WAIT   0x00000040UL           // Thread is waiting for event
#define FAPEX_SYS_EVENT_TYPE_MAILQUEUE      0x00000044UL           // Event is of eventType mailqueue
#define FAPEX_SYS_THREAD_STATE_DESTROY_WAIT 0x00000080UL           // Thread is waiting for event to be destroyed

#define FAPEX_SYS_ERR_BASE                  -100000
#define FAPEX_SYS_ERR_INVALID_HANDLE        FAPEX_SYS_ERR_BASE-104 //!< Bad handle passed
#define FAPEX_SYS_ERR_INVALID_PARAMETER     FAPEX_SYS_ERR_BASE-105 //!< Bad parameter passed
#define FAPEX_SYS_ERR_TIMEOUT               FAPEX_SYS_ERR_BASE-106 //!< operation timed out
#define FAPEX_SYS_ERR_OUT_OF_MEMORY         FAPEX_SYS_ERR_BASE-107 //!< Memory allocation failed
#define FAPEX_SYS_ERR_NOT_ALLOWED           FAPEX_SYS_ERR_BASE-108 //!< Not allowed in interrupt context
#define FAPEX_SYS_ERR_HANDLE_DESTROYED      FAPEX_SYS_ERR_BASE-109 //!< Handle destroyed in the meantime

typedef struct
{
    FAPEX_SYS_ThreadT ownerThread;  // thread who owns this event, non-NULL means used.
    uint32_t      eventType;    // event eventType, like ie. FAPEX_SYS_EVENT_TYPE_SEMAPHORE
    uint32_t      refCount;     // reference counter
    char_t        name[FAPEX_SYS_EVENT_NAME_SIZE]; // event name
    uint32_t      actCount;     // actual used queue entries
    uint32_t      messageBytes; // number of bytes per message, must be a multiple of 4
    uint8_t*      messageArray; // the real mailqueue message pointer array
    uint32_t      maxIndex;     // max. index of mail queue
    uint32_t      readIndex;    // current available read index of queue
    uint32_t      writeIndex;   // current available write index of queue

} FAPEX_SYS_MailqueueEventT;

typedef struct
{
    FAPEX_SYS_ThreadT ownerThread; // thread who owns this event, non-NULL means used.
    uint32_t      eventType;   // event eventType, like ie. FAPEX_SYS_EVENT_TYPE_SEMAPHORE
    uint32_t      refCount;    // reference counter
    char_t        name[FAPEX_SYS_EVENT_NAME_SIZE]; // event name
    uint32_t      __unused0;   // dummy data
    uint32_t      __unused1;   // dummy data
    uint32_t      __unused2;   // dummy data
    uint32_t      __unused3;   // dummy data
    uint32_t      __unused4;   // dummy data
    uint32_t      __unused5;   // dummy data

} FAPEX_SYS_GenericEventT;

typedef union
{
    FAPEX_SYS_GenericEventT*   generic;   // mapped as generic event
//  FAPEX_SYS_SemaphoreEventT* semaphore; // mapped as semaphore event
//  FAPEX_SYS_MailboxEventT*   mailbox;   // mapped as mailbox event
    FAPEX_SYS_MailqueueEventT* mailqueue; // mapped as mailqueue event
//  FAPEX_SYS_TimerEventT*     timer;     // mapped as timer event

} FAPEX_SYS_EventInfoT;

uint32_t fapex_sys_EventMailqueueCount = 0;
uint32_t fapex_sys_IsrActive             FAPEX_SYS_SET_NULL_VALUE;
uint32_t fapex_sys_TimerTickPeriodMsecs  FAPEX_SYS_SET_NULL_VALUE;
FAPEX_SYS_EventInfoT    fapex_sys_EventInfoArray[FAPEX_SYS_EVENT_TABLE_SIZE];
FAPEX_SYS_GenericEventT fapex_sys_GenericEventArray[FAPEX_SYS_EVENT_TABLE_SIZE];
uint32_t fapex_sys_GenericEventLastIndex = 0;

#define fapex_sys_CheckEventHandle(handle) \
            (((uint32_t)handle>=(uint32_t)&(fapex_sys_GenericEventArray[0]))\
            &&((uint32_t)handle<=(uint32_t)&(fapex_sys_GenericEventArray[FAPEX_SYS_EVENT_TABLE_SIZE-1]))\
            &&(((FAPEX_SYS_GenericEventT*)handle)->ownerThread))

#define fapex_sys_CheckMailqueueHandle(handle) \
            ((handle)&&(handle->eventType==FAPEX_SYS_EVENT_TYPE_MAILQUEUE))

/*
********************************************************************************
** \brief The thread scheduler
**
** \param from_interrupt  a flag that defines whether the call was initiated
**                        by the end of an ISR or because of an event wait
**                        signal.
********************************************************************************
*/
void fapex_sys_RunScheduler( uint32_t from_interrupt )
{
    FAPI_SYS_SLEEP(1);
    
    return;
}
int fapex_sys_flg_inited = 0;

void fapex_sys_init()
{
    int32_t index;
    for( index=0; index < FAPEX_SYS_EVENT_TABLE_SIZE; index++ )
    {
        memset( &(fapex_sys_GenericEventArray[index]), 0, sizeof(FAPEX_SYS_GenericEventT) );
        fapex_sys_EventInfoArray[index].generic = &(fapex_sys_GenericEventArray[index]);
    }

//  printf("fapex_sys_init() init OK\n");
}
/*
********************************************************************************
** \brief Allocates the next available event handle
**
** This function allocates the next available event info element from the
** event info array.
**
** \retval !0 the pointer to the allocated event info structure
** \retval 0  if no more events available.
********************************************************************************
*/
FAPEX_SYS_EventInfoT* fapex_sys_AllocateEvent( void )
{
    uint32_t          index      = 0;
    FAPEX_SYS_EventInfoT* event_info = 0;

    if ( fapex_sys_flg_inited == 0 )
    {
        fapex_sys_init();
        fapex_sys_flg_inited = 1;
    }
    
    for( index=fapex_sys_GenericEventLastIndex; index < FAPEX_SYS_EVENT_TABLE_SIZE; index++ )
    {
        fapex_sys_GenericEventLastIndex = index;
        if( fapex_sys_GenericEventArray[index].ownerThread == 0 )
        {
            event_info = &(fapex_sys_EventInfoArray[index]);
            event_info->generic->ownerThread = FAPEX_SYS_GetThread();
            /*lint -save -e970 */
            sprintf( event_info->generic->name, "evt#%03d", (int)index );
            /*lint -restore */
            return( event_info );
        }
    }

    for( index=0; index < FAPEX_SYS_EVENT_TABLE_SIZE; index++ )
    {
        fapex_sys_GenericEventLastIndex = index;
        if( fapex_sys_GenericEventArray[index].ownerThread == 0 )
        {
            event_info = &(fapex_sys_EventInfoArray[index]);
            event_info->generic->ownerThread = FAPEX_SYS_GetThread();
            /*lint -save -e970 */
            sprintf( event_info->generic->name, "evt#%03d", (int)index );
            /*lint -restore */
            return( event_info );
        }
    }

    return( 0 );
}

/*!
********************************************************************************
** \brief Temporarilly adjust a thread priority
**
** \param threadHandle The thread handle to access.
** \param adjustMode   The adjust mode to perform, must bei either
**                     #FAPEX_SYS_PRIORITY_ADJUST_CEILING or
**                     #FAPEX_SYS_PRIORITY_ADJUST_RESET.
**
** \retval 1 if thread priority was sucessfully adjusted
** \retval 0 in case of error, if either the given thread identifier or the mode
**           value was invalid.
********************************************************************************
*/
int32_t fapex_sys_ThreadAdjustPriority( FAPEX_SYS_ThreadT threadHandle, uint32_t adjustMode )
{

    return( 1 );
}

/*
********************************************************************************
** \brief Get the current time stamp value
**
** This function simply wraps the call to the FAPI timer function
** "FAPI_TIMER_GetTimeStamp()" to extract the current timer stamp value.
**
** \return  the current time stamp value
********************************************************************************
*/
uint64_t fapex_sys_GetTimerStamp( void )
{
/*
    return( FAPI_TIMER_GetTimeStamp( FAPI_TIMER_RESOLUTION_1_USEC ) );
*/
    /* to do ..... */
    return 1;
}

/*
********************************************************************************
** \brief Release the given event handle
**
** This function releases the given event handle.
**
** \param eventPtr The event info structure to release
********************************************************************************
*/
void fapex_sys_ReleaseEvent( void* eventPtr )
{
    FAPEX_SYS_GenericEventT* event_ptr = (FAPEX_SYS_GenericEventT*)eventPtr;

    if( fapex_sys_CheckEventHandle( eventPtr ) )
    {
        event_ptr->ownerThread = 0;
        event_ptr->eventType = FAPEX_SYS_EVENT_TYPE_UNDEFINED;
        event_ptr->refCount = 0;
        event_ptr->__unused1 = 0;
        event_ptr->__unused2 = 0;
        event_ptr->__unused3 = 0;
        event_ptr->__unused4 = 0;
        memset( event_ptr->name, 0, FAPEX_SYS_EVENT_NAME_SIZE );
    }
}
#endif /* _LINUX_FAPI */

/*!
********************************************************************************
** \brief Create a new mailqueue event structure.
**
** \param queueElements The number of void pointers in the internal queue table
** \param elementBytes  The number of void pointers in the internal queue table
**                      The special value #FAPEX_SYS_MSG_IS_POINTER can be used
**                      to tell the mailqueue to transport a pointers to
**                      messages and not their contentses.
**
** \return
** - An identifier to the newly created mailqueue event structure
** - NULL if either no more mailqueue are available or the requested
**   amount of mailqueue elements could not be dynamically allocated
********************************************************************************
*/
FAPEX_SYS_MailqueueT FAPEX_SYS_CreateMailqueue( uint32_t queueElements,
                                                uint32_t elementBytes )
{
#ifndef _LINUX_FAPI
    return( (RTOS_MailqueueT)RTOS_CreateMailqueue( queueElements,
                                                    elementBytes ) );
#else
    uint32_t               cpu_status      = 0;
//    FAPEX_SYS_ThreadInfoT*     act_thread      = 0;
    fapex_sys_thread_listT*    act_thread      = 0;
    FAPEX_SYS_EventInfoT*      event_info      = 0;
    FAPEX_SYS_MailqueueEventT* mailqueue_event = 0;
    uint8_t*               message_array   = 0;
    uint32_t               bytes           = 0;

    act_thread = (FAPEX_SYS_ThreadInfoT*)FAPEX_SYS_GetThread();
    act_thread->lastError = FAPEX_SYS_OK;

    if( queueElements > FAPEX_SYS_MAX_MAILQUEUE_ENTRIES )
        return( 0 );


    if( elementBytes == FAPEX_SYS_MSG_IS_POINTER )
    {
        bytes = queueElements * sizeof(void*);
    }
    else
    {
        elementBytes = ( elementBytes + 3 ) & ~0x3UL;
        bytes = queueElements * elementBytes;
    }

    //////////////////////////////////
    FAPEX_SYS_EnterCritical( cpu_status );
    //////////////////////////////////

    event_info = fapex_sys_AllocateEvent();
    if( event_info )
    {
        mailqueue_event = event_info->mailqueue;
        message_array = (uint8_t*)FAPI_SYS_MALLOC( bytes );
        if( message_array )
        {
            mailqueue_event->ownerThread = FAPEX_SYS_GetThread();
            mailqueue_event->eventType = FAPEX_SYS_EVENT_TYPE_MAILQUEUE;
            mailqueue_event->refCount = 1;
            mailqueue_event->messageBytes = elementBytes;
            mailqueue_event->messageArray = message_array;
            mailqueue_event->readIndex = 0;
            mailqueue_event->writeIndex = 0;
            mailqueue_event->maxIndex = queueElements-1;
            mailqueue_event->actCount = 0;
            mailqueue_event->name[0] = 'm';
            mailqueue_event->name[1] = 'q';
            mailqueue_event->name[2] = 'u';
            fapex_sys_EventMailqueueCount++;
        }
        else
        {
            fapex_sys_ReleaseEvent( event_info );
            mailqueue_event = 0;
        }
    }
    else act_thread->lastError = FAPEX_SYS_ERR_OUT_OF_MEMORY;

    //////////////////////////////////
    FAPEX_SYS_LeaveCritical( cpu_status );
    //////////////////////////////////

    return( (FAPEX_SYS_MailqueueT)mailqueue_event );

    /* to do....*/
#endif /* !_LINUX_FAPI */
}

/*!
********************************************************************************
** \brief Destroy a mailqueue event structure
**
** \param mailqueuePtr The mailqueue handle to destroy.
**
** \retval 1 if the mailqueue could be destroyed
** \retval 0 if the given mailqueue identifier is invalid.
********************************************************************************
*/
uint32_t FAPEX_SYS_DestroyMailqueue( FAPEX_SYS_MailqueueT mailqueuePtr )
{
#ifndef _LINUX_FAPI
    return( RTOS_DestroyMailqueue( (RTOS_MailqueueT)mailqueuePtr ) );
#else
    /* to do ... */
    uint32_t               cpu_status      = 0;
    FAPEX_SYS_ThreadInfoT*     act_thread      = 0;
    FAPEX_SYS_MailqueueEventT* mailqueue_event = 0;

    act_thread = (FAPEX_SYS_ThreadInfoT*)FAPEX_SYS_GetThread();
    act_thread->lastError = FAPEX_SYS_OK;

    mailqueue_event = (FAPEX_SYS_MailqueueEventT*)mailqueuePtr;
    if( !fapex_sys_CheckMailqueueHandle( mailqueue_event ) )
    {
        act_thread->lastError = FAPEX_SYS_ERR_INVALID_HANDLE;
        return( 0 );
    }

    //////////////////////////////////
    FAPEX_SYS_EnterCritical( cpu_status );
    //////////////////////////////////

    if( mailqueue_event->refCount > 0 )
        mailqueue_event->refCount--;

    if( mailqueue_event->refCount > 0 )
    {
        act_thread->actState = FAPEX_SYS_THREAD_STATE_DESTROY_WAIT;
        act_thread->eventPtr = mailqueue_event;

        //////////////////////////////////
        FAPEX_SYS_LeaveCritical( cpu_status );
        //////////////////////////////////

        fapex_sys_RunScheduler( 0 );

        //////////////////////////////////
        FAPEX_SYS_EnterCritical( cpu_status );
        //////////////////////////////////
    }

    FAPI_SYS_FREE( mailqueue_event->messageArray );
    fapex_sys_ReleaseEvent( mailqueue_event );
    fapex_sys_EventMailqueueCount--;

    act_thread->eventPtr = 0;

    //////////////////////////////////
    FAPEX_SYS_LeaveCritical( cpu_status );
    //////////////////////////////////

    return( 1 );

#endif /* !_LINUX_FAPI */
}

/*!
********************************************************************************
** \brief Get a mailqueue message
**
** \param mailqueueHandle The mailbox event handle to access.
** \param resultBuffer    The buffer where to copy the mailqueue message into,
**                        must be big enough to get the message. The size of
**                        a message must have been given in a previous call
**                        of RTOS_CreateMailqueue().
** \param msecsTimeout    The timeout value in msecs to wait before returning
**                        to the calling function, the following value are
**                        allowed:
**                        - #FAPEX_SYS_SUSPEND<br>
**                          The current thread should wait until the semaphore is free
**                          (blocking mode).
**                        - #FAPEX_SYS_NO_SUSPEND<br>
**                          The current thread should not be blocked (non-blocking mode).
**                        - other<br>
**                          The value represents the time in milliseconds to (at most)
**                          wait for the semaphore (timeout mode).
**
** \retval 1 if a message was successfully read from the mailqueue
** \retval 0 if either the given mailqueue handle is invalid or if this
**           function is called in ISR context
********************************************************************************
*/
uint32_t FAPEX_SYS_GetMailqueue( FAPEX_SYS_MailqueueT mailqueueHandle,
                                 void* resultBuffer, uint32_t msecsTimeout )
{
#ifndef _LINUX_FAPI
    return( RTOS_GetMailqueue( (RTOS_MailqueueT)mailqueueHandle,
                                resultBuffer, msecsTimeout ) );
#else
    /* to do ... */
        uint32_t               cpu_status      = 0;
    FAPEX_SYS_ThreadInfoT*     act_thread      = 0;
    FAPEX_SYS_MailqueueEventT* mailqueue_event = 0;
    uint8_t*               message_ptr8    = 0;
    uint32_t*              message_ptr32   = 0;
    uint32_t               byte_offset     = 0;
    uint32_t               result          = 0;
#if 0  /* get time optim */
#else
    FAPEX_SYS_TIME_T       time_start;
#endif

    act_thread = (FAPEX_SYS_ThreadInfoT*)FAPEX_SYS_GetThread();
    act_thread->lastError = FAPEX_SYS_OK;

    if( !resultBuffer)
    {
        act_thread->lastError = FAPEX_SYS_ERR_INVALID_PARAMETER;
        return( result );
    }

    mailqueue_event = (FAPEX_SYS_MailqueueEventT*)mailqueueHandle;
    if( !fapex_sys_CheckMailqueueHandle( mailqueue_event ) )
    {
        act_thread->lastError = FAPEX_SYS_ERR_INVALID_HANDLE;
        return( result );
    }

    if( mailqueue_event->messageBytes == FAPEX_SYS_MSG_IS_POINTER )
    {
        *((void**)resultBuffer) = 0;
    }
    else
    {
        memset( resultBuffer, 0, mailqueue_event->messageBytes );
    }

    if( ( fapex_sys_IsrActive ) && ( msecsTimeout != FAPEX_SYS_NO_SUSPEND ) )
    {
        act_thread->lastError = FAPEX_SYS_ERR_NOT_ALLOWED;
        return( result );
    }

    //////////////////////////////////
    FAPEX_SYS_EnterCritical( cpu_status );
    //////////////////////////////////

    if( msecsTimeout != FAPEX_SYS_NO_SUSPEND )
    {
#if 0 /* get time optim */
#else
        fapex_sys_time_checkstart( &time_start );
#endif
        while( mailqueue_event->actCount == 0 )
        {
            mailqueue_event->refCount++;

            act_thread->actState = FAPEX_SYS_THREAD_STATE_EVENT_WAIT;
            act_thread->eventPtr = mailqueue_event;
            act_thread->delayCount = msecsTimeout;
            if ( msecsTimeout != FAPEX_SYS_SUSPEND )
            {
#if 0  /* get time optim */
                if( msecsTimeout < fapex_sys_TimerTickPeriodMsecs )
                    msecsTimeout = fapex_sys_TimerTickPeriodMsecs;
                act_thread->delayCount = msecsTimeout / fapex_sys_TimerTickPeriodMsecs;
                act_thread->delayCount++;
#else
                if ( FAPEX_SYS_TIME_TRUE == fapex_sys_time_check( &time_start, msecsTimeout ) )
                {
                    break;
                }
#endif
            }

            //////////////////////////////////
            FAPEX_SYS_LeaveCritical( cpu_status );
            //////////////////////////////////

            fapex_sys_RunScheduler( 0 );

            //////////////////////////////////
            FAPEX_SYS_EnterCritical( cpu_status );
            //////////////////////////////////

            if( mailqueue_event->eventType != FAPEX_SYS_EVENT_TYPE_MAILQUEUE )
            {
                break;
            }

            mailqueue_event->refCount--;
#if 0  /* get time optim */
            if( act_thread->delayCount == 0 )
                break;
#else
#endif
        }
    }

    if( mailqueue_event->eventType == FAPEX_SYS_EVENT_TYPE_MAILQUEUE )
    {
        if( mailqueue_event->actCount == 0 )
        {
            act_thread->lastError = FAPEX_SYS_ERR_TIMEOUT;
        }
        else
        {
//          if( mailqueue_event->actCount > 0 )
//          {
                if( mailqueue_event->messageBytes == FAPEX_SYS_MSG_IS_POINTER )
                {
                    byte_offset = mailqueue_event->readIndex * sizeof(void*);
                    message_ptr32 = (uint32_t*)(void*)&(mailqueue_event->messageArray[byte_offset]);
                    *((void**)resultBuffer) = (void*)(*message_ptr32);
                }
                else
                {
                    byte_offset = mailqueue_event->readIndex * mailqueue_event->messageBytes;
                    message_ptr8 = (uint8_t*)(void*)&(mailqueue_event->messageArray[byte_offset]);
                    memcpy( resultBuffer, message_ptr8, mailqueue_event->messageBytes );
                }

                mailqueue_event->actCount--;
                mailqueue_event->readIndex++;
                if( mailqueue_event->readIndex > mailqueue_event->maxIndex )
                    mailqueue_event->readIndex = 0;
                result = 1;
//          }
//          else act_thread->lastError = FAPEX_SYS_ERR_TIMEOUT;
        }

        act_thread->eventPtr = 0;
    }
    else act_thread->lastError = FAPEX_SYS_ERR_HANDLE_DESTROYED;

    //////////////////////////////////
    FAPEX_SYS_LeaveCritical( cpu_status );
    //////////////////////////////////

    return( result );

#endif /* !_LINUX_FAPI */
}

/*!
********************************************************************************
** \brief Send a message to a mailqueue
**
** \param mailqueueHandle The mailequeue event handle to access.
** \param messagePtr      The message pointer to store in the mailbox.
** \param toFront         A flag which decides whether to put the given message
**                        to the front of the queue (if '1') or to the end.
** \param msecsTimeout    The timeout value in msecs to wait before returning
**                        to the calling function, the following value are
**                        allowed:
**                        - #FAPEX_SYS_SUSPEND<br>
**                          The current thread should wait until the semaphore is free
**                          (blocking mode).
**                        - #FAPEX_SYS_NO_SUSPEND<br>
**                          The current thread should not be blocked (non-blocking mode).
**                        - other<br>
**                          The value represents the time in milliseconds to (at most)
**                          wait for the semaphore (timeout mode).
**
** \retval 1 if the given message was successfully send to the mailqueue
** \retval 0 if either the given mailqueue handle is invalid or if the given message
**           points to NULL.
********************************************************************************
*/
uint32_t FAPEX_SYS_SetMailqueue( FAPEX_SYS_MailqueueT mailqueueHandle,
                                 void* messagePtr, uint32_t toFront,
                                 uint32_t msecsTimeout )
{
#ifndef _LINUX_FAPI
    return( RTOS_SetMailqueue( (RTOS_MailqueueT)mailqueueHandle,
                                messagePtr, toFront, msecsTimeout ) );
#else
    /* to do ... */
       uint32_t               cpu_status      = 0;
    FAPEX_SYS_ThreadInfoT*     act_thread      = 0;
    FAPEX_SYS_MailqueueEventT* mailqueue_event = 0;
    uint8_t*               message_ptr8    = 0;
    uint32_t*              message_ptr32   = 0;
    uint32_t               status          = 0;
#if 0  /* get time optim */
    uint64_t               time_start      = 0;
    uint32_t               time_msecs      = 0;
#else
    FAPEX_SYS_TIME_T       time_start;
#endif
    uint32_t               byte_offset     = 0;
    uint32_t               insert_index    = 0;

    act_thread = (FAPEX_SYS_ThreadInfoT*)FAPEX_SYS_GetThread();
    act_thread->lastError = FAPEX_SYS_OK;

    mailqueue_event = (FAPEX_SYS_MailqueueEventT*)mailqueueHandle;
    if( !fapex_sys_CheckMailqueueHandle( mailqueue_event ) )
    {
        act_thread->lastError = FAPEX_SYS_ERR_INVALID_HANDLE;
        return( 0 );
    }

    if( fapex_sys_IsrActive && ( msecsTimeout != FAPEX_SYS_NO_SUSPEND ) )
    {
        msecsTimeout = FAPEX_SYS_NO_SUSPEND;
    }

    (void)fapex_sys_ThreadAdjustPriority( act_thread, FAPEX_SYS_PRIORITY_ADJUST_CEILING );

    //////////////////////////////////
    FAPEX_SYS_EnterCritical( cpu_status );
    //////////////////////////////////
#if 0  /* get time optim */
    time_start = fapex_sys_GetTimerStamp();
#else
    fapex_sys_time_checkstart( &time_start );
#endif    
    for(;;)
    {
        if( mailqueue_event->actCount <= mailqueue_event->maxIndex )
        {
            if( toFront )
            {
                if( mailqueue_event->readIndex > 0 ) mailqueue_event->readIndex--;
                else mailqueue_event->readIndex = mailqueue_event->maxIndex;
                insert_index = mailqueue_event->readIndex;
            }
            else
            {
                insert_index = mailqueue_event->writeIndex;
                mailqueue_event->writeIndex++;
                if( mailqueue_event->writeIndex > mailqueue_event->maxIndex )
                    mailqueue_event->writeIndex = 0;
            }

            if( mailqueue_event->messageBytes == FAPEX_SYS_MSG_IS_POINTER )
            {
                byte_offset = insert_index * sizeof(void*);
                message_ptr32 = (uint32_t*)(void*)&(mailqueue_event->messageArray[byte_offset]);
                *message_ptr32 = (uint32_t)messagePtr;
            }
            else
            {
                byte_offset = insert_index * mailqueue_event->messageBytes;
                message_ptr8 = (uint8_t*)(void*)&(mailqueue_event->messageArray[byte_offset]);
                memcpy( message_ptr8, messagePtr, mailqueue_event->messageBytes );
            }
            mailqueue_event->actCount++;
            status = 1;
            break;
        }
        else if( msecsTimeout == FAPEX_SYS_NO_SUSPEND )
        {
           act_thread->lastError = FAPEX_SYS_ERR_OUT_OF_MEMORY;
           break;
        }
        else // msecsTimeout is either FAPEX_SYS_SUSPEND or a real msecs value
        {
            //////////////////////////////////
            FAPEX_SYS_LeaveCritical( cpu_status );
            //////////////////////////////////

            (void)fapex_sys_ThreadAdjustPriority( act_thread, FAPEX_SYS_PRIORITY_ADJUST_RESET );
            (void)FAPI_SYS_SLEEP( fapex_sys_TimerTickPeriodMsecs );
            (void)fapex_sys_ThreadAdjustPriority( act_thread, FAPEX_SYS_PRIORITY_ADJUST_CEILING );

            if( msecsTimeout != FAPEX_SYS_SUSPEND )
            {
#if 0  /* get time optim */
                time_msecs = (uint32_t)( ( ( fapex_sys_GetTimerStamp() - time_start )
                                           / 1000 ) & 0xFFFFFFFFUL );
                if( time_msecs > msecsTimeout )
#else
                if ( FAPEX_SYS_TIME_TRUE == fapex_sys_time_check( &time_start, msecsTimeout ) )
#endif
                {
                    act_thread->lastError = FAPEX_SYS_ERR_TIMEOUT;
                    break;
                }
            }

            //////////////////////////////////
            FAPEX_SYS_EnterCritical( cpu_status );
            //////////////////////////////////
        }
    }

    //////////////////////////////////
    FAPEX_SYS_LeaveCritical( cpu_status );
    //////////////////////////////////

    (void)fapex_sys_ThreadAdjustPriority( act_thread, FAPEX_SYS_PRIORITY_ADJUST_RESET );

    if( !fapex_sys_IsrActive )
    {
        fapex_sys_RunScheduler( 0 );
    }
    return( status );

#endif /* !_LINUX_FAPI */
}

/*
********************************************************************************
********************************************************************************
**
** API functions related to File System
**
********************************************************************************
********************************************************************************
*/
#ifdef _LINUX_FAPI

//#define FAPEX_SYS_FILE_DEBUG

#ifdef FAPEX_SYS_FILE_DEBUG
#define FAPEX_SYS_FILE_DMSG(fmt,...)    printf(fmt,##__VA_ARGS__)
#else
#define FAPEX_SYS_FILE_DMSG(fmt,...)
#endif

/*!
*******************************************************************************/
static int32_t fapex_sys_GetFState(FAPEX_SYS_FFindStrT *find, struct stat *tmp_stat)
{
    struct dirent *tmp_dirent = NULL;
    char_t        tmp_name[FAPEX_SYS_FMAXPATH];
    char_t        *str_ptr;
    int32_t       length;

    while(1) {
        str_ptr    = NULL;
        tmp_dirent = readdir(find->dir);
        if(tmp_dirent == NULL) { /* empty directory */
            closedir(find->dir);
            find->dir = 0;
            return -1;
        }
        /* check extension */
        if(find->findfsname.ext[0] != '\0') {
            str_ptr = strrchr(tmp_dirent->d_name, '.');
            if(str_ptr == NULL) {
                continue;
            }
            if( (strcmp(find->findfsname.ext, "*") != 0)
             && (strcmp(find->findfsname.ext, str_ptr+1) != 0) ) {
                continue;
            }
            if( (strcmp(tmp_dirent->d_name, "." ) == 0)
             || (strcmp(tmp_dirent->d_name, "..") == 0) ) {
                continue;
            }
        }
        /* check name */
        if(find->findfsname.name[0] != '\0') {
            length = strlen(find->findfsname.name);
            if(strncmp(tmp_dirent->d_name, find->findfsname.name, length) != 0) {
                continue;
            }
        }
        break;
    }

    length = snprintf(tmp_name, FAPEX_SYS_FMAXPATH, "%s/%s", find->filepath, tmp_dirent->d_name);
    if(length > FAPEX_SYS_FMAXPATH){
        FAPEX_SYS_FILE_DMSG("ERROR:filename is too long.\n");
        closedir(find->dir);
        find->dir = 0;
        return -1;
    }

    if(lstat(tmp_name, tmp_stat) == -1) {
        FAPEX_SYS_FILE_DMSG("lstat error \"%s\"\n", strerror(errno));
        return -1;
    }

    strncpy(find->filename, tmp_dirent->d_name, FAPEX_SYS_FMAXPATH);

    if(find->findfsname.ext[0] == '\0') {
        str_ptr = strrchr(tmp_dirent->d_name, '.');
    }
    if( (str_ptr != NULL)
     && (strcmp(tmp_dirent->d_name, "." ) != 0)
     && (strcmp(tmp_dirent->d_name, "..") != 0) ) {
        strncpy(find->ext, str_ptr+1, FAPEX_SYS_FMAXNAME);
        *str_ptr = '\0';
    }
    else {
        find->ext[0] = '\0';
    }
    strncpy(find->name, tmp_dirent->d_name, FAPEX_SYS_FMAXNAME);

    return 0;
}

/*!
*******************************************************************************/
static uint16_t fapex_sys_GetCTime(struct tm *now)
{
    uint16_t  ctime = 0;
    
    ctime =  (now->tm_sec  >> 1                          ) & FAPEX_SYS_FCTIME_SEC_MASK;
    ctime |= (now->tm_min  << FAPEX_SYS_FCTIME_MIN_SHIFT ) & FAPEX_SYS_FCTIME_MIN_MASK;
    ctime |= (now->tm_hour << FAPEX_SYS_FCTIME_HOUR_SHIFT) & FAPEX_SYS_FCTIME_HOUR_MASK;

    return ctime;
}

/*!
*******************************************************************************/
static uint16_t fapex_sys_GetCDate(struct tm *now)
{
    uint16_t  cdate = 0;
    
    cdate =  (now->tm_mday                                       ) & FAPEX_SYS_FCDATE_DAY_MASK;
    cdate |= ((now->tm_mon+1)     << FAPEX_SYS_FCDATE_MONTH_SHIFT) & FAPEX_SYS_FCDATE_MONTH_MASK;
    cdate |= ((now->tm_year - 80) << FAPEX_SYS_FCDATE_YEAR_SHIFT ) & FAPEX_SYS_FCDATE_YEAR_MASK;

    FAPEX_SYS_FILE_DMSG("tm_mday:%d, tm_mon:%d, tm_year:%d\n",
                            now->tm_mday, now->tm_mon, now->tm_year);

    return cdate;
}

/*!
*******************************************************************************/
static int32_t fapex_sys_GetFPropaty(FAPEX_SYS_FFindStrT *find, struct stat *tmp_stat)
{
    struct tm  *tmp_tm = 0;

    /* check args */
    if(find == NULL) {
        return -1;
    }
    if(tmp_stat == NULL) {
        return -1;
    }

    /* get file propaty */
    switch (tmp_stat->st_mode & S_IFMT) {
        case S_IFBLK:  
            FAPEX_SYS_FILE_DMSG("block device\n");
            tmp_tm = localtime(&(tmp_stat->st_mtime));
            find->ctime = fapex_sys_GetCTime(tmp_tm);
            find->cdate = fapex_sys_GetCDate(tmp_tm);
            find->attr  = FAPEX_SYS_FATTR_SYSTEM;
            find->filesize = 0;
            break;
        case S_IFCHR:  
            FAPEX_SYS_FILE_DMSG("character device\n");
            tmp_tm = localtime(&(tmp_stat->st_mtime));
            find->ctime = fapex_sys_GetCTime(tmp_tm);
            find->cdate = fapex_sys_GetCDate(tmp_tm);
            find->attr  = FAPEX_SYS_FATTR_SYSTEM;
            find->filesize = 0;
            break;
        case S_IFDIR:  
            tmp_tm = localtime(&(tmp_stat->st_mtime));
            find->ctime = fapex_sys_GetCTime(tmp_tm);
            find->cdate = fapex_sys_GetCDate(tmp_tm);
            find->attr  = FAPEX_SYS_FATTR_DIR;
            find->filesize = 0;
            break;
        case S_IFIFO:  
            FAPEX_SYS_FILE_DMSG("FIFO/pipe\n");
            tmp_tm = localtime(&(tmp_stat->st_mtime));
            find->ctime = fapex_sys_GetCTime(tmp_tm);
            find->cdate = fapex_sys_GetCDate(tmp_tm);
            find->attr  = FAPEX_SYS_FATTR_SYSTEM;
            find->filesize = 0;
            break;
        case S_IFLNK:  
            FAPEX_SYS_FILE_DMSG("symlink\n");  
            tmp_tm = localtime(&(tmp_stat->st_mtime));
            find->ctime = fapex_sys_GetCTime(tmp_tm);
            find->cdate = fapex_sys_GetCDate(tmp_tm);
            find->attr  = FAPEX_SYS_FATTR_SYSTEM;
            find->filesize = 0;
            break;
        case S_IFREG:  
            FAPEX_SYS_FILE_DMSG("regular file\n");
            tmp_tm = localtime(&(tmp_stat->st_mtime));
            find->ctime = fapex_sys_GetCTime(tmp_tm);
            find->cdate = fapex_sys_GetCDate(tmp_tm);
            find->attr  = 0;
            find->filesize = (unsigned long)(tmp_stat->st_size);
            break;
        case S_IFSOCK: 
            FAPEX_SYS_FILE_DMSG("socket\n");
            tmp_tm = localtime(&(tmp_stat->st_mtime));
            find->ctime = fapex_sys_GetCTime(tmp_tm);
            find->cdate = fapex_sys_GetCDate(tmp_tm);
            find->attr  = FAPEX_SYS_FATTR_SYSTEM;
            find->filesize = 0;
            break;
        default:
            FAPEX_SYS_FILE_DMSG("unknown?\n");
            ;
    }
    return 0;
}

/*!
*******************************************************************************
** \brief   Find first file or subdirectory in a specified directory.
**
** \param   filename    name of file to find.
** \param   find        where to store find information.
**
** \retval  0  sucessful
** \retval -1  unsuccessful
******************************************************************************/
int32_t FAPEX_SYS_FFindFirst(const char *filename, FAPEX_SYS_FFindStrT *find)
{
    struct stat  tmp_stat;
    char_t       tmp_name[FAPEX_SYS_FMAXPATH];
    char_t       *str_ptr;
    int32_t      ret;

    /* check args */
    if(filename == NULL) {
        return -1;
    }
    if(find == NULL) {
        return -1;
    }

    /* Initialize find structure */
    find->attr               = 0;
    find->ctime              = 0;
    find->cdate              = 0;
    find->filesize           = 0;
    find->cluster            = 0;
    find->dir                = 0;
    find->name[0]            = '\0';
    find->ext [0]            = '\0';
    find->findfsname.name[0] = '\0';
    find->findfsname.ext [0] = '\0';

    strncpy(tmp_name, filename, FAPEX_SYS_FMAXPATH);

    /* check wild-card */
    if(strstr(tmp_name, "*") != NULL) {
        /* check extension */
        str_ptr = strrchr(tmp_name, '.');
        if((str_ptr != NULL) && (strrchr(str_ptr, '/') == NULL)) {
            if(strcmp(str_ptr, ".*") == 0) {
                strcpy(find->findfsname.ext, "*");
            }
            else {
                strncpy(find->findfsname.ext, str_ptr+1, FAPEX_SYS_FMAXNAME);
            }
        }

        /* check directory path */
        str_ptr = strrchr(tmp_name, '/');
        if(str_ptr == NULL) {
            strncpy(find->findfsname.name, tmp_name, FAPEX_SYS_FMAXNAME);
            strcpy(tmp_name, ".");
        }
        else {
            *str_ptr = '\0';
            strncpy(find->findfsname.name, str_ptr+1, FAPEX_SYS_FMAXNAME);
        }
        str_ptr = strchr(find->findfsname.name, '*');
        *str_ptr = '\0';

        FAPEX_SYS_FILE_DMSG("name: \"%s\"\n", find->findfsname.name);
        FAPEX_SYS_FILE_DMSG("ext : \"%s\"\n", find->findfsname.ext);
    }

    /* check file/dir exist */
    if(lstat(tmp_name, &tmp_stat) == -1) {
        FAPEX_SYS_FILE_DMSG("lstat error \"%s\"\n", strerror(errno));
        return -1;
    }

    /* check dir or not ? */
    if((tmp_stat.st_mode & S_IFMT) == S_IFDIR) {
        FAPEX_SYS_FILE_DMSG("directory\n");

        find->dir = opendir(tmp_name);
        strncpy(find->filepath, tmp_name, FAPEX_SYS_FMAXPATH);

        ret = fapex_sys_GetFState(find, &tmp_stat);
        if(ret < 0) {
            return ret;
        }
    }
    else {
        strncpy(find->filename, tmp_name, FAPEX_SYS_FMAXPATH);
    }

    /* get file propaty */
    ret = fapex_sys_GetFPropaty(find, &tmp_stat);

    return ret;
}

/*!
*******************************************************************************
** \brief   Finds the next file or subdirectory in a specified directory after
**          a previous call to FAPEX_SYS_FFindFirst or FAPEX_SYS_FFindNext
**
** \param   find   find information (created by FAPEX_SYS_FFindFirst call)
**
** \retval  0  sucessful
** \retval -1  unsuccessful
******************************************************************************/
int32_t FAPEX_SYS_FFindNext(FAPEX_SYS_FFindStrT *find)
{
    struct stat  tmp_stat;
    int32_t      ret;

    /* check args */
    if(find == NULL) {
        return -1;
    }
    if(find->dir == 0) {
        return -1;
    }

    /* next file read */
    ret = fapex_sys_GetFState(find, &tmp_stat);
    if(ret < 0) {
        return ret;
    }

    /* get file propaty */
    ret = fapex_sys_GetFPropaty(find, &tmp_stat);

    return ret;
}

/*!
*******************************************************************************
******************************************************************************/
long FAPEX_SYS_FFileLength(const char *filename)
{
    FILE  *fd = NULL;

    long f_len = 0;

    fd = fopen(filename, "r");

    if (NULL == fd){
        return -1;
    }

    fseek(fd, 0, SEEK_END);
    f_len = ftell(fd);

    if (fd != NULL){
        fclose(fd);
        fd = NULL;
    }
    return f_len;
}
#endif /* _LINUX_FAPI */

/*
********************************************************************************
********************************************************************************
**
** other API functions
**
********************************************************************************
********************************************************************************
*/
#ifdef _LINUX_FAPI
/*!
*******************************************************************************
**
*******************************************************************************/
int32_t FAPEX_SYS_KEYIN( char_t *keycode, int32_t *length )
{
    struct termio ti_bak;
    struct termio ti_tmp;
    int32_t read_byte = 0;

    memset( keycode, 0, *length );


    /* save term info */
    ioctl(0, TCGETA, &ti_bak);

    /* chage term info and update */
    ti_tmp = ti_bak;
    ti_tmp.c_cc[VTIME] = 1; /* no input wait time */
    ti_tmp.c_cc[VMIN] = 1;
    ti_tmp.c_lflag &= ~(ECHO | ICANON); /* set noecho and raw mode */
    //ti_tmp.c_lflag &= ~(ECHO); /* set noecho and raw mode */
    ioctl(0, TCSETAF, &ti_tmp); /* set data */

    for(;;)
    {
        read_byte = read(0, keycode, *length );
        if ( read_byte == -1)
        {
            /* restore term info */
            ioctl(0, TCSETAF, &ti_bak);
            return FAPEX_NG;
        }
        else if ( *keycode != 0x00)
        {
            break;
        }
        else {
        //  printf("No keycode data\n"); /* debug */
            break;
        }

        memset( keycode, 0, *length );
    }

  /* restore term info */
  ioctl(0, TCSETAF, &ti_bak);

  *length = read_byte;

  return FAPEX_OK;
}
#endif

#ifdef _LINUX_FAPI
/*!
*******************************************************************************
**
** \brief   Calculate CRC32 checksum for a given data array.
**
** \param   dataPtr     Pointer to data array.
** \param   dataLength  Size of data array in bytes.
** \param   crcTable    Pointer to the CRC LUT to be used. This array should
**                      be included as a header file by the user of the CRC
**                      function. (eg: \#include <fapi/sys_crc_0x82608EDB.h>)
**                      The hex number represents the polynom used for
**                      calculating the CRC.
**
** \return  CRC32 checksum value.
******************************************************************************/
uint32_t FAPEX_SYS_CalculateCrc32(const uint8_t* dataPtr,
                                 const uint32_t dataLength,
                                 const uint32_t* const crcTable)
{
    register uint32_t index;
    uint32_t          crc = 0xFFFFFFFF;

    for(index = 0; index < dataLength; index++)
        crc = (crc << 8) ^ crcTable[((crc >> 24) ^ *dataPtr++) & 0xFF];

    return crc;
}
#endif /* _LINUX_FAPI */

#ifdef _LINUX_FAPI
/*!
*******************************************************************************
** \brief Retrievs the current counter of a given timer
**
** This function retrieves the current 64 bit counter value of the hardware
** timer. Based on the given resolution value the current hardware timer
** counter will be divided to return the current value in the requested
** resolution.
**
** \param resolution  The resolution value to set for the given timer block as follows.
**              - #FAPEX_TIMER_RESOLUTION_1_USEC<br>
**                timer resolution 1 usec
**              - #FAPEX_TIMER_RESOLUTION_10_USEC<br>
**                timer resolution 10 usec
**              - #FAPEX_TIMER_RESOLUTION_100_USEC<br>
**                timer resolution 100 usec
**              - #FAPEX_TIMER_RESOLUTION_1_MSEC<br>
**                timer resolution 1 msec
**              - #FAPEX_TIMER_RESOLUTION_10_MSEC<br>
**                timer resolution 10 msec
**              - #FAPEX_TIMER_RESOLUTION_100_MSEC<br>
**                timer resolution 100 msec
**              - #FAPEX_TIMER_RESOLUTION_1_SEC<br>
**                timer resolution 1 sec
**
** \retval 0   unsuccessful, if the resolution value is out of range
** \retval >0  the current timer counter value 1..0xFFFFFFFFFFFFFFFF
*******************************************************************************/
uint64_t FAPEX_TIMER_GetTimeStamp( uint32_t resolution )
{
    struct timeval  t_value;
    uint64_t        ret_value;

    gettimeofday(&t_value, NULL);

    switch(resolution) {
        case FAPEX_TIMER_RESOLUTION_1_USEC:
            ret_value  = (uint64_t)t_value.tv_sec * 1000000;
            ret_value += (uint64_t)t_value.tv_usec;
            break;
        case FAPEX_TIMER_RESOLUTION_10_USEC:
            ret_value  = (uint64_t)t_value.tv_sec  * 100000;
            ret_value += (uint64_t)t_value.tv_usec / 10;
            break;
        case FAPEX_TIMER_RESOLUTION_100_USEC:
            ret_value  = (uint64_t)t_value.tv_sec  * 10000;
            ret_value += (uint64_t)t_value.tv_usec / 100;
            break;
        case FAPEX_TIMER_RESOLUTION_1_MSEC:
            ret_value  = (uint64_t)t_value.tv_sec  * 1000;
            ret_value += (uint64_t)t_value.tv_usec / 1000;
            break;
        case FAPEX_TIMER_RESOLUTION_10_MSEC:
            ret_value  = (uint64_t)t_value.tv_sec  * 100;
            ret_value += (uint64_t)t_value.tv_usec / 10000;
            break;
        case FAPEX_TIMER_RESOLUTION_100_MSEC:
            ret_value  = (uint64_t)t_value.tv_sec  * 10;
            ret_value += (uint64_t)t_value.tv_usec / 10000;
            break;
        case FAPEX_TIMER_RESOLUTION_1_SEC:
            ret_value = (uint64_t)t_value.tv_sec;
            break;
        default:
            ret_value = 0;
    }
    return ret_value;
}
#endif /* _LINUX_FAPI */
