/*!
*****************************************************************************
**
** \file        fapex/abst_libaudec_ddpdcv.h
** 
** \version     $Id: drv_audec_ddpdcv.h 3088 2009-04-02 15:16:31Z spurke $
**
** \brief       DD+ Decoder Converter specific parameters.
**
** This file contains the declaration of specific parameters for the DDPDCV codec.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** (C) Copyright 2006-2010 Fujitsu Semiconductor Europe GmbH, All Rights Reserved.
** (C) Copyright 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*****************************************************************************
*/

#ifndef _FAPEX_ABST_LIBAUDEC_DDPDCV_H
    #define _FAPEX_ABST_LIBAUDEC_DDPDCV_H

    #include <fapex/abst_lib.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************


    /*!
    *************************************************************************
    ** \brief Parameters for the DD+ decoder converter.
    **
    ** This data structure covers all parameters which can be set for the DD+
    ** decoder converter.
    ** Setting these parameters is optional. If not the codec applies the 
    ** mentioned defaults. 
    **
    ** \note The various parameters are defined by the Dolby Digital Plus
    **       Subroutine API
    **
    ** \sa FAPEX_AUDEC_Ioctl/FAPEX_IOCCOM_AUDEC_SET_CODEC_PARAMS
    *************************************************************************
    */
    typedef struct {

        /*!
           The audio decoder ID.
           \note This field must be set to FAPEX_AUDEC_CODEC_TYPE_DDPDCV.
        */
        int32_t  decoder;
        
        /*!
           Enable AC-3 through SPDIF. 
           If set this switch tells the DD+ decoder/converter to output the converted 
           AC-3 via SPDIF. Otherwise it will output the PCM of the first 
           two speakers in the associated audio output (AUOUT) via SPDIF as usual.
           \note This switch takes only effect if SPDIF has been enabled for the
                 audio output associated with the DD+ decoder/converter.
           A value of 0 or 1. (Codec's default: 0)
        */
        uint32_t ac3ToSpdif;
        
        /*!
            This parameter is not supported and must be set to NULL.
        */
        void  *ac3OutBuffer;
        
        /*!
           Dolby Digital Plus upsampling mode.
           - 0: Upsample based on bitstream metadata.
           - 1: Always perform upsampling.
           - 2: Never perform upsampling.
           (Codec's default: 0)
        */
        uint32_t upsampleMode;
        
        /*!
           Dolby Digital Plus compression mode.
           A value between 0 and 3. (Codec's default: 2)
        */
        uint32_t compMode;
        
        /*!
           Dolby Digital Plus stereo output mode.
           A value between 0 and 2. (Codec's default: 0)
        */
        uint32_t stereoMode;
        
        /*!
           Dolby Digital Plus dual mono reproduction mode.
           A value between 0 and 3. (Codec's default: 0)
        */
        uint32_t dualMonoMode;
        
        /*!
           Dolby Digital Plus output channel configuration.
           A value between 1 to 7 or 9 to 15. (Codec's default: -1)
           
           \note Bit 3 determines the LFE presence and bit 0-2
           corresponds to the channel configuration code as described
           by the Dolby Digital Plus Subroutine API document.
           
           The output channel configuration is usually automatically derived 
           from the AUOUT driver's speaker configuration. Sometimes 
           it could be desirable to overwrite this approach. For this purpose 
           this parameter can be used. If the codec's current channel 
           configuration should not be overwritten it must be set to -1.
        */
        int32_t outChannelCfg;
        
        /*!
           Dolby Digital Plus maximum repeat count before muting. 
           A value between -1 and 0. (Codec's default: -1)
        */
        int32_t maxRepCount;
        
        /*!
           Dummy to fill the gap.
        */
        uint32_t dummy;
        
        /*!
           Dolby Digital Plus dynamic range scale factor high level signals.
           A value between 0.0 and 1.0. (Codec's default: 1.0)
        */        
        double dynsclh;
        
        /*!
           Dolby Digital Plus dynamic range scale factor low level signals.
           A value between 0.0 and 1.0. (Codec's default: 1.0)
        */  
        double dynscll;
        
        /*!
           Dolby Digital Plus PCM scale factor.
           A value between 0.0 and 1.0. (Codec's default: 1.0)
        */  
        double pcmscl;

    } FAPEX_AUDEC_DdpdcvParamStrT;

#endif
