/*!
*************************************************************************
**
** \file    fapex/abst_libhdmi.h
**
** \brief   abstraction library header file (HDMI driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBHDMI_H
#define _FAPEX_ABST_LIBHDMI_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex HDMI error code base value */
#define FAPEX_HDMI_ERR_BASE             (FAPEX_HDMI_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_HDMI_ERR_BAD_PARAMETER    (FAPEX_HDMI_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_HDMI_ERR_OUT_OF_MEMORY    (FAPEX_HDMI_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*! Timeout occured. */
#define FAPEX_HDMI_ERR_TIMEOUT          (FAPEX_HDMI_ERR_BASE + FAPEX_ERR_TIMEOUT)

/*! Invalid status. */
#define FAPEX_HDMI_ERR_INVALID_STATUS   (FAPEX_HDMI_ERR_BASE + FAPEX_ERR_INVALID_HANDLE)

/*!
********************************************************************************
** driver status error codes
********************************************************************************
*/
/*! \brief HDMI status ok */
#define FAPEX_HDMI_STATUS_OK                            (FAPEX_OK)

/* \brief HDMI status error base */
#define FAPEX_HDMI_STATUS_ERR_BASE                      (FAPEX_HDMI_MODULE_BASE - 100)

/*! \brief HDMI EDID read error */
#define FAPEX_HDMI_STATUS_ERR_EDID                      (FAPEX_HDMI_STATUS_ERR_BASE - 20)
/*! \brief HDMI DDC timeout error */
#define FAPEX_HDMI_STATUS_ERR_DDC_TIMEOUT               (FAPEX_HDMI_STATUS_ERR_BASE - 21)
/*! \brief HDMI HDCP timeout occured */
#define FAPEX_HDMI_STATUS_ERR_HDCP_TIMEOUT              (FAPEX_HDMI_STATUS_ERR_BASE - 30)
/*! \brief HDMI HDCP exceed device count */
#define FAPEX_HDMI_STATUS_ERR_HDCP_MAX_DEVS_EXCEED      (FAPEX_HDMI_STATUS_ERR_BASE - 31)
/*! \brief HDMI HDCP cascade exceeded */
#define FAPEX_HDMI_STATUS_ERR_HDCP_MAX_CASCADE_EXCEEDED (FAPEX_HDMI_STATUS_ERR_BASE - 32)
/*! \brief HDMI HDCP SHA-1 mismatch */
#define FAPEX_HDMI_STATUS_ERR_HDCP_SHA_1_MISMATCH       (FAPEX_HDMI_STATUS_ERR_BASE - 33)
/*! \brief HDMI HDCP AN DDC Error */
#define FAPEX_HDMI_STATUS_ERR_HDCP_AN_DDC               (FAPEX_HDMI_STATUS_ERR_BASE - 34)
/*! \brief HDMI HDCP AKSV DDC Error */
#define FAPEX_HDMI_STATUS_ERR_HDCP_AKSV_DDC             (FAPEX_HDMI_STATUS_ERR_BASE - 35)
/*! \brief HDMI HDCP BKSV Error */
#define FAPEX_HDMI_STATUS_ERR_HDCP_BKSV                 (FAPEX_HDMI_STATUS_ERR_BASE - 36)
/*! \brief HDMI HDCP RI failure */
#define FAPEX_HDMI_STATUS_ERR_HDCP_RI_FAILURE           (FAPEX_HDMI_STATUS_ERR_BASE - 37)
/*! \brief HDMI HDCP RI failure */
#define FAPEX_HDMI_STATUS_ERR_HDCP_RI_NOT_ABAILABLE     (FAPEX_HDMI_STATUS_ERR_BASE - 38)
/*! \brief HDMI HPD not detect */
#define FAPEX_HDMI_STATUS_ERR_HPD_NOT_DETECT            (FAPEX_HDMI_STATUS_ERR_BASE - 40)
/*! \brief HDMI HDCP BKSV exist in Revocation Lists */
#define FAPEX_HDMI_STATUS_ERR_EXIST_IN_REVOCATION_LISTS (FAPEX_HDMI_STATUS_ERR_BASE - 41)
/*! |brief HDCP feature is not available. */
#define FAPEX_HDMI_STATUS_ERR_HDCP_NOT_SUPPORT          (FAPEX_HDMI_STATUS_ERR_BASE - 42)
/*! \brief HDMI HDCP AKSV Error */
#define FAPEX_HDMI_STATUS_ERR_HDCP_AKSV                 (FAPEX_HDMI_STATUS_ERR_BASE - 43)

/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

// HDMI Video Input/Output Type
#define FAPEX_HDMI_RGB            0           //!< RGB
#define FAPEX_HDMI_YUV422         1           //!< YUV422
#define FAPEX_HDMI_YUV444         2           //!< YUV444

// HDMI Scan information
#define FAPEX_HDMI_SCAN_NO_DATA   0           //!< No data
#define FAPEX_HDMI_OVERSCAN       1           //!< overscanned display
#define FAPEX_HDMI_UNDERSCAN      2           //!< underscanned display

/* HDCP status */
#define FAPEX_HDMI_HDCP_STATUS_FIRST_AUTHENTICATION     (0x10)
#define FAPEX_HDMI_HDCP_STATUS_WAIT_FOR_RXRDY           (0x30)
#define FAPEX_HDMI_HDCP_STATUS_SECOND_AUTHENTICATION    (0x40)
#define FAPEX_HDMI_HDCP_STATUS_WAIT_FIFORDY             (0x50)
#define FAPEX_HDMI_HDCP_STATUS_FINISH_REPEATER_CHECK    (0x60)
#define FAPEX_HDMI_HDCP_STATUS_THIRD_AUTHENTICATION     (0x70)
#define FAPEX_HDMI_HDCP_STATUS_REQ_NOHDCP               (0x80)
#define FAPEX_HDMI_HDCP_STATUS_NO_ENCRYPTION            (0x90)

/* HDCP dynamic setting flag */
#define FAPEX_HDMI_HDCP_DYNAMIC_ON          1           //!< HDCP Dynamic ON
#define FAPEX_HDMI_HDCP_DYNAMIC_OFF         0           //!< HDCP Dynamic OFF

/*!
*************************************************************************
** \brief  get the EDID information data
**
** The related ioctl command type :
** \ref hdmi_cmd_get_edid_info "FAPEX_IOCCOM_HDMI_GET_EDID_INFO"
**
** \sa FAPEX_HDMI_Ioctl
*************************************************************************/
typedef struct {
    uint32_t            supportData;  //!< support data refer to CEA861-B Table28 Byte3
    uint32_t            videoCode1;   //!< bitmap of video short descriptor data refer to CEA861-B Table37
    uint32_t            videoCode2;   //!< bitmap of video short descriptor data 
    uint32_t            videoCode3;   //!< bitmap of video short descriptor data 
    uint32_t            videoCode4;   //!< bitmap of video short descriptor data
    uint32_t            audioData;    //!< audio short descriptor data refer to CEA861-B Table33,34
    uint32_t            speakerData;  //!< speaker allocation data block refer to CEA861-B Table36
    uint32_t            vsdbData6;    //!< extension data VSDB Byte6
    uint32_t            videoLatency; //!< video latency VSDB Byte 9
    uint32_t            audioLatency; //!< audio latency VSDB byte 10
    uint32_t            interlacedVideoLatency;    //!< interlaced video VSDB Byte11
    uint32_t            interlacedAudioLatency;    //!< interlaced audio VSDB Byte12
    uint32_t            hdmiMode;     //!< 0:DVI, 1:HDMI
    uint16_t            manufacturerId;  //!< manufacturer ID
    uint16_t            productId;       //!< product ID
    uint32_t            serialNumber;    //!< serial number
    uint8_t             monitorSerialNumber[13];   //!< monitor serial number
    uint8_t             asciiDataString[13];       //!< ASCII Data String
    uint8_t             monitorName[13];           //!< monitor serial number

} FAPEX_HDMI_EdidInfoStrT;


/*!
*************************************************************************
** \brief ioctl specific command for the HDMI driver
*************************************************************************/
#define FAPEX_IOCCOM_HDMI_CHECK_HPD         _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMI + 0, int32_t)
#define FAPEX_IOCCOM_HDMI_READ_EDID         _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMI + 1, int32_t)
#define FAPEX_IOCCOM_HDMI_GET_EDID_INFO     _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMI + 2, FAPEX_HDMI_EdidInfoStrT)
#define FAPEX_IOCCOM_HDMI_GET_EDID_PHYSADDR _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMI + 3, uint32_t)
#define FAPEX_IOCCOM_HDMI_LINK_HDCP         _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMI + 4, int32_t)
#define FAPEX_IOCCOM_HDMI_CHECK_HDCP_STATE  _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMI + 5, int32_t)
#define FAPEX_IOCCOM_HDMI_SET_DYNAMIC_HDCP  _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMI + 6, int32_t)


/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T FAPEX_HDMI_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_HDMI_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_HDMI_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBHDMI_H */

