/*!
*************************************************************************
**
** \file    abst_libaudec.c
**
** \brief   abstraction library source file (Audio Decoder driver)
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libaudec.h"
#include "abst_common_priv.h"

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The device identifier.
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (#FAPEX_AUDEC_OpenParamStrT)<br>
**                If descriptor for this '\e id' has already been opened,
**                this '\e argptr' will be invalid.
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_AUDEC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_AUDEC_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for this descriptor.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_AUDEC_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  read method.
**         This function is called with the systemcall "read".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the destination buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually read.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXREADRET_T FAPEX_AUDEC_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Read(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)read(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  write method.
**         This function is called with the systemcall "write".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the source buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually write.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXWRTRET_T FAPEX_AUDEC_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Write(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)write(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}
#endif

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
** <hr>
** ioctl command(\e cmd) type:
** \anchor audec_cmd_start
** \li #FAPEX_IOCCOM_START<br>
**      start Audio Decoder. (The \e pArg is ignored)
**
** \anchor audec_cmd_stop
** \li #FAPEX_IOCCOM_STOP<br>
**      stop Audio Decoder. (The \e pArg is ignored)
**
** \anchor audec_cmd_reset
** \li #FAPEX_IOCCOM_RESET<br>
**      reset Audio Decoder. (The \e pArg is ignored)
**
** \anchor audec_cmd_get_status
** \li #FAPEX_IOCCOM_AUDEC_GET_STATUS<br>
**      get status of Audio Decoder.<br>
**      Set pointer of #FAPEX_AUDEC_StatusStrT data field to \e pArg.
**      The status value will be set to this field.
**
** \anchor audec_cmd_set_output
** \li #FAPEX_IOCCOM_AUDEC_SET_OUTPUT<br>
**      set output parameter.<br>
**      Set pointer of #FAPEX_AUDEC_OutputParamStrT data field to \e pArg.
**
** \anchor audec_cmd_set_codec_params
** \li #FAPEX_IOCCOM_AUDEC_SET_CODEC_PARAMS<br>
**      set codec parameter.<br>
**      Set pointer of #FAPEX_AUDEC_CodecSpecificParamStrT data field to \e pArg.
**
** \anchor audec_cmd_get_capabilities
** \li #FAPEX_IOCCOM_AUDEC_GET_CAPABILITIES<br>
**      get capabilities of Audio Decoder.<br>
**      Set pointer of #FAPEX_AUDEC_CapabilitiesParamStrT data field to \e pArg.
**      The capabilities value will be set to this field.
**
** \anchor audec_cmd_set_output_delay
** \li FAPEX_IOCCOM_AUDEC_SET_OUTPUT_DELAY<br>
**      set the delay for audio presentation.<br>
**      Set value of delay in milliseconds to \e pArg.<br>
**
******************************************************************************/
FAPEXIOCRET_T FAPEX_AUDEC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Ioctl(id, cmd, pArg);
#else /* !_LINUX_FAPI */
    return ioctl(id, (int)cmd, pArg);
#endif /* !_LINUX_FAPI */
}
