/*!
*************************************************************************
**
** \file    abst_libauout.c
**
** \brief   abstraction library source file (Audio Output driver)
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libauout.h"
#include "abst_common_priv.h"

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The device identifier.
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (#FAPEX_AUOUT_OpenParamStrT)<br>
**                If descriptor for this '\e id' has already been opened,
**                this '\e argptr' will be invalid.
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_AUOUT_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_AUOUT_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for this descriptor.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_AUOUT_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  read method.
**         This function is called with the systemcall "read".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the destination buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually read.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXREADRET_T FAPEX_AUOUT_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Read(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)read(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  write method.
**         This function is called with the systemcall "write".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the source buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually write.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXWRTRET_T FAPEX_AUOUT_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Write(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)write(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}
#endif

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
** <hr>
** ioctl command(\e cmd) type:
** \anchor auout_cmd_start
** \li #FAPEX_IOCCOM_START<br>
**      start Audio Output (The \e pArg is ignored).
**
** \anchor auout_cmd_stop
** \li #FAPEX_IOCCOM_STOP<br>
**      stop Audio Output (The \e pArg is ignored).
**
** \anchor auout_cmd_set_pcm_info
** \li #FAPEX_IOCCOM_AUOUT_SET_PCM_INFO<br>
**      set PCM information.<br>
**      Set pointer of #FAPEX_AUOUT_PcmInfoStrT data field to \e pArg.
**
** \anchor auout_cmd_get_pcm_info
** \li #FAPEX_IOCCOM_AUOUT_GET_PCM_INFO<br>
**      get PCM information.<br>
**      Set pointer of #FAPEX_AUOUT_PcmInfoStrT data field to \e pArg.<br>
**      The PCM information value will be set to this field.
**
** \anchor auout_cmd_set_stream_info
** \li #FAPEX_IOCCOM_AUOUT_SET_STREAM_TYPE<br>
**      set stream type.<br>
**      Set stream type data (int32_t) to \e pArg.<br>
**      ex.
**      - #FAPEX_AUOUT_STREAM_AAC<br>
**        AAC compressed audio.
**      - #FAPEX_AUOUT_STREAM_AC3<br>
**        AC3 compressed audio.
**      - #FAPEX_AUOUT_STREAM_MPEG_L1<br>
**        MPEG1/2 Layer 1 compressed audio.
**      - #FAPEX_AUOUT_STREAM_MPEG_L2<br>
**        MPEG1/2 Layer 2 compressed audio.
**      - #FAPEX_AUOUT_STREAM_MPEG_L3<br>
**        MPEG1/2 Layer 3 (MP3) compressed audio.
**      - #FAPEX_AUOUT_STREAM_WMA<br>
**        WMA compressed audio.
**      - #FAPEX_AUOUT_STREAM_EC3<br>
**        Enhanced AC3 compressed audio.
**      - #FAPEX_AUOUT_STREAM_HEAAC<br>
**        AACplus v1/v2 compressed audio.
**
** \anchor auout_cmd_get_stream_info
** \li #FAPEX_IOCCOM_AUOUT_GET_STREAM_TYPE<br>
**      get stream type.<br>
**      Set pointer of int32_t data field to \e pArg.<br>
**      The stream type value as follows will be set to this field.
**      - #FAPEX_AUOUT_STREAM_UNDEF<br>
**        Audio stream undefined.
**      - #FAPEX_AUOUT_STREAM_AAC<br>
**        AAC compressed audio.
**      - #FAPEX_AUOUT_STREAM_AC3<br>
**        AC3 compressed audio.
**      - #FAPEX_AUOUT_STREAM_MPEG_L1<br>
**        MPEG1/2 Layer 1 compressed audio.
**      - #FAPEX_AUOUT_STREAM_MPEG_L2<br>
**        MPEG1/2 Layer 2 compressed audio.
**      - #FAPEX_AUOUT_STREAM_MPEG_L3<br>
**        MPEG1/2 Layer 3 (MP3) compressed audio.
**      - #FAPEX_AUOUT_STREAM_WMA<br>
**        WMA compressed audio.
**      - #FAPEX_AUOUT_STREAM_EC3<br>
**        Enhanced AC3 compressed audio.
**      - #FAPEX_AUOUT_STREAM_HEAAC<br>
**        AACplus v1/v2 compressed audio.
**
** \anchor auout_cmd_set_mute
** \li #FAPEX_IOCCOM_AUOUT_SET_MUTE<br>
**      set mute.<br>
**      Set pointer of #FAPEX_AUOUT_MuteParamStrT data field to \e pArg.
**
** \anchor auout_cmd_set_dualmono
** \li #FAPEX_IOCCOM_AUOUT_SET_DUALMONO<br>
**      set dual mono.<br>
**      Set pointer of #FAPEX_AUOUT_DualMonoParamStrT data field to \e pArg.
**
** \anchor auout_cmd_set_volume
** \li #FAPEX_IOCCOM_AUOUT_SET_VOLUME<br>
**      set volume.<br>
**      Set pointer of #FAPEX_AUOUT_VolumeParamStrT data field to \e pArg.
**
** \anchor auout_cmd_get_volume
** \li #FAPEX_IOCCOM_AUOUT_GET_VOLUME<br>
**      get volume.<br>
**      Set pointer of #FAPEX_AUOUT_VolumeParamStrT data field to \e pArg.<br>
**      The volume value will be set to this field.
**
** \anchor auout_cmd_get_speakers
** \li #FAPEX_IOCCOM_AUOUT_GET_SPEAKERS<br>
**      get supported speakers.<br>
**      Set pointer of #FAPEX_AUOUT_SpeakersParamStrT data field to \e pArg.<br>
**      The speaker parameter value will be set to this field.
**
** \anchor auout_cmd_get_i2sbuffers
** \li #FAPEX_IOCCOM_AUOUT_GET_I2SBUFFERS<br>
**      get I2S buffers in use.<br>
**      Set pointer of #FAPEX_AUOUT_BufferParamStrT data field to \e pArg.<br>
**      The I2S buffer parameter value will be set to this field.
**
******************************************************************************/
FAPEXIOCRET_T FAPEX_AUOUT_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Ioctl(id, cmd, pArg);
#else /* !_LINUX_FAPI */
    return ioctl(id, (int)cmd, pArg);
#endif /* !_LINUX_FAPI */
}
