/*!
*************************************************************************
**
** \file    abst_libicc.c
**
** \brief   abstraction library source file (ICC driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libicc.h"
#include "abst_common_priv.h"


static FAPEX_SYS_ThreadT                 FAPEX_ICC_Slot1CardThreadHandle;
static FAPEX_SYS_ThreadT                 FAPEX_ICC_Slot2CardThreadHandle;
static FAPEX_ICC_CardNotifierCALLBACK_T  FAPEX_ICC_CardInCallback1  = NULL;   //ICC slot1 card-in callback
static FAPEX_ICC_CardNotifierCALLBACK_T  FAPEX_ICC_CardOutCallback1 = NULL;   //ICC slot1 card-out callback
static FAPEX_ICC_CardNotifierCALLBACK_T  FAPEX_ICC_CardInCallback2  = NULL;   //ICC slot2 card-in callback
static FAPEX_ICC_CardNotifierCALLBACK_T  FAPEX_ICC_CardOutCallback2 = NULL;   //ICC slot2 card-out callback
static uint32_t                          Sub_id1                    = 0;
static uint32_t                          Sub_id2                    = 0;



/*!
*******************************************************************************
**
** \brief  smart card status  poll thread slot1.
**         This function is called with FAPEX_ICC_Open.
**
**
******************************************************************************/
static void FAPEX_ICC_Slot1CardInOutThreadFunction(void* optArg)
{
    int32_t     retVal      = 0;
    uint32_t    CardNotify  = 0;

    while(1){
        if ( Sub_id1 != 0 ){
            retVal = FAPEX_ICC_Ioctl( (FAPEXIOCARG1_T)Sub_id1,
                                      (FAPEXIOCARG2_T)FAPEX_IOCCOM_ICC_GET_CARDNOTIFY,
                                      (FAPEXIOCARG3_T)&CardNotify );
            if ( retVal == 0 ){
                if ( (CardNotify & FAPEX_ICC_STATUS_CARD_IN) == FAPEX_ICC_STATUS_CARD_IN ){
                    if ( FAPEX_ICC_CardInCallback1 != NULL ){
                        FAPEX_ICC_CardInCallback1();
                    }
                }
                else if ( (CardNotify & FAPEX_ICC_STATUS_CARD_OUT) == FAPEX_ICC_STATUS_CARD_OUT ){
                    if ( FAPEX_ICC_CardOutCallback1 != NULL ){
                        FAPEX_ICC_CardOutCallback1();
                    }
                }
            }
        }
        else{
            FAPI_SYS_PRINT_MSG("FAPEX_ICC_CardInOutThreadFunction Sub_id = 0 sleep\n");
            FAPI_SYS_SLEEP(100);
        }
    }
}


/*!
*******************************************************************************
**
** \brief  smart card status  poll thread slot2.
**         This function is called with FAPEX_ICC_Open.
**
**
******************************************************************************/
static void FAPEX_ICC_Slot2CardInOutThreadFunction(void* optArg)
{
    int32_t     retVal      = 0;
    uint32_t    CardNotify  = 0;

    while(1){
        if ( Sub_id2 != 0 ){
            retVal = FAPEX_ICC_Ioctl( (FAPEXIOCARG1_T)Sub_id2,
                                      (FAPEXIOCARG2_T)FAPEX_IOCCOM_ICC_GET_CARDNOTIFY,
                                      (FAPEXIOCARG3_T)&CardNotify );
            if ( retVal == 0 ){
                if ( (CardNotify & FAPEX_ICC_STATUS_CARD_IN) == FAPEX_ICC_STATUS_CARD_IN ){
                    if ( FAPEX_ICC_CardInCallback2 != NULL ){
                        FAPEX_ICC_CardInCallback2();
                    }
                }
                else if ( (CardNotify & FAPEX_ICC_STATUS_CARD_OUT) == FAPEX_ICC_STATUS_CARD_OUT ){
                    if ( FAPEX_ICC_CardOutCallback2 != NULL ){
                        FAPEX_ICC_CardOutCallback2();
                    }
                }
            }
        }
        else{
            FAPI_SYS_PRINT_MSG("FAPEX_ICC_CardInOutThreadFunction Sub_id = 0 sleep\n");
            FAPI_SYS_SLEEP(100);
        }
    }
}


/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The identifier for ICC block descriptor (\b common descriptor).
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (#FAPEX_ICC_OpenParamStrT)<br>
**                If descriptor for this '\e id' has already been opened,
**                this '\e argptr' will be invalid.
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_ICC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_ICC_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for ICC block descriptor (\b common descriptor).
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_ICC_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}


/*!
*******************************************************************************
**
** \brief  read method.
**         This function is called with the systemcall "read".
**
** \param  id     The identifier for ICC card descriptor (\b sub descriptor).
** \param  buf    The pointer to the destination buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually read.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXREADRET_T FAPEX_ICC_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Read(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)read(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  write method.
**         This function is called with the systemcall "write".
**
** \param  id     The identifier for ICC card descriptor (\b sub descriptor).
** \param  buf    The pointer to the source buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually write.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXWRTRET_T FAPEX_ICC_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Write(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)write(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
** <hr>
** ioctl command(\e cmd) type:
** \anchor icc_cmd_open_subid
** \li #FAPEX_IOCCOM_ICC_OPEN_SUBID<br>
**      open ICC channel.<br>
**      Set pointer of #FAPEX_ICC_OpenSubIdParamStrT data field to \e pArg.<br>
**      The \e id should be \b common descriptor for ICC block.<br>
**      The \b sub descriptor will be returned in ((FAPEX_ICC_OpenSubIdParamStrT *)\e pArg)->sub_id.<br>
**      If \b sub descriptor for this '\e sub_id' has already been opened,
**      parameters except '\e sub_id' of FAPEX_ICC_OpenSubIdParamStrT will be invalid.
**
** \anchor icc_cmd_close_subid
** \li #FAPEX_IOCCOM_CLOSE_SUBID<br>
**      close ICC channel. (\e pArg is ignored)<br>
**      The \e id should be \b sub descriptor opened in \ref icc_cmd_open_subid "FAPEX_IOCCOM_ICC_OPEN_SUBID".
**
** \anchor icc_cmd_set_ioport
** \li #FAPEX_IOCCOM_ICC_SET_IOPORT<br>
**      set I/O port parameter for accessing the smartcard.<br>
**      Set pointer of #FAPEX_ICC_IOportParamStrT data field to \e pArg.<br>
**      The \e id should be \b sub descriptor opened in \ref icc_cmd_open_subid "FAPEX_IOCCOM_ICC_OPEN_SUBID".
**
** \anchor icc_cmd_get_ioport
** \li #FAPEX_IOCCOM_ICC_GET_IOPORT<br>
**      get I/O port parameter for accessing the smartcard.<br>
**      Set pointer of #FAPEX_ICC_IOportParamStrT data field to \e pArg.<br>
**      The read information value will be set to this field.<br>
**      The \e id should be \b sub descriptor opened in \ref icc_cmd_open_subid "FAPEX_IOCCOM_ICC_OPEN_SUBID".
**
** \anchor icc_cmd_reset
** \li #FAPEX_IOCCOM_RESET<br>
**      reset the smartcard.<br>
**      Set reset mode flag to \e pArg as follows.<br>
**      - #FAPEX_ICC_RESET_MODE_WARM<br>
**        Requests a warm reset of the ICC
**      - #FAPEX_ICC_RESET_MODE_COLD<br>
**        Requests a cold reset of the ICC
**      .
**      The \e id should be \b sub descriptor opened in \ref icc_cmd_open_subid "FAPEX_IOCCOM_ICC_OPEN_SUBID".<br>
**      The completion of reset can be confirmed by \ref icc_cmd_get_status
**      "FAPEX_IOCCOM_ICC_GET_STATUS" command or FAPEX_SYS_SelectCall().
**
** \anchor icc_cmd_get_atr
** \li #FAPEX_IOCCOM_ICC_GET_ATR<br>
**      get ATR parameter from the smartcard.<br>
**      Set pointer of #FAPEX_ICC_AtrParamStrT data field to \e pArg.<br>
**      The read information value will be set to this field.<br>
**      The \e id should be \b sub descriptor opened in \ref icc_cmd_open_subid "FAPEX_IOCCOM_ICC_OPEN_SUBID".
**
** \anchor icc_cmd_get_data
** \li #FAPEX_IOCCOM_ICC_GET_DATA<br>
**      get data from the smartcard.<br>
**      Set pointer of #FAPEX_ICC_DataParamStrT data field to \e pArg.<br>
**      The read data value will be set to this field.<br>
**      The \e id should be \b sub descriptor opened in \ref icc_cmd_open_subid "FAPEX_IOCCOM_ICC_OPEN_SUBID".
**
** \anchor icc_cmd_get_status
** \li #FAPEX_IOCCOM_ICC_GET_STATUS<br>
**      get data from the smartcard.<br>
**      Set pointer of status data field (uint32_t) to \e pArg.<br>
**      The value of status is the bitwise-inclusive OR of values from the following list.<br>
**      - #FAPEX_ICC_STATUS_NORMAL<br>
**        normal status
**      - #FAPEX_ICC_STATUS_CARD_IN<br>
**        ICC card inserted
**      - #FAPEX_ICC_STATUS_CARD_OUT<br>
**        ICC card removed
**      - #FAPEX_ICC_STATUS_FINISHED_RESET_COLD<br>
**        cold reset finished
**      - #FAPEX_ICC_STATUS_FINISHED_RESET_WARM<br>
**        warm reset finished
**      - #FAPEX_ICC_STATUS_ERR_SLOT_EMPTY<br>
**        card slot is empty
**      - #FAPEX_ICC_STATUS_ERR_RUNNING<br>
**        an ICC function is currently running
**      - #FAPEX_ICC_STATUS_ERR_OVERFLOW<br>
**        ICC read buffer overflow detected while reading data
**      - #FAPEX_ICC_STATUS_ERR_NO_ANSWER<br>
**        no answer from smartcard
**      - #FAPEX_ICC_STATUS_ERR_TIMEOUT<br>
**        Timeout occured
**      - #FAPEX_ICC_STATUS_ERR_UNKNOWN<br>
**        unknown error
**      .
**      The \e id should be \b sub descriptor opened in \ref icc_cmd_open_subid "FAPEX_IOCCOM_ICC_OPEN_SUBID".<br>
**      The status change as #FAPEX_ICC_STATUS_CARD_IN, #FAPEX_ICC_STATUS_CARD_OUT,
**      #FAPEX_ICC_STATUS_FINISHED_RESET_COLD, and #FAPEX_ICC_STATUS_FINISHED_RESET_WARM
**      can be confirmed by FAPEX_SYS_SelectCall().<br>
**      After the \ref icc_cmd_reset "FAPEX_IOCCOM_RESET" command is executed, this command is waited
**      for until the reset event is finished.
**
******************************************************************************/
FAPEXIOCRET_T FAPEX_ICC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
    FAPEXIOCRET_T                  retVal;
    FAPEXIOCARG2_T                 cmdtemp     = cmd;
    FAPEXIOCARG1_T                 idtemp      = id;
    FAPEX_ICC_OpenSubIdParamStrT*  pSbuIdParam = NULL;

    retVal = fapex_COMMON_Ioctl(id, cmd, pArg);
    if ( cmdtemp == FAPEX_IOCCOM_ICC_OPEN_SUBID ){
        if ( idtemp == FAPEXDEV_ICC_SUB_MINOR_FIRST ){
            pSbuIdParam = (FAPEX_ICC_OpenSubIdParamStrT*) pArg;
            Sub_id1 = pSbuIdParam->sub_id;
            FAPEX_ICC_CardInCallback1  = pSbuIdParam->cardInCallback;
            FAPEX_ICC_CardOutCallback1 = pSbuIdParam->cardOutCallback;

            FAPEX_ICC_Slot1CardThreadHandle = FAPEX_SYS_CreateThread( NULL,
                                                                      FAPEX_ICC_CARD_THREAD_STACKSIZE,
                                                                      FAPEX_ICC_CARD_THREAD_PRIORITY,
                                                                      FAPEX_ICC_Slot1CardInOutThreadFunction,
                                                                      0, 0, FAPEX_ICC_CARD_THREAD_NAME );

            if( FAPEX_ICC_Slot1CardThreadHandle == NULL ){
                FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                                    FAPEX_ICC_CARD_THREAD_NAME );
                exit(1);
            }
        }
        else if ( idtemp == FAPEXDEV_ICC_SUB_MINOR_LAST ){
            pSbuIdParam = (FAPEX_ICC_OpenSubIdParamStrT*) pArg;
            Sub_id2 = pSbuIdParam->sub_id;
            FAPEX_ICC_CardInCallback2  = pSbuIdParam->cardInCallback;
            FAPEX_ICC_CardOutCallback2 = pSbuIdParam->cardOutCallback;

            FAPEX_ICC_Slot2CardThreadHandle = FAPEX_SYS_CreateThread( NULL,
                                                                      FAPEX_ICC_CARD_THREAD_STACKSIZE,
                                                                      FAPEX_ICC_CARD_THREAD_PRIORITY,
                                                                      FAPEX_ICC_Slot2CardInOutThreadFunction,
                                                                      0, 0, FAPEX_ICC_CARD_THREAD_NAME );

            if( FAPEX_ICC_Slot2CardThreadHandle == NULL ){
                FAPI_SYS_PRINT_MSG( "ERROR:[main.c] creating thread \"%s\" failed\n",
                                    FAPEX_ICC_CARD_THREAD_NAME );
                exit(1);
            }
        }
    }
    else if ( cmdtemp == FAPEX_IOCCOM_CLOSE_SUBID ){
        if ( idtemp == Sub_id1 ){
            FAPEX_SYS_DestroyThread( FAPEX_ICC_Slot1CardThreadHandle );
            Sub_id1 = 0;
            FAPEX_ICC_CardInCallback1  = NULL;
            FAPEX_ICC_CardOutCallback1 = NULL;
        }
        if ( idtemp == Sub_id2 ){
            FAPEX_SYS_DestroyThread( FAPEX_ICC_Slot2CardThreadHandle );
            Sub_id2 = 0;
            FAPEX_ICC_CardInCallback2  = NULL;
            FAPEX_ICC_CardOutCallback2 = NULL;
        }
    }

    return retVal;
}


