/*!
*************************************************************************
**
** \file    abst_libir.c
**
** \brief   abstraction library source file (IR driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libir.h"
#include "abst_common_priv.h"

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The device identifier.
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (#FAPEX_IR_OpenParamStrT)<br>
**                If descriptor for this '\e id' has already been opened,
**                this '\e argptr' will be invalid.
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_IR_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_IR_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for this descriptor.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_IR_Close(FAPEXRELARG1_T id){

#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  read method.
**         This function is called with the systemcall "read".<br>
**         This function is waited for until the key event is generated.
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the destination buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually read.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXREADRET_T FAPEX_IR_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count){
#ifndef _LINUX_FAPI
    return FAPEXSAL_Read(id,buf,count);
#else /* !_LINUX_FAPI */
    return (int)read(id,buf,(size_t)count);
#endif /* !_LINUX_FAPI */
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  write method.
**         This function is called with the systemcall "write".
**
** \param  id     The identifier for this descriptor.
** \param  buf    The pointer to the source buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually write.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXWRTRET_T FAPEX_IR_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count){
#ifndef _LINUX_FAPI
    return FAPEXSAL_Write(id,buf,count);
#else /* !_LINUX_FAPI */
    return (int)write(id,buf,(size_t)count);
#endif /* !_LINUX_FAPI */
}
#endif

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
** <hr>
** ioctl command(\e cmd) type:
** \anchor ir_cmd_get_status
** \li #FAPEX_IOCCOM_IR_GET_STATUS<br>
**      get status.<br>
**      Set pointer of #FAPEX_IR_StatusStrT data field to \e pArg.<br>
**      The status value will be set to this field.
**
** \anchor ir_cmd_get_key
** \li #FAPEX_IOCCOM_IR_GET_KEY<br>
**      get key value.<br>
**      Set pointer of #FAPEX_IR_KeyParamStrT data field to \e pArg.<br>
**      The key parameter value will be set to this field.<br>
**      This command is waited for until the key event is generated.<br>
**      This key status change can be confirmed by FAPEX_SYS_SelectCall(), too.
**
** \anchor ir_cmd_set_maptable
** \li #FAPEX_IOCCOM_IR_SET_MAPTABLE<br>
**      set a key mapping table.<br>
**      Set pointer of #FAPEX_IR_MapTableParamStrT data field to \e pArg.<br>
**      The key mapping table value will be set to this field.
**
** \anchor ir_cmd_remove_maptable
** \li #FAPEX_IOCCOM_IR_REMOVE_MAPTABLE<br>
**      remove key mapping table by deviceId.<br>
**      Set devideId data (int32_t) to \e pArg.
**
******************************************************************************/
FAPEXIOCRET_T FAPEX_IR_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg){
#ifndef _LINUX_FAPI
    return FAPEXSAL_Ioctl(id,cmd,pArg);
#else /* !_LINUX_FAPI */
    return ioctl(id,(int)cmd,pArg);
#endif /* !_LINUX_FAPI */
}
