/*!
*************************************************************************
**
** \file    abst_libvienc.c
**
** \brief   abstraction library source file (Video Output Encoder driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#else
#include <linux/ioctl.h>
#endif
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libvienc.h"
#include "abst_common_priv.h"

/*!
*******************************************************************************
**
** \brief  open method.
**         This function is called with the systemcall "open".
**
** \param  id     The identifier for all encoder (\b common descriptor).
** \param  flags  This parameter indicate a open mode (read only/write only/read-write).<br>
**                This implementation supports only read-write mode.
** \param  argptr argument pointer (value of VIENC system mode)
**                - #FAPEX_VIENC_SYSTEM_MODE_UNKNOWN
**                - #FAPEX_VIENC_SYSTEM_MODE_1920x1080x2397P
**                - #FAPEX_VIENC_SYSTEM_MODE_1920x1080x24P
**                - #FAPEX_VIENC_SYSTEM_MODE_1920x1080x25P
**                - #FAPEX_VIENC_SYSTEM_MODE_1920x1080x2997P
**                - #FAPEX_VIENC_SYSTEM_MODE_1920x1080x30P
**                - #FAPEX_VIENC_SYSTEM_MODE_1920x1080x50I
**                - #FAPEX_VIENC_SYSTEM_MODE_1920x1080x5994I
**                - #FAPEX_VIENC_SYSTEM_MODE_1920x1080x60I
**                - #FAPEX_VIENC_SYSTEM_MODE_1360x768x5994P
**                - #FAPEX_VIENC_SYSTEM_MODE_1280x720x2397P
**                - #FAPEX_VIENC_SYSTEM_MODE_1280x720x24P
**                - #FAPEX_VIENC_SYSTEM_MODE_1280x720x25P
**                - #FAPEX_VIENC_SYSTEM_MODE_1280x720x2997P
**                - #FAPEX_VIENC_SYSTEM_MODE_1280x720x30P
**                - #FAPEX_VIENC_SYSTEM_MODE_1280x720x50P
**                - #FAPEX_VIENC_SYSTEM_MODE_1280x720x5994P
**                - #FAPEX_VIENC_SYSTEM_MODE_1280x720x60P
**                - #FAPEX_VIENC_SYSTEM_MODE_1024x768x5994P
**                - #FAPEX_VIENC_SYSTEM_MODE_800x600x5994P
**                - #FAPEX_VIENC_SYSTEM_MODE_720x576x50P
**                - #FAPEX_VIENC_SYSTEM_MODE_720x576x50I
**                - #FAPEX_VIENC_SYSTEM_MODE_720x480x5994P
**                - #FAPEX_VIENC_SYSTEM_MODE_720x480x5994I
**                - #FAPEX_VIENC_SYSTEM_MODE_640x480x5994P
**                - #FAPEX_VIENC_SYSTEM_MODE_LCD_640x480x50P
**                - #FAPEX_VIENC_SYSTEM_MODE_LCD_1024x768x50P
**                .
**                If descriptor for this '\e id' has already been opened,
**                this '\e argptr' will be invalid.
**
** \retval >=0 process OK, new file descriptor is returned.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXOPENRET_T FAPEX_VIENC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr)
{
    return fapex_COMMON_Open(id, flags, argptr, FAPEX_IOCCOM_VIENC_OPEN);
}

/*!
*******************************************************************************
**
** \brief  release method.
**         This function is called with the systemcall "close".
**
** \param  id     The identifier for all encoder (\b common descriptor).
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXRELRET_T FAPEX_VIENC_Close(FAPEXRELARG1_T id)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Release(id);
#else /* !_LINUX_FAPI */
    return close(id);
#endif /* !_LINUX_FAPI */
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  read method.
**         This function is called with the systemcall "read".
**
** \param  id     The identifier for HD/SD encoder (\b sub descriptor).
** \param  buf    The pointer to the destination buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually read.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXREADRET_T FAPEX_VIENC_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Read(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)read(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}

/*!
*******************************************************************************
**
** \brief  write method.
**         This function is called with the systemcall "write".
**
** \param  id     The identifier for HD/SD encoder (\b sub descriptor).
** \param  buf    The pointer to the source buffer which is in the user space.
** \param  count  The data size.
**
** \retval >=0 process OK, its value is the number of bytes actually write.
** \retval <0  process fails, its value is set to errno.
**
******************************************************************************/
FAPEXWRTRET_T FAPEX_VIENC_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count)
{
#ifndef _LINUX_FAPI
    return FAPEXSAL_Write(id, buf, count);
#else /* !_LINUX_FAPI */
    return (int)write(id, buf, (size_t)count);
#endif /* !_LINUX_FAPI */
}
#endif

/*!
*******************************************************************************
**
** \brief  ioctl method.
**         This function is called with the systemcall "ioctl".
**
** \param  id     The identifier for this descriptor.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
** \retval 0   process OK
** \retval <0  process fails, its value is set to errno.
**
** <hr>
** ioctl command(\e cmd) type:
** \anchor vienc_cmd_start
** \li #FAPEX_IOCCOM_START<br>
**      start Video Encoder. (\e pArg is ignored)<br>
**      When \e id is \b common descriptor, all Video Encoder on the ready state will start.<br>
**      When \e id is \b sub descriptor for a Video Encoder on the ready state, only this
**      Encoder will start.
**
** \anchor vienc_cmd_stop
** \li #FAPEX_IOCCOM_STOP<br>
**      stop Video Encoder. (\e pArg is ignored)<br>
**      When \e id is \b common descriptor, all Video Encoder on the running state will stop.<br>
**      When \e id is \b sub descriptor for a Video Encoder on the running state, only this
**      Encoder will start.
**
** \anchor vienc_cmd_open_subid
** \li #FAPEX_IOCCOM_VIENC_OPEN_SUBID<br>
**      open Video Encoder (HD or SD).<br>
**      Set pointer of #FAPEX_VIENC_OpenSubIDParamStrT data field to \e pArg.<br>
**      The \e id should be \b common descriptor.<br>
**      The \b sub descriptor will be returned in ((FAPEX_VIENC_OpenSubIDParamStrT *)\e pArg)->sub_id.<br>
**      If \b sub descriptor for this '\e sub_id' has already been opened,
**      parameters except '\e sub_id' of FAPEX_VIENC_OpenSubIDParamStrT will be invalid.
**
** \anchor vienc_cmd_close_subid
** \li #FAPEX_IOCCOM_CLOSE_SUBID<br>
**      close Video Encoder (HD or SD). (\e pArg is ignored)<br>
**      The \e id should be \b sub descriptor opened in \ref vienc_cmd_open_subid "FAPEX_IOCCOM_VIENC_OPEN_SUBID".
**
** \anchor vienc_cmd_get_status
** \li #FAPEX_IOCCOM_VIENC_GET_STATUS<br>
**      get status.<br>
**      Set pointer of #FAPEX_VIENC_StatusStrT data field to \e pArg.<br>
**      The status value will be set to this field.
**
** \anchor vienc_cmd_get_wssdata
** \li #FAPEX_IOCCOM_VIENC_GET_WSSDATA<br>
**      get WSS data.<br>
**      Set pointer of variable data field (uint32_t) to \e pArg.<br>
**      The wss data value will be set to this field.<br>
**      The \e id should be \b sub descriptor for SD encoder.
**
** \anchor vienc_cmd_set_wssdata
** \li #FAPEX_IOCCOM_VIENC_SET_WSSDATA<br>
**      set WSS data.<br>
**      Set value of wss data (uint32_t) to \e pArg.<br>
**      The wss data value will be set.<br>
**      The \e id should be \b sub descriptor for SD encoder.
**
** \anchor vienc_cmd_set_component
** \li #FAPEX_IOCCOM_VIENC_SET_CCOMPONENT<br>
**      set component data.<br>
**      Set value of component (uint32_t) to \e pArg.<br>
**      FAPEX_VIENC_CCOMPONENT_UNKNOWN specifies an unknown colour component
**      FAPEX_VIENC_CCOMPONENT_DAC_POWER_DOWN  turns off the signal to the DACs
**                of VE SD or VE HD and instead sends a constant 0V to the DACs 
**      FAPEX_VIENC_CCOMPONENT_DIGITAL_RGB specifies Digital RGB output of VE HD
**                on Gpios. Mode valid for VE HD Only
**      FAPEX_VIENC_CCOMPONENT_ANALOG_RGB specifies Analog RGB output of VE HD on
**                Gpios. Mode valid for VE HD Only
**      FAPEX_VIENC_CCOMPONENT_RGB Specifies Analog and digital RGB output on VE
**                HD ONLY
**      FAPEX_VIENC_CCOMPONENT_DIGITAL_YPrPb specifies Digital YPrPb output of VE
**                HD on Gpios. Mode valid for VE HD Only                     
**      FAPEX_VIENC_CCOMPONENT_ANALOG_YPrPb specifies Analog YPrPb output of VE 
**               HD on Gpios. Mode valid for VE HD Only
**      FAPEX_VIENC_CCOMPONENT_YPrPb specifies Analog and digital YPrPb output on
**               VE HD ONLY
**      FAPEX_VIENC_CCOMPONENT_COLOURBAR specifies a standard \b COLOUR \b BAR. on 
**               VE SD only
**      FAPEX_VIENC_CCOMPONENT_CVBS specifies only CVBS on VE SD ONLY
**      FAPEX_VIENC_CCOMPONENT_YC specifies only CVBS on VE SD ONLY
**      FAPEX_VIENC_CCOMPONENT_CVBS_YC specifies CVBS with YC output on VE SD ONLY 
**               for S Video
**      
** \anchor vienc_cmd_get_component
** \li #FAPEX_IOCCOM_VIENC_GET_CCOMPONENT<br>
**      get component.<br>
**      Get component data to \e pArg.<br>
**
** \anchor vienc_cmd_set_enable_wss
** \li #FAPEX_IOCCOM_VIENC_ENABLE_WSS<br>
**      set enable wss function.<br>
**
** \anchor vienc_cmd_set_disable_wss
** \li #FAPEX_IOCCOM_VIENC_DISABLE_WSS<br>
**      set disable wss function.<br>
**
** \anchor vienc_cmd_set_dac
** \li #FAPEX_IOCCOM_VIENC_SET_DAC<br>
**      set dac configuration.<br>
**      FAPEX_VIENC_DAC_ASSIGNMENT_012_HD_345_SD specifies DACs 012 for HD and 
**              DACs345 for SD analog encoder
**      FAPEX_VIENC_DAC_ASSIGNMENT_012_SD_345_HD specifies DACs 012 for SD and 
**              DACs345 for HD analog encoder
******************************************************************************/
FAPEXIOCRET_T FAPEX_VIENC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
    return fapex_COMMON_Ioctl(id, cmd, pArg);
}

