/*!
*************************************************************************
**
** \file    abst_libvrflash.c
**
** \brief   abstraction library source file (Video Output driver).
**
** These function is executed in user space.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2009-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#include <stdio.h>
#include "fapex/abst_types.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_libvrflash.h"

#include "facs.h"
#define FACS_MALLOC FAPI_SYS_MALLOC
#define FACS_FREE   FAPI_SYS_FREE

#if 0
#ifndef _LINUX_FAPI
#include "fapex_cfdl_adapter.h"
#endif
#endif

#define FLASH_HANDLE_ID 255
#define FAPEX_VRFLASH_REGION_COUNT_MAX 4

#define FAPEX_VRFLASH_ERASE_SECTOR_TIMEOUT -2

int32_t flashInitialized = 0;


#if 0

typedef struct _flashRegionT
{
    uint32_t sectorCount; // number of sectors in erase region
    uint32_t sectorBytes; // sector size in erase region
}
flashRegionT;

typedef struct _flashDescriptorT
{
    const char_t* deviceName;       // human readable flash device string
    uint32_t      eraseRegionCount; // number of used erase regions
    flashRegionT  eraseRegionTable[FAPEX_VRFLASH_REGION_COUNT_MAX];
    uint32_t      portBits;         // the width of the port
    uint32_t      dataBits;         // the width of the chip
    uint32_t      manufacturerId;   // manufacturer id
    uint32_t      deviceId;         // device id
    uint32_t      commandSet;       // vendor command set identifier
    uint32_t      resetCommand;     // vendor specific reset command
    uint32_t      legacyUnlock;     // support Intel legacy (un)locking
}
flashDescriptorT;

typedef struct _flashDeviceT
{
    uint32_t          deviceId;           // combined device & manufacturer code
    uint32_t          deviceBytes;        // total bank size in bytes
    uint32_t          sectorCount;        // total number of erase units (sectors)
    uint32_t          cfiVersion;         // cfi version
    uint32_t          cfiOffset;          // offset for cfi query
    flashDescriptorT* descriptorPtr;      // a pointer to the current device info block
    uint32_t*         sectorStartArray;   // holding the start addresses for all sectors
    uint32_t*         sectorBytesArray;   // holding the sector sizes for all sectors
    uint8_t*          sectorProtectArray; // holding protection flags for all sectors
    void*             sectorBuffer;       // allocated buffer of max. sector size
    uint32_t          sectorIndex;        // index of sector currently in buffer
    uint32_t          sectorStart;        // start address of sector currently in buffer
    uint32_t          sectorBytes;        // size of sector currently in buffer
    uint32_t          dirtyFlag;          // decides whether flush is necessary
} flashDeviceT;

typedef int32_t(*flashDetectChipT)(flashDeviceT** devicePtrPtr, uint32_t timeout);
typedef int32_t(*flashEraseChipT)(flashDeviceT* devicePtr, uint32_t timeout);
typedef int32_t(*flashEraseSectorT)(flashDeviceT* devicePtr, uint32_t timeout);
typedef int32_t(*flashReadSectorT)(flashDeviceT* devicePtr, uint32_t timeout);
typedef int32_t(*flashWriteSectorT)(flashDeviceT* devicePtr, uint32_t timeout);

typedef int32_t(*flashReadDirectT)(flashDeviceT* devicePtr, uint32_t sectorIndex,
                                   uint32_t byteOffset, uint32_t byteCount,
                                   void* bufferPtr, uint32_t timeout);
typedef int32_t(*flashWriteDirectT)(flashDeviceT* devicePtr, uint32_t sectorIndex,
                                    uint32_t byteOffset, uint32_t byteCount,
                                    const void* bufferPtr, uint32_t timeout);


typedef struct _flashBlockT
{
    uint32_t            inUse;               // specifies if block is in use
    uint32_t            deviceType;          // device type, either parallel or serial
    FAPEX_SYS_SemaphoreT semaphore;           // device lock semaphore
    flashDeviceT*       devicePtr;           // information about the initialized device
    flashDetectChipT    detectChipFunction;  // specific device detection function
    flashEraseChipT     eraseChipFunction;   // specific erase chip function
    flashEraseSectorT   eraseSectorFunction; // specific erase sector function
    flashReadSectorT    readSectorFunction;  // specific read sector function
    flashWriteSectorT   writeSectorFunction; // specific write sector function
    flashReadDirectT    readDirectFunction;   // specific read bytes function
    flashWriteDirectT   writeDirectFunction;  // specific write bytes function
} flashBlockT;

typedef struct _flashHandleT
{
    uint32_t               id;         // handle ID
    uint32_t               inUse;      // specifies if handle is in use
    FAPEX_VRFLASH_OpenParamsT openParams; // open params of the handle
    flashBlockT*           blockPtr;   // pointer to the flash block structure
    FAPEX_VRFLASH_VolumeT     volumeInfo; // FS specific valuem information
} flashHandleT;
#endif

int32_t FAPEX_VRFLASH_GetSectorBytes( FAPEX_SYS_HandleT handle, uint32_t sectorIndex );

/*
********************************************************************************
** \brief Checks a given handle.
**
** This function checks a given FLASH handle.
**
** \return
** - (1) If handle is OK
** - (0) If handle is invalid
********************************************************************************
*/
uint32_t fapex_vrflashCheckHandle( const FACS_CONTROL* handlePtr )
{
    if ( handlePtr != NULL ) return 1;
    else return 0;
}

FAPEX_SYS_HandleT *
FAPEX_VRFLASH_Open( FAPEX_VRFLASH_OpenParamsT *paramsPtr, int32_t* errorCodePtr )
{
    FACS_CONTROL*    flash_handle_ptr = NULL;

    flash_handle_ptr = facs_open( NULL );
    
    return (FAPEX_SYS_HandleT)flash_handle_ptr;

}

int32_t
FAPEX_VRFLASH_Close( FAPEX_SYS_HandleT *flash_handle_ptr )
{
    
    facs_close( (FACS_CONTROL*)flash_handle_ptr );
    
    return 0;
}

/*!
********************************************************************************
** \brief Get current flash devive name
**
** This function retrieves the flash device name from the given flash handle.
**
** \param handle Valid driver instance handle previously returned by a call
**               of FAPI_FLASH_Open().
**
** \return
** - 'non-0' a pointer to a string containing the flash device name.
** - '0' if a given parameter was invalid.
**
** \sa FAPI_FLASH_Open
********************************************************************************
*/
const char_t* FAPEX_VRFLASH_GetDeviceName( FAPEX_SYS_HandleT handle )
{
    FACS_CONTROL* flash_handle_ptr = (FACS_CONTROL*)handle;
    const char_t* result;

    if( !fapex_vrflashCheckHandle( flash_handle_ptr ) )
        return(0);

    result = flash_handle_ptr->deviceID;
    
    return( result );
}
/*!
********************************************************************************
** \brief Get sector index for a given flash address
**
** This function calculates the sector index for a given flash address.
** Sector indexing starts with '0', the address must be a byte address.
**
** \param handle  Valid driver instance handle previously returned by a call
**                of FAPEX_VRFLASH_Open().
** \param address The address to get the sector index for.
**
** \return
** - '>=0' the index of the sector hosting the given byte address.
** - #FAPI_ERR_BAD_PARAMETER if a given parameter was invalid.
**
** \sa FAPEX_VRFLASH_Open
********************************************************************************
*/
int32_t FAPEX_VRFLASH_GetSectorIndex( FAPEX_SYS_HandleT handle, uint32_t address )
{
    FACS_CONTROL* flash_handle_ptr = (FACS_CONTROL*)handle;
    int32_t sector_no;
    
    if( !fapex_vrflashCheckHandle( flash_handle_ptr ) )
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );

    if ( facs_is_addressrange( flash_handle_ptr, address ) != FACS_OK )
    {
        return ( FAPEX_VRFLASH_ERR_BAD_PARAMETER );
    }
    
    sector_no = facs_address_to_sector( flash_handle_ptr, address );
    
    
    return( (int32_t)sector_no );
}

/*!
********************************************************************************
** \brief Get sector start address for a given flash sector index
**
** This function retrieves the sector start address based on the given sector
** index.
** Sector indexing starts with '0', the address must be a byte address.
**
** \param handle      Valid driver instance handle previously returned by a call
**                    of FAPEX_VRFLASH_Open().
** \param sectorIndex The index of the sector to retrieve the start address from.
**
** \return
** - '>=0' the start address of the given sector, relative to flash memory.
** - #FAPEX_VRFLASH_ERR_BAD_PARAMETER if a given parameter was invalid.
**
** \sa FAPEX_VRFLASH_Open
********************************************************************************
*/
int32_t FAPEX_VRFLASH_GetSectorStart( FAPEX_SYS_HandleT handle, uint32_t sectorIndex )
{
    FACS_CONTROL* flash_handle_ptr = (FACS_CONTROL*)handle;
    uint32_t      start_address;

    if( !fapex_vrflashCheckHandle( flash_handle_ptr ) )
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );

    if ( facs_is_sectorrange( flash_handle_ptr, sectorIndex ) != FACS_OK )
    {
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );
    }
    
    start_address = facs_sector_to_address( flash_handle_ptr, (uint32_t)sectorIndex );

    return( (int32_t)start_address );
}

/*!
********************************************************************************
** \brief Get sector size for a given flash sector index
**
** This function retrieves the sector size in bytes based on the given sector
** index.
** Sector indexing starts with '0', the address must be a byte address.
**
** \param handle      Valid driver instance handle previously returned by a call
**                    of FAPEX_VRFLASH_Open().
** \param sectorIndex The index of the sector to retrieve the start address from.
**
** \return
** - '>=0' the start address of the given sector.
** - #FAPI_ERR_BAD_PARAMETER if a given parameter was invalid.
**
** \sa FAPEX_VRFLASH_Open
********************************************************************************
*/
int32_t FAPEX_VRFLASH_GetSectorBytes( FAPEX_SYS_HandleT handle, uint32_t sectorIndex )
{
    FACS_CONTROL* flash_handle_ptr = (FACS_CONTROL*)handle;
    int32_t sector_byte;

    if( !fapex_vrflashCheckHandle( flash_handle_ptr ) )
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );

    if ( facs_is_sectorrange( flash_handle_ptr, sectorIndex ) != FACS_OK )
    {
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );
    }

    sector_byte = facs_get_sectorbyte( flash_handle_ptr, sectorIndex );
    

    return( sector_byte );
}

/*!
********************************************************************************
** \brief Read data from flash
**
** This function reads the given amount of bytes from flash. The data will be
** taken from the flash sector buffer. If needed the current buffered sector
** will be flushed first and the sector containing the given address will
** be read into the current sector buiffer before reading data from it.
**
** \param handle    Valid driver instance handle previously returned by a call
**                  of FAPEX_VRFLASH_Open().
** \param address   The byte address where to read data from, this address
**                  will be interpreted as relativ address inside flash memory.
** \param byteCount The number of bytes to read from flash.
** \param bufferPtr The target buffer where to write the read flash data into.
**
** \return
** - #FAPI_OK if operation was successful
** - #FAPI_ERR_BAD_PARAMETER if a given parameter was invalid.
** - #FAPI_ERR_TIMEOUT if operation timed out.
**
** \sa FAPEX_VRFLASH_Open
********************************************************************************
*/
int32_t FAPEX_VRFLASH_ReadBytes( FAPEX_SYS_HandleT handle, uint32_t address,
                                 uint32_t byteCount, void* bufferPtr )
{
    FACS_CONTROL* flash_handle_ptr = (FACS_CONTROL*)handle;

    if( !fapex_vrflashCheckHandle( flash_handle_ptr ) )
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );

    if( !bufferPtr )
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );

    if ( facs_is_addressrange( flash_handle_ptr, address ) != FACS_OK )
    {
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );
    }
    
    if ( facs_is_addressrange( flash_handle_ptr, address + byteCount - 1 ) != FACS_OK )
    {
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );
    }
    
    facs_read_byte( flash_handle_ptr, address, byteCount, bufferPtr );
    

    return( FAPI_OK );
}

/*!
********************************************************************************
** \brief Write data to flash
**
** This function writes the given amount of bytes to flash. The data will be
** stored into the flash sector buffer. If needed the current buffered sector
** will be flushed first and the sector containing the given address will
** be read into the current sector buffer before writing to it.
**
** \param handle    Valid driver instance handle previously returned by a call
**                  of FAPEX_VRFLASH_Open().
** \param address   The byte address where to read data from, this address
**                  will be interpreted as relativ address inside flash memory.
** \param byteCount The number of bytes to read from flash.
** \param bufferPtr The target buffer where to write the read flash data into.
**
** \return
** - #FAPI_OK if operation was successful
** - #FAPI_ERR_BAD_PARAMETER if a given parameter was invalid.
** - #FAPI_ERR_TIMEOUT if operation timed out.
**
** \sa FAPEX_VRFLASH_Open
********************************************************************************
*/
int32_t FAPEX_VRFLASH_WriteBytes( FAPEX_SYS_HandleT handle, uint32_t address,
                                  uint32_t byteCount, const void* bufferPtr )
{
    FACS_CONTROL*  flash_handle_ptr = (FACS_CONTROL*)handle;

    if( !fapex_vrflashCheckHandle( flash_handle_ptr ) )
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );

    if( !bufferPtr )
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );

    if ( facs_is_addressrange( flash_handle_ptr, address ) != FACS_OK )
    {
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );
    }
    
    if ( facs_is_addressrange( flash_handle_ptr, address + byteCount - 1 ) != FACS_OK )
    {
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );
    }
    
    facs_write_byte( flash_handle_ptr, address, byteCount, (char_t*)bufferPtr );

    return( FAPI_OK );
}

/*!
********************************************************************************
** \brief Erase the given flash sector of a flash device
**
** This function erases one sector of the flash device specified in the
** given flash handle.
**
** \param handle      Valid driver instance handle previously returned by a call
**                    of FAPEX_VRFLASH_Open().
** \param sectorIndex The index of the sector to erase
**
** \return
** - #FAPI_OK if operation was successful
** - #FAPI_ERR_BAD_PARAMETER if a given parameter was invalid, either an
**       invalid flash handle or a sector index which is out of range.
** - #FAPI_ERR_TIMEOUT if operation timed out.
**
** \sa FAPEX_VRFLASH_Open
********************************************************************************
*/
int32_t FAPEX_VRFLASH_EraseSector( FAPEX_SYS_HandleT handle, uint32_t sectorIndex )
{
    FACS_CONTROL*     flash_handle_ptr = (FACS_CONTROL*)handle;

    if( !fapex_vrflashCheckHandle( flash_handle_ptr ) )
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );

    if ( facs_is_sectorrange( flash_handle_ptr, sectorIndex ) != FACS_OK )
    {
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );
    }

    
    facs_erase_sector( flash_handle_ptr, sectorIndex );
    
    

    return( FAPI_OK );
}
/*!
********************************************************************************
** \brief Write data to flash without buffering
**
** This function writes the given amount of bytes directly into flash without
** using the internal sector buffer.
** If parts of the data to be written are already buffered in the internal
** sector buffer, data will be written into the sector buffer as well as
** directly into physical flash memory to keep the buffered data in sync.
**
** \param handle    Valid driver instance handle previously returned by a call
**                  of FAPEX_FLASH_Open().
** \param address   The byte address where to read data from, this address
**                  will be interpreted as relativ address inside flash memory.
** \param byteCount The number of bytes to read from flash.
** \param bufferPtr The target buffer where to write the read flash data into.
**
** \note Both parameters, byteCount as well as address must be 4-byte
**       aligned values.
**
** \return
** - #FAPI_OK if operation was successful
** - #FAPI_ERR_BAD_PARAMETER if a given parameter was invalid.
** - #FAPI_ERR_TIMEOUT if operation timed out.
**
** \sa FAPI_FLASH_Open
********************************************************************************
*/
int32_t FAPEX_VRFLASH_WriteBytesDirect( FAPEX_SYS_HandleT handle, uint32_t address,
                                  uint32_t byteCount, const void* bufferPtr )
{
    return FAPEX_VRFLASH_WriteBytes( handle, address, byteCount, bufferPtr );
}

/*!
********************************************************************************
** \brief Get current flash devive size
**
** This function retrieves the flash device size in bytes from the given
** flash handle.
**
** \param handle Valid driver instance handle previously returned by a call
**               of FAPEX_VRFLASH_Open().
**
** \return
** - 'non-0' the current flash device size in bytes.
** - '0' if a given parameter was invalid.
********************************************************************************
*/
int32_t FAPEX_VRFLASH_GetDeviceSize( FAPEX_SYS_HandleT handle )
{
    FACS_CONTROL* flash_handle_ptr = (FACS_CONTROL*)handle;

    if( !fapex_vrflashCheckHandle( flash_handle_ptr ) )
        return( 0 );

    return( (int32_t)(facs_get_vrflash_size(flash_handle_ptr)) );
}

/*!
********************************************************************************
** \brief Get total number of flash sectors
**
** This function retrieves the total number of flash sectors within the given
** flash device.
**
** \param handle  Valid driver instance handle previously returned by a call
**                of FAPEX_VRFLASH_Open().
**
** \return
** - '>0' the total number of flash sectors.
** - #FAPI_ERR_BAD_PARAMETER if a given parameter was invalid.
**
** \sa FAPEX_VRFLASH_Open
********************************************************************************
*/
int32_t FAPEX_VRFLASH_GetSectorCount( FAPEX_SYS_HandleT handle )
{
    FACS_CONTROL* flash_handle_ptr = (FACS_CONTROL*)handle;

    if( !fapex_vrflashCheckHandle( flash_handle_ptr ) )
        return( FAPEX_VRFLASH_ERR_BAD_PARAMETER );

    return( (int32_t)(facs_get_sectorcount( flash_handle_ptr )) );
}

/*!
********************************************************************************
** \brief Flushes all buffered flash data.
**
** This function flushes all buffered flash data to the real flash device
** memory.
**
** \param handle Valid driver instance handle previously returned by
**               FAPEX_VRFLASH_Open.
**
** \return
** - #FAPEX_OK if operation was successful
** - #FAPEX_VRFLASH_ERR_BAD_PARAMETER if the given handle is invalid.
**
** \sa FAPI_FLASH_Open
********************************************************************************
*/
int32_t FAPEX_VRFLASH_Flush( FAPEX_SYS_HandleT handle )
{
    FACS_CONTROL* flash_handle_ptr = (FACS_CONTROL*)handle;
    int32_t       result;


    if( fapex_vrflashCheckHandle( flash_handle_ptr ) )
    {
        result = facs_flashFlushBuffer( flash_handle_ptr );
    }
    else
    {
        result = FAPEX_VRFLASH_ERR_BAD_PARAMETER;
    }


    return( result );
}

#include "facs.c"
