/*
********************************************************************************
********************************************************************************
**
** \file        shell.c
**
** \brief       A generic command shell implementation.
**
** This source file implements a simple shell handler.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
********************************************************************************
********************************************************************************
*/

#include <string.h>
#include <fapex/abst_types.h>
#include "shell.h"

//***************************************************************************
//***************************************************************************
//** Defines and Macros
//***************************************************************************
//***************************************************************************

#define SHELL_ColorFgBlack     "\033[30m"
#define SHELL_ColorFgRed       "\033[31m"
#define SHELL_ColorFgGreen     "\033[32m"
#define SHELL_ColorFgBlue      "\033[34m"
#define SHELL_ColorBgWhite     "\033[47m"
#define SHELL_CommandTableSize  64
#define SHELL_CommandStringSize 80
#define SHELL_HistoryTableSize  16
#define SHELL_ArgvSize          16

//***************************************************************************
//***************************************************************************
//** Local Types
//***************************************************************************
//***************************************************************************
typedef struct SHELL_CommandStruct {
    char_t*    command;
    char_t*    brief;
    char_t*    description;
    uint32_t   (*function)(int32_t, char_t**);
} SHELL_CommandStr;

//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************
static SHELL_CommandStr  commandTable[SHELL_CommandTableSize];
static char_t            inputCommand[SHELL_CommandStringSize];
static char_t            historyTable[SHELL_HistoryTableSize][SHELL_CommandStringSize];
static int32_t           historyIndex;
static int32_t           updownIndex;

//***************************************************************************
//***************************************************************************
//** Global Data
//***************************************************************************
//***************************************************************************

//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/**
*******************************************************************************
**
** \brief Callback function the "help" command
**
** This function will be called by the shell thread if the "help" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
uint32_t SHELL_HelpCommand( int32_t argc, char_t* argv[] )
{
    int32_t length;
    int32_t index;

    if( argc == 1 )
    {
        FAPI_SYS_PRINT_MSG( "\n" );
        FAPI_SYS_PRINT_MSG( "Available commands:\n" );
        FAPI_SYS_PRINT_MSG( "-------------------\n" );
        for( index=0; index < SHELL_CommandTableSize; index++ )
        {
            if( commandTable[index].description != NULL ) {
                length = (int32_t)strlen( commandTable[index].description );
                if( ( length > 0 )
                 && ( commandTable[index].description[length-1] == '\n' ) )
                    commandTable[index].description[length-1] = '\0';
                if( ( commandTable[index].command  != NULL )
                 && ( commandTable[index].function != NULL )
                 && ( commandTable[index].brief    != NULL ) )
                {
                    FAPI_SYS_PRINT_MSG( "[%s%s%s] : %s\n",
                                        SHELL_ColorFgBlue,
                                        commandTable[index].command,
                                        SHELL_ColorFgBlack,
                                        commandTable[index].brief );
                }
            }
        }
        FAPI_SYS_PRINT_MSG( "\n" );
        return( 1 );
    }

    if( argc == 2 )
    {
        FAPI_SYS_PRINT_MSG( "Command help:\n" );
        FAPI_SYS_PRINT_MSG( "-------------\n" );
        for( index=0; index < SHELL_CommandTableSize; index++ )
        {
            if( commandTable[index].command != NULL ) {
                if( strcmp( commandTable[index].command, argv[1] ) == 0 )
                {
                    if( commandTable[index].description != NULL ) {
                        length = (int32_t)strlen( commandTable[index].description );
                        if( ( length > 0 )
                         && ( commandTable[index].description[length-1] == '\n' ) )
                            commandTable[index].description[length-1] = '\0';
                        if( ( commandTable[index].command  != NULL )
                         && ( commandTable[index].function != NULL )
                         && ( commandTable[index].brief    != NULL ) )
                        {
                            FAPI_SYS_PRINT_MSG( "[%s%s%s] : %s\n",
                                                SHELL_ColorFgBlue,
                                                commandTable[index].command,
                                                SHELL_ColorFgBlack,
                                                commandTable[index].brief );
                            if( commandTable[index].description
                                && commandTable[index].description[0] )
                                FAPI_SYS_PRINT_MSG( "%s\n",
                                                    commandTable[index].description );
                        }
                    }
                }
            }
        }
        FAPI_SYS_PRINT_MSG( "\n" );
        return( 1 );
    }
    return(1);
}

/**
*******************************************************************************
**
** \brief Callback function the "history" command
**
** This function will be called by the shell thread if the "help" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
uint32_t SHELL_HistoryCommand( int32_t argc, char_t* argv[] )
{
    char_t  active;
    int32_t index;

    argc = argc;
    argv = argv;

    FAPI_SYS_PRINT_MSG( "\n" );
    FAPI_SYS_PRINT_MSG( "Command history:\n" );
    FAPI_SYS_PRINT_MSG( "----------------\n" );
    for( index=0; index < SHELL_HistoryTableSize; index++ )
    {
        if( index == historyIndex )
        {
            strcpy( historyTable[index], "h" );
            active = '>';
        }
        else active = ' ';
        FAPI_SYS_PRINT_MSG( "%c %2d : %s%s%s\n",
                            active,
                            index,
                            SHELL_ColorFgBlue,
                            historyTable[index],
                            SHELL_ColorFgBlack );
    }
    FAPI_SYS_PRINT_MSG( "\n" );
    return(1);
}

/**
*******************************************************************************
**
** \brief Callback function the "!<##>" command
**
** This function will be called by the shell thread if the "help" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \retval >=0 historyTable index
** \retval  <0 error
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
uint32_t SHELL_RedoCommand( int32_t argc, char_t* argv[] )
{
    char_t* string;
    int32_t number;

    if( ( argc == 0 ) || ( argv[0] == 0 ) )
        return( 0 );

    if( argv[0][0] == '!' )
    {
        if( strcmp( argv[0], "!!" ) == 0 )
        {
            if( historyIndex > 0 ) {
                return(historyIndex - 1);
            }
            else {
                return(SHELL_HistoryTableSize - 1);
            }
        }

        string = argv[0] + 1;
        number = 0;
        while( *string )
            number = ( number * 10 ) + ( (*string++) - '0' );
        if( ( number < SHELL_HistoryTableSize )
         && ( historyTable[number][0] != '\0' ) )
        {
            return( number );
        }
        FAPI_SYS_PRINT_MSG( "Error: history index %d is out of range [0..%d]\n",
                            number, SHELL_HistoryTableSize-1 );
        return( (uint32_t)-1 );
    }

    return( (uint32_t)-2 );
}

/**
*******************************************************************************
**
** \brief Callback function the "cursor up/down" commands
**
** This function will be called by the shell thread if the "help" command
** is found in "argv[0]".
**
** \param argc A counter that holds the number of strings stored in argv[].
** \param argv A string array contains the command itself and all additional
**             parameters (strings).
**
** \return
** - 1  if command was handled successfully.
**
** \note "argc" contains at least '1' as "argv[0]" always contains the
**       command itself.
**
*******************************************************************************
*/
uint32_t SHELL_CursorUpDownCommand( int32_t argc, char_t* argv[] )
{
    int32_t copy_index = updownIndex;
    int32_t count;
    if( ( argc == 0 ) || ( argv[0] == 0 ) )
        return( 0 );

    if( strcmp( argv[0], "kbd:cursor-up" ) == 0 )
    {
        for( count=0; count < SHELL_HistoryTableSize; count++ )
        {
            if( copy_index > 0 ) copy_index--;
            else copy_index = SHELL_HistoryTableSize - 1;

            if( historyTable[copy_index][0] != '\0' )
            {
                strcpy( inputCommand,
                        historyTable[copy_index] );
                updownIndex = copy_index;
                break;
            }
        }

        return( 1 );
    }

    if( strcmp( argv[0], "kbd:cursor-down" ) == 0 )
    {
        for( count=0; count < SHELL_HistoryTableSize; count++ )
        {
            if( copy_index < ( SHELL_HistoryTableSize - 1 ) ) copy_index++;
            else copy_index = 0;

            if( historyTable[copy_index][0] != '\0' )
            {
                strcpy( inputCommand,
                        historyTable[copy_index] );
                updownIndex = copy_index;
                break;
            }
        }
        return( 1 );
    }

    return( 0 );
}

/**
*******************************************************************************
**
** \brief Perform the given command.
**
** This function performs (executes) the given command.
**
** \param command The command to execute.
**
*******************************************************************************
*/
void SHELL_DoCommand( char_t* command )
{
    int32_t   i, argc, index;
    char_t    *argv[SHELL_ArgvSize+1];
    uint32_t  result;

    if( command && command[0] )
    {
        if( inputCommand != command ) {
            strcpy( inputCommand, command );
        }

        /* split string */
        i = argc = 0;
        argv[argc] = command;
        while( command[i] != '\0' ) {
            if( command[i] == ' ' ) {
                while( command[i] == ' ' ) {
                    command[i++] = '\0';
                }
                argv[++argc] = &command[i];
            }
            else {
                i++;
            }
        }
        argv[++argc] = &command[i];

        /* check redo command */
        if( argv[0][0] == '!' ) {
            i = (int32_t)SHELL_RedoCommand( argc, argv );
            if(i >= 0) {
                SHELL_DoCommand( historyTable[i] );
                /* restore command */
                strcpy( historyTable[i], inputCommand );
            }
            return;
        }

        /* check command */
        for( index=0; index < SHELL_CommandTableSize; index++ ) {
            if( ( commandTable[index].command  != NULL )
            &&  ( commandTable[index].function != NULL )
            &&  ( strcmp( commandTable[index].command, argv[0] ) == 0 ) )
            {
                result = commandTable[index].function( argc, argv );
                if( result ) {
                    strcpy( historyTable[historyIndex], inputCommand );
                    historyIndex++;
                    if( historyIndex >= SHELL_HistoryTableSize )
                        historyIndex = 0;
                    updownIndex = historyIndex;
                }
                else {
                    FAPI_SYS_PRINT_MSG( "%s\nError: bad/unknown sub command(s) for \"%s\"\n%s",
                                        SHELL_ColorFgRed, argv[0], SHELL_ColorFgBlack );
                    strcpy( inputCommand, "? " );
                    strcat( inputCommand, argv[0] );
                    SHELL_DoCommand( inputCommand );
                }
                return;
            }
        }
        FAPI_SYS_PRINT_MSG( "%s\nError: unknown command \"%s\"\n%s",
                            SHELL_ColorFgRed, argv[0], SHELL_ColorFgBlack );
    }
}

/**
*******************************************************************************
**
** \brief Register a new command.
**
** This function registers the given command to the internal command table.
** An already existing command will be unregistered first and replaced by the
** given parameters. The buffers for the brief and the description texts will
** be dynamically allocated to avoid buffer overflows.
**
** \param command     The command (string) to register.
** \param brief       A short (one line) help message.
** \param description The full command description, maybe NULL.
** \param function    The function to be called when the user enters the
**                    "command" (with optional parameters), this functions
**                    must accept the argc, argv argument pair for additional
**                    optional parameters [strings].
**
*******************************************************************************
*/
int32_t SHELL_RegisterCommand( char_t*  command,
                               uint32_t (*function)(int32_t, char_t**),
                               char_t*  brief,
                               char_t*  description )
{
    int32_t status = -100;
    int32_t index;

    if( ! ( command && function && brief ) )
        return -1;

    for( index=0; index < SHELL_CommandTableSize; index++ )
    {
        if( commandTable[index].command == NULL )
        {
            if( description == NULL )
                description = "";
            commandTable[index].command  =
                                 (char_t*)FAPI_SYS_MALLOC( strlen(command)+1 );
            commandTable[index].brief =
                                 (char_t*)FAPI_SYS_MALLOC( strlen(brief)+1 );
            commandTable[index].description =
                                 (char_t*)FAPI_SYS_MALLOC( strlen(description)+1);
            if( ( commandTable[index].command     == NULL )
            ||  ( commandTable[index].brief       == NULL )
            ||  ( commandTable[index].description == NULL ) )
            {
                FAPI_SYS_FREE( (commandTable[index].command) );
                FAPI_SYS_FREE( (commandTable[index].brief) );
                FAPI_SYS_FREE( (commandTable[index].description) );
                commandTable[index].command     = NULL;
                commandTable[index].brief       = NULL;
                commandTable[index].description = NULL;
                commandTable[index].function    = NULL;
                status = -2;
                FAPI_SYS_PRINT_MSG( "%s\nError: cannot register command\n%s",
                                    SHELL_ColorFgRed, SHELL_ColorFgBlack );
                break;
            }
            strcpy( commandTable[index].command,     command     );
            strcpy( commandTable[index].brief,       brief       );
            strcpy( commandTable[index].description, description );
            commandTable[index].function = function;
            status = 0;
            break;
        }
    }
    return( status );
}

