/*
*****************************************************************************
**
** \file        ./sample_application/fapex_av_demo/src/sidebug.c
**
** \version     $Id$
**
** \brief       Module providing terminal output functions for DVBSI/PSI table parsing.
**
** This file contains the declaration of the scan driver API. It initializes
** and configures TSD including all related buffers. Furthermore it provides
** a simple API to start/stop transport stream scan.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/
#include <fapex/abst_types.h>
#include <fapex/abst_libtsd.h>
#include "sidebug.h"


//***********************************************************************
//***********************************************************************
//** Local Defines and Macros
//***********************************************************************
//***********************************************************************
#define DESC_OFFSET                                 2

/* DEFINES USED FOR PMT-PARSING */
/* MPEG-2 DESCRIPTORS */
#define VIDEO_STREAM_DESCRIPTOR                    0x02
#define AUDIO_STREAM_DESCRIPTOR                    0x03
//#define HIERARCHY_DESCRIPTOR                       0x04
//#define REGISTRATION_DESCRIPTOR                    0x05
//#define DATA_STREAM_ALIGNMENT_DESCRIPTOR           0x06
#define TARGET_BACKGROUND_GRID_DESCRIPTOR          0x07
//#define VIDEO_WINDOW_DESCRIPTOR                    0x08
#define CA_DESCRIPTOR                              0x09
#define ISO_639_DESCRIPTOR                         0x0a
//#define SYSTEM_CLOCK_DESCRIPTOR                    0x0b
//#define MULTIPLEX_BUFFER_UTILIZATION_DESCRIPTOR    0x0c
#define COPYRIGHT_DESCRIPTOR                       0x0d
#define MAXIMUM_BITRATE_DESCRIPTOR                 0x0e
#define PRIVATE_DATA_INDICATOR_DESCRIPTOR          0x0f
//#define SMOOTHING_BUFFER_DESCRIPTOR                0x10
#define STD_DESCRIPTOR                             0x11
//#define IBP_DESCRIPTOR                             0x12
#define MPEG4_VIDEO_DESCRIPTOR                     0x1B
#define MPEG4_AUDIO_DESCRIPTOR                     0x1C
#define AVC_VIDEO_DESCRIPTOR                       0x28
#define AVC_TIMING_HRD_DESCRIPTOR                  0x2A
#define MPEG2_AAC_AUDIO_DESCRIPTOR                 0x2B

/* DVB - DESCRIPTORS (extension of MPEG desc.) */
//#define VBI_DATA_DESCRIPTOR                        0x45
//#define VBI_TELETEXT_DESCRIPTOR                    0x46
//#define MOSAIC_DESCRIPTOR                          0x51
#define STREAM_IDENTIFIER_DESCRIPTOR               0x52
#define TELETEXT_DESCRIPTOR                        0x56
#define SUBTITLING_DESCRIPTOR                      0x59
#define PRIVATE_DATA_DESCRIPTOR                    0x5f
//#define SERVICE_MOVE_DESCRIPTOR                    0x60
//#define CA_SYSTEM_DESCRIPTOR                       0x65
#define DATA_BROADCAST_ID_DESCRIPTOR               0x66
#define AC_3_DESCRIPTOR                            0x6a
#define ANCILLARY_DATA_DESCRIPTOR                  0x6b
#define ADAPTATION_FIELD_DATA_DESCRIPTOR           0x6f
#define EAC_3_DESCRIPTOR                           0x7a
/* OPENTV - DESCRIPTORS */
#define MODULE_TRACK_DESCRIPTOR                    0x90
#define TRACK_ATTRIBUTE_DESCRIPTOR                 0xfd
#define TRACK_TAG_DESCRIPTOR                       0xfe
/* Special custom descriptor */
#define USER_DEF_MULTI_AUDIO                       0xc5
#define USER_DEF_AC_3                              0x6a06
#define USER_DEF_EAC_3                             0x7a06




/* DEFINES USED FOR DVB SI-PARSING OF DVB DESCRIPTORS ETS 300468 */
#define NETWORK_NAME_DESCRIPTOR             0x40
#define SERVICE_LIST_DESCRIPTOR             0x41
#define SATELLITE_DELIVERY_DESCRIPTOR       0x43
#define BOUQUET_NAME_DESCRIPTOR             0x47
#define SERVICE_DESCRIPTOR                  0x48
#define LINKAGE_DESCRIPTOR                  0x4A
#define SHORT_EVENT_DESCRIPTOR              0x4D
#define EXTENDED_EVENT_DESCRIPTOR           0x4E
#define COMPONENT_DESCRIPTOR                0x50
#define CA_IDENTIFIER_DESCRIPTOR            0x53
#define CONTENT_DESCRIPTOR                  0x54
#define PARENTAL_RATING_DESCRIPTOR          0x55
#define PRIVATE_DATA_SPECIFIER_DESCRIPTOR   0x5F
#define PDC_DESCRIPTOR                      0x69



//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************
static void PsiDescriptor(uint8_t *descData, uint32_t len);
static void DvbsiDescriptor(uint8_t *descData, uint32_t len);
static void DvbsiConvertDeliveryFec(uint8_t fec);
static void DvbsiConvertSatDeliveryFlags(uint8_t flags);
static void DvbsiServiceFlags(uint8_t flags);
static uint16_t DvbsiText2String(uint8_t *sourceP, uint8_t len, uint8_t *destP, uint32_t noEmphasis);
static void MemoryDumpByte(uint8_t *data, int32_t size);

/*!
*******************************************************************************

\c PsiDescriptor() should be used just for debugging. Call this
function if you want to know which descriptors are inside the given descriptor
loop.

\param  descData  Pointer to buffer containing the descriptor loop.
\param  len       Length of descriptor loop.

******************************************************************************/
void PsiDescriptor(uint8_t *descData, uint32_t len)
{
    uint8_t     i, descLength, descTag, idx=0;
    uint8_t     *desc;
    uint32_t    currentLen = 0;
    uint8_t     lang[5];
    uint8_t     name[20];

    /* parse whole PSI descriptor loop */
    while(currentLen<len)
    {
        desc = &descData[currentLen];
        descTag = desc[0];
        descLength = desc[1];
        currentLen += descLength + 2;
        FAPI_SYS_PRINT_MSG( "   %i. (desc:0x%02hx) ", idx, desc[0]);

        switch(descTag)
        {
            case PRIVATE_DATA_INDICATOR_DESCRIPTOR:
                /* private_data_indicator */
                FAPI_SYS_PRINT_MSG( "-> private data indicator MPEG-2-defined(0x");
                for(i=0; i<descLength; i++)
                    FAPI_SYS_PRINT_MSG( "%hx", desc[DESC_OFFSET+i]);
                FAPI_SYS_PRINT_MSG( " = " );
                for(i=0; i<descLength; i++)
                {
                    if(desc[DESC_OFFSET + i] >= 0x30)
                        FAPI_SYS_PRINT_MSG( "%c", desc[DESC_OFFSET+i]);
                }
                FAPI_SYS_PRINT_MSG( ")\n");
                break;
            case TRACK_ATTRIBUTE_DESCRIPTOR:
                /* user_defined track_attribute_descriptor */
                FAPI_SYS_PRINT_MSG( "-> track attribute ");
                if(desc[DESC_OFFSET] >= 0x80) {
                    FAPI_SYS_PRINT_MSG("(DVB user defined):%i = default track\n", desc[2]);
                }
                else
                    FAPI_SYS_PRINT_MSG(":%i\n", desc[2]);
                break;
            case TRACK_TAG_DESCRIPTOR:
                /* user_defined track_tag_descriptor */
                FAPI_SYS_PRINT_MSG("-> OTV track tag (DVB user defined) (0x");
                for(i=0; i<descLength; i++)
                    FAPI_SYS_PRINT_MSG( "%hx", desc[DESC_OFFSET+i]);
                FAPI_SYS_PRINT_MSG(" = ");
                for(i=0; i<descLength; i++)
                {
                    if(desc[DESC_OFFSET+i] >= 0x30)
                        FAPI_SYS_PRINT_MSG("%c", desc[DESC_OFFSET+i]);
                }
                FAPI_SYS_PRINT_MSG(")\n");
                break;
            case MODULE_TRACK_DESCRIPTOR:
                /* user_defined module_track_descriptor, sets the table ID */
                FAPI_SYS_PRINT_MSG( "-> Module track (DVB user defined)(table id:%i)\n",
                    desc[2]);
                break;
            case PRIVATE_DATA_DESCRIPTOR:
                /* private data specifier */
                FAPI_SYS_PRINT_MSG( "-> private data specifier DVB define(0x");
                for(i=0; i<descLength; i++)
                    FAPI_SYS_PRINT_MSG( "%hx", desc[DESC_OFFSET+i]);
                FAPI_SYS_PRINT_MSG( " = ");
                for(i=0; i<descLength; i++)
                {
                    if(desc[DESC_OFFSET+i] >= 0x30)
                        FAPI_SYS_PRINT_MSG( "%c", desc[DESC_OFFSET+i]);
                }
                FAPI_SYS_PRINT_MSG( ")\n");
                break;
            case ISO_639_DESCRIPTOR:
                /* language descriptor ID */
                for(i=0; i<3; i++)
                    lang[i] = desc[DESC_OFFSET+i];
                lang[3] = 0;
                FAPI_SYS_PRINT_MSG( "-> ISO639 Language (%s)\n", lang);
                break;
            case AC_3_DESCRIPTOR:
                /* AC-3 descriptor */
                FAPI_SYS_PRINT_MSG( "-> AC-3 descriptor(len:%i). Flags:0x%hx ",
                    descLength, desc[DESC_OFFSET]);
                i = 0;
                if(( desc[DESC_OFFSET] & 0x80) == 0x80)
                {
                    FAPI_SYS_PRINT_MSG( "AC-3_type:0x%hx ", desc[DESC_OFFSET+1+i]);
                    i++;
                }
                if(( desc[DESC_OFFSET] & 0x40) == 0x40)
                {
                    FAPI_SYS_PRINT_MSG( "BSID(version):0x%hx ", desc[DESC_OFFSET+1+i]);
                    i++;
                }
                if(( desc[DESC_OFFSET] & 0x20) == 0x20)
                {
                    FAPI_SYS_PRINT_MSG( "MAINID(main service):0x%hx ",
                        desc[DESC_OFFSET+1+i]);
                    i++;
                }
                if(( desc[DESC_OFFSET] & 0x10) == 0x10)
                {
                    FAPI_SYS_PRINT_MSG( "ASVC(assoz. service):0x%hx",
                        desc[DESC_OFFSET+1+i]);
                    i++;
                }
                FAPI_SYS_PRINT_MSG( ")\n");
                break;
            case EAC_3_DESCRIPTOR:
                /* EAC-3 descriptor (DD+) */
                FAPI_SYS_PRINT_MSG( "-> EAC-3 descriptor(len:%i Flags:0x%hx)\n",
                    descLength, desc[DESC_OFFSET]);
                break;
            case TELETEXT_DESCRIPTOR:
                /* teletext descriptor ID */
                FAPI_SYS_PRINT_MSG( "-> Teletext (%c%c%c type/page:%i magazine%i)\n",
                    desc[DESC_OFFSET],desc[DESC_OFFSET+1],
                    desc[4],desc[5],desc[6]);
                break;
            case STREAM_IDENTIFIER_DESCRIPTOR:
                /* Stream identifier descriptor */
                FAPI_SYS_PRINT_MSG( "-> Stream identifier descriptor(");
                FAPI_SYS_PRINT_MSG( "component tag:0x%x)\n", desc[DESC_OFFSET]);
                break;
            case DATA_BROADCAST_ID_DESCRIPTOR:
                /* data broadcast ID */
                FAPI_SYS_PRINT_MSG( "-> data broadcast id descriptor(");
                FAPI_SYS_PRINT_MSG( "len: 0x%x, 1.byte:0x%x)\n",
                    descLength, desc[DESC_OFFSET]);
                break;
            case VIDEO_STREAM_DESCRIPTOR:
                /* video stream descriptor ID */
                FAPI_SYS_PRINT_MSG( "-> video stream descriptor (MPEG-2-defined)\n");
                break;
            case AUDIO_STREAM_DESCRIPTOR:
                /* audio stream descriptor ID */
                FAPI_SYS_PRINT_MSG( "-> audio stream descriptor (MPEG-2-defined)\n");
                break;
            case TARGET_BACKGROUND_GRID_DESCRIPTOR:
                /* target background grid descriptor ID */
                FAPI_SYS_PRINT_MSG( "-> target background grid(");
                FAPI_SYS_PRINT_MSG( "Hor.Ver.Size:%ix%i, asp.ratio:%i)\n",
                    ((desc[DESC_OFFSET]<<6)|((desc[DESC_OFFSET+1]&0xfc)>>2)),
                    (((desc[DESC_OFFSET+1]&0x03)<<12)|(desc[DESC_OFFSET+2]<<4)|
                      ((desc[DESC_OFFSET+3]&0xf0)>>4)), (desc[DESC_OFFSET+3]&0x0f));
                break;
            case MAXIMUM_BITRATE_DESCRIPTOR:
                /* maximum bitrate descriptor ID */
                FAPI_SYS_PRINT_MSG( "-> max. bitrate descriptor(");
                FAPI_SYS_PRINT_MSG( "len: 0x%x, max.bitrate:0x%x%x%x)\n",
                        descLength, (desc[DESC_OFFSET] & 0x3f),
                        desc[DESC_OFFSET+1], desc[DESC_OFFSET+2]);
                break;
            case COPYRIGHT_DESCRIPTOR:
                /* copyright descriptor ID */
                FAPI_SYS_PRINT_MSG( "-> copyright descriptor(");
                FAPI_SYS_PRINT_MSG( "len: 0x%x, identifier:0x%02x%02x%02x%02x)\n",
                        descLength, desc[DESC_OFFSET], desc[DESC_OFFSET+1],
                        desc[DESC_OFFSET+2], desc[DESC_OFFSET+3]);
                break;
            case STD_DESCRIPTOR:
                /* STD descriptor */
                FAPI_SYS_PRINT_MSG( "-> STD descriptor(");
                FAPI_SYS_PRINT_MSG( "leak valid flag:0x%x)\n", (desc[DESC_OFFSET]&0xFE));
                break;
            case SUBTITLING_DESCRIPTOR:
                /* Subtitling descriptor */
                FAPI_SYS_PRINT_MSG( "-> Subtitling descriptor(");
                FAPI_SYS_PRINT_MSG( "len: 0x%x, 1.byte:0x%x, 2.byte:0x%x)\n",
                    descLength, desc[DESC_OFFSET], desc[DESC_OFFSET+1]);
                break;
            case ADAPTATION_FIELD_DATA_DESCRIPTOR:
                /* adaptation field data descriptor */
                FAPI_SYS_PRINT_MSG( "-> adaptation field data descriptor(");
                FAPI_SYS_PRINT_MSG( "len: 0x%x, 1.byte:0x%x, 2.byte:0x%x)\n",
                    descLength, desc[DESC_OFFSET], desc[DESC_OFFSET+1]);
                break;
            case ANCILLARY_DATA_DESCRIPTOR:
                /* ancillary data descriptor */
                FAPI_SYS_PRINT_MSG( "-> ancillary data descriptor(");
                FAPI_SYS_PRINT_MSG( "tag: 0x%x, length: 0x%x, 1.byte:0x%x)\n",
                    descTag, descLength, desc[DESC_OFFSET]);
                break;
            case USER_DEF_MULTI_AUDIO:
                FAPI_SYS_PRINT_MSG( "-> user defined multi audio descriptor(");
                FAPI_SYS_PRINT_MSG( "len: 0x%x, 1.byte:0x%x, 2.byte:0x%x)\n",
                    descLength, desc[DESC_OFFSET], desc[DESC_OFFSET+1]);
                if(descLength > 19)
                    descLength = 19;
                for(i=0;i<descLength;i++)
                    name[i] = desc[DESC_OFFSET +1 +i];
                name[i] = 0;
                FAPI_SYS_PRINT_MSG( "%i. TrackDescription:%s\n", desc[DESC_OFFSET], name);
                break;
            case CA_DESCRIPTOR:
                /* CA descriptor */
                FAPI_SYS_PRINT_MSG( "-> CA descriptor()\n");
                FAPI_SYS_PRINT_MSG( "-> CA descriptor(");
                FAPI_SYS_PRINT_MSG( "len: 0x%x, 1.byte:0x%x)\n",
                    descLength, desc[DESC_OFFSET]);
                break;

            case MPEG4_VIDEO_DESCRIPTOR:
                /* MPEG-4 video descriptor */
                FAPI_SYS_PRINT_MSG( "-> MPEG-4 video descriptor(");
                FAPI_SYS_PRINT_MSG( "profile and level:0x%x)\n", desc[DESC_OFFSET]);
                break;
            case MPEG4_AUDIO_DESCRIPTOR:
                /* MPEG-4 audio descriptor */
                FAPI_SYS_PRINT_MSG( "-> MPEG-4 audio descriptor(");
                FAPI_SYS_PRINT_MSG( "profile and level:0x%x)\n", desc[DESC_OFFSET]);
                break;
            case AVC_VIDEO_DESCRIPTOR:
                /* AVC video descriptor */
                FAPI_SYS_PRINT_MSG( "-> AVC video descriptor(");
                FAPI_SYS_PRINT_MSG( "profileIdc:0x%x levelIdc:0x%x flags:0x%x)\n", desc[DESC_OFFSET], desc[DESC_OFFSET+1],desc[DESC_OFFSET+2]);
                break;
            case AVC_TIMING_HRD_DESCRIPTOR:
                /* AVC timing and HRD descriptor */
                FAPI_SYS_PRINT_MSG( "-> AVC timing and HRD descriptor(");
                FAPI_SYS_PRINT_MSG( "tag: 0x%x, length: 0x%x, 1.byte:0x%x)\n",
                    descTag, descLength, desc[DESC_OFFSET]);
                break;
            case MPEG2_AAC_AUDIO_DESCRIPTOR:
                /* AAC audio descriptor */
                FAPI_SYS_PRINT_MSG( "-> AAC audio descriptor(");
                FAPI_SYS_PRINT_MSG( "profile:0x%x channelConf:0x%x addInfo:0x%x)\n", desc[DESC_OFFSET], desc[DESC_OFFSET+1],desc[DESC_OFFSET+2]);
                break;

            default:
                 FAPI_SYS_PRINT_MSG("-> unknown(len: 0x%x)\n", descLength);
                 break;
        }
        idx++;
    }
}

/*!
*******************************************************************************

\c DvbsiDescriptor() should be used just for debugging. Call this function
if you want to know which descriptors are inside the given descriptor loop.

\param  descData  Pointer to buffer containing the descriptor loop.
\param  len       Length of descriptor loop.

******************************************************************************/
static void DvbsiDescriptor(uint8_t *descData, uint32_t len)
{
    uint8_t   i, descLength, descTag, idx=0, nibble1, nibble2;
    uint32_t  currentLen = 0;
    uint8_t   *desc;
    uint8_t   name[256] = "";
    uint32_t  num;

    /* parse whole DVBSI descriptor loop */
    while(currentLen<len)
    {
        desc = &descData[currentLen];
        descTag = desc[0];
        descLength = desc[1];
        currentLen += descLength + 2;
        FAPI_SYS_PRINT_MSG( "   %i. (0x%02hx) ", idx, desc[0]);

        switch(descTag)
        {
            case BOUQUET_NAME_DESCRIPTOR:
                FAPI_SYS_PRINT_MSG("-> bouquet name desc (name:");
                num = DvbsiText2String(&desc[2], desc[1], name, 1);
                FAPI_SYS_PRINT_MSG("%s)\n", name);
                break;
            case CA_IDENTIFIER_DESCRIPTOR:
                FAPI_SYS_PRINT_MSG("-> CA identifier desc - (len:%i)", desc[1]);
                for( i=2; i<desc[1]; i+=2 )
                    FAPI_SYS_PRINT_MSG("\n      CA systemID:%i", (desc[i]<<8) + desc[i+1]);
                FAPI_SYS_PRINT_MSG("\n");
                break;
            case COMPONENT_DESCRIPTOR:
                FAPI_SYS_PRINT_MSG("-> component_desc (len:%i) - ", desc[1]);
                FAPI_SYS_PRINT_MSG("(%c%c%c) tag:%i ", desc[5], desc[6], desc[7], desc[4] );
                switch(desc[2] & 0x0F)
                {
                    case 1: /* video */
                        FAPI_SYS_PRINT_MSG( "video: " );
                        switch(desc[3])
                        {
                            case 1:  FAPI_SYS_PRINT_MSG( "4:3" ); break;
                            case 2:  FAPI_SYS_PRINT_MSG( "16:9(pan vec)" ); break;
                            case 3:  FAPI_SYS_PRINT_MSG( "16:9" ); break;
                            case 4:  FAPI_SYS_PRINT_MSG( ">16:9" ); break;
                            default: FAPI_SYS_PRINT_MSG( "(%i?)", desc[3]); break;
                        }
                        break;
                    case 2: /* audio */
                        FAPI_SYS_PRINT_MSG( "audio: " );
                        switch(desc[3])
                        {
                            case 1:  FAPI_SYS_PRINT_MSG( "MONO" ); break;
                            case 2:  FAPI_SYS_PRINT_MSG( "dual MONO" ); break;
                            case 3:  FAPI_SYS_PRINT_MSG( "STEREO" ); break;
                            case 4:  FAPI_SYS_PRINT_MSG( "multi channel" ); break;
                            case 5:  FAPI_SYS_PRINT_MSG( "SURROUND" ); break;
                            default: FAPI_SYS_PRINT_MSG( "(%i?)", desc[3]); break;
                        }
                        break;
                    case 3: /* subtitle/teletext */
                        switch(desc[3])
                        {
                            case 1:    FAPI_SYS_PRINT_MSG("Teletext/Subtitling"); break;
                            case 2:    FAPI_SYS_PRINT_MSG("Teletext"); break;
                            case 0x10:
                            case 0x11:
                            case 0x12:
                            case 0x13: FAPI_SYS_PRINT_MSG("DVB Subtitling"); break;
                            case 0x20:
                            case 0x21:
                            case 0x22:
                            case 0x23: FAPI_SYS_PRINT_MSG("DVB Subtitle(hard hearing)");break;
                            default:   FAPI_SYS_PRINT_MSG("(%i?)", desc[3]); break;
                        }
                        break;
                    default:
                        FAPI_SYS_PRINT_MSG( "(%i?)", desc[2] & 0x0F);
                        break;
                }
                FAPI_SYS_PRINT_MSG("\n");
                break;
            case CONTENT_DESCRIPTOR:
                FAPI_SYS_PRINT_MSG("-> content desc (len:%i)", descLength);
                i=2;
                while(i<=desc[1])
                {
                    nibble1 = (desc[i] & 0xF0) >> 4;
                    nibble2 = desc[i] & 0x0F;
                    i +=2;
                    FAPI_SYS_PRINT_MSG("\n      (%i/%i) ", nibble1, nibble2);
                    switch( nibble1 )
                    {
                        case 1: FAPI_SYS_PRINT_MSG("Movie/Drama"); break;
                        case 2: FAPI_SYS_PRINT_MSG("News/Current Affairs"); break;
                        case 3: FAPI_SYS_PRINT_MSG("Show/Game Show"); break;
                        case 4: FAPI_SYS_PRINT_MSG("Sport"); break;
                        case 5: FAPI_SYS_PRINT_MSG("Children/Youth"); break;
                        case 6: FAPI_SYS_PRINT_MSG("Music/Ballet/Dance"); break;
                        case 7: FAPI_SYS_PRINT_MSG("Arts/Culture"); break;
                        case 8: FAPI_SYS_PRINT_MSG("Social/Politics/Economics"); break;
                        case 9: FAPI_SYS_PRINT_MSG("Education/Science/Factual"); break;
                        case 10: FAPI_SYS_PRINT_MSG("Leisure hobbies"); break;
                        case 11:
                            FAPI_SYS_PRINT_MSG("MISC - ");
                            switch( nibble2 )
                            {
                                case 0:  FAPI_SYS_PRINT_MSG("Original Language"); break;
                                case 1:  FAPI_SYS_PRINT_MSG("Black & White"); break;
                                case 2:  FAPI_SYS_PRINT_MSG("Unpublished"); break;
                                case 3:  FAPI_SYS_PRINT_MSG("Live"); break;
                                default: FAPI_SYS_PRINT_MSG("???"); break;
                            }
                            break;
                        default: FAPI_SYS_PRINT_MSG( "???" ); break;
                    }
                }
                FAPI_SYS_PRINT_MSG("\n");
                break;
            case EXTENDED_EVENT_DESCRIPTOR:
                FAPI_SYS_PRINT_MSG("-> extended event desc (len:%i) - ISO639 Lang(%c%c%c)",
                    desc[1], desc[3], desc[4], desc[5] );
                FAPI_SYS_PRINT_MSG(" %i/%i (item len:%i)\n       text:", (desc[2]>>4)+1,
                    (desc[2] & 0x0F)+1, desc[6]);
                num = DvbsiText2String(&desc[desc[6]+8],desc[desc[6]+7],
                    name, 1);
                FAPI_SYS_PRINT_MSG("%s\n", name);
                break;
            case LINKAGE_DESCRIPTOR:
                FAPI_SYS_PRINT_MSG("-> linkage desc (len:%i) ONID:%i TSID:%i SVCID:%i\n",
                    desc[1], (desc[4] << 8) + desc[5],
                    (desc[2] << 8) + desc[3], (desc[6] << 8) + desc[7]);
                FAPI_SYS_PRINT_MSG("      type: ");
                switch(desc[8])
                {
                    case 1:   FAPI_SYS_PRINT_MSG("information service\n"); break;
                    case 2:   FAPI_SYS_PRINT_MSG("EPG service\n"); break;
                    case 3:   FAPI_SYS_PRINT_MSG("CA replacement service\n"); break;
                    case 4:   FAPI_SYS_PRINT_MSG("TS containing complete NIT/BAT\n"); break;
                    case 5:   FAPI_SYS_PRINT_MSG("service replacement service\n"); break;
                    case 6:   FAPI_SYS_PRINT_MSG("data broadcast service\n"); break;
                    default : FAPI_SYS_PRINT_MSG("(%i)???\n", desc[8]); break;
                }
                break;
            case NETWORK_NAME_DESCRIPTOR:
                FAPI_SYS_PRINT_MSG("-> network name desc - name:");
                num = DvbsiText2String(&desc[2], desc[1], name, 1);
                FAPI_SYS_PRINT_MSG("%s\n", name);
                break;
            case PARENTAL_RATING_DESCRIPTOR:
                FAPI_SYS_PRINT_MSG("-> parental rating desc (len:%i)", descLength);
                i=2;
                while(i<=desc[1])
                {
                    FAPI_SYS_PRINT_MSG("\n      in %c%c%c = %i years", desc[i],
                        desc[i+1], desc[i+2], desc[i+3] + 3);
                    i +=4;
                }
                FAPI_SYS_PRINT_MSG("\n");
                break;
            case PDC_DESCRIPTOR:
                FAPI_SYS_PRINT_MSG("-> PDC desc - PIL=0x%x -> ",
                    ((desc[2] << 16) + (desc[3] << 8) + desc[4]) & 0x000FFFFF);
                FAPI_SYS_PRINT_MSG("date:%i.%i  time:%i:%i\n",
                    ((desc[2] & 0x0F) << 1) + ((desc[3] & 0x80) >> 7),
                    (desc[3] & 0x78) >> 3,
                    ((desc[3] & 0x07) << 2) + ((desc[4] & 0xC0) >> 6),
                    desc[4] & 0x3F);
                break;
            case PRIVATE_DATA_SPECIFIER_DESCRIPTOR:
                FAPI_SYS_PRINT_MSG("-> private data specifier desc - specifier=0x%x\n",
                    (desc[2] << 24) + (desc[3] << 16) + (desc[4] << 8) + desc[5]);
                break;
            case SATELLITE_DELIVERY_DESCRIPTOR:
                FAPI_SYS_PRINT_MSG("-> satellie delivery desc - ");
                DvbsiConvertSatDeliveryFlags(desc[8]);
                FAPI_SYS_PRINT_MSG("orb.pos=%hx\n", (desc[6] << 8) + desc[7]);
                FAPI_SYS_PRINT_MSG("      freq=%x  ",
                    (desc[2] << 24) + (desc[3] << 16) + (desc[4] << 8) + desc[5]);
                FAPI_SYS_PRINT_MSG("sym=%x  FEC=", (desc[9] << 24) + (desc[10] << 16) +
                                      (desc[11] << 8) + (desc[12] & 0xF0));
                DvbsiConvertDeliveryFec((desc[12] & 0x0F));
                FAPI_SYS_PRINT_MSG("\n");
                break;
            case SERVICE_DESCRIPTOR:
                FAPI_SYS_PRINT_MSG("-> service desc (type=");
                SI_DEBUG_DvbsiConvertSvcType(desc[2]);
                num = DvbsiText2String(&desc[4], desc[3], name, 1);
                FAPI_SYS_PRINT_MSG("provider:%s)\n", name);
                i = desc[3] + 4;
                num = DvbsiText2String(&desc[i+1], desc[i], name, 1);
                FAPI_SYS_PRINT_MSG("       name:%s(%i)\n", name, num);
                /* here we check if a short name is available */
                num = DvbsiText2String(&desc[i+1], desc[i], name, 0);
                if(num!=0)
                    FAPI_SYS_PRINT_MSG("       short:%s(%i)\n", name, num);
                break;
            case SERVICE_LIST_DESCRIPTOR:
                FAPI_SYS_PRINT_MSG("-> service list desc");
                i=2;
                while(i<=desc[1])
                {
                    FAPI_SYS_PRINT_MSG("\n      SVC ID:%i ", (desc[i] << 8) + desc[i+1]);
                    SI_DEBUG_DvbsiConvertSvcType(desc[i+2]);
                    i +=3;
                }
                FAPI_SYS_PRINT_MSG("\n");
                break;
            case SHORT_EVENT_DESCRIPTOR:
                FAPI_SYS_PRINT_MSG("-> short event desc - ISO639 Lang(%c%c%c)\n       name:",
                    desc[2], desc[3], desc[4]);
                num = DvbsiText2String(&desc[6], desc[5], name, 1);
                FAPI_SYS_PRINT_MSG("%s\n       text:", name);
                if( desc[desc[5] + 6]>0 )
                {
                    num = DvbsiText2String(&desc[desc[5]+7], desc[desc[5]+6],
                                name, 1);
                    FAPI_SYS_PRINT_MSG("%s\n", name);
                }
                else
                    FAPI_SYS_PRINT_MSG("\n");
                break;

            default:
                FAPI_SYS_PRINT_MSG("-> unknown(len: 0x%x)\n", descLength);
                break;
        }
        idx++;
    }
}


/*!
*******************************************************************************

\c DvbsiConvertSatDeliveryFlags() should be used just for debugging. Call
this function if you want to know which satellite delivery flags are available.

\param  flags     Satellite delivery flags as defined in DVB.

******************************************************************************/
static void DvbsiConvertSatDeliveryFlags(uint8_t flags)
{
    if((flags & 0x01) == 0x01) {
        FAPI_SYS_PRINT_MSG( "QPSK " );
    } else
        FAPI_SYS_PRINT_MSG( "???? " );

    switch(flags & 0x60)
    {
        case 0x60: FAPI_SYS_PRINT_MSG("cir-r   "); break;
        case 0x40: FAPI_SYS_PRINT_MSG("cir-l   "); break;
        case 0x20: FAPI_SYS_PRINT_MSG("vertical   "); break;
        default:   FAPI_SYS_PRINT_MSG("horizontal "); break;
    }
    if((flags & 0x80) == 0x80) {
        FAPI_SYS_PRINT_MSG("EAST ");
    } else
        FAPI_SYS_PRINT_MSG("WEST ");
    return;
}

/*!
*******************************************************************************

\c DvbsiConvertDeliveryFec() should be used just for debugging. Call this
function if you want to know which forward error correction (FEC) is available.

\param  fec       Forward error correction as defined in DVB.

******************************************************************************/
static void DvbsiConvertDeliveryFec(uint8_t fec)
{
    switch(fec)
    {
        case 1:  FAPI_SYS_PRINT_MSG("1/2 "); break;
        case 2:  FAPI_SYS_PRINT_MSG("2/3 "); break;
        case 3:  FAPI_SYS_PRINT_MSG("3/4 "); break;
        case 4:  FAPI_SYS_PRINT_MSG("5/6 "); break;
        case 5:  FAPI_SYS_PRINT_MSG("7/8 "); break;
        default: FAPI_SYS_PRINT_MSG("??? "); break;
    }
    return;
}


/*!
*******************************************************************************

\c DvbsiText2String() is a utility function to handle decoding of DVBSI
text characters. It clean up any text array within DVB descriptors. In case of
single byte characters you will get a string which is able to be printed on
OSD or terminal.

The following operations are done (valid for single and double byte characters):
- First it removes control codes for the selection of the character table.
- It also removes control codes for emphasis on/off.
- A carriage return / line feed is converted to '\n' (=0x10 or =0x0010).
- when \c noEmphasis is set to \c '0' it will only decode the short form
  of the text(emphased text). If no short form is available the returned length
  is zero. Otherwise the the full text is returned.
- The returned string is terminated with zero.
- The returned length is equal to C-lib function \c strlen().

\note  Usually only service or provider names make use of the emphaised feature.
       If you can't fit the full string try the function with \c noEmphasis
       set to \c FALSE. If a emphaised version is allowed the returned string
       length is non zero. This should be used since broadcasters don't like
       if the service name is just cut off.

\param sourceP    Pointer to the start of the text character descriptor field.
\param len        Length of the descriptor. In case the destination buffer is
                  limited to a maximum size then set the maximum size for
                  the destination string in case the descriptor length is longer.
\param destP      Pointer to the destination buffer to be filled.
\param noEmphasis If set to \c '1' then the full text is returned.
                  If set to \c '0' then only the emphaised text is returned.

\return       Length of decode string in bytes (equal to\c strlen()).

******************************************************************************/
static uint16_t DvbsiText2String( uint8_t *sourceP, uint8_t len, uint8_t *destP, uint32_t noEmphasis)
{
    uint16_t    i, k=0;
    uint32_t    shortOn = 0, doubleByte = 0;


    /* first check if we have to use double or single byte character */
    switch(sourceP[0])
    {
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
            #ifdef DEBUG_DVBSI
            FAPI_SYS_PRINT_MSG( "(char table=%i)", sourceP[0] );
            #endif
            sourceP++;
            len--;
            break;
        case 0x10:
            #ifdef DEBUG_DVBSI
            FAPI_SYS_PRINT_MSG("(double byte char table)");
            #endif
            doubleByte = 0;
            sourceP++;
            len--;
            break;
        default: break;
    }

    if(noEmphasis == 1)
    {   /* calling function wishes to get the full text */
        for(i=0; i<len; i++)
        {
            switch( sourceP[i] )
            {
                case 0x86:
                case 0x87:
                    /* remove/ignore  emphasis */
                    if((doubleByte == 1) && (sourceP[i-1] == 0xE0))
                        k--;
                    break;
                case 0x8A:
                    #ifdef DEBUG_DVBSI
                    FAPI_SYS_PRINT_MSG( "(CR %i)", '\n' );
                    #endif
                    /* use normal line feed also for double byte */
                    if((doubleByte == 1) && (sourceP[i-1] == 0xE0))
                    {
                        destP[k-1] = 0x00; destP[k] = '\n';
                        k++;
                    }
                    else
                    {
                        destP[k] = '\n';
                        k++;
                    }
                    break;
                default:
                    if(sourceP[i]>0x10)
                    {
                        destP[k] = sourceP[i];
                        k++;
                    }
                    break;
            }
        }
    }
    else
    {   /* calling function wishes to get only the emphaised text - if available */
        for(i=0; i<len; i++)
        {
            if( shortOn == 1 )
            {
                if(doubleByte == 0)
                {
                    if( sourceP[i] == 0x87 )
                    {
                        shortOn = 0;
                        #ifdef DEBUG_DVBSI
                        FAPI_SYS_PRINT_MSG("(s off)");
                        #endif
                    }
                    else
                    {
                        destP[k] = sourceP[i];
                        k++;
                    }
                }
                else
                {
                    if((sourceP[i] == 0x87 ) && (sourceP[i-1] == 0xE0))
                    {
                        shortOn = 0;
                        k--;
                        #ifdef DEBUG_DVBSI
                        FAPI_SYS_PRINT_MSG("(s off)");
                        #endif
                    }
                    else
                    {
                        destP[k] = sourceP[i];
                        k++;
                    }
                }
            }
            else
            {
                if(doubleByte == 0)
                {
                    if( sourceP[i] == 0x86 )
                    {
                        shortOn = 1;
                        #ifdef DEBUG_DVBSI
                        FAPI_SYS_PRINT_MSG("(s on)");
                        #endif
                    }
                }
                else
                {
                    if((sourceP[i] == 0x86 ) && (sourceP[i-1] == 0xE0))
                    {
                        shortOn = 1;
                        k--;
                        #ifdef DEBUG_DVBSI
                        FAPI_SYS_PRINT_MSG("(s on)");
                        #endif
                    }
                }
            }
        }
    }
    destP[k] = 0;
    return(k);
}


/*!
*******************************************************************************

\c DvbsiServiceFlags() should be used just for debugging. Call this
function if you want to know which service flags are available.

\param  flags     Service flags as defined in DVB.

******************************************************************************/
static void DvbsiServiceFlags(uint8_t flags)
{
    uint8_t val;

    FAPI_SYS_PRINT_MSG("FLAGS: ");

    val = (flags & 0xE) >> 1;
    switch(val)
    {
        case 1:  FAPI_SYS_PRINT_MSG("not running "); break;
        case 2:  FAPI_SYS_PRINT_MSG("starts soon "); break;
        case 3:  FAPI_SYS_PRINT_MSG("pausing     "); break;
        case 4:  FAPI_SYS_PRINT_MSG("running     "); break;
        default: FAPI_SYS_PRINT_MSG("RST=(%i)??? ", val); break;
    }
    if( ( flags & 0x01 ) == 0x01) {
        FAPI_SYS_PRINT_MSG("CA ");
    } else
        FAPI_SYS_PRINT_MSG("   ");
    if( ( flags & 0x20 ) == 0x20) {
        FAPI_SYS_PRINT_MSG("EITschedule ");
    } else
        FAPI_SYS_PRINT_MSG("            ");
    if( ( flags & 0x10 ) == 0x10) {
        FAPI_SYS_PRINT_MSG("EITp/f ");
    } else
        FAPI_SYS_PRINT_MSG("       ");
    return;
}


/*!
*******************************************************************************

\c MemoryDumpByte() should be used just for debugging.

\param  data     pointer to data.
\param  size     number of bytes.

******************************************************************************/
static void MemoryDumpByte(uint8_t *data, int32_t size)
{
    uint32_t i = 0;
    uint32_t j = 0;
    uint32_t s = (uint32_t)size;

    for(i=0; i<(s>>4); i++)
    {
        FAPI_SYS_PRINT_MSG("0x%08x   ", (i*16));
        for (j=0; j<16; j++)
            FAPI_SYS_PRINT_MSG("%02x ", data[(i*16)+j]);
        FAPI_SYS_PRINT_MSG("\n");
    }
}


//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
*******************************************************************************

\c SI_DEBUG_Pat() should be used just for debugging. The function dump the
content of the PAT table on the terminal.

\param  data  Pointer to buffer containing the descriptor loop.

******************************************************************************/
void SI_DEBUG_Pat(uint8_t *data)
{
    uint32_t secLen, tsid, nbPrg, index, pid, prgNum;

    if( data[0] != (uint8_t)FAPEX_TSD_PAT_TABLE) {
        FAPI_SYS_PRINT_MSG("table ID is not PAT\n");
        MemoryDumpByte(data, 32);
        return;
    }

    secLen = ((data[1]<<8) + data[2]) & 0x0FFF;
    tsid   = (data[3]<<8) + data[4];
    nbPrg  = (secLen - 9) >> 2; /* 9bytes for header and CRC, one prg consist of 4bytes */;

    FAPI_SYS_PRINT_MSG("-> PAT START ###########################\n");
    FAPI_SYS_PRINT_MSG("PAT tableId:%i sectionLen:%i\n", data[0], secLen);
    FAPI_SYS_PRINT_MSG("version:0x%hx  section(%i/%i)\n", data[5], data[6], data[7]);
    FAPI_SYS_PRINT_MSG("TSID:%i ->numbOfProg:%i\n", tsid, nbPrg);

    for(index=0; index<nbPrg; index++)
    {
        prgNum =  (data[8+(index*4)]<<8)  + data[9+(index*4)];
        pid    = ((data[10+(index*4)]<<8) + data[11+(index*4)])&0x1FFF;
        FAPI_SYS_PRINT_MSG("%i. prgNum:%i  pid:%i\n", index, prgNum, pid);
    }
    FAPI_SYS_PRINT_MSG("-> PAT End   ###########################\n");
}

/*!
*******************************************************************************

\c SI_DEBUG_Cat() should be used just for debugging. The function dump the
content of the CAT table on the terminal.

\param  data  Pointer to buffer containing the descriptor loop.

******************************************************************************/
void SI_DEBUG_Cat(uint8_t *data)
{
    uint32_t secLen, descLen;

    if( data[0] != (uint8_t)FAPEX_TSD_CAT_TABLE) {
        FAPI_SYS_PRINT_MSG("table ID is not CAT\n");
        MemoryDumpByte(data, 32);
        return;
    }

    secLen = ((data[1]<<8) + data[2]) & 0x0FFF;
    descLen = (secLen - 9); /* 9bytes for header and CRC */

    FAPI_SYS_PRINT_MSG("-> CAT START ###########################\n");
    FAPI_SYS_PRINT_MSG("CAT tableId:%i sectionLen:%i\n", data[0], secLen);
    FAPI_SYS_PRINT_MSG("version:0x%hx  section(%i/%i)\n", data[5], data[6], data[7]);
    PsiDescriptor(&data[8], descLen);
    FAPI_SYS_PRINT_MSG("-> CAT End   ###########################\n");
}

/*!
*******************************************************************************

\c SI_DEBUG_DvbsiConvertSvcType() should be used just for debugging. Call this
function if you want to know which service type is available.

\param  type     Service type as defined in DVB.

******************************************************************************/
void SI_DEBUG_DvbsiConvertSvcType(uint8_t type)
{
    switch( type )
    {
        case 1:  FAPI_SYS_PRINT_MSG("DigTV      "); break;
        case 2:  FAPI_SYS_PRINT_MSG("DigRadio   "); break;
        case 3:  FAPI_SYS_PRINT_MSG("Teletext   "); break;
        case 4:  FAPI_SYS_PRINT_MSG("NVOD ref   "); break;
        case 5:  FAPI_SYS_PRINT_MSG("NVOD shift "); break;
        case 6:  FAPI_SYS_PRINT_MSG("Mosaic     "); break;
        case 7:  FAPI_SYS_PRINT_MSG("PAL        "); break;
        case 8:  FAPI_SYS_PRINT_MSG("SECAM      "); break;
        case 9:  FAPI_SYS_PRINT_MSG("D2-Mac     "); break;
        case 10: FAPI_SYS_PRINT_MSG("FM Radio   "); break;
        case 11: FAPI_SYS_PRINT_MSG("NTSC       "); break;
        case 12: FAPI_SYS_PRINT_MSG("Data svc   "); break;
        case 16: FAPI_SYS_PRINT_MSG("MHP svc    "); break;
        default: FAPI_SYS_PRINT_MSG("Unknown(%i)", type); break;
    }
    return;
}

/*!
*******************************************************************************

\c SI_DEBUG_Pmt() should be used just for debugging. The function dump the
content of the PMT table on the terminal.

\param  data  Pointer to buffer containing the descriptor loop.

******************************************************************************/
void SI_DEBUG_Pmt(uint8_t *data)
{
    uint32_t secLen, descLen, prgNumber, streamOffset, esInfoLen, pid;

    if( data[0] != (uint8_t)FAPEX_TSD_PMT_TABLE) {
        FAPI_SYS_PRINT_MSG("table ID is not PMT\n");
        MemoryDumpByte(data, 32);
        return;
    }

    secLen    = ((data[1]<<8) + data[2]) & 0x0FFF;
    prgNumber = (data[3]<<8) + data[4];

    FAPI_SYS_PRINT_MSG("-> PMT START #########################################\n");
    FAPI_SYS_PRINT_MSG("PMT tableId:%i sectionLen:%i\n", data[0], secLen);
    FAPI_SYS_PRINT_MSG("version:0x%hx  section(%i/%i)\n", data[5], data[6], data[7]);
    FAPI_SYS_PRINT_MSG("ProgramNumber:%i   PCR pid:%i\n", prgNumber, ((data[8]<<8) + data[9]) & 0x1FFF);
    /* first descriptor loop */
    descLen   = ((data[10]<<8) + data[11]) & 0x0FFF;
    FAPI_SYS_PRINT_MSG("ProgInfoLen:%i\n", descLen);
    PsiDescriptor(&data[12], descLen);

    streamOffset = 12 + descLen;
    while(streamOffset < (secLen-1))
    {   /* loop until stream offset is on first CRC32 byte */
        pid       = ((data[streamOffset+1]<<8) + data[streamOffset+2]) & 0x1FFF;
        esInfoLen = ((data[streamOffset+3]<<8) + data[streamOffset+4]) & 0x0FFF;
        FAPI_SYS_PRINT_MSG("stream type:");
        SI_DEBUG_PsiStreamType(data[streamOffset]);
        FAPI_SYS_PRINT_MSG(" pid:%i len:%i\n", pid, esInfoLen);
        PsiDescriptor(&data[streamOffset+5], esInfoLen);
        /* calculate offset for next descriptor */
        streamOffset+= 5 + esInfoLen;
    }
    FAPI_SYS_PRINT_MSG("-> PMT End   #########################################\n");
}

/*!
*******************************************************************************

\c SI_DEBUG_PsiStreamType() should be used just for debugging. Call this
function if you want to know which stream type is available.

\param  type      Stream type as defined in MPEG-2/DVB.

******************************************************************************/
void SI_DEBUG_PsiStreamType(uint32_t type)
{
    switch(type)
    {
        case 1:  FAPI_SYS_PRINT_MSG("Video MPEG1  "); break;
        case 2:  FAPI_SYS_PRINT_MSG("Video MPEG2  "); break;
        case 3:  FAPI_SYS_PRINT_MSG("Audio MPEG1  "); break;
        case 4:  FAPI_SYS_PRINT_MSG("Audio MPEG2  "); break;
        case 5:  FAPI_SYS_PRINT_MSG("Priv Section "); break;
        case 6:  FAPI_SYS_PRINT_MSG("Priv PESdata "); break;
        case 7:  FAPI_SYS_PRINT_MSG("MHEG         "); break;
        case 8:  FAPI_SYS_PRINT_MSG("DSMCC        "); break;
        case 9:  FAPI_SYS_PRINT_MSG("ITU_T H.222.1"); break;
        case 10: FAPI_SYS_PRINT_MSG("DSMCC TYPE A "); break;
        case 11: FAPI_SYS_PRINT_MSG("DSMCC TYPE B "); break;
        case 12: FAPI_SYS_PRINT_MSG("DSMCC TYPE C "); break;
        case 13: FAPI_SYS_PRINT_MSG("DSMCC TYPE D "); break;
        case 14: FAPI_SYS_PRINT_MSG("ISO 13818-6 auxiliary"); break;
        case 15: FAPI_SYS_PRINT_MSG("ISO 13818-7 audio ADTS "); break;
        case 16: FAPI_SYS_PRINT_MSG("ISO 14496-2  "); break;
        case 17: FAPI_SYS_PRINT_MSG("ISO 14496-3  "); break;
        case 18: FAPI_SYS_PRINT_MSG("ISO 14496-1 section"); break;
        case 19: FAPI_SYS_PRINT_MSG("ISO 14496-1 PES "); break;
        case 20: FAPI_SYS_PRINT_MSG("ISO 13818-6 sync download"); break;
        case 21: FAPI_SYS_PRINT_MSG("Metadata PES "); break;
        case 22: FAPI_SYS_PRINT_MSG("Metadata section "); break;
        case 23: FAPI_SYS_PRINT_MSG("Metadata data carousel "); break;
        case 24: FAPI_SYS_PRINT_MSG("Metadata object carousel "); break;
        case 25: FAPI_SYS_PRINT_MSG("Metadata sync donload "); break;
        case 26: FAPI_SYS_PRINT_MSG("IPMP stream  "); break;
        case 27: FAPI_SYS_PRINT_MSG("Video H.264  "); break;
        case 0x7F: FAPI_SYS_PRINT_MSG("IPMP stream  "); break;
        case 0x81: FAPI_SYS_PRINT_MSG("AC-3 ATSC A52"); break;

        case USER_DEF_AC_3:  FAPI_SYS_PRINT_MSG("AC-3 Priv PES"); break;
        case USER_DEF_EAC_3: FAPI_SYS_PRINT_MSG("EAC-3 Priv PES"); break;

        default: FAPI_SYS_PRINT_MSG("Type:%-3i ", type); break;
    }
}

/*!
*******************************************************************************

\c SI_DEBUG_Sdt() should be used just for debugging. The function dump the
content of the SDT table on the terminal.

\param  data  Pointer to buffer containing the descriptor loop.

******************************************************************************/
void SI_DEBUG_Sdt(uint8_t *data)
{
    uint32_t secLen, tsId, streamOffset, descLoopLen, svcId, flags;

    if( data[0] != (uint8_t)FAPEX_TSD_SDT_ATS_TABLE) {
        FAPI_SYS_PRINT_MSG("table ID is not SDT\n");
        MemoryDumpByte(data, 32);
        return;
    }

    secLen  = ((data[1]<<8) + data[2]) & 0x0FFF;
    tsId    = (data[3]<<8) + data[4];

    FAPI_SYS_PRINT_MSG("-> SDT START #########################################\n");
    FAPI_SYS_PRINT_MSG("SDT tableId:0x%hx sectionLen:%i\n", data[0], secLen);
    FAPI_SYS_PRINT_MSG("version:0x%hx  section(%i/%i)\n", data[5], data[6], data[7]);
    FAPI_SYS_PRINT_MSG("TSID:%i   ONID:%i\n", tsId, (data[8]<<8) + data[9]);

    /* descriptor loop */
    streamOffset = 11;
    while(streamOffset < (secLen-1))
    {   /* loop until stream offset is on first CRC32 byte */
        svcId       = (data[streamOffset]<<8) + data[streamOffset+1];
        descLoopLen = ((data[streamOffset+3]<<8) + data[streamOffset+4]) & 0x0FFF;
        flags       = ((data[streamOffset+2]<<4)&0x30) + (data[streamOffset+3]>>4);
        FAPI_SYS_PRINT_MSG("SVCID:%i ", svcId);
        DvbsiServiceFlags((uint8_t)flags);
        FAPI_SYS_PRINT_MSG(" len:0x%hx\n", descLoopLen);

        DvbsiDescriptor(&data[streamOffset+5], descLoopLen);
        /* calculate offset for next descriptor */
        streamOffset+= 5 + descLoopLen;
    }
    FAPI_SYS_PRINT_MSG("-> SDT End   #########################################\n");
}


