/*
********************************************************************************
**
** \file        ./sample_application/fapex_av_demo_fs/src/tsd.c
**
** \version     $Id$
**
** \brief       TSD input block setup.
**
** This file contains the declaration of the scan driver API. It initializes
** and configures TSD including all related buffers. Furthermore it provides
** a simple API to start/stop transport stream scan.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
********************************************************************************
*/

#include <fapex/abst_types.h>
#include <fapex/abst_libtsd.h>
#include <val/val.h>
#include "demo_app.h"
#include "tsd.h"
#include "audio.h"

//******************************************************************************
//******************************************************************************
//** Defines and Macros
//******************************************************************************
//******************************************************************************
#define APP_TSD_CHUNK_PACKET_NUM        512     // 512 packets

//******************************************************************************
//******************************************************************************
//** thread related defines and macros
//******************************************************************************
//******************************************************************************

#define APP_TSD_READ_THREAD_STACKSIZE   1024
#define APP_TSD_READ_THREAD_PRIORITY    FAPEX_SYS_PRIORITY_HIGH
#define APP_TSD_READ_THREAD_NAME        "app:tsd_read"
#define APP_TSD_INPUT_THREAD_STACKSIZE  1024
#define APP_TSD_INPUT_THREAD_PRIORITY   FAPEX_SYS_PRIORITY_HIGH
#define APP_TSD_INPUT_THREAD_NAME       "app:tsd_input"

//******************************************************************************
//******************************************************************************
//** Local data
//******************************************************************************
//******************************************************************************
static TSD_InputFileInfo    tsdInputFileInfo;

static FAPEX_SYS_ThreadT    appTSreadThreadHandle = NULL;
static FAPEX_SYS_ThreadT    appTSfeedThreadHandle = NULL;
static APP_ThreadStatus     appTSreadThreadStatus;
static APP_ThreadStatus     appTSfeedThreadStatus;
static volatile int32_t     appTSinputEnable      = -1;

static uint32_t tsd_InputBuffBusyStatus[FAPEX_TSD_PACKET_INPUT_BUFF_NUM] = {
    FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF0,
    FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF1,
    FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF2,
    FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF3
};

//******************************************************************************
//******************************************************************************
//** Global Data
//******************************************************************************
//******************************************************************************
int32_t fd_TSD[] = { -1, -1, -1, -1 };

extern VAL_OpenT* vidOpenPtr;

//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

/*!
********************************************************************************
** \brief TS file read thread
********************************************************************************/
static void appTSreadThreadFunction(void *optArg)
{
    int32_t   diff_count;
    uint8_t   *read_ptr;
    uint32_t  read_size;
    uint32_t  wait_count = 0;
    int32_t   ret;

    if(tsdInputFileInfo.filePtr == NULL) {
        return;
    }
    FAPI_SYS_PRINT_DEBUG(1, "appTSreadThreadFunction() start\n");

    tsdInputFileInfo.lastWriteSize = -1;

    APP_ThreadStatus_SetValue( &appTSreadThreadStatus,
                               (APP_THREAD_STATUS_OPENED | APP_THREAD_STATUS_SUSPENDED) );
    // wait start trigger
    FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );

    while(appTSinputEnable >= 0) {
        APP_ThreadStatus_SetValue( &appTSreadThreadStatus, APP_THREAD_STATUS_READY );

        if(appTSinputEnable == 0) {
            /* wait for request from other thread */
            APP_ThreadStatus_Lock( &appTSreadThreadStatus );
            appTSreadThreadStatus.value |= APP_THREAD_STATUS_SUSPENDED;
            APP_ThreadStatus_Unlock( &appTSreadThreadStatus );

            FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );
            continue;
        }

        /* check bufferring status */
        diff_count = tsdInputFileInfo.chunkWriteCount - tsdInputFileInfo.chunkReadCount;
        if(diff_count >= 2) {
            APP_ThreadStatus_Lock( &appTSfeedThreadStatus );
            if((appTSfeedThreadStatus.value & APP_THREAD_STATUS_SUSPENDED)) {
                APP_ThreadStatus_Unlock( &appTSfeedThreadStatus );
                FAPEX_SYS_WakeupThread( appTSfeedThreadHandle );
            }
            else {
                APP_ThreadStatus_Unlock( &appTSfeedThreadStatus );
                wait_count += diff_count;
                FAPI_SYS_SLEEP(10 * wait_count);
            }
            continue;
        }
        wait_count = 0;
        APP_ThreadStatus_SetValue( &appTSreadThreadStatus, APP_THREAD_STATUS_STARTED );

        /* read stream file data to chunk buffer */
        read_size = tsdInputFileInfo.chunkBufferSize;
        read_ptr  = tsdInputFileInfo.chunkBuffer;
        if(tsdInputFileInfo.chunkWriteCount & 0x00000001) {
            read_ptr += read_size;
        }

        ret = FAPEX_SYS_FRead( read_ptr, sizeof(uint8_t), read_size,
                               tsdInputFileInfo.filePtr );

        tsdInputFileInfo.chunkWriteCount ++;

        if(ret < read_size) {
            if(ret >= 0) {
                tsdInputFileInfo.lastWriteSize = ret;

                if(ret == 0) {
                    tsdInputFileInfo.chunkWriteCount --;
                }
                else {
                    /* clear rest area */
                    memset((read_ptr + ret), 0, (read_size - ret));
                }
            }
            /* check error type */
            if( FAPEX_SYS_FEof(tsdInputFileInfo.filePtr) == 0) {
                FAPI_SYS_PRINT_MSG("ERROR:[tsd.c] file access failed\n");
                break;
            }
            /* detect EOF */
            APP_ThreadStatus_SetValue( &appTSreadThreadStatus, APP_THREAD_STATUS_OPENED );

//          FAPEX_SYS_FRewind(tsdInputFileInfo.filePtr);
//          tsdInputFileInfo.syncOffset = 0;
            FAPEX_SYS_FSeek(tsdInputFileInfo.filePtr,
                            tsdInputFileInfo.syncOffset, FAPEX_SYS_FSEEK_SET);

            if(appTSinputEnable >= 0) {
                /* wait for request from other thread */
                APP_ThreadStatus_Lock( &appTSreadThreadStatus );
                appTSreadThreadStatus.value |= APP_THREAD_STATUS_SUSPENDED;
                APP_ThreadStatus_Unlock( &appTSreadThreadStatus );

                FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );
            }
        }
    }
    FAPI_SYS_PRINT_DEBUG(1, "appTSreadThreadFunction() end\n");

    APP_ThreadStatus_SetValue( &appTSreadThreadStatus, APP_THREAD_STATUS_DISABLE );
}

/*!
********************************************************************************
** \brief memory copy for little endian data
********************************************************************************/
static void tsdInputPacketForLittleEndian(void *dst, const void *src, int32_t n)
{
    uint32_t  *dst_ptr, *src_ptr;
    uint32_t  swap_data;
    int32_t   k, size;

    src_ptr = (uint32_t *)src;

    /* filter stuffing packet */
    if(((*src_ptr) & 0x00FF1F00) != 0x00FF1F00) {
        dst_ptr = (uint32_t *)dst;
        size    = n >> 2;
        for(k=0; k<size; k++) {
            swap_data = *src_ptr;
            *dst_ptr = ( swap_data               << 24) |
                       ((swap_data & 0x0000FF00) <<  8) |
                       ((swap_data & 0x00FF0000) >>  8) |
                       ( swap_data               >> 24);
            src_ptr ++;
            dst_ptr ++;
        }
    }
}

/*!
********************************************************************************
** \brief memory copy for big endian data
********************************************************************************/
static void tsdInputPacketForBigEndian(void *dst, const void *src, int32_t n)
{
    uint32_t  *src_ptr;

    src_ptr = (uint32_t *)src;

    /* filter stuffing packet */
    if(((*src_ptr) & 0x001FFF00) != 0x001FFF00) {
        memcpy(dst, src, n);
    }
}

/*!
********************************************************************************
** \brief feed packet data to input buffer
********************************************************************************/
static int32_t tsdFeedPacketData(TSD_InputFileInfo *fileInfo)
{
    FAPEX_TSD_PacketInInfoStrT  input_status;
    int32_t   out_buff_level;
    int32_t   audio_fill_level, video_fill_level;
    uint32_t  busy_flag;
    int32_t   chunk_cnt  = 0;
    uint32_t  wait_count = 0;
    uint8_t   *chunk_buff;
    uint8_t   *input_buff;
    uint32_t  *src_ptr;
    int32_t   i, result;

    chunk_buff = fileInfo->chunkBuffer;
    if(fileInfo->chunkReadCount & 0x00000001) {
        chunk_buff += fileInfo->chunkBufferSize;
    }

    /* check chunk buffer endian */
    if(fileInfo->inputFunc == NULL) {
        src_ptr = (uint32_t *)chunk_buff;
        if((*src_ptr & 0x000000FF) == 0x00000047) {
            FAPI_SYS_PRINT_DEBUG(3, "little endian data\n");
            fileInfo->inputFunc = tsdInputPacketForLittleEndian;
        }
        else {
            FAPI_SYS_PRINT_DEBUG(3, "big endian data\n");
            fileInfo->inputFunc = tsdInputPacketForBigEndian;
        }
    }

    /* feed chunk buffer data to TS input ports */
    wait_count = 0;
    while(chunk_cnt < APP_TSD_CHUNK_PACKET_NUM) {
        if(appTSinputEnable <= 0) {
            break;
        }
        out_buff_level = 0;

        /* check Audio buffer status */
        if(fileInfo->audioBuffSize <= 0) {
            fileInfo->audioBuffSize = AUDIO_GetBufferSize();
        }
        if(fileInfo->audioBuffSize > 0) {
            audio_fill_level = AUDIO_GetBufferLevel();
            if(audio_fill_level > 0) {
                if(audio_fill_level > (fileInfo->audioBuffSize >> 1)) {
                    out_buff_level ++;
                }
                else if(audio_fill_level < (fileInfo->audioBuffSize >> 4)) {
                    out_buff_level -= 4;
                }
                else if(audio_fill_level < (fileInfo->audioBuffSize >> 3)) {
                    out_buff_level -= 2;
                }
                else if(audio_fill_level < (fileInfo->audioBuffSize >> 2)) {
                    out_buff_level -= 1;
                }
            }
        }

        /* check Video buffer status */
        if(fileInfo->videoBuffSize <= 0) {
            VAL_GetBufferSize(vidOpenPtr, (uint32_t*)&fileInfo->videoBuffSize);
        }
        if(fileInfo->videoBuffSize > 0) {
            result = VAL_GetBufferLevel(vidOpenPtr, (uint32_t*)&video_fill_level);
            if((result == 0) && (video_fill_level > 0)) {
                if(video_fill_level > (fileInfo->videoBuffSize >> 1)) {
                    out_buff_level ++;
                }
                else if(video_fill_level < (fileInfo->videoBuffSize >> 4)) {
                    out_buff_level -= 4;
                }
                else if(video_fill_level < (fileInfo->videoBuffSize >> 3)) {
                    out_buff_level -= 2;
                }
                else if(video_fill_level < (fileInfo->videoBuffSize >> 2)) {
                    out_buff_level -= 1;
                }
            }
        }

        /* check Audio/Video buffer status */
        if(out_buff_level > 0) {
            wait_count += out_buff_level;
            FAPI_SYS_SLEEP(10 * wait_count);
            continue;
        }
        wait_count = 0;

        /* check input buffer status */
        result = FAPEX_TSD_Ioctl( fd_TSD[0],
                                  FAPEX_IOCCOM_TSD_GET_INPUT_PACKETINFO,
                                  (FAPEXIOCARG3_T)&input_status );
        if(result < 0) {
            FAPI_SYS_PRINT_ERR("[TSD] Error: get TSD input status failed %d.\n",
                               result);
            return result;
        }
        busy_flag = input_status.inputStatus;

        if(busy_flag != FAPEX_TSD_PACKET_INPUT_STATUS_BUSY_BUFF_ALL) {
            input_buff = fileInfo->inputPortBase;
            for(i=0; i<FAPEX_TSD_PACKET_INPUT_BUFF_NUM; i++) {
                /* check non-busy buffer */
                if(!(busy_flag & tsd_InputBuffBusyStatus[i])) {
                    /* input TS packet data */
                    fileInfo->inputFunc(input_buff, chunk_buff,
                                        FAPEX_TSD_PACKET_INPUT_BYTES);
                    chunk_buff += fileInfo->packetSize;
                    chunk_cnt  ++;
                }
                input_buff += fileInfo->inputPortOffset;
            }
        }
    }
    return 0;
}

/*!
********************************************************************************
** \brief TS packet data feeding thread
********************************************************************************/
static void appTSfeedThreadFunction(void *optArg)
{
    FAPEX_TSD_PacketInMapInfoStrT  input_map_info;
    uint8_t   *input_map;
    int32_t   diff_count;
    uint32_t  wait_count = 0;
    int32_t   result;

    if(tsdInputFileInfo.filePtr == NULL) {
        return;
    }
    FAPI_SYS_PRINT_DEBUG(1, "appTSfeedThreadFunction() start\n");

    APP_ThreadStatus_SetValue( &appTSfeedThreadStatus,
                               (APP_THREAD_STATUS_OPENED | APP_THREAD_STATUS_SUSPENDED) );
    // wait start trigger
    FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );

    /* get TSD input buffer mapping information */
    result = FAPEX_TSD_Ioctl( fd_TSD[0],
                              FAPEX_IOCCOM_TSD_GET_INPUT_MAP_INFO,
                              (FAPEXIOCARG3_T)&input_map_info );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("[TSD] Error: get TSD input buffer mmapping info failed %d.\n",
                           result);
        return;
    }

    /* get TSD input buffer mapping address */
    input_map = (uint8_t*)FAPEX_TSD_Mmap( NULL, input_map_info.mapSize,
                                          (PROT_READ | PROT_WRITE), MAP_SHARED,
                                          fd_TSD[0], 0 );
    if(input_map == (void *)-1) {
        result = -1;
        FAPI_SYS_PRINT_ERR("[TSD] Error: TSD input buffer mmap failed %d.\n", result);
        return;
    }
    tsdInputFileInfo.inputPortBase   = input_map + input_map_info.mapOffset;
    tsdInputFileInfo.inputPortOffset = input_map_info.packetOffset;

    /* feed TS packet data */
    while(appTSinputEnable >= 0) {
        APP_ThreadStatus_SetValue( &appTSfeedThreadStatus, APP_THREAD_STATUS_READY );

        if(appTSinputEnable == 0) {
            APP_ThreadStatus_Lock( &appTSfeedThreadStatus );
            appTSfeedThreadStatus.value |= APP_THREAD_STATUS_SUSPENDED;
            APP_ThreadStatus_Unlock( &appTSfeedThreadStatus );

            FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );
            continue;
        }

        /* check bufferring status */
        diff_count = tsdInputFileInfo.chunkWriteCount - tsdInputFileInfo.chunkReadCount;
        if(diff_count == 0) {
            APP_ThreadStatus_Lock( &appTSreadThreadStatus );
            if(appTSreadThreadStatus.value < APP_THREAD_STATUS_CREATED) {
                APP_ThreadStatus_Unlock( &appTSreadThreadStatus );

                /* read error or thread destory status */
                break;
            }
            else if((appTSreadThreadStatus.value & ~APP_THREAD_STATUS_SUSPENDED) == APP_THREAD_STATUS_OPENED) {
                APP_ThreadStatus_Unlock( &appTSreadThreadStatus );

                /* detect EOF */
                FAPI_SYS_PRINT_MSG("[TSD] detect EOF\n");

                APP_ThreadStatus_SetValue( &appTSfeedThreadStatus,
                                           (APP_THREAD_STATUS_OPENED | APP_THREAD_STATUS_SUSPENDED) );

                FAPEX_SYS_SuspendThread( FAPEX_SYS_GetThread() );

                APP_ThreadStatus_SetValue( &appTSfeedThreadStatus, APP_THREAD_STATUS_OPENED );

                wait_count = 0;
                if(appTSinputEnable <= 0) {
                    continue;
                }

                /* reset counter */
                tsdInputFileInfo.chunkWriteCount = 0;
                tsdInputFileInfo.chunkReadCount  = 0;

                /* restart read thread */
                FAPEX_SYS_WakeupThread( appTSreadThreadHandle );
            }
            else if(appTSreadThreadStatus.value >= APP_THREAD_STATUS_READY) {
                APP_ThreadStatus_Unlock( &appTSreadThreadStatus );

                /* to read next data */
                FAPEX_SYS_WakeupThread( appTSreadThreadHandle );
            }
            else {
                APP_ThreadStatus_Unlock( &appTSreadThreadStatus );
            }
            wait_count ++;
            FAPI_SYS_SLEEP(10 * wait_count);
            continue;
        }
        else if(diff_count < 2) {
            APP_ThreadStatus_Lock( &appTSreadThreadStatus );
            if(appTSreadThreadStatus.value >= APP_THREAD_STATUS_READY) {
                APP_ThreadStatus_Unlock( &appTSreadThreadStatus );

                /* to read next data */
                FAPEX_SYS_WakeupThread( appTSreadThreadHandle );
            }
            else {
                APP_ThreadStatus_Unlock( &appTSreadThreadStatus );
            }
        }
        wait_count = 0;
        APP_ThreadStatus_SetValue( &appTSfeedThreadStatus, APP_THREAD_STATUS_STARTED );

        /* feed stream data */
        result = tsdFeedPacketData(&tsdInputFileInfo);
        if(result < 0) {
            break;
        }
        tsdInputFileInfo.chunkReadCount ++;
    }
    APP_ThreadStatus_SetValue( &appTSfeedThreadStatus, APP_THREAD_STATUS_OPENED );

    FAPEX_TSD_Munmap(input_map, input_map_info.mapSize);

    tsdInputFileInfo.inputPortBase = NULL;

    FAPI_SYS_PRINT_DEBUG(1, "appTSfeedThreadFunction() end %02x\n", appTSreadThreadStatus.value);

    APP_ThreadStatus_SetValue( &appTSfeedThreadStatus, APP_THREAD_STATUS_DISABLE );
}

/*!
********************************************************************************
** \brief setup TSD input file
********************************************************************************/
static int32_t tsdSetupInputFile(const char_t *fileName, int32_t startOffset,
                                 FAPEX_TSD_InputParamStrT *inputParams)
{
    FAPEX_SYS_FFILE  *file_ptr;
    int32_t   sync_point, sync_point_next, max_point;
    uint32_t  packet_size = 0;
    uint32_t  chunk_buff_size;
    uint8_t   data;
    int32_t   result;

    // Open TS input file
    file_ptr = FAPEX_SYS_FOpen(fileName, "rb");
    if(file_ptr == NULL)
    {
        FAPI_SYS_PRINT_MSG( "ERROR:[tsd.c] open file \"%s\" failed\n",
                            fileName );
        return -1;
    }

    // seek offset
    result = FAPEX_SYS_FSeek(file_ptr, startOffset, FAPEX_SYS_FSEEK_SET);
    if(result < 0) {
        return -1;
    }

    // check the distance between SYNC(0x47) bytes
    sync_point = startOffset;
    while(1) {
        /************************/
        /* search 1st sync_byte */
        /***********************/
        max_point = startOffset + FAPEX_TSD_PACKET_INPUT_LENGTH_B;
        while( sync_point < max_point ) {
            result = FAPEX_SYS_FRead(&data, sizeof(uint8_t), 1, file_ptr);
            if(result <= 0) {
                return -1;
            }
            if(data == 0x47) {
                break;
            }
            sync_point ++;
        }
        if(sync_point >= max_point) {
            FAPI_SYS_PRINT_MSG( "ERROR:[tsd.c] TS-packet synchronization error" );
            return -1;
        }
        packet_size     = FAPEX_TSD_PACKET_INPUT_LENGTH;
        max_point       = sync_point + FAPEX_TSD_PACKET_INPUT_LENGTH_B;
        sync_point_next = sync_point + packet_size;

        /************************/
        /* search 2nd sync_byte */
        /***********************/
        result = FAPEX_SYS_FSeek(file_ptr, (packet_size - 1), FAPEX_SYS_FSEEK_CUR);
        if(result < 0) {
            break;
        }
        result = FAPEX_SYS_FRead(&data, sizeof(uint8_t), 1, file_ptr);
        if(result <= 0) {
            break;
        }

        if(data != 0x47) {
            /* check 192,204 bytes packet */
            while( sync_point_next < max_point ) {
                packet_size ++;
                result = FAPEX_SYS_FRead(&data, sizeof(uint8_t), 1, file_ptr);
                if(result <= 0) {
                    goto tsdSetupInputFile_SetSeekPoint;
                }
                if(data == 0x47) {
                    break;
                }
                sync_point_next ++;
                if(packet_size > FAPEX_TSD_PACKET_INPUT_LENGTH_B) {
                    break;
                }
            }
            if( (packet_size != FAPEX_TSD_PACKET_INPUT_LENGTH_A)
             && (packet_size != FAPEX_TSD_PACKET_INPUT_LENGTH_B) ) {
                if(sync_point_next > max_point) {
                    break;
                }
                else {
                    /* reset */
                    sync_point ++;
                    FAPEX_SYS_FSeek(file_ptr, sync_point, FAPEX_SYS_FSEEK_SET);
                    continue;
                }
            }
        }
        /***********************/
        /* check 3rd sync_byte */
        /***********************/
        result = FAPEX_SYS_FSeek(file_ptr, (packet_size - 1), FAPEX_SYS_FSEEK_CUR);
        if(result < 0) {
            break;
        }
        result = FAPEX_SYS_FRead(&data, sizeof(uint8_t), 1, file_ptr);
        if(result <= 0) {
            break;
        }

        if(data == 0x47) {
            break;
        }
        sync_point ++;
    }

tsdSetupInputFile_SetSeekPoint:
    /* seek to 1st sync_byte position */
    FAPEX_SYS_FSeek(file_ptr, sync_point, FAPEX_SYS_FSEEK_SET);

    FAPI_SYS_PRINT_DEBUG(2, "seek point  : %d\n", sync_point);
    FAPI_SYS_PRINT_DEBUG(2, "packet size : %d\n", packet_size);

    inputParams->syncByteDistance = packet_size;

    memset(&tsdInputFileInfo, 0, sizeof(TSD_InputFileInfo));

    tsdInputFileInfo.filePtr    = file_ptr;
    tsdInputFileInfo.syncOffset = sync_point;
    tsdInputFileInfo.packetSize = packet_size;

    chunk_buff_size = APP_TSD_CHUNK_PACKET_NUM * packet_size;

    tsdInputFileInfo.chunkBuffer = (uint8_t *)FAPI_SYS_MALLOC(chunk_buff_size * 2);
    if(tsdInputFileInfo.chunkBuffer == NULL) {
        FAPI_SYS_PRINT_MSG( "ERROR:[tsd.c] cannot allocate chunk buffer\n" );
        return -1;
    }
    tsdInputFileInfo.chunkBufferSize = chunk_buff_size;

    appTSinputEnable = 1;

    // Open TS read thread
    APP_ThreadStatus_Init( &appTSreadThreadStatus );
    appTSreadThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_TSD_READ_THREAD_STACKSIZE,
            APP_TSD_READ_THREAD_PRIORITY,
            appTSreadThreadFunction,
            0, 0, APP_TSD_READ_THREAD_NAME );
    if( appTSreadThreadFunction == NULL )
    {
        APP_ThreadStatus_Exit( &appTSreadThreadStatus );
        FAPI_SYS_PRINT_MSG( "ERROR:[tsd.c] creating thread \"%s\" failed\n",
                            APP_TSD_READ_THREAD_NAME );
        return -1;
    }

    // Open TS packet feeding thread
    APP_ThreadStatus_Init( &appTSfeedThreadStatus );
    appTSfeedThreadHandle = FAPEX_SYS_CreateThread(
            NULL,
            APP_TSD_INPUT_THREAD_STACKSIZE,
            APP_TSD_INPUT_THREAD_PRIORITY,
            appTSfeedThreadFunction,
            0, 0, APP_TSD_INPUT_THREAD_NAME );
    if( appTSfeedThreadHandle == NULL )
    {
        APP_ThreadStatus_Exit( &appTSfeedThreadStatus );
        FAPI_SYS_PRINT_MSG( "ERROR:[tsd.c] creating thread \"%s\" failed\n",
                            APP_TSD_INPUT_THREAD_NAME );
        return -1;
    }

    return 0;
}

/*!
********************************************************************************
** \brief TS packet data feeding thread
********************************************************************************/
static int32_t tsdCloseThreads(void)
{
    if(appTSinputEnable < 0) {
        return 0;
    }

    APP_ThreadStatus_Lock( &appTSfeedThreadStatus );

    if(appTSfeedThreadStatus.value >= APP_THREAD_STATUS_CREATED) {
        APP_ThreadStatus_Unlock( &appTSfeedThreadStatus );

        if(appTSinputEnable >= 0) {
            int32_t  closed_cnt;

            // Close TS input thread
            appTSinputEnable = -1;

            while(1) {
                closed_cnt = 0;
                APP_ThreadStatus_Lock( &appTSreadThreadStatus );
                if( (appTSreadThreadStatus.value >= APP_THREAD_STATUS_CREATED)
                 && (appTSreadThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) ) {
                    APP_ThreadStatus_Unlock( &appTSreadThreadStatus );
                    FAPEX_SYS_WakeupThread( appTSreadThreadHandle );
                }
                else {
                    APP_ThreadStatus_Unlock( &appTSreadThreadStatus );
                    closed_cnt ++;
                }

                APP_ThreadStatus_Lock( &appTSfeedThreadStatus );
                if(appTSfeedThreadStatus.value >= APP_THREAD_STATUS_CREATED) {
                    if(appTSfeedThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                        APP_ThreadStatus_Unlock( &appTSfeedThreadStatus );

                        FAPEX_SYS_WakeupThread( appTSfeedThreadHandle );
                    }
                    else {
                        APP_ThreadStatus_Unlock( &appTSfeedThreadStatus );
                    }
                }
                else {
                    APP_ThreadStatus_Unlock( &appTSfeedThreadStatus );
                    closed_cnt ++;
                }
                if(closed_cnt >= 2) {
                    break;
                }
                FAPI_SYS_SLEEP(10);
            }
            FAPEX_SYS_WaitEndThread( appTSreadThreadHandle );
            FAPEX_SYS_WaitEndThread( appTSfeedThreadHandle );

            APP_ThreadStatus_Exit( &appTSreadThreadStatus );
            APP_ThreadStatus_Exit( &appTSfeedThreadStatus );
            appTSreadThreadHandle = NULL;
            appTSfeedThreadHandle = NULL;
        }
    }
    else {
        APP_ThreadStatus_Unlock( &appTSfeedThreadStatus );
        FAPI_SYS_PRINT_MSG( "ERROR:[tsd.c] cannot close threads\n" );
        return -1;
    }

    if(tsdInputFileInfo.filePtr != NULL) {
        // Close input file
        FAPEX_SYS_FClose( tsdInputFileInfo.filePtr );
        tsdInputFileInfo.filePtr = NULL;
        if(tsdInputFileInfo.chunkBuffer != NULL) {
            FAPI_SYS_FREE(tsdInputFileInfo.chunkBuffer);
            tsdInputFileInfo.chunkBuffer = NULL;
        }
    }
    return 0;
}

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
*******************************************************************************
** \brief Open TSD.
**
** This function opens the TSD block.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************/
int32_t TSD_Open(void)
{
    FAPEX_TSD_OpenParamStrT  tsdOpenParam;

    /* input settings */
    tsdOpenParam.useStc           = 1;
    tsdOpenParam.tsSelect         = FAPEX_TSD_INPUT_CH_A;
    tsdOpenParam.syncByteDistance = 0;
    tsdOpenParam.iflags           = FAPEX_TSD_IFLAGS_SP_MODE3;
    tsdOpenParam.oflags           = 0;

    fd_TSD[0] = FAPEX_TSD_Open( DRV_ID_TSD0,
                                O_RDWR,
                                &tsdOpenParam );
    if(fd_TSD[0] < 0) {
        FAPI_SYS_PRINT_ERR("FAPEX_TSD_Open() : error %d\n", fd_TSD[0]);
        return -1;
    }

    /* input settings */
    tsdOpenParam.useStc = 0;

    fd_TSD[1] = FAPEX_TSD_Open( DRV_ID_TSD1,
                                O_RDWR,
                                &tsdOpenParam );
    if(fd_TSD[1] < 0) {
        FAPI_SYS_PRINT_ERR("FAPEX_TSD_Open() : error %d\n", fd_TSD[1]);
        return -1;
    }
    return 0;
}

/*!
*******************************************************************************
** \brief Close TSD.
**
** This function closes the TSD block.
**
** \return #FAPI_OK if sucessful, otherwise error code.
*******************************************************************************
*/
int32_t TSD_Close(void)
{
    int32_t  result = 0;

    if(fd_TSD[0] > 0) {
        result = tsdCloseThreads();
        if(result < 0) {
            return result;
        }

        result = FAPEX_TSD_Close( fd_TSD[0] );
        fd_TSD[0] = -1;
    }

    if(fd_TSD[1] > 0) {
        result = FAPEX_TSD_Close( fd_TSD[1] );
        fd_TSD[1] = -1;
    }

    return result;
}

/*!
*******************************************************************************
** \brief Start TSD.
*******************************************************************************/
int32_t TSD_Start(FAPEX_TSD_InputParamStrT *inputParams, const char_t *fileName,
                  int32_t startOffset)
{
    int32_t  opened_cnt;
    int32_t  result;

    result = tsdCloseThreads();
    if(result < 0) {
        return result;
    }

    /* check TS file input mode */
    if((inputParams->tsSelect > FAPEX_TSD_INPUT_CH_C) && (fileName != NULL)) {
        result = tsdSetupInputFile(fileName, startOffset, inputParams);
        if(result < 0) {
            return result;
        }
    }
    else {
        inputParams->syncByteDistance = 0;
    }

    result = FAPEX_TSD_Ioctl( fd_TSD[0],
                              FAPEX_IOCCOM_TSD_SET_INPUT,
                              (FAPEXIOCARG3_T)inputParams );
    if(result < 0) {
        FAPI_SYS_PRINT_ERR("ERROR:[tsd.c] set input error %d\n",
                            result );
        return result;
    }

    if(inputParams->tsSelect <= FAPEX_TSD_INPUT_CH_C) {
        return 0;
    }

    // check thread status
    while(1) {
        opened_cnt = 0;
        APP_ThreadStatus_Lock( &appTSreadThreadStatus );
        if(appTSreadThreadStatus.value >= APP_THREAD_STATUS_OPENED) {
            opened_cnt ++;
        }
        APP_ThreadStatus_Unlock( &appTSreadThreadStatus );

        APP_ThreadStatus_Lock( &appTSfeedThreadStatus );
        if(appTSfeedThreadStatus.value >= APP_THREAD_STATUS_OPENED) {
            opened_cnt ++;
        }
        APP_ThreadStatus_Unlock( &appTSfeedThreadStatus );

        if(opened_cnt >= 2) {
            break;
        }
        FAPI_SYS_SLEEP(10);
    }

    // Start TS input thread
    appTSinputEnable = 1;
    FAPEX_SYS_WakeupThread( appTSreadThreadHandle );
    FAPEX_SYS_WakeupThread( appTSfeedThreadHandle );

    return 0;
}

/*!
*******************************************************************************
** \brief Stop TSD.
*******************************************************************************/
int32_t TSD_Stop(void)
{
    int32_t  stoped_cnt;

    if(appTSinputEnable < 0) {
        return 0;
    }

    APP_ThreadStatus_Lock( &appTSfeedThreadStatus );

    if(appTSfeedThreadStatus.value >= APP_THREAD_STATUS_CREATED) {
        APP_ThreadStatus_Unlock( &appTSfeedThreadStatus );

        // Stop TS input thread
        appTSinputEnable = 0;

        while(1) {
            stoped_cnt = 0;
            APP_ThreadStatus_Lock( &appTSreadThreadStatus );
            if(appTSreadThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                stoped_cnt ++;
            }
            APP_ThreadStatus_Unlock( &appTSreadThreadStatus );

            APP_ThreadStatus_Lock( &appTSfeedThreadStatus );
            if(appTSfeedThreadStatus.value & APP_THREAD_STATUS_SUSPENDED) {
                stoped_cnt ++;
            }
            APP_ThreadStatus_Unlock( &appTSfeedThreadStatus );

            if(stoped_cnt >= 2) {
                break;
            }
            FAPI_SYS_SLEEP(10);
        }

        if(tsdInputFileInfo.filePtr != NULL) {
            FAPEX_SYS_FSeek(tsdInputFileInfo.filePtr,
                            tsdInputFileInfo.syncOffset, FAPEX_SYS_FSEEK_SET);
        }

        /* reset counter */
        tsdInputFileInfo.chunkWriteCount = 0;
        tsdInputFileInfo.chunkReadCount  = 0;
    }
    else {
        APP_ThreadStatus_Unlock( &appTSfeedThreadStatus );
    }
    return 0;
}

/*!
*******************************************************************************
** \brief Pause TSD.
*******************************************************************************/
int32_t TSD_Pause(void)
{
    if(appTSinputEnable < 0) {
        return 0;
    }
    appTSinputEnable = 0;
    return 0;
}

/*!
*******************************************************************************
** \brief Restart TSD.
*******************************************************************************/
int32_t TSD_Restart(void)
{
    if(appTSinputEnable < 0) {
        return 0;
    }
    // Restart TS input thread
    appTSinputEnable = 1;
    FAPEX_SYS_WakeupThread( appTSreadThreadHandle );
    FAPEX_SYS_WakeupThread( appTSfeedThreadHandle );
    return 0;
}

/*!
********************************************************************************
** \brief
********************************************************************************/
uint64_t TSD_GetFileReadPosition(void)
{
    uint64_t  read_size;

    read_size = (uint64_t)tsdInputFileInfo.chunkBufferSize
              * (uint64_t)tsdInputFileInfo.chunkReadCount;

    if( (tsdInputFileInfo.lastWriteSize > 0)
     && (tsdInputFileInfo.chunkWriteCount == tsdInputFileInfo.chunkReadCount) ) {
        read_size += tsdInputFileInfo.lastWriteSize;
    }
    return read_size;
}

/*!
********************************************************************************
** \brief
********************************************************************************/
uint64_t TSD_GetTimeStamp(void)
{
    uint64_t stc_value;
    int32_t  result;

    if(fd_TSD[0] > 0) {
        result = FAPEX_TSD_Ioctl( fd_TSD[0],
                                  FAPEX_IOCCOM_TSD_GET_STC_VALUE,
                                  (FAPEXIOCARG3_T)&stc_value );
        if(result < 0) {
            stc_value = (uint64_t)-1;
        }
    }
    else {
        stc_value = (uint64_t)-1;
    }
    return stc_value;
}
