/******************************************************************************
**
** \file      ./subsystems/fe/ce6353/src/ce6353.c
**
** \brief     Front-end device driver for Intel OFDM CE6353 with Thomson Tuner.
**
**            (C) Fujitsu Semiconductor Europe 2002 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: ce6353.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
// #define CE6353_DEBUG
// #define CE6353_DEBUG_2 /* Tuner debug information */
// #define CE6353_DEBUG_3 /* Get status debug information */

/* Thomson DTT75404 and Philips TD1316 tuners are supported by default */

#include "../src/fe_priv.h"
#include "ce6353.h"
#include "ce6353_priv.h"

/* tuner includes */
#include "dtt75404.h"
#include "td911.h"
#include "td1316.h"


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/* initialize for 20.48MHz oscillator and 36MHz IFC */
#define XTAL_FREQ 4
#define SCAN_STEP       500000          /* step width for blind scan in Hz */
/* defines for timer flags */
#define TIM_SYN_TIM 1
#define TIM_RES_TIM 2
#define TIM_GET_TIM 4
/* defines the timer resolution in ms, needed for rounding */
#define TIM_RESOLUTION 1

/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static FE_OFDM_SYSTEM_PARAMS_S FdFeCe6353Sparams[FE_MAX_NUM];

static uint8_t CE6353ResetFull[]      = {CE6353_RESET,
                                    CE6353_RESET_FULL
                                   };

/* secondary I2C connections of Intel CE6353 are directly connected to   */
/* primary I2C bus. External uP has direct communication with Thomson    */
/* Tuner. Intel's internal tune functionality is not used here.          */
/* After a communication towards the Tuner the secondary I2C bus is      */
/* disconnected again to keep off I2C traffic from the tuner.            */

static uint8_t ce6353ConnectTuner[]    = {CE6353_SCANCTL, 0x1A };
static uint8_t ce6353DisconnectTuner[] = {CE6353_SCANCTL, 0x0A };

/* According to Intel's Design Manual page 25 the ADC control register   */
/* is set to calibrate the ADC.                                          */
static uint8_t CE6353ADC_CTL_0[]      = {CE6353_ADCCTL0, 0x00 };
static uint8_t CE6353ADC_CTL_1[]      = {CE6353_ADCCTL0, 0x01 };
                                   
/* to power down the demodulator in case of stdby status                 */
static uint8_t CE6353PowerDown[]     = {CE6353_ENABLE,
                                    CE6353_ENABLE_DVRPORTDIS|CE6353_ENABLE_MPEGDIS
                                   };

/* increase the captureRange */
static uint8_t CE6353CaptureRange[]  = {CE6353_CAPTURE,   0x11 };

/* disable Automatic Frequency Control (AFC) */
static uint8_t CE6353DisableAFC[]    = {CE6353_AFCCTRL,   0x28 };

/* undocumented register settings. Refer to paragraph 7, page 39 ff of   */
/* Intel's Design Manual.                                                */ 
static uint8_t CE6353TriggerSweep[]  = {CE6353_TRSWEEPEN, 0xFD };
static uint8_t CE6353ImpulseCtrl[]   = {CE6353_IMPCTRL,   0xA1 };

static feHandleT* pData;
static FE_TUNER_INIT_FCT_T     tempInitFct = NULL;
static FE_TUNER_INIT_FCT_T     pInitFct[FE_MAX_NUM];
static FE_TUNER_OPEN_FCT_T     tempTuneFct = NULL;
static FE_TUNER_OPEN_FCT_T     pTuneFct[FE_MAX_NUM];
static FE_TUNER_CLOSE_FCT_T    tempCloseFct = NULL;
static FE_TUNER_CLOSE_FCT_T    pCloseFct[FE_MAX_NUM];
static uint32_t                tempIfFreq = 0;
static uint32_t                pIfFreq[FE_MAX_NUM];
static uint32_t                tempXtalFreq = 0;
static uint32_t                pXtalFreq[FE_MAX_NUM];
static uint32_t                startTimes[FE_MAX_NUM];

/*---------------------------------------------------------------------------*/
/* callback functions                                                        */
/*---------------------------------------------------------------------------*/
int32_t CE6353_Close(uint8_t index);
int32_t CE6353_Reset(uint8_t index);
int32_t CE6353_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams);
int32_t CE6353_Scan(uint8_t index, FE_SCAN_PARAMS_S* pScanParams);
int32_t CE6353_GetStatus(uint8_t index, FE_STATUS_E* pStatus);
int32_t CE6353_GetAFC(uint8_t index, int32_t* pAFC);
int32_t CE6353_GetBER(uint8_t index, uint32_t* pBER);
int32_t CE6353_GetFEC(uint8_t index, FE_FEC_E* pFEC);
int32_t CE6353_GetSignalQuality(uint8_t index, uint16_t* pQuality);
int32_t CE6353_GetSignalStrength(uint8_t index, uint16_t* pStrength);
int32_t CE6353_GetTuneParams(uint8_t index, FE_TUNE_PARAMS_S *pTuneParams);
int32_t CE6353_SetAntennaPower(uint8_t index, uint8_t antPower);
int32_t CE6353_SetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams);

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
static int32_t ce6353Initialize(uint8_t index);
// !?! static int32_t ce6353WaitDemodLock (uint8_t index);
static int32_t ce6353UserTimers(uint8_t ucFlag, uint8_t ucIndex, uint32_t *pulDiffTime);

/*!
*******************************************************************************
**
** \brief  Inits the Intel OFDM CE6353 demodulator
**
** \param  index for frontend data structure 
**
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_FE_OPEN        when Chip ID read does not match 0x14
**         - error values returned by #FE_I2C_Read(), #FE_I2C_Write() and #FE_I2C_SetProtocol()
**
** \sa
**
*******************************************************************************
*/
int32_t ce6353Initialize(uint8_t index)
{
    int32_t ferr = FE_OK;
    uint8_t   i2cBuffer[2];
    uint8_t   CE6353InitialSet[6];

    i2cBuffer[0] = CE6353_CHIPID;
    i2cBuffer[1] = 0xFF;
    
    do
    {
        /* make basic intialisation after hard reset */
        /* The initial 6 values set up the demodulator with following parameters */
        CE6353InitialSet[0]   = CE6353_ENABLE;
        CE6353InitialSet[1]   = 0x08 | CE6353_ENABLE_ADCEN | CE6353_ENABLE_CHIPEN;
        CE6353InitialSet[2]   = CE6353_CLKCTL0_MCLKINVERT | CE6353_CLKCTL0_HIGHSAMPLE;

        if (pXtalFreq[index] == 4000000)
        {
            /* External clock frequency is  4.00 [MHz] */
            /* Internal ADC clock is       45.33 [MHz] */
            /* Internal OFDM clock is      45.33 [MHz] */
            /* Internal FEC clock is       68.00 [MHz] */
            CE6353InitialSet[3]   = 0x22;
            CE6353InitialSet[4]   =   67;
            CE6353InitialSet[5]   =    3;
        }
        else
        {
            /* External clock frequency is 20.48 [MHz] */
            /* Internal ADC clock is       45.06 [MHz] */
            /* Internal OFDM clock is      45.06 [MHz] */
            /* Internal FEC clock is       56.32 [MHz] */
            CE6353InitialSet[3]   = 0x46;
            CE6353InitialSet[4]   = 0x15;
            CE6353InitialSet[5]   = 0x0F;
        }

        FE_DPrintf("[FFE_CE6353]Initial Set\n");
        ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, CE6353InitialSet, 6);
        if (ferr != FE_OK)
            break;
        FE_Wait(10); /* some wait time needed to settle the PLL's and signal generators */

        /* apply soft reset */
        FE_DPrintf("[FFE_CE6353]Soft reset\n");
        ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, CE6353ResetFull, 2);
        if (ferr != FE_OK)
            break;
        FE_Wait(10);

        /* calibrate ADC */
        FE_DPrintf("[FFE_CE6353]ADC1\n");
        ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, CE6353ADC_CTL_1, 2);
        if (ferr != FE_OK)
            break;

        FE_DPrintf("[FFE_CE6353]ADC2\n");
        ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, CE6353ADC_CTL_0, 2);
        if (ferr != FE_OK)
            break;

        /* overwrite some default values of the undocumented registers described in */
        /* Intel's Design Manual, page 39                                           */
        ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, CE6353TriggerSweep, 2);
        if (ferr != FE_OK)
            break;

        ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, CE6353ImpulseCtrl, 2);
        if (ferr != FE_OK)
            break;

        /* set AGC target value according to recommendation in Intel's Design Manual */
        /* page 45 to achieve optimum performance                                    */
        i2cBuffer[0] = CE6353_AGCTARGET;
        i2cBuffer[1] = 0x2B;
        ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, i2cBuffer, 2);
        if (ferr != FE_OK)
            break;

        /* enable ADC for reading RF level */
        i2cBuffer[0] = CE6353_OUTCTL0;
        i2cBuffer[1] = 0xC8;
        ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, i2cBuffer, 2);
        if (ferr != FE_OK)
            break;
        
        /* control antenna power - default power is off */
        ferr = CE6353_SetAntennaPower (index, 0);
        if (ferr != FE_OK)
            break;

        if (pIfFreq[index] == 4570000)
        {
            /* set input frequency like in MaxLinear code */
            i2cBuffer[0] = CE6353_ITBFREQ;
            i2cBuffer[1] = 0x19;
            ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, i2cBuffer, 2);
            if (ferr != FE_OK)
                break;
            i2cBuffer[0] = CE6353_ITBFREQ+1;
            i2cBuffer[1] = 0xF8;
            ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, i2cBuffer, 2);
            if (ferr != FE_OK)
                break;
        }

        FE_Wait(10);
    } while(0);
    return ferr;
}

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/

/*!
*******************************************************************************
**
** \brief  Opens communication and handle to one front end and sets initial
**         values
**
** \param   index                               Index of frontend asked for
** \param   #feHandleT* pDeviceData   pointer to device data
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_RFMOD_DEVICE_NOT_SUPPORTED
**         - #FE_ERR_FE_OPEN
**         - failure codes returned from #FE_I2C_Write()
**         - failure codes returned from #FE_I2C_SetProtocol()
**
** \sa     CE6353_Close()
**
*******************************************************************************
*/
int32_t CE6353_Open(uint8_t index, FE_HANDLE* pHandle)
{
    int32_t    ferr = FE_OK;

    if (index >= FE_MAX_NUM) 
        return FE_ERR_BAD_PARAMETER;

    /* check device data */
    //if (pDeviceData->pSystemParams == NULL) return FE_ERR_BAD_PARAMETER;

    /* store pointer to device data */
    *pHandle = pData = &feHandleArray[index];

    if (tempXtalFreq == 4000000)
    {
        FdFeCe6353Sparams[index].demodADCFreq = 45333;    // 45.33MHz
        FdFeCe6353Sparams[index].demodFECFreq = 68000;    // 68.00 MHz
    }
    else
    {
        FdFeCe6353Sparams[index].demodADCFreq = 45056;    // 45MHz
        FdFeCe6353Sparams[index].demodFECFreq = 56320;    // 56.32 MHz
    }
    FE_DPrintf("[FFE_OFDM] Opening\n");
    FdFeCe6353Sparams[index].feStatus     = FE_STATUS_UNKNOWN;
    if (tempTuneFct != NULL)
    {   /* in this case the open call came from detect function */
        pInitFct[index] = tempInitFct;
        tempInitFct     = NULL; /* reset to NULL for next open call */
        pTuneFct[index] = tempTuneFct;
        tempTuneFct     = NULL; /* reset to NULL for next open call */
        if (tempCloseFct == NULL)
            return FE_ERR_BAD_PARAMETER;
        pCloseFct[index] = tempCloseFct;
        tempCloseFct = NULL; /* reset to NULL for next open call */
        if (tempIfFreq == 0)
            return FE_ERR_BAD_PARAMETER;
        pIfFreq[index]   = tempIfFreq;
        pXtalFreq[index] = tempXtalFreq;
        tempIfFreq      = 0; /* reset to 0 for next open call */
        tempXtalFreq    = 0; /* reset to 0 for next open call */
    }
    else
    {   /* in this case the open call came from direct open call of the application.
           using default settings (Thomson tuner) now */
        pInitFct[index]  = NULL;
        pTuneFct[index]  = dtt75404TuneToFrequency;
        pCloseFct[index] = dtt75404Close;
        pIfFreq[index]   = 36166667;
        pXtalFreq[index] = 20480000;
    }
    FE_DPrintf("[FFE_OFDM] Reset\n");
    
    /* apply a hard reset */
    ferr = CE6353_Reset(index);
    if (ferr != FE_OK) return ferr;

    /* download initial values to the demodulator - has to be done first,*/
    /* because Tuner is yet not visible to the uP.                       */
    FE_DPrintf("[FFE_OFDM] Demod init\n");
    ferr = ce6353Initialize(index);
    if (ferr != FE_OK) return ferr;

    FE_DPrintf("[FFE_OFDM] tuner init\n");
    if (pInitFct[index] != NULL) /* call init function if it exists */
    {
        ferr = pInitFct[index](index);
        if (ferr != FE_OK)
            return ferr;
    }
    FE_DPrintf("[FFE_OFDM] test tuning\n");
    ferr = pTuneFct[index](index, 474000000, 8000000);
    if (ferr != FE_OK)
        return ferr;
    FdFeCe6353Sparams[index].frequency = 474000000;
    FdFeCe6353Sparams[index].bandwidth = FE_BANDWIDTH_8_MHZ;
    FdFeCe6353Sparams[index].feStatus = FE_STATUS_TUNING;
    FE_DPrintf("[FFE_OFDM] Tuner Initialised\n");

    FdFeCe6353Sparams[index].feInitialized = 1;
    
    return FE_OK;
}

/*---------------------------------------------------------------------------*/
/* callback functions                                                        */
/*---------------------------------------------------------------------------*/

/*!
*******************************************************************************
**
** \brief  Closes communication and handle for one front end and sets hardware
**         values for power saving.
**
** \param   index                               Index of frontend asked for
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - failure codes returned from #FE_I2C_Write()
**
** \sa     CE6353_Open()
**
*******************************************************************************
*/
int32_t CE6353_Close(uint8_t index)
{
    int32_t        ferr = FE_OK;
    uint8_t          outpCtrl[2];
    uint8_t          gppCtrl[2];

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    
    outpCtrl[0] = CE6353_OUTCTL0;
    outpCtrl[1] = 0x48;
    gppCtrl[0] = CE6353_GPPCTL;
    gppCtrl[1] = CE6353_GPPCTL_GPP3_OUT | CE6353_GPPCTL_GPP3;

    do
    {
        /* put tuner into Power Down mode */
        pCloseFct[index](index); /* close tuner */

        /* disable OFDM */
        ferr = FE_I2C_Write(pData->i2cId,
                            pData->i2cAddrDemod,
                            CE6353PowerDown,
                            2);
        if(ferr != FE_OK) break;
        /* disable ADC for reading RF level */    
        ferr = FE_I2C_Write(pData->i2cId,
                            pData->i2cAddrDemod,
                            outpCtrl,
                            2);
        if(ferr != FE_OK) break;
        /* switch off antenna power */    
        ferr = FE_I2C_Write(pData->i2cId,
                            pData->i2cAddrDemod,
                            gppCtrl,
                            2);
        if(ferr != FE_OK) break;
        FdFeCe6353Sparams[index].feInitialized = 0;
    } while (0);
    if(ferr != FE_OK)
        FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, ce6353DisconnectTuner, 2);

    return ferr;
}

int32_t CE6353_ConnectToTuner(uint8_t index)
{
    int32_t ferr;
    ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, ce6353ConnectTuner, 2);
    return ferr;
}

int32_t CE6353_DisconnectTuner(uint8_t index)
{
    int32_t ferr;
    ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, ce6353DisconnectTuner, 2);
    return ferr;
}

/*!
*******************************************************************************
**
** \brief  Apply a hard reset
**
** \param   index                               Index of frontend asked for
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**
** \sa
**
*******************************************************************************
*/
int32_t CE6353_Reset(uint8_t index)
{
    int32_t error;

    if (index >= FE_MAX_NUM)
        return FE_ERR_BAD_PARAMETER;

    error = FE_ControlFeReset(index, FE_RESET_PULSE);
    FE_Wait(20);

    return error;
}

/*!
*******************************************************************************
**
** \brief  Sets system parameters
** Currently this function is not populated.
**
** \return Possible return codes:
**         - #FE_OK
**
** \sa
**
*******************************************************************************
*/
int32_t CE6353_SetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pSystemParams == NULL) return FE_ERR_BAD_PARAMETER;
    
    /* copy provided params */
    FdFeCe6353Sparams[index].sysParams.terr.autoBandwidth = pSystemParams->terr.autoBandwidth;
    return FE_OK;
}

/*!
*******************************************************************************
**
** \brief  Tune the frontend to the parameters passed
**
** \param   index                               Index of frontend asked for
** \param   #FE_TUNE_PARAMS_S* pTuneParams   pointer to device data
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_FE_OTHER
**         - #FE_ERR_FE_TIMER
**         - failure codes returned from #FE_I2C_Read(), #FE_I2C_Write()
**
** \sa     CE6353_Scan()
**
*******************************************************************************
*/
int32_t CE6353_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams)
{
    int32_t        ferr = FE_OK;
    uint8_t          bwCtrl[2];
    uint8_t          filterCoeff[3];
    uint8_t          trlNomRate[3];
    uint8_t          tpsGiven[3];
    uint8_t          acqCtrl[2];
    uint8_t          mclkCtrl[2];
    uint8_t          bwCtrl6Mhz[3];
    uint8_t          fsmGo[2];
    uint16_t         tpsTemp;
    uint32_t         baw;
    uint32_t         time;


    bwCtrl[0] = CE6353_BWCTRL;
    filterCoeff[0] = CE6353_COEFF0;
    trlNomRate[0] = CE6353_TRLRATE;
    tpsGiven[0] = CE6353_TPSGIVEN;
    acqCtrl[0] = CE6353_ACQCTL; acqCtrl[1] = 0x40;
    mclkCtrl[0] = CE6353_MCLKCTL;
    /* this is an undocumented register for bandwidth control */
    bwCtrl6Mhz[0] = CE6353_BWCTRL6MHZ;
    bwCtrl6Mhz[1] = 0x20;
    bwCtrl6Mhz[2] = 0x03;
    
    fsmGo[0] = CE6353_FSMGO; fsmGo[1] = 0x1;
    
    if(pTuneParams == NULL) return FE_ERR_BAD_PARAMETER;
    if(index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if(FdFeCe6353Sparams[index].feInitialized == 0) return FE_ERR_FE_OTHER;
    
    FdFeCe6353Sparams[index].feStatus = FE_STATUS_TUNING;
    do
    {
        /* set antenna power on/off according to defined setting */
        ferr = CE6353_SetAntennaPower (index, FdFeCe6353Sparams[index].antPower);
        if (ferr != FE_OK) break;

        /* set bandwidth value that is used for later frequency calculations */
        /* and prepare some other initial values for bandwidth control       */
        switch(pTuneParams->terr.bandwidth)
        {
            case FE_BANDWIDTH_6_MHZ:
                baw = 6000000;
                bwCtrl6Mhz[1] = 0x06;
                bwCtrl6Mhz[2] = 0x07;
                break;
            case FE_BANDWIDTH_7_MHZ:
                baw = 7000000;
                break;
            case FE_BANDWIDTH_8_MHZ:
                baw = 8000000;
                break;
            default:
                baw = 0;
                ferr = FE_ERR_BAD_PARAMETER;
                break;
        } /* end of switch(pTuneParams->terr.bandwidth) */
        if (ferr != FE_OK) break;
        
        // read frequency and set Tuner accordingly

        ferr = pTuneFct[index](index, pTuneParams->terr.frequency, baw);
        if(ferr != FE_OK)
        {
            FE_DPrintf("[CE6353_Tune] Tune error: 0x%08x\n", ferr);
            break;
        }
        FdFeCe6353Sparams[index].frequency = pTuneParams->terr.frequency;
        FdFeCe6353Sparams[index].bandwidth = pTuneParams->terr.bandwidth;
        FdFeCe6353Sparams[index].feStatus = FE_STATUS_TUNING;


        /* Since we have an external clock of 20.48MHz and ADC clock is set to*/
        /* 45.06MHz and FEC clock is set to 56.32 MHz constantly the next two */
        /* clock parameters depending upon required bandwidth need not to be  */
        /* calculated explicitly. Values can be taken from tables given in    */
        /* Intel's design manual on page 52 resp. 70.                         */
         
        /*
        Program the TRL register, according to the bandwidth:
        TRL_nom_rate= 2^16 (64/7) * (ucBW/8) *M / Fadc(MHz)
        = 2^16*64*4000/7/8 * ucBW *M/(4*Fadc(kHz))
        =299593142.86 *ucBW*M/(4*Fadc(kHz))
        where M i2 2 for high sample rate, 1 otherwise
        */  
        /* 
        tempFreq = 299593143; 
        tempFreq *= bw;
        tempFreq /=  FdFeCe6353Sparams[index].demodADCFreq;
        if (FdFeCe6353Sparams[index].demodADCFreq>30000)
        {
            tempFreq *=2;
        }
        tempFreq +=2;
        tempFreq/=4;
        trlNomRate[1] = (uint8_t)((tempFreq >> 8) & 0xFF);
        trlNomRate[2] = (uint8_t)(tempFreq & 0xFF);
        */
        /* program MCLK_CTL to optimize the MPEG data clock
        MCLK_CTL = 128 *8/BW * FECclk(kHz)/61440
        = FEC_Clk/BW /60
        */
        /*
        tempFreq = FdFeCe6353Sparams[index].demodFECFreq;
        tempFreq /= bw;
        tempFreq /=60;
        mclkCtrl[1] = (uint8_t)(tempFreq & 0xFF);
        */

        switch (pTuneParams->terr.bandwidth)
        {
            /* in case of 6MHz bandwidth use own filter coefficients as des-  */
            /* cribed on page 51 of Intel's Design Manual                     */
            case FE_BANDWIDTH_6_MHZ:
                bwCtrl[1] = 0x3C;       /* initialize to write new filter coefficients */
                if (pXtalFreq[index] == 4000000)
                {
                    trlNomRate[1] = 0x4D;   /* refer to Intel's design manual page 52 */
                    trlNomRate[2] = 0x73;
                    mclkCtrl[1] = 0xBD;     /* refer to Intel's design manual page 70 */
                }
                else
                {
                    trlNomRate[1] = 0x4D;   /* refer to Intel's design manual page 52 */
                    trlNomRate[2] = 0xEC;
                    mclkCtrl[1] = 0x9C;     /* refer to Intel's design manual page 70 */
                }
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      bwCtrl, 2);
                /* write new coefficients for 6MHz bandwidth */
                /* step 1 */
                filterCoeff[1] = 0xB4;
                filterCoeff[2] = 0x04;
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      filterCoeff, 3);
                /* step 2 */
                filterCoeff[1] = 0xCD;
                filterCoeff[2] = 0x19;
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      filterCoeff, 3);
                /* step 3 */
                filterCoeff[1] = 0x87;
                filterCoeff[2] = 0x22;
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      filterCoeff, 3);
                /* step 4 */
                filterCoeff[1] = 0xBD;
                filterCoeff[2] = 0x3D;
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      filterCoeff, 3);
                /* step 5 */
                filterCoeff[1] = 0x19;
                filterCoeff[2] = 0x4A;
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      filterCoeff, 3);
                /* step 6*/
                filterCoeff[1] = 0x09;
                filterCoeff[2] = 0x53;
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      filterCoeff, 3);
                /* step 7 */
                filterCoeff[1] = 0xC1;
                filterCoeff[2] = 0x6B;
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      filterCoeff, 3);
                /* step 8 */
                filterCoeff[1] = 0x5C;
                filterCoeff[2] = 0x7A;
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      filterCoeff, 3);
                /* step 9 */
                filterCoeff[1] = 0x8B;
                filterCoeff[2] = 0x83;
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      filterCoeff, 3);
                /* step 10 */
                filterCoeff[1] = 0x37;
                filterCoeff[2] = 0x9B;
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      filterCoeff, 3);
                /* step 11*/
                filterCoeff[1] = 0x6E;
                filterCoeff[2] = 0xAA;
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      filterCoeff, 3);
                /* step 12 */
                filterCoeff[1] = 0xDF;
                filterCoeff[2] = 0xB3;
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      filterCoeff, 3);
                /* step 13 */
                filterCoeff[1] = 0x34;
                filterCoeff[2] = 0xC0;
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      filterCoeff, 3);
                /* step 14 */
                filterCoeff[1] = 0x51;
                filterCoeff[2] = 0xD0;
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      filterCoeff, 3);
                
                bwCtrl[1] = 0x30;       /* now use internal programmed coefficients */
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      bwCtrl, 2);
                break;
            case FE_BANDWIDTH_7_MHZ:
                bwCtrl[1] = 0x35;       /* now use internal programmed coefficients */
                if (pXtalFreq[index] == 4000000)
                {
                    trlNomRate[1] = 0x5A;   /* refer to Intel's design manual page 52 */
                    trlNomRate[2] = 0x5C;
                    mclkCtrl[1] = 0xA2;     /* refer to Intel's design manual page 70 */
                }
                else
                {
                    trlNomRate[1] = 0x5A;   /* refer to Intel's design manual page 52 */
                    trlNomRate[2] = 0xE9;
                    mclkCtrl[1] = 0x86;     /* refer to Intel's design manual page 70 */
                }
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      bwCtrl, 2);
                break;
            case FE_BANDWIDTH_8_MHZ:
                bwCtrl[1] = 0x36;       /* now use internal programmed coefficients */
                if (pXtalFreq[index] == 4000000)
                {
                    trlNomRate[1] = 0x67;   /* refer to Intel's design manual page 52 */
                    trlNomRate[2] = 0x45;
                    mclkCtrl[1] = 0x8E;     /* refer to Intel's design manual page 70 */
                }
                else
                {
                    trlNomRate[1] = 0x67;   /* refer to Intel's design manual page 52 */
                    trlNomRate[2] = 0xE5;
                    mclkCtrl[1] = 0x75;     /* refer to Intel's design manual page 70 */
                }
                ferr = FE_I2C_Write(pData->i2cId,
                                      pData->i2cAddrDemod,
                                      bwCtrl, 2);
               break;
            default:
                ferr = FE_ERR_BAD_PARAMETER;
                break; 
        } /* end of switch (pTuneParams->terr.bandwidth) */
        if (ferr != FE_OK) break;
        
        /* enlarge the frequency capture range */
        ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod,
                            CE6353CaptureRange, 2);
        if (ferr != FE_OK) break;
        
        /* disable Automatic Frequency Control */        
        ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod,
                            CE6353DisableAFC, 2);
        if (ferr != FE_OK) break;
                
        /* set TPS values according to passed parmeters                      */
        /* if a parameter is not set programm the devices default value.     */
        /* Refer to Intel's design manual page 57ff                          */

        /* set HP code rate value */
        switch(pTuneParams->terr.codeRateHP)
        {
            case FE_FEC_NONE:
                tpsTemp = 0x0080;               /* default is HP FEC 2/3 */
                break;
            case FE_FEC_1_2:
                tpsTemp = 0x0000;
                break;
            case FE_FEC_2_3:
                tpsTemp = 0x0080;
                break;            
            case FE_FEC_3_4:
                tpsTemp = 0x0100;
                break;
            case FE_FEC_5_6:
                tpsTemp = 0x0180;
                break;
            case FE_FEC_7_8:
                tpsTemp = 0x0200;
                break;
            case FE_FEC_AUTO:
                tpsTemp = 0x0080;               /* default is HP FEC 2/3 */
                break;
            default:
                tpsTemp = 0;
                ferr = FE_ERR_BAD_PARAMETER;
                break;
        } /* end of switch(pTuneParams->terr.codeRateHP) */
        if (ferr != FE_OK) break;
  
        /* set LP code rate value */
        switch(pTuneParams->terr.codeRateLP)
        {
            case FE_FEC_NONE:
                tpsTemp |= 0x0000;               /* default is LP FEC 1/2 */
                break;
            case FE_FEC_1_2:
                tpsTemp |= 0x0000;
                break;
            case FE_FEC_2_3:
                tpsTemp |= 0x0010;
                break;
            case FE_FEC_3_4:
                tpsTemp |= 0x0020;
                break;
            case FE_FEC_5_6:
                tpsTemp |= 0x0030;
                break;
            case FE_FEC_7_8:
                tpsTemp |= 0x0040;
                break;
            case FE_FEC_AUTO:
                tpsTemp |= 0x0000;               /* default is LP FEC 1/2 */
                break;
            default:
                ferr = FE_ERR_BAD_PARAMETER;
                break;
        } /* end of switch(pTuneParams->terr.codeRateLP) */
        if (ferr != FE_OK) break;

        /* set modulation value */
        switch(pTuneParams->terr.modulation)
        {
            case FE_MOD_NONE:
                tpsTemp |= 0x4000;               /* default is QAM64 */
                break;
            case FE_MOD_QPSK:
                tpsTemp |= 0x0000;
                break;
            case FE_MOD_QAM_16:
                tpsTemp |= 0x2000;
                break;
            case FE_MOD_QAM_64:
                tpsTemp |= 0x4000;
                break;
            case FE_MOD_QAM_32:
            case FE_MOD_QAM_128:
            case FE_MOD_QAM_256:
                ferr = FE_ERR_BAD_PARAMETER;
                break;
            case FE_MOD_AUTO:
                tpsTemp |= 0x4000;               /* default is QAM64 */
                break;
            default:
                ferr = FE_ERR_BAD_PARAMETER;
                break;
        } /* end of switch(pTuneParams->terr.modulation) */
        if (ferr != FE_OK) break;

        /* set transmission value */
        switch(pTuneParams->terr.transmission)
        {
            case FE_TRANS_MODE_NONE:
            case FE_TRANS_MODE_2K:
                tpsTemp |= 0x0000;                      /* default is 2K */
                acqCtrl[1] |= CE6353_ACQCTL_FORCEMODE;  /* disable Force_Mode */ 
                break;
            case FE_TRANS_MODE_8K:
                tpsTemp |= 0x0001;
                acqCtrl[1] |= 0x01;                     /* disable Force_Mode */
                break;
            case FE_TRANS_MODE_AUTO:
                acqCtrl[1] &= ~CE6353_ACQCTL_FORCEMODE; /* enable Force_Mode */
                tpsTemp |= 0x0000;                      /* default is 2K */
                break;
            default:
                ferr = FE_ERR_BAD_PARAMETER;
                break;
        } /* end of switch(pTuneParams->terr.transmission) */
        if (ferr != FE_OK) break;

        /* set guard interval value */
        switch(pTuneParams->terr.guardInterval)
        {
            case FE_GUARD_INTERVAL_NONE:
                tpsTemp |= 0x0000;                      /* default is 1/32 */
                acqCtrl[1] |= CE6353_ACQCTL_FORCEGUARD; /* disable Force_Guard */
                break;
            case FE_GUARD_INTERVAL_1_32:
                tpsTemp |= 0x0000;
                acqCtrl[1] |= CE6353_ACQCTL_FORCEGUARD; /* disable Force_Guard */
                break;
            case FE_GUARD_INTERVAL_1_16:
                tpsTemp |= 0x0004;
                acqCtrl[1] |= CE6353_ACQCTL_FORCEGUARD; /* disable Force_Guard */
                break;
            case FE_GUARD_INTERVAL_1_8:
                tpsTemp |= 0x0008;
                acqCtrl[1] |= CE6353_ACQCTL_FORCEGUARD; /* disable Force_Guard */
                break;
            case FE_GUARD_INTERVAL_1_4:
                tpsTemp |= 0x000C;
                acqCtrl[1] |= CE6353_ACQCTL_FORCEGUARD; /* disable Force_Guard */
                break;
            case FE_GUARD_INTERVAL_AUTO:
                acqCtrl[1] &= ~CE6353_ACQCTL_FORCEGUARD;/* enable Force_Guard */
                tpsTemp |= 0x0000;                      /* default is 1/32 */
                break;
            default:
                ferr = FE_ERR_BAD_PARAMETER;
                break;
        } /* end of switch(pTuneParams->terr.guardInterval) */
        if (ferr != FE_OK) break;

        /* set hierarchy value */
        switch(pTuneParams->terr.hierarchy)
        {
            case FE_HIERARCHY_NONE:
                tpsTemp |= 0x0000;               /* default is NONE */
                break;
            case FE_HIERARCHY_1:
                tpsTemp |= 0x0400;
                break;
            case FE_HIERARCHY_2:
                tpsTemp |= 0x0800;
                break;
            case FE_HIERARCHY_4:
                tpsTemp |= 0x0C00;
                break;
            case FE_HIERARCHY_AUTO:
                tpsTemp |= 0x0000;               /* default is NONE */
                break;
            default:
                ferr = FE_ERR_BAD_PARAMETER;
                break;
        } /* end of switch(pTuneParams->terr.hierarchy) */
        if (ferr != FE_OK) break;

        /* map tpsGiven  */
        tpsGiven[1] = (uint8_t)((tpsTemp >> 8) & 0xFF);
        tpsGiven[2] = (uint8_t)(tpsTemp & 0xFF);
        
        /* write to all registers */
        ferr = FE_I2C_Write(pData->i2cId,
                              pData->i2cAddrDemod,
                              bwCtrl, 2);
        if (ferr != FE_OK) break;
        ferr = FE_I2C_Write(pData->i2cId,
                              pData->i2cAddrDemod,
                              bwCtrl6Mhz, 3);
        if (ferr != FE_OK) break;
        ferr = FE_I2C_Write(pData->i2cId,
                              pData->i2cAddrDemod,
                              trlNomRate, 3);
        if (ferr != FE_OK) break;
        ferr = FE_I2C_Write(pData->i2cId,
                              pData->i2cAddrDemod,
                              mclkCtrl, 2);
        if (ferr != FE_OK) break;
        ferr = FE_I2C_Write(pData->i2cId,
                              pData->i2cAddrDemod,
                              tpsGiven, 3);
        if (ferr != FE_OK) break;
        ferr = FE_I2C_Write(pData->i2cId,
                              pData->i2cAddrDemod,
                              acqCtrl, 2);
        if (ferr != FE_OK) break;
        ferr = FE_I2C_Write(pData->i2cId,
                              pData->i2cAddrDemod,
                              bwCtrl6Mhz, 3);
        if (ferr != FE_OK) break;

        ferr = FE_I2C_Write(pData->i2cId,
                              pData->i2cAddrDemod,
                              fsmGo, 2);
        if (ferr != FE_OK) break;


        ferr = ce6353UserTimers(TIM_RES_TIM, index, &time);

// !?!       ferr = ce6353WaitDemodLock(index);

    } while(0);
    
    if (ferr == FE_OK)
//        FdFeCe6353Sparams[index].feStatus = FE_STATUS_LOCKED;
        FdFeCe6353Sparams[index].feStatus = FE_STATUS_TUNING;
    else
    {
        FdFeCe6353Sparams[index].feStatus = FE_STATUS_NOT_LOCKED;
        if (ferr == FE_ERR_FE_TIMER)
            ferr = FE_OK; /* no error, just not locked */
        /* else: keep error message */
    }

    return ferr;
}

/*!
*******************************************************************************
**
** \brief  Scan a frequency range and report found areas the FE locks onto
**
** \param  - index                    for frontend data structure
**         - pScanParams            pointer to structure with scan parameters
**                                  nested inside there is a pointer to
**                                  structure of tune parameters. 
**
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER  bad parameter has been paased
**         - #FE_ERR_FE_OTHER       when front end was not intitilized
**         - error values returned by #FE_I2C_Read() or #FE_I2C_Write()
**
** \sa
**
*******************************************************************************
*/

/*!
*******************************************************************************
**
** \brief  Scan the frontend according to frequency range passed
** The scan function uses on internal loop to optimize the frequency settings
** since the OFDM demodulator provides some function to obtain the offset.
** The "custom" parameter is a pointer used to Tune Paramter settings.
**
** Currently this function should not be used because it is still under construction
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_FE_OTHER
**         - #FE_ERR_FE_TIMER
**         - failure codes returned from #FE_I2C_Read(), #FE_I2C_Write()
**
** \sa     CE6353_Tune()
**
*******************************************************************************
*/
int32_t CE6353_Scan(uint8_t index, FE_SCAN_PARAMS_S* pScanParams)
{
    FE_TUNE_PARAMS_S*    pTuneParams;
    int32_t                    ferr = FE_OK;
    int32_t                     afcValue;           /* frequency offset value in kHz */
    uint8_t                      afcCtrl[2];
    uint8_t                      captureRange[2];
    uint32_t                     tempScanFreq;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if(pScanParams == NULL) return FE_ERR_BAD_PARAMETER;
    
    /* check custom parameter pointer and use for tune parameters */
    if(pScanParams->custom == NULL) return FE_ERR_BAD_PARAMETER;
    pTuneParams = (FE_TUNE_PARAMS_S*)(void*)pScanParams->custom;
    
    if(FdFeCe6353Sparams[index].feInitialized == 0) return FE_ERR_FE_OTHER;
    /* frequency range is checked when tuning function is called */
    if(pScanParams->startFrequency > pScanParams->stopFrequency) return FE_ERR_BAD_PARAMETER;

    /* disable automatic frequency control for blind scan */
    afcCtrl[0] = CE6353_AFCCTRL; afcCtrl[1] = 0x28;
    /* and increase the capture range                     */
    captureRange[0] = CE6353_CAPTURE; captureRange[1] = 0x12;

    do
    {
        /* enlarge the frequency capture range */
        ferr = FE_I2C_Write(pData->i2cId,
                              pData->i2cAddrDemod,
                              captureRange, 2);
        if (ferr != FE_OK) break;
        /* disable AFC control */
        ferr = FE_I2C_Write(pData->i2cId,
                              pData->i2cAddrDemod,
                              afcCtrl, 2);
        if (ferr != FE_OK) break;
        
        for(tempScanFreq = pScanParams->startFrequency;
            tempScanFreq <= pScanParams->stopFrequency;
            tempScanFreq += SCAN_STEP)
        {
            FE_DPrintf("[FFE_OFDM] Try at %dHz\n",tempScanFreq);
            pTuneParams->terr.frequency = tempScanFreq;
            ferr = CE6353_Tune(index, pTuneParams);
            if(ferr == FE_OK)
            {
                /* tune success, enter next loop */
                ferr = CE6353_GetAFC(index, &afcValue);
                FE_DPrintf("[FFE_OFDM] First AFC Value = %d\n",afcValue);
                if(ferr == FE_OK && (afcValue > 83 || afcValue < -83))
                {
                    pTuneParams->terr.frequency = FdFeCe6353Sparams[index].frequency - afcValue * 1000;
                    /* reduce the frequency capture range to default*/
                    captureRange[1] = 0x11;
                    ferr = FE_I2C_Write(pData->i2cId,
                                          pData->i2cAddrDemod,
                                          captureRange, 2);
                    if (ferr != FE_OK) break;
                    ferr = CE6353_Tune(index, pTuneParams);
                    ferr = CE6353_GetAFC(index, &afcValue);
                    FE_DPrintf("[FFE_OFDM] Second AFC Value = %d\n",afcValue);
                    /* enlarge the frequency capture range */
                    captureRange[1] = 0x12;
                    ferr = FE_I2C_Write(pData->i2cId,
                                          pData->i2cAddrDemod,
                                          captureRange, 2);

                } /* end of if ... */
                if (ferr != FE_OK) break;
                ferr = CE6353_GetTuneParams(index, pTuneParams);
                pScanParams->feScanNotifyFct(pTuneParams);
            } /* end of if ... */
            FE_DPrintf("[FFE_OFDM] ferr = %x\n",ferr);
            if (ferr != FE_ERR_FE_TIMER && ferr != FE_OK) break;
        } /* end of for .... */
    
        /* enable automatic frequency control again */
        afcCtrl[0] = CE6353_AFCCTRL; afcCtrl[1] = 0x29;
        ferr = FE_I2C_Write(pData->i2cId,
                              pData->i2cAddrDemod,
                              afcCtrl, 2);
        if (ferr != FE_OK) break;
    } while (0);
    
    return ferr;
}

/*!
*******************************************************************************
**
** \brief  Get current status of front end
**
** \param   index                       Index of frontend asked for
** \param   #FE_STATUS_E* pStatus    pointer to store current status
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_FE_OTHER
**         - #FE_ERR_FE_TIMER
**         - failure codes returned from #FE_I2C_Read(), #FE_I2C_Write()
**
** \sa     CE6353_Tune()
**
*******************************************************************************
*/
int32_t CE6353_GetStatus(uint8_t index, FE_STATUS_E* pStatus)
{
    int32_t        ferr = FE_OK;
    uint8_t          readStatusCE6353[3];
    uint32_t         expTime; /* expired time since tune attempt */
    uint32_t         timeout;
    
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    readStatusCE6353[0] = CE6353_STATUS;

    /* set a default status */
    *pStatus = FdFeCe6353Sparams[index].feStatus = FE_STATUS_NOT_LOCKED;
    /* read CE6353 status */
    ferr = FE_I2C_Read(pData->i2cId,
                        pData->i2cAddrDemod,
                        readStatusCE6353,
                        1,
                        &(readStatusCE6353[1]),
                        2);
    if (ferr != FE_OK)
        return ferr;
    /* we check full lock first to save time */
    if(readStatusCE6353[1] & CE6353_STATUS_FULL_LOCK)
    {
        *pStatus = FdFeCe6353Sparams[index].feStatus = FE_STATUS_LOCKED;
        #ifdef CE6353_DEBUG_3
        FM_PrStr("\n[CE6353_GetStatus] Tuner locked");
        #endif
        return FE_OK;
    }
    /* if not locked we preset status to tuning */
    ce6353UserTimers(TIM_GET_TIM, index, &expTime);
    FE_DPrintf("\n[CE6353_GetStatus] Time get: %d", expTime);
    timeout = 200;
    if (((readStatusCE6353[1] & CE6353_STATUS_AGC_LOCK) == 0) && (expTime > timeout))
    {   /* time expired but no AGC lock */
        *pStatus = FdFeCe6353Sparams[index].feStatus = FE_STATUS_NOT_LOCKED;
        FE_DPrintf("\n[CE6353_GetStatus] Tuner not locked 1");
        return FE_OK;
    }
    timeout += 200;
    if (((readStatusCE6353[1] & CE6353_STATUS_SYM_LOCK) == 0) && (expTime > timeout))
    {   /* time expired but no Symbol lock */
        *pStatus = FdFeCe6353Sparams[index].feStatus = FE_STATUS_NOT_LOCKED;
        FE_DPrintf("\n[CE6353_GetStatus] Tuner not locked 2");
        return FE_OK;
    }
    timeout += 400;
    if (((readStatusCE6353[1] & CE6353_STATUS_OFDM_FOUND) == 0) && (expTime > timeout))
    {   /* time expired but no OFDM lock */
        *pStatus = FdFeCe6353Sparams[index].feStatus = FE_STATUS_NOT_LOCKED;
        FE_DPrintf("\n[CE6353_GetStatus] Tuner not locked 3");
        return FE_OK;
    }
    timeout += 500;
    if (((readStatusCE6353[1] & CE6353_STATUS_TPS_LOCK) == 0) && (expTime > timeout))
    {   /* time expired but no TPS lock */
        *pStatus = FdFeCe6353Sparams[index].feStatus = FE_STATUS_NOT_LOCKED;
        FE_DPrintf("\n[CE6353_GetStatus] Tuner not locked 4");
        return FE_OK;
    }
    timeout += 200;
    if (((readStatusCE6353[1] & CE6353_STATUS_FEC_LOCK) == 0) && (expTime > timeout))
    {   /* time expired but no FEC lock */
        *pStatus = FdFeCe6353Sparams[index].feStatus = FE_STATUS_NOT_LOCKED;
        FE_DPrintf("\n[CE6353_GetStatus] Tuner not locked 5");
        return FE_OK;
    }
    timeout += 5000;
    /* Now we know from above that full lock was not achieved, so just check final timeout */
    if (expTime > timeout)
    {   /* total lock time expired but no full lock */
        *pStatus = FdFeCe6353Sparams[index].feStatus = FE_STATUS_NOT_LOCKED;
        FE_DPrintf("\n[CE6353_GetStatus] Tuner not locked 6");
        return FE_OK;
    }
    /* after running through all these checks, we know that there was no
       timeout and no full lock, hence the status is still tuning */
    *pStatus = FdFeCe6353Sparams[index].feStatus = FE_STATUS_TUNING;
    FE_DPrintf("\n[CE6353_GetStatus] Tuner tuning");

    return FE_OK;
}

/*!
*******************************************************************************
**
** \brief  Get AFC value from OFDM demodulator
** The scan function uses on internal loop to optimize the frequency settings
** since the OFDM demodulator provides some function to obtain the offset.
** The "custom" parameter is a pointer used to Tune Paramter settings.
**
** \param   index                       Index of frontend asked for
** \param   #int32_t* pAFC                  pointer to store AFC value
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - failure codes returned from #FE_I2C_Read(), #FE_I2C_Write()
**
** \sa     CE6353_Open()
**
*******************************************************************************
*/
int32_t CE6353_GetAFC(uint8_t index, int32_t* pAFC)
{
    int32_t        ferr = FE_OK;
    uint8_t          readStatusCE6353[2];
    uint8_t          freqOffset[6];
    uint8_t          tpsReceived[3];
    uint8_t          afcCtrl[2];
    uint32_t         temp1;
    int16_t         temp2;
    uint8_t          changeAFCCtrl;
    
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pAFC == NULL) return FE_ERR_BAD_PARAMETER;
    
    readStatusCE6353[0] = CE6353_STATUS;
    freqOffset[0] = CE6353_ITBFREQOFFSET;
    tpsReceived[0] = CE6353_TPSRECEIVED;
    afcCtrl[0] = CE6353_AFCCTRL;
    changeAFCCtrl = 0;
    
    do
    {
        /* first read status of demodulator to check if OFDM lock has occured */
        ferr = FE_I2C_Read(pData->i2cId,
                            pData->i2cAddrDemod,
                            readStatusCE6353,
                            1,
                            &(readStatusCE6353[1]),
                            1);
        if (ferr != FE_OK) break;
        if((readStatusCE6353[1] & 0x04) == 0)
        {
            /* no OFDM lock results in no offset available */
            ferr = FE_ERR_FE_NOT_LOCKED;
            break;
        }
        /* read first AFC control to check if it still has to be disabled */
        /* at the end original state should not be changed                */
        /* ACF is disable when doing a frequency scan for example         */
        ferr = FE_I2C_Read(pData->i2cId,
                            pData->i2cAddrDemod,
                            afcCtrl,
                            1,
                            &(afcCtrl[1]),
                            1);
        if (ferr != FE_OK) break;
        if(afcCtrl[1] & 0x01)
            changeAFCCtrl = 1;
        /* disable automatic AFC to read accurate offset value */
        if(changeAFCCtrl == 1)
        {
            afcCtrl[1] = 0x28;
            ferr = FE_I2C_Write(pData->i2cId,
                                  pData->i2cAddrDemod,
                                  afcCtrl, 2);
        }
        if (ferr != FE_OK) break;

        /* read offset value and calculate frequency offset */
        ferr = FE_I2C_Read(pData->i2cId,
                            pData->i2cAddrDemod,
                            freqOffset,
                            1,
                            &(freqOffset[1]),
                            5);
        if (ferr != FE_OK) break;

        /* read TPS received value to get mode and bandwidth */
                ferr = FE_I2C_Read(pData->i2cId,
                            pData->i2cAddrDemod,
                            tpsReceived,
                            1,
                            &(tpsReceived[1]),
                            2);
        if (ferr != FE_OK) break;
        
        /* enable automatic AFC to read accurate offset value */
        if(changeAFCCtrl == 1)
        {
            afcCtrl[1] = 0x29;
            ferr = FE_I2C_Write(pData->i2cId,
                                  pData->i2cAddrDemod,
                                  afcCtrl, 2);
        }
        if (ferr != FE_OK) break;
        temp1 = (uint32_t)(freqOffset[3]<<16) | (uint32_t)(freqOffset[4]<<8) | (uint32_t)freqOffset[5];
        temp2 = (uint32_t)(freqOffset[1]<<8) | (uint32_t)freqOffset[2];

        /* Refer to page 54 of Intel's design manual to get more info how the */
        /* frequency offset is calculated.                                    */
        /* Formula:                                                           */
        /* freqOff[kHz] = offsetRead/8192 * 64000/7 *1/(2048*(1+(3*mode))) * bw/8 */
        /* where mode = 0 for 2k and 1 for 8k                                     */
        /* The full equation simplifies to                                        */
        /* (offsetRead x bw) / (14680 x mode). Really.                            */

        /* sign extend */
        if (temp1&0x00800000)
        {
            temp1 |=0xFF000000;
        }
        *pAFC = temp1;
        switch(FdFeCe6353Sparams[index].bandwidth)
        {
            case FE_BANDWIDTH_6_MHZ:
                *pAFC *= 6;
                break;
            case FE_BANDWIDTH_7_MHZ:
                *pAFC *= 7;
                break;
            case FE_BANDWIDTH_8_MHZ:
                *pAFC *= 8;
                break;
            /* otherwise frequency offset cannot be calculated */
            case FE_BANDWIDTH_NONE:
            case FE_BANDWIDTH_AUTO:
            default:
            ferr = FE_ERR_FE_OTHER;
        } /* end of switch(FdFeCe6353Sparams[index].bandwidth) */
         
        if (tpsReceived[2] & 0x03)
        {/* 8K mode, divide by 4*/
             *pAFC /= 4;
        }
        *pAFC /= 14680;

        /* this second part has been ported from Intel's application source  */
        /* since register 0x16 and 0x17 are not documented                   */
        /* Offset += (ItbOffset  * demod->internalClks.AdcClk) / (1 << 20);  */
        *pAFC += (temp2 * 11) / 256;
    } while (0);
     
    return ferr;
}

/*!
*******************************************************************************
**
** \brief  Get BER after Viterbi decoder
** The returned error count is expected to be based upon 1 million. The 
** parameters are set to count approx. 0.5 seconds.
**
** \param   index                       Index of frontend asked for
** \param   #uint32_t* pBER                  pointer to store BER value
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - failure codes returned from #FE_I2C_Read(), #FE_I2C_Write()
**
** \sa     CE6353_Open()
**
*******************************************************************************
*/
/* read BER after Viterbi Decoder */
int32_t CE6353_GetBER(uint8_t index, uint32_t* pBER)
{
    int32_t        ferr = FE_ERR_FE_TIMER;
    uint8_t          readStatusCE6353[2];
    uint8_t          berCnt[4];
    uint8_t          rsErrPeriod[3];
    
    readStatusCE6353[0] = CE6353_STATUS;
    berCnt[0] = CE6353_RSBERCNT;
    *pBER = 500000;
    
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pBER == NULL) return FE_ERR_BAD_PARAMETER;
    
    do
    {
        /* check first if OFDM signal is being detected */
        ferr = FE_I2C_Read(pData->i2cId,
                            pData->i2cAddrDemod,
                            readStatusCE6353,
                            1,
                            &(readStatusCE6353[1]),
                            1);
        if(ferr != FE_OK) break;
        if((readStatusCE6353[1] & CE6353_STATUS_FEC_LOCK) == 0)
        {
            ferr = FE_OK;
            break;
        }
        if(ferr != FE_OK) break;
        
        /* according to Intel's design manual, page 77 the post Viterbi      */
        /* error rate is calculated as follows:                              */
        /* Viterbi_BER = rsErrorCnt/(rsErrorPeriod * 1024 * 1632)            */
        /* the rsErrPeriod value is set to count aprox. half a second at     */
        /* OFDM parameters 8MHz, 2k, Guard-1/32, non hierarchical, QAM-64,   */
        /* code-rate 2/3).                                                   */
        /* An OFDM Superframe contains 1008 RS packet transferred within     */
        /* 62.832ms. To cover half a second you need 7.8 * 1024 * RS-Packets */
        /* Using 8*1024 RS packets we receive 204*8*1024 = 1369344 Bits, so  */
        /* we need to divide by 13.69344.                                    */
        /* Since we receive from the demodulator a 24 bit value we can expand*/
        /* to a 32bit value by multiplying by 128 which results finally in   */
        /* division 1752.                                                    */
        rsErrPeriod[1] = 0x00;
        rsErrPeriod[2] = 0x08;
        ferr = FE_I2C_Write(pData->i2cId,
                            pData->i2cAddrDemod,
                            rsErrPeriod,
                            3);
        if(ferr != FE_OK) break;
        ferr = FE_I2C_Read(pData->i2cId,
                            pData->i2cAddrDemod,
                            berCnt,
                            1,
                            &(berCnt[1]),
                            3);
        if(ferr != FE_OK) break;
        
        *pBER = ((berCnt[1]<<24) + (berCnt[2]<<16) + (berCnt[3]<<8)) / 1752;
    } while (0);
              
    return ferr;
}

/*!
*******************************************************************************
**
** \brief  Get FEC after RS decoder
** The returned error count is expected to be based upon 1 million. The 
** parameters are set to count approx. 0.5 seconds.
**
** \param   index                       Index of frontend asked for
** \param   #uint32_t* pBER                  pointer to store BER value
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - failure codes returned from #FE_I2C_Read(), #FE_I2C_Write()
**
** \sa     CE6353_Open()
**
*******************************************************************************
*/
/* read FEC count after RS Decoder */
/******************************************************************************
**
** \brief  Returns current FEC.
**
** \param   index                       Index of frontend asked for
** \param   pFEC                        Pointer to FEC.
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_FE_NOT_LOCKED
**         - #FE_ERR_FE_I2C
**
******************************************************************************/
int32_t CE6353_GetFEC(uint8_t index, FE_FEC_E* pFEC)
{
    int32_t        ferr = FE_ERR_FE_TIMER;   
    uint8_t          tpsReceived[3];
    uint16_t         tpsParams;
    
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pFEC == NULL) return FE_ERR_BAD_PARAMETER;

    tpsReceived[0] = CE6353_TPSRECEIVED;
    tpsParams = 0;   

    if(FdFeCe6353Sparams[index].feStatus == FE_STATUS_TUNING)
        ferr = FE_ERR_FE_TUNE;
    else if(FdFeCe6353Sparams[index].feStatus == FE_STATUS_LOCKED)
    {
        /* read TPS received data */
        ferr = FE_I2C_Read(pData->i2cId,
                            pData->i2cAddrDemod,
                            tpsReceived,
                            1,
                            &(tpsReceived[1]),
                            2);
        tpsParams = (uint16_t)(tpsReceived[1]<<8) + (uint16_t)tpsReceived[2];
    }
    
    else
        ferr = FE_ERR_FE_OTHER;
    if(ferr == FE_OK)
    {
        /* check first if parameter data is flagged as valid */
        if((tpsParams & 0x8000) == 0)
            return FE_ERR_FE_NOT_LOCKED;
        /* evaluate HP FEC code rate */
        switch((tpsParams>>7) & 0x7)
        {
            case 0:
                *pFEC = FE_FEC_1_2;
                break;
            case 1:
                *pFEC = FE_FEC_2_3;
                break;
            case 2:
                *pFEC = FE_FEC_3_4;
                break;
            case 3:
                *pFEC = FE_FEC_5_6;
                break;
            case 4:
                *pFEC = FE_FEC_7_8;
                break;
            default:
                *pFEC = FE_FEC_NONE;
                break;
        }
    }
              
    return ferr;
}

/*!
*******************************************************************************
**
** \brief  Get signal quality
** Intel's CE6353 offers a register that provides a value for the S/N ratio
**
** \param   index                       Index of frontend asked for
** \param   #uint16_t* pQuality              pointer to store S/N value
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - failure codes returned from #FE_I2C_Read(), #FE_I2C_Write()
**
** \sa     CE6353_Open()
**
*******************************************************************************
*/
int32_t CE6353_GetSignalQuality(uint8_t index, uint16_t* pQuality)
{
    int32_t        ferr;
    uint8_t          snrValue[2];
    
    snrValue[0] = CE6353_SNR;
    ferr = FE_I2C_Read(pData->i2cId,
                        pData->i2cAddrDemod,
                        snrValue,
                        1,
                        &(snrValue[1]),
                        1);
    /*
        bits [0..7] to be filled with 0xFF to allow a 100% display in the application.
    */
    *pQuality = (uint16_t)((snrValue[1]<<8) | 0x00FF);
    return ferr;
}

/*!
*******************************************************************************
**
** \brief  Get signal strength
** The AGC level value of the Thomson tuner is passed via an ADC of the
** CE6353. The value read is 7 bit. 
**
** \param   index                       Index of frontend asked for
** \param   #uint16_t* pStrength             pointer to store signal strength
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - failure codes returned from #FE_I2C_Read(), #FE_I2C_Write()
**
** \sa     CE6353_Open()
**
*******************************************************************************
*/
int32_t CE6353_GetSignalStrength(uint8_t index, uint16_t* pStrength)
{
    int32_t           ferr = FE_OK;
    FE_STATUS_E status;
    uint8_t             rfLev[2];
    
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    rfLev[0] = CE6353_RFADCLEVEL;
    ferr = FE_I2C_Read(pData->i2cId,
                        pData->i2cAddrDemod,
                        rfLev,
                        1,
                        &(rfLev[1]),
                        1);
    if(ferr == FE_OK)
    {
        /* with current hardware AGC values read are correspond to:           */
        /*  -10dB = 0x15                                                      */
        /*  -20dB = 0x2B                                                      */
        /*  -30dB = 0x3A                                                      */
        /*  -40dB = 0x47                                                      */
        /*  -50dB = 0x56                                                      */
        /*  -60dB = 0x74                                                      */
        /* Let's define our range from -10dB downwards                        */
        #ifdef CE6353_DEBUG
        FM_PrStr("[FE_OFDM] rfLevel: "); FM_PrHex(rfLev[1], 2);
        #endif
        if(rfLev[1] < 0x20)
            rfLev[1] = 0x20;
        else if(rfLev[1] > 0x75)
            rfLev[1] = 0x75;
        /* Now rfLev is in the range from 0x20 to 0x75 */
        rfLev[1] -= 0x20;
        /* Now rfLev is in the range from 0 to 0x55 */
        rfLev[1] = (0x55 - rfLev[1]) * 3; /* multiplication * 3 results in max 0xFF */
        /* fit to 16 bit value */
        if (rfLev[1] < 0x30)
        {   /* check lock and simulate some level */
            ferr = CE6353_GetStatus(index, &status);
            if (ferr != FE_OK)
                return (ferr);
            if (status == FE_STATUS_LOCKED)
                rfLev[1] = 0x30;
        }
        /*
            bits [0..7] to be filled with 0xFF to allow a 100% display in the application.
        */
        *pStrength = (uint16_t)((rfLev[1] << 8) | 0x00FF);
    }
    
    return ferr;
}

/*!
*******************************************************************************
**
** \brief  Get current received TPS parameters
**
** \param   index                               Index of frontend asked for
** \param   #FE_TUNE_PARAMS_S *pTuneParams   pointer to store tune parameters
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - failure codes returned from #FE_I2C_Read(), #FE_I2C_Write()
**
** \sa
**
*******************************************************************************
*/

int32_t CE6353_GetTuneParams(uint8_t index, FE_TUNE_PARAMS_S *pTuneParams)
{
    int32_t        ferr = FE_OK;
    uint8_t          status[2];
    uint8_t          tpsReceived[3];
    uint16_t         tpsParams;
    
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pTuneParams == NULL) return FE_ERR_BAD_PARAMETER;

    status[0] = CE6353_STATUS;
    tpsReceived[0] = CE6353_TPSRECEIVED;
    tpsParams = 0;   
    if(FdFeCe6353Sparams[index].feStatus == FE_STATUS_TUNING)
        ferr = FE_ERR_FE_TUNE;
    else if(FdFeCe6353Sparams[index].feStatus == FE_STATUS_LOCKED)
    {
        /* read TPS received data */
        ferr = FE_I2C_Read(pData->i2cId,
                            pData->i2cAddrDemod,
                            tpsReceived,
                            1,
                            &(tpsReceived[1]),
                            2);
        if(ferr != FE_OK) return ferr;
        
        /* read Status 0 data */
        ferr = FE_I2C_Read(pData->i2cId,
                            pData->i2cAddrDemod,
                            status,
                            1,
                            &(status[1]),
                            1);
        if(ferr != FE_OK) return ferr;
        tpsParams = (uint16_t)(tpsReceived[1]<<8) + (uint16_t)tpsReceived[2];
    }
    
    else
        ferr = FE_ERR_FE_OTHER;
    if(ferr == FE_OK)
    {
        pTuneParams->terr.frequency = FdFeCe6353Sparams[index].frequency;
        pTuneParams->terr.bandwidth = FdFeCe6353Sparams[index].bandwidth;
        
        /* check first if parameter data is flagged as valid */
        if((tpsParams & 0x8000) == 0)
            return FE_ERR_FE_NOT_LOCKED;
        /* evaluate HP FEC code rate */
        switch((tpsParams>>7) & 0x7)
        {
            case 0:
                pTuneParams->terr.codeRateHP = FE_FEC_1_2;
                break;
            case 1:
                pTuneParams->terr.codeRateHP = FE_FEC_2_3;
                break;
            case 2:
                pTuneParams->terr.codeRateHP = FE_FEC_3_4;
                break;
            case 3:
                pTuneParams->terr.codeRateHP = FE_FEC_5_6;
                break;
            case 4:
                pTuneParams->terr.codeRateHP = FE_FEC_7_8;
                break;
            default:
                pTuneParams->terr.codeRateHP = FE_FEC_NONE;
                break;
        }

        /* evaluate LP FEC code rate */
        switch((tpsParams>>4) & 0x7)
        {
            case 0:
                pTuneParams->terr.codeRateLP = FE_FEC_1_2;
                break;
            case 1:
                pTuneParams->terr.codeRateLP = FE_FEC_2_3;
                break;
            case 2:
                pTuneParams->terr.codeRateLP = FE_FEC_3_4;
                break;
            case 3:
                pTuneParams->terr.codeRateLP = FE_FEC_5_6;
                break;
            case 4:
                pTuneParams->terr.codeRateLP = FE_FEC_7_8;
                break;
            default:
                pTuneParams->terr.codeRateLP = FE_FEC_NONE;
                break;
        }
        /* evaluate modulation */
        switch((tpsParams>>13) & 0x3)
        {
            case 0:
                pTuneParams->terr.modulation = FE_MOD_QPSK;
                break;
            case 1:
                pTuneParams->terr.modulation = FE_MOD_QAM_16;
                break;
            case 2:
                pTuneParams->terr.modulation = FE_MOD_QAM_64;
                break;
            default:
                pTuneParams->terr.modulation = FE_MOD_NONE;
                break;
        }
        /* evaluate transmission */
        switch((tpsParams) & 0x3)
        {
            case 0:
                pTuneParams->terr.transmission = FE_TRANS_MODE_2K;
                break;
            case 1:
                pTuneParams->terr.transmission = FE_TRANS_MODE_8K;
                break;
            default:
                pTuneParams->terr.transmission = FE_TRANS_MODE_NONE;
                break;
        }
        /* evaluate guard interval */
        switch((tpsParams>>2) & 0x3)
        {
            case 0:
                pTuneParams->terr.guardInterval = FE_GUARD_INTERVAL_1_32;
                break;
            case 1:
                pTuneParams->terr.guardInterval = FE_GUARD_INTERVAL_1_16;
                break;
            case 2:
                pTuneParams->terr.guardInterval = FE_GUARD_INTERVAL_1_8;
                break;
            case 3:
                pTuneParams->terr.guardInterval = FE_GUARD_INTERVAL_1_4;
                break;
            default:
                pTuneParams->terr.guardInterval = FE_GUARD_INTERVAL_NONE;
                break;
        }
        /* evaluate hierarchy */
        switch((tpsParams>>10) & 0x7)
        {
            case 0:
                pTuneParams->terr.hierarchy = FE_HIERARCHY_NONE;
                break;
            case 1:
                pTuneParams->terr.hierarchy = FE_HIERARCHY_1;
                break;
            case 2:
                pTuneParams->terr.hierarchy = FE_HIERARCHY_2;
                break;
            case 3:
                pTuneParams->terr.hierarchy = FE_HIERARCHY_4;
                break;
            default:
                pTuneParams->terr.hierarchy = FE_HIERARCHY_NONE;
                break;
        }
        if((status[1] & 0x80) == 0)
            pTuneParams->terr.iq = FE_IQ_NORMAL;
        else
            pTuneParams->terr.iq = FE_IQ_INVERTED;
    } /* end of if(ferr == FE_OK) */
    
        
    return ferr;
}

/*!
*******************************************************************************
**
** \brief  Enable/Disable antenna power output flag
** Antenna power output however will be set first when tune function is called.
**
** \param   index        Index of frontend asked for
** \param   antPower     uint8_t    type 1 = power on, 0 = power off,
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**
** \sa
**
*******************************************************************************
*/
int32_t CE6353_SetAntennaPower (uint8_t index, uint8_t antPower)
{
    int32_t        ferr;
    uint8_t   i2cBuffer[2];

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    pData = &feHandleArray[index];
    /* on SmartBOX it is controlled via a SmartMPEG GPIO, so we need to
    call the FFE driver. On our NIM it is controlled via a GPIO of the
    CE6353, so we also need to control this GPIO here. */
    i2cBuffer[0] = CE6353_GPPCTL;
    if (antPower == 0)
    {
        /* set GPP3 as output to control antenna power - default power is off */
        i2cBuffer[1] = CE6353_GPPCTL_GPP3_OUT | CE6353_GPPCTL_GPP3; /* GPIO high = off */
        ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, i2cBuffer, 2);
        /* in case of the power is controlled via CE6353 GPIOs, the ffe will
           not have a control function and report an error
           -> we have to ignore the error here */
        FE_ControlAntPow (index, 0);
    }
    else /* ant. power on */
    {
        i2cBuffer[1] = CE6353_GPPCTL_GPP3_OUT; /* GPIO low = on */
        ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, i2cBuffer, 2);
        /* in case of the power is controlled via CE6353 GPIOs, the ffe will
           not have a control function and report an error
           -> we have to ignore the error here */
        FE_ControlAntPow (index, 1);
    }
    FdFeCe6353Sparams[index].antPower = antPower;
    return ferr;
}

/*!
*******************************************************************************
**
** \brief   sets the used tuner frequency (1/6MHz raster)
**
** \param   index        Index of frontend asked for
** \param   frequency    Tuner frequency
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**
** \sa
**
*******************************************************************************
*/
int32_t CE6353_StoreTunerFrequency(uint8_t index, uint32_t freq)
{
    int32_t        ferr = FE_OK;
    
    if (index >= FE_MAX_NUM)
        return FE_ERR_BAD_PARAMETER;

    FdFeCe6353Sparams[index].frequency = freq;
    return ferr;
}

/*!
*******************************************************************************
**
** \brief  sets the tuner status in the sParam
**
** \param   index        Index of frontend asked for
** \param   status       Tuner status
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**
** \sa
**
*******************************************************************************
*/
int32_t CE6353_SetTunerStatus(uint8_t index, FE_STATUS_E status)
{
    int32_t        ferr = FE_OK;
    
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    
    FdFeCe6353Sparams[index].feStatus = status;
    return ferr;
}


/*
*******************************************************************************
**
** \brief  Reads from tuner without connecting and disconnecting.
**         Subb addresses are not supported.
**
** \param  index
** \param  data    pointer to the data buffer
** \param  length  number of bytes to read
**
** \return Possible return codes:
**         - failure codes returned from #FE_I2C_Read()
**
** \sa  #CE6353_WriteDirectToTuner
**
*******************************************************************************
*/
int32_t CE6353_ReadDirectFromTuner(uint8_t index, uint8_t *data, uint8_t length)
{
    uint8_t dummy;
    
    /* write to tuner */
    return (FE_I2C_Read(pData->i2cId, pData->i2cAddrTuner, &dummy, 0, 
                        data, length));

}

#define TUNER_LOCK_TIME 1000
int32_t CE6353_WaitTunerLock (uint8_t index, uint8_t wrLength)
{
    int32_t     ferr      = FE_ERR_FE_NOT_LOCKED;
    uint32_t    currTime  = FE_GetMillis();
    uint8_t     writeByte = 0x80;
    uint8_t     readByte;
    
    FE_Wait(50);
    do
    {
        ferr = FE_I2C_Read(pData->i2cId, pData->i2cAddrTuner, &writeByte, wrLength, &readByte, 1 );
        if (ferr != FE_OK) break;
        if((readByte & 0x40) > 0)
        {
            ferr = FE_OK;
            break;
        }
        FE_Wait(20);
    } while(currTime + TUNER_LOCK_TIME > FE_GetMillis());
    
    return ferr;
}

/*
*******************************************************************************
**
** \brief  Reads from tuner with connecting and disconnecting.
**         Subb addresses are supported.
**
** \param  index
** \param  data    pointer to the data buffer
** \param  length  number of bytes to read
**
** \return Possible return codes:
**         - failure codes returned from #FE_I2C_Read()
**
** \sa  #CE6353_WriteDirectToTuner
**
*******************************************************************************
*/
int32_t CE6353_ReadFromTuner(uint8_t index, uint8_t *wData, uint8_t wLen, uint8_t *rData, uint8_t rLen)
{
    int32_t ferr;

    /* Connect tuner to primary I2C bus */
    ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, ce6353ConnectTuner, 2);
    if (ferr != FE_OK)
    {
        /* Disconnect tuner from primary I2C bus */
        FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, ce6353DisconnectTuner, 2);
        return (ferr);
    }
    /* read from tuner */
    ferr = FE_I2C_Read(pData->i2cId,
                          pData->i2cAddrTuner,
                          wData, wLen, rData, rLen);
    if (ferr != FE_OK)
    {
        /* Disconnect tuner from primary I2C bus */
        FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, ce6353DisconnectTuner, 2);
        return (ferr);
    }
    /* Disconnect tuner from primary I2C bus */
    ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, ce6353DisconnectTuner, 2);
    return (ferr);

}


/*
*******************************************************************************
**
** \brief  Timer routines to check lock status.
**
** \param  ucFlag
**               TIM_SYN_TIM
**                 This flag causes the routine to wait for a transition of the system tick.
**                 If you use TIM_RES_TIM and TIM_GET_TIM and the routine was not
**                 synchronised before then you have to add one tick time
**                 to your timeout in order to guarantee at least the timeout length.
**                 ucIndex is ignored if only this flag is set.
**               TIM_RES_TIM Resets the timer counter with Index ucIndex.
**                 The reset is done immediately without synchronisation to the system
**                 timer. If your timer tick is long (like the 40ms in our example) then it is
**                 recommended to call User_Timers with TIM_SYN_TIM before.
**                 This gives the maximum accuracy but requires in worst case 40 ms for
**                 synchronisation.
**               TIM_GET_TIM requests the elapsed time of Index ucIndex since its last reset
** \param  ucIndex Timer index (currently from 0 to FE_MAX_NUM-1)
** \param  ulDiffTime  holds the elapsed time of a request in ms.
**                 The precision is depending on the system clock.
**                 Here we have a resolution of 40 resp. 33 ms
**                 This gives in total a max time of more than 1000 hours.
**
** \return Possible return codes:
**         - failure codes returned from #FE_I2C_Read()
**
** \sa  #CE6353_WriteDirectToTuner
**
*******************************************************************************
*/
int32_t ce6353UserTimers(uint8_t ucFlag, uint8_t ucIndex, uint32_t *pulDiffTime)
{
    uint32_t ulTempTime = 0;

    *pulDiffTime = 0;

    if (ucFlag > TIM_GET_TIM)
        return(FE_ERR_BAD_PARAMETER);
    if (ucIndex > FE_MAX_NUM)
        return(FE_ERR_BAD_PARAMETER);

    /* sync */
    if ((ucFlag & TIM_SYN_TIM) == TIM_SYN_TIM)
    {
        /* wait here until timer changes */
        ulTempTime = FE_GetMillis();
        while (FE_GetMillis() != ulTempTime); /* wait */
    }

    /* reset */
    if ((ucFlag & TIM_RES_TIM) == TIM_RES_TIM)
    {
        /* set start time to current time */
        startTimes[ucIndex] = FE_GetMillis();
        *pulDiffTime = 0;
    }

    /* get */
    if ((ucFlag & TIM_GET_TIM) == TIM_GET_TIM)
    {
        /* return current time */
        ulTempTime = FE_GetMillis();
        if (ulTempTime >= startTimes[ucIndex])
        {
           *pulDiffTime = (uint32_t)(ulTempTime - startTimes[ucIndex]);
        }
        else
        {
           /* in case of timer overflow reset start time */
            startTimes[ucIndex] = FE_GetMillis();
            *pulDiffTime = 0;
        }
    }

    /* Mp_VoIntTimeClock increments every 1/25 sec (1/30 for NTSC) */
    /* Other systems may have better reference and need to correct the pulDiffTime in a different way */
    *pulDiffTime *= TIM_RESOLUTION; /* with this multiplication we get an overflow if time is longer than 1000 hours => shouldn't be a problem */

    return FE_OK;
}

/*
*******************************************************************************
**
** \brief  Writes to tuner without connecting and disconnecting.
**
** \param  index
** \param  data    pointer to the data buffer
** \param  length  number of bytes excluding the device address
**
** \return Possible return codes:
**         - failure codes returned from #FE_I2C_Write()
**
** \sa  #CE6353_WriteToTuner
**
*******************************************************************************
*/
int32_t CE6353_WriteDirectToTuner(uint8_t index, uint8_t *data, uint8_t length)
{
    #ifdef CE6353_DEBUG_2
    uint8_t i;
    #endif
    /* write to tuner */
    #ifdef CE6353_DEBUG_2
    FM_PrStr("\nWrite to tuner: 0x"); FM_PrHex(pData->i2cAddrTuner,2);
    for (i = 0; i < length; i++)
    {
        FM_PrStr(", 0x"); FM_PrHex(data[i],2);
    }
    #endif
    return (FE_I2C_Write(pData->i2cId, pData->i2cAddrTuner, data, length));
}

/*
*******************************************************************************
**
** \brief  Enables the I2C connect to the tuner and performs a write,
**         disconnects afterwards.
**
** \param  index
** \param  data    pointer to the data buffer
** \param  length  number of bytes excluding the device address
**
** \return Possible return codes:
**         - failure codes returned from #FE_I2C_Write()
**
** \sa  #CE6353_WriteDirectToTuner
**
*******************************************************************************
*/
int32_t CE6353_WriteToTuner(uint8_t index, uint8_t *data, uint8_t length)
{
    int32_t ferr;
    #ifdef CE6353_DEBUG_2
    uint8_t i;
    #endif
    
    /* Connect tuner to primary I2C bus */
    ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, ce6353ConnectTuner, 2);
    if (ferr != FE_OK)
        return ferr;

    /* write to tuner */
    ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrTuner, data, length);
    if (ferr != FE_OK)
        return ferr;
    #ifdef CE6353_DEBUG_2
    FM_PrStr("\nWrite to tuner: 0x"); FM_PrHex(pData->i2cAddrTuner,2);
    for (i = 0; i < length; i++)
    {
        FM_PrStr(", 0x"); FM_PrHex(data[i],2);
    }
    #endif
    /* Disconnect tuner from primary I2C bus */
    ferr = FE_I2C_Write(pData->i2cId, pData->i2cAddrDemod, ce6353DisconnectTuner, 2);
    if (ferr != FE_OK)
        return ferr;

    return (FE_OK);
}

int32_t ce6353CheckChipId(uint8_t i2cAddress)
{
    uint8_t i2cBuffer[2];
    int32_t ferr;
    
    i2cBuffer[0] = CE6353_CHIPID;
    i2cBuffer[1] = 0xFF;
    ferr = FE_I2C_Read(pData->i2cId, i2cAddress, &(i2cBuffer[0]), 1,
                                                    &(i2cBuffer[1]), 1);
    FE_DPrintf("Checking for CE6353 on address %02x ferr %x read %x\n", i2cAddress, ferr, i2cBuffer[1]);
    if(ferr != FE_OK)
        return ferr;
    if(i2cBuffer[1] != 0x14)
        return -1;
    return FE_OK;
}

void ce6353SetFunctions(feHandleT *h)
{
    h->systemType             = FE_SYSTEM_TYPE_TERRESTRIAL;
    h->feOpenFct              = CE6353_Open;
    h->feCloseFct             = CE6353_Close;
    h->feSetSystemParamsFct   = CE6353_SetSystemParams;
    h->feResetFct             = CE6353_Reset;
    h->feTuneFct              = CE6353_Tune;
    h->feScanFct              = CE6353_Scan;
    h->feSendDiseqcCommandFct = NULL;
    h->feGetStatusFct         = CE6353_GetStatus;
    h->feGetFECFct            = CE6353_GetFEC;
    h->feGetAFCFct            = CE6353_GetAFC;
    h->feGetBERFct            = CE6353_GetBER;
    h->feGetSignalQualityFct  = CE6353_GetSignalQuality;
    h->feGetSignalStrengthFct = CE6353_GetSignalStrength;
    h->feGetTuneParamsFct     = CE6353_GetTuneParams;
    h->feSetAntennaPowerFct   = CE6353_SetAntennaPower;
    h->isPresent              = 1;
}

int32_t CE6353_DTT75404_Detect(uint32_t slotNum, feHandleT *h)
{
    pData = &feHandleArray[slotNum];
    if(slotNum == 0)
        h->i2cAddrDemod = 0x3c;
    else
        h->i2cAddrDemod = 0x3e;
    if(ce6353CheckChipId(h->i2cAddrDemod) != FE_OK)
        return FE_ERR_FE_UNKNOWN_DEVICE;
    ce6353SetFunctions(h);
    h->i2cAddrTuner = 0xc2;
    tempXtalFreq = 20480000;
    tempIfFreq   = 36166667;
    tempInitFct  = NULL;
    tempTuneFct  = dtt75404TuneToFrequency;
    tempCloseFct = dtt75404Close;
    return FE_OK;
}

int32_t CE6353_DTT75404_IDTV_Detect(uint32_t slotNum, feHandleT *h)
{
    pData = &feHandleArray[slotNum];
    if(slotNum)
        return FE_ERR_FE_UNKNOWN_DEVICE;
    h->i2cAddrDemod = 0x1e;
    if(ce6353CheckChipId(h->i2cAddrDemod) != FE_OK)
        return FE_ERR_FE_UNKNOWN_DEVICE;
    ce6353SetFunctions(h);
    h->i2cAddrTuner = 0xc2;
    tempXtalFreq = 20480000;
    tempIfFreq   = 36166667;
    tempInitFct  = NULL;
    tempTuneFct  = dtt75404TuneToFrequency;
    tempCloseFct = dtt75404Close;
    return FE_OK;
}

int32_t CE6353_TD911_Detect(uint32_t slotNum, feHandleT *h)
{
    pData = &feHandleArray[slotNum];
    if(slotNum)
        return FE_ERR_FE_UNKNOWN_DEVICE;
    h->i2cAddrDemod = 0x1c;
    if(ce6353CheckChipId(h->i2cAddrDemod) != FE_OK)
        return FE_ERR_FE_UNKNOWN_DEVICE;
    ce6353SetFunctions(h);
    h->i2cAddrTuner = 0xc0;
    tempXtalFreq = 4000000;
    tempIfFreq   = 36166667;
    tempInitFct  = NULL;
    tempTuneFct  = td911TuneToFrequency;
    tempCloseFct = td911Close;
    return FE_OK;
}

int32_t CE6353_TD1316_IDTV_Detect(uint32_t slotNum, feHandleT *h)
{
    pData = &feHandleArray[slotNum];
    if(slotNum)
        return FE_ERR_FE_UNKNOWN_DEVICE;
    h->i2cAddrDemod = 0x1e;
    if(ce6353CheckChipId(h->i2cAddrDemod) != FE_OK)
        return FE_ERR_FE_UNKNOWN_DEVICE;
    ce6353SetFunctions(h);
    ce6353SetFunctions(h);
    h->i2cAddrTuner = 0xc0;
    tempXtalFreq = 20480000;
    tempIfFreq   = 36166667;
    tempInitFct  = NULL;
    tempTuneFct  = td1316TuneToFrequency;
    tempCloseFct = td1316Close;
    return FE_OK;
}

/*
*******************************************************************************
**
** \brief  Returns the handle used for this index.
**
** \param  index
**
** \return Pointer to the handle
**
*******************************************************************************
*/
int32_t CE6353_Geti2cId(uint8_t index)
{
    return pData->i2cId;
}

/*---------------------------------------------------------------------------*/


/* Version History */
/******************************************************************************
$Log: ce6353.c,v $
Revision 1.3  2008/07/11 13:36:31  sgehre
- DSF89000 activated by default
- added support for MB86667 DVB-S frontend
- use SUBSYS_FE_MODULE_BASE for errors

Revision 1.2  2008/07/09 15:08:37  sgehre
- set I2C to hardware/400kbps
- fix some MISRA warnings

Revision 1.1  2008/05/07 11:10:18  wlaris
Initial creation.

Revision 1.1  2008/05/07 08:39:06  wlaris
Initial creation.


******************************************************************************/
