/******************************************************************************
**
** \file      ./subsystems/fe/ce6353/src/dtt75404.c
**
** \brief     Front-end device driver for Intel OFDM CE6353 with Thomson Tuner.
**
**            (C) Fujitsu Semiconductor Europe 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: dtt75404.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
// #define FE_FE_DTT75404_DEBUG

#include "../src/fe_priv.h"
#include "ce6353_priv.h"
#include "dtt75404.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
/* internal AGC take over point bit definitions */
#define    ATP_112dBuV  0
#define    ATP_109dBuV  REG_BIT_3
#define    ATP_106dBuV  REG_BIT_4
#define    ATP_103dBuV (REG_BIT_4 | REG_BIT_3)
#define    ATP_100dBuV  REG_BIT_5
#define    ATP_97dBuV  (REG_BIT_5 | REG_BIT_3)
#define    ATP0_94dBuV (REG_BIT_5 | REG_BIT_4)
#define    ATP_Disable (REG_BIT_5 | REG_BIT_4 | REG_BIT_3)

/* Reference Frequency definitions */
#define     dtt75404_fref166kHz667    0
// #define     dtt75404_fref142kHz857    REG_BIT_0
// #define     dtt75404_fref80kHz       REG_BIT_1
// #define     dtt75404_fref62kHz5      (REG_BIT_1 | REG_BIT_0)
// #define     dtt75404_fref31kHz25     REG_BIT_2
// #define     dtt75404_fref50kHz       (REG_BIT_2 | REG_BIT_0)

/* bandwidth selection */
#define     dtt75404_bw8MHz         0
#define     dtt75404_bw7MHz         REG_BIT_4

/* Charge Pump Current Select */
#define     dtt75404_cpcSelect0      0
#define     dtt75404_cpcSelect1      REG_BIT_6
#define     dtt75404_cpcSelect2      REG_BIT_7
#define     dtt75404_cpcSelect3      (REG_BIT_7 | REG_BIT_6)

/* band select definition */
#define dtt75404_bandSelectVL    REG_BIT_0
#define dtt75404_bandSelectVH    REG_BIT_1
#define dtt75404_bandSelectUHF   REG_BIT_3

#define gppoInactive    0
#define gppoActive      REG_BIT_2


/* crystal output selection */
#define    xtalOutEnabled   0
#define    xtalOutDisabled  REG_BIT_0

/* RF_AGC source current level definition */
#define    atcLow       0
#define    atcHigh      REG_BIT_5

/* StandBy mode definition */
#define    stbyActive   0
#define    stbySleep    REG_BIT_4


/*-------- Register Bit Definitions -----------*/

/* Bit Definition */
#define CB1_FIXED_BITS    (REG_BIT_7)
#define CB2_FIXED_BITS    (REG_BIT_7 | REG_BIT_6 | REG_BIT_1)

/*---------------------------------------------------------------------------*/
/* local types                                                             */
/*---------------------------------------------------------------------------*/
enum
{
    REG_BIT_0  = (0x01), 
    REG_BIT_1  = (0x02), 
    REG_BIT_2  = (0x04), 
    REG_BIT_3  = (0x08), 
    REG_BIT_4  = (0x10), 
    REG_BIT_5  = (0x20), 
    REG_BIT_6  = (0x40), 
    REG_BIT_7  = (0x80) 
};

/* ------- tuner register structure ---------------------------- */
typedef struct
{
    /* write registers */
    uint8_t    dividerByte1;
    uint8_t    dividerByte2;
    uint8_t    controlByte1;
    uint8_t  bandSwitchByte;
    uint8_t    controlByte2;
    /* read register */
    uint8_t  statusByte;
} TUNER_I2C_DATA_S;
/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static TUNER_I2C_DATA_S registerMirror[FE_MAX_NUM];

/*---------------------------------------------------------------------------*/
/* callback functions                                                        */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

/*
*******************************************************************************
**
** \brief   Shut down Thomson tuner
**
** \param   none
**
** \return Possible return codes:
**         - #FE_OK
**         - error values returned by #FE_I2C_Write()
**
** \sa
**
*******************************************************************************
*/
int32_t dtt75404Close(uint8_t index)
{
    int32_t ferr;

    registerMirror[index].controlByte2 = CB2_FIXED_BITS |
                                                 atcLow |
                                              stbySleep |
                                        xtalOutDisabled;

    ferr = CE6353_WriteToTuner(index, &(registerMirror[index].controlByte2), 1);

    return (ferr);
}

/*
*******************************************************************************
**
** \brief  Tune Thomson DTT75404 tuner to given frequency
**
** \param   index           for frontend data structure
** \param   frequency       given in Hz
** \param   bandwidth       passed in Hz
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_FE_NOT_LOCKED
**         - error values returned by #FE_I2C_Write(), #FE_I2C_READ()
**
** \sa
**
*******************************************************************************
*/
int32_t dtt75404TuneToFrequency (uint8_t index, uint32_t frequency, uint32_t _bandwidth)
{
    int32_t              ferr = FE_OK;
    uint16_t               divider;
    FE_BANDWIDTH_E bandwidth;

    FE_DPrintf("[DTT75404] Tuning to %d\n", frequency);
    /* frequency range should be in between 50.5MHz and 854 MHz */
    if(frequency > 858000000 || frequency < 50500000) return FE_ERR_BAD_PARAMETER;
    CE6353_SetTunerStatus(index, FE_STATUS_TUNING);

    if (_bandwidth == 6000000)
        bandwidth = FE_BANDWIDTH_6_MHZ;
    else if (_bandwidth == 7000000)
        bandwidth = FE_BANDWIDTH_7_MHZ;
    else if (_bandwidth == 8000000)
        bandwidth = FE_BANDWIDTH_8_MHZ;
    else
        return FE_ERR_BAD_PARAMETER;

    frequency += 36166667;      /* add IF frequency */
    frequency += 83333;         /* add on for rounding purposes */
    divider = (uint16_t)((((frequency)/10)*6)/100000);
    /* set internal frequency to the real value that the tuner is using */
    CE6353_StoreTunerFrequency(index, (divider * 166667) - 36166667);

    /* get appropriate settings for bandwidth selection byte */
    if(frequency >= 83000000 && frequency <= 121000000)
        registerMirror[index].bandSwitchByte = dtt75404_cpcSelect0 | dtt75404_bandSelectVL;
    else if (frequency > 121000000 && frequency <= 141000000)
        registerMirror[index].bandSwitchByte = dtt75404_cpcSelect1 | dtt75404_bandSelectVL;
    else if (frequency > 141000000 && frequency <= 166000000)
        registerMirror[index].bandSwitchByte = dtt75404_cpcSelect2 | dtt75404_bandSelectVL;
    else if (frequency > 166000000 && frequency <= 182000000)
        registerMirror[index].bandSwitchByte = dtt75404_cpcSelect3 | dtt75404_bandSelectVL;
    else if (frequency > 182000000 && frequency <= 286000000)
        registerMirror[index].bandSwitchByte = dtt75404_cpcSelect0 | dtt75404_bandSelectVH;
    else if (frequency > 286000000 && frequency <= 386000000)
        registerMirror[index].bandSwitchByte = dtt75404_cpcSelect1 | dtt75404_bandSelectVH;
    else if (frequency > 386000000 && frequency <= 446000000)
        registerMirror[index].bandSwitchByte = dtt75404_cpcSelect2 | dtt75404_bandSelectVH;
    else if (frequency > 446000000 && frequency <= 466000000)
        registerMirror[index].bandSwitchByte = dtt75404_cpcSelect3 | dtt75404_bandSelectVH;
    else if (frequency > 466000000 && frequency <= 506000000)
        registerMirror[index].bandSwitchByte = dtt75404_cpcSelect0 | dtt75404_bandSelectUHF;
    else if (frequency > 506000000 && frequency <= 761000000)
        registerMirror[index].bandSwitchByte = dtt75404_cpcSelect1 | dtt75404_bandSelectUHF;
    else if (frequency > 761000000 && frequency <= 846000000)
        registerMirror[index].bandSwitchByte = dtt75404_cpcSelect2 | dtt75404_bandSelectUHF;
    else
        registerMirror[index].bandSwitchByte = dtt75404_cpcSelect3 | dtt75404_bandSelectUHF;
    /* set bandwidth bit */
    registerMirror[index].bandSwitchByte |= ((bandwidth == FE_BANDWIDTH_8_MHZ) ?
                                                     dtt75404_bw8MHz :
                                                     dtt75404_bw7MHz);
    registerMirror[index].dividerByte1    = (uint8_t)((divider>>8) & 0xFF);
    registerMirror[index].dividerByte2    = (uint8_t)(divider & 0xFF);
    registerMirror[index].controlByte1 = CB1_FIXED_BITS |
                                                 ATP_109dBuV |
                                                 dtt75404_fref166kHz667;
    registerMirror[index].controlByte2 = CB2_FIXED_BITS |
                                                 atcHigh |
                                                 stbyActive |
                                                 xtalOutDisabled;
    
    /* start communication sequence to Tuner */
    do
    {
        /* Connect tuner to primary I2C bus */
        FE_DPrintf("[DTT75404] Connect tuner\n");
        ferr = CE6353_ConnectToTuner(index);
        if (ferr != FE_OK)
            break;

        FE_DPrintf("[DTT75404] Write direct to tuner\n");
        ferr = CE6353_WriteDirectToTuner(index, &(registerMirror[index].dividerByte1), 5);
        if (ferr != FE_OK) break;
        FE_DPrintf("[DTT75404] wait lock\n");
        ferr = CE6353_WaitTunerLock(index, 0);
        if (ferr != FE_OK) break;

        FE_DPrintf("[DTT75404] set tuner status\n");
        CE6353_SetTunerStatus(index, FE_STATUS_CARRIER);
    
        /* set RF-AGC current to a low value */
        registerMirror[index].controlByte2 = CB2_FIXED_BITS |
                                                     atcLow |
                                                 stbyActive |
                                            xtalOutDisabled ;
        FE_DPrintf("[DTT75404] Write direct to tuner 2\n");
        ferr = CE6353_WriteDirectToTuner(index, &(registerMirror[index].controlByte2), 1);
        if (ferr != FE_OK) break;
        /* Disconnect tuner from primary I2C bus */
        FE_DPrintf("[DTT75404] Disconnect tuner\n");
        ferr = CE6353_DisconnectTuner(index);
    } while (0);
    FE_DPrintf("[DTT75404]tuning returns %x\n", ferr);
    
    if(ferr != FE_OK)
        CE6353_DisconnectTuner(index);

    return ferr;
}

/* Version History */
/******************************************************************************
$Log: dtt75404.c,v $
Revision 1.1  2008/05/07 11:10:18  wlaris
Initial creation.

Revision 1.1  2008/05/07 08:39:06  wlaris
Initial creation.


******************************************************************************/
