/******************************************************************************
**
** \file      ./subsystems/fe/ce6353/src/td1316.c
**
** \brief     Front-end device driver for Intel OFDM CE6353 with Philips Tuner.
**
**            (C) Fujitsu Semiconductor Europe 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: td1316.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
// #define FE_FE_TD1316_DEBUG

#include "../src/fe_priv.h"
#include "ce6353_priv.h"

#include "td1316.h"

#ifdef FE_TD1316_DEBUG
    #include "fm_lib/fm_debug.h"
#endif


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
/* internal AGC take over point bit definitions */
#define     agcLoop0dB            REG_BIT_4
#define     agcLoopM3dB           REG_BIT_5
#define     agcLoopM6dB          (REG_BIT_5 | REG_BIT_4)
#define     agcLoopM9dB           REG_BIT_6
#define     agcLoopM12dB         (REG_BIT_6 | REG_BIT_4)
#define     agcLoopExternal      (REG_BIT_6 | REG_BIT_5)
#define     agcLoopDisabled      (REG_BIT_6 | REG_BIT_5 | REG_BIT_4)

#define     lowAgcCurrent        0
#define     highAgcCurrent       REG_BIT_7

/* Reference Frequency definitions */
#define     td1316_fref166kHz667 REG_BIT_2
#define     fref62kHz5     (REG_BIT_2 | REG_BIT_1)

/* charge pump selection */
#define     lowCP          (REG_BIT_5 | REG_BIT_4 | REG_BIT_3)
#define     mediumCP       (REG_BIT_6 | REG_BIT_5 | REG_BIT_4)
#define     highCP         (REG_BIT_6 | REG_BIT_5 | REG_BIT_4 | REG_BIT_3)
#define     auxByteEnabled (REG_BIT_4 | REG_BIT_3)

/* bandwidth and SAW selection */
#define     lowBand         REG_BIT_0
#define     midBand         REG_BIT_1
#define     highBand        REG_BIT_2
#define     saw7MHz         0
#define     saw8MHz         REG_BIT_3
#define     stdbyMode      (REG_BIT_1 | REG_BIT_0)

/* PLL Step Size */
#define     pll166kHz66     REG_BIT_2
#define     pll62kHz5      (REG_BIT_2 | REG_BIT_1)

#define     CDB_FIXED_BIT  (REG_BIT_7)

/* Read Register Masks */
#define     readMaskPOR     REG_BIT_7
#define     readLockFlag    REG_BIT_6
#define     readAgcFlag     REG_BIT_3

/*---------------------------------------------------------------------------*/
/* local types                                                             */
/*---------------------------------------------------------------------------*/
enum
{
    REG_BIT_0  = (0x01), 
    REG_BIT_1  = (0x02), 
    REG_BIT_2  = (0x04), 
    REG_BIT_3  = (0x08), 
    REG_BIT_4  = (0x10), 
    REG_BIT_5  = (0x20), 
    REG_BIT_6  = (0x40), 
    REG_BIT_7  = (0x80) 
};

/* ------- tuner register structure ---------------------------- */
typedef struct
{
     /* write registers */
    uint8_t dividerByte1;
    uint8_t dividerByte2;
    uint8_t controlDataByte;
    uint8_t switchPortByte;
    uint8_t auxiliaryByte;
    /* read register */
    uint8_t  statusByte;
} TUNER_I2C_DATA_S;

/*---------------------------------------------------------------------------*/
/* global data                                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static TUNER_I2C_DATA_S registerMirror[FE_MAX_NUM];

/*---------------------------------------------------------------------------*/
/* callback functions                                                        */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
int32_t td1316Close(uint8_t index);

/*
*******************************************************************************
**
** \brief   Shut down Philips TD1316 tuner
**
** \param   none
**
** \return Possible return codes:
**         - #FE_OK
**         - error values returned by #FE_I2C_Write()
**
** \sa
**
*******************************************************************************
*/
int32_t td1316Close(uint8_t index)
{
    int32_t ferr;

    registerMirror[index].switchPortByte = stdbyMode;
    ferr = CE6353_WriteToTuner(index, &(registerMirror[index].controlDataByte), 2);
    return (ferr);
}

/*
*******************************************************************************
**
** \brief  Tune Philips TD1316 tuner to given frequency
**
** \param   index           for frontend data structure
** \param   frequency       given in Hz
** \param   bandwidth       passed in Hz
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_FE_NOT_LOCKED
**         - error values returned by #FE_I2C_Write(), #FE_I2C_READ()
**
** \sa
**
*******************************************************************************
*/
int32_t td1316TuneToFrequency (uint8_t index, uint32_t frequency, uint32_t _bandwidth)
{
    int32_t              ferr = FE_OK;
    uint16_t               divider;
    uint8_t                writeBuffer[2];
    FE_BANDWIDTH_E bandwidth;

    /* frequency range should be in between 51.0MHz and 858 MHz */
    if(frequency > 858000000 || frequency < 51000000) return FE_ERR_BAD_PARAMETER;
    CE6353_SetTunerStatus(index, FE_STATUS_TUNING);
    
    if (_bandwidth == 6000000)
        bandwidth = FE_BANDWIDTH_6_MHZ;
    else if (_bandwidth == 7000000)
        bandwidth = FE_BANDWIDTH_7_MHZ;
    else if (_bandwidth == 8000000)
        bandwidth = FE_BANDWIDTH_8_MHZ;
    else
        return FE_ERR_BAD_PARAMETER;

    frequency += 36166667;      /* add IF frequency */
    frequency += 83333;         /* add on for rounding purposes */
    divider = (uint16_t)((((frequency)/10)*6)/100000);
    /* set internal frequency to the real value that the tuner is using */
    CE6353_StoreTunerFrequency(index, (divider * 166667) - 36166667);

    /*  initialise aux byte: set AGC current to high value and AGC loop to -3dB */
    registerMirror[index].controlDataByte = writeBuffer[0] = CDB_FIXED_BIT |
                                                            auxByteEnabled |
                                                               pll166kHz66;
    registerMirror[index].auxiliaryByte = writeBuffer[1] = highAgcCurrent | agcLoopM3dB;
    
    do
    {
        /* start communication sequence to Tuner */
        ferr = CE6353_WriteToTuner(index, &(writeBuffer[0]), 2);
        if(ferr != FE_OK)
        {
            CE6353_DisconnectTuner(index);
            break;
        }
        /* set PLL charge pump to low value for scan search */
        registerMirror[index].controlDataByte = CDB_FIXED_BIT  |
                                                        lowCP |
                                                        pll166kHz66;
        
        /* set SAW filter according to the selected band and bandwidth */
        if(frequency >= 84000000 && frequency <= 197000000)
            registerMirror[index].switchPortByte = lowBand;
        else if (frequency > 197000000 && frequency <= 484000000)
            registerMirror[index].switchPortByte = midBand;
        else
            registerMirror[index].switchPortByte = highBand;
        
        registerMirror[index].switchPortByte |= ((bandwidth == FE_BANDWIDTH_8_MHZ) ?
                                                         saw8MHz :
                                                         saw7MHz);
        
        registerMirror[index].dividerByte1 = (uint8_t)((divider>>8) & 0xFF);
        registerMirror[index].dividerByte2 = (uint8_t)(divider & 0xFF);
        
        /* start next communication sequence to Tuner */
        do
        {
            /* Connect tuner to primary I2C bus */
            ferr = CE6353_ConnectToTuner(index);
            if (ferr != FE_OK) break;
        
            ferr = CE6353_WriteDirectToTuner(index, &(registerMirror[index].dividerByte1), 4);
            if (ferr != FE_OK) break;
            ferr = CE6353_WaitTunerLock(index, 0);
            if (ferr != FE_OK) break;

            CE6353_SetTunerStatus(index, FE_STATUS_CARRIER);

            /* preset PLL charge pump value according to selected frequency range */
            if(frequency >= 84000000 && frequency <= 180000000)
                registerMirror[index].controlDataByte = CDB_FIXED_BIT  | lowCP | pll166kHz66;
            else if (frequency > 180000000 && frequency <= 197000000)
                registerMirror[index].controlDataByte = CDB_FIXED_BIT  | mediumCP | pll166kHz66;
            else if (frequency > 197000000 && frequency <= 366000000)
                registerMirror[index].controlDataByte = CDB_FIXED_BIT  | lowCP | pll166kHz66;
            else if (frequency > 366000000 && frequency <= 484000000)
                registerMirror[index].controlDataByte = CDB_FIXED_BIT  | mediumCP | pll166kHz66;
            else if (frequency > 484000000 && frequency <= 646000000)
                registerMirror[index].controlDataByte = CDB_FIXED_BIT  | lowCP | pll166kHz66;
            else if (frequency > 646000000 && frequency <= 790000000)
                registerMirror[index].controlDataByte = CDB_FIXED_BIT  | mediumCP | pll166kHz66;
            else
                registerMirror[index].controlDataByte = CDB_FIXED_BIT  | highCP | pll166kHz66;
        
            /* set RF-AGC current to a low value */
            /* take care that CP bit remains unchanged (refer to p13 of specification) */
            writeBuffer[0] = (registerMirror[index].controlDataByte & 0xC6) | auxByteEnabled;
            registerMirror[index].auxiliaryByte =   writeBuffer[1] = 
                                                            lowAgcCurrent | agcLoopM3dB;
            ferr = CE6353_WriteDirectToTuner(index, &(writeBuffer[0]), 2);
            if (ferr != FE_OK) break;
            /* write control data byte and switch port byte */
            ferr = CE6353_WriteDirectToTuner(index, &(registerMirror[index].controlDataByte), 2);
            /* Disconnect tuner from primary I2C bus */
            ferr = CE6353_DisconnectTuner(index);
        } while (0);
    } while (0);
   
    if(ferr != FE_OK)
        CE6353_DisconnectTuner(index);
    
    return ferr;
}

/* Version History */
/******************************************************************************
$Log: td1316.c,v $
Revision 1.1  2008/05/07 11:10:18  wlaris
Initial creation.

Revision 1.1  2008/05/07 08:39:06  wlaris
Initial creation.


******************************************************************************/

