/******************************************************************************
**
** \file      ./subsystems/fe/ce6353/src/td911.c
**
** \brief     Front-end device driver for Intel OFDM CE6353 with Thomson Tuner.
**
**            (C) Fujitsu Semiconductor Europe 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: td911.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
// #define FE_TD911_DEBUG

#include <string.h>

#include "../src/fe_priv.h"
#include "ce6353_priv.h"

#include "td911.h"

#ifdef FE_TD911_DEBUG
    #include "fm_lib/fm_debug.h"
#endif

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define FE_911_CB_CP075 0x04
#define FE_911_CB_CP150 0x08
#define FE_911_CB_CP300 0x10
#define FE_911_CB_CP600 0x20


/*---------------------------------------------------------------------------*/
/* local types                                                             */
/*---------------------------------------------------------------------------*/
typedef struct
{
    /* write registers, first byt of each reg is index!! */
    uint8_t  md[3];   // main divider
    uint8_t  cb[3];   // control bytes
    uint8_t  rd[3];   // reference divider
    uint8_t  ac[3];   // AGC control
    uint8_t  dc[3];   // DC-DC-Control
    uint8_t  dacs[4]; // DAC control
    uint8_t  sc[3];   // standby control

    /* read register */
    uint8_t  status[2];

    /* eeprom information */
    uint8_t eeprom[1024];

} TUNER_I2C_DATA_S;

/* ------- tuner register structure ---------------------------- */
/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static TUNER_I2C_DATA_S registerMirror[FE_MAX_NUM];

/*---------------------------------------------------------------------------*/
/* callback functions                                                        */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
int32_t td911Init(uint8_t index);
int32_t td911Close(uint8_t index);

/*
*******************************************************************************
**
** \brief   Initialise NXP tuner
**
** \param   none
**
** \return Possible return codes:
**         - #FE_OK
**         - error values returned by #FE_I2C_Write()
**
** \sa
**
*******************************************************************************
*/
int32_t td911Init(uint8_t index)
{
    uint16_t   i;
    int32_t  ferr = FE_OK;
    uint8_t dummy;
    int32_t hp = CE6353_Geti2cId(index);

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    /************** initialise **********/
    registerMirror[index].md[0]     = 0x00;
    registerMirror[index].md[1]     = 0x0B;
    registerMirror[index].md[2]     = 0x95;
    registerMirror[index].cb[0]     = 0x01;
    registerMirror[index].cb[1]     = 0x7C;
    registerMirror[index].cb[2]     = 0x21;
    registerMirror[index].rd[0]     = 0x02;
    registerMirror[index].rd[1]     = 0x30;
    registerMirror[index].rd[2]     = 0x18;
    registerMirror[index].ac[0]     = 0x03;
    registerMirror[index].ac[1]     = 0x00;
    registerMirror[index].ac[2]     = 0x42;
    registerMirror[index].dc[0]     = 0x04;
    registerMirror[index].dc[1]     = 0x06;
    registerMirror[index].dc[2]     = 0x06;
    registerMirror[index].dacs[0]   = 0x05;
    registerMirror[index].dacs[1]   = 0x00;
    registerMirror[index].dacs[2]   = 0x00;
    registerMirror[index].dacs[3]   = 0x00;
    registerMirror[index].sc[0]     = 0x06;
    registerMirror[index].sc[1]     = 0x00;
    registerMirror[index].sc[2]     = 0x00;
    registerMirror[index].status[0] = 0x80;
    registerMirror[index].status[1] = 0x00;

    #ifdef FE_TD911_DEBUG
    FM_Printf("[FE_OFDM] reg md = %04x\n", registerMirror[index].md[1] << 8 | registerMirror[index].md[2] );
    FM_Printf("[FE_OFDM] reg cb = %04x\n", registerMirror[index].cb[1] << 8 | registerMirror[index].cb[2] );
    FM_Printf("[FE_OFDM] reg rd = %04x\n", registerMirror[index].rd[1] << 8 | registerMirror[index].rd[2] );
    FM_Printf("[FE_OFDM] reg ac = %04x\n", registerMirror[index].ac[1] << 8 | registerMirror[index].ac[2] );
    FM_Printf("[FE_OFDM] reg dc = %04x\n", registerMirror[index].dc[1] << 8 | registerMirror[index].dc[2] );
    FM_Printf("[FE_OFDM] reg dacs = %04x\n", registerMirror[index].dacs[1] << 24 | registerMirror[index].dacs[2] << 8 | registerMirror[index].dacs[3] );
    FM_Printf("[FE_OFDM] reg sc = %04x\n", registerMirror[index].sc[1] << 8 | registerMirror[index].sc[2] );
    #endif

    ferr = CE6353_ConnectToTuner(index);
    if (ferr != FE_OK) return (ferr);

    CE6353_WriteDirectToTuner(index, registerMirror[index].md, 3);
    CE6353_WriteDirectToTuner(index, registerMirror[index].cb, 3);
    CE6353_WriteDirectToTuner(index, registerMirror[index].rd, 3);
    CE6353_WriteDirectToTuner(index, registerMirror[index].ac, 3);
    CE6353_WriteDirectToTuner(index, registerMirror[index].dc, 3);
    CE6353_WriteDirectToTuner(index, registerMirror[index].dacs, 4);
    CE6353_WriteDirectToTuner(index, registerMirror[index].sc, 3);

    // read in the EEPROm contents
    (void)FE_I2C_SetProtocol(hp, FE_I2C_COMMON_PROTOCOL);
    FE_I2C_Read(hp, 0xA9, &dummy, 0, registerMirror[index].eeprom    , 256 );
    FE_I2C_Read(hp, 0xAB, &dummy, 0, registerMirror[index].eeprom+256, 256 );
    FE_I2C_Read(hp, 0xAD, &dummy, 0, registerMirror[index].eeprom+512, 256 );
    FE_I2C_Read(hp, 0xAF, &dummy, 0, registerMirror[index].eeprom+768, 256 );
    (void)FE_I2C_SetProtocol(hp, FE_I2C_RESTART_PROTOCOL);

    ferr = CE6353_DisconnectTuner(index);
    if(ferr != FE_OK) return(ferr);

    #ifdef FE_TD911_DEBUG
    for( i = 0; i < 256; i += 16 )
    {
        FM_Printf( "eep[%04x]: ", i );
        for( k = 0; k < 16; k++ )
        {
            FM_Printf( "%02x,", registerMirror[index].eeprom[i+k] );
        }
        FM_Printf( "\n" );
    }
    #endif
    if( strcmp( (const char*)registerMirror[index].eeprom, "RFSolutionsTD911L" ) )
    {
        #ifdef FE_TD911_DEBUG
        FM_Printf( "[FE_OFDM] ***W: eeprom not valid\n" );
        #endif
        for (i = 0; i < 1024; i++)
            registerMirror[index].eeprom[i] = 0;
    }

    return ferr;
}

/*
*******************************************************************************
**
** \brief   Shut down NXP tuner
**
** \param   none
**
** \return Possible return codes:
**         - #FE_OK
**         - error values returned by #FE_I2C_Write()
**
** \sa
**
*******************************************************************************
*/
int32_t td911Close(uint8_t index)
{
    int32_t        ferr = FE_OK;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    /* put NXP tuner into Power Down mode */
    registerMirror[index].sc[1] = 0x07;
    registerMirror[index].sc[2] = 0xFC;
    ferr = CE6353_WriteToTuner(index, &(registerMirror[index].sc[0]), 3);

    return ferr;
}

/*
*******************************************************************************
**
** \brief  Tune Philips TD911 tuner to given frequency
**
** \param   index           for frontend data structure
** \param   frequency       given in Hz
** \param   bandwidth       passed as #FE_BANDWIDTH_E
**
** \return Possible return codes:
**         - #FE_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_FE_NOT_LOCKED
**         - error values returned by #FE_I2C_Write(), #FE_I2C_READ()
**
** \sa
**
*******************************************************************************
*/
int32_t td911TuneToFrequency (uint8_t index, uint32_t f, uint32_t _bandwidth)
{
    int32_t        ferr = FE_OK;
    uint32_t div;  // main divider
    uint32_t cb;
    const uint8_t * p;
    #ifdef FE_TD911_DEBUG
    uint16_t k;
    #endif

    /************** initialise **********/

    #ifdef FE_TD911_DEBUG
    FM_Printf( "[FE_OFDM] tune( %d, %d )\n", f, _bandwidth );
    #endif
    if( f >= 174000000 && f <= 230000000 )
    {
      cb   = 0x0281; // VHF
    }
    else if( f >= 474000000 && f <= 858000000 )
    {
      cb   = 0x7C01; //UHF
    }
    else
    {
      #ifdef FE_TD911_DEBUG
      FM_Printf( "[FE_OFDM] ***E: tune: invalid frequency: %d\n", (uint32_t)ferr );
      #endif
      return FE_ERR_BAD_PARAMETER;
    }

    // ---------------------------- calculate the main divider
    div = (f + 36166667 + (166667/2)) / 166667;
    registerMirror[index].md[1] = (div >> 8) & 0xFF;
    registerMirror[index].md[2] = (div     ) & 0xFF;

    // ---------------------------- control bytes
    if     ( f < 200000000 ) cb |= 4 << 2; // 300uA
    else if( f < 250000000 ) cb |= 5 << 2; // 450uA
    else if( f < 474000000 ) cb |= 2 << 2; // 150; // UHF
    else if( f < 518000000 ) cb |= 4 << 2; // 300;
    else if( f < 590000000 ) cb |= 6 << 2; // 450;
    else if( f < 662000000 ) cb |= 8 << 2; // 600;
    else if( f < 704000000 ) cb |=10 << 2; // 750;
    else if( f < 758000000 ) cb |=13 << 2; // 975;
    else if( f < 798000000 ) cb |=14 << 2; // 1050;
    else                     cb |=15 << 2; // 1125;

    registerMirror[index].cb[1] = (cb >> 8) & 0xFF;
    registerMirror[index].cb[2] = (cb     ) & 0xFF;

    // ---------------------------- DACs
    p = registerMirror[index].eeprom;
    registerMirror[index].dacs[1] = 0;
    registerMirror[index].dacs[2] = 0;
    registerMirror[index].dacs[3] = 0;

    for( p = (registerMirror[index].eeprom + 0x30); (p[0]|p[1]|p[2]|p[3]) != 0; p+=16 )
    {
      uint32_t f1, f2, step;
      int addr;
      f1 = p[0] << 24 | p[1] << 16 | p[2] << 8 | p[3];
      f2 = p[4] << 24 | p[5] << 16 | p[6] << 8 | p[7];
      if( f >= f1 && f <= f2 )
      {
        step = p[8] << 24 | p[9] << 16 | p[10] << 8 | p[11];
        if( 0 == step )
        {
            #ifdef FE_TD911_DEBUG
            FM_Printf( "***E: invalid EEPROM, step = 0!\n" );
            #endif
            break;
        }
        addr = (f - f1) / step * p[12] * p[13] + (p[14] << 8 | p[15]);
        #ifdef FE_TD911_DEBUG
        FM_Printf( "--- dac index: %04x\n", addr );
        #endif
        registerMirror[index].dacs[3] = registerMirror[index].eeprom[addr+0];
        registerMirror[index].dacs[2] = registerMirror[index].eeprom[addr+1];
        registerMirror[index].dacs[1] = registerMirror[index].eeprom[addr+2]; /* changed in revision 1.2 */
        break;
      }
    }

    #ifdef FE_TD911_DEBUG
    FM_Printf( "    md = %04x, cb = %04x, dac = %02x,%02x,%02x\n"
             , registerMirror[index].md[1] << 8 | registerMirror[index].md[2]
             , registerMirror[index].cb[1] << 8 | registerMirror[index].cb[2]
             , registerMirror[index].dacs[1], registerMirror[index].dacs[2], registerMirror[index].dacs[3]
             );
    #endif
    ferr = CE6353_ConnectToTuner(index);
    if (ferr != FE_OK) return (ferr);

    CE6353_WriteDirectToTuner(index, registerMirror[index].md, 3);
    CE6353_WriteDirectToTuner(index, registerMirror[index].cb, 3);
    CE6353_WriteDirectToTuner(index, registerMirror[index].dacs, 4);

    CE6353_WaitTunerLock(index, 1);
    ferr = CE6353_DisconnectTuner(index);
    return ferr;
}

/* Version History */
/******************************************************************************
$Log: td911.c,v $
Revision 1.2  2008/05/07 12:27:59  wlaris
Corrected wrong function signature.

Revision 1.1  2008/05/07 11:10:18  wlaris
Initial creation.

Revision 1.1  2008/05/07 08:39:06  wlaris
Initial creation.


******************************************************************************/
