/*!
********************************************************************************
**
** \file        drv_i2c.h
**
** \brief       I2C Driver.
**
** This file contains the declaration of the I2C driver API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2008 by Fujitsu Semiconductor Europe GmbH
**
********************************************************************************
*/

#ifndef FAPI_I2C_H
    #define FAPI_I2C_H


    #ifdef __cplusplus
    extern "C" {
    #endif


                                      
    /*!
    ********************************************************************************
    ** \brief Read data from an I2C channel
    **
    **         This function reads data from an I2C block.
    **         Two modes are supported:
    **         - Direct receive of \c rxDataLength bytes (set \c txDataLength = 0).
    **         - First transmit \c txDataLength bytes, then restart and
    **           receive \c rxDataLength bytes.
    **
    ** \param handle          Valid driver instance handle previously returned by
    **                        #FAPI_I2C_Open.
    ** \param  msSlaveAddress Slave address of external I2C device.
    ** \param  txDataPtr      Pointer to data to be transmitted to an I2C device.
    ** \param  txDataLength   Length of data to be transmitted in bytes.
    ** \param  rxDataPtr      Pointer to data to be received from I2C device.
    ** \param  rxDataLength   Length of data to be received from an I2C device.
    **
    ** \return Possible return codes:
    **         - \c #FAPI_OK on success.
    **         - \c #FAPI_ERR_INVALID_HANDLE
    **         - \c #FAPI_ERR_BAD_PARAMETER
    **         - \c #FAPI_I2C_ERR_BUSY
    **         - \c #FAPI_I2C_ERR_BUS_OCCUP
    **         - \c #FAPI_ERR_TIMEOUT
    **
    ** \sa FAPI_I2C_Write           <br>
    **     FAPI_I2C_Process         <br>
    **     FAPI_I2C_CancelProcess   <br>
    **
    ********************************************************************************
    */                                  
    int32_t          FAPI_I2C_Read(const FAPI_SYS_HandleT handle,
                                   const uint8_t msSlaveAddress,
                                   uint8_t* txDataPtr,
                                   const uint32_t txDataLength,
                                   uint8_t* rxDataPtr,
                                   uint32_t rxDataLength);
    /*!
    ********************************************************************************
    **
    ** \brief  Write data to I2C channel.
    **
    **         This function writes data to an I2C block.
    **
    ** \param handle          Valid driver instance handle previously returned by
    **                        #FAPI_I2C_Open.
    ** \param  msSlaveAddress Slave address of external I2C device.
    ** \param  txDataPtr      Pointer to data to be transmitted to I2C device
    **                        including subaddress if needed.
    ** \param  txDataLength   Length of data to be transmitted.
    **
    ** \return Possible return codes:
    **         - \c #FAPI_OK on success.
    **         - \c #FAPI_ERR_INVALID_HANDLE
    **         - \c #FAPI_ERR_BAD_PARAMETER
    **         - \c #FAPI_I2C_ERR_BUSY
    **         - \c #FAPI_I2C_ERR_BUS_OCCUP
    **         - \c #FAPI_ERR_TIMEOUT
    **
    ** \sa     FAPI_I2C_CancelProcess() <br>
    **         FAPI_I2C_Process         <br>
    **         FAPI_I2C_Read()          <br>
    **
    ********************************************************************************
    */                               
    int32_t          FAPI_I2C_Write(const FAPI_SYS_HandleT handle,
                                    const uint8_t msSlaveAddress,
                                    uint8_t* txDataPtr,
                                    const uint32_t txDataLength);

    #ifdef __cplusplus
    }
    #endif
#endif
