/****************************************************************************/
/*                   MONTAGE PROPRIETARY AND CONFIDENTIAL                   */
/*                   Montage Technology (Shanghai) Inc.                     */
/*                          All Rights Reserved                             */
/****************************************************************************/
/*
 * Filename:      mt_fe_tn_montage_ts2020.c
 *
 * Description:   Montage M88TS2020 Digital Satellite Tuner IC driver.
 *                
 *
 * Author:        Daniel Zhu
 * 
 * Version:       1.09-2
 * Date:          2009.7.14
/****************************************************************************/


#include "mt_fe_def.h"
#include "../src/fe_priv.h"
#if (MT_FE_TN_SUPPORT_TS2020 > 0)

#include "mt_fe_i2c.h"


#define MT_FE_CRYSTAL_FREQ_MHZ_TS2020        27


extern void _mt_sleep(U32 ms);
extern U8 feIndex;

static S32    g_tuner_freq_offset_KHz[2] = {0,0};


#define AVG_NUM   16          
#define SIGNAL_STRENGTH_RATIO    0xFFFF


/*****************************************************************
** Function: mt_fe_tn_init_ts2020
**
**
** Discription:    This function is used to initial ts2020. 
**
**
** Inputs:   None
**
** Outputs:  None
**
******************************************************************/
MT_FE_RET mt_fe_tn_init_ts2020(void)
{ 
    U8 buf;
    
    #if(M88TS2020_CKOUT_ON != 0)
        _mt_fe_tn_set_reg(0x42, 0x73);

        buf = M88TS2020_CKOUT_DIV;
        _mt_fe_tn_set_reg(0x05, buf);
    #else
        _mt_fe_tn_set_reg(0x42, 0x63);
    #endif


    #if(M88TS2020_RFBYPASS_ON != 0)
        _mt_fe_tn_set_reg(0x62, 0xf5);
    #else
        _mt_fe_tn_set_reg(0x62, 0xb5);
    #endif


    return MtFeErr_Ok;
}




/*************************************************************************
** Function: mt_fe_tn_get_tuner_freq_offset_ts2020
**
**
** Discription: return the tuner freq. offset, unit: KHz
**
**
** Inputs: none
**
**
** Outputs: none
**
**
*************************************************************************/
S32 mt_fe_tn_get_tuner_freq_offset_ts2020(void)
{
    return g_tuner_freq_offset_KHz[feIndex];
}


/*****************************************************************
** Function: mt_fe_tn_init_ts2020
**
**
** Discription:    This function is used to initial ts2020. 
**
**
** Input:
**
**   Parameter          Type        Description
**  ----------------------------------------------------------
**   freq_KHz           U32         from 950000 to 2150000
**   sym_rate_KSs       U32         from 1000 to 45000
**   lpf_offset_KHz     S16         low pass filter offset 
**
** Outputs:     None
**
******************************************************************/
MT_FE_RET mt_fe_tn_set_freq_ts2020(U32 freq_KHz, U32 sym_rate_KSs, S16 lpf_offset_KHz)
{
    U8      buf, div4, changePLL, K, lpf_mxdiv, divMax, divMin;
    U16     gdiv28;
    U32     N, lpf_gm, f3dB;


    //Initialize the tuner
    mt_fe_tn_init_ts2020();
    

    _mt_fe_tn_set_reg(0x07, 0x02);
    _mt_fe_tn_set_reg(0x10, 0x00);
    _mt_fe_tn_set_reg(0x60, 0x79);
    _mt_fe_tn_set_reg(0x08, 0x01);
    _mt_fe_tn_set_reg(0x00, 0x01);


    div4 = 0;
    changePLL = 0;

    
    //  set the PLL
     K = (U8)(MT_FE_CRYSTAL_FREQ_MHZ_TS2020/2 - 8);

    if(freq_KHz < 1146000)
    {
        _mt_fe_tn_set_reg(0x10, 0x11);
        div4 = 1;

        N = (U32)(((freq_KHz/1000) * (K + 8) * 8 / MT_FE_CRYSTAL_FREQ_MHZ_TS2020 + 1) / 2 - 1024);
    }
    else 
    {
        _mt_fe_tn_set_reg(0x10, 0x01);
        
        N = (U32)(((freq_KHz/1000) * (K+8) * 4 / MT_FE_CRYSTAL_FREQ_MHZ_TS2020 + 1) / 2 - 1024);
    }

    buf = (N >>8) & 0x0f;
    _mt_fe_tn_set_reg(0x01, buf);

    buf = N & 0xff;
    _mt_fe_tn_set_reg(0x02, buf);

    buf = K;
    _mt_fe_tn_set_reg(0x03, buf);
    _mt_fe_tn_set_reg(0x51, 0x0f);
    _mt_fe_tn_set_reg(0x51, 0x1f);
    _mt_fe_tn_set_reg(0x50, 0x10);
    _mt_fe_tn_set_reg(0x50, 0x00);

    _mt_sleep(5);

    _mt_fe_tn_get_reg(0x66, &buf);
    changePLL = (((buf & 0x80) >> 7) != div4); 
    if(changePLL)
    {
        _mt_fe_tn_set_reg( 0x10, 0x11);

        div4 = 1;

        N = (U32)(((freq_KHz/1000) * (K + 8) * 8 / MT_FE_CRYSTAL_FREQ_MHZ_TS2020 + 1) / 2 - 1024);
        buf = (N >>8) & 0x0f;
        _mt_fe_tn_set_reg(0x01, buf);

        buf = N & 0xff;
        _mt_fe_tn_set_reg(0x02, buf); 

        _mt_fe_tn_set_reg(0x51, 0x0f);
        _mt_fe_tn_set_reg(0x51, 0x1f);  
        _mt_fe_tn_set_reg(0x50, 0x10);
        _mt_fe_tn_set_reg(0x50, 0x00);
        _mt_sleep(5);
    }


    _mt_fe_tn_set_reg(0x51, 0x17);  
    _mt_fe_tn_set_reg(0x51, 0x1f);   
    _mt_fe_tn_set_reg(0x50, 0x08);       
    _mt_fe_tn_set_reg(0x50, 0x00); 

    _mt_sleep(5);

    _mt_fe_tn_get_reg(0x3d, &buf);
    buf &= 0x0f;
    if(buf < 15)
    {
        if(buf < 4)  buf = 0;
        else         buf = buf - 3;

        buf = ((buf << 3) | 0x01) & 0x79;
        _mt_fe_tn_set_reg(0x60, buf);

        _mt_fe_tn_set_reg(0x51, 0x17);
        _mt_fe_tn_set_reg(0x51, 0x1f);
        _mt_fe_tn_set_reg(0x50, 0x08);
        _mt_fe_tn_set_reg(0x50, 0x00);
    }
    

    f3dB = sym_rate_KSs * 4 / 5 + 2000;
    if(sym_rate_KSs < 5000)   f3dB += lpf_offset_KHz;
    if(f3dB < 7000 )    f3dB = 7000;
    if(f3dB > 40000)    f3dB = 40000;



    gdiv28 = (U16)((MT_FE_CRYSTAL_FREQ_MHZ_TS2020 * 1694 + 500) / 1000);
    buf = (U8)gdiv28;
    _mt_fe_tn_set_reg(0x04, buf); 


    _mt_fe_tn_set_reg(0x51, 0x1b);
    _mt_fe_tn_set_reg(0x51, 0x1f);
    _mt_fe_tn_set_reg(0x50, 0x04);
    _mt_fe_tn_set_reg(0x50, 0x00); 

    _mt_sleep(5);

    _mt_fe_tn_get_reg(0x26, &buf);
    gdiv28 = gdiv28 * 207 / (buf * 2 + 151);

    divMax = gdiv28 * 135 / 100;                           
    divMin = gdiv28 * 78 / 100;

    if(divMax>63)   divMax=63;

    lpf_gm = (U32)((f3dB * gdiv28 * 2 / 2766 / MT_FE_CRYSTAL_FREQ_MHZ_TS2020  + 1) / 2) ;

    if (lpf_gm>23)  lpf_gm = 23;
    if (lpf_gm<1)   lpf_gm = 1;

    lpf_mxdiv = (U8)((lpf_gm * MT_FE_CRYSTAL_FREQ_MHZ_TS2020 * 2766 * 2 / f3dB + 1) / 2);

    if (lpf_mxdiv<divMin) 
    {
        lpf_gm++;
        lpf_mxdiv = (U8)((lpf_gm * MT_FE_CRYSTAL_FREQ_MHZ_TS2020 * 2766 * 2 / f3dB + 1) / 2);
    }

    if (lpf_mxdiv>divMax)
        lpf_mxdiv=divMax;

    buf = lpf_mxdiv;
    _mt_fe_tn_set_reg(0x04, buf);

    buf = lpf_gm;
    _mt_fe_tn_set_reg(0x06, buf);

    _mt_fe_tn_set_reg(0x51, 0x1b);
    _mt_fe_tn_set_reg(0x51, 0x1f);
    _mt_fe_tn_set_reg(0x50, 0x04);
    _mt_fe_tn_set_reg(0x50, 0x00);

    _mt_sleep(5);

    _mt_fe_tn_set_reg(0x51, 0x1e);
    _mt_fe_tn_set_reg(0x51, 0x1f);
    _mt_fe_tn_set_reg(0x50, 0x01);
    _mt_fe_tn_set_reg(0x50, 0x00);
    _mt_sleep(60);


    _mt_fe_tn_get_reg(0x21, &buf);
    buf &= 0x0f;
    if(buf < 3)
    {
        _mt_fe_tn_get_reg(0x3d, &buf);
        buf &= 0x0f;
        if(buf == 15)
        {
            _mt_fe_tn_set_reg(0x60, 0x61);
            _mt_fe_tn_set_reg(0x51, 0x17);   
            _mt_fe_tn_set_reg(0x51, 0x1f);   
            _mt_fe_tn_set_reg(0x50, 0x08);       
            _mt_fe_tn_set_reg(0x50, 0x00);
        }
    }



    g_tuner_freq_offset_KHz[feIndex] = (S32)((N - N%2 + 1024) * (MT_FE_CRYSTAL_FREQ_MHZ_TS2020*1000) / (K + 8)/(div4 + 1) / 2 - freq_KHz);

    return MtFeErr_Ok;
}



/*****************************************************************
** Function: mt_fe_tn_sleep_ts2020
**
**
** Discription:    This function is used to set tuner in sleep mode. 
**
**
** Inputs:   None
**
** Outputs:  None
**
**
******************************************************************/
MT_FE_RET mt_fe_tn_sleep_ts2020(void)
{
    _mt_fe_tn_set_reg(0x00, 0x00); 
    _mt_sleep(50);
    
    return MtFeErr_Ok;
}



/*****************************************************************
** Function: mt_fe_tn_wake_up_ts2020
**
**
** Discription:    This function is used to wake up ts2020. 
**
**
** Inputs:   None
**
** Outputs:     None
**
**
******************************************************************/
MT_FE_RET mt_fe_tn_wake_up_ts2020(void)
{
    _mt_fe_tn_set_reg(0x00, 0x01);
    _mt_sleep(50);
    
    return MtFeErr_Ok;
}


/*****************************************************************
** Function: mt_fe_tn_get_tuner_gain
**
**
** Discription:
**
**
** Inputs:   vagc
**
**
**
******************************************************************/
U16 mt_fe_tn_get_tuner_gain(U16  vagc)
{

    U8  gain1,gain2;
    U16 gain;

    _mt_fe_tn_get_reg(0x3d, &gain1);
    _mt_fe_tn_get_reg(0x21, &gain2);
    gain1 &= 0x1f;  
    gain2 &= 0x1f;

    if(gain1<0)     gain1 = 0;
    if(gain1>15)    gain1 = 15;      
    if(gain2<0 )    gain2 =  0;
    if(gain2>13)    gain2 = 13;

    if(vagc < 400)       vagc = 400;     
    if(vagc > 1100)      vagc = 1100;

    gain = (U16) gain1 * 233 + (U16) gain2 * 350 + vagc * 24 / 10; 

    return gain; 
}


/*****************************************************************
** Function: mt_fe_tn_get_strength_ts2020
**
**
** Discription:    This function is used to get signal strength. 
**
**
** Inputs:   None
**
** Outputs:
**
**   Parameter      Type            Description
**  ----------------------------------------------------------
**   p_strength     S8*             strength value pointer
**
**
**Note:
**  This is a demo function to calculate the signal strength use M88TS2020GetTunerGain
**  This is defined for applications using Montage DVB-S2 demodulator M88DS3000
**  This function should be defined by users if they use a different type of demodulator
**  Return: the signal strength          0% to 100%
******************************************************************/
MT_FE_RET mt_fe_tn_get_strength_ts2020(S8 *p_strength)
{ 
    U8      i, strength, AgcPWM;
    U16     sum, gain;
    S16     Vagc;
    MT_FE_LOCK_STATE lockstate;

    static U8 signal_strength[AVG_NUM];
    static U8 signal_index = 0;

    // Step 1: read the AGC PWM rate from the demodulator register
    _mt_fe_dmd_get_reg(0x3f, &AgcPWM);

    // Step 2: Calculate the AGC voltage based on the AGC PWM rate
    Vagc = AgcPWM * 20 - 1166;                  //Convert the AGC PWM rate to AGC Voltage,unit: mV

    if(Vagc < 0) Vagc = 0;

    // Step 3: Calculate the total gain of the tuner
    gain = mt_fe_tn_get_tuner_gain(Vagc);       //Get the total tuner gain

    // Step 4: Calculate the signal strength based on the total gain of the tuner
    if(gain > 7300)         strength =  0;                              //0%           no signal or weak signal
    else if(gain > 5800)    strength = 0 + (7300 - gain)  * 13 / 300;   //0% - 65%     weak signal
    else if(gain > 4200)    strength = 65 + (5800 - gain) * 1 / 80;     //65% - 85%    normal signal
    else                    strength = 85 + (4200 - gain) * 3 / 580;    //85% - 100%   strong signal

    // Step 5: Adjust the display of the signal strength according to your requirements
    mt_fe_dmd_ds3k_get_lock_state(&lockstate);        // Get the lock status of the demodulator M88DS3000;

    // when the channel is locked, set the signal strength bigger
    if(strength < 40 && lockstate == MtFeLockState_Locked)      
        strength = 20 + strength / 2;


    //smooth the display of signal strength when the signal fluctuates, average AVG_NUM times
    signal_strength[signal_index] = strength;   
    signal_index++;
    if (signal_index == AVG_NUM)
        signal_index = 0;

    sum = 0;
    for (i = 0; i < AVG_NUM; i++)
        sum += signal_strength[i];
    strength = sum / AVG_NUM;

    //Scale the display by multiplying the signal strength with a coefficient
    strength = (U16)strength * SIGNAL_STRENGTH_RATIO / 100; 
    
    //Limit the display within 0% to 100% to avoid errors
    if(strength > 100)  strength = 100;        
    if(strength < 0)    strength = 0;
    
    *p_strength = strength;

    return MtFeErr_Ok;    
}


#endif/*#if (MT_FE_TN_SUPPORT_TS2020 > 0)*/

