/******************************************************************************
**
** \file      ./subsystems/fe/mb86667/src/MB86667_Api.h
**
** \brief     Public functions of MB86667 driver.
**
**            (C) Fujitsu Semiconductor Europe 2006 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: MB86667_Api.h 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
/*-------- compilation control switches -----------------------------*/
#ifndef MB86667_API_H
#define MB86667_API_H

#define USE_WITHIN_FAPI /* settings used for FAPI environment */

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "mb86667_types.h"


/*-------- type definitions -----------------------------------------*/


typedef enum
{                 LOW   = 0,
                  HIGH  = 1
} FAPI_MB86667_BAND_E;

typedef enum
{                 SET = 0,
                  GET = 1
} FAPI_MB86667_GETSET_E;

/* parameter types */
enum
{   POL_HOR         = 0x00,
    POL_VER         = 0x01,
    CIRC_LEFT       = 0x02, /* currently not supported */
    CIRC_RIGHT      = 0x03  /* currently not supported */
};

enum
{   MB86667_IQ_INV      = 0x04,
    MB86667_AUTO_IQ     = 0x08
};

enum
{   FEC_1_2         = 0x01,
    FEC_2_3         = 0x02,
    FEC_3_4         = 0x03,
    FEC_5_6         = 0x04,
    FEC_7_8         = 0x05,
    FEC_AUTO        = 0x0F
};

typedef enum
{
    NO_DIS_FLAG     = 0x00,
    CLASSIC_CTR     = 0x01, /* enables 22KHz continuous tone and 13/18 V */
    MINI_DISEQC     = 0x02, /* enables tone burst to select satellite */
    DISEQC1_0       = 0x04, /* full DiSEqC 1.0 (LNB power is set to 13V if CONT_TONE and MINI_DISEQC are off */
    DISEQC1_1DTH    = 0x08, /* DiSEqC 1.1 for DTH */
    DISEQC1_1SMA    = 0x10, /* DiSEqC 1.1 for SMATV */
    DISEQC1_2       = 0x20, /* DiSEqC 1.1 for SMATV */
    LNB_POWER       = 0x80  /* enables LNB power, if this is off, no LNB control is possible */
} FAPI_MB86667_DIS_FLAGS_E;

/* ------- start parameters -------------------------------------------- */
typedef struct
{   FAPI_MB86667_DIS_FLAGS_E flags;        /* see also enum type above
                                             bit0: H/V selection: Classic (22kHz, 14/18V)
                                             bit1: miniDiseqc (toneburst), bit0 must be set in addition to select H/V, Hi-Low
                                             bit2: Diseqc 1.0
                                             bit3: DiSeqC 1.1 for DTH
                                             bit4: DiSeqC 1.1 for SMATV
                                             bit5: DiSeqC 1.2 for Motor control (only "Goto position" implemented)
                                             bit6: reserved
                                             bit7: LNB power */
    uint8_t                   RepComSw;      /* Number of repetitions for committed switches (H/V, Hi/Lo, Pos and Opt)
                                             for DiSEqC 1.0 (0 means cmd is sent once) */
    uint8_t                   RepUncSw;      /* Number of repetitions for uncommitted switches (S1, S2, S3 and S4)
                                             for DiSEqC 1.1 DTH only (0 means cmd is sent once) */
}Downlink_Type;

typedef struct
{   uint32_t           LNBlowfreq;     /* LNBs lower oscillator frequency in KHz, in case of SMATVFreq
                                               this entry is used to hold the SMATV IF frequency */
    uint32_t           LNBhighfreq;    /* LNBs higher oscillator frequency in KHz */
    uint16_t           satID;          /*  0..63 for Diseqc switch
                                        ID   Tone burst      Pos     Option  S1  S2  S3  S4
                                        0   0               A       A
                                        1   1               B       A
                                        2   -               A       B
                                        3   -               B       B
                                      ------- only if DiSEqC 1.1 for DTH is enabled -------
                                        0..3                s.a.    s.a.    A   A   A   A
                                        4..7                s.a.    s.a.    B   A   A   A
                                        8..11               s.a.    s.a.    A   B   A   A
                                                 ...
                                        60..63              s.a.    s.a.    B   B   B   B */
}MB86667_SATELLITE_T;

typedef struct
{   uint16_t          symbolrate;     /* In kSamples / s */
    uint32_t          frequency;      /* Transponder frequency in kHz */
    uint8_t           fec;            /* FEC_1_2...FEC_7_8, FEC_AUTO  */
    uint16_t          params;         /* bits[0,1]: Polarisation
                                      bit2: IQ Inversion
                                      bit3: AutoIQ */
}MB86667_TRANSPONDER_T;

/* ------- required includes --------------------------------------- */

/* ------- limits -------------------------------------------------- */
#define MB86667_MIN_FREQ  950000
#define MB86667_MAX_FREQ 2240000
#define MB86667_BAND_SW  1950000 /* switch frequency from low- to high-band */
#define MB86667_MIN_SYM     1000
#define MB86667_MAX_SYM    45000


#ifdef  __cplusplus
extern "C" {
#endif

/* ------- public MB86667 API functions -------------------------------- */
/*-------- basic API functions ------------------------------------- */
int32_t    MB86667_DisBer(uint8_t bersel);
int32_t    MB86667_GetBer(uint32_t *pulErrors, uint8_t bersel);
int32_t    MB86667_GetLock(int8_t *scStatus);
int32_t    MB86667_GetFEC(int8_t *ucFEC);
int32_t    MB86667_GetSigLevel(uint8_t *ucLevel);
int32_t    MB86667_GetSigQual(uint8_t *ucQuality);
int32_t    MB86667_GetSRErr(int32_t *iSRE);
int32_t    MB86667_GetAFC(int32_t *iAFC);
int32_t    MB86667_Init(Downlink_Type *pDownlink);
int32_t    MB86667_RegReset(void);
int32_t    MB86667_SendDiseqcCommand( uint8_t  length,
                                   uint8_t* pCommands,
                                   uint8_t* pReturnLength,
                                   uint8_t* pReturnBuffer);
int32_t    MB86667_SetAntennaPower(FBOOL power);
int32_t    MB86667_Tune(MB86667_TRANSPONDER_T *pTransponder, MB86667_SATELLITE_T *pSatellite, FBOOL quicklock);

#ifdef  __cplusplus
}
#endif

#endif         /* MB86667_API_H */

