/******************************************************************************
**
** \file      ./subsystems/fe/mb86667/src/mb86667.c
**
** \brief     Front-end device driver for Fujitsu MB86667 sat dvbs frontend.
**
**            (C) Fujitsu Semiconductor Europe 2002 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: mb86667.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
//#include "../src/fe_priv.h"
#include "MB86667_Api.h"
#include "MB86667_Customise.h"
#include "MB86667_Tuner.h"
#include "MB86667_Scan.h"
#ifdef FAPI_FEMB86667_DEBUG01
   #include "fapex/abst_types.h"
#endif



/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static FE_MB86667_RF_INI_FCT_T tempRfIniFct = NULL, rf_ini_fct[FE_MAX_NUM];
static FE_MB86667_RF_SET_FCT_T tempRfSetFct = NULL, rf_set_fct[FE_MAX_NUM];
static feHandleT            *pData[FE_MAX_NUM];


/*---------------------------------------------------------------------------*/
/* callback functions                                                        */
/*---------------------------------------------------------------------------*/
int32_t FE_MB86667_Open(uint8_t index, FE_HANDLE *pDeviceData);
int32_t FE_MB86667_Close(uint8_t index);
int32_t FE_MB86667_SetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams);
int32_t FE_MB86667_Reset(uint8_t index);
int32_t FE_MB86667_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams);
int32_t FE_MB86667_GetStatus(uint8_t index, FE_STATUS_E* pStatus);
int32_t FE_MB86667_GetAFC(uint8_t index, int32_t* pAFC);
int32_t FE_MB86667_GetBER(uint8_t index, uint32_t* pBER);
int32_t FE_MB86667_GetSignalQuality(uint8_t index, uint16_t* pQuality);
int32_t FE_MB86667_GetSignalStrength(uint8_t index, uint16_t* pStrength);
int32_t FE_MB86667_GetTuneParams(uint8_t index, FE_TUNE_PARAMS_S *pTuneParams);
int32_t FE_MB86667_SendDiseqcCommand(uint8_t  index, 
                                 uint8_t  length, 
                                 uint8_t* pCommands, 
                                 uint8_t* pReturnLength, 
                                 uint8_t* pReturnBuffer);
int32_t FE_MB86667_GetFEC(uint8_t index, FE_FEC_E* pFEC);
int32_t FE_MB86667_Scan(uint8_t index, FE_SCAN_PARAMS_S* pScanParams);

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
Name:        mb86667_fillDownlinkType
-------------------------------------------------------------------------------*/
Downlink_Type fe_mb86667_fillDownlinkType(FE_SYSTEM_PARAMS_S* pSystemParams)
{
    Downlink_Type dlt;

    dlt.RepComSw = (uint8_t)pSystemParams->sat.diseqc10Rep;
    dlt.RepUncSw = (uint8_t)pSystemParams->sat.diseqc11Rep;

    dlt.flags = NO_DIS_FLAG;
    if (pSystemParams->sat.classic == FTRUE)   dlt.flags |= CLASSIC_CTR;
    if (pSystemParams->sat.toneburst == FTRUE) dlt.flags |= MINI_DISEQC;
    dlt.flags |= LNB_POWER;
    switch(pSystemParams->sat.diseqc)
    {
        case FE_DISEQC_1_0: dlt.flags |= DISEQC1_0; break;
        case FE_DISEQC_1_1_DTH: dlt.flags |= DISEQC1_1DTH; break;
        case FE_DISEQC_1_1_SMATV: dlt.flags |= DISEQC1_1SMA; break;
        case FE_DISEQC_1_2: dlt.flags |= (DISEQC1_0|DISEQC1_2); break;
        case FE_DISEQC_2_0: break;
        default: break;
    }

    return dlt;
}

int32_t fe_mb86667CheckChipId(feHandleT *hp, uint8_t i2cAddress)
{
    uint8_t i2cBuffer[2];
    int32_t ferr;
    
    i2cBuffer[0] = 0x7F;
    i2cBuffer[1] = 0xFF;
    ferr = FE_I2C_Read(hp->i2cId, i2cAddress, &(i2cBuffer[0]), 1,
                                              &(i2cBuffer[1]), 1);
    FE_DPrintf("Checking for MB86667 on address %02x ferr %x read %x\n", i2cAddress, ferr, i2cBuffer[1]);
    if(ferr != FE_OK)
        return ferr;
    if(i2cBuffer[1] != 0xFD)
        return -1; 
    return FE_OK;
}

void fe_mb86667SetFunctions(feHandleT *hp)
{
    hp->systemType             = FE_SYSTEM_TYPE_SATELLITE;
    hp->feOpenFct              = FE_MB86667_Open;
    hp->feCloseFct             = FE_MB86667_Close;
    hp->feSetSystemParamsFct   = FE_MB86667_SetSystemParams;
    hp->feResetFct             = FE_MB86667_Reset;
    hp->feTuneFct              = FE_MB86667_Tune;
    hp->feScanFct              = NULL /*FE_MB86667_Scan*/;
    hp->feSendDiseqcCommandFct = FE_MB86667_SendDiseqcCommand;
    hp->feGetStatusFct         = FE_MB86667_GetStatus;
    hp->feGetFECFct            = FE_MB86667_GetFEC;
    hp->feGetAFCFct            = FE_MB86667_GetAFC;
    hp->feGetBERFct            = FE_MB86667_GetBER;
    hp->feGetSignalQualityFct  = FE_MB86667_GetSignalQuality;
    hp->feGetSignalStrengthFct = FE_MB86667_GetSignalStrength;
    hp->feGetTuneParamsFct     = NULL;
    hp->feSetAntennaPowerFct   = NULL;
    hp->isPresent              = 1;
}

/*---------------------------------------------------------------------------*/
/* call back functions                                                       */
/*---------------------------------------------------------------------------*/

/* used on Fujitsu MB86667 board */
int32_t MB86667_S7HZ1402_Detect( uint8_t slot, feHandleT *hp)
{
    if (slot == 0)
        hp->i2cAddrDemod  = 0x10;
    else if (slot == 1)
        hp->i2cAddrDemod  = 0x12;
    else
        return (FE_ERR_BAD_PARAMETER);
    if(fe_mb86667CheckChipId(hp, hp->i2cAddrDemod) != FE_OK)
        return FE_ERR_FE_UNKNOWN_DEVICE;
    fe_mb86667SetFunctions(hp);
    hp->i2cAddrTuner = 0xC0;
    return FE_OK;
}


int32_t FE_MB86667_Open(uint8_t index, FE_HANDLE* pDeviceData)
{
    Downlink_Type dlt;
    int32_t error;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    /* store pointer to device data */
    pData[index] = *pDeviceData = &feHandleArray[index];

    /* set depending on "tuner_type" */
    if (tempRfIniFct != NULL)
        rf_ini_fct[index] = tempRfIniFct;
    else /* set default */
        rf_ini_fct[index] = MB86667_Rf_Val_Set_Ini_S7HZ1402;
    tempRfIniFct = NULL;
    if (tempRfSetFct != NULL)
        rf_set_fct[index] = tempRfSetFct;
    else /* set default */
        rf_set_fct[index] = MB86667_Rf_Val_Set_S7HZ1402;
    tempRfSetFct = NULL;

    /* set context for this front-end instance */
    MB86667_SetContext(index,
                   (void*)FE_MB86667_Reset, 
                   pData[index]->i2cId,
                   pData[index]->i2cAddrDemod,
                   pData[index]->i2cAddrTuner,
                   rf_ini_fct[index],
                   rf_set_fct[index]);

    dlt = fe_mb86667_fillDownlinkType(&pData[index]->pSystemParams);
    #ifdef FAPI_FE_MB86667_DEBUG01
    FAPI_SYS_PRINT_MSG("[FAPI_FE_MB86667_Open]: Try to detect MB86667\n");
    FAPI_SYS_PRINT_MSG("I2C Address Demod: 0x%02X\n",pData[index]->i2cAddrDemod);
    FAPI_SYS_PRINT_MSG("I2C Address Tuner: 0x%02X\n",pData[index]->i2cAddrTuner);
    #endif
    error = MB86667_Init(&dlt);
    if (error == FE_ERR_FE_UNKNOWN_DEVICE)
    {
        #ifdef FAPI_FE_MB86667_DEBUG01
        FAPI_SYS_PRINT_MSG("[FAPI_FE_MB86667_Open]: MB86667 not found\n");
        #endif
        return(FE_ERR_FE_UNKNOWN_DEVICE);
    }
    else if (error != FE_OK)
        return error;

    #ifdef FAPI_FE_MB86667_DEBUG01
        FAPI_SYS_PRINT_MSG("[FAPI_FE_MB86667_Open]: MB86667 found\n");
    #endif

    /* store all statics of the device driver to allow multiple instances */
    MB86667_SaveContext(index);


    return FE_OK;
}

int32_t FE_MB86667_Close(uint8_t index)
{
    Downlink_Type dlt;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    dlt = fe_mb86667_fillDownlinkType(&pData[index]->pSystemParams);
    dlt.flags &= (!LNB_POWER); /* clear LNB power flag */

    MB86667_SetContext(index,
                   (void*)FE_MB86667_Reset,
                   pData[index]->i2cId,
                   pData[index]->i2cAddrDemod,
                   pData[index]->i2cAddrTuner,
                   rf_ini_fct[index],
                   rf_set_fct[index]);
    MB86667_Init(&dlt); /* for switching off LNB power */
    MB86667_SaveContext(index);

    return FE_OK;
}

int32_t FE_MB86667_SetSystemParams(uint8_t index,
                                    FE_SYSTEM_PARAMS_S* pSystemParams)
{
    Downlink_Type dlt;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pSystemParams == NULL) return FE_ERR_BAD_PARAMETER;

    pData[index]->pSystemParams = *pSystemParams;
    dlt = fe_mb86667_fillDownlinkType(&pData[index]->pSystemParams);
    MB86667_ChangeFlags(index, dlt.flags, dlt.RepComSw, dlt.RepUncSw);
    return FE_OK;
}

int32_t FE_MB86667_Reset(uint8_t index)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    FE_ControlFeReset(index, FE_RESET_PULSE);
    FE_Wait(20);

    return FE_OK;
}

int32_t FE_MB86667_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams)
{
    FE_IQ_E iq;
    int32_t error = FE_OK;
    /* MB86667 types: */
    MB86667_TRANSPONDER_T tpt;
    MB86667_SATELLITE_T st;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    tpt.frequency  = (uint32_t)(pTuneParams->sat.frequency);
    /* use the direct symbol rate value when an external 4MHz crystal is used */
    tpt.symbolrate = (uint16_t)(pTuneParams->sat.symbolRate);

    /* fec: */
    if (pData[index]->pSystemParams.sat.autoFEC == FTRUE) tpt.fec = FEC_AUTO;
    else
    {
        switch (pTuneParams->sat.fec)
        {
            case FE_FEC_1_2: tpt.fec = FEC_1_2;  break;
            case FE_FEC_2_3: tpt.fec = FEC_2_3;  break;
            case FE_FEC_3_4: tpt.fec = FEC_3_4;  break;
            case FE_FEC_5_6: tpt.fec = FEC_5_6;  break;
            case FE_FEC_7_8: tpt.fec = FEC_7_8;  break;
            default:            tpt.fec = FEC_AUTO; break;
        }
    }

    tpt.params = 0;
    /* polarization: */
    /* ignore circular types because they are not supported */
    if (pTuneParams->sat.pol == FE_POL_VER) tpt.params |= POL_VER;
    else tpt.params |= POL_HOR;
    /* IQ: */
    iq = pData[index]->pSystemParams.sat.defaultIQ; /* first use global */
    if (iq == FE_IQ_NONE) iq = pTuneParams->sat.iq;
    switch (iq)
    {
        case FE_IQ_NONE:     tpt.params |= MB86667_AUTO_IQ; break;
        case FE_IQ_NORMAL:                              break;
        case FE_IQ_INVERTED: tpt.params |= MB86667_IQ_INV;  break;
        case FE_IQ_AUTO:     tpt.params |= MB86667_AUTO_IQ; break;
        default: break;
    }

    st.LNBlowfreq  = (uint32_t)(pData[index]->pSystemParams.sat.lowFrequency);
    st.LNBhighfreq = (uint32_t)(pData[index]->pSystemParams.sat.highFrequency);
    st.satID       = (uint16_t)(pTuneParams->sat.satID);

    if (st.LNBlowfreq == 0)  st.LNBlowfreq = st.LNBhighfreq;
    if (st.LNBhighfreq == 0) st.LNBhighfreq = st.LNBlowfreq;
    //todo: frequency for SMATV

    MB86667_SetContext(index,
                   (void*)FE_MB86667_Reset,
                   pData[index]->i2cId,
                   pData[index]->i2cAddrDemod,
                   pData[index]->i2cAddrTuner,
                   rf_ini_fct[index],
                   rf_set_fct[index]);
// quick lock
    error = MB86667_Tune(&tpt, &st, FTRUE);

// slow lock
//    error = MB86667_Tune(&tpt, &st, FFALSE);

    #ifdef FAPI_FE_MB86667_DEBUG01
        FAPI_SYS_PRINT_MSG("MB86667_Tune(%i) = 0x%X\n",index,(uint32_t)error);
    #endif

    MB86667_SaveContext(index);

    return error;
}

int32_t FE_MB86667_GetStatus(uint8_t index, FE_STATUS_E* pStatus)
{
    int8_t status;
    int32_t error;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pStatus == NULL) return FE_ERR_BAD_PARAMETER;

    MB86667_SetContext(index,
                   (void*)FE_MB86667_Reset,
                   pData[index]->i2cId,
                   pData[index]->i2cAddrDemod,
                   pData[index]->i2cAddrTuner,
                   rf_ini_fct[index],
                   rf_set_fct[index]);
    error = MB86667_GetLock(&status);
    MB86667_SaveContext(index);

    if (status == 1)
        *pStatus = FE_STATUS_LOCKED;
    else if (status == 0)
        *pStatus = FE_STATUS_NOT_LOCKED;
    else if ((status == -1)||(status == -2))
        *pStatus = FE_STATUS_TUNING;
    else /* error */
        *pStatus = FE_STATUS_UNKNOWN;

    return error;
}

int32_t FE_MB86667_GetAFC(uint8_t index, int32_t* pAFC)
{
    int32_t error;
    int32_t afc;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pAFC == NULL) return FE_ERR_BAD_PARAMETER;

    *pAFC = 0;

    MB86667_SetContext(index,
                   (void*)FE_MB86667_Reset,
                   pData[index]->i2cId,
                   pData[index]->i2cAddrDemod,
                   pData[index]->i2cAddrTuner,
                   rf_ini_fct[index],
                   rf_set_fct[index]);
    error = MB86667_GetAFC(&afc);
    MB86667_SaveContext(index);

    if (error == FE_OK)
        *pAFC = afc;
    else
        *pAFC = 0;

    return error;
}

int32_t FE_MB86667_GetBER(uint8_t index, uint32_t* pBER)
{
    int32_t error;
    uint32_t ber;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pBER == NULL) return FE_ERR_BAD_PARAMETER;

    MB86667_SetContext(index,
                   (void*)FE_MB86667_Reset,
                   pData[index]->i2cId,
                   pData[index]->i2cAddrDemod,
                   pData[index]->i2cAddrTuner,
                   rf_ini_fct[index],
                   rf_set_fct[index]);
    error = MB86667_GetBer(&ber, 1); /* post Viterbi BER */
    MB86667_SaveContext(index);

    if (error == FE_OK)
        *pBER = (uint32_t)ber;
    else
        *pBER = (uint32_t)500000; /* set to 0.5 in case of error */

    return error;
}

int32_t FE_MB86667_GetSignalQuality(uint8_t index, uint16_t* pQuality)
{
    int32_t error;
    uint8_t qual;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pQuality == NULL) return FE_ERR_BAD_PARAMETER;

    MB86667_SetContext(index,
                   (void*)FE_MB86667_Reset,
                   pData[index]->i2cId,
                   pData[index]->i2cAddrDemod,
                   pData[index]->i2cAddrTuner,
                   rf_ini_fct[index],
                   rf_set_fct[index]);
    error = MB86667_GetSigQual(&qual);
    MB86667_SaveContext(index);

    if (error == FE_OK)
        *pQuality = ((uint16_t)qual)<<8;
    else
        *pQuality = 0;

    return error;
}

int32_t FE_MB86667_GetSignalStrength(uint8_t index, uint16_t* pStrength)
{
    int32_t error;
    uint8_t level;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pStrength == NULL) return FE_ERR_BAD_PARAMETER;

    MB86667_SetContext(index,
                   (void*)FE_MB86667_Reset,
                   pData[index]->i2cId,
                   pData[index]->i2cAddrDemod,
                   pData[index]->i2cAddrTuner,
                   rf_ini_fct[index],
                   rf_set_fct[index]);
    error = MB86667_GetSigLevel(&level);
    MB86667_SaveContext(index);

    if (error == FE_OK)
        *pStrength = ((uint16_t)level)<<8;
    else
        *pStrength = 0;

    return error;
}

int32_t FE_MB86667_SendDiseqcCommand(uint8_t  index, 
                                 uint8_t  length, 
                                 uint8_t* pCommands, 
                                 uint8_t* pReturnLength, 
                                 uint8_t* pReturnBuffer)
{
    int32_t error;
    
    if (index >= FE_MAX_NUM)    return FE_ERR_BAD_PARAMETER;
    if ((length < 3) || (length > 6) || (pCommands == NULL) )
        return FE_ERR_BAD_PARAMETER;
    if (pReturnLength == NULL)  return FE_ERR_BAD_PARAMETER;
    if (pReturnBuffer == NULL)  return FE_ERR_BAD_PARAMETER;

    MB86667_SetContext(index,
                   (void*)FE_MB86667_Reset,
                   pData[index]->i2cId,
                   pData[index]->i2cAddrDemod,
                   pData[index]->i2cAddrTuner,
                   rf_ini_fct[index],
                   rf_set_fct[index]);
        
    error = MB86667_SendDiseqcCommand(length,
                                     pCommands,
                                     pReturnLength,
                                     pReturnBuffer);
    MB86667_SaveContext(index);
    
    return error;
}

int32_t FE_MB86667_GetFEC(uint8_t index, FE_FEC_E* pFEC)
{
    int32_t error;
    int8_t   fec;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    MB86667_SetContext(index,
                   (void*)FE_MB86667_Reset,
                   pData[index]->i2cId,
                   pData[index]->i2cAddrDemod,
                   pData[index]->i2cAddrTuner,
                   rf_ini_fct[index],
                   rf_set_fct[index]);
    error = MB86667_GetFEC(&fec);
    MB86667_SaveContext(index);

    switch (fec)
    {
        case 1:  *pFEC = FE_FEC_1_2; break;
        case 2:  *pFEC = FE_FEC_2_3; break;
        case 3:  *pFEC = FE_FEC_3_4; break;
        case 4:  *pFEC = FE_FEC_5_6; break;
        case 5:  *pFEC = FE_FEC_7_8; break;
        case 0:  error = FE_ERR_FE_NOT_LOCKED; break; /* not locked */
        default: error = FE_ERR_FE_I2C; break; /* other error */
    }

    return error;
}

#if 0
int32_t FE_MB86667_Scan(uint8_t index, FE_SCAN_PARAMS_S* pScanParams)
{
    int32_t error;
    MB86667_SCAN_PARAMS_T scanParams;
    MB86667_SATELLITE_T satType;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if(pScanParams == NULL)  return FE_ERR_BAD_PARAMETER;

    scanParams.feScanAbortFct=pScanParams->feScanAbortFct;
    scanParams.feScanNotifyFct=pScanParams->feScanNotifyFct;
    scanParams.feScanProgressFct=pScanParams->feScanProgressFct;
    scanParams.pol=pScanParams->pol;
    scanParams.startFrequency=pScanParams->startFrequency;
    scanParams.stopFrequency=pScanParams->stopFrequency;
    scanParams.startSymbolRate=pScanParams->startSymbolRate;
    scanParams.stopSymbolRate=pScanParams->stopSymbolRate;

    satType.satID=pScanParams->satID;
    satType.LNBhighfreq=pData[index]->pSystemParams.sat.highFrequency;
    satType.LNBlowfreq=pData[index]->pSystemParams.sat.lowFrequency;

    MB86667_SetContext(index,
                   (void*)FE_MB86667_Reset,
                   pData[index]->i2cId,
                   pData[index]->i2cAddrDemod,
                   pData[index]->i2cAddrTuner,
                   rf_ini_fct[index],
                   rf_set_fct[index]);
    error = MB86667_Scan(&scanParams,&satType);
    MB86667_SaveContext(index);

    return error;
}
#endif

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

/* Version History */
/******************************************************************************

$Log: mb86667.c,v $
Revision 1.1  2008/07/11 13:36:32  sgehre
- DSF89000 activated by default
- added support for MB86667 DVB-S frontend
- use SUBSYS_FE_MODULE_BASE for errors

Revision 1.1  2008/05/07 11:10:19  wlaris
Initial creation.

Revision 1.1  2008/05/07 08:39:07  wlaris
Initial creation.


******************************************************************************/
