/******************************************************************************
**
** \file      ./subsystems/fe/mb86668/src/mb86668.c
**
** \brief     Front-end device driver for Fujitsu MB86668 cable frontend.
**
**            (C) Fujitsu Semiconductor Europe 2002 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: mb86668.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
#include "../src/fe_priv.h"
#include "mb86668_fe_types.h"
#include "mb86668_fe_tools.h"
#include "mb86668_fe.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define WAIT_AND_PROBE_TIL_CHANGE_QAM 10 /* in units of 50 micro secs */

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static FE_Device            *FeHandle[FE_MAX_NUM];
static feHandleT            *pData[FE_MAX_NUM];
static tuner_type_t         tempTunerType = NO_TUNER;


/*---------------------------------------------------------------------------*/
/* callback functions                                                        */
/*---------------------------------------------------------------------------*/
int32_t MB86668_Open(uint8_t index, FE_HANDLE *pDeviceData);
int32_t MB86668_Close(uint8_t index);
int32_t MB86668_SetSystemParams(uint8_t index, FE_SYSTEM_PARAMS_S* pSystemParams);
int32_t MB86668_Reset(uint8_t index);
int32_t MB86668_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams);
int32_t MB86668_GetStatus(uint8_t index, FE_STATUS_E* pStatus);
int32_t MB86668_GetAFC(uint8_t index, int32_t* pAFC);
int32_t MB86668_GetBER(uint8_t index, uint32_t* pBER);
int32_t MB86668_GetSignalQuality(uint8_t index, uint16_t* pQuality);
int32_t MB86668_GetSignalStrength(uint8_t index, uint16_t* pStrength);
int32_t MB86668_GetTuneParams(uint8_t index, FE_TUNE_PARAMS_S *pTuneParams);

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

int32_t mb86668CheckChipId(feHandleT *hp, uint8_t i2cAddress)
{
    uint8_t i2cBuffer[2];
    int32_t ferr;
    
    i2cBuffer[0] = 0xFF;
    i2cBuffer[1] = 0xFF;
    ferr = FE_I2C_Read(hp->i2cId, i2cAddress, &(i2cBuffer[0]), 1,
                                              &(i2cBuffer[1]), 1);
    FE_DPrintf("Checking for MB86668 on address %02x ferr %x read %x\n", i2cAddress, ferr, i2cBuffer[1]);
    if(ferr != FE_OK)
        return ferr;
    /* if(i2cBuffer[1] != 0x14)
        return -1; */
    return FE_OK;
}

void mb86668SetFunctions(feHandleT *hp)
{
    hp->systemType             = FE_SYSTEM_TYPE_CABLE;
    hp->feOpenFct              = MB86668_Open;
    hp->feCloseFct             = MB86668_Close;
    hp->feSetSystemParamsFct   = MB86668_SetSystemParams;
    hp->feResetFct             = MB86668_Reset;
    hp->feTuneFct              = MB86668_Tune;
    hp->feScanFct              = NULL;
    hp->feSendDiseqcCommandFct = NULL;
    hp->feGetStatusFct         = MB86668_GetStatus;
    hp->feGetFECFct            = NULL;
    hp->feGetAFCFct            = MB86668_GetAFC;
    hp->feGetBERFct            = MB86668_GetBER;
    hp->feGetSignalQualityFct  = MB86668_GetSignalQuality;
    hp->feGetSignalStrengthFct = MB86668_GetSignalStrength;
    hp->feGetTuneParamsFct     = MB86668_GetTuneParams;
    hp->feSetAntennaPowerFct   = NULL;
    hp->isPresent              = 1;
}

/*---------------------------------------------------------------------------*/
/* call back functions                                                       */
/*---------------------------------------------------------------------------*/

/* used on Fujitsu MB86668-DK01 */
int32_t MB86668_DCT70701_Detect( uint8_t slot, feHandleT *hp)
{
    if (slot == 0)
        hp->i2cAddrDemod  = 0x30;
    else if (slot == 1)
        hp->i2cAddrDemod  = 0x32;
    else
        return (FE_ERR_BAD_PARAMETER);
    if(mb86668CheckChipId(hp, hp->i2cAddrDemod) != FE_OK)
        return FE_ERR_FE_UNKNOWN_DEVICE;
    mb86668SetFunctions(hp);
    hp->i2cAddrTuner = 0xC0;
    tempTunerType             = C_DCT70701;
    return FE_OK;
}

/* used on Fujitsu old DK */
int32_t MB86668_CD1600_Detect( uint8_t slot, feHandleT *hp)
{
    if (slot == 0)
        hp->i2cAddrDemod  = 0x32;
    else
        return (FE_ERR_BAD_PARAMETER);
    if(mb86668CheckChipId(hp, hp->i2cAddrDemod) != FE_OK)
        return FE_ERR_FE_UNKNOWN_DEVICE;
    mb86668SetFunctions(hp);
    hp->i2cAddrTuner  = 0xC2;
    tempTunerType             = C_CD1600L_3;
    return FE_OK;
}

/* used on Fujitsu old DK */
int32_t MB86668_DCT70602_Detect( uint8_t slot, feHandleT *hp)
{
    if (slot == 0)
        hp->i2cAddrDemod  = 0x32;
    else
        return (FE_ERR_BAD_PARAMETER);
    if(mb86668CheckChipId(hp, hp->i2cAddrDemod) != FE_OK)
        return FE_ERR_FE_UNKNOWN_DEVICE;
    mb86668SetFunctions(hp);
    hp->i2cAddrTuner  = 0xC0;
    tempTunerType             = C_DCT70602;
    return FE_OK;
}

/* not used on Fujitsu DKs */
int32_t MB86668_CD1316_Detect(uint8_t slot, feHandleT *hp)
{
    if (slot == 0)
        hp->i2cAddrDemod  = 0x30;
    else if (slot == 1)
        hp->i2cAddrDemod  = 0x32;
    else
        return (FE_ERR_BAD_PARAMETER);
    if(mb86668CheckChipId(hp, hp->i2cAddrDemod) != FE_OK)
        return FE_ERR_FE_UNKNOWN_DEVICE;
    mb86668SetFunctions(hp);
    hp->i2cAddrTuner  = 0xC2;
    tempTunerType             = C_CD1316L_3;
    return FE_OK;
}

int32_t MB86668_Open(uint8_t index, FE_HANDLE* pDeviceData)
{
    int32_t error;

    /* store pointer to device data */
    pData[index] = *pDeviceData = &feHandleArray[index];

    /* try to detect depending on "tuner_type" */
    if ((tempTunerType > NO_TUNER) && (tempTunerType < LAST_TUNER))
    {
        if((FeHandle[index] = FEC_Create(C_MB86668, pData[index]->i2cAddrDemod,
                      tempTunerType, pData[index]->i2cAddrTuner))==NULL)
        {
            tempTunerType = NO_TUNER;
            return FE_ERR_BAD_PARAMETER;
        }
        tempTunerType = NO_TUNER;
    }
    /* try to detect depending on I2C address */
    else if(pData[index]->i2cAddrTuner==0xC2)
    {   /* Philips tuner on evalboard have 0xC2 (they are identical) */
        #if 1   /* CD1316 and CD1600 */
        if((FeHandle[index] = FEC_Create(C_MB86668, pData[index]->i2cAddrDemod,
                      C_CD1600L_3, pData[index]->i2cAddrTuner))==NULL)
            return FE_ERR_BAD_PARAMETER;
        #endif
    }
    else
    {   /* Thomson tuner have 0xC0 */
        #if 0
        if((FeHandle[index] = FEC_Create(C_MB86668, pData[index]->i2cAddrDemod,
                      C_DCT70602, pData[index]->i2cAddrTuner))==NULL)
            return FE_ERR_BAD_PARAMETER;
        #endif
        #if 1
        if((FeHandle[index] = FEC_Create(C_MB86668, pData[index]->i2cAddrDemod,
                      C_DCT70701, pData[index]->i2cAddrTuner))==NULL)
            return FE_ERR_BAD_PARAMETER;
        #endif
    }

    error = FEC_Reset(pData[index]);
    if (error != FE_OK)
    {
        FEC_Destroy(FeHandle[index]);
        return (error);
    }

    if(FEC_TestDevice(FeHandle[index])!=FE_OK)
    {
        FEC_Destroy(FeHandle[index]);
        return FE_ERR_FE_UNKNOWN_DEVICE;
    }
    error = FEC_Reset(pData[index]);
    if (error != FE_OK)
    {
        FEC_Destroy(FeHandle[index]);
        return (error);
    }

    if(FEC_Init(FeHandle[index])!=FE_OK)
    {
        FEC_Destroy(FeHandle[index]);
        return FE_ERR_FE_I2C;
    }

    return FE_OK;
}

int32_t MB86668_Close(uint8_t index)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    FEC_Destroy(FeHandle[index]);
    return FE_OK;
}

int32_t MB86668_SetSystemParams(uint8_t index,
                                    FE_SYSTEM_PARAMS_S* pSystemParams)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pSystemParams == NULL) return FE_ERR_BAD_PARAMETER;

    return FE_OK;
}

int32_t MB86668_Reset(uint8_t index)
{
    int32_t error;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    error = FEC_Reset(pData[index]);
    if (error != FE_OK)
        return (error);

    FEC_Init(FeHandle[index]);
    return FE_OK;
}

int32_t MB86668_Tune(uint8_t index, FE_TUNE_PARAMS_S* pTuneParams)
{
    T_TuningParam param;
    int32_t err=FE_OK;
    int8_t rIndex=1,i;
    uint8_t status;
    fe_modulation_t Mod[5];
    int32_t ret;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    if (pTuneParams == NULL) return FE_ERR_BAD_PARAMETER;

    param.frequency=pTuneParams->cable.frequency;
    param.qam.symbol_rate=pTuneParams->cable.symbolRate*1000;      // we need Hz
    switch(pTuneParams->cable.modulation)
    {
       case FE_MOD_QAM_16:
          Mod[0]=QAM_16;
          break;
       case FE_MOD_QAM_32:
          Mod[0]=QAM_32;
          break;
       case FE_MOD_QAM_64:
          Mod[0]=QAM_64;
          break;
       case FE_MOD_QAM_128:
          Mod[0]=QAM_128;
          break;
       case FE_MOD_QAM_256:
          Mod[0]=QAM_256;
          break;
       case FE_MOD_AUTO:
          rIndex=5; // we check modulations in reverse mode index 4 to 0
          Mod[0]=QAM_16;
          Mod[1]=QAM_32;
          Mod[2]=QAM_128;
          Mod[3]=QAM_256;
          Mod[4]=QAM_64;
          break;
       default:
          return FE_ERR_BAD_PARAMETER;
    }
    do
    {
       param.qam.modulation=Mod[rIndex-1];
       ret=FEC_Tune( FeHandle[index], &param);
       switch(ret)
       {
          case FE_ERR_FE_I2C:
             err=FE_ERR_FE_I2C;
             rIndex=0;
             break;
          case    FE_ERR_FE_TUNE:
             err=FE_ERR_FE_TUNE;
             rIndex=0;
             break;
          case    FE_ERR_FE_OTHER:
             err=FE_ERR_FE_TUNE;
             rIndex=0;
             break;
          case FE_ERR_BAD_PARAMETER:
             err=FE_ERR_BAD_PARAMETER;
             rIndex=0;
             break;
          case    FE_OK:
             err=FE_OK;
             rIndex--;
             if(rIndex==0)
                break;
             for(i=0;i<WAIT_AND_PROBE_TIL_CHANGE_QAM;i++)
             {
                if(FEC_GetStatus( FeHandle[index], &status)!=FE_OK)
                   return FE_ERR_FE_I2C;
                if(status!=0) //locked
                   return FE_OK;
                FE_Wait(50);
             }
             break;
          default:
             err=FE_ERR_FE_OTHER;
             rIndex=0;
             break;
       }
    } while (rIndex>0);
    return err;
}

int32_t MB86668_GetStatus(uint8_t index, FE_STATUS_E* pStatus)
{
    uint8_t status;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pStatus == NULL) return FE_ERR_BAD_PARAMETER;

    if(FEC_GetStatus( FeHandle[index], &status)!=FE_OK)
       return FE_ERR_FE_I2C;
    if(status==0) //unlocked
       *pStatus=FE_STATUS_NOT_LOCKED;
    else
       *pStatus=FE_STATUS_LOCKED;

    return FE_OK;
}

int32_t MB86668_GetAFC(uint8_t index, int32_t* pAFC)
{
    if (index >= FE_MAX_NUM)
        return FE_ERR_BAD_PARAMETER;
    if (pAFC == NULL)
        return FE_ERR_BAD_PARAMETER;
    *pAFC = 0;
// still open

    return FE_OK;
}

int32_t MB86668_GetBER(uint8_t index, uint32_t* pBER)
{
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pBER == NULL) return FE_ERR_BAD_PARAMETER;

    if(FEC_GetBer( FeHandle[index], pBER)!=FE_OK)
       return FE_ERR_FE_I2C;

    return FE_OK;
}

int32_t MB86668_GetSignalQuality(uint8_t index, uint16_t* pQuality)
{
    uint16_t qual;

    if (pQuality == NULL) return FE_ERR_BAD_PARAMETER;
    *pQuality = 0;
    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;

    if(FEC_GetCN( FeHandle[index], &qual)!=FE_OK)
       return FE_ERR_FE_I2C;
    if (qual > 1424)
        qual = 1424;
    *pQuality = 65535 - (qual * 46);
    return FE_OK;
}

int32_t MB86668_GetSignalStrength(uint8_t index, uint16_t* pStrength)
{
    uint8_t level;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pStrength == NULL) return FE_ERR_BAD_PARAMETER;

    if(FEC_GetAgc( FeHandle[index], &level)!=FE_OK)
       return FE_ERR_FE_I2C;
    *pStrength = level * 256;

    return FE_OK;
}

int32_t MB86668_GetTuneParams(uint8_t index, FE_TUNE_PARAMS_S *pTuneParams)
{
    T_TuningParam param;

    if (index >= FE_MAX_NUM) return FE_ERR_BAD_PARAMETER;
    if (pTuneParams == NULL) return FE_ERR_BAD_PARAMETER;

    if(FEC_GetTuningParam( FeHandle[index], &param)!=FE_OK)
       return FE_ERR_FE_I2C;

    pTuneParams->cable.frequency=param.frequency;
    pTuneParams->cable.symbolRate=param.qam.symbol_rate/1000; // we got Hz
    pTuneParams->cable.fec=FE_FEC_AUTO;
    pTuneParams->cable.iq=FE_IQ_AUTO;

    switch(param.qam.modulation)
    {
       case QAM_16:
          pTuneParams->cable.modulation=FE_MOD_QAM_16;
          break;
       case QAM_32:
          pTuneParams->cable.modulation=FE_MOD_QAM_32;
          break;
       case QAM_64:
          pTuneParams->cable.modulation=FE_MOD_QAM_64;
          break;
       case QAM_128:
          pTuneParams->cable.modulation=FE_MOD_QAM_128;
          break;
       case QAM_256:
          pTuneParams->cable.modulation=FE_MOD_QAM_256;
          break;
       default:
          pTuneParams->cable.modulation=FE_MOD_NONE;
          break;
    }
    return FE_OK;
}

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

/* Version History */
/******************************************************************************

$Log: mb86668.c,v $
Revision 1.2  2008/07/31 10:33:25  sgehre
fix reset handling of MB86668 frontends

Revision 1.1  2008/05/07 11:10:19  wlaris
Initial creation.

Revision 1.1  2008/05/07 08:39:07  wlaris
Initial creation.


******************************************************************************/
