/******************************************************************************
**
** \file      ./subsystems/fe/mb86668/src/mb86668_fec.c
**
** \brief     Front-end device driver for Fujitsu MB86668 FEC.
**
**            (C) Fujitsu Semiconductor Europe 2002 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: mb86668_fec.c 1911 2009-02-02 10:17:47Z wlaris $
**
******************************************************************************/
/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include "../src/fe_priv.h"
#include "mb86668_fe_types.h"
#include "mb86668_fe_tools.h"
#include "mb86668_fe_c.h"
#include "mb86668_fe.h"

/*-----------------------------------------------------------------------------
   function:    FE_device *FEC_Create(demod_type_t dem_type, uint8_t dem_i2c,
                                      tuner_type_t tun_type, uint8_t tun_i2c)
   description: creates access handle for frontend device
   input:       demod_type_t dem_type, uint8_t dem_i2c
                tuner_type_t tun_type, uint8_t tun_i2c
   output:      nothing
   return:      FE_device * handle or NULL (on error)
-----------------------------------------------------------------------------*/
FE_Device *FEC_Create(demod_type_t dem_type, uint8_t dem_i2c,
                      tuner_type_t tun_type, uint8_t tun_i2c)
{
   FE_Device *handle=0;
   FEC_device *inthandle;


   // check supported demod type
   switch(dem_type)
   {
      case C_MB86668: break;
      default: return 0;
   }

   // check supported tuner type
   switch(tun_type)
   {
      case C_CD1316L_3:
      case C_CD1600L_3:
      case C_DCT70602:
      case C_DCT70701:
          break;
      default:
          return 0;
   }

   handle =(FE_Device *) malloc(sizeof(FE_Device));
   if (handle==0)
      return handle;

   // set handle data structure
   handle->devicetype=DVB_C;
   handle->demod_i2c_adr=dem_i2c;
   handle->demod_type=dem_type;
   handle->tuner_i2c_adr=tun_i2c;
   handle->tuner_type=tun_type;
   handle->internal_data= (void *) malloc(sizeof(FEC_device));
   if(handle->internal_data==0)
   {
      free(handle);
      return 0;
   }
   inthandle=(FEC_device *)handle->internal_data;

   inthandle->Demod.I2cAddress=handle->demod_i2c_adr;
   switch(handle->demod_type)
   {
      case C_MB86668: inthandle->Demod.Type=MB86668;
          break;
      default:
          free(handle);
          free(handle->internal_data);
          return 0;
   }

   inthandle->Tuner.I2cAddress=handle->tuner_i2c_adr;
   switch(handle->tuner_type)
   {
      case C_CD1316L_3:
          inthandle->Tuner.Type=CD1316L_3;
          break;
      case C_CD1600L_3:
          inthandle->Tuner.Type=CD1600L_3;
          break;
      case C_DCT70602:
          inthandle->Tuner.Type=DCT70602;
          break;
      case C_DCT70701:
          inthandle->Tuner.Type=DCT70701;
          break;
      default:
          free(handle);
          free(handle->internal_data);
          return 0;
   }

   return handle;
}


/*-----------------------------------------------------------------------------
   function:    void FEC_Destroy(FE_device *handle)
   description: frees access handle for frontend c device
   input:       FE_device *handle
   output:      nothing
   return:      void
-----------------------------------------------------------------------------*/
void FEC_Destroy(FE_Device *handle)
{

   if (handle!=0)
   {
      if(handle->internal_data!=0)
         free(handle->internal_data);
      free(handle);
      handle=0;
   }
}


/*****************************************************
--FUNCTION   ::  TUN_C_TestDevice
--ACTION     ::  tests if devices are available
--PARAMS IN  ::  inside handle
--PARAMS OUT ::  NONE
--RETURN     ::  FeErr
--***************************************************/
int32_t FEC_TestDevice(FE_Device * handle)
{
   FEC_device *inthandle;

   if(handle==0)
      return FE_ERR_BAD_PARAMETER;

   inthandle=(FEC_device *)handle->internal_data;
   if(DEM_86668_TestDevice(inthandle)!=FeOk)
   {
      FE_DPrintf("DEMOD MB86668 0x%02X not found!\n",inthandle->Demod.I2cAddress);
      return FE_ERR_FE_OTHER;
   }
   if(TUN_C_TestDevice(inthandle)!=FeOk)
   {
      FE_DPrintf("Tuner 0x%02X not found!\n",inthandle->Tuner.I2cAddress);
      return FE_ERR_FE_TUNE;
   }
   return FE_OK;
}

extern unsigned char initialised;
/*-----------------------------------------------------------------------------
   function:    int32_t FEC_Init(FE_device *handle)
   description: initialises tuner and demod
   input:       FE_device *handle
   output:      nothing
   return:      int32_t: error treatment
-----------------------------------------------------------------------------*/
int32_t FEC_Init(FE_Device *handle)
{
   FEC_device *inthandle;
   int ret;

   if(handle==0)
      return FE_ERR_BAD_PARAMETER;

   inthandle=(FEC_device *)handle->internal_data;
   if((ret=TUN_C_init(inthandle))!=FeOk)
   {
      if(ret==TUN_I2C_ERR)
         return FE_ERR_FE_I2C;
      else
         return FE_ERR_FE_TUNE;
   }
   if((ret=DEM_86668_Init(inthandle))!=FeOk)
   {
      if(ret==DEM_I2C_ERR)
         return FE_ERR_FE_I2C;
      else
         return FE_ERR_FE_OTHER;
   }
   return FE_OK;

}

/*-----------------------------------------------------------------------------
   function:    int32_t FEC_Tune( FEC_device *handle, T_TuningParam *param)
   description: tunes frontend to new transponder
   input:       FE_device *handle
   output:      nothing
   return:      int32_t: error treatment
-----------------------------------------------------------------------------*/
int32_t FEC_Tune( FE_Device *handle, T_TuningParam *param)
{
   FEC_device *inthandle;
   FE_86668_SearchParams_t searchparam;
   FeErr err;

   if(handle==0 || param==0)
      return FE_ERR_BAD_PARAMETER;

    FE_DPrintf("\n-----------------------------");
    FE_DPrintf("\nMB86668_Tune got \nFrequency: %u KHz", param->frequency/1000);
    FE_DPrintf("\nSymbolrate:  %u KS/s", param->qam.symbol_rate/1000);
    switch (param->qam.modulation)
    {
        case QPSK:
           FE_DPrintf("\nModulation:  QPSK");
        break;
        case QAM_16:
           FE_DPrintf("\nModulation:  QAM 16");
        break;
        case QAM_32:
           FE_DPrintf("\nModulation:  QAM 32");
        break;
        case QAM_64:
           FE_DPrintf("\nModulation:  QAM 64");
        break;
        case QAM_128:
           FE_DPrintf("\nModulation:  QAM 128");
        break;
        case QAM_256:
           FE_DPrintf("\nModulation:  QAM 256");
        break;
        case QAM_AUTO:
           FE_DPrintf("\nModulation:  auto");
        break;
        default:
           FE_DPrintf("\nUnknown modulation");
    }
    FE_DPrintf("\n-----------------------------\n");


   inthandle=(FEC_device *)handle->internal_data;

   // setup internal search params
   searchparam.Frequency=param->frequency/1000;  // we come with absolute freq
   searchparam.SymbolRate=param->qam.symbol_rate/1000; // we come with Hz
   switch(param->qam.modulation)
   {
      case QAM_16:
         searchparam.Qam=QAM16;
         break;
      case QAM_32:
         searchparam.Qam=QAM32;
         break;
      case QAM_64:
         searchparam.Qam=QAM64;
         break;
      case QAM_128:
         searchparam.Qam=QAM128;
         break;
      case QAM_256:
         searchparam.Qam=QAM256;
         break;
      case QPSK:
      case QAM_AUTO:
      default:
         return FE_ERR_BAD_PARAMETER;
   }

   if((err=DEM_86668_SearchTransponder(inthandle,&searchparam))!=FeOk)
   {
      FE_DPrintf("DEM_86668_SearchTransponder()=%i\n",err);
      if(err==DEM_I2C_ERR)
         return FE_ERR_FE_I2C;
      else
         return FE_ERR_FE_OTHER;
   }
   return FE_OK;
}

/*-----------------------------------------------------------------------------
   function:    int32_t FEC_GetStatus( FE_Device *handle, uint8_t *status)
   description: gets status from frontend
   input:       FE_device *handle
   output:      uint8_t *status  (0=unlocked 1=locked)
   return:      int32_t: error treatment
-----------------------------------------------------------------------------*/
int32_t FEC_GetStatus( FE_Device *handle, uint8_t *status)
{
   FEC_device *inthandle;

   if(handle==0)
      return FE_ERR_BAD_PARAMETER;

   inthandle=(FEC_device *)handle->internal_data;
   if(DEM_86668_GetLock(inthandle,status)!=FeOk)
      return FE_ERR_FE_I2C;
   return FE_OK;
}


/*-----------------------------------------------------------------------------
   function:    int32_t FEC_GetAgc( FE_Device *handle, uint16_t *agcVal)
   description: gets AGC from frontend
   input:       FE_device *handle
   output:      uint8_t *agcVal in the range from 0 to 255
   return:      int32_t: error treatment
-----------------------------------------------------------------------------*/
int32_t FEC_GetAgc( FE_Device *handle, uint8_t *agcVal)
{
   FEC_device *inthandle;
   uint16_t        agc;
   uint8_t         status;

   if(handle==0)
      return FE_ERR_BAD_PARAMETER;

   *agcVal = 0;
   inthandle=(FEC_device *)handle->internal_data;
   if(DEM_86668_GetAgc(inthandle, &agc)!=FeOk)
      return FE_ERR_FE_I2C;
   agc = (uint16_t)(1024 - agc); /* swap polarity */
   if (agc > 542)
      agc = 542;
   if (agc < 32)
      agc = 32;
   agc = (uint16_t)(agc - 32);
   agc = (uint16_t)(agc / 2);
   if (agc == 0)
   {
      if(DEM_86668_GetLock(inthandle, &status) != FeOk)
         return FE_ERR_FE_I2C;
      if (status == 1)
         agc = 32; /* indicate some level in case of lock */
   }
   *agcVal = (uint8_t)agc;
   FE_DPrintf("\n agcVal: %d\n", *agcVal);
   return FE_OK;
}

/*-----------------------------------------------------------------------------
   function:    int32_t FEC_GetCN( FE_Device *handle, uint16_t *cnVal)
   description: gets CN from frontend
   input:       FE_device *handle
   output:      uint16_t *cnVal
   return:      int32_t: error treatment
-----------------------------------------------------------------------------*/
int32_t FEC_GetCN( FE_Device *handle, uint16_t *cnVal)
{
   FEC_device *inthandle;
   uint32_t Noise;

   if(handle==0)
      return FE_ERR_BAD_PARAMETER;

   inthandle=(FEC_device *)handle->internal_data;
   if(DEM_86668_GetCN(inthandle,&Noise)!=FeOk)
      return FE_ERR_FE_I2C;
   *cnVal=(uint16_t) Noise;
   return FE_OK;
}

/*-----------------------------------------------------------------------------
   function:    int32_t FET_GetBer(FE_Device *handle, uint32_t *ber)
   description: gets BER from frontend
   input:       FE_device *handle
   output:      uint32_t *ber
   return:      int32_t: error treatment
-----------------------------------------------------------------------------*/
int32_t FEC_GetBer(FE_Device *handle, uint32_t *ber)
{
   FEC_device *inthandle;

   if(handle==0)
      return FE_ERR_BAD_PARAMETER;

   inthandle=(FEC_device *)handle->internal_data;
   if(DEM_86668_GetBer(inthandle,ber)!=FeOk)
      return FE_ERR_FE_I2C;
   return FE_OK;
}


/*-----------------------------------------------------------------------------
   function:    int32_t FEC_GetTuningParam( FE_device *handle, T_TuningParam *param)
   description: reads back frontend tuning params
   input:       FE_device *handle
   output:      T_TuningParam *param
   return:      int32_t: error treatment
-----------------------------------------------------------------------------*/
int32_t FEC_GetTuningParam(FE_Device *handle, T_TuningParam *param)
{
   FEC_device *inthandle;
   uint32_t offset;

   if(handle==0 || param==0)
      return FE_ERR_BAD_PARAMETER;

   inthandle=(FEC_device *)handle->internal_data;

   param->frequency= 1000*(inthandle->Demod.params.Frequency);
   param->qam.symbol_rate=1000*(inthandle->Demod.params.SymbolRate);

   switch(inthandle->Demod.params.Qam)
   {
      case QAM16:
         param->qam.modulation=QAM_16;
         break;
      case QAM32:
         param->qam.modulation=QAM_32;
         break;
      case QAM64:
         param->qam.modulation=QAM_64;
         break;
      case QAM128:
         param->qam.modulation=QAM_128;
         break;
      case QAM256:
         param->qam.modulation=QAM_256;
         break;
      default:
         return FE_ERR_BAD_PARAMETER;
   }

   if(DEM_86668_GetCarrierOffset(inthandle, &offset)!=FeOk)
      return FE_ERR_FE_I2C;
   FE_DPrintf("Carrier Offset now at: %d Hz\n",offset);
   return FE_OK;
}

/*-----------------------------------------------------------------------------
   function:    int32_t FEC_Reset(feHandleT *handle)
   description: resets frontend - not implemented at the moment
   input:       FE_device *handle
   output:      nothing
   return:      int32_t: error treatment
-----------------------------------------------------------------------------*/
int32_t FEC_Reset(feHandleT *handle)
{
   return (TOOL_resetFe(handle->slotNum));
}

/* Version History */
/******************************************************************************

$Log: mb86668_fec.c,v $
Revision 1.2  2008/07/31 10:33:25  sgehre
fix reset handling of MB86668 frontends

Revision 1.1  2008/05/07 11:10:19  wlaris
Initial creation.

Revision 1.1  2008/05/07 08:39:07  wlaris
Initial creation.


******************************************************************************/
