/*
*****************************************************************************
**
** \file        ./subsystems/fe/src/fe_tools.c
**
** \version     $Id: fe_tools.c 4063 2009-06-29 09:28:12Z ebende $
**
** \brief       functions commonly used by frontend-specific drivers.
**
** This files contains the implementation of the FE driver's core API.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2009 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2009      by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/

#include "fapex/abst_types.h"
#include "fapex/abst_libi2c.h"
#include "fapex/abst_libgpio.h"
#include "fe_priv.h"

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************
#define FMC_PDK_VX

#ifdef FMC_PDK_VX
#define HardwareReset_pin   12
#else
#include "fapex/abst_libupi.h"
#define DRV_ID_UPI0      (FAPEXDEV_UPI_MINOR_FIRST)
#define DRV_ID_UPI1      (FAPEXDEV_UPI_MINOR_FIRST + 1)
#define DRV_ID_UPI2      (FAPEXDEV_UPI_MINOR_FIRST + 2)
#endif


//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************
//static int32_t fd_UPI    = -1;
static uint64_t tuneEndTime[FE_MAX_NUM];



/*lint -save -e715 -e923*/
/*
********************************************************************************
** \brief Wait for a certain time
**
** \param millis Time to wait in ms
**
********************************************************************************
*/
void FE_Wait(uint32_t millis)
{
    FAPI_SYS_SLEEP(millis);
}

/*
********************************************************************************
** \brief Perform a frontend reset
**
** This function uses board-specific means to reset a frontend. The reset
** signal is active low
**
** \param slotNum Hardware/board slot at which the frontend is located
** \param type Type of reset like pulse, level set etc
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
********************************************************************************
*/
int32_t FE_ControlFeReset(uint8_t slotNum, FE_RESET_E type)
{
#ifndef FMC_PDK_VX
    uint32_t              base_address    = 0;
    uint32_t*             device_ptr      = 0;
    uint32_t              store_cs        = 0;
    uint32_t              store_mode      = 0;
    FAPEX_UPI_ConfigT     configuration;
    FAPEX_UPI_SetParamsT  upi_set_Params;
    int32_t               return_value = FE_ERR_FE_UPI_CONFIGURATION;

    uint8_t offMask;

    if(slotNum == 0)
        offMask = 0x08;
    else if(slotNum == 1)
        offMask = 0x04;
    else
        return FE_ERR_BAD_PARAMETER;

    fd_UPI = FAPEX_UPI_Open(DRV_ID_UPI2, O_RDWR, NULL);
    if(fd_UPI < 0) {
        FAPI_SYS_PRINT_ERR("ERROR FAPEX_UPI_Open %d\n", fd_UPI);
        return FE_ERR_FE_UPI_CONFIGURATION;
    }

    return_value = FAPEX_UPI_Ioctl( (FAPEXIOCARG1_T)fd_UPI, FAPEX_IOCCOM_UPI_GET_BASEADDRESS, (FAPEXIOCARG3_T)&base_address );
    if( return_value >= 0 )
    {
        device_ptr = (uint32_t*)base_address;
        if(!device_ptr)
        {
            (void)FAPEX_UPI_Close( (FAPEXIOCARG1_T)fd_UPI );
            return FE_ERR_FE_UPI_CONFIGURATION;
        }

        // Save UPI configuration
        (void)FAPEX_UPI_Ioctl((FAPEXIOCARG1_T)fd_UPI, FAPEX_IOCCOM_UPI_GET_CONFIG, (FAPEXIOCARG3_T)&configuration );
        store_cs   = configuration.chipSelect;
        store_mode = configuration.mode;

        // Set new configuration
        configuration.chipSelect = 3;                    // Set UPI select for external latch
        configuration.mode       = FAPEX_UPI_MODE_SRAM;   // SRAM mode on device 2
        (void)FAPEX_UPI_Ioctl( (FAPEXIOCARG1_T)fd_UPI, FAPEX_IOCCOM_UPI_SET_CONFIG, (FAPEXIOCARG3_T)&configuration );

        ////////////////
        FAPEX_UPI_Ioctl((FAPEXIOCARG1_T)fd_UPI, FAPEX_IOCCOM_UPI_LOCK, (FAPEXIOCARG3_T)NULL);

        // Set external latch: TSA/B low
        upi_set_Params.base_address = (uint32_t)device_ptr;
        upi_set_Params.offMask = offMask;
        (void)FAPEX_UPI_Ioctl( (FAPEXIOCARG1_T)fd_UPI, FAPEX_IOCCOM_UPI_SET_PARAM, (FAPEXIOCARG3_T)&upi_set_Params );
        FAPI_SYS_SLEEP(20);

        // Set external latch: TSA_RES, TSB_RES, FLED high
        upi_set_Params.base_address = (uint32_t)device_ptr;
        upi_set_Params.offMask = 0x8C;
        (void)FAPEX_UPI_Ioctl( (FAPEXIOCARG1_T)fd_UPI, FAPEX_IOCCOM_UPI_SET_PARAM, (FAPEXIOCARG3_T)&upi_set_Params );
        FAPI_SYS_SLEEP(20);

        FAPEX_UPI_Ioctl((FAPEXIOCARG1_T)fd_UPI, FAPEX_IOCCOM_UPI_UNLOCK, (FAPEXIOCARG3_T)NULL);
        //////////////////

        // Restore configuration
        configuration.chipSelect = store_cs;             // Set UPI select to stored value
        configuration.mode       = store_mode;           // Set mode on stored value
        (void)FAPEX_UPI_Ioctl( (FAPEXIOCARG1_T)fd_UPI, FAPEX_IOCCOM_UPI_SET_CONFIG, (FAPEXIOCARG3_T)&configuration );

        return_value = FAPI_OK;
    }

    (void)FAPEX_UPI_Close( (FAPEXIOCARG1_T)fd_UPI );

    return( return_value );
#else

    #ifdef _LINUX_FAPI
    FAPEX_GPIO_OpenParamStrT   paramsGPIO;
    #else
    FAPI_GPIO_OpenParamsT   paramsGPIO;
    FAPI_SYS_HandleT        gpioHandle;
    #endif
    uint32_t                gpiofuc;
    int32_t                    GPIO_ID;

    paramsGPIO.pin = HardwareReset_pin;
    paramsGPIO.function = FAPEX_GPIO_OUT_0;//0;
    #ifndef _LINUX_FAPI
    paramsGPIO.function = FAPI_GPIO_OUT_0;//0;
    paramsGPIO.callback = NULL;
    #endif

    #ifdef _LINUX_FAPI
    FAPI_SYS_PRINT_MSG("\nFE_Reset GPIO num: %d\n",paramsGPIO.pin);
    GPIO_ID = FAPEX_GPIO_Open(FAPEXDEV_GPIO_MINOR_FIRST,O_RDWR,(FAPEXOPENARG3_T)&paramsGPIO);

    #if 0
    gpiofuc = FAPEX_GPIO_OUT_0;
    FAPEX_GPIO_Ioctl(GPIO_ID,FAPEX_IOCCOM_GPIO_WRITEBIT,(FAPEXIOCARG3_T)&gpiofuc);
    FAPI_SYS_PRINT_MSG("\nFE_Reset GPIO low\n");
    FE_Wait(20);

    gpiofuc = FAPEX_GPIO_OUT_1;
    FAPEX_GPIO_Ioctl(GPIO_ID,FAPEX_IOCCOM_GPIO_WRITEBIT,(FAPEXIOCARG3_T)&gpiofuc);
    FAPI_SYS_PRINT_MSG("\nFE_Reset GPIO high\n");
    FE_Wait(20);
    #else
    FE_Wait(20);
    FAPEX_GPIO_Close(GPIO_ID);
    paramsGPIO.function = FAPEX_GPIO_OUT_1;//0;
    GPIO_ID = FAPEX_GPIO_Open(FAPEXDEV_GPIO_MINOR_FIRST,O_RDWR,(FAPEXOPENARG3_T)&paramsGPIO);
    #endif

    FAPEX_GPIO_Close(GPIO_ID);
    #else
    gpioHandle = FAPI_GPIO_Open(&paramsGPIO,NULL);
    FAPI_GPIO_WriteBit(gpioHandle,1);
    FAPI_GPIO_Close(gpioHandle);
    #endif
    return( 0 );
#endif
}

/*
********************************************************************************
** \brief Set antenna power
**
** This function uses board-specific means to set antenna power.
**
** \param slotNum Hardware/board slot at which the frontend is located
** \param doPower 1 = voltage, 0 = no voltage
**
** \return Possible return codes:
**         - #FAPI_OK
**         - #FE_ERR_BAD_PARAMETER
**         - #FE_ERR_FEATURE_NOT_SUPPORTED
********************************************************************************
*/
int32_t FE_ControlAntPow(uint8_t slotNum, uint8_t doPower)
{
    /* TODO */
    return FE_ERR_FEATURE_NOT_SUPPORTED;
}

uint32_t FE_GetMillis(void)
{
    return (uint32_t)FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC);
}

int32_t FE_I2C_SetProtocol(int32_t hp, FAPEX_I2C_ProtocolT protocol)
{
    /* TODO */
    #ifdef _LINUX_FAPI
    FAPEX_I2C_ProtocolT params;

    params = protocol;
    FAPEX_I2C_Ioctl(hp,
                    FAPEX_IOCCOM_I2C_SET_PROTOCOL,
                    (FAPEXIOCARG3_T)&params);
    #endif
    return( 0 );
}

/* +++*********************************************************************** \
*  FE_CreateSemaphore
*
*  parameters       *name:          pointer to semaphore name
*
*  return           *void:          pointer to semaphore created
*
*  description
*       This function should create a semaphore to be used by the fe driver.  A value of 0
*       should be returned if the semaphore could not be created.  If more than 1 thread
*       can read or write to the the fe at a time then they must be protected.  If only 1 thread
*       can access a fe ic at a time then the function does not need to create semaphore but
*       should still return a non zero value.
*
*       This is processor and OS dependant so this function will need to be modified for the
*       implemented processor and OS.
\************************************************************************--- */
void* FE_CreateSemaphore(const uint8_t* name)
{
    return (void *)FAPI_SYS_CREATE_SEMAPHORE(1);
}

/* +++*********************************************************************** \
*  FE_GetSemaphore
*
*  parameters       *id:            pointer to semaphore to get
*
*  return           void:
*
*  description
*       This waits for a designated semaphore to become available and then grabs it.    If more than
*       1 thread can read or write to the the fe at a time then they must be protected.  If only
*       1 thread can access a fe ic at a time then the function does not need to do anything.
*
*       This is processor and OS dependant so this function will need to be modified for the
*       implemented processor and OS.
\************************************************************************--- */
void FE_GetSemaphore(void* id)
{
    FAPI_SYS_GET_SEMAPHORE((FAPI_SYS_SemaphoreT)id, FAPI_SYS_SUSPEND);
}

/* +++*********************************************************************** \
*  fe_sem_pu
*
*  parameters       *id:            pointer to semaphore to put/release
*
*  return           void:
*
*  description
*       This releases the designated semaphore so it is available for another thread.    If more than
*       1 thread can read or write to the the fe at a time then they must be protected.  If only
*       1 thread can access a fe ic at a time then the function does not need to do anything.
*
*       This is processor and OS dependant so this function will need to be modified for the
*       implemented processor and OS.
\************************************************************************--- */
void FE_SetSemaphore(void* id)
{
    FAPI_SYS_SET_SEMAPHORE((FAPI_SYS_SemaphoreT)id, FAPI_SYS_NO_SUSPEND);
}

void FE_DestroySemaphore(void* id)
{
    FAPI_SYS_DESTROY_SEMAPHORE((FAPI_SYS_SemaphoreT)id);
}

/*!
********************************************************************************
** \brief Read data from an I2C channel
**
**         This function reads data from an I2C block.
**         Two modes are supported:
**         - Direct receive of \c rxDataLength bytes (set \c txDataLength = 0).
**         - First transmit \c txDataLength bytes, then restart and
**           receive \c rxDataLength bytes.
**
** \param  id             Valid driver identifier previously returned by
**                        #FAPEX_I2C_Open.
** \param  msSlaveAddress Slave address of external I2C device.
** \param  txDataPtr      Pointer to data to be transmitted to an I2C device.
** \param  txDataLength   Length of data to be transmitted in bytes.
** \param  rxDataPtr      Pointer to data to be received from I2C device.
** \param  rxDataLength   Length of data to be received from an I2C device.
**
**
********************************************************************************
*/
int32_t FE_I2C_Read(const int32_t id, const uint8_t msSlaveAddress,
                    uint8_t* txDataPtr, const uint32_t txDataLength,
                    uint8_t* rxDataPtr, uint32_t rxDataLength)
{
    FAPEX_I2C_RWParamStrT  params;
    int32_t  ret;

    params.msSlaveAddress = msSlaveAddress;
    params.txDataPtr      = txDataPtr;
    params.txDataLength   = txDataLength;
    params.rxDataPtr      = rxDataPtr;
    params.rxDataLength   = rxDataLength;

    ret = FAPEX_I2C_Ioctl(id, FAPEX_IOCCOM_I2C_READ, (FAPEXIOCARG3_T)&params);

    if(ret < 0) {
        ret = -errno;
    }
    return ret;
}
#ifdef _LINUX_FAPI
//int32_t FAPI_I2C_Read(const FAPI_SYS_HandleT handle, const uint8_t msSlaveAddress,
int32_t FAPI_I2C_Read(const int32_t handle, const uint8_t msSlaveAddress,

                      uint8_t* txDataPtr, const uint32_t txDataLength,
                      uint8_t* rxDataPtr, uint32_t rxDataLength)
{
    int32_t  id = (int32_t)handle;

    return FE_I2C_Read(id, msSlaveAddress, txDataPtr, txDataLength, rxDataPtr, rxDataLength);
}
#endif

/*!
********************************************************************************
**
** \brief  Write data to I2C channel.
**
**         This function writes data to an I2C block.
**
** \param  id             Valid driver identifier previously returned by
**                        #FAPEX_I2C_Open.
** \param  msSlaveAddress Slave address of external I2C device.
** \param  txDataPtr      Pointer to data to be transmitted to I2C device
**                        including subaddress if needed.
** \param  txDataLength   Length of data to be transmitted.
**
**
********************************************************************************
*/
int32_t FE_I2C_Write(const int32_t id,
                     const uint8_t msSlaveAddress,
                     uint8_t* txDataPtr,
                     const uint32_t txDataLength)
{
    FAPEX_I2C_RWParamStrT  params;
    int32_t  ret;

    params.msSlaveAddress = msSlaveAddress;
    params.txDataPtr      = txDataPtr;
    params.txDataLength   = txDataLength;

    ret = FAPEX_I2C_Ioctl(id, FAPEX_IOCCOM_I2C_WRITE, (FAPEXIOCARG3_T)&params);

    if(ret < 0) {
        ret = -errno;
    }
    return ret;
}
#ifdef _LINUX_FAPI
//int32_t FAPI_I2C_Write(const FAPI_SYS_HandleT handle, const uint8_t msSlaveAddress,
int32_t FAPI_I2C_Write(const int32_t handle, const uint8_t msSlaveAddress,

                       uint8_t* txDataPtr, const uint32_t txDataLength)
{
    int32_t  id = (int32_t)handle;

    return FE_I2C_Write(id, msSlaveAddress, txDataPtr, txDataLength);
}
#endif

static void fillStdDiseqc(uint8_t *cmd, FE_TUNE_SAT_S *tuneParams,
                          uint8_t isHighBand)
{
    /* set the committed switches anyhow */
    cmd[0] = 0xE0;
    cmd[1] = 0x10;
    cmd[2] = 0x38;
    cmd[3] = 0xF0;
    /* check hi/lo band */
    if (isHighBand)
        cmd[3] |= 0x01; /* switch to high band */
    /* check polarity */
    if (tuneParams->pol == FE_POL_HOR)
        cmd[3] |= 0x02; /* switch on for horizontal */

    /* check sat ID, position */
    if ((tuneParams->satID & 0x01) != 0)
        cmd[3] |= 0x04; /* position 1 or 3 */
    /* check sat ID, option */
    if ((tuneParams->satID & 0x02) != 0)
        cmd[3] |= 0x08; /* position 2 or 3 */
}
#if 0
void FE_PrepareDiseqcCmd(FE_DISEQC_E diseqcType, FE_TUNE_SAT_S *tuneParams,
                         uint8_t isHighBand, uint8_t *cmd, uint8_t *lenCmd)
{
//    uint8_t     ucBytes[2][6],DCCNUM[2];

    /*************************************************
     DiSEqC command settings
     *************************************************/
    /* check, which DiSEqC is selected */
    if (diseqcType == FE_DISEQC_1_0)
    { /* send committed switches */
        fillStdDiseqc(cmd, tuneParams, isHighBand);
        *lenCmd = 4;
    }
    else if (diseqcType ==FE_DISEQC_1_1_DTH)
    { /* send uncommitted and committed switches */
        cmd[0] = 0xE0;
        cmd[1] = 0x10;
        cmd[2] = 0x39;
        cmd[3] = 0xF0;
        if ((tuneParams->satID & 0x04) == 0x04) /* check lowest bit */
            cmd[3] |= 0x01;
        if ((tuneParams->satID & 0x08) == 0x08) /* check next bit */
            cmd[3] |= 0x02;
        if ((tuneParams->satID & 0x10) == 0x10) /* check next bit */
            cmd[3] |= 0x04;
        if ((tuneParams->satID & 0x20) == 0x20) /* check next bit */
            cmd[3] |= 0x08;
        fillStdDiseqc(cmd+4, tuneParams, isHighBand);
        *lenCmd = 8;
    }
    else if (diseqcType == FE_DISEQC_1_1_SMATV)
    { /* send frequency and committed switches */
        uint32_t ulFreq = tuneParams->frequency;
        uint8_t len = 4;

        cmd[0] = 0xE0;
        cmd[1] = 0x71;
        cmd[2] = 0x58;
        /* first two nibbles */
        cmd[3]  = (uint8_t)((ulFreq / 10000000) << 4);
        ulFreq -= (ulFreq / 10000000) * 10000000;
        cmd[3] += (uint8_t)(ulFreq /  1000000);
        ulFreq -= (ulFreq / 1000000) * 1000000;

        if(ulFreq != 0)
        {                 /* one more byte to send */
            len++;
            cmd[4]  = (uint8_t)((ulFreq / 100000) << 4);
            ulFreq     -= (ulFreq / 100000) * 100000;
            cmd[4] += (uint8_t)((ulFreq /  10000));
            ulFreq     -= (ulFreq /  10000) *  10000;
        }
        if(ulFreq != 0)
        {                 /* one more byte to send */
            len++;
            cmd[5]  = (uint8_t)((ulFreq / 1000) << 4);
            ulFreq     -= (ulFreq / 1000) * 1000;
            cmd[5] += (uint8_t)(ulFreq /  100);
            ulFreq     -= (ulFreq /  100) *  100;
        }
        fillStdDiseqc(cmd+len, tuneParams, isHighBand);
        *lenCmd = len+4;
    }
    else if (diseqcType == FE_DISEQC_1_2)
    { /* send position and committed switches */
        cmd[0] = 0xE0;
        cmd[1] = 0x30;
        cmd[2] = 0x6B;
        cmd[3] = (uint8_t)tuneParams->satID;
        fillStdDiseqc(cmd+4, tuneParams, isHighBand);
        *lenCmd = 8;
    }
    else
    {
        *lenCmd = 0;
    }
}
#endif
int32_t FE_SetSignalRoute(feHandleT *hp,
                          FE_TUNE_PARAMS_S* pTuneParams)
{
    FE_DISEQC_CMD           msg[2];
    uint8_t                 cnt;
    int32_t                 error = FE_OK;

    if(hp->pSystemParams.sat.posBy22kHz)
    {
        if(pTuneParams->sat.satID == 0)
        {
            if(hp->feControl22kHz != NULL)
            {
                error = hp->feControl22kHz(hp->slotNum,FE_22KHZ_ON);
                if(error != FE_OK)
                {
                    return error;
                }
            }
        }
        else
        {
            if(hp->feControl22kHz != NULL)
            {
                error = hp->feControl22kHz(hp->slotNum,FE_22KHZ_OFF);
                if(error != FE_OK)
                {
                    return error;
                }
            }
        }
    }
    else
    {
        if(hp->feControl22kHz != NULL)
        {
            error = hp->feControl22kHz(hp->slotNum,FE_22KHZ_OFF);
            if(error != FE_OK)
            {
                return error;
            }
            FE_Wait(15);
        }

        if(hp->pSystemParams.sat.toneburst)
        {
            if(pTuneParams->sat.satID == 0)
            {
                if(hp->feControl22kHz != NULL)
                {
                    error = hp->feControl22kHz(hp->slotNum,FE_22KHZ_TONE_0);
                    if(error != FE_OK)
                    {
                        return error;
                    }
                }
            }
            else
            {
                if(hp->feControl22kHz != NULL)
                {
                    error = hp->feControl22kHz(hp->slotNum,FE_22KHZ_TONE_1);
                    if(error != FE_OK)
                    {
                        return error;
                    }
                }
            }
            FE_Wait(15);
        }
        if( hp->pSystemParams.sat.diseqc != FE_DISEQC_1_1_SMATV)
        {
            if(hp->feControlLnbVoltage != NULL)
            {
                error = hp->feControlLnbVoltage(hp->slotNum,
                    (pTuneParams->sat.pol==FE_POL_HOR)?FE_LNB_18V:FE_LNB_13V);
                if(error != FE_OK)
                {
                    return error;
                }
            }
        }
        else
        {
            if(hp->feControlLnbVoltage != NULL)
            {
                error = hp->feControlLnbVoltage(hp->slotNum,FE_LNB_18V);
                if(error != FE_OK)
                {
                    return error;
                }
            }
        }
        /* wait for voltage to stabilise */
        FE_Wait(15);

        if((hp->feSendDiseqcCommandFct   != NULL) &&
           (hp->pSystemParams.sat.diseqc != FE_DISEQC_NONE))
        {
            memset(msg, 0, sizeof(msg));

            FE_PrepareDiseqcCmd(&hp->pSystemParams,
                                FE_DISEQC_1_0,
                                &pTuneParams->sat,
                                &msg[0]);

            if(hp->pSystemParams.sat.diseqc > FE_DISEQC_1_0)
            {
                FE_PrepareDiseqcCmd(&hp->pSystemParams,
                                    hp->pSystemParams.sat.diseqc,
                                    &pTuneParams->sat,
                                    &msg[1]);
            }

            if( hp->pSystemParams.sat.diseqc == FE_DISEQC_1_0 )
            {
                /*!
                 * Recommended sequence of DiSEqC 1.0 commands:
                 *
                 * XX  => Continous Tone (if present)
                 * ##  => Commited messages
                 * **  => Uncommited messages
                 * oo  => SA/SB Tone Burst
                 *
                 *    |-| <- >15ms after Stop continous Tone to switch Voltage
                 * XXX_
                 *     \ |-| <- >15ms wait after Voltage switch befor start DiSEqC
                 *      \| |
                 *       --******--OO--XXX
                 *               |--| <- min 25 msec
                 *         |-----| <- typicaly 40.5 msec to 54 msec
                 */
                error = hp->feSendDiseqcCommandFct(hp->slotNum,
                                                   msg[0].txSize,
                                                   msg[0].txData,
                                                   NULL,NULL);
                if(error != FE_OK)
                {
                    return error;
                }
                /* change Framing Byte from first transmission (0xE0) to Repeated
                 * transmission (0xE1) for DiSEqC command.
                 */
                msg[0].txData[0] = 0xE1;
                for(cnt=0;cnt<hp->pSystemParams.sat.diseqc10Rep;cnt++)
                {
                    FE_Wait(25);
                    error = hp->feSendDiseqcCommandFct(hp->slotNum,
                                                       msg[0].txSize,
                                                       msg[0].txData,
                                                       NULL,NULL);
                    if(error != FE_OK)
                    {
                        return error;
                    }
                }
            }
            else if (( hp->pSystemParams.sat.diseqc == FE_DISEQC_1_1_DTH ) ||
                     ( hp->pSystemParams.sat.diseqc == FE_DISEQC_1_1_SMATV))
            {
                /*!
                 * Recommended sequence of DiSEqC 1.1 commands for repeated
                 * messages:
                 *
                 * XX  => Continous Tone (if present)
                 * ##  => Commited messages
                 * **  => Uncommited messages
                 * oo  => SA/SB Tone Burst
                 *
                 *    |-| <- >15ms after Stop continous Tone to switch Voltage
                 * XXX_
                 *     \ |-| <- >15ms wait after Voltage switch befor start DiSEqC
                 *      \| |
                 *       --######---******---######---******---######--OO--XXXXXX
                 *               |--| <- min 25 msec
                 *               |-----------| <- min 100msec
                 *         |-----| <- typicaly 45 msec to 54 msec
                 */
                error = hp->feSendDiseqcCommandFct(hp->slotNum,
                                                   msg[1].txSize,
                                                   msg[1].txData,
                                                   NULL,NULL);
                if(error != FE_OK)
                {
                    return error;
                }
                FE_Wait(25);
                error = hp->feSendDiseqcCommandFct(hp->slotNum,
                                                   msg[0].txSize,
                                                   msg[0].txData,
                                                   NULL,NULL);
                if(error != FE_OK)
                {
                    return error;
                }
                /* change Framing Byte from first transmission (0xE0) to Repeated
                 * transmission (0xE1) for both DiSEqC commands.
                 */
                msg[0].txData[0] = 0xE1;
                msg[1].txData[0] = 0xE1;
                for(cnt=0;cnt<hp->pSystemParams.sat.diseqc11Rep;cnt++)
                {
                    FE_Wait(25);
                    error = hp->feSendDiseqcCommandFct(hp->slotNum,
                                                       msg[1].txSize,
                                                       msg[1].txData,
                                                       NULL,NULL);
                    if(error != FE_OK)
                    {
                        return error;
                    }
                    if(cnt == (hp->pSystemParams.sat.diseqc11Rep-1)) break;
                    FE_Wait(25);
                    error = hp->feSendDiseqcCommandFct(hp->slotNum,
                                                       msg[0].txSize,
                                                       msg[0].txData,
                                                       NULL,NULL);
                    if(error != FE_OK)
                    {
                        return error;
                    }
                }
            }
            else if ( hp->pSystemParams.sat.diseqc == FE_DISEQC_1_2 )
            {
                /*!
                 * Recommended sequence of DiSEqC 1.2 command:
                 *
                 * XX  => Continous Tone (if present)
                 * ##  => Commited messages
                 * **  => Uncommited messages
                 * oo  => SA/SB Tone Burst
                 *
                 *    |-| <- >15ms after Stop continous Tone to switch Voltage
                 * XXX_
                 *     \ |-| <- >15ms wait after Voltage switch befor start DiSEqC
                 *      \| |
                 *       --######---******--OO--XXXXXX
                 *               |--| <- min 25 msec
                 *               |-----------| <- min 100msec
                 *         |-----| <- typicaly 45 msec to 54 msec
                 */
                error = hp->feSendDiseqcCommandFct(hp->slotNum,
                                                   msg[1].txSize,
                                                   msg[1].txData,
                                                   NULL,NULL);
                if(error != FE_OK)
                {
                    return error;
                }
                FE_Wait(25);
                error = hp->feSendDiseqcCommandFct(hp->slotNum,
                                                   msg[0].txSize,
                                                   msg[0].txData,
                                                   NULL,NULL);
                if(error != FE_OK)
                {
                    return error;
                }
            }
            FE_Wait(100);
        }
        if(labs(pTuneParams->sat.frequency - hp->pSystemParams.sat.lowFrequency)
                >= FE_BAND_SW)
        {
            if(hp->feControl22kHz != NULL)
            {
                error = hp->feControl22kHz(hp->slotNum,FE_22KHZ_ON);
                if(error != FE_OK)
                {
                    return error;
                }
            }
        }
    }
    return FE_OK;
}

void FE_PrepareDiseqcCmd(FE_SYSTEM_PARAMS_S* h,
                         FE_DISEQC_E diseqcType,
                         FE_TUNE_SAT_S *tuneParams,
                         FE_DISEQC_CMD *msg)
{
    uint32_t    freq_MHz;
    uint32_t    ifFreq_MHz;
    uint32_t    lowFreq_MHz;
    uint32_t    highFreq_MHz;
    uint8_t     isHighBand = 0;

    if( ( h          == NULL) ||
        ( tuneParams == NULL) ||
        ( msg        == NULL) ) return;

    memset(msg, 0, sizeof(FE_DISEQC_CMD));

    lowFreq_MHz  = h->sat.lowFrequency / 1000;
    highFreq_MHz = h->sat.highFrequency / 1000;
    freq_MHz = tuneParams->frequency /1000;

    if (lowFreq_MHz  == 0)
    {
        lowFreq_MHz  = highFreq_MHz;
    }
    if (highFreq_MHz == 0)
    {
        highFreq_MHz = lowFreq_MHz;
    }

    if (labs(freq_MHz - lowFreq_MHz) > 1950)/* switching frequency between High and Low Band*/
    {
        isHighBand = 1;
    }
    /*************************************************
     DiSEqC command settings
     *************************************************/
    /* check, which DiSEqC is selected */
    switch(diseqcType)
    {
        case FE_DISEQC_1_0:
            /*!
             * DiSEqC 1.0 Command
             *
             * Committed switch: 0xE0 0x10 0x38 0xFx
             * Committed switch repeated command: 0xE1 0x10 0x38 0xFx
             *
             * - bit 0
             *   High/Low band selection
             *   1: High Band 0: Low Band
             * - bit 1:
             *   Polarisation selection:
             *   1: Horizontal; 0: Vertical
             * bit 2 and 3
             *   Satellite Id:
             *   00: Sat Pos A
             *   01: Sat Pos B
             *   10: Sat Pos C
             *   11: Sat Pos D
             *
             */
            /* send committed switches */
            msg->txData[0] = 0xE0;
            msg->txData[1] = 0x10;
            msg->txData[2] = 0x38;
            msg->txData[3] = 0xF0;
            /* check hi/lo band */
            if (isHighBand)
                msg->txData[3] |= 0x01; /* switch to high band */
            /* check polarity */
            if (tuneParams->pol == FE_POL_HOR)
                msg->txData[3] |= 0x02; /* switch on for horizontal */

            /* check sat ID, position */
            if ((tuneParams->satID & 0x01) != 0)
                msg->txData[3] |= 0x04; /* position 1 or 3 */
            /* check sat ID, option */
            if ((tuneParams->satID & 0x02) != 0)
                msg->txData[3] |= 0x08; /* position 2 or 3 */
            msg->txSize = 4;
            /*only send data to device*/
            msg->rxEnabled = 0;
            break;
        case FE_DISEQC_1_1_DTH:
            /* send uncommitted switches */
            msg->txData[0] = 0xE0;
            msg->txData[1] = 0x10;
            msg->txData[2] = 0x39;
            msg->txData[3] = 0xF0;
            if ((tuneParams->satID & 0x04) == 0x04) /* check lowest bit */
                msg->txData[3] |= 0x01;
            if ((tuneParams->satID & 0x08) == 0x08) /* check next bit */
                msg->txData[3] |= 0x02;
            if ((tuneParams->satID & 0x10) == 0x10) /* check next bit */
                msg->txData[3] |= 0x04;
            if ((tuneParams->satID & 0x20) == 0x20) /* check next bit */
                msg->txData[3] |= 0x08;
            msg->txSize = 4;
            /*only send data to device*/
            msg->rxEnabled = 0;
            break;
        case FE_DISEQC_1_1_SMATV:
            msg->txData[0] = 0xE0;
            msg->txData[1] = 0x10;
            msg->txData[2] = 0x5A;
            msg->txData[3] = 0x00;
            /*
             * Data0:
             *  b7-b5: Slot Nummer 0-7 (SCR ID)
             *  b4   : PosA/PosB
             *  b3   : Vert/Horiz
             *  b2   : L-Band/H-Band,
             */
            switch(tuneParams->ifFrequency)
            {
                default:
                case 1284: msg->txData[3] = 0x00; break; // 000x xxxx
                case 1400: msg->txData[3] = 0x20; break; // 001x xxxx
                case 1516: msg->txData[3] = 0x40; break; // 010x xxxx
                case 1632: msg->txData[3] = 0x60; break; // 011x xxxx
                case 1748: msg->txData[3] = 0x80; break; // 100x xxxx
                case 1864: msg->txData[3] = 0xA0; break; // 101x xxxx
                case 1980: msg->txData[3] = 0xC0; break; // 110x xxxx
                case 2196: msg->txData[3] = 0xE0; break; // 111x xxxx
            }
            /* check Position */
            if( tuneParams->satID)
            { /* PosA / PosB */
                msg->txData[3] |= 0x10; // xxx1 xxxx
            }
            /* check polarity */
            if (tuneParams->pol == FE_POL_HOR)
            {  /* switch on for horizontal */
                msg->txData[3] |= 0x08; // xxxx 1xxx
            }
            /* check high/low band */
            if (isHighBand)
            { /* switch to high band */
                msg->txData[3] |= 0x04; // xxxx x1xx
            }

            /*!
             * Data0:
             *  b1-b0: 2 MSB bits for tuning word
             *  Data1:
             *  b7-b0: 8 LSB bits fr tuning word
             *
             * rnd((Ft - Floc + Fslot)/4 - 350),
             * Ft:      transponder frequenz
             * Floc:    LNB local osc.frequenz (High or Low Band)
             * Fslot:   slot Frequenz (IF-Frequency)
             */
            if(isHighBand)
            {
                ifFreq_MHz = labs(freq_MHz - highFreq_MHz);
            }
            else
            {
                ifFreq_MHz = labs(freq_MHz - lowFreq_MHz);
            }
            ifFreq_MHz += tuneParams->ifFrequency;
            ifFreq_MHz /= 4;
            ifFreq_MHz -= 350;

            msg->txData[3] |= (ifFreq_MHz>>8) & 0x03;  // xxxx xx11
            msg->txData[4]  =  ifFreq_MHz     & 0xFF;

            msg->txSize= 5;
            /*only send data to device*/
            msg->rxEnabled = 0;
            break;
        case FE_DISEQC_1_2:
            /* send position */
            msg->txData[0] = 0xE0;
            msg->txData[1] = 0x30;
            msg->txData[2] = 0x6B;
            msg->txData[3] = (uint8_t)tuneParams->satID;
            msg->txSize = 4;
            /*only send data to device*/
            msg->rxEnabled = 0;
            break;
        default:
            msg->txSize = 0;
            /*only send data to device*/
            msg->rxEnabled = 0;
            break;
    }
}

void FE_SetTuneTimeout(uint8_t index, uint32_t symbolRate)
{
    uint64_t uiTimeth;

    if(index >= FE_MAX_NUM) return;

    //Channel lock time increase while symbol rate decrease.Give the max waiting time for different symbolrates.
    if(symbolRate < 5000)
    {
        uiTimeth = 7500;
    }
    else if(symbolRate < 10000)
    {
        uiTimeth = 2000;
    }
    else
    {
        uiTimeth = 1000;
    }
    #ifdef _LINUX_FAPI
    tuneEndTime[index] = FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC) + uiTimeth;
    #else
    tuneEndTime[index] = FAPI_TIMER_GetTimeStamp(FAPI_TIMER_RESOLUTION_1_MSEC) + uiTimeth;
    #endif
}


uint8_t FE_IsTuneTimeout(uint8_t index)
{
    if(index >= FE_MAX_NUM) return 0;

    #ifdef _LINUX_FAPI
    if( tuneEndTime[index] >= FAPEX_TIMER_GetTimeStamp(FAPEX_TIMER_RESOLUTION_1_MSEC))
    #else
    if( tuneEndTime[index] >= FAPI_TIMER_GetTimeStamp(FAPI_TIMER_RESOLUTION_1_MSEC))
    #endif
    {
        return 0;
    }
    else
    {
        return 1;
    }
}

