/*!
*******************************************************************************
**
** \file      ./subsystems/paint/JPEG/include/JPEG_api.h
**
** \brief     JPEG decoder library API.
**
**            (C) Fujitsu Semiconductor Europe 2002 - 2008
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   $Id: JPEG_api.h 2574 2009-03-09 13:26:03Z wlaris $
**
******************************************************************************/
#ifndef _JPEG_API_H_
#define _JPEG_API_H_

// #include <ftypes.h>
// #include <fmodids.h>
#include <stdint.h>
#include <fapi/sys_error.h>

#include <jpeg/jpeglib.h>

/*---------------------------------------------------------------------------*/
/* constants and macros                                                      */
/*---------------------------------------------------------------------------*/

#define     U16     uint16_t
#define     U32     uint32_t

// min. buffer size for JPEG_environment_t:staticP
#define     JPEG_MIN_STATIC_BUFFER_SIZE     32768

/*---------------------------------------------------------------------------*/
/* types, enums and structures                                               */
/*---------------------------------------------------------------------------*/

enum J_MESSAGE_CODE
{   JERR_ARITH_NOTIMPL= -100,   //"Sorry, there are legal restrictions on arithmetic coding"
    JERR_BAD_COMPONENT_ID,      //"Invalid component ID in SOS"
    JERR_BAD_DCT_COEF,          //"DCT coefficient out of range"
    JERR_BAD_DCTSIZE,           //"IDCT output block size not supported"
    JERR_BAD_HUFF_TABLE,        //"Bogus Huffman table definition"
    JERR_BAD_J_COLORSPACE,      //"Bogus JPEG colorspace"
    JERR_BAD_LENGTH,            //"Bogus marker length"
    JERR_BAD_MCU_SIZE,          //"Sampling factors too large for interleaved scan"
    JERR_BAD_PRECISION,         //"Unsupported JPEG data precision"
    JERR_BAD_SAMPLING,          //"Bogus sampling factors"
    JERR_BAD_STATE,             //"Improper call to JPEG library"
    JERR_COMPONENT_COUNT,       //"Too many color components"
    JERR_EOI_EXPECTED,          //"Didn't expect more than one scan"
    JERR_IMAGE_TOO_BIG,         //"Maximum supported image dimension is 65500 pixels"
    JERR_INPUT_EOF,             //"Premature end of input file"
    JERR_NO_QUANT_TABLE,        //"Quantization table was not defined"
    JERR_NO_SOI,                //"Not a JPEG file"
    JERR_SOF_DUPLICATE,         //"Invalid JPEG file structure: two SOF markers"
    JERR_SOF_NO_SOS,            //"Invalid JPEG file structure: missing SOS marker"
    JERR_SOF_UNSUPPORTED,       //"Unsupported JPEG process"
    JERR_SOI_DUPLICATE,         //"Invalid JPEG file structure: two SOI markers"
    JERR_SOS_NO_SOF,            //"Invalid JPEG file structure: SOS before SOF"
    JERR_BAD_SCALING            //"Enlargement not implemented"
};

typedef enum
{   SCREEN_PAL43= 0,            //720 x 576
    SCREEN_PAL169,              //720 x 576 wide
    SCREEN_NTSC43,              //720 x 480
    SCREEN_NTSC169,             //720 x 480 wide
    SCREEN_LCD15,               //1024 x 768
    SCREEN_LCD4,                //320 x 240
    SCREEN_LCD7W,               //480 x 270 wide
    SCREEN_HD,                  //1920 x 1080 wide
    SCREEN_CUSTOM
}JPEG_screen_t;

typedef enum
{   ROT_0,
    ROT_90,
    ROT_180,
    ROT_270
}JPEG_orientation_t;

typedef struct
{   unsigned short      sizeX;          // horizontal size of source image in pixels
    unsigned short      sizeY;          // vertical size of source image in pixels
    unsigned short      dpiX;           // resolution in DPI or 0 if not known
    unsigned short      dpiY;           // resolution in DPI or 0 if not known
    char                color;          // TRUE for color, FALSE for grayscale
    char                progressive;    // TRUE for progressive mode, FALSE for baseline
}JPEG_info_t;

typedef struct
{   void *              staticP;        // a pointer to at least 32 kByte of RAM for header storage
    JPEG_screen_t       screen;         // defines the current display
    unsigned long       temporalbytes;  // amount of memory required to decode this image.
    void *              temporalP;      // pointer to at least the amount of bytes requested in temporalbytes;
    void *              userDataP;      // this pointer will be passed to the data loading function
    short               (*loadFct)(short required, void* userDataP, char* destP);
}JPEG_environment_t;

typedef struct
{   unsigned short      left;           // leftmost pixel of source image that should be displayed
    unsigned short      right;          // rightmost pixel of source image that should be displayed
    unsigned short      top;            // top row of source image that should be displayed
    unsigned short      bottom;         // bottom row of source image that should be displayed
    JPEG_orientation_t  orient;         // allows to rotate the resulting picture on the fly
    char                fast;           // TRUE for use of a faster but less accurate DCT method
}JPEG_image_t;

typedef struct
{   unsigned char *     screenP;        // a pointer to the YUV 4:2:2 graphics plane
    unsigned long       rowlongs;       // amount of 32-bit words between two lines of the graphics plane
    unsigned short      left;           // leftmost position of image in pixels. Must be smaller than right. Even numbers only.
    unsigned short      right;          // rightmost position of the image in pixels. Must be below 720.
    unsigned short      top;            // topmost position of the image in pixels. Must be smaller than bottom.
    unsigned short      bottom;         // lowest position of the image in pixels. Must be below 576 (480).
    unsigned long       bgcolor;        // color for unused regions of the screen as Cb Y0 Cr Y1. 0 for transparency
}JPEG_destination_t;

typedef struct
{   JPEG_environment_t  env;
    JPEG_info_t         info;
    JPEG_image_t        image;
    JPEG_destination_t  dest;
}JPEG_t;

/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

short JPEG_ReadHeader(JPEG_t* jpegP);   // Always read the header first

short JPEG_Decode(JPEG_t* jpegP);       // Decode it as a single piece

short JPEG_StartDecode(JPEG_t* jpegP);  //          OR

char JPEG_Continue(JPEG_t* jpegP);      // Decode it in multiple pieces

void JPEG_FinishDecode(JPEG_t* jpegP);

#ifdef __cplusplus
}
#endif

#endif /* _JPEG_API_H_ */
/* end of JPEG_api.h */
