/*
 * jpeglib.h
 *
 * Copyright (C) 1991-1998, Thomas G. Lane.
 * This file is part of the Independent JPEG Group's software.
 * For conditions of distribution and use, see the accompanying README file.
 *
 * This file defines the application interface for the JPEG library.
 * Most applications using the library need only include this file,
 * and perhaps jerror.h if they want to know the exact error codes.
 */

#ifndef JPEGLIB_H
#define JPEGLIB_H

/*
 * First we include the configuration files that record how this
 * installation of the JPEG library is set up.  jconfig.h can be
 * generated automatically for many systems.  jmorecfg.h contains
 * manual configuration options that most people need not worry about.
 */

//#ifndef JCONFIG_INCLUDED    /* in case jinclude.h already did */
//#include "jconfig.h"        /* widely used configuration options */
//#endif
//#include <sys/types.h>
#include <stddef.h>
#include <jpeg/jmorecfg.h>        /* seldom changed options */
#include <stdint.h>
#include <fapi/sys_error.h>

typedef struct
{    unsigned char *     div[10];
    short                Y[1900];
    short                U[1900];
    short                V[1900];
    unsigned char        Y2[1080];
    long                ratioX;
    long                ratioY;
    long                accuY;
    unsigned short        leftsource;
    unsigned short        firstsourceY;
    unsigned short        sizeX;
    long                sizeY;
    short                countY;
    short                orient;
    short                blowup;
    volatile unsigned long*    screenP;
    unsigned long        rowlongs;
}JPEG_scaler_t;



/* Various constants determining the sizes of things.
 * All of these are specified by the JPEG standard, so don't change them
 * if you want to be compatible.
 */

#define DCTSIZE            8    /* The basic DCT block is 8x8 samples */
#define DCTSIZE2        64    /* DCTSIZE squared; # of elements in a block */
#define NUM_QUANT_TBLS      4    /* Quantization tables are numbered 0..3 */
#define NUM_HUFF_TBLS       4    /* Huffman tables are numbered 0..3 */
#define NUM_ARITH_TBLS      16    /* Arith-coding tables are numbered 0..15 */
#define MAX_COMPS_IN_SCAN   4    /* JPEG limit on # of components in one scan */
#define MAX_SAMP_FACTOR     4    /* JPEG limit on sampling factors */

/* Unfortunately, some bozo at Adobe saw no reason to be bound by the standard;
 * the PostScript DCT filter can emit files with many more than 10 blocks/MCU.
 * If you happen to run across such a file, you can up D_MAX_BLOCKS_IN_MCU
 * to handle it.  We even let you do this from the jconfig.h file.  However,
 * we strongly discourage changing C_MAX_BLOCKS_IN_MCU; just because Adobe
 * sometimes emits noncompliant files doesn't mean you should too.
 */
#define D_MAX_BLOCKS_IN_MCU   10     /* decompressor's limit on blocks per MCU */


/* Data structures for images (arrays of samples and of DCT coefficients).
 * On 80x86 machines, the image arrays are too big for near pointers,
 * but the pointer arrays can fit in near memory.
 */

typedef JSAMPLE *JSAMPROW;            /* ptr to one image row of pixel samples. */
typedef JSAMPROW *JSAMPARRAY;        /* ptr to some rows (a 2-D sample array) */
typedef JSAMPARRAY *JSAMPIMAGE;        /* a 3-D sample array: top index is color */

typedef JCOEF JBLOCK[DCTSIZE2];        /* one block of coefficients */
typedef JBLOCK *JBLOCKROW;            /* pointer to one row of coefficient blocks */
typedef JBLOCKROW *JBLOCKARRAY;        /* a 2-D array of coefficient blocks */
typedef JBLOCKARRAY *JBLOCKIMAGE;    /* a 3-D array of coefficient blocks */

typedef JCOEF *JCOEFPTR;            /* useful in a couple of places */


/* Types for JPEG compression parameters and working tables. */


/* DCT coefficient quantization tables. */

typedef struct {
  /* This array gives the coefficient quantizers in natural array order
   * (not the zigzag order in which they are stored in a JPEG DQT marker).
   * CAUTION: IJG versions prior to v6a kept this array in zigzag order.
   */
  UINT16 quantval[DCTSIZE2];    /* quantization step for each coefficient */
 
} JQUANT_TBL;


/* Huffman coding tables. */

typedef struct {
  /* These two fields directly represent the contents of a JPEG DHT marker */
  UINT8 bits[17];        /* bits[k] = # of symbols with codes of */
                /* length k bits; bits[0] is unused */
  UINT8 huffval[256];        /* The symbols, in order of incr code length */
 
} JHUFF_TBL;


/* Basic info about one component (color channel). */

typedef struct {
  /* These values are fixed over the whole image. */
  /* For compression, they must be supplied by parameter setup; */
  /* for decompression, they are read from the SOF marker. */
  int component_id;        /* identifier for this component (0..255) */
  int component_index;        /* its index in SOF or cinfo->comp_info[] */
  int h_samp_factor;        /* horizontal sampling factor (1..4) */
  int v_samp_factor;        /* vertical sampling factor (1..4) */
  int quant_tbl_no;        /* quantization table selector (0..3) */
  /* These values may vary between scans. */
  /* For compression, they must be supplied by parameter setup; */
  /* for decompression, they are read from the SOS marker. */
  /* The decompressor output side may not use these variables. */
  int dc_tbl_no;        /* DC entropy table selector (0..3) */
  int ac_tbl_no;        /* AC entropy table selector (0..3) */
  
  /* Remaining fields should be treated as private by applications. */
  
  /* These values are computed during compression or decompression startup: */
  /* Component's size in DCT blocks.
   * Any dummy blocks added to complete an MCU are not counted; therefore
   * these values do not depend on whether a scan is interleaved or not.
   */
  JDIMENSION width_in_blocks;
  JDIMENSION height_in_blocks;
  /* Size of a DCT block in samples.  Always DCTSIZE for compression.
   * For decompression this is the size of the output from one DCT block,
   * reflecting any scaling we choose to apply during the IDCT step.
   * Values of 1,2,4,8 are likely to be supported.  Note that different
   * components may receive different IDCT scalings.
   */
  int DCT_scaled_size;
  /* The downsampled dimensions are the component's actual, unpadded number
   * of samples at the main buffer (preprocessing/compression interface), thus
   * downsampled_width = ceil(image_width * Hi/Hmax)
   * and similarly for height.  For decompression, IDCT scaling is included, so
   * downsampled_width = ceil(image_width * Hi/Hmax * DCT_scaled_size/DCTSIZE)
   */
  JDIMENSION downsampled_width;     /* actual width in samples */
  JDIMENSION downsampled_height; /* actual height in samples */

  /* These values are computed before starting a scan of the component. */
  /* The decompressor output side may not use these variables. */
  int MCU_width;        /* number of blocks per MCU, horizontally */
  int MCU_height;        /* number of blocks per MCU, vertically */
  int MCU_blocks;        /* MCU_width * MCU_height */
  int MCU_sample_width;        /* MCU width in samples, MCU_width*DCT_scaled_size */
  int last_col_width;        /* # of non-dummy blocks across in last MCU */
  int last_row_height;        /* # of non-dummy blocks down in last MCU */

  /* Saved quantization table for component; NULL if none yet saved.
   * See jdinput.c comments about the need for this information.
   * This field is currently used only for decompression.
   */
  JQUANT_TBL * quant_table;

  /* Private per-component storage for DCT or IDCT subsystem. */
  void * dct_table;
} jpeg_component_info;


/* Known color spaces. */

typedef enum {
    JCS_UNKNOWN,        /* error/unspecified */
    JCS_GRAYSCALE,        /* monochrome */
    JCS_RGB,        /* red/green/blue */
    JCS_YCbCr,        /* Y/Cb/Cr (also known as YUV) */
    JCS_CMYK,        /* C/M/Y/K */
    JCS_YCCK        /* Y/Cb/Cr/K */
} J_COLOR_SPACE;

/* DCT/IDCT algorithm options. */

typedef enum {
    JDCT_ISLOW,        /* slow but accurate integer algorithm */
    JDCT_IFAST,        /* faster, less accurate integer method */
} J_DCT_METHOD;


/* Routines that are to be used by both halves of the library are declared
 * to receive a pointer to this structure.  There are no actual instances of
 * jpeg_common_struct, only of jpeg_compress_struct and jpeg_decompress_struct.
 */

typedef struct jpeg_decompress_struct * j_decompress_ptr;


/* Master record for a decompression instance */

struct jpeg_decompress_struct {
  struct jpeg_memory_mgr * mem;        /* Memory manager module */
  int global_state;                    /* For checking call sequence validity */

  char *            pool[2];
  unsigned long        used[2];

  /* Source of compressed data */
  struct jpeg_source_mgr * src;

  /* Basic description of image --- filled in by jpeg_read_header(). */
  /* Application may inspect these values to decide how to process image. */

  JDIMENSION image_width;    /* nominal image width (from SOF marker) */
  JDIMENSION image_height;    /* nominal image height */
  int num_components;        /* # of color components in JPEG image */
  J_COLOR_SPACE jpeg_color_space; /* colorspace of JPEG image */

  /* Decompression processing parameters --- these fields must be set before
   * calling jpeg_start_decompress().  Note that jpeg_read_header() initializes
   * them to default values.
   */

  unsigned int scale_num, scale_denom; /* fraction by which to scale image */

  boolean buffered_image;    /* TRUE=multiple output passes */

  J_DCT_METHOD dct_method;    /* IDCT algorithm selector */
  boolean do_fancy_upsampling;    /* TRUE=apply fancy upsampling */
  boolean do_block_smoothing;    /* TRUE=apply interblock smoothing */

  /* Description of actual output image that will be returned to application.
   * These fields are computed by jpeg_start_decompress().
   * You can also use jpeg_calc_output_dimensions() to determine these values
   * in advance of calling jpeg_start_decompress().
   */

  JDIMENSION output_width;    /* scaled image width */
  JDIMENSION output_height;    /* scaled image height */
  
  /* State variables: these variables indicate the progress of decompression.
   * The application may examine these but must not modify them.
   */

  /* Row index of next scanline to be read from jpeg_read_scanlines().
   * Application may use this to control its processing loop, e.g.,
   * "while (output_scanline < output_height)".
   */
  JDIMENSION output_scanline;    /* 0 .. output_height-1  */

  /* Current input scan number and number of iMCU rows completed in scan.
   * These indicate the progress of the decompressor input side.
   */
  int input_scan_number;    /* Number of SOS markers seen so far */
  JDIMENSION input_iMCU_row;    /* Number of iMCU rows completed */

  /* The "output scan number" is the notional scan being displayed by the
   * output side.  The decompressor will not allow output scan/row number
   * to get ahead of input scan/row, but it can fall arbitrarily far behind.
   */
  int output_scan_number;    /* Nominal scan number being displayed */
  JDIMENSION output_iMCU_row;    /* Number of iMCU rows read */

  /* Current progression status.  coef_bits[c][i] indicates the precision
   * with which component c's DCT coefficient i (in zigzag order) is known.
   * It is -1 when no data has yet been received, otherwise it is the point
   * transform (shift) value for the most recent scan of the coefficient
   * (thus, 0 at completion of the progression).
   * This pointer is NULL when reading a non-progressive file.
   */
  int (*coef_bits)[DCTSIZE2];    /* -1 or current Al value for each coef */

  /* Internal JPEG parameters --- the application usually need not look at
   * these fields.  Note that the decompressor output side may not use
   * any parameters that can change between scans.
   */

  /* Quantization and Huffman tables are carried forward across input
   * datastreams when processing abbreviated JPEG datastreams.
   */

  JQUANT_TBL * quant_tbl_ptrs[NUM_QUANT_TBLS];
  /* ptrs to coefficient quantization tables, or NULL if not defined */

  JHUFF_TBL * dc_huff_tbl_ptrs[NUM_HUFF_TBLS];
  JHUFF_TBL * ac_huff_tbl_ptrs[NUM_HUFF_TBLS];
  /* ptrs to Huffman coding tables, or NULL if not defined */

  /* These parameters are never carried across datastreams, since they
   * are given in SOF/SOS markers or defined to be reset by SOI.
   */

  int data_precision;        /* bits of precision in image data */

  jpeg_component_info * comp_info;
  /* comp_info[i] describes component that appears i'th in SOF */

  boolean progressive_mode;    /* TRUE if SOFn specifies progressive mode */
  boolean arith_code;        /* TRUE=arithmetic coding, FALSE=Huffman */

  unsigned int restart_interval; /* MCUs per restart interval, or 0 for no restart */

  /* These fields record data obtained from optional markers recognized by
   * the JPEG library.
   */
  boolean saw_JFIF_marker;    /* TRUE iff a JFIF APP0 marker was found */
  /* Data copied from JFIF marker; only valid if saw_JFIF_marker is TRUE: */
  UINT8 density_unit;        /* JFIF code for pixel size units */
  UINT16 X_density;        /* Horizontal pixel density */
  UINT16 Y_density;        /* Vertical pixel density */
  boolean saw_Adobe_marker;    /* TRUE iff an Adobe APP14 marker was found */
  UINT8 Adobe_transform;    /* Color transform code from Adobe marker */

  /* Remaining fields are known throughout decompressor, but generally
   * should not be touched by a surrounding application.
   */

  /*
   * These fields are computed during decompression startup
   */
  int max_h_samp_factor;    /* largest h_samp_factor */
  int max_v_samp_factor;    /* largest v_samp_factor */

  int min_DCT_scaled_size;    /* smallest DCT_scaled_size of any component */

  JDIMENSION total_iMCU_rows;    /* # of iMCU rows in image */
  /* The coefficient controller's input and output progress is measured in
   * units of "iMCU" (interleaved MCU) rows.  These are the same as MCU rows
   * in fully interleaved JPEG scans, but are used whether the scan is
   * interleaved or not.  We define an iMCU row as v_samp_factor DCT block
   * rows of each component.  Therefore, the IDCT output contains
   * v_samp_factor*DCT_scaled_size sample rows of a component per iMCU row.
   */

  JSAMPLE * sample_range_limit; /* table for fast range-limiting */

  /*
   * These fields are valid during any one scan.
   * They describe the components and MCUs actually appearing in the scan.
   * Note that the decompressor output side must not use these fields.
   */
  int comps_in_scan;        /* # of JPEG components in this scan */
  jpeg_component_info * cur_comp_info[MAX_COMPS_IN_SCAN];
  /* *cur_comp_info[i] describes component that appears i'th in SOS */

  JDIMENSION MCUs_per_row;    /* # of MCUs across the image */
  JDIMENSION MCU_rows_in_scan;    /* # of MCU rows in the image */

  int blocks_in_MCU;        /* # of DCT blocks per MCU */
  int MCU_membership[D_MAX_BLOCKS_IN_MCU];
  /* MCU_membership[i] is index in cur_comp_info of component owning */
  /* i'th block in an MCU */

  int Ss, Se, Ah, Al;        /* progressive JPEG parameters for scan */

  /* This field is shared between entropy decoder and marker parser.
   * It is either zero or the code of a JPEG marker that has been
   * read from the data source, but has not yet been processed.
   */
  int unread_marker;

  /*
   * Links to decompression subobjects (methods, private variables of modules)
   */
  struct jpeg_decomp_master * master;
  struct jpeg_d_main_controller * main;
  struct jpeg_d_coef_controller * coef;
  struct jpeg_input_controller * inputctl;
  struct jpeg_marker_reader * marker;
  struct jpeg_entropy_decoder * entropy;
  struct jpeg_inverse_dct * idct;
  struct jpeg_upsampler * upsample;
};


/* "Object" declarations for JPEG modules that may be supplied or called
 * directly by the surrounding application.
 * As with all objects in the JPEG library, these structs only define the
 * publicly visible methods and state variables of a module.  Additional
 * private fields may exist after the public ones.
 */


/* Data source object for decompression */

struct jpeg_source_mgr {
  const JOCTET * next_input_byte; /* => next byte to read from buffer */
  size_t bytes_in_buffer;    /* # of bytes remaining in buffer */

  JMETHOD(void, init_source, (j_decompress_ptr cinfo));
  JMETHOD(void, fill_input_buffer, (j_decompress_ptr cinfo));
  JMETHOD(void, skip_input_data, (j_decompress_ptr cinfo, long num_bytes));
  JMETHOD(boolean, resync_to_restart, (j_decompress_ptr cinfo, int desired));
  
  void * userP;
  short (*loadFct)(short required, void* userDataP, char* destP);
};


/* Memory manager object.
 * Allocates "small" objects (a few K total), "large" objects (tens of K),
 * and "really big" objects (virtual arrays with backing store if needed).
 * The memory manager does not allow individual objects to be freed; rather,
 * each created object is assigned to a pool, and whole pools can be freed
 * at once.  This is faster and more convenient than remembering exactly what
 * to free, especially where malloc()/free() are not too speedy.
 * NB: alloc routines never return NULL.  They exit to error_exit if not
 * successful.
 */

#define JPOOL_PERMANENT    0    /* lasts until master record is destroyed */
#define JPOOL_IMAGE    1    /* lasts until done with image/datastream */

typedef struct jvirt_sarray_control * jvirt_sarray_ptr;
typedef struct jvirt_barray_control * jvirt_barray_ptr;


struct jpeg_memory_mgr {
  /* Method pointers */
  JMETHOD(void *, alloc_small, (j_decompress_ptr cinfo, int pool_id, size_t sizeofobject));
  JMETHOD(JSAMPARRAY, alloc_sarray, (j_decompress_ptr cinfo, int pool_id, JDIMENSION samplesperrow, JDIMENSION numrows));
  JMETHOD(JBLOCKARRAY, alloc_barray, (j_decompress_ptr cinfo, int pool_id, JDIMENSION blocksperrow, JDIMENSION numrows));
  JMETHOD(jvirt_barray_ptr, request_virt_barray, (j_decompress_ptr cinfo, int pool_id, boolean pre_zero, JDIMENSION blocksperrow, JDIMENSION numrows, JDIMENSION maxaccess));
  JMETHOD(void, realize_virt_arrays, (j_decompress_ptr cinfo));
  JMETHOD(JBLOCKARRAY, access_virt_barray, (j_decompress_ptr cinfo, jvirt_barray_ptr ptr, JDIMENSION start_row, JDIMENSION num_rows, boolean writable));
};


/* Routine signature for application-supplied marker processing methods.
 * Need not pass marker code since it is stored in cinfo->unread_marker.
 */
typedef JMETHOD(boolean, jpeg_marker_parser_method, (j_decompress_ptr cinfo));


/* Declarations for routines called by application.
 * The JPP macro hides prototype parameters from compilers that can't cope.
 * Note JPP requires double parentheses.
 */

#ifdef HAVE_PROTOTYPES
#define JPP(arglist)    arglist
#else
#define JPP(arglist)    ()
#endif


/* Initialization of JPEG compression objects.
 * jpeg_create_compress() and jpeg_create_decompress() are the exported
 * names that applications should call.  These expand to calls on
 * jpeg_CreateCompress and jpeg_CreateDecompress with additional information
 * passed for version mismatch checking.
 * NB: you must set up the error-manager BEFORE calling jpeg_create_xxx.
 */

EXTERN(void) jpeg_CreateDecompress JPP((j_decompress_ptr cinfo));

/* Standard data source and destination managers: stdio streams. */
/* Caller is responsible for opening the file before and closing after. */
EXTERN(void) jpeg_stdio_src JPP((j_decompress_ptr cinfo, FILE * infile));

EXTERN(JQUANT_TBL *) jpeg_alloc_quant_table JPP((j_decompress_ptr cinfo));
EXTERN(JHUFF_TBL *) jpeg_alloc_huff_table JPP((j_decompress_ptr cinfo));

/* Decompression startup: read start of JPEG datastream to see what's there */
EXTERN(int) jpeg_read_header JPP((j_decompress_ptr cinfo));
/* Return value is one of: */
#define JPEG_SUSPENDED        0 /* Suspended due to lack of input data */
#define JPEG_HEADER_OK        1 /* Found valid image datastream */
#define JPEG_HEADER_TABLES_ONLY    2 /* Found valid table-specs-only datastream */
/* If you pass require_image = TRUE (normal case), you need not check for
 * a TABLES_ONLY return code; an abbreviated file will cause an error exit.
 * JPEG_SUSPENDED is only possible if you use a data source module that can
 * give a suspension return (the stdio source module doesn't).
 */

/* Main entry points for decompression */
EXTERN(short) jpeg_start_decompress JPP((j_decompress_ptr cinfo));
EXTERN(char) jpeg_read_scanlines JPP((j_decompress_ptr cinfo, JPEG_scaler_t * scalerP));
EXTERN(boolean) jpeg_finish_decompress JPP((j_decompress_ptr cinfo));

/* Additional entry points for buffered-image mode. */
EXTERN(boolean) jpeg_has_multiple_scans JPP((j_decompress_ptr cinfo));
EXTERN(int) jpeg_consume_input JPP((j_decompress_ptr cinfo));
/* Return value is one of: */
/* #define JPEG_SUSPENDED    0    Suspended due to lack of input data */
#define JPEG_REACHED_SOS    1 /* Reached start of new scan */
#define JPEG_REACHED_EOI    2 /* Reached end of image */
#define JPEG_ROW_COMPLETED    3 /* Completed one iMCU row */
#define JPEG_SCAN_COMPLETED    4 /* Completed last iMCU row of a scan */

/* Precalculate output dimensions for current decompression parameters. */
EXTERN(void) jpeg_calc_output_dimensions JPP((j_decompress_ptr cinfo));

/* Read or write raw DCT coefficients --- useful for lossless transcoding. */
EXTERN(jvirt_barray_ptr *) jpeg_read_coefficients JPP((j_decompress_ptr cinfo));

/* Default restart-marker-resync procedure for use by data source modules */
EXTERN(boolean) jpeg_resync_to_restart JPP((j_decompress_ptr cinfo, int desired));


/* These marker codes are exported since applications and data source modules
 * are likely to want to use them.
 */

#define JPEG_RST0    0xD0    /* RST0 marker code */
#define JPEG_EOI    0xD9    /* EOI marker code */
#define JPEG_APP0    0xE0    /* APP0 marker code */
#define JPEG_COM    0xFE    /* COM marker code */


/*
 * The JPEG library modules define JPEG_INTERNALS before including this file.
 * The internal structure declarations are read only when that is true.
 * Applications using the library should not include jpegint.h, but may wish
 * to include jerror.h.
 */

#ifdef JPEG_INTERNALS
#include "jpegint.h"        /* fetch private declarations */
#include <jpeg/JPEG_api.h>
//#include "jerror.h"        /* fetch error codes too */
#endif

#endif /* JPEGLIB_H */
