/*
 * jdhuff.c
 *
 * Copyright (C) 1991-1997, Thomas G. Lane.
 * This file is part of the Independent JPEG Group's software.
 * For conditions of distribution and use, see the accompanying README file.
 *
 * This file contains Huffman entropy decoding routines.
 *
 * Much of the complexity here has to do with supporting input suspension.
 * If the data source module demands suspension, we want to be able to back
 * up to the start of the current MCU.  To do this, we copy state variables
 * into local working storage, and update them back to the permanent
 * storage only upon successful completion of an MCU.
 */

#define JPEG_INTERNALS
#include "jinclude.h"
#include <jpeg/jpeglib.h>
#include "jdhuff.h"        /* Declarations shared with jdphuff.c */


/*
 * Expanded entropy decoder object for Huffman decoding.
 *
 * The savable_state subrecord contains fields that change within an MCU,
 * but must not be updated permanently until we complete the MCU.
 */

typedef struct {
  int last_dc_val[MAX_COMPS_IN_SCAN]; /* last DC coef for each component */
} savable_state;


typedef struct {
  struct jpeg_entropy_decoder pub; /* public fields */

  /* These fields are loaded into local variables at start of each MCU.
   * In case of suspension, we exit WITHOUT updating them.
   */
  bitread_perm_state bitstate;    /* Bit buffer at start of MCU */
  savable_state saved;        /* Other state at start of MCU */

  /* These fields are NOT loaded into local working state. */
  unsigned int restarts_to_go;    /* MCUs left in this restart interval */

  /* Pointers to derived tables (these workspaces have image lifespan) */
  d_derived_tbl * dc_derived_tbls[NUM_HUFF_TBLS];
  d_derived_tbl * ac_derived_tbls[NUM_HUFF_TBLS];

  /* Precalculated info set up by start_pass for use in decode_mcu: */

  /* Pointers to derived tables to be used for each block within an MCU */
  d_derived_tbl * dc_cur_tbls[D_MAX_BLOCKS_IN_MCU];
  d_derived_tbl * ac_cur_tbls[D_MAX_BLOCKS_IN_MCU];
  /* Whether we care about the DC and AC coefficient values for each block */
} huff_entropy_decoder;

typedef huff_entropy_decoder * huff_entropy_ptr;


/*
 * Initialize for a Huffman-compressed scan.
 */

void start_pass_huff_decoder(j_decompress_ptr cinfo)
{
  huff_entropy_ptr        entropy= (huff_entropy_ptr) cinfo->entropy;
  int                    ci, blkn, dctbl, actbl;
  jpeg_component_info * compptr;

  for(ci= 0; ci < cinfo->comps_in_scan; ci++)
  { 
    compptr= cinfo->cur_comp_info[ci];
    dctbl= compptr->dc_tbl_no;
    actbl= compptr->ac_tbl_no;
    /* Compute derived values for Huffman tables */
    /* We may do this more than once for a table, but it's not expensive */
    jpeg_make_d_derived_tbl(cinfo, TRUE, dctbl, &entropy->dc_derived_tbls[dctbl]);
    jpeg_make_d_derived_tbl(cinfo, FALSE, actbl, &entropy->ac_derived_tbls[actbl]);
    /* Initialize DC predictions to 0 */
    entropy->saved.last_dc_val[ci]= 0;
  }

  /* Precalculate decoding info for each block in an MCU of this scan */
  for(blkn= 0; blkn < cinfo->blocks_in_MCU; blkn++)
  {
    ci= cinfo->MCU_membership[blkn];
    compptr= cinfo->cur_comp_info[ci];
    /* Precalculate which table to use for each block */
    entropy->dc_cur_tbls[blkn]= entropy->dc_derived_tbls[compptr->dc_tbl_no];
    entropy->ac_cur_tbls[blkn]= entropy->ac_derived_tbls[compptr->ac_tbl_no];
  }

  /* Initialize bitread state variables */
  entropy->bitstate.bits_left= 0;
  entropy->bitstate.get_buffer= 0; /* unnecessary, but keeps Purify quiet */
  entropy->pub.insufficient_data= FALSE;

  /* Initialize restart counter */
  entropy->restarts_to_go= cinfo->restart_interval;
}


/*
 * Compute the derived values for a Huffman table.
 * This routine also performs some validation checks on the table.
 *
 * Note this is also used by jdphuff.c.
 */

void jpeg_make_d_derived_tbl (j_decompress_ptr cinfo, boolean isDC, int tblno, d_derived_tbl ** pdtbl)
{
  JHUFF_TBL *            htbl;
  d_derived_tbl *        dtbl;
  int                    p, i, l, si, numsymbols;
  int                    lookbits, ctr;
  char                    huffsize[257];
  unsigned int            huffcode[257];
  unsigned int            code;

  /* Note that huffsize[] and huffcode[] are filled in code-length order,
   * paralleling the order of the symbols themselves in htbl->huffval[].
   */

  /* Find the input Huffman table */
  htbl= isDC ? cinfo->dc_huff_tbl_ptrs[tblno] : cinfo->ac_huff_tbl_ptrs[tblno];

  /* Allocate a workspace if we haven't already done so. */
  if(*pdtbl == NULL)
    *pdtbl= (d_derived_tbl *)(*cinfo->mem->alloc_small) (cinfo, JPOOL_IMAGE, SIZEOF(d_derived_tbl));
  dtbl= *pdtbl;
  dtbl->pub= htbl;        /* fill in back link */
  
  /* Figure C.1: make table of Huffman code length for each symbol */

  p= 0;
  for(l= 1; l <= 16; l++)
  {
    i= (int) htbl->bits[l];
    while (i--)
      huffsize[p++]= (char) l;
  }
  huffsize[p]= 0;
  numsymbols= p;
  
  /* Figure C.2: generate the codes themselves */
  /* We also validate that the counts represent a legal Huffman code tree. */
  
  code= 0;
  si= huffsize[0];
  p= 0;
  while (huffsize[p])
  {
    while(((int) huffsize[p]) == si)
    {
      huffcode[p++]= code;
      code++;
    }
    code <<= 1;
    si++;
  }

  /* Figure F.15: generate decoding tables for bit-sequential decoding */

  p= 0;
  for(l= 1; l <= 16; l++)
  {
    if(htbl->bits[l])
    { /* valoffset[l] = huffval[] index of 1st symbol of code length l,
       * minus the minimum code of length l
       */
      dtbl->valoffset[l]= (INT32) p - (INT32) huffcode[p];
      p += htbl->bits[l];
      dtbl->maxcode[l]= huffcode[p-1]; /* maximum code of length l */
    }
    else
      dtbl->maxcode[l] = -1;    /* -1 if no codes of this length */
  }
  dtbl->maxcode[17]= 0xFFFFFL; /* ensures jpeg_huff_decode terminates */

  /* Compute lookahead tables to speed up decoding.
   * First we set all the table entries to 0, indicating "too long";
   * then we iterate through the Huffman codes that are short enough and
   * fill in all the entries that correspond to bit sequences starting
   * with that code.
   */

  MEMZERO(dtbl->look_nbits, SIZEOF(dtbl->look_nbits));

  p= 0;
  for(l= 1; l <= HUFF_LOOKAHEAD; l++)
  {
    for(i= 1; i <= (int) htbl->bits[l]; i++, p++)
    { /* l = current code's length, p = its index in huffcode[] & huffval[]. */
      /* Generate left-justified code followed by all possible bit sequences */
      lookbits= huffcode[p] << (HUFF_LOOKAHEAD-l);
      for(ctr= 1 << (HUFF_LOOKAHEAD-l); ctr > 0; ctr--)
      { dtbl->look_nbits[lookbits]= l;
        dtbl->look_sym[lookbits]= htbl->huffval[p];
        lookbits++;
      }
    }
  }
}


/*
 * Out-of-line code for bit fetching (shared with jdphuff.c).
 * See jdhuff.h for info about usage.
 * Note: current values of get_buffer and bits_left are passed as parameters,
 * but are returned in the corresponding fields of the state struct.
 */
 
#define MIN_GET_BITS  (BIT_BUF_SIZE-7)


void jpeg_fill_bit_buffer(j_decompress_ptr cinfo, bit_buf_type * get_buffer, int * bits_left, int nbits)
/* Load up the bit buffer to a depth of at least nbits */
{
    unsigned char        c;

  /* Attempt to load at least MIN_GET_BITS bits into get_buffer. */
  /* (It is assumed that no request will be for more than that many bits.) */
  /* We fail to do so only if we hit a marker or are forced to suspend. */

  if(cinfo->unread_marker == 0)
  {    /* cannot advance past a marker */
    while((*bits_left) < MIN_GET_BITS)
    { /* Attempt to read a byte */
      if(cinfo->src->bytes_in_buffer == 0)
        (*cinfo->src->fill_input_buffer) (cinfo);

      cinfo->src->bytes_in_buffer--;
      c= *cinfo->src->next_input_byte++;

      /* If it's 0xFF, check and discard stuffed zero byte */
      if(c == 0xFF)
      { /* Loop here to discard any padding FF's on terminating marker,
         * so that we can save a valid unread_marker value.  NOTE: we will
         * accept multiple FF's followed by a 0 as meaning a single FF data
         * byte.  This data pattern is not valid according to the standard.
         */
        do 
        { if(cinfo->src->bytes_in_buffer == 0)
            (*cinfo->src->fill_input_buffer)(cinfo);
            
          cinfo->src->bytes_in_buffer--;
          c= *cinfo->src->next_input_byte++;
        }while (c == 0xFF);

        if(c == 0)
          /* Found FF/00, which represents an FF data byte */
          c= 0xFF;
        else
        { /* Oops, it's actually a marker indicating end of compressed data.
           * Save the marker code for later use.
           * Fine point: it might appear that we should save the marker into
           * bitread working state, not straight into permanent state.  But
           * once we have hit a marker, we cannot need to suspend within the
           * current MCU, because we will read no more bytes from the data
           * source.  So it is OK to update permanent state right away.
           */
          cinfo->unread_marker= c;
          /* See if we need to insert some fake zero bits. */
          goto no_more_bytes;
        }
      }
      /* OK, load c into get_buffer */
      *get_buffer= (*get_buffer << 8) | c;
      *bits_left= (*bits_left) + 8;
    } /* end while */
  }
  else
  { 
    no_more_bytes:
    /* We get here if we've read the marker that terminates the compressed
     * data segment.  There should be enough bits in the buffer register
     * to satisfy the request; if so, no problem.
     */
    if(nbits > (*bits_left))
    {
      /* Uh-oh.  Report corrupted data to user and stuff zeroes into
       * the data stream, so that we can produce some kind of image.
       * We use a nonvolatile flag to ensure that only one warning message
       * appears per data segment.
       */
      cinfo->entropy->insufficient_data= TRUE;
      
        /* Fill the buffer with zero bits */
      *get_buffer= (*get_buffer) << (MIN_GET_BITS - *bits_left);
      *bits_left= MIN_GET_BITS;
    }
  }
}


/*
 * Out-of-line code for Huffman code decoding.
 * See jdhuff.h for info about usage.
 */

int jpeg_huff_decode(j_decompress_ptr cinfo, bit_buf_type * get_buffer, int * bits_left, d_derived_tbl * htbl, int min_bits)
{
  int            l= min_bits;
  INT32            code;

  /* HUFF_DECODE has determined that the code is at least min_bits */
  /* bits long, so fetch that many bits in one swoop. */

  if((*bits_left) < l)
   jpeg_fill_bit_buffer(cinfo, get_buffer, bits_left, l);

  code= ((*get_buffer >> (*bits_left -= l))) & ((1<<l)-1);

  /* Collect the rest of the Huffman code one bit at a time. */
  /* This is per Figure F.16 in the JPEG spec. */

  while(code > htbl->maxcode[l])
  { code <<= 1;
    if((*bits_left) < l)
      jpeg_fill_bit_buffer(cinfo, get_buffer, bits_left, 1);
         
    code |= ((*get_buffer >> (*bits_left -= 1)) & 1);
    l++;
  }

  /* With garbage input we may reach the sentinel value l = 17. */
  if(l > 16)
    return 0;            /* fake a zero as the safest result */

  return htbl->pub->huffval[code + htbl->valoffset[l]];
}



/*
 * Check for a restart marker & resynchronize decoder.
 * Returns FALSE if must suspend.
 */

boolean process_restart(j_decompress_ptr cinfo)
{
  huff_entropy_ptr    entropy= (huff_entropy_ptr) cinfo->entropy;
  int                ci;

  /* Throw away any unused bits remaining in bit buffer; */
  /* include any full bytes in next_marker's count of discarded bytes */
  cinfo->marker->discarded_bytes += entropy->bitstate.bits_left / 8;
  entropy->bitstate.bits_left= 0;

  /* Advance past the RSTn marker */
  if(! (*cinfo->marker->read_restart_marker) (cinfo))
    return FALSE;

  /* Re-initialize DC predictions to 0 */
  for(ci= 0; ci < cinfo->comps_in_scan; ci++)
    entropy->saved.last_dc_val[ci]= 0;

  /* Reset restart counter */
  entropy->restarts_to_go= cinfo->restart_interval;

  /* Reset out-of-data flag, unless read_restart_marker left us smack up
   * against a marker.  In that case we will end up treating the next data
   * segment as empty, and we can avoid producing bogus output pixels by
   * leaving the flag set.
   */
  if (cinfo->unread_marker == 0)
    entropy->pub.insufficient_data= FALSE;

  return TRUE;
}


/*
 * Decode and return one MCU's worth of Huffman-compressed coefficients.
 * The coefficients are reordered from zigzag order into natural array order,
 * but are not dequantized.
 *
 * The i'th block of the MCU is stored into the block pointed to by
 * MCU_data[i].  WE ASSUME THIS AREA HAS BEEN ZEROED BY THE CALLER.
 * (Wholesale zeroing is usually a little faster than retail...)
 *
 * Returns FALSE if data source requested suspension.  In that case no
 * changes have been made to permanent state.  (Exception: some output
 * coefficients may already have been assigned.  This is harmless for
 * this module, since we'll just re-assign them on the next call.)
 */

boolean decode_mcu(j_decompress_ptr cinfo, JBLOCKROW *MCU_data)
{
  huff_entropy_ptr            entropy= (huff_entropy_ptr) cinfo->entropy;
  int                        blkn;
  bit_buf_type                get_buffer;
  int                        bits_left;
  int                        s, k, r;
  int                        nb, look, ci;
  JBLOCKROW                    block;
  d_derived_tbl *            dctbl;
  d_derived_tbl *            actbl;

  /* Process restart marker if needed; may have to suspend */
  if(cinfo->restart_interval)
  { if(entropy->restarts_to_go == 0)
      if(! process_restart(cinfo))
        return FALSE;
  }

  /* If we've run out of data, just leave the MCU set to zeroes.
   * This way, we return uniform gray for the remainder of the segment.
   */
  if(! entropy->pub.insufficient_data)
  {
    get_buffer= entropy->bitstate.get_buffer;
    bits_left= entropy->bitstate.bits_left;

    /* Outer loop handles each block in the MCU */

    for(blkn= 0; blkn < cinfo->blocks_in_MCU; blkn++)
    {
      block= MCU_data[blkn];
      dctbl= entropy->dc_cur_tbls[blkn];
      actbl= entropy->ac_cur_tbls[blkn];

      /* Decode a single block's worth of coefficients */

      /* Section F.2.2.1: decode the DC coefficient difference */
      if(bits_left < HUFF_LOOKAHEAD)
      { jpeg_fill_bit_buffer(cinfo, &get_buffer, &bits_left, 0);
        if(bits_left < HUFF_LOOKAHEAD)
        { nb= 1;
          goto label1;
        }
      }
      look= (((get_buffer >> (bits_left - HUFF_LOOKAHEAD))) & ((1<<HUFF_LOOKAHEAD)-1));
      if((nb= dctbl->look_nbits[look]) != 0)
      { bits_left -= nb;
        s= dctbl->look_sym[look];
      }
      else
      { nb= HUFF_LOOKAHEAD+1;
label1:    s= jpeg_huff_decode(cinfo, &get_buffer, &bits_left, dctbl, nb);
      }
      
      if(s)
      { if(bits_left < s)
          jpeg_fill_bit_buffer(cinfo, &get_buffer, &bits_left, s);
     
        r= (((get_buffer >> (bits_left -= s))) & ((1<<s)-1));
        s= (r < (1<<(s-1)) ? r + ((-1<<s) + 1) : r);
      }

      /* Convert DC difference to actual value, update last_dc_val */
      ci= cinfo->MCU_membership[blkn];
      s += entropy->saved.last_dc_val[ci];
      entropy->saved.last_dc_val[ci]= s;
      /* Output the DC coefficient (assumes jpeg_natural_order[0] = 0) */
      (*block)[0]= s;

      /* Section F.2.2.2: decode the AC coefficients */
      /* Since zeroes are skipped, output area must be cleared beforehand */
      for(k= 1; k < DCTSIZE2; k++)
      { if(bits_left < HUFF_LOOKAHEAD)
        { jpeg_fill_bit_buffer(cinfo, &get_buffer, &bits_left, 0);
          if(bits_left < HUFF_LOOKAHEAD)
          { nb= 1;
            goto label2;
          }
        }
        look= (((get_buffer >> (bits_left - HUFF_LOOKAHEAD))) & ((1<<HUFF_LOOKAHEAD)-1));
        if((nb= actbl->look_nbits[look]) != 0)
        { bits_left -= nb;
          s= actbl->look_sym[look];
        }
        else
        { nb= HUFF_LOOKAHEAD+1;
label2:      s= jpeg_huff_decode(cinfo, &get_buffer, &bits_left, actbl, nb);
        }
            
        r= s >> 4;
        s &= 15;
      
        if(s)
        { k += r;
          if(bits_left < s)
            jpeg_fill_bit_buffer(cinfo, &get_buffer, &bits_left, s);
             
          r= (((get_buffer >> (bits_left -= s))) & ((1<<s)-1));
          s= (r < (1<<(s-1)) ? r + ((-1<<s) + 1) : r);
            /* Output coefficient in natural (dezigzagged) order.
             * Note: the extra entries in jpeg_natural_order[] will save us
             * if k >= DCTSIZE2, which could happen if the data is corrupted.
             */
          (*block)[jpeg_natural_order[k]]= s;
        }
        else
        { if(r != 15)
            break;
          k += 15;
        }
      }
    }
    entropy->bitstate.get_buffer= get_buffer;
    entropy->bitstate.bits_left= bits_left;
  }

  /* Account for restart interval (no-op if not using restarts) */
  entropy->restarts_to_go--;

  return TRUE;
}


/*
 * Module initialization routine for Huffman entropy decoding.
 */

void jinit_huff_decoder(j_decompress_ptr cinfo)
{
  huff_entropy_ptr    entropy;
  int                i;

  entropy= (huff_entropy_ptr)(*cinfo->mem->alloc_small) (cinfo, JPOOL_IMAGE, SIZEOF(huff_entropy_decoder));
  cinfo->entropy= (struct jpeg_entropy_decoder *) entropy;
  entropy->pub.start_pass= start_pass_huff_decoder;
  entropy->pub.decode_mcu= decode_mcu;

  /* Mark tables unallocated */
  for(i= 0; i < NUM_HUFF_TBLS; i++)
  { entropy->dc_derived_tbls[i]= NULL;
    entropy->ac_derived_tbls[i]= NULL;
  }
}
