/*
 * jdmaster.c
 *
 * Copyright (C) 1991-1997, Thomas G. Lane.
 * This file is part of the Independent JPEG Group's software.
 * For conditions of distribution and use, see the accompanying README file.
 *
 * This file contains master control logic for the JPEG decompressor.
 * These routines are concerned with selecting the modules to be executed
 * and with determining the number of passes and the work to be done in each
 * pass.
 */

#define JPEG_INTERNALS
#include "jinclude.h"
#include <jpeg/jpeglib.h>


/* Private state */

typedef struct {
  struct jpeg_decomp_master pub; /* public fields */

  int pass_number;        /* # of passes completed */

} my_decomp_master;

typedef my_decomp_master * my_master_ptr;


/*
 * Compute output image dimensions and related values.
 * NOTE: this is exported for possible use by application.
 * Hence it mustn't do anything that can't be done twice.
 * Also note that it may be called before the master module is initialized!
 */

void jpeg_calc_output_dimensions (j_decompress_ptr cinfo)
/* Do computations that are needed before master selection phase */
{ int                    ci;
  jpeg_component_info    *compptr;

  /* Compute actual output image dimensions and DCT scaling choices. */
  if(cinfo->scale_num * 8 <= cinfo->scale_denom)
  { /* Provide 1/8 scaling */
    cinfo->output_width= (JDIMENSION)jdiv_round_up((long) cinfo->image_width, 8L);
    cinfo->output_height= (JDIMENSION)jdiv_round_up((long) cinfo->image_height, 8L);
    cinfo->min_DCT_scaled_size= 1;
  } 
  else if(cinfo->scale_num * 4 <= cinfo->scale_denom)
  { /* Provide 1/4 scaling */
    cinfo->output_width= (JDIMENSION)jdiv_round_up((long) cinfo->image_width, 4L);
    cinfo->output_height= (JDIMENSION)jdiv_round_up((long) cinfo->image_height, 4L);
    cinfo->min_DCT_scaled_size= 2;
  } 
  else if(cinfo->scale_num * 2 <= cinfo->scale_denom)
  { /* Provide 1/2 scaling */
    cinfo->output_width= (JDIMENSION)jdiv_round_up((long) cinfo->image_width, 2L);
    cinfo->output_height= (JDIMENSION)jdiv_round_up((long) cinfo->image_height, 2L);
    cinfo->min_DCT_scaled_size= 4;
  }
  else
  { /* Provide 1/1 scaling */
    cinfo->output_width= cinfo->image_width;
    cinfo->output_height= cinfo->image_height;
    cinfo->min_DCT_scaled_size= DCTSIZE;
  }
  /* In selecting the actual DCT scaling for each component, we try to
   * scale up the chroma components via IDCT scaling rather than upsampling.
   * This saves time if the upsampler gets to use 1:1 scaling.
   * Note this code assumes that the supported DCT scalings are powers of 2.
   */
  for(ci= 0, compptr= cinfo->comp_info; ci < cinfo->num_components; ci++, compptr++)
  { int ssize= cinfo->min_DCT_scaled_size;
    while (ssize < DCTSIZE && (compptr->h_samp_factor * ssize * 2 <= cinfo->max_h_samp_factor * cinfo->min_DCT_scaled_size) && (compptr->v_samp_factor * ssize * 2 <= cinfo->max_v_samp_factor * cinfo->min_DCT_scaled_size))
    { ssize= ssize * 2;
    }
    compptr->DCT_scaled_size= ssize;
  }

  /* Recompute downsampled dimensions of components;
   * application needs to know these if using raw downsampled data.
   */
  for(ci= 0, compptr= cinfo->comp_info; ci < cinfo->num_components; ci++, compptr++)
  { /* Size in samples, after IDCT scaling */
    compptr->downsampled_width= (JDIMENSION)jdiv_round_up((long) cinfo->image_width * (long) (compptr->h_samp_factor * compptr->DCT_scaled_size), (long) (cinfo->max_h_samp_factor * DCTSIZE));
    compptr->downsampled_height = (JDIMENSION)jdiv_round_up((long) cinfo->image_height * (long) (compptr->v_samp_factor * compptr->DCT_scaled_size), (long) (cinfo->max_v_samp_factor * DCTSIZE));
  }
}


/*
 * Several decompression processes need to range-limit values to the range
 * 0..MAXJSAMPLE; the input value may fall somewhat outside this range
 * due to noise introduced by quantization, roundoff error, etc.  These
 * processes are inner loops and need to be as fast as possible.  On most
 * machines, particularly CPUs with pipelines or instruction prefetch,
 * a (subscript-check-less) C table lookup
 *        x = sample_range_limit[x];
 * is faster than explicit tests
 *        if (x < 0)  x = 0;
 *        else if (x > MAXJSAMPLE)  x = MAXJSAMPLE;
 * These processes all use a common table prepared by the routine below.
 *
 * For most steps we can mathematically guarantee that the initial value
 * of x is within MAXJSAMPLE+1 of the legal range, so a table running from
 * -(MAXJSAMPLE+1) to 2*MAXJSAMPLE+1 is sufficient.  But for the initial
 * limiting step (just after the IDCT), a wildly out-of-range value is 
 * possible if the input data is corrupt.  To avoid any chance of indexing
 * off the end of memory and getting a bad-pointer trap, we perform the
 * post-IDCT limiting thus:
 *        x = range_limit[x & MASK];
 * where MASK is 2 bits wider than legal sample data, ie 10 bits for 8-bit
 * samples.  Under normal circumstances this is more than enough range and
 * a correct output will be generated; with bogus input data the mask will
 * cause wraparound, and we will safely generate a bogus-but-in-range output.
 * For the post-IDCT step, we want to convert the data from signed to unsigned
 * representation by adding CENTERJSAMPLE at the same time that we limit it.
 * So the post-IDCT limiting table ends up looking like this:
 *   CENTERJSAMPLE,CENTERJSAMPLE+1,...,MAXJSAMPLE,
 *   MAXJSAMPLE (repeat 2*(MAXJSAMPLE+1)-CENTERJSAMPLE times),
 *   0          (repeat 2*(MAXJSAMPLE+1)-CENTERJSAMPLE times),
 *   0,1,...,CENTERJSAMPLE-1
 * Negative inputs select values from the upper half of the table after
 * masking.
 *
 * We can save some space by overlapping the start of the post-IDCT table
 * with the simpler range limiting table.  The post-IDCT table begins at
 * sample_range_limit + CENTERJSAMPLE.
 *
 * Note that the table is allocated in near data space on PCs; it's small
 * enough and used often enough to justify this.
 */

void prepare_range_limit_table (j_decompress_ptr cinfo)
/* Allocate and fill in the sample_range_limit table */
{ JSAMPLE *        table;
  int            i;

  table= (JSAMPLE *)(*cinfo->mem->alloc_small) (cinfo, JPOOL_IMAGE, (5 * (MAXJSAMPLE+1) + CENTERJSAMPLE) * SIZEOF(JSAMPLE));
  table += (MAXJSAMPLE+1);    /* allow negative subscripts of simple table */
  cinfo->sample_range_limit= table;
  /* First segment of "simple" table: limit[x] = 0 for x < 0 */
  MEMZERO(table - (MAXJSAMPLE+1), (MAXJSAMPLE+1) * SIZEOF(JSAMPLE));
  /* Main part of "simple" table: limit[x] = x */
  for(i= 0; i <= MAXJSAMPLE; i++)
    table[i]= (JSAMPLE) i;
  table += CENTERJSAMPLE;    /* Point to where post-IDCT table starts */
  /* End of simple table, rest of first half of post-IDCT table */
  for(i= CENTERJSAMPLE; i < 2*(MAXJSAMPLE+1); i++)
    table[i] = MAXJSAMPLE;
  /* Second half of post-IDCT table */
  MEMZERO(table + (2 * (MAXJSAMPLE+1)), (2 * (MAXJSAMPLE+1) - CENTERJSAMPLE) * SIZEOF(JSAMPLE));
  MEMCOPY(table + (4 * (MAXJSAMPLE+1) - CENTERJSAMPLE), cinfo->sample_range_limit, CENTERJSAMPLE * SIZEOF(JSAMPLE));
}


/*
 * Master selection of decompression modules.
 * This is done once at jpeg_start_decompress time.  We determine
 * which modules will be used and give them appropriate initialization calls.
 * We also initialize the decompressor input side to begin consuming data.
 *
 * Since jpeg_read_header has finished, we know what is in the SOF
 * and (first) SOS markers.  We also have all the application parameter
 * settings.
 */

short master_selection (j_decompress_ptr cinfo)
{ my_master_ptr        master= (my_master_ptr) cinfo->master;
  boolean            use_c_buffer;
  long                samplesperrow;
  JDIMENSION        jd_samplesperrow;

  /* Initialize dimensions and other stuff */
  jpeg_calc_output_dimensions(cinfo);
  prepare_range_limit_table(cinfo);

  /* Width of an output scanline must be representable as JDIMENSION. */
  samplesperrow= (long) cinfo->output_width * (long) cinfo->num_components;
  jd_samplesperrow= (JDIMENSION) samplesperrow;
  
  /* Initialize my private state */
  master->pass_number = 0;

  jinit_upsampler(cinfo);
//  jinit_d_post_controller(cinfo, 0);
  
  /* Inverse DCT */
  jinit_inverse_dct(cinfo);

  if(cinfo->progressive_mode)
    jinit_phuff_decoder(cinfo);
  else
    jinit_huff_decoder(cinfo);

  /* Initialize principal buffer controllers. */
  use_c_buffer= cinfo->inputctl->has_multiple_scans || cinfo->buffered_image;
  jinit_d_coef_controller(cinfo, use_c_buffer);

  jinit_d_main_controller(cinfo, FALSE /* never need full buffer here */);

  /* We can now tell the memory manager to allocate virtual arrays. */
  (*cinfo->mem->realize_virt_arrays) (cinfo);

  /* Initialize input side of decompressor to consume first scan. */
  return (*cinfo->inputctl->start_input_pass) (cinfo);
}


/*
 * Per-pass setup.
 * This is called at the beginning of each output pass.  We determine which
 * modules will be active during this pass and give them appropriate
 * start_pass calls. 
 */

void prepare_for_output_pass(j_decompress_ptr cinfo)
{ 
  (*cinfo->idct->start_pass) (cinfo);
  (*cinfo->coef->start_output_pass) (cinfo);
  (*cinfo->upsample->start_pass) (cinfo);
//  (*cinfo->post->start_pass) (cinfo, JBUF_PASS_THRU);
  (*cinfo->main->start_pass) (cinfo, JBUF_PASS_THRU);
}


/*
 * Finish up at end of an output pass.
 */

void finish_output_pass (j_decompress_ptr cinfo)
{
  my_master_ptr        master= (my_master_ptr) cinfo->master;

  master->pass_number++;
}


/*
 * Initialize master decompression control and select active modules.                
 * This is performed at the start of jpeg_start_decompress.
 */

short jinit_master_decompress(j_decompress_ptr cinfo)
{
  my_master_ptr        master;

  master= (my_master_ptr)(*cinfo->mem->alloc_small) (cinfo, JPOOL_IMAGE, SIZEOF(my_decomp_master));
  cinfo->master= (struct jpeg_decomp_master *) master;
  master->pub.prepare_for_output_pass= prepare_for_output_pass;
  master->pub.finish_output_pass= finish_output_pass;

  return master_selection(cinfo);
}
