/*!
*****************************************************************************
**
** \file        ./subsystems/paint/osdhandler/src/osdhandler_priv.h
**
** \version     $Id: osdhandler_priv.h 3750 2009-06-04 15:21:43Z rklose $
**
** \brief       Privately used header file.
**
** This file contains the declaration of private data for the OSDHANDLER.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2007 - 2009 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008 - 2009 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/

#ifndef OSDHANDLER_PRIV_H
    #define OSDHANDLER_PRIV_H

    #include <stdio.h>
    #include <stdint.h>
    #include <fapex/abst_types.h>
    #include <fapex/abst_libviout.h>
    #include "osdhandler.h"

    //***********************************************************************
    //***********************************************************************
    //** Private Defines and Macros
    //***********************************************************************
    //***********************************************************************

    /*! when converting 16bit (5551) colour to 32 bit true colour,
    **  the 1 bit alpha value 0.
    */
    #define OSDHANDLER_5551_Alpha0            0x80
    #define OSDHANDLER_5551_Alpha1            0xFF
    /*! when converting 8 bit alpha (32bit mode) to 1 bit
    **  alpha (5551 16bit mode), the threshold value
    */
    #define OSDHANDLER_Alpha_Threshold        0x80

    /*! Alignemnt of base pointer for OSD frame buffer(hardware requirement)
    ** in terms of byte addressing. */
    #define OSDHANDLER_OSD_BASE_ADDR_ALIGN 8
    
    /*! The line length of an OSD frame buffer must be a multiple of this 
    ** value in bytes. */
    #define OSDHANDLER_OSD_BURST_LENGTH 8
  
    /*! Alignment of base pointer for cursor Plane frame buffer
    ** in terms of byte addressing. */
    #define OSDHANDLER_CP_BASE_ADDR_ALIGN   4
   
    /*! The line length of cursor Plane frame buffer must be a multiple of this
    ** value in bytes. */
    #define OSDHANDLER_CP_BURST_LENGTH  4

    /*! Maximum size of cursor plane memory in bytes */
    #define OSDHANDLER_CP_MAX_SIZE       4096

    //***********************************************************************
    //***********************************************************************
    //** Private Data types
    //***********************************************************************
    //***********************************************************************

    /*!
    *************************************************************************
    ** \brief Keeps track of variables that are needed for the usage of the
    **        underlying Viscale driver (that provides the OSDs and the
    **        Cursor Plane). One instance of this structure is related to one
    **        specific OSD/CP.
    **
    ** handle:                  OSD/CP handle returned by the viscale driver when
    **                          the layer was opened
    ** allocatedMemPtr:         points to a field that has been allocated for the
    **                          usage of the OSD/CP
    ** isMemoryDynamic:         determines whether the allocatedMemPtr points to a
    **                          memory address that has been returned by
    **                          #FAPI_SYS_MALLOC (any value but 0) or if the
    **                          memory adress is static (0).
    ** fillColour:              colour code that will be used when the whole plane
    **                          is going to be filled up with a colour - this may
    **                          also be a 'transparent colour' in order to clear
    **                          the whole OSD layer and see the underlying layers
    ** scalerParams:            structure defined by the viscale driver that
    **                          keeps important values for an OSD when starting /
    **                          repositioning / rescaling it.
    *************************************************************************
    */
    typedef struct
    {
        int32_t                     fd;
        uint8_t*                    allocatedMemPtr;
        uint32_t                    isMemoryDynamic;
        uint32_t                    fillColour;
        FAPEX_VIOUT_ScalerParamStrT scalerParams;
        OSDHANDLER_ColourModeT      colourMode;
        OSDHANDLER_OsdLayoutEnumT   osdLayout;
        uint32_t                    clut[256];
        uint32_t                    layerEnable;
        uint32_t                    picSize;
        uint32_t                    picBase;
        uint32_t                    useGlobalAlpha;
        uint32_t                    globalAlpha;
    } OSDHANDLER_OsdCpParamsT;

    #if OSDHANDLER_USE_VIRTUAL_LAYERS
    /*!
    *************************************************************************
    ** \brief   Colour parameters used by the virtual layer system.
    *************************************************************************
    */
    typedef struct
    {
        uint32_t                    colour_mask;
        uint32_t*                   osd_base_ptr;
        uint32_t                    pixel_bit_offset;
        uint32_t                    data_mask;
        uint32_t                    temp_data;
        uint32_t                    data;
        uint32_t                    bpp;
        OSDHANDLER_ARGB__T          rgb;
    } OSDHANDLER_ColourParamsT;
    #endif // OSDHANDLER_USE_VIRTUAL_LAYERS

    //***********************************************************************
    //***********************************************************************
    //** Private Data
    //***********************************************************************
    //***********************************************************************

        #ifdef OSDHANDLER_C

        uint32_t osdhandlerInitialized = 0;
        uint32_t osdhandlerMaxHRES = OSDHANDLER_MAX_HRES;
        uint32_t osdhandlerMaxVRES = OSDHANDLER_MAX_VRES;
        OSDHANDLER_OsdCpParamsT osdhandlerOsd1Params;
        OSDHANDLER_OsdCpParamsT osdhandlerOsd2Params;
        OSDHANDLER_OsdCpParamsT osdhandlerOsd3Params;
        OSDHANDLER_OsdCpParamsT osdhandlerOsd4Params;
        OSDHANDLER_OsdCpParamsT osdhandlerCpParams;

        #if OSDHANDLER_USE_VIRTUAL_LAYERS
        // virtual layer system related data:
        uint32_t osdhandlerGroup1Started = 0;
        uint32_t osdhandlerGroup2Started = 0;
        uint32_t osdhandlerGroup3Started = 0;
        uint32_t osdhandlerGroup4Started = 0;
        uint32_t osdhandlerGroup1Shown   = 0;
        uint32_t osdhandlerGroup2Shown   = 0;
        uint32_t osdhandlerGroup3Shown   = 0;
        uint32_t osdhandlerGroup4Shown   = 0;
        uint32_t osdhandlerGroup1Updated = 1;
        uint32_t osdhandlerGroup2Updated = 1;
        uint32_t osdhandlerGroup3Updated = 1;
        uint32_t osdhandlerGroup4Updated = 1;

        /* Similar to a semaphore that locks the virtual layer system
        ** when the osdhandler is combining the virtual OSDs.
        */
        uint32_t osdhandlerFifoBusy      = 0;
        #endif // OSDHANDLER_USE_VIRTUAL_LAYERS

        #else // OSDHANDLER_C

        extern uint32_t osdhandlerInitialized;
        extern uint32_t osdhandlerMaxHRES;
        extern uint32_t osdhandlerMaxVRES;
        extern OSDHANDLER_OsdCpParamsT osdhandlerOsd1Params;
        extern OSDHANDLER_OsdCpParamsT osdhandlerOsd2Params;
        extern OSDHANDLER_OsdCpParamsT osdhandlerOsd3Params;
        extern OSDHANDLER_OsdCpParamsT osdhandlerOsd4Params;
        extern OSDHANDLER_OsdCpParamsT osdhandlerCpParams;

        #if OSDHANDLER_USE_VIRTUAL_LAYERS
        // virtual layer system related data:
        extern uint32_t osdhandlerGroup1Started;
        extern uint32_t osdhandlerGroup2Started;
        extern uint32_t osdhandlerGroup3Started;
        extern uint32_t osdhandlerGroup4Started;
        extern uint32_t osdhandlerGroup1Shown;
        extern uint32_t osdhandlerGroup2Shown;
        extern uint32_t osdhandlerGroup3Shown;
        extern uint32_t osdhandlerGroup4Shown;
        extern uint32_t osdhandlerGroup1Updated;
        extern uint32_t osdhandlerGroup2Updated;
        extern uint32_t osdhandlerGroup3Updated;
        extern uint32_t osdhandlerGroup4Updated;
        extern uint32_t osdhandlerFifoBusy;
        #endif // OSDHANDLER_USE_VIRTUAL_LAYERS

        #endif // OSDHANDLER_C

    //***********************************************************************
    //***********************************************************************
    //** Private Function Declarations
    //***********************************************************************
    //***********************************************************************

    #ifdef __cplusplus
    extern "C" {
    #endif

    int32_t osdhandlerGetOsdColourMode
                           (const OSDHANDLER_ColourModeT osdhandlerColourMode);
    int32_t osdhandlerGetCpColourMode
                           (const OSDHANDLER_ColourModeT osdhandlerColourMode);
    int32_t osdhandlerGetOsdLayout
                           (const OSDHANDLER_OsdLayoutEnumT osdhandlerLayout);
    OSDHANDLER_OsdCpParamsT* osdhandlerGetOsdCpParams (const uint32_t osdIndex);
    uint32_t osdhandlerGetBitsPerPixel
                           (const OSDHANDLER_OsdCpParamsT* osdCpParamsPtr);
    int32_t osdhandlerSetUpOsdPlane(OSDHANDLER_OsdCpParamsT* osdParamsPtr,
                                    uint8_t* memoryBasePtr);
    void osdhandlerFill (const OSDHANDLER_OsdCpParamsT* osdParamsPtr,
                         const uint32_t bpp, uint32_t colour);
    void osdhandlerGetPixelAddress(const uint32_t xx,
                                   const uint32_t yy,
                                   const uint32_t bpp,
                                   OSDHANDLER_OsdCpParamsT* osd_params_ptr,
                                   uint32_t* pixelBaseAddrPtr,
                                   uint32_t* pixelBitOffsetPtr);

#if OSDHANDLER_USE_VIRTUAL_LAYERS
    // virtual layer system related private functions:
    OSDHANDLER_OsdCpParamsT* osdvirtualGetOsdCpParams(const uint32_t layerId);
    int32_t osdvirtualStartLayer(const uint32_t layerId,
                                 OSDHANDLER_OsdCpParamsT* osdCpParamsPtr,
                                 const OSDHANDLER_StartParamsT* startParamsPtr);
    int32_t osdvirtualShowLayer(const uint32_t layerId,
                                OSDHANDLER_OsdCpParamsT* osdCpParamsPtr,
                                uint32_t* errorCodePtr);
    int32_t osdvirtualGlobalAlpha(OSDHANDLER_OsdCpParamsT* osdParamsPtr,
                                  const OSDHANDLER_ModeParamT mode,
                                  const uint32_t level);
#endif // OSDHANDLER_USE_VIRTUAL_LAYERS

    #ifdef __cplusplus
    }
    #endif

#endif // OSDHANDLER_PRIV_H
