/*
*****************************************************************************
**
** \file        ./subsystems/paint/osdhandler/src/osdhandler_virtual.c
** 
** \version     $Id: osdhandler_virtual.c 1911 2009-02-02 10:17:47Z wlaris $
**
** \brief       OSD handler for virtual layers.
**
** This file contains the implementation of the OSDHANDLER API for virtual
** layer related procedures.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2007-2009 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2009 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/

#include <stdint.h>
#include <fapex/abst_types.h>
#include "osdhandler.h"
#include "osdhandler_priv.h"

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************

#define OSDHANDLER_MAX_FIFO_ELEMENT        255

//***************************************************************************
//***************************************************************************
//** Local Data
//***************************************************************************
//***************************************************************************

static OSDHANDLER_OsdCpParamsT osdhandlerGroup1Params[OSDHANDLER_MAX_NUM_VIRTUAL_LAYER];
static OSDHANDLER_OsdCpParamsT osdhandlerGroup2Params[OSDHANDLER_MAX_NUM_VIRTUAL_LAYER];
static OSDHANDLER_PositionT    osdhandlerGroup1Fifo[OSDHANDLER_MAX_FIFO_ELEMENT];
static OSDHANDLER_PositionT    osdhandlerGroup2Fifo[OSDHANDLER_MAX_FIFO_ELEMENT];

//***************************************************************************
//***************************************************************************
//** Local Functions Declaration
//***************************************************************************
//***************************************************************************

static OSDHANDLER_OsdCpParamsT* osdhandlerGetOsdGroupParams(const uint32_t groupId);
static OSDHANDLER_PositionT* osdhandlerGetGroupFifo(const uint32_t groupId);
static uint8_t osdhandlerSearchBestColour(const uint32_t col,
                                          const uint32_t* clut_ptr,
                                          const uint8_t clutSize );
static void osdhandlerGetRGBAColour(OSDHANDLER_OsdCpParamsT* osd_params_ptr,
                                    const uint32_t  data,
                                    const uint32_t  bpp,
                                    OSDHANDLER_ARGB__T* rgba_ptr );
static void osdhandlerGetARGBColour(OSDHANDLER_OsdCpParamsT* osd_params_ptr,
                                    const uint32_t  data,
                                    const uint32_t  bpp,
                                    OSDHANDLER_ARGB__T* argb_ptr );
static void osdhandlerCombine2Colour(OSDHANDLER_ARGB__T* rgb1_ptr,
                                     OSDHANDLER_ARGB__T* rgb2_ptr,
                                     OSDHANDLER_ARGB__T* rgb_ptr);
static void osdhandlerCombine(const uint32_t groudId,
                              OSDHANDLER_PositionT* pos_ptr);

//***************************************************************************
//***************************************************************************
//** Private Functions
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief   Returns that OSDHANDLER_OsdCpParamsT structure that is
**          related to the virtual layer identified by the layerId.
** \param   layerId
                identifies an virtual OSD layer
** \return  Pointer to an OSDHANDLER_OsdCpParamsT structure if possible,
**          otherwise NULL.
*****************************************************************************
*/
OSDHANDLER_OsdCpParamsT* osdvirtualGetOsdCpParams(const uint32_t layerId)
{
    if (((layerId & 0x0000FFFFUL) > OSDHANDLER_MAX_NUM_VIRTUAL_LAYER)
      ||((layerId & 0x0000FFFFUL) == 0))
    {
        return(NULL);
    }
    else if((layerId >> 16) == 1)
    {
        return &osdhandlerGroup1Params[(layerId & 0x0000FFFFUL) - 1];
    }
    else if((layerId >> 16) == 2)
    {
        return &osdhandlerGroup2Params[(layerId & 0x0000FFFFUL) - 1];
    }
    else
    {
        return NULL;
    }
}

/*!
*****************************************************************************
** \brief   Starts a virtual layer internally at given coordinates and
**          with given dimensions. The coordinates and the dimensions
**          have to be equal to those of the corresponding hardware
**          layer.
**
** \param   layerId
**              Layer identifier that is used to extract the group.
** \param   osdCpParamsPtr
**              The OSD structure of the virtual layer that will be started.
** \param   startParamsPtr
**              Points to a structure that specifies scaling,
**              dimensions and the position of the layer.
**
** \return
**          -   #FAPI_OK on success.
**          -   #OSDHANDLER_ERR_BAD_LAYER_ID
**              if the layerId is bad.
**          -   #OSDHANDLER_ERR_NO_OSD_HANDLE if the corresponding
**              hardware layer is not allocated by the OSDHANDLER.
**          -   #OSDHANDLER_ERR_LAYER_NOT_STARTED
**              if the corresponding HW layer has not been started yet.
**          -   #OSDHANDLER_ERR_OUT_OF_SIZE
**              if the dimensions of the virtual layer do not fit those of
**              the hardware layer.
**          -   #OSDHANDLER_ERR_OUT_OF_MEMORY
**              if there was not enough memory available for allocation in order
**              to drive the OSD at the given resolution and in the current
**              colour mode.
**
** \sa
**          - OSDHANDLER_StartLayer()
**
*****************************************************************************
*/
int32_t osdvirtualStartLayer(const uint32_t layerId,
                             OSDHANDLER_OsdCpParamsT* osdCpParamsPtr,
                             const OSDHANDLER_StartParamsT* startParamsPtr)
{
    OSDHANDLER_OsdCpParamsT*    hw_osd_params_ptr   = NULL;  //underlining hw layer
    uint32_t                    group               = 0;
    int32_t                     result              = 0;

    if((osdCpParamsPtr == NULL)
    || (startParamsPtr == NULL))
    {
        return(OSDHANDLER_ERR_BAD_PARAMETER);
    }
    group = layerId >> 16;
    hw_osd_params_ptr = osdhandlerGetOsdCpParams(group);
    if (hw_osd_params_ptr == NULL)
    {
        // Must be prevented by the calling API function.
        return(OSDHANDLER_ERR_BAD_LAYER_ID);
    }
    // the underlining hw should already be allocated and started:
    if(hw_osd_params_ptr->fd == -1)
    {
        return(OSDHANDLER_ERR_NO_OSD_HANDLE);
    }
    if(hw_osd_params_ptr->allocatedMemPtr == NULL)
    {
        return(OSDHANDLER_ERR_LAYER_NOT_STARTED);
    }
    //virtual layers should have the same size as its underlining hw layer
    if(!(startParamsPtr->xPos   == hw_osd_params_ptr->scalerParams.outputWindowX0)||
       !(startParamsPtr->yPos   == hw_osd_params_ptr->scalerParams.outputWindowY0)||
       !(startParamsPtr->inputWidth  == hw_osd_params_ptr->scalerParams.inputWindowWidth)||
       !(startParamsPtr->inputHeight == hw_osd_params_ptr->scalerParams.inputWindowHeight) ||
       !(startParamsPtr->outputWidth  == hw_osd_params_ptr->scalerParams.outputWindowWidth) ||
       !(startParamsPtr->outputHeight == hw_osd_params_ptr->scalerParams.outputWindowHeight) ||
       !(startParamsPtr->frameBased == hw_osd_params_ptr->scalerParams.flagFrameField)
       )
    {
        return(OSDHANDLER_ERR_OUT_OF_SIZE);
    }

    // ************** start the OSD plane  **********************
    osdCpParamsPtr->scalerParams.outputWindowX0     = startParamsPtr->xPos;
    osdCpParamsPtr->scalerParams.outputWindowY0     = startParamsPtr->yPos;
    osdCpParamsPtr->scalerParams.inputWindowWidth   = startParamsPtr->inputWidth;
    osdCpParamsPtr->scalerParams.inputWindowHeight  = startParamsPtr->inputHeight;
    osdCpParamsPtr->scalerParams.outputWindowWidth  = startParamsPtr->outputWidth;
    osdCpParamsPtr->scalerParams.outputWindowHeight = startParamsPtr->outputHeight;
    osdCpParamsPtr->scalerParams.flagFrameField     = startParamsPtr->frameBased;

    result = osdhandlerSetUpOsdPlane(osdCpParamsPtr,
                                     startParamsPtr->memoryBasePtr);
    if(result != FAPI_OK)
    {
        return(result);
        // OSDHANDLER_ERR_OUT_OF_MEMORY
        // other error codes already checked
    }
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Set a virtual OSD layer visible. The content of the virtual
**          layer will be committed to that of the corresponding
**          hardware OSD layer.
**
** \param   layerId
**              Layer identifier that is used to extract the group of the
**              virtual OSD layer.
** \param   osdCpParamsPtr
**              The OSD structure of the virtual layer that will be set
**              visible.
** \param   errorCodePtr
**              Will receive error codes related to specific hardware layers.
**              If there occured an error during the underlying driver calls,
**              one or more of the following error messages may be contained:
**              - #OSDHANDLER_ERR_CODE_OSD1_ENABLE
**              - #OSDHANDLER_ERR_CODE_OSD2_ENABLE
**
** \return
**          -   #FAPI_OK on success.
**          -   #OSDHANDLER_ERR_BAD_LAYER_ID
**              if the corresponding hardware OSD layer ID is invalid.
**          -   #OSDHANDLER_ERR_BAD_PARAMETER
**              if the passed osdCpParamsPtr is NULL.
**          -   #OSDHANDLER_ERR_NO_OSD_HANDLE
**              if the corresponding HW layer is not allocated by the OSDHANDLER.
**          -   #OSDHANDLER_ERR_LAYER_NOT_STARTED
**              if the corresponding HW layer has not been started yet.
**          -   #OSDHANDLER_ERR_BAD_COLOUR_MODE
**              if either the virtual layer or the corresponding hardware OSD
**              layer is driven in a bad colour mode.
**
** \sa
**          - OSDHANDLER_ShowLayer()
**
*****************************************************************************
*/
int32_t osdvirtualShowLayer(const uint32_t layerId,
                            OSDHANDLER_OsdCpParamsT* osdCpParamsPtr,
                            uint32_t* errorCodePtr)
{
    OSDHANDLER_OsdCpParamsT*    hw_osd_params_ptr   = NULL;     //underlining hw layer
    OSDHANDLER_ColourParamsT    osd_colour_params;              //virtual osd layer color params
    OSDHANDLER_ColourParamsT    layer_colour_params;            //hw layer colour params
    int32_t                     result              = 0;
    uint32_t                    group               = 0;
    uint16_t                    col16               = 0;        //temporary 16bit colour
    uint32_t                    col32               = 0;        //temporary 32bit colour
    uint32_t                    x_pos               = 0;
    uint32_t                    x_size              = 0;
    uint32_t                    y_size              = 0;
    uint32_t                    global_alpha        = 0;
    uint8_t                     red                 = 0;
    uint8_t                     green               = 0;
    uint8_t                     blue                = 0;
    uint8_t                     alpha               = 0;

#if OSDHANDLER_DEBUG
    uint8_t                     best_colour_dbg     = 0;
#endif

    if(osdCpParamsPtr == NULL)
    {
        // Must be prevented by the calling API function.
        return(OSDHANDLER_ERR_BAD_PARAMETER);
    }
    group = layerId >> 16;
    hw_osd_params_ptr = osdhandlerGetOsdCpParams(group);
    if(hw_osd_params_ptr == NULL)
    {
        return(OSDHANDLER_ERR_BAD_LAYER_ID);
    }
    if(hw_osd_params_ptr->fd == -1)
    {
        return(OSDHANDLER_ERR_NO_OSD_HANDLE);
    }
    if(hw_osd_params_ptr->allocatedMemPtr == NULL)
    {
        return(OSDHANDLER_ERR_LAYER_NOT_STARTED);
    }
    //show the underlining hardware layer if it is hidden
    if (!(hw_osd_params_ptr->layerEnable))
    {
        result = FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)hw_osd_params_ptr->fd,
                                    FAPEX_IOCCOM_VIOUT_ENABLE,
                                    (FAPEXIOCARG3_T)NULL );
        if((result != 0) && (errorCodePtr != NULL))
        {
            if(group == 1) *errorCodePtr |= OSDHANDLER_ERR_CODE_OSD1_ENABLE;
            if(group == 2) *errorCodePtr |= OSDHANDLER_ERR_CODE_OSD2_ENABLE;
        }
        hw_osd_params_ptr->layerEnable = 1;
    }

    memset(&osd_colour_params,   0, sizeof(OSDHANDLER_ColourParamsT));
    memset(&layer_colour_params, 0, sizeof(OSDHANDLER_ColourParamsT));

    osd_colour_params.bpp = osdhandlerGetBitsPerPixel(osdCpParamsPtr);
    if(osd_colour_params.bpp == 0)
    {
        return(OSDHANDLER_ERR_BAD_COLOUR_MODE);
    }
    layer_colour_params.bpp = osdhandlerGetBitsPerPixel(hw_osd_params_ptr);
    if(layer_colour_params.bpp == 0)
    {
        return(OSDHANDLER_ERR_BAD_COLOUR_MODE);
    }
    if(osd_colour_params.bpp >> 5) // bpp1 >= 32 ?
    {
        osd_colour_params.colour_mask = 0xFFFFFFFFUL;
    }
    else
    {
        osd_colour_params.colour_mask =
                    (0x00000001UL << osd_colour_params.bpp) - 0x00000001UL;
    }
    if(layer_colour_params.bpp >> 5) // bpp >= 32 ?
    {
        layer_colour_params.colour_mask = 0xFFFFFFFFUL;
    }
    else
    {
        layer_colour_params.colour_mask =
                    (0x00000001UL << layer_colour_params.bpp) - 0x00000001UL;
    }
    if (osdCpParamsPtr->useGlobalAlpha)
    {
        global_alpha = osdCpParamsPtr->globalAlpha;
    }

    x_size = osdCpParamsPtr->scalerParams.inputWindowWidth;
    y_size = osdCpParamsPtr->scalerParams.inputWindowHeight;
    /*lint -save -e923 */
    osd_colour_params.osd_base_ptr = (uint32_t*)(osdCpParamsPtr->picBase);
    layer_colour_params.osd_base_ptr = (uint32_t*)(hw_osd_params_ptr->picBase);
    /*lint -restore */
    osd_colour_params.data = *osd_colour_params.osd_base_ptr++;

    while (y_size)
    {
        // 32-bit virtual OSD
        if (osd_colour_params.bpp >> 5)
        {
            // 32-bit virtual OSD and 32-bit HW OSD
            if(layer_colour_params.bpp >> 5)
            {
                if (osdCpParamsPtr->useGlobalAlpha)
                {
                    if (osdCpParamsPtr->osdLayout == OSDHANDLER_OSD_LAYOUT_RGBA)
                    {
                        osd_colour_params.data =
                                (osd_colour_params.data&0xFFFFFF00UL)|global_alpha;
                    }
                    else if (osdCpParamsPtr->osdLayout == OSDHANDLER_OSD_LAYOUT_ARGB)
                    {
                        osd_colour_params.data =
                                (global_alpha<<24)|(osd_colour_params.data&0x00FFFFFFUL);
                    }
                }
                *layer_colour_params.osd_base_ptr++ = osd_colour_params.data;
            }
            // 32-bit virtual OSD and 16-bit HW OSD
            else if (layer_colour_params.bpp >> 4)
            {
                if (osdCpParamsPtr->osdLayout == OSDHANDLER_OSD_LAYOUT_RGBA)
                {
                    red = osd_colour_params.data>>27;
                    green = (osd_colour_params.data&0x00FF0000)>>19;
                    blue = (osd_colour_params.data&0x0000FF00)>>11;
                    if (osdCpParamsPtr->useGlobalAlpha)
                    {
                        alpha = (global_alpha > 255) ? 255 : (uint8_t)global_alpha;
                    }
                    else
                    {
                        alpha = (osd_colour_params.data & 0x000000FF);
                    }
                    if (alpha >= OSDHANDLER_Alpha_Threshold)    alpha = 0x1;
                    else                                        alpha = 0x0;
                    layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                    layer_colour_params.data =
                       ((layer_colour_params.data) & (~layer_colour_params.data_mask)) |
                       (((uint32_t)((red<<11)|(green<<6)|(blue<<1)|alpha)) <<
                           (0x20UL - (layer_colour_params.pixel_bit_offset +
                           layer_colour_params.bpp)));
                }
                else if (osdCpParamsPtr->osdLayout == OSDHANDLER_OSD_LAYOUT_ARGB)
                {
                    red = (osd_colour_params.data & 0x00FF0000UL)>>19;
                    green = (osd_colour_params.data & 0x0000FF00UL)>>11;
                    blue = (osd_colour_params.data & 0x000000FFUL)>>3;
                    if (osdCpParamsPtr->useGlobalAlpha)
                    {
                        alpha = (global_alpha > 255) ? 255 : (uint8_t)global_alpha;
                    }
                    else
                    {
                        alpha = (osd_colour_params.data & 0xFF000000UL)>>24;
                    }
                    if (alpha >= OSDHANDLER_Alpha_Threshold)    alpha = 0x1;
                    else                                        alpha = 0x0;
                    layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                    layer_colour_params.data =
                       ((layer_colour_params.data) & (~layer_colour_params.data_mask)) |
                       (((uint32_t)((alpha<<15)|(red<<10)|(green<<5)|blue)) <<
                           (0x20UL - (layer_colour_params.pixel_bit_offset +
                           layer_colour_params.bpp)));
                }
                layer_colour_params.pixel_bit_offset += layer_colour_params.bpp;
                if (layer_colour_params.pixel_bit_offset >> 5)
                {
                    *layer_colour_params.osd_base_ptr++ = layer_colour_params.data;
                    layer_colour_params.pixel_bit_offset = 0;
                }
            }
            // 32-bit virtual OSD and (8-bit or 4-bit) HW OSD
            else
            {
                if (osdCpParamsPtr->useGlobalAlpha)
                {
                    if (osdCpParamsPtr->osdLayout == OSDHANDLER_OSD_LAYOUT_RGBA)
                    {
                        osd_colour_params.data =
                                (osd_colour_params.data & 0xFFFFFF00UL)|global_alpha;
                    }
                    else if (osdCpParamsPtr->osdLayout == OSDHANDLER_OSD_LAYOUT_ARGB)
                    {
                        osd_colour_params.data =
                                (global_alpha<<24)|(osd_colour_params.data & 0x00FFFFFFUL);
                    }
                }
                layer_colour_params.temp_data =
                    osdhandlerSearchBestColour
                            (osd_colour_params.data,
                             hw_osd_params_ptr->clut,
                             (uint8_t)((1UL << layer_colour_params.bpp)-1UL) ); // is <256

#if OSDHANDLER_DEBUG
                if(!best_colour_dbg)
                {
                    FAPI_SYS_PRINT_MSG("[osdhandler/debug] best colour: %d\n",
                                       layer_colour_params.temp_data);
                    best_colour_dbg = 1;
                }
#endif
                layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                layer_colour_params.data = ((layer_colour_params.data) &
                        (~layer_colour_params.data_mask)) |(layer_colour_params.temp_data <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset +
                        layer_colour_params.bpp)));
                layer_colour_params.pixel_bit_offset += layer_colour_params.bpp;
                if (layer_colour_params.pixel_bit_offset >> 5)
                {
                    *layer_colour_params.osd_base_ptr++ = layer_colour_params.data;
                    layer_colour_params.pixel_bit_offset = 0;
                }
            }
            osd_colour_params.data = *osd_colour_params.osd_base_ptr++;
        }
        // 16-bit virtual OSD
        else if (osd_colour_params.bpp >> 4)
        {
            osd_colour_params.data_mask = osd_colour_params.colour_mask <<
                    (0x20UL - (osd_colour_params.pixel_bit_offset + osd_colour_params.bpp));
            // 16-bit virtual OSD and 32-bit HW OSD
            if(layer_colour_params.bpp >> 5)
            {
                col16 = (osd_colour_params.data & osd_colour_params.data_mask) >>
                        (0x20UL - (osd_colour_params.pixel_bit_offset + osd_colour_params.bpp));
                if (osdCpParamsPtr->osdLayout == OSDHANDLER_OSD_LAYOUT_RGBA)
                {
                    red   = (col16 & 0xF800) >> 8;
                    green = (col16 & 0x07C0) >> 3;
                    blue  = (col16 & 0x003E) << 2;
                    alpha =  col16 & 0x0001;
                    if (osdCpParamsPtr->useGlobalAlpha)
                    {
                        alpha = (global_alpha > 255) ? 255 : (uint8_t)global_alpha;
                    }
                    else
                    {
                        if (alpha == 0) alpha = OSDHANDLER_5551_Alpha0;
                        else            alpha = OSDHANDLER_5551_Alpha1;
                    }
                    *layer_colour_params.osd_base_ptr++ = \
                        ((red<<24)|(green<<16)|(blue<<8)|alpha);
                }
                else if (osdCpParamsPtr->osdLayout == OSDHANDLER_OSD_LAYOUT_ARGB)
                {
                    red   = (col16 & 0x7C00) >>  7;
                    green = (col16 & 0x03E0) >>  2;
                    blue  = (col16 & 0x001F) <<  3;
                    alpha = (col16 & 0x8000) >> 15;
                    if (osdCpParamsPtr->useGlobalAlpha)
                    {
                        alpha = (global_alpha > 255) ? 255 : (uint8_t)global_alpha;
                    }
                    else
                    {
                        if (alpha == 0) alpha = OSDHANDLER_5551_Alpha0;
                        else            alpha = OSDHANDLER_5551_Alpha1;
                    }
                    *layer_colour_params.osd_base_ptr++ = \
                        ((alpha<<24)|(red<<16)|(green<<8)|blue);
                }
            }
            // 16-bit virtual OSD and 16-bit HW OSD
            // NOTE:    The global alpha setting of the virtual OSD will be
            //          overridden by the setting of the corresponding hardware OSD.
            else if(layer_colour_params.bpp >> 4)
            {
                if (osd_colour_params.pixel_bit_offset >> 4)
                {
                    *layer_colour_params.osd_base_ptr++ = osd_colour_params.data;
                }
            }
            // 16-bit virtual OSD and (8-bit or 4-bit) HW OSD
            else
            {
                col16 = (osd_colour_params.data & osd_colour_params.data_mask) >>
                        (0x20UL - (osd_colour_params.pixel_bit_offset + osd_colour_params.bpp));
                if (osdCpParamsPtr->osdLayout == OSDHANDLER_OSD_LAYOUT_RGBA)
                {
                    red   = (col16 & 0xF800) >> 8;
                    green = (col16 & 0x07C0) >> 3;
                    blue  = (col16 & 0x003E) << 2;
                    alpha = (col16 & 0x0001);
                    if (osdCpParamsPtr->useGlobalAlpha)
                    {
                        alpha = (global_alpha > 255) ? 255 : (uint8_t)global_alpha;
                    }
                    else
                    {
                        if (alpha == 0) alpha = OSDHANDLER_5551_Alpha0;
                        else            alpha = OSDHANDLER_5551_Alpha1;
                    }
                    layer_colour_params.temp_data =
                        osdhandlerSearchBestColour(
                            ((red<<24)|(green<<16)|(blue<<8)|alpha),
                            hw_osd_params_ptr->clut,
                            (uint8_t)((1UL << layer_colour_params.bpp)-1UL) ); // is <256
                }
                else if (osdCpParamsPtr->osdLayout == OSDHANDLER_OSD_LAYOUT_ARGB)
                {
                    red   = (col16 & 0x7C00) >>  7;
                    green = (col16 & 0x03E0) >>  2;
                    blue  = (col16 & 0x001F) <<  3;
                    alpha = (col16 & 0x8000) >> 15;
                    if (osdCpParamsPtr->useGlobalAlpha)
                    {
                        alpha = (global_alpha > 255) ? 255 : (uint8_t)global_alpha;
                    }
                    else
                    {
                        if (alpha == 0) alpha = OSDHANDLER_5551_Alpha0;
                        else            alpha = OSDHANDLER_5551_Alpha1;
                    }
                    layer_colour_params.temp_data =
                        osdhandlerSearchBestColour(
                            ((alpha<<24)|(red<<16)|(green<<8)|blue),
                            hw_osd_params_ptr->clut,
                            (uint8_t)((1UL << layer_colour_params.bpp)-1UL) ); // is <256
                }
                layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                layer_colour_params.data = ((layer_colour_params.data) &
                        (~layer_colour_params.data_mask)) |(layer_colour_params.temp_data <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset +
                        layer_colour_params.bpp)));
                layer_colour_params.pixel_bit_offset += layer_colour_params.bpp;
                if (layer_colour_params.pixel_bit_offset >> 5)
                {
                    *layer_colour_params.osd_base_ptr++ = layer_colour_params.data;
                    layer_colour_params.pixel_bit_offset = 0;
                }
            }
            osd_colour_params.pixel_bit_offset += osd_colour_params.bpp;
            if (osd_colour_params.pixel_bit_offset >> 5)
            {
                osd_colour_params.pixel_bit_offset = 0;
                osd_colour_params.data = *osd_colour_params.osd_base_ptr++;
            }
        }
        // 8-bit or 4-bit virtual OSD
        else
        {
            osd_colour_params.data_mask = osd_colour_params.colour_mask <<
                    (0x20UL - (osd_colour_params.pixel_bit_offset + osd_colour_params.bpp));
            osd_colour_params.temp_data = (osd_colour_params.data & osd_colour_params.data_mask) >>
                    (0x20UL - (osd_colour_params.pixel_bit_offset + osd_colour_params.bpp));
            // 8/4-bit virtual OSD and 32-bit HW OSD
            if(layer_colour_params.bpp >> 5)
            {
                col32 = osdCpParamsPtr->clut[osd_colour_params.temp_data];
                if (osdCpParamsPtr->useGlobalAlpha)
                {
                    if (osdCpParamsPtr->osdLayout == OSDHANDLER_OSD_LAYOUT_RGBA)
                    {
                        col32 = (col32&0xFFFFFF00UL)|global_alpha;
                    }
                    else if (osdCpParamsPtr->osdLayout == OSDHANDLER_OSD_LAYOUT_ARGB)
                    {
                        col32 = (global_alpha<<24)|(col32&0x00FFFFFFUL);
                    }
                }
                *layer_colour_params.osd_base_ptr++ = col32;
            }
            // 8/4-bit virtual OSD and 16-bit HW OSD
            else if(layer_colour_params.bpp >> 4)
            {
                col32 = osdCpParamsPtr->clut[osd_colour_params.temp_data];

                if (osdCpParamsPtr->osdLayout == OSDHANDLER_OSD_LAYOUT_RGBA)
                {
                    red = col32>>27;
                    green = (col32&0x00FF0000)>>19;
                    blue = (col32&0x0000FF00)>>11;
                    if (osdCpParamsPtr->useGlobalAlpha)
                    {
                        alpha = (global_alpha > 255) ? 255 : (uint8_t)global_alpha;
                    }
                    else
                    {
                        alpha = (col32&0x000000FF);
                    }
                    if (alpha >= OSDHANDLER_Alpha_Threshold) alpha = 0x1;
                    else                                     alpha = 0x0;
                    layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                    layer_colour_params.data =
                       ((layer_colour_params.data) & (~layer_colour_params.data_mask)) |
                       (((uint32_t)((red<<11)|(green<<6)|(blue<<1)|alpha)) <<
                          (0x20UL - (layer_colour_params.pixel_bit_offset +
                          layer_colour_params.bpp)));
                }
                else if (osdCpParamsPtr->osdLayout == OSDHANDLER_OSD_LAYOUT_ARGB)
                {
                    red   = (col32 & 0x00FF0000UL) >> 19;
                    green = (col32 & 0x0000FF00UL) >> 11;
                    blue  = (col32 & 0x000000FFUL) >>  3;
                    if (osdCpParamsPtr->useGlobalAlpha)
                    {
                        alpha = (global_alpha > 255) ? 255 : (uint8_t)global_alpha;
                    }
                    else
                    {
                        alpha = (col32 & 0xFF000000) >> 24;
                    }
                    if (alpha >= OSDHANDLER_Alpha_Threshold) alpha = 0x1;
                    else                                     alpha = 0x0;
                    layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                    layer_colour_params.data =
                       ((layer_colour_params.data) & (~layer_colour_params.data_mask)) |
                       (((uint32_t)((alpha<<15)|(red<<10)|(green<<5)|blue)) <<
                          (0x20UL - (layer_colour_params.pixel_bit_offset +
                          layer_colour_params.bpp)));
                }
                layer_colour_params.pixel_bit_offset += layer_colour_params.bpp;
                if (layer_colour_params.pixel_bit_offset >> 5)
                {
                    *layer_colour_params.osd_base_ptr++ = layer_colour_params.data;
                    layer_colour_params.pixel_bit_offset = 0;
                }
            }
            // 8/4-bit virtual OSD and 8/4-bit HW OSD
            else
            {
                // TODO: is that assumption correct? --RK
                //by default the virtual OSD layer and its underlining hardware layer use the same CLUT
                layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                layer_colour_params.data = ((layer_colour_params.data) &
                        (~layer_colour_params.data_mask)) |(osd_colour_params.temp_data <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset +
                        layer_colour_params.bpp)));
                layer_colour_params.pixel_bit_offset += layer_colour_params.bpp;
                if (layer_colour_params.pixel_bit_offset >> 5)
                {
                    *layer_colour_params.osd_base_ptr++ = layer_colour_params.data;
                    layer_colour_params.pixel_bit_offset = 0;
                }
            }
            osd_colour_params.pixel_bit_offset += osd_colour_params.bpp;
            // if (scr_pixel_offset > 31)
            if (osd_colour_params.pixel_bit_offset >> 5)
            {
                osd_colour_params.pixel_bit_offset = 0;
                osd_colour_params.data = *osd_colour_params.osd_base_ptr++;
            }
        }
        x_pos++;
        if (x_pos >= x_size) // check end of Pixel-Line
        {
            x_pos = 0;
            y_size--;
            if (!y_size)
            {
                break;
            }
        }
    }
    if(group == 1)
    {
        osdhandlerGroup1Shown     = 1;
    }
    else
    {
        osdhandlerGroup2Shown     = 1;
    }
    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Operates on the GlobalAlpha settings of an OSD/CP.
**
** \param   layerId
**              A VIRTUAL layer ID.
** \param   mode
**              Specifies how to operate on the global alpha settings of the
**              OSD/CP and if the level parameter is utilized, how to apply it.
**              Possible modes are:
**              - #OSDHANDLER_PARAMETER_GET returns the global alpha value
**                (even though global alpha may be disabled). No FAPI status
**                word will be returned here.
**              - #OSDHANDLER_PARAMETER_SET sets the global alpha value. If you
**                pass a value greater than 255, 255 is applied to the
**                alpha value.
**              - #OSDHANDLER_PARAMETER_INCREASE increases the existing global
**                alpha value by 'level' steps. If you excess 255, the setting
**                will not go beyond that value.
**              - #OSDHANDLER_PARAMETER_DECREASE deceases the existing global
**                alpha value by 'level' steps. If you fall below 0, the
**                setting will stay at 0.
**              - #OSDHANDLER_PARAMETER_ON switches the global alpha mode on.
**              - #OSDHANDLER_PARAMETER_OFF switches the global alpha mode off.
** \param   level
**              The level value will be applied to the global alpha value
**              according to the mode parameter.
**
** \return
**          -   #FAPI_OK on success.
**          -   #OSDHANDLER_ERR_BAD_PARAMETER if the mode qualifier is bad
**              or if the passed osdParamsPtr is NULL.
**
*****************************************************************************
*/
int32_t osdvirtualGlobalAlpha(OSDHANDLER_OsdCpParamsT* osdParamsPtr,
                              const OSDHANDLER_ModeParamT mode,
                              const uint32_t level)
{
    uint32_t                    newLevel            = 0;
    uint32_t                    result              = 0;

    if(osdParamsPtr == NULL)
    {
        // Must be prevented by the calling API function.
        return(OSDHANDLER_ERR_BAD_PARAMETER);
    }
    switch(mode)
    {
        case(OSDHANDLER_PARAMETER_GET):
            result = osdParamsPtr->globalAlpha;
            return((int32_t)result);
        case(OSDHANDLER_PARAMETER_SET):
            newLevel = (level >> 8) ? 255 : level;
            osdParamsPtr->globalAlpha = newLevel;
            return(FAPI_OK);
        case(OSDHANDLER_PARAMETER_INCREASE):
            if((osdParamsPtr->globalAlpha >> 8)
             ||(level >> 8)
             ||((osdParamsPtr->globalAlpha + level) >> 8))
            {
                newLevel = 255;
            }
            else
            {
                newLevel = osdParamsPtr->globalAlpha + level;
            }
            osdParamsPtr->globalAlpha = newLevel;
            return(FAPI_OK);
        case(OSDHANDLER_PARAMETER_DECREASE):
            if(level > osdParamsPtr->globalAlpha)
            {
                newLevel = 0;
            }
            else
            {
                newLevel = osdParamsPtr->globalAlpha - level;
            }
            osdParamsPtr->globalAlpha = newLevel;
            return(FAPI_OK);
        case(OSDHANDLER_PARAMETER_ON):
            osdParamsPtr->useGlobalAlpha = 1;
            return(FAPI_OK);
        case(OSDHANDLER_PARAMETER_OFF):
            osdParamsPtr->useGlobalAlpha = 0;
            return(FAPI_OK);
        default:
            return(OSDHANDLER_ERR_BAD_PARAMETER);
    }
}

//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief   Returns that OSDHANDLER_OsdCpParamsT structure that is
**          related to the virtual layer identified by the layerId.
**
** \param   groupId
**          For virtual osd layers:
**               - group 1: 0x10000
**               - group 2: 0x20000
**
** \return  Pointer to an OSDHANDLER_OsdCpParamsT structure if possible,
**          otherwise NULL.
*****************************************************************************
*/
static OSDHANDLER_OsdCpParamsT* osdhandlerGetOsdGroupParams(const uint32_t groupId)
{
    switch(groupId)
    {
        case(0x10000):
            return osdhandlerGroup1Params;
        case(0x20000):
            return osdhandlerGroup2Params;
        default:
            return NULL;
    }
}

/*!
*****************************************************************************
** \brief   Returns that internal OSDHANDLER_PositionT structure that is
**          related to the given qualifier.
**
** \param   groupId
**          For virtual osd layers:
**               - group 1: 0x10000
**               - group 2: 0x20000
** \return
**          -   Internal OSDHANDLER_PositionT structures of virtual layer
**              group on success.
**          -   NULL if the qualifier is invalid.
**
*****************************************************************************
*/
static OSDHANDLER_PositionT* osdhandlerGetGroupFifo(const uint32_t groupId)
{
    /*Only for Virtual OSD layers*/
    switch(groupId)
    {
        case(0x10000):
            return osdhandlerGroup1Fifo;
        case(0x20000):
            return osdhandlerGroup2Fifo;
        default:
            return NULL;
    }
}

/*!
*****************************************************************************
** \brief Looks up the best matching colour for a 32-bit colour in a CLUT.
**
** \param   col
**              32-bit color to be matched
** \param   clut_ptr
**              point to the clut array
** \param   clutSize
**              number of colours in the CLUT to take into account
**
** \return  Index of best matching colour in the CLUT.
*****************************************************************************
*/
static uint8_t osdhandlerSearchBestColour(const uint32_t col,
                                          const uint32_t* clut_ptr,
                                          const uint8_t clutSize )
{
    uint32_t            diffmin = 0;
    uint32_t            ii      = 0;
    uint32_t            red2    = 0;
    uint32_t            green2  = 0;
    uint32_t            blue2   = 0;
    uint32_t            alpha2  = 0;
    uint8_t             red     = 0;
    uint8_t             green   = 0;
    uint8_t             blue    = 0;
    uint8_t             alpha   = 0;
    uint8_t             index   = 0;

    diffmin  = 0xFFFFFFFF;
    if (clut_ptr == NULL)
    {
        return 0;
    }
    red    = (col >> 24) & 0xFF;
    green  = (col >> 16) & 0xFF;
    blue   = (col >>  8) & 0xFF;
    alpha  = (col)       & 0xFF;

    for (ii = 0; ii < clutSize; ii++)
    {

//        red2   = (red > ((clut_ptr[ii]>>24) & 0xFF))        ?
//                    (red - ((clut_ptr[ii]>>24) & 0xFF))     :
//                    (((clut_ptr[ii]>>24) & 0xFF) - red);
//        green2 = (green > ((clut_ptr[ii]>>16) & 0xFF))      ?
//                    (green - ((clut_ptr[ii]>>16)& 0xFF))    :
//                    (((clut_ptr[ii]>>16)& 0xFF) - green);
//        blue2  = (blue > ((clut_ptr[ii]>>8) & 0xFF))        ?
//                    (blue - ((clut_ptr[ii]>>8) & 0xFF))     :
//                    (((clut_ptr[ii]>>8) & 0xFF) - blue);
//        alpha2 = (alpha > ((clut_ptr[ii]) & 0xFF))          ?
//                    (alpha - ((clut_ptr[ii]) & 0xFF))       :
//                    (((clut_ptr[ii]) & 0xFF) - alpha);

        red2   = red - ((clut_ptr[ii]>>24) & 0xFF);
        green2 = green - ((clut_ptr[ii]>>16)& 0xFF);
        blue2  = blue - ((clut_ptr[ii]>>8) & 0xFF);
        alpha2 = alpha - ((clut_ptr[ii]) & 0xFF);
        red2   = red2 * red2;
        green2 = green2 * green2;
        blue2  = blue2 * blue2;
        alpha2 = alpha2 * alpha2;

        if ((red2 + green2 + blue2 + alpha2) < diffmin)
        {
            diffmin = red2 + green2 + blue2 + alpha2;
            index = (uint8_t)ii; // must be < 256
        }
    }
    return(index);
}

/*!
*****************************************************************************
** \brief   Gets the RGBA colour for each pixel of virtual OSD layer memory
**          for displaying or combining virtual osd layers in RGBA layout mode.
**
** \param   osd_params_ptr
**              OSD layer parameters
** \param   index
**              point to the clut array
** \param   bpp
**              count of bits per pixel of the OSD layer
** \param   rgba_ptr
**              RGBA colour
**
*****************************************************************************
*/
static void osdhandlerGetRGBAColour(OSDHANDLER_OsdCpParamsT* osd_params_ptr,
                                    const uint32_t  data,
                                    const uint32_t  bpp,
                                    OSDHANDLER_ARGB__T* rgba_ptr )
{
    uint32_t                col32 = 0;

    // No pointer checks for efficiency reasons here.
    //if((osd_params_ptr == NULL)
    //|| (rgba_ptr == NULL))
    //{
    //    // Must be prevented by the calling API function.
    //    return;
    //}
    if (bpp >> 5) // 32-bit
    {
        rgba_ptr->comp.r = data>>24;
        rgba_ptr->comp.g = (data&0x00FF0000)>>16;
        rgba_ptr->comp.b = (data&0x0000FF00)>>8;
        if (osd_params_ptr->useGlobalAlpha)
        {
            rgba_ptr->comp.alpha = osd_params_ptr->globalAlpha;
        }
        else
        {
            rgba_ptr->comp.alpha = (data&0x000000FF);
        }
    }
    else if (bpp >> 4) // 16-bit
    {
        rgba_ptr->comp.r        = (data&0xF800) >> 8;
        rgba_ptr->comp.g        = (data&0x07C0) >> 3;
        rgba_ptr->comp.b        = (data&0x003E) << 2;
        rgba_ptr->comp.alpha    =  data&0x0001;
        if (osd_params_ptr->useGlobalAlpha)
        {
            rgba_ptr->comp.alpha = osd_params_ptr->globalAlpha;
        }
        else
        {
            if (rgba_ptr->comp.alpha == 0)
            {
                rgba_ptr->comp.alpha = OSDHANDLER_5551_Alpha0;
            }
            else
            {
                rgba_ptr->comp.alpha = OSDHANDLER_5551_Alpha1;
            }
        }
    }
    else // CLUT modes
    {
        col32 = osd_params_ptr->clut[data];
        rgba_ptr->comp.r =  col32>>24;
        rgba_ptr->comp.g = (col32&0x00FF0000)>>16;
        rgba_ptr->comp.b = (col32&0x0000FF00)>>8;
        if (osd_params_ptr->useGlobalAlpha)
        {
            rgba_ptr->comp.alpha = osd_params_ptr->globalAlpha;
        }
        else
        {
            rgba_ptr->comp.alpha = (col32&0x000000FF);
        }
    }
    return;
}

/*!
*****************************************************************************
** \brief   Gets the ARGB colour for each pixel of virtual OSD layer memory
**          for displaying or combining virtual OSD layers in ARGB layout mode.
**
** \param   osd_params_ptr
**              OSD layer parameters
** \param   index
**              point to the clut array
** \param   bpp
**              count of bits per pixel of the OSD layer
** \param   rgba_ptr
**              ARGB colour
**
*****************************************************************************
*/
static void osdhandlerGetARGBColour(OSDHANDLER_OsdCpParamsT* osd_params_ptr,
                                    const uint32_t  data,
                                    const uint32_t  bpp,
                                    OSDHANDLER_ARGB__T* argb_ptr )
{
    // No pointer checks for efficiency reasons here.
    //if((osd_params_ptr == NULL)
    //|| (argb_ptr == NULL))
    //{
    //    // Must be prevented by the calling API function.
    //    return;
    //}
    if (bpp >> 5) // 32-bit
    {
        if (osd_params_ptr->useGlobalAlpha)
        {
            argb_ptr->all =
                    (osd_params_ptr->globalAlpha << 24) |
                    (data & 0x00FFFFFFUL);
        }
        else
        {
            argb_ptr->all = data;
        }
    }
    else if (bpp >> 4) // 16-bit
    {
        argb_ptr->comp.r        = (data&0x7C00) >> 7;
        argb_ptr->comp.g        = (data&0x03E0) >> 2;
        argb_ptr->comp.b        = (data&0x001F) << 3;
        argb_ptr->comp.alpha    =  data >> 15;

        if (osd_params_ptr->useGlobalAlpha)
        {
            argb_ptr->comp.alpha = osd_params_ptr->globalAlpha;
        }
        else
        {
            if (argb_ptr->comp.alpha == 0)
            {
                argb_ptr->comp.alpha = OSDHANDLER_5551_Alpha0;
            }
            else
            {
                argb_ptr->comp.alpha = OSDHANDLER_5551_Alpha1;
            }
        }
    }
    else // CLUT modes
    {
        if (osd_params_ptr->useGlobalAlpha)
        {
            argb_ptr->all =
                (osd_params_ptr->globalAlpha << 24) |
                (osd_params_ptr->clut[data]&0x00FFFFFFUL);
        }
        else
        {
            argb_ptr->all = osd_params_ptr->clut[data];
        }
    }
    return;
}

/*!
*****************************************************************************
** \brief Combine the RGBA / ARGB colour of two pixels from 2 virtual layers.
**
** \param   rgb1_ptr
**              RGB colour input 1
** \param   rgb2_ptr
**              RGB colour input 2
** \param   rgb_ptr
**              RGB colour output
**
*****************************************************************************
*/
static void osdhandlerCombine2Colour(OSDHANDLER_ARGB__T* rgb1_ptr,
                                     OSDHANDLER_ARGB__T* rgb2_ptr,
                                     OSDHANDLER_ARGB__T* rgb_ptr)
{
    uint32_t    alpha_sum = 0;

    // No pointer checks for efficiency reasons here.

    alpha_sum = rgb1_ptr->comp.alpha + rgb2_ptr->comp.alpha;

    rgb_ptr->comp.r = ((rgb1_ptr->comp.r*rgb1_ptr->comp.alpha)/(alpha_sum))+
                      ((rgb2_ptr->comp.r*rgb2_ptr->comp.alpha)/(alpha_sum));
    rgb_ptr->comp.g = ((rgb1_ptr->comp.g*rgb1_ptr->comp.alpha)/(alpha_sum))+
                      ((rgb2_ptr->comp.g*rgb2_ptr->comp.alpha)/(alpha_sum));
    rgb_ptr->comp.b = ((rgb1_ptr->comp.b*rgb1_ptr->comp.alpha)/(alpha_sum))+
                      ((rgb2_ptr->comp.b*rgb2_ptr->comp.alpha)/(alpha_sum));

    if (rgb1_ptr->comp.alpha >= rgb2_ptr->comp.alpha)
    {
        rgb_ptr->comp.alpha = rgb1_ptr->comp.alpha;
    }
    else
    {
        rgb_ptr->comp.alpha = rgb2_ptr->comp.alpha;
    }
}

/*!
*****************************************************************************
** \brief   Combine a virtual layer group to the hardware layer at a
**          specific position on the OSD.
**
** \param   groupId
**              Is defined by the hardware OSD layer.
**              groupId = hw OSD layer << 16
** \param   pos_ptr
**              The area to be updated on the hardware layer.
**
*****************************************************************************
*/
static void osdhandlerCombine(const uint32_t groupId,
                                OSDHANDLER_PositionT*       pos_ptr)
{
    OSDHANDLER_OsdCpParamsT*    layer_params_ptr   = NULL;     //underlining hw layer
    OSDHANDLER_OsdCpParamsT*    group_params_ptr   = NULL;     //virtual osd layer group
    OSDHANDLER_ColourParamsT    layer_colour_params;
    OSDHANDLER_ColourParamsT    osd_colour_params[OSDHANDLER_MAX_NUM_VIRTUAL_LAYER];
    uint32_t                    pixel_base_addr     = 0;
    uint32_t                    x_pos               = 0;
    uint32_t                    y_pos               = 0;
    uint32_t                    x_size              = 0;
    uint32_t                    y_size              = 0;
    uint32_t                    group               = 0;
    uint32_t                    ii                  = 0;
    uint32_t                    jj                  = 0;

    group = groupId >> 16;

    layer_params_ptr = osdhandlerGetOsdCpParams(group);
    if((layer_params_ptr == NULL)
    || (pos_ptr == NULL))
    {
        // Must be prevented by the calling API function.
        return;
    }
    memset(&layer_colour_params, 0, sizeof(OSDHANDLER_ColourParamsT));
    layer_colour_params.bpp = osdhandlerGetBitsPerPixel(layer_params_ptr);

    group_params_ptr = osdhandlerGetOsdGroupParams(groupId);
    if(group_params_ptr == NULL)
    {
        // Must be prevented by the calling API function.
        return;
    }
    for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
    {
        memset(&osd_colour_params[ii], 0, sizeof(OSDHANDLER_ColourParamsT));
        if(group_params_ptr->allocatedMemPtr != NULL)
        {
            osd_colour_params[ii].bpp = osdhandlerGetBitsPerPixel(group_params_ptr);
        }
        group_params_ptr++;
    }

    x_pos = pos_ptr->x;
    y_pos = pos_ptr->y;
    x_size = pos_ptr->width;
    y_size = pos_ptr->height;


    // Get colour mask, base address and pixel offset for HW layer...

    if(layer_colour_params.bpp >> 5) // bpp >= 32 ?
    {
        layer_colour_params.colour_mask = 0xFFFFFFFFUL;
    }
    else
    {
        layer_colour_params.colour_mask = \
                (0x00000001UL << layer_colour_params.bpp) - 0x00000001UL;
    }
    osdhandlerGetPixelAddress(pos_ptr->x, y_pos, layer_colour_params.bpp,
                              layer_params_ptr, &pixel_base_addr,
                              &layer_colour_params.pixel_bit_offset);
    /*lint -save -e923 */
    layer_colour_params.osd_base_ptr = (uint32_t*)pixel_base_addr;
    /*lint -restore */

    // Get colour mask, base address and pixel offset for virtual layers...

    group_params_ptr = osdhandlerGetOsdGroupParams(groupId);

    for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
    {
        if(osd_colour_params[ii].bpp)
        {
            if(osd_colour_params[ii].bpp >> 5) // bpp >= 32 ?
            {
                osd_colour_params[ii].colour_mask = 0xFFFFFFFFUL;
            }
            else
            {
                osd_colour_params[ii].colour_mask = \
                        (0x00000001UL << osd_colour_params[ii].bpp) - 0x00000001UL;
            }

            osdhandlerGetPixelAddress(pos_ptr->x, y_pos, osd_colour_params[ii].bpp,
                                group_params_ptr, &pixel_base_addr,
                                &osd_colour_params[ii].pixel_bit_offset);
            /*lint -save -e923 */
            osd_colour_params[ii].osd_base_ptr = (uint32_t*)pixel_base_addr;
            /*lint -restore */

            /* temp_data keeps the 32-bit value referenced by the last
            ** osd_base_ptr.
            */
            osd_colour_params[ii].temp_data = *osd_colour_params[ii].osd_base_ptr++;
        }
        group_params_ptr++;
    }

    if(layer_params_ptr->osdLayout == OSDHANDLER_OSD_LAYOUT_RGBA)
    {
        while (y_size)
        {
            // Get the unified RGB value of the current pixel for each virtual layer...
            group_params_ptr = osdhandlerGetOsdGroupParams(groupId);

            for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
            {
                if(osd_colour_params[ii].bpp) // if virtual layer is active ...
                {
                    osd_colour_params[ii].data_mask = osd_colour_params[ii].colour_mask <<
                            (0x20UL - (osd_colour_params[ii].pixel_bit_offset +
                            osd_colour_params[ii].bpp));
                    /* Member 'data' becomes the value of the pixel indicated by
                    ** osd_base_ptr and pixel_bit_offset on the virtual layer.
                    */
                    osd_colour_params[ii].data = (osd_colour_params[ii].temp_data &
                            osd_colour_params[ii].data_mask) >> (0x20UL -
                            (osd_colour_params[ii].pixel_bit_offset + osd_colour_params[ii].bpp));
                    osdhandlerGetRGBAColour(group_params_ptr, osd_colour_params[ii].data,
                            osd_colour_params[ii].bpp, &osd_colour_params[ii].rgb);
                }
                group_params_ptr++;
            }

            // Combine colours from virtual layers to the HW layer...
            for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
            {
                if(osd_colour_params[ii].bpp)
                {
                    layer_colour_params.rgb.all = osd_colour_params[ii].rgb.all;

                    // TODO: this is NOT the arithmetic mean of all colours... --RK
                    for(jj = ii + 1; jj < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; jj++)
                    {
                        if (osd_colour_params[jj].rgb.all)
                        {
                            osdhandlerCombine2Colour(&layer_colour_params.rgb,
                                &osd_colour_params[jj].rgb, &layer_colour_params.rgb);
                        }
                    }
                    break;
                }
            }

            /*write pixel to hardware OSD memory*/
            // 32-bit HW OSD:
            if(layer_colour_params.bpp >> 5)
            {
                *layer_colour_params.osd_base_ptr++ = \
                                        (layer_colour_params.rgb.comp.r<<24) |
                                        (layer_colour_params.rgb.comp.g<<16) |
                                        (layer_colour_params.rgb.comp.b<< 8) |
                                        (layer_colour_params.rgb.comp.alpha);
            }
            // 16-bit HW OSD:
            else if(layer_colour_params.bpp >> 4)
            {
                layer_colour_params.rgb.comp.r = layer_colour_params.rgb.comp.r>>3;
                layer_colour_params.rgb.comp.g = layer_colour_params.rgb.comp.g>>3;
                layer_colour_params.rgb.comp.b = layer_colour_params.rgb.comp.b>>3;
                if (layer_colour_params.rgb.comp.alpha >= OSDHANDLER_Alpha_Threshold)
                {
                    layer_colour_params.rgb.comp.alpha = 0x1;
                }
                else
                {
                    layer_colour_params.rgb.comp.alpha = 0x0;
                }
                layer_colour_params.temp_data = \
                                        (layer_colour_params.rgb.comp.r<<11)  |
                                        (layer_colour_params.rgb.comp.g<< 6)  |
                                        (layer_colour_params.rgb.comp.b<< 1)  |
                                        (layer_colour_params.rgb.comp.alpha);
                layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                    (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                layer_colour_params.data = ((layer_colour_params.data) &
                    (~layer_colour_params.data_mask)) |(layer_colour_params.temp_data <<
                    (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp)));
                layer_colour_params.pixel_bit_offset += layer_colour_params.bpp;
                if (layer_colour_params.pixel_bit_offset >> 5)
                {
                    *layer_colour_params.osd_base_ptr++ = layer_colour_params.data;
                    layer_colour_params.pixel_bit_offset = 0;
                }
            }
            // 8-bit or 4-bit HW OSD:
            else
            {
                layer_colour_params.temp_data =
                    osdhandlerSearchBestColour(
                        layer_colour_params.rgb.all,
                        layer_params_ptr->clut,
                        (uint8_t)((1UL << layer_colour_params.bpp)-1UL) ); // is <256
                layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                    (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                layer_colour_params.data = ((layer_colour_params.data) &
                    (~layer_colour_params.data_mask)) |(layer_colour_params.temp_data <<
                    (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp)));
                layer_colour_params.pixel_bit_offset += layer_colour_params.bpp;
                if (layer_colour_params.pixel_bit_offset >> 5)
                {
                    *layer_colour_params.osd_base_ptr++ = layer_colour_params.data;
                    layer_colour_params.pixel_bit_offset = 0;
                }
            }
            // Update pixel bit offset and OSD base pointer for all virtual layers...
            for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
            {
                if(osd_colour_params[ii].bpp)
                {
                    osd_colour_params[ii].pixel_bit_offset += osd_colour_params[ii].bpp;
                    if (osd_colour_params[ii].pixel_bit_offset >> 5)
                    {
                        osd_colour_params[ii].temp_data = *osd_colour_params[ii].osd_base_ptr++;
                        osd_colour_params[ii].pixel_bit_offset = 0;
                    }
                }
            }

            x_pos++;
            if (x_pos >= x_size) // check end of Pixel-Line
            {
                x_pos = 0;
                y_pos++;
                y_size--;
                if (!y_size)
                {
                    break;
                }

                /* After line breaks get new osd_base_ptr and pixel_bit_offset
                ** for the HW layer.
                */
                osdhandlerGetPixelAddress(pos_ptr->x, y_pos, layer_colour_params.bpp,
                            layer_params_ptr, &pixel_base_addr,
                            &layer_colour_params.pixel_bit_offset);
                /*lint -save -e923 */
                layer_colour_params.osd_base_ptr = (uint32_t*)pixel_base_addr;
                /*lint -restore */


                /* After line breaks get new osd_base_ptr and pixel_bit_offset
                ** for all active virtual layers.
                */
                group_params_ptr = osdhandlerGetOsdGroupParams(groupId);

                for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
                {
                    if(osd_colour_params[ii].bpp)
                    {
                        osdhandlerGetPixelAddress(pos_ptr->x, y_pos, osd_colour_params[ii].bpp,
                                            group_params_ptr, &pixel_base_addr,
                                            &osd_colour_params[ii].pixel_bit_offset);
                        /*lint -save -e923 */
                        osd_colour_params[ii].osd_base_ptr = (uint32_t*)pixel_base_addr;
                        /*lint -restore */
                        osd_colour_params[ii].temp_data = *osd_colour_params[ii].osd_base_ptr++;
                    }
                    group_params_ptr++;
                }
            }
        }
    }
    else if(layer_params_ptr->osdLayout == OSDHANDLER_OSD_LAYOUT_ARGB)
    {
        while (y_size)
        {
            group_params_ptr = osdhandlerGetOsdGroupParams(groupId);

            // Get the unified RGB value of the current pixel for each virtual layer...
            for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
            {
                if(osd_colour_params[ii].bpp) // if virtual layer is active ...
                {
                    osd_colour_params[ii].data_mask = osd_colour_params[ii].colour_mask <<
                            (0x20UL - (osd_colour_params[ii].pixel_bit_offset +
                            osd_colour_params[ii].bpp));
                    /* Member 'data' becomes the value of the pixel indicated by
                    ** osd_base_ptr and pixel_bit_offset on the virtual layer.
                    */
                    osd_colour_params[ii].data = (osd_colour_params[ii].temp_data &
                            osd_colour_params[ii].data_mask) >> (0x20UL -
                            (osd_colour_params[ii].pixel_bit_offset + osd_colour_params[ii].bpp));
                    osdhandlerGetARGBColour(group_params_ptr, osd_colour_params[ii].data,
                            osd_colour_params[ii].bpp, &osd_colour_params[ii].rgb);
                }
                group_params_ptr++;
            }

            // Combine colours from virtual layers to the HW layer...
            for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
            {
                if(osd_colour_params[ii].bpp)
                {
                    layer_colour_params.rgb.all = osd_colour_params[ii].rgb.all;

                    // TODO: this is NOT the arithmetic mean of all colours... --RK
                    for(jj = ii + 1; jj < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; jj++)
                    {
                        if (osd_colour_params[jj].rgb.all)
                        {
                            osdhandlerCombine2Colour(&layer_colour_params.rgb,
                                &osd_colour_params[jj].rgb, &layer_colour_params.rgb);
                        }
                    }
                    break;
                }
            }

            /*write pixel to hardware OSD memory*/
            // 32-bit HW OSD:
            if(layer_colour_params.bpp >> 5)
            {
                *layer_colour_params.osd_base_ptr++ = layer_colour_params.rgb.all;
            }
            // 16-bit HW OSD:
            else if(layer_colour_params.bpp >> 4)
            {
                layer_colour_params.rgb.comp.r = layer_colour_params.rgb.comp.r>>3;
                layer_colour_params.rgb.comp.g = layer_colour_params.rgb.comp.g>>3;
                layer_colour_params.rgb.comp.b = layer_colour_params.rgb.comp.b>>3;
                if (layer_colour_params.rgb.comp.alpha >= OSDHANDLER_Alpha_Threshold)
                {
                    layer_colour_params.rgb.comp.alpha = 0x1;
                }
                else
                {
                    layer_colour_params.rgb.comp.alpha = 0x0;
                }
                layer_colour_params.temp_data = (layer_colour_params.rgb.comp.alpha<<15) |
                                                (layer_colour_params.rgb.comp.r<<10)     |
                                                (layer_colour_params.rgb.comp.g<<5)      |
                                                 layer_colour_params.rgb.comp.b;
                layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                    (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                layer_colour_params.data = ((layer_colour_params.data) &
                    (~layer_colour_params.data_mask)) |(layer_colour_params.temp_data <<
                    (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp)));
                layer_colour_params.pixel_bit_offset += layer_colour_params.bpp;
                if (layer_colour_params.pixel_bit_offset >> 5)
                {
                    *layer_colour_params.osd_base_ptr++ = layer_colour_params.data;
                    layer_colour_params.pixel_bit_offset = 0;
                }
            }
            // 8-bit or 4-bit HW OSD:
            else
            {
                layer_colour_params.temp_data =
                    osdhandlerSearchBestColour(
                        layer_colour_params.rgb.all,
                        layer_params_ptr->clut,
                        (uint8_t)((1UL << layer_colour_params.bpp)-1UL) ); // is <256
                layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                    (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                layer_colour_params.data = ((layer_colour_params.data) &
                    (~layer_colour_params.data_mask)) |(layer_colour_params.temp_data <<
                    (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp)));
                layer_colour_params.pixel_bit_offset += layer_colour_params.bpp;
                if (layer_colour_params.pixel_bit_offset >> 5)
                {
                    *layer_colour_params.osd_base_ptr++ = layer_colour_params.data;
                    layer_colour_params.pixel_bit_offset = 0;
                }
            }

            // Update pixel bit offset and OSD base pointer for all virtual layers...
            for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
            {
                if(osd_colour_params[ii].bpp)
                {
                    osd_colour_params[ii].pixel_bit_offset += osd_colour_params[ii].bpp;
                    if (osd_colour_params[ii].pixel_bit_offset >> 5)
                    {
                        osd_colour_params[ii].temp_data = *osd_colour_params[ii].osd_base_ptr++;
                        osd_colour_params[ii].pixel_bit_offset = 0;
                    }
                }
            }

            x_pos++;
            if (x_pos >= x_size) // check end of Pixel-Line
            {
                x_pos = 0;
                y_pos++;
                y_size--;
                if (!y_size)
                {
                    break;
                }
                /* After line breaks get new osd_base_ptr and pixel_bit_offset
                ** for the HW layer.
                */
                osdhandlerGetPixelAddress(pos_ptr->x, y_pos, layer_colour_params.bpp,
                            layer_params_ptr, &pixel_base_addr,
                            &layer_colour_params.pixel_bit_offset);
                /*lint -save -e923*/
                layer_colour_params.osd_base_ptr = (uint32_t*)pixel_base_addr;
                /*lint -restore */

                /* After line breaks get new osd_base_ptr and pixel_bit_offset
                ** for all active virtual layers.
                */
                group_params_ptr = osdhandlerGetOsdGroupParams(groupId);

                for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
                {
                    if(osd_colour_params[ii].bpp)
                    {
                        osdhandlerGetPixelAddress(pos_ptr->x, y_pos, osd_colour_params[ii].bpp,
                                            group_params_ptr, &pixel_base_addr,
                                            &osd_colour_params[ii].pixel_bit_offset);
                        /*lint -save -e923 */
                        osd_colour_params[ii].osd_base_ptr = (uint32_t*)pixel_base_addr;
                        /*lint -restore */
                        osd_colour_params[ii].temp_data = *osd_colour_params[ii].osd_base_ptr++;
                    }
                    group_params_ptr++;
                }
            }
        }
    }
}

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief   Allocate the hardware OSD layer for a virtual layer group (groupID).
**
**          After calling OSDHANDLER_StartVirtualLayerGroup() the hardware
**          OSD layer is not accessible by any other functions.
**          If direct access is needed OSDHANDLER_HideVirtualLayerGroup()
**          has to be called.
**
** \note    The virtual layer system works only with the following layout modes:
**          - OSDHANDLER_OSD_LAYOUT_RGBA
**          - OSDHANDLER_OSD_LAYOUT_ARGB
**          .
**          The virtual layer system works only with the following colour modes:
**          - OSDHANDLER_OSD_MODE_CLUT_4BIT
**          - OSDHANDLER_OSD_MODE_CLUT_8BIT
**          - OSDHANDLER_OSD_MODE_5551
**          - OSDHANDLER_OSD_MODE_32_24BIT
**          .
** \param   groupId
**              Is defined by the hardware OSD layer.
**              groupId = hardware OSD layer << 16
** \param   initParamsPtr
**              Initialization parameters for the virtual layer group.
**
** \return
**          -   #FAPI_OK on success.
**          -   #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          -   #OSDHANDLER_ERR_BAD_PARAMETER
**              if the passed initParamsPtr is NULL.
**          -   #OSDHANDLER_ERR_BAD_LAYER_ID
**              if the groupId is bad.
**          -   #OSDHANDLER_ERR_BAD_PARAMETER
**              if the layout mode or the colour mode is unknown or not supported.
**          -   #OSDHANDLER_ERR_NO_OSD_HANDLE
**              if the OSDHANDLER has not allocated the corresponding handle of
**              the OSD/CP at initialization.
**          -   #OSDHANDLER_ERR_LAYER_NOT_STARTED
**              if the corresponding hardware layer has not been started yet.
**          -   #OSDHANDLER_ERR_VL_ALREADY_EXISTS
**              if the virtual layer group already exists.
**          -   #OSDHANDLER_ERR_BAD_COLOUR_MODE
**              if the OSD is driven in a bad colour mode.
**          -   #OSDHANDLER_ERR_VL_NO_FREE_LAYER
**              if the maximum number of virtual OSD layer has been reached.
**
*****************************************************************************
*/
int32_t OSDHANDLER_StartVirtualLayerGroup(
                        const uint32_t groupId,
                        const OSDHANDLER_VirtualGroupInitParamsT* initParamsPtr)
{
    OSDHANDLER_OsdCpParamsT*    osd_cp_params_ptr   = NULL;
    uint32_t                    group               = 0;
    uint32_t                    ii                  = 0;
    uint32_t                    jj                  = 0;

    if(!osdhandlerInitialized)
    {
        return (OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    if(initParamsPtr == NULL)
    {
        // Must be prohibited by calling API function.
        return(OSDHANDLER_ERR_BAD_PARAMETER);
    }
    group = groupId >> 16;  // virtual layer group - hw layer
    osd_cp_params_ptr = osdhandlerGetOsdCpParams(group);
    if(osd_cp_params_ptr == NULL)
    {
        return(OSDHANDLER_ERR_BAD_LAYER_ID);
    }
    if(osd_cp_params_ptr->fd == -1)
    {
        return(OSDHANDLER_ERR_NO_OSD_HANDLE);
    }
    if(osd_cp_params_ptr->allocatedMemPtr == NULL)
    {
        return(OSDHANDLER_ERR_LAYER_NOT_STARTED);
    }

    if(osd_cp_params_ptr == &osdhandlerOsd1Params)        // hw layer 1
    {
        if(osdhandlerGroup1Started)
        {
            return(OSDHANDLER_ERR_VL_ALREADY_EXISTS);
        }

        // ************* Virtual OSD layer group 1 Initialization *************//
        for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
        {
            memset(&osdhandlerGroup1Params[ii], 0, sizeof(OSDHANDLER_OsdCpParamsT));
            memset(&osdhandlerGroup1Fifo[ii], 0, sizeof(OSDHANDLER_PositionT));

            if(initParamsPtr->osdInitParams[ii].inUse)
            {
                osdhandlerGroup1Params[ii].fd = 1;     // dummy descriptor
                osdhandlerGroup1Params[ii].allocatedMemPtr = NULL;
                osdhandlerGroup1Params[ii].fillColour = 0x0;                // valid for all colour modes
                osdhandlerGroup1Params[ii].colourMode = initParamsPtr->osdInitParams[ii].colourMode;
                osdhandlerGroup1Params[ii].osdLayout = initParamsPtr->osdInitParams[ii].osdLayout;
                osdhandlerGroup1Params[ii].scalerParams.mode = \
                    osdhandlerGetOsdColourMode(initParamsPtr->osdInitParams[ii].colourMode);
                osdhandlerGroup1Params[ii].scalerParams.osdByteOrder = \
                    osdhandlerGetOsdLayout(initParamsPtr->osdInitParams[ii].osdLayout);
                if(osdhandlerGetBitsPerPixel(&osdhandlerGroup1Params[ii]) == 0)
                {
                    return(OSDHANDLER_ERR_BAD_COLOUR_MODE);
                }
                //support ONLY RGB layout modes
                if(!((osdhandlerGroup1Params[ii].osdLayout == FAPEX_VIOUT_OSD_LAYOUT_ARGB)
                   ||(osdhandlerGroup1Params[ii].osdLayout == FAPEX_VIOUT_OSD_LAYOUT_RGBA)))
                {
                    return(OSDHANDLER_ERR_BAD_PARAMETER);
                }
                //virtual layer must in the same layout mode with its underlining hw layer
                if(!(osdhandlerGroup1Params[ii].osdLayout == \
                        osd_cp_params_ptr->osdLayout))
                {
                    return(OSDHANDLER_ERR_BAD_PARAMETER);
                }
                //suport ONLY these colour mode
                if( !(osdhandlerGroup1Params[ii].colourMode == OSDHANDLER_OSD_MODE_CLUT_4BIT)&&
                    !(osdhandlerGroup1Params[ii].colourMode == OSDHANDLER_OSD_MODE_CLUT_8BIT)&&
                    !(osdhandlerGroup1Params[ii].colourMode == OSDHANDLER_OSD_MODE_5551)&&
                    !(osdhandlerGroup1Params[ii].colourMode == OSDHANDLER_OSD_MODE_32_24BIT))
                {
                    return(OSDHANDLER_ERR_BAD_PARAMETER);
                }
                for (jj = 0; jj < 256; jj++)
                {
                    osdhandlerGroup1Params[ii].clut[jj] =
                                *(initParamsPtr->osdInitParams[ii].defaultClutPtr + jj);
                }
                osdhandlerGroup1Params[ii].useGlobalAlpha                   = \
                                initParamsPtr->osdInitParams[ii].useGlobalAlpha;
                osdhandlerGroup1Params[ii].globalAlpha                      = \
                                initParamsPtr->osdInitParams[ii].globalAlpha;
                osdhandlerGroup1Params[ii].layerEnable                      = 0;
                osdhandlerGroup1Params[ii].scalerParams.flagFrameField      = 0;
                osdhandlerGroup1Params[ii].scalerParams.inputWindowX0       = 0;
                osdhandlerGroup1Params[ii].scalerParams.inputWindowY0       = 0;
                osdhandlerGroup1Params[ii].scalerParams.outputWindowX0      = 0;
                osdhandlerGroup1Params[ii].scalerParams.outputWindowY0      = 0;
                osdhandlerGroup1Params[ii].scalerParams.outputWindowWidth   = 0;
                osdhandlerGroup1Params[ii].scalerParams.outputWindowHeight  = 0;
            }

        }
        //hw layer is locked once the virtual layer groupp is started
        osdhandlerGroup1Started = 1;
        osdhandlerGroup1Shown   = 1;
    }
    else if (osd_cp_params_ptr == &osdhandlerOsd2Params)
    {
        if(osdhandlerGroup2Started)
        {
            return(OSDHANDLER_ERR_VL_ALREADY_EXISTS);
        }

        // ************* Virtual OSD layer groupp 2 Initialization *************//
        for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
        {
            memset(&osdhandlerGroup2Params[ii], 0, sizeof(OSDHANDLER_OsdCpParamsT));
            memset(&osdhandlerGroup2Fifo[ii], 0, sizeof(OSDHANDLER_PositionT));

            if(initParamsPtr->osdInitParams[ii].inUse)
            {
                osdhandlerGroup2Params[ii].fd = 1;     // dummy descriptor
                osdhandlerGroup2Params[ii].allocatedMemPtr = NULL;
                osdhandlerGroup2Params[ii].fillColour = 0x0;                // valid for all colour modes
                osdhandlerGroup2Params[ii].colourMode = initParamsPtr->osdInitParams[ii].colourMode;
                osdhandlerGroup2Params[ii].osdLayout = initParamsPtr->osdInitParams[ii].osdLayout;
                osdhandlerGroup2Params[ii].scalerParams.mode = \
                    osdhandlerGetOsdColourMode(initParamsPtr->osdInitParams[ii].colourMode);
                osdhandlerGroup2Params[ii].scalerParams.osdByteOrder = \
                    osdhandlerGetOsdLayout(initParamsPtr->osdInitParams[ii].osdLayout);
                if(osdhandlerGetBitsPerPixel(&osdhandlerGroup1Params[ii]) == 0)
                {
                    return(OSDHANDLER_ERR_BAD_COLOUR_MODE);
                }
                //support ONLY RGB layout modes
                if(!((osdhandlerGroup2Params[ii].osdLayout == FAPEX_VIOUT_OSD_LAYOUT_ARGB)
                   ||(osdhandlerGroup2Params[ii].osdLayout == FAPEX_VIOUT_OSD_LAYOUT_RGBA)))
                {
                    return(OSDHANDLER_ERR_BAD_PARAMETER);
                }
                //virtual layer must in the same layout mode with its underlining hw layer
                if(!(osdhandlerGroup2Params[ii].osdLayout == \
                        osd_cp_params_ptr->osdLayout))
                {
                    return(OSDHANDLER_ERR_BAD_PARAMETER);
                }
                //suport ONLY these colour mode
                if( !(osdhandlerGroup2Params[ii].colourMode == OSDHANDLER_OSD_MODE_CLUT_4BIT)&&
                    !(osdhandlerGroup2Params[ii].colourMode == OSDHANDLER_OSD_MODE_CLUT_8BIT)&&
                    !(osdhandlerGroup2Params[ii].colourMode == OSDHANDLER_OSD_MODE_5551)&&
                    !(osdhandlerGroup2Params[ii].colourMode == OSDHANDLER_OSD_MODE_32_24BIT))
                {
                    return(OSDHANDLER_ERR_BAD_PARAMETER);
                }
                for (jj = 0; jj < 256; jj++)
                {
                    osdhandlerGroup2Params[ii].clut[jj] =
                                *(initParamsPtr->osdInitParams[ii].defaultClutPtr + jj);
                }
                osdhandlerGroup2Params[ii].useGlobalAlpha                   = \
                                initParamsPtr->osdInitParams[ii].useGlobalAlpha;
                osdhandlerGroup2Params[ii].globalAlpha                      = \
                                initParamsPtr->osdInitParams[ii].globalAlpha;
                osdhandlerGroup2Params[ii].layerEnable                      = 0;
                osdhandlerGroup2Params[ii].scalerParams.flagFrameField      = 0;
                osdhandlerGroup2Params[ii].scalerParams.inputWindowX0       = 0;
                osdhandlerGroup2Params[ii].scalerParams.inputWindowY0       = 0;
                osdhandlerGroup2Params[ii].scalerParams.outputWindowX0      = 0;
                osdhandlerGroup2Params[ii].scalerParams.outputWindowY0      = 0;
                osdhandlerGroup2Params[ii].scalerParams.outputWindowWidth   = 0;
                osdhandlerGroup2Params[ii].scalerParams.outputWindowHeight  = 0;
            }
        }

        //hw layer is locked once the virtual layer groupp is started
        osdhandlerGroup2Started = 1;
        osdhandlerGroup2Shown   = 1;
    }
    else // no virtual layers for the Cursor Plane available
    {
        return(OSDHANDLER_ERR_BAD_LAYER_ID);
    }

    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Free the previously allocated hardware OSD layer.
**          This function closes all virtual OSDs of the given virtual
**          layer group, frees allocated memory.
**
** \param   groupId
**              is defined by the hardware OSD layer.
**              groupId = hw OSD layer << 16
**
** \return
**          -   #FAPI_OK on success.
**          -   #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          -   #OSDHANDLER_ERR_LAYER_NOT_STARTED
**              if the virtual layer group has not been started yet.
**          -   #OSDHANDLER_ERR_BAD_LAYER_ID
**              if the passed groupId is invalid.
**
*****************************************************************************
*/
int32_t  OSDHANDLER_StopVirtualLayerGroup(const uint32_t groupId)
{
    uint32_t            group = 0;
    uint32_t            ii    = 0;

    group = groupId >> 16;

    if(!osdhandlerInitialized)
    {
        return (OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    if(group == 1)
    {
        if (!osdhandlerGroup1Started)
        {
            return (OSDHANDLER_ERR_LAYER_NOT_STARTED);
        }
        // close virtual OSD layer group 1:
        for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
        {
            if((osdhandlerGroup1Params[ii].isMemoryDynamic)
             &&(osdhandlerGroup1Params[ii].allocatedMemPtr))
            {
                FAPI_SYS_FREE(osdhandlerGroup1Params[ii].allocatedMemPtr);
                osdhandlerGroup1Params[ii].allocatedMemPtr = NULL;
            }
            osdhandlerGroup1Params[ii].fd = -1; // destroy dummy handle
        }
        osdhandlerGroup1Started = 0;
        osdhandlerGroup1Shown   = 0;
    }
    else if(group == 2)
    {
        if (!osdhandlerGroup2Started)
        {
            return (OSDHANDLER_ERR_LAYER_NOT_STARTED);
        }
        // close virtual OSD layer group 2:
        for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
        {
            if((osdhandlerGroup2Params[ii].isMemoryDynamic)
             &&(osdhandlerGroup2Params[ii].allocatedMemPtr))
            {
                FAPI_SYS_FREE(osdhandlerGroup2Params[ii].allocatedMemPtr);
                osdhandlerGroup2Params[ii].allocatedMemPtr = NULL;
            }
            osdhandlerGroup2Params[ii].fd = -1; // destroy dummy handle
        }
        osdhandlerGroup2Started = 0;
        osdhandlerGroup2Shown   = 0;
    }
    else
    {
        return (OSDHANDLER_ERR_BAD_LAYER_ID);
    }

    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Hide all virtual OSD layers of group groupId. After calling
**          this function, direct access to the hardware OSD layer
**          is possible.
**          While the virtual OSD layer group is hidden, it is possible to
**          draw in background to the virtual OSD layer.
**          The change becomes visible after calling
**          OSDHANDLER_ShowVirtualLayerGroup();
**
** \param   groupId
**              is defined by the hardware OSD layer.
**              groupId = hw OSD layer << 16
**
** \return
**          -   #FAPI_OK on success.
**          -   #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          -   #OSDHANDLER_ERR_VL_ALREADY_HIDDEN
**              if the layer is already hidden.
**          -   #OSDHANDLER_ERR_LAYER_NOT_STARTED
**              if the virtual layer group has not been started yet.
**          -   #OSDHANDLER_ERR_BAD_LAYER_ID
**              if the passed groupId is invalid.
**
*****************************************************************************
*/
int32_t  OSDHANDLER_HideVirtualLayerGroup(const uint32_t groupId)
{
    if(!osdhandlerInitialized)
    {
        return (OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    if((groupId != 0x00010000UL)
    && (groupId != 0x00020000UL))
    {
        return(OSDHANDLER_ERR_BAD_LAYER_ID);
    }
    if(groupId == 0x00010000UL)
    {
        if (!osdhandlerGroup1Started)
        {
            return (OSDHANDLER_ERR_LAYER_NOT_STARTED);
        }
        if (osdhandlerGroup1Shown)
        {
            osdhandlerGroup1Shown     = 0;
        }
        else
        {
            return (OSDHANDLER_ERR_VL_ALREADY_HIDDEN);
        }
    }
    else if(groupId == 0x00020000UL)
    {
        if (!osdhandlerGroup2Started)
        {
            return (OSDHANDLER_ERR_LAYER_NOT_STARTED);
        }
        if (osdhandlerGroup2Shown)
        {
            osdhandlerGroup2Shown     = 0;
        }
        else
        {
            return (OSDHANDLER_ERR_VL_ALREADY_HIDDEN);
        }
    }
    else
    {
        return (OSDHANDLER_ERR_BAD_LAYER_ID);
    }

    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Show all virtual OSD layer of group groupId. If there is anything
**          to present on one of the virtual OSD layer of this group it will
**          be presented otherwise the hardware OSD layer will be filled
**          with OSD fill colour.
**
** \param   groupId
**              is defined by the hardware OSD layer.
**              groupId = hw OSD layer << 16
** \param   errorCodePtr
**              Will receive error codes related to specific hardware layers.
**              If there occured an error during the underlying driver calls,
**              one or more of the following error messages may be contained:
**              - #OSDHANDLER_ERR_CODE_OSD1_ENABLE
**              - #OSDHANDLER_ERR_CODE_OSD2_ENABLE
**              - #OSDHANDLER_ERR_CODE_CP_ENABLE
**              - #OSDHANDLER_ERR_CODE_OSD1_START
**              - #OSDHANDLER_ERR_CODE_OSD2_START
**              - #OSDHANDLER_ERR_CODE_CP_START
**              - #OSDHANDLER_ERR_CODE_OSD1_CLUT
**              - #OSDHANDLER_ERR_CODE_OSD2_CLUT
**              - #OSDHANDLER_ERR_CODE_CP_CLUT
**              .
**              If no errors occured, the word equals FAPI_OK. <br>
**
** \return
**          -   #FAPI_OK on success.
**          -   #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          -   #OSDHANDLER_ERR_BAD_LAYER_ID
**              if the passed groupId is invalid.
**          -   #OSDHANDLER_ERR_NO_OSD_HANDLE
**              if the OSDHANDLER has not allocated the handle of
**              the corresponding hardware OSD at initialization.
**          -   #OSDHANDLER_ERR_LAYER_NOT_STARTED
**              if the corresponding hardware layer or the virtual
**              layer group has not been started yet.
**          -   #OSDHANDLER_ERR_BAD_COLOUR_MODE
**              if the corresponding hardware OSD layer is driven
**              in a bad colour mode.
**
*****************************************************************************
*/
int32_t  OSDHANDLER_ShowVirtualLayerGroup(const uint32_t groupId,
                                          uint32_t* errorCodePtr)
{
    OSDHANDLER_OsdCpParamsT*    layer_params_ptr   = NULL;     //underlining hw layer
    OSDHANDLER_OsdCpParamsT*    group_params_ptr   = NULL;     //virtual osd layer group
    OSDHANDLER_ColourParamsT    layer_colour_params;
    OSDHANDLER_ColourParamsT    osd_colour_params[OSDHANDLER_MAX_NUM_VIRTUAL_LAYER];
    uint32_t                    group               = 0;
    uint32_t                    x_pos               = 0;
//  uint32_t                    y_line              = 0;
    uint32_t                    x_size              = 0;
    uint32_t                    y_size              = 0;
    uint32_t                    ii                  = 0;
    uint32_t                    jj                  = 0;
    uint32_t                    count               = 0;
    uint32_t                    currentErrorCode    = 0;

    group = groupId>>16;

    if(!osdhandlerInitialized)
    {
        return (OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    // the underlining hw should already started
    layer_params_ptr = osdhandlerGetOsdCpParams(group);
    if(layer_params_ptr == NULL)
    {
        return(OSDHANDLER_ERR_BAD_LAYER_ID);
    }
    if((groupId != 0x00010000UL)
    && (groupId != 0x00020000UL))
    {
        return(OSDHANDLER_ERR_BAD_LAYER_ID);
    }
    if(layer_params_ptr->fd == -1)
    {
        return(OSDHANDLER_ERR_NO_OSD_HANDLE);
    }
    if(layer_params_ptr->allocatedMemPtr == NULL)
    {
        return(OSDHANDLER_ERR_LAYER_NOT_STARTED);
    }

    memset(&layer_colour_params, 0, sizeof(OSDHANDLER_ColourParamsT));

    layer_colour_params.bpp = osdhandlerGetBitsPerPixel(layer_params_ptr);
    if(layer_colour_params.bpp== 0)
    {
        return(OSDHANDLER_ERR_BAD_COLOUR_MODE);
    }

    if(group == 1)
    {
        if (!osdhandlerGroup1Started)
        {
            return (OSDHANDLER_ERR_LAYER_NOT_STARTED);
        }
        group_params_ptr = osdhandlerGetOsdGroupParams(groupId);

    }
    else if (group == 2)
    {
        if (!osdhandlerGroup2Started)
        {
            return (OSDHANDLER_ERR_LAYER_NOT_STARTED);
        }
        group_params_ptr = osdhandlerGetOsdGroupParams(groupId);
    }
    else
    {
        return(OSDHANDLER_ERR_BAD_LAYER_ID);
    }
    if(group_params_ptr == NULL)
    {
        return(OSDHANDLER_ERR_BAD_LAYER_ID);
    }
    for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
    {
        memset(&osd_colour_params[ii], 0, sizeof(OSDHANDLER_ColourParamsT));

        if(group_params_ptr->allocatedMemPtr != NULL)
        {
            osd_colour_params[ii].bpp = osdhandlerGetBitsPerPixel(group_params_ptr);
            if(osd_colour_params[ii].bpp)
            {
                // bpp indicates whether the virtual layer is active
                count++;
            }
        }
        group_params_ptr++;
    }

    x_size = layer_params_ptr->scalerParams.inputWindowWidth;
    y_size = layer_params_ptr->scalerParams.inputWindowHeight;

    if(layer_colour_params.bpp >> 5) // bpp >= 32 ?
    {
        layer_colour_params.colour_mask = 0xFFFFFFFFUL;
    }
    else
    {
        layer_colour_params.colour_mask =
                (0x00000001UL << layer_colour_params.bpp) - 0x00000001UL;
    }

    /*lint -save -e923 */
    layer_colour_params.osd_base_ptr = (uint32_t*)(layer_params_ptr->picBase);
    /*lint -restore */

    //NO virtual layer is started
    if (count == 0)
    {
        osdhandlerFill(layer_params_ptr,
                       layer_colour_params.bpp,
                       layer_params_ptr->fillColour);
    }
    //ONLY 1 virtual layer is started
    else if (count == 1)
    {
        for(ii = 0; ii< OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
        {
            if(osd_colour_params[ii].bpp)
            {
                // FAPI error codes already checked
                currentErrorCode = 0UL;
                (void)OSDHANDLER_ShowLayer(((groupId & 0x00030000UL)|(ii+1)),
                                           &currentErrorCode);
                if(errorCodePtr != NULL)
                {
                    *errorCodePtr |= currentErrorCode;
                }
                break;
            }
        }
    }
    //More than 1 virtual layers are started
    else
    {
        group_params_ptr = osdhandlerGetOsdGroupParams(groupId);

        for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
        {
            if(osd_colour_params[ii].bpp)
            {
                if(osd_colour_params[ii].bpp >> 5) // bpp >= 32 ?
                {
                    osd_colour_params[ii].colour_mask = 0xFFFFFFFFUL;
                }
                else
                {
                    osd_colour_params[ii].colour_mask = \
                            (0x00000001UL << osd_colour_params[ii].bpp) - 0x00000001UL;
                }
                /*lint -save -e923 */
                osd_colour_params[ii].osd_base_ptr = \
                            (uint32_t*)(group_params_ptr->picBase);
                /*lint -restore */
                // osd_colour_params.pixel_bit_offset is initialized with 0
                osd_colour_params[ii].temp_data = *osd_colour_params[ii].osd_base_ptr++;
            }
            group_params_ptr++;
        }

        if (layer_params_ptr->osdLayout == OSDHANDLER_OSD_LAYOUT_RGBA)
        {
            while (y_size)
            {
                group_params_ptr = osdhandlerGetOsdGroupParams(groupId);

                // get the current pixel data for each active virtual layer...
                for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
                {
                    if(osd_colour_params[ii].bpp)
                    {
                        osd_colour_params[ii].data_mask = osd_colour_params[ii].colour_mask <<
                                (0x20UL - (osd_colour_params[ii].pixel_bit_offset +
                                osd_colour_params[ii].bpp));
                        osd_colour_params[ii].data = (osd_colour_params[ii].temp_data &
                                osd_colour_params[ii].data_mask) >> (0x20UL -
                                (osd_colour_params[ii].pixel_bit_offset + osd_colour_params[ii].bpp));
                        osdhandlerGetRGBAColour(group_params_ptr, osd_colour_params[ii].data,
                                osd_colour_params[ii].bpp, &osd_colour_params[ii].rgb);
                    }
                    group_params_ptr++;
                }

                // Combine all pixel data of the active virtual layers to
                // layer_colour_params.rgb ...
                for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
                {
                    if(osd_colour_params[ii].bpp)
                    {
                        layer_colour_params.rgb.all = osd_colour_params[ii].rgb.all;

                        // TODO: this is NOT the arithmetic mean of all colours... --RK
                        for(jj = ii + 1; jj < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; jj++)
                        {
                            if (osd_colour_params[jj].rgb.all)
                            {
                                osdhandlerCombine2Colour(&layer_colour_params.rgb,
                                                         &osd_colour_params[jj].rgb,
                                                         &layer_colour_params.rgb);
                            }
                        }
                        break;
                    }
                }

                /*Write pixel to hardware OSD memory */
                // 32-bit HW OSD:
                if(layer_colour_params.bpp >> 5)
                {
                    *layer_colour_params.osd_base_ptr++ = (layer_colour_params.rgb.comp.r<<24) |
                                                          (layer_colour_params.rgb.comp.g<<16) |
                                                          (layer_colour_params.rgb.comp.b<<8)  |
                                                           layer_colour_params.rgb.comp.alpha;
                }
                // 16-bit HW OSD:
                else if(layer_colour_params.bpp >> 4)
                {
                    layer_colour_params.rgb.comp.r = layer_colour_params.rgb.comp.r>>3;
                    layer_colour_params.rgb.comp.g = layer_colour_params.rgb.comp.g>>3;
                    layer_colour_params.rgb.comp.b = layer_colour_params.rgb.comp.b>>3;
                    if (layer_colour_params.rgb.comp.alpha >= OSDHANDLER_Alpha_Threshold)
                    {
                        layer_colour_params.rgb.comp.alpha = 0x1;
                    }
                    else
                    {
                        layer_colour_params.rgb.comp.alpha = 0x0;
                    }
                    layer_colour_params.temp_data = (layer_colour_params.rgb.comp.r<<11) |
                                                    (layer_colour_params.rgb.comp.g<<6)  |
                                                    (layer_colour_params.rgb.comp.b<<1)  |
                                                     layer_colour_params.rgb.comp.alpha;
                    layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                    layer_colour_params.data = ((layer_colour_params.data) &
                        (~layer_colour_params.data_mask)) |(layer_colour_params.temp_data <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp)));
                    layer_colour_params.pixel_bit_offset += layer_colour_params.bpp;
                    if (layer_colour_params.pixel_bit_offset >> 5)
                    {
                        *layer_colour_params.osd_base_ptr++ = layer_colour_params.data;
                        layer_colour_params.pixel_bit_offset = 0;
                    }
                }
                // 8-bit or 4-bit HW OSD:
                else
                {
                    layer_colour_params.temp_data =
                        osdhandlerSearchBestColour(
                            layer_colour_params.rgb.all,
                            layer_params_ptr->clut,
                            (uint8_t)((1UL << layer_colour_params.bpp) - 1UL)); // is <256
                    layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                    layer_colour_params.data = ((layer_colour_params.data) &
                        (~layer_colour_params.data_mask)) |(layer_colour_params.temp_data <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp)));
                    layer_colour_params.pixel_bit_offset += layer_colour_params.bpp;
                    if (layer_colour_params.pixel_bit_offset >> 5)
                    {
                        *layer_colour_params.osd_base_ptr++ = layer_colour_params.data;
                        layer_colour_params.pixel_bit_offset = 0;
                    }
                }

                for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
                {
                    if(osd_colour_params[ii].bpp)
                    {
                        osd_colour_params[ii].pixel_bit_offset += osd_colour_params[ii].bpp;
                        if (osd_colour_params[ii].pixel_bit_offset >> 5)
                        {
                            osd_colour_params[ii].temp_data = *osd_colour_params[ii].osd_base_ptr++;
                            osd_colour_params[ii].pixel_bit_offset = 0;
                        }
                    }
                }

                x_pos++;
                if (x_pos >= x_size) // check end of Pixel-Line
                {
                    x_pos = 0;
                    // y_line++;
                    y_size--;
                    if (!y_size)
                    {
                        break;
                    }
                }
            }
        }

        else if(layer_params_ptr->osdLayout == OSDHANDLER_OSD_LAYOUT_ARGB)
        {
            while (y_size)
            {
                group_params_ptr = osdhandlerGetOsdGroupParams(groupId);

                // get the current pixel data for each active virtual layer...
                for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
                {
                    if(osd_colour_params[ii].bpp)
                    {
                        osd_colour_params[ii].data_mask = osd_colour_params[ii].colour_mask <<
                                (0x20UL - (osd_colour_params[ii].pixel_bit_offset +
                                osd_colour_params[ii].bpp));
                        osd_colour_params[ii].data = (osd_colour_params[ii].temp_data &
                                osd_colour_params[ii].data_mask) >> (0x20UL -
                                (osd_colour_params[ii].pixel_bit_offset + osd_colour_params[ii].bpp));
                        osdhandlerGetARGBColour(group_params_ptr, osd_colour_params[ii].data,
                                osd_colour_params[ii].bpp, &osd_colour_params[ii].rgb);
                    }
                    group_params_ptr++;
                }

                // Combine all pixel data of the active virtual layers to
                // layer_colour_params.rgb ...
                for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
                {
                    if(osd_colour_params[ii].bpp)
                    {
                        layer_colour_params.rgb.all = osd_colour_params[ii].rgb.all;

                        // TODO: this is NOT the arithmetic mean of all colours... --RK
                        for(jj = ii + 1; jj < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; jj++)
                        {
                            if (osd_colour_params[jj].rgb.all)
                            {
                                osdhandlerCombine2Colour(&layer_colour_params.rgb,
                                    &osd_colour_params[jj].rgb, &layer_colour_params.rgb);
                            }
                        }
                        break;
                    }
                }

                /*Write pixel to hardware OSD memory */
                // 32-bit HW OSD:
                if(layer_colour_params.bpp >> 5)
                {
                    *layer_colour_params.osd_base_ptr++ = layer_colour_params.rgb.all;
                }
                // 16-bit HW OSD:
                else if(layer_colour_params.bpp >> 4)
                {
                    layer_colour_params.rgb.comp.r = layer_colour_params.rgb.comp.r>>3;
                    layer_colour_params.rgb.comp.g = layer_colour_params.rgb.comp.g>>3;
                    layer_colour_params.rgb.comp.b = layer_colour_params.rgb.comp.b>>3;
                    if (layer_colour_params.rgb.comp.alpha >= OSDHANDLER_Alpha_Threshold)
                    {
                        layer_colour_params.rgb.comp.alpha = 0x1;
                    }
                    else
                    {
                        layer_colour_params.rgb.comp.alpha = 0x0;
                    }
                    layer_colour_params.temp_data = (layer_colour_params.rgb.comp.alpha<<15) |
                                                    (layer_colour_params.rgb.comp.r<<10)     |
                                                    (layer_colour_params.rgb.comp.g<<5)      |
                                                    layer_colour_params.rgb.comp.b;
                    layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                    layer_colour_params.data = ((layer_colour_params.data) &
                        (~layer_colour_params.data_mask)) |(layer_colour_params.temp_data <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp)));
                    layer_colour_params.pixel_bit_offset += layer_colour_params.bpp;
                    if (layer_colour_params.pixel_bit_offset >> 5)
                    {
                        *layer_colour_params.osd_base_ptr++ = layer_colour_params.data;
                        layer_colour_params.pixel_bit_offset = 0;
                    }
                }
                // 8-bit or 4-bit HW OSD:
                else
                {
                    layer_colour_params.temp_data =
                        osdhandlerSearchBestColour(
                            layer_colour_params.rgb.all,
                            layer_params_ptr->clut,
                            (uint8_t)((1UL << layer_colour_params.bpp) - 1UL)); // is <256
                    layer_colour_params.data_mask = layer_colour_params.colour_mask <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp));
                    layer_colour_params.data = ((layer_colour_params.data) &
                        (~layer_colour_params.data_mask)) |(layer_colour_params.temp_data <<
                        (0x20UL - (layer_colour_params.pixel_bit_offset + layer_colour_params.bpp)));
                    layer_colour_params.pixel_bit_offset += layer_colour_params.bpp;
                    if (layer_colour_params.pixel_bit_offset >> 5)
                    {
                        *layer_colour_params.osd_base_ptr++ = layer_colour_params.data;
                        layer_colour_params.pixel_bit_offset = 0;
                    }
                }

                for(ii = 0; ii < OSDHANDLER_MAX_NUM_VIRTUAL_LAYER; ii++)
                {
                    if(osd_colour_params[ii].bpp)
                    {
                        osd_colour_params[ii].pixel_bit_offset += osd_colour_params[ii].bpp;
                        if (osd_colour_params[ii].pixel_bit_offset >> 5)
                        {
                            osd_colour_params[ii].temp_data = *osd_colour_params[ii].osd_base_ptr++;
                            osd_colour_params[ii].pixel_bit_offset = 0;
                        }
                    }
                }

                x_pos++;
                if (x_pos >= x_size) // check end of Pixel-Line
                {
                    x_pos = 0;
                    // y_line++;
                    y_size--;
                    if (!y_size)
                    {
                        break;
                    }
                }
            }
        }
    }
    if(group == 1)
    {
        osdhandlerGroup1Shown     = 1;
    }
    else
    {
        osdhandlerGroup2Shown     = 1;
    }

    return(FAPI_OK);
}

/*!
*****************************************************************************
** \brief   Sets the update area FIFO which indicates the area were the
**          OSD has been updated. The change becomes visible after calling
**          OSDHANDLER_CombineVirtualLayers();
**
** \param   layerId
**              layer identifier
** \param   pos
**              Update area FIFO
**
** \return
**          -   #FAPI_OK on success.
**          -   #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          -   #OSDHANDLER_ERR_BAD_LAYER_ID
**              if the passed layerId is invalid.
**          -   #OSDHANDLER_ERR_NO_OSD_HANDLE
**              if the OSDHANDLER has not allocated the handle of
**              the corresponding hardware OSD at initialization.
**          -   #OSDHANDLER_ERR_OUT_OF_BOUNDS
**              if the passed position is out of bounds of the OSD picture.
**          -   #OSDHANDLER_ERR_VL_FIFO_OVERFLOW
**              if the FIFO has overflowed.
**
*****************************************************************************
*/
int32_t OSDHANDLER_SetUpdateArea (uint32_t layerId, OSDHANDLER_PositionT pos)
{
    OSDHANDLER_OsdCpParamsT*        osd_cp_params_ptr   = NULL;
    OSDHANDLER_PositionT*           group_fifo_ptr      = NULL;
    uint32_t                        group               = 0;
    uint32_t                        ii                  = 0;

    if (!osdhandlerInitialized)
    {
        return(OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    osd_cp_params_ptr = osdhandlerGetOsdCpParams(layerId);
    if(osd_cp_params_ptr == NULL)
    {
        return(OSDHANDLER_ERR_BAD_LAYER_ID);
    }
    if(osd_cp_params_ptr->fd == -1)
    {
        return(OSDHANDLER_ERR_NO_OSD_HANDLE);
    }

    group = layerId >> 16;
    group_fifo_ptr = osdhandlerGetGroupFifo(group << 16);
    if(group_fifo_ptr == NULL)
    {
        return(OSDHANDLER_ERR_BAD_LAYER_ID);
    }
    if(pos.width||pos.height)
    {
        if(((pos.x + pos.width) > osd_cp_params_ptr->scalerParams.inputWindowWidth)||
           ((pos.y + pos.height)> osd_cp_params_ptr->scalerParams.inputWindowHeight))
        {
            return(OSDHANDLER_ERR_OUT_OF_BOUNDS);
        }

        for(ii = 0; ii < OSDHANDLER_MAX_FIFO_ELEMENT; ii++)
        {
            if(!(group_fifo_ptr->width||group_fifo_ptr->height))
            {
                group_fifo_ptr->x = pos.x;
                group_fifo_ptr->y = pos.y;
                group_fifo_ptr->width = pos.width;
                group_fifo_ptr->height = pos.height;

                break;
            }
            group_fifo_ptr++;
            if(ii == (OSDHANDLER_MAX_FIFO_ELEMENT - 1))
            {
                return(OSDHANDLER_ERR_VL_FIFO_OVERFLOW);
            }
        }
    }

    if (group == 1)
    {
        osdhandlerGroup1Updated = 0;
    }
    else if(group == 2)
    {
        osdhandlerGroup2Updated = 0;
    }

    return (FAPI_OK);
}

/*!
*****************************************************************************
** \brief   This function triggers the combination of the previously updated
**          virtual OSD layers by reading the update area FIFO.
**          Before calling this function the update area FIFO has to be set
**          by calling OSDHANLDER_SetUpdateArea().
**          Otherwise OSDHANDLER_CombineVirtualLayers() will return with an
**          error.
**
** \param   groupId
**              is defined by the hardware OSD layer.
**              groupId = hw OSD layer << 16
**
** \return
**          -   #FAPI_OK on success.
**          -   #OSDHANDLER_ERR_NOT_INITIALIZED
**              if the OSDHANDLER is not initialized.
**          -   #OSDHANDLER_ERR_VL_BUSY
**              if the combiner is running.
**          -   #OSDHANDLER_ERR_BAD_LAYER_ID
**              if the passed groupId is invalid.
**          -   #OSDHANDLER_ERR_NO_OSD_HANDLE
**              if the OSDHANDLER has not allocated the handle of
**              the corresponding hardware OSD at initialization.
**          -   #OSDHANDLER_ERR_LAYER_NOT_STARTED
**              if the corresponding hardware layer or the virtual
**              layer group has not been started yet.
**          -   #OSDHANDLER_ERR_VL_ALREADY_UPDATED
**              if the updated area FIFO is empty.
**          -   #OSDHANDLER_ERR_BAD_COLOUR_MODE
**              if the corresponding hardware OSD layer is driven
**              in a bad colour mode.
**
*****************************************************************************
*/
int32_t OSDHANDLER_CombineVirtualLayers (const uint32_t groupId)
{
    OSDHANDLER_OsdCpParamsT*    layer_params_ptr    = NULL;     //underlining hw layer
    OSDHANDLER_PositionT*       group_fifo_ptr      = NULL;
    uint32_t                    ii                  = 0;
    uint32_t                    bpp                 = 0;
    uint32_t                    group               = 0;

    group = groupId >>16;

    if(!osdhandlerInitialized)
    {
        return (OSDHANDLER_ERR_NOT_INITIALIZED);
    }
    /*if the combiner is running.*/
    if(osdhandlerFifoBusy)
    {
        return(OSDHANDLER_ERR_VL_BUSY);
    }
    /* the underlining HW should already be started */
    layer_params_ptr = osdhandlerGetOsdCpParams(group);
    if(layer_params_ptr == NULL)
    {
        return(OSDHANDLER_ERR_BAD_LAYER_ID);
    }
    if(layer_params_ptr->fd == -1)
    {
        return(OSDHANDLER_ERR_NO_OSD_HANDLE);
    }
    if(layer_params_ptr->allocatedMemPtr == NULL)
    {
        return(OSDHANDLER_ERR_LAYER_NOT_STARTED);
    }
    if(group == 1)
    {
        if (!osdhandlerGroup1Started)
        {
            return (OSDHANDLER_ERR_LAYER_NOT_STARTED);
        }
        if (osdhandlerGroup1Updated)
        {
            return (OSDHANDLER_ERR_VL_ALREADY_UPDATED);
        }

        group_fifo_ptr = osdhandlerGetGroupFifo(groupId);
    }
    else if (group == 2)
    {
        if (!osdhandlerGroup2Started)
        {
            return (OSDHANDLER_ERR_LAYER_NOT_STARTED);
        }
        if (osdhandlerGroup2Updated)
        {
            return (OSDHANDLER_ERR_VL_ALREADY_UPDATED);
        }

        group_fifo_ptr = osdhandlerGetGroupFifo(groupId);
    }

    bpp = osdhandlerGetBitsPerPixel(layer_params_ptr);
    if(bpp== 0)
    {
        return(OSDHANDLER_ERR_BAD_COLOUR_MODE);
    }
    if(group_fifo_ptr == NULL)
    {
        return(OSDHANDLER_ERR_BAD_LAYER_ID);
    }
    osdhandlerFifoBusy = 1;
    for(ii = 0; ii < OSDHANDLER_MAX_FIFO_ELEMENT; ii++)
    {
        if((group_fifo_ptr->width)||(group_fifo_ptr->height))
        {
            osdhandlerCombine(groupId, group_fifo_ptr);
            memset(group_fifo_ptr, 0, sizeof(OSDHANDLER_PositionT));
        }
        group_fifo_ptr++;
    }
    osdhandlerFifoBusy = 0;

    if(group == 1)
    {
        osdhandlerGroup1Shown     = 1;
        osdhandlerGroup1Updated   = 1;
    }
    else
    {
        osdhandlerGroup2Shown     = 1;
        osdhandlerGroup1Updated   = 1;
    }
    return (FAPI_OK);
}
