/*!
*******************************************************************************
**
** \file      sfal.h
**
** \brief     Section filter driver supporting circular filtering.
**
**            (C) Fujitsu Semiconductor Europe 2002 - 2007
**
** \attention THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**            ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**            OMMISSIONS.
**
** \version   \$Id: secfilter.h 1652 2009-01-22 10:07:13Z wlaris $
**
******************************************************************************/
#ifndef _SFAL_H_
#define _SFAL_H_

#include "fapex/abst_types.h"
#include "fapex/abst_libtsd.h"

/*!
 *******************************************************************************
 ** Standard error codes for the driver
 *******************************************************************************
*/

/*!< Bad parameter passed. */
#define SUBSYS_SFAL_ERR_BAD_PARAMETER         (SUBSYS_SFAL_MODULE_BASE +\
                                                 FAPI_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define SUBSYS_SFAL_ERR_OUT_OF_MEMORY         (SUBSYS_SFAL_MODULE_BASE +\
                                                  FAPI_ERR_OUT_OF_MEMORY)

/*! Device already initialised. */
#define SUBSYS_SFAL_ERR_ALREADY_INITIALIZED   (SUBSYS_SFAL_MODULE_BASE +\
                                                  FAPI_ERR_ALREADY_INITIALIZED)
/*! Device not initialised. */
#define SUBSYS_SFAL_ERR_NOT_INITIALIZED       (SUBSYS_SFAL_MODULE_BASE +\
                                                  FAPI_ERR_NOT_INITIALIZED)

/*! Feature or function is not available. */
#define SUBSYS_SFAL_ERR_FEATURE_NOT_SUPPORTED (SUBSYS_SFAL_MODULE_BASE +\
                                                 FAPI_ERR_FEATURE_NOT_SUPPORTED)

/*! Timeout occured. */
#define SUBSYS_SFAL_ERR_TIMEOUT               (SUBSYS_SFAL_MODULE_BASE +\
                                                   FAPI_ERR_TIMEOUT)

/*! The device is busy, try again later. */
#define SUBSYS_SFAL_ERR_DEVICE_BUSY           (SUBSYS_SFAL_MODULE_BASE +\
                                                   FAPI_ERR_DEVICE_BUSY)

/*! Invalid handle was passed. */
#define SUBSYS_SFAL_ERR_INVALID_HANDLE        (SUBSYS_SFAL_MODULE_BASE +\
                                                   FAPI_ERR_INVALID_HANDLE)

/*! Semaphore could not be created. */
#define SUBSYS_SFAL_ERR_SEMAPHORE_CREATE      (SUBSYS_SFAL_MODULE_BASE +\
                                                   FAPI_ERR_SEMAPHORE_CREATE)
/*! The driver's used version is not supported. */
#define SUBSYS_SFAL_ERR_UNSUPPORTED_VERSION   (SUBSYS_SFAL_MODULE_BASE +\
                                                  FAPI_ERR_UNSUPPORTED_VERSION)

/*! The mailqueue could not be created. */
#define SUBSYS_SFAL_ERR_MAILQUEUE_CREATE      (SUBSYS_SFAL_MODULE_BASE +\
                                                  FAPI_ERR_MAILQUEUE_CREATE)

/*!
********************************************************************************
** Additional driver error codes (reserved from -100 to -999)
**
** Each error code shall be decreased in steps of 1.
********************************************************************************
*/
/* SFAL error base */
#define SUBSYS_SFAL_ERR_BASE            (SUBSYS_SFAL_MODULE_BASE - 100)
/*! \brief The provided PID is invalid. */
#define SUBSYS_SFAL_ERR_INVALID_PID                (SUBSYS_SFAL_ERR_BASE -  0)
/*! \brief No PID is associated with the provided to filter handle. */
#define SUBSYS_SFAL_ERR_NO_PID_ASSIGNED            (SUBSYS_SFAL_ERR_BASE -  1)
/*! \brief The thread could not be created. */
#define SUBSYS_SFAL_ERR_THREAD_CREATE              (SUBSYS_SFAL_ERR_BASE -  2)
/*! \brief No free channel found to start filter. */
#define SUBSYS_SFAL_ERR_NO_FREE_CHANNEL            (SUBSYS_SFAL_ERR_BASE -  3)
/*! \brief No free filter found. */
#define SUBSYS_SFAL_ERR_NO_FREE_FILTER             (SUBSYS_SFAL_ERR_BASE -  4)


//!< Maximum depth of section bytes.
#define SFAL_MAX_DEPTH            12

//!< Minumum filter depth (tableId + section length).
#define SFAL_FILTERDEPTH_MIN      3

//!< Maximum filter depth
#define SFAL_FILTERDEPTH_MAX      ((SFAL_MAX_DEPTH) + (SFAL_FILTERDEPTH_MIN))

//!< Filter runs till SFAL_StopFilter() is called
#define SFAL_NO_TIMEOUT           0

/*!
** \brief  Loop modes for filtering.
** \sa     SFAL_FilterStart()
*/
typedef enum
{
    /*! \brief Continuous filtering mode.
    **
    **  The filter works continuously.
    **  Only a set timeout could stop it automatically. */
    SFAL_LOOP_CONTINUOUS = 0,

    /*! \brief One shot filtering mode.
    **
    **  After a matching section arrives the
    **  filter will be stopped automatically. */
    SFAL_LOOP_ONE_SHOT
    
} SFAL_LOOP_MODE_E;


/*!
** \brief  CRC check modes.
** \sa     SFAL_FilterStart()
*/
typedef enum
{
    /*! \brief The checksum (CRC) of an arrived section is verified.
    **
    **  If the checksums do not match, the section will be notified. */
    SFAL_CRC_CHECK = 0, 

    /*! \brief The CRC of an arrived section is ignored. */
    SFAL_CRC_IGNORE
    
} SFAL_CRC_MODE_E;

/*!
*******************************************************************************
**
** \brief  Section status used by the section received notification function.
**
** \sa     SFAL_SECTION_RECEIVED_F
**
*******************************************************************************
*/
typedef enum
{
    /*! \brief The section is available.
     *
     *  The CRC is valid if CRC checking was enabled.
     */
    SFAL_SECTION_AVAILABLE = 0,

    /*! \brief A timeout has occurred on this filter and the filter was stopped. */
    SFAL_SECTION_TIMEOUT_ERROR,

    /*! \brief The section was received but with an invalid CRC.*/
    SFAL_SECTION_CRC_ERROR

} SFAL_SECTION_STATUS_E;

/*!
** \brief  Filterset configuration data 
** \sa     SFAL_OpenFilterset()
** \sa     SFAL_CloseFilterset()
*/
typedef struct
{
    /*!
    **  \brief TSD input to be used.
    **
    **  Possible values are FAPI_TSD0, FAPI_TSD1, FAPI_TSD2 or FAPI_TSD3.
    */
    uint8_t  tsdInput;

    /*!
    **  \brief Number of PIDs to filter simultaneously.
    **
    **  Each filter channel is assinged one unique PID.
    */
    uint8_t  numberOfChannels;

    /*!
    **  \brief Number of filters per channel.
    **
    **  This is also the maximum number of filtered TIDs per PID channel.
    **  Possibles values are 1, 2, 4, 8, 16 or 32.
    */
    int32_t  filtersPerChannel;

    /*!
    **  \brief Number of filter bytes per filter.
    **
    **  It includes the TID and the two section length bytes.
    **  For example, set this value to 7 if 4 section bytes shall be filtered.
    */
    uint8_t  filterDepth;

#if 0 /* FAPex */
    /*!
    **  \brief Address of the first channel buffer.
    **
    **  To each PID channel a memory buffer has to assigned.
    **  This address shows the driver where to locate the first of these buffers.
    **  All buffers have the same size.
    */
    uint32_t bufferAddress;
#endif
    /*!
    **  \brief Size of one channel buffer.
    */
    uint32_t bufferSize;

} SFAL_FILTERSET_S;


/*!
*******************************************************************************
**
** \brief  Notify function for requesting a buffer.
**
** \note   A section is copied to an internal buffer anyway.
**         So, providing another buffer by function does not improve the
**         performance of copying the section.
**
** \param  handle          Handle of filter requesting a buffer.
** \param  ppBuffer        Pointer to buffer pointer to be set.
** \param  pBufferSize     Pointer to buffer size to be set.
** \param  pPrivateData    Private data pointer set with SFAL_FilterStart().
**
** \sa     SFAL_FilterStart()
**
******************************************************************************/
typedef void (*SFAL_QUERY_BUFFER_F)( FAPI_SYS_HandleT handle,
                                   uint8_t**        ppBuffer,
                                   uint16_t*        pBufferSize,
                                   void*            pPrivateData);

/*!
*******************************************************************************
**
** \brief  Notify function for informing about a filtered section.
**
** \code
**  pSectionBufferStart ---> --------------------------------
**                           | ############################ |
**                           | ############################ |
**                           |                              |
**                           |                              |
**                           |                              |
**  pSectionBuffer      ---> | ############################ |
**                           | ############################<=== section data
**                           | ############################ |
**  pSectionBufferEnd   ---> --------------------------------
**
** \endcode
**
** \param  handle          Handle of filter.
** \param  pSectionBufferStart  Pointer to beginning of the BM-buffer.
** \param  pSectionBufferEnd    Pointer to end of the BM-buffer.
** \param  pSectionBuffer  Pointer to buffer containing the section.
** \param  sectionSize     Section size in bytes.
** \param  pPrivateData    Private data pointer set with SFAL_FilterStart().
**
** \return 1 to keep section, 0 to discard it.
** 
** \sa     SFAL_FilterStart()
**
******************************************************************************/
typedef uint32_t (*SFAL_SECTION_FILTERED_F)(FAPI_SYS_HandleT        handle,
                                    const volatile uint8_t* pSectionBufferStart,
                                    const volatile uint8_t* pSectionBufferEnd,
                                         const volatile uint8_t* pSectionBuffer,
                                         uint16_t                sectionSize,
                                         void*                   pPrivateData);

/*!
*******************************************************************************
**
** \brief  Notify function for informing about a received section.
**
** \param  handle          Handle of filter requesting a buffer.
** \param  sectionStatus   Status of the provided section.
** \param  pSectionBuffer  Pointer to buffer containing the section.
** \param  sectionSize     Section size in bytes.
** \param  pPrivateData    Private data pointer set with SFAL_FilterStart().
**
** \sa     SFAL_FilterStart()
**
******************************************************************************/
typedef void (*SFAL_SECTION_RECEIVED_F)(FAPI_SYS_HandleT handle,
                                      SFAL_SECTION_STATUS_E sectionStatus,
                                      const uint8_t*   pSectionBuffer,
                                      uint16_t         sectionSize,
                                      void*            pPrivateData);

/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif

int32_t SFAL_Init                     (void);
void SFAL_Exit                          (void);
int32_t SFAL_OpenFilterset            (SFAL_FILTERSET_S* pFiltersetConfig,
                                     FAPI_SYS_HandleT* pFilterset);
void SFAL_CloseFilterset            (FAPI_SYS_HandleT filterset);
int32_t SFAL_OpenFilter               (FAPI_SYS_HandleT filterset,
                                     uint8_t depth,
                                     FAPI_SYS_HandleT* pFilter);
void SFAL_CloseFilter                (FAPI_SYS_HandleT filter);
int32_t SFAL_SetFilterPatterns        (FAPI_SYS_HandleT filter,
                                     const uint8_t*  pMatch,
                                     const uint8_t*  pEqualMask,
                                     const uint8_t*  pNotEqualMask);
int32_t SFAL_ClearFilterPatterns     (FAPI_SYS_HandleT filter);
int32_t SFAL_SetFilterPid             (FAPI_SYS_HandleT filter,
                                     uint16_t pid);
int32_t SFAL_SetSectionFilteredNotify (FAPI_SYS_HandleT filter,
                                     SFAL_SECTION_FILTERED_F sectionFilteredNotify);
int32_t SFAL_StartFilter              (FAPI_SYS_HandleT      filter,
                                     uint32_t                timeout,
                                     SFAL_LOOP_MODE_E        loopMode,
                                     SFAL_CRC_MODE_E         crcMode,
                                     SFAL_QUERY_BUFFER_F     queryBufferNotify,
                                     SFAL_SECTION_RECEIVED_F sectionReceivedNotify,
                                     const void*           pPrivateData);
int32_t SFAL_StopFilter               (FAPI_SYS_HandleT filter);
void SFAL_ThreadFunction             (void);
#ifdef __cplusplus
}
#endif

#endif /* _SFAL_H_ */
