/****************************************************************************
 * MODULE     : DsubTDecoderTypes
 * FILE       : DsubTDecoderTypes.c
 * COPYRIGHT  : TARA Systems GmbH
 * DESCRIPTION:
 *   This module contains the all types needed to access the DVB subtitle 
 *   decoder.
 ****************************************************************************/
 
 
#ifndef DSUBTDECODERTYPES_H
#define DSUBTDECODERTYPES_H


#include <subtitle/DsubStddef.h>


/****************************************************************************
 * DEFINE: DSUB_DECODER_MIN_MEMORY_SIZE_VERSION_1_2_1
 *
 * DESCRIPTION:
 *   DSUB_DECODER_MIN_MEMORY_SIZE_VERSION_1_2_1 defines the minimal memory
 *   requirements of the DVB subtitle decoder, if the decoder works according
 *   to the norm Version 1.2.1. When a decoder is created with this size, a
 *   DecoderHandle will returned by the DsubDecoderCreate() function, but the
 *   decoder will not decode any subtitle regions. See chapter 'Resource
 *   usage' of DvbSubtitleDecoderRefManual.pdf for details.
 ****************************************************************************/

#define DSUB_DECODER_MIN_MEMORY_SIZE_VERSION_1_2_1   ( 155 * 1024UL )


/****************************************************************************
 * DEFINE: DSUB_DECODER_MIN_MEMORY_SIZE_VERSION_1_3_1
 *
 * DESCRIPTION:
 *   DSUB_DECODER_MIN_MEMORY_SIZE_VERSION_1_3_1 defines the minimal memory
 *   requirements of the DVB subtitle decoder, if the decoder works according
 *   to the norm Version 1.3.1. When a decoder is created with this size, a
 *   DecoderHandle will returned by the DsubDecoderCreate() function, but the
 *   decoder will not decode any subtitle regions. See chapter 'Resource
 *   usage' of DvbSubtitleDecoderRefManual.pdf for details.
 ****************************************************************************/

#define DSUB_DECODER_MIN_MEMORY_SIZE_VERSION_1_3_1   ( 170 * 1024UL )


/****************************************************************************
 * TYPE: DsubTNormVersion
 *
 * DESCRIPTION:
 *   This type defines the various supported version of the norm ETSI EN
 *   300 743.
 *
 * COMPONENTS:
 *   DSUB_DECODER_NORM_VERSION_1_2_1 -
 *     If a decoder is created passing this version, the following
 *     constraints are given:
 *
 *     The norm V1.2.1 is supported. Streams containing
 *     display_definition_segments will be ignored.
 *
 *   DSUB_DECODER_NORM_VERSION_1_3_1 -
 *     If a decoder is created passing this version:
 *
 *     The norm V1.3.1 is supported.
 ****************************************************************************/

typedef enum
{
  DSUB_DECODER_NORM_VERSION_1_2_1 = 1,
  DSUB_DECODER_NORM_VERSION_1_3_1
  
} DsubTNormVersion;


/***************************************************************************
 * TYPE: DsubTStcComposition
 *
 * DESCRIPTION:
 *    This type defines the composition of the STC. The STC is normally 
 *    derived from the PCR. The pcr_base is built-up of 33 bits. The 
 *    decoder only uses 32 bits to compare the STC to the PTS. This type
 *    is used to define which bits of the STC are passed to the Decoder.
 *
 * COMPONENTS:
 *    DSUB_STC_COMPOSITION_HIGH_BITS - The highest 32 bits of pcr_base 
 *                                     are used.
 *    DSUB_STC_COMPOSITION_LOW_BITS  - The lowest 32 bits of pcr_base 
 *                                     are used.
 *****************************************************************************/

typedef enum
{
  DSUB_STC_COMPOSITION_HIGH_BITS,
  DSUB_STC_COMPOSITION_LOW_BITS
  
} DsubTStcComposition;


/****************************************************************************
 * CALLBACK: DsubTGetTime
 *
 * DESCRIPTION:
 *   An implementation of DsubTGetTime must return the current value of the
 *   platform's clock in seconds. It must be implemented by the user. The
 *   callback is called by the decoder to check if a timeout has occurred and
 *   the currently displayed subtitle page has to be removed from the OSD.
 *
 * PARAMETERS:
 *   aUserData - The user data pointer which was passed to
 *               DsubDecoderCreate().
 *
 * RETURNS:
 *   Current value of the platform's clock in seconds.
 *
 * SEE ALSO:
 *   DsubDecoderCreate
 ****************************************************************************/

typedef
unsigned long ( * DsubTGetTime ) ( 
  void * aUserData );


/****************************************************************************
 * CALLBACK: DsubTGetStcTime
 *
 * DESCRIPTION:
 *   A function of type DsubTGetStcTime must be implemented by the user of
 *   the DsubDecoder. It must return the current value of the System Time
 *   Clock (STC).
 *
 *   The System Time Clock is calculated using the
 *   program_clock_reference_base value found in the adaption_field of a
 *   Transport Stream (TS) packet. The decoder only compares 32-bits of the
 *   33-bit pcr_base value. Which bits the decoder will compare defines a
 *   parameter of DsubDecoderSetStcCallback().
 *
 *   When this function returns 0, the subtitles are decoded instantly.
 *
 *   NOTE: The implementation of this function is recommended but optional.
 *
 * PARAMETERS:
 *   aUserData - The user data pointer which was passed to
 *               DsubDecoderSetStcCallback().
 *
 * RETURNS:
 *   Current value of the STC.
 *
 * SEE ALSO:
 *   DsubDecoderSetStcCallback
 *   DsubTStcComposition
 ****************************************************************************/

typedef
unsigned long ( * DsubTGetStcTime ) ( 
  void * aUserData );


/****************************************************************************
 * TYPE: DsubTDecoderDisplayClip
 *
 * DESCRIPTION:
 *   This type contains various possibilities of the decoder which are used
 *   when the incomming subtitles do not fit onto the screen.
 *
 * COMPONENTS:
 *   DSUB_DECODER_CLIP_DISCARD -
 *     Subtitles will be discarded.
 *
 *   DSUB_DECODER_CLIP_CLIP -
 *     Subtitles will be clipped.
 *
 *   DSUB_DECODER_CLIP_NO_CLIP -
 *     Subtitles will not be clipped. They are returned exactly as they are
 *     decoded.
 ****************************************************************************/

typedef enum
{
  DSUB_DECODER_CLIP_DISCARD = 1,
  DSUB_DECODER_CLIP_CLIP,
  DSUB_DECODER_CLIP_NO_CLIP
  
} DsubTDecoderDisplayClip;


/****************************************************************************
 * TYPE: DsubTDecoderAlignH
 *
 * DESCRIPTION:
 *   The horizontal alignment of the subtitles on the screen.
 *
 * COMPONENTS:
 *   DSUB_DECODER_ALIGN_H_LEFT -
 *     Subtitles will be positioned on the left.
 *
 *   DSUB_DECODER_ALIGN_H_MIDDLE -
 *     Subtitles will bepositioned in the middle.
 *
 *   DSUB_DECODER_ALIGN_H_RIGHT -
 *     Subtitles will bepositioned on the right.
 ****************************************************************************/

typedef enum
{
  DSUB_DECODER_ALIGN_H_LEFT = 1,
  DSUB_DECODER_ALIGN_H_MIDDLE,
  DSUB_DECODER_ALIGN_H_RIGHT

} DsubTDecoderAlignH;


/****************************************************************************
 * TYPE: DsubTDecoderAlignV
 *
 * DESCRIPTION:
 *   The vertical alignment of the subtitles on the screen.
 *
 * COMPONENTS:
 *   DSUB_DECODER_ALIGN_V_TOP -
 *     Subtitles will be positioned on top.
 *
 *   DSUB_DECODER_ALIGN_V_CENTER -
 *     Subtitles will be positioned in the center.
 *
 *   DSUB_DECODER_ALIGN_V_BOTTOM -
 *     Subtitles will be positioned on bottom.
 ****************************************************************************/

typedef enum
{
  DSUB_DECODER_ALIGN_V_TOP = 1,
  DSUB_DECODER_ALIGN_V_CENTER,
  DSUB_DECODER_ALIGN_V_BOTTOM

} DsubTDecoderAlignV;


/****************************************************************************
 * TYPE: DsubTDecoderInfo
 *
 * DESCRIPTION:
 *   Used to retrieve information about the subtitle decoder.
 *
 *   When DsubDecoderReset is called also the DsubTDecoderInfo will be reset.
 *   This does not affect the MemoryTotal/Used/Free info, the
 *   SelectedCompositionPageId info and the SelectedAncillaryPageId info.
 *
 * COMPONENTS:
 *   MemoryTotal -
 *     The number of bytes which are managed by the decoder.
 *
 *   MemoryUsed -
 *     The number of bytes currently in use.
 *
 *   MemoryFree -
 *     The number of free bytes.
 *
 *   PesPacketsProcessed -
 *     The number of processed PES packets.
 *
 *   PesPacketsDiscarded -
 *     The number of discarded PES packets.
 *
 *   PesPacketsSizeMin -
 *     The minimal PES packet size.
 *
 *   PesPacketsSizeAverage -
 *     The average PES packet size.
 *
 *   PesPacketsSizeMax -
 *     The maximal PES packets size.
 *
 *   PesQueuePacketCount -
 *     The number of packets hold in the PES queue.
 *
 *   PesQueueOverflows -
 *     Number of PES queue overflows.
 *
 *   PagesProcessed -
 *     The number of processed pages (successful and unsuccessful).
 *
 *   PagesDiscarded -
 *     The number of discarded pages.
 *
 *   RegionsProcessed -
 *     The number of totally processed regions (successful and unsuccessful).
 *
 *   RegionsDiscarded -
 *     The number of discarded regions.
 *
 *   RegionsVisible -
 *     The number of currently visible regions.
 *
 *   ClutsProcessed -
 *     The number of totally processed CLUTS (successful and unsuccessful).
 *
 *   ClutsDiscarded -
 *     The number of discarded CLUTS.
 *
 *   ObjectsProcessed -
 *     The number of totally processed objects (successful and unsuccessful).
 *
 *   ObjectsDiscarded -
 *     The number of discarded objects.
 *
 *   InternalResets -
 *     The number of internal decoder resets.
 *
 *   SelectedCompositionPageId -
 *     The currently selected composition page ID.
 *
 *   SelectedAncillaryPageId -
 *     The currently selected ancillary page ID.
 ****************************************************************************/

typedef struct
{
  unsigned long MemoryTotal;
  unsigned long MemoryUsed;
  unsigned long MemoryFree;

  unsigned long PesPacketsProcessed;
  unsigned long PesPacketsDiscarded;
  unsigned long PesPacketsSizeMin;
  unsigned long PesPacketsSizeAverage;
  unsigned long PesPacketsSizeMax;

  unsigned long PesQueuePacketCount;
  unsigned long PesQueueOverflows;

  unsigned long PagesProcessed;
  unsigned long PagesDiscarded;

  unsigned long RegionsProcessed;
  unsigned long RegionsDiscarded;
  unsigned long RegionsVisible;

  unsigned long ClutsProcessed;
  unsigned long ClutsDiscarded;

  unsigned long ObjectsProcessed;
  unsigned long ObjectsDiscarded;

  unsigned long InternalResets;

  unsigned long SelectedCompositionPageId ;
  unsigned long SelectedAncillaryPageId;
  
} DsubTDecoderInfo;


/****************************************************************************
 * TYPE: DsubTLogId
 *
 * DESCRIPTION:
 *   Enumeration type for all supported log ids. The log ids are used in the
 *   log handler for the parameter aLogId. The log ids are separated into
 *   different log levels (error, warning and info). Depending on the platform
 *   only a subset of all log ids is supported.
 *
 * SEE ALSO:
 *   DsubDecoderSetLogHandler()
 *   DsubDecoderEnableLog()
 *   DsubDecoderDisableLog()
 *   DsubDecoderLogHandler
 ****************************************************************************/

typedef enum
{
  /* errors */
  DSUB_ERROR_FIRST                                     = 0x1000,
  
  DSUB_ERROR_ALL                                       = 0x1000,
  DSUB_ERROR_ILLEGAL_LOG_ID                            = 0x1001,
  DSUB_ERROR_UNSUPPORTED_LOG_ID                        = 0x1002,
  DSUB_ERROR_TOO_LESS_MEMORY                           = 0x1003,
  DSUB_ERROR_PARAMETER                                 = 0x1004,
  DSUB_ERROR_PARAMETER_NULL                            = 0x1005,
  DSUB_ERROR_PARAMETER_UINT                            = 0x1006,
  DSUB_ERROR_PARAMETER_CHAR                            = 0x1007,
  
  DSUB_ERROR_SUBDEC_END_OF_PES                         = 0x1008,
  DSUB_ERROR_SUBDEC_BAD_SYNC_BYTE                      = 0x1009,
  DSUB_ERROR_SUBDEC_BAD_INPUT_CONTINUE                 = 0x100A,
  DSUB_ERROR_SUBDEC_BAD_SEGMENT_TYPE                   = 0x100B,
  
  DSUB_ERROR_PESDEC_NO_OPTIONAL_HEADER                 = 0x100C,
  DSUB_ERROR_PESDEC_HEADER_TO_LONG                     = 0x100D,
  DSUB_ERROR_PESDEC_HEADER_TO_SHORT                    = 0x100E,
  DSUB_ERROR_PESDEC_INVALID_MARKER_BITS_IN_PTS         = 0x100F,
  DSUB_ERROR_PESDEC_BAD_DATA_IDENTIFIER                = 0x1010,
  DSUB_ERROR_PESDEC_BAD_SUBTITLE_STREAM_ID             = 0x1011,

  DSUB_ERROR_REGDEC_REGION_INVALID                     = 0x1012,
  DSUB_ERROR_REGDEC_REGION_SIZE_INVALID                = 0x1013,

  DSUB_ERROR_OBJDEC_SEGMENT_LENGTH_TOO_SHORT           = 0x1014,
  DSUB_ERROR_OBJDEC_UNSUPPORTED_CODING_METHOD_STRING   = 0x1015,
  DSUB_ERROR_OBJDEC_BAD_DATA_TYPE                      = 0x1016,
  DSUB_ERROR_OBJDEC_BAD_END_OF_PIXEL_CODE_STRING       = 0x1017,
  DSUB_ERROR_OBJDEC_BAD_PIXEL_CODE_STRING              = 0x1018,

  DSUB_ERROR_REGCOMP_BAD_CLUT_ENTRY_INDEX              = 0x1019,
  DSUB_ERROR_REGCOMP_BAD_BACKGROUND_COLOR              = 0x101A,
  DSUB_ERROR_REGCOMP_BITMAP_INTERFACE_CREATE           = 0x101B,
  
  DSUB_ERROR_DDSDEC_ILLEGAL_DISPLAY_SIZE               = 0x101C,
  DSUB_ERROR_DDSDEC_ILLEGAL_WINDOW                     = 0x101D,
  
  DSUB_ERROR_LAST                                      = 0x101D,


  /* warnings */
  DSUB_WARNING_FIRST                                   = 0x2000,

  DSUB_WARNING_ALL                                     = 0x2000,
  
  DSUB_WARNING_SUBDEC_RESET                            = 0x2001,

  DSUB_WARNING_SUBDEC_WRONG_PAGE_ID                    = 0x2002,

  DSUB_WARNING_PESDEC_WRONG_LENGTH                     = 0x2003,

  DSUB_WARNING_REGDEC_REGION_VALUES_INVALID            = 0x2004,
  DSUB_WARNING_REGDEC_REGION_IGNORED                   = 0x2005,
  DSUB_WARNING_REGDEC_OBJECT_IGNORED                   = 0x2006,

  DSUB_WARNING_CLUTDEC_CLUT_ENTRY_IGNORED              = 0x2007,
  
  DSUB_WARNING_DISPLAY_SET_TOO_MANY_REGIONS            = 0x2008,
  DSUB_WARNING_DISPLAY_SET_TOO_MANY_CLUTS              = 0x2009,
  DSUB_WARNING_DISPLAY_SET_TOO_MANY_OBJECTS            = 0x200A,
  
  DSUB_WARNING_PAGEDEC_ILLEGAL_SEGMENT_LENGTH          = 0x200B,
  
  DSUB_WARNING_REGDEC_BITMAP_INTERFACE_MAX_MEM         = 0x200C,
  DSUB_WARNING_NO_CLIP_BUT_DISCARD                     = 0x200D,
  
  DSUB_WARNING_LAST                                    = 0x200D,


  /* infos */
  DSUB_INFO_FIRST                                      = 0x3000,
  
  DSUB_INFO_ALL                                        = 0x3000,
  
  DSUB_INFO_SUBDEC_NEW_PAGE_IDS                        = 0x3001,
  DSUB_INFO_SUBDEC_SEGMENT_SYNC_BYTE                   = 0x3002,
  DSUB_INFO_SUBDEC_SEGMENT_TYPE                        = 0x3003,
  DSUB_INFO_SUBDEC_PAGE_ID                             = 0x3004,
  DSUB_INFO_SUBDEC_DRAW_REGION                         = 0x3005,

  DSUB_INFO_PESDEC_START_CODE_PREFIX_1                 = 0x3006,
  DSUB_INFO_PESDEC_START_CODE_PREFIX_2                 = 0x3007,
  DSUB_INFO_PESDEC_START_CODE_PREFIX_3                 = 0x3008,
  DSUB_INFO_PESDEC_STREAM_ID                           = 0x3009,
  DSUB_INFO_PESDEC_PACKET_LENGTH                       = 0x300A,
  DSUB_INFO_PESDEC_HEADER_DATA_LENGTH                  = 0x300B,
  DSUB_INFO_PESDEC_DATA_IDENTIFIER                     = 0x300C,
  DSUB_INFO_PESDEC_SUBTITLE_STREAM_ID                  = 0x300D,
  DSUB_INFO_PESDEC_DECODED_PTS                         = 0x300E,
  DSUB_INFO_PESDEC_SCRAMBLING_CONTROL                  = 0x300F,
  DSUB_INFO_PESDEC_EXTENSION_FLAGS                     = 0x3010,
  DSUB_INFO_PESDEC_NO_PTS_AVAILABLE                    = 0x3011,
  DSUB_INFO_PESDEC_REST_IGNORED                        = 0x3012,

  DSUB_INFO_PAGEDEC_SEGMENT_LENGTH                     = 0x3013,
  DSUB_INFO_PAGEDEC_PAGE_TIMEOUT                       = 0x3014,
  DSUB_INFO_PAGEDEC_PAGE_VERSION                       = 0x3015,
  DSUB_INFO_PAGEDEC_PAGE_STATE                         = 0x3016,
  DSUB_INFO_PAGEDEC_REGION_IN_PAGE                     = 0x3017,
  DSUB_INFO_PAGEDEC_PES_DISCARDED_NORMAL_CASE          = 0x3018,

  DSUB_INFO_REGDEC_SEGMENT_LENGTH                      = 0x3019,
  DSUB_INFO_REGDEC_REGION_ID                           = 0x301A,
  DSUB_INFO_REGDEC_REGION_VERSION                      = 0x301B,
  DSUB_INFO_REGDEC_REGION_FILL_FLAG                    = 0x301C,
  DSUB_INFO_REGDEC_REGION_LEVEL_OF_COMPATIBILITY       = 0x301D,
  DSUB_INFO_REGDEC_REGION_DEPTH                        = 0x301E,
  DSUB_INFO_REGDEC_REGION_8_BIT_PIXEL_CODE             = 0x301F,
  DSUB_INFO_REGDEC_REGION_4_BIT_PIXEL_CODE             = 0x3020,
  DSUB_INFO_REGDEC_REGION_2_BIT_PIXEL_CODE             = 0x3021,
  DSUB_INFO_REGDEC_REGION_WIDTH                        = 0x3022,
  DSUB_INFO_REGDEC_REGION_HEIGHT                       = 0x3023,
  DSUB_INFO_REGDEC_REGION_CLUT_ID                      = 0x3024,
  DSUB_INFO_REGDEC_REGION_OBJECT_ID                    = 0x3025,
  DSUB_INFO_REGDEC_REGION_OBJECT_TYPE                  = 0x3026,
  DSUB_INFO_REGDEC_REGION_OBJECT_PROVIDER              = 0x3027,
  DSUB_INFO_REGDEC_REGION_OBJECT_HOR_POS               = 0x3028,
  DSUB_INFO_REGDEC_REGION_OBJECT_VER_POS               = 0x3029,

  DSUB_INFO_CLUTDEC_SEGMENT_LENGTH                     = 0x302A,
  DSUB_INFO_CLUTDEC_CLUT_ID                            = 0x302B,
  DSUB_INFO_CLUTDEC_CLUT_VERSION_NUMBER                = 0x302C,
  DSUB_INFO_CLUTDEC_CLUT_ENTRY_ID                      = 0x302D,
  DSUB_INFO_CLUTDEC_CLUT_FLAGS                         = 0x302E,
  DSUB_INFO_CLUTDEC_CLUT_FULL_RANGE_FLAG               = 0x302F,
  DSUB_INFO_CLUTDEC_DECODED_N_BIT_ENTRY                = 0x3030,
  DSUB_INFO_CLUTDEC_Y_EQUALS_ZERO_SET_TO               = 0x3031,
  DSUB_INFO_CLUTDEC_Y_CR_CB_T_VALUES                   = 0x3032,
  DSUB_INFO_CLUTDEC_R_G_B_A_VALUES                     = 0x3033,

  DSUB_INFO_OBJDEC_DECODING_TOP_FIELD                  = 0x3034,
  DSUB_INFO_OBJDEC_DECODING_BOTTOM_FIELD               = 0x3035,
  DSUB_INFO_OBJDEC_SKIPPING_BYTES                      = 0x3036,
  DSUB_INFO_OBJDEC_PROCESSED_BYTES                     = 0x3037,
  DSUB_INFO_OBJDEC_SEGMENT_LENGTH                      = 0x3038,
  DSUB_INFO_OBJDEC_OBJECT_ID                           = 0x3039,
  DSUB_INFO_OBJDEC_FLAGS                               = 0x303A,
  DSUB_INFO_OBJDEC_CODING_METHOD                       = 0x303B,
  DSUB_INFO_OBJDEC_NON_MODIFYING_COLOR_FLAG            = 0x303C,
  DSUB_INFO_OBJDEC_TOP_FIELD_DATA_BLOCK_LENGTH         = 0x303D,
  DSUB_INFO_OBJDEC_BOTTOM_FIELD_DATA_BLOCK_LENGTH      = 0x303E,
  DSUB_INFO_OBJDEC_SUBBLOCK_DATA_TYPE                  = 0x303F,
  DSUB_INFO_OBJDEC_2_TO_4_BIT_MAP_TABLE                = 0x3040,
  DSUB_INFO_OBJDEC_2_TO_8_BIT_MAP_TABLE                = 0x3041,
  DSUB_INFO_OBJDEC_4_TO_8_BIT_MAP_TABLE                = 0x3042,
  DSUB_INFO_OBJDEC_2_BIT_RUN_LENGTH                    = 0x3043,
  DSUB_INFO_OBJDEC_4_BIT_RUN_LENGTH                    = 0x3044,
  DSUB_INFO_OBJDEC_8_BIT_RUN_LENGTH                    = 0x3045,

  DSUB_INFO_REGCOMP_FILL_REGION                        = 0x3046,
  DSUB_INFO_REGCOMP_WRITE_PIXELS                       = 0x3047,
  DSUB_INFO_REGCOMP_INVALIDATE_RECT                    = 0x3048,
  
  DSUB_INFO_PAGEDEC_PAGE_VERSION_EQUAL                 = 0x3049,
  DSUB_INFO_CLUTDEC_HDTV_VALUES                        = 0x304A,
  
  DSUB_INFO_DDSDEC_SEGMENT_LENGTH                      = 0x304B,
  DSUB_INFO_DDSDEC_DDS_VERSION                         = 0x304C,
  DSUB_INFO_DDSDEC_DDS_WINDOW_FLAG                     = 0x304D,
  DSUB_INFO_DDSDEC_DDS_DISPLAY_WIDTH                   = 0x304E,
  DSUB_INFO_DDSDEC_DDS_DISPLAY_HEIGHT                  = 0x304F,
  DSUB_INFO_DDSDEC_DDS_WINDOW_H_MIN                    = 0x3050,
  DSUB_INFO_DDSDEC_DDS_WINDOW_H_MAX                    = 0x3051,
  DSUB_INFO_DDSDEC_DDS_WINDOW_V_MIN                    = 0x3052,
  DSUB_INFO_DDSDEC_DDS_WINDOW_V_MAX                    = 0x3053,
  
  DSUB_INFO_PAGE_TIMEOUT_EXPIRED                       = 0x3054,
  
  DSUB_INFO_LAST                                       = 0x3054

} DsubTLogId;


/****************************************************************************
 * TYPE: DsubTLogLevel
 *
 * DESCRIPTION:
 *   Defines the different log levels (fatal, error, warning, info). The
 *   log level is passed to the log handler as parameter aLogLevel.
 *
 * SEE ALSO:
 *   DsubDecoderSetLogHandler()
 ****************************************************************************/

typedef enum
{
  DSUB_LOG_LEVEL_UNDEFINED,
  DSUB_LOG_LEVEL_FATAL,
  DSUB_LOG_LEVEL_ERROR,
  DSUB_LOG_LEVEL_WARNING,
  DSUB_LOG_LEVEL_INFO
  
} DsubTLogLevel;


/****************************************************************************
 * CALLBACK: DsubTLogHandler
 *
 * DESCRIPTION:
 *   DsubTLogHandler defines the function type for the log handler. A log
 *   handler can be registered with the function DsubDecoderSetLogHandler().
 *   The called handler is given a pointer to a user defined data area that
 *   is initialized during the registration and passed unchanged to the log
 *   handler callback function, the name of the function that caused the log,
 *   a log id, a log level (error, warning, info) and a log description.
 *
 * PARAMETERS:
 *   aUserData     - A pointer to a user defined area.
 *   aFunctionName - The name of the function that initiated the log.
 *   aLogId        - The log id.
 *   aLogLevel     - The log level (fatal, error, warning, info, debug)
 *   aLogText      - A textual description of the log.
 *
 * RETURN:
 *   Currently not used. Must be set to 0.
 *
 * SEE ALSO:
 *   DsubDecoderSetLogHandler()
 ****************************************************************************/

typedef int
( * DsubTLogHandler )(
  void *                 aUserData,
  const char *           aFunctionName,
  DsubTLogId             aLogId,
  DsubTLogLevel          aLogLevel,
  const char *           aLogText
);



#endif /* DSUBTDECODERTYPES_H */



/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
