/****************************************************************************
 * MODULE     : TT_Core
 * FILE       : TT_Core.h
 * COPYRIGHT  : TARA Systems GmbH
 * DESCRIPTION: 
 *   The mudole TT_Core is the root module to initialize all modules of the
 *   Teletext core library.
 *
 ****************************************************************************/

#ifndef TT_CORE_H
#define TT_CORE_H


/****************************************************************************
 * SECTION: #include
 ****************************************************************************/

#include "teletext/TT_Stdinc.h"


/****************************************************************************
 * SECTION: typedef
 ****************************************************************************/


/****************************************************************************
 * TYPE: TT_DECODING_MODE
 *
 * DESCRIPTION:
 *   Decoding mode used by the SPC.
 *
 * COMPONENTS: 
 *   TT_DECODING_MODE_CLOCK -
 *     Clock only
 *
 *   TT_DECODING_MODE_HEADER_CLOCK -
 *     Header and clock
 *
 *   TT_DECODING_MODE_HEADER -
 *     Header only
 *
 *   TT_DECODING_MODE_PAGE -
 *     whole page
 ****************************************************************************/
typedef enum
{
  /* 
  process clock only
  display at column 32
  starting attributes : black & white (black or other background color is SPT parameter)
  process level 1.5 only
  */
  TT_DECODING_MODE_CLOCK,

  /* 
  process clock only
  display at column 32
  starting attributes : attributes from last header at clock position
  process level 1.5 only
  */
  TT_DECODING_MODE_HEADER_CLOCK,

  /* 
  process header only
  process level 1.5 or 2.5 (SPT parameter)
  if the header isn't found, fall back to "TT_SPT_Processing_Mode_Header_Clock"
  */
  TT_DECODING_MODE_HEADER,

  /* 
  process complete page
  process level 1.5 or 2.5 (SPT parameter)
  if the page isn't found, fall back to "TT_SPT_Processing_Mode_Header_Clock" 
  */
  TT_DECODING_MODE_PAGE

} TT_DECODING_MODE;


/****************************************************************************
 * TYPE: TT_DECODING_LEVEL
 *
 * DESCRIPTION:
 *   Decoding level used by the SPC.
 *
 * COMPONENTS: 
 *   TT_DECODING_LEVEL_15 -
 *     Teletext Level 1.5
 *
 *   TT_DECODING_LEVEL_25 -
 *     Teletext Level 2.5
 *
 *   TT_DECODING_LEVEL_25_SIDEPANEL -
 *     Teletext Level 2.5 with sidepanels
 *
 *   TT_DECODING_LEVEL_10 -
 *     Teletext Level 1.0
 ****************************************************************************/
typedef enum
{
  /*
  level 1.5
  */
  TT_DECODING_LEVEL_15,

  /*
  level 2.5 without sidepanel
  */
  TT_DECODING_LEVEL_25,

  /*
  level 2.5 with sidepanel
  */
  TT_DECODING_LEVEL_25_SIDEPANEL,

  /*
  level 1.0
  */
  TT_DECODING_LEVEL_10

} TT_DECODING_LEVEL;

/****************************************************************************
 * TYPE: TT_CORE_DECODING_PARAMS
 *
 * DESCRIPTION:
 *   All decoding parameters used by the SPC.
 *
 * COMPONENTS:
 *   MagPage -
 *     Teletext page number
 *   
 *   SubPage -
 *     Subpage number
 *
 *   DefaultBackgroundColor -
 *     default background color
 *
 *   DecodingLevel -
 *     see TT_CORE_DECODING_LEVEL
 *
 *   DecodingMode -
 *     see TT_CORE_DECODING_MODE
 ****************************************************************************/
typedef struct
{
  U_INT16            MagPage;
  U_INT16            SubPage;
  U_INT8             DefaultBackgroundColor;
  TT_DECODING_LEVEL  DecodingLevel;
  TT_DECODING_MODE   DecodingMode;

} TT_CORE_DECODING_PARAMS;




#ifdef __cplusplus
  extern "C" {
#endif



/****************************************************************************
 * FUNCTION: TT_CoreCreate
 *
 * DESCRIPTION:
 *   Creates all needed modules (SPC, Display, ACQ, packet buffer, TOP, 
 *   database and database lookup table).
 *
 * PARAMETERS:
 *   aMemoryStart - The start address of the internal memory used by the 
 *                  decoder. If there is no malloc available on the used
 *                  system, this parameter is used to define the decoders
 *                  internal memory area.
 *   aMemorySize  - The size of the internal memory used by the decoder. Both
 *                  parameters are used on dynamic memory allocation only.
 *' A rough calculation for the needed memory size can be:
 *' internal size + no of pages * 1400
 * 
 * RETURN:
 *   TRUE if successful, FALSE otherwise.
 ****************************************************************************/
PUBLIC TT_BOOL
TT_CoreCreate( 
  unsigned long   aMemoryStart,
  unsigned long   aMemorySize);


/****************************************************************************
 * FUNCTION: TT_CoreDelete
 *
 * DESCRIPTION:
 *   Deletes the core instance and all depending modules.
 ****************************************************************************/
PUBLIC void
TT_CoreDelete( void );


/****************************************************************************
 * FUNCTION: TT_CoreGetVersion
 *
 * DESCRIPTION:
 *   Returns the core version string. Format: <major version>.<minor version>
 *   e.g. 1.01
 *
 * RETURN:
 *   Version string
 ****************************************************************************/

PUBLIC const char*
TT_CoreGetVersion( void );


/****************************************************************************
 * FUNCTION: TT_CoreCheckVersion
 *
 * DESCRIPTION:
 *   Checks if the given Version String is compatible with the current
 *   Version of the lib. Format: <major version>.<minor version> e.g. 1.01
 *
 * PARAMETERS:
 *   aVersionString - The version string required by the caller
 *   aReturnMessage - text buffer for error messages
 *   aMessageSize   - size of the text buffer
 *
 * RETURN:
 *   true if compatible false otherwise
 ****************************************************************************/
PUBLIC TT_BOOL 
TT_CoreCheckVersion( 
  const char *    aRequiredVersion,
  char *          aErrorMessage,
  int             aSize );


#if ( defined LEVEL_15 )
/****************************************************************************
 * FUNCTION: TT_CoreIsCompiledForTeletextLevel15
 *
 * DESCRIPTION:
 *   This function exists only within a built for LEVEL_15.
 ****************************************************************************/
PUBLIC void
TT_CoreIsCompiledForTeletextLevel15( void );


#elif ( defined LEVEL_25 )
/****************************************************************************
 * FUNCTION: TT_CoreIsCompiledForTeletextLevel25
 *
 * DESCRIPTION:
 *   This function exists only within a built for LEVEL_25.
 ****************************************************************************/
PUBLIC void
TT_CoreIsCompiledForTeletextLevel25( void );
#else
  #error "Either LEVEL_15 or LEVEL_25 has to be defined!"
#endif


/****************************************************************************
 * FUNCTION: TT_CoreGetNumberOfPagesInDb
 *
 * DESCRIPTION:
 *   Returns the amount of pages stored inside the database. In case
 *   TT_DEC_STAT_FILE is defined a text file is generated. This file can be
 *   used e.g. in EXCEL for further statistics.
 *
 * PARAMETERS:
 *   aNumberOfPages      - number of pages inside database
 *   aNumberOfSubPages   - number of additional stored subpages 
 *   aNumberOfTotalPages - number of all pages stored inside database
 ****************************************************************************/
PUBLIC void
TT_CoreGetNumberOfPagesInDb(
  U_INT32 * aNumberOfPages,
  U_INT32 * aNumberOfSubPages,
  U_INT32 * aNumberOfTotalPages );


/****************************************************************************
 * FUNCTION: TT_CoreSpcProcess
 *
 * DESCRIPTION:
 *   This function starts the serial/parallel conversion which reads the
 *   teletext data from the database and writes into the display page buffer.
 *
 * PARAMETERS:
 *   aParams      - decoding parameters TT_CORE_DECODING_PARAMS
 *
 * RETURN:
 *   success
 ****************************************************************************/
PUBLIC TT_BOOL
TT_CoreSpcProcess( TT_CORE_DECODING_PARAMS* aParams );


/****************************************************************************
 * FUNCTION: TT_CoreSpcReset
 *
 * DESCRIPTION:
 *   Resets the serial/parallel conversion module. 
 *
 * PARAMETERS:
 *   none
 ****************************************************************************/
PUBLIC void
TT_CoreSpcReset( void );



#ifdef __cplusplus
  }
#endif

#endif /* TT_CORE_H */

/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
