/****************************************************************************
 * MODULE     : TT_Display
 * FILE       : TT_Display.h
 * COPYRIGHT  : TARA Systems GmbH
 * DESCRIPTION:
 *   The Teletext display module stores all information for displaying the
 *   basic Teletext information. It contains the decoded characters with
 *   there attributes for one Teletext page and additional information about
 *   used colors and sidepanels.
 ****************************************************************************/

#ifndef TT_DISPLAY_H
#define TT_DISPLAY_H


/****************************************************************************
 * SECTION: #define
 ****************************************************************************/

#define TT_DSP_NOF_ROWS                 26
#define TT_DSP_NOF_L1_COLUMNS           40

#define TT_DSP_ALL_ROWS                 0xFF


#if ( defined LEVEL_15 )
  #define TT_DSP_NOF_COLUMNS            40
#elif ( defined LEVEL_25 )
  #define TT_DSP_NOF_COLUMNS            56 /* L1 + Sidepanel */
#else
  #error "Either LEVEL_15 or LEVEL_25 has to be defined!"
#endif 

#if ( defined LEVEL_15 )
  #define TT_DSP_NO_OF_COLORS           16
#elif ( defined LEVEL_25 )
  #define TT_DSP_NO_OF_COLORS           38
#else
  #error "Either LEVEL_15 or LEVEL_25 has to be defined!"
#endif

#define TT_DSP_NOF_CHANGE_COLORS        16

#define TT_DSP_ROW_ATTR_DOUBLE_HEIGHT   0x80u
#define TT_DSP_ROW_ATTR_IS_PROCESSED    0x01u
#define TT_DSP_ROW_ATTR_IS_DISPLAYED    0x02u
#define TT_DSP_ROW_ATTR_PROCESS_MASK    0x03u


/**************** color definitions *******************/

#if ( defined LEVEL_15 )

#define TT_COLOR_BLACK        0x00
#define TT_COLOR_RED          0x01
#define TT_COLOR_GREEN        0x02
#define TT_COLOR_YELLOW       0x03
#define TT_COLOR_BLUE         0x04
#define TT_COLOR_MAGENTA      0x05
#define TT_COLOR_CYAN         0x06
#define TT_COLOR_WHITE        0x07

#define TT_COLOR_TRANSPARENT  0x08
#define TT_COLOR_GREY         0x09
#define TT_COLOR_LIGHT_GREY   0x0A
#define TT_COLOR_DARK_GREY    0x0B
#define TT_COLOR_USER0        0x0C
#define TT_COLOR_USER1        0x0D
#define TT_COLOR_USER2        0x0E
#define TT_COLOR_USER3        0x0F

#elif ( defined LEVEL_25 )

#define TT_COLOR_BLACK        0x00
#define TT_COLOR_RED          0x01
#define TT_COLOR_GREEN        0x02
#define TT_COLOR_YELLOW       0x03
#define TT_COLOR_BLUE         0x04
#define TT_COLOR_MAGENTA      0x05
#define TT_COLOR_CYAN         0x06
#define TT_COLOR_WHITE        0x07

#define TT_COLOR_TRANSPARENT  0x08
#define TT_COLOR_HALF_RED     0x09
#define TT_COLOR_HALF_GREEN   0x0A
#define TT_COLOR_HALF_YELLOW  0x0B
#define TT_COLOR_HALF_BLUE    0x0C
#define TT_COLOR_HALF_MAGENTA 0x0D
#define TT_COLOR_HALF_CYAN    0x0E
#define TT_COLOR_GREY         0x0F

/* color 0x10 - 0x1F can be overwritten by broadcast */
#define TT_COLOR_CLUT2_0      0x10
#define TT_COLOR_CLUT2_1      0x11
#define TT_COLOR_CLUT2_2      0x12
#define TT_COLOR_CLUT2_3      0x13
#define TT_COLOR_CLUT2_4      0x14
#define TT_COLOR_CLUT2_5      0x15
#define TT_COLOR_CLUT2_6      0x16
#define TT_COLOR_CLUT2_7      0x17

#define TT_COLOR_CLUT3_0      0x18
#define TT_COLOR_CLUT3_1      0x19
#define TT_COLOR_CLUT3_2      0x1A
#define TT_COLOR_CLUT3_3      0x1B
#define TT_COLOR_CLUT3_4      0x1C
#define TT_COLOR_CLUT3_5      0x1D
#define TT_COLOR_CLUT3_6      0x1E
#define TT_COLOR_CLUT3_7      0x1F

#ifdef TT_DISPLAY_COLOR_LIMIT_32
#define TT_COLOR_LIGHT_GREY   TT_COLOR_WHITE
#define TT_COLOR_DARK_GREY    TT_COLOR_BLACK

#define TT_COLOR_USER0        TT_COLOR_HALF_BLUE
#define TT_COLOR_USER1        TT_COLOR_HALF_MAGENTA
#define TT_COLOR_USER2        TT_COLOR_HALF_CYAN
#define TT_COLOR_USER3        TT_COLOR_HALF_YELLOW
#else /* TT_DISPLAY_COLOR_LIMIT_32 */
#define TT_COLOR_LIGHT_GREY   0x20
#define TT_COLOR_DARK_GREY    0x21

#define TT_COLOR_USER0        0x22
#define TT_COLOR_USER1        0x23
#define TT_COLOR_USER2        0x24
#define TT_COLOR_USER3        0x25
#endif /* TT_DISPLAY_COLOR_LIMIT_32 */

#else
#error "Either LEVEL_15 or LEVEL_25 has to be defined!"
#endif



/*
max DRCS characters
*/
#define TT_DISPLAY_MAX_DRCS 24

/****************************************************************************
 * SECTION: typedef
 ****************************************************************************/

/****************************************************************************
 * CALLBACK: TT_DRCS_CALLBACK
 *
 * DESCRIPTION:
 *   This callback function will be called any time a DRCS character is
 *   decoded for the current Teletext page. The character data is passed to
 *   the callback function and can be stored for display purposes. If DRCS
 *   scalling is needed, it is recommended to do this within the callback
 *   function.
 *
 * PARAMETERS:
 *   aIndex - The DRCS character index (0..24)
 *   aData  - The DRCS character data
 ****************************************************************************/

typedef void
( * TT_DRCS_CALLBACK ) (
  U_INT8      aIndex,
  U_INT8    * aData);


/****************************************************************************
 * TYPE: TT_CHAR_STRUCT
 *
 * DESCRIPTION:
 *   The structure TT_CHAR_STRUCT includes the character code and all
 *   attributes for a Teletext character. L2.5 size on PC: 6 Byte -> with
 *   pragma pack 5 Byte
 *
 * COMPONENTS:
 *   CharCode     - character code, begins with 0x20
 *   FontSelector - defines the font for the character
 *   DoubleWidth  - only level 2.5
 *   DoubleHeight - DH character upper and lower half
 *   LowerHalf    - DH character only lower half
 *   FlashMode    - blinking character
 *   Conceal      - hidden character, shown with reveal mode
 *   FgColor      - foreground color ot the character
 *   BgColor      - background color ot the character
 *   Protected    - protected bit for internal use
 *
 * SEE ALSO:
 *   TT_CHAR_HANDLE
 ****************************************************************************/


#if ( defined LEVEL_25 )

typedef struct
{
  U_INT8 CharCode;

  BitField  FgColor      : 8;
  BitField  BgColor      : 8;

  BitField  FontSelector : 5;
  BitField  DoubleWidth  : 1;
  BitField  DoubleHeight : 1;
  BitField  LowerHalf    : 1;
  BitField  FlashMode    : 5;
  BitField  Underline    : 1;
  BitField  Conceal      : 1;
  BitField  Protected    : 1;

} TT_CHAR_STRUCT;

/* (LEVEL_25) */

#elif ( defined LEVEL_15 )
typedef struct
{
  BitField  CharCode     : 8;

  BitField  FgColor      : 4;
  BitField  BgColor      : 4;

  BitField  FontSelector : 5;
  BitField  DoubleWidth  : 1;
  BitField  DoubleHeight : 1;
  BitField  LowerHalf    : 1;
  BitField  FlashMode    : 1;
  BitField  Underline    : 1;
  BitField  Conceal      : 1;
  BitField  Protected    : 1;

} TT_CHAR_STRUCT;
#else
#error "Either LEVEL_15 or LEVEL_25 has to be defined!"
#endif /* (LEVEL_15) */


/****************************************************************************
 * TYPE: TT_CHAR_HANDLE
 *
 * DESCRIPTION:
 *   Pointer to structure TT_CHAR_STRUCT.
 *
 * SEE ALSO:
 *   TT_CHAR_STRUCT
 ****************************************************************************/
typedef TT_CHAR_STRUCT * TT_CHAR_HANDLE;

/****************************************************************************
 * TYPE: TT_CURSOR_STRUCT
 *
 * DESCRIPTION:
 *   The structure TT_DISPLAY_CURSOR defines a cursor area relative to the
 *   LOP coordinates of the display page.
 *
 * COMPONENTS:
 *   X        - X position
 *   Y        - Y position
 *   W        - Width (no of Characters in a row)
 *   Reserved - Reserved
 *   FlagOn   - Cursor On / Off
 *
 * SEE ALSO:
 *   TT_DISPLAY
 ****************************************************************************/
typedef struct
{
  U_INT8          X;
  U_INT8          Y;
  U_INT8          W;
  U_INT8          Reserved;
  TT_BOOL         FlagOn;

} TT_CURSOR_STRUCT;

/****************************************************************************
 * TYPE: TT_CURSOR_HANDLE
 *
 * DESCRIPTION:
 *   Pointer to structure TT_CURSOR_STRUCT.
 *
 * SEE ALSO:
 *   TT_CHAR_STRUCT
 ****************************************************************************/
typedef TT_CURSOR_STRUCT * TT_CURSOR_HANDLE;


/****************************************************************************
 * TYPE: TT_DISPLAY
 *
 * DESCRIPTION:
 *   The structure TT_DISPLAY defines the parallel buffer for all Teletext
 *   character of one display page. Each character is defined by
 *   TT_CHAR_STRUCT.
 *
 * COMPONENTS:
 *   Page           - Teletext display page
 *   RowAttribute   - Row dependend display attributes
 *   ColorScreen    - Screen background color
 *   ColorRow       - Row background colors
 *   SidePanelLeft  - No of characters for left sidepannel
 *   SidePanelRight - No of characters for right sidepannel If both
 *                    sidepannels are 0, no sidpannel information is
 *                    available.
 *   UpdateColors   - Color table for re-definable colors
 *   DrcsCallback   - Callback function for DRCS characters
 *
 * SEE ALSO:
 *   TT_DISPLAY_HANDLE
 ****************************************************************************/
typedef struct
{
  TT_CHAR_STRUCT    Page[TT_DSP_NOF_ROWS][TT_DSP_NOF_COLUMNS];

  U_INT8            RowAttribute[TT_DSP_NOF_ROWS+1];

  U_INT8            ColorScreen;
  U_INT8            ColorRow[TT_DSP_NOF_ROWS];
  U_INT8            SidePanelLeft;
  U_INT8            SidePanelRight;

  TT_CURSOR_STRUCT  Cursor;

  #if ( defined LEVEL_25)
  TT_DRCS_CALLBACK  DrcsCallback;
  U_INT16           UpdateColors[TT_DSP_NOF_CHANGE_COLORS];
  #endif /* (LEVEL_25) */

} TT_DISPLAY;


/****************************************************************************
 * TYPE: TT_DISPLAY_HANDLE
 *
 * DESCRIPTION:
 *   Pointer to structure TT_DISPLAY.
 *
 * SEE ALSO:
 *   TT_DISPLAY
 ****************************************************************************/
#if (TT_DISPLAY_MEM_ALLOC == TT_MEM_ALLOC_SRAM)
typedef TT_DISPLAY far * TT_DISPLAY_HANDLE;
#else
typedef TT_DISPLAY * TT_DISPLAY_HANDLE;
#endif



#ifdef __cplusplus
  extern "C" {
#endif



/****************************************************************************
 * FUNCTION: TT_DisplaySetScreenColor
 *
 * DESCRIPTION:
 *   Sets the background color.
 *
 * PARAMETERS:
 *   aColor - The full screen color
 ****************************************************************************/

PUBLIC void
TT_DisplaySetScreenColor(
  U_INT8          aColor );


/****************************************************************************
 * FUNCTION: TT_DisplayGetScreenColor
 *
 * DESCRIPTION:
 *   Returns the fullscreen color.
 *
 * RETURN:
 *   fullscreen color
 ****************************************************************************/

PUBLIC U_INT8
TT_DisplayGetScreenColor( void );


/****************************************************************************
 * FUNCTION: TT_DisplayGetRowColor
 *
 * DESCRIPTION:
 *   Returns the rows background color.
 *
 * RETURN:
 *   The row background color.
 ****************************************************************************/

PUBLIC U_INT8
TT_DisplayGetRowColor(
  U_INT8   aRow );


/****************************************************************************
 * FUNCTION: TT_DisplaySetRowColor
 *
 * DESCRIPTION:
 *   Sets the rows background color.
 *
 * PARAMETER:
 *   aRow   - the row number background color.
 *   aColor - the background color
 ****************************************************************************/

PUBLIC void
TT_DisplaySetRowColor(
  U_INT8   aRow,
  U_INT8   aColor );


/****************************************************************************
 * FUNCTION: TT_DisplayGetSidePanelLeft
 *
 * DESCRIPTION:
 *   Returns the number of characters for the left sidepannel.
 *
 * RETURN:
 *   No of characters for left sidepannel
 ****************************************************************************/

PUBLIC U_INT8
TT_DisplayGetSidePanelLeft( void );


/****************************************************************************
 * FUNCTION: TT_DisplaySetSidePanelLeft
 *
 * DESCRIPTION:
 *   Sets the number of characters for the left sidepannel.
 *
 * PARAMETERS:
 *   size - number of character for the left sidepannel
 ****************************************************************************/

PUBLIC void
TT_DisplaySetSidePanelLeft( 
  U_INT8 size);


/****************************************************************************
 * FUNCTION: TT_DisplayGetSidePanelRight
 *
 * DESCRIPTION:
 *   Returns the number of characters for the right sidepannel.
 *
 * RETURN:
 *   No of characters for right sidepannel
 ****************************************************************************/

PUBLIC U_INT8
TT_DisplayGetSidePanelRight( void );


/****************************************************************************
 * FUNCTION: TT_DisplaySetSidePanelRight
 *
 * DESCRIPTION:
 *   Sets the number of characters for the right sidepannel.
 *
 * PARAMETERS:
 *   size - number of character for the right sidepannel
 ****************************************************************************/

PUBLIC void
TT_DisplaySetSidePanelRight( 
  U_INT8 size );


/****************************************************************************
 * FUNCTION: TT_DisplaySetChar
 *
 * DESCRIPTION:
 *   Insert a Teletext character into the teletext page at absolute position.
 *   No attributes are checked.
 *
 * PARAMETERS:
 *   aRow    - row number in the teletext page
 *   aColumn - column number in the teletext page
 *   aChar   - structure pointer to teletext character structure
 ****************************************************************************/

PUBLIC void
TT_DisplaySetChar(
  U_INT8          aRow,
  U_INT8          aColumn,
  TT_CHAR_HANDLE  aChar );


/****************************************************************************
 * FUNCTION: TT_DisplayProtectArea
 *
 * DESCRIPTION:
 *   Protects or unprotects a specific area to be overwritten from the
 *   SPC/SPT.
 *
 * PARAMETERS:
 *   aStartRow    - start row of the area
 *   aStartColumn - start column of the area
 *   aEndRow      - end row of the area
 *   aEndColumn   - end column of the area
 *   aProtect     - TRUE: protect, FALSE: unprotect
 ****************************************************************************/

PUBLIC void
TT_DisplayProtectArea(
  U_INT8          aStartRow,
  U_INT8          aStartColumn,
  U_INT8          aEndRow,
  U_INT8          aEndColumn,
  TT_BOOL         aProtect );


/****************************************************************************
 * FUNCTION: TT_DisplaySetCharInContext
 *
 * DESCRIPTION:
 *   Insert a Teletext character into the teletext page at absolute position.
 *   Lower half of double height characters is processed within this
 *   function.
 *
 * PARAMETERS:
 *   aRow    - row number in the teletext page
 *   aColumn - column number in the teletext page
 *   aChar   - structure pointer to teletext character structure
 ****************************************************************************/

PUBLIC void
TT_DisplaySetCharInContext (
  U_INT8          aRow,
  U_INT8          aColumn,
  TT_CHAR_HANDLE  aChar );


/****************************************************************************
 * FUNCTION: TT_DisplayGetChar
 *
 * DESCRIPTION:
 *   Get a Teletext character from the teletext page
 *
 * PARAMETERS:
 *   aRow    - row number in the teletext page
 *   aColumn - column number in the teletext page
 *
 * RETURN:
 *   Pointer to character or NULL for invalid character
 ****************************************************************************/

PUBLIC TT_CHAR_HANDLE
TT_DisplayGetChar(
  U_INT8        aRow,
  U_INT8        aColumn );


/****************************************************************************
 * FUNCTION: TT_DisplayGetCharCode
 *
 * DESCRIPTION:
 *   Returns the character code of the specified teletext character
 *
 * PARAMETERS:
 *   aRow    - row number in the teletext page
 *   aColumn - column number in the teletext page
 *
 * RETURN:
 *   Character code or 0 for invalid character.
 ****************************************************************************/

PUBLIC U_INT8
TT_DisplayGetCharCode(
  U_INT8        aRow,
  U_INT8        aColumn );


/****************************************************************************
 * FUNCTION: TT_DisplayClear
 *
 * DESCRIPTION:
 *   This function clears a single row or all rows of the display.
 *
 * PARAMETERS:
 *   aRow         - row number or TT_DSP_ALL_ROWS for all rows
 *   aTransparent - row color ( TRANSPARENT or BLACK )
 ****************************************************************************/

PUBLIC void
TT_DisplayClear( 
  U_INT8    aRow, 
  U_INT8    aTransparent );


/****************************************************************************
 * FUNCTION: TT_DisplayRowSetProcessed
 *
 * DESCRIPTION:
 *   Marks a row as processed.
 *
 * PARAMETERS:
 *   aRow - row number
 ****************************************************************************/

PUBLIC void 
TT_DisplayRowSetProcessed(
  U_INT8         aRow );


/****************************************************************************
 * FUNCTION: TT_DisplayPageSetProcessed
 *
 * DESCRIPTION:
 *   Marks all rows as processed.
 ****************************************************************************/

PUBLIC void 
TT_DisplayPageSetProcessed( void );


/****************************************************************************
 * FUNCTION: TT_DisplayRowSetDoubleHeight
 *
 * DESCRIPTION:
 *   Marks a row containing at least one double height attribute.
 *
 * PARAMETERS:
 *   aRow - row number
 ****************************************************************************/

PUBLIC void 
TT_DisplayRowSetDoubleHeight(
  U_INT8         aRow );


/****************************************************************************
 * FUNCTION: TT_DisplayRowRemoveAttributes
 *
 * DESCRIPTION:
 *   Removes the row attributes.
 *
 * PARAMETERS:
 *   aRow - row number
 *
 * SEE ALSO:
 *   TT_DisplayRowSetDoubleHeight
 ****************************************************************************/

PUBLIC void 
TT_DisplayRowRemoveAttributes(
  U_INT8         aRow );


/****************************************************************************
 * FUNCTION: TT_DisplayCheckPage
 *
 * DESCRIPTION:
 *   Creates a checksum for the current Teletext display. The checksum data
 *   includes all character atrtributes from row 1 to row 24 for the level 1
 *   Teletxt page area. Row 0 (Header), row 24 (navigation) and row 25
 *   (subpage navigation) are not part of this checksum.
 *
 * RETURN:
 *   checksum
 ****************************************************************************/

PUBLIC U_INT32 
TT_DisplayCheckPage( void );


/****************************************************************************
 * FUNCTION: TT_DisplaySetCursor
 *
 * DESCRIPTION:
 *   Define the cursor position and color. Only one cursor is handled.
 *
 * PARAMETERS:
 *   aX  - X position
 *   aY  - Y position
 *   aW  - Width (no of Characters in a row)
 *   aOn - Cursor On / Off
 ****************************************************************************/

PUBLIC void
TT_DisplaySetCursor(
  U_INT8   aX,
  U_INT8   aY,
  U_INT8   aW,
  TT_BOOL  aOn );


/****************************************************************************
 * FUNCTION: TT_DisplayGetCursor
 *
 * DESCRIPTION:
 *   Returns the cursor handle.
 *
 * RETURN:
 *   cursor handle
 ****************************************************************************/

PUBLIC TT_CURSOR_HANDLE
TT_DisplayGetCursor( void );


#if ( defined LEVEL_25 )
/****************************************************************************
 * FUNCTION: TT_DisplaySetDrcsCallback
 *
 * DESCRIPTION:
 *   Sets the callback function for DRCS character handling. This function is
 *   used for Level 2.5 only.
 *
 * PARAMETERS:
 *   aCallback - The callback function to set
 ****************************************************************************/

PUBLIC void
TT_DisplaySetDrcsCallback(
  TT_DRCS_CALLBACK    aCallback );


/****************************************************************************
 * FUNCTION: TT_DisplayGetUpdateColor
 *
 * DESCRIPTION:
 *   Gets an update color as 16 bit value in the format 4444 ARGB with A
 *   undefined. The indices 0-15 are mapped to the redefinable Teletext
 *   colors 16-31. This function is used for Level 2.5 only.
 *
 * PARAMETERS:
 *   aIndex - the update color index (0..15 allowed)
 *
 * RETURN:
 *   The update color value as 4444 ARGB
 ****************************************************************************/

PUBLIC U_INT16
TT_DisplayGetUpdateColor(
  U_INT8   aIndex );


#endif /* (LEVEL_25) */



#ifdef __cplusplus
  }
#endif

#endif /* TT_DISPLAY_H */

/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
