/***********************************************************************
 * MODULE     : TT_DriverDisplay
 * FILE       : TT_DriverDisplay.h
 * COPYRIGHT  : TARA Systems GmbH
 * DESCRIPTION:
 *   The display driver is a platform specific module which allows to 
 *   adapt the Teletext decoders drawing operations.
 ***********************************************************************/

#ifndef _TT_DRIVER_DISPLAY_H_
#define _TT_DRIVER_DISPLAY_H_

/***********************************************************************
 * SECTION: #include
 ***********************************************************************/

#include <teletext/TT_Stdinc.h>
#include <teletext/TT_DriverFont.h>

#ifdef __cplusplus
extern "C" {
#endif

/***********************************************************************
 * SECTION: #define
 ***********************************************************************/

#define TT_DISPLAY_MARGIN    10

#define TT_DSP_NORMAL_SIZE   0u
#define TT_DSP_UPPER_HALF    1u
#define TT_DSP_LOWER_HALF    2u
#define TT_DSP_READY         3u
#define TT_DSP_SIZE_MASK     3u

#define TT_DSP_REVEAL_ENABLE 4u
#define TT_DSP_SEMI_TRANSP   8u



/***********************************************************************
 * FUNCTION: TT_DriverDisplayOpen
 *
 * DESCRIPTION:
 *   Initialisation of the display driver. Depending on the compiler
 *   settings either the character based hardware display or bitmap
 *   based display is initialised.
 ***********************************************************************/
PUBLIC void 
TT_DriverDisplayOpen( void );


typedef struct 
{
    int layerIdx;
    int firstClutIdx;
    
} TT_InitParamsT;

PUBLIC TT_BOOL
TT_DriverDisplayInit( TT_InitParamsT* params );

/***********************************************************************
 * FUNCTION: TT_DriverDisplayClose
 *
 * DESCRIPTION:
 *   Close the display driver.
 ***********************************************************************/
PUBLIC void 
TT_DriverDisplayClose( void );


/***********************************************************************
 * FUNCTION: TT_DriverDisplayIsOpen
 *
 * DESCRIPTION:
 *   Get information about the open state of the display driver.
 *
 * RETURN:
 *   TRUE if driver is open, otherwise false.
 ***********************************************************************/
PUBLIC TT_BOOL
TT_DriverDisplayIsOpen( void );


/***********************************************************************
 * FUNCTION: TT_DriverDisplaySetFont
 *
 * DESCRIPTION:
 *   Set a new font of type TT_FONT to the Teletext display driver. 
 *   This font is then used for rendering.
 *
 * PARAMETERS:
 *   aFont   - The new font to set
 ***********************************************************************/
PUBLIC void
TT_DriverDisplaySetFont( 
  TT_FONT   aFont );


/***********************************************************************
 * FUNCTION: TT_DriverDisplayGetFont
 *
 * DESCRIPTION:
 *   Get the current font of type TT_FONT used by the Teletext display 
 *   driver.
 *
 * Return:
 *   a pointer to the TT_FONT
 ***********************************************************************/
PUBLIC TT_FONT
TT_DriverDisplayGetFont( void );


/***********************************************************************
 * FUNCTION: TT_DriverDisplayGetColorIndex
 *
 * DESCRIPTION:
 *   Get the color index of a Teletext color which is used on the target 
 *   implementation.
 *
 * PARAMETERS:
 *   aTeletextColor   - The Teletext color as defined in TT_Display.h
 *
 * Return:
 *   The used color index or 0xFF if no valid color is given.
 ***********************************************************************/
PUBLIC U_INT8
TT_DriverDisplayGetColorIndex(
   U_INT8  aTeletextColor );


/***********************************************************************
 * FUNCTION: TT_DriverDisplayStartPageDraw
 *
 * DESCRIPTION:
 *   This function is called at start of a new page drawing. This 
 *   allows to initialize internal settings in the display driver.
 ***********************************************************************/
PUBLIC void 
TT_DriverDisplayStartPageDraw( void );


/***********************************************************************
 * FUNCTION: TT_DriverDisplayEndPageDraw
 *
 * DESCRIPTION:
 *   This function is called at the end of a page drawing. This 
 *   allows to do some post processing within the display driver.
 ***********************************************************************/
PUBLIC void 
TT_DriverDisplayEndPageDraw( void );


/***********************************************************************
 * FUNCTION: TT_DriverDisplayDrawRow
 *
 * DESCRIPTION:
 *   This function is rendering a Teletext row.
 *   For rendering the following modes can be selected:
 *    1. Size modes:
 *       - TT_DSP_NORMAL_SIZE
 *       - TT_DSP_UPPER_HALF
 *       - TT_DSP_LOWER_HALF
 *    2. Drawing modes
 *       - TT_DSP_REVEAL_ENABLE
 *       - TT_DSP_SEMI_TRANSP
 *
 * PARAMETERS:
 *   aSrcRow   - Teletext row number to display
 *   aDestRow  - display row number where to place the row
 *   aMode     - display mode (normal or double size)
 ***********************************************************************/
PUBLIC void
TT_DriverDisplayDrawRow(
  U_INT8     aSrcRow,
  U_INT8     aDestRow,
  U_INT8     aMode );

/***********************************************************************
 * FUNCTION: TT_DriverDisplayFlushClut
 *
 * DESCRIPTION:
 *   Must be called periodcally every 166 ms to handle the clut animation.
 ***********************************************************************/
PUBLIC void 
TT_DriverDisplayFlushClut( void );

/***********************************************************************
 * FUNCTION: TT_DriverDisplaySetPaletteColor
 *
 * DESCRIPTION:
 *   Set the specified color into the internal palette at the given index.
 *
 * PARAMETERS:
 *   aIndex  - the palette index to set
 *   aRed    - value of red
 *   aGreen  - value of green
 *   aBlue   - value blue
 ***********************************************************************/
PUBLIC void 
TT_DriverDisplaySetPaletteColor(
  U_INT8 aIndex, 
  U_INT8 aRed, 
  U_INT8 aGreen, 
  U_INT8 aBlue);


#if ( defined LEVEL_25 )
/***********************************************************************
 * FUNCTION: TT_DriverDisplaySetDrcsChar
 *
 * DESCRIPTION:
 *   define a DRCS character. there are up to 96 DRCS characters,
 *   each character is given by 10 rows, each row has 12 column pixels.
 *
 * PARAMETERS:
 *   aIndex      - number of DRCS character 0 ... 95
 *   aData       - pointer to DRCS data
 ***********************************************************************/
PUBLIC void
TT_DriverDisplaySetDrcsChar(
  U_INT8      aIndex,
  U_INT8    * aData);
#endif /* #if (LEVEL_25) */


#ifdef __cplusplus
  }
#endif

#endif /* _TT_DRIVER_DISPLAY_H_ */
