/****************************************************************************
 * MODULE     : TT_PacketBuffer
 * FILE       : TT_PacketBuffer.h
 * COPYRIGHT  : TARA Systems GmbH
 * DESCRIPTION:
 *   The Teletext packet buffer is a thread save ring buffer used to cache
 *   Teletext packets. The packets can be added to the buffer from a slicer
 *   task or from VBI interrupt. The packets are taken from the buffer when
 *   calling the function TT_AcqExecuteProcessing. The size of the packet
 *   bffer defines the maximum delay of processing Teletext packets.
 ****************************************************************************/

#ifndef TT_PACKETBUFFER_H
#define TT_PACKETBUFFER_H


/****************************************************************************
 * SECTION: #define
 ****************************************************************************/

#define TT_PACKETBUFFER_PACKET_LENGTH 44

/****************************************************************************
 * SECTION: typedef
 ****************************************************************************/

/****************************************************************************
 * TYPE: TT_PACKETBUFFER_PACKET_HANDLE
 ****************************************************************************/
typedef struct
{
  U_INT8
    Data[ TT_PACKETBUFFER_PACKET_LENGTH ];

} TT_PACKETBUFFER_PACKET;

typedef TT_PACKETBUFFER_PACKET * TT_PACKETBUFFER_PACKET_HANDLE;

#ifdef __cplusplus
  extern "C" {
#endif



/****************************************************************************
 * FUNCTION: TT_PacketBufferReset
 *
 * DESCRIPTION:
 *   Resets the Teletext PacketBuffer. All Data is removed from the buffer
 *   and the overflow counter is set to zero.
 ****************************************************************************/

PUBLIC void
TT_PacketBufferReset( void );


/****************************************************************************
 * FUNCTION: TT_PacketBufferClear
 *
 * DESCRIPTION:
 *   Remove all data from the packet buffer and reset the overflow flag. The
 *   overflow counter is not resetted.
 ****************************************************************************/

PUBLIC void
TT_PacketBufferClear( void );


/****************************************************************************
 * FUNCTION: TT_PacketBufferSetPacket
 *
 * DESCRIPTION:
 *   Copy a Teletex packet to the packet buffer. The Teletext packet set to
 *   the packet buffer has to fulfill the following format: 2 byte framing
 *   code, 42 byte Teletext data => 44 bytes of data
 *
 * PARAMETERS:
 *   aPacket - pointer to packet data
 *
 * RETURN:
 *   TRUE if successfull, FALSE otherwise.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_PacketBufferSetPacket(
  TT_PACKETBUFFER_PACKET_HANDLE  aPacket );


/****************************************************************************
 * FUNCTION: TT_PacketBufferGetPacket
 *
 * DESCRIPTION:
 *   Retrieves packet data from packet buffer.
 *
 * RETURN:
 *   On success: pointer to packet data, NULL on failure.
 ****************************************************************************/

PUBLIC TT_PACKETBUFFER_PACKET_HANDLE
TT_PacketBufferGetPacket( void );


/****************************************************************************
 * FUNCTION: TT_PacketBufferIsEmpty
 *
 * DESCRIPTION:
 *   Check if the packet buffer is empty.
 *
 * RETURN:
 *   TRUE if the packet buffer is empty, NULL otherwise.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_PacketBufferIsEmpty( void );


/****************************************************************************
 * FUNCTION: TT_PacketBufferIsOverflow
 *
 * DESCRIPTION:
 *   Check if there is a packet buffer overflow. If there is an overflow, the
 *   packet buffer should be cleared from the reading task.
 *
 * RETURN:
 *   TRUE if there is a packet buffer overflow, NULL otherwise.
 ****************************************************************************/

PUBLIC TT_BOOL
TT_PacketBufferIsOverflow( void );


/****************************************************************************
 * FUNCTION: TT_PacketBufferGetNoOfOverflows
 *
 * DESCRIPTION:
 *   Returns the number of packet buffer overflows since the last reset or
 *   creation time.
 *
 * RETURN:
 *   The number of packet buffer overflows.
 ****************************************************************************/

PUBLIC U_INT16
TT_PacketBufferGetNoOfOverflows( void );


/****************************************************************************
 * FUNCTION: TT_PacketBufferGetNoOfPackets
 *
 * DESCRIPTION:
 *   Returns the number of packets currently stored in the packet buffer.
 *
 * RETURN:
 *   The number of packets in packet buffer.
 ****************************************************************************/

PUBLIC U_INT16
TT_PacketBufferGetNoOfPackets( void );


/****************************************************************************
 * FUNCTION: TT_PacketBufferGetMaxNoOfPackets
 *
 * DESCRIPTION:
 *   Returns the maximum number of packets that can be stored in the packet
 *   buffer.
 *
 * RETURN:
 *   The max number of packets in packet buffer.
 ****************************************************************************/

PUBLIC U_INT16
TT_PacketBufferGetMaxNoOfPackets( void );



#ifdef __cplusplus
  }
#endif

#endif /* TT_PACKETBUFFER_H */

/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
