/****************************************************************************
 * MODULE     : TT_TOP
 * FILE       : TT_TOP.h
 * COPYRIGHT  : TARA Systems GmbH
 * DESCRIPTION:
 *   Teletext TOP acquisition
 *
 ****************************************************************************/

#ifndef TT_TOP_H
#define TT_TOP_H


/****************************************************************************
 * SECTION: #include
 ****************************************************************************/


/****************************************************************************
 * SECTION: #define
 ****************************************************************************/

#define TT_TOP_AI_STRING_LENGTH           12
#define TT_TOP_MAX_NO_OF_AI_ENTRIES       TT_TOP_MAX_STRINGS
#define TT_TOP_MAX_NO_OF_AI_PLT_ENTRIES   8

/* parameters for TT_TopCheckBttEntry() */
#define TT_TOP_PAGE_TYPE_NONE         0x00
#define TT_TOP_PAGE_TYPE_IN_CYC       0x01
#define TT_TOP_PAGE_TYPE_NORMAL       0x02
#define TT_TOP_PAGE_TYPE_UT           0x04
#define TT_TOP_PAGE_TYPE_PREV         0x08
#define TT_TOP_PAGE_TYPE_BLOCK        0x10
#define TT_TOP_PAGE_TYPE_GROUP        0x20
#define TT_TOP_PAGE_TYPE_GROUP_BLOCK  0x30
#define TT_TOP_PAGE_TYPE_MULTI        0x40
#define TT_TOP_PAGE_TYPE_AI           0x80



/****************************************************************************
 * SECTION: typedef
 ****************************************************************************/




#ifdef __cplusplus
  extern "C" {
#endif

/****************************************************************************
 * SECTION: Definition of public functions
 ****************************************************************************/

/****************************************************************************
 * FUNCTION: TT_TopDecodeRow23Data
 *
 * DESCRIPTION:
 *   This function anables or disabled the row 23 data decoding within the
 *   basic top table. The decoding ir related to the patent 'EP 0 625 298'.
 *   By default the row 23 decoding is disabled.
 *
 * PARAMETERS:
 *   aRow23Decoding - the decoding setting ( TRUE or FALSE )
 ****************************************************************************/

PUBLIC void
TT_TopDecodeRow23Data(
  TT_BOOL       aRow23Decoding );


/****************************************************************************
 * FUNCTION: TT_TopGetLinkText
 *
 * DESCRIPTION:
 *   Returns the block and group link strings corresponding to the given BCD
 *   coded page number. The returned strings contains the odd parity bit.
 *
 * PARAMETERS:
 *   aBcdPageNo -
 *     the page number
 *
 *   aBlockString -
 *     the pointer to the block string
 *
 *   aGroupString -
 *     the pointer to the group string
 *
 *   aPreviousBlockString -
 *     the pointer to the previous block string
 *
 *   aNatOptBlock -
 *     the pointer to the national options / block
 *
 *   aNatOptGroup -
 *     the pointer to the national options / group
 *
 *   aPreviousNatOptBlock -
 *     the pointer to the previous national options / block
 ****************************************************************************/

PUBLIC void
TT_TopGetLinkText(
  U_INT16       aBcdPageNo, 
  U_INT8        *aBlockString, 
  U_INT8        *aGroupString,
  U_INT8        *aPreviousBlockString,
  U_INT8        *aNatOptBlock, 
  U_INT8        *aNatOptGroup,
  U_INT8        *aPreviousNatOptBlock );


/****************************************************************************
 * FUNCTION: TT_TopGetPageTitle
 *
 * DESCRIPTION:
 *   Returns the TOP page title as string including national option bits for
 *   the selected BDC coded page number. If there is no AI information
 *   available, the coresponding BCD coded page number is returned as string.
 *   If the page number is not valid, the return buffers are not touched. The
 *   returned string contains the odd parity bit.
 *
 * PARAMETERS:
 *   aBcdPageNo   - the page number
 *   aTitleString - the pointer to the title string
 *   aNatOptTitle - the pointer to the national options
 ****************************************************************************/

PUBLIC void
TT_TopGetPageTitle(
  U_INT16       aBcdPageNo, 
  U_INT8        *aTitleString, 
  U_INT8        *aNatOptTitle );


/****************************************************************************
 * FUNCTION: TT_TopIsTopAvailable
 *
 * DESCRIPTION:
 *   Returns 1 as soon as the first TOP informations are available in the
 *   other case 0 is returned.
 *
 * RETURN:
 *   Top available flag.
 ****************************************************************************/

PUBLIC U_INT8
TT_TopIsTopAvailable( void );


/****************************************************************************
 * FUNCTION: TT_TopNavigation
 *
 * DESCRIPTION:
 *   Updates the given bcd coded page number to the following pagenumber
 *   depending on the given command.
 *
 * PARAMETERS:
 *   aCmd       - TOP command
 *   aBcdPageNo - pointer to the pagenumber
 ****************************************************************************/

PUBLIC void 
TT_TopNavigation(
  U_INT8        aCmd, 
  U_INT16       *aBcdPageNo );


/****************************************************************************
 * FUNCTION: TT_TopGetNoOfBlockPages
 *
 * DESCRIPTION:
 *   Returns the number of block - pages
 *
 * RETURN:
 *   The number of block pages.
 ****************************************************************************/

PUBLIC U_INT16 
TT_TopGetNoOfBlockPages( void );


/****************************************************************************
 * FUNCTION: TT_TopGetNoOfGroupPages
 *
 * DESCRIPTION:
 *   Returns the number of group - pages up to the next block - pages
 *
 * PARAMETERS:
 *   aBcdPageNo         - the start page number
 *   aFirstBCDGroupPage - first group - page
 *
 * RETURN:
 *   The number of group pages belonging to the current block page.
 ****************************************************************************/

PUBLIC U_INT16 
TT_TopGetNoOfGroupPages(
  U_INT16   aBcdPageNo, 
  U_INT16*  aFirstBCDGroupPage );


/****************************************************************************
 * FUNCTION: TT_TopGetNextGroupPageOfCurrentBlock
 *
 * DESCRIPTION:
 *   Returns the next group - page and link strings corresponding to the page
 *   number
 *
 * PARAMETERS:
 *   aBcdPageNo   - the start page number
 *   aGroupString - the pointer to the group string
 *   aNatOptGroup - the pointer to the national options / group
 *
 * RETURN:
 *   The group page belonging to the current block page.
 ****************************************************************************/

PUBLIC U_INT16 
TT_TopGetNextGroupPageOfCurrentBlock(
  U_INT16  aBcdPageNo,
  U_INT8  *aGroupString,
  U_INT8  *aNatOptGroup );


/****************************************************************************
 * FUNCTION: TT_TopGetNextBlockPage
 *
 * DESCRIPTION:
 *   Returns the next block - page and link strings corresponding to the page
 *   number
 *
 * PARAMETERS:
 *   aBcdPageNo   - the start page number
 *   aBlockString - the pointer to the block string
 *   aNatOptBlock - the pointer to the national options
 *
 * RETURN:
 *   the next block page no.
 ****************************************************************************/

PUBLIC U_INT16 
TT_TopGetNextBlockPage(
  U_INT16  aBcdPageNo,
  U_INT8  *aBlockString,
  U_INT8  *aNatOptBlock );


/****************************************************************************
 * FUNCTION: TT_TopGetNextBlockPageNo
 *
 * DESCRIPTION:
 *   Returns the next block - page number after the given page number.
 *
 * PARAMETERS:
 *   aBcdPageNo - the start page number
 *
 * RETURN :
 *   the next block page no.
 ****************************************************************************/

PUBLIC U_INT16 
TT_TopGetNextBlockPageNo(
  U_INT16  aBcdPageNo );


/****************************************************************************
 * FUNCTION: TT_TopGetPreviousBlockPageNo
 *
 * DESCRIPTION:
 *   Returns the previous block - page number previous the given number.
 *
 * PARAMETERS:
 *   aBcdPageNo - the start page number
 *
 * RETURN :
 *   the previous block page no.
 ****************************************************************************/

PUBLIC U_INT16 
TT_TopGetPreviousBlockPageNo(
  U_INT16  aBcdPageNo );


/****************************************************************************
 * FUNCTION: TT_TopGetNextGroupPageNo
 *
 * DESCRIPTION:
 *   Returns the next group - page number after the given page number.
 *
 * PARAMETERS:
 *   aBcdPageNo - the start page number
 *
 * RETURN :
 *   the next group page no.
 ****************************************************************************/

PUBLIC U_INT16 
TT_TopGetNextGroupPageNo(
  U_INT16  aBcdPageNo );


/****************************************************************************
 * FUNCTION: TT_TopGetPreviousGroupPageNo
 *
 * DESCRIPTION:
 *   Returns the previous group - page number previous to the given number.
 *
 * PARAMETERS:
 *   aBcdPageNo - the start page number
 *
 * RETURN :
 *   the previous group page no.
 ****************************************************************************/

PUBLIC U_INT16 
TT_TopGetPreviousGroupPageNo(
  U_INT16  aBcdPageNo );


/****************************************************************************
 * FUNCTION: TT_TopCheckBttEntry
 *
 * DESCRIPTION:
 *   Checks if the specified page number is marked with the given check type
 *   inside the BTT (basic top table).
 *
 * PARAMETERS:
 *   aBcdPageNo - the page number to check
 *   aCheckType - the check type to verfiy
 *
 * RETURN:
 *   TRUE if checktype is marked inside BTT or FALSE.
 ****************************************************************************/

PUBLIC TT_BOOL 
TT_TopCheckBttEntry (
  U_INT16       aBcdPageNo, 
  U_INT8        aCheckType );



#ifdef __cplusplus
  }
#endif

#endif /* TT_TOP_H */

/* This Headerfile was generated with C2H.exe ( C2H ) */
/* (c) 2006 by TARA Systems GmbH Munich */
