/*!
*****************************************************************************
**
** \file        ./subsystems/video/val/src/val.h
**
** \version     $Id: val.h 5874 2010-03-02 14:42:55Z wlaris $
**
** \brief       Video abstraction layer (VAL) suitable for live and PVR
**              playback presentation.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
**
*****************************************************************************
*/

#ifndef VAL_H
#define VAL_H

#include <fapex/abst_types.h>
#include <fapex/abst_libvidec.h>
#include <fapex/abst_libviout.h>
#include <fapex/abst_libhdmi.h>

//*****************************************************************************
//*****************************************************************************
//** Defines and Macros
//*****************************************************************************
//*****************************************************************************

/*!
********************************************************************************
** Standard error codes for the module
********************************************************************************
*/

/*! Bad parameter passed. */
#define SUBSYS_VIDEO_VAL_ERR_BAD_PARAMETER         (SUBSYS_VIDEO_VAL_MODULE_BASE +\
                                                           FAPI_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define SUBSYS_VIDEO_VAL_ERR_OUT_OF_MEMORY         (SUBSYS_VIDEO_VAL_MODULE_BASE +\
                                                           FAPI_ERR_OUT_OF_MEMORY)

/*! Module already initialised. */
#define SUBSYS_VIDEO_VAL_ERR_ALREADY_INITIALIZED   (SUBSYS_VIDEO_VAL_MODULE_BASE +\
                                                           FAPI_ERR_ALREADY_INITIALIZED)

/*! Module not initialised. */
#define SUBSYS_VIDEO_VAL_ERR_NOT_INITIALIZED       (SUBSYS_VIDEO_VAL_MODULE_BASE +\
                                                           FAPI_ERR_NOT_INITIALIZED)

/*! Feature or function is not available. */
#define SUBSYS_VIDEO_VAL_ERR_FEATURE_NOT_SUPPORTED (SUBSYS_VIDEO_VAL_MODULE_BASE +\
                                                           FAPI_ERR_FEATURE_NOT_SUPPORTED)

/*! Timeout occured. */
#define SUBSYS_VIDEO_VAL_ERR_TIMEOUT               (SUBSYS_VIDEO_VAL_MODULE_BASE +\
                                                           FAPI_ERR_TIMEOUT)

/*! The device is busy, try again later. */
#define SUBSYS_VIDEO_VAL_ERR_DEVICE_BUSY           (SUBSYS_VIDEO_VAL_MODULE_BASE +\
                                                           FAPI_ERR_DEVICE_BUSY)

/*! Invalid handle was passed. */
#define SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE        (SUBSYS_VIDEO_VAL_MODULE_BASE +\
                                                           FAPI_ERR_INVALID_HANDLE)

/*! Semaphore could not be created. */
#define SUBSYS_VIDEO_VAL_ERR_SEMAPHORE_CREATE      (SUBSYS_VIDEO_VAL_MODULE_BASE +\
                                                           FAPI_ERR_SEMAPHORE_CREATE)

/*! The driver's used version is not supported. */
#define SUBSYS_VIDEO_VAL_ERR_UNSUPPORTED_VERSION   (SUBSYS_VIDEO_VAL_MODULE_BASE +\
                                                           FAPI_ERR_UNSUPPORTED_VERSION)

/*! Module has not been opened. */
#define SUBSYS_VIDEO_VAL_ERR_NOT_OPEN              (SUBSYS_VIDEO_VAL_MODULE_BASE +\
                                                           FAPI_ERR_NOT_OPEN)

/*! Module is already open. */
#define SUBSYS_VIDEO_VAL_ERR_ALREADY_OPEN          (SUBSYS_VIDEO_VAL_MODULE_BASE +\
                                                           FAPI_ERR_ALREADY_OPEN)

/*!
********************************************************************************
** Additional driver error codes (reserved from -100 to -999)
**
** Each error code shall be decreased in steps of 1.
********************************************************************************
*/

/*! VAL specific error code base. */
#define SUBSYS_VIDEO_VAL_ERR_BASE   (SUBSYS_VIDEO_VAL_MODULE_BASE - 100)

/*! VAL specific errors. */
#define SUBSYS_VIDEO_VAL_ERR_DATA_NOT_AVAILABLE  (SUBSYS_VIDEO_VAL_ERR_BASE - 0)
#define SUBSYS_VIDEO_VAL_ERR_STATE               (SUBSYS_VIDEO_VAL_ERR_BASE - 1)
#define SUBSYS_VIDEO_VAL_ERR_ALREADY_STARTED     (SUBSYS_VIDEO_VAL_ERR_BASE - 2)
#define SUBSYS_VIDEO_VAL_ERR_BAD_MODE            (SUBSYS_VIDEO_VAL_ERR_BASE - 3)
#define SUBSYS_VIDEO_VAL_ERR_INTERNAL            (SUBSYS_VIDEO_VAL_ERR_BASE - 4)
#define SUBSYS_VIDEO_VAL_ERR_THREAD_CREATE       (SUBSYS_VIDEO_VAL_ERR_BASE - 5)

//*****************************************************************************
//*****************************************************************************
//** Data types
//*****************************************************************************
//*****************************************************************************

/*!
*******************************************************************************
** \brief Environment for which VAL instance is opened.
**
** This enumeration contains the presentation environments supported by VAL.
*******************************************************************************
*/
typedef enum
{
    VAL_ENV_NONE = -1,      //!< None, internally used only.
    VAL_ENV_LIVE = 0,       //!< Decoding of live broadcast content.
    VAL_ENV_PLAY,           //!< Playback of recorded content via PVR lib.
    VAL_ENV_STREAMING,      //!< Playback of Stream by streaming from ethernet or CMMB
    VAL_ENV_NUMS            //!< Number of modes, internally used.

} VAL_EnvironmentT;

/*!
*******************************************************************************
** \brief Video output definitions.
**
** This enumeration contains the video output types supported by VAL.
*******************************************************************************
*/
typedef enum
{
    VAL_VOUT_NONE = -1,     //!< None, internally used only.
    VAL_VOUT_SD,            //!< SD video output.
    VAL_VOUT_HD,            //!< HD video output.
    VAL_VOUT_NUMS           //!< Number of outputs, internally used.

} VAL_VideoOutputT;

/*!
*******************************************************************************
** \brief HD analog video encoder mode.
**
** This enumeration contains the possible modes that are supported by
** analog HD encoder, they also correspond the to respective HDMI
** video settings.
*******************************************************************************
*/
typedef enum
{
    VAL_1920x1080x2397P = 0,        //!< 1080p, 23.97 Hz.
    VAL_1920x1080x24P,              //!< 1080p, 24    Hz.
    VAL_1920x1080x25P,              //!< 1080p, 25    Hz.
    VAL_1920x1080x2997P,            //!< 1080p, 29.97 Hz.
    VAL_1920x1080x30P,              //!< 1080p, 30    Hz.
    VAL_1920x1080x50I,              //!< 1080i, 50    Hz.
    VAL_1920x1080x50P,              //!< 1080p, 50    Hz.
    VAL_1920x1080x5994I,            //!< 1080i, 59.94 Hz.
    VAL_1920x1080x5994P,            //!< 1080p, 59.94 Hz.
    VAL_1920x1080x60I,              //!< 1080i, 60    Hz.
    VAL_1920x1080x60P,              //!< 1080p, 60    Hz.
    VAL_1280x720x2397P,             //!<  720p, 23.97 Hz.
    VAL_1280x720x24P,               //!<  720p, 24    Hz.
    VAL_1280x720x25P,               //!<  720p, 25    Hz.
    VAL_1280x720x2997P,             //!<  720p, 29.97 Hz.
    VAL_1280x720x30P,               //!<  720p, 30    Hz.
    VAL_1280x720x50P,               //!<  720p, 50    Hz.
    VAL_1280x720x5994P,             //!<  720p, 59.94 Hz.
    VAL_1280x720x60P,               //!<  720p, 60    Hz.
    VAL_720x576x50I,                //!<  576i, 50    Hz.
    VAL_720x576x50P,                //!<  576p, 50    Hz.
    VAL_720x480x5994I,              //!<  480i, 59.94 Hz.
    VAL_720x480x5994P,              //!<  480p, 59.94 Hz.
    VAL_VGA_640x480x5994P,          //!<  480p, 59.94 Hz.
    VAL_VGA_800x600x5994P,          //!<  600p, 59.94 Hz.
    VAL_VGA_1024x768x5994P,         //!<  768p, 59.94 Hz.
    VAL_VGA_1360x768x5994P,         //!<  768p, 59.94 Hz.

} VAL_HdEncoderT;

/*!
*******************************************************************************
** \brief SD analog video encoder mode.
**
** This enumeration contains the possible modes that are supported by the
** analog SD encoder.
*******************************************************************************
*/
typedef enum
{
    VAL_SD_OFF = 0,                 //!< VE SD turned off.
    VAL_SD_SECAM,                   //!< Secam System.
    VAL_SD_NTSC,                    //!< NTSC System.
    VAL_SD_NTSC_4_43,               //!< NTSC 443 System.
    VAL_SD_PAL,                     //!< PAL System.
    VAL_SD_PAL60,                   //!< PAL 60 System.
    VAL_SD_PAL_M,                   //!< PAL M System.
    VAL_SD_PAL_N,                   //!< PAL N System.
    VAL_SD_PAL_N_ARG,               //!< PAL N Argentina System.
    VAL_SD_COLOURBAR                //!< PAL System Colourbar.

} VAL_SdEncoderT;

/*!
*******************************************************************************
** \brief VAL operation mode.
**
** This enumeration contains the internal operation modes of an opened VAL
** instance.
*******************************************************************************
*/
typedef enum
{
    VAL_OP_ERROR = -1,      //!< Module is in an error state that needs
                            //!< VAL_Close to be called for recovery.
    VAL_OP_NONE = 0,        //!< No operation.
    VAL_OP_LIVE,            //!< Live stream presentation.
    VAL_OP_PLAY,            //!< PVR playback presentation.
    VAL_OP_STREAMING,         //!< Streaming playback presentation.
    VAL_OP_IFRAME,          //!< Decoding of single I-frames (MPEG only!)
    VAL_OP_NUMS             //!< Number of available modes.

} VAL_OpEnumT;

/*!
*******************************************************************************
** \brief VAL playback mode.
**
** This enumeration contains all modes available for playback environment
** #VAL_ENV_PLAY. For live environment, mode is always #VAL_PLAYMODE_NORMAL.
*******************************************************************************
*/
typedef enum
{
    VAL_PLAYMODE_NONE = 0,            //!< Not in playback.
    VAL_PLAYMODE_NORMAL,              //!< Normal playback.
    VAL_PLAYMODE_PAUSE,               //!< Paused playback.
    VAL_PLAYMODE_FAST_DEC_IONLY,      //!< I-frame-only mode controlled by
                                      //!< decoder.
    VAL_PLAYMODE_FAST_DEC_IP,         //!< I/P-frame-only mode controlled by
                                      //!< decoder.
    VAL_PLAYMODE_FAST_LIB_IONLY,      //!< "Any-I" mode controlled by PVR lib.
    VAL_PLAYMODE_FAST_NOFEED,         //!< Fast seek mode controlled by PVR lib.
    VAL_PLAYMODE_FAST_CONTROLLEDFEED, //!< I-frame-only mode, data fed by PVR
                                      //!< lib, feed control and frame
                                      //!< extraction done by decoder.
    VAL_PLAYMODE_SLOW_2X,             //!< 1/2 speed slow mode.
    VAL_PLAYMODE_SLOW_4X,             //!< 1/4 speed slow mode.
    VAL_PLAYMODE_SLOW_8X,             //!< 1/8 speed slow mode.
    VAL_PLAYMODE_NUMS

} VAL_PlayModeEnumT;

/*!
*******************************************************************************
** \brief Video decoder status.
**
** This enumeration contains all internal decoder states.
*******************************************************************************
*/
typedef enum
{
    VAL_STATUS_STOPPED = 0,             //!< Decoder is stopped.
    VAL_STATUS_RUNNING,                 //!< Decoder is running.
    VAL_STATUS_NUMS                     //!< Number of states.

} VAL_StatusEnumT;

/*!
*******************************************************************************
** \brief Video decoder type.
**
** This structure describes the decoder type by system and frame bank size.
*******************************************************************************
*/
typedef struct
{
    int32_t                              system;        //!< Decoder system.
    int32_t                              frameBankSize; //!< Frame bank size.
    
} VAL_DecoderTypeT;

/*!
*******************************************************************************
** \brief Video frame parameters.
**
** This structure contains the video frame parameters relevant for judging if
** the decoder needs to be rebooted on change.
*******************************************************************************
*/
typedef struct
{
    uint32_t                             decodedWidth;
    uint32_t                             decodedHeight;

} VAL_FrameParamsT;

/*!
*******************************************************************************
** \brief HDMI connection, EDID availability and HDCP parameters
**
*******************************************************************************
*/
typedef struct
{
    uint8_t                              connected; //!< HDMI connection status.
    uint8_t                              haveEDID;  //!< EDID availability
                                                    //!< status.
} VAL_HDMIConnectParamsT;


typedef void (*VAL_HDMIConnectCallbackT) (const VAL_HDMIConnectParamsT*
                                          paramsPtr, void* arg);
/*!
*******************************************************************************
** \brief HDMI HDCP authentication state.
**
** This enumeration contains all HDCP authentication states.
*******************************************************************************
*/
typedef enum
{
    VAL_HDMI_HDCPSTATE_OFF =0,
    VAL_HDMI_HDCPSTATE_AUTH_INIT,
    VAL_HDMI_HDCPSTATE_AUTH_SUCC,
    VAL_HDMI_HDCPSTATE_AUTH_FAIL

} VAL_HdmiHdcpStateT;

typedef void (*VAL_HdmiHdcpStateCallbackT)(const VAL_HdmiHdcpStateT state,
                                         void* arg);

/*!
*******************************************************************************
** \brief Parameters for initialization of VAL module.
**
** This structure contains parameters that are passed when VAL is initialized.
*******************************************************************************
*/
typedef struct
{
    /*!
    ** Priority of HDMI connection thread. Set to 0 in order to disable thread
    ** usage. As an alternative, VAL_UpdateHDMIConnectionStatus can be used
    ** in order to detect HDMI connection events and proceed with HDCP
    ** authentication protocol.
    */
    uint32_t                             threadPriority;

} VAL_InitParamsT;
/*!
*******************************************************************************
** \brief Parameters for transition to "up" mode.
**
** This structure contains parameters that are passed when bringing the VAL
** system up, i.e. enabling the video output.
*******************************************************************************
*/
typedef struct
{
    VAL_HdEncoderT                       hdSystemType;
    VAL_SdEncoderT                       sdSystemType;
    int32_t                              displayAspectRatio[VAL_VOUT_NUMS];
    int32_t                              scalingMethod;
    uint8_t                              enableHDCP;   //!< Set to 1 to enable
                                                       //!< HDCP encryption via
                                                       //!< HDMI.

} VAL_UpParamsT;


/*!
*******************************************************************************
** \brief Access unit delimiter type for H264 decoding.
**
** This enumeration contains all internal delimiter modes.
*******************************************************************************
*/
typedef enum
{
    VAL_H264_AUD_TYPE_AUTO = 0,    //!< Auto detect delimiter
    VAL_H264_AUD_TYPE_AUD,         //!< AU is separated by AUD header
    VAL_H264_AUD_TYPE_NO_AUD       //!< AU is separated by PES Header

} VAL_H264AudTypeT;

/*!^M
 * *******************************************************************************^M
 * ** \brief API to set the stream segment Mode. For normal decoding it is .^M
 * **        Full segment and for ISDB-T it can be 1 segment
 * ** This enumeration contains all internal delimiter modes.^M
 * *******************************************************************************^M
 * */
typedef enum
{
    VAL_STREAM_SEGMENT_FULL = 0,    //!< Full stream segment for normal decoding
    VAL_STREAM_SEGMENT_1,           //!< 1 segment for ISDB-T
} VAL_StreamSegmentTypeT;



/*!
*******************************************************************************
** \brief Parameters for opening a VAL instance.
**
** This structure contains parameters that are passed when opening a VAL
** instance, i.e. preparing for video decoding.
*******************************************************************************
*/
typedef struct
{
    VAL_EnvironmentT                     environment;  //!< Decoder environment.
    VAL_DecoderTypeT                     decoder;      //!< Requested decoder.
    uint8_t                              enableAfd;    //!< AFD enable request.
    VAL_H264AudTypeT                     h264AudType;  //!< AU delimiter
                                                       //!< type, needed for
                                                       //!< H.264 video only.
                                                       //!< Use TYPE_AUTO
                                                       //!< option for default
                                                       //!< broadcast streams.
} VAL_OpenParamsT;

/*!
*******************************************************************************
** \brief Internal data valid during "up" phase, internally accessed via
**        pointer returned by VAL_GoUp.
*******************************************************************************
*/
typedef struct
{
    FAPEX_SYS_SemaphoreT                 mutexHandle;
    FAPEX_SYS_SemaphoreT                 triggerHandle;
    FAPEX_SYS_ThreadT                    threadHandle;
    uint8_t                              threadSleepReq;
    uint8_t                              threadSleepAck;

    uint8_t                              up;
    void*                                openPtr;
    
    int32_t                              viencId;
    int32_t                              viencSubId[VAL_VOUT_NUMS];
    int32_t                              vioutId;
    int32_t                              vioutSubId[VAL_VOUT_NUMS];
    int32_t                              hdmiId;
    
    VAL_HdEncoderT                       appHdSystemType;
    VAL_SdEncoderT                       appSdSystemType;
    VAL_HdEncoderT                       viencHdEncoder;
    VAL_SdEncoderT                       viencSdEncoder;
    FAPEX_VIOUT_ScalerParamStrT          vioutParams[VAL_VOUT_NUMS];
    int32_t                              vioutDispAr[VAL_VOUT_NUMS];
    int32_t                              vioutHdScalingMethod;
    uint8_t                              videoBlanked;
    void                                 (*vioutParamChangeCb)(void *, void*);
    void*                                vioutParamChangeCbArg;
    
    uint8_t                              hdmiHDCPEnabled;
    VAL_HdmiHdcpStateT                   hdmiHDCPState;
    VAL_HdmiHdcpStateCallbackT           hdmiHDCPStateCb;
    void*                                hdmiHDCPStateCbArg;
    VAL_HDMIConnectParamsT               hdmiConnectParams;
    int32_t                              hdmiPictureAr;
    uint8_t                              hdmiActiveFormat;
    FAPEX_HDMI_EdidInfoStrT              hdmiEdid;
    VAL_HDMIConnectCallbackT             hdmiConnectCb;
    void*                                hdmiConnectCbArg;

} VAL_DataT;

/*!
*******************************************************************************
** \brief Internal data valid during "open" phase, internally accessed via
**        pointer returned by VAL_Open.
*******************************************************************************
*/
typedef struct
{
    VAL_OpenParamsT                      openParams;
    VAL_DataT*                           dataPtr;
    
    int32_t                              tsdId;
    int32_t                              videcId;
    
    VAL_DecoderTypeT                     decoder;    //!< Current decoder.
    uint32_t                             startFlags;
    
    void                                 (*frameAvlCb)(void* optData);
    void*                                frameAvlCbArg;
    void                                 (*paramChangeCb)
                                         (FAPEX_VIDEC_DecFrameInfoStrT* frameInfoPtr,
                                          void* optData);
    void*                                paramChangeCbArg;

    uint32_t                             bufferSize;

    VAL_OpEnumT                          operation;
    uint32_t                             pid;
    uint32_t                             pcrFlag;
    VAL_PlayModeEnumT                    mode;
    uint8_t                              frameAvailable;
    uint8_t                              frameDecoded;
    VAL_StatusEnumT                      decoderStatus;
    uint32_t                             delimiterType; //!< delimiter type for
                                                        //!< H264 decoding only.
    uint64_t                             h264AudTimeOut;

    FAPEX_VIDEC_DecFrameInfoStrT         videcFrameInfo;
    uint8_t                              videcFrameInfoAvailable;

    FAPEX_SYS_ThreadT                    checkThread;

} VAL_OpenT;

/*!
*******************************************************************************
** \brief Publicly available VAL status parameters.
**
** This structure contains VAL's public runtime parameters.
*******************************************************************************
*/
typedef struct
{
    VAL_EnvironmentT                     environment;   //!< Selected
                                                        //!< environment.
    VAL_OpEnumT                          operation;     //!< Current operation.
    VAL_PlayModeEnumT                    mode;          //!< Current playback
                                                        //!< mode.
    VAL_DecoderTypeT                     decoder;       //!< Current decoder.
    VAL_StatusEnumT                      decoderStatus; //!< Current decoder
                                                        //!< status.
} VAL_StatusT;

/*!
*******************************************************************************
** \brief .
*******************************************************************************
*/
typedef FAPEX_HDMI_EdidInfoStrT         VAL_HDMI_EdidInfoStrT;
typedef FAPEX_VIDEC_DecFrameInfoStrT    VAL_VIDEC_DecFrameInfoStrT;

//*****************************************************************************
//*****************************************************************************
//** Global Data
//*****************************************************************************
//*****************************************************************************

//*****************************************************************************
//*****************************************************************************
//** API Functions
//*****************************************************************************
//*****************************************************************************

#ifdef __cplusplus
extern "C" {
#endif

/* general interface */

int32_t          VAL_Init               (const VAL_InitParamsT* paramsPtr);
VAL_DataT*       VAL_GoUp               (const VAL_UpParamsT* paramsPtr,
                                         int32_t* errCodePtr);
int32_t          VAL_GoDown             (VAL_DataT* dataPtr);
int32_t          VAL_UpdateHDMIConnectionStatus
                                        (VAL_DataT* dataPtr,
                                         uint8_t* connectedPtr,
                                         uint8_t* changePtr);
int32_t          VAL_GetHDMIConnectionStatus (VAL_DataT* dataPtr,
                                         VAL_HDMIConnectParamsT* paramsPtr);
int32_t          VAL_SetHDMIConnectionCb(VAL_DataT* dataPtr,
                                         VAL_HDMIConnectCallbackT cb,
                                         void* cbArg);
int32_t          VAL_SetHdmiHdcpStateCb(VAL_DataT* dataPtr,
                                         VAL_HdmiHdcpStateCallbackT cb,
                                         void* cbArg);
int32_t          VAL_GetHDMI_EDID       (VAL_DataT* dataPtr,
                                         VAL_HDMI_EdidInfoStrT* edidPtr);
int32_t          VAL_SetOutputMode      (VAL_DataT* dataPtr,
                                         VAL_HdEncoderT hdSystemType,
                                         VAL_SdEncoderT sdSystemType);
int32_t          VAL_GetOutputMode      (VAL_DataT* dataPtr,
                                         VAL_HdEncoderT* hdSystemType,
                                         VAL_SdEncoderT* sdSystemType);
int32_t          VAL_SetOutputDelay     (VAL_DataT* dataPtr,
                                         int32_t outputDelayMsec);
int32_t          VAL_SetDisplayAspectRatio (VAL_DataT* dataPtr,
                                         VAL_VideoOutputT output,
                                         int32_t displayAr);
int32_t          VAL_GetDisplayAspectRatio (VAL_DataT* dataPtr,
                                         VAL_VideoOutputT output,
                                         int32_t* displayAr);
int32_t          VAL_SetHdScalingMethod (VAL_DataT* dataPtr,
                                         int32_t scalingMethod);
int32_t          VAL_GetHdScalingMethod (VAL_DataT* dataPtr,
                                         int32_t* scalingMethod);
int32_t          VAL_Resize             (VAL_DataT* dataPtr,
                                         uint32_t x0, uint32_t y0,
                                         uint32_t width, uint32_t height);
int32_t          VAL_Crop               (VAL_DataT* dataPtr,
                                         uint32_t x0, uint32_t y0,
                                         uint32_t width, uint32_t height);
int32_t          VAL_Blank              (VAL_DataT* dataPtr,
                                         uint8_t blankOrUnblank);
int32_t          VAL_SetScalingParamChangeCb(VAL_DataT* dataPtr,
                                         void (*paramChangeCb)(void *paramChangePtr, void* optData),
                                         void* paramChangeCbArg);
int32_t          VAL_HdmiUpdateAviInfoFrame(VAL_DataT* dataPtr, 
                                         int32_t pictureAr,
                                         uint8_t activeFormat);
int32_t          VAL_HdmiHdcpEnable (VAL_DataT* dataPtr,
                                         uint32_t onOffFlag);
int32_t          VAL_GetHdmiHdcpOnOffStatus(VAL_DataT* dataPtr);

int32_t          VAL_GetIdHdmi          (VAL_DataT* dataPtr);
int32_t          VAL_GetIdViencSd       (VAL_DataT* dataPtr);
int32_t          VAL_GetIdViencHd       (VAL_DataT* dataPtr);
int32_t          VAL_GetIdViscaleSd     (VAL_DataT* dataPtr);
int32_t          VAL_GetIdViscaleHd     (VAL_DataT* dataPtr);

VAL_OpenT*       VAL_Open               (VAL_DataT* dataPtr,
                                         const VAL_OpenParamsT* paramsPtr,
                                         int32_t* errCodePtr);
int32_t          VAL_Close              (VAL_OpenT* openPtr);

int32_t          VAL_Task               (VAL_OpenT* openPtr);
int32_t          VAL_SetFrameAvailableCb(VAL_OpenT* openPtr,
                                         void (*frameAvlCb)(void* optData),
                                         void* frameAvlCbArg);
int32_t          VAL_SetParamChangeCb   (VAL_OpenT* openPtr,
                                         void (*paramChangeCb)(VAL_VIDEC_DecFrameInfoStrT *, void*),
                                         void* paramChangeCbArg);
int32_t          VAL_Start              (VAL_OpenT* openPtr, uint32_t pid,
                                         VAL_PlayModeEnumT mode, uint32_t pcrFlag);
int32_t          VAL_Stop               (VAL_OpenT* openPtr, uint8_t freeze);
int32_t          VAL_Freeze             (VAL_OpenT* openPtr, uint8_t freezeOn);
int32_t          VAL_Reboot             (VAL_OpenT* openPtr,
                                         const VAL_DecoderTypeT* decoderPtr);
int32_t          VAL_GetStatus          (VAL_OpenT* openPtr,
                                         VAL_StatusT* statusPtr);
int32_t          VAL_GetVideoFrameInfo  (VAL_OpenT* openPtr,
                                         VAL_VIDEC_DecFrameInfoStrT*
                                         videcInfoPtr);

int32_t          VAL_Recover            (VAL_OpenT* openPtr);
int32_t          VAL_GetBufferSize      (VAL_OpenT* openPtr, uint32_t* sizePtr);
int32_t          VAL_GetBufferLevel     (VAL_OpenT* openPtr, uint32_t*
                                         levelPtr);
int32_t          VAL_IsFrameAvailable   (VAL_OpenT* openPtr, uint8_t* avlPtr);

int32_t          VAL_GetIdTsd           (VAL_OpenT* openPtr);
int32_t          VAL_GetIdVidec         (VAL_OpenT* openPtr);

/* specific for I-frame presentation operation */

int32_t          VAL_IFrameDecode       (VAL_OpenT* openPtr,
                                         const uint32_t* dataPtr,
                                         uint32_t sizeInBytes);
int32_t          VAL_IFrameDisplay      (VAL_OpenT* openPtr);

/* specific for PVR playback operation */

int32_t          VAL_PlayChangeMode     (VAL_OpenT* openPtr,
                                         VAL_PlayModeEnumT mode);
int32_t          VAL_PlayShowIFrame     (VAL_OpenT* openPtr,
                                         const uint32_t* dataPtr,
                                         uint32_t dataBytes);
int32_t          VAL_PlayPrepareNextFrame(VAL_OpenT* openPtr);
int32_t          VAL_SetStreamSegment(VAL_OpenT* openPtr,
                      VAL_StreamSegmentTypeT segmentType);
int32_t          VAL_SetBrightness(VAL_OpenT* openPtr, int32_t level);
int32_t          VAL_SetContrast(VAL_OpenT* openPtr,int32_t level);
int32_t          VAL_SetSaturation(VAL_OpenT* openPtr,int32_t level);

#ifdef __cplusplus
}
#endif

#endif /* VAL_H */
