/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_guided.c
**
**              $Id: app_guided.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Guided installation mode management.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
**
*****************************************************************************
*/
/*lint -save -e641 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/* NOTE: Assumption: Max. number of frontends is 3 */
#define MENU_GUIDED_MENUS_MAX     9


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    MENU_GUIDED_S   guidedMenu[MENU_GUIDED_MENUS_MAX];
    int32_t             guidedNums;     //!< Number of menues in guided mode
    int32_t             guidedCurr;     //!< Current index in guided mode

} MENU_GUIDED_INST_S;


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC MENU_GUIDED_INST_S* menuGuidedDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

int32_t MENU_GuidedInit (void)
{
    if ( menuGuidedDat != NULL ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    menuGuidedDat = UTILS_Malloc(sizeof(MENU_GUIDED_INST_S));
    if ( menuGuidedDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    return MENU_GuidedClear();
}

int32_t MENU_GuidedClear (void)
{
     MENU_GUIDED_INST_S* inst = menuGuidedDat;

     if ( inst == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

     memset(menuGuidedDat, 0, sizeof(MENU_GUIDED_INST_S));

     inst->guidedCurr = -1;

     return FAPI_OK;
}

int32_t MENU_GuidedAdd (const MENU_GUIDED_S* guidedParams)
{
    MENU_GUIDED_INST_S* inst = menuGuidedDat;

    if ( inst == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if ( inst->guidedNums >= MENU_GUIDED_MENUS_MAX )
    {
        return SMARTGO_APPL_ERR_CAPACITY;
    }

    inst->guidedMenu[inst->guidedNums++] = *guidedParams;

    return FAPI_OK;
}

fbool_t MENU_GuidedStart (void)
{
    MENU_GUIDED_INST_S* inst = menuGuidedDat;
    fbool_t                retVal;

    if ( inst == NULL ) return FFALSE;

    inst->guidedCurr = 0;

    retVal = MENUSTACK_Up (inst->guidedMenu[inst->guidedCurr].entry,
                          (FAPI_SYS_HandleT)
                          (&(inst->guidedMenu[inst->guidedCurr].params)));

    if ( retVal == FFALSE ) inst->guidedCurr--;
    return retVal;
}

fbool_t MENU_GuidedGotoNext (void)
{
    MENU_GUIDED_INST_S* inst = menuGuidedDat;
    fbool_t                retVal;

    DBG_Assert(inst != NULL);
    if(inst == NULL)return FFALSE; /* to satisfy MISRA */

    if ( (inst->guidedCurr < 0)                       ||
         ((inst->guidedCurr + 1) >= inst->guidedNums) ||
         (inst->guidedMenu[inst->guidedCurr + 1].entry == NULL) )
    {
        /* return from guided menus if next menu does not exist */
        retVal = MENUSTACK_Back(NULL);

        if ( retVal == FTRUE )
        {
            inst->guidedCurr = -1;
        }
    }
    else
    {
        inst->guidedCurr++;

        retVal = MENUSTACK_Switch (inst->guidedMenu[inst->guidedCurr].entry,
                                  (FAPI_SYS_HandleT)
                                  (&(inst->guidedMenu[inst->guidedCurr].params)),
                                  -1);

        if ( retVal == FFALSE ) inst->guidedCurr--;
    }

    return retVal;
}

fbool_t MENU_GuidedGotoPrevious (void)
{
    MENU_GUIDED_INST_S* inst = menuGuidedDat;
    fbool_t                retVal;

    DBG_Assert(inst != NULL);
    if(inst == NULL)return FFALSE; /* to satisfy MISRA */

    if ( (inst->guidedCurr <= 0)                ||
         (inst->guidedCurr >= inst->guidedNums) ||
         (inst->guidedMenu[inst->guidedCurr - 1].entry == NULL) )
    {
        /* return from guided menus if previous menu does not exist */
        retVal = MENUSTACK_Back(NULL);

        if ( retVal == FTRUE )
        {
            inst->guidedCurr = -1;
        }
    }
    else
    {
        inst->guidedCurr--;

        retVal = MENUSTACK_Switch (inst->guidedMenu[inst->guidedCurr].entry,
                                  (FAPI_SYS_HandleT)
                                  (&(inst->guidedMenu[inst->guidedCurr].params)),
                                  -1);

        if ( retVal == FFALSE ) inst->guidedCurr++;
    }
    return retVal;
}

uint32_t MENU_GuidedGetNextStr (void)
{
    MENU_GUIDED_INST_S* inst = menuGuidedDat;

    DBG_Assert(inst != NULL);
    if(inst == NULL)return STR_EMPTYSTRING; /* to satisfy MISRA */

    if ( (inst->guidedCurr + 1) >= inst->guidedNums )
    {
         return STR_ENTRY_ANTPROG_ENDOFGUIDED;
    }
    else if ( (inst->guidedCurr < 0) ||
              (inst->guidedMenu[inst->guidedCurr + 1].entry == NULL) )
    {
        return STR_EMPTYSTRING;
    }
    else
    {
        return inst->guidedMenu[inst->guidedCurr + 1].strIdx;
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
