/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_epg_ext.c
**
**              $Id: app_menu_epg_ext.c 5874 2010-03-02 14:42:55Z wlaris $
**
** \brief       Extended EPG menu implementation.
**
** Detail description. [\ref app_smartgo_app_ref_epg]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: loss of precision
** - Info    715: symbol 'anySymb' not referenced
** - Info    737: loss of sign in promotion from int to unsigned int.
** - Info    774: Boolean within 'if' always evaluates to True.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1,
**                dependence placed on C's operator precedence;
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e737 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"
#include "app_menu_info.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define OSDINF_EXT_LINES          10

#define EXTDESC_MAXLEN          4096
#define EXTDESC_MAXLINES          36


/* start pixels: left-aligned items: relative to title box */
#define OSDINF_TITLE_X        (SCALER_GetCurrVal(MAIN_TEXT_PAD_X)+ \
                               SCALER_GetCurrVal(INFO_NUM_WIDTH) + \
                               SCALER_GetCurrVal(INNER_FRAME_WIDTH) + \
                               (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X)))

/* start pixels: right-aligned items: relative to title box end */
#define OSDINF_TIME_X         (SCALER_GetCurrVal(MAIN_TEXT_PAD_X) + \
                               SCALER_GetCurrVal(INFO_TIME_WIDTH))

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
enum
{
    MENU_EPGEXT_ID_LIST,
    MENU_EPGEXT_ID_TITLE,
    MENU_EPGEXT_ID_SERVICENUM,
    MENU_EPGEXT_ID_TIME,
    MENU_EPGEXT_ID_REST,
    MENU_EPGEXT_ID_DESCBOX,
    MENU_EPGEXT_ID_STARTTIMEBOX,
    MENU_EPGEXT_ID_ENDTIMEBOX,
    MENU_EPGEXT_ID_SEPARATOR,
    MENU_EPGEXT_ID_DAYBOX,
    MENU_EPGEXT_ID_DATEBOX
};

typedef struct
{
    SYS_STRING_S     lineString;     //!< Line string (not terminated!).
    uint16_t         numBytes;       //!< Number of bytes in line.

} EXTDESC_LINE_S;


typedef struct
{
    FAPI_SYS_HandleT    pnlHdl;
    FAPI_SYS_HandleT    lstHdl;
    FAPI_SYS_HandleT    descHdl;

    MENU_EPGEXT_OPEN_S  data;
    FGS_PANEL_MEMBER_S* pItem;
    FGS_POS_S           itemPos;

    SYS_STRING_S        eitExtCurr;
    char_t              eitExtBuf[EXTDESC_MAXLEN];    //!< Buffer for ext. event
    uint32_t            eitExtLen;                    //!< size of ext. event desc.
    EXTDESC_LINE_S      eitExtLine[EXTDESC_MAXLINES]; //!< line info for ext. desc.
    uint32_t            eitExtNumLines;               //!< number of lines
    uint16_t            eitExtPixPerLine;             //!< max. pixels per line

    EIT_EVT_S           event;
    fbool_t             eventValid;

} MENU_EPGEXT_INST_S;


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
FSTATIC MENU_EPGEXT_INST_S*    epgExtDat   = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_EpgExtLeave       (FAPI_SYS_HandleT h,
                                        fbool_t force);
FSTATIC void    MENU_EpgExtOnFocusSet  (FAPI_SYS_HandleT h,
                                        fbool_t hasFocus);
FSTATIC fbool_t MENU_EpgExtKeys        (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_EpgExtTrans       (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_EpgExtChgEvent    (FAPI_SYS_HandleT h);
FSTATIC void    MENU_EpgExtGetLine     (FAPI_SYS_HandleT h,
                                        int32_t id,
                                        uint32_t index,
                                        FGS_ITEMSTATE_E state,
                                        FGS_ITEM_S* item);
FSTATIC int32_t MENU_EpgExtDraw        (FAPI_SYS_HandleT h);
FSTATIC void    MENU_EpgExtGetItem     (FAPI_SYS_HandleT h,
                                        int32_t id,
                                        uint32_t index,
                                        FGS_ITEMSTATE_E state,
                                        FGS_ITEM_S* item);
FSTATIC void    MENU_EpgExtParseExtDesc(MENU_EPGEXT_INST_S* inst,
                                        uint8_t font);
FSTATIC fbool_t MENU_EpgExtLineParsed  (uint16_t lineIdx,
                                        SYS_STRING_S lineStart,
                                        uint16_t lineBytes);

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/
/*!
******************************************************************************
** \brief   Initialize the extended EPG Menu
**
** This function allocat the memory for the menu handle \p epgExtDat and
** clears the handle structure.
**
** \return  one of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_ALREADY_INITIALIZED
**          - #APPL_SMARTGO_ERR_OUT_OF_MEMORY
**
******************************************************************************
*/
int32_t MENU_EpgExtInit (void)
{
    if ( epgExtDat != NULL )
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    epgExtDat = UTILS_Malloc(sizeof(MENU_EPGEXT_INST_S));
    if ( epgExtDat == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    memset (epgExtDat, 0, sizeof(MENU_EPGEXT_INST_S));

    STR_Clear(&(epgExtDat->eitExtCurr));

    epgExtDat->eitExtLen      = 0;
    epgExtDat->eitExtNumLines = 0;
    epgExtDat->eventValid     = FFALSE;

    return FAPI_OK;
}

/*!
******************************************************************************
** \brief   With this function you enter the Extended EPG menu.
**
** This function is called twice. First with \p test flag #FTRUE to check if
** the menu can be opend. If the first call was successful and return #FTRUE
** the second call will open the menu (\p test flag #FFALSE).
**
** \param   h       Handle of the Extended EPG Instants.
** \param   test    A flag to test if opening this menu is possible.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
fbool_t MENU_EpgExtEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    FGS_PANEL_OPEN_S     pnlPars;
    FGS_PANEL_MEMBER_S   member;
    FGS_LNAVI_OPEN_S     scrollDat;
    FGS_BOX_OPEN_S       boxPars;
    MENU_EPGEXT_INST_S*  inst = epgExtDat;
    int32_t              retVal;
    MENU_EPGEXT_OPEN_S*  openParams = (MENU_EPGEXT_OPEN_S*)h;

    if ( ( inst         == NULL ) ||
         ( inst->pnlHdl != NULL ) ||  // test if already open
         ( openParams   == NULL ) )
    {
        return FFALSE;
    }

    if ( test == FTRUE )
    {
        return FTRUE;
    }

    memset(&pnlPars,   0, sizeof(FGS_PANEL_OPEN_S));
    memset(&scrollDat, 0, sizeof(FGS_LNAVI_OPEN_S));
    memset(&member,    0, sizeof(member));

    inst->data             = *openParams;
    inst->eitExtPixPerLine = SCALER_GetCurrVal(EPG_EXT_WIDTH) -
                             (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH)) -
                             SCALER_GetCurrVal(SCROLLBAR_WIDTH) -
                             (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X)) - 4;

    do
    {
        pnlPars.bkgrType      = OSD_PNLTYPE_EPGEXT;
        pnlPars.pos.width     = SCALER_GetCurrVal(EPG_EXT_WIDTH);
        pnlPars.pos.height    = (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH)) +
                                OSD_GetPanelTitleHeight(pnlPars.bkgrType) +
                                (2 * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                                (1 * SCALER_GetCurrVal(INNER_FRAME_WIDTH)) +
                                OSD_GetListHeight(OSDINF_EXT_LINES);
        pnlPars.pos.x         = 0;
        pnlPars.pos.y         = -((int16_t)(SCALER_GetCurrVal(MAIN_PAD_Y)
                                + pnlPars.pos.height));
        pnlPars.pos.alignment = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_BOTTOM;
        pnlPars.onFocusSet    = MENU_EpgExtOnFocusSet;
        pnlPars.onFocusSetArg = (FAPI_SYS_HandleT)inst;
        pnlPars.pInst         = &(appDat->pnlInst);

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_EPGEXT, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create scroll list ---------------------------------------- */

        scrollDat.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        scrollDat.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH) +
                                    OSD_GetPanelTitleHeight(pnlPars.bkgrType) +
                                    (2 * SCALER_GetCurrVal(LIST_LINE_HEIGHT)) +
                                    (1 * SCALER_GetCurrVal(INNER_FRAME_WIDTH));
        scrollDat.pos.width       = pnlPars.pos.width -
                                    (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH));
        scrollDat.pos.height      = OSD_GetListHeight(OSDINF_EXT_LINES);
        scrollDat.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        scrollDat.bkgrType        = OSD_NAVIBKGR_NORMAL |
                                    OSD_CORNER_BOT_LEFT | OSD_CORNER_BOT_RIGHT;
        scrollDat.type            = FGS_LNAVI_TYPE_TXTVIEW;
        scrollDat.rowHeight       = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        scrollDat.scrollbarWidth  = SCALER_GetCurrVal(SCROLLBAR_WIDTH);
        scrollDat.numVisLines     = OSDINF_EXT_LINES;
        scrollDat.numColumns      = 1;
        scrollDat.params.numItems = 0;
        scrollDat.params.index    = 0;
        scrollDat.params.posInVis = 0;
        scrollDat.colStart[0]     = SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        scrollDat.topDownOrient   = FTRUE;
        scrollDat.wraparound      = FFALSE;
        scrollDat.getItem         = MENU_EpgExtGetLine;
        scrollDat.getItemArg      = (FAPI_SYS_HandleT)inst;
        scrollDat.pInst           = &(appDat->lnaviInst);

        retVal = FGS_LNAVI_Open (&scrollDat, MENU_EPGEXT_ID_LIST,
                                 inst->pnlHdl, &(inst->lstHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create service number textbox ----------------------------- */

        retVal = MENU_InfoServiceNumBoxCreate(inst->pnlHdl, MENU_EPGEXT_ID_SERVICENUM,
                                          MENUID_EPGEXT);
        if ( retVal != FAPI_OK ) break;

        /* ----- create service name textbox ------------------------------- */

        retVal = MENU_InfoServiceNameBoxCreate(inst->pnlHdl,
                                  pnlPars.pos.width -
                                  (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH)) -
                                  OSDINF_TITLE_X - OSDINF_TIME_X -
                                  SCALER_GetCurrVal(INNER_FRAME_WIDTH) -
                                  (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X)),
                                  MENU_EPGEXT_ID_TITLE,
                                  MENUID_EPGEXT);
        if ( retVal != FAPI_OK ) break;

        /* ----- create time textbox --------------------------------------- */

        retVal = MENU_InfoTimeBoxCreate(inst->pnlHdl, MENU_EPGEXT_ID_TIME);
        if ( retVal != FAPI_OK ) break;

        /* ----- Create EIT description text box --------------------------- */

        memset(&boxPars, 0, sizeof(boxPars));

        boxPars.pos.x           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                                + SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        boxPars.pos.y           = SCALER_GetCurrVal(OUTER_FRAME_WIDTH)
                                + OSD_GetPanelTitleHeight(pnlPars.bkgrType);
        boxPars.pos.width       = pnlPars.pos.width
                                - (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH))
                                - (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X));
        boxPars.pos.height      = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.alignment   = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType        = OSD_BOX_BKGR_TITLE;
        boxPars.getItem         = MENU_EpgExtGetItem;
        boxPars.getItemArg      = (FAPI_SYS_HandleT)inst;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPGEXT_ID_DESCBOX,
                                inst->pnlHdl, &(inst->descHdl));
        if ( retVal != FAPI_OK ) break;

        /*! \note
         * Create some boxes to show starttime, endtime, day and date in one
         * row.
         * 12:34  -  23:45  Sunday  2009-07-26
         * -----  -  -----  ------  ----------
         *   |    |    |       |         |
         *   |    |    |       |         --- MENU_EPGEXT_ID_DATEBOX
         *   |    |    |       ------------- MENU_EPGEXT_ID_DAYBOX
         *   |    |    --------------------- MENU_EPGEXT_ID_ENDTIMEBOX
         *   |    -------------------------- MENU_EPGEXT_ID_SEPARATOR
         *   ------------------------------- MENU_EPGEXT_ID_STARTTIMEBOX
         *
         */
        boxPars.pos.y          += boxPars.pos.height;
        boxPars.pos.width       = SCALER_GetCurrVal(EPG_EXT_TIME_WIDTH);
        retVal = FGS_BOX_Open (&boxPars, MENU_EPGEXT_ID_STARTTIMEBOX,
                                inst->pnlHdl, &(inst->descHdl));

        boxPars.pos.x          += boxPars.pos.width;
        boxPars.pos.width       = SCALER_GetCurrVal(EPG_EXT_SEPARATOR_WIDTH);
        retVal = FGS_BOX_Open (&boxPars, MENU_EPGEXT_ID_SEPARATOR,
                                inst->pnlHdl, &(inst->descHdl));
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x          += boxPars.pos.width;
        boxPars.pos.width       = SCALER_GetCurrVal(EPG_EXT_TIME_WIDTH);
        retVal = FGS_BOX_Open (&boxPars, MENU_EPGEXT_ID_ENDTIMEBOX,
                                inst->pnlHdl, &(inst->descHdl));
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x          += boxPars.pos.width;
        boxPars.pos.width       = SCALER_GetCurrVal(EPG_EXT_DAY_WIDTH);
        retVal = FGS_BOX_Open (&boxPars, MENU_EPGEXT_ID_DAYBOX,
                                inst->pnlHdl, &(inst->descHdl));
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.x          += boxPars.pos.width +
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        boxPars.pos.width       = SCALER_GetCurrVal(EPG_EXT_DATE_WIDTH);
        retVal = FGS_BOX_Open (&boxPars, MENU_EPGEXT_ID_DATEBOX,
                                inst->pnlHdl, &(inst->descHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

        inst->itemPos.x         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        inst->itemPos.y         = SCALER_GetCurrVal(OUTER_FRAME_WIDTH);
        inst->itemPos.width     = pnlPars.pos.width -
                                  (2 * SCALER_GetCurrVal(OUTER_FRAME_WIDTH));
        inst->itemPos.height    = SCALER_GetCurrVal(TITLE_LINE_HEIGHT);
        inst->itemPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h             = (FAPI_SYS_HandleT)inst;
        member.pos           = &(inst->itemPos);
        member.id            = MENU_EPGEXT_ID_REST;
        member.isHidden      = FFALSE;
        member.setFocus      = NULL;
        member.show          = MENU_EpgExtDraw;
        member.close         = NULL;

        retVal = FGS_PANEL_AddMember(inst->pnlHdl, &member,
                                     &(inst->pItem));
        if ( retVal != FAPI_OK ) break;

        /* ----------------------------------------------------------------- */

        MENU_InfoServiceNumBoxUpdate();
        MENU_EpgExtUpdate(NULL);

        (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_EPGEXT_ID_LIST);
        (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_EPGEXT);
        (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE);

        /* Raise minimum key repeat interval for smoother zapping */
        (void)KEYIN_SetRepeatInterval (125, 125);
    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL )
        {
            (void)FGS_PANEL_Close(inst->pnlHdl);
        }

        inst->pnlHdl = NULL;
        return FFALSE;
    }

    MENUSTACK_SetCleanup (MENU_EpgExtLeave, (FAPI_SYS_HandleT)inst);
    return FTRUE;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   params  Update informations.
**
******************************************************************************
*/
void MENU_EpgExtUpdate (const MENU_EPGEXT_OPEN_S* params)
{
    MENU_EPGEXT_INST_S* inst = epgExtDat;
    EIT_EXTEVT_S        extDesc;
    FGS_LNAVI_PARAMS_S  naviParams;
    SYS_PRESSTATE_E     presState;
    SYS_SERVLOC_S*      pServLoc   = NULL;
    fbool_t             getEvtData = FTRUE;
    int32_t             retVal     = FAPI_OK;

    if ( ( inst         == NULL ) ||
         ( inst->pnlHdl == NULL ) )
    {
        return;
    }

    if ( params != NULL )
    {
        inst->data = *params;
    }

    STR_Clear(&(inst->eitExtCurr));

    inst->eitExtLen      = 0;
    inst->eitExtNumLines = 0;

    inst->eventValid     = FFALSE;

    /* ----- get event data ------------------------------------------------ */

    memset (&extDesc, 0, sizeof(extDesc));

    extDesc.pBuf    = inst->eitExtBuf;
    extDesc.maxLen  = EXTDESC_MAXLEN;

    if ( SYS_GetPresentationState(&presState) != FAPI_OK ) return;

    switch ( presState )
    {
        default:
        case SYS_PRESSTATE_NONE:
            getEvtData = FFALSE;
            break;

        case SYS_PRESSTATE_LIVETV:
            pServLoc = MENU_InfoGetCurrServLoc();
            break;

        case SYS_PRESSTATE_PLAYBACK:
            pServLoc = NULL;
            break;
    }

    if ( getEvtData )
    {
        if ( inst->data.schedMode )
        {
            retVal = EIT_GetSchedByReference (0, &(inst->event), &extDesc,
                                              &(inst->data.currRef));
        }
        else
        {
            retVal = EIT_GetEvent (pServLoc, inst->data.currType,
                                   &(inst->event));

            if ( retVal == FAPI_OK )
            {
                (void)EIT_GetExtEvt (pServLoc, inst->data.currType, &(extDesc));
            }
        }

        if ( retVal == FAPI_OK )
        {
            inst->eventValid = FTRUE;
        }

        if ( extDesc.descStrLen > 0 )
        {
            inst->eitExtCurr = extDesc.descStr;
            inst->eitExtLen  = extDesc.descStrLen;

            MENU_EpgExtParseExtDesc(inst, MAIN_FONT);
        }
    }

    /* ----- update EIT description text box ------------------------------- */

    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPGEXT_ID_DESCBOX);
    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPGEXT_ID_STARTTIMEBOX);
    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPGEXT_ID_SEPARATOR);
    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPGEXT_ID_ENDTIMEBOX);
    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPGEXT_ID_DAYBOX);
    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPGEXT_ID_DATEBOX);

    if ( FGS_LNAVI_GetParams(inst->lstHdl, &naviParams) == FAPI_OK )
    {
        naviParams.numItems = inst->eitExtNumLines;
        naviParams.index    = 0;

        (void)FGS_LNAVI_SetParams(inst->lstHdl, &naviParams);
    }
}

/*!
******************************************************************************
** \brief   This function is a callback for the menustack. It close the
**          extended EPG.
**
** \param   h       Handle of the Extended EPG Instants.
** \param   force   (unused)
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgExtLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_EPGEXT_INST_S* inst = (MENU_EPGEXT_INST_S*)h;

    DBG_Assert(inst->pnlHdl != 0);

    /* Restore minimum key repeat interval */
    (void)KEYIN_SetRepeatInterval (200, 100);

    if ( inst->pnlHdl != NULL )
    {
        (void)FGS_PANEL_Close(inst->pnlHdl);
    }

    MENU_InfoTimeBoxRemove();
    MENU_InfoServiceNumBoxRemove();
    MENU_InfoListSymbolBoxRemove();
    MENU_InfoServiceNameBoxRemove();

    MENU_InfoServiceNumBoxInputAbort(); /* reset service number input mode */

    (void)MENUSTACK_FocusRelease((int32_t)MENUID_EPGEXT);

    inst->pnlHdl = NULL;
    return FTRUE;
}

/*!
******************************************************************************
** \brief   This function is a callback for the menustack. It sets or remove
**          the focus to the extended EPG and allocatets the keys.
**
** The following Keys are used by this menu:
** - #FGS_KEY_EXIT      -> To close all opend menus.
** - #FGS_KEY_BACK      -> Go back to the previous menu.
** - #FGS_KEY_MENU      -> Go back to the previous menu.
** - #FGS_KEY_NUMBER    ->
** - #FGS_KEY_NAVI      ->
** - #FGS_KEY_OK        ->
** - #FGS_KEY_INFO      ->
** - #FGS_KEY_LEFT      ->
** - #FGS_KEY_RIGHT     ->
** - #FGS_KEY_CHAN_UP   -> Channel up
** - #FGS_KEY_CHAN_DOWN -> Channel down
** - #FGS_KEY_TV        -> Switch between TV and Radio list
**
** \param   h           Handle (unused).
** \param   hasFocus    #FTRUE set the focus to this menu #FFALSE remove the
**                      focus.
**
******************************************************************************
*/
FSTATIC void MENU_EpgExtOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_EPGEXT_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleZappingKeys(hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler   (FGS_KEY_MENU,   MAIN_DirectMenuTrans,    NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_BACK,   MENUSTACK_Exit,          NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_EXIT,   MENUSTACK_Exit,          NULL, FTRUE, FFALSE);
        FGS_SetAppNumKeyHandler(FGS_KEY_NUMBER, MENU_InfoServiceNumBoxInput, NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_NAVI,   MENU_EpgExtTrans,        h,    FTRUE, FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_OK,     MENU_EpgExtKeys,         h,    FTRUE, FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_INFO,   MAIN_DirectMenuTrans,    NULL, FTRUE, FFALSE);
        FGS_SetAppKeyHandler   (FGS_KEY_LEFT,   MENU_EpgExtChgEvent,     h,    FTRUE, FTRUE);
        FGS_SetAppKeyHandler   (FGS_KEY_RIGHT,  MENU_EpgExtChgEvent,     h,    FTRUE, FTRUE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_MENU);
        FGS_ClearAppKeyHandler(FGS_KEY_BACK);
        FGS_ClearAppKeyHandler(FGS_KEY_EXIT);
        FGS_ClearAppKeyHandler(FGS_KEY_NUMBER);
        FGS_ClearAppKeyHandler(FGS_KEY_NAVI);
        FGS_ClearAppKeyHandler(FGS_KEY_OK);
        FGS_ClearAppKeyHandler(FGS_KEY_INFO);
        FGS_ClearAppKeyHandler(FGS_KEY_LEFT);
        FGS_ClearAppKeyHandler(FGS_KEY_RIGHT);
    }
}

/*!
*******************************************************************************
**
** \brief   Handles the Keys adapted by this menu.
**
** This function is used as callback for the FGS_SetAppKeyHandler(). and
** handles the following Keys adapted by the extended EPG:
** - #FGS_KEY_OK
**
** \param   h   Address of extended EPG handle.
**
** \return  One of the following status codes:
**          - #FTRUE    If the key was used by this menu.
**          - #FFALSE   If the key wasn't used.
**
** \sa
**          - FGS_SetAppKeyHandler()
**          - MENU_ExtEpgOnFocusSet()
**
*******************************************************************************-
*/
FSTATIC fbool_t MENU_EpgExtKeys (FAPI_SYS_HandleT h)
{
    fbool_t keyUsed = FTRUE;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_OK:
            if(MENU_InfoIsInputMode() == FTRUE)
            {
                MAIN_SelectServiceByInput();
                keyUsed = FTRUE;
            }
            else
            {
                keyUsed = MENU_EpgExtTrans(h);
            }
            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}

/*!
******************************************************************************
**
** \brief   Handles the direct transfer between SmartGo-HD EPG's.
**
** This function is used as callback for the FGS_SetAppKeyHandler() and
** handles the following Keys adapted by the extended EPG:
** - #FGS_KEY_OK    only if not in Service number input mode.
** - #FGS_KEY_NAVI
**
** \param   h   Address of extended EPG handle.
**
** \return  One of the following status codes:
**          - #FTRUE    If the key was used by this menu.
**          - #FFALSE   If the key wasn't used.
**
** \sa
**          - FGS_SetAppKeyHandler()
**          - MENU_ExtEpgOnFocusSet()
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgExtTrans (FAPI_SYS_HandleT h)
{
    MENU_EPGEXT_INST_S* inst = (MENU_EPGEXT_INST_S*)h;
    MENUSTACK_ENTRY     newEntry;
    FAPI_SYS_HandleT    arg = NULL;

    if(h == NULL)
    {
        return FFALSE;
    }
    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_NAVI:
            if(inst->data.calledFrom != NULL)
            {
                newEntry = inst->data.calledFrom;
            }
            else
            {
                newEntry = MENU_EpgSchedEntry;
            }
            break;

        default:
            return FFALSE;
    }

    return MENUSTACK_Switch(newEntry, arg, -1);
}

/*!
******************************************************************************
**
** \brief   Handles the Keys adapted by this menu.
**
** This function is used as callback for the FGS_SetAppKeyHandler(). and
** handles the following Keys adapted by the extended EPG:
** - #FGS_KEY_LEFT
** - #FGS_KEY_RIGHT
**
** \param   h   Address of extended EPG handle.
**
** \return  One of the following status codes:
**          - #FTRUE    If the key was used by this menu.
**          - #FFALSE   If the key wasn't used.
**
** \sa
**          - FGS_SetAppKeyHandler()
**          - MENU_ExtEpgOnFocusSet()
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgExtChgEvent (FAPI_SYS_HandleT h)
{
    MENU_EPGEXT_INST_S* inst = (MENU_EPGEXT_INST_S*)h;
    FGS_KEY_E           key;
    fbool_t             retVal = FTRUE;

    key = FGS_GetCurrentKeyCode();

    if ( !(inst->data.schedMode) )
    {
        inst->data.currType = (inst->data.currType == EIT_TYPE_CURR) ?
                               EIT_TYPE_NEXT : EIT_TYPE_CURR;
        retVal = FTRUE;
    }
    else
    {
        if ( key == FGS_KEY_LEFT )
        {
            if ( EIT_GetSchedPrevious(NULL, NULL, &(inst->data.currRef))
                 != FAPI_OK )
            {
                retVal = FFALSE;
            }
        }
        else
        {
            if ( EIT_GetSchedNext(NULL, NULL, &(inst->data.currRef))
                 != FAPI_OK )
            {
                retVal = FFALSE;
            }
        }
    }

    if ( retVal == FTRUE )
    {
        MENU_EpgExtUpdate(NULL);
    }
    return retVal;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the Extended EPG Instants.
** \param   id      The item identifier.
** \param   index   TODO
** \param   state   The status of the item, FGS_FOCUS; FGS_NONFOCUS or
**                  FGS_DISABLED
** \param   item    Pointer to the item structure.
**
******************************************************************************
*/
FSTATIC void MENU_EpgExtGetLine (FAPI_SYS_HandleT h,
                                 int32_t id,
                                 uint32_t index,
                                 FGS_ITEMSTATE_E state,
                                 FGS_ITEM_S* item)
{
    MENU_EPGEXT_INST_S* inst = (MENU_EPGEXT_INST_S*)h;

    if ( index < inst->eitExtNumLines )
    {
        (void)STR_BufNAppend (NULL, 0,
                              inst->eitExtLine[index].lineString,
                              MAIN_LINEBUFCHARSET,
                              inst->eitExtLine[index].numBytes);

        item[0].type                = FGS_ITEM_TYPE_STRING;
        item[0].align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
        item[0].item.str.chr        = STR_IntLineStr();
        item[0].item.str.font       = MAIN_FONT;
        item[0].item.str.lineHeight = 0;

        switch ( state )
        {
            default:
            case FGS_NONFOCUS:
                item[0].item.str.col = OSD_COL_TEXT;
                break;
            case FGS_FOCUS:
                item[0].item.str.col = OSD_COL_TEXT_HIGHLIGHTED;
                break;
            case FGS_DISABLED:
                item[0].item.str.col = OSD_COL_TEXT_DISABLED;
                break;
        }
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h   Handle of the Extended EPG Instants.
**
** \return  one of the following status codes:
**          - #FAPI_OK
**
******************************************************************************
*/
FSTATIC int32_t MENU_EpgExtDraw (FAPI_SYS_HandleT h)
{
    MENU_EPGEXT_INST_S* inst = (MENU_EPGEXT_INST_S*)h;

    /* Vertical separators in top line */
    OSD_DrawSeparatorVertical((uint16_t)inst->pItem->abspos.x +
                              OSDINF_TITLE_X -
                              SCALER_GetCurrVal(MAIN_TEXT_PAD_X) -
                              SCALER_GetCurrVal(INNER_FRAME_WIDTH),
                              (uint16_t)inst->pItem->abspos.y,
                              SCALER_GetCurrVal(TITLE_LINE_HEIGHT));

    OSD_DrawSeparatorVertical((uint16_t)inst->pItem->abspos.x +
                              inst->itemPos.width -
                              OSDINF_TIME_X -
                              SCALER_GetCurrVal(MAIN_TEXT_PAD_X) -
                              SCALER_GetCurrVal(INNER_FRAME_WIDTH),
                              (uint16_t)inst->pItem->abspos.y,
                              SCALER_GetCurrVal(TITLE_LINE_HEIGHT));

    return FAPI_OK;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the Extended EPG Instants.
** \param   id      The item identifier.
** \param   index   TODO
** \param   state   The status of the item, FGS_FOCUS; FGS_NONFOCUS or
**                  FGS_DISABLED
** \param   item    Pointer to the item structure.
**
******************************************************************************
*/
FSTATIC void MENU_EpgExtGetItem (FAPI_SYS_HandleT h,
                                 int32_t id,
                                 uint32_t index,
                                 FGS_ITEMSTATE_E state,
                                 FGS_ITEM_S* item)
{
    MENU_EPGEXT_INST_S* inst = (MENU_EPGEXT_INST_S*)h;
    SYS_TIME_S          tmpTime;
    SYS_DATE_S          tmpDate;
    uint32_t            evtDuration;
    uint32_t            bufPos = 0;
    uint32_t            strIdx;

    switch ( id )
    {
        case MENU_EPGEXT_ID_DESCBOX:

            if ( !(inst->data.schedMode) )
            {
                if ( inst->data.currType == EIT_TYPE_CURR )
                {
                    strIdx = STR_GNR_EVTCURR;
                }
                else
                {
                    strIdx = STR_GNR_EVTNEXT;
                }

                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(strIdx),
                                         MAIN_LINEBUFCHARSET);

                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(STR_GNR_COLON),
                                         MAIN_LINEBUFCHARSET);
            }

            if ( inst->eventValid )
            {
                bufPos += STR_BufAppend (NULL, bufPos,
                                         STR_Build(inst->event.nameCharSet,
                                                   inst->event.name),
                                         MAIN_LINEBUFCHARSET);
            }

            item->type                = FGS_ITEM_TYPE_STRING;
            item->align               = FGS_ALIGN_HOR_LEFT |
                                        FGS_ALIGN_VER_CENTER;
            item->item.str.chr        = STR_IntLineStr();
            item->item.str.font       = MAIN_FONT;
            item->item.str.lineHeight = 0;
            item->item.str.col        = OSD_COL_TITLE_TEXT;
            break;

        case MENU_EPGEXT_ID_STARTTIMEBOX:
            if ( inst->eventValid )
            {
                tmpTime = inst->event.startTime;

                (void)TIME_AddUtcOffset (&tmpTime,NULL);

                item->type            = FGS_ITEM_TYPE_TIME;
                item->align           = FGS_ALIGN_HOR_RIGHT |
                                        FGS_ALIGN_VER_CENTER;
                item->item.time.value  = tmpTime;
                item->item.time.format = appDat->timeFormat;
                item->item.time.font   = MAIN_FONT;
                item->item.time.col    = OSD_COL_TITLE_TEXT;
            }
            break;

        case MENU_EPGEXT_ID_SEPARATOR:
            if ( inst->eventValid )
            {
                item->type                = FGS_ITEM_TYPE_STRING;
                item->align               = FGS_ALIGN_HOR_CENTER |
                                            FGS_ALIGN_VER_CENTER;
                item->item.str.chr        = OSD_GetStr(STR_GNR_MINUS);
                item->item.str.font       = MAIN_FONT;
                item->item.str.lineHeight = 0;
                item->item.str.col        = OSD_COL_TITLE_TEXT;
            }
            break;

        case MENU_EPGEXT_ID_ENDTIMEBOX:
            if ( inst->eventValid )
            {
                tmpTime = inst->event.startTime;

                (void)TIME_AddUtcOffset (&tmpTime,NULL);

                evtDuration = TIME_ConvToSeconds(&(inst->event.duration));

                TIME_AddSeconds (&tmpTime, NULL, evtDuration);

                item->type             = FGS_ITEM_TYPE_TIME;
                item->align            = FGS_ALIGN_HOR_LEFT |
                                         FGS_ALIGN_VER_CENTER;
                item->item.time.value  = tmpTime;
                item->item.time.format = appDat->timeFormat;
                item->item.time.font   = MAIN_FONT;
                item->item.time.col    = OSD_COL_TITLE_TEXT;
            }
            break;

        case MENU_EPGEXT_ID_DAYBOX:
            if ( inst->eventValid )
            {
                (void)STR_BufAppend (NULL,
                                     0,
                                     OSD_GetWeekDayStr(inst->event.startDate.dayOfWeek),
                                     MAIN_LINEBUFCHARSET);
                item->type                = FGS_ITEM_TYPE_STRING;
                item->align               = FGS_ALIGN_HOR_RIGHT |
                                            FGS_ALIGN_VER_CENTER;
                item->item.str.chr        = STR_IntLineStr();
                item->item.str.font       = MAIN_FONT;
                item->item.str.lineHeight = 0;
                item->item.str.col        = OSD_COL_TITLE_TEXT;
            }
            break;

        case MENU_EPGEXT_ID_DATEBOX:
            if ( inst->eventValid )
            {
                tmpTime = inst->event.startTime;
                tmpDate = inst->event.startDate;

                (void)TIME_AddUtcOffset (&tmpTime,&tmpDate);
                item->type             = FGS_ITEM_TYPE_DATE;
                item->align            = FGS_ALIGN_HOR_LEFT |
                                         FGS_ALIGN_VER_CENTER;
                item->item.date.value  = tmpDate;
                item->item.date.format = appDat->dataFormatNorm;
                item->item.date.font   = MAIN_FONT;
                item->item.date.col    = OSD_COL_TITLE_TEXT;
            }
            break;

        default:
            break;
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst    The Extended EPG Instants.
** \param   font    The font index used for drawing.
**
******************************************************************************
*/
FSTATIC void MENU_EpgExtParseExtDesc (MENU_EPGEXT_INST_S* inst, uint8_t font)
{
    inst->eitExtNumLines = 0;

    if ( inst->eitExtLen < 2 )
    {
        return;
    }

    /* let FGS format lines to be displayed */
    (void)FGS_GetLinesFromString (inst->eitExtCurr, inst->eitExtPixPerLine,
                                  font, MENU_EpgExtLineParsed);
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   lineIdx     The number of the line.
** \param   lineStart   The string of this line.
** \param   lineBytes   The number of bytes of this line.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgExtLineParsed (uint16_t lineIdx,
                                       SYS_STRING_S lineStart,
                                       uint16_t lineBytes)
{
    MENU_EPGEXT_INST_S* inst = epgExtDat;

    if ( (lineIdx >= EXTDESC_MAXLINES) ||
         (lineIdx != inst->eitExtNumLines) )
    {
        return FFALSE;
    }

    inst->eitExtLine[inst->eitExtNumLines].lineString = lineStart;
    inst->eitExtLine[inst->eitExtNumLines].numBytes   = lineBytes;

    inst->eitExtNumLines++;

    return FTRUE;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
