/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_epg_grid.c
**
**              $Id: app_menu_epg_grid.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Grid EPG menu implementation.
**
** Detail description. [\ref app_smartgo_app_ref_epg]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 574: Signed-unsigned mix with relational.
** - Warning 641: Converting enum 'anyEnum' to int.
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Note    946: Relational or subtract operator applied to pointers
**                [MISRA 2004 Rule 17.3]
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e574 -e641 -e713 -e715 -e732 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"
#include "app_menu_info.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
/*!
 * \note Units must be aligned on hours! (MENU_EPG_GRID_MINS_PER_UNIT = 60/n)
 */
#define MENU_EPG_GRID_MINS_PER_UNIT      2

#define MENU_EPG_GRID_UNITS_VISIBLE      120
#define MENU_EPG_GRID_UNITS_PER_DAY      ((60 * 24) / MENU_EPG_GRID_MINS_PER_UNIT)
#define MENU_EPG_GRID_DAYS_MAX           AV_SCHED_DAYS_MAX
#define MENU_EPG_GRID_UNIT_GRID          10 //!< Number of units to be scrolled
                                            //!< if no event is in focus.

#define MENU_EPG_GRID_RECT_PAD           2 //!< Rectangle padding in pixels.

#define MENU_EPG_GRID_GAP_TIME           60 //!< Maximum gap between events that
                                            //!< is ignored in OSD
                                            //!< representation in seconds.
                                            //!< This is done for ensuring a
                                            //!< seamless presentation.

#define MENU_EPG_GRID_TITLE_HEIGHT       (SCALER_GetCurrVal(LIST_LINE_HEIGHT) + SCALER_GetCurrVal(LIST_LINE_PADDING))
#define MENU_EPG_GRID_SCHED_HEIGHT       ((MENU_EPG_GRID_UNITS_VISIBLE                  \
                                           * SCALER_GetCurrVal(EPG_GRID_PIX_PER_UNIT))  \
                                         + MENU_EPG_GRID_RECT_PAD)
#define MENU_EPG_GRID_DESCRBOX_HEIGHT    (5 * SCALER_GetCurrVal(LIST_LINE_HEIGHT))

#define MENU_EPG_GRID_NUM_SERVICES       3
#define MENU_EPG_GRID_EVTPERDAY_MAX      100

#if ( MENU_EPG_GRID_NUM_SERVICES > AV_EIT_SCHED_SERV_MAX )
    #error EPG Menu: Configuration error
#endif

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

struct menu_epg_grid_s;

enum
{
    MENU_EPG_GRID_ID_FOCUSNAME = 0,
    MENU_EPG_GRID_ID_FOCUSTIME,
    MENU_EPG_GRID_ID_FOCUSDESCR,
    MENU_EPG_GRID_ID_DATE,
    MENU_EPG_GRID_ID_TIMELINE,
    MENU_EPG_GRID_ID_HELP_GREEN_ICON,
    MENU_EPG_GRID_ID_HELP_GREEN_TEXT,
    MENU_EPG_GRID_ID_HELP_YELLOW_ICON,
    MENU_EPG_GRID_ID_HELP_YELLOW_TEXT,
    MENU_EPG_GRID_ID_HELP_OK_ICON,
    MENU_EPG_GRID_ID_HELP_OK_TEXT,
    MENU_EPG_GRID_ID_HELP_INFO_ICON,
    MENU_EPG_GRID_ID_HELP_INFO_TEXT,
    MENU_EPG_GRID_ID_HELP_REC_ICON,
    MENU_EPG_GRID_ID_HELP_REC_TEXT,
    MENU_EPG_GRID_ID_HELP_EXIT_ICON,
    MENU_EPG_GRID_ID_HELP_EXIT_TEXT,
    MENU_EPG_GRID_ID_SERVTITLE,               //!< Service titles
    MENU_EPG_GRID_ID_SERVSCHED = MENU_EPG_GRID_ID_SERVTITLE + MENU_EPG_GRID_NUM_SERVICES
};

typedef struct
{
    int32_t                 index;              //!< index in EIT schedule list.
    uint16_t                id;                 //!< event_id
    SYS_CHARSET             nameCharSet;
    char_t                  name[AV_EIT_NAMELEN_MAX];
    uint32_t                startUnit;          //!< Index of first unit.
    uint32_t                endUnit;            //!< Index after last unit.

    #ifdef APPL_RECTIMER_ENABLED
    RECTIMER_IDX            timerIdx;           //!< Timer index if assigned.
    #endif /* APPL_RECTIMER_ENABLED */

} MENU_EPG_GRID_EVT_S;


typedef struct
{
    uint8_t                  index;
    struct menu_epg_grid_s*  inst;
    fbool_t                  cleared;

    FGS_BOX_INST_S           titleInst;    //!< Service title box.
    FGS_BOX_INST_S           schedInst;    //!< Service schedule box.

    SYS_SERVLOC_S            servLoc;
    SLIST_SERVIDX            servIdx;
    SLIST_ITEM_S             servItem;

    MENU_EPG_GRID_EVT_S      evt[MENU_EPG_GRID_EVTPERDAY_MAX];
    int16_t                  numEvts;

} MENU_EPG_GRID_SERVICE_S;

struct menu_epg_grid_s
{
    FAPI_SYS_HandleT         pnlHdl;
    FGS_BOX_INST_S           boxInst[5];   //!< Several boxes.

    SYS_TIME_S               topTime;      //!< UTC time at start of current date.
    SYS_DATE_S               topDate;      //!< Current date (UTC).
    int32_t                  dateOffset;   //!< Current offset from topTime/-Date.
    int32_t                  topUnit;      //!< First unit in current display.

    MENU_EPG_GRID_SERVICE_S  service[MENU_EPG_GRID_NUM_SERVICES];

    MENU_EPG_GRID_SERVICE_S* focusService; //!< Currently focussed column.
    int16_t                  focusEvt;     //!< Index of focussed event (-1: none)
    uint8_t                  numColumns;   //!< Number of columns currently used.

    EIT_PARAMS_S             eitParams;

    fbool_t                  evtDataValid;
    EIT_EVT_S                evtData;
    SYS_STRING_S             evtExtData;
    char_t                   evtExtBuf[AV_EIT_EXTLEN_SCHED];

    #ifdef APPL_RECTIMER_ENABLED
    fbool_t                  timerChanged;
    #endif /* APPL_RECTIMER_ENABLED */
};

typedef struct menu_epg_grid_s MENU_EPG_GRID_INST_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

MENU_EPG_GRID_INST_S* menEpgGridDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_EpgGridLeave              (FAPI_SYS_HandleT h,
                                                fbool_t force);
FSTATIC fbool_t MENU_EpgGridKeys               (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_EpgGridBuild              (MENU_EPG_GRID_INST_S* inst);
FSTATIC void    MENU_EpgGridOnFocusSet         (FAPI_SYS_HandleT h,
                                                fbool_t hasFocus);
FSTATIC void    MENU_EpgGridDrawSched          (FAPI_SYS_HandleT h,
                                                uint8_t align,
                                                uint16_t x,
                                                uint16_t y,
                                                uint16_t width,
                                                uint16_t height);
FSTATIC void    MENU_EpgGridDrawTimeLine       (FAPI_SYS_HandleT h,
                                                uint8_t align,
                                                uint16_t x,
                                                uint16_t y,
                                                uint16_t width,
                                                uint16_t height);
FSTATIC void    MENU_EpgGridTimeUpdate         (void);
FSTATIC void    MENU_EpgGridMoveServiceFocus   (MENU_EPG_GRID_INST_S* inst,
                                                MENU_EPG_GRID_SERVICE_S* pNewFocus);
FSTATIC void    MENU_EpgGridMoveEventFocus     (MENU_EPG_GRID_INST_S* inst,
                                                int16_t newFocusEvt);
FSTATIC fbool_t MENU_EpgGridNavigateUp         (MENU_EPG_GRID_INST_S* inst);
FSTATIC fbool_t MENU_EpgGridNavigateDown       (MENU_EPG_GRID_INST_S* inst);
FSTATIC fbool_t MENU_EpgGridNavigateLeft       (MENU_EPG_GRID_INST_S* inst);
FSTATIC fbool_t MENU_EpgGridNavigateRight      (MENU_EPG_GRID_INST_S* inst);
FSTATIC void    MENU_EpgGridSetDisplayTime     (MENU_EPG_GRID_INST_S* inst,
                                                int32_t newTopUnit);
FSTATIC int16_t MENU_EpgGridGetPrevVisibleEvt  (MENU_EPG_GRID_INST_S* inst,
                                                int32_t topUnit,
                                                int16_t refIdx,
                                                fbool_t complete);
FSTATIC int16_t MENU_EpgGridGetNextVisibleEvt  (MENU_EPG_GRID_INST_S* inst,
                                                int32_t topUnit,
                                                int16_t refIdx,
                                                fbool_t complete);
FSTATIC void    MENU_EpgGridAssignServices     (MENU_EPG_GRID_INST_S* inst,
                                                const SYS_SERVLOC_S* pFirstServLoc);
FSTATIC void    MENU_EpgGridAssignFocusEvent   (MENU_EPG_GRID_INST_S* inst,
                                                const MENU_EPG_GRID_EVT_S* refEvent,
                                                fbool_t newService);
FSTATIC void    MENU_EpgGridGetServiceDescr    (FAPI_SYS_HandleT h,
                                                int32_t id,
                                                uint32_t index,
                                                FGS_ITEMSTATE_E state,
                                                FGS_ITEM_S* item);
FSTATIC void    MENU_EpgGridGetServiceName     (FAPI_SYS_HandleT h,
                                                int32_t id,
                                                uint32_t index,
                                                FGS_ITEMSTATE_E state,
                                                FGS_ITEM_S* item);
FSTATIC void    MENU_EpgGridGetDate            (FAPI_SYS_HandleT h,
                                                int32_t id,
                                                uint32_t index,
                                                FGS_ITEMSTATE_E state,
                                                FGS_ITEM_S* item);
FSTATIC void    MENU_EpgGridGetItem            (FAPI_SYS_HandleT h,
                                                int32_t id,
                                                uint32_t index,
                                                 FGS_ITEMSTATE_E state,
                                                 FGS_ITEM_S* item);
FSTATIC MENU_EPG_GRID_SERVICE_S* MENU_EpgGridGetServiceSlot (MENU_EPG_GRID_INST_S* inst,
                                                SLIST_SERVIDX servIdx);
FSTATIC void    MENU_EpgGridUpdateDisplay      (MENU_EPG_GRID_INST_S* inst,
                                                int32_t reqTopUnit);
FSTATIC void    MENU_EpgGridUpdateServiceSlot  (MENU_EPG_GRID_INST_S* inst,
                                                MENU_EPG_GRID_SERVICE_S* pService);
FSTATIC void    MENU_EpgGridUpdateFocusEvtData (MENU_EPG_GRID_INST_S* inst);
FSTATIC uint32_t MENU_EpgGridGetUnit           (MENU_EPG_GRID_INST_S* inst,
                                                SYS_TIME_S* time);
FSTATIC int32_t MENU_EpgGridGetTopUnitForEvent (MENU_EPG_GRID_INST_S* inst,
                                                int32_t refTopUnit);
FSTATIC void    MENU_EpgGridGetHelpItem        (FAPI_SYS_HandleT h,
                                                int32_t id,
                                                uint32_t index,
                                                FGS_ITEMSTATE_E state,
                                                FGS_ITEM_S* item);
#ifdef APPL_RECTIMER_ENABLED
FSTATIC fbool_t MENU_EpgGridHandleRecTimer     (MENU_EPG_GRID_SERVICE_S* pService,
                                                MENU_EPG_GRID_EVT_S* pEvt);
#endif /* APPL_RECTIMER_ENABLED */

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/
/*!
******************************************************************************
** \brief   This function initialize the grid EPG.
**
** Here the grid epg instants memory is allocated, and the grid EPG service
** data base is filled with default data.
**
** \return  one of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_ALREADY_INITIALIZED
**          - #APPL_SMARTGO_ERR_OUT_OF_MEMORY
**
******************************************************************************
*/
int32_t MENU_EpgGridInit (void)
{
    MENU_EPG_GRID_INST_S* inst;
    uint8_t               cnt;

    if ( menEpgGridDat != NULL )
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    menEpgGridDat = UTILS_Malloc(sizeof(MENU_EPG_GRID_INST_S));
    if ( menEpgGridDat == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    memset(menEpgGridDat, 0, sizeof(MENU_EPG_GRID_INST_S));

    inst = menEpgGridDat;

    for ( cnt = 0; cnt < MENU_EPG_GRID_NUM_SERVICES; cnt++ )
    {
        inst->service[cnt].index   = cnt;
        inst->service[cnt].inst    = inst;
        inst->service[cnt].cleared = FFALSE;
    }

    return FAPI_OK;
}

/*!
******************************************************************************
** \brief   With this function you enter the grid EPG menu.
**
** This function is called twice. First with \p test flag #FTRUE to check if
** the menu can be opend. If the first call was successful and return #FTRUE
** the second call will open the menu (\p test flag #FFALSE).
**
** \param   h       Handle of the Grid EPG Instants.
** \param   test    A flag to test if opening this menu is possible.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
fbool_t MENU_EpgGridEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_EPG_GRID_INST_S*   inst = menEpgGridDat;
    SYS_SERVLOC_S           currServLoc;
    int16_t                 utcOffset   = 0;
    fbool_t                 dstIsActive = FFALSE;
    fbool_t                 hasSynced;
    uint32_t                firstTopUnit;
    EIT_PARAMS_S            eitParams;
    SYS_TIME_S              currTime;

    if( inst == NULL)
    {
        return FFALSE;
    }

    if ( test == FTRUE )
    {
        return FTRUE;
    }

    inst->focusService = NULL;
    inst->focusEvt     = -1;
    inst->numColumns   = 0;
    inst->evtDataValid = FFALSE;

    inst->dateOffset   = 0;
    inst->topUnit      = 0;

    #ifdef APPL_RECTIMER_ENABLED
    inst->timerChanged = FFALSE;
    #endif /* APPL_RECTIMER_ENABLED */

    /* store current EIT parameters in order to restore them when leaving
       menu. */
    if ( EIT_GetParams(&(inst->eitParams)) != FAPI_OK )
    {
        inst->eitParams.modeCurrNext = EIT_FILTERMODE_OFF;
        inst->eitParams.modeSchedule = EIT_FILTERMODE_OFF;
    }

    /* Make sure to filter maximum number of events */
    eitParams.modeCurrNext = EIT_FILTERMODE_ALL;
    eitParams.modeSchedule = EIT_FILTERMODE_ALL;

    (void)EIT_SetParams(&eitParams);

    /* --------------------------------------------------------------------- */

    (void)SETTINGS_TimeOffsetGet (&utcOffset, &dstIsActive);

    if ( dstIsActive ) utcOffset += 60;

    (void)TIME_TimeAndDateGetUtc(&(inst->topTime), &(inst->topDate),
                                 &hasSynced);

    // TODO: perform error handling if not synced!

    currTime = inst->topTime; /* store current UTC time */

    /* add UTC offset */
    TIME_AddSeconds(&(inst->topTime), &(inst->topDate), (int32_t)utcOffset * 60);

    memset(&(inst->topTime), 0, sizeof(inst->topTime));

    /* re-convert time to UTC */
    TIME_AddSeconds(&(inst->topTime), &(inst->topDate), -((int32_t)utcOffset * 60));

    /* --------------------------------------------------------------------- */
    /* start timer for periodic updating of dynamic items */
    appDat->osdUpdateCb = MENU_EpgGridTimeUpdate;
    (void)TIME_PeriodicTimerLaunch(OSD_PERIODTIMER_GENERAL, 600); // period: 1min


    if ( MENU_EpgGridBuild(inst) == FFALSE ) return FFALSE;

    /* Assign services */
    currServLoc.listId = SLIST_NONE;
    currServLoc.index  = -1;

    (void)AV_GetRequestedService (&currServLoc);

    MENU_EpgGridAssignServices(inst, &currServLoc);

    /* use current time for calculation of top unit */
    firstTopUnit = MENU_EpgGridGetUnit(inst, &currTime);

    /* place current time at 1/4 of viewable display */
    if ( firstTopUnit >= (MENU_EPG_GRID_UNITS_VISIBLE / 4) )
    {
        firstTopUnit -= (MENU_EPG_GRID_UNITS_VISIBLE / 4);
    }

    MENU_EpgGridSetDisplayTime (inst, firstTopUnit);

    MENU_EpgGridAssignFocusEvent(inst, NULL, FTRUE);

    /* --------------------------------------------------------------------- */

    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_EPG_GRID_ID_FOCUSNAME); /* dummy */
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_EPG);
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE);

    MENUSTACK_SetCleanup(MENU_EpgGridLeave, (FAPI_SYS_HandleT)inst);

    return FTRUE;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   servIdx TODO
**
******************************************************************************
*/
void MENU_EpgGridUpdate(SLIST_SERVIDX servIdx)
{
    MENU_EPG_GRID_INST_S*    inst       = menEpgGridDat;
    MENU_EPG_GRID_SERVICE_S* pService   = MENU_EpgGridGetServiceSlot(inst, servIdx);
    MENU_EPG_GRID_EVT_S*     pEvt       = NULL;
    int32_t                  newTopUnit;
    MENU_EPG_GRID_EVT_S      evtFocus;

    if ( ( inst         == NULL ) ||
         ( inst->pnlHdl == NULL ) ||
         ( pService     == NULL ) )
    {
        return;
    }

    if ( (inst->focusService == pService) &&
         (inst->focusEvt     >= 0) )
    {
        /* store current focus event in order to restore it afterwards */
        evtFocus = inst->focusService->evt[inst->focusEvt];
        pEvt     = &evtFocus;
    }

    MENU_EpgGridUpdateServiceSlot (inst, pService);

    newTopUnit = inst->topUnit;

    if ( inst->focusService == pService )
    {
        MENU_EpgGridAssignFocusEvent (inst, pEvt, FFALSE);

        newTopUnit = MENU_EpgGridGetTopUnitForEvent (inst, newTopUnit);
    }

    if ( newTopUnit != inst->topUnit )
    {
        MENU_EpgGridSetDisplayTime (inst, newTopUnit);
    }
    else
    {
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_EPG_GRID_ID_SERVTITLE +
                                       pService->index);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_EPG_GRID_ID_SERVSCHED +
                                       pService->index);
    }
}

/*!
******************************************************************************
** \brief   This function sets up the menu structure of the grid EPG.
**
** \param   inst    Grid EPG Instants.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgGridBuild (MENU_EPG_GRID_INST_S* inst)
{
    FGS_PANEL_OPEN_S            pnlPars;
    FGS_BOX_OPEN_S              boxPars;
    FGS_BOX_OPEN_S              schedPars;
    int32_t                     retVal = FAPI_OK;
    FAPI_SYS_HandleT            itemHdl;
    uint8_t                     cnt;
    MENU_EPG_GRID_SERVICE_S*    pService;
    FGS_ITEM_S                  item;
    int32_t                     numHelpLines = 4;

    memset(&pnlPars,   0, sizeof(pnlPars));
    memset(&boxPars,   0, sizeof(boxPars));
    memset(&schedPars, 0, sizeof(schedPars));

    do
    {
        pnlPars.bkgrType      = OSD_PNLTYPE_PROGGUIDE;
        pnlPars.pos.x         = 0;
        pnlPars.pos.y         = 0;
        pnlPars.pos.width     = SCALER_GetCurrVal(MAIN_LEFT_PADDING)  +
                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING) +
                                SCALER_GetCurrVal(EPG_GRID_TIMELINE_WIDTH) +
                               (MENU_EPG_GRID_NUM_SERVICES *
                                (SCALER_GetCurrVal(EPG_GRID_COLUMN_WIDTH)
                               + SCALER_GetCurrVal(EPG_GRID_COLUMN_PAD)));

        pnlPars.pos.height    = SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                SCALER_GetCurrVal(MAIN_BOT_PADDING) +
                                MENU_EPG_GRID_TITLE_HEIGHT +
                                MENU_EPG_GRID_SCHED_HEIGHT +
                                SCALER_GetCurrVal(EPG_GRID_COLUMN_PAD) +
                                MENU_EPG_GRID_DESCRBOX_HEIGHT;
        pnlPars.pos.alignment = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
        pnlPars.onFocusSet    = MENU_EpgGridOnFocusSet;
        pnlPars.onFocusSetArg = (FAPI_SYS_HandleT)inst;
        pnlPars.pInst         = &(appDat->pnlInst);

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_EPG, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- Service description boxes --------------------------------- */

        boxPars.pos.width           = (uint16_t)(pnlPars.pos.width -
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                      SCALER_GetCurrVal(MAIN_RIGHT_PADDING)) / 2;
        boxPars.pos.x               = -(int16_t)(SCALER_GetCurrVal(MAIN_RIGHT_PADDING) + boxPars.pos.width);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_BLACK;
        boxPars.getItem             = MENU_EpgGridGetServiceDescr;
        boxPars.getItemArg          = (FAPI_SYS_HandleT)inst;

        boxPars.pos.height          = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.y               = -(int16_t)SCALER_GetCurrVal(MAIN_BOT_PADDING) -
                                       MENU_EPG_GRID_DESCRBOX_HEIGHT;
        boxPars.pInst               = inst->boxInst + 0;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_FOCUSNAME, inst->pnlHdl,
                               &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.height          = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pos.y               = -(int16_t)SCALER_GetCurrVal(MAIN_BOT_PADDING) -
                                       MENU_EPG_GRID_DESCRBOX_HEIGHT +
                                       SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        boxPars.pInst               = inst->boxInst + 1;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_FOCUSTIME, inst->pnlHdl,
                               &itemHdl);
        if ( retVal != FAPI_OK ) break;

        boxPars.pos.height          = MENU_EPG_GRID_DESCRBOX_HEIGHT -
                                      (2 * SCALER_GetCurrVal(LIST_LINE_HEIGHT));
        boxPars.pos.y               = -(int16_t)(SCALER_GetCurrVal(MAIN_BOT_PADDING) + boxPars.pos.height);
        boxPars.pInst               = inst->boxInst + 2;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_FOCUSDESCR, inst->pnlHdl,
                               &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----- Date ------------------------------------------------------ */

        boxPars.pos.x               = SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = SCALER_GetCurrVal(MAIN_TOP_PADDING);
        boxPars.pos.width           = SCALER_GetCurrVal(EPG_GRID_TIMELINE_WIDTH);
        boxPars.pos.height          = MENU_EPG_GRID_TITLE_HEIGHT;
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType            = OSD_BOX_BKGR_NORMAL;
        boxPars.getItem             = MENU_EpgGridGetDate;
        boxPars.getItemArg          = (FAPI_SYS_HandleT)inst;
        boxPars.pInst               = inst->boxInst + 3;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_DATE, inst->pnlHdl,
                               &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----- Timeline -------------------------------------------------- */

        boxPars.pos.x               = SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = SCALER_GetCurrVal(MAIN_TOP_PADDING) +
                                      MENU_EPG_GRID_TITLE_HEIGHT;
        boxPars.pos.width           = SCALER_GetCurrVal(EPG_GRID_TIMELINE_WIDTH);
        boxPars.pos.height          = MENU_EPG_GRID_SCHED_HEIGHT;
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType            = OSD_BOX_BKGR_NORMAL;
        boxPars.getItem             = MENU_EpgGridGetItem;
        boxPars.getItemArg          = (FAPI_SYS_HandleT)inst;
        boxPars.pInst               = inst->boxInst + 4;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_TIMELINE, inst->pnlHdl,
                               &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* ----- Service columns ------------------------------------------- */

        boxPars.pos.x               = SCALER_GetCurrVal(MAIN_LEFT_PADDING) +
                                      SCALER_GetCurrVal(EPG_GRID_TIMELINE_WIDTH) +
                                      SCALER_GetCurrVal(EPG_GRID_COLUMN_PAD);
        boxPars.pos.y               = SCALER_GetCurrVal(MAIN_TOP_PADDING);
        boxPars.pos.width           = (uint16_t)(pnlPars.pos.width -
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                      SCALER_GetCurrVal(MAIN_RIGHT_PADDING) -
                                      SCALER_GetCurrVal(EPG_GRID_TIMELINE_WIDTH) -
                                      (MENU_EPG_GRID_NUM_SERVICES * SCALER_GetCurrVal(EPG_GRID_COLUMN_PAD)))
                                      / MENU_EPG_GRID_NUM_SERVICES;
        boxPars.pos.height          = MENU_EPG_GRID_TITLE_HEIGHT;
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        boxPars.bkgrType            = OSD_BOX_BKGR_NORMAL;
        boxPars.getItem             = MENU_EpgGridGetServiceName;
        boxPars.getItemArg          = (FAPI_SYS_HandleT)inst;

        schedPars.pos.x             = boxPars.pos.x;
        schedPars.pos.y             = boxPars.pos.y + boxPars.pos.height;
        schedPars.pos.width         = boxPars.pos.width;
        schedPars.pos.height        = MENU_EPG_GRID_SCHED_HEIGHT;
        schedPars.pos.alignment     = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
        schedPars.bkgrType          = OSD_BOX_BKGR_NONE;
        schedPars.getItem           = MENU_EpgGridGetItem;
        schedPars.getItemArg        = (FAPI_SYS_HandleT)inst;

        for ( cnt = 0, pService = inst->service;
              cnt < MENU_EPG_GRID_NUM_SERVICES;
              cnt++, pService++ )
        {
            pService->cleared = FFALSE;

            boxPars.pInst      = &(pService->titleInst);
            schedPars.pInst    = &(pService->schedInst);

            retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_SERVTITLE + cnt,
                                   inst->pnlHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            retVal = FGS_BOX_Open (&schedPars, MENU_EPG_GRID_ID_SERVSCHED + cnt,
                                   inst->pnlHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            boxPars.pos.x   += boxPars.pos.width   + SCALER_GetCurrVal(EPG_GRID_COLUMN_PAD);
            schedPars.pos.x += schedPars.pos.width + SCALER_GetCurrVal(EPG_GRID_COLUMN_PAD);
        }

        if ( retVal != FAPI_OK ) break;

        /* ----- Help items ------------------------------------------------ */

        memset(&item,    0, sizeof(item));
        memset(&boxPars, 0, sizeof(boxPars));

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNGREENSMALL);

        boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = 0 - (SCALER_GetCurrVal(OUTER_FRAME_ALL)/2)
                                      - SCALER_GetCurrVal(HELP_TB_PADDING)
                                      - (numHelpLines * SCALER_GetCurrVal(HELP_LINE_HEIGHT))
                                      - ((numHelpLines - 1) * SCALER_GetCurrVal(HELP_LINE_PADDING));
        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.width           = SCALER_GetCurrVal(EPG_GRID_HELPKEY_WIDTH_SHORT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_HELP_GREEN_ICON,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_EPG_HELP_PREV_DAY);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = pnlPars.pos.width - boxPars.pos.x;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_HELP_GREEN_TEXT,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /*--------------------------------------------------------------*/

        MENU_COMMON_ITEM_BUTTON(&item, ICON_BTNYELLOWSMALL);

        boxPars.pos.x               = pnlPars.pos.width>>2;
        boxPars.pos.width           = SCALER_GetCurrVal(EPG_GRID_HELPKEY_WIDTH_SHORT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_HELP_YELLOW_ICON,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_EPG_HELP_NEXT_DAY);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = pnlPars.pos.width - boxPars.pos.x;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_HELP_YELLOW_TEXT,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        /*--------------------------------------------------------------*/

        MENU_COMMON_ITEM_HELPSTR(&item, STR_KEY_OK);

        boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y              += SCALER_GetCurrVal(HELP_LINE_HEIGHT) +
                                      SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.width           = SCALER_GetCurrVal(EPG_GRID_HELPKEY_WIDTH_SHORT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_HELP_OK_ICON,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_EPG_HELP_SELECT_SERVICE);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = pnlPars.pos.width - boxPars.pos.x;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_HELP_OK_TEXT,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /*--------------------------------------------------------------*/

        MENU_COMMON_ITEM_HELPSTR(&item, STR_KEY_INFO);

        boxPars.pos.x               = pnlPars.pos.width>>2;
        boxPars.pos.width           = SCALER_GetCurrVal(EPG_GRID_HELPKEY_WIDTH_SHORT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_HELP_INFO_ICON,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_EPG_HELP_EXTINFO);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = pnlPars.pos.width - boxPars.pos.x;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_HELP_INFO_TEXT,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /*--------------------------------------------------------------*/

        MENU_COMMON_ITEM_HELPSTR(&item, STR_KEY_REC);

        boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y              += SCALER_GetCurrVal(HELP_LINE_HEIGHT) +
                                      SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.width           = SCALER_GetCurrVal(EPG_GRID_HELPKEY_WIDTH_SHORT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_HELP_REC_ICON,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_EPG_HELP_ADDTOTIMERS);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = pnlPars.pos.width - boxPars.pos.x;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_HELP_REC_TEXT,
                               inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /*--------------------------------------------------------------*/

        boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                      SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y              += SCALER_GetCurrVal(HELP_LINE_HEIGHT) +
                                      SCALER_GetCurrVal(HELP_LINE_PADDING);
        boxPars.pos.width           = SCALER_GetCurrVal(EPG_GRID_HELPKEY_WIDTH_LONG);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.getItem             = MENU_EpgGridGetHelpItem;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_HELP_EXIT_ICON, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        MENU_COMMON_ITEM_HELPSTR(&item, STR_HELP_MENU_EXIT);

        boxPars.pos.x              += boxPars.pos.width;
        boxPars.pos.width           = pnlPars.pos.width - boxPars.pos.x;
        boxPars.item[FGS_NONFOCUS]  = item;
        boxPars.getItem = NULL;

        retVal = FGS_BOX_Open (&boxPars, MENU_EPG_GRID_ID_HELP_EXIT_TEXT, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;


    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

        inst->pnlHdl = NULL;
        return FFALSE;
    }

    return FTRUE;
}

/*!
******************************************************************************
** \brief   This function gives the box item for the exit keys to the FGS.
**
** To save some space on the screen the 3 exit key are putted together to a
** single entry "EXIT\MENU\BACK"
**
** \param   h       Handle of the Grid EPG Instants. (unused)
** \param   id      The item identifier. (unused)
** \param   index   TODO. (unused)
** \param   state   The status of the item, FGS_FOCUS; FGS_NONFOCUS or
**                  FGS_DISABLED. (unused)
** \param   item    Pointer to the item structure.
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridGetHelpItem(FAPI_SYS_HandleT h,
                                     int32_t id,
                                     uint32_t index,
                                     FGS_ITEMSTATE_E state,
                                     FGS_ITEM_S* item)
{
    uint32_t            bufPos = 0;

    bufPos += STR_BufAppend (NULL, bufPos,
                             OSD_GetStr(STR_KEY_EXIT),
                             MAIN_LINEBUFCHARSET);
    bufPos += STR_BufAppend (NULL, bufPos,
                             OSD_GetStr(STR_GNR_BACKSLASH),
                             MAIN_LINEBUFCHARSET);
    bufPos += STR_BufAppend (NULL, bufPos,
                             OSD_GetStr(STR_KEY_MENU),
                             MAIN_LINEBUFCHARSET);
    bufPos += STR_BufAppend (NULL, bufPos,
                             OSD_GetStr(STR_GNR_BACKSLASH),
                             MAIN_LINEBUFCHARSET);
    bufPos += STR_BufAppend (NULL, bufPos,
                             OSD_GetStr(STR_KEY_EPG),
                             MAIN_LINEBUFCHARSET);

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->item.str.col        = appDat->txtAttr.colNonFocus;
    item->item.str.font       = (int32_t)MAIN_FONT;
    item->item.str.lineHeight = 0;
    item->item.str.chr        = STR_IntLineStr();
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the Grid EPG Instants.
** \param   force   TODO
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgGridLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_EPG_GRID_INST_S*   inst = (MENU_EPG_GRID_INST_S*)h;

    #ifdef APPL_RECTIMER_ENABLED

    if ( !force )
    {
        if ( inst->timerChanged )
        {
            MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);

            (void)RECTIMER_Save(FFALSE);

            (void)MAIN_MsgBoxHide(&(appDat->msgBox));
        }
    }

    #endif /* APPL_RECTIMER_ENABLED */

    appDat->osdUpdateCb = NULL;

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_EPG);

    (void)EIT_RemoveFocusCurrNextAll();
    (void)EIT_RemoveFocusSchedAll();

    (void)EIT_SetParams(&(inst->eitParams));

    inst->pnlHdl = NULL;
    return FTRUE;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h           Handle of the Grid EPG Instants.
** \param   hasFocus    TODO
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_EPG_GRID_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_UP,     MENU_EpgGridKeys,   h,    FFALSE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_DOWN,   MENU_EpgGridKeys,   h,    FFALSE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_LEFT,   MENU_EpgGridKeys,   h,    FFALSE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_RIGHT,  MENU_EpgGridKeys,   h,    FFALSE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_NAVI,   MENUSTACK_Exit, NULL, FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_GREEN,  MENU_EpgGridKeys,   h,    FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_YELLOW, MENU_EpgGridKeys,   h,    FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_OK,     MENU_EpgGridKeys,   h,    FTRUE,  FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_INFO,   MENU_EpgGridKeys,   h,    FTRUE,  FFALSE);

        #ifdef APPL_RECTIMER_ENABLED
        FGS_SetAppKeyHandler(FGS_KEY_RECORD, MENU_EpgGridKeys,   h,    FTRUE,  FFALSE);
        #endif /* APPL_RECTIMER_ENABLED */
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_UP);
        FGS_ClearAppKeyHandler(FGS_KEY_DOWN);
        FGS_ClearAppKeyHandler(FGS_KEY_LEFT);
        FGS_ClearAppKeyHandler(FGS_KEY_RIGHT);
        FGS_ClearAppKeyHandler(FGS_KEY_NAVI);
        FGS_ClearAppKeyHandler(FGS_KEY_GREEN);
        FGS_ClearAppKeyHandler(FGS_KEY_YELLOW);
        FGS_ClearAppKeyHandler(FGS_KEY_OK);
        FGS_ClearAppKeyHandler(FGS_KEY_INFO);

        #ifdef APPL_RECTIMER_ENABLED
        FGS_ClearAppKeyHandler(FGS_KEY_RECORD);
        #endif /* APPL_RECTIMER_ENABLED */
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h   Handle of the Grid EPG Instants.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgGridKeys (FAPI_SYS_HandleT h)
{
    MENU_EPG_GRID_INST_S*   inst    = (MENU_EPG_GRID_INST_S*)h;
    fbool_t                 keyUsed = FTRUE;
    SYS_SERVLOC_S           newServLoc;
    MENU_EPGEXT_OPEN_S      epgExtParams;
    int32_t                 retVal;
    EIT_POS_S               refPos;

    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_LEFT:
            keyUsed = MENU_EpgGridNavigateLeft (inst);
            break;

        case FGS_KEY_RIGHT:
            keyUsed = MENU_EpgGridNavigateRight (inst);
            break;

        case FGS_KEY_UP:
            keyUsed = MENU_EpgGridNavigateUp (inst);
            break;

        case FGS_KEY_DOWN:
            keyUsed = MENU_EpgGridNavigateDown (inst);
            break;

        case FGS_KEY_GREEN: /* previous day */
            if ( inst->dateOffset > 0 )
            {
                inst->dateOffset--;
            }
            else
            {
                inst->dateOffset = MENU_EPG_GRID_DAYS_MAX - 1;
            }

            (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPG_GRID_ID_DATE);
            (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPG_GRID_ID_TIMELINE);

            /* NOTE: topUnit is not changed! */

            newServLoc = inst->service[0].servLoc;

            MENU_EpgGridAssignServices (inst, &newServLoc);

            /* set focus on first (completely) visible event */
            inst->focusEvt = MENU_EpgGridGetNextVisibleEvt (inst, inst->topUnit,
                             -1, FTRUE);

            MENU_EpgGridUpdateFocusEvtData (inst);
            MENU_EpgGridUpdateDisplay      (inst, inst->topUnit);
            break;

        case FGS_KEY_YELLOW: /* next day */
            inst->dateOffset++;

            if ( inst->dateOffset >= MENU_EPG_GRID_DAYS_MAX )
            {
                inst->dateOffset = 0;
            }

            newServLoc = inst->service[0].servLoc;

            (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPG_GRID_ID_DATE);
            (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPG_GRID_ID_TIMELINE);

            /* NOTE: topUnit is not changed! */

            newServLoc = inst->service[0].servLoc;

            MENU_EpgGridAssignServices (inst, &newServLoc);

            /* set focus on first (completely) visible event */
            inst->focusEvt = MENU_EpgGridGetNextVisibleEvt (inst, inst->topUnit,
                             -1, FTRUE);

            MENU_EpgGridUpdateFocusEvtData (inst);
            MENU_EpgGridUpdateDisplay      (inst, inst->topUnit);
            break;

        #ifdef APPL_RECTIMER_ENABLED

        case FGS_KEY_RECORD: /* Assign for recording/remove assignment */
            if ( (inst->focusService == NULL) || (inst->focusEvt < 0) )
            {
                keyUsed = FFALSE;
            }
            else
            {
                keyUsed = MENU_EpgGridHandleRecTimer (inst->focusService,
                                                  inst->focusService->evt
                                                  + inst->focusEvt);
            }
            break;

        #endif /* APPL_RECTIMER_ENABLED */

        case FGS_KEY_OK:
            if ( inst->focusService == NULL )
            {
                keyUsed = FFALSE;
            }
            else if ( MAIN_SelectService (&(inst->focusService->servLoc),
                      FFALSE) != FAPI_OK )
            {
                keyUsed = FFALSE;
            }
            break;

        case FGS_KEY_INFO:
            if ( ( inst->evtDataValid == FTRUE ) &&
                 ( inst->focusService != NULL  ) )
            {
                memset (&epgExtParams, 0, sizeof(epgExtParams));
                memset (&refPos,       0, sizeof(refPos));
                retVal =  EIT_GetSchedByIndex (&(inst->focusService->servLoc),
                                                  NULL, NULL, &refPos,
                                                  inst->focusService->evt[inst->focusEvt].index);
                if(retVal != FAPI_OK )
                {
                    keyUsed = FFALSE;
                    break;
                }
                epgExtParams.schedMode  = FTRUE;
                epgExtParams.currRef    = refPos;
                epgExtParams.calledFrom = MENU_EpgGridEntry;

                (void)MENUSTACK_Switch(MENU_EpgExtEntry,
                                 (FAPI_SYS_HandleT)(&epgExtParams),
                                 -1);
                MENU_InfoUpdateSched(inst->focusService->servLoc.index);
            }
            else
            {
                keyUsed = FFALSE;
            }
            break;
        default:
            keyUsed = FFALSE;
            break;
    }

    return keyUsed;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst        Grid EPG Instants.
** \param   refEvent    Previously focussed event, NULL if no focus event
**                      existed.
** \param   newService  #FTRUE if service is newly assigned, #FFALSE if service
**                      is refreshed.
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridAssignFocusEvent (MENU_EPG_GRID_INST_S* inst,
                                           const MENU_EPG_GRID_EVT_S* refEvent,
                                           fbool_t newService)
{
    fbool_t              selByCurrTime = FFALSE;
    SYS_TIME_S           currTime;
    SYS_DATE_S           currDate;
    fbool_t              hasSynced;
    uint32_t             currUnit;
    uint16_t             cnt;
    MENU_EPG_GRID_EVT_S* pEvt;
    int16_t              tmpEvent;

    inst->focusEvt = -1;

    if ( (inst->focusService == NULL) ||        /* no service in focus */
         (inst->focusService->numEvts < 1) )    /* no events available */
    {
        MENU_EpgGridUpdateFocusEvtData(inst);
        return;
    }

    if ( refEvent == NULL )
    {
        /* no reference event available */
        selByCurrTime = FTRUE;
    }
    else
    {
        if ( newService )
        {
            tmpEvent = -1;

            /* choose visible event by start- and end-time of reference event */
            for ( cnt = 0; cnt < (uint16_t)inst->focusService->numEvts; cnt++ )
            {
                pEvt = inst->focusService->evt + cnt;

                /* test for visibility */
                if ( pEvt->endUnit <= inst->topUnit )
                {
                    continue;
                }
                else if ( pEvt->startUnit >=
                          (inst->topUnit + MENU_EPG_GRID_UNITS_VISIBLE) )
                {
                    break;
                }

                tmpEvent = cnt;

                if ( pEvt->startUnit < refEvent->startUnit )
                {
                    if ( pEvt->endUnit >= refEvent->endUnit )
                    {
                        /* Event starts earlier but ends later than reference
                           event. */
                        inst->focusEvt = cnt;
                        break;
                    }
                }
                else
                {
                    /* Event starts later or at the same time than
                       reference event */
                    inst->focusEvt = cnt;
                    break;
                }
            }

            /* choose last visible event in list if no match has been found */
            if ( inst->focusEvt < 0 )
            {
                inst->focusEvt = tmpEvent;
            }
        }
        else
        {
            /* Try to choose event by the previous event's event_id. In this
               case visibility is not mandatory. */
            for ( cnt = 0; cnt < inst->focusService->numEvts; cnt++ )
            {
                if ( inst->focusService->evt[cnt].id == refEvent->id )
                {
                    inst->focusEvt = cnt;
                    break;
                }
            }

            /* if no match has been found select event by current time */
            if ( inst->focusEvt < 0 )
            {
                selByCurrTime = FTRUE;
            }
        }
    }

    if ( selByCurrTime )
    {
        if ( (TIME_TimeAndDateGetUtc (&currTime, &currDate, &hasSynced)
              == FAPI_OK) && hasSynced )
        {
            currUnit = MENU_EpgGridGetUnit(inst, &currTime);
            tmpEvent = -1;

            /* select first visible event that ends after current unit */
            for ( cnt = 0; cnt < inst->focusService->numEvts; cnt++ )
            {
                pEvt = inst->focusService->evt + cnt;

                /* test for visibility */
                if ( pEvt->endUnit <= inst->topUnit )
                {
                    continue;
                }
                else if ( pEvt->startUnit >=
                          (inst->topUnit + MENU_EPG_GRID_UNITS_VISIBLE) )
                {
                    break;
                }

                tmpEvent = cnt;

                if ( pEvt->endUnit >= currUnit )
                {
                    inst->focusEvt = cnt;
                    break;
                }
            }

            /* choose last visible event in list if no match has been found */
            if ( inst->focusEvt < 0 )
            {
                inst->focusEvt = tmpEvent;
            }
        }
    }

    MENU_EpgGridUpdateFocusEvtData(inst);
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst        Grid EPG Instants.
** \param   pNewFocus   TODO
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridMoveServiceFocus (MENU_EPG_GRID_INST_S* inst,
                                           MENU_EPG_GRID_SERVICE_S* pNewFocus)
{
    int32_t                  newTopUnit;
    MENU_EPG_GRID_SERVICE_S* pCurrFocus = inst->focusService;
    MENU_EPG_GRID_EVT_S*     pEvt = NULL;
    MENU_EPG_GRID_EVT_S      evtFocus;

    if ( (inst->focusService != NULL) &&
         (inst->focusEvt >= 0) )
    {
        /* store current focus event as reference for new focus
           event to be assigned */
        evtFocus = inst->focusService->evt[inst->focusEvt];
        pEvt     = &evtFocus;
    }

    inst->focusService = pNewFocus;

    MENU_EpgGridAssignFocusEvent (inst, pEvt, FTRUE);

    newTopUnit = MENU_EpgGridGetTopUnitForEvent (inst, inst->topUnit);

    if ( newTopUnit != inst->topUnit )
    {
        MENU_EpgGridSetDisplayTime (inst, newTopUnit);
    }
    else
    {
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_EPG_GRID_ID_SERVTITLE +
                                        pCurrFocus->index);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_EPG_GRID_ID_SERVTITLE +
                                       pNewFocus->index);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_EPG_GRID_ID_SERVSCHED +
                                       pCurrFocus->index);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_EPG_GRID_ID_SERVSCHED +
                                       pNewFocus->index);
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst    Grid EPG Instants.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgGridNavigateUp (MENU_EPG_GRID_INST_S* inst)
{
    int32_t         newTopUnit = inst->topUnit;
    fbool_t         prevDay    = FFALSE;
    SYS_SERVLOC_S   newServLoc;
    int16_t         newEvt;

    if ( inst->focusService == NULL ) return FFALSE;

    if ( inst->focusEvt < 0 )
    {
        if ( newTopUnit < 1 )
        {
            prevDay = FTRUE;
        }
        else
        {
            newTopUnit -= MENU_EPG_GRID_UNIT_GRID;

            if ( newTopUnit < 0 ) newTopUnit = 0;

            /* event visible in up-shifted display? */
            newEvt = MENU_EpgGridGetPrevVisibleEvt (inst, newTopUnit, -1, FFALSE);

            if ( newEvt >= 0 )
            {
                MENU_EpgGridMoveEventFocus (inst, newEvt);
            }
            else
            {
                /* shift display only */
                MENU_EpgGridSetDisplayTime (inst, newTopUnit);
            }
        }
    }
    else
    {
        /* event visible in current display? */
        newEvt = MENU_EpgGridGetPrevVisibleEvt (inst, newTopUnit, inst->focusEvt,
                                            FFALSE);

        if ( newEvt >= 0 )
        {
            MENU_EpgGridMoveEventFocus (inst, newEvt);
        }
        else if ( newTopUnit == 0 )
        {
            prevDay = FTRUE;
        }
        else
        {
            newTopUnit -= MENU_EPG_GRID_UNIT_GRID;

            if ( newTopUnit < 0 ) newTopUnit = 0;

            /* event visible in up-shifted display? */
            newEvt = MENU_EpgGridGetPrevVisibleEvt (inst, newTopUnit,
                                                inst->focusEvt, FFALSE);
            if ( newEvt >= 0 )
            {
                MENU_EpgGridMoveEventFocus (inst, newEvt);
            }
            else
            {
                /* shift display only */
                MENU_EpgGridSetDisplayTime (inst, newTopUnit);
            }
        }
    }

    if ( prevDay )
    {
        /* move to end of previous day */
        if ( inst->dateOffset > 0 )
        {
            inst->dateOffset--;
        }
        else
        {
            inst->dateOffset = MENU_EPG_GRID_DAYS_MAX - 1;
        }

        (void)FGS_PANEL_RefreshMember   (inst->pnlHdl, MENU_EPG_GRID_ID_DATE);

        /* set display frame to end of day */
        newTopUnit = MENU_EPG_GRID_UNITS_PER_DAY - MENU_EPG_GRID_UNITS_VISIBLE;

        newServLoc = inst->service[0].servLoc;

        MENU_EpgGridAssignServices (inst, &newServLoc);

        /* set focus on last visible event */
        inst->focusEvt = MENU_EpgGridGetPrevVisibleEvt (inst, newTopUnit,
                         -1, FFALSE);

        MENU_EpgGridUpdateFocusEvtData (inst);
        MENU_EpgGridUpdateDisplay      (inst, newTopUnit);
    }

    return FTRUE;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst    Grid EPG Instants.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgGridNavigateDown (MENU_EPG_GRID_INST_S* inst)
{
    int32_t         newTopUnit = inst->topUnit;
    fbool_t         nextDay    = FFALSE;
    SYS_SERVLOC_S   newServLoc;
    int16_t         newEvt;

    if ( inst->focusService == NULL ) return FFALSE;

    if ( inst->focusEvt < 0 )
    {
        if ( (newTopUnit + MENU_EPG_GRID_UNITS_VISIBLE) >=
              MENU_EPG_GRID_UNITS_PER_DAY )
        {
            nextDay = FTRUE;
        }
        else
        {
            newTopUnit += MENU_EPG_GRID_UNIT_GRID;

            if ( (newTopUnit + MENU_EPG_GRID_UNITS_VISIBLE) > MENU_EPG_GRID_UNITS_PER_DAY )
            {
                newTopUnit = MENU_EPG_GRID_UNITS_PER_DAY - MENU_EPG_GRID_UNITS_VISIBLE;
            }

            /* event visible in down-shifted display? */
            newEvt = MENU_EpgGridGetNextVisibleEvt (inst, newTopUnit, -1, FFALSE);

            if ( newEvt >= 0 )
            {
                MENU_EpgGridMoveEventFocus (inst, newEvt);
            }
            else
            {
                /* shift display only */
                MENU_EpgGridSetDisplayTime (inst, newTopUnit);
            }
        }
    }
    else
    {
        /* event visible in current display? */
        newEvt = MENU_EpgGridGetNextVisibleEvt (inst, newTopUnit, inst->focusEvt,
                                            FFALSE);

        if ( newEvt >= 0 )
        {
            MENU_EpgGridMoveEventFocus (inst, newEvt);
        }
        else if ( (newTopUnit + MENU_EPG_GRID_UNITS_VISIBLE) >=
                  MENU_EPG_GRID_UNITS_PER_DAY )
        {
            nextDay = FTRUE;
        }
        else
        {
            newTopUnit += MENU_EPG_GRID_UNIT_GRID;

            if ( (newTopUnit + MENU_EPG_GRID_UNITS_VISIBLE) > MENU_EPG_GRID_UNITS_PER_DAY )
            {
                newTopUnit = MENU_EPG_GRID_UNITS_PER_DAY - MENU_EPG_GRID_UNITS_VISIBLE;
            }

            /* event visible in down-shifted display? */
            newEvt = MENU_EpgGridGetNextVisibleEvt (inst, newTopUnit,
                                                inst->focusEvt, FFALSE);
            if ( newEvt >= 0 )
            {
                MENU_EpgGridMoveEventFocus (inst, newEvt);
            }
            else
            {
                /* shift display only */
                MENU_EpgGridSetDisplayTime (inst, newTopUnit);
            }
        }
    }

    if ( nextDay )
    {
        /* move to start of next day */
        inst->dateOffset++;

        if ( inst->dateOffset >= MENU_EPG_GRID_DAYS_MAX )
        {
            inst->dateOffset = 0;
        }

        (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPG_GRID_ID_DATE);

        /* set display frame to start of day */
        newTopUnit = 0;

        newServLoc = inst->service[0].servLoc;

        MENU_EpgGridAssignServices (inst, &newServLoc);

        /* set focus on firstvisible event */
        inst->focusEvt = MENU_EpgGridGetNextVisibleEvt (inst, newTopUnit,
                         -1, FFALSE);

        MENU_EpgGridUpdateFocusEvtData (inst);
        MENU_EpgGridUpdateDisplay      (inst, newTopUnit);
    }

    return FTRUE;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst    Grid EPG Instants.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgGridNavigateLeft (MENU_EPG_GRID_INST_S* inst)
{
    fbool_t                      keyUsed = FTRUE;
    const MENU_EPG_GRID_EVT_S*   pEvt;
    SYS_SERVLOC_S                newServLoc;
    MENU_EPG_GRID_EVT_S          evtFocus;

    if ( inst->focusService == NULL )
    {
        keyUsed = FFALSE;
    }
    /*lint -save -e946 */
    else if ( inst->focusService > inst->service )
    /*lint -restore */
    {
        /* move left by one column */
        MENU_EpgGridMoveServiceFocus (inst, inst->focusService - 1);
    }
    else if ( inst->numColumns < MENU_EPG_GRID_NUM_SERVICES )
    {
        keyUsed = FFALSE;
    }
    else if ( inst->focusService->servLoc.listId == SLIST_INVISIBLE )
    {
        /* Navigation among hidden services is prohibited */
        keyUsed = FFALSE;
    }
    else
    {
        /* move left out of currently displayed area */
        newServLoc = inst->service[0].servLoc;

        if ( newServLoc.index == 0 )
        {
            newServLoc.index = (int16_t)SLIST_ListSizeGet
                               (newServLoc.listId);
        }

        if ( newServLoc.index > 0 ) newServLoc.index--;

        if ( inst->focusEvt >= 0 )
        {
            /* store current focus event as reference for new focus
               event to be assigned */
            evtFocus = inst->focusService->evt[inst->focusEvt];
            pEvt     = &evtFocus;
        }
        else
        {
            pEvt = NULL;
        }

        MENU_EpgGridAssignServices   (inst, &newServLoc);
        MENU_EpgGridAssignFocusEvent (inst, pEvt, FTRUE);
        MENU_EpgGridUpdateDisplay    (inst, inst->topUnit);
    }

    return keyUsed;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst    Grid EPG Instants.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgGridNavigateRight (MENU_EPG_GRID_INST_S* inst)
{
    fbool_t                     keyUsed = FTRUE;
    const MENU_EPG_GRID_EVT_S*  pEvt;
    SYS_SERVLOC_S               newServLoc;
    MENU_EPG_GRID_EVT_S         evtFocus;

    if ( inst->focusService == NULL )
    {
        keyUsed = FFALSE;
    }
    /*lint -save -e946 */
    else if ( (inst->focusService + 1) <
              (inst->service + inst->numColumns) )
    /*lint -restore */
    {
        /* move right by one column */
        MENU_EpgGridMoveServiceFocus (inst, inst->focusService + 1);
    }
    else if ( inst->numColumns < MENU_EPG_GRID_NUM_SERVICES )
    {
        keyUsed = FFALSE;
    }
    else if ( inst->focusService->servLoc.listId == SLIST_INVISIBLE )
    {
        /* Navigation among hidden services is prohibited */
        keyUsed = FFALSE;
    }
    else
    {
        /* move right out of currently displayed area */
        newServLoc = inst->service[0].servLoc;

        newServLoc.index++;

        if ( newServLoc.index >= (int16_t)SLIST_ListSizeGet
                                 (newServLoc.listId) )
        {
            newServLoc.index = 0;
        }

        if (  inst->focusEvt >= 0 )
        {
            /* store current focus event as reference for new focus
               event to be assigned */
            evtFocus = inst->focusService->evt[inst->focusEvt];
            pEvt     = &evtFocus;
        }
        else
        {
            pEvt = NULL;
        }

        MENU_EpgGridAssignServices   (inst, &newServLoc);
        MENU_EpgGridAssignFocusEvent (inst, pEvt, FTRUE);
        MENU_EpgGridUpdateDisplay    (inst, inst->topUnit);
    }

    return keyUsed;
}

/*!
*******************************************************************************

** \brief  Get previous event visible in a virtual display frame.
**
** \param  inst     Pointer to the global Grid EPG data structure.
** \param  topUnit  Top unit used for calculation, not necessarily top unit
**                  currently active!
** \param  refIdx   If >= 0, first visible event prior to refIdx is returned,
**                  else last visible event is returned.
** \param  complete FTRUE: prefer completely visible event. If not available,
**                  partially visible event is returned if available.
**
** \return
**          - -1         No matching event found.
**          - others     Index of matching event.
**
** \sa
**          - MENU_EpgGridGetNextVisibleEvt()
**
*******************************************************************************
*/
FSTATIC int16_t MENU_EpgGridGetPrevVisibleEvt (MENU_EPG_GRID_INST_S* inst,
                                               int32_t topUnit,
                                               int16_t refIdx,
                                               fbool_t complete)
{
    int16_t              cnt;
    int16_t              maxCnt;
    MENU_EPG_GRID_EVT_S* pEvt;
    int16_t              evtIdx = -1;
    int16_t              altIdx = -1;

    if ( inst->focusService == NULL ) return evtIdx;

    if ( refIdx < 0 ) maxCnt = inst->focusService->numEvts - 1;
    else              maxCnt = refIdx - 1;

    for ( cnt = maxCnt; cnt >= 0; cnt-- )
    {
        pEvt = inst->focusService->evt + cnt;

        if ( pEvt->startUnit >= (uint32_t)(topUnit + MENU_EPG_GRID_UNITS_VISIBLE) )
        {
            /* not yet visible */
            continue;
        }
        else if ( pEvt->endUnit <= (uint32_t)topUnit )
        {
            /* not visible anymore */
            break;
        }
        else
        {
            if ( complete )
            {
                if ( altIdx < 0 ) altIdx = cnt;

                if ( pEvt->endUnit <= (uint32_t)
                                      (topUnit + MENU_EPG_GRID_UNITS_VISIBLE) )
                {
                    evtIdx = cnt;
                    break;
                }
            }
            else
            {
                evtIdx = cnt;
                break;
            }
        }
    }

    if ( evtIdx < 0 ) evtIdx = altIdx;

    return evtIdx;
}

/*!
*******************************************************************************
**
** \brief  Get next event visible in a virtual display frame.
**
** \param  inst     Pointer to the global Grid EPG data structure.
** \param  topUnit  Top unit used for calculation, not necessarily top unit
**                  currently active!
** \param  refIdx   If >= 0, next visible event after refIdx is returned, else
**                  first visible event is returned.
** \param  complete FTRUE: prefer completely visible event. If not available,
**                  partially visible event is returned if available.
**
** \return
**          - -1         No matching event found.
**          - others     Index of matching event.
**
** \sa
**          - MENU_EpgGridGetPrevVisibleEvt()
*******************************************************************************
*/
FSTATIC int16_t MENU_EpgGridGetNextVisibleEvt (MENU_EPG_GRID_INST_S* inst,
                                               int32_t topUnit,
                                               int16_t refIdx,
                                               fbool_t complete)
{
    int16_t              cnt;
    MENU_EPG_GRID_EVT_S* pEvt;
    int16_t              evtIdx = -1;
    int16_t              altIdx = -1;

    if ( inst->focusService == NULL ) return evtIdx;

    if ( refIdx < 0 ) cnt = 0;
    else              cnt = refIdx + 1;

    for ( ; cnt < inst->focusService->numEvts; cnt++ )
    {
        pEvt = inst->focusService->evt + cnt;

        if ( pEvt->endUnit <= (uint32_t)topUnit )
        {
            /* not yet visible */
            continue;
        }
        else if ( pEvt->startUnit
                  >= (uint32_t)(topUnit + MENU_EPG_GRID_UNITS_VISIBLE) )
        {
            /* not visible anymore */
            break;
        }
        else
        {
            if ( complete )
            {
                if ( altIdx < 0 ) altIdx = cnt;

                if ( pEvt->startUnit >= (uint32_t)topUnit )
                {
                    evtIdx = cnt;
                    break;
                }
            }
            else
            {
                evtIdx = cnt;
                break;
            }
        }
    }

    if ( evtIdx < 0 ) evtIdx = altIdx;

    return evtIdx;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst        Grid EPG Instants.
** \param   reqTopUnit  TODO
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridUpdateDisplay (MENU_EPG_GRID_INST_S* inst,
                                        int32_t reqTopUnit)
{
    uint8_t  cnt;

    reqTopUnit = MENU_EpgGridGetTopUnitForEvent (inst, reqTopUnit);

    if ( reqTopUnit != inst->topUnit )
    {
        /* update timeline + service slots */
        MENU_EpgGridSetDisplayTime (inst, reqTopUnit);
    }
    else
    {
        /* update service slots only */
        for ( cnt = 0; cnt < inst->numColumns; cnt++ )
        {
            (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                           MENU_EPG_GRID_ID_SERVTITLE + cnt);
            (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                           MENU_EPG_GRID_ID_SERVSCHED + cnt);
        }
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst        Grid EPG Instants.
** \param   newFocusEvt TODO
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridMoveEventFocus (MENU_EPG_GRID_INST_S* inst,
                                         int16_t newFocusEvt)
{
    int32_t newTopUnit;

    if ( inst->focusService == NULL ) return;

    inst->focusEvt = newFocusEvt;

    newTopUnit = MENU_EpgGridGetTopUnitForEvent (inst, inst->topUnit);

    if ( newTopUnit != inst->topUnit )
    {
        MENU_EpgGridSetDisplayTime (inst, newTopUnit);
    }
    else
    {
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPG_GRID_ID_SERVSCHED +
                                 inst->focusService->index);
    }

    MENU_EpgGridUpdateFocusEvtData(inst);
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst        Grid EPG Instants.
** \param   newTopUnit  TODO
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridSetDisplayTime (MENU_EPG_GRID_INST_S* inst,
                                         int32_t newTopUnit)
{
    uint8_t cnt;

    if ( newTopUnit < 0 ) return; /* invalid top unit: not an error! */

    inst->topUnit = newTopUnit;

    (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPG_GRID_ID_TIMELINE);

    for ( cnt = 0; cnt < inst->numColumns; cnt++ )
    {
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_EPG_GRID_ID_SERVTITLE + cnt);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl,
                                       MENU_EPG_GRID_ID_SERVSCHED + cnt);
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst            Grid EPG Instants.
** \param   pFirstServLoc   TODO
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridAssignServices (MENU_EPG_GRID_INST_S* inst,
                                         const SYS_SERVLOC_S* pFirstServLoc)
{
    MENU_EPG_GRID_SERVICE_S* pService = inst->service;
    uint8_t             cnt        = 0;
    uint16_t            listSize   = SLIST_ListSizeGet(pFirstServLoc->listId);
    int16_t             firstIndex = pFirstServLoc->index;
    SYS_SERVLOC_S       servLoc    = *pFirstServLoc;

    (void)EIT_RemoveFocusCurrNextAll();
    (void)EIT_RemoveFocusSchedAll();

    inst->numColumns = 0;

    if ( (servLoc.listId != SLIST_NONE) &&
         (servLoc.index  >= 0) )
    {
        for ( ; cnt < MENU_EPG_GRID_NUM_SERVICES; cnt++, pService++ )
        {
            if ( cnt > 0 )
            {
                if ( servLoc.index == firstIndex )
                {
                    /* avoid wraparound */
                    break;
                }
                else if ( servLoc.listId == SLIST_INVISIBLE )
                {
                    /* In case of hidden service, do not show other services! */
                    break;
                }
            }

            pService->servLoc = servLoc;
            pService->servIdx = SLIST_GetServiceIndex(&(pService->servLoc));

            memset (&(pService->servItem), 0,
                            sizeof(pService->servItem));

            (void)SLIST_EntryGet (&(pService->servLoc),
                            &(pService->servItem));

            /* set focus if not already done */
            if ( inst->focusService == NULL )
            {
                inst->focusService = pService;
            }

            /* Notify EIT database */
            (void)EIT_AddFocusCurrNext (&(pService->servLoc));
            (void)EIT_AddFocusSched    (&(pService->servLoc));

            /* Get events currently available for given service */
            MENU_EpgGridUpdateServiceSlot (inst, pService);

            servLoc.index++;

            if ( servLoc.index >= (int16_t)listSize )
            {
                servLoc.index = 0;
            }

            inst->numColumns++;
        }
    }

    /* remaining entries are empty/invalid */
    for ( ; cnt < MENU_EPG_GRID_NUM_SERVICES; cnt++, pService++ )
    {
        pService->servLoc.listId = SLIST_NONE;
        pService->servLoc.index  = -1;
        pService->servIdx        = SLIST_SERVIDX_NONE;
        pService->numEvts        = 0;

        memset(&(pService->servItem), 0, sizeof(pService->servItem));
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the Grid EPG Instants.
** \param   id      The item identifier.
** \param   index   TODO
** \param   state   The status of the item, FGS_FOCUS; FGS_NONFOCUS or
**                  FGS_DISABLED
** \param   item    Pointer to the item structure.
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridGetServiceDescr (FAPI_SYS_HandleT h,
                                          int32_t id,
                                          uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item)
{
    MENU_EPG_GRID_INST_S*   inst     = (MENU_EPG_GRID_INST_S*)h;
    SYS_TIME_S              timeEnd;
    uint32_t                evtDuration;

    if ( !(inst->evtDataValid) ) return;

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;
    item->item.str.font       = MAIN_FONT;
    item->item.str.lineHeight = 0;
    item->item.str.col        = OSD_COL_TEXT;

    switch ( id )
    {
        default:
        case MENU_EPG_GRID_ID_FOCUSNAME:
            item->item.str.chr = STR_Build (inst->evtData.nameCharSet,
                                            inst->evtData.name);
            break;

        case MENU_EPG_GRID_ID_FOCUSTIME:
            timeEnd = inst->evtData.startTime;

            evtDuration = TIME_ConvToSeconds(&(inst->evtData.duration));

            TIME_AddSeconds (&timeEnd, NULL, (int32_t)evtDuration);

            sprintf (appDat->asciiBuf, "%02d:%02d - %02d:%02d",
                     inst->evtData.startTime.hour,
                     inst->evtData.startTime.minute,
                     timeEnd.hour,
                     timeEnd.minute);

            item->item.str.chr = STR_IntASCIIStr();
            break;

        case MENU_EPG_GRID_ID_FOCUSDESCR:
            /* Show extended event decription if available. */
            if ( !(STR_IsEmpty(inst->evtExtData)) )
            {
                item->item.str.chr = inst->evtExtData;
            }
            else
            {
                item->item.str.chr = STR_Build (inst->evtData.descCharSet,
                                                inst->evtData.desc);
            }

            item->item.str.lineHeight = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
            break;
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the Grid EPG Instants.
** \param   id      The item identifier.
** \param   index   TODO
** \param   state   The status of the item, FGS_FOCUS; FGS_NONFOCUS or
**                  FGS_DISABLED
** \param   item    Pointer to the item structure.
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridGetServiceName (FAPI_SYS_HandleT h,
                                         int32_t id,
                                         uint32_t index,
                                         FGS_ITEMSTATE_E state,
                                         FGS_ITEM_S* item)
{
    MENU_EPG_GRID_INST_S*    inst     = (MENU_EPG_GRID_INST_S*)h;
    MENU_EPG_GRID_SERVICE_S* pService = inst->service + (id - MENU_EPG_GRID_ID_SERVTITLE);

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
    item->item.str.chr        = STR_Build (pService->servItem.nameCharSet,
                                           pService->servItem.name);
    item->item.str.font       = MAIN_FONT;
    item->item.str.lineHeight = 0;

    if ( pService == inst->focusService )
    {
        item->item.str.col    = OSD_COL_TEXT_HIGHLIGHTED;
    }
    else
    {
        item->item.str.col    = OSD_COL_TEXT_DISABLED;
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the Grid EPG Instants.
** \param   id      The item identifier.
** \param   index   TODO
** \param   state   The status of the item, FGS_FOCUS; FGS_NONFOCUS or
**                  FGS_DISABLED
** \param   item    Pointer to the item structure.
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridGetDate (FAPI_SYS_HandleT h,
                                  int32_t id,
                                  uint32_t index,
                                  FGS_ITEMSTATE_E state,
                                  FGS_ITEM_S* item)
{
    MENU_EPG_GRID_INST_S*   inst     = (MENU_EPG_GRID_INST_S*)h;
    SYS_TIME_S              currTime = inst->topTime;
    SYS_DATE_S              currDate = inst->topDate;

    (void)TIME_AddUtcOffset(&currTime, &currDate);

    TIME_AddDays(&currDate, inst->dateOffset);

    item->type                = FGS_ITEM_TYPE_DATE;
    item->align               = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
    item->item.date.value     = currDate;
    item->item.date.format    = appDat->dataFormatShort;
    item->item.date.font      = MAIN_FONT;
    item->item.date.col       = OSD_COL_TITLE_TEXT;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the Grid EPG Instants.
** \param   id      The item identifier.
** \param   index   TODO
** \param   state   The status of the item, FGS_FOCUS; FGS_NONFOCUS or
**                  FGS_DISABLED
** \param   item    Pointer to the item structure.
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridGetItem (FAPI_SYS_HandleT h,
                                  int32_t id,
                                  uint32_t index,
                                  FGS_ITEMSTATE_E state,
                                  FGS_ITEM_S* item)
{
    MENU_EPG_GRID_INST_S* inst = (MENU_EPG_GRID_INST_S*)h;

    item->type           = FGS_ITEM_TYPE_GENERIC;
    item->align          = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

    switch ( id )
    {
        case MENU_EPG_GRID_ID_TIMELINE:
            item->item.gnr.cb    = MENU_EpgGridDrawTimeLine;
            item->item.gnr.cbArg = h;
            break;

        default:
            item->item.gnr.cb    = MENU_EpgGridDrawSched;
            item->item.gnr.cbArg = (FAPI_SYS_HandleT)(inst->service +
                                   (id - MENU_EPG_GRID_ID_SERVSCHED));
            break;
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst    Grid EPG Instants.
** \param   servIdx TODO
**
** \return  Pointer to grid epg service type.
**
******************************************************************************
*/
FSTATIC MENU_EPG_GRID_SERVICE_S* MENU_EpgGridGetServiceSlot (MENU_EPG_GRID_INST_S* inst,
                                                    SLIST_SERVIDX servIdx)
{
    MENU_EPG_GRID_SERVICE_S* pService = inst->service;
    uint8_t                  cnt;

    if ( servIdx < 0 ) return NULL;

    for ( cnt = 0; cnt < inst->numColumns; cnt++, pService++ )
    {
        if ( pService->servIdx == servIdx ) return pService;
    }

    return NULL;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst        Grid EPG Instants.
** \param   pService    TODO
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridUpdateServiceSlot (MENU_EPG_GRID_INST_S* inst,
                                            MENU_EPG_GRID_SERVICE_S* pService)
{
    SYS_DATE_S           targetDate;
    SYS_DATE_S           nextDate;
    EIT_EVT_S            currEvt;
    EIT_EVT_S            nextEvt;
    EIT_POS_S            refPos;
    int32_t              index;
    MENU_EPG_GRID_EVT_S* pEntry  = pService->evt;
    uint32_t             refUnit = 0;
    uint32_t             evtDuration;
    SYS_TIME_S           endTime;
    SYS_DATE_S           endDate;
    int32_t              nextEvtStat;

    /* clear event list */
    pService->numEvts = 0;

    /* calculate current date */
    targetDate = inst->topDate;

    TIME_AddDays(&targetDate, inst->dateOffset);

    nextDate = targetDate;

    TIME_AddDays(&nextDate, 1);

    /* Get first event by date */
    index = EIT_GetSchedByDate (&(pService->servLoc), &currEvt, NULL, &refPos,
                                &(inst->topTime), &targetDate, FFALSE);

    /* If event exists, ensure that start time is on given day */
    if ( (index >= 0)
      && (TIME_IsLaterThan(&(inst->topTime), &nextDate,
                           &(currEvt.startTime), &(currEvt.startDate))) )
    {
        /* already try to get next event in order to be "ahead" by one event.
           This allows internal comparison of start and end times. */
        nextEvtStat = EIT_GetSchedNext(&nextEvt, NULL, &refPos);

        do
        {
            if ( pService->numEvts >= MENU_EPG_GRID_EVTPERDAY_MAX ) break;

            /* Test if start time of next event is on given day */
            if ( nextEvtStat == FAPI_OK )
            {
                if ( !(TIME_IsLaterThan(&(inst->topTime), &nextDate,
                                        &(nextEvt.startTime),
                                        &(nextEvt.startDate))) )
                {
                    nextEvtStat = SMARTGO_SYS_ERR_DATA_NOT_AVL;
                }
            }

            evtDuration  = TIME_ConvToSeconds (&(currEvt.duration));
            evtDuration -= 1;

            endTime = currEvt.startTime;
            endDate = currEvt.startDate;

            TIME_AddSeconds (&endTime, &endDate, (int32_t)evtDuration);

            if ( nextEvtStat == FAPI_OK )
            {
                /* If end time of current event is near start time of next
                   event, close that gap in order to ensure seamless
                   presentation on OSD. */

                if ( TIME_GetLengthInSeconds (&endTime, &(nextEvt.startTime))
                     <= MENU_EPG_GRID_GAP_TIME )
                {
                    endTime = nextEvt.startTime;
                }
            }

            /* Test if start time is on previous day */
            if ( TIME_IsLaterThan (&(inst->topTime), &targetDate,
                                   &(currEvt.startTime), &(currEvt.startDate)) )
            {
                pEntry->startUnit = 0;
            }
            else
            {
                pEntry->startUnit = MENU_EpgGridGetUnit (inst, &(currEvt.startTime));
            }

            /* Test if end time is on next day */
            if ( TIME_IsLaterThan (&endTime, &endDate,
                                   &(inst->topTime), &nextDate) )
            {
                pEntry->endUnit = MENU_EPG_GRID_UNITS_PER_DAY;
            }
            else
            {
                pEntry->endUnit = MENU_EpgGridGetUnit (inst, &endTime);
            }

            if ( pEntry->startUnit < refUnit )
            {
                pEntry->startUnit = refUnit;
            }

            /* add event to internal list */
            if ( pEntry->endUnit > pEntry->startUnit )
            {
                pEntry->index = index;
                pEntry->id    = currEvt.id;

                (void)STR_Copy (STR_Build(currEvt.nameCharSet, currEvt.name),
                          pEntry->name, &(pEntry->nameCharSet),
                          currEvt.nameCharSet,
                          AV_EIT_NAMELEN_MAX);

                refUnit = pEntry->endUnit;

                #ifdef APPL_RECTIMER_ENABLED

                if ( RECTIMER_GetIndexByEvtId (pService->servIdx,
                                               currEvt.id,
                                               &(pEntry->timerIdx)) != FAPI_OK )
                {
                    pEntry->timerIdx = RECTIMER_INVALID;
                }

                #endif /* APPL_RECTIMER_ENABLED */

                pService->numEvts++;
                pEntry++;
            }

            index++;

            /* abort if no more events */
            if ( nextEvtStat != FAPI_OK )
            {
                break;
            }

            currEvt = nextEvt;

            nextEvtStat = EIT_GetSchedNext (&nextEvt, NULL, &refPos);

        /*lint -save -e506 */
        } while ( 1 );
        /*lint -restore */
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst        Grid EPG Instants.
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridUpdateFocusEvtData (MENU_EPG_GRID_INST_S* inst)
{
    fbool_t      refreshNeeded = FFALSE;
    EIT_POS_S    refPos;
    EIT_EXTEVT_S extDesc;

    if ( (inst->focusService == NULL) || (inst->focusEvt < 0) )
    {
        if ( inst->evtDataValid )
        {
            inst->evtDataValid = FFALSE;
            refreshNeeded      = FTRUE;
        }
    }
    else
    {
        memset (&extDesc, 0, sizeof(extDesc));

        extDesc.pBuf    = inst->evtExtBuf;
        extDesc.maxLen  = AV_EIT_EXTLEN_SCHED;

        if ( EIT_GetSchedByIndex (&(inst->focusService->servLoc),
                            &(inst->evtData), &extDesc, &refPos,
                            (uint32_t)
                            (inst->focusService->evt[inst->focusEvt].index))
                            == FAPI_OK )
        {
            (void)TIME_AddUtcOffset (&(inst->evtData.startTime), NULL);

            inst->evtDataValid = FTRUE;
            refreshNeeded      = FTRUE;
        }
        else
        {
            if ( inst->evtDataValid )
            {
                inst->evtDataValid = FFALSE;
                refreshNeeded      = FTRUE;
            }
        }

        inst->evtExtData = extDesc.descStr;
    }

    if ( refreshNeeded )
    {
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPG_GRID_ID_FOCUSNAME);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPG_GRID_ID_FOCUSTIME);
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPG_GRID_ID_FOCUSDESCR);
    }
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst    Grid EPG Instants.
** \param   time    TODO
**
** \return  The number of units.
**
******************************************************************************
*/
FSTATIC uint32_t MENU_EpgGridGetUnit (MENU_EPG_GRID_INST_S* inst,
                                      SYS_TIME_S* time)
{
    uint32_t minutes;
    uint32_t units;
    uint32_t offset;

    time->second = 0; /* seconds are not taken into account */

    minutes = TIME_GetLengthInSeconds(&(inst->topTime), time) / 60;
    offset  = minutes % MENU_EPG_GRID_MINS_PER_UNIT;
    units   = minutes / MENU_EPG_GRID_MINS_PER_UNIT;

    if ( (offset * 2) >= MENU_EPG_GRID_MINS_PER_UNIT ) /* round upwards */
    {
        units++;
    }

    return units;
}

/*!
******************************************************************************
** \brief   TODO
**
** \param   inst        Grid EPG Instants.
** \param   refTopUnit  TODO
**
** \return  The id of start unit.
**
******************************************************************************
*/
FSTATIC int32_t MENU_EpgGridGetTopUnitForEvent (MENU_EPG_GRID_INST_S* inst,
                                                int32_t refTopUnit)
{
    MENU_EPG_GRID_EVT_S* pEvt;
    int32_t         newTopUnit = refTopUnit;

    if ( (inst->focusService != NULL) &&
         (inst->focusEvt     >= 0) )
    {
        pEvt = inst->focusService->evt + inst->focusEvt;

        if ( pEvt->endUnit > (uint32_t)(newTopUnit + MENU_EPG_GRID_UNITS_VISIBLE) )
        {
            newTopUnit = (int32_t)(pEvt->endUnit - MENU_EPG_GRID_UNITS_VISIBLE);
        }

        if (  ((int32_t)(pEvt->startUnit)) < newTopUnit )
        {
            newTopUnit = (int32_t)pEvt->startUnit;
        }
    }

    return newTopUnit;
}

#ifdef APPL_RECTIMER_ENABLED
/*!
******************************************************************************
** \brief   TODO
**
** \param   pService    TODO
** \param   pEvt        TODO
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_EpgGridHandleRecTimer (MENU_EPG_GRID_SERVICE_S* pService,
                                            MENU_EPG_GRID_EVT_S* pEvt)
{
    fbool_t         retVal = FTRUE;
    RECTIMER_ITEM_S timer;
    EIT_EVT_S       evtItem;
    EIT_POS_S       refPos;
    int32_t         lengthInSecs;
    int16_t         newPos;

    if ( pEvt->timerIdx == RECTIMER_INVALID ) /* Add new timer */
    {
        memset(&timer, 0, sizeof(timer));

        timer.serviceIndex = pService->servIdx;

        if ( EIT_GetSchedByIndex (&(pService->servLoc), &evtItem, NULL,
                                  &refPos, (uint32_t)(pEvt->index)) != FAPI_OK )
        {
            return FFALSE;
        }

        lengthInSecs = (int32_t)TIME_ConvToSeconds(&(evtItem.duration));

        (void)STR_Copy (STR_Build(evtItem.nameCharSet, evtItem.name),
                            timer.desc, &(timer.descCharSet),
                            evtItem.nameCharSet,
                            RECTIMER_DESC_LENGTH);

        (void)STR_Copy (STR_Build(evtItem.descCharSet, evtItem.desc),
                            timer.extDesc, &(timer.extDescCharSet),
                            evtItem.descCharSet,
                            RECTIMER_DESC_LENGTH);

        timer.type      = RECTIMER_TYPE_SINGLE;
        timer.enabled   = FTRUE;
        timer.startTime = evtItem.startTime;
        timer.startDate = evtItem.startDate;

        /* clear seconds: record at full minute only! */
        timer.startTime.second = 0;

        /* set duration + post-recording interval */
        TIME_AddSeconds (&(evtItem.startTime),
                         &(evtItem.startDate), lengthInSecs);

        timer.endTime     = evtItem.startTime;
        timer.endDate     = evtItem.startDate;
        timer.eventId     = pEvt->id;
        timer.autoStandby = FFALSE;

        #ifdef APPL_PVR_ENABLED

            timer.operation    = RECTIMER_OP_RECORD;
            timer.targetDevice = MAIN_PvrGetDefaultRecDevice ();

            if ( timer.targetDevice == SYS_DEV_NULL )
            {
                /* no recording device found */
                MENU_CommonMsgShow(STR_MSG_NORECDEVICE, FFALSE);
                retVal = FFALSE;
            }

        #else

            timer.operation    = RECTIMER_OP_SWITCH_SERVICE;
            timer.targetDevice = SYS_DEV_NULL;

        #endif /* APPL_PVR_ENABLED */

        /*lint -save -e774 */
        if ( retVal )
        /*lint -restore */
        {
            if ( RECTIMER_Add(&timer, &newPos) != FAPI_OK )
            {
                retVal = FFALSE;
            }
            else
            {
                (void)RECTIMER_GetIndex(newPos, &(pEvt->timerIdx));
                pService->inst->timerChanged = FTRUE;
            }
        }
    }
    else /* Remove timer */
    {
        if ( RECTIMER_RemoveByIndex(pEvt->timerIdx) == FAPI_OK )
        {
            pService->inst->timerChanged = FTRUE;
        }

        /* Remove timer indication in any case: Removal might have failed
           because of already expired timer, ... */
        pEvt->timerIdx = RECTIMER_INVALID;
    }

    if ( retVal )
    {
        (void)FGS_PANEL_RefreshMember (pService->inst->pnlHdl,
                                       MENU_EPG_GRID_ID_SERVSCHED + pService->index);
    }

    return retVal;
}
#endif /* APPL_RECTIMER_ENABLED */

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the Grid EPG Instants.
** \param   align   TODO
** \param   x       TODO
** \param   y       TODO
** \param   width   TODO
** \param   height  TODO
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridDrawTimeLine (FAPI_SYS_HandleT h,
                                       uint8_t align,
                                       uint16_t x,
                                       uint16_t y,
                                       uint16_t width,
                                       uint16_t height)
{
    MENU_EPG_GRID_INST_S* inst     = (MENU_EPG_GRID_INST_S*)h;
    uint16_t              posY     = 0;
    FGS_ITEM_S            item;
    SYS_TIME_S            currTime;
    SYS_DATE_S            currDate;
    SYS_TIME_S            timeShown = inst->topTime;
    SYS_DATE_S            dateShown = inst->topDate;
    fbool_t               timeSynced = FFALSE;
    uint32_t              pixelPerMin;

    item.type             = FGS_ITEM_TYPE_TIME;
    item.align            = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
    item.item.time.value  = inst->topTime;
    item.item.time.format = appDat->timeFormat;
    item.item.time.font   = MAIN_FONT;
    item.item.time.col    = OSD_COL_TITLE_TEXT;

    (void)TIME_AddUtcOffset (&(item.item.time.value), NULL);

    TIME_AddSeconds (&(item.item.time.value), NULL,
                    inst->topUnit * MENU_EPG_GRID_MINS_PER_UNIT * 60);

    while ( (uint16_t)(posY + SCALER_GetCurrVal(LIST_LINE_HEIGHT)) <= height )
    {
        /* draw every hour */
        if ( item.item.time.value.minute == 0 )
        {
            (void)FGS_DrawItem (&item, x, y + posY, width, SCALER_GetCurrVal(LIST_LINE_HEIGHT));
        }

        TIME_AddSeconds (&item.item.time.value, NULL,
                         MENU_EPG_GRID_MINS_PER_UNIT * 60);
        posY += SCALER_GetCurrVal(EPG_GRID_PIX_PER_UNIT);
    }

    item.type                = FGS_ITEM_TYPE_STRING;
    item.align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
    item.item.str.chr        = OSD_GetStr(STR_KEY_RIGHT);
    item.item.str.lineHeight = 0;
    item.item.str.font       = MAIN_FONT;
    item.item.time.col    = OSD_COL_TITLE_TEXT;

    (void)TIME_AddUtcOffset (&timeShown, &dateShown);
    TIME_AddDays(&dateShown, inst->dateOffset);

    (void)TIME_TimeAndDateGet(&currTime,&currDate,&timeSynced);
    if( timeSynced == FFALSE)
    {
        return;
    }
    TIME_AddSeconds (&timeShown, &dateShown,
                    inst->topUnit * MENU_EPG_GRID_MINS_PER_UNIT * 60);

    if  ( ( dateShown.day   != currDate.day   ) ||
          ( dateShown.month != currDate.month ) ||
          ( dateShown.year  != currDate.year  ) )
    {
        /* if not today */
        return;
    }
    pixelPerMin = SCALER_GetCurrVal(EPG_GRID_PIX_PER_UNIT)/ MENU_EPG_GRID_MINS_PER_UNIT;
    posY  = 60 * pixelPerMin * (currTime.hour - timeShown.hour);
    posY += pixelPerMin * (currTime.minute - timeShown.minute);

    /* check if inside visible area */
    if( ( posY >= (height - SCALER_GetCurrVal(LIST_LINE_HEIGHT)) ) ||
        ( posY <= (SCALER_GetCurrVal(LIST_LINE_HEIGHT)/2) ) )
    {
        return;
    }
    posY -= SCALER_GetCurrVal(LIST_LINE_HEIGHT)/2;
    (void)FGS_DrawItem (&item, x, y + posY, width,
                SCALER_GetCurrVal(LIST_LINE_HEIGHT));
}

/*!
******************************************************************************
** \brief   This function refresh the time line.
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridTimeUpdate (void)
{
    MENU_EPG_GRID_INST_S* inst     = menEpgGridDat;

    if(inst->pnlHdl != NULL)
    {
        (void)FGS_PANEL_RefreshMember (inst->pnlHdl, MENU_EPG_GRID_ID_TIMELINE);
    }

}

/*!
******************************************************************************
** \brief   TODO
**
** \param   h       Handle of the Grid EPG Instants.
** \param   align   TODO
** \param   x       TODO
** \param   y       TODO
** \param   width   TODO
** \param   height  TODO
**
******************************************************************************
*/
FSTATIC void MENU_EpgGridDrawSched (FAPI_SYS_HandleT h,
                                    uint8_t align,
                                    uint16_t x,
                                    uint16_t y,
                                    uint16_t width,
                                    uint16_t height)
{
    MENU_EPG_GRID_SERVICE_S* pService = (MENU_EPG_GRID_SERVICE_S*)h;
    MENU_EPG_GRID_EVT_S*     pEvt     = pService->evt;
    FGS_ITEM_S               item;
    uint16_t                 botUnit;
    int16_t                  evtCnt;
    uint16_t                 startDispUnit;
    uint16_t                 endDispUnit;
    uint8_t                  rectPad = MENU_EPG_GRID_RECT_PAD;
    uint16_t                 posY1;
    uint16_t                 posY2;
    uint16_t                 bkgrPosY;
    uint8_t                  colBkgr;
    uint16_t                 recWidth = 0;

    item.type                = FGS_ITEM_TYPE_STRING;
    item.align               = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
    item.item.str.font       = MAIN_FONT;
    item.item.str.lineHeight  = SCALER_GetCurrVal(LIST_LINE_HEIGHT);

    /*! Clear complete rectangle at first rendering after building of menu
     *  in order to fill also (static) horizontal padding space with
     *  background color. Only vertical padding is included in rendering
     * process.
     */

    if ( !(pService->cleared) )
    {
        OSD_DrawFilledRectangle (x, y,
                                 x + width - 1,
                                 y + height - 1,
                                 OSD_GetSchemeColor(OSD_SCHEMECOL_BLACK_BKGR));

        pService->cleared = FTRUE;
    }

    botUnit = (uint32_t)pService->inst->topUnit + MENU_EPG_GRID_UNITS_VISIBLE;

    if ( botUnit > MENU_EPG_GRID_UNITS_PER_DAY )
    {
        botUnit = MENU_EPG_GRID_UNITS_PER_DAY;
    }

    bkgrPosY = y;

    /* NOTE: Each event rectangle is padded by a rectangle of width
             MENU_EPG_GRID_RECT_PAD. Drawing of events is done as follows:

             for ( each event )
             {
                (1) Draw padding and empty space (time intervals where no
                    events are existing for) from top of column or from
                    last event.

                (2) + (3) Draw event rectangle and service name. Part of the
                          rectangle will be drawn in another color if a
                          recording timer is set on it.
             }

             (4) Draw remaining padding/empty space until bottom end of the
                 column.
    */

    for ( evtCnt = 0; evtCnt < pService->numEvts; evtCnt++, pEvt++ )
    {
        if ( pEvt->startUnit >= botUnit )
        {
            /* below visible area */
            break;
        }
        else if ( pEvt->endUnit <= (uint32_t)pService->inst->topUnit )
        {
            /* above visible area */
            continue;
        }

        if ( pEvt->startUnit < (uint32_t)pService->inst->topUnit )
        {
            startDispUnit = 0;
        }
        else
        {
            startDispUnit = pEvt->startUnit - (uint32_t)pService->inst->topUnit;
        }

        if ( pEvt->endUnit > botUnit )
        {
            endDispUnit = MENU_EPG_GRID_UNITS_VISIBLE;
        }
        else
        {
            endDispUnit = pEvt->endUnit - (uint32_t)pService->inst->topUnit;
        }

        posY1 = y + (startDispUnit * SCALER_GetCurrVal(EPG_GRID_PIX_PER_UNIT));
        posY2 = y + (endDispUnit   * SCALER_GetCurrVal(EPG_GRID_PIX_PER_UNIT));

        if ( (pService == pService->inst->focusService) &&
             (evtCnt   == pService->inst->focusEvt) )
        {
            colBkgr           = OSD_GetSchemeColor(OSD_SCHEMECOL_MAIN_BKGR_HIGHL);
            item.item.str.col = OSD_COL_TEXT_HIGHLIGHTED;
        }
        else
        {
            colBkgr           = OSD_GetSchemeColor(OSD_SCHEMECOL_TITLE_BKGR);
            item.item.str.col = OSD_COL_TEXT;
        }

        if ( (posY1 + rectPad) <= posY2 )
        {
            /* (1) draw column background between previous item and this item */
            if ( (posY1 + rectPad) > bkgrPosY )
            {
                OSD_DrawFilledRectangle (x + rectPad,
                                         bkgrPosY,
                                         x + width - rectPad - 1,
                                         posY1 + rectPad - 1,
                                         OSD_GetSchemeColor(OSD_SCHEMECOL_BLACK_BKGR));
            }

            #ifdef APPL_RECTIMER_ENABLED

            if ( pEvt->timerIdx == RECTIMER_INVALID )
            {
                recWidth = 0;
            }
            else
            {
                recWidth = SCALER_GetCurrVal(EPG_GRID_REC_COLUMN_WIDTH);

                /* (2) draw rectangle indicating that timer is set on event */
                if ( posY2 > (posY1 + rectPad) )
                {
                    OSD_DrawFilledRectangle (x + rectPad,
                                             posY1 + rectPad,
                                             x + rectPad + recWidth - 1,
                                             posY2 - 1,
                                             OSD_COL_RED);
                }
            }

            #endif /* APPL_RECTIMER_ENABLED */

            if ( posY2 > (posY1 + rectPad) )
            {
                /* (3) draw event rectangle + service name */
                OSD_DrawFilledRectangle (x + rectPad + recWidth,
                                         posY1 + rectPad,
                                         x + width - rectPad - 1,
                                         posY2 - 1,
                                         colBkgr);

                if ( (posY2 - posY1 - rectPad)
                     >= SCALER_GetCurrVal(LIST_LINE_HEIGHT) )
                {
                    /* draw service name */
                    item.item.str.chr = STR_Build(pEvt->nameCharSet, pEvt->name);

                    (void)FGS_DrawItem (&item,
                                  x      + rectPad,
                                  posY1  + rectPad,
                                  width  - (2 * rectPad),
                                  (posY2 - posY1 - rectPad));
                }
            }

            bkgrPosY = posY2;
        }
    }

    /* (4) draw remaining column background until bottom end of column */
    if ( (y + height) > bkgrPosY )
    {
        OSD_DrawFilledRectangle (x + rectPad,
                                 bkgrPosY,
                                 x + width - rectPad - 1,
                                 y + height - 1,
                                 OSD_GetSchemeColor(OSD_SCHEMECOL_BLACK_BKGR));
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
