/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_menu_streaminfo.c
**
**              $Id: app_menu_streaminfo.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       OSD Stream Information Menu.
**
** Detail description. [\ref app_smartgo_app_ref_streaminfo]
**
** For more detail documentation refer to the SmartGo-HD page:
** \ref app_smartgo_app_ref_streaminfo
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    715: symbol 'anySymb' not referenced
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1,
**                dependence placed on C's operator precedence;
**
*****************************************************************************
*/
/*lint -save -e641 -e715 -e961*/
#define DEBUG 1

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

/*!
 * FGS Item index for Stream Information Menu.
 */
enum
{
    MENU_STREAMINFO_ID_VIDEOPID = 0,
    MENU_STREAMINFO_ID_VIDEOPIC,
    MENU_STREAMINFO_ID_VIDEOFRATE,
    MENU_STREAMINFO_ID_VIDEOFORMAT,
    MENU_STREAMINFO_ID_VIDEOMAXBITRATE,
    MENU_STREAMINFO_ID_VIDEOREALBITRATE,
    MENU_STREAMINFO_ID_AUDIOPID,
    MENU_STREAMINFO_ID_AUDIOFORMAT,
    MENU_STREAMINFO_ID_AUDIOMODE,
    MENU_STREAMINFO_ID_AUDIOSAMPLE,
    MENU_STREAMINFO_ID_PCRPID,
    MENU_STREAMINFO_ID_TTXPID,
    MENU_STREAMINFO_ID_AUDIOPIDSPDIF,
    MENU_STREAMINFO_ID_TITLE,
    MENU_STREAMINFO_ID_LIST,
    MENU_STREAMINFO_ID_REST

};

/*!
 * Data storage structure for the Stream Information Menu.
 */
typedef struct
{
    FAPI_SYS_HandleT        pnlHdl;         //!< Handle for the menu Panel.
    FAPI_SYS_HandleT        listHdl;        //!< Handle for the menu list.
    FGS_PANEL_MEMBER_S*     pItem;          //!< Item pointer shared by all items.
    FGS_POS_S               itemPos;        //!< Position of the separator lines.

    int16_t                 currVideoPid;   //!< Currently shown Video PID.
    uint8_t                 currVideoType;  //!< Currently shown Video Type.
    int16_t                 currAudioPid;   //!< Currently shown Audio PID.
    int16_t                 currSpdifPid;   //!< Currently shown Audio PID on S/P-Dif.
    int16_t                 currPcrPid;     //!< Currently shown PCR PID.
    int16_t                 currTtxPid;     //!< Currently shown Teletext PID.
    AV_VIDEO_STREAMINFO_S   currVidInf;     //!< Other currently shon Video informations.
    AV_AUDIO_STREAMINFO_S   currAudInf;     //!< Other currently shon Audio informations.
    uint32_t                currMaxBitRate; //!< Maximum Bit Rate counter.

}MENU_STREAMINFO_INST_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
/*!
 * Pointer to the data storage structure for the Stream Information Menu.
 * The memory is allocated in MENU_StreamInfoInit().
 */
FSTATIC MENU_STREAMINFO_INST_S* menuStreamInfoDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC fbool_t MENU_StreamInfoLeave              (FAPI_SYS_HandleT h,
                                                   fbool_t force);
FSTATIC void    MENU_StreamInfoGetDescrItem       (FAPI_SYS_HandleT h,
                                                   int32_t id,
                                                   uint32_t index,
                                                   FGS_ITEMSTATE_E state,
                                                   FGS_ITEM_S* item);
FSTATIC void    MENU_StreamInfoGetItem            (FAPI_SYS_HandleT h,
                                                   int32_t id,
                                                   uint32_t index,
                                                   FGS_ITEMSTATE_E state,
                                                   FGS_ITEM_S* item);
FSTATIC fbool_t MENU_StreamInfoBuild              (FAPI_SYS_HandleT h);
FSTATIC void    MENU_StreamInfoUpdate             (void);
FSTATIC void    MENU_StreamInfoOnFocusSet         (FAPI_SYS_HandleT h,
                                                   fbool_t hasFocus);
FSTATIC fbool_t MENU_StreaminfoChannelUp          (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_StreaminfoChannelDown        (FAPI_SYS_HandleT h);
FSTATIC int32_t MENU_StreamInfoDrawSeparatorLines (FAPI_SYS_HandleT h);

/*!
******************************************************************************
** \brief   This function initialise the Stream Information Menu. It just
**          allocate the memory for the menuStreamInfoDat structure.
**
** \return  one of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_ALREADY_INITIALIZED
**          - #APPL_SMARTGO_ERR_OUT_OF_MEMORY
**
******************************************************************************
*/
int32_t MENU_StreamInfoInit (void)
{
    if ( menuStreamInfoDat != NULL )
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    menuStreamInfoDat = UTILS_Malloc(sizeof(MENU_STREAMINFO_INST_S));
    if ( menuStreamInfoDat == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    memset(menuStreamInfoDat, 0, sizeof(MENU_STREAMINFO_INST_S));

    return FAPI_OK;
}

/*!
******************************************************************************
** \brief   With this function you enter the Stream Information Menu.
**
** This function is called twice. First with \p test flag #FTRUE to check if
** the menu can be opend. If the first call was successful and return #FTRUE
** the second call will open the menu (\p test flag #FFALSE).
**
** \param   h       Handle of Stream Info Menu.
** \param   test    A flag to test if opening this menu is possible.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
fbool_t MENU_StreamInfoEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_STREAMINFO_INST_S* inst = (MENU_STREAMINFO_INST_S*)menuStreamInfoDat;

    if ( inst == NULL )
    {
        return FFALSE;
    }

    if ( test == FTRUE )
    {
        return FTRUE;
    }

    inst->currMaxBitRate = 0;

    (void)MENU_StreamInfoBuild((FAPI_SYS_HandleT)inst);
    MENU_StreamInfoUpdate ();

    /* start timer for periodic updating of dynamic items */
    appDat->osdUpdateCb = MENU_StreamInfoUpdate;

    // period: 400 ms
    (void)TIME_PeriodicTimerLaunch(OSD_PERIODTIMER_GENERAL, 4);
    /* set list focus on first item */

    // temporary used to enter the Debug menu
//   (void) FGS_LIST_SetMemberFocus  (inst->listHdl, 0);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_STREAMINFO_ID_LIST);
    /* set global focus on menu panel */
    (void)MENUSTACK_FocusClaim ((int32_t)MENUID_STRINF);
    /* turn visibility on */
    (void)FGS_PANEL_Show (inst->pnlHdl, FFALSE);

    (void)MENU_StreamInfoDrawSeparatorLines((FAPI_SYS_HandleT)inst);

    MENUSTACK_SetCleanup((MENUSTACK_CLEANUP)MENU_StreamInfoLeave,
                         (FAPI_SYS_HandleT)inst);

    return FTRUE;
}

/*!
******************************************************************************
** \brief   This function is a callback for the menustack. It sets or remove
**          the focus to the Stream Information Menu and allocatets the keys.
**
** \param   h           Handle (unused).
** \param   hasFocus    #FTRUE set the focus to this menu #FFALSE remove the
**                      focus.
**
******************************************************************************
*/
FSTATIC void MENU_StreamInfoOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_STREAMINFO_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);

    if ( hasFocus )
    {
        FGS_SetAppKeyHandler(FGS_KEY_UP,        MENU_StreaminfoChannelUp,   h, FFALSE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_CHAN_UP,   MENU_StreaminfoChannelUp,   h, FFALSE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_DOWN,      MENU_StreaminfoChannelDown, h, FFALSE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_CHAN_DOWN, MENU_StreaminfoChannelDown, h, FFALSE, FFALSE);
    }
    else
    {
        FGS_ClearAppKeyHandler(FGS_KEY_UP);
        FGS_ClearAppKeyHandler(FGS_KEY_DOWN);
        FGS_ClearAppKeyHandler(FGS_KEY_CHAN_UP);
        FGS_ClearAppKeyHandler(FGS_KEY_CHAN_DOWN);
    }
}

/*!
******************************************************************************
** \brief   This function is a callback for the menustack. It close the Stream
**          Information Menu.
**
** \param   h       Handle of Stream Info Menu.
** \param   force   (unused).
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_StreamInfoLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_STREAMINFO_INST_S* inst = (MENU_STREAMINFO_INST_S*)h;

    if ( inst->pnlHdl != NULL )
    {
        (void)TIME_PeriodicTimerAbort  (OSD_PERIODTIMER_GENERAL);
        (void) FGS_PANEL_Close(inst->pnlHdl);
    }

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_STRINF);

    inst->pnlHdl = NULL;
    return FTRUE;
}

/*!
******************************************************************************
** \brief   This Function is a callback for the FGS and creates the
**          description of the items.
**
** \param   h       Handle of Stream Info Menu.
** \param   id      The item identifier.
** \param   index   The current index of the item. (unused)
** \param   state   The status of the item, FGS_FOCUS; FGS_NONFOCUS or
**                  FGS_DISABLED
** \param   item    Pointer to the item structure.
**
******************************************************************************
*/
FSTATIC void MENU_StreamInfoGetDescrItem (FAPI_SYS_HandleT h,
                                          int32_t id,
                                          uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item)
{
    uint32_t strIdx;

    switch ( id )
    {
        case MENU_STREAMINFO_ID_VIDEOPID:
            strIdx = STR_ENTRY_STREAMINFO_VIDEOPID;
            break;
        case MENU_STREAMINFO_ID_VIDEOPIC:
            strIdx = STR_ENTRY_STREAMINFO_VIDEOPIC;
            break;
        case MENU_STREAMINFO_ID_VIDEOFRATE:
            strIdx = STR_ENTRY_STREAMINFO_VIDEOFRATE;
            break;
        case MENU_STREAMINFO_ID_VIDEOFORMAT:
            strIdx = STR_ENTRY_STREAMINFO_VIDEOFORMAT;
            break;
        case MENU_STREAMINFO_ID_VIDEOMAXBITRATE:
            strIdx = STR_ENTRY_STREAMINFO_VIDEOMAXBITRATE;
            break;
        case MENU_STREAMINFO_ID_VIDEOREALBITRATE:
            strIdx = STR_ENTRY_STREAMINFO_VIDOEREALBITRATE;
            break;
        case MENU_STREAMINFO_ID_AUDIOPID:
            strIdx = STR_ENTRY_STREAMINFO_AUDIOPID;
            break;
        case MENU_STREAMINFO_ID_AUDIOFORMAT:
            strIdx = STR_ENTRY_STREAMINFO_AUDIOFORMAT;
            break;
        case MENU_STREAMINFO_ID_AUDIOMODE:
            strIdx = STR_ENTRY_STREAMINFO_AUDIOMODE;
            break;
        case MENU_STREAMINFO_ID_AUDIOSAMPLE:
            strIdx = STR_ENTRY_STREAMINFO_AUDIOSAMPLE;
            break;
        case MENU_STREAMINFO_ID_PCRPID:
            strIdx = STR_ENTRY_STREAMINFO_PCRPID;
            break;
        case MENU_STREAMINFO_ID_TTXPID:
            strIdx = STR_ENTRY_STREAMINFO_TTXPID;
            break;
        case MENU_STREAMINFO_ID_AUDIOPIDSPDIF:
            strIdx = STR_ENTRY_STREAMINFO_AUDIOPIDSPDIF;
            break;
        default:
            strIdx = STR_EMPTYSTRING;
            break;
    }

    MENU_COMMON_ITEM_MENUSTR(item, strIdx, state);
}

/*!
******************************************************************************
** \brief   This Function is a callback for the FGS and creates the Stream
**          Information items.
**
**
** \param   h       Handle of Stream Info Menu.
** \param   id      The item identifier.
** \param   index   The current index of the item. (unused)
** \param   state   The status of the item, FGS_FOCUS; FGS_NONFOCUS or
**                  FGS_DISABLED
** \param   item    Pointer to the item structure.
**
******************************************************************************
*/
FSTATIC void MENU_StreamInfoGetItem (FAPI_SYS_HandleT h,
                                     int32_t id,
                                     uint32_t index,
                                     FGS_ITEMSTATE_E state,
                                     FGS_ITEM_S* item)
{
    MENU_STREAMINFO_INST_S* inst = (MENU_STREAMINFO_INST_S*)h;
    uint32_t                bufPos = 0;
    fbool_t                 invalid = FFALSE;
    uint32_t                strIdx;
    const char_t*           tmpStr;
    AV_AUDIO_STREAMINFO_S   audioInfo;
    AV_VIDEO_STREAMINFO_S   videoInfo;

    memset(&audioInfo,  0, sizeof(audioInfo));
    memset(&videoInfo,  0, sizeof(videoInfo));

    (void)AV_GetAudioStreamInfo(&audioInfo);
    (void)AV_GetVideoStreamInfo(&videoInfo);
    (void)STR_BufClear (NULL); /* default: empty string */

    switch ( id )
    {
        case MENU_STREAMINFO_ID_VIDEOPID:
            if(inst->currVideoPid > 0)
            {
                sprintf (appDat->asciiBuf, "%i", inst->currVideoPid);
                bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                         MAIN_LINEBUFCHARSET);
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(STR_GNR_N_A),
                                         MAIN_LINEBUFCHARSET);
            }

        break;
        case MENU_STREAMINFO_ID_VIDEOPIC:
            if(inst->currVideoPid > 0)
            {
                sprintf (appDat->asciiBuf, "%ix%i   ",
                         (int16_t)(videoInfo.decoder.displayWidth),
                         (int16_t)(videoInfo.decoder.displayHeight));

                bufPos += STR_BufAppend (NULL, bufPos,
                                         STR_IntASCIIStr(), MAIN_LINEBUFCHARSET);

                switch ( videoInfo.decoder.aspectRatio )
                {
                    case FAPEX_VIDEC_AR_4_3:
                        tmpStr = "4:3";
                    break;
                    case FAPEX_VIDEC_AR_16_9:
                        tmpStr = "16:9";
                    break;
                    case FAPEX_VIDEC_AR_14_9:
                        tmpStr = "14:9";
                    break;
                    case FAPEX_VIDEC_AR_221_100:
                        tmpStr = "2,21:1";
                    break;
                    case FAPEX_VIDEC_AR_235_100:
                        tmpStr = "2,35:1";
                    break;
                    default:
                        tmpStr = "";
                        invalid = FTRUE;
                    break;
                }
                if ( invalid )
                {
                    bufPos += STR_BufAppend (NULL, bufPos,
                            OSD_GetStr(STR_VALUE_STREAMINFO_VIDEOFRATE_INVALID),
                            MAIN_LINEBUFCHARSET);

                    sprintf (appDat->asciiBuf, " %i",
                             videoInfo.decoder.aspectRatio);

                    bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                             MAIN_LINEBUFCHARSET);
                }
                else
                {
                    bufPos += STR_BufAppend (NULL, bufPos, STR_GetASCII(tmpStr),
                                             MAIN_LINEBUFCHARSET);
                }
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos, OSD_GetStr(STR_GNR_N_A),
                                         MAIN_LINEBUFCHARSET);
            }
            break;
        case MENU_STREAMINFO_ID_VIDEOFRATE:
            if(inst->currVideoPid > 0)
            {
                switch ( videoInfo.decoder.frameRate )
                {
                    case 2397 : tmpStr = "23.97 "; break;
                    case 2400 : tmpStr = "24 ";    break;
                    case 2500 : tmpStr = "25 ";    break;
                    case 2997 : tmpStr = "29.97 "; break;
                    case 3000 : tmpStr = "30 ";    break;
                    case 5000 : tmpStr = "50 ";    break;
                    case 5994 : tmpStr = "59.94 "; break;
                    case 6000 : tmpStr = "60 ";    break;
                    default:
                        tmpStr = "";
                        invalid = FTRUE;
                    break;
                }
                if ( invalid )
                {
                    bufPos += STR_BufAppend (NULL, bufPos,
                             OSD_GetStr(STR_VALUE_STREAMINFO_VIDEOFRATE_INVALID),
                             MAIN_LINEBUFCHARSET);

                    sprintf (appDat->asciiBuf, " %i",
                             (int16_t)(videoInfo.decoder.frameRate));

                    bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                             MAIN_LINEBUFCHARSET);
                }
                else
                {
                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_GetASCII(tmpStr),
                                             MAIN_LINEBUFCHARSET);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             OSD_GetStr(STR_UNIT_HZ),
                                             MAIN_LINEBUFCHARSET);
                }
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos, OSD_GetStr(STR_GNR_N_A),
                                         MAIN_LINEBUFCHARSET);
            }
            break;
        case MENU_STREAMINFO_ID_VIDEOFORMAT:
            if(inst->currVideoPid > 0)
            {
                switch(inst->currVideoType)
                {
                    case 0x01:
                        strIdx = STR_VALUE_STREAMINFO_VIDEOFORMAT_MPEG1;
                        break;
                    case 0x02:
                        strIdx = STR_VALUE_STREAMINFO_VIDEOFORMAT_MPEG2;
                        break;
                    case 0x1B:
                        strIdx = STR_VALUE_STREAMINFO_VIDEOFORMAT_H264;
                        break;
                    case 0xEA:
                        strIdx = STR_VALUE_STREAMINFO_VIDEOFORMAT_VC1;
                        break;
                    default:
                        strIdx = STR_VALUE_STREAMINFO_VIDEOFORMAT_UNKNOWN;
                        break;
                }
                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(strIdx),
                                         MAIN_LINEBUFCHARSET);
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos, OSD_GetStr(STR_GNR_N_A),
                                         MAIN_LINEBUFCHARSET);
            }
        break;

        case MENU_STREAMINFO_ID_VIDEOMAXBITRATE:
            if(inst->currVideoPid > 0)
            {
                sprintf (appDat->asciiBuf," %li ", inst->currMaxBitRate/1024);

                bufPos += STR_BufAppend (NULL, bufPos,
                                         STR_IntASCIIStr(),
                                         MAIN_LINEBUFCHARSET);

                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(STR_UNIT_KBITSEC),
                                         MAIN_LINEBUFCHARSET);
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos, OSD_GetStr(STR_GNR_N_A),
                                         MAIN_LINEBUFCHARSET);
            }
        break;

        case MENU_STREAMINFO_ID_VIDEOREALBITRATE:
            if(inst->currVideoPid > 0)
            {
                sprintf (appDat->asciiBuf, "%li ",
                         videoInfo.decoder.currBitRate/1024);

                bufPos += STR_BufAppend (NULL, bufPos,
                                         STR_IntASCIIStr(),
                                         MAIN_LINEBUFCHARSET);

                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(STR_UNIT_KBITSEC),
                                         MAIN_LINEBUFCHARSET);
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos, OSD_GetStr(STR_GNR_N_A),
                                         MAIN_LINEBUFCHARSET);
            }
        break;

        case MENU_STREAMINFO_ID_AUDIOPID:
            if(inst->currAudioPid > 0)
            {
                sprintf (appDat->asciiBuf, "%i", inst->currAudioPid);
                bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                         MAIN_LINEBUFCHARSET);
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(STR_GNR_N_A),
                                         MAIN_LINEBUFCHARSET);
            }
        break;

        case MENU_STREAMINFO_ID_AUDIOFORMAT:
            if(inst->currAudioPid > 0)
            {
                switch ( audioInfo.stream.type )
                {
                    case FAPEX_AUOUT_STREAM_AAC:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOFORMAT_AAC;
                        break;
                    case FAPEX_AUOUT_STREAM_AC3:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOFORMAT_AC3;
                        break;
                    case FAPEX_AUOUT_STREAM_WMA:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOFORMAT_WMA;
                        break;
                    case FAPEX_AUOUT_STREAM_EC3:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOFORMAT_EC3;
                        break;
                    case FAPEX_AUOUT_STREAM_MPEG_L1:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOFORMAT_I;
                        break;
                    case FAPEX_AUOUT_STREAM_MPEG_L2:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOFORMAT_II;
                        break;
                    case FAPEX_AUOUT_STREAM_MPEG_L3:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOFORMAT_III;
                        break;
                    case FAPEX_AUOUT_STREAM_HEAAC:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOFORMAT_HEAAC;
                        break;
                    default:    strIdx = STR_VALUE_STREAMINFO_AUDIOFORMAT_INVALID;
                                invalid = FTRUE;
                                break;
                }

                bufPos += STR_BufAppend (NULL, bufPos, OSD_GetStr(strIdx),
                                         MAIN_LINEBUFCHARSET);

                if ( invalid )
                {
                    sprintf (appDat->asciiBuf, " %i", audioInfo.stream.type);

                    bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                             MAIN_LINEBUFCHARSET);
                }
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos, OSD_GetStr(STR_GNR_N_A),
                                         MAIN_LINEBUFCHARSET);
            }
        break;

        case MENU_STREAMINFO_ID_AUDIOMODE:
            if(inst->currAudioPid > 0)
            {
                switch ( audioInfo.stream.channels )
                {
                    case AAL_AUOUT_CHANNEL_MONO:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOMODE_MONO;
                        break;
                    case AAL_AUOUT_CHANNEL_STEREO:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOMODE_STEREO;
                        break;
                    case AAL_AUOUT_CHANNEL_DUALMONO:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOMODE_DUALMONO;
                        break;
                    case AAL_AUOUT_CHANNEL_C:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOMODE_C;
                        break;
                    case AAL_AUOUT_CHANNEL_L_C_R:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOMODE_L_C_R;
                        break;
                    case AAL_AUOUT_CHANNEL_L_R_S:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOMODE__L_R_S;
                        break;
                    case AAL_AUOUT_CHANNEL_L_C_R_S:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOMODE_L_C_R_S;
                        break;
                    case AAL_AUOUT_CHANNEL_L_R_SL_SR:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOMODE_L_R_SL_SR;
                        break;
                    case AAL_AUOUT_CHANNEL_L_C_R_SL_SR:
                        strIdx = STR_VALUE_STREAMINFO_AUDIOMODE_L_C_R_SL_SR;
                        break;
                    default: strIdx = STR_VALUE_STREAMINFO_AUDIOMODE_INVALID;
                             invalid = FTRUE;
                             break;
                }
                bufPos += STR_BufAppend (NULL, bufPos, OSD_GetStr(strIdx),
                                         MAIN_LINEBUFCHARSET);

                if ( invalid )
                {
                    sprintf (appDat->asciiBuf, " %i", audioInfo.stream.channels);

                    bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                             MAIN_LINEBUFCHARSET);
                }
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos, OSD_GetStr(STR_GNR_N_A),
                                         MAIN_LINEBUFCHARSET);
            }
        break;

        case MENU_STREAMINFO_ID_AUDIOSAMPLE:
            if(inst->currAudioPid > 0)
            {
                switch ( audioInfo.stream.sampleFreq )
                {
                    case AAL_AUOUT_8000_HZ:  tmpStr = "8";     break;
                    case AAL_AUOUT_12000_HZ: tmpStr = "12";    break;
                    case AAL_AUOUT_16000_HZ: tmpStr = "16";    break;
                    case AAL_AUOUT_22050_HZ: tmpStr = "22,05"; break;
                    case AAL_AUOUT_24000_HZ: tmpStr = "24";    break;
                    case AAL_AUOUT_32000_HZ: tmpStr = "32";    break;
                    case AAL_AUOUT_44100_HZ: tmpStr = "44,1";  break;
                    case AAL_AUOUT_48000_HZ: tmpStr = "48";    break;
                    case AAL_AUOUT_96000_HZ: tmpStr = "96";    break;
                    default:
                        tmpStr = "";
                        invalid = FTRUE;
                    break;
                }

                if ( invalid )
                {
                    bufPos += STR_BufAppend (NULL, bufPos,
                             OSD_GetStr(STR_VALUE_STREAMINFO_AUDIOSAMPLE_INVALID),
                             MAIN_LINEBUFCHARSET);

                    sprintf (appDat->asciiBuf, " %i", audioInfo.stream.sampleFreq);

                    bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                             MAIN_LINEBUFCHARSET);
                }
                else
                {
                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_GetASCII(tmpStr),
                                             MAIN_LINEBUFCHARSET);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             OSD_GetStr(STR_UNIT_KHZ),
                                             MAIN_LINEBUFCHARSET);
                }
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos, OSD_GetStr(STR_GNR_N_A),
                                         MAIN_LINEBUFCHARSET);
            }
        break;

        case MENU_STREAMINFO_ID_PCRPID:
            if(inst->currPcrPid > 0)
            {
                sprintf (appDat->asciiBuf, "%i", inst->currPcrPid);
                bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                         MAIN_LINEBUFCHARSET);
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(STR_GNR_N_A),
                                         MAIN_LINEBUFCHARSET);
            }
        break;

        case MENU_STREAMINFO_ID_TTXPID:
            if(inst->currTtxPid > 0)
            {
                sprintf (appDat->asciiBuf, "%i", inst->currTtxPid);
                bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                         MAIN_LINEBUFCHARSET);
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(STR_GNR_N_A),
                                         MAIN_LINEBUFCHARSET);
            }
        break;
        case MENU_STREAMINFO_ID_AUDIOPIDSPDIF:
            if( inst->currSpdifPid  > 0 )
            {
                sprintf (appDat->asciiBuf, "%i", inst->currSpdifPid);
                bufPos += STR_BufAppend (NULL, bufPos, STR_IntASCIIStr(),
                                         MAIN_LINEBUFCHARSET);
            }
            else
            {
                bufPos += STR_BufAppend (NULL, bufPos,
                                         OSD_GetStr(STR_GNR_N_A),
                                         MAIN_LINEBUFCHARSET);
            }
            break;

        default:
            break;
    }

    item->item.str.font       = MAIN_FONT;
    item->align               = FGS_ALIGN_HOR_RIGHT | FGS_ALIGN_VER_CENTER;
    item->type                = FGS_ITEM_TYPE_STRING;
    item->item.str.lineHeight = 0;
    item->item.str.chr        = STR_IntLineStr();

    switch ( state )
    {
        default:
        case FGS_NONFOCUS:
            item->item.str.col = appDat->txtAttr.colNonFocus;
            break;
        case FGS_FOCUS:
            item->item.str.col = appDat->txtAttr.colFocus;
            break;
        case FGS_DISABLED:
            item->item.str.col = appDat->txtAttr.colDisabled;
            break;
    }
}

/*!
******************************************************************************
** \brief   This function is used as callback for TIME_PeriodicTimerLaunch()
**          it is call every 200 ms.
**          All items of the menu will be checked here if the have to be
**          updated.
**
** \sa
**          - MENU_StreamInfoEntry()
******************************************************************************
*/
FSTATIC void MENU_StreamInfoUpdate (void)
{
    MENU_STREAMINFO_INST_S* inst = menuStreamInfoDat;
    SYS_SERVLOC_S           servLoc;
    SLIST_ITEM_S            listItem;
    int8_t                  audioStrIdx = -1;
    int8_t                  spdifStrIdx = -1;
    AV_SPDIF_STATUS_E       spdifType;
    AV_AUDIO_STREAMINFO_S   audioInfo;
    AV_VIDEO_STREAMINFO_S   videoInfo;
    uint8_t                 videoType = 0;
    int16_t                 videoPid = -1;
    int16_t                 audioPid = -1;
    int16_t                 pcrPid   = -1;
    int16_t                 ttxPid   = -1;
    int16_t                 spdifPid = -1;
    int32_t                 retVal;
    SYS_PRESSTATE_E         presState;
    #ifdef APPL_PVR_ENABLED
    SYS_STREAM_INFO_S       playInfo;
    #endif

    memset(&audioInfo,  0, sizeof(audioInfo));
    memset(&videoInfo,  0, sizeof(videoInfo));
    memset(&servLoc,    0, sizeof(servLoc));
    memset(&listItem,   0, sizeof(listItem));

    retVal = SYS_GetPresentationState(&presState);
    if ( retVal != FAPI_OK )
    {
        return;
    }

    (void)AV_GetAudioStreamInfo(&audioInfo);
    (void)AV_GetVideoStreamInfo(&videoInfo);

    switch( presState )
    {
        default:
        case SYS_PRESSTATE_LIVETV:
            (void)AV_GetRequestedService(&servLoc);
            (void)SLIST_EntryGet(&servLoc,&listItem);
            (void)AV_GetAudioStream(&audioStrIdx);
            (void)AV_GetSPDIFStatus(&spdifType,&spdifStrIdx);

            if(listItem.strPid.valid)
            {
                videoPid  = (int16_t)listItem.strPid.video;
                videoType = listItem.strPid.videoType;
                audioPid  = listItem.strPid.audio.stream[audioStrIdx].pid;
                switch(spdifType)
                {
                    case SPDIFSTATUS_PASSTHROUGH:
                        spdifPid  = listItem.strPid.audio.stream[spdifStrIdx].pid;
                        break;
                    case SPDIFSTATUS_CURRPCM:
                        spdifPid  = audioPid;
                        break;
                    default:
                        spdifPid = -1;
                        break;
                }
                pcrPid    = listItem.strPid.pcr;
                ttxPid    = (int16_t)listItem.strPid.ttx;
            }
            break;
        #ifdef APPL_PVR_ENABLED

        case SYS_PRESSTATE_PLAYBACK:
            memset(&playInfo, 0, sizeof(playInfo));
            retVal = PVRSYS_PlayGetStreamInfo (&playInfo);
            if ( ( playInfo.valid ) &&
                 ( retVal == FAPI_OK ) )
            {
                (void)PVRSYS_PlayGetAudioStream(&audioStrIdx);
                (void)PVRSYS_PlayGetSPDIFStatus(&spdifType,&spdifStrIdx);
                videoPid  = (int16_t)playInfo.video;
                videoType = playInfo.videoType;
                audioPid  = playInfo.audio.stream[audioStrIdx].pid;
                switch(spdifType)
                {
                    case SPDIFSTATUS_PASSTHROUGH:
                        spdifPid  = playInfo.audio.stream[spdifStrIdx].pid;
                        break;
                    case SPDIFSTATUS_CURRPCM:
                        spdifPid  = audioPid;
                        break;
                    default:
                        spdifPid = -1;
                        break;
                }
                pcrPid    = playInfo.pcr;
                ttxPid    = (int16_t)playInfo.ttx;
            }
            break;
        #endif /* APPL_PVR_ENABLED */
    }

    if(videoPid != inst->currVideoPid)
    {
        inst->currVideoPid = videoPid;
        (void)FGS_LIST_RefreshMember(inst->listHdl,
                                     MENU_STREAMINFO_ID_VIDEOPID );
    }
    if ( (videoInfo.decoder.aspectRatio   !=
                                     inst->currVidInf.decoder.aspectRatio)  ||
         (videoInfo.decoder.decodedWidth  !=
                                     inst->currVidInf.decoder.decodedWidth) ||
         (videoInfo.decoder.decodedHeight !=
                                     inst->currVidInf.decoder.decodedHeight) )
    {
        inst->currVidInf.decoder.aspectRatio   = videoInfo.decoder.aspectRatio;
        inst->currVidInf.decoder.decodedWidth  = videoInfo.decoder.decodedWidth;
        inst->currVidInf.decoder.decodedHeight = videoInfo.decoder.decodedHeight;
        (void)FGS_LIST_RefreshMember(inst->listHdl,
                                     MENU_STREAMINFO_ID_VIDEOPIC );

    }
    if(videoInfo.decoder.frameRate != inst->currVidInf.decoder.frameRate)
    {
        inst->currVidInf.decoder.frameRate = videoInfo.decoder.frameRate;
        (void)FGS_LIST_RefreshMember(inst->listHdl,
                                     MENU_STREAMINFO_ID_VIDEOFRATE );
    }
    if(videoType != inst->currVideoType)
    {
        inst->currVideoType = videoType;
        (void)FGS_LIST_RefreshMember(inst->listHdl,
                                     MENU_STREAMINFO_ID_VIDEOFORMAT );
    }

    if( (videoInfo.decoder.currBitRate /1000) !=
                            (inst->currVidInf.decoder.currBitRate/1000) )
    {
        inst->currVidInf.decoder.currBitRate =
                                        videoInfo.decoder.currBitRate;
        (void)FGS_LIST_RefreshMember(inst->listHdl,
                                     MENU_STREAMINFO_ID_VIDEOREALBITRATE );
    }
    if(videoInfo.decoder.currBitRate > inst->currMaxBitRate)
    {
        inst->currMaxBitRate = videoInfo.decoder.currBitRate;
        (void)FGS_LIST_RefreshMember(inst->listHdl,
                                     MENU_STREAMINFO_ID_VIDEOMAXBITRATE);
    }
    if(audioPid != inst->currAudioPid)
    {
        inst->currAudioPid = audioPid;
        (void)FGS_LIST_RefreshMember(inst->listHdl,
                                     MENU_STREAMINFO_ID_AUDIOPID );
    }
    if ( audioInfo.stream.type != inst->currAudInf.stream.type )
    {
        inst->currAudInf.stream.type = audioInfo.stream.type;
        (void)FGS_LIST_RefreshMember(inst->listHdl,
                                     MENU_STREAMINFO_ID_AUDIOFORMAT );
    }
    if ( audioInfo.stream.channels != inst->currAudInf.stream.channels )
    {
        inst->currAudInf.stream.channels = audioInfo.stream.channels;
        (void)FGS_LIST_RefreshMember(inst->listHdl,
                                     MENU_STREAMINFO_ID_AUDIOMODE );
    }
    if ( audioInfo.stream.sampleFreq != inst->currAudInf.stream.sampleFreq )
    {
        inst->currAudInf.stream.sampleFreq = audioInfo.stream.sampleFreq;
        (void)FGS_LIST_RefreshMember(inst->listHdl,
                                     MENU_STREAMINFO_ID_AUDIOSAMPLE );
    }
    if ( pcrPid != inst->currPcrPid )
    {
        inst->currPcrPid = pcrPid;
        (void)FGS_LIST_RefreshMember(inst->listHdl,
                                     MENU_STREAMINFO_ID_PCRPID );
    }
    if ( ttxPid != inst->currTtxPid )
    {
        inst->currTtxPid = ttxPid;
        (void)FGS_LIST_RefreshMember(inst->listHdl,
                                     MENU_STREAMINFO_ID_TTXPID );
    }
    if ( spdifPid != inst->currSpdifPid )
    {
        inst->currSpdifPid = spdifPid;
        (void)FGS_LIST_RefreshMember(inst->listHdl,
                                     MENU_STREAMINFO_ID_AUDIOPIDSPDIF );
    }
}

/*!
******************************************************************************
** \brief   This function sets up the menu structure and creates the FGS items.
**
** \param   h       Handle of Stream Info Menu.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_StreamInfoBuild(FAPI_SYS_HandleT h)
{
    MENU_STREAMINFO_INST_S* inst = (MENU_STREAMINFO_INST_S*)h;
    FGS_PANEL_OPEN_S        pnlPars;
    FGS_LIST_OPEN_S         listPars;
    FAPI_SYS_HandleT        itemHdl;
    FGS_ITEM_TXT_OPEN_S     txtPars;
    FGS_POS_S               itemPos;
    FGS_POS_S               valuePos;
    FGS_POS_S               descrPos;
    FGS_PANEL_MEMBER_S      member;
    int32_t                 retVal;
    uint16_t                helpWidth;
    uint16_t                stackedFrameWidth;
    uint16_t                innerHeight;
    uint32_t                numLines = 13;

    memset(&txtPars,  0, sizeof(txtPars));
    memset(&member,   0, sizeof(member));

    helpWidth         = 2*SCALER_GetCurrVal(HELP_LR_PADDING) +
                          OSD_GetStrWidth(OSD_GetStr(STR_ENTRY_OK),
                                          -1,SYMBOL_FONT);
    innerHeight       = numLines *
                        (SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                         SCALER_GetCurrVal(LIST_LINE_PADDING));

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        MENU_CommonSetParamsTitleWHelp (&pnlPars, SCALER_GetCurrVal(MAIN_WIDTH),
                                        innerHeight, helpWidth, 1,
                                        &stackedFrameWidth);

        pnlPars.onFocusSet      = MENU_StreamInfoOnFocusSet;
        pnlPars.onFocusSetArg   = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_STRINF, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_STREAMINFO_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     stackedFrameWidth, STR_TITLE_STREAMINF,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_StreamInfoGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;
        listPars.getItem         = MENU_StreamInfoGetItem;
        listPars.getItemArg      = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_STREAMINFO_ID_LIST,
                                            stackedFrameWidth, 0,
                                            SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, &descrPos, &valuePos, 66,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----- menu lines -------------------------------------------------*/

        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos           = descrPos;
        txtPars.valuePos           = valuePos;
        txtPars.pos                = itemPos;

        /* --- VIDEOPID -----------------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_STREAMINFO_ID_VIDEOPID,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT) +
                     (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- VIDEOPIC -----------------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_STREAMINFO_ID_VIDEOPIC,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- VIDEO FRAME RATE ---------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_STREAMINFO_ID_VIDEOFRATE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- VIDEOFORMAT --------------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_STREAMINFO_ID_VIDEOFORMAT,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- VIDEOMAXBITRATE ----------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars,
                                    MENU_STREAMINFO_ID_VIDEOMAXBITRATE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- VIDEOREALBITRATE ---------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars,
                                    MENU_STREAMINFO_ID_VIDEOREALBITRATE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- AUDIOPID -----------------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_STREAMINFO_ID_AUDIOPID,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- AUDIOFORMAT --------------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_STREAMINFO_ID_AUDIOFORMAT,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- AUDIOMODE ----------------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_STREAMINFO_ID_AUDIOMODE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- AUDIOSAMPLE --------------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_STREAMINFO_ID_AUDIOSAMPLE,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- AUDIOPCRPID --------------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_STREAMINFO_ID_PCRPID,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- AUDIOTTXPID --------------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_STREAMINFO_ID_TTXPID,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* --- AUDIOPIDSPDIF ------------------------------------------------*/

        txtPars.pos = itemPos;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_STREAMINFO_ID_AUDIOPIDSPDIF,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                   + (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING);

        /* ----- Separator lines ------------------------------------------- */

        inst->itemPos.x         = listPars.pos.x;
        inst->itemPos.y         = listPars.pos.y;
        inst->itemPos.width     = SCALER_GetCurrVal(MAIN_WIDTH);
        inst->itemPos.height    = (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                +  SCALER_GetCurrVal(INNER_FRAME_WIDTH));
        inst->itemPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        member.h             = (FAPI_SYS_HandleT)inst;
        member.pos           = &(inst->itemPos);
        member.id            = MENU_STREAMINFO_ID_REST;
        member.isHidden      = FFALSE;
        member.show          = MENU_StreamInfoDrawSeparatorLines;

        retVal = FGS_PANEL_AddMember(inst->pnlHdl, &member,
                                     &(inst->pItem));
        if ( retVal != FAPI_OK ) break;

        /* -----Bottom help lines ------------------------------------------ */

        retVal = MENU_CommonAddMenuExitHelpLine(
                (int16_t)(stackedFrameWidth +
                SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                SCALER_GetCurrVal(MAIN_LEFT_PADDING)),
                (int16_t)(- (int16_t)SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2 -
                (int16_t)SCALER_GetCurrVal(HELP_TB_PADDING) -
                (int16_t)SCALER_GetCurrVal(HELP_LINE_HEIGHT)),
                pnlPars.pos.width -
                stackedFrameWidth -
                2 * SCALER_GetCurrVal(OUTER_FRAME_ALL) -
                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;

    } while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);
        return FFALSE;
    }

    return FTRUE;
}

/*!
******************************************************************************
** \brief   This function handles the channel up key to zap through the
**          current selected Servicelist. Used as callback for the FGS key
**          handling.
**
** \note    In Playback mode channel change is not possible.
**
** \param   h       Handle of Stream Info Menu.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_StreaminfoChannelUp (FAPI_SYS_HandleT h)
{
    MENU_STREAMINFO_INST_S* inst = (MENU_STREAMINFO_INST_S*)h;

    (void)MAIN_SelectNextService();

    inst->currMaxBitRate = 0;

    (void)FGS_LIST_RefreshMember(inst->listHdl,
                                 MENU_STREAMINFO_ID_VIDEOMAXBITRATE );

    return FTRUE;
}

/*!
******************************************************************************
** \brief   This function handles the channel down key to zapp through the
**          current selected Servicelist. Used as callback for the FGS key
**          handling.
**
** \note    In Playback mode channel change is not possible.
**
** \param   h       Handle of Stream Info Menu.
**
** \return  one of the following status codes:
**          - #FTRUE
**          - #FFALSE
**
******************************************************************************
*/
FSTATIC fbool_t MENU_StreaminfoChannelDown (FAPI_SYS_HandleT h)
{
    MENU_STREAMINFO_INST_S* inst = (MENU_STREAMINFO_INST_S*)h;

    (void)MAIN_SelectPreviousService();

    inst->currMaxBitRate = 0;

    (void)FGS_LIST_RefreshMember(inst->listHdl,
                                 MENU_STREAMINFO_ID_VIDEOMAXBITRATE );

    return FTRUE;
}

/*!
******************************************************************************
** \brief   Draw a separator line between Video and Audio informations and
**          over PCR Pid info. Used as callback for the FGS list member.
**
** \param   h       Handle of Stream Info Menu.
**
** \return  one of the following status codes:
**          - #FAPI_OK
**
******************************************************************************
*/
FSTATIC int32_t  MENU_StreamInfoDrawSeparatorLines (FAPI_SYS_HandleT h)
{
    MENU_STREAMINFO_INST_S* inst = (MENU_STREAMINFO_INST_S*)h;
    FGS_LIST_INST_S *       listInst = (FGS_LIST_INST_S*)inst->listHdl;
    FGS_LOC_S               absPos;

    memset(&absPos, 0, sizeof(absPos));

    absPos = listInst->member[0].abspos;
    OSD_DrawSeparatorHorizontal(
                            (uint16_t)(absPos.x),
                            (uint16_t)(absPos.y -
                             (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING) +
                             (int16_t)(6*(SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                     + SCALER_GetCurrVal(LIST_LINE_PADDING)))),
                            (uint16_t)SCALER_GetCurrVal(MAIN_WIDTH));
    OSD_DrawSeparatorHorizontal(
                            (uint16_t)(absPos.x),
                            (uint16_t)(absPos.y -
                             (int16_t)SCALER_GetCurrVal(LIST_LINE_PADDING) +
                             (int16_t)(10*(SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                     + SCALER_GetCurrVal(LIST_LINE_PADDING)))),
                            (uint16_t)SCALER_GetCurrVal(MAIN_WIDTH));
    return FAPI_OK;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
