/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_mosaic.c
**
**              $Id: app_mosaic.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Implementation of "Mosaic" Preview Menu.
**
** Detail description. [\ref app_smartgo_app_ref_mosaic]
**
** Mosaic preview scans available services and displays a static frame per
** visible window. By using the arrow keys of the remote control a window
** can be selected and the underlaying service can be chosen by the OK key.
** If the selection runs outside the visible windows the selection of available
** services is shifted accordingly.
** At the end of the service list a wrap around is done.
** In playback mode no mosaic is possible.
** In case of active recording some services might not be available.
** Aspect ratio of preview windows are set to 16:9
**
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    774: Boolean within 'if' always evaluates to True.
** - Info    785: Too few initializers for aggregate of type
**           'unsigned short [13]' ( line 140 - 168 )
*****************************************************************************
*/
/*lint -save -e715 -e732 -e774 */
#define DEBUG 0

#if (DEBUG > 0)
    //#define MOSAIC_PRINT_DBG
    #define MOSAIC_SERV_DBG
    //#define MOSAIC_VIDEO_DEBUG
#endif

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <voc/voc.h>
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define MOSAIC_NUM_MAX      13

//#define MOSAIC_CCIR_ENABLE

#define MOSAIC_NUM_DEFAULT   9

#define MOSAIC_NUM_4         4
#define MOSAIC_NUM_6         6
#define MOSAIC_NUM_9         9
#define MOSAIC_NUM_13       13

#define MENU_MOSAIC_LAYOUT   0
#define MOSAIC_WAIT_TIME     4

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef enum
{
    MOSAIC_SHIFT_DOWN =    -1,
    MOSAIC_SHIFT_NONE =     0,
    MOSAIC_SHIFT_UP   =     1

} MENU_MOSAIC_SHIFT_E;

typedef struct
{
    FAPI_SYS_HandleT          pnlHdl;
    OSDHANDLER_StartParamsT   osdParams;
    fbool_t                   haveOsdLayer;            //!< get control of osd layer
    SYS_SERVLOC_S             storedServLoc;           //!< service Loc
    FAPI_SYS_HandleT          vocInst;                 //!< VOC instance
    VOC_OpenParamsT           vocPars;
    VOC_PictureWindowT        picturePos[MOSAIC_NUM_MAX]; //!< picture positions
    SLIST_TYPE                listId;                  //!< ID of service list
    uint16_t                  servIdx[MOSAIC_NUM_MAX]; //!< services shown
    uint16_t                  numServices;             //!< number of services
    uint16_t                  nextServ;                //!< service shown next
    uint16_t                  hasFocusServiceId;       //!< ID of window which has
                                                       //!< focus
    uint16_t                  listSize;                //!< no. of entries in current
                                                       //!< selected service list
    uint16_t                  numWindows;              //!< number of windows max.
                                                       //!< displayed
    fbool_t                   rearrangeWindows;        //!< use this flag if windows
                                                       //!< have to be shifted
    fbool_t                   timerOn;                 //!< timer on/off
    uint8_t                   timerCount;              //!< timer counter
    SYS_SERVICE_STAT_E        servStat;                //!< service selection state
    RTOS_SemaphoreT           timerMutex;              //!< timer timerMutex
    fbool_t                   servAvailable[MOSAIC_NUM_MAX];
    fbool_t                   mosaicOn;                //!< start progress and timer
                                                       //!< handle
} MENU_MOSAIC_INST_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*lint -save -e785 */
FSTATIC uint16_t indexTabel[4][4][13] =
    {
/* 2 */ {
/* up */    { 0, 1, 0, 1},
/* down */  { 2, 3, 2, 3},
/* left */  { 0, 0, 2, 2},
/* right */ { 1, 1, 3, 3},
        },
/* 6 */ {
/* up */    { 0, 1, 2, 0, 3, 1},
/* down */  { 3, 5, 5, 4, 4, 5},
/* left */  { 0, 0, 1, 3, 3, 3},
/* right */ { 1, 2, 2, 5, 5, 5},
        },
/* 9 */ {
/* up */    { 0, 1, 2, 0, 1, 2, 3, 4, 5},
/* down */  { 3, 4, 5, 6, 7, 8, 6, 7, 8},
/* left */  { 0, 0, 1, 3, 3, 4, 6, 6, 7},
/* right*/  { 1, 2, 2, 4, 5, 5, 7, 8, 8},
        },
/* 13 */{
/* up */    { 0,  1,  2, 3,  0,  3, 4,  5, 6, 12, 12,  7, 1},
/* down */  { 4, 12, 12, 5,  6,  7, 8, 11, 8,  9, 10, 11, 9},
/* left */  { 0,  0,  1, 2,  4, 12, 6, 12, 8,  8,  9, 10, 4},
/* right */ { 1,  2,  3, 3, 12,  5, 12, 7, 9, 10, 11, 11, 5},
        },
    };
/*lint -restore */

FSTATIC MENU_MOSAIC_INST_S* mosaicDat  = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC fbool_t MENU_MosaicLeave     (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC void    MENU_MosaicPrintServName
                                     (MENU_MOSAIC_INST_S* inst, uint16_t index);
FSTATIC void    MENU_MosaicDrawString(MENU_MOSAIC_INST_S* inst, uint16_t index,
                                      uint32_t posX, uint32_t posY,
                                      SYS_STRING_S string);
FSTATIC void    MENU_MosaicSetSolid  (FAPI_SYS_HandleT h, uint32_t layerId,
                                      uint32_t windowNum, uint32_t color);
FSTATIC fbool_t MENU_MosaicKeys      (FAPI_SYS_HandleT h);
FSTATIC void    MENU_MosaicDrawFrames(FAPI_SYS_HandleT h, uint32_t width);
FSTATIC void    MENU_MosaicDrawFrame (MENU_MOSAIC_INST_S* inst, uint32_t width,
                                      uint16_t windowNum, uint8_t colour);
FSTATIC void    MENU_MosaicDrawLayout (FAPI_SYS_HandleT h, uint8_t align,
                                      uint16_t x, uint16_t y, uint16_t width,
                                      uint16_t height);
FSTATIC void    MENU_MosaicGetItem   (FAPI_SYS_HandleT h, int32_t id,
                                      uint32_t index, FGS_ITEMSTATE_E state,
                                      FGS_ITEM_S* item);
FSTATIC void    MENU_MosaicPrintAllServiceNames(FAPI_SYS_HandleT h);
FSTATIC void    MENU_MosaicSetServPars(FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_MosaicBuild     (FAPI_SYS_HandleT h);
FSTATIC int32_t MENU_MosaicPrepareWindow(FAPI_SYS_HandleT h,
                                      uint16_t windowNum, fbool_t enable);
FSTATIC int32_t MENU_MosaicShowWindow (FAPI_SYS_HandleT h, uint16_t windowNum);
FSTATIC void    MENU_MosaicPrintServState (FAPI_SYS_HandleT h,
                                      uint16_t windowNum,fbool_t testOn,
                                      uint8_t testCnt);
FSTATIC int32_t MENU_MosaicGetLayer  (FAPI_SYS_HandleT h, fbool_t isFirstHit);


int32_t MENU_MosaicInit (void)
{
    if ( mosaicDat != NULL )
    {
        return FAPI_ERR_ALREADY_INITIALIZED;
    }

    mosaicDat = UTILS_Malloc (sizeof(MENU_MOSAIC_INST_S));
    if ( mosaicDat == NULL )
    {
        return FAPI_ERR_OUT_OF_MEMORY;
    }

    memset(mosaicDat, 0, sizeof(MENU_MOSAIC_INST_S));

    mosaicDat->timerMutex = RTOS_CreateSemaphore(0);

    return FAPI_OK;
}

fbool_t MENU_MosaicEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_MOSAIC_INST_S*     inst = mosaicDat;
    int16_t                 currIdx;
    SYS_HDTV_STANDARD       hdStandard;
    SYS_SDTV_STANDARD       sdStandard;
    SYS_PRESSTATE_E         state;
    fbool_t                 retVal = FTRUE;

    for(;;)
    {
        if ( test == FTRUE )
        {
            retVal =  FTRUE;
            break;
        }
        #ifdef APPL_MHEG_ENABLED
        MHEG_Enable(FTRUE,SYS_SUSPENDSRC_USER);
        #endif

        inst->numWindows        = MOSAIC_NUM_DEFAULT;
        inst->listId            = SLIST_NONE;
        inst->listSize          = 0;
        inst->numServices       = 0;
        inst->nextServ          = 0;
        inst->hasFocusServiceId = 0;
        inst->rearrangeWindows  = FFALSE;
        inst->timerOn           = FFALSE;
        inst->timerCount        = 0;

        //stop aspect ratio handling and disable output format update
        AV_EnableOutputFormatUpdate(FFALSE);

        //open VOC
        inst->vocPars.functionMode = VOC_MODE_MOSAIC;
        if(VOC_Open(&inst->vocPars, &inst->vocInst) !=  FAPI_OK)
        {
            retVal = FFALSE;
            break;
        }

        /* check first if a service is available */
        (void)SYS_GetPresentationState(&state);
        if ( state != SYS_PRESSTATE_LIVETV )
        {
            MENU_CommonMsgShow(STR_MSG_NOMOSAIC, FFALSE);
            retVal = FFALSE;
            break;
        }

        if ( AV_GetRequestedService(&(inst->storedServLoc)) != FAPI_OK )
        {
            retVal = FFALSE;
            break;
        }

        inst->listId = inst->storedServLoc.listId;
        currIdx      = inst->storedServLoc.index;


        /* set to default values in case no service is currently selected */
        if ( (inst->listId == SLIST_NONE) ||
             (inst->listId == SLIST_INVISIBLE) )
        {
            inst->listId = SLIST_TV;
            currIdx      = 0;
        }
        else if ( currIdx < 0 )
        {
            currIdx = 0;
        }

        inst->listSize = SLIST_ListSizeGet(inst->listId);

        /* abort in case of empty service list */
        if ( inst->listSize == 0 )
        {
            retVal = FFALSE;
            break;
        }

        if ( AV_OutputTvStandardCurrGet(&hdStandard, &sdStandard) != FAPI_OK )
        {
            retVal = FFALSE;
            break;
        }
        #ifdef APPL_PARENTAL_ENABLED
        (void)PCTRL_SetEnableStatus(FFALSE);
        #endif /* APPL_PARENTAL_ENABLED */

        if (MENU_MosaicGetLayer(inst, FTRUE)!= FAPI_OK)
        {
            retVal = FFALSE;
            break;
        }

        if ( MENU_MosaicBuild((FAPI_SYS_HandleT)inst) == FFALSE )
        {
            #ifdef APPL_PARENTAL_ENABLED
            (void)PCTRL_SetEnableStatus(FTRUE);
            #endif /* APPL_PARENTAL_ENABLED */

            retVal = FFALSE;
            break;
        }

        inst->mosaicOn = FTRUE;

        (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_MOSAIC);
        (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE);

        /* ---------------------------------------------------------------------- */
        MAIN_HandleExitKeys(FTRUE);
        FGS_SetAppKeyHandler(FGS_KEY_UP,   MENU_MosaicKeys,
                                            (FAPI_SYS_HandleT) inst, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_DOWN, MENU_MosaicKeys,
                                            (FAPI_SYS_HandleT) inst, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_LEFT, MENU_MosaicKeys,
                                            (FAPI_SYS_HandleT) inst, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_RIGHT,MENU_MosaicKeys,
                                            (FAPI_SYS_HandleT) inst, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_OK,   MENU_MosaicKeys,
                                            (FAPI_SYS_HandleT) inst, FTRUE, FFALSE);
        FGS_SetAppKeyHandler(FGS_KEY_BLUE, MENU_MosaicKeys,
                                            (FAPI_SYS_HandleT) inst, FTRUE, FFALSE);

        MENUSTACK_SetCleanup (MENU_MosaicLeave, (FAPI_SYS_HandleT)inst);
        break; /* end of for(;;) */
    }
    if(retVal != FTRUE)
    {
        if(inst->vocInst != NULL)
        {
            (void) VOC_Close(inst->vocInst);
        }
    }

    return retVal;
}

FSTATIC fbool_t MENU_MosaicLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_MOSAIC_INST_S* inst = (MENU_MOSAIC_INST_S*)h;

    MAIN_HandleExitKeys(FFALSE);
    FGS_ClearAppKeyHandler(FGS_KEY_UP);
    FGS_ClearAppKeyHandler(FGS_KEY_DOWN);
    FGS_ClearAppKeyHandler(FGS_KEY_LEFT);
    FGS_ClearAppKeyHandler(FGS_KEY_RIGHT);
    FGS_ClearAppKeyHandler(FGS_KEY_OK);
    FGS_ClearAppKeyHandler(FGS_KEY_BLUE);

    (void)AV_Stop();
    (void)AV_Nop();

    // close VOC
    if ( inst->vocInst != NULL )
    {
        (void)VOC_Close(inst->vocInst);
        inst->vocInst = NULL;
    }
    (void) MENU_MosaicPrepareWindow(inst, 0, FFALSE);

    #ifdef MOSAIC_VIDEO_DEBUG
    (void) OSDHANDLER_PlaneOrder(OSDHANDLER_PARAMETER_SET,3,2,1,0,&errorCode);
    #endif /* MOSAIC_VIDEO_DEBUG */

    /* Release mosaic layers */
    if ( inst->haveOsdLayer )
    {
        (void) SYS_GiveOsdLayer(OSD_LAYER_MOSAIC);
        inst->haveOsdLayer = FFALSE;
    }

    if ( inst->pnlHdl != NULL )
    {
        (void) FGS_PANEL_Close(inst->pnlHdl);
        inst->pnlHdl = NULL;
    }
    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_MOSAIC);

    inst->mosaicOn = FFALSE;

    #ifdef APPL_PARENTAL_ENABLED
    (void)PCTRL_SetEnableStatus(FTRUE);
    #endif /* APPL_PARENTAL_ENABLED */

    /* restart previous service */
    (void) MAIN_SelectService (&(inst->storedServLoc), FFALSE);

    inst->timerOn = FFALSE;

    //start aspect ratio handling and enable output format update
    AV_EnableOutputFormatUpdate(FTRUE);

    return FTRUE;
}


void MENU_MosaicProgress (const SYS_EVT_SERVICE_S* status)
{
    MENU_MOSAIC_INST_S*  inst = mosaicDat;
    int32_t              retVal;
    SYS_SERVLOC_S        servLoc;

    if (( inst == NULL )||(inst->mosaicOn == FFALSE))
    {
        return;
    }

    if (inst->rearrangeWindows == FTRUE)
    {
        return;
    }

    inst->servStat = status->status;

    switch ( inst->servStat )
    {
        case SERVICE_STAT_STOPPED:
            #ifdef MOSAIC_SERV_DBG
                FAPI_SYS_PRINT_DEBUG(1,
                      "[MOSAIC] (%2d) SERVICE_STAT_STOPPED\n", inst->nextServ);
            #endif /* MOSAIC_SERV_DBG */

            /* prepare next service */
            retVal = MENU_MosaicPrepareWindow(inst, inst->nextServ, FTRUE);
            if ( retVal != FAPI_OK )
            {
                FAPI_SYS_PRINT_DEBUG(1,
                        "\n[MOSAIC]  Error from MENU_MosaicPrepareWindow %d\n",
                        retVal);
            }

            servLoc.listId = inst->listId;
            servLoc.index  = (int16_t)inst->servIdx[inst->nextServ];

            /* tune to next service */
            retVal = MAIN_SelectService (&servLoc, FTRUE);

            //reset timer for service status change
            inst->timerCount = 0;
            break;

        // start a internal timer since SERVICE_STAT_DECODED may not always
        // be reached, e.g. radio or empty services.
        case SERVICE_STAT_START: //SERVICE_STAT_SUCCESS :
            #ifdef MOSAIC_SERV_DBG
                FAPI_SYS_PRINT_DEBUG(1,
                        "[MOSAIC] (%2d) SERVICE_STAT_START\n", inst->nextServ);
            #endif /* MOSAIC_SERV_DBG */

            inst->timerOn = FTRUE;
            retVal = (int32_t)RTOS_SetSemaphore(
                                            inst->timerMutex, RTOS_NO_SUSPEND );
            if (retVal == 0)
            {
                FAPI_SYS_PRINT_DEBUG(1, "[MOSAIC] RTOS_SetSemaphore failed\n");
            }
            break;
        case SERVICE_STAT_DECODED :
            #ifdef MOSAIC_SERV_DBG
                FAPI_SYS_PRINT_DEBUG(1,
                    "[MOSAIC] (%2d) SERVICE_STAT_DECODED change to (%2d)\n",
                    inst->nextServ, inst->nextServ+1);
            #endif /* MOSAIC_SERV_DBG */

            retVal = (int32_t)RTOS_GetSemaphore(
                                            inst->timerMutex, RTOS_NO_SUSPEND);
            if(retVal==0)
            {
                if(inst->servAvailable[inst->nextServ]!=FFALSE)
                {
                    inst->servAvailable[inst->nextServ] = FFALSE;
                    (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                                        MENU_MOSAIC_LAYOUT);
                }
            }
            else
            {
                retVal = MENU_MosaicShowWindow(inst, inst->nextServ);
                if (retVal != FAPI_OK)
                {
                    FAPI_SYS_PRINT_DEBUG(1,
                            "\n[MOSAIC] Error from MENU_MosaicShowWindow %d\n",
                            retVal);
                }
                if(inst->servAvailable[inst->nextServ]!=FTRUE)
                {
                    inst->servAvailable[inst->nextServ] = FTRUE;
                    (void)FGS_PANEL_RefreshMember(inst->pnlHdl,
                                                        MENU_MOSAIC_LAYOUT);
                }
            }

            (void)AV_Stop();
            (void)AV_Nop();

            /* choose next service */
            inst->nextServ++;

            if ( (uint16_t)inst->nextServ >= inst->numServices )
            {
                inst->nextServ = 0;
            }
            inst->timerOn = FFALSE;
            break;

        default:
            break;
    }
}

FSTATIC fbool_t MENU_MosaicKeys (FAPI_SYS_HandleT h)
{
    MENU_MOSAIC_INST_S*  inst = (MENU_MOSAIC_INST_S*) h;
    MENU_MOSAIC_SHIFT_E  shiftMosaic = MOSAIC_SHIFT_NONE;
    int16_t              currIdx;
    fbool_t              keyUsed = FTRUE;
    uint8_t              i;
    uint32_t             clearColor = 0x8010;
    uint8_t              tmp = 0;

    switch(inst->numWindows)
    {
        case MOSAIC_NUM_4:  tmp = 0; break;
        case MOSAIC_NUM_6:  tmp = 1; break;
        default:
        case MOSAIC_NUM_9:  tmp = 2; break;
        case MOSAIC_NUM_13: tmp = 3; break;
    }

    /* evaluate arrow keys and OK key. If shift is beyond displayed mosaics, */
    /* the complete view onto the service list is shifted and drawn again    */
    switch ( FGS_GetCurrentKeyCode() )
    {
        case FGS_KEY_UP:
            inst->rearrangeWindows = FTRUE;
            if((inst->numServices < inst->numWindows) ||
                            (inst->listSize == inst->numWindows))
            {
                inst->hasFocusServiceId =
                            indexTabel[tmp][0][inst->hasFocusServiceId];
            }
            else
            {
                if(inst->hasFocusServiceId ==
                            indexTabel[tmp][0][inst->hasFocusServiceId])
                {
                    shiftMosaic = MOSAIC_SHIFT_UP;
                }
                inst->hasFocusServiceId =
                            indexTabel[tmp][0][inst->hasFocusServiceId];
            }
            break;
        case FGS_KEY_DOWN:
            inst->rearrangeWindows = FTRUE;
            if((inst->numServices < inst->numWindows) ||
                            (inst->listSize == inst->numWindows))
            {
                inst->hasFocusServiceId =
                            indexTabel[tmp][1][inst->hasFocusServiceId];
                if(inst->hasFocusServiceId > inst->numServices)
                {
                    inst->hasFocusServiceId = 0;
                }
            }
            else
            {
                if(inst->hasFocusServiceId ==
                            indexTabel[tmp][1][inst->hasFocusServiceId])
                {
                    shiftMosaic = MOSAIC_SHIFT_DOWN;
                }
                inst->hasFocusServiceId =
                            indexTabel[tmp][1][inst->hasFocusServiceId];
            }
            break;
        case FGS_KEY_LEFT:
            inst->rearrangeWindows = FTRUE;
            if((inst->numServices < inst->numWindows) ||
                            (inst->listSize == inst->numWindows))
            {
                inst->hasFocusServiceId =
                            indexTabel[tmp][2][inst->hasFocusServiceId];
                if(inst->hasFocusServiceId > inst->numServices)
                {
                    inst->hasFocusServiceId = 0;
                }
            }
            else
            {
                if(inst->hasFocusServiceId ==
                            indexTabel[tmp][2][inst->hasFocusServiceId])
                {
                    shiftMosaic = MOSAIC_SHIFT_UP;
                }
                inst->hasFocusServiceId =
                            indexTabel[tmp][2][inst->hasFocusServiceId];
            }
            break;
        case FGS_KEY_RIGHT:
            inst->rearrangeWindows = FTRUE;
            if((inst->numServices < inst->numWindows) ||
                            (inst->listSize == inst->numWindows))
            {
                inst->hasFocusServiceId =
                            indexTabel[tmp][3][inst->hasFocusServiceId];
                if(inst->hasFocusServiceId > inst->numServices)
                {
                    inst->hasFocusServiceId = 0;
                }
            }
            else
            {
                if(inst->hasFocusServiceId ==
                            indexTabel[tmp][3][inst->hasFocusServiceId])
                {
                    shiftMosaic = MOSAIC_SHIFT_DOWN;
                }
                inst->hasFocusServiceId =
                            indexTabel[tmp][3][inst->hasFocusServiceId];
            }
            break;
        case FGS_KEY_OK:
            inst->storedServLoc.index =
                                (int16_t)inst->servIdx[inst->hasFocusServiceId];
            return MENUSTACK_Exit(NULL);

        case FGS_KEY_BLUE:
            inst->hasFocusServiceId = 0;
            //reset semaphore
            while(RTOS_GetSemaphore (inst->timerMutex, RTOS_NO_SUSPEND)>0) ;

            (void)AV_Stop();
            (void)AV_Nop();

            switch(inst->numWindows)
            {
                case MOSAIC_NUM_4:
                    inst->numWindows = MOSAIC_NUM_6;
                    break;
                case MOSAIC_NUM_6:
                    inst->numWindows = MOSAIC_NUM_9;
                    break;
                case MOSAIC_NUM_9:
                    inst->numWindows = MOSAIC_NUM_13;
                    break;
                case MOSAIC_NUM_13:
                    inst->numWindows = MOSAIC_NUM_4;
                    break;
                default:
                    inst->numWindows = MOSAIC_NUM_DEFAULT;
                    break;
            }

            if ( inst->pnlHdl != NULL )
            {
                (void) FGS_PANEL_Close(inst->pnlHdl);
                inst->pnlHdl = NULL;
            }
            (void)MENUSTACK_FocusRelease ((int32_t)MENUID_MOSAIC);

            /* Release OSD layer */
            if ( inst->haveOsdLayer )
            {
                (void) SYS_GiveOsdLayer(OSD_LAYER_MOSAIC);
                inst->haveOsdLayer = FFALSE;
            }
            if(MENU_MosaicGetLayer(inst, FFALSE)!=FAPI_OK) break;

            (void) MENU_MosaicBuild((FAPI_SYS_HandleT)inst);
            (void) MENUSTACK_FocusClaim((int32_t)MENUID_MOSAIC);
            (void) FGS_PANEL_Show(inst->pnlHdl, FFALSE);

            break;

        default:
            keyUsed = FFALSE;
            break;
    }

    /* if mosaics have to be shifted clear the mosaics first */
    if(shiftMosaic != MOSAIC_SHIFT_NONE)
    {
        for ( i = 0; i < inst->numServices; i++ )
        {
            MENU_MosaicSetSolid((FAPI_SYS_HandleT)inst,
                                        OSD_LAYER_MOSAIC, i, clearColor);
            inst->servAvailable[i] = FTRUE;
        }

        switch(shiftMosaic)
        {
            case MOSAIC_SHIFT_UP:
                if(inst->listSize > inst->numWindows)
                {
                    currIdx = (int16_t)inst->storedServLoc.index;
                    currIdx -= (int16_t)inst->numWindows;
                    if(currIdx < 0)
                        currIdx += (int16_t)inst->listSize;
                    inst->storedServLoc.index = currIdx;
                }
                break;
                case MOSAIC_SHIFT_DOWN:
                if(inst->listSize > inst->numWindows)
                {
                    currIdx = (int16_t)inst->storedServLoc.index;
                    currIdx += (int16_t)inst->numWindows;
                    if(currIdx >= (int16_t)(inst->listSize-1))
                        currIdx -= (int16_t)inst->listSize;
                    inst->storedServLoc.index = currIdx;
                }
                break;
            case MOSAIC_SHIFT_NONE:
                break;
            default:
                break;
        }
        /* ----- set services to be shown ---------------------------------- */
        MENU_MosaicSetServPars((FAPI_SYS_HandleT)inst);
    }

    if(inst->rearrangeWindows == FTRUE)
    {
        (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MOSAIC_LAYOUT);
        inst->rearrangeWindows = FFALSE;
    }

    return keyUsed;
}

FSTATIC void MENU_MosaicSetServPars(FAPI_SYS_HandleT h)
{
    MENU_MOSAIC_INST_S*  inst = (MENU_MOSAIC_INST_S*) h;
    SYS_SERVLOC_S        localServLoc;
    uint8_t              i = 0;

    /* number of available services is calculated while scanning through
     * actual service list. Services currently not available due to PVR
     * activity are not selected. */

/*    inst->numServices = (inst->listSize < inst->numWindows ?
                         inst->listSize :
                         inst->numWindows;
*/
    inst->numServices = 0;
    inst->nextServ    = 0;
    localServLoc.index = inst->storedServLoc.index;
    localServLoc.listId = inst->storedServLoc.listId;
    inst->servIdx[0] = (uint16_t)localServLoc.index;

    for ( i = 1; i <= inst->numWindows; i++ )
    {
        if(MAIN_ServiceSwitchGetAvailability (&(localServLoc)) == FAPI_OK)
        {
            inst->servIdx[i-1] = (uint16_t)localServLoc.index++;
            inst->numServices = i;
        }
        else
        {
            localServLoc.index++;
            i--;
        }
        /* wrap around at end of service list */
        if ( (uint16_t)localServLoc.index >= inst->listSize )
        {
            localServLoc.index = 0;
        }
        /* finish when first service found is reached again */
        if( (uint16_t)localServLoc.index == inst->servIdx[0])
        {
            return;
        }
    }
}


FSTATIC void MENU_MosaicDrawFrames(FAPI_SYS_HandleT h, uint32_t width)
{
    MENU_MOSAIC_INST_S*  inst = (MENU_MOSAIC_INST_S*) h;
    OSD_COLSCHEME_E      scheme;
    uint8_t              frameColorHasFocus;
    uint8_t              frameColor = 0;
    uint8_t              cnt;


    (void)OSD_GetColorScheme (&scheme);
    frameColorHasFocus =
                    appDat->schemeData[scheme][OSD_SCHEMECOL_MAIN_BKGR_HIGHL];
    frameColor = appDat->schemeData[scheme][OSD_SCHEMECOL_FRAME];

    for (cnt = 0; cnt < inst->numServices; cnt++)
    {
        if(cnt == inst->hasFocusServiceId)
        {
            MENU_MosaicDrawFrame(inst, width, cnt, frameColorHasFocus);
        }
        else
        {
            MENU_MosaicDrawFrame(inst, width, cnt, frameColor);
        }
    }
}

FSTATIC void MENU_MosaicDrawFrame(MENU_MOSAIC_INST_S* inst, uint32_t width,
                                    uint16_t windowNum, uint8_t colour)
{
    uint32_t x, y;

    if ( windowNum >= inst->numWindows )
    {
        return;
    }

    for (y = 0; y < width; y++)
    {
        OSD_DrawHorizontalLine(inst->picturePos[windowNum].left,
                                inst->picturePos[windowNum].left +
                                inst->picturePos[windowNum].hsize,
                                (inst->picturePos[windowNum].top - width) + y,
                                colour);

        OSD_DrawHorizontalLine(inst->picturePos[windowNum].left - width,
                                inst->picturePos[windowNum].left
                                + inst->picturePos[windowNum].hsize +(width -1),
                                inst->picturePos[windowNum].top
                                + inst->picturePos[windowNum].vsize + y,
                                colour);
    }

    for (x = 0; x < width; x++)
    {
        OSD_DrawVerticalLine((inst->picturePos[windowNum].left - width) + x,
                                inst->picturePos[windowNum].top - width,
                                inst->picturePos[windowNum].top
                                + inst->picturePos[windowNum].vsize +(width -1),
                                colour);

        OSD_DrawVerticalLine(inst->picturePos[windowNum].left
                                + inst->picturePos[windowNum].hsize + x,
                                inst->picturePos[windowNum].top - width,
                                inst->picturePos[windowNum].top
                                + inst->picturePos[windowNum].vsize +(width -1),
                                colour);
    }
}

FSTATIC void MENU_MosaicDrawLayout (FAPI_SYS_HandleT h, uint8_t align,
                    uint16_t x, uint16_t y, uint16_t width, uint16_t height)
{
    MENU_MOSAIC_INST_S* inst = (MENU_MOSAIC_INST_S*)h;

    MENU_MosaicDrawFrames((FAPI_SYS_HandleT)inst,
                                        SCALER_GetCurrVal(MOSAIC_FRAME_WIDTH));
    MENU_MosaicPrintAllServiceNames((FAPI_SYS_HandleT)inst);
}

FSTATIC void MENU_MosaicGetItem (FAPI_SYS_HandleT h, int32_t id, uint32_t index,
                              FGS_ITEMSTATE_E state, FGS_ITEM_S* item)
{
    MENU_MOSAIC_INST_S* inst = (MENU_MOSAIC_INST_S*)h;

    item->type           = FGS_ITEM_TYPE_GENERIC;
    item->align          = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;
    item->item.gnr.cb    = MENU_MosaicDrawLayout;
    item->item.gnr.cbArg = inst;
}

FSTATIC void MENU_MosaicPrintAllServiceNames(FAPI_SYS_HandleT h)
{
    MENU_MOSAIC_INST_S* inst = (MENU_MOSAIC_INST_S*) h;
    uint8_t             cnt;

    for ( cnt = 0; cnt < inst->numServices; cnt++ )
    {
        MENU_MosaicPrintServName (inst, cnt);
        /* Print message on screen if service is not available */
        if(inst->servAvailable[cnt]==FFALSE)
        {
            MENU_MosaicPrintServState(inst, cnt, FFALSE, 0);
        }
    }
}

FSTATIC void MENU_MosaicPrintServName (MENU_MOSAIC_INST_S* inst, uint16_t index)
{
    SLIST_ITEM_S    service;
    SYS_SERVLOC_S   servLoc;
    SYS_STRING_S    sys_string;

    memset(&sys_string,0,sizeof(SYS_STRING_S));

    servLoc.listId = inst->listId;
    servLoc.index  = (int16_t)inst->servIdx[index];

    if ( SLIST_EntryGet (&servLoc, &service) == FAPI_OK )
    {
        sys_string.cs = service.nameCharSet;
        sys_string.chr = service.name;
        MENU_MosaicDrawString(inst, index, 10, 0, sys_string);
    }
}

FSTATIC void MENU_MosaicDrawString (MENU_MOSAIC_INST_S* inst, uint16_t index,
                             uint32_t posX, uint32_t posY, SYS_STRING_S string)
{
    uint32_t stringColor;

    stringColor = OSD_COL_LIST_TEXT_HIGHL;

    MENU_MosaicSetSolid((FAPI_SYS_HandleT)inst,
                                OSD_LAYER_MENU, index, OSD_COL_BLANK);

    OSD_DrawStringLtd ((uint16_t)(inst->picturePos[index].left + posX),
                       (uint16_t)(inst->picturePos[index].top  + posY),
                       stringColor, (uint8_t)MAIN_FONT, -1,
                       (uint16_t)(inst->picturePos[index].hsize - (2*posX)),
                       (int32_t)(inst->picturePos[index].hsize - (2*posX)),
                       string, FTRUE);
}

FSTATIC void MENU_MosaicSetSolid(FAPI_SYS_HandleT h, uint32_t layerId,
                                    uint32_t windowNum, uint32_t color)
{
    MENU_MOSAIC_INST_S*     inst = (MENU_MOSAIC_INST_S*) h;
    VOC_PictureWindowT      outputSize;

    if ( windowNum >= inst->numWindows )
    {
        return;
    }

    outputSize = inst->picturePos[windowNum];

    (void)OSDHANDLER_BitBlitFill(layerId,
                                outputSize.left, outputSize.top,
                                outputSize.hsize, outputSize.vsize,
                                color);
}

FSTATIC fbool_t MENU_MosaicBuild (FAPI_SYS_HandleT h)
{
    MENU_MOSAIC_INST_S*     inst = (MENU_MOSAIC_INST_S*) h;
    FGS_PANEL_OPEN_S        pnlPars;
    FGS_BOX_OPEN_S          boxPars;
    FAPI_SYS_HandleT        itemHdl;

    uint16_t                i = 0;
    uint16_t                cnt;
    int32_t                 retVal = FAPI_OK;
    int32_t                 jj;

    /* ----- set services to be shown ---------------------------------- */
        MENU_MosaicSetServPars((FAPI_SYS_HandleT)inst);
        if ( inst->numServices == 0 )
        {
            retVal = FAPI_ERR_BAD_PARAMETER;
            return FFALSE;
        }

        (void)AV_Stop();
        (void)AV_Nop();

    /* ----- set picture positions ----------------------------------------- */

    if( inst->numWindows == MOSAIC_NUM_4 )
    {
        /*  -------------
            |  1  |  2  |
            -------------
            |  3  |  4  |
            -------------
            */

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_2_4_POS_Y);
        inst->picturePos[i].hsize = SCALER_GetCurrVal(MOSAIC_2_4_WIDTH);
        inst->picturePos[i].vsize = SCALER_GetCurrVal(MOSAIC_2_4_HEIGHT);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                    SCALER_GetCurrVal(MOSAIC_2_4_WIDTH) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_2_4_POS_Y);
        inst->picturePos[i].hsize = SCALER_GetCurrVal(MOSAIC_2_4_WIDTH);
        inst->picturePos[i].vsize = SCALER_GetCurrVal(MOSAIC_2_4_HEIGHT);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_2_4_POS_Y)+
                                    SCALER_GetCurrVal(MOSAIC_2_4_HEIGHT) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_Y);
        inst->picturePos[i].hsize = SCALER_GetCurrVal(MOSAIC_2_4_WIDTH);
        inst->picturePos[i].vsize = SCALER_GetCurrVal(MOSAIC_2_4_HEIGHT);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                    SCALER_GetCurrVal(MOSAIC_2_4_WIDTH) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_2_4_POS_Y)+
                                    SCALER_GetCurrVal(MOSAIC_2_4_HEIGHT) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_Y);
        inst->picturePos[i].hsize = SCALER_GetCurrVal(MOSAIC_2_4_WIDTH);
        inst->picturePos[i].vsize = SCALER_GetCurrVal(MOSAIC_2_4_HEIGHT);
        i++;
    }
    else if( inst->numWindows == MOSAIC_NUM_6 )
    {
      /*    -------------
            | 1 | 2 | 3 |
            | 4 |       |
            | 5 |   6   |
            ------------- */

        for ( cnt = 0; cnt < inst->numWindows; cnt++ )
        {
            inst->picturePos[cnt].hsize = SCALER_GetCurrVal(MOSAIC_6_9_WIDTH);
            inst->picturePos[cnt].vsize = SCALER_GetCurrVal(MOSAIC_6_9_HEIGHT);
        }

        inst->picturePos[i].left  =  SCALER_GetCurrVal(MOSAIC_POS_X);
        inst->picturePos[i].top   =  SCALER_GetCurrVal(MOSAIC_6_9_POS_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                    SCALER_GetCurrVal(MOSAIC_6_9_WIDTH) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_6_9_POS_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                    (2 * SCALER_GetCurrVal(MOSAIC_6_9_WIDTH)) +
                                    (2 * SCALER_GetCurrVal(MOSAIC_PAD_X));
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_6_9_POS_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_6_9_POS_Y)  +
                                    SCALER_GetCurrVal(MOSAIC_6_9_HEIGHT) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_6_9_POS_Y)  +
                                    (2 * SCALER_GetCurrVal(MOSAIC_6_9_HEIGHT)) +
                                    (2 * SCALER_GetCurrVal(MOSAIC_PAD_Y));
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                    SCALER_GetCurrVal(MOSAIC_6_9_WIDTH) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_6_9_POS_Y)  +
                                    SCALER_GetCurrVal(MOSAIC_6_9_HEIGHT) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_Y);
        inst->picturePos[i].hsize = (2 * SCALER_GetCurrVal(MOSAIC_6_9_WIDTH)) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_X);
        inst->picturePos[i].vsize = (2 * SCALER_GetCurrVal(MOSAIC_6_9_HEIGHT)) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_Y);
        i++;

    }
    else if( inst->numWindows == MOSAIC_NUM_9 )
    {
      /*    -------------
            | 1 | 2 | 3 |
            | 4 | 5 | 6 |
            | 7 | 8 | 9 |
            ------------- */
        for ( cnt = 0; cnt < inst->numWindows; cnt++ )
        {
            inst->picturePos[cnt].hsize = SCALER_GetCurrVal(MOSAIC_6_9_WIDTH);
            inst->picturePos[cnt].vsize = SCALER_GetCurrVal(MOSAIC_6_9_HEIGHT);
        }

        inst->picturePos[i].left  =  SCALER_GetCurrVal(MOSAIC_POS_X);
        inst->picturePos[i].top   =  SCALER_GetCurrVal(MOSAIC_6_9_POS_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                    SCALER_GetCurrVal(MOSAIC_6_9_WIDTH) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_6_9_POS_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                    (2 * SCALER_GetCurrVal(MOSAIC_6_9_WIDTH)) +
                                    (2 * SCALER_GetCurrVal(MOSAIC_PAD_X));
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_6_9_POS_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_6_9_POS_Y)  +
                                    SCALER_GetCurrVal(MOSAIC_6_9_HEIGHT) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                    SCALER_GetCurrVal(MOSAIC_6_9_WIDTH) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_6_9_POS_Y)  +
                                    SCALER_GetCurrVal(MOSAIC_6_9_HEIGHT) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                    (2 * SCALER_GetCurrVal(MOSAIC_6_9_WIDTH)) +
                                    (2 * SCALER_GetCurrVal(MOSAIC_PAD_X));
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_6_9_POS_Y)  +
                                    SCALER_GetCurrVal(MOSAIC_6_9_HEIGHT) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_6_9_POS_Y)  +
                                    (2 * SCALER_GetCurrVal(MOSAIC_6_9_HEIGHT)) +
                                    (2 * SCALER_GetCurrVal(MOSAIC_PAD_Y));
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                    SCALER_GetCurrVal(MOSAIC_6_9_WIDTH) +
                                    SCALER_GetCurrVal(MOSAIC_PAD_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_6_9_POS_Y)  +
                                    (2 * SCALER_GetCurrVal(MOSAIC_6_9_HEIGHT)) +
                                    (2 * SCALER_GetCurrVal(MOSAIC_PAD_Y));
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                    (2 * SCALER_GetCurrVal(MOSAIC_6_9_WIDTH)) +
                                    (2 * SCALER_GetCurrVal(MOSAIC_PAD_X));
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_6_9_POS_Y)  +
                                    (2 * SCALER_GetCurrVal(MOSAIC_6_9_HEIGHT)) +
                                    (2 * SCALER_GetCurrVal(MOSAIC_PAD_Y));
        i++;
    }
    else if( inst->numWindows == MOSAIC_NUM_13 )
    {
      /*    -----------------
            | 1 | 2 | 3 | 4 |
            | 5 |       | 6 |
            | 7 |   13  | 8 |
            | 9 | 10| 11| 12|
            ----------------- */
        for ( cnt = 0; cnt < inst->numWindows; cnt++ )
        {
            inst->picturePos[cnt].hsize = SCALER_GetCurrVal(MOSAIC_13_WIDTH);
            inst->picturePos[cnt].vsize = SCALER_GetCurrVal(MOSAIC_13_HEIGHT);
        }

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_13_POS_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                  SCALER_GetCurrVal(MOSAIC_13_WIDTH) +
                                  SCALER_GetCurrVal(MOSAIC_PAD_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_13_POS_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                  (2 * SCALER_GetCurrVal(MOSAIC_13_WIDTH)) +
                                  (2 * SCALER_GetCurrVal(MOSAIC_PAD_X));
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_13_POS_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_13_WIDTH)) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_PAD_X));
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_13_POS_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_13_POS_Y) +
                                  SCALER_GetCurrVal(MOSAIC_13_HEIGHT) +
                                  SCALER_GetCurrVal(MOSAIC_PAD_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_13_WIDTH)) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_PAD_X));
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_13_POS_Y) +
                                  SCALER_GetCurrVal(MOSAIC_13_HEIGHT) +
                                  SCALER_GetCurrVal(MOSAIC_PAD_Y);
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_13_POS_Y) +
                                  (2 * SCALER_GetCurrVal(MOSAIC_13_HEIGHT)) +
                                  (2 * SCALER_GetCurrVal(MOSAIC_PAD_Y));
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_13_WIDTH)) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_PAD_X));
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_13_POS_Y) +
                                  (2 * SCALER_GetCurrVal(MOSAIC_13_HEIGHT)) +
                                  (2 * SCALER_GetCurrVal(MOSAIC_PAD_Y));
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_13_POS_Y) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_13_HEIGHT)) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_PAD_Y));
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                  SCALER_GetCurrVal(MOSAIC_13_WIDTH) +
                                  SCALER_GetCurrVal(MOSAIC_PAD_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_13_POS_Y) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_13_HEIGHT)) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_PAD_Y));
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                  (2 * SCALER_GetCurrVal(MOSAIC_13_WIDTH)) +
                                  (2 * SCALER_GetCurrVal(MOSAIC_PAD_X));
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_13_POS_Y) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_13_HEIGHT)) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_PAD_Y));
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_13_WIDTH)) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_PAD_X));
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_13_POS_Y) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_13_HEIGHT)) +
                                  (3 * SCALER_GetCurrVal(MOSAIC_PAD_Y));
        i++;

        inst->picturePos[i].left  = SCALER_GetCurrVal(MOSAIC_POS_X) +
                                  SCALER_GetCurrVal(MOSAIC_13_WIDTH) +
                                  SCALER_GetCurrVal(MOSAIC_PAD_X);
        inst->picturePos[i].top   = SCALER_GetCurrVal(MOSAIC_13_POS_Y) +
                                  SCALER_GetCurrVal(MOSAIC_13_HEIGHT) +
                                  SCALER_GetCurrVal(MOSAIC_PAD_Y);
        inst->picturePos[i].hsize = (2 * SCALER_GetCurrVal(MOSAIC_13_WIDTH)) +
                                  SCALER_GetCurrVal(MOSAIC_PAD_X);
        inst->picturePos[i].vsize = (2 * SCALER_GetCurrVal(MOSAIC_13_HEIGHT)) +
                                  SCALER_GetCurrVal(MOSAIC_PAD_Y);
        i++;
    }

    for ( cnt = 0; cnt < inst->numWindows; cnt++ )
    {
        inst->servAvailable[cnt] = FTRUE;
    }

    /* build fgs panel and box for service name and availability print*/
    for (jj=1; jj>0; jj--)
    {
        memset(&pnlPars, 0, sizeof(pnlPars));
        memset(&boxPars, 0, sizeof(boxPars));

        pnlPars.bkgrType      = OSD_PNLTYPE_TRANSPARENT;
        pnlPars.pos.x         = (int32_t)SCALER_GetCurrVal(MOSAIC_OSD_POS_X);
        pnlPars.pos.y         = (int32_t)SCALER_GetCurrVal(MOSAIC_OSD_POS_Y);
        pnlPars.pos.width     = SCALER_GetCurrVal(MOSAIC_OSD_WIDTH);
        pnlPars.pos.height    = SCALER_GetCurrVal(MOSAIC_OSD_HEIGHT);
        pnlPars.pos.alignment = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
        pnlPars.onFocusSet    = NULL;
        pnlPars.onFocusSetArg = NULL;
        pnlPars.pInst         = &(appDat->pnlInst);

        retVal = FGS_PANEL_Open(&pnlPars,
                                (int32_t)MENUID_MOSAIC, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* frames of preview windows*/
        boxPars.bkgrType      = OSD_BOX_BKGR_NONE;
        boxPars.pos.x         = (int32_t)SCALER_GetCurrVal(MOSAIC_OSD_POS_X);
        boxPars.pos.y         = (int32_t)SCALER_GetCurrVal(MOSAIC_OSD_POS_Y);
        boxPars.pos.width     = SCALER_GetCurrVal(MOSAIC_OSD_WIDTH);
        boxPars.pos.height    = SCALER_GetCurrVal(MOSAIC_OSD_HEIGHT);
        boxPars.pos.alignment = FGS_ALIGN_HOR_CENTER | FGS_ALIGN_VER_CENTER;
        boxPars.getItem       = MENU_MosaicGetItem;
        boxPars.getItemArg    = (FAPI_SYS_HandleT)inst;

        retVal = FGS_BOX_Open (&boxPars,
                                (int32_t)MENU_MOSAIC_LAYOUT, inst->pnlHdl,
                               &itemHdl);
        if ( retVal != FAPI_OK ) break;
    }

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL )
        {
            (void) FGS_PANEL_Close(inst->pnlHdl);
            inst->pnlHdl = NULL;
        }

        return FFALSE;
    }
    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief  Time update event handler for managing service state after
**          SERVICE_STAT_START.
**
** This function is called when EVT_TIME_UPDATE is arrived. It works together
**   with MENU_MosaicProgress to handle different types of state changes in
**   service selection process. Since SERVICE_STAT_DECODED is never reached
**   for radio and empty channel, it is necessary to set a time counter
**   after SERVICE_STAT_START and react to unavilable services
**   before a certain amount of time.
**
** \sa MENU_MosaicProgress
******************************************************************************/

void MENU_MosaicHandleTime(void)
{
    MENU_MOSAIC_INST_S*  inst = mosaicDat;
    int32_t retVal=FAPI_OK;

    if ( ( inst == NULL )             ||
         ( inst->mosaicOn == FFALSE ) ||
         ( !inst->timerOn ) )
    {
        return;
    }

    retVal = (int32_t)RTOS_GetSemaphore (inst->timerMutex, RTOS_NO_SUSPEND);
    if ( retVal == 0 )
    {
        return;
    }
    if(inst->servStat == SERVICE_STAT_DECODED)
    {
        /*MENU_MosaicSetSolid((FAPI_SYS_HandleT)inst, OSD_LAYER_MENU,
                                inst->nextServ, OSD_COL_BLANK);
        */
        retVal = MENU_MosaicShowWindow(inst, inst->nextServ);
        inst->servAvailable[inst->nextServ] = FTRUE;
        (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MOSAIC_LAYOUT);

        if (retVal != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1,
                   "\n[MOSAIC] Error from MENU_MosaicShowWindow %d\n", retVal);
        }

        (void)AV_Stop();
        (void)AV_Nop();

        /* choose next service */
        inst->nextServ++;

        if ( (uint16_t)inst->nextServ >= inst->numServices )
        {
            inst->nextServ = 0;
        }
        inst->timerOn = FFALSE;

    }
    else
    {
        inst->timerCount++;

        if(inst->timerCount == MOSAIC_WAIT_TIME)
        {
            if (inst->servAvailable[inst->nextServ] != FFALSE)
            {
                inst->servAvailable[inst->nextServ] = FFALSE;
                (void)FGS_PANEL_RefreshMember(inst->pnlHdl, MENU_MOSAIC_LAYOUT);
            }
            /* choose next service */
            inst->nextServ++;

            if ( (uint16_t)inst->nextServ >= inst->numServices )
            {
                inst->nextServ = 0;
            }

            (void)AV_Stop();
            (void)AV_Nop();

            inst->timerOn = FFALSE;
        }
        else
        {
            (void) RTOS_SetSemaphore (inst->timerMutex, RTOS_NO_SUSPEND);
        }
    }
}

/*!
*******************************************************************************
**
** \brief   Resize and reposition the video to a given window for
**          capturing afterward
**
** \param   h           Mosaic handle
** \param   windowNum   Index of window to perform operation on.
** \param   enable      #FTRUE for resizing and #FFALSE for full screen
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
FSTATIC int32_t MENU_MosaicPrepareWindow (FAPI_SYS_HandleT h,
                            uint16_t windowNum, fbool_t enable)
{
    MENU_MOSAIC_INST_S*     inst   = (MENU_MOSAIC_INST_S*) h;
    int32_t                 retVal = FAPI_OK;

    #ifdef MOSAIC_VIDEO_DEBUG //disable because of aspect ratio handling problem
    AV_VO_PICTURE_WINDOW_S      avPos;
    #endif /* MOSAIC_VIDEO_DEBUG */

    if (( inst == NULL )||( windowNum >= inst->numServices ))
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    #ifdef MOSAIC_VIDEO_DEBUG //disable because of aspect ratio handling problem
          //it is necessary only when VOC_CopyIFrameCCIR is used
    if (enable)
    {
        avPos = *(AV_VO_PICTURE_WINDOW_S *)(inst->picturePos + windowNum);
        retVal = AV_SetVideoArea(FFALSE, &avPos);
    }
    else
    {
        retVal = AV_SetVideoArea(FTRUE, NULL);
    }
    #endif  /* MOSAIC_VIDEO_DEBUG */

    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[MOSAIC] AV resize failed!\n");
        return(retVal);
    }
    else
    {
        return(FAPI_OK);
    }
}

/*!
*******************************************************************************
**
** \brief   Capture I-frame and scale/copy it to a specified window on OSD.
**
** \param   h           Mosaic handle
** \param   windowNum   Index of window to perform operation on.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**         - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
******************************************************************************/
FSTATIC int32_t MENU_MosaicShowWindow (FAPI_SYS_HandleT h, uint16_t windowNum)
{
    MENU_MOSAIC_INST_S*             inst = (MENU_MOSAIC_INST_S*)h;
    int32_t                         retVal= FAPI_OK;
    OSDHANDLER_LayerParamsT         osdlayerParams;
    AV_VIDEO_STREAMINFO_S           videoInfo;
    FAPI_VIDEC_VideoDecInfoStateT   frameInfo;
    VOC_VidecInfoT                  videcInfo;
    VOC_PictureWindowT              *winPos;

    if ( inst == NULL )
    {
        FAPI_SYS_PRINT_DEBUG(1, "[MOSAIC] Mosaic is not available\n");
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    winPos = inst->picturePos + windowNum;
    memset(&osdlayerParams, 0 , sizeof(OSDHANDLER_LayerParamsT));
    for(;;)
    {
        retVal = OSDHANDLER_GlobalAlpha(OSD_LAYER_MOSAIC,
                                     OSDHANDLER_PARAMETER_SET, 0xFF);
        if (retVal != FAPI_OK)
        {
            break;
        }
        retVal = OSDHANDLER_GetLayerParameters(OSD_LAYER_MOSAIC,
                                                        &osdlayerParams);
        if (retVal != FAPI_OK)
        {
            break;
        }

        retVal = AV_FreezeVideo(FTRUE);
        if (retVal != FAPI_OK)
        {
            break;
        }

        retVal = AV_Nop();
        if (retVal != FAPI_OK)
        {
            break;
        }

        retVal = AV_GetVideoStreamInfo (&videoInfo);
        if (retVal != FAPI_OK)
        {
            break;
        }

        retVal = AV_GetVideoFrameInfo (&frameInfo);
        if (retVal != FAPI_OK)
        {
            break;
        }

        #ifdef MOSAIC_PRINT_DBG
        FAPI_SYS_PRINT_DEBUG(1,
            "FBSizeType=%d, displayWidth=%d, decodedHeight=%d, frameBank=%d\n",
                      videoInfo.decoder.decodeFrameBankSizeType,
                      videoInfo.decoder.displayWidth,
                      videoInfo.decoder.decodedHeight,
                      frameInfo.frameBank);
        #endif /* MOSAIC_PRINT_DBG */

        videcInfo.frameBank     = frameInfo.frameBank;
        videcInfo.decodedHeight = videoInfo.decoder.decodedHeight;
        videcInfo.displayWidth  = videoInfo.decoder.displayWidth;
        videcInfo.decodeFrameBankSizeType =
                                videoInfo.decoder.decodeFrameBankSizeType;

        retVal = VOC_CopyIFrame(inst->vocInst, winPos, &videcInfo,
                                                        &osdlayerParams);

        #ifdef MOSAIC_PRINT_DBG
        FAPI_SYS_PRINT_DEBUG(1, "[MOSAIC] VOC_CopyIFrame is called\n");
        #endif /* MOSAIC_PRINT_DBG */
        if (retVal != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1, "[MOSAIC] VOC_CopyIFrame failed\n");
            return(retVal);
        }

        break; /*end of for(;;) */
    }

    return(retVal);
}
/*!
*******************************************************************************
**
** \brief  Print information in a given window on OSD when its service is
**         not available.
**
** \param h            Mosaic handle
** \param windowNum    Index of window to perform operation on.
** \param testOn       consider testCnt when it is on
** \param testCnt      indicate number of times when handletime is called
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
FSTATIC void MENU_MosaicPrintServState (FAPI_SYS_HandleT h, uint16_t windowNum,
                                fbool_t testOn, uint8_t testCnt)
{
    MENU_MOSAIC_INST_S*     inst = (MENU_MOSAIC_INST_S*)h;
    VOC_PictureWindowT     *winPos;
    #ifdef MOSAIC_PRINT_DBG
    SYS_STRING_S            string;
    #endif /* MOSAIC_PRINT_DBG */

    winPos = inst->picturePos + windowNum;

    if(testOn)
    {
        #ifdef MOSAIC_PRINT_DBG
        if (testCnt==3)
        {
            string =  STR_GetASCII("wait !!");

            MENU_MosaicSetSolid((FAPI_SYS_HandleT)inst, OSD_LAYER_MENU,
                                                    windowNum, OSD_COL_BLANK);
            OSD_DrawStringLtd ((uint16_t)(winPos->left + 10),
                       (uint16_t)(winPos->top  + (winPos->vsize/2)),
                        OSD_COL_LIST_TEXT_HIGHL,
                        (uint8_t)MAIN_FONT, -1, (uint16_t)(winPos->hsize - 10),
                       (int32_t)(winPos->hsize - 10),
                        string, FFALSE);
        }
        #endif /* MOSAIC_PRINT_DBG */
    }
    else
    {
        OSD_DrawStringLtd ((uint16_t)(winPos->left + 10),
                       (uint16_t)(winPos->top  + (winPos->vsize/2)),
                        OSD_COL_TEXT,
                        (uint8_t)MAIN_FONT, -1, (uint16_t)(winPos->hsize - 10),
                       (int32_t)(winPos->hsize - 10),
                        STR_GetASCII("Not available!"), FFALSE);
    }
}

FSTATIC int32_t MENU_MosaicGetLayer(FAPI_SYS_HandleT h, fbool_t isFirstHit)
{
    MENU_MOSAIC_INST_S*     inst = (MENU_MOSAIC_INST_S*) h;
    SYS_OSDLAYER_S          layerParams;
    SYS_HDTV_STANDARD       hdSys;
    SYS_SDTV_STANDARD       sdSys;
    fbool_t                 frameBased = FFALSE;
    int32_t                 retVal = FAPI_OK;
    uint32_t                errorCode;

    if( inst->haveOsdLayer == FTRUE)
    {
        return FAPI_OK;
    }
    /*------set osd parameters-----------------------------------------------*/
    (void)SETTINGS_OutputTvStandardGet(&hdSys, &sdSys);
    switch (hdSys)
    {
        case HDSYS_576P:
        case HDSYS_720P:
            frameBased = FTRUE;
            break;
        default:
        case HDSYS_576I:
        case HDSYS_1080I:
        case HDSYS_NUMS:
            break;
    }

    inst->osdParams.xPos            = 0;
    inst->osdParams.yPos            = 0;
    inst->osdParams.inputWidth      = SCALER_GetCurrVal(MOSAIC_OSD_WIDTH);
    inst->osdParams.inputHeight     = SCALER_GetCurrVal(MOSAIC_OSD_HEIGHT);
    inst->osdParams.outputWidth     = 0;
    inst->osdParams.outputHeight    = 0;
    inst->osdParams.frameBased      = (int32_t)frameBased;
    inst->osdParams.memoryBasePtr   = NULL;

    for (;;)
    {
        /*****Prepare MOSAIC layer for displaying mosaic windows*****/
        //set layer mode to 16-bit CCIR mode
        layerParams.params = inst->osdParams;
        layerParams.layerIdx = OSD_LAYER_MOSAIC;
        layerParams.colourMode = OSDHANDLER_OSD_MODE_16BIT;
        layerParams.layoutMode = OSDHANDLER_OSD_LAYOUT_CbYCrA;
        layerParams.fillColour = 0x8010;
        retVal = SYS_GetOsdLayer(&layerParams, NULL);
        if(retVal != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1,
                    "[MOSAIC]SYS_GetOsdLayer(OSD_LAYER_MOSAIC) failed, %d\n",
                    retVal);
            break;
        }

        retVal = OSDHANDLER_ShowLayer(OSD_LAYER_MOSAIC, &errorCode);
        if(retVal != FAPI_OK)
        {
            break;
        }

        inst->haveOsdLayer = FTRUE;

        #ifdef MOSAIC_VIDEO_DEBUG
        retVal = OSDHANDLER_PlaneOrder(OSDHANDLER_PARAMETER_SET,
                                                        3,2,0,1,&errorCode);
        #endif /* MOSAIC_VIDEO_DEBUG */

        break; /* end of for(;;) */
    }

    if ( retVal != FAPI_OK )
    {
        /* Release OSD layer */
        if ( inst->haveOsdLayer )
        {
            (void)SYS_GiveOsdLayer(OSD_LAYER_MOSAIC);
            inst->haveOsdLayer = FFALSE;
        }

        /* restart previous service if required*/
        (void)MAIN_SelectService (&(inst->storedServLoc), FFALSE);

        return retVal;
    }
    else
    {
        return FAPI_OK;
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
