/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_pvr.c
**
**              $Id: app_pvr.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       PVR handling and "record-stop" menu implementation.
**
** Detail description. [\ref app_smartgo_app_ref_pvr]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: Loss of precision
** - Info    715: Symbol not referenced
** - Info    774: Boolean within 'if' always evaluates to True.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1,
**                dependence placed on C's operator precedence;
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e774 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_menu.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
// TODO: choose sensible value: curr. 1 GB
#define MENU_PVR_BKGRFILESIZE 1024    // Circular file size in MBytes

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum
{
    MENU_RECSTOP_ID_TITLE = 0,
    MENU_RECSTOP_ID_LIST,
    MENU_RECSTOP_ID_HELPBOTTOM

};

enum
{
    MENU_RECSTOP_STOP1 = 0,
    MENU_RECSTOP_STOP2,
    MENU_RECSTOP_ABORT
};

typedef struct
{
    FAPI_SYS_HandleT    pnlHdl;
    FAPI_SYS_HandleT    listHdl;

    PVRSYS_STATUS_S     pvrStat;

} MENU_RECSTOP_INST_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

static MENU_RECSTOP_INST_S* recstopDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC fbool_t MENU_RecordStopEntry        (FAPI_SYS_HandleT h, fbool_t test);
FSTATIC void    MENU_RecordStopOnFocusSet   (FAPI_SYS_HandleT h, fbool_t hasFocus);
FSTATIC fbool_t MENU_RecordStopLeave        (FAPI_SYS_HandleT h, fbool_t force);
FSTATIC void    MENU_RecordStopGetDescrItem (FAPI_SYS_HandleT h, int32_t id,
                                             uint32_t index,
                                             FGS_ITEMSTATE_E state,
                                             FGS_ITEM_S* item);
FSTATIC fbool_t MENU_RecordStopBuild        (FAPI_SYS_HandleT h);
FSTATIC fbool_t MENU_RecordStopKeys         (FAPI_SYS_HandleT h);

FSTATIC fbool_t MAIN_PvrStartBackgroundRecording (uint8_t* recIdx);

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/

int32_t MENU_PvrInit ()
{
    if ( (recstopDat != NULL) ) return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;

    recstopDat = UTILS_Malloc (sizeof(MENU_RECSTOP_INST_S));
    if ( recstopDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset (recstopDat, 0, sizeof(MENU_RECSTOP_INST_S));

    return FAPI_OK;
}

FSTATIC fbool_t MENU_RecordStopEntry (FAPI_SYS_HandleT h, fbool_t test)
{
    MENU_RECSTOP_INST_S* inst    = recstopDat;
    PVRSYS_STATUS_S*     pvrStat = (PVRSYS_STATUS_S*)h;

    if ( inst == NULL )
    {
        return FFALSE;
    }
    else if ( pvrStat == NULL )
    {
        return FFALSE;
    }
    else if ( inst->pnlHdl != NULL )
    {
        FAPI_SYS_PRINT_DEBUG(2,"MENU_RecordStopEntry: already open!\n");
        return FFALSE;
    }

    if ( test == FTRUE ) return FTRUE;

    inst->pvrStat = *pvrStat;

    (void)MENU_RecordStopBuild( (FAPI_SYS_HandleT) inst);

    (void)FGS_LIST_SetMemberFocus  (inst->listHdl, MENU_RECSTOP_ABORT);
    (void)FGS_PANEL_SetMemberFocus (inst->pnlHdl, MENU_RECSTOP_ID_LIST);
    (void)MENUSTACK_FocusClaim     ((int32_t)MENUID_RECSTOP);
    (void)FGS_PANEL_Show           (inst->pnlHdl, FFALSE); /* turn visibility on */

    MENUSTACK_SetCleanup (MENU_RecordStopLeave, (FAPI_SYS_HandleT)inst);
    return FTRUE;
}

FSTATIC fbool_t MENU_RecordStopLeave (FAPI_SYS_HandleT h, fbool_t force)
{
    MENU_RECSTOP_INST_S* inst = (MENU_RECSTOP_INST_S*)h;

    if ( inst->pnlHdl != NULL ) (void)FGS_PANEL_Close(inst->pnlHdl);

    (void)MENUSTACK_FocusRelease ((int32_t)MENUID_RECSTOP);

    inst->pnlHdl = NULL;
    return FTRUE;
}

FSTATIC void MENU_RecordStopOnFocusSet (FAPI_SYS_HandleT h, fbool_t hasFocus)
{
    DBG_Assert(((MENU_RECSTOP_INST_S*)h)->pnlHdl != NULL);

    MAIN_HandleExitKeys(hasFocus);
    if ( hasFocus )
        FGS_SetAppKeyHandler(FGS_KEY_STOP,   MENU_RecordStopKeys, h, FTRUE, FFALSE);
    else
        FGS_ClearAppKeyHandler(FGS_KEY_STOP);
}

FSTATIC void MENU_RecordStopGetDescrItem (FAPI_SYS_HandleT h, int32_t id,
                                          uint32_t index,
                                          FGS_ITEMSTATE_E state,
                                          FGS_ITEM_S* item)
{
    MENU_RECSTOP_INST_S* inst   = (MENU_RECSTOP_INST_S*)h;
    int8_t               recIdx = -1;
    SLIST_ITEM_S         servItem;
    PVR_LIST_ITEM_S      recPars;
    uint16_t             bufPos = 0;

    switch ( id )
    {
        case MENU_RECSTOP_STOP1:
            recIdx = 0;
            break;

        case MENU_RECSTOP_STOP2:
            recIdx = 1;
            break;

        case MENU_RECSTOP_ABORT:
            break;

        default:
            return;
    }

    if ( recIdx >= 0 )
    {
        (void)STR_BufClear(NULL);

        switch ( inst->pvrStat.rec[recIdx].type )
        {
            case REC_TYPE_NORMAL:
                if ( SLIST_GetServByIndex(inst->pvrStat.rec[recIdx].servIdx,
                                          &servItem) == FAPI_OK )
                {
                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_Build(servItem.nameCharSet,
                                                       servItem.name),
                                             MAIN_LINEBUFCHARSET);

                    bufPos += STR_BufAppend (NULL, bufPos,
                                             STR_GetASCII(" - "),
                                             MAIN_LINEBUFCHARSET);
                }
                break;

            case REC_TYPE_DESCRREC:
                // to do: add string!
                bufPos += STR_BufAppend (NULL, bufPos,
                                         STR_GetASCII("DESCRAMBLING"),
                                         MAIN_LINEBUFCHARSET);

                bufPos += STR_BufAppend (NULL, bufPos,
                                         STR_GetASCII(" - "),
                                         MAIN_LINEBUFCHARSET);
                break;

            default:
                break;
        }

        if ( PVRSYS_RecordGetInfo (recIdx, &recPars, NULL) == FAPI_OK )
        {
            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_Build(recPars.nameCharSet,
                                               recPars.name),
                                     MAIN_LINEBUFCHARSET);
        }

        if ( bufPos < 1 ) /* Fallback if all other attempts fail... */
        {
            sprintf(appDat->asciiBuf, "Record %i", recIdx);

            bufPos += STR_BufAppend (NULL, bufPos,
                                     STR_IntASCIIStr(),
                                     MAIN_LINEBUFCHARSET);
        }

        item->item.str.chr = STR_IntLineStr();
    }
    else
    {
        item->item.str.chr = OSD_GetStr(STR_EMPTYSTRING);
    }

    item->type                = FGS_ITEM_TYPE_STRING;
    item->align               = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_CENTER;

    item->item.str.font       = MAIN_FONT;
    item->item.str.lineHeight = 0;

    switch ( state )
    {
        default:
        case FGS_NONFOCUS: item->item.str.col = OSD_COL_TEXT;             break;
        case FGS_FOCUS:    item->item.str.col = OSD_COL_TEXT_HIGHLIGHTED; break;
        case FGS_DISABLED: item->item.str.col = OSD_COL_TEXT_DISABLED;    break;
    }
}

FSTATIC fbool_t MENU_RecordStopBuild (FAPI_SYS_HandleT h)
{
    MENU_RECSTOP_INST_S* inst   = (MENU_RECSTOP_INST_S*)h;
    FGS_PANEL_OPEN_S     pnlPars;
    FGS_LIST_OPEN_S      listPars;
    FGS_ITEM_TXT_OPEN_S  txtPars;
    FGS_BOX_OPEN_S       boxPars;
    FGS_ITEM_S           item;
    FGS_POS_S            itemPos;
    FAPI_SYS_HandleT     itemHdl;
    int32_t              retVal;
    uint16_t             numLines = 1;
    uint16_t             innerHeight;
    uint8_t              numHelpLines = 2;
    int32_t              cnt;
    uint16_t             mainPanelWidth;
    uint16_t             bottomHelpKeyWidth;
    uint16_t             bottomHelpItemWidth;
    int32_t              idCnt;

    /* get number of current active recordings */
    for ( cnt = 0; cnt < RSRC_REC_MAX; cnt++ )
    {
        if ( (inst->pvrStat.rec[cnt].status != REC_STATUS_INACTIVE) &&
             (inst->pvrStat.rec[cnt].type   != REC_TYPE_BACKGROUND) )
        {
            numLines++;
        }
    }

    do
    {
        memset (&pnlPars, 0, sizeof(pnlPars));

        innerHeight = numLines *
                     (SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                    + SCALER_GetCurrVal(LIST_LINE_PADDING));

        MENU_CommonSetParamsTitleWBotHelp (&pnlPars,
                                           SCALER_GetCurrVal(MAIN_WIDTH),
                                           innerHeight,
                                           numHelpLines, NULL);

        mainPanelWidth          = pnlPars.pos.width
                                - (2 * SCALER_GetCurrVal(OUTER_FRAME_ALL));
        bottomHelpKeyWidth      = SCALER_GetCurrVal(HELP_BOT_XOFF) -
                                  SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        bottomHelpItemWidth     = (mainPanelWidth / 2)
                                - bottomHelpKeyWidth
                                - SCALER_GetCurrVal(MAIN_RIGHT_PADDING);

        pnlPars.onFocusSet    = MENU_RecordStopOnFocusSet;
        pnlPars.onFocusSetArg = (FAPI_SYS_HandleT)inst;

        MENU_CommonSetPanelPos(&(pnlPars.pos));

        retVal = FGS_PANEL_Open(&pnlPars, MENUID_RECSTOP, &(inst->pnlHdl));
        if ( retVal != FAPI_OK ) break;

        /* ----- create textbox holding title ------------------------------ */

        retVal = MENU_CommonAddTitle(MENU_RECSTOP_ID_TITLE,
                                     inst->pnlHdl, pnlPars.pos.width,
                                     0, STR_TITLE_STOPREC,
                                     NULL, NULL);
        if ( retVal != FAPI_OK ) break;

        /* ----- create list ----------------------------------------------- */

        memset (&listPars, 0, sizeof(listPars));

        listPars.getDescrItem    = MENU_RecordStopGetDescrItem;
        listPars.getDescrItemArg = (FAPI_SYS_HandleT)inst;

        inst->listHdl = MENU_CommonAddList (&listPars, MENU_RECSTOP_ID_LIST,
                                            0, 0, SCALER_GetCurrVal(MAIN_WIDTH),
                                            innerHeight, inst->pnlHdl,
                                            &itemPos, NULL, NULL, 100,
                                            &retVal);
        if ( inst->listHdl == NULL ) break;

        /* ----------------------------------------------------------------- */

        memset (&txtPars, 0, sizeof(FGS_ITEM_TXT_OPEN_S));

        for ( cnt = 0; cnt < RSRC_REC_MAX; cnt++ )
        {
            if ( (inst->pvrStat.rec[cnt].status == REC_STATUS_INACTIVE) ||
                 (inst->pvrStat.rec[cnt].type   == REC_TYPE_BACKGROUND) )
            {
                continue;
            }

            txtPars.pos                = itemPos;
            txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
            txtPars.descrPos.x         = SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
            txtPars.descrPos.y         = 0;
            txtPars.descrPos.width     = listPars.pos.width
                                       - (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X));
            txtPars.descrPos.height    = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
            txtPars.descrPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

            retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_RECSTOP_STOP1 + cnt,
                                        inst->listHdl, &itemHdl);
            if ( retVal != FAPI_OK ) break;

            itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                                 + SCALER_GetCurrVal(LIST_LINE_PADDING));
        }

        /* ----------------------------------------------------------------- */

        txtPars.pos                = itemPos;
        txtPars.bkgrType           = OSD_LSTITEM_BKGR_NORMAL;
        txtPars.descrPos.x         = SCALER_GetCurrVal(MAIN_TEXT_PAD_X);
        txtPars.descrPos.y         = 0;
        txtPars.descrPos.width     = listPars.pos.width
                                   - (2 * SCALER_GetCurrVal(MAIN_TEXT_PAD_X));
        txtPars.descrPos.height    = SCALER_GetCurrVal(LIST_LINE_HEIGHT);
        txtPars.descrPos.alignment = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_TOP;

        retVal = FGS_ITEM_TXT_Open (&txtPars, MENU_RECSTOP_ABORT,
                                    inst->listHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        itemPos.y += (int16_t)SCALER_GetCurrVal(LIST_LINE_HEIGHT);

        /* -----Bottom help lines ------------------------------------------ */

        itemPos.y += (int16_t)(SCALER_GetCurrVal(LIST_LINE_HEIGHT)
                             + SCALER_GetCurrVal(LIST_LINE_PADDING));

        idCnt = MENU_RECSTOP_ID_HELPBOTTOM;

        memset (&item, 0, sizeof(FGS_ITEM_S));

        MENU_COMMON_ITEM_HELPSTR(&item, STR_KEY_STOP);

        memset (&boxPars, 0, sizeof(boxPars));

        boxPars.pos.x               = SCALER_GetCurrVal(OUTER_FRAME_ALL)
                                    + SCALER_GetCurrVal(MAIN_LEFT_PADDING);
        boxPars.pos.y               = - (int16_t)((SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2)
                                      + SCALER_GetCurrVal(HELP_TB_PADDING)
                                      + (numHelpLines * SCALER_GetCurrVal(HELP_LINE_HEIGHT))
                                      + ((numHelpLines - 1) * SCALER_GetCurrVal(HELP_LINE_PADDING)));
        boxPars.pos.width           = bottomHelpKeyWidth;
        boxPars.pos.height          = SCALER_GetCurrVal(HELP_LINE_HEIGHT);
        boxPars.pos.alignment       = FGS_ALIGN_HOR_LEFT | FGS_ALIGN_VER_BOTTOM;
        boxPars.bkgrType            = OSD_BOX_BKGR_NONE;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        /* -----Colon ------------------------------------------ */
        item.item.str.chr           = OSD_GetStr(STR_GNR_COLON);
        boxPars.pos.x               += (int16_t)SCALER_GetCurrVal(HELP_BOT_XOFF);
        boxPars.pos.width           = OSD_GetStrWidth(item.item.str.chr, -1,
                                      HELP_FONT);
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;

        item.item.str.chr           = OSD_GetStr(STR_HELP_RECORDING_STOP);
        boxPars.pos.x              += (int16_t)boxPars.pos.width;
        boxPars.pos.width           = bottomHelpItemWidth;
        boxPars.item[FGS_NONFOCUS]  = item;

        retVal = FGS_BOX_Open (&boxPars, idCnt++, inst->pnlHdl, &itemHdl);
        if ( retVal != FAPI_OK ) break;
        retVal = MENU_CommonAddMenuExitHelpLine(SCALER_GetCurrVal(OUTER_FRAME_ALL) +
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING),
                                              - (int16_t)((SCALER_GetCurrVal(OUTER_FRAME_ALL) / 2) +
                                                SCALER_GetCurrVal(HELP_TB_PADDING) +
                                                SCALER_GetCurrVal(HELP_LINE_HEIGHT)),
                                                pnlPars.pos.width -
                                                (2 * SCALER_GetCurrVal(OUTER_FRAME_ALL)) -
                                                SCALER_GetCurrVal(MAIN_LEFT_PADDING) -
                                                SCALER_GetCurrVal(MAIN_RIGHT_PADDING),
                                                inst->pnlHdl);
        if ( retVal != FAPI_OK ) break;

    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->pnlHdl != NULL )
        {
            (void)FGS_PANEL_Close(inst->pnlHdl);
            inst->pnlHdl = NULL;
        }
        return FFALSE;
    }
    return FTRUE;
}


FSTATIC fbool_t MENU_RecordStopKeys (FAPI_SYS_HandleT h)
{
    MENU_RECSTOP_INST_S* inst   = (MENU_RECSTOP_INST_S*)h;
    int32_t              pnlFocus;
    int8_t               recIdx = -1;

    pnlFocus = FGS_LIST_GetMemberFocusID(inst->listHdl);

    switch (pnlFocus)
    {
        case MENU_RECSTOP_STOP1:
            recIdx = 0;
            break;

        case MENU_RECSTOP_STOP2:
            recIdx = 1;
            break;

        default:
            return FFALSE;
    }

    if ( (recIdx >= 0) &&
         (recIdx < RSRC_REC_MAX) )
    {
        /* stop recording */
        (void)MAIN_PvrStopRecording(recIdx);

        if ( inst->pvrStat.rec[recIdx].timerIdx != RECTIMER_INVALID )
        {
            /* remove/re-insert timer */
            (void)RECTIMER_Reinsert(inst->pvrStat.rec[recIdx].timerIdx);

            MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);

            (void)RECTIMER_Save(FFALSE);

            (void)MAIN_MsgBoxHide(&(appDat->msgBox));
        }
    }

    return MENUSTACK_Back(NULL);
}

/* ------------------------------------------------------------------------- */

int32_t MAIN_PvrStopRecording (int8_t recIdx)
{
    PVRSYS_STATUS_S pvrStatus;
    int32_t         retVal;
    fbool_t         keepFile = FFALSE;

    retVal = PVRSYS_GetStatus(&pvrStatus);
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    if ( (recIdx < 0) || (recIdx >= RSRC_REC_MAX) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    switch ( pvrStatus.rec[recIdx].type )
    {
        case REC_TYPE_DESCRREC:
        case REC_TYPE_NORMAL:
            keepFile = FTRUE;
            break;

        case REC_TYPE_BACKGROUND:
            keepFile = FFALSE;
            break;

        default:
            return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    /* stop potentially running playback on recording */
    if ( (pvrStatus.play.type   == PLAY_TYPE_TIMESHIFT) &&
         (pvrStatus.play.recIdx == recIdx) )
    {
        (void)MAIN_PvrResumeLiveTv(&pvrStatus);
    }

    /* stop recording */
    return PVRSYS_RecordStop(recIdx, keepFile);
}

/*!
*******************************************************************************
**
** \brief  Starts instant recording after pressing REC key.
**
** This function prepares all data that is neccessary to start instant recording
** of the current selected service. If a harddisk is installed and a recording
** slot is free. Otherwise the request is denied.
** Next the Edit Menu is called to allow changes to the timer parameters. Pressing
** REC Key a second time leads to immediate storage and start of recording.
**
** \return Possible return codes:
**         - #FTRUE
**         - #FFALSE
*******************************************************************************
*/

fbool_t MAIN_PvrStartInstantRecording ()
{
    RECTIMER_ITEM_S         timerItem;
    EIT_EVT_S               evtCurr;
    SLIST_SERVIDX           servIdx;
    fbool_t                 hasSynced;
    MENU_TIMEREDIT_PARAMS_S editPars;
    fbool_t                 retVal;

    memset (&editPars, 0, sizeof(MENU_TIMEREDIT_PARAMS_S));

    if ( AV_GetRequestedService (&(editPars.newService)) != FAPI_OK )
    {
        return FFALSE;
    }

    servIdx = SLIST_GetServiceIndex(&(editPars.newService));

    if ( EIT_GetEvent (&(editPars.newService), EIT_TYPE_CURR, &evtCurr)
         != FAPI_OK )
    {
        memset (&evtCurr, 0, sizeof(evtCurr));
    }

    /* default is a recording timer */
    memset (&timerItem, 0, sizeof(timerItem));

    timerItem.operation = RECTIMER_OP_RECORD;
    timerItem.type      = RECTIMER_TYPE_SINGLE;
    timerItem.enabled   = FTRUE;

    (void)STR_Copy (STR_Build(evtCurr.nameCharSet, evtCurr.name),
                        timerItem.desc, &(timerItem.descCharSet),
                        evtCurr.nameCharSet,
                        RECTIMER_DESC_LENGTH);

    (void)STR_Copy (STR_Build(evtCurr.descCharSet, evtCurr.desc),
                        timerItem.extDesc,
                        &(timerItem.extDescCharSet),
                        evtCurr.descCharSet,
                        RECTIMER_DESC_LENGTH);

    if ( TIME_TimeAndDateGetUtc (&(timerItem.startTime),
                                 &(timerItem.startDate),
                                 &hasSynced) != FAPI_OK )
    {
        return FFALSE;
    }

    /* ensure that valid time information is already available */
    if ( !hasSynced ) return FFALSE;

    /* clear seconds: record at full minute only! */
    timerItem.startTime.second = 0;
    timerItem.endTime = evtCurr.startTime;
    timerItem.endDate = evtCurr.startDate;

    (void)TIME_AddUtcOffset(&(evtCurr.startTime), &(evtCurr.startDate));

    /* calculate end time of current service and add 3 minutes */
    TIME_AddSeconds (&(timerItem.endTime),
                     &(timerItem.endDate),
                     (int32_t)TIME_ConvToSeconds(&(evtCurr.duration)) + (3 * 60));


    timerItem.serviceIndex = servIdx;
    timerItem.eventId      = 0;
    timerItem.autoStandby  = FFALSE;
    timerItem.targetDevice = MAIN_PvrGetDefaultRecDevice();

    if ( timerItem.targetDevice == SYS_DEV_NULL )
    {
        /* no recording device found */
        MENU_CommonMsgShow(STR_MSG_NORECDEVICE, FFALSE);
        retVal = FFALSE;
    }
    else
    {
        editPars.timerIndex = -1;
        editPars.newDat     = &timerItem;

        retVal = MENUSTACK_Switch (MENU_TimerEditEntry,
                                  (FAPI_SYS_HandleT)(&editPars), 1);
    }

    return retVal;
}

int32_t MAIN_PvrStartTimerRecording (RECTIMER_IDX idx)
{
    int32_t            retVal = FAPI_OK;
    RECTIMER_ITEM_S    recTimer;
    uint8_t            recIdx;
    const char_t*      targetDir;

    retVal = RECTIMER_GetByIndex(idx, &recTimer);
    if ( retVal != FAPI_OK ) return retVal;

    switch ( recTimer.targetDevice )
    {
        case SYS_DEV_HDD:
            targetDir = MAIN_DIR_HDD_PVR;
            break;

        default:
            targetDir = MAIN_DIR_OTH_PVR;
            break;
    }

    retVal = PVRSYS_RecordStart (recTimer.targetDevice, targetDir,
                                 STR_Build(recTimer.descCharSet, recTimer.desc),
                                 recTimer.serviceIndex, idx,
                                 &recIdx);

    if ( retVal == FAPI_OK )
    {
        MENU_CommonMsgShow(STR_MSG_RECSTARTED, FFALSE);
    }

    return retVal;
}

FSTATIC fbool_t MAIN_PvrStartBackgroundRecording (uint8_t* recIdx)
{
    EIT_EVT_S           evtCurr;
    SYS_SERVLOC_S       servDesc;
    SLIST_SERVIDX       servIdx;
    const char_t*       targetDir;
    SYS_DEVICE_E        targetDevice = MAIN_PvrGetDefaultRecDevice();
    fbool_t             retVal;

    if ( targetDevice == SYS_DEV_NULL )
    {
        /* no recording device found */
        MENU_CommonMsgShow(STR_MSG_NORECDEVICE, FFALSE);
        retVal = FFALSE;
    }
    else if ( AV_GetRequestedService (&servDesc) != FAPI_OK )
    {
        retVal = FFALSE;
    }
    else
    {
        servIdx = SLIST_GetServiceIndex(&servDesc);

        if ( EIT_GetEvent (&servDesc, EIT_TYPE_CURR, &evtCurr) != FAPI_OK )
        {
            memset (&evtCurr, 0, sizeof(evtCurr));
        }

        switch ( targetDevice )
        {
            case SYS_DEV_HDD:
                targetDir = MAIN_DIR_HDD_PVR;
                break;

            default:
                targetDir = MAIN_DIR_OTH_PVR;
                break;
        }

        if ( PVRSYS_RecordBackgroundStart (targetDevice, targetDir,
                                           STR_Build(evtCurr.nameCharSet,
                                           evtCurr.name), servIdx,
                                           MENU_PVR_BKGRFILESIZE, recIdx)
                                           == FAPI_OK )
        {
            retVal = FTRUE;
        }
        else
        {
            retVal = FFALSE;
        }
    }

    return retVal;
}

fbool_t MAIN_PvrKeyStop ()
{
    fbool_t         retVal = FFALSE;
    PVRSYS_STATUS_S pvrStatus;
    uint8_t         cnt;

    if ( PVRSYS_GetStatus(&pvrStatus) != FAPI_OK )
    {
        return FFALSE;
    }

    /* ----- (Timeshift) playback is running ------------------------------- */

    if ( pvrStatus.play.type != PLAY_TYPE_NONE )
    {
        /* stop any playback (timeshift or normal) */
        (void)MAIN_PvrResumeLiveTv(&pvrStatus);

        retVal = FTRUE;
    }

    /* ----- Record(s) running --------------------------------------------- */

    else if ( pvrStatus.numActiveRecs > 0 )
    {
        for ( cnt = 0; cnt < RSRC_REC_MAX; cnt++ )
        {
            if ( (pvrStatus.rec[cnt].status != REC_STATUS_INACTIVE) &&
                 (pvrStatus.rec[cnt].type   != REC_TYPE_BACKGROUND) )
            {
                break;
            }
        }

        /* Show record stop dialogue only if non-background recordings are
           active. */
        if ( cnt < RSRC_REC_MAX )
        {
            retVal = MENUSTACK_Switch (MENU_RecordStopEntry,
                                      (FAPI_SYS_HandleT)(&pvrStatus), 1);
        }
    }

    /* --------------------------------------------------------------------- */

    return retVal;
}

fbool_t MAIN_PvrKeyPlay ()
{
    fbool_t         keyUsed = FTRUE;
    PVRSYS_STATUS_S pvrStatus;
    int32_t         retVal;

    if ( PVRSYS_GetStatus(&pvrStatus) != FAPI_OK )
    {
        return FFALSE;
    }

    if ( pvrStatus.play.type != PLAY_TYPE_NONE ) /* during playback */
    {
        switch ( pvrStatus.play.status )
        {
            case PLAY_STATUS_STOP:
            case PLAY_STATUS_PAUSE:
            case PLAY_STATUS_FAST:
            case PLAY_STATUS_SLOW:
                /* resume from any fast mode, release pause */
                retVal = PVRSYS_PlaySetMode (PLAY_STATUS_NORMAL, 0);

                if ( retVal != FAPI_OK )
                {
                    keyUsed = FFALSE;
                }
                break;

            case PLAY_STATUS_NORMAL:
            default:
                /* do nothing */
                break;
        }
    }
    else /* not in playback */
    {
        keyUsed = MENUSTACK_Switch(MENU_PlayListEntry, NULL, 1);
    }

    return keyUsed;
}

fbool_t MAIN_PvrKeyPause ()
{
    fbool_t         keyUsed = FTRUE;
    PVRSYS_STATUS_S pvrStatus;
    uint8_t         recIdx;
    SLIST_SERVIDX   servIdx = SLIST_SERVIDX_NONE;
    SYS_SERVLOC_S   currServLoc;
    int32_t         retVal;

    if ( PVRSYS_GetStatus(&pvrStatus) != FAPI_OK )
    {
        return FFALSE;
    }

    /* during playback */
    if ( pvrStatus.play.type != PLAY_TYPE_NONE )
    {
        switch ( pvrStatus.play.status )
        {
            case PLAY_STATUS_STOP:
            case PLAY_STATUS_PAUSE:
                retVal = PVRSYS_PlaySetMode (PLAY_STATUS_NORMAL, 0);
                break;

            default:
                retVal = PVRSYS_PlaySetMode (PLAY_STATUS_PAUSE, 0);
                break;
        }

        if ( retVal != FAPI_OK )
        {
            keyUsed = FFALSE;
        }
    }
    else /* not in playback: start/enter timeshift */
    {
        /* before starting background recording, test if other
           recordings are available */

        /* test if a background recording is already active */
        if ( pvrStatus.bkgrRecIdx >= 0 )
        {
            (void)AV_Freeze();

            /* start playback on current recording position in pause mode */
            if ( MAIN_PvrStartTimeShiftPlayback (pvrStatus.bkgrRecIdx,
                                            PVRSYS_TIMESHIFT_CURRRECPOS,
                                            PLAY_STATUS_PAUSE, 0) != FAPI_OK )
            {
                keyUsed = FFALSE;
            }
        }
        else /* no background recording active */
        {
            /* test if any recording is active on current service */
            if ( AV_GetRequestedService(&currServLoc) == FAPI_OK )
            {
                servIdx = SLIST_GetServiceIndex (&currServLoc);
            }

            for ( recIdx = 0; recIdx < RSRC_REC_MAX; recIdx++ )
            {
                if ( (pvrStatus.rec[recIdx].status  != REC_STATUS_INACTIVE) &&
                     (pvrStatus.rec[recIdx].servIdx == servIdx) )
                {
                    (void)AV_Freeze();

                    if ( MAIN_PvrStartTimeShiftPlayback((int8_t)recIdx,
                                                   PVRSYS_TIMESHIFT_CURRRECPOS,
                                                   PLAY_STATUS_PAUSE, 0) !=
                                                   FAPI_OK )
                    {
                        keyUsed = FFALSE;
                    }
                    break;
                }
            }

            if ( recIdx >= RSRC_REC_MAX ) /* no recording active on current
                                             service */
            {
                /* No recording active: start background recording */
                keyUsed = MAIN_PvrStartBackgroundRecording(&recIdx);

                if ( keyUsed )
                {
                    /* freeze current live presentation */
                    (void)AV_Freeze();

                    /* Start playback in pause mode */
                    (void)MAIN_PvrStartTimeShiftPlayback((int8_t)recIdx,
                                                   PVRSYS_TIMESHIFT_BEGIN,
                                                   PLAY_STATUS_PAUSE, 0);
                }
            }
        }
    }

    return keyUsed;
}

fbool_t MAIN_PvrKeyForward ()
{
    fbool_t                 retVal     = FTRUE;
    PVRSYS_PLAY_STATUS_E    mode       = PLAY_STATUS_INACTIVE;
    int16_t                 speedIndex = 0;
    PVRSYS_STATUS_S         pvrStatus;

    if ( PVRSYS_GetStatus(&pvrStatus) != FAPI_OK )
    {
        return FFALSE;
    }

    switch ( pvrStatus.play.status )
    {
        case PLAY_STATUS_NORMAL:
        case PLAY_STATUS_SLOW:
            mode       = PLAY_STATUS_FAST;
            speedIndex = 0;         /* set initial fast forward mode */
            break;

        case PLAY_STATUS_FAST:
            if ( pvrStatus.play.speedIndex < 0 )
            {
                mode       = PLAY_STATUS_FAST;
                speedIndex = 0;     /* set initial fast forward mode */
            }
            else if ( (pvrStatus.play.speedIndex + 1) <
                      PVRSYS_PLAYMODE_FF_NUMS )
            {
                mode       = PLAY_STATUS_FAST;
                speedIndex = pvrStatus.play.speedIndex + 1; /* next fast mode */
            }
            else
            {
                retVal = FFALSE;
            }
            break;

        case PLAY_STATUS_PAUSE:

            #ifdef APPL_EDITMARKER_ENABLED

            /* NOTE: In case of TS recording playback the marker initialisation
                     might have failed at start of TS recording because CAT and
                     PMT also must have been received. */
            if (!(appDat->playStatus.cutModeActive) &&
                 (pvrStatus.play.recIdx >= 0) )
            {
                MAIN_PvrInitMarker ();
            }

            #endif /* APPL_EDITMARKER_ENABLED */

            mode       = PLAY_STATUS_FAST;
            speedIndex = 0;         /* set initial fast forward mode */
            break;

        default:
        case PLAY_STATUS_INACTIVE:
            retVal = FFALSE;
            break;
    }

    if ( mode != PLAY_STATUS_INACTIVE )
    {
        if ( PVRSYS_PlaySetMode (mode, speedIndex) != FAPI_OK )
        {
            retVal = FFALSE;
        }
    }

    return retVal;
}

fbool_t MAIN_PvrKeyRewind ()
{
    fbool_t                 retVal     = FTRUE;
    SLIST_SERVIDX           servIdx    = SLIST_SERVIDX_NONE;
    PVRSYS_PLAY_STATUS_E    mode       = PLAY_STATUS_INACTIVE;
    int16_t                 speedIndex = 0;
    PVRSYS_STATUS_S         pvrStatus;
    SYS_SERVLOC_S           currServLoc;
    int8_t                  cnt;

    #ifdef APPL_EDITMARKER_ENABLED
    PVR_RecordInfoT         recordInfo;
    #endif /* APPL_EDITMARKER_ENABLED */

    if ( PVRSYS_GetStatus(&pvrStatus) != FAPI_OK )
    {
        return FFALSE;
    }

    if ( pvrStatus.play.type != PLAY_TYPE_NONE ) /* during playback */
    {
        switch ( pvrStatus.play.status )
        {
            case PLAY_STATUS_NORMAL:
            case PLAY_STATUS_PAUSE:
            case PLAY_STATUS_SLOW:
                /* set initial fast rewind mode */
                mode       = PLAY_STATUS_FAST;
                speedIndex = -1;
                break;

            case PLAY_STATUS_FAST:
                if ( pvrStatus.play.speedIndex >= 0 )
                {
                    mode       = PLAY_STATUS_FAST;
                    speedIndex = -1;        /* set initial fast rewind mode */
                }
                else if ( (-(pvrStatus.play.speedIndex - 1)) <=
                          PVRSYS_PLAYMODE_FR_NUMS )
                {
                    mode  = PLAY_STATUS_FAST;
                    speedIndex = pvrStatus.play.speedIndex - 1; /* next mode */
                }
                else
                {
                    retVal = FFALSE;
                }
                break;

            case PLAY_STATUS_INACTIVE:
            default:
                retVal = FFALSE;
                break;
        }

        if ( mode != PLAY_STATUS_INACTIVE )
        {
            if ( PVRSYS_PlaySetMode (mode, speedIndex) != FAPI_OK )
            {
                retVal = FFALSE;
            }
        }
    }
    else if ( pvrStatus.bkgrRecIdx >= 0 ) /* background recording is active */
    {
        #ifdef APPL_EDITMARKER_ENABLED

        memset (&(appDat->blockMarkerData), 0, sizeof(PVR_BLOCK_MARKER_ITEM_S));

        #endif /* APPL_EDITMARKER_ENABLED */

        /* rewind into background recording if possible */
        if ( MAIN_PvrStartTimeShiftPlayback(pvrStatus.bkgrRecIdx,
                                            PVRSYS_TIMESHIFT_END,
                                            PLAY_STATUS_FAST,
                                            -1) != FAPI_OK )
        {
            /* nothing to do */
        }

        #ifdef APPL_EDITMARKER_ENABLED

        /* NOTE: Load marker data and enable driver only if playback start was
                 successful and if circular recording has not yet looped. */
        else if ( PVRSYS_RecordGetInfo (pvrStatus.bkgrRecIdx, NULL,
                                        &recordInfo) == FAPI_OK )
        {
            if( !(recordInfo.hasLooped) )
            {
                MAIN_PvrInitMarker ();
                MAIN_PvrGetABRepeatStatus();
            }
        }

        #endif /* APPL_EDITMARKER_ENABLED */

        else
        {
            /* nothing to do */
        }
    }
    else
    {
        /* test if a non-background recording is active on current service */

        if ( AV_GetRequestedService(&currServLoc) == FAPI_OK )
        {
            servIdx = SLIST_GetServiceIndex (&currServLoc);
        }

        for ( cnt = 0; cnt < RSRC_REC_MAX; cnt++ )
        {
            if ( (pvrStatus.rec[cnt].status  != REC_STATUS_INACTIVE) &&
                 (pvrStatus.rec[cnt].servIdx == servIdx) )
            {
                /* rewind into background recording if possible */
                if ( (MAIN_PvrStartTimeShiftPlayback(cnt,
                                                     PVRSYS_TIMESHIFT_END,
                                                     PLAY_STATUS_FAST,
                                                     -1)) != FAPI_OK )
                {
                    break;
                }

                #ifdef APPL_EDITMARKER_ENABLED

                /* load marker data and enable driver */
                MAIN_PvrInitMarker ();
                MAIN_PvrGetABRepeatStatus();

                #endif /* APPL_EDITMARKER_ENABLED */

                break;
            }
        }

        if ( cnt >= RSRC_REC_MAX )
        {
            retVal = FFALSE;
        }
    }

    return retVal;
}

fbool_t MAIN_PvrKeyForwSlow (void)
{
    fbool_t                 retVal     = FTRUE;
    PVRSYS_PLAY_STATUS_E    mode       = PLAY_STATUS_INACTIVE;
    int16_t                 speedIndex = 0;
    PVRSYS_STATUS_S         pvrStatus;

    if ( PVRSYS_GetStatus(&pvrStatus) != FAPI_OK )
    {
        return FFALSE;
    }

    if ( pvrStatus.play.type != PLAY_TYPE_NONE ) /* during playback */
    {
        switch ( pvrStatus.play.status )
        {
            case PLAY_STATUS_NORMAL:
            case PLAY_STATUS_PAUSE:
            case PLAY_STATUS_FAST:
                /* set initial slow mode */
                mode       = PLAY_STATUS_SLOW;
                speedIndex = 0;
                break;

            case PLAY_STATUS_SLOW:
                if (  (pvrStatus.play.speedIndex >= 0)
                  && ((pvrStatus.play.speedIndex +  1) <
                     PVRSYS_PLAYMODE_SF_NUMS) )
                {
                    mode       = PLAY_STATUS_SLOW;
                    speedIndex = pvrStatus.play.speedIndex + 1; /* next mode */
                }
                else
                {
                    retVal = FFALSE;
                }
                break;

            case PLAY_STATUS_INACTIVE:
            default:
                retVal = FFALSE;
                break;
        }
    }

    if ( mode != PLAY_STATUS_INACTIVE )
    {
        if ( PVRSYS_PlaySetMode (mode, speedIndex) != FAPI_OK )
        {
            retVal = FFALSE;
        }
    }

    return retVal;
}

fbool_t MAIN_PvrKeyRewSlow (void)
{
    fbool_t                 retVal     = FTRUE;
    PVRSYS_PLAY_STATUS_E    mode       = PLAY_STATUS_INACTIVE;
    int16_t                 speedIndex = 0;
    PVRSYS_STATUS_S         pvrStatus;

    if ( PVRSYS_GetStatus(&pvrStatus) != FAPI_OK )
    {
        return FFALSE;
    }

    if ( pvrStatus.play.type != PLAY_TYPE_NONE ) /* during playback */
    {
        switch ( pvrStatus.play.status )
        {
            case PLAY_STATUS_NORMAL:
            case PLAY_STATUS_PAUSE:
            case PLAY_STATUS_FAST:
            case PLAY_STATUS_INACTIVE:
            default:
                retVal = FFALSE;
                break;

            case PLAY_STATUS_SLOW:
                if ( pvrStatus.play.speedIndex == 0 )
                {
                    mode       = PLAY_STATUS_NORMAL;
                    speedIndex = 0;      /* back to normal playback */
                }
                else if (  (pvrStatus.play.speedIndex >  0)
                       && ((pvrStatus.play.speedIndex -  1) >= 0)
                       && ((pvrStatus.play.speedIndex -  1) <
                            PVRSYS_PLAYMODE_SF_NUMS) )
                {
                    mode       = PLAY_STATUS_SLOW;
                    speedIndex = pvrStatus.play.speedIndex - 1; /* prev. mode */
                }
                else
                {
                    retVal = FFALSE;
                }
                break;
        }
    }

    if ( mode != PLAY_STATUS_INACTIVE )
    {
        if ( PVRSYS_PlaySetMode (mode, speedIndex) != FAPI_OK )
        {
            retVal = FFALSE;
        }
    }

    return retVal;
}

fbool_t MAIN_PvrKeyJump(fbool_t jumpForward)
{
    int32_t         retVal  = FAPI_OK;
    fbool_t         keyUsed = FFALSE;
    PVRSYS_STATUS_S pvrStatus;
    int32_t         jumpTime = SETTINGS_DEFAULT_PVRJUMPTIME; // in minutes

    if ( PVRSYS_GetStatus(&pvrStatus) == FAPI_OK )
    {
        if ( pvrStatus.play.status != PLAY_STATUS_INACTIVE )
        {
            retVal = MAIN_UserDataGet(USERDAT_PVRJUMPTIME, (uint32_t*)&jumpTime);
            if(retVal != FAPI_OK)
            {
                FAPI_SYS_PRINT_DEBUG(4,"MAIN_UserDataGet() failed! %d\n", retVal);
            }
            else
            {
                if(jumpTime != 0)
                {
                    if(!jumpForward)
                    {
                        jumpTime = -jumpTime;
                    }
                    (void)PVRSYS_PlayJump((int32_t)(60*jumpTime),
                                                            PVR_JUMP_TIME_REL);
                    (void)PVRSYS_PlaySetMode(pvrStatus.play.status,
                                                    pvrStatus.play.speedIndex);
                }

                keyUsed = FTRUE;
            }
        }
        else
        {
            /* do nothing */
        }
    }
    return keyUsed;
}

#ifdef APPL_EDITMARKER_ENABLED

/*
    handle defining AB-Repeat area. First time pressed set
    A-mark, second time pressed set B-mark and enable AB repeat,
    third time pressed delete both marks again.
*/
fbool_t MAIN_PvrKeyABRepeat ()
{
    PVRSYS_STATUS_S         pvrStatus;
    PVR_LIST_ITEM_S         pvrListItem;
    fbool_t                 keyUsed;
    int32_t                 ferr;
    int8_t                  cnt;

    if ( PVRSYS_GetStatus(&pvrStatus) != FAPI_OK )
    {
        return FFALSE;
    }

    /* now read playlist data */
    if (PVR_LIST_GetItemByIndex ( pvrStatus.play.lstIdx,
                                  &pvrListItem,
                                  NULL
                                ) != FAPI_OK )
    {
        return FFALSE;
    }
    /* in case of no video PID this key is disabled */
    if ( pvrListItem.service.strPid.video == 0 &&
         pvrListItem.service.strPid.video == 0x1FFF
       )
    {
        return FFALSE;
    }

    keyUsed = FTRUE;

    switch ( appDat->playStatus.abRepeatMode )
    {
        default:
        case AB_REPEAT_MODE_NONE:
            if ( (ferr = PVRSYS_PlaySetMarker ( FDVR_PLAY_MARKER_AB_REPEAT_IN, MAX_BLOCK_MARKERS)) == FAPI_OK)
            {
                appDat->playStatus.abRepeatMode = AB_REPEAT_MODE_A_POS;
                PVRSYS_PlayGetBlockData(&(appDat->blockMarkerData));
            }

            else
                keyUsed = FFALSE;
            break;
        case AB_REPEAT_MODE_A_POS:
            if ( (ferr = PVRSYS_PlaySetMarker ( FDVR_PLAY_MARKER_AB_REPEAT_OUT, MAX_BLOCK_MARKERS)) == FAPI_OK)
            {
                appDat->playStatus.abRepeatMode = AB_REPEAT_MODE_B_POS;
                PVRSYS_PlayGetBlockData(&(appDat->blockMarkerData));
            }
            else
                keyUsed = FFALSE;
            break;
        case AB_REPEAT_MODE_B_POS:
            /* look for all AB Repeat Markers and delete them */
            for ( cnt = 0 ; ((uint8_t)cnt < appDat->blockMarkerData.numBlockMarker) ; cnt++)
            {
                if ( (appDat->blockMarkerData.marker[cnt].type == FDVR_PLAY_MARKER_AB_REPEAT_IN) ||
                     (appDat->blockMarkerData.marker[cnt].type == FDVR_PLAY_MARKER_AB_REPEAT_OUT)
                   )
                {
                    PVRSYS_PlayRemoveMarker ( MAX_BLOCK_MARKERS,
                                           &(appDat->blockMarkerData.marker[cnt].position) );
                    PVRSYS_PlayGetBlockData(&(appDat->blockMarkerData));
                    /* restart search because marker list has changed */
                    cnt = -1;
                }
            }
            appDat->playStatus.abRepeatMode = AB_REPEAT_MODE_NONE;
            break;
    }
    if ( ferr == FDVR_ERR_OUT_OF_MEMORY )
    {
        MENU_CommonMsgShow(STR_MSG_NOMOREMARKER, FFALSE);
        keyUsed = FFALSE;
    }
    return keyUsed;
}

#endif /* APPL_EDITMARKER_ENABLED */

void MAIN_PvrHandlePlayEvent (PVR_PlayNotifyT event)
{
    PVRSYS_STATUS_S pvrStatus;

    if ( PVRSYS_GetStatus(&pvrStatus) != FAPI_OK )
    {
        return;
    }

    if ( pvrStatus.play.type == PLAY_TYPE_NONE ) return;

    switch ( event )
    {
        case PVR_PLAY_NOTIFY_FILE_ERROR:
        case PVR_PLAY_NOTIFY_END_OF_DATA:
            /* stop any playback (timeshift or normal) */
            (void)MAIN_PvrResumeLiveTv(&pvrStatus);
            break;

        case PVR_PLAY_NOTIFY_BEGIN_OF_DATA:
            /* restart playback in normal speed */
            (void)PVRSYS_PlaySetMode (PLAY_STATUS_NORMAL, 0);
            break;

        default:
            break;
    }
}

void MAIN_PvrHandleRecEvent (PVR_RecordNotifyT event, uint8_t recIdx)
{
    PVRSYS_STATUS_S pvrStatus;
    STR_IDX_S       msgText = STR_EMPTYSTRING;

    switch ( event )
    {
        case PVR_RECORD_NOTIFY_FILE_ERROR:
            msgText = STR_MSG_RECSTOPPED;
            break;

        case PVR_RECORD_NOTIFY_DISK_FULL:
            msgText = STR_MSG_RECSTOPPED_DISKFULL;
            break;

        case PVR_RECORD_NOTIFY_PERFORMANCE_LIMIT:
            msgText = STR_MSG_RECSTOPPED_PERFORMANCE;
            break;

        default:
            break;
    }

    if ( msgText != STR_EMPTYSTRING )
    {
        if ( (recIdx < RSRC_REC_MAX) &&
             (PVRSYS_GetStatus(&pvrStatus) == FAPI_OK) )
        {
            /* Recording engine already stopped: Handle corresponding timers
               and playback operations, inform user, ... */

            /* stop recording */
            (void)MAIN_PvrStopRecording((int8_t)recIdx);

            if ( pvrStatus.rec[recIdx].timerIdx != RECTIMER_INVALID )
            {
                /* remove/re-insert timer */
                (void)RECTIMER_Reinsert(pvrStatus.rec[recIdx].timerIdx);
                MENU_CommonMsgShow(STR_MSG_PLEASEWAIT, FTRUE);

                (void)RECTIMER_Save(FFALSE);

                (void)MAIN_MsgBoxHide(&(appDat->msgBox));
            }

            /* show stop reason as separate message */
            MENU_CommonMsgShow(msgText, FFALSE);
        }
    }
}

int32_t MAIN_PvrStartTimeShiftPlayback (int8_t recIdx,
                                        PVRSYS_TIMESHIFTPOS_E pos,
                                        PVRSYS_PLAY_STATUS_E mode,
                                        int16_t speedIndex)
{
    int32_t           retVal;
    SYS_SERVLOC_S     servLoc;
    SLIST_ITEM_S      service;
    int8_t            audioIdx    = -1;
    AV_SPDIF_STATUS_E spdifStatus = SPDIFSTATUS_CURRPCM;
    int8_t            spdifIdx    = -1;
    SYS_PLAYPREF_S    pref;

    pref.spdifPref = FTRUE;
    pref.langAudio = LANG_NOTSPECIFIED;
    pref.langSpdif = LANG_NOTSPECIFIED;

    /* ----- get audio language from live-TV ------------------------------- */

    retVal = AV_GetRequestedService(&servLoc);
    if ( retVal != FAPI_OK ) return retVal;

    retVal = SLIST_EntryGet (&servLoc, &service);
    if ( retVal != FAPI_OK ) return retVal;

    (void)AV_GetAudioStream (&audioIdx);
    (void)AV_GetSPDIFStatus (&spdifStatus, &spdifIdx);

    if ( (audioIdx >= 0) &&
         (audioIdx <  (int8_t)service.strPid.audio.numStreams) )
    {
        pref.langAudio = service.strPid.audio.stream[audioIdx].langCode;
    }

    if ( spdifStatus == SPDIFSTATUS_PASSTHROUGH )
    {
        pref.spdifMode = SPDIFMODE_PREFERPASSTHROUGH;

        if ( (spdifIdx >= 0) &&
             (spdifIdx <  (int8_t)service.strPid.audio.numStreams) )
        {
            pref.langSpdif = service.strPid.audio.stream[spdifIdx].langCode;
        }
    }
    else
    {
        pref.spdifMode = SPDIFMODE_PCMONLY;
    }

    retVal = PVRSYS_PlayRecStart(recIdx, pos, mode, speedIndex, &pref, NULL);

    return retVal;
}

int32_t MAIN_PvrResumeLiveTv (const PVRSYS_STATUS_S* pvrStatus)
{
    int32_t           retVal    = FAPI_OK;
    int8_t            audioIdx  = -1;
    AV_SPDIF_STATUS_E spdifType = SPDIFSTATUS_DISABLED;
    int8_t            spdifIdx  = -1;
    SYS_SERVLOC_S     servLoc;
    SYS_PLAYPREF_S    playPref;

    switch ( pvrStatus->play.type )
    {
        default:
        case PLAY_TYPE_NONE:
            /* do nothing */
            break;

        case PLAY_TYPE_NORMAL:
            (void)PVRSYS_PlayStop();

            /* resume previous live service */
            (void)AV_RestartService();
            break;

        case PLAY_TYPE_TIMESHIFT:

            /* resume live service but adopt language settings from playback
               stream */

            playPref.spdifPref = FTRUE;
            playPref.langAudio = LANG_NOTSPECIFIED;
            playPref.langSpdif = LANG_NOTSPECIFIED;

            (void)PVRSYS_PlayGetAudioStream (&audioIdx);
            (void)PVRSYS_PlayGetSPDIFStatus (&spdifType, &spdifIdx);

            if ( (audioIdx >= 0) &&
                 (audioIdx <  (int8_t)appDat->playStreamInf.audio.numStreams) )
            {
                playPref.langAudio = appDat->playStreamInf.audio.
                                     stream[audioIdx].langCode;
            }

            if ( (spdifType == SPDIFSTATUS_PASSTHROUGH)
              && (spdifIdx  >= 0)
              && (spdifIdx  <  (int8_t)appDat->playStreamInf.audio.numStreams) )
            {
                playPref.spdifMode = SPDIFMODE_PREFERPASSTHROUGH;
                playPref.langSpdif = appDat->playStreamInf.audio.stream[spdifIdx].
                                     langCode;
            }
            else
            {
                playPref.spdifMode = SPDIFMODE_PCMONLY;
            }

            (void)PVRSYS_PlayStop();

            servLoc.listId = SLIST_NONE;
            servLoc.index  = -1;

            (void)AV_GetRequestedService (&servLoc);

            retVal = AV_SelectService(&servLoc, &playPref, SERVICE_CTRL_NORMAL, FTRUE);
            break;
    }

    return retVal;
}

SYS_DEVICE_E MAIN_PvrGetDefaultRecDevice ()
{
    SYS_DEVICE_E    device = SYS_DEV_NULL;
    SYS_DEV_STAT_S  devStat;

    /* first check for availability of internal HDD */
    if ( SYS_DeviceGetStatus(SYS_DEV_HDD, &devStat) == FAPI_OK )
    {
        if ( (devStat.mountStatus == SYS_DEV_FS_MOUNTED)
          && (PVRSYS_DeviceIsSynced(SYS_DEV_HDD)) )
        {
            device = SYS_DEV_HDD;
        }
    }

    /* if not available, test for USB availability */
    if ( device == SYS_DEV_NULL )
    {
        if ( SYS_DeviceGetStatus(SYS_DEV_USB, &devStat) == FAPI_OK )
        {
            if ( (devStat.mountStatus == SYS_DEV_FS_MOUNTED)
              && (PVRSYS_DeviceIsSynced(SYS_DEV_USB)) )
            {
                device = SYS_DEV_USB;
            }
        }
    }

    return device;
}

#ifdef APPL_EDITMARKER_ENABLED

/*
    this function scans the complete marker list up to the current
    read position. If at current position is already a marker,
    FTRUE is returned. Otherwise FFALSE.
    For cut marker this function evaluates if current position is behind
    an In point. Then pType is converted into Out type
    Be sure that list with marker data is up to date
*/
fbool_t MAIN_PvrCheckMarker( FDVR_PLAY_MARKER_TYPE_E* pType )
{
    PVR_PlayInfoT   playInf;
    uint8_          cnt;
    fbool_t         cutArea;

    cutArea = FFALSE;

    if ( PVRSYS_PlayGetInfo(&playInf) != FAPI_OK )
    {
        return FFALSE;
    }

    PVRSYS_PlayGetBlockData(&(appDat->blockMarkerData));

    for ( cnt = 0 ; cnt < appDat->blockMarkerData.numBlockMarker ; cnt++)
    {
        if ( (playInf.position > appDat->blockMarkerData.marker[cnt].position) )
        {
            if ( appDat->blockMarkerData.marker[cnt].type == FDVR_PLAY_MARKER_JUMP_IN)
                cutArea = FTRUE;
            else if ( appDat->blockMarkerData.marker[cnt].type == FDVR_PLAY_MARKER_JUMP_OUT)
                cutArea = FFALSE;
        }
        else if ( playInf.position == appDat->blockMarkerData.marker[cnt].position)
        {
            return FTRUE;
        }
        else
            break;
    }

    if ( *pType == FDVR_PLAY_MARKER_JUMP_IN && cutArea == FTRUE )
        *pType = FDVR_PLAY_MARKER_JUMP_OUT;
    return FFALSE;
}

/*!
*******************************************************************************
**
** \brief  Jump from current position to previous or next jump index
**
** This function reads the current displayed chunk number (position) and
** scans through the internal index list for the previous or next jump
** position, depending upon the direction passed.
** Jump to next index continues with playback, jump to previous index remains
** in PAUSE mode, otherwise you could only jump to the nearest previous index.
**
** \params direction to jump
** \return No return codes:
**
** \sa
*****************************************************************************
*/
int32_t MAIN_PvrJumpMarker (PVR_BLOCK_MARKER_ITEM_S*  pMarker,
                            uint8_t                   direction )
{
    PVR_PlayInfoT   playInf;
    int32_t         ferr = APPL_SMARTGO_ERR_BAD_PARAMETER;
    int8_t          cnt;

    if (pMarker == NULL)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    if ( pMarker->numBlockMarker > MAX_BLOCK_MARKERS)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    if ( PVRSYS_PlayGetInfo(&playInf) != FAPI_OK ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    if(direction == JUMP_MARKER_NEXT)  /* look for next index to jump to */
    {
        for(cnt = 0; (uint8_t)cnt < pMarker->numBlockMarker ; cnt++)
        {
            if(pMarker->marker[cnt].position > playInf.position)
            {
                PVRSYS_PlayJump (pMarker->marker[cnt].position, PVR_JUMP_DATA_ABS);
                if ( appDat->playStatus.playbackMode == PLAYBACK_MODE_PLAY )
                    PVRSYS_PlaySetMode(PLAY_STATUS_NORMAL,0);
                else
                {
                    PVRSYS_PlaySetMode(PLAY_STATUS_PAUSE,0);
                    appDat->playStatus.playbackMode = PLAYBACK_MODE_PAUSE;
                }
                ferr = FAPI_OK;
                break;
            }
            if ( cnt == ( pMarker->numBlockMarker - 1) )
            {
                PVRSYS_PlayJump (playInf.size - 1, PVR_JUMP_DATA_ABS);
                if ( appDat->playStatus.playbackMode == PLAYBACK_MODE_PLAY )
                    PVRSYS_PlaySetMode(PLAY_STATUS_NORMAL,0);
                else
                {
                    PVRSYS_PlaySetMode(PLAY_STATUS_PAUSE,0);
                    appDat->playStatus.playbackMode = PLAYBACK_MODE_PAUSE;
                }
                ferr = FAPI_OK;
                break;
            }
        }
    }
    else
    {
        for(cnt = (pMarker->numBlockMarker - 1) ; cnt >= 0 ; cnt--)
        {
            if(pMarker->marker[cnt].position < playInf.position)
            {
                PVRSYS_PlayJump (pMarker->marker[cnt].position, PVR_JUMP_DATA_ABS);
                //if ( appDat->playStatus.playbackMode == PLAYBACK_MODE_PLAY )
                //    PVRSYS_PlaySetMode(PLAY_STATUS_NORMAL,0);
                PVRSYS_PlaySetMode(PLAY_STATUS_PAUSE,0);
                appDat->playStatus.playbackMode = PLAYBACK_MODE_PAUSE;
                ferr = FAPI_OK;
                break;
            }
            /* jump to beginning of recording */
            if ( cnt == 0)
            {
                PVRSYS_PlayJump (0, PVR_JUMP_DATA_ABS);
                PVRSYS_PlaySetMode(PLAY_STATUS_PAUSE,0);
                appDat->playStatus.playbackMode = PLAYBACK_MODE_PAUSE;
                ferr = FAPI_OK;
            }
        }
    }

    return ferr;
}

void MAIN_PvrInitMarker ()
{
    PVRSYS_STATUS_S pvrStatus;

    memset (&pvrStatus, 0, sizeof(PVRSYS_STATUS_S));
    memset (&(appDat->blockMarkerData), 0, sizeof(PVR_BLOCK_MARKER_ITEM_S));

    if ( PVRSYS_GetStatus(&pvrStatus) != FAPI_OK )
    {
        return;
    }
    if ( pvrStatus.play.type != PLAY_TYPE_NONE )
    {
        /* now look for existing marker file and enable setting of marker */
        if ( PVRSYS_PlayOpenAndValidateBlockMarker(pvrStatus.play.lstIdx) != FAPI_OK)
        /*
            no block marker file found, reset structure and intitialise. At play stop
            a new block marker file will be created.
        */
        {
            PVRSYS_PlaySetBlockData(&(appDat->blockMarkerData));
        }
        //if ( (ferr = PVR_PlayInitMarker( &(appDat->blockMarkerData.marker[0]),
        //                         ((uint32_t*)&(appDat->blockMarkerData.numBlockMarker)))) == FAPI_OK)
        //{
            PVRSYS_PlayGetBlockData(&(appDat->blockMarkerData));
            PVRSYS_PlayEnableMarker(FTRUE);
            appDat->playStatus.cutModeActive = FTRUE;
        //}
    }
}

void MAIN_PvrGetABRepeatStatus()
{
    uint8_t cnt;

    appDat->playStatus.abRepeatMode = AB_REPEAT_MODE_NONE;

    for ( cnt = 0 ; cnt < appDat->blockMarkerData.numBlockMarker ; cnt++)
    {
        switch ( appDat->blockMarkerData.marker[cnt].type )
        {
            case FDVR_PLAY_MARKER_AB_REPEAT_IN:
                appDat->playStatus.abRepeatMode = AB_REPEAT_MODE_A_POS;
                break;
            case FDVR_PLAY_MARKER_AB_REPEAT_OUT:
                if ( appDat->playStatus.abRepeatMode == AB_REPEAT_MODE_A_POS )
                    appDat->playStatus.abRepeatMode = AB_REPEAT_MODE_B_POS;
                else
                    appDat->playStatus.abRepeatMode = AB_REPEAT_MODE_NONE;
                break;
            default:
                break;
        }

    }
}

#endif /* APPL_EDITMARKER_ENABLED */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
