/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/app_ref/src/app_scaler.c
**
**              $Id: app_scaler.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       This module supports scaleable menus by defining a transform
**              matix and min and max values. The org value in #SCALER_VAL_S
**              defines the size for the given reference resulution.
**
** Detail description. [\ref app_smartgo_app_ref_scaler]
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "app_main.h"
#include <fontserver/fontserver_engine_fbf.h>
#include <fontserver/fontserver_engine_ft.h>

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
typedef struct
{
    SCALER_TRANSFORM_S  transform;                  //!< Currently use transform matrix.
    uint32_t            numVal;                     //!< The number of currently register menu values.
    uint32_t            numPic;                     //!< The number of currently registered icons.
    uint32_t            menuLayer;                  //!< The layer for the menu.
    uint32_t            font[SCALER_FONT_NUMS];     //!< List of font handler
    uint32_t            defaultFontSize;            //!< The default font size for the reference resolution.
    uint8_t             fontSize[SCALER_FONT_NUMS]; //!< The current font size for all fonts;

    SCALER_VAL_S        val[SCALER_VAL_ID_NUMS]; //!< scaler value look up table
    DRAW_PicT           pic[SCALER_PIC_ID_NUMS]; //!< scaler icon look up table

} SCALER_DATA_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC SCALER_DATA_S*  scalerDat    = NULL;
        SCALER_VAL_S*   scalerValPtr = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC int32_t calcWidth   (SCALER_VAL_S *width);
FSTATIC int32_t calcHeight  (SCALER_VAL_S *height);
FSTATIC int32_t calcXPos    (SCALER_VAL_S *xPos);
FSTATIC int32_t calcYPos    (SCALER_VAL_S *yPos);
FSTATIC int32_t reCalc      (void);
FSTATIC int32_t calcFontSize(void);

#ifdef APPL_BMP_SCALER_ENABLED
    FSTATIC int32_t resizePicAll(void);
#endif /* #ifdef APPL_BMP_SCALER_ENABLED */
/*!
*******************************************************************************
**
** \brief   Initialise the Application Scaler unit.
**
** This function initialize all scaler values, fontsizes and pictures and
** calculates the current values base on the given transform matrix. To rescale
** you can change the transform matrix wit SCALER_SetTransformMatrix().
**
** \param   pParams Pointer to the init parameters.
**
** \return  One of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_OUT_OF_MEMORY
**
** \sa
**          - SCALER_Exit()
******************************************************************************/
int32_t SCALER_Init(SCALER_INIT_PARAMS_S * pParams)
{
    int32_t                 retVal = FAPI_OK;
    uint32_t                cnt;
    int32_t                 currId;
    DRAW_PictureFileTypeT   fileType;

    if( scalerDat != NULL ) return FAPI_OK;

    if((pParams == NULL)         ||
       (pParams->valPtr == NULL) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    for(;;)
    {
        scalerDat = UTILS_Malloc(sizeof(SCALER_DATA_S));
        if(scalerDat == NULL)
        {
            retVal = APPL_SMARTGO_ERR_OUT_OF_MEMORY;
            break;
        }

        memset(scalerDat, 0, sizeof(SCALER_DATA_S));

        scalerDat->transform = pParams->transform;
        scalerDat->menuLayer = pParams->menuLayer;
        for(cnt = 0; cnt < (uint32_t)SCALER_VAL_ID_NUMS; cnt++)
        {
            currId = (int32_t)((pParams->valPtr) + cnt)->id;
            /* check for end of list */
            if ( currId == (int32_t)SCALER_VAL_NONE ) break;

            scalerDat->val[currId] = *((pParams->valPtr) + cnt);

            switch ( scalerDat->val[currId].type )
            {
                case SCALER_TYPE_WIDTH:
                    retVal = calcWidth(scalerDat->val + (int32_t)currId);
                    break;
                case SCALER_TYPE_HEIGHT:
                    retVal = calcHeight(scalerDat->val + (int32_t)currId);
                    break;
                case SCALER_TYPE_X_POS:
                    retVal = calcXPos(scalerDat->val + (int32_t)currId);
                    break;
                case SCALER_TYPE_Y_POS:
                    retVal = calcYPos(scalerDat->val + (int32_t)currId);
                    break;
                default:
                case SCALER_TYPE_NUMS:
                    break;
            }

            if(retVal != FAPI_OK) break;
        }
        scalerDat->numVal = cnt;

        scalerDat->defaultFontSize       = pParams->defaultFontSize;
        scalerDat->font[TITLE_FONT]      = pParams->fonts[TITLE_FONT];
        scalerDat->font[MAIN_FONT]       = pParams->fonts[MAIN_FONT];
        scalerDat->font[HELP_FONT]       = pParams->fonts[HELP_FONT];
        scalerDat->font[SYMBOL_FONT]     = pParams->fonts[SYMBOL_FONT];
        scalerDat->font[SYMBOL_TITLE]    = pParams->fonts[SYMBOL_TITLE];

        retVal = calcFontSize();
        if(retVal != FAPI_OK) break;

        if ( pParams->picPtr != NULL )
        {

            for(cnt = 0; cnt < (uint32_t)SCALER_PIC_ID_NUMS; cnt++)
            {
                currId = (int32_t)(((pParams->picPtr) + cnt)->id);
                /* check for end of list */
                if ( currId == (int32_t)ICON_NONE )
                {
                    break;
                }
                retVal = DRAW_DetectFile(&fileType,(uint8_t*)(pParams->picPtr[cnt].ptr));
                if(retVal != FAPI_OK)
                {
                    FAPI_SYS_PRINT_DEBUG(2,"[SCALER] Can't detect file type [id %i]",cnt);
                    break;
                }
                switch(fileType)
                {
                    case DRAW_PIC_FILE_TYPE_BMP:
                        retVal = DRAW_ReadFile_BMP(
                                        ((uint32_t*)(pParams->picPtr[cnt].ptr)),
                                        &(scalerDat->pic[cnt].orgPic));
                        break;
                    case DRAW_PIC_FILE_TYPE_FBM:
                        retVal = DRAW_ReadFile_FBM(
                                        ((uint32_t*)(pParams->picPtr[cnt].ptr)),
                                        &(scalerDat->pic[cnt].orgPic));
                        break;
                    case DRAW_PIC_FILE_TYPE_DAT:
                        retVal = DRAW_ReadFile_DAT(
                                        ((uint32_t*)(pParams->picPtr[cnt].ptr)),
                                        &(scalerDat->pic[cnt].orgPic));

                        break;
                    default:
                    case DRAW_PIC_FILE_TYPE_UNKNOWN:
                        FAPI_SYS_PRINT_DEBUG(2,"[SCALER] Can't read unknown file type [id %i]",cnt);
                        break;
                }

                if ( retVal != FAPI_OK )
                {
                    FAPI_SYS_PRINT_DEBUG(2,"[SCALER] Can't read file type [id %i]",cnt);
                    break;
                }

                scalerDat->pic[cnt].id                   = cnt;
                scalerDat->pic[cnt].orgPic.id            = cnt;
                scalerDat->pic[cnt].orgPic.clutPtr       = NULL;
                scalerDat->pic[cnt].orgPic.clutId        = scalerDat->menuLayer;
                scalerDat->pic[cnt].orgPic.clutStartIdx  = 0;
                scalerDat->pic[cnt].orgPic.clutEndIdx    = 255;
                scalerDat->pic[cnt].orgPic.globalAlpha   = 256;
            }

            scalerDat->numPic = cnt;

            if(retVal != FAPI_OK) break;

            #ifdef APPL_BMP_SCALER_ENABLED

            retVal = resizePicAll();
            if(retVal != FAPI_OK) break;

            #endif /* #ifdef APPL_BMP_SCALER_ENABLED */

            scalerValPtr = scalerDat->val;
        }

        break; /* end of for(;;) */
    }

    if ( retVal != FAPI_OK )
    {
        if ( scalerDat != NULL )
        {
            UTILS_Free(scalerDat);

            scalerDat = NULL;
        }

        scalerValPtr = NULL;
    }

    return retVal;
}


/*!
*******************************************************************************
**
** \brief   Exit the SmartGo-HD scaler module.
**
** All regisred values and pictures are cleaned up and all memory pointers are
** freed.
**
** \return  One of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
** \sa
**          - SCALER_Init()
******************************************************************************/
int32_t SCALER_Exit(void)
{
    SCALER_DATA_S*      inst = scalerDat;
    uint32_t            id;

    if ( scalerDat == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        for(id = 0; id < inst->numPic; id++)
        {
            DRAW_FreeFbm(&(inst->pic[id].orgPic));
            memset(&(inst->pic[id].orgPic), 0, sizeof(DRAW_PicItemT));

            DRAW_FreeFbm(&(inst->pic[id].convPic));
            memset(&(inst->pic[id].convPic), 0, sizeof(DRAW_PicItemT));
        }
        UTILS_Free(scalerDat);

        scalerDat    = NULL;
        scalerValPtr = NULL;

        return FAPI_OK;
    }
}

/*!
*******************************************************************************
**
** \brief   Set the Transform matrix.
**
** If the transform matrix was set successfully all scaler values and fontsizes
** arerecalulated based on the new transform matrix. If APPL_BMP_SCALER_ENABLED
** is enabled in the app_ref/makefile.app also all registered picture are
** scaled.
**
** \param transform    Pointer to Transform matrix.
**
** \return  One of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**          - #APPL_SMARTGO_ERR_BAD_PARAMETER
**
** \sa
**          - SCALER_GetTransformMatrix()
**          - #SCALER_TRANSFORM_S
**
******************************************************************************/
int32_t SCALER_SetTransformMatrix(SCALER_TRANSFORM_S * transform)
{
    SCALER_DATA_S*  inst = scalerDat;
    int32_t         retVal = FAPI_OK;

    if(inst      == NULL) return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if(transform == NULL) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    for(;;)
    {
        /* avoid division by '0' */
        if((transform->sxDenom == 0) ||
           (transform->syDenom == 0) ||
           (transform->txDenom == 0) ||
           (transform->tyDenom == 0) )
        {
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
            break;
        }

        /* check if minimum one parameter has changed */
        if((transform->sxNum == inst->transform.sxNum) &&
           (transform->sxDenom == inst->transform.sxDenom) &&
           (transform->syNum == inst->transform.syNum) &&
           (transform->syDenom == inst->transform.syDenom) &&
           (transform->txNum == inst->transform.txNum) &&
           (transform->txDenom == inst->transform.txDenom) &&
           (transform->tyNum == inst->transform.tyNum) &&
           (transform->tyDenom == inst->transform.tyDenom))
        {
            retVal = FAPI_OK;
            break;
        }

        inst->transform = *transform;

        retVal = reCalc();
        if(retVal != FAPI_OK) break;

        retVal = calcFontSize();
        if(retVal != FAPI_OK) break;

        #ifdef APPL_BMP_SCALER_ENABLED

        retVal = resizePicAll();
        if(retVal != FAPI_OK) break;

        #endif

        break; /* end of for(;;) */
    }

    return (retVal);
}

/*!
*******************************************************************************
**
** \brief   Reads out the current Transformmatrix.
**
** \param   transform   Pointer to Transform matrix.
**
** \return  One of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_BAD_PARAMETER
**
** \sa
**          - SCALER_SetTransformMatrix
**          - #SCALER_TRANSFORM_S
******************************************************************************/
int32_t SCALER_GetTransformMatrix(SCALER_TRANSFORM_S * transform)
{
    SCALER_DATA_S* inst = scalerDat;

    if(inst      == NULL) return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if(transform == NULL) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    *transform = inst->transform;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   Reads out the current Font for given id.
**
** This function returns the font id, which is defined for the current y-scale
** factor. This is only needed for fbf fonts. In case of using ttf fonts, the
** font size is change internaly by the scaler modul, so this function returns
** always the same font id. If you need to modify the font size for different
** scale factors, you can change it.
**
** \param   id  Value Id.
**
** \return  The current Font id or '0'in case of an error.
**
** \sa
**          - #SCALER_FONT_E
**          - SCALER_SetTransformMatrix()
******************************************************************************/
int32_t SCALER_GetFont(SCALER_FONT_E id)
{
    SCALER_DATA_S* inst = scalerDat;

    if( inst == NULL )
    {
        return 0;
    }
    else
    {
        return (int32_t)inst->font[id];
    }
}
/*!
*******************************************************************************
**
** \brief   Reads out the current Font size for given id.
**
** This function returns the font size, which is defined for the current y-scale
** factor. This is only needed for ftf fonts. In case of using ttf fonts, the
** font size is change internaly by the scaler modul, so this function returns
** always the current font size.
**
** \param   id  Value Id.
**
** \return  The current Fontsize or '0' in case of error.
**
** \sa SCALER_FONT_E, SCALER_SetTransformMatrix
******************************************************************************/
int32_t SCALER_GetFontSize(SCALER_FONT_E id)
{
    SCALER_DATA_S* inst = scalerDat;

    if( inst == NULL )
    {
        return 0;
    }
    else
    {
        return inst->fontSize[id];
    }
}

/*!
*******************************************************************************
**
** \brief   Return a pointer to a picture.
**
** If APPL_BMP_SCALER_ENABLED is enabled in the app_ref/makefile.app the scaled
** picture is return, otherwise the original picture will be return.
**
** \param   id   Picture Id.
**
** \return  Pointer to converted fbm file (-array)
**
** \sa
**          - #SCALER_PIC_ID_E
******************************************************************************/
const DRAW_PicItemT *SCALER_GetPic(SCALER_PIC_ID_E id)
{
    SCALER_DATA_S* inst = scalerDat;

    if( (inst == NULL)      ||
        (id   == ICON_NONE) ||
        (id   >= SCALER_PIC_ID_NUMS))
    {
        return NULL;
    }

    #ifdef APPL_BMP_SCALER_ENABLED

    return &(inst->pic[id].convPic);

    #else

    return &(inst->pic[id].orgPic);

    #endif /* #ifdef APPL_BMP_SCALER_ENABLED */
}
/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

/*!
*******************************************************************************
**
** \brief   Calculate new width.
**
** \param   width   Pointer to the scaler width value.
**
** \return  One of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
** \sa
**          - SCALER_SetTransformMatrix()
**          - calcHeight()
**          - calcXPos()
**          - calcYPos()
**
******************************************************************************/
FSTATIC int32_t calcWidth(SCALER_VAL_S * width)
{
    SCALER_DATA_S*  inst = scalerDat;
    uint32_t        temp;

    if(inst == NULL) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    temp = width->org;
    temp *= inst->transform.sxNum;
    temp += (inst->transform.sxDenom / 2);
    temp /= inst->transform.sxDenom;

    if(temp < width->min)
    {
        temp = width->min;
    } else if(temp > width->max)
    {
        temp = width->max;
    }

    width->curr = temp;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   Calculate new height.
**
** \param   height  Pointer to scaler height value.
**
** \return  One of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
** \sa
**          - SCALER_SetTransformMatrix()
**          - calcWidth()
**          - calcXPos()
**          - calcYPos()
**
******************************************************************************/
FSTATIC int32_t calcHeight(SCALER_VAL_S * height)
{
    SCALER_DATA_S*  inst = scalerDat;
    uint32_t        temp;

    if(inst == NULL) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    temp = height->org;
    temp *= inst->transform.syNum;
    temp += (inst->transform.syDenom / 2);
    temp /= inst->transform.syDenom;

    if(temp < height->min)
    {
        temp = height->min;
    } else if(temp > height->max)
    {
        temp = height->max;
    }

    height->curr = temp;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   Calculate new x pos.
**
** \param   xPos    Pointer to scaler x-position value.
**
** \return  One of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
** \sa
**          - SCALER_SetTransformMatrix()
**          - calcWidth()
**          - calcHeight()
**          - calcYPos()
**
******************************************************************************/
FSTATIC int32_t calcXPos(SCALER_VAL_S * xPos)
{
    SCALER_DATA_S* inst = scalerDat;
    uint32_t       temp;

    if(inst == NULL) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    temp = xPos->org;
    temp *= inst->transform.sxNum;
    temp += (inst->transform.sxDenom / 2);
    temp /= inst->transform.sxDenom;

    if(temp < xPos->min)
    {
        temp = xPos->min;
    }
    else if(temp > xPos->max)
    {
        temp = xPos->max;
    }

    xPos->curr = temp;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   Calculate new y pos.
**
** \param   yPos    Pointer to scaler y-position value.
**
** \return  One of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
** \sa
**          - SCALER_SetTransformMatrix()
**          - calcWidth()
**          - calcHeight()
**          - calcXPos()
**
******************************************************************************/
FSTATIC int32_t calcYPos(SCALER_VAL_S * yPos)
{
    SCALER_DATA_S* inst = scalerDat;
    uint32_t       temp;

    if(inst == NULL) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    temp = yPos->org;
    temp *= inst->transform.syNum;
    temp += (inst->transform.syDenom / 2);
    temp /= inst->transform.syDenom;

    if(temp < yPos->min)
    {
        temp = yPos->min;
    }
    else if(temp > yPos->max)
    {
        temp = yPos->max;
    }

    yPos->curr = temp;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   Calculate new pixel position.
**
** \return  One of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
** \sa
**          - SCALER_SetTransformMatrix()
**          - calcWidth()
**          - calcHeight()
**          - calcXPos()
**          - calcYPos()
**
******************************************************************************/
FSTATIC int32_t reCalc(void)
{
    SCALER_DATA_S* inst = scalerDat;
    uint32_t       valId;
    int32_t        retVal = FAPI_OK;

    if(inst == NULL) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    for(valId = 0; valId < inst->numVal; valId++)
    {
        switch (inst->val[valId].type)
        {
            case SCALER_TYPE_WIDTH:
                retVal = calcWidth(inst->val + valId);
                break;
            case SCALER_TYPE_HEIGHT:
                retVal = calcHeight(inst->val + valId);
                break;
            case SCALER_TYPE_X_POS:
                retVal = calcXPos(inst->val + valId);
                break;
            case SCALER_TYPE_Y_POS:
                retVal = calcYPos(inst->val + valId);
                break;

            default:
            case SCALER_TYPE_NUMS:
                break;
        }
    }

    return (retVal);
}

/*!
*******************************************************************************
**
** \brief   Calculate new font size.
**
** \return  One of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
** \sa
**          - #SCALER_INIT_PARAMS_S
**          - SCALER_SetTransformMatrix()
**          - calcWidth()
**          - calcHeight()
**          - calcXPos()
**          - calcYPos()
**
******************************************************************************/
FSTATIC int32_t calcFontSize(void)
{
    SCALER_DATA_S*          inst = scalerDat;
    uint8_t                 fontIndex = 0;
    FONTSERVER_InformationT fontInfo;
    int32_t                 retVal = FAPI_OK;
    uint8_t                 newFontSize;
    SCALER_VAL_S            newHeight;

    if(inst == NULL)   return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    for(;;)
    {
        memset(&fontInfo,  0, sizeof(fontInfo));
        memset(&newHeight, 0, sizeof(newHeight));

        (void)FONTSERVER_SetFont(appDat->fontHdl, fontIndex);

        retVal = FONTSERVER_GetFontInfo(appDat->fontHdl, &fontInfo);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        if ( fontInfo.fontFormat == FONTSERVER_FORMAT_TTF )
        {
            newFontSize = (uint8_t)((inst->defaultFontSize *
                                    ((uint32_t)(inst->transform.syNum))) /
                                               (inst->transform.syDenom));

            newHeight.id   = (SCALER_VAL_ID_E)0;
            newHeight.org  = 4;
            newHeight.min  = 2;
            newHeight.max  = 4;
            newHeight.type = SCALER_TYPE_HEIGHT;

            (void)calcHeight( &newHeight );

            inst->fontSize[TITLE_FONT]   = (uint8_t)(newFontSize + newHeight.curr);
            inst->fontSize[MAIN_FONT]    = newFontSize;
            inst->fontSize[HELP_FONT]    = newFontSize;
            inst->fontSize[SYMBOL_FONT]  = newFontSize;
            inst->fontSize[SYMBOL_TITLE] = (uint8_t)(newFontSize + newHeight.curr);
        }

        break; //end of for (;;)
    }

    return retVal;
}


#ifdef APPL_BMP_SCALER_ENABLED
/*!
*******************************************************************************
**
** \brief   Calculate new Picture sizes and rescale all pictures.
**
** \return  One of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**          - return values from OSDHANDLER_GetClut()
**          - return values form DRAW_ScalePicture
**
** \sa
**          - SCALER_SetTransformMatrix()
**          - calcWidth()
**          - calcHeight()
**          - calcXPos()
**          - calcYPos()
**
******************************************************************************/
FSTATIC int32_t resizePicAll(void)
{
    SCALER_DATA_S*      inst = scalerDat;
    SCALER_VAL_S        newWidth;
    SCALER_VAL_S        newHeight;
    int32_t             retVal = FAPI_OK;
    uint32_t            id;

    if(inst == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    for(id = 0; id < inst->numPic; id++)
    {
        newWidth.id   = MAIN_ICON_WIDTH;
        newWidth.org  = inst->pic[id].orgPic.fbmHeaderPtr->width;
        newWidth.min  = newWidth.org>>1;
        newWidth.max  = newWidth.org<<1;
        newWidth.curr = 0;
        newWidth.type = SCALER_TYPE_WIDTH;

        newWidth.id    = MAIN_ICON_HEIGHT;
        newHeight.org  = inst->pic[id].orgPic.fbmHeaderPtr->height;
        newHeight.min  = newHeight.org>>1;
        newHeight.max  = newHeight.org<<1;
        newHeight.curr = 0;
        newHeight.type = SCALER_TYPE_HEIGHT;

        retVal = calcWidth ( &newWidth );
        if(retVal != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(2,"[SCALER] calcWidth() failed %ld\n",retVal);
            break;
        }
        retVal = calcHeight( &newHeight );
        if(retVal != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(2,"[SCALER] calcHeight() failed %ld\n",retVal);
            break;
        }

        DRAW_FreeFbm(&(inst->pic[id].convPic));
        memset(&(inst->pic[id].convPic), 0, sizeof(DRAW_PicItemT));
        inst->pic[id].orgPic.clutEndIdx  = 215;
        inst->pic[id].orgPic.clutId      = OSD_LAYER_MENU;
        if( ( inst->pic[id].orgPic.fbmHeaderPtr->height == newHeight.curr) &&
            ( inst->pic[id].orgPic.fbmHeaderPtr->width == newWidth.curr) )
        {
            memcpy(&(inst->pic[id].convPic),&(inst->pic[id].orgPic),sizeof(DRAW_PicItemT));
            inst->pic[id].convPic.allocInformation = 0;
        }
        else
        {
            inst->pic[id].convPic.clutEndIdx = 215;
            inst->pic[id].convPic.clutId     = OSD_LAYER_MENU;
            retVal = DRAW_ScalePicture( &(inst->pic[id].orgPic),
                                        &(inst->pic[id].convPic),
                                        newWidth.curr,
                                        newHeight.curr);
            if(retVal != FAPI_OK)
            {
                FAPI_SYS_PRINT_DEBUG(2,"[SCALER] DRAW_ScalePicture() failed %ld\n",retVal);
                //break;
            }
        }
    }
    return (retVal);
}
#endif /* #ifdef APPL_BMP_SCALER_ENABLED */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
