/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/inc/av.h
**
**              $Id: av.h 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Internal audio and video control functions.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
*/
#ifndef _AV_H_
#define _AV_H_

//#include <fapi/board_ifc.h>

#include <val/val.h>

#include "sys_priv.h"
#include "init.h"

#ifdef APPL_MHEG_ENABLED
#include "mheg.h"
#endif /* APPL_MHEG_ENABLED */

/* ------------------------------------------------------------------------- */

/* MPEG-2 descriptor tags */
#define CA_DESCRIPTOR                            0x09
#define LANGUAGE_DESCRIPTOR                      0x0A
#define STREAM_IDENTIFIER_DESCRIPTOR             0x52

/* ETSI descriptor tags */
#define SAT_DEL_SYSTEM_DESCRIPTOR                0x43
#define CAB_DEL_SYSTEM_DESCRIPTOR                0x44
#define SERVICE_DESCRIPTOR                       0x48
#define SHORT_EVENT_DESCRIPTOR                   0x4D
#define EXT_EVENT_DESCRIPTOR                     0x4E
#define STREAM_IDENTIFIER_DESCRIPTOR             0x52
#define CA_IDENTIFIER_DESCRIPTOR                 0x53
#define CONTENT_DESCRIPTOR                       0x54
#define PARENTAL_RATING_DESCRIPTOR               0x55
#define TELETEXT_DESCRIPTOR                      0x56
#define LOCAL_TIME_OFFSET_DESCRIPTOR             0x58
#define SUBTITLING_DESCRIPTOR                    0x59
#define TER_DEL_SYSTEM_DESCRIPTOR                0x5A
#define PRIVATE_DATA_SPECIFIER_DESCRIPTOR        0x5F
#define DATA_BROADCAST_ID_DESCRIPTOR             0x66
#define AC_3_DESCRIPTOR                          0x6A
#define DDPDCV_DESCRIPTOR                        0x7A

/* UK-DTT private descriptors */
#define LOGICAL_CHANNEL_DESCRIPTOR               0x83
#define SERVICE_ATTRIBUTE_DESCRIPTOR             0x86
#define SERVICE_LIST_DESCRIPTOR                  0x41
#define HD_SIMULCAST_LOGICAL_CHANNEL_DESCRIPTOR  0x88

/* DVB stream types */
#define STRTYPE_VIDEO_MPEG1                      0x01
#define STRTYPE_VIDEO_MPEG2                      0x02
#define STRTYPE_VIDEO_H264                       0x1B
#define STRTYPE_VIDEO_VC1                        0xEA

#define STRTYPE_AUDIO_MPEG1                      0x03
#define STRTYPE_AUDIO_MPEG2                      0x04
#define STRTYPE_AUDIO_AC3_ATSC_A52               0x81
#define STRTYPE_AUDIO_AAC_MPEG2                  0x0F
#define STRTYPE_AUDIO_AAC_MPEG4                  0x11

#define STRTYPE_PRIVDATA                         0x06

/* MPEG2-PSI constants */
#define CRC_LEN                 4       //!< CRC field length in bytes

#define PAT_SECTSIZE_MAX        1024    //!< max. size of one PAT section

#define PAT_SECTSIZE_MIN        12      //!< Min. size of one PAT section
#define NIT_SECTSIZE_MIN        16      //!< Min. size of one NIT section
#define PMT_SECTSIZE_MIN        12      //!< Min. size of one PMT section
#define SDT_SECTSIZE_MIN        15      //!< Min. size of one SDT section
#define CAT_SECTSIZE_MIN        12      //!< Min. size of one CAT section

#define AV_USESTATUS_LIVETV     0x01
#define AV_USESTATUS_REC_TSD2   0x02
#define AV_USESTATUS_REC_TSD3   0x04
#define AV_USESTATUS_PIP        0x08

/* Mute request sources, stored in bitfield and handled in parallel */
#define AV_MUTEREQ_USER         0x01    //!< Mute requested by user.
#define AV_MUTEREQ_SYNC         0x02    //!< Mute requested by synced service switch.
#define AV_MUTEREQ_PARENTAL     0x04    //!< Mute requested by parental control.
#define AV_MUTEREQ_MHEG         0x08    //!< Mute requested by MHEG-5 engine.

#define AV_PID_NONE             0xFFFF

#define AV_VIDEO_USERDATA_MAX   8

#define FAPI_AUOUT_MAX_VOL      0xFE
#define AUDIO_TSD_CHANNEL       8

/* ------------------------------------------------------------------------- */

typedef enum
{
    AV_STATE_SLEEP = 0,
    AV_STATE_ACTIVE

} AV_STATE_E;

typedef enum                   /* Possible states for each frontend */
{
    AV_FESTATE_IDLE = 0,    //!< No operation.
    AV_FESTATE_NORMAL,      //!< Any combination of live-TV and first/second
                            //!< recording or simply tuning with no operation.
    AV_FESTATE_SCANNING,    //!< Used exclusively for scanning.
    AV_FESTATE_OAD,         //!< Used exclusively for Oad search.

} AV_FE_STATE_E;

typedef enum
{
    AV_SCANSTATE_START = 0,           //!< Scan state: Start.
    AV_SCANSTATE_GETCARRIERS,         //!< Scan state: perform NIT scan.
    AV_SCANSTATE_GETSERVICES,         //!< Scan state: Parse PAT/PMT/SDT for services.
    AV_SCANSTATE_FINISHED             //!< Scan state: Finished.

} AV_SCANSTATE_E;

typedef enum
{
    AV_SCANTP_TUNE = 0,
    AV_SCANTP_PATSDT,
    AV_SCANTP_PMTSTART,
    AV_SCANTP_PMTWAIT,
    AV_SCANTP_FINISHED

} AV_SCANTPSTATE_E;

typedef enum
{
    AV_SCANNIT_TUNE = 0,
    AV_SCANNIT_NIT,
    AV_SCANNIT_PARSE,
    AV_SCANNIT_FINISHED

} AV_SCANNITSTATE_E;

typedef struct
{
    SYS_NETWORK_S       network;        //!< target network
    fbool_t             manualMode;      //!< FTRUE: carrier index is invalid!
    FE_TUNE_PARAMS_S    par;    //!< Tune parameters.
    TPLIST_IDX          tpIdx;  //!< Non-manual-mode carrier index

} AV_TUNEPARS_S;

typedef struct
{
    int8_t              feIdx;  // own index in frontend array
    FE_SYSTEM_TYPE_E    systemType;
    FAPI_SYS_HandleT    drvHdl;

    #ifdef APPL_USALS_ENABLED
    FAPI_SYS_HandleT    usalsHdl;
    #endif /* APPL_USALS_ENABLED */

    AV_FE_STATE_E       state;
    SYS_FE_LOCK_E       tuneLockStatus;
    uint16_t            sigStrengthCurr;
    uint16_t            sigQualiCurr;
    uint32_t            sigLastUpdateTime;      // last update timestamp
    uint32_t            lastLockCheck;  // last lock-checking timestamp

    #ifdef APPL_FESAT_ENABLED
    fbool_t             autoMotorMode;
    #endif /* APPL_FESAT_ENABLED */

    uint8_t             useStatus;

    /* current tune parameters */
    AV_TUNEPARS_S       currTunePar;
    AV_TUNEPARS_S       reqTunePar;

    fbool_t             forceLnbSettsUpdate;    //!< forces update of LNB settings
    fbool_t             forceTuning;    // forces tuning on next service change

    /* Scan parameters */
    SYS_SCAN_MODE_E     scanMode;
    AV_SCANSTATE_E      scanState;
    AV_SCANTPSTATE_E    scanTpState;
    AV_SCANNITSTATE_E   scanNitState;
    fbool_t             scanProgressSent;
    uint64_t            scanTimeStamp;
    uint16_t            sdtLoopPos;
    SLIST_ITEM_S        scanService;
    uint16_t            scanNumTs;
    int32_t             scanNetworkId;
    uint16_t            scanTsCnt;
    int32_t             scanTsId;
    uint16_t            scanPrgIdx;
    fbool_t             scanSdtInfoAvl;
    fbool_t             scanSdtStarted;

    AV_SCAN_OPTS_S      scanOpts;
    SYS_NETWORK_S       scanNetwork;

    uint8_t             scanTpDone[TPLIST_CARRIERS_MAX];
    // TODO: to be optimised: use bitfield instead of bytes!
    uint16_t            scanNumTpScanned;
    TPLIST_IDX          scanIdxNext;

    SYS_EVT_SCAN_S      scanEvent;

    /* Data stored from NIT during scan */
    PSISI_NIT_TP_INFO_S nitTpInfo;

    /* filtering parameters */
    PSISI_PAT_DATA_S    pat;

} AV_FEDAT_S;

typedef struct
{
    uint16_t            adPid;
    fbool_t             adOverride;
    uint8_t             adType;
    uint16_t            vdPid;
    fbool_t             vdOverride;
    uint8_t             vdType;
    uint16_t            pcrPid;
    fbool_t             pcrOverride;

} AV_PID_OVERRIDE_S;

typedef struct
{
    AV_FEDAT_S         *pFe;
    RSRC_CFG_S          cfg;
    fbool_t             streamsStarted; //!<
    uint32_t            streamsStartTime;
    fbool_t             videoFreezed;   //!< Video is currently freeze
    fbool_t             serviceDecoded; //!< FTRUE if streams can be decoded.
    fbool_t             haveVideoStream;        //!< FTRUE if status of
    //!< 'serviceDecoded' is based on audio
    //!< decoding status (used for radio
    //!< services only).
    fbool_t            timeoutSent;     //!< Stores sending status of decoding
    //!< status event.

    SYS_SERVLOC_S       reqServLoc;     // requested service: not necessarily tuned
    SYS_SERVLOC_S       quietServLoc;   // requested quiet service

    uint8_t             currVersionPat; //!< Version of last PAT filtered.
    uint8_t             currVersionPmt; //!< Version of last PMT filtered.
    uint8_t             currVersionSdt; //!< Version of last SDT filtered.
    uint8_t             currVersionNit; //!< Version of last NIT filtered.
    uint8_t             currVersionBat; //!< Version of last BAT filtered. //Newly added...Ravi
    SYS_RUNSTAT_E       currRunstat;    //!< Current service running status.

    /* selected service */
    AV_PID_OVERRIDE_S   pidOverride;    //!< curr PID if != selected PID
    SYS_SERVLOC_S       currServLoc;    // loc. of currently selected service
    SLIST_SERVIDX       currServIdx;    // index of currently selected service
    SLIST_ITEM_S        currentService; // current service
    SYS_PLAYPREF_S      pref;   //!< Stream preferences
    int8_t              selectedAudioStream;    // index of selected I2S audio
                                                // stream
    int8_t              currentAudioStream;     // index of currently presented
                                                // I2S audio stream
    int8_t              selectedSpdifStream;    // index of selected SPDIF stream
    int8_t              currentSpdifStream;     // index of currently presented
                                                // SPDIF stream
    fbool_t             radioScreenVisible;

    /* previously selected services */
    SYS_SERVLOC_S       selServLoc;     // loc. of last/current selected service
    SYS_SERVLOC_S       prevServLoc;    // previously selected service
    SYS_SERVLOC_S       swapServLoc;    // prev. selected service: swappable
    int16_t             prevIdxGlob[SLIST_GLOB_NUMLISTS];       // prev. service for each list
    int16_t             prevIdxUsr[SLIST_USR_NUMLISTS]; // prev. service for each list
    uint32_t            selectionTime;  // selection timestamp of current service
    fbool_t             swapFlag;

    uint32_t            VBV_Readpointer;
    uint32_t            VBV_PTS;
    fbool_t             ptsSynced;

} AV_LIVETV_S;

#ifdef APPL_PVR_ENABLED

typedef struct
{
    fbool_t             active;
    AV_FEDAT_S*         pFe;
    RSRC_CFG_S          cfg;
    SLIST_ITEM_S        service;
    uint8_t             currVersionPat; //!< Version of last PAT filtered.
    uint8_t             currVersionPmt; //!< Version of last PMT filtered.

} AV_REC_S;

typedef struct /* used by both playback and descramble-recording */
{
    fbool_t             active;         //!< Play or descr. rec. is active
    fbool_t             internal;       //!< 0: std. playback, 1:
                                        //!< descr. recording
    RSRC_CFG_S          cfg;            //!< Resource configuration.
    PSISI_PAT_DATA_S    pat;            //!< PAT data.
    uint8_t             currVersionPat; //!< Version of last PAT filtered.
    uint8_t             currVersionPmt; //!< Version of last PMT filtered.

} AV_PLAY_S;

#endif /* APPL_PVR_ENABLED */

typedef struct
{
    VAL_DataT*          valDataPtr;
    VAL_OpenT*          valOpenPtr;

    SYS_HDTV_STANDARD   outHdSysCurr;       //!< Current HDTV output Standard.
    SYS_SDTV_STANDARD   outSdSysCurr;       //!< Current SDTV output Standard.
    SYS_PICFORMAT_E     picFormat;          //!< Current picture format.
    SYS_TV_STANDARD     inSysCurr;          //!< Current video input system.

    fbool_t             videoFullScreen;    //!< FTRUE: Video in fullscreen
                                            //!< mode.
    fbool_t             bootScreenActive;
    fbool_t             firstFrameWait;     //!< if FTRUE, no frame has been
                                            //!< decoded since wakeup.
    VAL_DecoderTypeT    freezeDecoder;      //!< Decoder information stored
                                            //!< when stopped in freeze mode
                                            //!< -> helpful for PVR timeshift.
    FAPEX_VISCALE_ParamChangeInfoT          //!< video scaler parameters
                        viscaleParam;
    fbool_t             haveFrameRate;      //!< the flag to check if the frame
                                            //!< rate of the current stream
                                            //!< is available
    fbool_t             hdmiConnected;      //!< HDMI connection status.
    fbool_t             hdmiHdcpEnabled;    //!< enable HDCP
    VAL_HdmiHdcpStateT  hdmiHdcpState;      //!< HDCP state of HDMI

    uint16_t            currPid;

} AV_VIDEO_S;

typedef struct
{
    uint8_t             lvlLeft;        //!< Left channel output volume.
    uint8_t             lvlRight;       //!< Right channel output volume.

    int8_t              balance;        //!< left right balance
    AV_STEREO_MODE_E    stereoMode;     //!< Current stereo mode.
    uint8_t             reqChannel;     //!< Requested audio channel if audio
                                        //!< stream is dual mono, one of
                                        //!< SYS_AUDIOCHAN_XXX.
    void                (*startNotification) (void);
    fbool_t             muteByUser;     //!< 1: Muted by user.
    uint8_t             muteStatus;     //!< Bitfield holding current mute req.

    FAPI_SYS_HandleT    audioHandle;

    /* list of audio formats compliant for SPDIF passthrough mode, sorted in
       order of preference. */
    uint8_t             spdifFormat[SYS_AUDIO_NUMS];
    uint8_t             spdifFormatNums;

    uint16_t            currPid;

} AV_AUDIO_S;

typedef struct
{
    fbool_t             appSelected;
    fbool_t             isDvb;
    SYS_LANGCODE        langCode;
    uint8_t             type;
    SUBTT_DISPLAY_E     displayType;

} AV_SUBTT_S;

typedef struct
{
    /* Resources */
    ITC_INST_S                      itcInst;
    uint8_t                         descBuf[4192];  /* 4K buffer for descriptors should be enough */

    /* parameters modified by av_api only (user thread!) */
    SYS_SERVICE_STAT_E              liveStatus;
    SYS_RUNSTAT_E                   runningStatus;

    fbool_t                         sleepMode;
    AV_STATE_E                      threadState;

    /* section filtering parameters */
    PSISI_SDT_S                     sdt;
    PSISI_BAT_S                     bat;

    #if (defined APPL_MHEG_ENABLED) || (defined APPL_OAD_ENABLED)

    fbool_t                         dsmccPatReq;
    fbool_t                         dsmccPmtReq;
    uint16_t                        dsmccPmtPrgNum;
    uint16_t                        dsmccPmtPid;
    uint8_t                         dsmccLastPmtVersion;

    #endif /* #if (defined APPL_MHEG_ENABLED) || (defined APPL_OAD_ENABLED) */

    /* Parameters */
    fbool_t                         outputFormatUpdateOn;
    SYS_VIDEO_OUT                   videoOutCurr;       //!< Current video output setting.
    FAPEX_VISCALE_AspectRatioEnumT   hdmiPictureAr;      //!< hdmi picture aspect ratio
    uint8_t                         hdmiActiveFormat;   //!< hdmi active format
    AV_WSS_E                        wssMode;            //!< current WSS bit setting
    fbool_t                         wssOverride;

    fbool_t             hdmiLipSyncSet;
    int32_t             avLipSyncDelay;     //!< the A/V delay value set either
                                            //!< from EDID info or from user
                                            //!< manually

    AV_FEDAT_S          feDat[RSRC_FE_MAX];     //!< parameters of each frontend
    AV_LIVETV_S         liveTv; //!< live-TV related parameters

    #ifdef APPL_PVR_ENABLED
    AV_PLAY_S                       play;               //!< PVR-playback related params.
    AV_REC_S                        rec[RSRC_REC_MAX];  //!< PVR-record related params.
    #endif /* APPL_PVR_ENABLED */

    /* intra-thread scheduling parameters */
    uint64_t                        nextSlotStamp[RSRC_FE_MAX + 1]; /* for each FE + Playback */

    AV_AUDIO_S                      audio;
    AV_VIDEO_S                      video;

    /* welcome audio jingle parameters */
    fbool_t                         jingleReq;
    uint64_t                        jingleStartStamp;

    //FAPI_BOARD_ScartSbInputT scartInputStatus;
    uint64_t            scartInputLastCheck;

    AV_SUBTT_S                      subtt;

#ifdef APPL_BLINDSCAN_ENABLED
    int8_t              scanBlindFe;
    uint32_t            scanBlindProgLvl;
#endif

} AV_DATA_S;

/* ------------------------------------------------------------------------- */

typedef enum
{
    AV_CMD_NOP = 0,             //!< NOP
    AV_CMD_WAKEUP,              //!< Activate AV operation.
    AV_CMD_GOTOSLEEP,           //!< Deactivate AV operation.
    AV_CMD_SELECT_SERVICE,      //!< Select live-TV service.
    AV_CMD_RESTART_SERVICE,     //!< Restart previous live service.
    AV_CMD_SELECT_AUDIOSTREAM,  //!< Select I2S audio stream for live-TV.
    AV_CMD_SELECT_SPDIFSTREAM,  //!< Select SPDIF stream for live-TV.
    AV_CMD_SELECT_SUBTTSTREAM,  //!< Select Subtitle stream
    AV_CMD_SET_STEREOMODE,      //!< Set stereo mode option for dual channel.
    AV_CMD_STOP_SERVICE,        //!< Stop live-TV service.
    AV_CMD_SCAN_CARRIER,        //!< Scan single carrier.
    AV_CMD_SCAN_LIST,           //!< Scan carrier list.
    AV_CMD_SCAN_INPUT,          //!< Scan current input TS.
    AV_CMD_SCAN_BLIND,          //!< Scan a frequency area for carriers.
    AV_CMD_STOP_SCAN,           //!< Stop scan.
    AV_CMD_REC_START,           //!< Prepare recording.
    AV_CMD_REC_STOP,            //!< Stop recording.
    AV_CMD_REC_REROUTE,         //!< Reroute recording's input path.
    AV_CMD_PLAY_START,          //!< Start playback.
    AV_CMD_PLAY_STOP,           //!< Stop playback.
    AV_CMD_DESCRREC_START,      //!< Prepare descramble-recording.
    AV_CMD_DESCRREC_STOP,       //!< Stop descramble-recording.
    AV_CMD_DESCRREC_REROUTE,    //!< Reroute descr.-recording's input path.
    AV_CMD_TUNEMODE,            //!< Start tunemode.
    AV_CMD_STOP_TUNEMODE,       //!< Stop tunemode.
    AV_CMD_DIS12_DRIVEMOTOR,    //!< DiSEqC 1.2: Move motor.
    AV_CMD_DIS12_HALTMOTOR,     //!< DiSEqC 1.2: Halt motor.
    AV_CMD_DIS12_RECALCPOS,     //!< DiSEqC 1.2: Recalculation of sat pos.
    AV_CMD_DIS12_STOREPOS,      //!< DiSEqC 1.2: Store current motor position.
    AV_CMD_DIS12_GOTOPOS,       //!< DiSEqC 1.2: Move motor to stored pos.
    AV_CMD_DIS12_SETLIMITWEST,  //!< DiSEqC 1.2: Store pos. as eastern limit.
    AV_CMD_DIS12_SETLIMITEAST,  //!< DiSEqC 1.2: Store pos. as western limit.
    AV_CMD_DIS12_CLEARLIMITS,   //!< DiSEqC 1.2: Clear positioning limits.
    AV_CMD_DIS12_GOTOZEROPOS,   //!< DiSEqC 1.2: Move to reference position.
    AV_CMD_USALS_GOTOPOS,       //!< USALS: Move motor to stored position.
    AV_CMD_USALS_GOTOZERO,      //!< USALS: Move motor to zero.
    AV_CMD_PIP_SELECT,          //!< Select a service for PIP presentation.
    AV_CMD_PIP_STOP,            //!< Stop PIP presentation.
    AV_CMD_SET_ANTENNAPOWER,    //!< Switch on/off antenna power.
    AV_CMD_SET_OUTPUT_SYSTEM,   //!< Set output TV system.
    AV_CMD_SET_VIDEOAREA,       //!< Set video position and size.
    AV_CMD_ZOOM_VIDEO,          //!< Zoom a cutout area to current video area.
    AV_CMD_VIDEO_FREEZE,        //!< Freeze current video frame.
    AV_CMD_VIDEO_IFRAME_SHOW,   //!< Shows IFrame.
    AV_CMD_VIDEO_IFRAME_HIDE,   //!< Hides IFrame.
    AV_CMD_SET_AUTOMOTORMODE,   //!< Control automatic motor mode.
    AV_CMD_MHEG_VIDEO,          //!<
    AV_CMD_MHEG_AUDIO,          //!<
    AV_CMD_MHEG_CONTROL,        //!<
    AV_CMD_MHEG_DEMUX,          //!<
    AV_CMD_START_OAD,           //!<
    AV_CMD_STOP_OAD,            //!<

} AV_CMDTYPE_E;

typedef struct
{
    SYS_SERVLOC_S       servLoc;
    SYS_PLAYPREF_S      pref;
    RSRC_CFG_S          cfg;
    uint32_t            ctrlInfo;

} AV_CMD_ARG_SEL_S;

typedef struct
{
    SYS_NETWORK_S       network;        //!< Network to be scanned.
    RSRC_CFG_S          cfg;    //!< Resource configuration parameters.
    uint16_t            tpLstPos;       //!< Index in network's carrier list.
    AV_SCAN_OPTS_S      opts;   //!< Scan options.

} AV_CMD_ARG_SCAN_S;

#ifdef APPL_PVR_ENABLED

typedef struct
{
    RSRC_CFG_S          cfg;    //!< Resource configuration parameters.

} AV_CMD_ARG_REC_S;

typedef struct
{
    PVRLST_IDX          lstIdx; //!< recording ID.
    SLIST_SERVIDX       servIdx;        //!< Service ID, for timeshift playback only!
    RSRC_CFG_S          cfg;    //!< Resource configuration parameters.

} AV_CMD_ARG_PLAY_S;

#endif /* APPL_PVR_ENABLED */

typedef struct
{
    AV_STEREO_MODE_E    mode;

} AV_CMD_ARG_STEREOMODE_S;

typedef struct
{
    fbool_t             freeze;
    fbool_t             sendNotification;
    fbool_t             resetFocus;
    fbool_t             keepRadioScreen;
    uint32_t            ctrlInfo;

} AV_CMD_ARG_SERVSTOP_S;

typedef struct
{
    int8_t              index;
    AV_SPDIF_STATUS_E   type;

} AV_CMD_ARG_AUDIO_S;

typedef struct
{
    int8_t  index; //!< subtitle index
    fbool_t isDvb;

} AV_CMD_ARG_SUBTT_S;

typedef struct
{
    SYS_NETWORK_S       network;        //!< Target network.
    RSRC_CFG_S          cfg;    //!< Resource configuration parameters.
    TPLIST_IDX          tpIdx;  //!< Target carrier in network (optional)
    TUNE_ITEM_S         tunePars;       //!< Alternative manual tune parameters.
    //!< (valid if tpIdx < 0)
} AV_CMD_ARG_TUNE_S;

typedef struct
{
    int8_t              feIdx;

} AV_CMD_ARG_ABORT_S;

typedef struct
{
    int8_t              feIdx;
    fbool_t             power;

} AV_CMD_ARG_ANTPWR_S;

typedef struct
{
    SYS_HDTV_STANDARD     hdSystem;
    SYS_SDTV_STANDARD     sdSystem;

} AV_CMD_ARG_OUTSYS_S;

typedef struct
{
    fbool_t             fullScreen;
    AV_VO_PICTURE_WINDOW_S area;

} AV_CMD_ARG_VAREA_S;

typedef struct
{
    int8_t              feIdx;
    fbool_t             enable;

} AV_CMD_ARG_MOTORMODE_S;

#ifdef APPL_DISEQC12_ENABLED

typedef struct
{
    int8_t              feIdx;
    FE_WE_FLAG_E        westEastFlag;
    uint8_t             numSteps;

} AV_CMD_ARG_DIS12MOVE_S;

typedef struct
{
    int8_t              feIdx;
    uint8_t             satPos;

} AV_CMD_ARG_DIS12IDX_S;

#endif /* APPL_DISEQC12_ENABLED */

#ifdef APPL_USALS_ENABLED

typedef struct
{
    int8_t              feIdx;
    SYS_ORBPOS_S        orbPos;

} AV_CMD_ARG_USALSPOS_S;

#endif /* APPL_USALS_ENABLED */

#ifdef APPL_OAD_ENABLED

typedef struct
{
    SYS_NETWORK_S       network;        //!< Network to be scanned.
    RSRC_CFG_S          cfg;    //!< Resource configuration parameters.
    int8_t                feIdx;

} AV_CMD_ARG_OAD_S;

#endif /* APPL_OAD_ENABLED */

typedef struct
{
    SYS_SERVLOC_S       servLoc;
    RSRC_CFG_S          cfg;

} AV_CMD_ARG_PIP_S;

typedef struct
{
    fbool_t             freeze;

} AV_CMD_ARG_VIDEOFREEZE_S;

typedef struct
{
    const uint32_t*     data;
          uint32_t      size;
          uint32_t      format;


} AV_CMD_ARG_VIDEOIFRAME_S;

typedef union
{
    AV_CMD_ARG_SEL_S            servsel;
    AV_CMD_ARG_SERVSTOP_S       servstop;
    AV_CMD_ARG_SCAN_S           scan;
    AV_CMD_ARG_AUDIO_S          audio;
    AV_CMD_ARG_SUBTT_S          subtt;
    AV_CMD_ARG_TUNE_S           tune;
    AV_CMD_ARG_ABORT_S          abort;
    AV_CMD_ARG_ANTPWR_S         antpwr;
    AV_CMD_ARG_OUTSYS_S         outsys;
    AV_CMD_ARG_VAREA_S          varea;
    AV_CMD_ARG_STEREOMODE_S     stereo;
    AV_CMD_ARG_VIDEOFREEZE_S    videoFreeze;
    AV_CMD_ARG_VIDEOIFRAME_S    videoIframe;

    #ifdef APPL_PVR_ENABLED
    AV_CMD_ARG_REC_S    record;
    AV_CMD_ARG_PLAY_S   play;
    #endif                          /* APPL_PVR_ENABLED */

    #ifdef APPL_FESAT_ENABLED
    AV_CMD_ARG_MOTORMODE_S motor;
    #endif                          /* APPL_FESAT_ENABLED */

    #ifdef APPL_DISEQC12_ENABLED
    AV_CMD_ARG_DIS12IDX_S dis12idx;
    AV_CMD_ARG_DIS12MOVE_S dis12move;
    #endif                          /* APPL_DISEQC12_ENABLED */

    #ifdef APPL_USALS_ENABLED
    AV_CMD_ARG_USALSPOS_S usalspos;
    #endif                          /* APPL_USALS_ENABLED */

    #ifdef APPL_OAD_ENABLED
    AV_CMD_ARG_OAD_S    oad;
    #endif                          /* APPL_OAD_ENABLED */

    #ifdef APPL_MHEG_ENABLED
    AV_CMD_MHEG_VD_S    mhegVd;
    AV_CMD_MHEG_AD_S    mhegAd;
    #endif                          /* APPL_MHEG_ENABLED */

    #ifdef APPL_MHEG_ENABLED
    AV_CMD_MHEG_DX_S    mhegDx;
    #endif                          /* APPL_MHEG_ENABLED */

    #ifdef APPL_PIP_ENABLED
    AV_CMD_ARG_PIP_S    pip;
    #endif                          /* APPL_PIP_ENABLED */

} AV_CMD_ARG_S;

typedef struct
{
    AV_CMDTYPE_E        type;
    AV_CMD_ARG_S        arg;

} AV_CMD_S;

/* ------------------------------------------------------------------------- */

extern AV_DATA_S   *avData;

#ifdef __cplusplus
extern "C"
{
#endif

int32_t             AV_Init(void);
int32_t             AV_Wakeup(void);
int32_t             AV_Shutdown(void);
int32_t             AV_PowerupScreenShow(fbool_t openVideo);
int32_t             AV_SetTvOutputSystem(SYS_HDTV_STANDARD hdSystem,
                                         SYS_SDTV_STANDARD sdSystem);
int32_t             AV_CmdSend(AV_CMD_S * pCmd);
int32_t             AV_CheckArgs(int8_t feIdx);
void                AV_SetVideoOutput(SYS_VIDEO_OUT videoOut);
int32_t             AV_SetAntennaPower(int8_t feIdx, fbool_t powerOn);
void                AV_OutputFormatUpdate(fbool_t force);

int32_t             AVINT_Tune(AV_FEDAT_S * pFe,
                               const AV_TUNEPARS_S * tunePars,
                               fbool_t abortable);
fbool_t             AVINT_TuningIsNecessary(AV_FEDAT_S * pFe,
                                            const AV_TUNEPARS_S * tunePars);
void                AVINT_RouteSignal(const RSRC_CFG_S * cfg);

void                AVINT_UpdateSignalParams(AV_FEDAT_S * pFe);
void                AVINT_OutputSystemUpdate(SYS_HDTV_STANDARD reqHdSystem,
                                             SYS_SDTV_STANDARD reqSdSystem,
                                             fbool_t force);
int32_t             AVINT_OutputTvStandardGet (VAL_HdEncoderT* hdSys,
                                               VAL_SdEncoderT* sdSys );
int32_t             AVINT_SetVideoArea(fbool_t fullScreen,
                                       fbool_t useDtgScallingFactors,
                                       const AV_VO_PICTURE_WINDOW_S * pos);
int32_t             AVINT_ZoomVideo(fbool_t full,
                                    fbool_t useDtgScallingFactors,
                                    const AV_VO_PICTURE_WINDOW_S * pos);

fbool_t             AV_SleepAndCheckAbortion(int32_t timeInMs,
                                             fbool_t abortable);
void                AV_ForceTuning(int8_t feIdx);
void                AV_ForceLnbSettingsUpdate(int8_t feIdx);
void                AV_UpdateTPParams(AV_FEDAT_S * pFe,
                                      const SYS_NETWORK_S *
                                      network, TPLIST_IDX carrierIdx);
void                AV_UpdateLiveStatus(SYS_SERVICE_STAT_E status);
void                AV_UpdateRunningStatus(SYS_RUNSTAT_E runStat);
void                AV_PrintSchedule(void); /* debug function */

int32_t             AV_ScanAddSdtInfos(const PSISI_SDT_SERVICE_S *
                                       sdtInfo, SLIST_ITEM_S * pService);

#ifdef APPL_FESAT_ENABLED

fbool_t             decodeSatDescriptor(uint8_t * pBuf,
                                        uint16_t len,
                                        FE_TUNE_PARAMS_S * pTunepar);

#endif                          /* APPL_FESAT_ENABLED */

#ifdef APPL_FETERR_ENABLED

fbool_t             decodeTerrDescriptor(uint8_t * pBuf,
                                         uint16_t len,
                                         FE_TUNE_PARAMS_S * pTunepar);

#endif                          /* APPL_FETERR_ENABLED */

#ifdef APPL_FECABLE_ENABLED

fbool_t             decodeCableDescriptor(uint8_t * pBuf,
                                          uint16_t len,
                                          FE_TUNE_PARAMS_S * pTunepar);

#endif                          /* APPL_FECABLE_ENABLED */

int32_t             AVINT_VideoInit  (void);
int32_t             AVINT_VideoOpen  (void);
int32_t             AVINT_VideoClose (void);
int32_t             AVINT_VideoTask  (void);
int32_t             AVINT_VideoSetOutputMode
                                     (SYS_HDTV_STANDARD hdOutput,
                                      SYS_SDTV_STANDARD sdOutput);
int32_t             AVINT_VideoStart (VAL_EnvironmentT environment,
                                      uint16_t pid, uint8_t streamType,
                                      VAL_PlayModeEnumT playMode,
                                      void (*videoStartCb)(void* optData),
                                      void* videoStartCbArg);
int32_t             AVINT_VideoStop  (fbool_t freezeVideo);
int32_t             AVINT_VideoBlank (fbool_t blank);
int32_t             AVINT_VideoFreeze(fbool_t freeze);
int32_t             AVINT_VideoIFrameShow (const uint32_t* data,
                                           uint32_t sizeInBytes,
                                           uint32_t format);
int32_t             AVINT_VideoGetStreamInfo (AV_VIDEO_STREAMINFO_S* info);
int32_t             AVINT_VideoGetFrameInfo  (VAL_VIDEC_DecFrameInfoStrT* info);
int32_t             AVINT_VideoSetAspectRatio(VAL_VideoOutputT output,
                          FAPEX_VISCALE_AspectRatioEnumT aspectRatio);
int32_t             AVINT_VideoGetAspectRatio (VAL_VideoOutputT output,
                          FAPEX_VISCALE_AspectRatioEnumT* aspectRatio);
int32_t             AVINT_VideoSetScaleMethod(
                          FAPEX_VISCALE_ScalingMethodEnumT newScaleMethod);
int32_t             AVINT_VideoGetScaleMethod(
                          FAPEX_VISCALE_ScalingMethodEnumT* scaleMethod);

int32_t             AVINT_AudioInit    (void);
int32_t             AVINT_AudioWakeup  (void);
int32_t             AVINT_AudioShutdown(void);
int32_t             AVINT_AudioOpen    (void);
int32_t             AVINT_AudioClose   (void);
int32_t             AVINT_AudioTask    (void);
void                AVINT_AudioMuteRequestSet(uint8_t req);
void                AVINT_AudioMuteRequestClear(uint8_t req);
int32_t             AVINT_AudioStart (uint16_t pid, uint8_t format,
                                      uint8_t reqChannel,
                                      void (*startNotification)(void));
void                AVINT_AudioStop  (void);
int32_t             AVINT_AudioPause (fbool_t enterPause);
int32_t             AVINT_AudioVolumeSet(uint8_t lvlLeft, uint8_t lvlRight);
void                AVINT_AudioSetStereoMode(AV_STEREO_MODE_E mode);
int32_t             AVINT_AudioGetStreamInfo(AV_AUDIO_STREAMINFO_S* info);
int32_t             AVINT_AudioSPDIFSetStatus(AV_SPDIF_STATUS_E type,
                                              uint16_t pid, uint8_t format);
int8_t              AVINT_AudioSPDIFFindFirst(SYS_STREAMINF_AUDIO_S* pStrAudio);
fbool_t             AVINT_AudioIsSPDIFCompliant(uint8_t format);
int8_t              AVINT_AudioChooseStream(const SYS_STREAMINF_AUDIO_S*
                                            pStrAudio, uint8_t format,
                                            SYS_LANGCODE directPref,
                                            fbool_t preferDecodable);
int8_t              AVINT_AudioChooseSPDIFStream(const SYS_STREAMINF_AUDIO_S*
                                                 pStrAudio,
                                                 SYS_LANGCODE directPref);
int32_t             AVINT_HDMIGetEDIDLipSyncDelay (int32_t* delay);
int32_t             AVINT_HDMISetLipSyncDelay( int32_t delay );
int32_t             AVINT_HDMIAutoSync(void);

int32_t             AVINT_LiveTvInit(void);
int32_t             AVINT_LiveTvSelAudioStream(int8_t index);
int32_t             AVINT_LiveTvSelSpdifStream(AV_SPDIF_STATUS_E
                                               type, int8_t index);
void                AVINT_LiveTvSignalAvailable(AV_FEDAT_S * pFe);
void                AVINT_LiveTvTasks(AV_FEDAT_S * pFe);
void                AVINT_LiveTvSignalLost(AV_FEDAT_S * pFe);
int32_t             AVINT_LiveTvSelectService(const SYS_SERVLOC_S* servLoc,
                                              const RSRC_CFG_S *cfg,
                                              const SYS_PLAYPREF_S* pref,
                                              uint32_t ctrlInfo);
int32_t             AVINT_LiveTvRestart(const SYS_SERVLOC_S *
                                        servLoc, const RSRC_CFG_S * cfg);
int32_t             AVINT_LiveTvStop(fbool_t freeze,
                                     fbool_t sendNotification,
                                     fbool_t resetFocus,
                                     fbool_t keepRadioScreen,
                                     uint32_t crtlInfo);
void                AVINT_LiveTvVideoStartNotify(void* optData);
void                AVINT_LiveTvAudioStartNotify(void);


int32_t             AVINT_StopStreams( fbool_t videoFreeze,
                                       fbool_t keepRadioScreen,
                                       fbool_t handleVideo,
                                       fbool_t handleAudio);
int32_t             AVINT_StartStreams(const SLIST_ITEM_S * pService,
                                       fbool_t handleVideo,
                                       fbool_t handleAudio);
#ifdef APPL_DISEQC12_ENABLED

int32_t             AV_Dis12GotoSatPosition(int8_t feIdx, uint8_t satPos);
int32_t             AV_Dis12StoreSatPosition(int8_t feIdx, uint8_t satPos);
int32_t             AV_Dis12DriveMotor(int8_t feIdx,
                                       FE_WE_FLAG_E westEastFlag,
                                       uint8_t numSteps);
int32_t             AV_Dis12HaltMotor(int8_t feIdx);
int32_t             AV_Dis12ReCalcPos(int8_t feIdx);
int32_t             AV_Dis12SetLimitWest(int8_t feIdx);
int32_t             AV_Dis12SetLimitEast(int8_t feIdx);
int32_t             AV_Dis12ClearLimits(int8_t feIdx);
int32_t             AV_Dis12GotoZeroPosition(int8_t feIdx);

#endif                          /* APPL_DISEQC12_ENABLED */

#ifdef APPL_USALS_ENABLED

int32_t             AV_UsalsGotoSatPosition(int8_t feIdx,
                                            const SYS_ORBPOS_S * orbPos);
int32_t             AV_UsalsGotoZero       (int8_t feIdx);

#endif /* APPL_USALS_ENABLED */

void                AVINT_StartScanOperation(AV_FEDAT_S * pFe,
                                             AV_CMD_S * pCmd);
void                AVINT_AbortScanOperation(AV_FEDAT_S * pFe);
uint32_t            AV_TaskScan(AV_FEDAT_S * pFe);

int32_t             AV_SetSystemParamsSat (int8_t feIdx, CFGSAT_IDX satIdx,
                                           FE_SYSTEM_PARAMS_S* pSys);
int32_t             AVINT_SetTsInput(uint8_t tsd, RSRC_TS_E ts);
void                AVINT_ScartHandleOutputSource(void);

#ifdef APPL_PVR_ENABLED

int32_t             AV_RecordStart(const RSRC_CFG_S * rsrcCfg);
int32_t             AV_RecordStop(uint8_t recIdx);
int32_t             AV_RecordReroute(const RSRC_CFG_S * rsrcCfg);
int32_t             AV_PlayStart(PVRLST_IDX lstIdx,
                                 SLIST_SERVIDX servIdx,
                                 const RSRC_CFG_S * rsrcCfg);
int32_t             AV_PlayStop(void);

int32_t             AVINT_PlayRecInit(void);

int32_t             AVINT_RecStart(const RSRC_CFG_S * cfg);
int32_t             AVINT_RecStop(uint8_t tsdIdx);
int32_t             AVINT_RecReroute(const RSRC_CFG_S * cfg);
void                AVINT_RecSignalAvailable(uint8_t tsdIdx);
void                AVINT_RecSignalLost(uint8_t tsdIdx);
void                AVINT_RecTasks(uint8_t tsdIdx);

void                AVINT_PlayStart(PVRLST_IDX lstIdx,
                                    SLIST_SERVIDX servIdx,
                                    const RSRC_CFG_S * cfg);
void                AVINT_PlayStop(void);
uint32_t            AVINT_PlayTasks(void);


#if defined(APPL_CI_ENABLED) && defined(APPL_DESCRREC_ENABLED)

int32_t             AV_DescrRecordStart(const RSRC_CFG_S * cfg);
int32_t             AV_DescrRecordStop(void);
int32_t             AV_DescrRecordReroute(const RSRC_CFG_S * cfg);
int32_t             AVINT_DescrRecStart(const RSRC_CFG_S * cfg);
int32_t             AVINT_DescrRecStop(void);
int32_t             AVINT_DescrRecReroute(const RSRC_CFG_S * cfg);
uint32_t            AVINT_DescrRecTasks(void);

#endif /* defined(APPL_CI_ENABLED) && defined(APPL_DESCRREC_ENABLED) */

#endif                          /* APPL_PVR_ENABLED */

#ifdef APPL_PIP_ENABLED

int32_t             AVINT_PIPInit(void);
int32_t             AVINT_PIPSelect(const SYS_SERVLOC_S * servLoc,
                                    const RSRC_CFG_S * cfg);
int32_t             AVINT_PIPStop(void);

void                AVINT_PIPSignalAvailable(AV_FEDAT_S * pFe);
void                AVINT_PIPTasks(AV_FEDAT_S * pFe);
void                AVINT_PIPSignalLost(AV_FEDAT_S * pFe);

#endif                          /* APPL_PIP_ENABLED */

#ifdef APPL_MHEG_ENABLED

int32_t             AVINT_MHEG_Init(void);

#endif /* APPL_MHEG_ENABLED */

#ifdef APPL_OAD_ENABLED

int32_t             AVINT_OAD_Start(AV_FEDAT_S * pFe, AV_CMD_S * pCmd);
int32_t             AVINT_OAD_Stop(AV_FEDAT_S * pFe);
uint32_t            AV_TaskOad(AV_FEDAT_S * pFe);
fbool_t             OAD_AbortReq(void);

#endif /* APPL_OAD_ENABLED */

#ifdef APPL_MPEG_ENCODER_ENABLED

int32_t             AVINT_MpencMonitorStart    (const RSRC_CFG_S* cfg);
int32_t             AVINT_MpencMonitorStop     (void);

int32_t             AVINT_MpencEncoderStart    (RSRC_OP op);
int32_t             AVINT_MpencEncoderStop     (RSRC_OP op);

#ifdef APPL_PVR_ENABLED

int32_t             AVINT_RecMpencStart        (const RSRC_CFG_S* cfg);
int32_t             AVINT_RecMpencStop         (uint8_t tsdIdx);

#endif /* APPL_PVR_ENABLED */

#endif /* APPL_MPEG_ENCODER_ENABLED */

#if (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED)

int32_t             AVINT_SubtSelectStream(int8_t index,
                                           fbool_t isDvb, fbool_t appSelected);

#endif /* (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED) */

void                AVINT_SubtResetUserSelect(void);
void                AVINT_SubtPmtUpdate(void);

#ifdef APPL_HDCP_ENABLED

int32_t             AVINT_DynamicEnableHDCP   (fbool_t enable);

#endif /* APPL_HDCP_ENABLED */

#ifdef __cplusplus
}
#endif

#endif /* _AV_H_ */

/******************************************************************************/
/* END OF FILE */
