/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/av_api.c
**
**              $Id: av_api.c 5881 2010-03-03 08:04:49Z jmorha $
**
** \brief       Public functions of AV module.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save  */
#define DEBUG 1

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <fapi/sys_driver.h>         /* global type definitions */

#include <aal/aal.h>
#include <val/val.h>
#include "config.h"
#include "utils.h"
#include "sys_api.h"
#include "sys_priv.h"

#include "av.h"
#include "event_queue.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/



/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC int32_t     AV_ScanGetFrontend(const SYS_NETWORK_S * network,
                                       RSRC_CFG_S * rsrcCfg);

int32_t AV_GetLiveStatus(AV_STATUS_LIVE_S * status)
{
    RSRC_STATUS_S*  rsrcStatus;
    int32_t         retVal = FAPI_OK;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        status->status        = avData->liveStatus;
        status->runningStatus = avData->runningStatus;
        status->feIdx =       -1;

        switch (avData->liveStatus)
        {
            case SERVICE_STAT_START:
            case SERVICE_STAT_SUCCESS:
            case SERVICE_STAT_DECODED:

                rsrcStatus = RSRC_GetStatus();

                if((rsrcStatus->opStatus & RSRC_OP_LIVE) != 0)
                {
                    status->feIdx = rsrcStatus->live.cfg.feIdx;
                }
                break;

            case SERVICE_STAT_STOPPED:
                status->feIdx = -1;
                break;

            default:
                break;
        }
    }

    return retVal;
}

void AV_UpdateLiveStatus(SYS_SERVICE_STAT_E status)
{
    DBG_Assert(avData != NULL);

    avData->liveStatus = status;
}

void AV_UpdateRunningStatus(SYS_RUNSTAT_E runStat)
{
    DBG_Assert(avData != NULL);

    avData->runningStatus = runStat;
}

/*!
*******************************************************************************
**
** \brief  Select next service in current service list.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa AV_SelectPreviousService, AV_SelectService
******************************************************************************/
int32_t AV_SelectNextService(void)
{
    int32_t         retVal = SMARTGO_SYS_ERR_PROHIBITED;
    uint16_t        listSize;
    SYS_SERVLOC_S   servLoc;
    int16_t         currIdx;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        listSize = SLIST_ListSizeGet(avData->liveTv.reqServLoc.listId);
        if ( listSize < 2 )
        {
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
        else
        {
            servLoc = avData->liveTv.reqServLoc;
            currIdx = avData->liveTv.reqServLoc.index;

            do
            {
                servLoc.index = (servLoc.index + 1) % (int16_t) listSize;

                /* Abort after complete list has been tried */
                if(servLoc.index == currIdx)
                    break;

                retVal = AV_SelectService(&servLoc, NULL, SERVICE_CTRL_NORMAL, FFALSE);
            }
            while ( retVal != FAPI_OK );
        }
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Select previous service in current service list.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa AV_SelectNextService, AV_SelectService
******************************************************************************/
int32_t AV_SelectPreviousService(void)
{
    int32_t         retVal = SMARTGO_SYS_ERR_PROHIBITED;
    uint16_t        listSize;
    SYS_SERVLOC_S   servLoc;
    int16_t         currIdx;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        listSize = SLIST_ListSizeGet(avData->liveTv.reqServLoc.listId);
        if ( listSize < 2 )
        {
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
        else
        {
            servLoc = avData->liveTv.reqServLoc;
            currIdx = avData->liveTv.reqServLoc.index;

            do
            {
                servLoc.index--;
                if(servLoc.index < 0)
                    servLoc.index = ((int16_t)listSize) - 1;

                /* Abort after complete list has been tried */
                if(servLoc.index == currIdx)
                    break;

                retVal = AV_SelectService(&servLoc, NULL, SERVICE_CTRL_NORMAL, FFALSE);
            }
            while ( retVal != FAPI_OK );
        }
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Swap to previously selected service.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa AV_SelectService
******************************************************************************/
int32_t AV_SwapService(void)
{
    int32_t         retVal;
    SYS_SERVLOC_S   servLoc;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        /* Use local copy because swapServLoc is modified in
           AV_SelectService()! */
        servLoc = avData->liveTv.swapServLoc;

        avData->liveTv.swapFlag = FTRUE;

        retVal = AV_SelectService(&servLoc, NULL, SERVICE_CTRL_NORMAL, FFALSE);

        avData->liveTv.swapFlag = FFALSE;
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Start a service for live-TV presentation.
**
** \param  pServLoc        Pointer to requested service.
** \param  pPref           Optional stream preferences
**                         (set to NULL if not needed).
** \param ctrlInfo         FFALSE: "quiet" selection requested
**                         (e.g front panel control will be omitted).
** \param forceSelection   FTRUE: Allow selection even if service is marked
**                         as non-selectable by service attribute descriptor
**                         (-> SLIST_FLAG_NONSELECT is set).
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
******************************************************************************/
int32_t AV_SelectService(const SYS_SERVLOC_S * pServLoc,
                         const SYS_PLAYPREF_S * pPref,
                         uint32_t ctrlInfo, fbool_t forceSelection)
{
    AV_CMD_S            cmd;
    SYS_EVT_SERVICE_S   ctrlEvent;
    int32_t             retVal = FAPI_OK;
    uint32_t            selTime;
    RSRC_CFG_S          rsrcCfg;
    SLIST_ITEM_S        servItem;
    fbool_t             freezeStatus;
    RSRC_OP             ops = RSRC_OP_LIVE | RSRC_OP_PLAY
                                           | RSRC_OP_MPENCMONITOR;

    FAPI_SYS_PRINT_DEBUG(1, "[av_api][AV_SelectService] enter\n");

    if ( avData == NULL )
    {
        FAPI_SYS_PRINT_DEBUG(1, "[av_api][AV_SelectService] avData == NULL\n");
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    retVal = SLIST_EntryGet(pServLoc, &servItem);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[av_api][AV_SelectService] SLIST_EntryGet() %s\n", SYS_GetErrorString(retVal));


        return retVal;
    }

    FAPI_SYS_PRINT_DEBUG(1, "[av_api][AV_SelectService] name(%s), v(%d,%d),a(%d,%d)\n",\
                            servItem.name,\
                            servItem.strPid.video,\
                            servItem.strPid.videoType,\
                            servItem.strPid.audio.stream[0].pid,\
                            servItem.strPid.audio.stream[0].type);

    if(SLIST_FlagIsSet(&servItem, SLIST_FLAG_NONSELECT) && (!forceSelection))
    {
        FAPI_SYS_PRINT_DEBUG(1, "[av_api][AV_SelectService] SMARTGO_SYS_ERR_PROHIBITED\n");
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    retVal = RSRC_ServiceAvlForLive(SLIST_GetServiceIndex(pServLoc),
                                    RSRC_OP_ANY & (~ops),
                                    NULL, &rsrcCfg);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[av_api][AV_SelectService] RSRC_ServiceAvlForLive() %s\n", SYS_GetErrorString(retVal));
        return retVal;
    }

    #ifdef APPL_CI_ENABLED

    if((!(rsrcCfg.useCi)) && ((rsrcCfg.ciMatch[RSRC_TS_A]) ||
                              (rsrcCfg.ciMatch[RSRC_TS_B])))
    {
        retVal = CISYS_GetLiveCfgByPrio(SLIST_GetServiceIndex(pServLoc),
                                        rsrcCfg.ciMatch[RSRC_TS_A] ? RSRC_TS_A
                                        : RSRC_TS_B, &rsrcCfg);
        if(retVal != FAPI_OK)
        {
            /* No rerouting available, choose default resource configuration. */

            retVal = RSRC_ServiceAvlForLive(SLIST_GetServiceIndex(pServLoc),
                                            RSRC_OP_ANY & (~ops),
                                            NULL, &rsrcCfg);
            if(retVal != FAPI_OK)
                return retVal;
        }
    }

    #endif /* APPL_CI_ENABLED */


    if(ctrlInfo == SERVICE_CTRL_NORMAL)
        avData->liveTv.reqServLoc = *pServLoc;

    avData->liveTv.quietServLoc = *pServLoc;

    #ifdef APPL_MHEG_ENABLED
    if(MHEG_ServiceGetFlags() == 0)
    {
        (void) MHEG_Stop();
    }
    #endif /* APPL_MHEG_ENABLED */

    /* ---------------------------------------------------------------------- */

    retVal = TIME_GetSystemTime(&selTime);      /* get selection timestamp */
    if(retVal != FAPI_OK)
        return retVal;

    /* update swap entry if current service has been active long enough */

    if((avData->liveTv.swapFlag) ||
       (TIME_GetSystemTimeInterval(avData->liveTv.selectionTime, selTime) >=
        AV_MIN_SWAP_INTERVAL))
    {
        avData->liveTv.swapServLoc = avData->liveTv.reqServLoc;
    }

    avData->liveTv.selectionTime = selTime;

    /* ---------------------------------------------------------------------- */
    (void)SYS_StopPresentation (RSRC_OP_PLAY | RSRC_OP_MPENCMONITOR, FFALSE);

    retVal = RSRC_LiveServiceStopped();
    if ( (retVal != FAPI_OK)
     && (retVal != SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE) )
    {
        FAPI_SYS_PRINT_DEBUG(1, "[av_api][AV_SelectService] RSRC_LiveServiceStopped() %s\n", SYS_GetErrorString(retVal));
        return retVal;
    }

    retVal = SETTINGS_GetServiceSwitchMode(&freezeStatus);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[av_api][AV_SelectService] SETTINGS_GetServiceSwitchMode() %s\n", SYS_GetErrorString(retVal));
        return retVal;
    }

    /* stop current live presentation */
    cmd.type                          = AV_CMD_STOP_SERVICE;
    cmd.arg.servstop.sendNotification = FFALSE;
    cmd.arg.servstop.resetFocus       = FFALSE;
    cmd.arg.servstop.freeze           = freezeStatus;
    cmd.arg.servstop.ctrlInfo         = ctrlInfo;

    /* Avoid hiding the radio background picture when switching between radio
       channels. */
    if((SYS_GetConfig()->radioScreen != NULL) &&
       (SLIST_EntryGet(&(avData->liveTv.reqServLoc), &servItem) == FAPI_OK) &&
       (servItem.strPid.video == 0) &&
       (SLIST_EntryGet(pServLoc, &servItem) == FAPI_OK) &&
       (servItem.strPid.video == 0))
    {
        cmd.arg.servstop.keepRadioScreen = FTRUE;
    }
    else
    {
        cmd.arg.servstop.keepRadioScreen = FFALSE;
    }

    retVal = AV_CmdSend(&cmd);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[av_api][AV_SelectService] AV_CmdSend(AV_CMD_STOP_SERVICE) %s\n", SYS_GetErrorString(retVal));
        return retVal;
    }

    /* update resource parameters */
    retVal = RSRC_SetLiveService(&rsrcCfg);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[av_api][AV_SelectService] RSRC_SetLiveService() %s\n", SYS_GetErrorString(retVal));
        return retVal;
    }

    /* ---------------------------------------------------------------------- */

    cmd.type = AV_CMD_SELECT_SERVICE;
    cmd.arg.servsel.servLoc = *pServLoc;
    cmd.arg.servsel.cfg = rsrcCfg;
    cmd.arg.servsel.ctrlInfo = ctrlInfo;

    if ( pPref != NULL )
    {
        cmd.arg.servsel.pref = *pPref;
    }
    else
    {
        cmd.arg.servsel.pref.spdifPref = FFALSE;
        cmd.arg.servsel.pref.spdifMode = SPDIFMODE_PREFERPASSTHROUGH;
        cmd.arg.servsel.pref.langAudio = LANG_NOTSPECIFIED;
        cmd.arg.servsel.pref.langSpdif = LANG_NOTSPECIFIED;
    }

    retVal = AV_CmdSend(&cmd);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[av_api][AV_SelectService] AV_CmdSend(AV_CMD_SELECT_SERVICE) %s\n", SYS_GetErrorString(retVal));
        return retVal;
    }
    avData->runningStatus = SYS_RUNSTAT_UNKNOWN;

    /* Generate event to notify start of service selection */
    ctrlEvent.hdr.type = EVT_SERVICE_SELECTED;
    ctrlEvent.hdr.length = sizeof(ctrlEvent);
    ctrlEvent.servLoc = *pServLoc;
    ctrlEvent.audioStr = 0; // invalid for this message
    ctrlEvent.status = SERVICE_STAT_START;
    ctrlEvent.ctrlInfo = ctrlInfo;

    retVal = EVT_Send(&ctrlEvent, "Service select");
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[av_api][AV_SelectService] EVT_Send() %s\n", SYS_GetErrorString(retVal));
        return retVal;
    }
    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Use one frontend in "tune mode".
**
** Tuning is available in manual mode where custom parameters are passed as well
** as in auto mode where an already-known carrier in requested network is
** referenced.
**
** \param feIdx    Requested frontend. If set to -1, frontend will be chosen
**                 automatically. If >= 0, function will fail if requested
**                 frontend is not available.
** \param network  Requested network to be accessed.
** \param tpLstPos Requested carrier in network (set to -1 for manual mode).
** \param tunePars Tune parameters for manual mode (ignored if tpLstPos >= 0).
** \param selFeIdx If function does not return an error code, index of selected
**                 frontend is returned here.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa AV_TuneModeStop, AV_TuneModeStartByCarrier
******************************************************************************/
int32_t AV_TuneModeStart(int8_t feIdx, const SYS_NETWORK_S * network,
                         int16_t tpLstPos, const TUNE_ITEM_S * tunePars,
                         int8_t * selFeIdx)
{
    AV_CMD_S            cmd;
    int32_t             retVal;
    RSRC_CFG_S          rsrcCfg;

    retVal = RSRC_NetworkAvlForTunemode(feIdx, network, RSRC_OP_ANY, &rsrcCfg);
    if(retVal != FAPI_OK)
        return retVal;

    /* ---------------------------------------------------------------------- */

    memset(&cmd, 0, sizeof(cmd));

    /* send "tune" command */
    cmd.type = AV_CMD_TUNEMODE;
    cmd.arg.tune.network = *network;
    cmd.arg.tune.cfg = rsrcCfg;

    if(tunePars != NULL)
    {
        cmd.arg.tune.tunePars = *tunePars;
    }

    if(tpLstPos >= 0)
    {
        retVal = TPLIST_GetIndex(network, tpLstPos, &(cmd.arg.tune.tpIdx));
        if(retVal != FAPI_OK)
            return retVal;
    }
    else
    {
        cmd.arg.tune.tpIdx = -1;
    }

    retVal = RSRC_SetTunemode(&rsrcCfg);
    if(retVal != FAPI_OK)
        return retVal;

    if(selFeIdx != NULL)
        *selFeIdx = rsrcCfg.feIdx;

    return AV_CmdSend(&cmd);
}

/*!
*******************************************************************************
**
** \brief  Use one frontend in non-manual "tune mode".
**
** \param feIdx    Requested frontend. If set to -1, frontend will be chosen
**                 automatically. If >= 0, function will fail if requested
**                 frontend is not available.
** \param network  Requested network to be accessed.
** \param tpIdx    Requested carrier in network.
** \param selFeIdx If function does not return an error code, index of selected
**                 frontend is returned here.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa AV_TuneModeStop AV_TuneModeStart
******************************************************************************/
int32_t AV_TuneModeStartByCarrier(int8_t feIdx, const SYS_NETWORK_S * network,
                                  TPLIST_IDX tpIdx, int8_t * selFeIdx)
{
    AV_CMD_S            cmd;
    int32_t             retVal;
    RSRC_CFG_S          rsrcCfg;

    if(tpIdx < 0)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    retVal = RSRC_NetworkAvlForTunemode(feIdx, network, RSRC_OP_ANY, &rsrcCfg);
    if(retVal != FAPI_OK)
        return retVal;

    /* ---------------------------------------------------------------------- */

    memset(&cmd, 0, sizeof(cmd));

    /* send "tune" command */
    cmd.type = AV_CMD_TUNEMODE;
    cmd.arg.tune.network = *network;
    cmd.arg.tune.cfg = rsrcCfg;
    cmd.arg.tune.tpIdx = tpIdx;

    retVal = RSRC_SetTunemode(&rsrcCfg);
    if(retVal != FAPI_OK)
        return retVal;

    if(selFeIdx != NULL)
        *selFeIdx = rsrcCfg.feIdx;

    return AV_CmdSend(&cmd);
}

int32_t AV_TuneModeStop(int8_t feIdx)
{
    AV_CMD_S            cmd;
    int32_t             retVal;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
    {
        return retVal;
    }

    #ifdef APPL_DISEQC12_ENABLED

    (void)DIS12_HaltMotor (feIdx);

    #endif
    retVal = RSRC_TunemodeStopped(feIdx);
    if((retVal != FAPI_OK) && (retVal != SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE))
    {
        return retVal;
    }

    cmd.type = AV_CMD_STOP_TUNEMODE;
    cmd.arg.abort.feIdx = feIdx;

    return AV_CmdSend(&cmd);
}

/*!
*******************************************************************************
**
** \brief  Returns index of the currently selected I2S audio stream.
**
** \return
**         - -1:    no stream has been selected yet
**         - 0...n: index in current service's audio stream list
**
** \sa AV_SetAudioStream
******************************************************************************/
int32_t AV_GetAudioStream(int8_t * index)
{
    int32_t retVal;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        switch (avData->liveStatus)
        {
            default:
            case SERVICE_STAT_START:
            case SERVICE_STAT_STOPPED:
                *index = -1;
                break;

            case SERVICE_STAT_SUCCESS:
            case SERVICE_STAT_DECODED:
                *index = avData->liveTv.selectedAudioStream;
                break;
        }

        retVal = FAPI_OK;
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Select an audio stream to decoded for I2S output.
**
** \attention  Only audio streams for I2S output may be selected by this
**             function. Use AV_SetSPDIFStatus() for selecting a stream for
**             the SDIF output.
**
** \param  index  Index of audio stream in current service's list.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa AV_SetSPDIFStatus
******************************************************************************/
int32_t AV_SetAudioStream(int8_t index)
{
    AV_CMD_S    cmd;
    int32_t     retVal;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else if ( avData->liveStatus == SERVICE_STAT_STOPPED )
    {
        retVal = SMARTGO_SYS_ERR_PROHIBITED;
    }
    else
    {
        cmd.type            = AV_CMD_SELECT_AUDIOSTREAM;
        cmd.arg.audio.index = index;
        /* NOTE: type argument is not used for I2S audio streams */

        retVal = AV_CmdSend(&cmd);
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Select an audio stream for the SPDIF output.
**
** \attention  There are two options for the SPDIF output: Either the currently
**             decoded audio stream is redirected as PCM stream or a compressed
**             stream is output in passthrough mode to be decoded by an
**             external device.
**
** \param  type   output type: curr. stream's PCM or an compressed stream
** \param  index  index of stream in list of current service
**                (parameter is only used for type #SPDIFSTATUS_PASSTHROUGH)
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa AV_SetAudioStream
******************************************************************************/
int32_t AV_SetSPDIFStatus(AV_SPDIF_STATUS_E type, int8_t index)
{
    AV_CMD_S    cmd;
    int32_t     retVal;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else if ( avData->liveStatus == SERVICE_STAT_STOPPED )
    {
        retVal = SMARTGO_SYS_ERR_PROHIBITED;
    }
    else
    {
        cmd.type            = AV_CMD_SELECT_SPDIFSTREAM;
        cmd.arg.audio.type  = type;
        cmd.arg.audio.index = index;

        retVal = AV_CmdSend(&cmd);
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Returns the current status of SPDIF output.
**
** For #SERVICE_STAT_START and #SERVICE_STAT_STOPPED the type is set to
** #SPDIFSTATUS_DISABLED and the index is set to -1.
**
** For #SERVICE_STAT_SUCCESS and #SERVICE_STAT_DECODED the settings depend
** on if an SPDIF stream is selected.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
******************************************************************************/
int32_t AV_GetSPDIFStatus(AV_SPDIF_STATUS_E* type, int8_t* index)
{
    int32_t retVal;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        switch ( avData->liveStatus )
        {
            default:
            case SERVICE_STAT_START:
            case SERVICE_STAT_STOPPED:
                *type  = SPDIFSTATUS_DISABLED;
                *index = -1;
                break;

            case SERVICE_STAT_SUCCESS:
            case SERVICE_STAT_DECODED:
                if ( avData->liveTv.selectedSpdifStream < 0 )
                {
                    if ( avData->liveTv.selectedAudioStream < 0 )
                    {
                        *type = SPDIFSTATUS_DISABLED;
                        *index = -1;
                    }
                    else
                    {
                        *type = SPDIFSTATUS_CURRPCM;
                        *index = -1;
                    }
                }
                else
                {
                    *type  = SPDIFSTATUS_PASSTHROUGH;
                    *index = avData->liveTv.selectedSpdifStream;
                }
                break;
        }

        retVal = FAPI_OK;
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Scan a transponder (sat frontend) or channel (cable/terrestrial).
**
** \param  network  Pointer to the network of the delivery system.
** \param  tpLstPos List index of requested carrier.
** \param  opts     Scan options.
** \param  feIdx    Frontend to be scanned (0...).
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa AV_ScanCarrierList, AV_ScanBlind
******************************************************************************/
int32_t AV_ScanSingleCarrier(const SYS_NETWORK_S * network, uint16_t tpLstPos,
                             const AV_SCAN_OPTS_S * opts, int8_t * feIdx)
{
    AV_CMD_S    cmd;
    int32_t     retVal;
    RSRC_CFG_S  rsrcCfg;

    if ( (retVal = AV_ScanGetFrontend(network, &rsrcCfg)) == FAPI_OK )
    {
        *feIdx = rsrcCfg.feIdx;

        cmd.type                = AV_CMD_SCAN_CARRIER;
        cmd.arg.scan.network    = *network;
        cmd.arg.scan.cfg        = rsrcCfg;
        cmd.arg.scan.tpLstPos   = tpLstPos;
        cmd.arg.scan.opts       = *opts;

        retVal = AV_CmdSend(&cmd);
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief   Scan a complete list of transponders/channels.
**
** \param   network Pointer to the network of the delivery system.
** \param   opts    Scan options.
** \param   feIdx   Frontend to be scanned (0...).
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa AV_ScanSingleCarrier, AV_ScanBlind
******************************************************************************/
int32_t AV_ScanCarrierList(const SYS_NETWORK_S * network,
                           const AV_SCAN_OPTS_S * opts, int8_t * feIdx)
{
    AV_CMD_S    cmd;
    int32_t     retVal;
    RSRC_CFG_S  rsrcCfg;

    if ( (retVal = AV_ScanGetFrontend(network, &rsrcCfg)) == FAPI_OK )
    {
        *feIdx = rsrcCfg.feIdx;

        cmd.type                = AV_CMD_SCAN_LIST;
        cmd.arg.scan.network    = *network;
        cmd.arg.scan.cfg        = rsrcCfg;
        cmd.arg.scan.tpLstPos   = 0;  /* not needed */
        cmd.arg.scan.opts       = *opts;

        retVal = AV_CmdSend(&cmd);
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief   Start a "blind scan".
**
** This is a rather slow but thourough scanning method. First, the entire
** frequency range is searched for transponders/channels. Next, all recognized
** transponders are scanned for services.
**
** \param   network Pointer to the network of the delivery system.
** \param   opts    Scan options.
** \param   feIdx   Frontend to be scanned (0...).
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**
** \sa
**          - AV_ScanSingleCarrier()
**          - AV_ScanCarrierList()
**
******************************************************************************/
int32_t AV_ScanBlind(const SYS_NETWORK_S * network, const AV_SCAN_OPTS_S * opts,
                     int8_t * feIdx)
{
    AV_CMD_S    cmd;
    int32_t     retVal;
    RSRC_CFG_S  rsrcCfg;

    FAPI_SYS_PRINT_DEBUG(1, "[av_api][AV_ScanBlind] enter\n");

    if ( (retVal = AV_ScanGetFrontend(network, &rsrcCfg)) == FAPI_OK )
    {
        *feIdx = rsrcCfg.feIdx;

        cmd.type = AV_CMD_SCAN_BLIND;
        cmd.arg.scan.network = *network;
        cmd.arg.scan.cfg = rsrcCfg;
        cmd.arg.scan.tpLstPos = 0;  /* not needed */
        cmd.arg.scan.opts = *opts;

        retVal = AV_CmdSend(&cmd);
    }
    FAPI_SYS_PRINT_DEBUG(1, "[av_api][AV_ScanBlind] quit\n");
    return retVal;
}

int32_t AV_ScanStop(int8_t feIdx)
{
    AV_CMD_S            cmd;
    int32_t             retVal;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    retVal = RSRC_ScanStopped(feIdx);
    if((retVal != FAPI_OK) && (retVal != SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE))
        return retVal;

    cmd.type = AV_CMD_STOP_SCAN;
    cmd.arg.abort.feIdx = feIdx;

    return AV_CmdSend(&cmd);
}

/*!
*******************************************************************************
**
** \brief  Stop current service presentation and keep the last frame on screen.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa AV_Stop
******************************************************************************/
int32_t AV_Freeze(void)
{
    AV_CMD_S            cmd;
    int32_t             retVal;

    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    #ifdef APPL_MHEG_ENABLED
    (void) MHEG_Stop();
    #endif /* APPL_MHEG_ENABLED */

    retVal = RSRC_LiveServiceStopped();
    if((retVal != FAPI_OK) && (retVal != SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE))
        return retVal;

    cmd.type                          = AV_CMD_STOP_SERVICE;
    cmd.arg.servstop.freeze           = FTRUE;
    cmd.arg.servstop.sendNotification = FTRUE;
    cmd.arg.servstop.resetFocus       = FFALSE;
    cmd.arg.servstop.keepRadioScreen  = FFALSE;
    cmd.arg.servstop.ctrlInfo         = SERVICE_CTRL_NORMAL;

    retVal = AV_CmdSend(&cmd);

    if(retVal == FAPI_OK)
    {
        avData->runningStatus = SYS_RUNSTAT_UNKNOWN;
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Stop current service presentation.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa AV_Freeze
******************************************************************************/
int32_t AV_Stop(void)
{
    AV_CMD_S            cmd;
    int32_t             retVal;

    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    #ifdef APPL_MHEG_ENABLED
    (void) MHEG_Stop();
    #endif /* APPL_MHEG_ENABLED */

    retVal = RSRC_LiveServiceStopped();
    if((retVal != FAPI_OK) && (retVal != SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE))
        return retVal;

    cmd.type = AV_CMD_STOP_SERVICE;
    cmd.arg.servstop.freeze           = FFALSE;
    cmd.arg.servstop.sendNotification = FTRUE;
    cmd.arg.servstop.resetFocus       = FFALSE;
    cmd.arg.servstop.keepRadioScreen  = FFALSE;
    cmd.arg.servstop.ctrlInfo         = SERVICE_CTRL_NORMAL;

    retVal = AV_CmdSend(&cmd);

    if(retVal == FAPI_OK)
    {
        avData->runningStatus = SYS_RUNSTAT_UNKNOWN;
    }

    return retVal;
}

int32_t AV_ResetService(void)
{
    AV_CMD_S            cmd;
    int32_t             retVal;

    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    retVal = RSRC_LiveServiceStopped();
    if((retVal != FAPI_OK) && (retVal != SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE))
        return retVal;

    avData->liveTv.reqServLoc.listId = SLIST_NONE;
    avData->liveTv.reqServLoc.index = -1;

    avData->liveTv.quietServLoc = avData->liveTv.reqServLoc;

    cmd.type = AV_CMD_STOP_SERVICE;
    cmd.arg.servstop.freeze           = FFALSE;
    cmd.arg.servstop.sendNotification = FTRUE;
    cmd.arg.servstop.resetFocus       = FTRUE;
    cmd.arg.servstop.keepRadioScreen  = FFALSE;
    cmd.arg.servstop.ctrlInfo         = SERVICE_CTRL_NORMAL;

    retVal = AV_CmdSend(&cmd);

    if(retVal == FAPI_OK)
    {
        avData->runningStatus = SYS_RUNSTAT_UNKNOWN;
    }

    return retVal;
}

int32_t AV_RestartService(void)
{
    AV_CMD_S            cmd;
    SYS_EVT_SERVICE_S   ctrlEvent;
    int32_t             retVal = FAPI_OK;
    RSRC_CFG_S          rsrcCfg;
    SYS_PRESSTATE_E     presState;

    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    retVal = SYS_GetPresentationState (&presState);
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    /* NOTE: Live service is only restarted if no other presentation is
             currently active! */
    if ( presState != SYS_PRESSTATE_NONE )
    {
        return FAPI_OK;
    }

    retVal = RSRC_ServiceAvlForLive(SLIST_GetServiceIndex
                                    (&(avData->liveTv.reqServLoc)),
                                    RSRC_OP_ANY & (~RSRC_OP_LIVE),
                                    NULL, &rsrcCfg);
    if(retVal != FAPI_OK)
        return retVal;

    /* update resource parameters */
    retVal = RSRC_SetLiveService(&rsrcCfg);
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    /* ---------------------------------------------------------------------- */

    cmd.type                       = AV_CMD_RESTART_SERVICE;
    cmd.arg.servsel.servLoc        = avData->liveTv.reqServLoc;
    cmd.arg.servsel.pref.spdifPref = FFALSE; /* not used for this command */
    cmd.arg.servsel.pref.spdifMode = SPDIFMODE_PREFERPASSTHROUGH;
    cmd.arg.servsel.pref.langAudio = LANG_NOTSPECIFIED; /* not used for this command */
    cmd.arg.servsel.pref.langSpdif = LANG_NOTSPECIFIED; /* not used for this command */
    cmd.arg.servsel.cfg            = rsrcCfg;

    retVal = AV_CmdSend(&cmd);
    if(retVal == FAPI_OK)
    {
        avData->runningStatus = SYS_RUNSTAT_UNKNOWN;

        /* Generate event to notify start of service selection */
        ctrlEvent.hdr.type   = EVT_SERVICE_SELECTED;
        ctrlEvent.hdr.length = sizeof(ctrlEvent);
        ctrlEvent.servLoc    = avData->liveTv.reqServLoc;
        ctrlEvent.audioStr   = 0; // invalid for this message
        ctrlEvent.status     = SERVICE_STAT_START;
        ctrlEvent.ctrlInfo   = SERVICE_CTRL_NORMAL;

        retVal = EVT_Send(&ctrlEvent, "Service select");
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Start the default service.
**
** This function can be used after system startup. It will first try to select
** the stored service from last system operation. On failure, selecting of
** the first service in one of the available service lists is attempted.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t AV_StartInitialService(void)
{
    SYS_SERVLOC_S       servLoc;

    servLoc.listId = SLIST_TV;
    servLoc.index = 0;

    /* 1. attempt to select stored service */
    if ( AV_SelectService(&(avData->liveTv.reqServLoc), NULL, SERVICE_CTRL_NORMAL, FTRUE) ==
         FAPI_OK )
    {
        return FAPI_OK;
    }

    /* 2. attempt to select first service in global TV list */
    else if ( AV_SelectService(&servLoc, NULL, SERVICE_CTRL_NORMAL, FFALSE) == FAPI_OK )
    {
        return FAPI_OK;
    }

    /* 3. attempt to select first service in global Radio list */
    else
    {
        servLoc.listId = SLIST_RADIO;

        return AV_SelectService(&servLoc, NULL, SERVICE_CTRL_NORMAL, FFALSE);
    }
}

/*!
*******************************************************************************
**
** \brief  Returns the currently active TV Standard.
**
** The output TV standard does not necessarily match the TV standard of the
** input stream. It depends on the preference setting that is configurable via
** SETTINGS_OutputTvStandardSet.
**
** \return Current TV Standard at the video output.
**
** \sa SETTINGS_OutputTvStandardSet
******************************************************************************/
int32_t AV_OutputTvStandardCurrGet(SYS_HDTV_STANDARD* hdTvStandard,
                                   SYS_SDTV_STANDARD* sdTvStandard)
{
    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    if ( hdTvStandard != NULL ) *hdTvStandard = avData->video.outHdSysCurr;
    if ( sdTvStandard != NULL ) *sdTvStandard = avData->video.outSdSysCurr;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Returns the current picture format.
**
** \return
**         - #PICFORMAT_4_3   4:3 format
**         - #PICFORMAT_16_9  16:9 format, the video frames are
                              anamorphically distorted
**
** \sa SETTINGS_DispFormatSet
******************************************************************************/
int32_t AV_PicFormatCurrGet(SYS_PICFORMAT_E * picFormat)
{
    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    *picFormat = avData->video.picFormat;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Returns the current TV standard of the input video stream.
**
** \return Current TV Standard of the input video stream.
**
** \sa AV_OutputTvStandardCurrGet
******************************************************************************/
int32_t AV_InputTvStandardCurrGet(SYS_TV_STANDARD * tvStandard)
{
    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    *tvStandard = avData->video.inSysCurr;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get the current signal strength on requested frontend.
**
** \param  feIdx      Frontend index (0...n).
** \param  levelCurr  current level is returned here.
** \param  lvlMax     maximum level is returned here.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**         - #APPL_SMARTGO_ERR_BAD_PARAMETER in case of invalid frontend selection
**
** \sa AV_GetSignalQuality
******************************************************************************/
int32_t AV_GetSignalStrength(int8_t feIdx, uint16_t * levelCurr,
                             uint16_t * lvlMax)
{
    int32_t             retVal;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    (*lvlMax) = 0xFFFF;
    (*levelCurr) = avData->feDat[feIdx].sigStrengthCurr;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get the current signal quality (based on BER) on requested frontend.
**
** \param  feIdx      Frontend index (0...n).
** \param  levelCurr  current level is returned here.
** \param  lvlMax     maximum level is returned here.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**         - #APPL_SMARTGO_ERR_BAD_PARAMETER in case of invalid frontend selection
**
** \sa AV_GetSignalStrength
******************************************************************************/
int32_t AV_GetSignalQuality(int8_t feIdx, uint16_t * levelCurr,
                            uint16_t * lvlMax)
{
    int32_t             retVal;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    (*lvlMax) = 0xFFFF;
    (*levelCurr) = avData->feDat[feIdx].sigQualiCurr;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get status parameters of requested frontend.
**
** \param  feIdx      Frontend index (0...n).
** \param  status     Pointer to structure where frontend parameters will be
**                    returned.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**         - #APPL_SMARTGO_ERR_BAD_PARAMETER in case of invalid frontend index
**
******************************************************************************/
int32_t AV_GetFeStatus(int8_t feIdx, SYS_FE_STATUS_S * status)
{
    RSRC_STATUS_S      *rsrcStatus;

    FAPI_SYS_PRINT_DEBUG(1,"[av_api][AV_GetFeStatus] enter\n");

    if(avData == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else if((feIdx < 0) || (feIdx >= (int8_t) SYS_GetHWConfig()->feNums))
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    rsrcStatus = RSRC_GetStatus();

    FAPI_SYS_PRINT_DEBUG(1,"[av_api][AV_GetFeStatus] rsrcStatus = %s\n", NULL == rsrcStatus ? "NULL" : "Valid");

    if((rsrcStatus->hw.fe[feIdx].use & RSRC_STAT_FE_OPENED) == 0)
    {
        status->use = SYS_FEUSAGE_NOTAVAILABLE;
        status->type = FE_SYSTEM_TYPE_DIRECT;
    }
    else
    {
        if(rsrcStatus->hw.fe[feIdx].use == RSRC_STAT_FE_OPENED)
        {
            status->use = SYS_FEUSAGE_FREE;
        }
        else
        {
            status->use = SYS_FEUSAGE_INUSE;
        }

        status->type = avData->feDat[feIdx].systemType;
    }

    status->lock = avData->feDat[feIdx].tuneLockStatus;

    return FAPI_OK;
}

int32_t AV_SetAntennaPower(int8_t feIdx, fbool_t powerOn)
{
    int32_t             retVal;
    AV_CMD_S            cmd;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    cmd.type = AV_CMD_SET_ANTENNAPOWER;
    cmd.arg.antpwr.feIdx = feIdx;
    cmd.arg.antpwr.power = powerOn;

    return AV_CmdSend(&cmd);
}

#ifdef APPL_FESAT_ENABLED

/*!
*******************************************************************************
**
** \brief  Control automatic satellite motor mode.
**
** In automatic mode (default), satellite position is adjusted every time the
** system performs a tuning operation. Disabling this mode might be needed
** to avoid conflicts in OSD menus where manual positioning is performed.
**
** \param  feIdx      Frontend index (0...n).
** \param  enable     1: Automatic mode enabled, 0: disabled.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t AV_SetAutoMotorMode(int8_t feIdx, fbool_t enable)
{
    int32_t             retVal;
    AV_CMD_S            cmd;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    cmd.type = AV_CMD_SET_AUTOMOTORMODE;
    cmd.arg.motor.feIdx = feIdx;
    cmd.arg.motor.enable = enable;

    return AV_CmdSend(&cmd);
}

#endif /* APPL_FESAT_ENABLED */

/*!
*******************************************************************************
**
** \brief  Get service list location of the currently \b requested/selected
**         live service.
**
** \param  pServLoc  Pointer to where service list location will be copied into.
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
******************************************************************************/
int32_t AV_GetRequestedService(SYS_SERVLOC_S * pServLoc)
{
    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (*pServLoc) = avData->liveTv.reqServLoc;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get service list location of the currently \b quiet requested/selected
**         live service.
**
** \param  pServLoc  Pointer to where service list location will be copied into.
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
******************************************************************************/
int32_t AV_GetQuietRequestedService(SYS_SERVLOC_S * pServLoc)
{
    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (*pServLoc) = avData->liveTv.quietServLoc;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get service list location of the previously selected live service.
**
** \param  pServLoc  Pointer to where service list location will be copied into.
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
******************************************************************************/
int32_t AV_GetPrevService(SYS_SERVLOC_S * pServLoc)
{
    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (*pServLoc) = avData->liveTv.prevServLoc;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   Get index of the previously selected live service out of
**          requested service list.
**
** \param   listId  identifier of list to get last selected service from.
** \param   index   pointer to store the requested service list index or -1 in
**                  case of failure or non-availability.
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**
******************************************************************************/
int32_t AV_GetPrevServiceByList(SLIST_TYPE listId, int16_t * index)
{
    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    if(listId == SLIST_NONE)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    else if(listId >= SLIST_OFFSET_GLOB)
    {
        if((uint16_t) (listId & SLIST_MASK_GLOB) >= SLIST_GLOB_NUMLISTS)
        {
            return APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
        else
        {
            *index = avData->liveTv.prevIdxGlob[listId & SLIST_MASK_GLOB];
        }
    }
    else
    {
        if(listId >= SLIST_USR_NUMLISTS)
            return APPL_SMARTGO_ERR_BAD_PARAMETER;
        else
            *index = avData->liveTv.prevIdxUsr[listId];
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get the current volume level and mute status.
**
** \param  lvlLeft      level of left channel
** \param  lvlRight     level of right channel
** \param  lvlMax       maximum level
** \param  isMuted      mute status (FTRUE: muted)
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa
**          - AV_VolumeSet()
**
******************************************************************************/
int32_t AV_VolumeGet(uint8_t * lvlLeft, uint8_t * lvlRight, uint8_t * lvlMax,
                     fbool_t * isMuted)
{
    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (*lvlLeft) = avData->audio.lvlLeft;
    (*lvlRight) = avData->audio.lvlRight;
    (*lvlMax) = FAPI_AUOUT_MAX_VOL;
    (*isMuted) = avData->audio.muteByUser;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get current stereo mode.
**
** \param  mode         Pointer to variable to be filled with value of
**                      type AV_STEREO_MODE_E.
**
** \return One of the following status codes:
**         - #FAPI_OK                     If successful.
**         - #APPL_SMARTGO_ERR_NOT_INITIALIZED    AV module not initialized yet.
**
** \sa AV_VolumeSetStereoMode
******************************************************************************/
int32_t AV_VolumeGetStereoMode(AV_STEREO_MODE_E * mode)
{
    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    *mode = avData->audio.stereoMode;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set current stereo mode.
**
** \param  mode       Requested stereo mode.
**
** \return One of the following status codes:
**         - #FAPI_OK                     If successful.
**         - #APPL_SMARTGO_ERR_NOT_INITIALIZED    AV module not initialized yet.
**
** \sa AV_VolumeGetStereMode
******************************************************************************/
int32_t AV_VolumeSetStereoMode(AV_STEREO_MODE_E mode)
{
    AV_CMD_S            cmd;

    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    if(mode >= AV_STEREO_MODE_NUMS)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    cmd.type = AV_CMD_SET_STEREOMODE;
    cmd.arg.stereo.mode = mode;

    return AV_CmdSend(&cmd);
}

/*!
*******************************************************************************
**
** \brief  Get current balance set value
**
** \param  balance      Pointer to variable to be filled with value of
**                      type int8_t.
**
** \return One of the following status codes:
**         - #FAPI_OK                     If successful.
**         - #APPL_SMARTGO_ERR_NOT_INITIALIZED    AV module not initialized yet.
**
** \sa AV_VolumeSetBalance
******************************************************************************/
int32_t AV_VolumeGetBalance(int8_t* balance)
{
    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    *balance = avData->audio.balance;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set new balance value
**
** \param  balance      Requested balance value
**
** \return One of the following status codes:
**         - #FAPI_OK                     If successful.
**         - #APPL_SMARTGO_ERR_NOT_INITIALIZED    AV module not initialized yet.
**
** \sa AV_VolumeGetBalance
******************************************************************************/
int32_t AV_VolumeSetBalance(int8_t balance)
{
    if ( avData == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if( ( balance < AV_AUDIO_BALANCE_LEFT_MAX  ) ||
        ( balance > AV_AUDIO_BALANCE_RIGHT_MAX ) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    avData->audio.balance = balance;

    return AV_VolumeSet(avData->audio.lvlLeft, avData->audio.lvlRight);
}

/*!
*******************************************************************************
**
** \brief  Set the audio volume level.
**
** \param  lvlLeft   level of left channel
** \param  lvlRight  level of right channel
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa AV_VolumeGet
******************************************************************************/
int32_t AV_VolumeSet(uint8_t lvlLeft, uint8_t lvlRight)
{
    int32_t retVal;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        retVal = AVINT_AudioVolumeSet(lvlLeft, lvlRight);
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Mute or unmute the audio output.
**
** \param  onOffFlag  FTRUE: Mute, FFALSE: Unmute
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t AV_Mute(fbool_t onOffFlag)
{
    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    if(avData->audio.muteByUser != onOffFlag)
    {
        if(onOffFlag)
            AVINT_AudioMuteRequestSet(AV_MUTEREQ_USER);
        else
            AVINT_AudioMuteRequestClear(AV_MUTEREQ_USER);

        avData->audio.muteByUser = onOffFlag;
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Modify size and position of the video output picture.
**
** This function can be used for realising application features like an OSD
** menu with a minimised live-TV window in one corner of the screen.
**
** \param  fullScreen  Set fullscreen video, pos argument is ignored.
** \param  pos         Video area positioning information (only used if
**                     fullscreen is FFALSE).
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t AV_SetVideoArea(fbool_t fullScreen, const AV_VO_PICTURE_WINDOW_S * pos)
{
    AV_CMD_S            cmd;

    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    cmd.type = AV_CMD_SET_VIDEOAREA;
    cmd.arg.varea.fullScreen = fullScreen;

    if(fullScreen)
    {
        memset(&(cmd.arg.varea.area), 0,
                       sizeof(AV_VO_PICTURE_WINDOW_S));
    }
    else
    {
        if(pos == NULL)
            return APPL_SMARTGO_ERR_BAD_PARAMETER;

        cmd.arg.varea.area = *pos;
    }

    return AV_CmdSend(&cmd);
}

/*!
*******************************************************************************
**
** \brief  Scale a given cutout area to current video area.
**
** \note Position and size is interpreted relative to a PAL frame,
**       regardless of the current video size and scaling method.
**
** \param  full        Revert to non-zoomed video, pos argument is ignored.
** \param  pos         Position and size of cutout area to be zoomed relative
**                     to a full-screen PAL frame (720x576).
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa FD_VO_SetCutoutPicture
******************************************************************************/
int32_t AV_ZoomVideo(fbool_t full, const AV_VO_PICTURE_WINDOW_S * pos)
{
    AV_CMD_S            cmd;

    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    cmd.type = AV_CMD_ZOOM_VIDEO;
    cmd.arg.varea.fullScreen = full;

    if(full)
    {
        memset(&(cmd.arg.varea.area), 0, sizeof(AV_VO_PICTURE_WINDOW_S));
    }
    else
    {
        if(pos == NULL)
            return APPL_SMARTGO_ERR_BAD_PARAMETER;

        cmd.arg.varea.area = *pos;
    }

    return AV_CmdSend(&cmd);
}

/*!
*******************************************************************************
**
** \brief  Freeze current video output or resume presentation
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
******************************************************************************/
int32_t AV_FreezeVideo (fbool_t freezeOn)
{
    AV_CMD_S    cmd;
    int32_t     retVal;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        cmd.type                   = AV_CMD_VIDEO_FREEZE;
        cmd.arg.videoFreeze.freeze = freezeOn;

        retVal = AV_CmdSend(&cmd);
    }

    return retVal;
}

int32_t AV_IFrameShow (const uint32_t* data,
                       uint32_t sizeInBytes,
                       uint32_t format)
{
    AV_CMD_S    cmd;
    int32_t     retVal;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else if ( (data == NULL) || (sizeInBytes == 0) )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        cmd.type                   = AV_CMD_VIDEO_IFRAME_SHOW;
        cmd.arg.videoIframe.data   = data;
        cmd.arg.videoIframe.size   = sizeInBytes;
        cmd.arg.videoIframe.format = format;

        retVal = AV_CmdSend(&cmd);

        if ( retVal == FAPI_OK )
        {
            /* NOTE: Wait for end of decoding to ensure integrity of data
                     passed via pointer! */
            retVal = AV_Nop();
        }
    }

    return retVal;
}

int32_t AV_IFrameHide (void)
{
    AV_CMD_S    cmd;
    int32_t     retVal;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        cmd.type = AV_CMD_VIDEO_IFRAME_HIDE;

        retVal = AV_CmdSend(&cmd);
    }

    return retVal;
}

int32_t AV_PowerupScreenHide (void)
{
    int32_t retVal;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else if ( avData->video.bootScreenActive &&
              (SYS_GetConfig()->bootScreen != NULL) )
    {
        retVal = AV_IFrameHide ();
    }
    else
    {
        retVal = FAPI_OK;
    }

    return retVal;
}

int32_t AV_GetVideoStreamInfo (AV_VIDEO_STREAMINFO_S* info)
{
    int32_t retVal = FAPI_OK;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        retVal = AVINT_VideoGetStreamInfo(info);
    }

    return retVal;
}

#ifndef _LINUX_FAPI
int32_t AV_GetVideoFrameInfo (FAPI_VIDEC_VideoDecInfoStateT* info)
{
    int32_t retVal = FAPI_OK;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        retVal = AVINT_VideoGetFrameInfo(info);
    }

    return retVal;
}
#else
/* Martin added at 20100512 */
int32_t AV_GetVideoFrameInfo (VAL_VIDEC_DecFrameInfoStrT* info)
{
    int32_t retVal = FAPI_OK;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        retVal = AVINT_VideoGetFrameInfo(info);
    }

    return retVal;
}
#endif

int32_t AV_GetAudioStreamInfo (AV_AUDIO_STREAMINFO_S* info)
{
    int32_t retVal = FAPI_OK;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        retVal = AVINT_AudioGetStreamInfo(info);
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Check if given audio format is decodable by current audio firmware.
**
** \param  format   Audio format where decoding capability should be checked,
**                  one of SYS_AUDIO_XXX
** \param  pResult  Pointer to boolean value where result will be stored on
**                  success (function needs to return FAPI_OK).
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t AV_AudioFormatIsDecodable (uint8_t format, fbool_t* pResult)
{
    int32_t          retVal;
    AAL_AudioFormatT aalFormat;
    uint8_t          decodable = 0;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        switch ( format )
        {
            case SYS_AUDIO_MPEG:    aalFormat = AAL_AUDIOFORMAT_MPEG;
                                    break;
            case SYS_AUDIO_AC3:     aalFormat = AAL_AUDIOFORMAT_AC3;
                                    break;
            case SYS_AUDIO_AAC:     aalFormat = AAL_AUDIOFORMAT_AAC;
                                    break;
            case SYS_AUDIO_HEAAC:   aalFormat = AAL_AUDIOFORMAT_HEAAC;
                                    break;
            case SYS_AUDIO_DDPDCV:  aalFormat = AAL_AUDIOFORMAT_DDP;
                                    break;
            default:                aalFormat = AAL_AUDIOFORMAT_NONE;
                                    break;
        }

        retVal = AAL_FormatIsDecodable (aalFormat, &decodable);

        if ( pResult != NULL )
        {
            *pResult = (decodable == 0) ? FFALSE : FTRUE;
        }
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Check if given audio format is suitable for SPDIF passthrough.
**
** \param  format   Audio format where SPDIF passthrough compliance should
**                  be checked, one of SYS_AUDIO_XXX
** \param  pResult  Pointer to boolean value where result will be stored on
**                  success (function needs to return FAPI_OK).
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t AV_AudioFormatIsSpdifCompliant (uint8_t format, fbool_t* pResult)
{
    int32_t retVal;
    fbool_t compliant;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        compliant = AVINT_AudioIsSPDIFCompliant (format);

        retVal = FAPI_OK;

        if ( pResult != NULL )
        {
            *pResult = compliant;
        }
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Get audio delay value from EDID info.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
******************************************************************************/
int32_t AV_HDMIGetEDIDLipSyncDelay (int32_t* delay)
{
    int32_t retVal = FAPI_OK;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        retVal = AVINT_HDMIGetEDIDLipSyncDelay(delay);
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Set audio delay value for lip sync purpose.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
******************************************************************************/
int32_t AV_HDMISetLipSyncDelay (int32_t delay)
{
    int32_t retVal = FAPI_OK;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        retVal = AVINT_HDMISetLipSyncDelay(delay);
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Set lip sync auto regard to the EDID info
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
******************************************************************************/
int32_t AV_HDMIAutoSync (void)
{
    int32_t retVal = FAPI_OK;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        retVal = AVINT_HDMIAutoSync();
    }

    return retVal;
}
/*!
*******************************************************************************
**
** \brief  Get size and position of the video output picture.
**
** \param  pos     Video coordinates requested by user are returned here
**                 (NULL if not needed).
** \param  realPos "Real" video coordinates are returned here. These might
**                 differ from the user request due to scaling limitations.
**                 (NULL if not needed).
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t AV_GetVideoArea(AV_VO_PICTURE_WINDOW_S * pos,
                        AV_VO_PICTURE_WINDOW_S * realPos)
{
    int32_t retVal = FAPI_OK;

    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    if(pos != NULL)
    {
        #if 0
        retVal = FD_VO_GetOutputPicture(0, pos);
        if(retVal != FAPI_OK)
            return retVal;
        #endif
    }

    if(realPos != NULL)
    {
        #if 0
        retVal = FD_VO_GetRealOutputPicture(0, realPos);
        #endif
    }
    return retVal;
}

/* ------------------------------------------------------------------------- */

int32_t AV_CheckArgs(int8_t feIdx)
{
    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else if((feIdx < 0) || (feIdx >= (int8_t) SYS_GetHWConfig()->feNums))
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if(avData->feDat[feIdx].drvHdl == NULL)
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }
    else
    {
        return FAPI_OK;
    }
}

/* Searches for a frontend available for scanning requested satellite */
FSTATIC int32_t AV_ScanGetFrontend(const SYS_NETWORK_S * network,
                                   RSRC_CFG_S * rsrcCfg)
{
    int32_t             retVal;

    #ifdef APPL_FESAT_ENABLED
    CFGSAT_SAT_S        satInf;
    #endif /* APPL_FESAT_ENABLED */

    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    #ifdef APPL_FESAT_ENABLED

    if(network->type == FE_SYSTEM_TYPE_SATELLITE)
    {
        /* NOTE: Currently this is only done to test if function parameters are
           valid */
        retVal = CFGSAT_GetDataByIndex(network->satIdx, &satInf);
        if(retVal != FAPI_OK)
            return retVal;
    }

    #endif /* APPL_FESAT_ENABLED */

    retVal = RSRC_NetworkAvlForScan(network, RSRC_OP_ANY, rsrcCfg);
    if(retVal != FAPI_OK)
        return retVal;

    retVal = RSRC_SetScan(rsrcCfg);
    if(retVal != FAPI_OK)
        return retVal;

    return FAPI_OK;
}

int32_t AV_CmdSend(AV_CMD_S * pCmd)
{
    return ITC_CmdSend(&(avData->itcInst), pCmd, NULL);
}

// TODO: this interface is to be replaced!
FAPI_SYS_HandleT AV_GetViencHdHandle(void)
{
    FAPI_SYS_HandleT handle = NULL;

    if ( avData->video.valDataPtr != NULL )
    {
        handle = VAL_GetIdViencHd(avData->video.valDataPtr);
    }

    return handle;
}

#if (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED)
/*!
*******************************************************************************
**
** \brief  Select a subtitle stream to decoder.
**
** \param  index  Index of subtitle stream in current service's list
**                   including DVB subtitle and teletext subtitle.
** \param  isDvb  DVB or teletext subtitle indication
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**         - #SMARTGO_SYS_ERR_PROHIBITED
**
** \sa
******************************************************************************/
int32_t AV_SetSubttStream(int8_t index, fbool_t isDvb)
{
    AV_CMD_S    cmd;
    int32_t     retVal;

    if ( avData == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else if ( avData->liveStatus == SERVICE_STAT_STOPPED )
    {
        retVal = SMARTGO_SYS_ERR_PROHIBITED;
    }
    else
    {
        cmd.type            = AV_CMD_SELECT_SUBTTSTREAM;
        cmd.arg.subtt.index = index;
        cmd.arg.subtt.isDvb = isDvb;

        retVal = AV_CmdSend(&cmd);
    }

    return retVal;
}

#endif /* (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED) */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
