/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/av_rec.c
**
**              $Id: av_rec.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       PVR recording extensions for AV module.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info    715: Symbol not referenced.
**
*****************************************************************************
*/
/*lint -save -e715 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "sys_api.h"
#include "sys_priv.h"
#include "utils.h"

#include "av.h"
#include "pvr_sys.h"
#include "pvr_list.h"
#include "eit.h"
#include "teletext.h"

#ifdef APPL_SUBTT_ENABLED
#include "subtitle.h"
#endif /* APPL_SUBTT_ENABLED */

#ifdef APPL_MPEG_ENCODER_ENABLED
#include "mpenc.h"
#endif /* APPL_MPEG_ENCODER_ENABLED */

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define AV_SLEEP_PLAY_IDLE     100
#define AV_SLEEP_PLAY_ACTIVE    50

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

int32_t AV_RecordStart (const RSRC_CFG_S* cfg)
{
    int32_t  retVal;
    AV_CMD_S cmd;

    /* NOTE: Resource allocation is done in PVR module. */

    retVal = RSRC_SetRecord(cfg);
    if ( retVal != FAPI_OK ) return retVal;

    switch ( cfg->servIdx )
    {
        #ifdef APPL_MPEG_ENCODER_ENABLED

        case SLIST_SERVIDX_MPENC:
            cmd.type = AV_CMD_REC_MPENC_START;
            break;

        #endif /* APPL_MPEG_ENCODER_ENABLED */

        default:
        case SLIST_SERVIDX_NONE:
        case SLIST_SERVIDX_PLAY:
            cmd.type = AV_CMD_REC_START;
            break;
    }

    cmd.arg.record.cfg = *cfg;

    return AV_CmdSend(&cmd);
}

int32_t AV_RecordStop (uint8_t recIdx)
{
    AV_CMD_S cmd;
    int32_t  retVal;

    cmd.arg.record.cfg = RSRC_GetStatus()->rec[recIdx].cfg;

    retVal = RSRC_RecordStopped(recIdx);
    if ( retVal == SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE )
    {
        retVal = FAPI_OK;
    }
    else if ( retVal == FAPI_OK )
    {
        switch ( cmd.arg.record.cfg.servIdx )
        {
            #ifdef APPL_MPEG_ENCODER_ENABLED

            case SLIST_SERVIDX_MPENC:
                cmd.type = AV_CMD_REC_MPENC_STOP;
                break;

            #endif /* APPL_MPEG_ENCODER_ENABLED */

            default:
            case SLIST_SERVIDX_NONE:
            case SLIST_SERVIDX_PLAY:
                cmd.type = AV_CMD_REC_STOP;
                break;
        }

        retVal = AV_CmdSend(&cmd);
    }

    return retVal;
}

int32_t AV_RecordReroute (const RSRC_CFG_S* cfg)
{
    int32_t  retVal;
    AV_CMD_S cmd;

    retVal = RSRC_RecordStopped (cfg->tsdIdx - RSRC_TSD_REC1);
    if ( retVal != FAPI_OK ) return retVal;

    retVal = RSRC_SetRecord (cfg);
    if ( retVal != FAPI_OK ) return retVal;

    cmd.type            = AV_CMD_REC_REROUTE;
    cmd.arg.record.cfg  = *cfg;

    return AV_CmdSend(&cmd);
}

int32_t AV_PlayStart (PVRLST_IDX lstIdx, SLIST_SERVIDX servIdx,
                      const RSRC_CFG_S* cfg)
{
    int32_t  retVal;
    AV_CMD_S cmd;

    /* NOTE: Resource allocation is done in PVR module. */

    retVal = RSRC_SetPlayback(cfg);
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    #ifdef APPL_MHEG_ENABLED
    MHEG_Enable(FFALSE,SYS_SUSPENDSRC_PLAY);
    #endif

    cmd.type             = AV_CMD_PLAY_START;
    cmd.arg.play.lstIdx  = lstIdx;
    cmd.arg.play.servIdx = servIdx;
    cmd.arg.play.cfg     = *cfg;

    return AV_CmdSend(&cmd);
}

int32_t AV_PlayStop ( void )
{
    AV_CMD_S cmd;
    int32_t  retVal;

    cmd.type = AV_CMD_PLAY_STOP;

    retVal = RSRC_PlaybackStopped();
    if ( retVal == SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE )
    {
        return FAPI_OK;
    }
    else if ( retVal != FAPI_OK )
    {
        return retVal;
    }
    else
    {
        return AV_CmdSend(&cmd);
    }
}

#if defined(APPL_CI_ENABLED) && defined(APPL_DESCRREC_ENABLED)

int32_t AV_DescrRecordStart (const RSRC_CFG_S* cfg)
{
    int32_t  retVal;
    AV_CMD_S cmd;

    /* NOTE: Resource allocation is done in PVR module. */

    retVal = RSRC_SetDescrRecord (cfg);
    if ( retVal != FAPI_OK ) return retVal;

    cmd.type           = AV_CMD_DESCRREC_START;
    cmd.arg.record.cfg = *cfg;

    return AV_CmdSend(&cmd);
}

int32_t AV_DescrRecordStop ()
{
    AV_CMD_S cmd;
    int32_t  retVal;

    cmd.type           = AV_CMD_DESCRREC_STOP;
    cmd.arg.record.cfg = RSRC_GetStatus()->descrRec.cfg;

    retVal = RSRC_DescrRecordStopped ();

    if      ( retVal == SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE ) return FAPI_OK;
    else if ( retVal != FAPI_OK )                         return retVal;

    else return AV_CmdSend(&cmd);
}

int32_t AV_DescrRecordReroute (const RSRC_CFG_S* cfg)
{
    int32_t  retVal;
    AV_CMD_S cmd;

    retVal = RSRC_DescrRecordStopped ();
    if ( retVal != FAPI_OK ) return retVal;

    retVal = RSRC_SetDescrRecord (cfg);
    if ( retVal != FAPI_OK ) return retVal;

    cmd.type           = AV_CMD_DESCRREC_REROUTE;
    cmd.arg.record.cfg = *cfg;

    return AV_CmdSend(&cmd);
}

#endif /* defined(APPL_CI_ENABLED) && defined(APPL_DESCRREC_ENABLED) */

/* ------------------------------------------------------------------------- */

int32_t AVINT_PlayRecInit()
{
    uint8_t cnt;

    memset (&(avData->play), 0, sizeof(avData->play));

    avData->play.active        = FFALSE;
    avData->play.internal      = FFALSE;
    avData->play.currVersionPat = PSISI_VERSION_ANY;
    avData->play.currVersionPmt = PSISI_VERSION_ANY;

    for ( cnt = 0; cnt < RSRC_REC_MAX; cnt++ )
    {
        memset (avData->rec + cnt, 0, sizeof(avData->rec[0]));

        avData->rec[cnt].active         = FFALSE;
        avData->rec[cnt].pFe            = NULL;
        avData->rec[cnt].currVersionPat = PSISI_VERSION_ANY;
        avData->rec[cnt].currVersionPmt = PSISI_VERSION_ANY;
    }

    #ifdef APPL_MPEG_ENCODER_ENABLED

    memset (&(avData->mpenc), 0, sizeof(avData->mpenc));

    #endif /* APPL_MPEG_ENCODER_ENABLED */

    return FAPI_OK;
}

int32_t AVINT_RecStart (const RSRC_CFG_S* cfg)
{
    AV_REC_S*       pRec = avData->rec + (cfg->tsdIdx - RSRC_TSD_REC1);
    AV_FEDAT_S*     pFe  = avData->feDat + cfg->feIdx;
    int32_t         retVal;
    TPLIST_ITEM_S   tpItem;

    if ( pRec->active ) return FAPI_OK; /* already active */

    do
    {
        retVal = SLIST_GetServByIndex(cfg->servIdx, &(pRec->service));
        if ( retVal != FAPI_OK ) break;

        retVal = TPLIST_GetDataByIndex(&(pRec->service.network),
                                       pRec->service.tpIdx,
                                       &tpItem);
        if ( retVal != FAPI_OK ) break;

        AVINT_RouteSignal(cfg);

        (void)PSISI_SetTsdUsage (cfg->op, cfg->tsdIdx,
                           &(avData->feDat[cfg->feIdx].pat));

        #if (defined APPL_CA_ENABLED) || (defined APPL_CI_ENABLED)

        /* NOTE: CAT is not filtered for updates in order to avoid filter
                 conflicts with recording engine. After CAT has been received,
                 recording will start and set a filter on the CAT PID. */

        (void)PSISI_CAT_Invalidate     (cfg->tsdIdx);
        (void)PSISI_CAT_StartFiltering (cfg->tsdIdx, PSISI_VERSION_ANY, FFALSE,
                                  FFALSE, NULL);

        #endif /* (defined APPL_CA_ENABLED) || (defined APPL_CI_ENABLED) */

        #ifdef APPL_CI_ENABLED

        if ( cfg->useCi )
        {
            CISYS_ServiceSelected (cfg->servIdx, cfg->ts);
        }

        #endif /* APPL_CI_ENABLED */

        pRec->active         = FTRUE;
        pRec->pFe            = pFe;
        pRec->cfg            = *cfg;
        pRec->currVersionPat = PSISI_VERSION_ANY;
        pRec->currVersionPmt = PSISI_VERSION_ANY;

        pFe->reqTunePar.network    = pRec->service.network;
        pFe->reqTunePar.manualMode = FFALSE;
        pFe->reqTunePar.par        = tpItem.pars;
        pFe->reqTunePar.tpIdx      = pRec->service.tpIdx;

        pFe->useStatus |= (uint8_t)cfg->op;
        pFe->state      = AV_FESTATE_NORMAL;
    }
    while ( 0 );

    return retVal;
}

int32_t AVINT_RecStop (uint8_t tsdIdx)
{
    AV_REC_S* pRec = avData->rec + (tsdIdx - RSRC_TSD_REC1);

    if ( !(pRec->active) ) return FAPI_OK;

    pRec->pFe->useStatus &= (~(pRec->cfg.op));

    if ( pRec->pFe->useStatus == 0 )
    {
        pRec->pFe->state = AV_FESTATE_IDLE;
    }

    #ifdef APPL_CI_ENABLED

    if ( pRec->cfg.useCi )
    {
        CISYS_ServiceStop (pRec->cfg.servIdx, pRec->cfg.ts);
    }

    #endif /* APPL_CI_ENABLED */

    #if (defined APPL_CA_ENABLED) || (defined APPL_CI_ENABLED)

    (void)PSISI_CAT_StopFiltering (tsdIdx);

    #endif /* (defined APPL_CA_ENABLED) || (defined APPL_CI_ENABLED) */

    (void)PSISI_ReleaseTsdUsage (pRec->cfg.op, tsdIdx);

    pRec->active = FFALSE;
    pRec->pFe    = NULL;

    return FAPI_OK;
}

int32_t AVINT_RecReroute (const RSRC_CFG_S* cfg)
{
    AV_REC_S*       pRec = avData->rec + (cfg->tsdIdx - RSRC_TSD_REC1);
    AV_FEDAT_S*     pFe  = avData->feDat + cfg->feIdx;
    int32_t         retVal;
    TPLIST_ITEM_S   tpItem;

    if ( (!(pRec->active)) ||
         (pRec->pFe == NULL) ) return FAPI_OK;

    /* NOTE: It is assumed here that the following configuration parameters
             have not changed:
                        - servIdx
                        - tsdIdx
    */

    /* ----- handle frontend usage status ---------------------------------- */

    if ( pFe != pRec->pFe )
    {
        /* Frontend change: update states of last frontend */

        pRec->pFe->useStatus &= (~(pRec->cfg.op));

        if ( pRec->pFe->useStatus == 0 )
        {
            pRec->pFe->state = AV_FESTATE_IDLE;
        }

        pRec->pFe = pFe;

        /* Frontend change: update states of new frontend */

        pFe->useStatus |= (uint8_t)cfg->op;
        pFe->state      = AV_FESTATE_NORMAL;
    }

    /* --------------------------------------------------------------------- */

    #ifdef APPL_CI_ENABLED

    if ( pRec->cfg.useCi )
    {
        CISYS_ServiceStop (pRec->cfg.servIdx, pRec->cfg.ts);
    }

    #endif /* APPL_CI_ENABLED */

    (void)PSISI_ReleaseTsdUsage (pRec->cfg.op, pRec->cfg.tsdIdx);

    do
    {
        retVal = TPLIST_GetDataByIndex(&(pRec->service.network),
                                       pRec->service.tpIdx,
                                       &tpItem);
        if ( retVal != FAPI_OK ) break;

        AVINT_RouteSignal(cfg);

        (void)PSISI_SetTsdUsage (cfg->op, cfg->tsdIdx,
                           &(avData->feDat[cfg->feIdx].pat));

        #ifdef APPL_CI_ENABLED

        if ( cfg->useCi )
        {
            CISYS_ServiceSelected (cfg->servIdx, cfg->ts);
        }

        #endif /* APPL_CI_ENABLED */

        pRec->cfg            = *cfg;
        pRec->currVersionPat = PSISI_VERSION_ANY;
        pRec->currVersionPmt = PSISI_VERSION_ANY;

        pFe->reqTunePar.network    = pRec->service.network;
        pFe->reqTunePar.manualMode = FFALSE;
        pFe->reqTunePar.par        = tpItem.pars;
        pFe->reqTunePar.tpIdx      = pRec->service.tpIdx;
    }
    while ( 0 );

    return retVal;
}

void AVINT_RecSignalAvailable (uint8_t tsdIdx)
{
    //AV_REC_S* pRec = avData->rec + (tsdIdx - RSRC_TSD_REC1);

    /* currently nothing to do... */
}

void AVINT_RecSignalLost (uint8_t tsdIdx)
{
    AV_REC_S* pRec = avData->rec + (tsdIdx - RSRC_TSD_REC1);

    if ( (!(pRec->active)) ||
         (pRec->pFe == NULL) ) return;

    (void)PSISI_PMT_StopFiltering(tsdIdx);

    /* Trigger new filtering of PAT/PMT after signal is available again,
       tables might have been updated during non-signal phase. */
    pRec->currVersionPat = PSISI_VERSION_ANY;
}

void AVINT_RecTasks (uint8_t tsdIdx)
{
    AV_REC_S*          pRec  = avData->rec + (tsdIdx - RSRC_TSD_REC1);
    const PSISI_PAT_S* pPat  = PSISI_PAT_GetCurrent(tsdIdx);
    const PSISI_PMT_S* pPmt  = PSISI_PMT_GetCurrent(tsdIdx);
    uint16_t           pmtPid;

    if ( !(pRec->active) ) return;

    /* --------------------------------------------------------------------- */

    if ( (pPat != NULL) && (pPat->sect.allAvailable) )
    {
        /* PAT update */
        if ( pPat->version_number != pRec->currVersionPat )
        {
            FAPI_SYS_PRINT_DEBUG(4,"[REC] PAT RECEIVED: version %i \n",
                       pPat->version_number);

            pRec->currVersionPat = pPat->version_number;
            pRec->currVersionPmt = PSISI_VERSION_ANY;

            /* Invalidate current PMT */
            (void)PSISI_PMT_Invalidate (tsdIdx);

            /* Get PMT PID from PAT */
            if ( PSISI_PAT_GetPmtPid (pPat, pRec->service.serviceId,
                                      &pmtPid) == FAPI_OK )
            {
                (void)PSISI_PMT_Invalidate     (tsdIdx);
                (void)PSISI_PMT_StartFiltering (tsdIdx, pRec->service.serviceId,
                                          pmtPid, PSISI_VERSION_ANY, FTRUE);
            }
            else
            {
                /* do nothing */
            }
        }

        /* PMT update */
        if ( (pPmt->available) &&
             (pPmt->version_number != pRec->currVersionPmt) )
        {
            FAPI_SYS_PRINT_DEBUG(4,"[REC] PMT received: version %i\n",
                       pPmt->version_number);

            pRec->currVersionPmt = pPmt->version_number;

            #ifdef APPL_CI_ENABLED

            if ( pRec->cfg.useCi )
            {
                CISYS_PmtReceived (pRec->cfg.servIdx, pRec->cfg.ts, pPmt);
            }

            #endif /* APPL_CI_ENABLED */

            (void)PSISI_PMT_NotifyReception (tsdIdx);

            /* re-start PMT filtering for non-equal version */
            (void)PSISI_PMT_StartFiltering (tsdIdx,
                                      pPmt->program_number,
                                      pPmt->pid,
                                      pPmt->version_number, FFALSE);
        }
    }

    /* --------------------------------------------------------------------- */

    #ifdef APPL_CI_ENABLED

    /* test if CI stack waits for PMT */
    if ( pPmt->available )
    {
        if ( pRec->cfg.useCi )
        {
            CISYS_Task (pRec->cfg.ts, pPmt);
        }
    }

    #endif /* APPL_CI_ENABLED */
}

/* ------------------------------------------------------------------------- */
/*
servIdx: for time shift playback only: ID of recorded service to be played
*/
void AVINT_PlayStart (PVRLST_IDX lstIdx, SLIST_SERVIDX servIdx,
                      const RSRC_CFG_S* cfg)
{
    AV_PLAY_S*        pPlay = &(avData->play);
    PVR_LIST_ITEM_S   pvrItem;

    if ( pPlay->active ) return;

    if ( PVR_LIST_GetItemByIndex (lstIdx, &pvrItem, NULL) != FAPI_OK )
    {
        return;
    }

    AVINT_RouteSignal(cfg);

    PSISI_PAT_Invalidate(&(pPlay->pat)); /* Invalidate playback PAT */

    (void)PSISI_SetTsdUsage (cfg->op, cfg->tsdIdx, &(pPlay->pat));

    AV_OutputFormatUpdate (FFALSE);
    /* ----- start EIT filtering ------------------------------------------- */

    (void)EIT_FilteringStart (SLIST_SERVIDX_PLAY, pvrItem.service.serviceId);

    /* --------------------------------------------------------------------- */

    #ifdef APPL_PARENTAL_ENABLED

    PCTRL_PlayStartHandling(&pvrItem, servIdx);

    #endif /* APPL_PARENTAL_ENABLED */

    #ifdef APPL_CI_ENABLED

    if ( cfg->useCi )
    {
        CISYS_ServiceSelected (cfg->servIdx, cfg->ts);
    }

    #endif /* APPL_CI_ENABLED */

    pPlay->active         = FTRUE;
    pPlay->internal       = FFALSE;
    pPlay->cfg            = *cfg;
    pPlay->currVersionPat = PSISI_VERSION_ANY;
    pPlay->currVersionPmt = PSISI_VERSION_ANY;
}

void AVINT_PlayStop ()
{
    AV_PLAY_S* pPlay = &(avData->play);

    if ( (!(pPlay->active)) ||
           (pPlay->internal) ) return;

    #ifdef APPL_SUBTT_ENABLED
    SUBTT_Clear  ();
    SUBTT_Enable (FTRUE, SYS_SUSPENDSRC_PLAY);
    #endif /* APPL_SUBTT_ENABLED */

    TTXSUBTT_Clear  ();

    #ifdef APPL_TTXT_ENABLED
    TTXSUBTT_Enable (FTRUE, SYS_SUSPENDSRC_PLAY);
    #endif /* APPL_TTXT_ENABLED */

    #ifdef APPL_MHEG_ENABLED
    MHEG_Enable(FTRUE,SYS_SUSPENDSRC_PLAY);
    #endif
    (void)EIT_FilteringStop ();

    #ifdef APPL_PARENTAL_ENABLED

    PCTRL_Stop();

    #endif /* APPL_PARENTAL_ENABLED */

    #ifdef APPL_CI_ENABLED

    if ( pPlay->cfg.useCi )
    {
        CISYS_ServiceStop(pPlay->cfg.servIdx, pPlay->cfg.ts);
    }

    #endif /* APPL_CI_ENABLED */

    (void)PSISI_ReleaseTsdUsage (pPlay->cfg.op, pPlay->cfg.tsdIdx);

    PSISI_PAT_Invalidate(&(pPlay->pat)); /* Invalidate playback PAT */

    pPlay->active = FFALSE;
}

uint32_t AVINT_PlayTasks ()
{
    AV_PLAY_S*         pPlay = &(avData->play);
    const PSISI_PAT_S* pPat  = PSISI_PAT_GetCurrent(pPlay->cfg.tsdIdx);
    const PSISI_PMT_S* pPmt  = PSISI_PMT_GetCurrent(pPlay->cfg.tsdIdx);
    uint16_t           prgIdx;
    uint16_t           prgNum;

    if ( (!(pPlay->active)) ||
           (pPlay->internal) ) return AV_SLEEP_PLAY_IDLE;

    /* --------------------------------------------------------------------- */

    if ( (pPat != NULL) && (pPat->sect.allAvailable) )
    {
        /* PAT update */
        if ( pPat->version_number != pPlay->currVersionPat )
        {
            FAPI_SYS_PRINT_DEBUG(4,"[PLAY] PAT RECEIVED: version %i \n",
                       pPat->version_number);

            pPlay->currVersionPat = pPat->version_number;
            pPlay->currVersionPmt = PSISI_VERSION_ANY;

            /* Invalidate current PMT */
            (void)PSISI_PMT_Invalidate (pPlay->cfg.tsdIdx);

            /* Get PMT PID from PAT */
            for ( prgIdx = 0; prgIdx < pPat->numPrograms; prgIdx++ )
            {
                prgNum = pPat->program[prgIdx].program_number;

                /* assume first non-zero program */
                if ( prgNum != 0 )
                {
                    (void)PSISI_PMT_Invalidate     (pPlay->cfg.tsdIdx);
                    (void)PSISI_PMT_StartFiltering (pPlay->cfg.tsdIdx, prgNum,
                                                    pPat->program[prgIdx].
                                                    program_map_pid,
                                                    PSISI_VERSION_ANY, FTRUE);
                    break;
                }
            }

            if ( prgIdx >= pPat->numPrograms )
            {
                /* do nothing for the moment */
            }
        }

        /* PMT update */
        if ( (pPmt->available) &&
             (pPmt->version_number != pPlay->currVersionPmt) )
        {
            FAPI_SYS_PRINT_DEBUG(4,"[PLAY] PMT RECEIVED: version %i \n",
                       pPmt->version_number);

            pPlay->currVersionPmt = pPmt->version_number;

            #ifdef APPL_SUBTT_ENABLED

            (void)SUBTT_PmtUpdate (pPmt, PVRSYS_PlayPidAvailable);

            #endif /* APPL_SUBTT_ENABLED */

            (void)TTXSUBTT_PmtUpdate (pPmt, PVRSYS_PlayPidAvailable);

            AVINT_SubtPmtUpdate();

            #ifdef APPL_CI_ENABLED

            if ( pPlay->cfg.useCi )
            {
                CISYS_PmtReceived (pPlay->cfg.servIdx, pPlay->cfg.ts, pPmt);
            }

            #endif /* APPL_CI_ENABLED */

            (void)PSISI_PMT_NotifyReception (pPlay->cfg.tsdIdx);
            (void)PSISI_PMT_StartFiltering  (pPlay->cfg.tsdIdx,
                                             pPmt->program_number,
                                             pPmt->pid,
                                             pPmt->version_number, FFALSE);
        }
    }

    if ( avData->itcInst.abortReq ) return 0;

    /* --------------------------------------------------------------------- */

    #ifdef APPL_CI_ENABLED

    /* test if CI stack waits for PMT */
    if ( pPmt->available )
    {
        if ( pPlay->cfg.useCi )
        {
            CISYS_Task (pPlay->cfg.ts, pPmt);
        }
    }

    #endif /* APPL_CI_ENABLED */

    if ( avData->itcInst.abortReq ) return 0;

    /* --------------------------------------------------------------------- */

    #ifdef APPL_PARENTAL_ENABLED

    PCTRL_Task();

    #endif /* APPL_PARENTAL_ENABLED */

    return AV_SLEEP_PLAY_ACTIVE;
}

/* ------------------------------------------------------------------------- */

#if defined(APPL_CI_ENABLED) && defined(APPL_DESCRREC_ENABLED)

int32_t AVINT_DescrRecStart (const RSRC_CFG_S* cfg)
{
    AV_PLAY_S*  pPlay = &(avData->play);
    RSRC_OP     op;

    if ( pPlay->active ) return FAPI_OK;

    AVINT_RouteSignal(cfg);

    PSISI_PAT_Invalidate(&(pPlay->pat)); /* Invalidate playback PAT */

    /* Use recording operation IDs here. */
    if   ( cfg->tsdIdx == RSRC_TSD_REC1 ) op = RSRC_OP_REC1;
    else                                  op = RSRC_OP_REC2;

    PSISI_SetTsdUsage (op, cfg->tsdIdx, &(pPlay->pat));

    /* NOTE: CAT is not filtered for updates in order to avoid filter
             conflicts with recording engine. After CAT has been received,
             recording will start and set a filter on the CAT PID. */

    PSISI_CAT_Invalidate     (cfg->tsdIdx);
    PSISI_CAT_StartFiltering (cfg->tsdIdx, PSISI_VERSION_ANY, FFALSE,
                              FFALSE, NULL);

    if ( cfg->useCi )
    {
        CISYS_ServiceSelected(cfg->servIdx, cfg->ts);
    }

    pPlay->active         = FTRUE;
    pPlay->internal       = FTRUE;
    pPlay->cfg            = *cfg;
    pPlay->currVersionPat = PSISI_VERSION_ANY;
    pPlay->currVersionPmt = PSISI_VERSION_ANY;

    return FAPI_OK;
}

int32_t AVINT_DescrRecStop ()
{
    AV_PLAY_S* pPlay = &(avData->play);
    RSRC_OP    op;

    if ( (!(pPlay->active)) ||
         (!(pPlay->internal)) ) return FAPI_OK;

    if ( pPlay->cfg.useCi )
    {
        CISYS_ServiceStop (pPlay->cfg.servIdx, pPlay->cfg.ts);
    }

    PSISI_CAT_StopFiltering (pPlay->cfg.tsdIdx);

    /* Use recording operation IDs here. */
    if   ( pPlay->cfg.tsdIdx == RSRC_TSD_REC1 ) op = RSRC_OP_REC1;
    else                                        op = RSRC_OP_REC2;

    PSISI_ReleaseTsdUsage (op, pPlay->cfg.tsdIdx);

    pPlay->active = FFALSE;

    return FAPI_OK;
}

int32_t AVINT_DescrRecReroute (const RSRC_CFG_S* cfg)
{
    AV_PLAY_S* pPlay = &(avData->play);

    if ( (!(pPlay->active)) ||
         (!(pPlay->internal)) ) return FAPI_OK;

    /* NOTE: It is assumed here that the following configuration parameters
             have not changed:
                        - servIdx
                        - tsdIdx
    */

    if ( pPlay->cfg.useCi )
    {
        CISYS_ServiceStop (pPlay->cfg.servIdx, pPlay->cfg.ts);
    }

    AVINT_RouteSignal(cfg);

    if ( cfg->useCi )
    {
        CISYS_ServiceSelected(cfg->servIdx, cfg->ts);
    }

    pPlay->cfg = *cfg;

    return FAPI_OK;
}

uint32_t AVINT_DescrRecTasks ()
{
    AV_PLAY_S*         pPlay = &(avData->play);
    const PSISI_PAT_S* pPat  = PSISI_PAT_GetCurrent(pPlay->cfg.tsdIdx);
    const PSISI_PMT_S* pPmt  = PSISI_PMT_GetCurrent(pPlay->cfg.tsdIdx);
    uint16_t           prgIdx;
    uint16_t           prgNum;

    if ( (!(pPlay->active)) ||
         (!(pPlay->internal)) ) return AV_SLEEP_PLAY_IDLE;

    /* --------------------------------------------------------------------- */

    if ( (pPat != NULL) && (pPat->sect.allAvailable) )
    {
        /* PAT update */
        if ( pPat->version_number != pPlay->currVersionPat )
        {
            FAPI_SYS_PRINT_DEBUG(4,"[DREC] PAT RECEIVED: version %i \n",
                       pPat->version_number);

            pPlay->currVersionPat = pPat->version_number;
            pPlay->currVersionPmt = PSISI_VERSION_ANY;

            /* Invalidate current PMT */
            PSISI_PMT_Invalidate (pPlay->cfg.tsdIdx);

            /* Get PMT PID from PAT */
            for ( prgIdx = 0; prgIdx < pPat->numPrograms; prgIdx++ )
            {
                prgNum = pPat->program[prgIdx].program_number;

                /* assume first non-zero program */
                if ( prgNum != 0 )
                {
                    PSISI_PMT_Invalidate     (pPlay->cfg.tsdIdx);
                    PSISI_PMT_StartFiltering (pPlay->cfg.tsdIdx, prgNum,
                                              pPat->program[prgIdx].
                                              program_map_pid,
                                              PSISI_VERSION_ANY, FTRUE);
                    break;
                }
            }

            if ( prgIdx >= pPat->numPrograms )
            {
                /* do nothing for the moment */
            }
        }

        /* PMT update */
        if ( (pPmt->available) &&
             (pPmt->version_number != pPlay->currVersionPmt) )
        {
            FAPI_SYS_PRINT_DEBUG(4,"[DREC] PMT RECEIVED: version %i \n",
                       pPmt->version_number);

            pPlay->currVersionPmt = pPmt->version_number;

            if ( pPlay->cfg.useCi )
            {
                CISYS_PmtReceived (pPlay->cfg.servIdx, pPlay->cfg.ts, pPmt);
            }

            PSISI_PMT_NotifyReception (pPlay->cfg.tsdIdx);
            PSISI_PMT_StartFiltering  (pPlay->cfg.tsdIdx,
                                       pPmt->program_number,
                                       pPmt->pid,
                                       pPmt->version_number, FFALSE);
        }
    }

    if ( avData->itcInst.abortReq ) return 0;

    /* --------------------------------------------------------------------- */

    /* test if CI stack waits for PMT */
    if ( pPmt->available )
    {
        if ( pPlay->cfg.useCi )
        {
            CISYS_Task (pPlay->cfg.ts, pPmt);
        }
    }

    return AV_SLEEP_PLAY_ACTIVE;
}

#endif /* defined(APPL_CI_ENABLED) && defined(APPL_DESCRREC_ENABLED) */

#ifdef APPL_MPEG_ENCODER_ENABLED

int32_t AVINT_RecMpencStart (const RSRC_CFG_S* cfg)
{
    AV_REC_S*   pRec   = avData->rec + (cfg->tsdIdx - RSRC_TSD_REC1);
    AV_MPENC_S* pMpenc = &(avData->mpenc);

    if ( pRec->active ) return FAPI_OK; /* already active */

    memset (&(pRec->service), 0, sizeof(pRec->service));

    pRec->service.serviceId = MPENC_SERVICE_ID;

    AVINT_RouteSignal(cfg);

    AVINT_MpencEncoderStart(cfg->op);

    PSISI_SetTsdUsage (cfg->op, cfg->tsdIdx, &(pMpenc->pat));

    pRec->active         = FTRUE;
    pRec->pFe            = NULL;
    pRec->cfg            = *cfg;
    pRec->currVersionPat = PSISI_VERSION_ANY;
    pRec->currVersionPmt = PSISI_VERSION_ANY;

    pMpenc->useStatus   |= cfg->op;

    return FAPI_OK;
}

int32_t AVINT_RecMpencStop (uint8_t tsdIdx)
{
    AV_REC_S*   pRec   = avData->rec + (tsdIdx - RSRC_TSD_REC1);
    AV_MPENC_S* pMpenc = &(avData->mpenc);

    if ( !(pRec->active) ) return FAPI_OK;

    PSISI_ReleaseTsdUsage (pRec->cfg.op, tsdIdx);

    AVINT_MpencEncoderStop(pRec->cfg.op);

    pMpenc->useStatus &= (~(pRec->cfg.op));

    pRec->active = FFALSE;
    pRec->pFe    = NULL;

    return FAPI_OK;
}

#endif /* APPL_MPEG_ENCODER_ENABLED */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
