/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/av_scan.c
**
**              $Id: av_scan.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       AV module extension for scan operation.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save -e715*/
#define DEBUG 1

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "fapex/abst_types.h"
#include "fapex/abst_libtsd.h"


#include "settings.h"
#include "init.h"

#include "av.h"
#include "event_queue.h"

#include "utils.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define AV_SLEEP_SCAN                  20 // TODO: to be defined
#define AV_SLEEP_SCANFINISHED        1000 // TODO: einfhren: unendliches schlafen!!!

#if 0
#define PAT_FILTERING_TIMEOUT       18000 // PAT/SDT filtering timeout in ms
#define PMT_FILTERING_TIMEOUT         500 // PMT filtering timeout in ms
#define NIT_FILTERING_TIMEOUT       25000 // NIT filtering timeout in ms
#else
#define PAT_FILTERING_TIMEOUT       18000 // PAT/SDT filtering timeout in ms
#define PMT_FILTERING_TIMEOUT        5000 // PMT filtering timeout in ms
#define NIT_FILTERING_TIMEOUT       25000 // NIT filtering timeout in ms

#endif

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC uint32_t    AV_TaskScanList(AV_FEDAT_S * pFe);
FSTATIC uint32_t    AV_TaskScanCurrTS(AV_FEDAT_S * pFe);

FSTATIC uint32_t    AV_ScanCarrier(AV_FEDAT_S * pFe,
                                   const SYS_NETWORK_S * network,
                                   TPLIST_IDX carrierIdx,
                                   AV_SCAN_OPTS_S * opts);
FSTATIC uint32_t    AV_ScanNIT(AV_FEDAT_S * pFe,
                               const SYS_NETWORK_S * network,
                               TPLIST_IDX carrierIdx);
FSTATIC void        AV_ScanAddService(const AV_SCAN_OPTS_S * opts,
                                      const SLIST_ITEM_S * service);
FSTATIC void        AV_ScanAddLcn(AV_FEDAT_S * pFe, SLIST_ITEM_S * service);

#ifdef APPL_BLINDSCAN_ENABLED
FSTATIC uint32_t    AV_TaskScanBlind(AV_FEDAT_S * pFe);
FSTATIC uint8_t     scanBlindAbortFct (void);
FSTATIC void        scanBlindProgressFct(uint16_t progress);
FSTATIC void        scanBlindNotifyFct(FE_TUNE_PARAMS_S * pFoundTuneParams);
#endif
/* ------------------------------------------------------------------------- */

void AVINT_StartScanOperation(AV_FEDAT_S * pFe, AV_CMD_S * pCmd)
{
    int16_t             numCarriers;

    /* reset general scan parameters */
    pFe->scanOpts = pCmd->arg.scan.opts;
    pFe->scanNetwork.type = pFe->systemType;    /* NOTE: Direct input not implemented yet! */
    pFe->scanNetwork = pCmd->arg.scan.network;
    pFe->scanNumTpScanned = 0;

    memset(pFe->scanTpDone, 0, TPLIST_CARRIERS_MAX);
    memset(&(pFe->scanEvent), 0, sizeof(pFe->scanEvent));

    pFe->scanEvent.hdr.type = EVT_SCAN_PROGRESS;
    pFe->scanEvent.hdr.length = sizeof(pFe->scanEvent);
    pFe->scanEvent.tpDone = 0;
    pFe->scanEvent.progCurr = 0;

    /* perform specific initialisations */
    switch (pCmd->type)
    {
        case AV_CMD_SCAN_CARRIER:
            pFe->scanMode = SCAN_MODE_SINGLE;

            pFe->scanEvent.status = SCAN_STAT_TPSCAN;

            pFe->scanEvent.tpMax = 1;
            pFe->scanEvent.tpNext = pCmd->arg.scan.tpLstPos;
            pFe->scanEvent.progMax = 1;

            if ( TPLIST_GetIndex(&(pFe->scanNetwork),
                                 (int16_t)(pFe->scanEvent.tpNext),
                                 &(pFe->scanIdxNext)) != FAPI_OK )
            {
                return;
            }

            pFe->scanState = AV_SCANSTATE_START;
            pFe->scanProgressSent = FFALSE;
            break;

        case AV_CMD_SCAN_LIST:
            pFe->scanMode = SCAN_MODE_LIST;

            pFe->scanEvent.status = SCAN_STAT_TPSCAN;

            if ( TPLIST_GetNumCarriers(&(pFe->scanNetwork), &numCarriers)
                 != FAPI_OK )
            {
                return;
            }

            pFe->scanEvent.tpMax   = (uint16_t)numCarriers;
            pFe->scanEvent.tpNext  = 0;
            pFe->scanEvent.progMax = (uint16_t)numCarriers;

            if ( TPLIST_GetIndex(&(pFe->scanNetwork),
                                 (int16_t)(pFe->scanEvent.tpNext),
                                 &(pFe->scanIdxNext)) != FAPI_OK )
            {
                return;
            }

            pFe->scanState = AV_SCANSTATE_START;
            pFe->scanProgressSent  = FFALSE;
            break;

        case AV_CMD_SCAN_INPUT:
            pFe->scanMode          = SCAN_MODE_CURRENTTS;

            pFe->scanEvent.status  = SCAN_STAT_TPSCAN;
            pFe->scanEvent.tpMax   = 1;
            pFe->scanEvent.tpNext  = 0;
            pFe->scanEvent.progMax = 1;
            break;
        #ifdef APPL_BLINDSCAN_ENABLED
        case AV_CMD_SCAN_BLIND:
            pFe->scanMode  = SCAN_MODE_BLIND;
            pFe->scanState = AV_SCANSTATE_START;

            pFe->scanEvent.status  = SCAN_STAT_SCAN_HOR; /* horizontal frequencies
                                                           will be scanned first */
            avData->scanBlindFe       = pCmd->arg.scan.cfg.feIdx; /* must be globally stored */
            avData->scanBlindProgLvl  = 0;

            pFe = avData->feDat + avData->scanBlindFe;
            pFe->scanEvent.hdr.type   = EVT_SCAN_PROGRESS;
            pFe->scanEvent.hdr.length = sizeof(pFe->scanEvent);
            pFe->scanEvent.mode       = SCAN_MODE_BLIND;
            pFe->scanEvent.tpDone     = 0;
            pFe->scanEvent.tpMax      = 0;
            pFe->scanEvent.tpNext     = 0;
            pFe->scanEvent.progMax    = 1000;
            pFe->scanEvent.status = SCAN_STAT_SCAN_HOR;
            pFe->scanEvent.progCurr = 0;

            if(TPLIST_GetNumCarriers(&(pFe->scanNetwork),&numCarriers) == FAPI_OK)
            {
                /* update total number of transponders in list */
                pFe->scanEvent.tpMax   = (uint16_t)numCarriers;
                pFe->scanEvent.progMax = (uint16_t)numCarriers;
            }
            break;
            #endif
        default:
            return;
    }

    AVINT_RouteSignal(&(pCmd->arg.scan.cfg));

    (void)PSISI_SetTsdUsage (RSRC_OP_SCAN, pCmd->arg.scan.cfg.tsdIdx,
                             &(pFe->pat));

    pFe->state = AV_FESTATE_SCANNING;

    pFe->scanEvent.mode = pFe->scanMode;

    /* send initial event: job started */
    (void)EVT_Send(&(pFe->scanEvent), "Scan start");
}

void AVINT_AbortScanOperation(AV_FEDAT_S * pFe)
{
    fbool_t sendEvent = FFALSE;

    if(pFe->state != AV_FESTATE_SCANNING)
        return;

    switch (pFe->scanMode)
    {
        case SCAN_MODE_SINGLE:
        case SCAN_MODE_LIST:
        case SCAN_MODE_CURRENTTS:
            sendEvent = FTRUE;
            break;

        case SCAN_MODE_BLIND:
            // TODO: Abort blind scan in separate thread!
            sendEvent = FTRUE;
            break;

        case SCAN_MODE_FINISHED:
            /* do nothing, do not send "aborted" event */
            break;

        default:
            break;
    }

    /* stop all filtering */
    (void)PSISI_SDT_StopFiltering();
    PSISI_SDT_Invalidate(&(avData->sdt));
    (void)PSISI_NIT_StopFiltering();
    (void)PSISI_ReleaseTsdUsage (RSRC_OP_SCAN, RSRC_TSD_SCAN);

    pFe->state = AV_FESTATE_IDLE;

    if(sendEvent)
    {
        pFe->scanEvent.status = SCAN_STAT_ABORTED;
        pFe->scanEvent.tpNext = 0;

        (void)EVT_Send(&(pFe->scanEvent), "Scan progress");
    }
}

uint32_t AV_TaskScan(AV_FEDAT_S * pFe)
{
    uint32_t            nextSlotTime = AV_SLEEP_SCAN;

    switch (pFe->scanMode)
    {
        case SCAN_MODE_SINGLE:
        case SCAN_MODE_LIST:
            nextSlotTime = AV_TaskScanList(pFe);
            break;
        #ifdef APPL_BLINDSCAN_ENABLED
        case SCAN_MODE_BLIND:
            nextSlotTime = AV_TaskScanBlind(pFe);
            break;
        #endif
        case SCAN_MODE_CURRENTTS:
            nextSlotTime = AV_TaskScanCurrTS(pFe);
            break;

        case SCAN_MODE_FINISHED:
            /* scan has been finished, wait for stop command */
            nextSlotTime = AV_SLEEP_SCANFINISHED;
            break;

        default:
            break;
    }

    return nextSlotTime;
}

FSTATIC uint32_t AV_TaskScanList(AV_FEDAT_S * pFe)
{
    uint32_t            nextSlotTime = 0;
    int16_t             numCarriers;
    int16_t             cnt;
    TPLIST_IDX          nextIdx;

    if(pFe->scanNumTpScanned >= pFe->scanEvent.tpMax)
    {
        /* final progress event */
        pFe->scanEvent.status = SCAN_STAT_COMPLETED;
        pFe->scanEvent.tpNext = 0;

        (void)EVT_Send(&(pFe->scanEvent), "Scan complete");

        pFe->scanMode = SCAN_MODE_FINISHED;
    }
    else
    {
        switch (pFe->scanState)
        {
            case AV_SCANSTATE_START:
                pFe->scanTpState  = AV_SCANTP_TUNE;
                pFe->scanNitState = AV_SCANNIT_TUNE;
                pFe->scanSdtStarted = FFALSE;
                pFe->scanTsId     = -1;

                pFe->scanState = AV_SCANSTATE_GETCARRIERS;
                break;

            case AV_SCANSTATE_GETCARRIERS:

                if(!(pFe->scanOpts.scanNIT))
                {
                    pFe->scanTpState = AV_SCANTP_TUNE;
                    pFe->scanState = AV_SCANSTATE_GETSERVICES;
                }
                else
                {
                    nextSlotTime = AV_ScanNIT(pFe, &(pFe->scanNetwork),
                                              pFe->scanIdxNext);

                    if(pFe->scanSdtStarted == FFALSE)
                    {
                        /* Start filtering for SDT (PAT is started automatically) */
                        FAPI_SYS_PRINT_DEBUG(1,"[SCAN] Search SDT: START...\n");
                        (void)PSISI_SDT_StopFiltering();
                        PSISI_SDT_Invalidate(&(avData->sdt));
                        if( PSISI_SDT_StartFiltering (PSISI_VERSION_ANY, FTRUE)
                           != FAPI_OK)
                        {
                            /* opening of filters failed */
                            pFe->scanTpState = AV_SCANTP_FINISHED;
                            break;
                        }
                        pFe->scanSdtStarted = FTRUE;
                    }
                    /* if NIT scanning is finished, proceed to next state */
                    if(pFe->scanNitState == AV_SCANNIT_FINISHED)
                    {
                        if(pFe->scanMode == SCAN_MODE_LIST)
                        {
                            /* In list scan mode, newfound carriers are
                               included in scan list. */

                            if(TPLIST_GetNumCarriers(&(pFe->scanNetwork),
                                                     &numCarriers) == FAPI_OK)
                            {
                                /* update total number of transponders in list */
                                pFe->scanEvent.tpMax   = (uint16_t)numCarriers;
                                pFe->scanEvent.progMax = (uint16_t)numCarriers;
                            }
                        }

                        //pFe->scanTpState = AV_SCANTP_TUNE;
                        pFe->scanTpState = AV_SCANTP_PATSDT;
                        pFe->scanTimeStamp = SYS_GET_TIMESTAMP_MS;
                        pFe->scanState = AV_SCANSTATE_GETSERVICES;
                    }
                }
                break;

            case AV_SCANSTATE_GETSERVICES:
                nextSlotTime = AV_ScanCarrier(pFe, &(pFe->scanNetwork),
                                              pFe->scanIdxNext,
                                              &(pFe->scanOpts));

                /* if scanning is finished, proceed to next state */
                if(pFe->scanTpState == AV_SCANTP_FINISHED)
                {
                    pFe->scanState = AV_SCANSTATE_FINISHED;
                }
                break;

            case AV_SCANSTATE_FINISHED:
                pFe->scanEvent.tpDone++;
                pFe->scanEvent.progCurr++;
                pFe->scanSdtStarted = FFALSE;

                /* Mark TP as scanned: If list scan is restarted due to NIT found,
                   it will not be scanned again! */
                if(pFe->scanMode == SCAN_MODE_LIST)
                {
                    DBG_Assert(pFe->scanIdxNext < TPLIST_CARRIERS_MAX);

                    pFe->scanTpDone[pFe->scanIdxNext] = 1;
                }

                pFe->scanNumTpScanned++;

                /* reset states for next carrier scan */
                pFe->scanState = AV_SCANSTATE_START;
                pFe->scanProgressSent  = FFALSE;

                /* Search for next carrier to scan */
                if(pFe->scanNumTpScanned < pFe->scanEvent.tpMax)
                {
                    for ( cnt = 0; cnt < (int16_t) (pFe->scanEvent.tpMax);
                          cnt++ )
                    {
                        /* get absolute carrier index */
                        if(TPLIST_GetIndex(&(pFe->scanNetwork), cnt,
                                           &nextIdx) != FAPI_OK)
                        {
                            continue;
                        }

                        /* ensure that entry has not been scanned before */
                        if(pFe->scanTpDone[nextIdx] != 0)
                        {
                            continue;
                        }

                        pFe->scanEvent.tpNext = (uint16_t)cnt;
                        pFe->scanIdxNext      = nextIdx;

                        if(!(pFe->scanProgressSent))    // FIXME: needed?
                        {
                            (void)EVT_Send(&(pFe->scanEvent), "Scan progress");
                            pFe->scanProgressSent = FTRUE;
                        }

                        break;
                    }

                    /* this should never happen */
                    if(cnt >= (int16_t) (pFe->scanEvent.tpMax))
                    {
                        pFe->scanNumTpScanned = pFe->scanEvent.tpMax;
                    }
                }

                nextSlotTime = 0;
                break;

            default:
                break;
        }
    }

    return nextSlotTime;
}

#ifdef APPL_BLINDSCAN_ENABLED
FSTATIC uint32_t AV_TaskScanBlind(AV_FEDAT_S * pFe)
{
    uint32_t            nextSlotTime = 0;
    int16_t             cnt;
    TPLIST_IDX          nextIdx;
    FE_SCAN_PARAMS_S    scanPars;
    int32_t             retVal;
    uint32_t            numSteps = 0;

    #ifdef APPL_FESAT_ENABLED

    CFGSAT_FEPARS_S     feCfg;
    CFGSAT_SAT_S        satInf;
    FE_SYSTEM_PARAMS_S  systemParams;

    #endif /* APPL_FESAT_ENABLED */

    if( ( pFe->scanNumTpScanned >= pFe->scanEvent.tpMax) &&
         (pFe->scanEvent.tpMax  != 0 ) )
    {
        pFe->scanEvent.progCurr = pFe->scanEvent.progMax;
        (void)EVT_Send(&(pFe->scanEvent), "Scan progress");

        /* final progress event */
        pFe->scanEvent.status = SCAN_STAT_COMPLETED;
        pFe->scanEvent.tpNext = 0;

        (void)EVT_Send(&(pFe->scanEvent), "Scan complete");

        pFe->scanMode = SCAN_MODE_FINISHED;
    }
    else
    {
        switch (pFe->scanState)
        {
            case AV_SCANSTATE_START:
                pFe->scanTpState  = AV_SCANTP_TUNE;
                pFe->scanNitState = AV_SCANNIT_TUNE;
                pFe->scanTsId     = -1;

                if(avData->scanBlindProgLvl == 0)
                {
                    pFe->scanState = AV_SCANSTATE_GETCARRIERS;
                }
                else
                {
                    pFe->forceTuning = FTRUE;
                    pFe->scanOpts.scanNIT = FFALSE;
                    pFe->scanTpState = AV_SCANTP_TUNE;
                    pFe->scanState = AV_SCANSTATE_GETSERVICES;
                    pFe->scanEvent.status =  SCAN_STAT_TPSCAN;
                    (void)EVT_Send(&(pFe->scanEvent), "Scan progress");
                }
                break;

            case AV_SCANSTATE_GETCARRIERS:
                pFe->scanEvent.hdr.type   = EVT_SCAN_PROGRESS;
                pFe->scanEvent.hdr.length = sizeof(pFe->scanEvent);
                pFe->scanEvent.mode       = SCAN_MODE_BLIND;
                pFe->scanEvent.tpDone     = 0;
                pFe->scanEvent.tpMax      = 0;
                pFe->scanEvent.tpNext     = 0;

                switch(pFe->scanNetwork.type)
                {
                    #ifdef APPL_FESAT_ENABLED

                    case FE_SYSTEM_TYPE_SATELLITE:
                        numSteps = 3;       /* 3 steps required */

                        /* get global frontend configuration */
                        retVal = CFGSAT_GetFrontendSetup (&feCfg);
                        if ( retVal != FAPI_OK )
                        {
                            pFe->scanState = AV_SCANSTATE_FINISHED;
                            break;
                        }

                        /* Get LNB's HI- and LO-band frequencies */
                        retVal = CFGSAT_GetSatData (pFe->scanNetwork.satIdx, &satInf);
                        if(retVal != FAPI_OK)
                        {
                            pFe->scanState = AV_SCANSTATE_FINISHED;
                            break;
                        }
                        if(satInf.lnbCfg[pFe->feIdx].active == FFALSE)
                        {
                            pFe->scanState = AV_SCANSTATE_FINISHED;
                            break;
                        }
                        scanPars.startFrequency  =  950000 + // LO-Band freq + QST_MIN
                                                   satInf.lnbCfg[pFe->feIdx].freqLo;
                        scanPars.stopFrequency   = 2150000 + // HI-Band freq + QST_MAX
                                                   satInf.lnbCfg[pFe->feIdx].freqHi;
                        scanPars.startSymbolRate =    2000;
                        scanPars.stopSymbolRate  =   45000;

                        switch ( feCfg.controlMode[pFe->feIdx] )
                        {
                            default:
                            case CFGSAT_FEMODE_CLASSIC:
                                scanPars.satID = 0;
                                break;

                            #ifdef APPL_SATPOSBY22KHZ_ENABLED
                            case CFGSAT_FEMODE_POSBY22KHZ:
                            #endif
                            case CFGSAT_FEMODE_TONEBURST:
                                scanPars.satID = satInf.lnbCfg[pFe->feIdx].toneburstId;
                                break;


                            case CFGSAT_FEMODE_DISEQC10:
                                scanPars.satID = satInf.lnbCfg[pFe->feIdx].diseqc10Id;
                                break;

                            case CFGSAT_FEMODE_DISEQC11_DTH:
                                scanPars.satID = satInf.lnbCfg[pFe->feIdx].diseqc11Id;
                                break;

                            #ifdef APPL_DISEQC11_SMATV_ENABLED

                            case CFGSAT_FEMODE_DISEQC11_SMATV:
                                scanPars.satID  = satInf.lnbCfg[pFe->feIdx].diseqc11Id_SMATV;
                                break;

                            #endif /* APPL_DISEQC11_SMATV_ENABLED */

                            #ifdef APPL_DISEQC12_ENABLED

                            case CFGSAT_FEMODE_DISEQC12:
                                scanPars.satID = satInf.lnbCfg[pFe->feIdx].diseqc12Pos;
                                break;

                            #endif /* APPL_DISEQC12_ENABLED */
                        }
                        break;

                    #endif /* APPL_FESAT_ENABLED */

                    #if 0 //def APPL_FETERR_ENABLED
                        //NOT supported yet
                    case FE_SYSTEM_TYPE_TERRESTRIAL:
                        numSteps = 2;       /* 2 steps required */

                        scanPars.startFrequency    = 177500;
                        scanPars.stopFrequency     = 858000;
                        scanPars.startSymbolRate   = 0;
                        scanPars.stopSymbolRate    = 0;
                        scanPars.satID             = 0;
                        break;

                    #endif /* APPL_FETERR_ENABLED */

                    #if 0 //def APPL_FECABLE_ENABLED
                        //NOT supported yet
                    case FE_SYSTEM_TYPE_CABLE:
                        numSteps = 2;       /* 2 steps required */

                        scanPars.startFrequency    = 100000;
                        scanPars.stopFrequency     = 900000;
                        scanPars.startSymbolRate   = 20000;
                        scanPars.stopSymbolRate    = 45000;
                        scanPars.satID             = 0;
                        break;

                    #endif /* APPL_FECABLE_ENABLED */

                    #if 0 //def APPL_FEDIRECT_ENABLED
                        //NOT supported yet
                    case FE_SYSTEM_TYPE_DIRECT:
                        numSteps = 2;       /* 2 steps required */

                        scanPars.startFrequency    = 100000;
                        scanPars.stopFrequency     = 100000;
                        scanPars.startSymbolRate   = 10000;
                        scanPars.stopSymbolRate    = 10000;
                        scanPars.satID             = 0;
                        break;

                    #endif /* APPL_FEDIRECT_ENABLED */

                    default:
                        pFe->scanState = AV_SCANSTATE_FINISHED;
                        break;
                }

                scanPars.custom            = NULL;
                scanPars.feScanAbortFct    = scanBlindAbortFct;
                scanPars.feScanProgressFct = scanBlindProgressFct;
                scanPars.feScanNotifyFct   = scanBlindNotifyFct;

                pFe->scanEvent.progMax  = numSteps * 1000;

                #ifdef APPL_FESAT_ENABLED

                /* ----- update system parameters ---------------------------------- */

                if(pFe->scanNetwork.type == FE_SYSTEM_TYPE_SATELLITE)
                {
                    memset(&systemParams,0,sizeof(systemParams));
                    retVal = FE_GetSystemParams(pFe->drvHdl, &systemParams);
                    if(retVal != FAPI_OK)
                    {
                        pFe->scanState = AV_SCANSTATE_FINISHED;
                        break;
                    }
                    retVal = AV_SetSystemParamsSat(pFe->feIdx,
                                                   pFe->scanNetwork.satIdx,
                                                   &systemParams);
                    if ( retVal != FAPI_OK )
                    {
                        pFe->scanState = AV_SCANSTATE_FINISHED;
                        break;
                    }
                    retVal = FE_SetSystemParams(pFe->drvHdl, &systemParams);
                    if(retVal != FAPI_OK)
                    {
                        pFe->scanState = AV_SCANSTATE_FINISHED;
                        break;
                    }
                }

                #endif /* APPL_FESAT_ENABLED */

                    /* ----- scan for horizontal polarisation -------------------------- */
                scanPars.pol = FE_POL_HOR;
                avData->scanBlindProgLvl = 1;
                pFe->scanEvent.status = SCAN_STAT_SCAN_HOR;
                pFe->scanEvent.progCurr = 0;

                (void)EVT_Send(&(pFe->scanEvent), "Scan progress");

                retVal = FE_Scan(pFe->drvHdl, &scanPars);
                if(retVal != FAPI_OK)
                {
                    pFe->scanState = AV_SCANSTATE_FINISHED;
                    break;
                }

                if(avData->itcInst.abortReq)
                    break;

                /* ----- scan for vertical polarisation ---------------------------- */

                #ifdef APPL_FESAT_ENABLED

                pFe->scanEvent.progCurr = 1000;
                (void)EVT_Send(&(pFe->scanEvent), "Scan progress");
                if(pFe->scanNetwork.type == FE_SYSTEM_TYPE_SATELLITE)
                {
                    scanPars.pol = FE_POL_VER;
                    avData->scanBlindProgLvl = 2;
                    pFe->scanEvent.status = SCAN_STAT_SCAN_VER;

                    (void)EVT_Send(&(pFe->scanEvent), "Scan progress");

                    retVal = FE_Scan(pFe->drvHdl, &scanPars);
                    if(retVal != FAPI_OK)
                    {
                        pFe->scanState = AV_SCANSTATE_FINISHED;
                        break;
                    }

                    if(avData->itcInst.abortReq)
                        break;
                    pFe->scanEvent.progCurr = 2000;
                    (void)EVT_Send(&(pFe->scanEvent), "Scan progress");
                }

                #endif /* APPL_FESAT_ENABLED */

                pFe->scanTpState = AV_SCANTP_TUNE;
                pFe->scanState = AV_SCANSTATE_GETSERVICES;
                break;

            case AV_SCANSTATE_GETSERVICES:
                nextSlotTime = AV_ScanCarrier(pFe, &(pFe->scanNetwork),
                                              pFe->scanIdxNext,
                                              &(pFe->scanOpts));
                /* if scanning is finished, proceed to next state */
                if(pFe->scanTpState == AV_SCANTP_FINISHED)
                {
                    pFe->scanState = AV_SCANSTATE_FINISHED;
                }
                break;

            case AV_SCANSTATE_FINISHED:
                pFe->scanEvent.tpDone++;
                pFe->scanEvent.progCurr+=10;

                /* Mark TP as scanned: If list scan is restarted due to NIT found,
                   it will not be scanned again! */
                if(pFe->scanMode == SCAN_MODE_LIST)
                {
                    DBG_Assert(pFe->scanIdxNext < TPLIST_CARRIERS_MAX);

                    pFe->scanTpDone[pFe->scanIdxNext] = 1;
                }

                pFe->scanNumTpScanned++;

                /* reset states for next carrier scan */
                pFe->scanState = AV_SCANSTATE_START;
                pFe->scanProgressSent  = FFALSE;

                /* Search for next carrier to scan */
                if(pFe->scanNumTpScanned < pFe->scanEvent.tpMax)
                {
                    for ( cnt = 0; cnt < (int16_t) (pFe->scanEvent.tpMax);
                          cnt++ )
                    {
                        /* get absolute carrier index */
                        if(TPLIST_GetIndex(&(pFe->scanNetwork), cnt,
                                           &nextIdx) != FAPI_OK)
                        {
                            continue;
                        }

                        /* ensure that entry has not been scanned before */
                        if(pFe->scanTpDone[nextIdx] != 0)
                        {
                            continue;
                        }
                        pFe->scanTpDone[nextIdx] = 1;
                        pFe->scanEvent.tpNext = (uint16_t)cnt;
                        pFe->scanIdxNext      = nextIdx;

                        if(!(pFe->scanProgressSent))    // FIXME: needed?
                        {
                            (void)EVT_Send(&(pFe->scanEvent), "Scan progress");
                            pFe->scanProgressSent = FTRUE;
                        }

                        break;
                    }

                    /* this should never happen */
                    if(cnt >= (int16_t) (pFe->scanEvent.tpMax))
                    {
                        pFe->scanNumTpScanned = pFe->scanEvent.tpMax;
                    }
                }

                nextSlotTime = 0;
                break;

            default:
                break;
        }
    }

    return nextSlotTime;
}
#endif

FSTATIC uint32_t AV_TaskScanCurrTS(AV_FEDAT_S * pFe)
{
    // TODO: needs to be rewritten
    if((int32_t)AV_ScanCarrier(pFe, &(pFe->scanNetwork), -1, &(pFe->scanOpts))
       == SMARTGO_SYS_ERR_ABORTED)
    {
        return 0;
    }

    /* generate event: job completed */
    pFe->scanEvent.status = SCAN_STAT_COMPLETED;
    pFe->scanEvent.tpDone = 1;
    pFe->scanEvent.tpNext = 0;
    pFe->scanEvent.progCurr = 1;

    (void)EVT_Send(&(pFe->scanEvent), "Scan complete");

    pFe->scanMode = SCAN_MODE_FINISHED;
    return 0;
}

/*
- carrierIdx  Target carrier in given network, -1 if tuning not requested
*/
FSTATIC uint32_t AV_ScanCarrier(AV_FEDAT_S * pFe, const SYS_NETWORK_S * network,
                                TPLIST_IDX carrierIdx, AV_SCAN_OPTS_S * opts)
{
    SYS_EVT_CARRIERADD_S ctrlEvent;
    uint32_t            nextSlotTime = 0;
    int32_t             retVal = FAPI_OK;
    uint16_t            pmtPid;
    PSISI_SDT_SERVICE_S sdtService;
    AV_TUNEPARS_S       tunePars;
    fbool_t             gotoNextState = FFALSE;
    TPLIST_ITEM_S       tpItem;
    const PSISI_PMT_S  *pPmt;


    switch (pFe->scanTpState)
    {
            /* ----------------------------------------------------------------- */

        case AV_SCANTP_TUNE:
            if(carrierIdx >= 0)
            {
                memset(&tunePars, 0, sizeof(tunePars));

                tunePars.network    = *network;
                tunePars.manualMode = FFALSE;
                tunePars.tpIdx      = carrierIdx;

                /* tune on requested transponder/channel */
                if(AVINT_TuningIsNecessary(pFe, &tunePars))
                {
                    retVal = AVINT_Tune (pFe, &tunePars, FTRUE);

                    switch (retVal)
                    {
                        case SMARTGO_SYS_ERR_ABORTED:
                            nextSlotTime = 0;
                            break;

                        case FAPI_OK:
                            /* check if stored carrier info can be updated */
                            AV_UpdateTPParams(pFe, network, carrierIdx);

                            AVINT_UpdateSignalParams(pFe);

                            /* proceed to next state */
                            gotoNextState = FTRUE;
                            break;

                        default:
                            /* tuning has failed: abort carrier scan */
                            pFe->scanTpState = AV_SCANTP_FINISHED;

                            AVINT_UpdateSignalParams(pFe);
                            break;
                    }
                }
                else
                {
                    /* check if stored carrier info can be updated */
                    AV_UpdateTPParams(pFe, network, carrierIdx);

                    AVINT_UpdateSignalParams(pFe);

                    gotoNextState = FTRUE;
                }
            }
            else
            {
                /* Tuning not requested */
                gotoNextState = FTRUE;
            }

            if ( gotoNextState )
            {
                /* Start filtering for SDT (PAT is started automatically) */
                FAPI_SYS_PRINT_DEBUG(1,"[SCAN] Search SDT: START...\n");
                (void)PSISI_SDT_StopFiltering();
                PSISI_SDT_Invalidate(&(avData->sdt));

                retVal = PSISI_SDT_StartFiltering (PSISI_VERSION_ANY, FTRUE);
                if(retVal != FAPI_OK)
                {
                    /* opening of filters failed */
                    pFe->scanTpState = AV_SCANTP_FINISHED;
                }
                else
                {
                    pFe->scanTpState = AV_SCANTP_PATSDT;
                    pFe->scanTimeStamp = SYS_GET_TIMESTAMP_MS;
                }
            }
            break;

            /* ----------------------------------------------------------------- */

        case AV_SCANTP_PATSDT:

            AVINT_UpdateSignalParams(pFe);

            if(pFe->pat.curr.sect.allAvailable && avData->sdt.sect.allAvailable)
            {
                /* PAT + SDT received: Update carrier information with
                   parameters if necessary. */
                if((carrierIdx >= 0) &&
                   (TPLIST_GetDataByIndex(network, carrierIdx, &tpItem)
                    == FAPI_OK))
                {
                    if((tpItem.tsId != (int32_t)avData->sdt.transport_stream_id) ||
                       (tpItem.onId != (int32_t)avData->sdt.original_network_id))
                    {
                        tpItem.tsId = (int32_t)avData->sdt.transport_stream_id;
                        tpItem.onId = (int32_t)avData->sdt.original_network_id;

                        if(TPLIST_SetDataByIndex(network, carrierIdx,
                                                 &tpItem, SYS_OWNER_SYSTEM)
                           == FAPI_OK)
                        {
                            ctrlEvent.hdr.type = EVT_SCAN_CARRIER_ADDED;
                            ctrlEvent.hdr.length = sizeof(ctrlEvent);
                            ctrlEvent.status =
                                SMARTGO_SYS_ERR_REPLACED_IN_LIST;
                            ctrlEvent.network = *network;

                            /* generate event: transponder/channel added */
                            (void)EVT_Send(&ctrlEvent, "Channel mod");
                        }
                    }
                }

                /* Ignore multiplexes declared for "private temporary use." */
                if ( avData->sdt.original_network_id >= ONID_PRIVATE_START )
                {
                    FAPI_SYS_PRINT_DEBUG(1,"[SCAN] Invalid onid (0x%0X), abort scan!\n",
                               avData->sdt.original_network_id);

                    pFe->scanTpState = AV_SCANTP_FINISHED;
                }
                else
                {
                    pFe->sdtLoopPos  = 0;
                    pFe->scanPrgIdx  = 0;
                    pFe->scanTpState = AV_SCANTP_PMTSTART;
                }
            }
            else
            {
                /* Test for timeout */
                if(UTILS_GetTimerStampInterval(pFe->scanTimeStamp,
                                               SYS_GET_TIMESTAMP_MS) >
                                               PAT_FILTERING_TIMEOUT)
                {
                    FAPI_SYS_PRINT_DEBUG(1,"[SCAN] SDT timeout...\n");
                    /* PAT and/or SDT not received in time. */
                    (void)PSISI_SDT_StopFiltering();

                    #ifdef APPL_SERVICE_SCAN_WITHOUT_SDT

                    FAPI_SYS_PRINT_DEBUG(1,"[SCAN] Search PAT: START...\n");
                    if ( pFe->pat.curr.sect.allAvailable )
                    {
                        FAPI_SYS_PRINT_DEBUG(1,"[SCAN] PAT found ...\n");
                        if ( (carrierIdx >= 0) &&
                             (TPLIST_GetDataByIndex (network, carrierIdx, &tpItem) == FAPI_OK) )
                        {
                            if ( tpItem.tsId
                                 != (int32_t)(pFe->pat.curr.transport_stream_id) )
                            {
                                if ( tpItem.tsId < 0 )
                                {
                                    tpItem.tsId = (int32_t)(pFe->pat.curr.transport_stream_id);
                                    (void)TPLIST_SetDataByIndex (network,
                                                                 carrierIdx,
                                                                 &tpItem,
                                                                 SYS_OWNER_SYSTEM);
                                }
                            }
                        }

                        pFe->scanPrgIdx  = 0;
                        pFe->scanTpState = AV_SCANTP_PMTSTART;
                    }
                    else
                    {
                        FAPI_SYS_PRINT_DEBUG(1,"[SCAN] PAT timeout...\n");

                    #endif /* APPL_SERVICE_SCAN_WITHOUT_SDT */

                        pFe->scanTpState = AV_SCANTP_FINISHED;

                    #ifdef APPL_SERVICE_SCAN_WITHOUT_SDT
                    }
                    #endif /* APPL_SERVICE_SCAN_WITHOUT_SDT */
                }
                else
                {
                    /* not yet in timeout, sleep for a while */
                    nextSlotTime = 30;
                }
            }
            break;

            /* ----------------------------------------------------------------- */

        case AV_SCANTP_PMTSTART:

            if ( PSISI_SDT_GetNextService(&(avData->sdt), &(pFe->sdtLoopPos),
                                          &sdtService) == FAPI_OK )
            {
                /* init service list item */
                memset(&(pFe->scanService), 0, sizeof(SLIST_ITEM_S));

                pFe->scanService.network      = *network;
                pFe->scanService.tpIdx        = carrierIdx;
                pFe->scanService.flags       |= SLIST_FLAG_VISIBLESVC; /* default */
                pFe->scanService.minAge       = PCTRL_MINAGE_UNLIMITED;
                pFe->scanService.lcn          = 0; /* default */
                pFe->scanService.strPid.valid = 0; /* default */

                /* Extract some SDT information  */
                if(AV_ScanAddSdtInfos(&sdtService, &(pFe->scanService))
                   != FAPI_OK)
                {
                    break;
                }

                /* add LCN and service attribute information if assigned */
                AV_ScanAddLcn(pFe, &(pFe->scanService));

                /* Filter PMT only if service is not in state "not running". */
                if(sdtService.running_status != PSISI_RUNSTAT_NOTRUNNING)
                {
                    /* get PMT-PID from PAT */
                    if(PSISI_PAT_GetPmtPid(&(pFe->pat.curr),
                                           pFe->scanService.serviceId,
                                           &pmtPid) == FAPI_OK)
                    {
                        (void)PSISI_PMT_Invalidate(RSRC_TSD_SCAN);

                        retVal = PSISI_PMT_StartFiltering(RSRC_TSD_SCAN,
                                                          pFe->scanService.serviceId, 
                                                          pmtPid,
                                                          PSISI_VERSION_ANY, 
                                                          FTRUE);
                        if ( retVal == FAPI_OK )
                        {
                            pFe->scanTimeStamp  = SYS_GET_TIMESTAMP_MS;
                            pFe->scanSdtInfoAvl = FTRUE;
                            pFe->scanTpState    = AV_SCANTP_PMTWAIT;
                        }
                        else
                        {
                            FAPI_SYS_PRINT_DEBUG(2,"PSISI_PMT_StartFiltering failed\n");
                        }
                    }
                }

                if ( pFe->scanTpState != AV_SCANTP_PMTWAIT )
                {
                    /* PMT is not filtered, add service immediately. */
                    AV_ScanAddService(opts, &(pFe->scanService));
                }
            }

            #ifdef APPL_SERVICE_SCAN_WITHOUT_SDT

            else if ( (!(avData->sdt.sect.allAvailable))
                   || (pFe->sdtLoopPos == 0) )
            {
                /* SDT is either not available or empty */

                /* init service list item */
                if ( pFe->scanPrgIdx < pFe->pat.curr.numPrograms )
                {
                    memset(&(pFe->scanService), 0, sizeof(SLIST_ITEM_S));

                    pFe->scanService.serviceId   = pFe->pat.curr.program
                                                   [pFe->scanPrgIdx].
                                                   program_number;
                    pFe->scanService.network     = *network;
                    pFe->scanService.serviceId   = pFe->scanService.serviceId;
                    pFe->scanService.flags      |= SLIST_FLAG_ALLFREE;
                    pFe->scanService.flags      |= SLIST_FLAG_VISIBLESVC;
                    pFe->scanService.tpIdx       = carrierIdx;
                    pFe->scanService.minAge      = PCTRL_MINAGE_UNLIMITED;
                    pFe->scanService.lcn         = 0;

                    if ( PSISI_PAT_GetPmtPid (&(pFe->pat.curr),
                               pFe->scanService.serviceId, &pmtPid) == FAPI_OK )
                    {
                        (void)PSISI_PMT_Invalidate(RSRC_TSD_SCAN);

                        retVal = PSISI_PMT_StartFiltering (
                                                    RSRC_TSD_SCAN,
                                                    pFe->scanService.serviceId,
                                                    pmtPid,
                                                    PSISI_VERSION_ANY, FTRUE);
                        if ( retVal == FAPI_OK )
                        {
                            pFe->scanTimeStamp  = SYS_GET_TIMESTAMP_MS;
                            pFe->scanSdtInfoAvl = FFALSE;
                            pFe->scanTpState    = AV_SCANTP_PMTWAIT;
                        }
                        else
                        {
                            FAPI_SYS_PRINT_DEBUG(2,"PSISI_PMT_StartFiltering failed\n");
                        }
                    }

                    pFe->scanPrgIdx++;
                }
                else
                {
                    pFe->scanTpState = AV_SCANTP_FINISHED;
                }
            }
            #endif /* APPL_SERVICE_SCAN_WITHOUT_SDT */
            else
            {
                pFe->scanTpState = AV_SCANTP_FINISHED;
            }

            break;
            /* ----------------------------------------------------------------- */

        case AV_SCANTP_PMTWAIT:

            AVINT_UpdateSignalParams(pFe);

            pPmt = PSISI_PMT_GetCurrent(RSRC_TSD_SCAN);

            if(!(pPmt->available))
            {
                /* test for timeout */
                if(UTILS_GetTimerStampInterval(pFe->scanTimeStamp,
                                               SYS_GET_TIMESTAMP_MS) >
                                               PMT_FILTERING_TIMEOUT)
                {
                    FAPI_SYS_PRINT_DEBUG(1,"[SCAN] PMT timeout...\n");

                    (void)PSISI_PMT_StopFiltering(RSRC_TSD_SCAN);

                    pFe->scanTpState = AV_SCANTP_PMTSTART;
                }
                else
                {
                    nextSlotTime = 30;
                }
            }
            else
            {
                (void)PSISI_PMT_StopFiltering (RSRC_TSD_SCAN);
                (void)PSISI_PMT_GetStreamPIDs (RSRC_TSD_SCAN,
                                               &(pFe->scanService.strPid));

                #ifdef APPL_SERVICE_SCAN_WITHOUT_SDT

                if( !(pFe->scanSdtInfoAvl) )
                {
                    if (pFe->scanService.strPid.video > 0 )
                    {
                        pFe->scanService.serviceType = SERVICE_TYPE_DIGITAL_TV;
                    }
                    else if ( pFe->scanService.strPid.audio.numStreams )
                    {
                        pFe->scanService.serviceType = SERVICE_TYPE_DIGITAL_RADIO;
                    }
                    else
                    {
                        pFe->scanService.serviceType = 0;
                    }

                    if ( pFe->scanService.strPid.video > 0 )
                    {
                        pFe->scanService.nameCharSet = SYS_CHARSET_ASCII;
                        sprintf (pFe->scanService.name,
                                 "SID %d VID %d",
                                 pFe->scanService.serviceId,
                                 pFe->scanService.strPid.video);
                    }
                    else
                    {
                        pFe->scanService.nameCharSet = SYS_CHARSET_ASCII;
                        sprintf (pFe->scanService.name,
                                 "SID %d AID %d",
                                 pFe->scanService.serviceId,
                                 pFe->scanService.strPid.audio.stream[0].pid);
                    }

                    AV_ScanAddLcn (pFe, &(pFe->scanService));
                }

                #endif /* APPL_SERVICE_SCAN_WITHOUT_SDT */

                /* add new service to global service list */
                AV_ScanAddService(opts, &(pFe->scanService));

                pFe->scanTpState = AV_SCANTP_PMTSTART;
            }
            break;

            /* ----------------------------------------------------------------- */

        default:
        case AV_SCANTP_FINISHED:
            break;
    }

    return nextSlotTime;
}

int32_t AV_ScanAddSdtInfos(const PSISI_SDT_SERVICE_S* sdtService,
                           SLIST_ITEM_S* pService)
{
    int32_t retVal;
    uint8_t service_type;

    pService->serviceId = sdtService->service_id;

    retVal = PSISI_GetServiceDescriptor(sdtService->descLoopStart,
                                        sdtService->descLoopLength,
                                        pService->name, SLIST_NAME_LENGTH,
                                        &(pService->nameCharSet),
                                        &service_type);
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }
    #ifdef SLIST_WITH_PROVIDER
    retVal = PSISI_GetServiceProviderName(sdtService->descLoopStart,
                                          sdtService->descLoopLength,
                                          pService->provider, SLIST_PROVIDER_NAME_LENGTH,
                                          &(pService->providerCharSet));
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }
    #endif
    SLIST_SetFlag(pService, SLIST_FLAG_ALLFREE, (fbool_t)(sdtService->free_ca_mode == 0));
    SLIST_SetFlag(pService, SLIST_FLAG_CURRNEXTAVL,
                  (fbool_t)(sdtService->eit_pf_flag != 0));
    SLIST_SetFlag(pService, SLIST_FLAG_SCHEDAVL,
                  (fbool_t)(sdtService->eit_schedule_flag != 0));

    /* ----- determine target list --------------------------------- */

    switch ( service_type )
    {
        case SERVICE_TYPE_DIGITAL_TV:
        case SERVICE_TYPE_DIGITAL_TV_HD:
        case SERVICE_TYPE_DIGITAL_TV_ADVANCED:
        case SERVICE_TYPE_DIGITAL_TV_HD_ADVANCED:
        case SERVICE_TYPE_DIGITAL_RADIO:
        case SERVICE_TYPE_DIGITAL_RADIO_ADVANCED:
        #ifdef APPL_MHEG_ENABLED
        case SERVICE_TYPE_DATA_BROADCAST:
        #endif /* APPL_MHEG_ENABLED */
            pService->serviceType = service_type;
            break;

        default:
            if( (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT) ||
                (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT_T2) ||
                (SYS_GetAppProfile() == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW))
            {
                /* NOTE: For UK-DTT profile, unknown service types are not
                         accepted. */

                FAPI_SYS_PRINT_DEBUG(1,"[SCAN] Invalid Service Type = 0x%02X\n",
                           service_type);
                retVal = SMARTGO_SYS_ERR_INVALID_DATA;
            }
            else
            {
                pService->serviceType = SERVICE_TYPE_DIGITAL_TV;
            }
    }

    return retVal;
}

/*
- carrierIdx  Target carrier in given network, -1 if tuning not requested
*/
FSTATIC uint32_t AV_ScanNIT(AV_FEDAT_S * pFe, const SYS_NETWORK_S * network,
                            TPLIST_IDX carrierIdx)
{
    uint32_t            nextSlotTime = 0;
    int32_t             retVal = FAPI_OK;
    uint16_t            len;
    fbool_t             tpFound = FFALSE;
    TPLIST_ITEM_S       tpItem;
    SYS_EVT_CARRIERADD_S ctrlEvent;
    AV_TUNEPARS_S       tunePars;
    fbool_t             gotoNextState = FFALSE;
    uint16_t            numOfTs;
    int32_t             networkId;
    int32_t             tsId;
    int32_t             onId;

    switch (pFe->scanNitState)
    {
            /* ----------------------------------------------------------------- */

        case AV_SCANNIT_TUNE:
            FAPI_SYS_PRINT_DEBUG(1,"[SCAN] Search NIT: START...\n");

            if(carrierIdx >= 0)
            {
                memset(&tunePars, 0, sizeof(tunePars));

                tunePars.network    = *network;
                tunePars.manualMode = FFALSE;
                tunePars.tpIdx      = carrierIdx;

                /* tune on requested transponder/channel */
                if(AVINT_TuningIsNecessary(pFe, &tunePars))
                {
                    retVal = AVINT_Tune (pFe, &tunePars, FTRUE);

                    switch (retVal)
                    {
                        case SMARTGO_SYS_ERR_ABORTED:
                            break;

                        case FAPI_OK:

                            AVINT_UpdateSignalParams(pFe);

                            /* proceed to next state */
                            gotoNextState = FTRUE;
                            break;

                        default:
                            /* tuning has failed: abort carrier scan */
                            pFe->scanNitState = AV_SCANNIT_FINISHED;

                            AVINT_UpdateSignalParams(pFe);
                            break;
                    }
                }
                else
                {
                    gotoNextState = FTRUE;
                }
            }
            else
            {
                /* Tuning not requested */
                gotoNextState = FTRUE;
            }

            if(gotoNextState)
            {
                /* Start filtering for NIT */
                do
                {
                    /* Ensure that new NIT will be filtered. */
                    (void)PSISI_NIT_Invalidate();

                    retVal = PSISI_NIT_StartFiltering(PSISI_VERSION_ANY,
                                                      FTRUE,
                                                      NIT_FILTERING_TIMEOUT);
                    if ( retVal != FAPI_OK )
                    {
                        break;
                    }
                }
                while(0);

                if ( retVal != FAPI_OK )
                {
                    /* opening of filters failed */
                    (void)PSISI_NIT_StopFiltering();

                    pFe->scanNitState = AV_SCANNIT_FINISHED;
                }
                else
                {
                    pFe->scanTimeStamp = SYS_GET_TIMESTAMP_MS;
                    pFe->scanNitState  = AV_SCANNIT_NIT;
                }
            }
            break;

            /* ----------------------------------------------------------------- */

        case AV_SCANNIT_NIT:
            AVINT_UpdateSignalParams(pFe);

            /* poll for PAT reception */
            if ( pFe->scanTsId < 0 )
            {
                if ( pFe->pat.curr.sect.allAvailable )
                {
                    /* PAT + received: Store current transport_stream_id. */
                    pFe->scanTsId = (int32_t)(pFe->pat.curr.transport_stream_id);
                }
                else if ( UTILS_GetTimerStampInterval(pFe->scanTimeStamp,
                                               SYS_GET_TIMESTAMP_MS)
                                             > PAT_FILTERING_TIMEOUT )
                {
                    /* PAT timeout! */
                    FAPI_SYS_PRINT_DEBUG(1,"[SCAN] pFe->scanTimeStamp = %d, now = %d\n",pFe->scanTimeStamp,  SYS_GET_TIMESTAMP_MS);
                    FAPI_SYS_PRINT_DEBUG(1,"[SCAN] PAT timeout during NIT scan!\n");
                    pFe->scanNitState = AV_SCANNIT_FINISHED;
                    break;
                }
            }

            /* poll for NIT reception */
            retVal = PSISI_NIT_GetInfo(&numOfTs, &networkId);
            if ( (pFe->scanTsId >= 0)
              && (retVal == FAPI_OK) )
            {
                FAPI_SYS_PRINT_DEBUG(1,"[SCAN] NIT FOUND: ID %X - %i TS\n", networkId,
                           numOfTs);

                pFe->scanNumTs     = numOfTs;
                pFe->scanNetworkId = networkId;
                pFe->scanTsCnt     = 0;
                pFe->scanNitState  = AV_SCANNIT_PARSE;

                /* set network_id of  current carrier (might be still
                   unknown). */

                if(TPLIST_GetDataByIndex(network, carrierIdx, &tpItem) ==
                   FAPI_OK)
                {
                    if ( tpItem.networkId != pFe->scanNetworkId )
                    {
                        tpItem.networkId = pFe->scanNetworkId;

                        if(TPLIST_SetDataByIndex(network, carrierIdx, &tpItem,
                                                 SYS_OWNER_SYSTEM) == FAPI_OK)
                        {
                            ctrlEvent.hdr.type = EVT_SCAN_CARRIER_ADDED;
                            ctrlEvent.hdr.length = sizeof(ctrlEvent);
                            ctrlEvent.status =
                                SMARTGO_SYS_ERR_REPLACED_IN_LIST;
                            ctrlEvent.network = *network;

                            /* generate event: transponder/channel added */
                            (void)EVT_Send(&ctrlEvent, "Channel added");
                        }
                    }
                }
            }
            else if(retVal == APPL_SMARTGO_ERR_TIMEOUT)
            {
                FAPI_SYS_PRINT_DEBUG(1,"[SCAN] NIT timeout...\n");
                pFe->scanNitState = AV_SCANNIT_FINISHED;
            }
            else
            {
                nextSlotTime = 30;
            }
            break;

            /* ------------------------------------------------------------- */

        case AV_SCANNIT_PARSE:

            if(pFe->scanTsCnt < pFe->scanNumTs)
            {
                if(PSISI_NIT_GetTransportInfo(pFe->scanTsCnt, &tsId, &onId)
                   != FAPI_OK)
                {
                    pFe->scanTsCnt++;
                    break;
                }

                if ( PSISI_NIT_GetTransportDescLoop(pFe->scanTsCnt,
                                                    avData->descBuf, &len)
                                                    != FAPI_OK )
                {
                    pFe->scanTsCnt++;
                    break;
                }

                FAPI_SYS_PRINT_DEBUG(3,"[NIT SCAN] TS %3i: id %04X onid %X desclen %i\n",
                                         pFe->scanTsCnt, tsId, onId, len);
                memset(&(pFe->nitTpInfo.tpItem), 0, sizeof(pFe->nitTpInfo.tpItem));

                pFe->nitTpInfo.tpItem.tsId       = tsId;
                pFe->nitTpInfo.tpItem.networkId  = pFe->scanNetworkId;
                pFe->nitTpInfo.tpItem.onId       = onId;
                pFe->nitTpInfo.tpItem.sigQuality = 0;

                tpFound = PSISI_NIT_ParseByTP(pFe->systemType, avData->descBuf, len, &(pFe->nitTpInfo), pFe->scanTsId);
                if(tpFound)
                {
                    ctrlEvent.hdr.type = EVT_SCAN_CARRIER_ADDED;
                    ctrlEvent.hdr.length = sizeof(ctrlEvent);
                    ctrlEvent.status = TPLIST_AddCarrier(network, &(pFe->nitTpInfo.tpItem),
                                                         SYS_OWNER_SYSTEM,
                                                         &ctrlEvent.lstIdx);
                    ctrlEvent.network = *network;

                    /* generate event: transponder/channel added */
                    (void)EVT_Send(&ctrlEvent, "Channel add");
                }

                pFe->scanTsCnt++;
            }
            else
            {
                (void)PSISI_NIT_StopFiltering();
                pFe->scanNitState = AV_SCANNIT_FINISHED;
            }
            break;

        default:
            break;
    }

    return nextSlotTime;
}

FSTATIC void AV_ScanAddService(const AV_SCAN_OPTS_S * opts,
                               const SLIST_ITEM_S * service)
{
    SYS_SERVLOC_S       targetLoc;
    SYS_EVT_SERVICE_S   ctrlEvent;
    #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)
        const PSISI_PMT_S  *pPmt = PSISI_PMT_GetCurrent(RSRC_TSD_SCAN);
    #endif

    #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

    SLIST_CA_INFO_S     caInfo;
    SLIST_SERVIDX       servIdx;

    #endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */

    targetLoc.listId = SLIST_NONE;
    targetLoc.index = -1;

    /* test for crypted/uncrypted option */
    switch (opts->filter)
    {
        default:
        case AV_SCANFILTER_ALL:
            break;

        case AV_SCANFILTER_FREEONLY:
            if((service->flags & SLIST_FLAG_ALLFREE) == 0)
                return;
            break;

        case AV_SCANFILTER_CRYPTEDONLY:
            if((service->flags & SLIST_FLAG_ALLFREE) != 0)
                return;
            break;
    }

    /* test for TV/Radio option */
    switch (opts->choice)
    {
        default:
        case AV_SCANCHOICE_ALL:
            break;

        case AV_SCANCHOICE_TVONLY:
            if( (service->serviceType != SERVICE_TYPE_DIGITAL_TV) &&
                (service->serviceType != SERVICE_TYPE_DIGITAL_TV_ADVANCED) &&
                (service->serviceType != SERVICE_TYPE_DIGITAL_TV_HD) &&
                (service->serviceType != SERVICE_TYPE_DIGITAL_TV_HD_ADVANCED) &&
                (service->serviceType != SERVICE_TYPE_DATA_BROADCAST))
            {
                return;
            }
            break;

        case AV_SCANCHOICE_RADIOONLY:
            if ( (service->serviceType != SERVICE_TYPE_DIGITAL_RADIO ) &&
                 (service->serviceType != SERVICE_TYPE_DIGITAL_RADIO_ADVANCED ) )
            {
                return;
            }
            break;
    }


    /* add to global list */
    ctrlEvent.addStat = SLIST_EntryAdd(service, &targetLoc);

    #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

    /* add CA information to service list */
    switch (ctrlEvent.addStat)
    {
        case FAPI_OK:
        case SMARTGO_SYS_ERR_ALREADY_IN_LIST: // TODO: hier auch ca-info ersetzen? wenn ja, strpid von slist holen statt aktuelle?
        case SMARTGO_SYS_ERR_REPLACED_IN_LIST:
            if(SLIST_CaInfoExtract(pPmt, &(service->strPid), &caInfo) ==
               FAPI_OK)
            {
                servIdx = SLIST_GetServiceIndex(&targetLoc);

                (void)SLIST_CaInfoSet(servIdx, &caInfo);
            }

            break;

        default:
            break;
    }

    #endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */

    /* generate event: service found */
    ctrlEvent.hdr.type = EVT_SCAN_SERVICE_ADDED;
    ctrlEvent.hdr.length = sizeof(ctrlEvent);
    ctrlEvent.servLoc = targetLoc;
    ctrlEvent.status = SERVICE_STAT_SUCCESS;    // not needed

    /* send event: service found */
    (void)EVT_Send(&ctrlEvent, "Service add");

    FAPI_SYS_PRINT_DEBUG(1,"[SCAN]  %-25s  PCR= %4d, V= %4d, A= %4d, TXT= %4d\n",
               service->name,
               service->strPid.pcr,
               service->strPid.video,
               service->strPid.audio.stream[0].pid, service->strPid.ttx);
}

/*
Parses through logical channel descriptor in order to find LCN for given
service.
*/
FSTATIC void AV_ScanAddLcn(AV_FEDAT_S * pFe, SLIST_ITEM_S * service)
{
    uint16_t            cnt;

    service->lcn = 0;

    for(cnt = 0; (uint16_t) cnt < pFe->nitTpInfo.serviceCount; cnt++)
    {
        if(pFe->nitTpInfo.nitServiceList[cnt].serviceId == service->serviceId)
        {
            FAPI_SYS_PRINT_DEBUG(3,"[LCN]serviceId=0x%x, lcn=%d\n", service->serviceId, pFe->nitTpInfo.nitServiceList[cnt].lcn);
            service->lcn = pFe->nitTpInfo.nitServiceList[cnt].lcn;
            SLIST_SetFlag(service, SLIST_FLAG_HIDDEN_UK, (pFe->nitTpInfo.nitServiceList[cnt].invisible));
            SLIST_SetFlag(service, SLIST_FLAG_NONSELECT, (pFe->nitTpInfo.nitServiceList[cnt].unselectable));

            break;
        }
    }
    return;
}

/* ------------------------------------------------------------------------- */

/* NOTE: Blind scan has not yet been implemented! */

#ifdef APPL_BLINDSCAN_ENABLED
FSTATIC uint8_t scanBlindAbortFct (void)
{
    return (uint8_t)avData->itcInst.abortReq;
}

FSTATIC void scanBlindProgressFct(uint16_t progress)
{
    AV_FEDAT_S*          pFe;

    pFe = avData->feDat + avData->scanBlindFe;

    pFe->scanEvent.hdr.type   = EVT_SCAN_PROGRESS;
    pFe->scanEvent.hdr.length = sizeof(pFe->scanEvent);
    pFe->scanEvent.mode       = SCAN_MODE_BLIND;
    pFe->scanEvent.tpDone     = 0;
    pFe->scanEvent.tpNext     = 0;
    if(pFe->scanEvent.status == SCAN_STAT_SCAN_HOR)
    {
        if ( pFe->scanEvent.progCurr <= 1000 )
        {
            pFe->scanEvent.progCurr  += 2;
        }
    }
    else if (pFe->scanEvent.status == SCAN_STAT_SCAN_VER)
    {
        if ( pFe->scanEvent.progCurr <= 2000 )
        {
            pFe->scanEvent.progCurr  += 2;
        }
    }


    (void)EVT_Send(&pFe->scanEvent, "Blind Scan progress");
}

FSTATIC void scanBlindNotifyFct(FE_TUNE_PARAMS_S * pFoundTuneParams)
{
    SYS_EVT_CARRIERADD_S ctrlEvent;
    TPLIST_ITEM_S        listItem;
    AV_FEDAT_S*          pFe;
    int16_t              numTp = 0;

    if(avData->scanBlindFe == -1) return;

    pFe = avData->feDat + avData->scanBlindFe;

    memset(&listItem, 0, sizeof(listItem));
    /* IDs are unknown at this time */
    listItem.tsId       = -1;
    listItem.networkId  = -1;
    listItem.onId       = -1;
    listItem.pars       = *pFoundTuneParams;
    listItem.sigQuality = 0;

    ctrlEvent.hdr.type   = EVT_SCAN_CARRIER_ADDED;
    ctrlEvent.hdr.length = sizeof(ctrlEvent);
    ctrlEvent.status     = TPLIST_AddCarrier(&(pFe->scanNetwork),
                                             &listItem,
                                             SYS_OWNER_SYSTEM,
                                             &ctrlEvent.lstIdx);
    ctrlEvent.network    = pFe->scanNetwork;

    if(TPLIST_GetNumCarriers(&(pFe->scanNetwork),&numTp) == FAPI_OK)
    {
        /* update total number of transponders in list */
        pFe->scanEvent.tpMax    = (uint16_t)numTp;
    }

    /* generate event: transponder/channel added */
    (void)EVT_Send(&ctrlEvent, "Channel added");
}

#endif

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
