/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/bootimage.c
**
**              $Id: bootimage.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Bootloader and -image related operations.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <stdio.h>

#include <fapi/drv_flash.h>
#include <fapi/drv_boot.h>

#include "sys_api.h"
#include "sys_priv.h"
#include "utils.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define FAKESLOTS 0

#define ABS(X) (((X) > 0) ? (X) : (-(X)))

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/* the fallback addresses BOOTER_SLOT_OFFSET0/1 are set in the Makefile */
static const uint32_t         fallbackAddress[2] = { BOOTER_SLOT_OFFSET0,
                                                     BOOTER_SLOT_OFFSET1 };

#if (FAKESLOTS)
FSTATIC FAPI_BOOT_InfoT bootDummy = {FAPI_BOOT_MODE_BOOTER, FAPI_BOOT_DEVICE_SFLASH,
                                  0, BOOTER_SLOT_OFFSET0,   BOOTER_SLOT_OFFSET1};
FSTATIC BOOTER_FlashSlotHeaderT slotDummy = {BOOTER_SLOTHDR_SIG, 0xFFFFFFFEUL,
                                         0x01020304UL, "dummyslot",
                                         0x00000001UL, 0, 1, 0x00100000UL};
#endif

FSTATIC FAPI_BOOT_InfoT      bootInf;
FSTATIC fbool_t              bootInfValid = FFALSE;
FSTATIC uint32_t             bootMode;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

int32_t SYS_GetBootloaderInfo(void)
{
    int32_t                 retVal;
    FAPI_SYS_HandleT        bootHdl;
    FAPI_BOOT_OpenParamsT   bootParams = { FAPI_BOOT_VERSION };

    bootHdl = FAPI_BOOT_Open(&bootParams, &retVal);
    if(retVal)
    {
        return retVal;
    }

    retVal = FAPI_BOOT_GetBootMode(bootHdl, &bootMode);
    if(retVal)
    {
        (void)FAPI_BOOT_Close(bootHdl);
        return retVal;
    }

    if( (bootMode == FAPI_BOOT_MODE_BOOTER) )
    {
        /* get boot information deposited by bootloader */
        (void)FAPI_BOOT_GetInfo(bootHdl, &bootInf);
        FAPI_SYS_PRINT_DEBUG(2,"FAPI_BOOT_GetInfo says mode %d device %d\n", bootInf.bootMode, bootInf.bootDevice);
        if(!bootInf.slotAddress1)
        {
            if(!bootInf.slotAddress0)
            {
                /* this should never happen, as we must have booted
                   this application from somewhere */
                bootInf.slotAddress0 = fallbackAddress[0];
                bootInf.slotAddress1 = fallbackAddress[1];
                bootInf.slotIndex = 0;
            }
            /* booter has found only one valid slot, find
               the fallback address with the highest distance
               to the valid slot */
            if(ABS((int32_t)bootInf.slotAddress0 - (int32_t)fallbackAddress[0]) >
               ABS((int32_t)bootInf.slotAddress0 - (int32_t)fallbackAddress[1]))
            {
                bootInf.slotAddress1 = fallbackAddress[0];
            }
            else
            {
                bootInf.slotAddress1 = fallbackAddress[1];
            }
        }
        bootInfValid = FTRUE;
    }
    else
    {
        #if (FAKESLOTS)

        bootInf = bootDummy;
        bootInfValid = FTRUE;

        #endif /* FAKESLOTS */
    }
    (void)FAPI_BOOT_Close(bootHdl);
    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Get boot status information.
**
** This function is only available if bootloader support is enabled in the
** makefile. It returns information that is passed by the bootloader to the
** application via a special RAM area.
**
** \param  pBoot  Pointer to boot information filled by the function.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**         - #SMARTGO_SYS_ERR_DATA_NOT_AVL if application has not been started
**           by the smartgo-HD-bootloader
******************************************************************************/
int32_t SYS_GetBootInfo (FAPI_BOOT_InfoT* pBoot)
{
    if ( bootInfValid == FFALSE )
    {
        return SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }
    else
    {
        (*pBoot) = bootInf;
        return FAPI_OK;
    }
}

int32_t SYS_GetBootSlotInfo (uint32_t slotIdx, BOOTER_FlashSlotHeaderT* pSlot)
{
    FAPI_SYS_HandleT        flHdl;
    FAPI_FLASH_OpenParamsT  flParams;
    int32_t                 retVal;
    uint32_t                slotAddr;

    if ( slotIdx >= BOOTER_NUM_SLOTS ) return FAPI_ERR_BAD_PARAMETER;
    if ( !bootInfValid )               return SMARTGO_SYS_ERR_DATA_NOT_AVL;

    #if (FAKESLOTS)

    if (bootMode != FAPI_BOOT_MODE_BOOTER)
    {
        *pSlot = slotDummy;
        return FAPI_OK;
    }

    #endif /* FAKESLOTS */

    memset (&flParams, 0, sizeof(flParams));

    flParams.version     = FAPI_FLASH_VERSION;
    flParams.deviceType  = sysData->flashDeviceType;
    flParams.firstSector = 0;
    flParams.sectorCount = sysData->flashUsrStartSector;

    flHdl = FAPI_FLASH_Open(&flParams, &retVal);

    if ( retVal == FAPI_OK )
    {
        slotAddr = (slotIdx == 0) ? bootInf.slotAddress0 : bootInf.slotAddress1;

        retVal = FAPI_FLASH_ReadBytes(flHdl, slotAddr,
                                      sizeof(BOOTER_FlashSlotHeaderT), pSlot);

        (void)FAPI_FLASH_Close(flHdl);
    }

    return retVal;
}

int32_t SYS_WriteBootImage (SYS_DEVICE_E device)
{
    SYS_DEV_STAT_S          devStatus;
    FAPI_BOOT_InfoT         bootInfo;
    BOOTER_FlashSlotHeaderT bootHdr;
    int32_t                 retVal;
    char_t                  name[32]; // TODO: strings anpassen
    char_t                  path[16]; // TODO: strings anpassen

    #ifdef APPL_FATFS_ENABLED
    F_FILE*                 fileP = NULL;
    FAPI_FLASH_OpenParamsT  flParams;
    FAPI_SYS_HandleT        flHdl;
    uint32_t                transferPos;
    uint32_t                flashaddr;
    uint32_t                transferBytes;
    uint32_t                chunkSize; //!< NOTE: must be multiple of 2 bytes!
    #endif /* APPL_FATFS_ENABLED */

    do
    {
        retVal = SYS_DeviceGetStatus(device, &devStatus);

        if ( retVal != FAPI_OK )
        {
            break;
        }
        else if ( devStatus.mountStatus != SYS_DEV_FS_MOUNTED )
        {
            retVal = SMARTGO_SYS_ERR_PROHIBITED;
            break;
        }

        retVal = SYS_GetBootInfo(&bootInfo);
        if ( retVal != FAPI_OK ) break;

        retVal = SYS_GetBootSlotInfo(bootInfo.slotIndex, &bootHdr);
        if ( retVal != FAPI_OK ) break;

        if ( SYS_DeviceGetMountPath(device, path, 16) == 0 )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        sprintf (name, "%s/V%ld_%02ld.SSU", path,
                 bootHdr.version >> 8,
                 bootHdr.version & 0xFF);

        retVal = SYS_DevicePreAccess (device);
        if ( retVal != FAPI_OK ) break;

        #ifdef APPL_FATFS_ENABLED

        fileP = f_open(name, "w");
        if( fileP == NULL )
        {
            #if (DEBUG > 0)
            int32_t rc = f_getlasterror();
            FAPI_SYS_PRINT_DEBUG(1,"SYS_WriteBootImage f_open failed: %d\n", rc);
            #endif
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        if ( bootInfo.slotIndex == 0 ) flashaddr = bootInfo.slotAddress0;
        else                           flashaddr = bootInfo.slotAddress1;

        memset (&flParams, 0, sizeof(flParams));

        flParams.version     = FAPI_FLASH_VERSION;
        flParams.deviceType  = sysData->flashDeviceType;
        flParams.firstSector = 0;
        flParams.sectorCount = 1;
        flParams.forceFlush  = 1;

        flHdl = FAPI_FLASH_Open(&flParams, &retVal);
        if ( retVal != FAPI_OK ) break;

        transferPos = 0;
        chunkSize   = sysData->bufSize;

        if ( (bootHdr.slotSize*4) < chunkSize )
        {
            chunkSize = bootHdr.slotSize*4;
        }

        chunkSize &= 0xFFFFFFFE;

        while ( transferPos < (bootHdr.slotSize*4) )
        {
            if ( ((bootHdr.slotSize*4) - transferPos) > chunkSize )
            {
                transferBytes = chunkSize;
            }
            else
            {
                transferBytes = (bootHdr.slotSize*4) - transferPos;
            }

            retVal = FAPI_FLASH_ReadBytes(flHdl, flashaddr, transferBytes,
                                         (uint8_t*)sysData->pBuf );

            if ( retVal != FAPI_OK ) break;

            if ( f_write (sysData->pBuf, 1, (int32_t)transferBytes, fileP) !=
                 (int32_t)transferBytes )
            {
                retVal = SMARTGO_SYS_ERR_INTERNAL;
                break;
            }

            flashaddr   += transferBytes;
            transferPos += transferBytes;
        }

        (void)FAPI_FLASH_Close(flHdl);

        retVal = FAPI_OK;

        #else

        retVal = SMARTGO_SYS_ERR_PROHIBITED;

        #endif /* APPL_FATFS_ENABLED */

    } while ( 0 );

    #ifdef APPL_FATFS_ENABLED

    if ( fileP != NULL )
    {
        (void)f_close(fileP);
    }

    #endif /* APPL_FATFS_ENABLED */

    return retVal;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
