/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/carrierlist.c
**
**              $Id: carrierlist.c 5871 2010-03-02 13:42:55Z jmorha $
**
** \brief       Carrier list implementation.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
**
*****************************************************************************
*/
/*lint -save -e713 -e732 -e737 */
//#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <fapi/sys_driver.h>         /* global type definitions */
#include <flashfs/flashfs.h>

#include "sys_api.h"
#include "sys_priv.h"
#include "statlist.h"
#include "utils.h"
/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    uint32_t            id;     // file identification
    uint32_t            revision;       // file structure revision
    uint16_t            numLists;       // number of TP lists included

} TPLIST_FILEHDR_S;

typedef struct
{
    SYS_NETWORK_S       network;
    fbool_t             dirty;
    STATLST_S           statLst;
    TPLIST_IDX          refLst[TPLIST_CARRIERS_MAX];

} TPLIST_LIST_S;

typedef struct
{
    #ifdef APPL_FESAT_ENABLED
    TPLIST_LIST_S       sat[CFGSAT_SAT_MAX];    //!< sat carrier list;
    #endif                          /* APPL_FESAT_ENABLED */

    #ifdef APPL_FETERR_ENABLED
    TPLIST_LIST_S       terr;   //!< terrestrial carrier list
    #endif                          /* APPL_FETERR_ENABLED */

    #ifdef APPL_FECABLE_ENABLED
    TPLIST_LIST_S       cable;  //!< cable carrier list
    #endif                          /* APPL_FECABLE_ENABLED  */

    uint8_t             dummy;
} TPLIST_DAT_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC TPLIST_DAT_S *tpListDat = NULL;
FSTATIC fbool_t                lstSatValid = FTRUE; //!< Status of stored list.
FSTATIC fbool_t                lstTerValid = FTRUE; //!< Status of stored list.
FSTATIC fbool_t                lstCabValid = FTRUE; //!< Status of stored list.

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC fbool_t     TPLIST_LoadLists(TPLIST_LIST_S * pList, uint16_t numLists,
                                     fbool_t lstValid);
FSTATIC fbool_t     TPLIST_SaveLists(TPLIST_LIST_S * pList, uint16_t numLists,
                                     fbool_t force, fbool_t * lstValid);
FSTATIC int32_t     TPLIST_ItemIsInList(TPLIST_LIST_S * pList,
                                        const TPLIST_ITEM_S * pTp);

FSTATIC int32_t     TPLIST_GetTargetList(const SYS_NETWORK_S * network,
                                         const int16_t * lstPos,
                                         TPLIST_LIST_S ** pList);
FSTATIC int32_t     TPLIST_GetFilename(FE_SYSTEM_TYPE_E feType,
                                      uint32_t* fileId,
                                      uint32_t* fileRev);
FSTATIC int32_t     TPLIST_GetListBySystem(FE_SYSTEM_TYPE_E feType,
                                           TPLIST_LIST_S ** ppList,
                                           uint16_t * numLists);
FSTATIC void        TPLIST_GenerateRefList(TPLIST_LIST_S * pList);
FSTATIC fbool_t     TPLIST_InsertEntry(TPLIST_LIST_S * pList, TPLIST_IDX idx,
                                       int16_t listSize, int16_t * newPos);
FSTATIC void        TPLIST_CarrierRemoved(const SYS_NETWORK_S * network,
                                          TPLIST_IDX tpIdx);
FSTATIC void        TPLIST_ClearCb(void *arg, int32_t index);

/* Marks stored files as invalid */
void TPLIST_Invalidate(FE_SYSTEM_TYPE_E feType)
{
    switch (feType)
    {
        case FE_SYSTEM_TYPE_SATELLITE:
            lstSatValid = FFALSE;
            break;
        case FE_SYSTEM_TYPE_TERRESTRIAL:
            lstTerValid = FFALSE;
            break;
        case FE_SYSTEM_TYPE_CABLE:
            lstCabValid = FFALSE;
            break;
        default:
            return;
    }
    /* Invalidate service lists -> dependency on carrier lists */
    SLIST_Invalidate();
}

int32_t TPLIST_Init(void)
{
    int32_t     retVal;
    uint32_t    maxFileSize = 0;

    #ifdef APPL_FESAT_ENABLED
    uint16_t    satCnt;
    #endif /* APPL_FESAT_ENABLED */

    DBG_Assert(tpListDat == NULL);

    tpListDat = UTILS_Malloc(sizeof(TPLIST_DAT_S));
    if ( tpListDat == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    memset(tpListDat, 0, sizeof(TPLIST_DAT_S));

    /* ----- Initialise satellite delivery system -------------------------- */

    #ifdef APPL_FESAT_ENABLED

    maxFileSize = sizeof(TPLIST_FILEHDR_S);

    for ( satCnt = 0; satCnt < CFGSAT_SAT_MAX; satCnt++ )
    {
        tpListDat->sat[satCnt].network.type = FE_SYSTEM_TYPE_SATELLITE;
        tpListDat->sat[satCnt].network.satIdx = satCnt;
        tpListDat->sat[satCnt].dirty          = FTRUE;

        retVal = STATLST_Init(&(tpListDat->sat[satCnt].statLst),
                              TPLIST_CARRIERS_MAX, sizeof(TPLIST_ITEM_S));
        if ( retVal != FAPI_OK )
        {
            return retVal;
        }

        maxFileSize += STATLST_GetMaxDataSize (TPLIST_CARRIERS_MAX,
                                               sizeof(TPLIST_ITEM_S));
    }

    /* Test if maximum flash file size is sufficient. */
    retVal = FLASH_CheckFileSize (FILEID_TPLIST_SAT, maxFileSize);

    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    (void)TPLIST_LoadLists(tpListDat->sat, CFGSAT_SAT_MAX, lstSatValid);

    #endif /* APPL_FESAT_ENABLED */

    /* ----- Initialise terrestrial delivery system ------------------------ */

    #ifdef APPL_FETERR_ENABLED

    tpListDat->terr.network.type = FE_SYSTEM_TYPE_TERRESTRIAL;
    tpListDat->terr.network.satIdx = -1;
    tpListDat->terr.dirty          = FTRUE;

    retVal = STATLST_Init(&(tpListDat->terr.statLst),
                          TPLIST_CARRIERS_MAX, sizeof(TPLIST_ITEM_S));
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    maxFileSize = sizeof(TPLIST_FILEHDR_S)
                + STATLST_GetMaxDataSize (TPLIST_CARRIERS_MAX,
                                          sizeof(TPLIST_ITEM_S));

    /* Test if maximum flash file size is sufficient. */
    retVal = FLASH_CheckFileSize (FILEID_TPLIST_TER, maxFileSize);

    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    (void)TPLIST_LoadLists(&(tpListDat->terr), 1, lstTerValid);

    #endif /* APPL_FETERR_ENABLED */

    /* ----- Initialise cable delivery system ------------------------------ */

    #ifdef APPL_FECABLE_ENABLED

    tpListDat->cable.network.type = FE_SYSTEM_TYPE_CABLE;
    tpListDat->cable.network.satIdx = -1;
    tpListDat->cable.dirty          = FTRUE;

    retVal = STATLST_Init(&(tpListDat->cable.statLst),
                          TPLIST_CARRIERS_MAX, sizeof(TPLIST_ITEM_S));
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    maxFileSize = sizeof(TPLIST_FILEHDR_S)
                + STATLST_GetMaxDataSize (TPLIST_CARRIERS_MAX,
                                          sizeof(TPLIST_ITEM_S));

    /* Test if maximum flash file size is sufficient. */
    retVal = FLASH_CheckFileSize (FILEID_TPLIST_CAB, maxFileSize);

    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    (void)TPLIST_LoadLists(&(tpListDat->cable), 1, lstCabValid);

    #endif /* APPL_FECABLE_ENABLED */

    tpListDat->dummy = 0; // to satisfy MISRA

    return FAPI_OK;
}

int32_t TPLIST_Clear(const SYS_NETWORK_S * network, SYS_OWNER_E owner)
{
    TPLIST_LIST_S      *pList = NULL;
    int32_t             retVal;

    if((retVal = TPLIST_GetTargetList(network, NULL, &pList)) != FAPI_OK)
    {
        return retVal;
    }

    retVal = STATLST_Clear(&(pList->statLst), owner,
                           TPLIST_ClearCb, (void *) network);
    if(retVal == FAPI_OK)
    {
        /* re-generate reference list */
        TPLIST_GenerateRefList(pList);

        pList->dirty = FTRUE;
    }

    return retVal;
}

/* re-load lists from flash */
int32_t TPLIST_Load(FE_SYSTEM_TYPE_E feType)
{
    if ( tpListDat == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    switch (feType)
    {
        #ifdef APPL_FESAT_ENABLED
        case FE_SYSTEM_TYPE_SATELLITE:
            (void)TPLIST_LoadLists(tpListDat->sat, CFGSAT_SAT_MAX, lstSatValid);
            break;
        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED
        case FE_SYSTEM_TYPE_TERRESTRIAL:
            (void)TPLIST_LoadLists(&(tpListDat->terr), 1, lstTerValid);
            break;
        #endif /* APPL_FETERR_ENABLED */

        #ifdef APPL_FECABLE_ENABLED
        case FE_SYSTEM_TYPE_CABLE:
            (void)TPLIST_LoadLists(&(tpListDat->cable), 1, lstCabValid);
            break;
        #endif /* APPL_FECABLE_ENABLED */

        #ifdef APPL_FEDIRECT_ENABLED
        case FE_SYSTEM_TYPE_DIRECT:

            break;
        #endif /* APPL_FEDIRECT_ENABLED */

        default:
            return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    return FAPI_OK;
}

fbool_t TPLIST_SavingIsNecessary(const SYS_NETWORK_S * network)
{
    TPLIST_LIST_S      *pList = NULL;

    if(TPLIST_GetTargetList(network, NULL, &pList) != FAPI_OK)
    {
        return FFALSE;
    }
    else
    {
        return pList->dirty;
    }
}

/* remove lists in flash, restore default lists in SDRAM */
void TPLIST_RestoreDefaults(FE_SYSTEM_TYPE_E feType, fbool_t keepUserData)
{
    uint16_t            cnt;
    uint32_t            fileId;
    uint32_t            fileRev;
    SYS_OWNER_E         owners;
    TPLIST_LIST_S      *pList = NULL;
    uint16_t            numLists;

    if ( TPLIST_GetFilename(feType, &fileId, &fileRev) != FAPI_OK )
    {
        return;
    }

    (void)FLASHFS_Delete(fileId);

    if ( TPLIST_GetListBySystem(feType, &pList, &numLists) != FAPI_OK )
    {
        return;
    }
    /* --------------------------------------------------------------------- */

    if(!keepUserData)
        owners = SYS_OWNER_ALL;
    else
        owners = SYS_OWNER_SYSTEM;

    for(cnt = 0; cnt < numLists; cnt++)
    {
        FAPI_SYS_PRINT_DEBUG(1,"[SYS] Set carrier defaults: type %i list %i\n",
                   pList[cnt].network.type, cnt);

        (void)STATLST_Clear(&(pList[cnt].statLst), owners, NULL, NULL);

        pList[cnt].dirty = FTRUE;

        TPLIST_GenerateRefList(pList + cnt);

        /* call "set defaults" callback */
        if(SYS_GetConfig()->setCarrierDefaults != NULL)
        {
            SYS_GetConfig()->setCarrierDefaults(&(pList[cnt].network));
        }
    }
}

/*!
*******************************************************************************
**
** \brief   Get number of available carriers for specified network.
**
** \attention: Transponder lists can be used for all frontend types. In case of
**             satellite system, multiple lists may exist (one for each
**             satellite referenced by a LNB).
**
** \param   network TODO
** \param   numTp   TODO
**
** \return  Transponder list size for requested LNB.
**
******************************************************************************/
int32_t TPLIST_GetNumCarriers(const SYS_NETWORK_S * network, int16_t * numTp)
{
    TPLIST_LIST_S      *pList = NULL;
    int32_t             retVal;

    if((retVal = TPLIST_GetTargetList(network, NULL, &pList)) != FAPI_OK)
    {
        return retVal;
    }

    *numTp = pList->statLst.stat.numEntries;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   Get data of requested carrier on requested network.
**
** \attention: Transponder lists can be used for all frontend types. Only in
**             case of a satellite system, multiple lists may exist,
**             one for each satellite referenced by a LNB.
**
** \param   network TODO
** \param   lstPos  TODO
** \param   pTp     parDesc
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**
******************************************************************************/
int32_t TPLIST_GetCarrierData(const SYS_NETWORK_S * network, int16_t lstPos,
                              TPLIST_ITEM_S * pTp)
{
    TPLIST_LIST_S      *pList = NULL;
    int32_t             retVal;
    TPLIST_ITEM_S      *tpItem;

    if((retVal = TPLIST_GetTargetList(network, &lstPos, &pList)) != FAPI_OK)
    {
        return retVal;
    }

    tpItem = STATLST_GetEntry(&(pList->statLst), pList->refLst[lstPos]);
    if(tpItem == NULL)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    *pTp = *tpItem;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Add a carrier to the requested network's list.
**
** \param  network  TODO
** \param  pTp      Carrier parameters to be set.
** \param  owner    TODO
** \param  newPos   Position of added/updated entry is returned here
**                  (-1 if no data has been written).
**
** \return  One of the following status codes:
**          - #FAPI_OK Data has been added to list.
**          - #SMARTGO_SYS_ERR_ALREADY_IN_LIST Transponder is already in list.
**          - #SMARTGO_SYS_ERR_REPLACED_IN_LIST Transponder is already in list
**            but data has been updated.
**          - #SMARTGO_SYS_ERR_CAPACITY No more free items in list
**
** \sa
**          - TPLIST_TpRemove()
**          - TPLIST_TpReplace()
**
******************************************************************************/
int32_t TPLIST_AddCarrier(const SYS_NETWORK_S * network,
                          const TPLIST_ITEM_S * pTp, SYS_OWNER_E owner,
                          int16_t * newPos)
{
    TPLIST_LIST_S      *pList = NULL;
    TPLIST_ITEM_S      *pItem;
    int32_t             retVal;
    int32_t             lstPos;
    TPLIST_ITEM_S       newItem;
    int32_t             newStatIdx;

    (*newPos) = -1;

    if((retVal = TPLIST_GetTargetList(network, NULL, &pList)) != FAPI_OK)
    {
        return retVal;
    }

    newItem = *pTp;

    FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    /* check if carrier is already in list */
    lstPos = TPLIST_ItemIsInList(pList, &newItem);

    if(lstPos >= 0)             /* carrier already is in list */
    {
        pItem = STATLST_GetEntry(&(pList->statLst), pList->refLst[lstPos]);
        if(pItem == NULL)
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
        }

        /* check if list item is to be overwritten */
        else if((newItem.tsId >= 0) && (pItem->tsId < 0))
        {
            /* If a carrier name exists: do not overwrite it! */
            if ( !(STR_IsEmpty(STR_Build(pItem->nameCharSet, pItem->name))) )
            {
                (void)STR_Copy (STR_Build(pItem->nameCharSet, pItem->name),
                          newItem.name, &(newItem.nameCharSet),
                          pItem->nameCharSet, TPLIST_CARRIERNAME_MAX);
            }

            /* If signal quality of new carrier is unknown, do not overwrite
               existing value. */
            if(newItem.sigQuality == 0)
            {
                newItem.sigQuality = pItem->sigQuality;
            }

            retVal = STATLST_SetEntry(&(pList->statLst), pList->refLst[lstPos],
                                      &newItem, owner);

            if(retVal == FAPI_OK)
            {
                pList->dirty = FTRUE;

                (*newPos) = (int16_t) lstPos;
                retVal = SMARTGO_SYS_ERR_REPLACED_IN_LIST;
            }
        }
        else
        {
            (*newPos) = (int16_t) lstPos;
            retVal = SMARTGO_SYS_ERR_ALREADY_IN_LIST;
        }
    }
    else
    {
        /* add item to list */
        retVal = STATLST_AddEntry(&(pList->statLst), &newItem, owner,
                                  &newStatIdx);

        if(retVal == FAPI_OK)
        {
            pList->dirty = FTRUE;

            (void)TPLIST_InsertEntry(pList, newStatIdx, -1, newPos);
        }
    }

    FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);
    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Remove a carrier from the list.
**
** \param  network  Target network.
** \param  lstPos   List position to be removed.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa TPLIST_TpAdd, TPLIST_TpReplace
******************************************************************************/
int32_t TPLIST_RemoveCarrier(const SYS_NETWORK_S * network, int16_t lstPos)
{
    TPLIST_LIST_S      *pList = NULL;
    int32_t             retVal;
    int16_t             cnt;
    TPLIST_IDX          idx;

    if((retVal = TPLIST_GetTargetList(network, &lstPos, &pList)) != FAPI_OK)
    {
        return retVal;
    }

    FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    idx = pList->refLst[lstPos];

    retVal = STATLST_RemoveEntry(&(pList->statLst), idx);

    if(retVal == FAPI_OK)
    {
        /* Remove entry in reference list by shifting subsequent entries. */
        for(cnt = lstPos; cnt < (int16_t) (pList->statLst.stat.numEntries);
            cnt++)
        {
            pList->refLst[cnt] = pList->refLst[cnt + 1];
        }

        pList->dirty = FTRUE;
    }

    FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    /* ----- Remove all services referencing removed carrier --------------- */

    if(retVal == FAPI_OK)
    {
        TPLIST_CarrierRemoved(network, idx);
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief   Replace a carrier list entry.
**
** \param   network TODO
** \param   lstPos  TODO
** \param   pTp     Carrier parameters to be set.
** \param   owner   TODO
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**
** \sa
**          - TPLIST_TpAdd()
**          - TPLIST_TpRemove()
**
******************************************************************************/
int32_t TPLIST_ReplaceCarrier(const SYS_NETWORK_S * network, int16_t lstPos,
                              const TPLIST_ITEM_S * pTp, SYS_OWNER_E owner)
{
    TPLIST_LIST_S      *pList = NULL;
    int32_t             retVal;

    if((retVal = TPLIST_GetTargetList(network, &lstPos, &pList)) != FAPI_OK)
    {
        return retVal;
    }

    FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    retVal = STATLST_SetEntry(&(pList->statLst), pList->refLst[lstPos],
                              pTp, owner);

    if(retVal == FAPI_OK)
        pList->dirty = FTRUE;

    FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);
    return retVal;
}

int32_t TPLIST_GetIndex(const SYS_NETWORK_S * network, int16_t lstPos,
                        TPLIST_IDX * idx)
{
    TPLIST_LIST_S      *pList = NULL;
    int32_t             retVal;

    if((retVal = TPLIST_GetTargetList(network, &lstPos, &pList)) != FAPI_OK)
    {
        return retVal;
    }

    *idx = pList->refLst[lstPos];
    return FAPI_OK;
}

int32_t TPLIST_GetDataByIndex(const SYS_NETWORK_S * network, TPLIST_IDX idx,
                              TPLIST_ITEM_S * pTp)
{
    TPLIST_LIST_S      *pList = NULL;
    int32_t             retVal;
    void               *pEntry = NULL;

    if((retVal = TPLIST_GetTargetList(network, NULL, &pList)) != FAPI_OK)
    {
        return retVal;
    }

    if(pList != NULL)
    {
        pEntry = STATLST_GetEntry(&(pList->statLst), idx);
    }

    if ((pEntry != NULL) && (pTp != NULL))
    {
        *pTp = *((TPLIST_ITEM_S *) pEntry);
    } else if( (network != NULL) &&
               (network->type == FE_SYSTEM_TYPE_DIRECT) )
    {
        if (pTp != NULL)
        {
            memset(pTp,0,sizeof(TPLIST_ITEM_S));
        }
        return FAPI_OK;
    } else {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    return FAPI_OK;
}

int32_t TPLIST_SetDataByIndex(const SYS_NETWORK_S * network, TPLIST_IDX idx,
                              const TPLIST_ITEM_S * pTp, SYS_OWNER_E owner)
{
    TPLIST_LIST_S      *pList = NULL;
    int32_t             retVal;

    if((retVal = TPLIST_GetTargetList(network, NULL, &pList)) != FAPI_OK)
    {
        return retVal;
    }

    retVal = STATLST_SetEntry(&(pList->statLst), idx, pTp, owner);

    if(retVal == FAPI_OK)
        pList->dirty = FTRUE;

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Save carrier lists.
**
** This saves all available carrier lists.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t TPLIST_Save(FE_SYSTEM_TYPE_E feType, fbool_t force)
{
    fbool_t retVal = FFALSE;

    if(tpListDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    switch (feType)
    {
        #ifdef APPL_FESAT_ENABLED
        case FE_SYSTEM_TYPE_SATELLITE:

            /* Store also lists referred by carrier lists */
            (void)CFGSAT_Save(FFALSE);

            retVal = TPLIST_SaveLists(tpListDat->sat,
                                      CFGSAT_SAT_MAX, force, &lstSatValid);
            break;
        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED
        case FE_SYSTEM_TYPE_TERRESTRIAL:
            retVal = TPLIST_SaveLists(&(tpListDat->terr), 1,
                                      force, &lstTerValid);
            break;
        #endif /* APPL_FETERR_ENABLED */

        #ifdef APPL_FECABLE_ENABLED
        case FE_SYSTEM_TYPE_CABLE:
            retVal = TPLIST_SaveLists(&(tpListDat->cable), 1,
                                      force, &lstCabValid);
            break;
        #endif /* APPL_FECABLE_ENABLED */

        #ifdef APPL_FEDIRECT_ENABLED
        case FE_SYSTEM_TYPE_DIRECT:

            break;
        #endif /* APPL_FEDIRECT_ENABLED */

        default:
            return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    if(retVal == FFALSE)
        return SMARTGO_SYS_ERR_INTERNAL;
    else
        return FAPI_OK;
}

int32_t TPLIST_ResortLists(FE_SYSTEM_TYPE_E feType)
{
    TPLIST_LIST_S      *pList = NULL;
    uint16_t            cnt;
    uint16_t            numLists;
    int32_t             retVal;

    if(tpListDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    retVal = TPLIST_GetListBySystem(feType, &pList, &numLists);
    if(retVal != FAPI_OK)
        return retVal;

    for(cnt = 0; cnt < numLists; cnt++)
    {
        TPLIST_GenerateRefList(pList + cnt);
    }

    return FAPI_OK;
}

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC fbool_t TPLIST_LoadLists(TPLIST_LIST_S * pList, uint16_t numLists,
                                fbool_t lstValid)
{
    int32_t             res;
    uint32_t            fileId;
    uint32_t            fileRev;
    fbool_t             retVal = FFALSE;
    TPLIST_FILEHDR_S    fileHdr;
    uint16_t            listsToLoad;
    uint16_t            cnt;
    FLASHFS_HANDLE      file = NULL;

    FAPI_SYS_PRINT_DEBUG(1,"[SYS] Load carrier lists: type %i \n", pList->network.type);

    if ( TPLIST_GetFilename(pList->network.type, &fileId, &fileRev) != FAPI_OK )
    {
        return FFALSE;
    }

    for(;;)
    {
        /* If list is marked as invalid, trigger setting of defaults */
        if(!lstValid)
            break;

        file = FLASHFS_Open (fileId, FLASHFS_OPEN_READ, NULL);
        if(file == NULL)
        {
            break;
        }

        /* ----- read file header ------------------------------------------ */

        res = FLASHFS_Read (file, sizeof(TPLIST_FILEHDR_S), &fileHdr, NULL);
        if ( res != sizeof(TPLIST_FILEHDR_S) )
        {
            break;              /* file read error, file too small */
        }
        else if((fileHdr.id       != fileId) ||
                (fileHdr.revision != fileRev))
        {
            break;              /* invalid revision/identifier */
        }

        /* ----- read all entries ------------------------------------------ */

        listsToLoad = (fileHdr.numLists <= numLists) ? fileHdr.numLists
                                                     : numLists;
        if(listsToLoad < 1)
            break;

        for(cnt = 0; cnt < numLists; cnt++)     /* clear all lists */
        {
            (void)STATLST_Clear(&(pList[cnt].statLst), SYS_OWNER_ALL, NULL, NULL);
        }

        for(cnt = 0; cnt < listsToLoad; cnt++)  /* load available lists */
        {
            if ( STATLST_LoadFromFS(&(pList[cnt].statLst), FFALSE, file)
                 == FFALSE )
            {
                break;
            }

            /* generate reference list */
            TPLIST_GenerateRefList(pList + cnt);

            pList[cnt].dirty = FFALSE;
        }

        if(cnt >= listsToLoad)
        {
            retVal = FTRUE; /* all lists successfully read */
        }
        break; //end of for(;;)
    }

    if ( file != NULL )
    {
        (void)FLASHFS_Close(file);
    }

    /* clear lists, call 'set defaults' function if available */
    if ( retVal == FFALSE )
    {
        /* mark lists referring current list as invalid */
        SLIST_Invalidate();

        for(cnt = 0; cnt < numLists; cnt++)
        {
            FAPI_SYS_PRINT_DEBUG(1,"[SYS] Set carrier defaults: type %i list %i\n",
                       pList[cnt].network.type, cnt);

            (void)STATLST_Clear(&(pList[cnt].statLst), SYS_OWNER_ALL, NULL, NULL);

            pList[cnt].dirty = FTRUE;

            /* call "set defaults" callback */
            if(SYS_GetConfig()->setCarrierDefaults != NULL)
            {
                SYS_GetConfig()->setCarrierDefaults(&(pList[cnt].network));
            }
        }
    }

    return retVal;
}

FSTATIC fbool_t TPLIST_SaveLists(TPLIST_LIST_S * pList,
                                 uint16_t numLists, fbool_t force,
                                 fbool_t* lstValid)
{
    uint32_t            fileId;
    uint32_t            fileRev;
    fbool_t             retVal = FTRUE;
    TPLIST_FILEHDR_S    fileHdr;
    uint16_t            cnt;
    FLASHFS_HANDLE      file = NULL;

    /* check if list must be saved */
    if(!force)
    {
        for(cnt = 0; cnt < numLists; cnt++)
        {
            if(pList[cnt].dirty)
                break;
        }

        if(cnt >= numLists)
            return FTRUE;
    }

    if ( TPLIST_GetFilename(pList->network.type, &fileId, &fileRev) != FAPI_OK )
    {
        return FFALSE;
    }

    file = FLASHFS_Open (fileId, FLASHFS_OPEN_WRITE, NULL);

    if ( file == NULL )
    {
        return FFALSE;
    }

    /* ----- write file header ---------------------------------------------- */

    memset(&fileHdr,0,sizeof(TPLIST_FILEHDR_S));
    fileHdr.id       = fileId;
    fileHdr.revision = fileRev;
    fileHdr.numLists = numLists;

    if ( FLASHFS_Write(file, sizeof(TPLIST_FILEHDR_S), &fileHdr, NULL)
         != sizeof(TPLIST_FILEHDR_S) )
    {
        retVal = FFALSE;
    }

    /* ----- write all entries ---------------------------------------------- */

    else
    {
        for ( cnt = 0; cnt < fileHdr.numLists; cnt++ )
        {
            retVal = STATLST_WriteToFS(&(pList[cnt].statLst), FFALSE, file);
            if ( retVal == FFALSE ) break;

            pList[cnt].dirty = FFALSE;
        }
    }

    (void)FLASHFS_Close(file);

    if ( retVal == FTRUE )
    {
        *lstValid = FTRUE;
    }

    return retVal;
}

FSTATIC int32_t TPLIST_ItemIsInList(TPLIST_LIST_S * pList,
                                    const TPLIST_ITEM_S * pTp)
{
    int32_t             cnt;
    int32_t             freqTol = 0;    // tolerance in kHz (Sat) or Hz (Cable, Terr)
    TPLIST_ITEM_S      *pItem;

    /* if item has a valid id, search for an ID match first */
    if(pTp->tsId >= 0)
    {
        for(cnt = 0; cnt < (int32_t)pList->statLst.stat.numEntries; cnt++)
        {
            pItem = STATLST_GetEntry(&(pList->statLst), pList->refLst[cnt]);
            if(pItem == NULL)
                continue;

            /* compare TS-IDs if possible (-> both have a valid ID) */
            if((pItem->tsId >= 0) && (pItem->tsId == pTp->tsId))
            {
                //FAPI_SYS_PRINT_DEBUG(1,"TP MATCH 1: %X - %X \n", pTp->tsId, pTpLst->tpRef[cnt].tsId);
                return cnt;
            }
        }
    }

    /* If no matching item was found, compare by parameters */

    /* calculate frequency tolerance */
    switch (pList->network.type)
    {
        #ifdef APPL_FESAT_ENABLED
        case FE_SYSTEM_TYPE_SATELLITE:
            freqTol = pTp->pars.sat.symbolRate / 2;
            if(freqTol > 2000)
                freqTol = 2000;
            break;
        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED
        case FE_SYSTEM_TYPE_TERRESTRIAL:
            freqTol = 1000000;
            break;
        #endif /* APPL_FETERR_ENABLED */

        #ifdef APPL_FECABLE_ENABLED
        case FE_SYSTEM_TYPE_CABLE:
            freqTol = 1000000;
            break;
        #endif /* APPL_FECABLE_ENABLED */

        #ifdef APPL_FEDIRECT_ENABLED
        case FE_SYSTEM_TYPE_DIRECT:
            return -1;
        #endif /* APPL_FEDIRECT_ENABLED */

        default:
            return -1;
    }

    for(cnt = 0; cnt < (int32_t)pList->statLst.stat.numEntries; cnt++)
    {
        pItem = STATLST_GetEntry(&(pList->statLst), pList->refLst[cnt]);
        if(pItem == NULL)
            continue;

        switch (pList->network.type)
        {
            #ifdef APPL_FESAT_ENABLED
            case FE_SYSTEM_TYPE_SATELLITE:
                if((pItem->pars.sat.pol == pTp->pars.sat.pol) &&
                   ((int32_t)pItem->pars.sat.frequency <=
                    ((int32_t) pTp->pars.sat.frequency + freqTol))
                   && ((int32_t)pItem->pars.sat.frequency >=
                       ((int32_t) pTp->pars.sat.frequency - freqTol)))
                {
                    return cnt;
                }
                break;
            #endif /* APPL_FESAT_ENABLED */

            #ifdef APPL_FETERR_ENABLED
            case FE_SYSTEM_TYPE_TERRESTRIAL:
                if(((int32_t)pItem->pars.terr.frequency <=
                    ((int32_t) pTp->pars.terr.frequency + freqTol))
                   && ((int32_t)pItem->pars.terr.frequency >=
                       ((int32_t) pTp->pars.terr.frequency - freqTol)))
                {
                    return cnt;
                }
                break;
            #endif /* APPL_FETERR_ENABLED */

            #ifdef APPL_FECABLE_ENABLED
            case FE_SYSTEM_TYPE_CABLE:
                if(((int32_t)pItem->pars.cable.frequency <=
                    ((int32_t) pTp->pars.cable.frequency + freqTol))
                   && ((int32_t)pItem->pars.cable.frequency >=
                       ((int32_t) pTp->pars.cable.frequency - freqTol)))
                {
                    return cnt;
                }
                break;
            #endif /* APPL_FECABLE_ENABLED */

            #ifdef APPL_FEDIRECT_ENABLED
            case FE_SYSTEM_TYPE_DIRECT:
                return -1;
            #endif /* APPL_FEDIRECT_ENABLED */

            default:
                /* do nothing */
                break;
        }
    }

    return -1;
}

FSTATIC int32_t TPLIST_GetTargetList(const SYS_NETWORK_S * network,
                                     const int16_t * lstPos,
                                     TPLIST_LIST_S ** pList)
{
    int32_t             retVal = FAPI_OK;
    TPLIST_LIST_S      *targetList = NULL;

    if(tpListDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if(network == NULL)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    switch (network->type)
    {
        #ifdef APPL_FESAT_ENABLED
        case FE_SYSTEM_TYPE_SATELLITE:
            if((network->satIdx >= 0) && (network->satIdx < CFGSAT_SAT_MAX))
            {
                targetList = tpListDat->sat + network->satIdx;
            }
            else
            {
                retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
            }
            break;
        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED
        case FE_SYSTEM_TYPE_TERRESTRIAL:
            targetList = &(tpListDat->terr);
            break;
        #endif /* APPL_FETERR_ENABLED */

        #ifdef APPL_FECABLE_ENABLED
        case FE_SYSTEM_TYPE_CABLE:
            targetList = &(tpListDat->cable);
            break;
        #endif /* APPL_FECABLE_ENABLED */

        #ifdef APPL_FEDIRECT_ENABLED
        case FE_SYSTEM_TYPE_DIRECT:
            break;
        #endif /* APPL_FEDIRECT_ENABLED */
        default:
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
            break;
    }

    /* check lstPos argument if requested */
    if((retVal == FAPI_OK) && (lstPos != NULL) && (targetList != NULL) )
    {
        if(((*lstPos) < 0) ||
           ((*lstPos) >= (int16_t)targetList->statLst.stat.numEntries))
        {
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
    }

    if(retVal == FAPI_OK)
    {
        *pList = targetList;
    }

    return retVal;
}

FSTATIC int32_t TPLIST_GetListBySystem(FE_SYSTEM_TYPE_E feType,
                                       TPLIST_LIST_S ** ppList,
                                       uint16_t * numLists)
{
    int32_t             retVal = FAPI_OK;

    switch (feType)
    {
        #ifdef APPL_FESAT_ENABLED
        case FE_SYSTEM_TYPE_SATELLITE:
            *ppList = tpListDat->sat;
            *numLists = CFGSAT_SAT_MAX;
            break;
        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED
        case FE_SYSTEM_TYPE_TERRESTRIAL:
            *ppList = &(tpListDat->terr);
            *numLists = 1;
            break;
        #endif /* APPL_FETERR_ENABLED */

        #ifdef APPL_FECABLE_ENABLED
        case FE_SYSTEM_TYPE_CABLE:
            *ppList = &(tpListDat->cable);
            *numLists = 1;
            break;
        #endif /* APPL_FECABLE_ENABLED */

        default:
            retVal = SMARTGO_SYS_ERR_CONFIG_ERROR;
            break;
    }

    return retVal;
}

FSTATIC int32_t TPLIST_GetFilename(FE_SYSTEM_TYPE_E feType,
                                   uint32_t* fileId,
                                   uint32_t* fileRev)
{
    switch (feType)
    {
        #ifdef APPL_FESAT_ENABLED
        case FE_SYSTEM_TYPE_SATELLITE:
            if(fileId)  *fileId  = FILEID_TPLIST_SAT;
            if(fileRev) *fileRev = FILEREV_TPLIST_SAT;
            break;
        #endif /* APPL_FESAT_ENABLED */

        #ifdef APPL_FETERR_ENABLED
        case FE_SYSTEM_TYPE_TERRESTRIAL:
            if(fileId)  *fileId  = FILEID_TPLIST_TER;
            if(fileRev) *fileRev = FILEREV_TPLIST_TER;
            break;
        #endif /* APPL_FETERR_ENABLED */

        #ifdef APPL_FECABLE_ENABLED
        case FE_SYSTEM_TYPE_CABLE:
            if(fileId)  *fileId  = FILEID_TPLIST_CAB;
            if(fileRev) *fileRev = FILEREV_TPLIST_CAB;
            break;
        #endif /* APPL_FECABLE_ENABLED */

        default:
            return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    return FAPI_OK;
}

FSTATIC void TPLIST_GenerateRefList(TPLIST_LIST_S * pList)
{
    TPLIST_IDX          refCnt;
    int16_t             numRefEntries = 1;
    int16_t             newPos;

    for(refCnt = 0; refCnt < pList->statLst.stat.afterLast; refCnt++)
    {
        if ( (TPLIST_InsertEntry(pList, refCnt, numRefEntries, &newPos)
              == FTRUE) )
        {
            numRefEntries++;
        }
    }
}

/*
listSize: current reference list size including item to be inserted. If < 0,
          reference list size matches FSTATIC list size.
*/
FSTATIC fbool_t TPLIST_InsertEntry(TPLIST_LIST_S * pList, TPLIST_IDX idx,
                                   int16_t listSize, int16_t * newPos)
{
    TPLIST_ITEM_S      *listEntry;
    TPLIST_ITEM_S      *refEntry;
    int16_t             index;
    int16_t             shiftCnt;
    fbool_t             insert = FFALSE;

    listEntry = STATLST_GetEntry (&(pList->statLst), idx);
    if ( listEntry == NULL ) return FFALSE;

    if ( listSize < 0 ) listSize = pList->statLst.stat.numEntries;

    index = listSize - 2;

    while ( index >= -1 )
    {
        if ( index == -1 )
        {
            insert = FTRUE;
        }
        else
        {
            refEntry = STATLST_GetEntry (&(pList->statLst),
                                         pList->refLst[index]);
            if ( refEntry != NULL )
            {
                /* test for time and date to keep chronological order */
                switch ( pList->network.type )
                {
                    #ifdef APPL_FESAT_ENABLED
                    case FE_SYSTEM_TYPE_SATELLITE:
                            insert = (fbool_t)(listEntry->pars.sat.frequency >
                                       refEntry->pars.sat.frequency);
                            break;
                    #endif /* APPL_FESAT_ENABLED */

                    #ifdef APPL_FETERR_ENABLED
                    case FE_SYSTEM_TYPE_TERRESTRIAL:
                            insert = (fbool_t)(listEntry->pars.terr.frequency >
                                       refEntry->pars.terr.frequency);
                            break;
                    #endif /* APPL_FETERR_ENABLED */

                    #ifdef APPL_FECABLE_ENABLED
                    case FE_SYSTEM_TYPE_CABLE:
                            insert = (fbool_t)(listEntry->pars.cable.frequency >
                                       refEntry->pars.cable.frequency);
                            break;
                    #endif /* APPL_FECABLE_ENABLED */

                    default:
                        /* do nothing */
                        break;
                }
            }
        }

        if ( insert )
        {
            /* shift subsequent entries */
            for(shiftCnt = listSize - 2; shiftCnt > index; shiftCnt--)
            {
                pList->refLst[shiftCnt + 1] = pList->refLst[shiftCnt];
            }

            *newPos = index + 1;

            pList->refLst[index + 1] = idx;
            return FTRUE;
        }

        index--;
    }

    return FFALSE;
}

FSTATIC void TPLIST_CarrierRemoved (const SYS_NETWORK_S* network,
                                   TPLIST_IDX tpIdx)
{
    int16_t             cnt;
    uint16_t            listSize;
    SLIST_ITEM_S   listItem;
    SYS_SERVLOC_S  servLoc;

    /* Remove all services referencing removed carrier */

    for ( cnt = 0; cnt < SLIST_GLOB_NUMLISTS; cnt++ )
    {
        servLoc.listId = cnt + SLIST_OFFSET_GLOB;

        listSize = SLIST_ListSizeGet (servLoc.listId);
        if ( listSize < 1 ) continue;

        for(servLoc.index = (listSize - 1); servLoc.index >= 0; servLoc.index--)
        {
            if(SLIST_EntryGet(&servLoc, &listItem) != FAPI_OK)
                continue;

            /* test if network matches */
            if ( listItem.network.type != network->type ) continue;
            else if((listItem.network.type == FE_SYSTEM_TYPE_SATELLITE) &&
                      (listItem.network.satIdx != network->satIdx) )
            {
                continue;
            }

            if ( listItem.tpIdx == tpIdx ) /* test if carrier index matches */
            {
                (void)SLIST_EntryRemove(&servLoc, FFALSE);
            }
        }
    }
}

FSTATIC void TPLIST_ClearCb(void *arg, int32_t index)
{
    TPLIST_CarrierRemoved(arg, index);
}

/*!
*******************************************************************************
**
** \brief   TPLIST_UpdateCarrier
**
** \note    This function update parameters of carrier
**
** \param   feType        calbe, satellite, terrestrial
** \param   pNitTPService the updated parameter of one carrier  get from NIT table
** \param   satIdx        Satellite index if feType is FE_SYSTEM_TYPE_SATELLITE.
**
** \return  -1 failed, or index of the updated carrier
**
*******************************************************************************
*/
int32_t TPLIST_UpdateCarrier(FE_SYSTEM_TYPE_E feType, PSISI_NIT_TP_INFO_S *pNitTPService, CFGSAT_IDX satIdx)
{
    int32_t             cnt;
    TPLIST_ITEM_S      *pItem;
    TPLIST_LIST_S      *pList = NULL;
    uint16_t            numLists = 0;
    fbool_t             checkflag = FFALSE;

    if(pNitTPService == NULL)
    {
        return -1;
    }
    if(feType == FE_SYSTEM_TYPE_DIRECT)
    {
        pNitTPService->tpIdx = 0;
        return 0;
    }
    if ( TPLIST_GetListBySystem(feType, &pList, &numLists) != FAPI_OK )
    {
        return -1;
    }

    //Satellite has more than one list
    if(feType == FE_SYSTEM_TYPE_SATELLITE)
    {
        if(satIdx < (int16_t)numLists)
        {
            pList += satIdx;
        }
        else
        {
            return -1;
        }

        if(pNitTPService->tpItem.pars.sat.frequency != 0)
        {
            checkflag = FTRUE;
        }
    }
    else if(feType == FE_SYSTEM_TYPE_CABLE)
    {
        if(pNitTPService->tpItem.pars.cable.frequency != 0)
        {
            checkflag = FTRUE;
        }
    }
    else if(feType == FE_SYSTEM_TYPE_TERRESTRIAL)
    {
        if(pNitTPService->tpItem.pars.terr.frequency != 0)
        {
            checkflag = FTRUE;
        }
    }

    FAPI_SYS_PRINT_DEBUG(5,"Target TP tsId=0x%x, networkId=0x%x\n", pNitTPService->tpItem.tsId, pNitTPService->tpItem.networkId);
    for(cnt = 0; cnt < (int32_t)pList->statLst.stat.numEntries; cnt++)
    {
        pItem = STATLST_GetEntry(&(pList->statLst), pList->refLst[cnt]);
        if(pItem == NULL)
        {
            continue;
        }
        //find match item
        if( (pItem->tsId == pNitTPService->tpItem.tsId) &&
            (pItem->networkId == pNitTPService->tpItem.networkId) &&
            (pItem->onId == pNitTPService->tpItem.onId))
        {
            FAPI_SYS_PRINT_DEBUG(5,"Find match TP cnt =%d, pItem->tsId=0x%x, pItem->networkId=0x%x\n", cnt,pItem->tsId, pItem->networkId);

            //check parameter
            /* calculate frequency tolerance */
            switch (pList->network.type)
            {
                #ifdef APPL_FESAT_ENABLED
                case FE_SYSTEM_TYPE_SATELLITE:
                    if(checkflag)
                    {
                        if(((int32_t)pItem->pars.sat.frequency != (int32_t)pNitTPService->tpItem.pars.sat.frequency) ||
                           ((int32_t)pItem->pars.sat.pol!= ((int32_t) pNitTPService->tpItem.pars.sat.pol))||
                           ((int32_t)pItem->pars.sat.rollOff!= ((int32_t) pNitTPService->tpItem.pars.sat.rollOff))||
                           ((int32_t)pItem->pars.sat.modulation!= ((int32_t) pNitTPService->tpItem.pars.sat.modulation))||
                           ((int32_t)pItem->pars.sat.symbolRate!= ((int32_t) pNitTPService->tpItem.pars.sat.symbolRate))||
                           ((int32_t)pItem->pars.sat.fec!= ((int32_t) pNitTPService->tpItem.pars.sat.fec)))
                        {
                            FAPI_SYS_PRINT_DEBUG(5,"%d, %d, %d, %d, %d, %d, \n", pItem->pars.sat.frequency, pItem->pars.sat.pol, pItem->pars.sat.rollOff, pItem->pars.sat.modulation, pItem->pars.sat.symbolRate, pItem->pars.sat.fec);
                            FAPI_SYS_PRINT_DEBUG(5,"%d, %d, %d, %d, %d, %d, \n", pNitTPService->tpItem.pars.sat.frequency, pNitTPService->tpItem.pars.sat.pol, pNitTPService->tpItem.pars.sat.rollOff, pNitTPService->tpItem.pars.sat.modulation, pNitTPService->tpItem.pars.sat.symbolRate, pNitTPService->tpItem.pars.sat.fec);
                            pItem->pars = pNitTPService->tpItem.pars;
                        }
                    }
                    pNitTPService->tpIdx = pList->refLst[cnt];
                    return pList->refLst[cnt];
                #endif /* APPL_FESAT_ENABLED */

                #ifdef APPL_FETERR_ENABLED
                case FE_SYSTEM_TYPE_TERRESTRIAL:
                    if(checkflag)
                    {
                        if(((int32_t)pItem->pars.terr.frequency != ((int32_t) pNitTPService->tpItem.pars.terr.frequency)) ||
                            ((int32_t)pItem->pars.terr.bandwidth != ((int32_t) pNitTPService->tpItem.pars.terr.bandwidth)) ||
                            ((int32_t)pItem->pars.terr.modulation != ((int32_t) pNitTPService->tpItem.pars.terr.modulation)) ||
                            ((int32_t)pItem->pars.terr.codeRateHP != ((int32_t) pNitTPService->tpItem.pars.terr.codeRateHP)) ||
                            ((int32_t)pItem->pars.terr.codeRateLP != ((int32_t) pNitTPService->tpItem.pars.terr.codeRateLP)) ||
                            ((int32_t)pItem->pars.terr.guardInterval != ((int32_t) pNitTPService->tpItem.pars.terr.guardInterval)) ||
                            ((int32_t)pItem->pars.terr.transmission != ((int32_t) pNitTPService->tpItem.pars.terr.transmission)))
                        {
                            pItem->pars = pNitTPService->tpItem.pars; //for test
                        }
                    }
                    pNitTPService->tpIdx = pList->refLst[cnt];
                    return pList->refLst[cnt];
                #endif /* APPL_FETERR_ENABLED */

                #ifdef APPL_FECABLE_ENABLED
                case FE_SYSTEM_TYPE_CABLE:
                    if(checkflag)
                    {
                        if(((int32_t)pItem->pars.cable.frequency != ((int32_t) pNitTPService->tpItem.pars.cable.frequency)) ||
                            ((int32_t)pItem->pars.cable.symbolRate != ((int32_t) pNitTPService->tpItem.pars.cable.symbolRate)) ||
                            ((int32_t)pItem->pars.cable.modulation != ((int32_t) pNitTPService->tpItem.pars.cable.modulation)) ||
                            ((int32_t)pItem->pars.cable.fec != ((int32_t) pNitTPService->tpItem.pars.cable.fec)))
                        {
                            pItem->pars = pNitTPService->tpItem.pars;
                        }
                    }
                    pNitTPService->tpIdx = pList->refLst[cnt];
                    return pList->refLst[cnt];
                #endif /* APPL_FECABLE_ENABLED */

                #ifdef APPL_FEDIRECT_ENABLED
                case FE_SYSTEM_TYPE_DIRECT:
                    return -1;
                #endif /* APPL_FEDIRECT_ENABLED */

                default:
                    return -1;
            }
        }
    }

    return -1;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
