/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/cfgsat.c
**
**              $Id: cfgsat.c 5871 2010-03-02 13:42:55Z jmorha $
**
** \brief       Satellite frontend configuration handling.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save -e574 -e713 -e737 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <flashfs/flashfs.h>

#include "sys_api.h"
#include "av.h"
#include "statlist.h"
#include "utils.h"

/* It is only needed if USALS is disabled to avoid flash file size change. */
#ifndef APPL_USALS_ENABLED
#include "usals.h"
#endif

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    uint32_t            id;             //!< file identification
    uint32_t            revision;       //!< file revision
    CFGSAT_FEPARS_S     feCfg;          //!< frontend configuration
    USALS_PARAMS_S      usalsParams;    //!< USALS parameters;
    uint32_t            idEnd;          //!< file identification

} CFGSAT_FILEHDR_S;

typedef struct
{
    fbool_t             dirty;                  //!< FTRUE: current data does
                                                //!< not match data stored in
                                                //!< Flash.
    CFGSAT_FILEHDR_S    satDat;

    STATLST_S           statLst;                //!< FSTATIC satellite list.
    CFGSAT_IDX          refLst[CFGSAT_SAT_MAX]; //!< Continuous sat list.

    fbool_t             usalsValid;             //!< USALS data validity status.

} CFGSAT_DAT_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC CFGSAT_DAT_S *cfgsatDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC fbool_t CFGSAT_SatDatLoad        (void);
FSTATIC void    CFGSAT_SatDatDefaultsSet (fbool_t keepUserData);
FSTATIC int32_t CFGSAT_CheckArgs         (int16_t lstPos);
FSTATIC void    CFGSAT_GenerateRefList   (void);

#ifdef APPL_DISEQC12_ENABLED
FSTATIC int32_t DIS12_Init               (void);
#endif /* APPL_DISEQC12_ENABLED */

int32_t CFGSAT_Init(void)
{
    int32_t     retVal;
    uint32_t    maxFileSize;

    DBG_Assert(cfgsatDat == NULL);

    /* Test if maximum flash file size is sufficient. */
    maxFileSize = sizeof(CFGSAT_FILEHDR_S)
                + STATLST_GetMaxDataSize (CFGSAT_SAT_MAX,
                                          sizeof(CFGSAT_SAT_S));

    retVal = FLASH_CheckFileSize (FILEID_SATDAT, maxFileSize);

    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    cfgsatDat = UTILS_Malloc(sizeof(CFGSAT_DAT_S));
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    memset(cfgsatDat, 0, sizeof(CFGSAT_DAT_S));

    cfgsatDat->dirty      = FTRUE;
    cfgsatDat->usalsValid = FFALSE;

    retVal = STATLST_Init (&(cfgsatDat->statLst), CFGSAT_SAT_MAX,
                           sizeof(CFGSAT_SAT_S));
    if(retVal != FAPI_OK)
    {
        return retVal;
    }

    #ifdef APPL_DISEQC12_ENABLED
    retVal = DIS12_Init();
    if(retVal != FAPI_OK)
    {
        return retVal;
    }
    #endif /* APPL_DISEQC12_ENABLED */

    (void)CFGSAT_Load();

    return FAPI_OK;
}

/* remove lists in flash, restore default lists in SDRAM */
void CFGSAT_RestoreDefaults (fbool_t keepUserData)
{
    /* Delete stored data */
    (void)FLASHFS_Delete(FILEID_SATDAT);

    CFGSAT_SatDatDefaultsSet(keepUserData);
}


fbool_t CFGSAT_SavingIsNecessary (void)
{
    if ( cfgsatDat == NULL )
    {
        return FFALSE;
    }

    return cfgsatDat->dirty;
}

int32_t CFGSAT_Load(void)
{
    FAPI_SYS_PRINT_DEBUG(1,"[SYS] Load satellite list\n");
    if ( CFGSAT_SatDatLoad() == FFALSE )
    {
        /* mark lists depending on current list as invalid */
        TPLIST_Invalidate(FE_SYSTEM_TYPE_SATELLITE);

        FAPI_SYS_PRINT_DEBUG(1,"[SYS] Set satellite defaults\n");
        CFGSAT_SatDatDefaultsSet(FFALSE);
    }
    else
    {
        cfgsatDat->usalsValid = FTRUE;
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Save LNB/Satellite data.
**
** \attention This function is valid for satellite frontends only!
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t CFGSAT_Save(fbool_t force)
{
    int32_t         retVal = FAPI_OK;
    FLASHFS_HANDLE  file;

    if ( cfgsatDat == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    /* test if data must be saved. */
    if((!force) && (!(cfgsatDat->dirty)))
    {
        return FAPI_OK;
    }

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    file = FLASHFS_Open(FILEID_SATDAT, FLASHFS_OPEN_WRITE, &retVal);

    if ( file != NULL )
    {
        cfgsatDat->satDat.id       = FILEID_SATDAT;
        cfgsatDat->satDat.idEnd    = FILEID_SATDAT;
        cfgsatDat->satDat.revision = FILEREV_SATDAT;

        /* write header */
        retVal = FLASHFS_Write(file, sizeof(CFGSAT_FILEHDR_S),
                               &(cfgsatDat->satDat), NULL);

        if ( retVal >= 0 )
        {
            if ( retVal != sizeof(CFGSAT_FILEHDR_S) )
            {
                retVal = SMARTGO_SYS_ERR_INTERNAL;
            }
            else
            {
                if ( STATLST_WriteToFS(&(cfgsatDat->statLst), FFALSE, file) ==
                     FFALSE )
                {
                    retVal = SMARTGO_SYS_ERR_INTERNAL;
                }
                else
                {
                    retVal = FAPI_OK;
                    cfgsatDat->dirty = FFALSE;
                }
            }
        }

        (void)FLASHFS_Close(file);
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);
    return retVal;
}

int32_t CFGSAT_AddSat(const CFGSAT_SAT_S* dat, SYS_OWNER_E owner,
                      int16_t* newPos)
{
    int32_t retVal;
    int32_t newStatIdx;

    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    retVal = STATLST_AddEntry(&(cfgsatDat->statLst), dat, owner, &newStatIdx);
    if(retVal != FAPI_OK)
    {
        return retVal;
    }

    *newPos = cfgsatDat->statLst.stat.numEntries - 1;

    DBG_Assert((*newPos) >= 0);

    cfgsatDat->refLst[*newPos] = newStatIdx;
    cfgsatDat->dirty = FTRUE;
    return FAPI_OK;
}

int32_t CFGSAT_RemoveSat (int16_t lstPos)
{
    int32_t         retVal;
    int16_t         cnt;
    SYS_NETWORK_S   network;
    CFGSAT_IDX      idx;

    if((retVal = CFGSAT_CheckArgs(lstPos)) != FAPI_OK)
    {
        return retVal;
    }

    idx = cfgsatDat->refLst[lstPos];

    retVal = STATLST_RemoveEntry(&(cfgsatDat->statLst), idx);
    if(retVal != FAPI_OK)
    {
        return retVal;
    }

    /* Remove entry in reference list by shifting subsequent entries. */
    for(cnt = lstPos; cnt < (int16_t) (cfgsatDat->statLst.stat.numEntries);
        cnt++)
    {
        cfgsatDat->refLst[cnt] = cfgsatDat->refLst[cnt + 1];
    }

    cfgsatDat->dirty = FTRUE;

    network.type = FE_SYSTEM_TYPE_SATELLITE;
    network.satIdx = idx;

    /* Clear corresponding carrier list */
    (void)TPLIST_Clear(&network, SYS_OWNER_ALL);

    return FAPI_OK;
}

int32_t CFGSAT_ReplaceSat(int16_t lstPos, const CFGSAT_SAT_S* dat,
                          SYS_OWNER_E owner)
{
    int32_t             retVal;

    if((retVal = CFGSAT_CheckArgs(lstPos)) != FAPI_OK)
    {
        return retVal;
    }

    return CFGSAT_SetDataByIndex(cfgsatDat->refLst[lstPos], dat, owner);
}

int32_t CFGSAT_GetSatData(int16_t lstPos, CFGSAT_SAT_S* dat)
{
    int32_t             retVal;

    retVal = CFGSAT_CheckArgs(lstPos);
    if(retVal != FAPI_OK)
    {
        return retVal;
    }

    return CFGSAT_GetDataByIndex(cfgsatDat->refLst[lstPos], dat);
}

/*!
*******************************************************************************
**
** \brief  Get number of available satellites.
**
** \attention This function is valid for satellite frontends only!
**
** \return Number of available satellites.
******************************************************************************/
int32_t CFGSAT_GetNumSats(int16_t* numSats)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    *numSats = cfgsatDat->statLst.stat.numEntries;
    return FAPI_OK;
}

int32_t CFGSAT_GetIndex (int16_t lstPos, CFGSAT_IDX* idx)
{
    int32_t retVal;

    retVal = CFGSAT_CheckArgs(lstPos);
    if(retVal != FAPI_OK)
    {
        return retVal;
    }

    *idx = cfgsatDat->refLst[lstPos];
    return FAPI_OK;
}

int32_t CFGSAT_GetListPos (CFGSAT_IDX idx, int16_t* lstPos)
{
    int16_t lstCnt;
    int32_t retVal;

    if ( cfgsatDat == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else if ( lstPos == NULL )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;

        for ( lstCnt = 0; lstCnt < cfgsatDat->statLst.stat.numEntries;
              lstCnt++ )
        {
            if ( cfgsatDat->refLst[lstCnt] == idx )
            {
                retVal  = FAPI_OK;
                *lstPos = lstCnt;
                break;
            }
        }
    }

    return retVal;
}

int32_t CFGSAT_GetDataByIndex (CFGSAT_IDX idx, CFGSAT_SAT_S* pSatInf)
{
    void* pEntry;

    if ( cfgsatDat == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    pEntry = STATLST_GetEntry(&(cfgsatDat->statLst), idx);

    if(pEntry == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    *pSatInf = *((CFGSAT_SAT_S *) pEntry);
    return FAPI_OK;
}

int32_t CFGSAT_SetDataByIndex(CFGSAT_IDX idx, const CFGSAT_SAT_S* pSatInf,
                              SYS_OWNER_E owner)
{
    int32_t             retVal;

    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    retVal = STATLST_SetEntry(&(cfgsatDat->statLst), idx, pSatInf, owner);
    if(retVal == FAPI_OK)
    {
        cfgsatDat->dirty = FTRUE;
    }
    return retVal;
}

int32_t CFGSAT_GetFrontendSetup(CFGSAT_FEPARS_S* cfg)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    *cfg = cfgsatDat->satDat.feCfg;
    return FAPI_OK;
}

int32_t CFGSAT_SetFrontendSetup(const CFGSAT_FEPARS_S* cfg)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    cfgsatDat->satDat.feCfg = *cfg;
    cfgsatDat->dirty        = FTRUE;
    return FAPI_OK;
}


FSTATIC void CFGSAT_SatDatDefaultsSet(fbool_t keepUserData)
{
    SYS_OWNER_E         owners;
    uint8_t             cnt;

    memset(&(cfgsatDat->satDat.feCfg),       0, sizeof(CFGSAT_FEPARS_S));
    memset(&(cfgsatDat->satDat.usalsParams), 0, sizeof(USALS_PARAMS_S));

    cfgsatDat->usalsValid = FFALSE;

    cfgsatDat->satDat.feCfg.installType = CFGSAT_INSTALLTYPE_INDEPENDENT;

    for(cnt = 0; cnt < RSRC_FE_MAX; cnt++)
    {
        cfgsatDat->satDat.feCfg.controlMode[cnt] = CFGSAT_FEMODE_DISEQC10;
    }

    /* --------------------------------------------------------------------- */

    if(!keepUserData)
    {
        owners = SYS_OWNER_ALL;
    }
    else
    {
        owners = SYS_OWNER_SYSTEM;
    }

    (void)STATLST_Clear(&(cfgsatDat->statLst), owners, NULL, NULL);

    CFGSAT_GenerateRefList();

    cfgsatDat->dirty = FTRUE;

    /* --------------------------------------------------------------------- */

    if(SYS_GetConfig()->setSatDefaults != NULL)
    {
        SYS_GetConfig()->setSatDefaults();
    }
}

FSTATIC fbool_t CFGSAT_SatDatLoad (void)
{
    fbool_t         retVal = FFALSE;
    FLASHFS_HANDLE  file;

    file = FLASHFS_Open(FILEID_SATDAT, FLASHFS_OPEN_READ,NULL);

    if ( file == NULL )
    {
        return FFALSE;
    }

    do
    {
        /* ----- read file header ------------------------------------------ */

        if(FLASHFS_Read(file,
                        sizeof(CFGSAT_FILEHDR_S),
                        &(cfgsatDat->satDat),NULL) != sizeof(CFGSAT_FILEHDR_S))
        {
            break;
        }

        if ( (cfgsatDat->satDat.id       != FILEID_SATDAT) ||
             (cfgsatDat->satDat.idEnd    != FILEID_SATDAT) ||
             (cfgsatDat->satDat.revision != FILEREV_SATDAT) )
        {
            break;
        }

        (void)STATLST_Clear(&(cfgsatDat->statLst), SYS_OWNER_ALL, NULL, NULL);

        /* ----- read satellite list --------------------------------------- */

        if ( STATLST_LoadFromFS(&(cfgsatDat->statLst), FFALSE, file) == FFALSE )
        {
            break;
        }

        /* ----- generate reference list ----------------------------------- */

        CFGSAT_GenerateRefList();

        retVal = FTRUE;

        cfgsatDat->dirty = FFALSE;
    }
    while ( 0 );

    (void)FLASHFS_Close(file);

    return retVal;
}

#ifdef APPL_USALS_ENABLED

int32_t CFGSAT_SetUsalsParams(const USALS_PARAMS_S* params)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    cfgsatDat->satDat.usalsParams = *params;
    cfgsatDat->dirty              = FTRUE;
    cfgsatDat->usalsValid         = FTRUE;

    return FAPI_OK;
}

int32_t CFGSAT_GetUsalsParams(USALS_PARAMS_S* params)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    if(!(cfgsatDat->usalsValid))
    {
        return SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }

    *params = cfgsatDat->satDat.usalsParams;
    return FAPI_OK;
}

#endif /* APPL_USALS_ENABLED */

/* ------------------------------------------------------------------------- */

FSTATIC int32_t CFGSAT_CheckArgs(int16_t lstPos)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else if((lstPos < 0) || (lstPos >= cfgsatDat->statLst.stat.numEntries))
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        return FAPI_OK;
    }
}

FSTATIC void CFGSAT_GenerateRefList(void)
{
    int32_t             cnt;
    void               *listEntry;
    int32_t             numEntries = 0;

    for(cnt = 0; cnt < cfgsatDat->statLst.stat.afterLast; cnt++)
    {
        listEntry = STATLST_GetEntry(&(cfgsatDat->statLst), cnt);

        /* If entry is non-empty, set reference on it */
        if(listEntry != NULL)
        {
            cfgsatDat->refLst[numEntries++] = cnt;
        }
    }
}

/* ------------------------------------------------------------------------- */

#ifdef APPL_DISEQC12_ENABLED

int32_t DIS12_Init(void)
{
    return FAPI_OK;
}

int32_t DIS12_DriveMotor(int8_t feIdx, FE_WE_FLAG_E westEastFlag,
                         uint8_t numSteps)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    return AV_Dis12DriveMotor(feIdx, westEastFlag, numSteps);
}

int32_t DIS12_HaltMotor(int8_t feIdx)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    return AV_Dis12HaltMotor(feIdx);
}

int32_t DIS12_StoreSatPosition(int8_t feIdx, uint8_t satPos)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    return AV_Dis12StoreSatPosition(feIdx, satPos);
}

int32_t DIS12_GotoSatPosition(int8_t feIdx, uint8_t satPos)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    return AV_Dis12GotoSatPosition(feIdx, satPos);
}

int32_t DIS12_ReCalcPos(int8_t feIdx)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    return AV_Dis12ReCalcPos(feIdx);
}

int32_t DIS12_SetLimitWest(int8_t feIdx)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    return AV_Dis12SetLimitWest(feIdx);
}

int32_t DIS12_SetLimitEast(int8_t feIdx)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    return AV_Dis12SetLimitEast(feIdx);
}

int32_t DIS12_ClearLimits(int8_t feIdx)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    return AV_Dis12ClearLimits(feIdx);
}

int32_t DIS12_GotoZeroPosition(int8_t feIdx)
{
    if(cfgsatDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    return AV_Dis12GotoZeroPosition(feIdx);
}

#endif /* APPL_DISEQC12_ENABLED */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
