/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/dev_hdd.c
**
**              $Id: dev_hdd.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Specific management of device HDD.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info    715: Symbol not referenced.
**
*****************************************************************************
*/
/*lint -save -e715 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <fapi/sys_driver.h>            /* global type definitions */

#include <rtos/rtos.h>
#include <fapi/drv_ata.h>

#include <fatfs/atadrv.h>

#include "sys_api.h"
#include "sys_priv.h"
#include "utils.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

// FIXME: is this used?
#define HDD_FILENAME_SETTINGS           "hddsetts"

#define HDD_SPINDOWNTIME_STANDBY        10      //!< in seconds
#define HDD_SPINDOWN_CHECK_INTERVAL     3000    //!< in milliseconds

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    uint32_t            id;                  //!< file identification
    uint32_t            revision;            //!< file revision
    uint32_t            spinDownTime;        //!< spindown time in seconds (0: disabled)
    uint32_t            idEnd;               //!< file identification

} HDD_FILEHDR_S;

typedef struct
{
    HDD_FILEHDR_S       cfg;

    FAPI_SYS_HandleT    devHdl;
    F_DRIVER*           devfsHdl;

    fbool_t             firstMount;
    fbool_t             active;            //!< module status: active/standby

    /* HDD spindown parameters */
    fbool_t             deviceAccessed;    //!< Current device access status,
                                           //!< controlled by device management
                                           //!< callback.
    uint32_t            idleStartTime;     //!< start-of-idle-mode timestamp
    uint32_t            spindownInterval;  //!< Current setting of spindown time.
    fbool_t             timerActive;
    uint64_t            lastSpindownStamp; //!< Timestamp of last spindown check.

    uint32_t            mountDelay;        //!< Mounting delay in milliseconds.
    uint64_t            mountTime;         //!< Time for next mounting, 0 if
                                           //!< none.

} HDD_DATA_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC HDD_DATA_S* hddDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC fbool_t  HDD_SettingsLoad            (void);
FSTATIC int32_t  HDD_SettingsSave            (void);
FSTATIC void     HDD_SettingsDefaultsSet     (void);
FSTATIC void     HDD_SettingsGetFilename     (void);

FSTATIC void     HDD_Wakeup                  (SYS_DEVICE_E device);
FSTATIC void     HDD_Shutdown                (SYS_DEVICE_E device);
FSTATIC int32_t  HDD_PreAccessHandler        (SYS_DEVICE_E device);
FSTATIC int32_t  HDD_Setup                   (void);
FSTATIC void     HDD_IntSetSpindownTime      (uint32_t timeInSeconds);
FSTATIC int32_t  HDD_GetPowerMode            (SYS_DEVICE_E device,
                                              SYS_DEV_PWRMODE_E* pMode);
FSTATIC void     HDD_AccessStatusChanged     (SYS_DEVICE_E device,
                                              fbool_t accessed);

int32_t HDD_Init (void)
{
    int32_t                 retVal = FAPI_OK;
    SYS_DEV_CFG_S           devCfg;
    uint32_t                masterFlag = 1;
    F_DRIVER*               devfsHdl;
    FAPI_ATA_DeviceInfoT    ataInfo;
    SYS_DEVICE_PARAMS_S     devParams;

    SYS_BootOsdPrint("Initializing HDD...");

    DBG_Assert(hddDat == NULL);

    hddDat = UTILS_Malloc(sizeof(HDD_DATA_S));
    if ( hddDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(hddDat, 0, sizeof(HDD_DATA_S));

    hddDat->firstMount        = FTRUE;
    hddDat->active            = FFALSE;
    hddDat->timerActive       = FFALSE;
    hddDat->deviceAccessed    = FFALSE;
    hddDat->lastSpindownStamp = 0;
    hddDat->mountDelay        = 0;
    hddDat->mountTime         = 0;

    for(;;)
    {
        /* ---- Register device -------------------------------------------- */

        memset(&devCfg, 0, sizeof(devCfg));

        devCfg.devName             = "Int. HDD";
        devCfg.wakeup              = HDD_Wakeup;
        devCfg.shutdown            = HDD_Shutdown;
        devCfg.preAccess           = HDD_PreAccessHandler;
        devCfg.getPowerMode        = HDD_GetPowerMode;
        devCfg.accessStatusChanged = HDD_AccessStatusChanged;

        retVal = SYS_DeviceRegister (SYS_DEV_HDD, &devCfg);
        if ( retVal != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(2,"[HDD] SYS_DeviceRegister failed %ld\n",retVal);
            break;
        }

        /* ----- Init HDD device for use with FATFS ------------------------ */

        retVal = f_createdriver(&devfsHdl, ATADRV_Init, masterFlag);
        if( retVal != F_NO_ERROR )
        {
            FAPI_SYS_PRINT_DEBUG(2,"[HDD] f_createdriver failed %ld\n",retVal);
            break;
        }

        retVal = FAPI_ATA_SpinUpWait(devfsHdl->user_ptr, HDD_TIMEOUT_READYWAIT);

        if (retVal != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(2,"[HDD] FAPI_ATA_PowerUp failed %ld\n",retVal);
            break;
        }

        hddDat->devfsHdl  = devfsHdl;
        hddDat->devHdl    = devfsHdl->user_ptr;

        /* ----- get device information and indicate availability ---------- */

        retVal = FAPI_ATA_GetDeviceInfo (hddDat->devHdl, &ataInfo);

        if ( retVal == FAPI_OK )
        {
            memset(&devParams, 0, sizeof(devParams));

            devParams.sectorCount = ataInfo.lbaCapacity;
            devParams.sectorSize  = FAPI_ATA_SECTOR_BYTES;

            (void)UTILS_strncpy (devParams.serialNumber, ataInfo.serialNumber,
                                 SYS_DEV_SERIALNUM_LEN);
            (void)UTILS_strncpy (devParams.firmwareRev,
                                 ataInfo.firmwareRevision,
                                 SYS_DEV_FIRMWARE_LEN);
            (void)UTILS_strncpy (devParams.modelNumber, ataInfo.modelNumber,
                                 SYS_DEV_MODEL_LEN);

            (void)SYS_DeviceSetAvailability (SYS_DEV_HDD, FTRUE,
                                             hddDat->devfsHdl,
                                             &devParams);
        }

        break;
    }

    if ( retVal == FAPI_OK )
    {
        FAPI_SYS_PRINT_DEBUG(1,"HDD Device found!\n");
    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(1,"HDD device not available!\n");
    }

    HDD_SettingsDefaultsSet();

    return FAPI_OK;
}

int32_t HDD_SetMountDelay (uint32_t delayInMs)
{
    int32_t retVal;

    if ( hddDat == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        hddDat->mountDelay = delayInMs;

        retVal = FAPI_OK;
    }

    return retVal;
}

void HDD_Tasks (void)
{
    int32_t         retVal;
    uint32_t        timeCurr = 0;
    uint64_t        currTime;
    SYS_DEV_STAT_S  devStat;
    fbool_t         standby;

    if ( hddDat == NULL ) return;

    currTime = SYS_GET_TIMESTAMP_MS;

    /* ----- handle spindown timer ----------------------------------------- */

    if ( UTILS_GetTimerStampInterval(hddDat->lastSpindownStamp, currTime)
         >= HDD_SPINDOWN_CHECK_INTERVAL )
    {
        hddDat->lastSpindownStamp = currTime;

        retVal = SYS_DeviceGetStatus (SYS_DEV_HDD, &devStat);

        if ( (retVal != FAPI_OK)
          || (hddDat->spindownInterval == 0)               /* Timer disabled */
          || (devStat.avlStatus != SYS_DEVAVL_AVAILABLE) ) /* HDD not avl.   */
        {
            hddDat->timerActive  = FFALSE;
        }
        else if ( !(hddDat->deviceAccessed) ) /* HDD is idle */
        {
            retVal = FAPI_ATA_GetPowerMode(hddDat->devHdl);
            if ( retVal == FAPI_ATA_ERR_DEVICE_IN_STANDBY ) standby = FTRUE;
            else                                            standby = FFALSE;

            if ( standby ) /* device is in standby mode */
            {
                /* deactivate timer */
                hddDat->timerActive  = FFALSE;
            }
            else
            {
                if ( hddDat->timerActive )
                {
                    (void)TIME_GetSystemTime(&timeCurr);
                    /* test for expiration */
                    if ( TIME_GetSystemTimeInterval(hddDat->idleStartTime,
                                                    timeCurr) >=
                       ((hddDat->spindownInterval * 1000)
                        / TIME_SYSTEMTIME_TICK) )
                    {
                        (void)HDD_Spindown();
                    }
                }
                else
                {
                    /* activate timer */
                    (void)TIME_GetSystemTime(&(hddDat->idleStartTime));
                    hddDat->timerActive = FTRUE;
                }
            }
        }
        else /* HDD is busy */
        {
            hddDat->timerActive = FFALSE;
        }
    }

    /* ----- handle delayed mounting --------------------------------------- */

    if ( (hddDat->mountTime != 0)
      && (currTime >= hddDat->mountTime) )
    {
        hddDat->mountTime = 0;

        (void)HDD_Setup();
    }
}

int32_t HDD_Spindown (void)
{
    int32_t         retVal;
    SYS_DEV_STAT_S  devStat;

    retVal = SYS_DeviceGetStatus (SYS_DEV_HDD, &devStat);

    if ( retVal == FAPI_OK )
    {
        if ( devStat.avlStatus == SYS_DEVAVL_AVAILABLE )
        {
            retVal = FAPI_ATA_PowerDown(hddDat->devHdl);
        }
        else
        {
            retVal = SMARTGO_SYS_ERR_PROHIBITED;
        }
    }

    return retVal;
}

int32_t HDD_Spinup (fbool_t waitForSpinup, uint32_t timeoutInMs)
{
    int32_t         retVal;
    SYS_DEV_STAT_S  devStat;

    retVal = SYS_DeviceGetStatus (SYS_DEV_HDD, &devStat);
    if ( retVal != FAPI_OK ) return retVal;

    if ( devStat.avlStatus != SYS_DEVAVL_NOTAVAILABLE )
    {
        if ( waitForSpinup )
        {
            (void)FAPI_ATA_PowerUp(hddDat->devHdl);
            retVal = FAPI_ATA_SpinUpWait(hddDat->devHdl, timeoutInMs);
        }
        else
        {
            retVal = FAPI_ATA_PowerUp(hddDat->devHdl);
        }
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Set spindown time of HDD in idle mode.
**
** If a value other than 0 is chosen, the harddisk will automatically spindown
** after an idle period of the requested length.
**
** \param  timeInSeconds  Spindown time in seconds (0 to disable timeout).
**
** \return One of the following status codes:
**         - #FAPI_OK                       if successful
**         - #SMARTGO_SYS_ERR_PROHIBITED if HDD is not connected.
******************************************************************************/
int32_t HDD_SetSpindownTime (uint32_t timeInSeconds)
{
    SYS_DEV_STAT_S  devStat;
    int32_t         retVal;

    if ( hddDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    retVal = SYS_DeviceGetStatus(SYS_DEV_HDD, &devStat);

    if ( retVal == FAPI_OK )
    {
        if ( devStat.avlStatus == SYS_DEVAVL_NOTAVAILABLE )
        {
            retVal = SMARTGO_SYS_ERR_PROHIBITED;
        }
        else
        {
            hddDat->cfg.spinDownTime = timeInSeconds;

            if ( hddDat->active ) HDD_IntSetSpindownTime(timeInSeconds);

            (void)HDD_SettingsSave();
        }
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Get current spindown time of HDD.
**
** \param  timeInSeconds  Pointer to variable where current spindown time
**                        will be copied into.
**
** \return One of the following status codes:
**         - #FAPI_OK                       if successful
**         - #SMARTGO_SYS_ERR_PROHIBITED if HDD is not connected.
******************************************************************************/
int32_t HDD_GetSpindownTime (uint32_t* timeInSeconds)
{
    SYS_DEV_STAT_S  devStat;
    int32_t         retVal;

    if ( hddDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    retVal = SYS_DeviceGetStatus(SYS_DEV_HDD, &devStat);

    if ( retVal == FAPI_OK )
    {
        if ( devStat.avlStatus == SYS_DEVAVL_NOTAVAILABLE )
        {
            retVal = SMARTGO_SYS_ERR_PROHIBITED;
        }
        else
        {
            *timeInSeconds = hddDat->cfg.spinDownTime;
        }
    }

    return retVal;
}

void HDD_MediaStatusHandling (SYS_EVT_DEVSTAT_S* pEvent)
{
    if ( hddDat == NULL ) return;

    hddDat->mountTime = 0;

    if ( pEvent->status.avlStatus == SYS_DEVAVL_AVAILABLE )
    {
        if ( pEvent->status.mountStatus == SYS_DEV_FS_NONE )
        {
            if ( hddDat->mountDelay > 0 )
            {
                /* delayed mounting activated: calculate time at which mounting
                   shall be started. NOTE: mountTime must not be zero, as this
                   means: no mounting time specified. */

                hddDat->mountTime = SYS_GET_TIMESTAMP_MS
                                  + hddDat->mountDelay;
            }
            else
            {
                (void)HDD_Setup();
            }
        }
        else if ( pEvent->status.mountStatus == SYS_DEV_FS_MOUNTED )
        {
            if ( hddDat->firstMount )
            {
                if ( HDD_SettingsLoad() == FFALSE )
                {
                    FAPI_SYS_PRINT_DEBUG(1,"[HDD] Set default settings\n");
                    HDD_SettingsDefaultsSet();
                }

                hddDat->firstMount = FFALSE;
            }
            else
            {
                (void)HDD_SettingsSave();
            }
        }
    }
}

FSTATIC int32_t HDD_Setup (void)
{
    int32_t         retVal;
    SYS_DEV_STAT_S  devStat;

    retVal = SYS_DeviceGetStatus (SYS_DEV_HDD, &devStat);
    if ( retVal != FAPI_OK ) return retVal;

    if ( devStat.avlStatus != SYS_DEVAVL_AVAILABLE )
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }
    else if ( (devStat.mountStatus == SYS_DEV_FS_MOUNTING)
           || (devStat.mountStatus == SYS_DEV_FS_MOUNTED)
           || (devStat.mountStatus == SYS_DEV_FS_FORMATTING) )
    {
        /* nothing to do... */
        return FAPI_OK;
    }

    /* spin up HDD */
    (void)HDD_Spinup (FTRUE, HDD_TIMEOUT_READYWAIT);
    //if ( retVal != FAPI_OK ) return retVal;

    /* Mount FS on first available partition */
    return SYS_DeviceMount(SYS_DEV_HDD, -1);
}

FSTATIC fbool_t HDD_SettingsLoad (void)
{
    fbool_t         retVal = FFALSE;
    F_FILE*         file;
    SYS_DEV_STAT_S  devStat;

    if ( SYS_DeviceGetStatus (SYS_DEV_HDD, &devStat) != FAPI_OK )
    {
        return FFALSE;
    }
    else if ( devStat.mountStatus != SYS_DEV_FS_MOUNTED )
    {
        return FFALSE;
    }

    FAPI_SYS_PRINT_DEBUG(1,"[HDD] Load settings\n");

    if ( HDD_Spinup (FTRUE, HDD_TIMEOUT_READYWAIT) != FAPI_OK ) return FFALSE;

    HDD_SettingsGetFilename();

    file = f_open (sysData->strBuf,"r");
    if ( file == NULL )
    {
        return FFALSE;
    }

    do
    {
        /* read file header */
        if ( f_read (&(hddDat->cfg), 1, sizeof(HDD_FILEHDR_S), file) !=
             (int32_t)sizeof(HDD_FILEHDR_S) )
        {
            break;
        }

        if ( (hddDat->cfg.id       != FILEID_HDDDAT) ||
             (hddDat->cfg.idEnd    != FILEID_HDDDAT) ||
             (hddDat->cfg.revision != FILEREV_HDDDAT) )
        {
            break;
        }

        if ( hddDat->active ) HDD_IntSetSpindownTime(hddDat->cfg.spinDownTime);

        retVal = FTRUE;
        break;
    }
    while ( 0 );

    (void)f_close(file);

    return retVal;
}

int32_t HDD_SettingsSave (void)
{
    int32_t         retVal = FAPI_OK;
    SYS_DEV_STAT_S  devStat;
    F_FILE*         file;

    if ( hddDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    retVal = SYS_DeviceGetStatus (SYS_DEV_HDD, &devStat);

    if ( retVal != FAPI_OK )
    {
        return retVal;
    }
    else if ( devStat.mountStatus != SYS_DEV_FS_MOUNTED )
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    retVal = HDD_Spinup (FTRUE, HDD_TIMEOUT_READYWAIT);
    if ( retVal != FAPI_OK ) return retVal;

    HDD_SettingsGetFilename();

    file = f_open(sysData->strBuf,"w");

    if ( file == NULL )
    {
        retVal = SMARTGO_SYS_ERR_INTERNAL;
    }
    else
    {
        hddDat->cfg.id       = FILEID_HDDDAT;
        hddDat->cfg.idEnd    = FILEID_HDDDAT;
        hddDat->cfg.revision = FILEREV_HDDDAT;

        /* write header */
        if ( f_write( &(hddDat->cfg), 1, sizeof(HDD_FILEHDR_S), file) !=
             (int32_t)sizeof(HDD_FILEHDR_S) )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
        }

        (void)f_close(file);
    }

    return retVal;
}

void HDD_RestoreDefaults (void)
{
    SYS_DEV_STAT_S  devStat;
    F_FIND          find;

    if ( hddDat == NULL ) return;

    if ( (SYS_DeviceGetStatus(SYS_DEV_HDD, &devStat) == FAPI_OK) &&
         (devStat.mountStatus == SYS_DEV_FS_MOUNTED) )
    {
        if ( HDD_Spinup (FTRUE, HDD_TIMEOUT_READYWAIT) != FAPI_OK ) return;

        HDD_SettingsGetFilename();

        /* Delete stored data */
        if ( f_findfirst(sysData->strBuf,&find) == FAPI_OK )
        {
            (void) f_delete(sysData->strBuf);
        }
    }

    HDD_SettingsDefaultsSet();
}

/* ------------------------------------------------------------------------- */

FSTATIC void HDD_SettingsDefaultsSet (void)
{
    hddDat->cfg.spinDownTime = 0;

    if ( hddDat->active ) HDD_IntSetSpindownTime(hddDat->cfg.spinDownTime);

    /* --------------------------------------------------------------------- */

    if ( SYS_GetConfig()->setHddDefaults != NULL )
    {
        SYS_GetConfig()->setHddDefaults();
    }
}

FSTATIC void HDD_SettingsGetFilename (void)
{
    uint32_t strPos;

    strPos = (uint32_t)SYS_DeviceGetMountPath (SYS_DEV_HDD, sysData->strBuf,
                                     SYS_STRBUF_SIZE);

    if ( (strPos + 2) < SYS_STRBUF_SIZE )
    {
        /* append separator */
        strPos += UTILS_strncpy (sysData->strBuf + strPos, "/",
                                 SYS_STRBUF_SIZE - strPos);

        /* append filename */
        (void)UTILS_strncpy (sysData->strBuf + strPos, HDD_FILENAME_SETTINGS,
                       SYS_STRBUF_SIZE - strPos);
    }
}

FSTATIC void HDD_Wakeup (SYS_DEVICE_E device)
{
    /* set user-defined spindown time */
    HDD_IntSetSpindownTime(hddDat->cfg.spinDownTime);

    hddDat->active = FTRUE;
}

FSTATIC void HDD_Shutdown (SYS_DEVICE_E device)
{
    /* set fixed spindown time for standby mode */
    HDD_IntSetSpindownTime(HDD_SPINDOWNTIME_STANDBY);
    hddDat->active = FFALSE;
}

FSTATIC int32_t HDD_PreAccessHandler (SYS_DEVICE_E device)
{
    return HDD_Spinup (FTRUE, HDD_TIMEOUT_READYWAIT);
}

FSTATIC int32_t HDD_GetPowerMode (SYS_DEVICE_E device, SYS_DEV_PWRMODE_E* pMode)
{
    int32_t retVal;

    if ( hddDat->devHdl != NULL )
    {
        retVal = FAPI_ATA_GetPowerMode(hddDat->devHdl);

        if ( retVal == FAPI_OK )
        {
            *pMode = SYS_DEV_PWRMODE_ACTIVE;
        }
        else if ( retVal == FAPI_ATA_ERR_DEVICE_IN_STANDBY )
        {
            *pMode = SYS_DEV_PWRMODE_STANDBY;
            retVal = FAPI_OK;
        }
    }
    else
    {
        retVal = SMARTGO_SYS_ERR_PROHIBITED;
    }

    return retVal;
}

FSTATIC void HDD_AccessStatusChanged (SYS_DEVICE_E device, fbool_t accessed)
{
    hddDat->deviceAccessed = accessed;
    hddDat->timerActive    = FFALSE; /* force relaunch of spindown timer */
}

FSTATIC void HDD_IntSetSpindownTime (uint32_t timeInSeconds)
{
    hddDat->spindownInterval = timeInSeconds;
    hddDat->timerActive      = FFALSE; /* force relaunch of spindown timer */
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
