/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/dev_usb.c
**
**              $Id: dev_usb.c 5896 2010-03-04 09:30:11Z jmorha $
**
** \brief       Specific management of device USB.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info    715: function argument not referenced
**
*****************************************************************************
*/
/*lint -save -e715 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <fapi/sys_driver.h>            /* global type definitions */

#include "sys_api.h"
#include "sys_priv.h"
#include "utils.h"

#include <fapi/drv_usb.h>
#include <fapi/drv_usb_volume.h>
#include <fapi/drv_usb_func_msc.h>
#include <fatfs/usbdrv.h>

#ifdef APPL_USB_FUNC_MSC_ENABLED

#include <fapi/drv_ata.h>

#endif /* APPL_USB_FUNC_MSC_ENABLED */

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#ifdef APPL_HDD_ENABLED
#define USB_FUNC_MSC_HDD_MOUNT_DELAY        2000        //!< in ms.
#endif

#define USB_WATCHDOG        0

#if(USB_WATCHDOG)
#define USB_BLOCK_ADDRESS   (uint8_t *)0xC7000800
#endif

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    MUSB_BusHandle        usbBusHandle;
    F_DRIVER*             usbFsHandle;
    fbool_t               running;

    USB_CONNECT_STATUS_S  currStatus;     //!< Current status.

    #ifdef APPL_USB_FUNC_MSC_ENABLED

    FAPI_SYS_HandleT      ataHandle;      //!< ATA handle used for accessing HDD
                                          //!< in function mode.
    fbool_t               hostConnected;  //!< USB host connection status.
    fbool_t               hostLocked;     //!< FTRUE if host access is blocked
                                          //!< by lock mechanism.
    #endif /* APPL_USB_FUNC_MSC_ENABLED */

} USB_DATA_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
FSTATIC USB_DATA_S* usbDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void     USB_MediaNotify (FAPI_USB_VolumeInfoT* infoPtr,
                                  uint32_t insertFlag);
FSTATIC void     USB_Wakeup      (SYS_DEVICE_E device);
FSTATIC void     USB_Shutdown    (SYS_DEVICE_E device);
FSTATIC int32_t  USB_Start       (USB_DATA_S* inst);
FSTATIC int32_t  USB_Stop        (USB_DATA_S* inst);

#ifdef APPL_USB_FUNC_MSC_ENABLED

FSTATIC int32_t  USB_SetupFunctionMode (USB_DATA_S* inst);
FSTATIC void     USB_FunctionHostConnectionNotify (uint32_t connectFlag);

#endif /* APPL_USB_FUNC_MSC_ENABLED */


extern uint16_t MGC_FillStorageClassPeripheralList(uint8_t* pList,
                                                   uint16_t wListLength);
extern MUSB_DeviceDriver* MUSB_GetStorageClassDriver(void);

extern MUSB_DeviceDriver* MUSB_GetHubClassDriver(void);
extern uint16_t MGC_FillHubClassPeripheralList( uint8_t* pList, uint16_t wListLength);

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

int32_t USB_Init (void)
{
    int32_t                     retVal = FAPI_OK;
    SYS_DEV_CFG_S               devCfg;
    FAPI_USB_OpenParamsT        usbOpenParams;
    FAPI_USB_VolumeOpenParamsT  usbVolParams;
    uint8_t                     periphList[FAPI_USB_PERIPHERAL_LIST_MAX];
    uint16_t                    periphListLength;

    #ifdef APPL_USB_FUNC_MSC_ENABLED
    SYS_DEV_STAT_S              hddStatus;
    #endif /* APPL_USB_FUNC_MSC_ENABLED */

    SYS_BootOsdPrint("Initializing USB...");

    DBG_Assert(usbDat == NULL);

    do
    {
        usbDat = UTILS_Malloc(sizeof(USB_DATA_S));
        if ( usbDat == NULL )
        {
            retVal = APPL_SMARTGO_ERR_OUT_OF_MEMORY;
            break;
        }

        memset(usbDat, 0, sizeof(USB_DATA_S));

        /* ---- open USB driver instance ----------------------------------- */

        memset (&usbOpenParams, 0, sizeof(usbOpenParams));

        usbOpenParams.version             = FAPI_USB_VERSION;
        usbOpenParams.ctrlId              = 0;
        usbOpenParams.desireHostRoleFlag  = 1;
        usbOpenParams.otgStateNotifyFct   = NULL;
        usbOpenParams.otgErrorNotifyFct   = NULL;

        usbDat->usbBusHandle = FAPI_USB_Open( &usbOpenParams , &retVal);
        if ( usbDat->usbBusHandle == NULL )
        {
            break;
        }

        /* ---- Setup USB host mode ---------------------------------------- */

        periphListLength = MGC_FillStorageClassPeripheralList(periphList,
                                                FAPI_USB_PERIPHERAL_LIST_MAX);

        retVal = FAPI_USB_RegHostDriver (usbDat->usbBusHandle,
                                         MUSB_GetStorageClassDriver(),
                                         periphList, periphListLength);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        periphListLength = MGC_FillHubClassPeripheralList(periphList,
                                                    FAPI_USB_PERIPHERAL_LIST_MAX);

        retVal = FAPI_USB_RegHostDriver(usbDat->usbBusHandle,
                                        MUSB_GetHubClassDriver(),
                                        periphList,periphListLength );

        if ( retVal != FAPI_OK )
        {
            break;
        }
        FAPI_USB_SetVolumeNotifyFct(USB_MediaNotify);

        /* ---- Register USB host device with the FATFS -------------------- */

        memset (&usbVolParams, 0, sizeof(usbVolParams));

        usbVolParams.version  = FAPI_USB_VOLUME_VERSION;
        usbVolParams.volumeId = 0;
        usbVolParams.useDMA   = 1;
        usbVolParams.useRTOS  = 1;

        retVal = f_createdriver(&(usbDat->usbFsHandle), USBDRV_Init,
                               (uint32_t)&usbVolParams);
        if ( retVal != F_NO_ERROR )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        /* ---- Register USB host device in device layer ------------------- */

        memset(&devCfg, 0, sizeof(devCfg));

        devCfg.devName    = "USB";
        devCfg.wakeup     = USB_Wakeup;
        devCfg.shutdown   = USB_Shutdown;

        retVal = SYS_DeviceRegister (SYS_DEV_USB, &devCfg);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        usbDat->currStatus.connected    = FFALSE;
        usbDat->currStatus.devType      = USB_DEVTYPE_NONE;
        usbDat->currStatus.accessStatus = USB_MSC_ACCESS_UNKNOWN;

        #ifdef APPL_USB_FUNC_MSC_ENABLED

        usbDat->ataHandle               = NULL;
        usbDat->hostConnected           = FFALSE;
        usbDat->hostLocked              = FFALSE;

        /* Setup USB function mode if HDD device is available */
        if ( SYS_DeviceGetStatus (SYS_DEV_HDD, &hddStatus) == FAPI_OK )
        {
            if ( hddStatus.avlStatus != SYS_DEVAVL_NOTAVAILABLE )
            {
                retVal = USB_SetupFunctionMode (usbDat);
                if ( retVal != FAPI_OK )
                {
                    break;
                }
            }
        }

        #endif /* APPL_USB_FUNC_MSC_ENABLED */
    }
    while ( 0 );

    return retVal;
}

FSTATIC void USB_Wakeup (SYS_DEVICE_E device)
{
    FAPI_SYS_PRINT_DEBUG(4,"[USB] Wakeup...\n");

    /* setup USB-OTG system */
    (void)USB_Start(usbDat);
    usbDat->running = FTRUE;
}

FSTATIC void USB_Shutdown (SYS_DEVICE_E device)
{
    usbDat->running = FFALSE;

    FAPI_SYS_PRINT_DEBUG(4,"[USB] Shutdown...\n");

    (void)USB_Stop(usbDat);
}

int32_t USB_GetConnectionStatus (USB_CONNECT_STATUS_S* pStatus)
{
    int32_t retVal;

    if ( usbDat == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        if ( pStatus != NULL )
        {
            *pStatus = usbDat->currStatus;
        }

        retVal = FAPI_OK;
    }

    return retVal;
}

void USB_ConnectStatusHandling (const SYS_EVT_USB_CONNECT_S* pEvent)
{
    SYS_DEVICE_PARAMS_S     devParams;
    FAPI_USB_VolumeInfoT*   pInfoUSB;

    #ifdef APPL_USB_FUNC_MSC_ENABLED
    SYS_EVT_USB_CONNECT_S ctrlEvent;
    #endif /* APPL_USB_FUNC_MSC_ENABLED */

    if ( usbDat == NULL ) return;

    switch ( pEvent->status.devType )
    {
        #ifdef APPL_USB_FUNC_MSC_ENABLED

        case USB_DEVTYPE_HOST:
            if ( pEvent->status.connected ) /* host has been connected */
            {
                /* NOTE: USB function device disconnection event might have
                         been missed. Ensure that it is properly shut down. */
                if ( (usbDat->currStatus.connected)
                  && (usbDat->currStatus.devType == USB_DEVTYPE_FUNCTION) )
                {
                    (void)SYS_DeviceSetAvailability (SYS_DEV_USB, FFALSE,
                                                     NULL, NULL);
                }

                if ( pEvent->status.accessStatus == USB_MSC_ACCESS_UNKNOWN )
                {
                    /* access rights have not been evaluated yet. */

                    ctrlEvent.hdr.type            = EVT_USB_CONNECT_STATUS;
                    ctrlEvent.hdr.length          = sizeof(ctrlEvent);
                    ctrlEvent.status.connected    = FTRUE;
                    ctrlEvent.status.devType      = USB_DEVTYPE_HOST;
                    ctrlEvent.status.accessStatus = USB_MSC_ACCESS_DENIED;

                    if ( SYS_DeviceSetExternalAccess(SYS_DEV_HDD, FTRUE)
                         == FAPI_OK )
                    {
                        if ( usbDat->hostLocked )
                        {
                            /* NOTE: This enables the host device's access
                                     to the internal HDD. */
                            FAPI_USB_FUNC_MSC_Unlock();

                            usbDat->hostLocked = FFALSE;
                        }
                        else
                        {
                            /* NOTE: This should never happen! */
                            FAPI_SYS_PRINT_DEBUG(1,"[USB] USB-MSC: access granted but was not locked!\n");
                        }

                        ctrlEvent.status.accessStatus = USB_MSC_ACCESS_GRANTED;
                    }

                    (void)EVT_Send(&ctrlEvent, "USB MSC");
                }
            }
            else /* host has been disconnected */
            {
                (void)SYS_DeviceSetExternalAccess(SYS_DEV_HDD, FFALSE);
            }
            break;

        #endif /* APPL_USB_FUNC_MSC_ENABLED */

        case USB_DEVTYPE_FUNCTION:

            if ( pEvent->status.connected )
            {
                pInfoUSB = FAPI_USB_GetVolumeInfo((FAPI_SYS_HandleT)
                                                  (usbDat->usbFsHandle->
                                                  user_ptr));

                if ( pInfoUSB != NULL )
                {
                    memset(&devParams, 0, sizeof(devParams));

                    devParams.sectorCount = pInfoUSB->sectorCount;
                    devParams.sectorSize  = pInfoUSB->sectorSize;

                    (void)UTILS_strncpy(devParams.serialNumber,
                                       (const char_t*)pInfoUSB->serialNumber,
                                        SYS_DEV_SERIALNUM_LEN);

                    (void)SYS_DeviceSetAvailability (SYS_DEV_USB, FTRUE,
                                                     usbDat->usbFsHandle,
                                                     &devParams);
                }
            }
            else
            {
                (void)SYS_DeviceSetAvailability (SYS_DEV_USB, FFALSE,
                                                 NULL, NULL);
            }
            break;

        default:
            /* do nothing */
            break;
    }

    /* store latest status */
    usbDat->currStatus = pEvent->status;
}

void USB_MediaStatusHandling (SYS_EVT_DEVSTAT_S* pEvent)
{
    if ( usbDat == NULL )
    {
        return;
    }

    if ( pEvent->status.avlStatus == SYS_DEVAVL_AVAILABLE )
    {
        if ( pEvent->status.mountStatus == SYS_DEV_FS_NONE )
        {
            /* Mount FS on first available partition */
            (void)SYS_DeviceMount(SYS_DEV_USB, -1);
        }
    }
}

FSTATIC void USB_MediaNotify (FAPI_USB_VolumeInfoT* infoPtr,
                              uint32_t insertFlag)
{
    SYS_EVT_USB_CONNECT_S ctrlEvent;

    FAPI_SYS_PRINT_DEBUG(4,"[USB] media notify: insert %i \n", insertFlag);

    ctrlEvent.hdr.type            = EVT_USB_CONNECT_STATUS;
    ctrlEvent.hdr.length          = sizeof(ctrlEvent);
    ctrlEvent.status.connected    = (fbool_t)insertFlag;
    ctrlEvent.status.devType      = USB_DEVTYPE_FUNCTION;
    ctrlEvent.status.accessStatus = USB_MSC_ACCESS_UNKNOWN;

    (void)EVT_Send(&ctrlEvent, "USB");
}

void USB_Reconnect (void)
{
    if ( usbDat == NULL )
    {
        return;
    }

    if ( usbDat->usbBusHandle != NULL )
    {
        (void)FAPI_USB_RequestBus(usbDat->usbBusHandle);
    }
}

void USB_Tasks (void)
{
    fbool_t autoConnect = FFALSE;

    if ( usbDat == NULL )
    {
        return;
    }
    if ( !(usbDat->running) )
    {
        return;
    }

    #ifdef APPL_USB_FUNC_MSC_ENABLED

    /* do no perform auto-connected if a USB host is currently connected. */
    if ( usbDat->hostConnected )
    {
        return;
    }

    #endif /* APPL_USB_FUNC_MSC_ENABLED */

    (void)SETTINGS_UsbAutoConnectGet(&autoConnect);

    if ( autoConnect )
    {
        USB_Reconnect();
    }

    /* For some USB devices the IRQ indicating their removal is not generated
       reliably. The following workaround detects this situation,
       re-initialises the USB OTG system and initiates an unmounting of the
       mass storage device. */

    /* NOTE: This workaround for SmartMPEG has been disabled, it does not work
             reliably for H60. */
    #if(USB_WATCHDOG)
    if ( usbDat->usbBusHandle != NULL )
    {
        uint8_t devctl = MGC_FAPI_ReadReg8 (USB_BLOCK_ADDRESS, 0x60UL);

        if ( !(devctl & 0x1) )
        {
            //FAPI_SYS_PRINT_DEBUG(1,"[USB] Watchdog hit!\n");

            /* Restart USB system */
            (void)USB_Stop  (usbDat);
            (void)USB_Start (usbDat);

            /* TODO: replace this by sending of a message if watchdog is ever
                     to be reactivated again! */
            (void)SYS_DeviceSetAvailability (SYS_DEV_USB, FFALSE, NULL, NULL);
        }
    }
    #endif /* USB_WATCHDOG */
}

FSTATIC int32_t USB_Start (USB_DATA_S* inst)
{
    int32_t retVal = FAPI_OK;

    /* NOTE: If the VBUS should be kept enabled (e.g. for host only systems),
             this has to be done in main.c in the APP_SET_BOARD_PARAMS macro. */

    if ( inst->usbBusHandle == NULL )
    {
        retVal = SMARTGO_SYS_ERR_INTERNAL;
    }
    else
    {
        retVal = FAPI_USB_Start( inst->usbBusHandle );

        if( (retVal != FAPI_OK)
         && (retVal != FAPI_USB_ERR_ALREADY_RUNNING) )
        {
            FAPI_SYS_PRINT_DEBUG(1,"[USB] FAPI_USB_Start() failed: %d\n", retVal);
        }
    }

    return retVal;
}

FSTATIC int32_t USB_Stop (USB_DATA_S* inst)
{
    if ( usbDat->usbBusHandle != NULL )
    {
        FAPI_USB_Stop(inst->usbBusHandle);
    }

    return FAPI_OK;
}

/* ----- functions for USB function mode ----------------------------------- */

#ifdef APPL_USB_FUNC_MSC_ENABLED

FSTATIC int32_t USB_SetupFunctionMode (USB_DATA_S* inst)
{
    FAPI_USB_FUNC_MSC_LunT          msc_luns;
    FAPI_USB_FUNC_MSC_InitParamsT   msc_params;
    FAPI_ATA_OpenParamsT            ata_open_params;
    FAPI_ATA_DeviceInfoT            ataDeviceInfo;
    int32_t                         retVal = FAPI_OK;

    if ( inst->ataHandle != NULL )
    {
        return FAPI_ERR_ALREADY_INITIALIZED;
    }

    do
    {
        memset(&ata_open_params, 0, sizeof(ata_open_params));

        ata_open_params.version     = FAPI_ATA_VERSION;
        ata_open_params.type        = FAPI_ATA_TYPE_PARALLEL;
        ata_open_params.channel     = FAPI_ATA_CHANNEL_MASTER;
        ata_open_params.sectorBytes = FAPI_ATA_SECTOR_BYTES;
        ata_open_params.use48BitLba = 1;
        ata_open_params.waitTimeout = HDD_TIMEOUT_READYWAIT;

        inst->ataHandle = FAPI_ATA_Open(&ata_open_params, &retVal);

        if ( inst->ataHandle == NULL )
        {
            break;
        }

        memset(&ataDeviceInfo, 0, sizeof(ataDeviceInfo));

        retVal = FAPI_ATA_GetDeviceInfo(inst->ataHandle, &ataDeviceInfo);

        if ( retVal != FAPI_OK )
        {
            break;
        }

        if ( ataDeviceInfo.lbaCapacity == 0 )
        {
            retVal = SMARTGO_SYS_ERR_INVALID_DATA;
            break;
        }

        memset (&msc_luns, 0, sizeof(msc_luns));

        msc_luns.type           = FAPI_USB_FUNC_MSC_LUN_HDD;
        msc_luns.namePtr        = "MB86H60 HDD 1";
        msc_luns.lbaCapacity    = ataDeviceInfo.lbaCapacity;
        msc_luns.blockLength    = FAPI_ATA_SECTOR_BYTES;
        msc_luns.startAddrOrLba = 0; // first LBA
        msc_luns.readOnly       = 0;
        msc_luns.ataHandle      = inst->ataHandle;

        memset (&msc_params, 0, sizeof(msc_params));

        msc_params.lunCount = 1;
        msc_params.lunList  = &msc_luns;

        retVal = FAPI_USB_FUNC_MSC_Init(&msc_params);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        retVal = FAPI_USB_RegFuncDriver (inst->usbBusHandle,
                                         FAPI_USB_FUNC_MSC_GetFunctionClient());

        if ( retVal != FAPI_OK )
        {
            break;
        }

        if ( FAPI_USB_FUNC_MSC_Lock() != 1 )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        usbDat->hostLocked = FTRUE;

        FAPI_USB_FUNC_MSC_SetHostNotifyFct(USB_FunctionHostConnectionNotify);

        /* NOTE: This will delay internal mounting of the HDD after external
                 access by USB host device has ended i.e. device has been
                 disconnected. It increases tolerance against sudden
                 disconnect/reconnect operations by the USB host device
                 (e.g. a PC). Under normal - not-delayed - conditions
                 internal mounting would immediately start after
                 disconnection, blocking all further attempts for external
                 access for the time of mounting (a few seconds). */

        retVal = HDD_SetMountDelay (USB_FUNC_MSC_HDD_MOUNT_DELAY);
        if ( retVal != FAPI_OK )
        {
            break;
        }

    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( inst->ataHandle != NULL )
        {
            (void)FAPI_ATA_Close(inst->ataHandle);

            inst->ataHandle = NULL;
        }
    }

    FAPI_SYS_PRINT_DEBUG(4,"[USB] USB_SetupFunctionMode: res: %i\n", retVal);

    return retVal;
}

FSTATIC void USB_FunctionHostConnectionNotify (uint32_t connectFlag)
{
    fbool_t                 connectStatus = (connectFlag != 0) ? FTRUE : FFALSE;
    SYS_EVT_USB_CONNECT_S   ctrlEvent;

    FAPI_SYS_PRINT_DEBUG(4,"[USB] Host connection notify: connectFlag %i \n", connectFlag);

    if ( connectStatus != usbDat->hostConnected )
    {
        usbDat->hostConnected = connectStatus;

        if ( !connectStatus )
        {
            /* on disconnection restore locked status for next connection */
            if ( !(usbDat->hostLocked) )
            {
                if ( FAPI_USB_FUNC_MSC_Lock() != 0 )
                {
                    usbDat->hostLocked = FTRUE;
                }
                else
                {
                    FAPI_SYS_PRINT_DEBUG(1,"[USB] FAPI_USB_FUNC_MSC_Lock failed!\n");
                }
            }
        }

        ctrlEvent.hdr.type            = EVT_USB_CONNECT_STATUS;
        ctrlEvent.hdr.length          = sizeof(ctrlEvent);
        ctrlEvent.status.connected    = connectStatus;
        ctrlEvent.status.devType      = USB_DEVTYPE_HOST;
        ctrlEvent.status.accessStatus = USB_MSC_ACCESS_UNKNOWN;

        (void)EVT_Send(&ctrlEvent, "USB MSC");
    }
}

#endif /* APPL_USB_FUNC_MSC_ENABLED */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
