/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/device.c
**
**              $Id: device.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Generic device management.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    737: Loss of sign in promotion from int to unsigned int.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715  -e737 */
//#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <fapi/sys_driver.h>         /* global type definitions */

#ifdef APPL_FATFS_ENABLED
#include <fatfs/fatfs.h>
#endif /* APPL_FATFS_ENABLED */

#include "sys_api.h"
#include "sys_priv.h"
#include "utils.h"

#ifdef APPL_PVR_ENABLED
#include "pvr_sys.h"
#endif /* APPL_PVR_ENABLED */

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define SYS_DEV_MAX_NUM_DRIVES          10      //!< Max. number of assigned
                                                //!< drive letters.
#define SYS_DEV_MAX_DRIVES_PER_DEVICE   4       //!< Max. number of drive
                                                //!< letters (= partitions)
                                                //!< per device.
#define SYS_DEV_CMDEND_TIMEOUT          15000   //!< in ms.
#define SYS_DEV_CMDEND_SLEEPTIME        100     //!< in ms.

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef enum
{
    SYS_DEVCMD_NONE = 0,
    SYS_DEVCMD_MOUNT,
    SYS_DEVCMD_FORMAT

} SYS_DEV_CMD_TYPE_E;

typedef struct
{
    SYS_DEV_CMD_TYPE_E            type;
    int8_t                        arg;

} SYS_DEV_CMD_S;

typedef struct
{
    SYS_DEVICE_E                  type;         //!< Device type.
    SYS_DEV_CFG_S                 cfg;          //!< Device parameters.

    /* static device information */
    SYS_DEVICE_PARAMS_S           params;       //!< Device information.
    SYS_SIZE_S                    capacity;     //!< Device capacity.
    void*                         devFsHdl;     //!< Device FS handle.

    SYS_DEV_AVL_E                 avlStatus;    //!< Availability status.
    SYS_DEV_FS_E                  mountStatus;  //!< Mount status.
    uint16_t                      numAccessOps; //!< Number of parallel access
                                                //!< operations.
    int32_t                       mountDriveNum;//!< Mounted drive index.
    fbool_t                       unmountReq;

    /* command parameters */
    SYS_DEV_CMD_S                 cmdReq;       //!< Requested command.
    fbool_t                       cmdActive;    //!< 1 if thread is currently
                                                //!< performing a command.
} SYS_DEV_DEVICE_S;

typedef struct
{
    SYS_DEVICE_E                  device;     //!< Device that drive is
                                              //!< assigned to, SYS_DEV_NULL
                                              //!< if none.
    int8_t                        partIdx;    //!< Partition number of device,
                                              //!< -1 if none.
    fbool_t                       available;  //!< Availability status.
    fbool_t                       mounted;    //!< Mount status.

} SYS_DEV_DRV_S;

typedef struct
{
    FAPEX_SYS_SemaphoreT          mutexId;      //!< Mutex semaphore.
    SYS_DEV_DEVICE_S*             dev[SYS_DEV_NUMS];//!< Array of pointers to
                                                //!< known devices.
    SYS_DEV_DRV_S                 drive[SYS_DEV_MAX_NUM_DRIVES];

} SYS_DEV_DATA_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC SYS_DEV_DATA_S* devDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

#ifdef APPL_FATFS_ENABLED

FSTATIC int32_t         SYS_DeviceGetDate       (uint16_t* yearPtr,
                                                 uint16_t* monthPtr,
                                                 uint16_t* dayPtr);
FSTATIC int32_t         SYS_DeviceGetTime       (uint16_t* hoursPtr,
                                                 uint16_t* minutesPtr,
                                                 uint16_t* secondsPtr);
FSTATIC uint32_t        SYS_DeviceGetTaskId     (void);

#if (DEBUG > 0)
FSTATIC char_t*         SYS_DeviceGetErrStr     (int32_t status);
#endif /* #if (DEBUG > 0) */

#endif /* APPL_FATFS_ENABLED */

FSTATIC SYS_DEV_DEVICE_S* SYS_CheckDevice       (SYS_DEVICE_E device,
                                                 int32_t* pErrCode);
FSTATIC int32_t         SYS_DeviceUnmount       (SYS_DEV_DATA_S* inst,
                                                 SYS_DEV_DEVICE_S* pDev);
FSTATIC void            SYS_DeviceGenerateStatus(const SYS_DEV_DEVICE_S* pDev,
                                                 SYS_DEV_STAT_S* pStatus);
FSTATIC void            SYS_DeviceThreadFunction(void* opt);
FSTATIC SYS_DEV_FS_E    SYS_DeviceCmdMount      (SYS_DEV_DATA_S* inst,
                                                 SYS_DEV_DEVICE_S* pDev,
                                                 int8_t partIdx);
FSTATIC SYS_DEV_FS_E    SYS_DeviceCmdFormat     (SYS_DEV_DATA_S* inst,
                                                 SYS_DEV_DEVICE_S* pDev);
FSTATIC int32_t         SYS_DeviceGetDriveNum   (SYS_DEV_DATA_S* inst,
                                                 SYS_DEVICE_E device,
                                                 int8_t partIdx,
                                                 fbool_t onlyIfAvailable);
FSTATIC int32_t         SYS_DeviceHandleDir     (SYS_DEVICE_E device,
                                                 const char_t* dirName,
                                                 fbool_t create);
FSTATIC int32_t         SYS_DeviceSetPartitionAvl(SYS_DEV_DATA_S* inst,
                                                 SYS_DEV_DEVICE_S* pDev);
FSTATIC int32_t         SYS_DeviceClearPartitionAvl (SYS_DEV_DATA_S* inst,
                                                 SYS_DEV_DEVICE_S* pDev);

int32_t SYS_DeviceInit (void)
{
    SYS_DEVICE_E        cnt;
    int32_t             retVal;
    uint32_t            driveCnt;

    #ifdef APPL_FATFS_ENABLED
    FATFS_InitParamsT   fatfsInitParams;
    #endif /* APPL_FATFS_ENABLED */

    DBG_Assert(devDat == NULL);

    devDat = UTILS_Malloc(sizeof(SYS_DEV_DATA_S));
    if ( devDat == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    memset(devDat, 0, sizeof(SYS_DEV_DATA_S));

    devDat->mutexId = FAPEX_SYS_CreateSemaphore(1);

    if ( devDat->mutexId == NULL )
    {
        return SMARTGO_SYS_ERR_RESOURCES;
    }

    for ( cnt = (SYS_DEVICE_E)0; cnt < SYS_DEV_NUMS; cnt++ )
    {
        devDat->dev[cnt] = NULL;
    }

    for ( driveCnt = 0; driveCnt < SYS_DEV_MAX_NUM_DRIVES; driveCnt++ )
    {
        devDat->drive[driveCnt].device    = SYS_DEV_NULL;
        devDat->drive[driveCnt].partIdx   = -1;
        devDat->drive[driveCnt].available = FFALSE;
        devDat->drive[driveCnt].mounted   = FFALSE;
    }

    #ifdef APPL_FATFS_ENABLED

    memset (&fatfsInitParams, 0 ,sizeof(FATFS_InitParamsT));

    fatfsInitParams.getDateFct     = SYS_DeviceGetDate;
    fatfsInitParams.getTimeFct     = SYS_DeviceGetTime;
    fatfsInitParams.getTaskIdFct   = SYS_DeviceGetTaskId;

    retVal = FATFS_Init (&fatfsInitParams);

    if ( retVal != FAPI_OK )
    {
        FAPI_SYS_PRINT_DEBUG(2,"[HDD] FATFS_Init failed %ld\n",retVal);
        return retVal;
    }

    #endif /* APPL_FATFS_ENABLED */
     FAPI_SYS_PRINT_MSG("SYS_DeviceInit: create SYS_DeviceThreadFunction\n ");
    retVal = SYS_ThreadCreate(SYS_DeviceThreadFunction, devDat,
                              THREAD_PRIO_DEVICEOP,
                              THREAD_STACKSZ_DEVICEOP,
                              THREAD_NAME_DEVICEOP,
                              NULL);

    FAPI_SYS_PRINT_MSG("SYS_DeviceInit quit (retVal = %d)(%s)\n",retVal, SYS_GetErrorString(retVal));
    return retVal;
}

int32_t SYS_DeviceWakeup(void)
{
    SYS_DEVICE_E cnt;

    FAPI_SYS_PRINT_DEBUG(1,"[device] SYS_DeviceWakeup\n");

    for ( cnt = (SYS_DEVICE_E)0; cnt < SYS_DEV_NUMS; cnt++ )
    {
        if ( devDat->dev[cnt] != NULL )
        {
            if ( devDat->dev[cnt]->cfg.wakeup != NULL )
            {
                devDat->dev[cnt]->cfg.wakeup(cnt);
            }
        }
    }
    FAPI_SYS_PRINT_DEBUG(1,"[device] SYS_DeviceWakeup done\n");
    return FAPI_OK;
}

int32_t SYS_DeviceShutdown (void)
{
    SYS_DEVICE_E cnt;

    for ( cnt = (SYS_DEVICE_E)0; cnt < SYS_DEV_NUMS; cnt++ )
    {
        if ( devDat->dev[cnt] != NULL )
        {
            if ( devDat->dev[cnt]->cfg.shutdown != NULL )
            {
                devDat->dev[cnt]->cfg.shutdown(cnt);
            }
        }
    }

    return FAPI_OK;
}

fbool_t SYS_DeviceStatusHandling (const SYS_EVT_DEVSTAT_S* pEvtData)
{
    SYS_DEV_DEVICE_S*   pDev   = SYS_CheckDevice (pEvtData->device, NULL);
    fbool_t             retVal = FFALSE;

    if ( pDev != NULL )
    {
        if ( pDev->unmountReq )
        {
            if ( pEvtData->status.mountStatus == SYS_DEV_FS_NONE )
            {
                pDev->unmountReq = FFALSE;
                retVal = FTRUE;
            }
            else
            {
                /* NOTE: This event is blocked. It was sent during an operation
                         in the device thread context with an unmount
                         requested already pending. */

                retVal = FFALSE;
            }
        }
        else
        {
            retVal = FTRUE;
        }

        if ( retVal )
        {
            pDev->avlStatus   = pEvtData->status.avlStatus;
            pDev->mountStatus = pEvtData->status.mountStatus;
        }
    }

    return retVal;
}

int32_t SYS_DeviceRegister (SYS_DEVICE_E device, SYS_DEV_CFG_S* cfg)
{
    SYS_DEV_DEVICE_S*   pDev;
    int32_t             retVal;
    uint32_t            driveCnt;
    uint8_t             partCnt;

    if ( (device <  (SYS_DEVICE_E)0)
      || (device >= SYS_DEV_NUMS) )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if ( devDat->dev[device] != NULL )
    {
        /* already registered! */
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        devDat->dev[device] = UTILS_Malloc(sizeof(SYS_DEV_DEVICE_S));
        if ( devDat->dev[device] == NULL )
        {
            retVal = APPL_SMARTGO_ERR_OUT_OF_MEMORY;
        }
        else
        {
            pDev = devDat->dev[device];

            memset(pDev, 0, sizeof(SYS_DEV_DEVICE_S));

            pDev->type          = device;
            pDev->cfg           = *cfg;
            pDev->avlStatus     = SYS_DEVAVL_NOTAVAILABLE;
            pDev->mountStatus   = SYS_DEV_FS_NONE;
            pDev->mountDriveNum = -1;
            pDev->unmountReq    = FFALSE;
            pDev->cmdReq.type   = SYS_DEVCMD_NONE;
            pDev->cmdActive     = FFALSE;

            /* Reserve drive letters for maximum number of partitions */
            partCnt = 0;

            for ( driveCnt = 0; driveCnt < SYS_DEV_MAX_NUM_DRIVES; driveCnt++ )
            {
                if ( partCnt >= SYS_DEV_MAX_DRIVES_PER_DEVICE )
                {
                    break;
                }

                if ( devDat->drive[driveCnt].device == SYS_DEV_NULL )
                {
                    devDat->drive[driveCnt].device    = device;
                    devDat->drive[driveCnt].partIdx   = (int8_t)partCnt++;
                    devDat->drive[driveCnt].available = FFALSE;
                    devDat->drive[driveCnt].mounted   = FFALSE;
                }
            }

            retVal = FAPI_OK;
        }
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Prepare thread for usage of FAT FS.
**
** This function must be called by any thread that potentially performs
** accesses on FAT file system.
**
** \return -
**
******************************************************************************/
void SYS_DevicePrepareThread(void)
{
    DBG_Assert(devDat != NULL);

    #ifdef APPL_FATFS_ENABLED

    (void)f_enterFS();

    #endif /* APPL_FATFS_ENABLED */
}

int32_t SYS_DeviceGetInfo (SYS_DEVICE_E device, SYS_DEVICE_INFO_S* pInf)
{
    SYS_DEV_DEVICE_S*   pDev;
    int32_t             retVal = FAPI_OK;

    #ifdef APPL_FATFS_ENABLED
    F_SPACE             space;
    #endif /* APPL_FATFS_ENABLED */

    pDev = SYS_CheckDevice (device, &retVal);

    if ( pDev != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(devDat->mutexId, FAPEX_SYS_SUSPEND);

        memset(pInf, 0, sizeof(SYS_DEVICE_INFO_S));

        SYS_DeviceGenerateStatus(pDev, &(pInf->status));

        pInf->capacity = pDev->capacity;

        (void)UTILS_strncpy(pInf->serialNumber, pDev->params.serialNumber,
                            SYS_DEV_SERIALNUM_LEN);
        (void)UTILS_strncpy(pInf->firmwareRev, pDev->params.firmwareRev,
                            SYS_DEV_FIRMWARE_LEN);
        (void)UTILS_strncpy(pInf->modelNumber, pDev->params.modelNumber,
                            SYS_DEV_MODEL_LEN);

        UTILS_remWhiteCharAtEnd(pInf->serialNumber, SYS_DEV_SERIALNUM_LEN);
        UTILS_remWhiteCharAtEnd(pInf->firmwareRev,  SYS_DEV_FIRMWARE_LEN);
        UTILS_remWhiteCharAtEnd(pInf->modelNumber,  SYS_DEV_MODEL_LEN);

        pInf->serialNumberCs = SYS_CHARSET_ASCII;
        pInf->firmwareRevCs  = SYS_CHARSET_ASCII;
        pInf->modelNumberCs  = SYS_CHARSET_ASCII;

        #ifdef APPL_FATFS_ENABLED

        if ( pDev->mountStatus == SYS_DEV_FS_MOUNTED )
        {
            /* get current amount of free space on device/medium */
            if ( f_getfreespace (pDev->mountDriveNum, &space) == F_NO_ERROR )
            {
                pInf->freeSpace.kbytes = (((space.free_high * 4) * 1024) * 1024)
                                         + (space.free >> 10);
            }
        }

        #endif /* APPL_FATFS_ENABLED */

        (void)FAPEX_SYS_SetSemaphore(devDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    }

    return retVal;
}

int32_t SYS_DeviceGetPowerMode (SYS_DEVICE_E device, SYS_DEV_PWRMODE_E* pMode)
{
    int32_t             retVal = FAPI_OK;
    SYS_DEV_DEVICE_S*   pDev;

    if ( pMode == NULL )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        pDev = SYS_CheckDevice (device, &retVal);

        if ( pDev != NULL )
        {
            if ( pDev->cfg.getPowerMode == NULL )
            {
                retVal = APPL_SMARTGO_ERR_FEATURE_NOT_SUPPORTED;
            }
            else
            {
                (void)FAPEX_SYS_GetSemaphore(devDat->mutexId, FAPEX_SYS_NO_SUSPEND);

                if ( pDev->avlStatus == SYS_DEVAVL_NOTAVAILABLE )
                {
                    retVal = SMARTGO_SYS_ERR_PROHIBITED;
                }
                else
                {
                    retVal = pDev->cfg.getPowerMode(device, pMode);
                }

                (void)FAPEX_SYS_SetSemaphore(devDat->mutexId, FAPEX_SYS_NO_SUSPEND);
            }
        }
    }

    return retVal;
}

/*
Determine device given path has been mounted on.
*/
int32_t SYS_DeviceGetByPath (const char_t* path, SYS_DEVICE_E* device,
                             uint16_t* pathOffset)
{
    int32_t      retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    int32_t      driveNum;
    SYS_DEVICE_E foundDevice;
    uint16_t     drivePathLen;

    (void)FAPEX_SYS_GetSemaphore(devDat->mutexId, FAPEX_SYS_SUSPEND);

    if ( (strlen(path) >= 2) && (path[1] == ':') )
    {
        driveNum = path[0] - 'a';

        if ( (driveNum >= 0) && (driveNum < SYS_DEV_MAX_NUM_DRIVES) )
        {
            foundDevice = devDat->drive[driveNum].device;

            if ( (foundDevice              != SYS_DEV_NULL)
              && (foundDevice               < SYS_DEV_NUMS)
              && (devDat->dev[foundDevice] != NULL) )
            {
                drivePathLen = 2;

                switch ( path[drivePathLen] )
                {
                    default:
                    case 0:
                        break;

                    case '/': /* skip path delimiter */
                        drivePathLen++;
                        break;
                }

                if ( device != NULL )
                {
                    *device = foundDevice;
                }

                if ( pathOffset != NULL )
                {
                    *pathOffset = drivePathLen;
                }

                retVal = FAPI_OK;
            }
        }
    }

    (void)FAPEX_SYS_SetSemaphore(devDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t SYS_DevicePreAccess(SYS_DEVICE_E device)
{
    int32_t           retVal = FAPI_OK;
    SYS_DEV_DEVICE_S* pDev;

    pDev = SYS_CheckDevice (device, &retVal);

    if ( pDev != NULL )
    {
        if ( pDev->cfg.preAccess != NULL )
        {
            retVal = pDev->cfg.preAccess(pDev->type);
        }
    }

    return retVal;
}

int32_t SYS_DeviceMkdirByPath (const char_t* dirPath)
{
    int32_t           retVal;
    SYS_DEVICE_E      device = SYS_DEV_NULL;
    SYS_DEV_DEVICE_S* pDev;

    retVal = SYS_DeviceGetByPath(dirPath, &device, NULL);
    if ( retVal != FAPI_OK ) return retVal;

    pDev = devDat->dev[device];

    if ( pDev->cfg.preAccess != NULL )
    {
        retVal = pDev->cfg.preAccess(pDev->type);
        if ( retVal != FAPI_OK )
        {
            return retVal;
        }
    }

    #ifdef APPL_FATFS_ENABLED

    if ( f_mkdir((char_t*)dirPath) != F_NO_ERROR )
    {
        retVal = SMARTGO_SYS_ERR_INTERNAL;
    }

    #endif /* APPL_FATFS_ENABLED */

    return retVal;
}

int32_t SYS_DeviceMkdir (SYS_DEVICE_E device, const char_t* dirName)
{
    return SYS_DeviceHandleDir (device, dirName, FTRUE);
}

int32_t SYS_DeviceRmdir (SYS_DEVICE_E device, const char_t* dirName)
{
    return SYS_DeviceHandleDir (device, dirName, FFALSE);
}

FSTATIC int32_t SYS_DeviceHandleDir (SYS_DEVICE_E device, const char_t* dirName,
                                     fbool_t create)
{
    int32_t             retVal = FAPI_OK;
    SYS_DEV_DEVICE_S*   pDev;
    uint32_t            strPos;

    #ifdef APPL_FATFS_ENABLED
    int32_t             result;
    #endif /* APPL_FATFS_ENABLED */

    pDev = SYS_CheckDevice (device, &retVal);

    if ( pDev == NULL )
    {
        return retVal;
    }
    else if ( (dirName == NULL) || (dirName[0] == 0) )
    {
        return FAPI_OK;
    }

    strPos = (uint32_t)SYS_DeviceGetMountPath (device, sysData->strBuf,
                                               SYS_STRBUF_SIZE);

    if ( (strPos + 2) < SYS_STRBUF_SIZE )
    {
        /* append separator */
        strPos += SYS_DeviceAddSeparator (sysData->strBuf + strPos,
                                          SYS_STRBUF_SIZE - strPos);

        /* append requested directory name */
        (void)UTILS_strncpy (sysData->strBuf + strPos, dirName,
                             SYS_STRBUF_SIZE - strPos);
    }

    if ( pDev->cfg.preAccess != NULL )
    {
        retVal = pDev->cfg.preAccess(pDev->type);
        if ( retVal != FAPI_OK )
        {
            return retVal;
        }
    }

    #ifdef APPL_FATFS_ENABLED

    if ( create )
    {
        result = f_mkdir(sysData->strBuf);
    }
    else
    {
        result = f_rmdir(sysData->strBuf);
    }

    if ( result != F_NO_ERROR )
    {
        retVal = SMARTGO_SYS_ERR_INTERNAL;
    }

    #endif /* APPL_FATFS_ENABLED */

    return retVal;
}


int32_t SYS_DeviceSetAvailability(SYS_DEVICE_E device, fbool_t available,
                                  void* devFsHdl,
                                  const SYS_DEVICE_PARAMS_S* pDevParams)
{
    SYS_EVT_DEVSTAT_S   ctrlEvent;
    int32_t             retVal;
    SYS_DEV_DEVICE_S*   pDev;
    uint32_t            kBytes;
    uint32_t            bytes;
    uint32_t            tmpBytes;

    FAPI_SYS_PRINT_DEBUG(4,"[DEV] Set availability: device %i, available %i \n", device,
               available);

    if ( (device < (SYS_DEVICE_E)0)
      || (device >= SYS_DEV_NUMS) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if ( available && (pDevParams == NULL) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    (void)FAPEX_SYS_GetSemaphore(devDat->mutexId, FAPEX_SYS_SUSPEND);

    pDev = devDat->dev[device];

    if ( pDev == NULL )
    {
        retVal = SMARTGO_SYS_ERR_PROHIBITED;
    }
    else if ( available && ((pDev->avlStatus == SYS_DEVAVL_AVAILABLE)
                         || (pDev->avlStatus == SYS_DEVAVL_EXT_ACCESS)) )

    {
        /* already available */
        retVal = FAPI_OK;
    }
    else if ( (!available) && (pDev->avlStatus == SYS_DEVAVL_NOTAVAILABLE) )
    {
        /* already unavailable */
        retVal = FAPI_OK;
    }
    else
    {
        if ( available )
        {
            pDev->params   = *pDevParams;
            pDev->devFsHdl = devFsHdl;

            kBytes   = (pDev->params.sectorCount / 1024)
                     *  pDev->params.sectorSize;
            tmpBytes = (pDev->params.sectorCount % 1024)
                     *  pDev->params.sectorSize;
            kBytes  += tmpBytes / 1024;
            bytes    = tmpBytes % 1024;

            SYS_SizeAdd(&(pDev->capacity), kBytes, bytes);

            /* print general device information */

            FAPI_SYS_PRINT_DEBUG(4,"-----------------------------------------\n");
            FAPI_SYS_PRINT_DEBUG(4,"Device detected:     %s\n", pDev->cfg.devName);
            FAPI_SYS_PRINT_DEBUG(4,"- Number of sectors: %i\n", pDev->params.sectorCount);
            FAPI_SYS_PRINT_DEBUG(4,"- Sector size:       %i bytes\n",
                                                    pDev->params.sectorSize);
            FAPI_SYS_PRINT_DEBUG(4,"- Model Number:      %s\n", pDev->params.modelNumber);
            FAPI_SYS_PRINT_DEBUG(4,"- Serial Number:     %s\n", pDev->params.serialNumber);
            FAPI_SYS_PRINT_DEBUG(4,"- Firmware Revision: %s\n", pDev->params.firmwareRev);

            if ( pDev->capacity.kbytes >= (1024 * 1024) )
            {
                FAPI_SYS_PRINT_DEBUG(4,"- Capacity         : %i GBytes\n",
                          (pDev->capacity.kbytes / 1024) / 1024);
            }
            else
            {
                FAPI_SYS_PRINT_DEBUG(4," Capacity          : %i MBytes\n",
                           pDev->capacity.kbytes / 1024);
            }

            FAPI_SYS_PRINT_DEBUG(4,"-----------------------------------------\n");

            (void)SYS_DeviceSetPartitionAvl(devDat, pDev);

            pDev->avlStatus = SYS_DEVAVL_AVAILABLE;
        }
        else
        {
            (void)SYS_DeviceUnmount(devDat, pDev);

            (void)SYS_DeviceClearPartitionAvl(devDat, pDev);

            memset(&(pDev->params), 0, sizeof(SYS_DEVICE_PARAMS_S));

            SYS_SizeClear(&(pDev->capacity));

            pDev->devFsHdl = NULL;

            pDev->avlStatus = SYS_DEVAVL_NOTAVAILABLE;
        }

        pDev->mountStatus   = SYS_DEV_FS_NONE;
        pDev->mountDriveNum = -1;
        pDev->numAccessOps  = 0;

        if ( pDev->cfg.accessStatusChanged != NULL )
        {
            pDev->cfg.accessStatusChanged (pDev->type, FFALSE);
        }

        ctrlEvent.hdr.type   = EVT_DEVICE_STATUS;
        ctrlEvent.hdr.length = sizeof(ctrlEvent);
        ctrlEvent.device     = device;

        SYS_DeviceGenerateStatus(pDev, &(ctrlEvent.status));

        (void)EVT_Send(&ctrlEvent, "DevStat");

        retVal = FAPI_OK;
    }

    (void)FAPEX_SYS_SetSemaphore(devDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t SYS_DeviceGetStatus(SYS_DEVICE_E device, SYS_DEV_STAT_S* status)
{
    int32_t             retVal = FAPI_OK;
    SYS_DEV_DEVICE_S*   pDev;

    pDev = SYS_CheckDevice (device, &retVal);

    if ( pDev != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(devDat->mutexId, FAPEX_SYS_SUSPEND);

        SYS_DeviceGenerateStatus(pDev, status);

        (void)FAPEX_SYS_SetSemaphore(devDat->mutexId, FAPEX_SYS_NO_SUSPEND);
    }

    return retVal;
}

int32_t SYS_DeviceCheckPartition (SYS_DEVICE_E device, int8_t partIdx)
{
    int32_t driveNum;
    int32_t retVal;

    FAPI_SYS_PRINT_DEBUG(4,"[DEV] SYS_DeviceCheckPartition\n");

    driveNum = SYS_DeviceGetDriveNum (devDat, device, partIdx, FTRUE);

    if ( driveNum < 0 )
    {
        retVal = SMARTGO_SYS_ERR_PROHIBITED;
    }
    else
    {
        #ifdef APPL_FATFS_ENABLED

        retVal = f_checkvolume(driveNum);

        if ( retVal != F_NO_ERROR )
        {
            retVal = SMARTGO_SYS_ERR_PROHIBITED;
        }
        else
        {
            retVal = FAPI_OK;
        }

        #else

        retVal = SMARTGO_SYS_ERR_PROHIBITED;

        #endif /* APPL_FATFS_ENABLED */

    }

    return retVal;
}

int32_t SYS_DeviceFormat(SYS_DEVICE_E device)
{
    int32_t           retVal = FAPI_OK;
    SYS_DEV_DEVICE_S* pDev;

    FAPI_SYS_PRINT_DEBUG(4,"[DEV] SYS_DeviceFormat: dev %i \n", device);

    pDev = SYS_CheckDevice (device, &retVal);

    if ( pDev != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(devDat->mutexId, FAPEX_SYS_SUSPEND);

        if ( pDev->avlStatus != SYS_DEVAVL_AVAILABLE )
        {
            retVal = SMARTGO_SYS_ERR_PROHIBITED;
        }
        else
        {
            switch ( pDev->mountStatus )
            {
                case SYS_DEV_FS_NONE:
                case SYS_DEV_FS_ERROR:
                case SYS_DEV_FS_UNFORMATTED:
                case SYS_DEV_FS_MOUNTED:
                    /* pass command to device operation thread */
                    pDev->cmdReq.type = SYS_DEVCMD_FORMAT;
                    pDev->cmdActive   = FTRUE;

                    pDev->mountStatus = SYS_DEV_FS_FORMATTING;
                    break;

                case SYS_DEV_FS_FORMATTING:
                case SYS_DEV_FS_MOUNTING:
                default:
                    retVal = APPL_SMARTGO_ERR_DEVICE_BUSY;
                    break;
            }
        }

        (void)FAPEX_SYS_SetSemaphore(devDat->mutexId, FAPEX_SYS_NO_SUSPEND);
    }

    return retVal;
}


/*!
*******************************************************************************
**
** \brief  Mount requested partition on a device.
**
** Please note that the actual mount operation is performed in a background
** thread. It might fail even if this function returns FAPI_OK. Completion
** status of the mount operation is indicated by events.
**
** \param  device   Device where partition shall be mounted.
** \param  partIdx  Index of partition to be mounted:
**                  - 0...(SYS_DEV_MAX_DRIVES_PER_DEVICE-1): partition index
**                  - -1: Mount first available partition
**
** \return One of the following return codes:
**   - #FAPI_OK                       Successful.
**   - APPL_SMARTGO_ERR_BAD_PARAMETER Bad argument passed.
**   - #SMARTGO_SYS_ERR_PROHIBITED    Target device (currently) not available.
******************************************************************************/
int32_t SYS_DeviceMount(SYS_DEVICE_E device, int8_t partIdx)
{
    int32_t           retVal = FAPI_OK;
    SYS_DEV_DEVICE_S* pDev;

    FAPI_SYS_PRINT_DEBUG(4,"[DEV] SYS_DeviceMount: dev %i, partIdx %i \n", device, partIdx);

    if ( partIdx >= SYS_DEV_MAX_DRIVES_PER_DEVICE )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    pDev = SYS_CheckDevice (device, &retVal);

    if ( pDev != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(devDat->mutexId, FAPEX_SYS_SUSPEND);

        if ( pDev->avlStatus != SYS_DEVAVL_AVAILABLE )
        {
            retVal = SMARTGO_SYS_ERR_PROHIBITED;
        }
        else
        {
            switch ( pDev->mountStatus )
            {
                case SYS_DEV_FS_MOUNTED:
                    /* already mounted */
                    retVal = FAPI_OK;
                    break;

                case SYS_DEV_FS_NONE:
                case SYS_DEV_FS_ERROR:
                case SYS_DEV_FS_UNFORMATTED:
                    /* pass command to device operation thread */
                    pDev->cmdReq.type = SYS_DEVCMD_MOUNT;
                    pDev->cmdReq.arg  = partIdx;
                    pDev->cmdActive   = FTRUE;

                    pDev->mountStatus = SYS_DEV_FS_MOUNTING;
                    break;

                case SYS_DEV_FS_FORMATTING:
                case SYS_DEV_FS_MOUNTING:
                default:
                    retVal = APPL_SMARTGO_ERR_DEVICE_BUSY;
                    break;
            }
        }

        (void)FAPEX_SYS_SetSemaphore(devDat->mutexId, FAPEX_SYS_NO_SUSPEND);
    }

    return retVal;
}

/* NOTE: This function assumes to be performed in a "locked"
         (=> semaphore-protected) area. The semaphore is given and regained
         in a special case. */
FSTATIC int32_t SYS_DeviceUnmount (SYS_DEV_DATA_S* inst, SYS_DEV_DEVICE_S* pDev)
{
    int32_t  retVal = FAPI_OK;
    uint64_t startTime;

    FAPI_SYS_PRINT_DEBUG(4,"[DEV] SYS_DeviceUnmount: dev %i\n", pDev->type);

    if ( pDev->avlStatus != SYS_DEVAVL_AVAILABLE )
    {
        retVal = SMARTGO_SYS_ERR_PROHIBITED;
    }
    else
    {
        if ( (pDev->mountStatus == SYS_DEV_FS_FORMATTING)
          || (pDev->mountStatus == SYS_DEV_FS_MOUNTING) )
        {
            /* device thread is still busy with an operation, wait until
               operation has ended. */

            (void)FAPEX_SYS_SetSemaphore(devDat->mutexId, FAPEX_SYS_NO_SUSPEND);

            /* NOTE: Semaphore is temporarily given here to allow polling
                     on device thread status. */

            startTime = SYS_GET_TIMESTAMP_MS;

            while ( pDev->cmdActive )
            {
                /* check timeout */
                if ( UTILS_GetTimerStampInterval(startTime,
                     SYS_GET_TIMESTAMP_MS)
                     >= SYS_DEV_CMDEND_TIMEOUT )
                {
                    retVal = APPL_SMARTGO_ERR_TIMEOUT;
                    break;
                }

                FAPI_SYS_SLEEP(SYS_DEV_CMDEND_SLEEPTIME);
            }

            pDev->unmountReq = FTRUE;

            (void)FAPEX_SYS_GetSemaphore(devDat->mutexId, FAPEX_SYS_SUSPEND);
        }

        if ( (retVal == FAPI_OK) && (pDev->mountDriveNum >= 0) )
        {
            /* ----- Perform necessary pre-unmount operations -------------- */

            #ifdef APPL_PVR_ENABLED

            (void)FAPEX_SYS_SetSemaphore(devDat->mutexId, FAPEX_SYS_NO_SUSPEND);

            PVRSYS_AbortDeviceOperations(pDev->type);

            (void)FAPEX_SYS_GetSemaphore(devDat->mutexId, FAPEX_SYS_SUSPEND);

            #endif /* APPL_PVR_ENABLED */

            /* Perform unmount, unregister all partitions from this device */

            #ifdef APPL_FATFS_ENABLED

            retVal = f_delvolume (pDev->mountDriveNum);

            // FIXME: do this also on failure?
            if ( retVal == F_NO_ERROR )
            {
                pDev->mountStatus = SYS_DEV_FS_NONE;
            }
            else
            {
                pDev->mountStatus = SYS_DEV_FS_ERROR;

                retVal = SMARTGO_SYS_ERR_INTERNAL;
            }

            #else

            retVal = SMARTGO_SYS_ERR_PROHIBITED;

            #endif /* APPL_FATFS_ENABLED */

            inst->drive[pDev->mountDriveNum].mounted = FFALSE;

            pDev->mountDriveNum = -1;
        }
    }

    return retVal;
}

FSTATIC int32_t SYS_DeviceSetPartitionAvl (SYS_DEV_DATA_S* inst,
                                           SYS_DEV_DEVICE_S* pDev)
{
    int32_t     retVal = FAPI_OK;

    #ifdef APPL_FATFS_ENABLED

    F_PARTITION dev_partition[SYS_DEV_MAX_NUM_DRIVES];
    int32_t     partCnt;
    int32_t     driveCnt;

    memset(dev_partition, 0, sizeof(dev_partition));

    /* check the number of partion available for the device */
    retVal = f_getpartition ((F_DRIVER*)(pDev->devFsHdl),
                             SYS_DEV_MAX_NUM_DRIVES, dev_partition);

    if ( retVal != F_NO_ERROR )
    {
        FAPI_SYS_PRINT_DEBUG(4,"[DEV] f_getpartition failed: %s (%i)\n",
                   SYS_DeviceGetErrStr(retVal), retVal);

        return retVal;
    }

    /* NOTE: Partitions beyond maximum number of partitions per device are
             ignored. dev_partition array is set to maximum size in order to
             avoid errors when used on devices having more than
             SYS_DEV_MAX_DRIVES_PER_DEVICE partitions. */

    FAPI_SYS_PRINT_DEBUG(4,"-----------------------------------------\n");
    FAPI_SYS_PRINT_DEBUG(4,"FATFS partitions detected: \n");

    for ( partCnt = 0; partCnt < SYS_DEV_MAX_DRIVES_PER_DEVICE; partCnt++ )
    {
        /* Search corresponding partition in drive list. */
        for ( driveCnt = 0; driveCnt < SYS_DEV_MAX_NUM_DRIVES; driveCnt++ )
        {
            if ( (inst->drive[driveCnt].device  == pDev->type)
              && (inst->drive[driveCnt].partIdx == partCnt) )
            {
                break;
            }
        }

        if ( driveCnt >= SYS_DEV_MAX_NUM_DRIVES )
        {
            /* partition not found */
            continue;
        }

        if ( dev_partition[partCnt].secnum > 0 )
        {
            FAPI_SYS_PRINT_DEBUG(4,"#%2i - drive '%c:', sectors: %i\n",
                       partCnt,
                       'a' + driveCnt,
                       dev_partition[partCnt].secnum);

            inst->drive[driveCnt].available = FTRUE;
        }
        else
        {
            inst->drive[driveCnt].available = FFALSE;
        }

        inst->drive[driveCnt].mounted = FFALSE;
    }

    FAPI_SYS_PRINT_DEBUG(4,"-----------------------------------------\n");

    #endif /* APPL_FATFS_ENABLED */

    return retVal;
}

FSTATIC int32_t SYS_DeviceClearPartitionAvl (SYS_DEV_DATA_S* inst,
                                             SYS_DEV_DEVICE_S* pDev)
{
    int32_t driveCnt;

    for ( driveCnt = 0; driveCnt < SYS_DEV_MAX_NUM_DRIVES; driveCnt++ )
    {
        if ( inst->drive[driveCnt].device == pDev->type )
        {
            inst->drive[driveCnt].available = FFALSE;
            inst->drive[driveCnt].mounted   = FFALSE;
        }
    }

    return FAPI_OK;
}

int32_t SYS_DeviceSetExternalAccess (SYS_DEVICE_E device, fbool_t access)
{
    SYS_DEV_DEVICE_S*   pDev;
    int32_t             retVal  = FAPI_OK;
    fbool_t             sendEvt = FFALSE;
    SYS_EVT_DEVSTAT_S   ctrlEvent;

    FAPI_SYS_PRINT_DEBUG(4,"[DEV] SYS_DeviceSetExternalAccess: dev %i access %i\n",
               device, access);

    pDev = SYS_CheckDevice (device, &retVal);

    if ( pDev != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(devDat->mutexId, FAPEX_SYS_SUSPEND);

        if ( access )
        {
            switch ( pDev->avlStatus )
            {
                default:
                case SYS_DEVAVL_NOTAVAILABLE:
                    retVal = SMARTGO_SYS_ERR_PROHIBITED;
                    break;

                case SYS_DEVAVL_EXT_ACCESS:
                    /* external access already set */
                    retVal = FAPI_OK;
                    break;

                case SYS_DEVAVL_AVAILABLE:

                    switch ( pDev->mountStatus )
                    {
                        case SYS_DEV_FS_NONE:
                        case SYS_DEV_FS_ERROR:
                        case SYS_DEV_FS_UNFORMATTED:

                            pDev->avlStatus   = SYS_DEVAVL_EXT_ACCESS;
                            pDev->mountStatus = SYS_DEV_FS_NONE;

                            sendEvt = FTRUE;
                            retVal  = FAPI_OK;
                            break;

                        case SYS_DEV_FS_MOUNTED:

                            if ( pDev->numAccessOps > 0 )
                            {
                                /* FS is currently accessed. */
                                retVal = APPL_SMARTGO_ERR_DEVICE_BUSY;
                            }
                            else
                            {
                                (void)SYS_DeviceUnmount(devDat, pDev);

                                // TODO: add error handling on unmount failure?

                                pDev->avlStatus   = SYS_DEVAVL_EXT_ACCESS;
                                pDev->mountStatus = SYS_DEV_FS_NONE;

                                sendEvt = FTRUE;
                                retVal  = FAPI_OK;
                            }
                            break;

                        default:
                        case SYS_DEV_FS_FORMATTING:
                        case SYS_DEV_FS_MOUNTING:

                            retVal = APPL_SMARTGO_ERR_DEVICE_BUSY;
                            break;
                    }
            }
        }
        else if ( pDev->avlStatus == SYS_DEVAVL_EXT_ACCESS )
        {
            pDev->avlStatus = SYS_DEVAVL_AVAILABLE;

            sendEvt = FTRUE;
            retVal  = FAPI_OK;
        }

        (void)FAPEX_SYS_SetSemaphore(devDat->mutexId, FAPEX_SYS_NO_SUSPEND);
    }

    if ( ( sendEvt ) &&
         ( pDev != NULL ) )
    {
        ctrlEvent.hdr.type   = EVT_DEVICE_STATUS;
        ctrlEvent.hdr.length = sizeof(ctrlEvent);
        ctrlEvent.device     = pDev->type;

        SYS_DeviceGenerateStatus(pDev, &(ctrlEvent.status));

        (void)EVT_Send(&ctrlEvent, "DevStat");
    }

    return retVal;
}

int32_t SYS_DeviceSetAccessStatus(SYS_DEVICE_E device, fbool_t access)
{
    SYS_DEV_DEVICE_S* pDev;
    int32_t           retVal = FAPI_OK;
    fbool_t           cbCall = FFALSE;
    fbool_t           cbVal  = FFALSE;

    pDev = SYS_CheckDevice (device, &retVal);

    if ( pDev != NULL )
    {
        (void)FAPEX_SYS_GetSemaphore(devDat->mutexId, FAPEX_SYS_SUSPEND);

        /* this feature is only available in 'available' state. */
        if ( pDev->avlStatus != SYS_DEVAVL_AVAILABLE )
        {
            retVal = SMARTGO_SYS_ERR_PROHIBITED;
        }
        else
        {
            if ( access )
            {
                if ( pDev->numAccessOps == 0 )
                {
                    cbCall = FTRUE;
                    cbVal  = FTRUE;
                }

                pDev->numAccessOps++;
            }
            else if ( pDev->numAccessOps > 0 )
            {
                if ( pDev->numAccessOps == 1 )
                {
                    cbCall = FTRUE;
                    cbVal  = FFALSE;
                }

                pDev->numAccessOps--;
            }
        }

        if ( cbCall && (pDev->cfg.accessStatusChanged != NULL) )
        {
            pDev->cfg.accessStatusChanged (pDev->type, cbVal);
        }

        (void)FAPEX_SYS_SetSemaphore(devDat->mutexId, FAPEX_SYS_NO_SUSPEND);
    }

    return retVal;
}

int32_t SYS_DeviceGetMountPath (SYS_DEVICE_E device, char_t* buf,
                                uint32_t bufSize)
{
    char_t   path[3] = "a:";
    int32_t  retVal  = 0;
    uint32_t driveCnt;

    if ( SYS_CheckDevice (device, NULL) != NULL )
    {
        for ( driveCnt = 0; driveCnt < SYS_DEV_MAX_NUM_DRIVES; driveCnt++ )
        {
            if ( devDat->drive[driveCnt].device == device )
            {
                break;
            }
        }

        if ( driveCnt < SYS_DEV_MAX_NUM_DRIVES )
        {
            path[0] += (char_t)driveCnt;

            retVal = (int32_t)UTILS_strncpy(buf, path, bufSize);
        }
    }

    return retVal;
}

int32_t SYS_DeviceGetLabel (SYS_DEVICE_E device, char_t* pStrBuf,
                            uint32_t bufSize)
{
    int32_t             retVal = FAPI_OK;
    SYS_DEV_DEVICE_S*   pDev;

    pDev = SYS_CheckDevice (device, &retVal);

    if ( pDev != NULL )
    {
        if ( pDev->mountStatus == SYS_DEV_FS_MOUNTED )
        {
            if ( pStrBuf != NULL )
            {
                if ( pDev->cfg.preAccess != NULL )
                {
                    retVal = pDev->cfg.preAccess(pDev->type);
                }

                #ifdef APPL_FATFS_ENABLED

                if ( retVal == FAPI_OK )
                {
                    if ( f_getlabel (pDev->mountDriveNum, pStrBuf, bufSize)
                         != F_NO_ERROR )
                    {
                        retVal = SMARTGO_SYS_ERR_INTERNAL;
                    }
                }

                #endif /* APPL_FATFS_ENABLED */
            }
        }
        else
        {
            retVal = SMARTGO_SYS_ERR_PROHIBITED;
        }
    }

    return retVal;
}

uint32_t SYS_DeviceAddSeparator(char_t* buf, uint32_t bufSize)
{
    return UTILS_strncpy(buf, "/", bufSize);
}

FSTATIC SYS_DEV_DEVICE_S* SYS_CheckDevice (SYS_DEVICE_E device,
                                           int32_t* pErrCode)
{
    SYS_DEV_DEVICE_S* pDev = NULL;
    int32_t           retVal;

    if ( (device <  (SYS_DEVICE_E)0)
      || (device >= SYS_DEV_NUMS) )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if ( devDat->dev[device] == NULL )
    {
        /* device is not registered */
        retVal = SMARTGO_SYS_ERR_PROHIBITED;
    }
    else
    {
        pDev   = devDat->dev[(int32_t)device];
        retVal = FAPI_OK;
    }

    if ( pErrCode != NULL )
    {
        *pErrCode = retVal;
    }

    return pDev;
}

FSTATIC void SYS_DeviceGenerateStatus (const SYS_DEV_DEVICE_S* pDev,
                                       SYS_DEV_STAT_S* pStatus)
{
    pStatus->avlStatus   = pDev->avlStatus;
    pStatus->mountStatus = pDev->mountStatus;
    pStatus->access      = (pDev->numAccessOps > 0) ? FTRUE : FFALSE;
}

FSTATIC void SYS_DeviceThreadFunction(void* opt)
{
    SYS_DEV_DATA_S*     inst = (SYS_DEV_DATA_S*)opt;
    SYS_EVT_DEVSTAT_S   ctrlEvent;
    uint32_t            devCnt;
    SYS_DEV_DEVICE_S*   pDev;
    SYS_DEV_CMD_S       cmdCurr;
    SYS_DEV_FS_E        newMountStatus;
    fbool_t             sendEvent;

    memset(&cmdCurr,0,sizeof(SYS_DEV_CMD_S));

    SYS_DevicePrepareThread();

    ctrlEvent.hdr.type   = EVT_DEVICE_STATUS;
    ctrlEvent.hdr.length = sizeof(ctrlEvent);

    for (;;)
    {
        for ( devCnt = 0; devCnt < (uint32_t)SYS_DEV_NUMS; devCnt++ )
        {
            pDev = inst->dev[devCnt];

            if ( pDev == NULL )
            {
                continue;
            }

            (void)FAPEX_SYS_GetSemaphore(inst->mutexId, FAPEX_SYS_SUSPEND);

            if ( pDev->cmdActive )
            {
                cmdCurr = pDev->cmdReq;
            }
            else
            {
                cmdCurr.type = SYS_DEVCMD_NONE;
                cmdCurr.arg  = 0;
            }

            (void)FAPEX_SYS_SetSemaphore(inst->mutexId, FAPEX_SYS_NO_SUSPEND);

            if ( pDev->cmdActive )
            {
                (void)SYS_DeviceSetAccessStatus(pDev->type, FTRUE);

                newMountStatus = pDev->mountStatus;

                switch ( cmdCurr.type )
                {
                    case SYS_DEVCMD_MOUNT:
                        newMountStatus = SYS_DeviceCmdMount(inst, pDev,
                                                            cmdCurr.arg);

                        FAPI_SYS_PRINT_DEBUG(4,"[DEV] Mount dev %i, new mount status: %i\n",
                                   pDev->type, newMountStatus);
                        break;

                    case SYS_DEVCMD_FORMAT:
                        newMountStatus = SYS_DeviceCmdFormat (inst, pDev);

                        FAPI_SYS_PRINT_DEBUG(4,"[DEV] Format dev %i, new mount status: %i\n",
                                   pDev->type, newMountStatus);
                        break;

                    default:
                    case SYS_DEVCMD_NONE:
                        break;
                }

                (void)SYS_DeviceSetAccessStatus(pDev->type, FFALSE);

                if ( newMountStatus != pDev->mountStatus )
                {
                    ctrlEvent.device             = pDev->type;
                    ctrlEvent.status.avlStatus   = pDev->avlStatus;
                    ctrlEvent.status.mountStatus = newMountStatus;
                    ctrlEvent.status.access      = (pDev->numAccessOps > 0)
                                                    ? FTRUE : FFALSE;
                    sendEvent = FTRUE;
                }
                else
                {
                    sendEvent = FFALSE;
                }

                pDev->cmdActive = FFALSE;

                if ( sendEvent )
                {
                    (void)EVT_Send(&ctrlEvent, "DevStat");
                }
            }
        }

        FAPI_SYS_SLEEP(200);
    }
}

/*!
*******************************************************************************
**
** \brief  Returns the available space in kilo bytes.
**
** \param device    The Device Id.
**
** \return The available spaice in kbytes or 0 if device is not available.
**
******************************************************************************/
uint32_t SYS_DeviceGetFreekBytes(SYS_DEVICE_E device)
{
    SYS_DEVICE_INFO_S devInfo;

    memset(&devInfo,0, sizeof(SYS_DEVICE_INFO_S));

    if ( SYS_DeviceGetInfo(device, &devInfo) == FAPI_OK )
    {
        if ( (devInfo.status.avlStatus   == SYS_DEVAVL_AVAILABLE)
           &&(devInfo.status.mountStatus == SYS_DEV_FS_MOUNTED) )
        {
            return devInfo.freeSpace.kbytes;
        }
    }
    return 0;
}

/*!
*******************************************************************************
**
** \brief  Internal mount procedure (performed in device thread).
**
** \param  inst     Pointer to internal data.
** \param  pDev     Pointer to target device data.
** \param  partIdx  Index of partition to be mounted:
**                  - 0...(SYS_DEV_MAX_DRIVES_PER_DEVICE-1): partition index
**                  - -1: Mount first available partition
**
** \return New mount status.
******************************************************************************/
FSTATIC SYS_DEV_FS_E SYS_DeviceCmdMount (SYS_DEV_DATA_S* inst,
                                         SYS_DEV_DEVICE_S* pDev, int8_t partIdx)
{
    SYS_DEV_FS_E  newMountStatus = SYS_DEV_FS_ERROR;

    #ifdef APPL_FATFS_ENABLED

    int32_t       driveNum = -1;
    F_SPACE       space;
    int32_t       retVal       = F_NO_ERROR;
    fbool_t       notFormatted = FFALSE;

    #endif /* APPL_FATFS_ENABLED */

    FAPI_SYS_PRINT_DEBUG(4,"[DEV] SYS_DeviceCmdMount\n");

    #ifdef APPL_FATFS_ENABLED

    do
    {
        if ( partIdx >= 0 ) /* specific partition requested */
        {
            driveNum = SYS_DeviceGetDriveNum (inst, pDev->type, partIdx, FTRUE);

            if ( driveNum < 0 )
            {
                retVal = SMARTGO_SYS_ERR_INTERNAL;
                break;
            }

            FAPI_SYS_PRINT_DEBUG(4,"[DEV] Mount device %i partition %i on drive '%c:'...\n",
                       pDev->type, partIdx, 'a' + driveNum);

            retVal = f_initvolumepartition (driveNum,
                                           (F_DRIVER*)(pDev->devFsHdl),
                                            partIdx);

            if ( retVal == F_ERR_NOTFORMATTED )
            {
                notFormatted = FTRUE;
            }
        }
        else /* first available partition requested */
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;

            for ( partIdx = 0; partIdx < SYS_DEV_MAX_DRIVES_PER_DEVICE;
                  partIdx++ )
            {
                driveNum = SYS_DeviceGetDriveNum (inst, pDev->type, partIdx,
                                                  FTRUE);

                if ( driveNum < 0 )
                {
                    continue;
                }

                FAPI_SYS_PRINT_DEBUG(4,"[DEV] Mount device %i partition %i on drive '%c:'...\n",
                           pDev->type, partIdx, 'a' + driveNum);

                retVal = f_initvolumepartition (driveNum,
                                               (F_DRIVER*)(pDev->devFsHdl),
                                                partIdx);

                if ( retVal == FAPI_OK )
                {
                    /* partition successfully mounted */
                    break;
                }
                else if ( retVal == F_ERR_NOTFORMATTED )
                {
                    notFormatted = FTRUE;
                }
            }
        }

        if ( retVal != FAPI_OK )
        {
            /* NOTE: Error code "not formatted" is specially treated here. The
                     user is notified on that problem by an event and can
                     cure it by formatting the volume. */

            if ( notFormatted )
            {
                newMountStatus = SYS_DEV_FS_UNFORMATTED;
            }

            break;
        }

        FAPI_SYS_PRINT_DEBUG(4,"[DEV] Mount successful!\n");

        /* perform dummy f_getfreespace: first call might take some seconds,
           but further calls are quite fast. */
        retVal = f_getfreespace(driveNum, &space);

        if ( retVal != F_NO_ERROR )
        {
            FAPI_SYS_PRINT_DEBUG(4,"[DEV] f_getfreespace failed: %s (%i)\n",
                       SYS_DeviceGetErrStr(retVal), retVal);
            break;
        }

        (void)RTOS_GetSemaphore(inst->mutexId, RTOS_SUSPEND);

        pDev->mountDriveNum = driveNum;

        inst->drive[driveNum].mounted = FTRUE;

        (void)RTOS_SetSemaphore(inst->mutexId, RTOS_NO_SUSPEND);

        newMountStatus = SYS_DEV_FS_MOUNTED;
    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        if ( driveNum >= 0 )
        {
            (void)f_delvolume (driveNum);

            pDev->mountDriveNum = -1;

            inst->drive[driveNum].mounted = FFALSE;
        }

        FAPI_SYS_PRINT_DEBUG(4,"[DEV] Device mounting failed: %s (%i)\n",
                   SYS_DeviceGetErrStr(retVal), retVal);
    }

    #endif /* APPL_FATFS_ENABLED */

    return newMountStatus;
}

FSTATIC SYS_DEV_FS_E SYS_DeviceCmdFormat (SYS_DEV_DATA_S* inst,
                                          SYS_DEV_DEVICE_S* pDev)
{
    SYS_DEV_FS_E  newMountStatus = SYS_DEV_FS_ERROR;

    #ifdef APPL_FATFS_ENABLED

    int32_t       retVal = FAPI_OK;
    F_PARTITION   dev_partition[1];
    F_SPACE       space;
    int32_t       newDriveNum;
    #if (DEBUG > 0)
    uint32_t      totalGB, freeGB, usedGB, badGB;
    #endif /* #if (DEBUG > 0) */

    #endif /* APPL_FATFS_ENABLED */

    FAPI_SYS_PRINT_DEBUG(4,"[DEV] SYS_DeviceCmdFormat\n");

    #ifdef APPL_FATFS_ENABLED

    do
    {
        if ( pDev->cfg.preAccess != NULL )
        {
            retVal = pDev->cfg.preAccess(pDev->type);
            if ( retVal != FAPI_OK )
            {
                break;
            }
        }

        /* create only one partition and remove the remaining partitions */

        dev_partition[0].system_indicator = F_SYSIND_DOSFAT32;

        dev_partition[0].secnum           = pDev->params.sectorCount;

        if ( dev_partition[0].secnum > 63 )
        {
            dev_partition[0].secnum -= 63; // F_SPACE_AFTER_MBR;
        }
        else
        {
            dev_partition[0].secnum = 0;
        }

        retVal = f_createpartition ((F_DRIVER*)(pDev->devFsHdl), 1,
                                    dev_partition);

        if ( retVal != F_NO_ERROR )
        {
            break;
        }

        (void)RTOS_GetSemaphore(inst->mutexId, RTOS_SUSPEND);

        pDev->mountDriveNum = -1;

        /* update partition availability */

        (void)SYS_DeviceClearPartitionAvl(inst, pDev);
        (void)SYS_DeviceSetPartitionAvl  (inst, pDev);

        /* select first partition for mounting */
        newDriveNum = SYS_DeviceGetDriveNum(inst, pDev->type, 0, FTRUE);

        (void)RTOS_SetSemaphore(inst->mutexId, RTOS_NO_SUSPEND);

        if ( newDriveNum < 0 )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        retVal = f_format (newDriveNum, F_FAT32_MEDIA);

        if ( retVal != F_NO_ERROR )
        {
            break;
        }

        retVal = f_getfreespace(newDriveNum, &space );

        if ( retVal != F_NO_ERROR )
        {
            break;
        }

        (void)RTOS_GetSemaphore(inst->mutexId, RTOS_SUSPEND);

        inst->drive[newDriveNum].mounted = FTRUE;

        pDev->mountDriveNum = newDriveNum;

        (void)RTOS_SetSemaphore(inst->mutexId, RTOS_NO_SUSPEND);

        newMountStatus = SYS_DEV_FS_MOUNTED;

        #if (DEBUG > 0)

        totalGB = (space.total_high * 4) + (space.total>>30);
        freeGB  = (space.free_high  * 4) + (space.free>>30);
        usedGB  = (space.used_high  * 4) + (space.used>>30);
        badGB   = (space.bad_high   * 4) + (space.bad>>30);

        FAPI_SYS_PRINT_DEBUG(1, "   total: %u.%u GB\n", totalGB, (space.total&0x3fffffff)>>20 );
        FAPI_SYS_PRINT_DEBUG(1, "   free : %u.%u GB\n", freeGB,  (space.free&0x3fffffff)>>20);
        FAPI_SYS_PRINT_DEBUG(1, "   used : %u.%u GB\n", usedGB,  (space.used&0x3fffffff)>>20);
        FAPI_SYS_PRINT_DEBUG(1, "   bad  : %u.%u GB\n", badGB,   (space.bad&0x3fffffff)>>20);

        #endif /* #if (DEBUG > 0) */
    }
    while ( 0 );

    if ( retVal != FAPI_OK )
    {
        // TODO: unmount/perform error handling on failure?

        FAPI_SYS_PRINT_DEBUG(4,"[DEV] Device formatting failed: %s (%i)\n",
                   SYS_DeviceGetErrStr(retVal), retVal);
    }

    #endif /* APPL_FATFS_ENABLED */

    return newMountStatus;
}


int32_t SYS_DeviceReadDir(char_t *dirpath)
{
    #if (DEBUG > 0)
    #ifdef APPL_FATFS_ENABLED

    F_FIND entry;
    uint32_t kbytes;
    uint32_t bytes;
    uint8_t day, month;
    uint16_t year = 1980;
    uint8_t sec, min, hour;

    FAPI_SYS_PRINT_DEBUG(1, "Directory contents: \"%s\"\n", dirpath );
    FAPI_SYS_PRINT_DEBUG(1, "Type Attr     Filename     kbytes.bytes             Date&Time  \n" );
    FAPI_SYS_PRINT_DEBUG(1, "---- ---- ---------------- ----------------------------------------\n" );
    if ( !f_findfirst(dirpath, &entry) ) {
        do
        {
            year   = 1980;
            kbytes = ((uint32_t)entry.filesize) / 1024;
            bytes  = ((uint32_t)entry.filesize) % 1024;
            day    = (entry.cdate & F_CDATE_DAY_MASK)   >> F_CDATE_DAY_SHIFT;
            month  = (entry.cdate & F_CDATE_MONTH_MASK) >> F_CDATE_MONTH_SHIFT;
            year  += (entry.cdate & F_CDATE_YEAR_MASK)  >> F_CDATE_YEAR_SHIFT;
            sec    = (entry.ctime & F_CTIME_SEC_MASK)   >> F_CTIME_SEC_SHIFT;
            min    = (entry.ctime & F_CTIME_MIN_MASK)   >> F_CTIME_MIN_SHIFT;
            hour   = (entry.ctime & F_CTIME_HOUR_MASK)  >> F_CTIME_HOUR_SHIFT;

            FAPI_SYS_PRINT_DEBUG(1, "  %c 0x%02X %16s %10u.%-10u",
                        (entry.attr & F_ATTR_DIR) ? 'D' : 'F',
                        entry.attr,
                        entry.filename,
                        kbytes,
                        bytes);
            FAPI_SYS_PRINT_DEBUG(1," %.2d.%.2d.%.4d", day, month, year);
            FAPI_SYS_PRINT_DEBUG(1," %.2d:%.2d:%.2d\n", hour, min, sec );
        } while (!f_findnext(&entry));
    }
    FAPI_SYS_PRINT_DEBUG(1,"\n" );

    #endif /* APPL_FATFS_ENABLED */
    #endif /* #if (DEBUG > 0) */

    return( 1 );
}

FSTATIC int32_t SYS_DeviceGetDriveNum (SYS_DEV_DATA_S* inst,
                                       SYS_DEVICE_E device, int8_t partIdx,
                                       fbool_t onlyIfAvailable)
{
    int32_t driveNum = 0;
    int32_t retVal;

    if ( (device >= SYS_DEV_NUMS)
      || (device == SYS_DEV_NULL)
      || (partIdx < 0) )
    {
        /* bad parameter passed */
        retVal = -1;
    }
    else
    {
        retVal = -1;

        while ( driveNum < SYS_DEV_MAX_NUM_DRIVES )
        {
            if ( (inst->drive[driveNum].device  == device)
              && (inst->drive[driveNum].partIdx == partIdx) )
            {
                if ( (!onlyIfAvailable)
                  || (inst->drive[driveNum].available) )
                {
                    retVal = driveNum;
                }
                break;
            }

            driveNum++;
        }
    }

    return retVal;
}

#ifdef APPL_FATFS_ENABLED

/*!
*******************************************************************************
**
** \brief Get Date callback function for FATFS.
**
** This type of function is accepted as \b SYS_DeviceGetDate callback by
** the initialization function of the FATFS filesystem.
**
** \param yearPtr  The address where the year should be stored.
**                 Valid value range 1980-2107.
** \param monthPtr The address where the month of the year should be stored.
**                 Valid value range 1-12.
** \param dayPtr   The address where the day of the month should be stored.
**                 Valid value range 1-31.
**
** \return One of the following return codes:
**   - #FAPI_OK                         to indicate successful operation
**   - #APPL_SMARTGO_ERR_BAD_PARAMETER  to indicate a wrong argument
**
*******************************************************************************
*/
FSTATIC int32_t SYS_DeviceGetDate (uint16_t* yearPtr, uint16_t* monthPtr,
                                   uint16_t* dayPtr)
{
    int32_t     retVal    = FAPI_OK;
    fbool_t     hasSynced = FFALSE;
    SYS_TIME_S  time;
    SYS_DATE_S  date;

    if( (yearPtr  == NULL) ||
        (monthPtr == NULL) ||
        (dayPtr   == NULL) )
    {
        return (APPL_SMARTGO_ERR_BAD_PARAMETER);
    }

    memset(&date, 0 ,sizeof(SYS_DATE_S));
    memset(&time, 0 ,sizeof(SYS_TIME_S));

    retVal = TIME_TimeAndDateGet(&time,&date,&hasSynced);
    if( (retVal != FAPI_OK) || ( hasSynced == FFALSE ) )
    {
        return (APPL_SMARTGO_ERR_BAD_PARAMETER);
    }

    *yearPtr  = date.year;
    *monthPtr = date.month;
    *dayPtr   = date.day;

    return (FAPI_OK);
}

/*!
*******************************************************************************
**
** \brief Get Time callback function.
**
** This type of function is accepted as \b SYS_DeviceGetTime callback by
** the initialization function of the FATFS filesystem.
**
** \param hoursPtr   The address where the hours of the day should be stored.
**                   Valid value range 0-23.
** \param minutesPtr The address where the minutes of the hour should be stored.
**                   Valid value range 0-59.
** \param secondsPtr The address where the seconds of the minute should be
**                   stored. Valid value range 0-59.
**
** \return One of the following return codes:
**   - #FAPI_OK                      to indicate successful operation
**   - #APPL_SMARTGO_ERR_BAD_PARAMETER       to indicate a wrong argument
**
*******************************************************************************
*/
FSTATIC int32_t SYS_DeviceGetTime (uint16_t* hoursPtr, uint16_t* minutesPtr,
                                   uint16_t* secondsPtr)
{
    int32_t     retVal    = FAPI_OK;
    fbool_t     hasSynced = FFALSE;
    SYS_TIME_S  time;
    SYS_DATE_S  date;

    if( (hoursPtr   == NULL) ||
        (minutesPtr == NULL) ||
        (secondsPtr == NULL) )
    {
        return (APPL_SMARTGO_ERR_BAD_PARAMETER);
    }

    memset(&date, 0 ,sizeof(SYS_DATE_S));
    memset(&time, 0 ,sizeof(SYS_TIME_S));

    retVal = TIME_TimeAndDateGet(&time,&date,&hasSynced);
    if( (retVal != FAPI_OK) || ( hasSynced == FFALSE ) )
    {
        return (APPL_SMARTGO_ERR_BAD_PARAMETER);
    }

    *hoursPtr   = time.hour;
    *minutesPtr = time.minute;
    *secondsPtr = time.second;

    return (FAPI_OK);
}

#if (DEBUG > 0)

FSTATIC char_t*  SYS_DeviceGetErrStr (int32_t status)
{
    switch( status ) {
        case F_NO_ERROR             :  return("OK");
        case F_ERR_INVALIDDRIVE     :  return("ERR_INVALIDDRIVE");
        case F_ERR_NOTFORMATTED     :  return("ERR_NOTFORMATTED");
        case F_ERR_INVALIDDIR       :  return("ERR_INVALIDDIR");
        case F_ERR_INVALIDNAME      :  return("ERR_INVALIDNAME");
        case F_ERR_NOTFOUND         :  return("ERR_NOTFOUND");
        case F_ERR_DUPLICATED       :  return("ERR_DUPLICATED");
        case F_ERR_NOMOREENTRY      :  return("ERR_NOMOREENTRY");
        case F_ERR_NOTOPEN          :  return("ERR_NOTOPEN");
        case F_ERR_EOF              :  return("ERR_EOF");
        case F_ERR_RESERVED         :  return("ERR_RESERVED");
        case F_ERR_NOTUSEABLE       :  return("ERR_NOTUSEABLE");
        case F_ERR_LOCKED           :  return("ERR_LOCKED");
        case F_ERR_ACCESSDENIED     :  return("ERR_ACCESSDENIED");
        case F_ERR_NOTEMPTY         :  return("ERR_NOTEMPTY");
        case F_ERR_INITFUNC         :  return("ERR_INITFUNC");
        case F_ERR_CARDREMOVED      :  return("ERR_CARDREMOVED");
        case F_ERR_ONDRIVE          :  return("ERR_ONDRIVE");
        case F_ERR_INVALIDSECTOR    :  return("ERR_INVALIDSECTOR");
        case F_ERR_READ             :  return("ERR_READ");
        case F_ERR_WRITE            :  return("ERR_WRITE");
        case F_ERR_INVALIDMEDIA     :  return("ERR_INVALIDMEDIA");
        case F_ERR_BUSY             :  return("ERR_BUSY");
        case F_ERR_WRITEPROTECT     :  return("ERR_WRITEPROTECT");
        case F_ERR_INVFATTYPE       :  return("ERR_INVFATTYPE");
        case F_ERR_MEDIATOOSMALL    :  return("ERR_MEDIATOOSMALL");
        case F_ERR_MEDIATOOLARGE    :  return("ERR_MEDIATOOLARGE");
        case F_ERR_NOTSUPPSECTORSIZE:  return("ERR_NOTSUPPSECTORSIZE");
        case F_ERR_DRVALREADYMNT    :  return("ERR_DRVALREADYMNT");
        case F_ERR_DELFUNC          :  return("ERR_DELFUNC");
        case F_ERR_ALLOCATION       :  return("ERR_ALLOCATION");
        default: return("UNKNOWN ERROR");
    }
}

#endif /* #if (DEBUG > 0) */

/*!
*******************************************************************************
**
** \brief Get Task ID callback function.
**
** This type of function is accepted as \b SYS_DeviceGetTaskId callback by
** the initialization function of the FATFS filesystem.
**
** \return The valid unique ID of the current task/thread.
**
*******************************************************************************
*/
FSTATIC uint32_t SYS_DeviceGetTaskId (void)
{
    #ifndef _LINUX_FAPI
    return ( (uint32_t)RTOS_GetThread() );
    #else
    return FAPEX_SYS_GetThread();
    #endif
}

#endif /* APPL_FATFS_ENABLED */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
