/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/eit.c
**
**              $Id: eit.c 5871 2010-03-02 13:42:55Z jmorha $
**
** \brief       EIT database module.
**
** See here for more detail documentation: \ref app_smartgo_eit
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info    754: Local structure member 'lstHdr' not referenced.
** - Note    946: Relational or subtract operator applied to pointers.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1,
**                dependence placed on C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e754 -e946 -e961 */
#define DEBUG 1

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <sfal/sfal.h>

#include "utils.h"

#include "settings.h"

#include "av.h"
#include "event_queue.h"
#include "sys_time.h"
#include "linklist.h"
#include "eit.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define EIT_TABLEID_PF_ACTUALTS     0x4E
#define EIT_TABLEID_PF_OTHERTS      0x4F

#define EIT_FILTER_DEPTH            6

#define EIT_SECTLEN_MIN             18

#define EIT_AVAILABLE_BASIC         0x01
#define EIT_AVAILABLE_EXT           0x02
#define EIT_AVAILABLE_NO            0x04

#define EIT_UPDATE_CURR             0x01
#define EIT_UPDATE_GNR              0x02

#define EIT_VERSION_INVALID         0x80        //!< Marks version as invalid.
#define EIT_VERSION_OTHER           0x40        //!< Marks version referencing "other TS".
/* NOTE: 5 least significant bits are reserved for version information! */

#define EIT_SCHED_TABLE_MAX     16      //!< max. EIT Schedule tables

#ifdef APPL_PVR_ENABLED

#define EIT_CURRNEXT_ENTRIES_MAX ((AV_EIT_CURRNEXT_SERV_MAX) + 1)
#define EIT_SCHED_ENTRIES_MAX    ((AV_EIT_SCHED_SERV_MAX) + 1)

#define EIT_CURRNEXT_IDX_PLAY    (AV_EIT_CURRNEXT_SERV_MAX)
#define EIT_SCHED_IDX_PLAY       (AV_EIT_SCHED_SERV_MAX)

#else

#define EIT_CURRNEXT_ENTRIES_MAX (AV_EIT_CURRNEXT_SERV_MAX)
#define EIT_SCHED_ENTRIES_MAX    (AV_EIT_SCHED_SERV_MAX)

#endif /* APPL_PVR_ENABLED */

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

enum                  /* EIT section filters */
{
    EIT_FILTER_PF_ACT = 0,      //!< Present/following, actual TS
    EIT_FILTER_PF_OTH,          //!< Present/following, other TS
    EIT_FILTER_SCHED_ACT,       //!< Schedule, actual TS
    EIT_FILTER_SCHED_OTH,       //!< Schedule, other TS
    EIT_FILTER_NUMS

};

typedef struct EIT_TablerevS
{
    uint8_t             active; //!< curr. version number
    //!< or'ed with
    //!< EIT_VERSION_XXX status.
    uint32_t            sectFound[PSISI_SECT_MAX_LW];   //!< Sections found for
    //!< curr. version number.

} EIT_TABLEREV_S;

typedef struct EIT_SchedS
{
    LINKLST_HDR_S   lstHdr;

    uint32_t        tableId;    // EIT table reference

    EIT_EVT_S       evt;        // basic EIT information

#if ( AV_SCHED_STORE_EXT != 0 )

    SYS_CHARSET        extCharSet;
    char_t          pExt[AV_EIT_EXTLEN_SCHED];      // (optional) extended event text

#endif                          /* AV_SCHED_STORE_EXT != 0 */

} EIT_SCHED_S;

typedef struct EIT_EntryHdrS
{
    SLIST_SERVIDX       servIdx;        //!< service identification (-1: free)
    uint8_t             update; //!< >0: update this entry from stream
    uint32_t            lastAccessTime;
    uint16_t            service_id;     //!< service_id
    uint16_t            ts_id;  //!< transport_stream_id

} EIT_ENTRY_HDR_S;

typedef struct EIT_CurrnextEvtS
{
    uint8_t             availability;   //!< Bitfield indicating
    //!< availability of items.
    EIT_EVT_S           event;  //!< Basic event data.

    SYS_CHARSET        extCharSet;
    char_t                ext[AV_EIT_EXTLEN_CURRNEXT];    //!< Ext. description.

} EIT_CURRNEXT_EVT_S;

typedef struct EIT_CurrnextEntryS
{
    EIT_ENTRY_HDR_S     hdr;

    EIT_CURRNEXT_EVT_S  curr;
    EIT_CURRNEXT_EVT_S  next;

    EIT_TABLEREV_S      sectVer;

} EIT_CURRNEXT_ENTRY_S;

typedef struct EIT_SchedEntryS
{
    EIT_ENTRY_HDR_S     hdr;

    EIT_SCHED_S         lst[AV_SCHED_ENTRIES_MAX];
    LINKLST_S           free;   //!< list holding all free entries
    LINKLST_S           used;   //!< list holding all used entries

    /* version numbers of currently stored EIT sections */
    EIT_TABLEREV_S      sectVerSched[EIT_SCHED_TABLE_MAX];
    EIT_TABLEREV_S      sectVerPf;

} EIT_SCHED_ENTRY_S;

/*!
 * \brief Internal data structure for EIT database.
 */
typedef struct EIT_DataS
{
    FAPEX_SYS_SemaphoreT  evtMutex;
    FAPI_SYS_HandleT    filterHdl[EIT_FILTER_NUMS];
    fbool_t             filtersActive;
    fbool_t             playMode;
    EIT_PARAMS_S        params;
    uint16_t            currServiceId;

    /* pointers to current live service */
    EIT_CURRNEXT_ENTRY_S *focusCurrNext;
    EIT_SCHED_ENTRY_S    *focusSched;

    EIT_CURRNEXT_ENTRY_S currNext[EIT_CURRNEXT_ENTRIES_MAX];
    EIT_SCHED_ENTRY_S    sched[EIT_SCHED_ENTRIES_MAX];

    uint16_t            currNextFirstFree;
    uint16_t            schedFirstFree;

    EIT_SCHED_S         tmpSched;

} EIT_DATA_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC EIT_DATA_S  *eitDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC int32_t     EIT_FilteringInit(void);
FSTATIC int32_t     EIT_FiltersOpen(void);
FSTATIC int32_t     EIT_FiltersClose(void);
FSTATIC void        EIT_SectPfNotify(FAPI_SYS_HandleT handle,
                                     SFAL_SECTION_STATUS_E   sectionStatus,
                                     const uint8_t * pSect,
                                     uint16_t length, void *pPrivateData);
FSTATIC void        EIT_SectSchedNotify(FAPI_SYS_HandleT handle,
                                        SFAL_SECTION_STATUS_E   sectionStatus,
                                        const uint8_t * pSect,
                                        uint16_t length, void *pPrivateData);
FSTATIC void        EIT_Notify(FAPI_SYS_HandleT handle,
                                     SFAL_SECTION_STATUS_E   sectionStatus,
                                     const uint8_t * pSect,
                                     uint16_t length, void *pPrivateData);

FSTATIC uint8_t     EIT_SectPfFiltered(FAPI_SYS_HandleT handle,
                                       const volatile uint8_t * pSectStart,
                                       const volatile uint8_t * pSectEnd,
                                       const volatile uint8_t * pSect,
                                       uint16_t length, void *pPrivateData);
FSTATIC uint8_t     EIT_SectSchedFiltered(FAPI_SYS_HandleT handle,
                                          const volatile uint8_t * pSectStart,
                                          const volatile uint8_t * pSectEnd,
                                          const volatile uint8_t * pSect,
                                          uint16_t length, void *pPrivateData);

FSTATIC void        EIT_CurrNextEntryClear(EIT_CURRNEXT_ENTRY_S * pEntry);
FSTATIC void        EIT_CurrNextEntryFree(uint16_t index);

FSTATIC void        EIT_SchedEntryClear(EIT_SCHED_ENTRY_S * pEntry);
FSTATIC void        EIT_SchedEntryFree(uint16_t index);

FSTATIC void        EIT_OccupyEntry(EIT_ENTRY_HDR_S * pHdr,
                                    SLIST_SERVIDX servIdx);
FSTATIC void        EIT_FreeEntry(EIT_ENTRY_HDR_S * pHdr);

FSTATIC int32_t     EIT_SchedListInsert(EIT_SCHED_ENTRY_S * pEntry,
                                        const EIT_SCHED_S * pSchedItem);
FSTATIC int32_t     EIT_SchedListRemoveTable(EIT_SCHED_ENTRY_S * pEntry,
                                             uint8_t tableId);

FSTATIC /* FINLINE */ void EIT_SchedListCopy(const EIT_SCHED_S * pItem,
                                       EIT_EVT_S * pEvt, EIT_EXTEVT_S * pExt);

FSTATIC fbool_t     EIT_ExtractCurrNext(EIT_EVT_S * pEvt, char_t *pExt,
                                        SYS_CHARSET* pExtCharSet,
                                        uint32_t extLen, uint8_t * avl,
                                        const uint8_t * pBuf, int32_t bufLen,
                                        uint8_t sectNum);
FSTATIC fbool_t     EIT_ExtractSched(EIT_SCHED_ENTRY_S * pSched,
                                     EIT_SCHED_S * pItem, const uint8_t * pBuf,
                                     int32_t len, uint8_t sectNum,
                                     uint8_t tableId);

FSTATIC void        EIT_GetShortEventDescr(const uint8_t * descData,
                                           uint16_t descLen,
                                           EIT_EVT_S* pEvt);
FSTATIC void        EIT_ProcessParentalDescr(const uint8_t * descData,
                                             uint16_t descLen,
                                             uint8_t * minAge);
FSTATIC void EIT_ProcessContentDescr(const uint8_t * descData,
                                      uint16_t descLen, uint8_t * content);
FSTATIC uint8_t     EIT_GetMinAge(SYS_CTRYCODE country, uint8_t rating);
FSTATIC void        EIT_GetDescrLang(const uint8_t * pLoop, uint16_t loopLen,
                                     SYS_LANGCODE * pShort,
                                     SYS_LANGCODE * pExt);

FSTATIC EIT_CURRNEXT_ENTRY_S *EIT_GetEntryCurrNext(SLIST_SERVIDX servIdx);
FSTATIC EIT_CURRNEXT_ENTRY_S *EIT_GetFreeEntryCurrNext(SLIST_SERVIDX servIdx);
FSTATIC EIT_CURRNEXT_ENTRY_S *EIT_GetTargetEntryCurrNext(EIT_DATA_S * inst,
                                                         uint16_t serviceId,
                                                         uint16_t tsId);

FSTATIC EIT_SCHED_ENTRY_S *EIT_GetEntrySched(SLIST_SERVIDX servIdx);
FSTATIC EIT_SCHED_ENTRY_S *EIT_GetFreeEntrySched(SLIST_SERVIDX servIdx);
FSTATIC EIT_SCHED_ENTRY_S *EIT_GetTargetEntrySched(EIT_DATA_S * inst,
                                                   uint16_t serviceId,
                                                   uint16_t tsId);

FSTATIC void        EIT_SetCurrService  (SLIST_SERVIDX servIdx);

#ifdef APPL_PVR_ENABLED

FSTATIC int32_t     EIT_PlayClear(void);

#endif /* APPL_PVR_ENABLED */

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/


/*
NOTE: Needs SF to be initialized.
*/
int32_t EIT_Init(void)
{
    int32_t             retVal = FAPI_OK;
    uint32_t            cnt;

    if(eitDat != NULL)
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    eitDat = UTILS_Malloc(sizeof(EIT_DATA_S));
    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    for(;;)
    {
        /* Inits all entries to 0. Non-zero initialisations are set below */
        memset(eitDat, 0, sizeof(EIT_DATA_S));

        eitDat->filtersActive       = FFALSE;
        eitDat->playMode            = FFALSE;

        eitDat->params.modeCurrNext = EIT_FILTERMODE_THISSERVICE;
        eitDat->params.modeSchedule = EIT_FILTERMODE_THISSERVICE;

        eitDat->focusCurrNext       = NULL;
        eitDat->focusSched          = NULL;

        for ( cnt = 0; cnt < EIT_CURRNEXT_ENTRIES_MAX; cnt++ )
        {
            EIT_CurrNextEntryClear(eitDat->currNext + cnt);
            EIT_CurrNextEntryFree (cnt);
        }

        for ( cnt = 0; cnt < EIT_SCHED_ENTRIES_MAX; cnt++ )
        {
            EIT_SchedEntryClear(eitDat->sched + cnt);
            EIT_SchedEntryFree (cnt);
        }

        /* ----------------------------------------------------------------- */

        eitDat->evtMutex = FAPEX_SYS_CreateSemaphore(1);
        if(eitDat->evtMutex == NULL)
        {
            retVal = SMARTGO_SYS_ERR_RESOURCES;
            break;
        }

        /* ----------------------------------------------------------------- */

        retVal = EIT_FilteringInit();
        if(retVal != FAPI_OK)
        {
            break;
        }

        break; //end of for(;;)
    }

    return retVal;
}

int32_t EIT_SetParams(const EIT_PARAMS_S* pParams)
{
    FAPI_SYS_PRINT_DEBUG(1, "[eit][EIT_SetParams] enter\n ");

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(pParams == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    // TODO: to be protected?
    eitDat->params = *pParams;

    if(eitDat->filtersActive)
    {
        /* Restart filtering in new modes */
        FAPI_SYS_PRINT_DEBUG(1, "[eit][EIT_SetParams] try to restart filtering\n ");
        (void)EIT_FiltersClose();
        (void)EIT_FiltersOpen();
        FAPI_SYS_PRINT_DEBUG(1, "[eit][EIT_SetParams] end restart filtering\n ");
    }
    FAPI_SYS_PRINT_DEBUG(1, "[eit][EIT_SetParams] quit success\n ");
    return FAPI_OK;
}

int32_t EIT_GetParams(EIT_PARAMS_S * pParams)
{
    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(pParams == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    *pParams = eitDat->params;

    return FAPI_OK;
}

int32_t EIT_ClearDatabase(void)
{
    uint32_t cnt;

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    for(cnt = 0; cnt < EIT_CURRNEXT_ENTRIES_MAX; cnt++)
    {
        EIT_CurrNextEntryClear(eitDat->currNext + cnt);
    }
    for(cnt = 0; cnt < EIT_SCHED_ENTRIES_MAX; cnt++)
    {
        EIT_SchedEntryClear(eitDat->sched + cnt);
    }

    return FAPI_OK;
}


FSTATIC int32_t EIT_FilteringInit(void)
{
    uint8_t                 cnt;
    int32_t                 retVal = FAPI_OK;
    FAPI_SYS_HandleT        filterSetHdl;
    SFAL_FILTERSET_S          filterSetPars;
    SFAL_SECTION_FILTERED_F   handler;

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    filterSetPars.tsdInput          = RSRC_TSD_LIVE;
    filterSetPars.numberOfChannels  = 1;
    filterSetPars.filtersPerChannel = EIT_FILTER_NUMS;
    filterSetPars.filterDepth       = EIT_FILTER_DEPTH;
    /*lint -save -e40 */
    /* defines place in makefiles */
    //filterSetPars.bufferAddress              =  INIT_EIT_ADDR;
    filterSetPars.bufferSize        = INIT_EIT_SIZE;
    /*lint -restore */

    for(;;)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[eit][EIT_FilteringInit] OpenFilterset for EIT (%d channel, %d filter)\n",
                                  filterSetPars.numberOfChannels,
                                  filterSetPars.filtersPerChannel);
        retVal = SFAL_OpenFilterset(&filterSetPars, &filterSetHdl);
        if(retVal != FAPI_OK)
        {
            break;
        }

        for(cnt = 0; cnt < EIT_FILTER_NUMS; cnt++)
        {
            retVal = SFAL_OpenFilter(filterSetHdl, EIT_FILTER_DEPTH,
                                      &(eitDat->filterHdl[cnt]));
            if(retVal != FAPI_OK)
            {
                break;
            }
            retVal = SFAL_SetFilterPid(eitDat->filterHdl[cnt], ISO_EIT_PID);
            if(retVal != FAPI_OK)
            {
                break;
            }
            switch (cnt)
            {
                default:
                case EIT_FILTER_PF_ACT:
                case EIT_FILTER_PF_OTH:
                    handler = EIT_SectPfFiltered;
                    break;
                case EIT_FILTER_SCHED_ACT:
                case EIT_FILTER_SCHED_OTH:
                    handler = EIT_SectSchedFiltered;
                    break;
            }
            //retVal = SFAL_SetSectionFilteredNotify(eitDat->filterHdl[cnt],
            //                                        handler);
        }
        break;// end of for(;;)
    }

    return retVal;
}

int32_t EIT_FilteringStart(SLIST_SERVIDX servIdx, uint16_t service_id)
{
    int32_t retVal = FAPI_OK;
    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(eitDat->filtersActive)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    eitDat->currServiceId = service_id;

    switch (servIdx)
    {
        #ifdef APPL_PVR_ENABLED

        case SLIST_SERVIDX_PLAY:
            (void)EIT_PlayClear();     /* Clear "play" entry in EIT database */

            eitDat->playMode = FTRUE;
            break;

        #endif /* APPL_PVR_ENABLED */

        case -1:
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
            break;

        default:
            eitDat->playMode = FFALSE;
            break;
    }

    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    EIT_SetCurrService(servIdx);        /* mark current service */

    #ifdef APPL_PVR_ENABLED

    if(eitDat->playMode)
    {
        if(eitDat->focusCurrNext != NULL)
        {
            eitDat->focusCurrNext->hdr.service_id = service_id;
            eitDat->focusCurrNext->hdr.servIdx    = SLIST_SERVIDX_PLAY;
        }

        if(eitDat->focusSched != NULL)
        {
            eitDat->focusSched->hdr.service_id = service_id;
            eitDat->focusSched->hdr.servIdx    = SLIST_SERVIDX_PLAY;
        }
    }

    #endif /* APPL_PVR_ENABLED */

    retVal = EIT_FiltersOpen();

    if(retVal == FAPI_OK)
    {
        eitDat->filtersActive = FTRUE;
    }

    return retVal;
}

int32_t EIT_FilteringStop(void)
{
    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(eitDat->filtersActive)
    {
        (void)EIT_FiltersClose();

        EIT_SetCurrService(-1);

        eitDat->filtersActive = FFALSE;
    }

    return FAPI_OK;
}

FSTATIC int32_t EIT_FiltersOpen(void)
{
    int32_t retVal = FAPI_OK;
    uint8_t filterData[EIT_FILTER_DEPTH];
    uint8_t filterEqual[EIT_FILTER_DEPTH];
    uint8_t filterNonEq[EIT_FILTER_DEPTH];

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    memset(filterData,  0, EIT_FILTER_DEPTH);
    memset(filterEqual, 0, EIT_FILTER_DEPTH);
    memset(filterNonEq, 0, EIT_FILTER_DEPTH);

    /* set current_next_indicator: always filter for current version */
    filterData [5] |= 0x01;
    filterEqual[5] |= 0x01;

    /* ----- Present/following, actual TS ---------------------------------- */

    /*lint -save -e548 */
    if ( eitDat->params.modeCurrNext >= EIT_FILTERMODE_THISSERVICE )
    {
        filterData[0]  = EIT_TABLEID_PF_ACTUALTS;
        filterEqual[0] = 0x00;//not check tableid

        if ( eitDat->params.modeCurrNext == EIT_FILTERMODE_THISSERVICE )
        {
            filterData [3] = (eitDat->currServiceId & 0xFF00) >> 8;
            filterData [4] = (eitDat->currServiceId & 0x00FF);
            filterEqual[3] = 0xFF;
            filterEqual[4] = 0xFF;
        }
        else
        {
            filterData [3] = 0;
            filterData [4] = 0;
            filterEqual[3] = 0;
            filterEqual[4] = 0;
        }

        retVal = SFAL_SetFilterPatterns (eitDat->filterHdl[EIT_FILTER_PF_ACT],
                                       filterData, filterEqual, filterNonEq);

        if ( retVal == FAPI_OK )
        {
            retVal = SFAL_StartFilter(eitDat->filterHdl[EIT_FILTER_PF_ACT],
                                    //SFAL_NO_TIMEOUT,
                                    100,
                                    SFAL_LOOP_CONTINUOUS,
                                    SFAL_CRC_CHECK,     /* CRC check enabled */
                                    NULL,
                                    //EIT_SectPfNotify,
                                    EIT_Notify,
                                    eitDat);
        }

        if ( retVal != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(4,"[EIT] Filter start failed! \n");
        }
    }
#if 0
    /* ----- Present/following, other TS ----------------------------------- */

    if ( eitDat->params.modeCurrNext >= EIT_FILTERMODE_ALL )
    {
        filterData [0] = EIT_TABLEID_PF_OTHERTS;
        filterData [3] = 0;
        filterData [4] = 0;

        filterEqual[0] = 0xFF;
        filterEqual[3] = 0;
        filterEqual[4] = 0;

        retVal = SFAL_SetFilterPatterns (eitDat->filterHdl[EIT_FILTER_PF_OTH],
                                       filterData, filterEqual, filterNonEq);

        if ( retVal == FAPI_OK )
        {
            retVal = SFAL_StartFilter(eitDat->filterHdl[EIT_FILTER_PF_OTH],
                                    SFAL_NO_TIMEOUT,
                                    SFAL_LOOP_CONTINUOUS,
                                    SFAL_CRC_CHECK,     /* CRC check enabled */
                                    NULL,
                                    EIT_SectPfNotify,
                                    eitDat);
        }

        if ( retVal != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(4,"[EIT] Filter start failed! \n");
        }
    }

    /* ----- Schedule, actual TS ------------------------------------------- */

    if ( eitDat->params.modeSchedule >= EIT_FILTERMODE_THISSERVICE )
    {
        filterData [0] = 0x50;
        filterEqual[0] = 0xF0;

        if ( eitDat->params.modeSchedule == EIT_FILTERMODE_THISSERVICE )
        {
            filterData [3] = (eitDat->currServiceId & 0xFF00) >> 8;
            filterData [4] = (eitDat->currServiceId & 0x00FF);

            filterEqual[3] = 0xFF;
            filterEqual[4] = 0xFF;
        }
        else
        {
            filterData [3] = 0;
            filterData [4] = 0;

            filterEqual[3] = 0;
            filterEqual[4] = 0;
        }

        retVal = SFAL_SetFilterPatterns (eitDat->filterHdl[EIT_FILTER_SCHED_ACT],
                                       filterData, filterEqual, filterNonEq);

        if ( retVal == FAPI_OK )
        {
            retVal = SFAL_StartFilter(eitDat->filterHdl[EIT_FILTER_SCHED_ACT],
                                      SFAL_NO_TIMEOUT,
                                      SFAL_LOOP_CONTINUOUS,
                                      SFAL_CRC_CHECK,     /* CRC check enabled */
                                      NULL,
                                      EIT_SectSchedNotify,
                                      eitDat);
        }

        if ( retVal != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(4,"[EIT]Filter start failed! \n");
        }
    }

    /* ----- Schedule, other TS -------------------------------------------- */

    if ( eitDat->params.modeSchedule >= EIT_FILTERMODE_ALL )
    {
        filterData [0] = 0x60;
        filterData [3] = 0;
        filterData [4] = 0;

        filterEqual[0] = 0xF0;
        filterEqual[3] = 0;
        filterEqual[4] = 0;

        retVal = SFAL_SetFilterPatterns (eitDat->filterHdl[EIT_FILTER_SCHED_OTH],
                                       filterData, filterEqual, filterNonEq);

        if ( retVal == FAPI_OK )
        {
            retVal = SFAL_StartFilter(eitDat->filterHdl[EIT_FILTER_SCHED_OTH],
                                      SFAL_NO_TIMEOUT,
                                      SFAL_LOOP_CONTINUOUS,
                                      SFAL_CRC_CHECK,     /* CRC check enabled */
                                      NULL,
                                      EIT_SectSchedNotify,
                                      eitDat);
        }

        if ( retVal != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(4,"[EIT] Filter start failed! \n");
        }
    }
    /*lint -restore */
#endif
    // TODO: improve error handling

    return FAPI_OK;
}

FSTATIC int32_t EIT_FiltersClose(void)
{
    uint8_t             cnt;

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    for(cnt = 0; cnt < EIT_FILTER_NUMS; cnt++)
    {
        (void)SFAL_StopFilter(eitDat->filterHdl[cnt]);
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Request EIT "present" or "following" event information from database.
**
** \param  pServLoc  Service, identified by service list reference,
**                   set to NULL for EIT information of current playback.
** \param  type      Type of event: EIT_TYPE_CURR (present) or
**                   EIT_TYPE_NEXT (following).
** \param  pEvent    EIT event data structure (filled by function).
**
** \return One of the following status codes:
** - #FAPI_OK if successful
** - #SMARTGO_SYS_ERR_DATA_NOT_AVL       if event data is not yet available
** - #SMARTGO_SYS_ERR_DATA_NOT_PROVIDED  if requested event data is not
**                                        provided by broadcaster
**
** \sa EIT_GetExtEvt
******************************************************************************/
int32_t EIT_GetEvent(const SYS_SERVLOC_S * pServLoc, EIT_EVT_TYPE_E type,
                     EIT_EVT_S * pEvent)
{
    int32_t                 retVal  = FAPI_OK;
    EIT_CURRNEXT_EVT_S *    pEvt    = NULL;
    EIT_CURRNEXT_ENTRY_S    *pEntry = NULL;
    SLIST_SERVIDX           servIdx = SLIST_SERVIDX_NONE;

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(pEvent == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    if(pServLoc != NULL)
    {
        servIdx = SLIST_GetServiceIndex(pServLoc);
    }

    #ifdef APPL_PVR_ENABLED

        else
        {
            servIdx = SLIST_SERVIDX_PLAY;
        }
    #else

        else
        {
            return APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
    #endif /* APPL_PVR_ENABLED */

    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    pEntry = EIT_GetEntryCurrNext(servIdx);

    if(pEntry == NULL)
    {
        retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }
    else
    {
        switch (type)
        {
            case EIT_TYPE_CURR:
                pEvt = &(pEntry->curr);
                break;

            case EIT_TYPE_NEXT:
                pEvt = &(pEntry->next);
                break;

            default:
                retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
                break;
        }

        if(pEvt != NULL)
        {
            if((pEvt->availability & EIT_AVAILABLE_NO) != 0)
            {
                retVal = SMARTGO_SYS_ERR_DATA_NOT_PROVIDED;
            }
            else if((pEvt->availability & EIT_AVAILABLE_BASIC) == 0)
            {
                retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
            }
            else
            {
                *pEvent = pEvt->event;
            }
        }

        (void)TIME_GetSystemTime(&(pEntry->hdr.lastAccessTime));
    }

    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Request extended information for "present" or "following" EIT event
**         from database.
**
** \param  pServLoc  Service, identified by service list reference,
**                   set to NULL for EIT information of current playback.
** \param  type      Type of event: EIT_TYPE_CURR (present) or
**                   EIT_TYPE_NEXT (following).
** \param  pExt      Parameters for extended event description.
**
** \return One of the following status codes:
** - #FAPI_OK if successful
** - #SMARTGO_SYS_ERR_DATA_NOT_AVL       if event data is not yet available
** - #SMARTGO_SYS_ERR_DATA_NOT_PROVIDED  if requested event data is not
**                                        provided by broadcaster
**
** \sa EIT_GetEvent
******************************************************************************/
int32_t EIT_GetExtEvt(const SYS_SERVLOC_S * pServLoc, EIT_EVT_TYPE_E type,
                      EIT_EXTEVT_S * pExt)
{
    int32_t             retVal = FAPI_OK;
    EIT_CURRNEXT_ENTRY_S *pEntry = NULL;
    EIT_CURRNEXT_EVT_S *pEvt = NULL;
    SLIST_SERVIDX       servIdx = SLIST_SERVIDX_NONE;

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if((pExt == NULL) || (pExt->pBuf == NULL))
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    if(pServLoc != NULL)
    {
        servIdx = SLIST_GetServiceIndex(pServLoc);
    }

    #ifdef APPL_PVR_ENABLED

    else
    {
        servIdx = SLIST_SERVIDX_PLAY;
    }
    #else

        else
        {
            return APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
    #endif /* APPL_PVR_ENABLED */

    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    pEntry = EIT_GetEntryCurrNext(servIdx);

    if(pEntry == NULL)
    {
        retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }
    else
    {
        switch (type)
        {
            case EIT_TYPE_CURR:
                pEvt = &(pEntry->curr);

                break;

            case EIT_TYPE_NEXT:
                pEvt = &(pEntry->next);
                break;

            default:
                retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
                break;
        }

        if(pEvt != NULL)
        {
            if((pEvt->availability & EIT_AVAILABLE_NO) != 0)
            {
                retVal = SMARTGO_SYS_ERR_DATA_NOT_PROVIDED;
            }
            else if((pEvt->availability & EIT_AVAILABLE_EXT) == 0)
            {
                /* use short event discription if ext desc is not available */
                pExt->descStrLen = STR_Copy (STR_Build(pEvt->event.descCharSet,
                                                       pEvt->event.desc),
                                                       pExt->pBuf,
                                                       &(pExt->descStr.cs),
                                                       pEvt->event.descCharSet,
                                                       pExt->maxLen);
                pExt->descStr.chr = pExt->pBuf;
            }
            else
            {
                pExt->descStrLen = STR_Copy (STR_Build(pEvt->extCharSet,
                                                       pEvt->ext),
                                             pExt->pBuf,
                                             &(pExt->descStr.cs),
                                             pEvt->extCharSet,
                                             pExt->maxLen);
                pExt->descStr.chr = pExt->pBuf;
            }
        }

        (void)TIME_GetSystemTime(&(pEntry->hdr.lastAccessTime));
    }

    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Get number of currently available schedule events for requested
**         service.
**
** \param  pServLoc  Service, identified by service list reference,
**                   set to NULL for EIT information of current playback.
**
** \return Schedule list size.
******************************************************************************/
uint32_t EIT_GetSchedListSize(const SYS_SERVLOC_S * pServLoc)
{
    EIT_SCHED_ENTRY_S  *pSched = NULL;
    uint32_t            currSize = 0;
    SLIST_SERVIDX       servIdx;

    if(eitDat == NULL)
    {
        return currSize;
    }
    /* get database entry */
    if(pServLoc != NULL)
    {
        servIdx = SLIST_GetServiceIndex(pServLoc);
    }

    #ifdef APPL_PVR_ENABLED

        else
        {
            servIdx = SLIST_SERVIDX_PLAY;
        }
    #else

        else
        {
            return 0;
        }
    #endif /* APPL_PVR_ENABLED */

    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    pSched = EIT_GetEntrySched(servIdx);

    if(pSched != NULL)
    {
        currSize = LINKLST_GetListSize(&(pSched->used));

        (void)TIME_GetSystemTime(&(pSched->hdr.lastAccessTime));
    }

    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);

    return currSize;
}


/*!
*******************************************************************************
**
** \brief  Get a schedule event by its time and date from schedule list.
**
** The schedule entry currently active or the first one starting after the
** requested time and date is returned (if matchStartTime is FFALSE).
**
** This function also initialises an internal iterator. Until the next call of
** either EIT_GetSchedByIndex or EIT_GetSchedByDate, navigation through the
** schedule list is possible via EIT_GetSchedNext and EIT_GetSchedPrevious.
**
** \param  pServLoc       Service, identified by service list reference,
**                        set to NULL for EIT information of current playback.
** \param  pEvt           EIT event data (NULL if not needed).
** \param  pExt           Parameters for extended event description, set to
**                        NULL if not needed.
** \param  pRef           Iterator reference (filled by function).
** \param  startTime      Requested start time.
** \param  startDate      Requested start date.
** \param  matchStartTime FTRUE: an event is returned only if start and end
**                        time match; FFALSE: the first event with an end time
**                        after requested time is returned.
**
** \return List index of returned event (>= 0) or -1 in case of failure.
**
** \sa EIT_GetSchedByIndex, EIT_GetSchedNext, EIT_GetSchedPrevious
******************************************************************************/
int32_t EIT_GetSchedByDate(const SYS_SERVLOC_S * pServLoc,
                           EIT_EVT_S * pEvt,
                           EIT_EXTEVT_S * pExt,
                           EIT_POS_S * pRef,
                           SYS_TIME_S * startTime, SYS_DATE_S * startDate,
                           fbool_t matchStartTime)
{
    int32_t             retVal = -1;
    uint32_t            index = 0;
    EIT_SCHED_ENTRY_S  *pSched = NULL;
    SYS_TIME_S          tempTime;
    SYS_DATE_S          tempDate;
    uint32_t            durInSecs;
    EIT_SCHED_S        *pItem;

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    pRef->pItem = NULL;

    /* get database entry */
    if(pServLoc != NULL)
    {
        pRef->servIdx = SLIST_GetServiceIndex(pServLoc);
    }

    #ifdef APPL_PVR_ENABLED

        else
        {
            pRef->servIdx = SLIST_SERVIDX_PLAY;
        }
    #else

        else
        {
            return -1;
        }
    #endif /* APPL_PVR_ENABLED */

    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    pSched = EIT_GetEntrySched(pRef->servIdx);

    if(pSched != NULL)
    {
        pItem = (EIT_SCHED_S *) (void *) LINKLST_GetFirst(&(pSched->used));

        while(pItem != NULL)
        {
            tempTime = pItem->evt.startTime;
            tempDate = pItem->evt.startDate;

            /* test for end time */
            durInSecs = TIME_ConvToSeconds(&(pItem->evt.duration));
            TIME_AddSeconds(&tempTime, &tempDate, (int32_t)durInSecs);

            /* Event's end time must be greater than current time */
            if ( TIME_IsLaterThan(&tempTime, &tempDate,
                                  startTime, startDate) == FTRUE )
            {
                /* test for start time */
                if(matchStartTime)
                {
                    /* Event's start time must be smaller of equal to current
                       time */
                    if ( TIME_IsLaterThan(&(pItem->evt.startTime),
                                          &(pItem->evt.startDate),
                                          startTime, startDate) == FTRUE )
                    {
                        /* failure, abort search */
                        break;
                    }
                }

                EIT_SchedListCopy(pItem, pEvt, pExt);
                pRef->pItem = pItem;
                retVal = (int32_t) index;
                break;
            }

            pItem = (EIT_SCHED_S *) (void *) LINKLST_GetNext(&(pSched->used),
                                                             (LINKLST_HDR_S
                                                              *) (void *)
                                                             pItem);
            index++;
        }

        (void)TIME_GetSystemTime(&(pSched->hdr.lastAccessTime));
    }

    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);
    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Get a schedule entry by its index in the schedule list.
**
** This function also initialises an internal iterator. Until the next call of
** either EIT_GetSchedByIndex or EIT_GetSchedByDate, navigation through the
** schedule list is possible via EIT_GetSchedNext and EIT_GetSchedPrevious.
**
** \param  pServLoc   Service, identified by service list reference,
**                    set to NULL for EIT information of current playback.
** \param  pEvt       EIT event data (NULL if not needed).
** \param  pExt       Parameters for extended event description, set to
**                    NULL if not needed.
** \param  pRef       Iterator reference (filled by function).
** \param  index      Requested list index (0...n).
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**         - #SMARTGO_SYS_ERR_DATA_NOT_AVL if requested event is not available
**
** \sa EIT_GetSchedByDate, EIT_GetSchedNext, EIT_GetSchedPrevious
******************************************************************************/
int32_t EIT_GetSchedByIndex(const SYS_SERVLOC_S * pServLoc,
                            EIT_EVT_S * pEvt,
                            EIT_EXTEVT_S * pExt,
                            EIT_POS_S * pRef, uint32_t index)
{
    int32_t             retVal = FAPI_OK;
    EIT_SCHED_ENTRY_S  *pSched = NULL;
    EIT_SCHED_S        *pItem;

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    pRef->pItem = NULL;

    /* get database entry */
    if(pServLoc != NULL)
    {
        pRef->servIdx = SLIST_GetServiceIndex(pServLoc);
    }

    #ifdef APPL_PVR_ENABLED

        else
        {
            pRef->servIdx = SLIST_SERVIDX_PLAY;
        }
    #else

        else
        {
            return APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
    #endif /* APPL_PVR_ENABLED */

    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    pSched = EIT_GetEntrySched(pRef->servIdx);

    if(pSched == NULL)
        retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
    else if(index >= LINKLST_GetListSize(&(pSched->used)))
    {
        retVal = SMARTGO_SYS_ERR_INTERNAL;
    }
    else
    {
        pItem =
            (EIT_SCHED_S *) (void *) LINKLST_GetByIndex(&(pSched->used), index);

        if(pItem == NULL)
            retVal = SMARTGO_SYS_ERR_INTERNAL;
        else
        {
            EIT_SchedListCopy(pItem, pEvt, pExt);
            pRef->pItem = pItem;
        }

        (void)TIME_GetSystemTime(&(pSched->hdr.lastAccessTime));
    }

    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);
    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Get next schedule list entry starting from last position.
**
** Before using this function, EIT_GetSchedByIndex or EIT_GetSchedByDate have to
** be called at least once in order to get a valid list position.
**
** \param  pEvt       EIT event data (NULL if not needed).
** \param  pExt       Parameters for extended event description, set to
**                    NULL if not needed.
** \param  pRef       Iterator reference from previous call of
**                    EIT_GetSchedXXX().
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**         - #SMARTGO_SYS_ERR_DATA_NOT_AVL if requested event is not available
**
** \sa EIT_GetSchedPrevious
******************************************************************************/
int32_t EIT_GetSchedNext(EIT_EVT_S * pEvt,
                         EIT_EXTEVT_S * pExt, EIT_POS_S * pRef)
{
    int32_t             retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
    EIT_SCHED_ENTRY_S  *pSched;
    EIT_SCHED_S        *pTemp;

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(pRef == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    /* get database entry */
    pSched = EIT_GetEntrySched(pRef->servIdx);

    if(pSched != NULL)
    {
        if(pRef->pItem == NULL)
        {
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
        else
        {
            pTemp = (EIT_SCHED_S *) (void *) LINKLST_GetNext(&(pSched->used),
                                                             (LINKLST_HDR_S
                                                              *) (void *) pRef->
                                                             pItem);

            if(pTemp == NULL)
                retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
            else
            {
                pRef->pItem = pTemp;

                EIT_SchedListCopy(pRef->pItem, pEvt, pExt);

                retVal = FAPI_OK;
            }
        }

        (void)TIME_GetSystemTime(&(pSched->hdr.lastAccessTime));
    }

    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);
    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Get previous schedule list entry starting from last position.
**
** Before using this function, EIT_GetSchedByIndex or EIT_GetSchedByDate have to
** be called at least once in order to get a valid list position.
**
** \param  pEvt       EIT event data (NULL if not needed).
** \param  pExt       Parameters for extended event description, set to
**                    NULL if not needed.
** \param  pRef       Iterator reference from previous call of
**                    EIT_GetSchedXXX().
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**         - #SMARTGO_SYS_ERR_DATA_NOT_AVL if requested event is not available
**
** \sa EIT_GetSchedByDate EIT_GetSchedByIndex
******************************************************************************/
int32_t EIT_GetSchedPrevious(EIT_EVT_S * pEvt,
                             EIT_EXTEVT_S * pExt, EIT_POS_S * pRef)
{
    int32_t             retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    EIT_SCHED_ENTRY_S  *pSched;
    EIT_SCHED_S        *pTemp;

    if(pRef == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    /* get database entry */
    pSched = EIT_GetEntrySched(pRef->servIdx);

    if(pSched != NULL)
    {
        if(pRef->pItem == NULL)
        {
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
        else
        {
            pTemp =
                (EIT_SCHED_S *) (void *) LINKLST_GetPrevious(&(pSched->used),
                                                             (LINKLST_HDR_S
                                                              *) (void *) pRef->
                                                             pItem);

            if(pTemp == NULL)
                retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
            else
            {
                pRef->pItem = pTemp;

                EIT_SchedListCopy(pRef->pItem, pEvt, pExt);

                retVal = FAPI_OK;
            }
        }

        (void)TIME_GetSystemTime(&(pSched->hdr.lastAccessTime));
    }

    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);
    return retVal;
}


/*!
*******************************************************************************
**
** \brief  Get schedule list entry by position relative to last reference.
**
** Before using this function, EIT_GetSchedByIndex or EIT_GetSchedByDate have to
** be called at least once in order to get a valid list position.
**
** \param  offset     list offset in positive or negative direction.
** \param  pEvt       EIT event data (NULL if not needed).
** \param  pExt       Parameters for extended event description, set to
**                    NULL if not needed.
** \param  pRef       Iterator reference from previous call of
**                    EIT_GetSchedXXX().
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**         - #SMARTGO_SYS_ERR_DATA_NOT_AVL if requested event is not available
**
** \sa EIT_GetSchedNext EIT_GetSchedPrevious
******************************************************************************/
int32_t EIT_GetSchedByReference(int16_t offset, EIT_EVT_S * pEvt,
                                EIT_EXTEVT_S * pExt, EIT_POS_S * pRef)
{
    int32_t             retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    EIT_SCHED_S        *pTemp = NULL;
    EIT_SCHED_ENTRY_S  *pSched;

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(pRef == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    /* get database entry */
    pSched = EIT_GetEntrySched(pRef->servIdx);

    if(pSched != NULL)
    {
        if(pRef->pItem == NULL)
        {
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
        else
        {
            if(offset == 0)
            {
                pTemp = pRef->pItem;
            }
            else if(offset > 0)
            {
                while((offset--) != 0)
                {
                    pTemp =
                        (EIT_SCHED_S *) (void *)
                        LINKLST_GetNext(&(pSched->used),
                                        (LINKLST_HDR_S *) (void *) pRef->pItem);

                    if(pTemp == NULL)
                        break;
                    else
                        pRef->pItem = pTemp;
                }
            }
            else
            {
                while((offset++) != 0)
                {
                    pTemp =
                        (EIT_SCHED_S *) (void *)
                        LINKLST_GetPrevious(&(pSched->used),
                                            (LINKLST_HDR_S *) (void *) pRef->
                                            pItem);

                    if(pTemp == NULL)
                        break;
                    else
                        pRef->pItem = pTemp;
                }
            }

            if(pTemp == NULL)
                retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
            else
            {
                pRef->pItem = pTemp;

                EIT_SchedListCopy(pRef->pItem, pEvt, pExt);

                retVal = FAPI_OK;
            }
        }

        (void)TIME_GetSystemTime(&(pSched->hdr.lastAccessTime));
    }

    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);
    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Triggers new aqcuisition of present/following events for given
**         service.
**
** Current EIT data remains valid until the next section is extracted from
** stream.
**
** \param  pServLoc   Service, identified by service list reference,
**                    set to NULL for EIT information of current playback.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t EIT_RequestNewEvents(const SYS_SERVLOC_S * pServLoc)
{
    EIT_CURRNEXT_ENTRY_S *pEntry = NULL;
    SLIST_SERVIDX       servIdx = SLIST_SERVIDX_NONE;
    int32_t             retVal;

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(pServLoc != NULL)
    {
        servIdx = SLIST_GetServiceIndex(pServLoc);
    }

    #ifdef APPL_PVR_ENABLED

        else
        {
            servIdx = SLIST_SERVIDX_PLAY;
        }
    #else

        else
        {
            return APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
    #endif /* APPL_PVR_ENABLED */

    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    pEntry = EIT_GetEntryCurrNext(servIdx);

    if(pEntry == NULL)
    {
        retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }
    else
    {
        memset(&(pEntry->sectVer), 0, sizeof(EIT_TABLEREV_S));
        pEntry->sectVer.active = EIT_VERSION_INVALID;
        retVal = FAPI_OK;
    }

    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Remove outdated schedule events from all database entries.
**
** An application that uses the EIT schedule database should call this function
** daily in order to remove outdated events from all database entries. All
** entries starting before today (according to local time setting) will be
** removed.
**
** Because the database has a fixed number of event per entry, using this
** function is mandatory in order to avoid new events from being refused due
** to the static database capacity.
**
** \return -
******************************************************************************/
void EIT_CleanupSched(void)
{
    fbool_t             changed = FFALSE;
    EIT_SCHED_ENTRY_S  *pEntry = eitDat->sched;
    uint32_t            cnt;
    SYS_EVT_EITEVT_S    ctrlEvent;
    EIT_SCHED_S        *pItem;
    EIT_SCHED_S        *pTemp;
    SYS_TIME_S          timeCalc;
    SYS_DATE_S          dateCalc;
    int16_t             utcOffset = 0;
    fbool_t             dstIsActive = FFALSE;
    fbool_t             hasSynced;

    if(eitDat == NULL)
    {
        return;
    }
    ctrlEvent.hdr.type = EVT_EIT_SCHED_UPDATE;
    ctrlEvent.hdr.length = sizeof(ctrlEvent);

    (void)TIME_TimeAndDateGetUtc(&timeCalc, &dateCalc, &hasSynced);

    if ( !hasSynced )
    {
        return;
    }

    (void)SETTINGS_TimeOffsetGet(&utcOffset, &dstIsActive);
    if(dstIsActive)
        utcOffset += 60;

    memset(&timeCalc, 0, sizeof(SYS_TIME_S));

    TIME_AddSeconds(&timeCalc, &dateCalc, -((int32_t) utcOffset * 60));

    for(cnt = 0; cnt < AV_EIT_SCHED_SERV_MAX; cnt++, pEntry++)
    {
        if(pEntry->hdr.servIdx < 0)
            continue;

        (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

        pItem = (EIT_SCHED_S *) (void *) LINKLST_GetFirst(&(pEntry->used));

        while(pItem != NULL)
        {
            pTemp = (EIT_SCHED_S *) (void *) LINKLST_GetNext(&(pEntry->used),
                                                             (LINKLST_HDR_S
                                                              *) (void *)
                                                             pItem);

            if(TIME_IsLaterThan(&timeCalc, &dateCalc,
                                &(pItem->evt.startTime),
                                &(pItem->evt.startDate)) == FTRUE)
            {
                /* Move from "used" to "free" list */
                (void)LINKLST_Remove(&(pEntry->used),
                               (LINKLST_HDR_S *) (void *) pItem);
                (void)LINKLST_PushBack(&(pEntry->free),
                                 (LINKLST_HDR_S *) (void *) pItem);
                changed = FTRUE;
            }
            else
            {
                break;
            }

            pItem = pTemp;
        }

        (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);

        if(changed)
        {
            /* generate notification event */
            ctrlEvent.servIdx = pEntry->hdr.servIdx;

            (void)EVT_Send(&ctrlEvent, "EIT sched update");
        }
    }
}

/*!
*******************************************************************************
**
** \brief  Remove a service from database.
**
** \param  servIdx    Service index to be removed.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**         - #APPL_SMARTGO_ERR_BAD_PARAMETER if service index is invalid.
******************************************************************************/
int32_t EIT_InvalidateService(SLIST_SERVIDX servIdx)
{
    EIT_SCHED_ENTRY_S  *pSched = eitDat->sched;
    EIT_CURRNEXT_ENTRY_S *pCurrNext = eitDat->currNext;
    uint32_t            cnt;

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(servIdx < 0)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    for(cnt = 0; cnt < AV_EIT_CURRNEXT_SERV_MAX; cnt++, pCurrNext++)
    {
        if(pCurrNext->hdr.servIdx == servIdx)
        {
            EIT_CurrNextEntryFree(cnt);
            break;
        }
    }

    for(cnt = 0; cnt < AV_EIT_SCHED_SERV_MAX; cnt++, pSched++)
    {
        if(pSched->hdr.servIdx == servIdx)
        {
            EIT_SchedEntryFree(cnt);
            break;
        }
    }

    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);

    // to do: send notification if database entries have been removed!

    return FAPI_OK;
}

int32_t EIT_AddFocusCurrNext(const SYS_SERVLOC_S * pServLoc)
{
    EIT_CURRNEXT_ENTRY_S *pEntry;
    SLIST_SERVIDX       servIdx;
    int32_t             retVal;

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(pServLoc == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    servIdx = SLIST_GetServiceIndex(pServLoc);
    if(servIdx < 0)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    pEntry = EIT_GetEntryCurrNext(servIdx);     /* already in database? */

    if(pEntry == NULL)
    {
        pEntry = EIT_GetFreeEntryCurrNext(servIdx);     /* get free entry */
    }
    if(pEntry != NULL)
    {
        pEntry->hdr.update |= EIT_UPDATE_GNR;
        retVal = FAPI_OK;
    }
    else
    {
        retVal = SMARTGO_SYS_ERR_CAPACITY;
    }
    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t EIT_AddFocusSched(const SYS_SERVLOC_S * pServLoc)
{
    EIT_SCHED_ENTRY_S  *pEntry;
    SLIST_SERVIDX       servIdx;
    int32_t             retVal;

    if(eitDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(pServLoc == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    servIdx = SLIST_GetServiceIndex(pServLoc);
    if(servIdx < 0)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    pEntry = EIT_GetEntrySched(servIdx);        /* already in database? */

    if(pEntry == NULL)
    {
        pEntry = EIT_GetFreeEntrySched(servIdx);        /* get free entry */
    }
    if(pEntry != NULL)
    {
        pEntry->hdr.update |= EIT_UPDATE_GNR;
        retVal = FAPI_OK;
    }
    else
    {
        retVal = SMARTGO_SYS_ERR_CAPACITY;
    }
    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t EIT_RemoveFocusCurrNextAll(void)
{
    EIT_CURRNEXT_ENTRY_S *pEntry = eitDat->currNext;
    uint32_t            cnt;

    for(cnt = 0; cnt < AV_EIT_CURRNEXT_SERV_MAX; cnt++)
    {
        pEntry->hdr.update &= ~(EIT_UPDATE_GNR);
        pEntry++;
    }

    return FAPI_OK;
}

int32_t EIT_RemoveFocusSchedAll(void)
{
    EIT_SCHED_ENTRY_S  *pEntry = eitDat->sched;
    uint32_t            cnt;

    for(cnt = 0; cnt < AV_EIT_SCHED_SERV_MAX; cnt++)
    {
        pEntry->hdr.update &= ~(EIT_UPDATE_GNR);
        pEntry++;
    }

    return FAPI_OK;
}

#ifdef APPL_PVR_ENABLED

/*!
*******************************************************************************
**
** \brief  Clear playback entry in EIT database.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
FSTATIC int32_t EIT_PlayClear(void)
{
    EIT_CURRNEXT_ENTRY_S *pCurrNext = eitDat->currNext + EIT_CURRNEXT_IDX_PLAY;
    EIT_SCHED_ENTRY_S  *pSched = eitDat->sched + EIT_SCHED_IDX_PLAY;

    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    EIT_CurrNextEntryClear(pCurrNext);
    EIT_SchedEntryClear(pSched);

    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);

    return FAPI_OK;
}

#endif /* APPL_PVR_ENABLED */

/* ------------------------------------------------------------------------- */

FSTATIC void EIT_CurrNextEntryClear(EIT_CURRNEXT_ENTRY_S * pEntry)
{
    pEntry->curr.availability = 0;
    pEntry->curr.ext[0] = 0;

    memset(&(pEntry->curr.event), 0, sizeof(EIT_EVT_S));

    pEntry->next.availability = 0;
    pEntry->next.ext[0] = 0;

    memset(&(pEntry->next.event), 0, sizeof(EIT_EVT_S));

    memset(&(pEntry->sectVer), 0, sizeof(EIT_TABLEREV_S));
    pEntry->sectVer.active = EIT_VERSION_INVALID;
}

FSTATIC void EIT_SchedEntryClear(EIT_SCHED_ENTRY_S * pEntry)
{
    uint32_t            cnt;

    (void)LINKLST_Init(&(pEntry->free), AV_SCHED_ENTRIES_MAX, sizeof(EIT_SCHED_S),
                 pEntry->lst);
    (void)LINKLST_Init(&(pEntry->used), 0, 0, NULL);

    /* clear version info and section availability */
    for(cnt = 0; cnt < EIT_SCHED_TABLE_MAX; cnt++)
    {
        memset(pEntry->sectVerSched + cnt, 0, sizeof(EIT_TABLEREV_S));
        pEntry->sectVerSched[cnt].active = EIT_VERSION_INVALID;
    }

    memset(&(pEntry->sectVerPf), 0, sizeof(EIT_TABLEREV_S));
    pEntry->sectVerPf.active = EIT_VERSION_INVALID;
}

FSTATIC void EIT_OccupyEntry(EIT_ENTRY_HDR_S * pHdr, SLIST_SERVIDX servIdx)
{
    SLIST_ITEM_S        servItem;
    TPLIST_ITEM_S       carrierItem;

    (void)TIME_GetSystemTime(&(pHdr->lastAccessTime));

    pHdr->servIdx = servIdx;
    pHdr->update = 0;
    pHdr->service_id = 0;
    pHdr->ts_id = 0;

    if(SLIST_GetServByIndex(servIdx, &servItem) == FAPI_OK)
    {
        pHdr->service_id = servItem.serviceId;

        if(TPLIST_GetDataByIndex(&(servItem.network),
                                 servItem.tpIdx, &carrierItem) == FAPI_OK)
        {
            if(carrierItem.tsId >= 0)
            {
                pHdr->ts_id = (uint16_t) carrierItem.tsId;
            }
        }
    }
}

FSTATIC void EIT_CurrNextEntryFree(uint16_t index)
{
    EIT_FreeEntry(&(eitDat->currNext[index].hdr));

    if(eitDat->currNextFirstFree > index)
    {
        eitDat->currNextFirstFree = index;
    }

    /* Remove focus if necessary */
    if(eitDat->focusCurrNext == (eitDat->currNext + index))
    {
        eitDat->focusCurrNext = NULL;
    }
}

FSTATIC void EIT_SchedEntryFree(uint16_t index)
{
    EIT_FreeEntry(&(eitDat->sched[index].hdr));

    if(eitDat->schedFirstFree > index)
    {
        eitDat->schedFirstFree = index;
    }

    /* Remove focus if necessary */
    if(eitDat->focusSched == (eitDat->sched + index))
    {
        eitDat->focusSched = NULL;
    }
}

FSTATIC void EIT_FreeEntry(EIT_ENTRY_HDR_S * pHdr)
{
    pHdr->servIdx = SLIST_SERVIDX_NONE;
    pHdr->update = 0;
    pHdr->lastAccessTime = 0;
    pHdr->service_id = 0;
    pHdr->ts_id = 0;
}

FSTATIC int32_t EIT_SchedListInsert(EIT_SCHED_ENTRY_S * pEntry,
                                    const EIT_SCHED_S * pSchedItem)
{
    int32_t             retVal = SMARTGO_SYS_ERR_INTERNAL;
    fbool_t             inserted = FFALSE;
    fbool_t             removeCurr;
    EIT_SCHED_S        *pNew;
    EIT_SCHED_S        *pItem;
    EIT_SCHED_S        *pTemp;
    SYS_TIME_S          tempTime;
    SYS_DATE_S          tempDate;
    uint32_t            durInSecs;

    if ( pEntry == NULL )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    /* get entry from "free" list */
    pNew = (EIT_SCHED_S*)(void*)LINKLST_GetFirst(&(pEntry->free));

    if ( pNew == NULL ) /* no more free entries available */
    {
        FAPI_SYS_PRINT_DEBUG(2,"[EIT] [SCH] Event entries for service exhausted!\n");

        /* try to get last entry from used list instead */
        pNew = (EIT_SCHED_S*)(void*)LINKLST_GetLast(&(pEntry->used));

        if ( pNew != NULL )
        {
            /* remove entry from list */
            (void)LINKLST_Remove(&(pEntry->used), (LINKLST_HDR_S*)(void*)pNew);
        }
    }
    else
    {
        /* remove entry from list */
        (void)LINKLST_Remove(&(pEntry->free), (LINKLST_HDR_S*)(void*)pNew);
    }

    if ( pNew == NULL ) /* no more entries available */
    {
        retVal = SMARTGO_SYS_ERR_CAPACITY;
    }
    else
    {
        (*pNew) = (*pSchedItem);

        pItem = (EIT_SCHED_S*)(void*)LINKLST_GetLast(&(pEntry->used));

        while ( pItem != NULL )
        {
            removeCurr = FFALSE;

            if(!inserted)
            {
                /* test for time and date to keep chronological order */
                if(TIME_IsLaterThan(&(pSchedItem->evt.startTime),
                                    &(pSchedItem->evt.startDate),
                                    &(pItem->evt.startTime),
                                    &(pItem->evt.startDate)) == FTRUE)
                {
                    tempTime = pItem->evt.startTime;
                    tempDate = pItem->evt.startDate;

                    /* calculate end time of reference event */
                    durInSecs = TIME_ConvToSeconds(&(pItem->evt.duration));
                    TIME_AddSeconds(&tempTime, &tempDate, (int32_t)durInSecs);

                    /* If events overlap, replace existing event. If not,
                       insert new event after current entry. */

                    if(TIME_IsLaterThan(&tempTime, &tempDate,
                                        &(pSchedItem->evt.startTime),
                                        &(pSchedItem->evt.startDate)) == FTRUE)
                    {
                        removeCurr = FTRUE;
                    }

                    /* insert after current entry */
                    retVal = LINKLST_Insert(&(pEntry->used),
                                            (LINKLST_HDR_S *) (void *) pNew,
                                            (LINKLST_HDR_S *) (void *) pItem,
                                            FFALSE);

                    inserted = FTRUE;
                }
            }

            pTemp = pItem;
            pItem = (EIT_SCHED_S*)(void*)LINKLST_GetPrevious(&(pEntry->used),
                                                            (LINKLST_HDR_S*)
                                                            (void*)pItem);

            /* remove duplicate or colliding events */
            if ( removeCurr || (pTemp->evt.id == pSchedItem->evt.id) )
            {
                (void)LINKLST_Remove(&(pEntry->used),
                               (LINKLST_HDR_S *) (void *) pTemp);
                (void)LINKLST_PushBack(&(pEntry->free),
                                 (LINKLST_HDR_S *) (void *) pTemp);
            }
        }

        /* Insert as first entry if not yet inserted */
        if ( !inserted )
        {
            /* insert as first entry */
            retVal = LINKLST_PushFront (&(pEntry->used),
                                       (LINKLST_HDR_S*)(void*)pNew);
        }

        /* Remove all following events colliding chronologically with inserted
           event. */

        if(retVal == FAPI_OK)
        {
            tempTime = pSchedItem->evt.startTime;
            tempDate = pSchedItem->evt.startDate;

            /* calculate end time of inserted event */
            durInSecs = TIME_ConvToSeconds(&(pSchedItem->evt.duration));
            TIME_AddSeconds(&tempTime, &tempDate, (int32_t)durInSecs);

            pItem = (EIT_SCHED_S *) (void *) LINKLST_GetNext(&(pEntry->used),
                                                             (LINKLST_HDR_S
                                                              *) (void *) pNew);

            while(pItem != NULL)
            {
                pTemp = pItem;
                pItem =
                    (EIT_SCHED_S *) (void *) LINKLST_GetNext(&(pEntry->used),
                                                             (LINKLST_HDR_S
                                                              *) (void *)
                                                             pItem);

                if(TIME_IsLaterThan(&tempTime, &tempDate,
                                    &(pTemp->evt.startTime),
                                    &(pTemp->evt.startDate))== FTRUE)
                {
                    (void)LINKLST_Remove(&(pEntry->used),
                                   (LINKLST_HDR_S *) (void *) pTemp);
                    (void)LINKLST_PushBack(&(pEntry->free),
                                     (LINKLST_HDR_S *) (void *) pTemp);
                }
                else
                {
                    break;
                }
            }
        }
    }

    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);
    return retVal;
}


FSTATIC /* FINLINE */ void EIT_SchedListCopy(const EIT_SCHED_S * pItem,
                                       EIT_EVT_S * pEvt, EIT_EXTEVT_S * pExt)
{
    if(pEvt != NULL)
        *pEvt = pItem->evt;

    if(pExt != NULL)
    {
        /* no target buffer available */
        if((pExt->pBuf == NULL) || (pExt->maxLen == 0))
        {
            STR_Clear(&(pExt->descStr));
            pExt->descStrLen = 0;
        }

        #if ( AV_SCHED_STORE_EXT != 0 )

        else
        {
            pExt->descStrLen = STR_Copy (STR_Build(pItem->extCharSet,
                                                   pItem->pExt),
                                         pExt->pBuf,
                                         &(pExt->descStr.cs),
                                         pItem->extCharSet,
                                         pExt->maxLen);

            pExt->descStr.chr = pExt->pBuf;
        }

        #else

        else
        {
            STR_Clear(&(pExt->descStr));
            pExt->descStrLen = 0;
        }

        #endif /* AV_SCHED_STORE_EXT != 0 */
    }
}

/* Removes all entries originating from a specified tableId. This is used to
   remove outdated entries in case of a version update of an EIT table. */
FSTATIC int32_t EIT_SchedListRemoveTable(EIT_SCHED_ENTRY_S * pEntry,
                                         uint8_t tableId)
{
    int32_t             retVal = FAPI_OK;
    EIT_SCHED_S        *pItem;
    EIT_SCHED_S        *pTemp;

    if(pEntry == NULL)
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    pItem = (EIT_SCHED_S *) (void *) LINKLST_GetFirst(&(pEntry->used));

    while(pItem != NULL)
    {
        pTemp = (EIT_SCHED_S *) (void *) LINKLST_GetNext(&(pEntry->used),
                                                         (LINKLST_HDR_S *) (void
                                                                            *)
                                                         pItem);

        if(pItem->tableId == tableId)
        {
            /* Move from "used" to "free" list */
            (void)LINKLST_Remove(&(pEntry->used), (LINKLST_HDR_S *) (void *) pItem);
            (void)LINKLST_PushBack(&(pEntry->free), (LINKLST_HDR_S *) (void *) pItem);
        }

        pItem = pTemp;
    }

    // to do: send notification if database entries have been removed!

    return retVal;
}

/* ------------------------------------------------------------------------- */

FSTATIC void EIT_SetCurrService(SLIST_SERVIDX servIdx)
{
    (void)FAPEX_SYS_GetSemaphore(eitDat->evtMutex, FAPEX_SYS_SUSPEND);

    /* ----- clear update flag of currently selected "current" service ----- */

    if(eitDat->focusCurrNext != NULL)
    {
        eitDat->focusCurrNext->hdr.update &= ~(EIT_UPDATE_CURR);
    }

    if(eitDat->focusSched != NULL)
    {
        eitDat->focusSched->hdr.update &= ~(EIT_UPDATE_CURR);
    }

    /* ----- set new "current" service ------------------------------------- */

    if(servIdx == SLIST_SERVIDX_NONE)
    {
        eitDat->focusCurrNext = NULL;
        eitDat->focusSched = NULL;
    }
    else
    {
        /* test if service already available in database */
        eitDat->focusCurrNext = EIT_GetEntryCurrNext(servIdx);
        eitDat->focusSched = EIT_GetEntrySched(servIdx);

        #ifdef APPL_PVR_ENABLED
        if(servIdx != SLIST_SERVIDX_PLAY)
        {
        #endif /* APPL_PVR_ENABLED */

            /* if not in database, prepare free entry */
            if(eitDat->focusCurrNext == NULL)
            {
                eitDat->focusCurrNext = EIT_GetFreeEntryCurrNext(servIdx);
            }

            if(eitDat->focusSched == NULL)
            {
                eitDat->focusSched = EIT_GetFreeEntrySched(servIdx);
            }

        #ifdef APPL_PVR_ENABLED
        }
        #endif /* APPL_PVR_ENABLED */

        if(eitDat->focusCurrNext != NULL)
        {
            eitDat->focusCurrNext->hdr.update |= EIT_UPDATE_CURR;
        }

        if(eitDat->focusSched != NULL)
        {
            eitDat->focusSched->hdr.update |= EIT_UPDATE_CURR;
        }
    }

    (void)FAPEX_SYS_SetSemaphore(eitDat->evtMutex, FAPEX_SYS_NO_SUSPEND);
}


/* searches for an entry to match given service index */
FSTATIC EIT_CURRNEXT_ENTRY_S *EIT_GetEntryCurrNext(SLIST_SERVIDX servIdx)
{
    uint32_t            cnt;
    EIT_CURRNEXT_ENTRY_S *pEntry = eitDat->currNext;
    EIT_CURRNEXT_ENTRY_S *retVal = NULL;

    switch (servIdx)
    {
        case -1:
            /* invalid service */
            break;

        #ifdef APPL_PVR_ENABLED

        case SLIST_SERVIDX_PLAY:
            retVal = &(pEntry[EIT_CURRNEXT_IDX_PLAY]);
            break;

        #endif /* APPL_PVR_ENABLED */

        default:
            for(cnt = 0; cnt < AV_EIT_CURRNEXT_SERV_MAX; cnt++)
            {
                if(pEntry->hdr.servIdx == servIdx)
                {
                    retVal = pEntry;
                    break;
                }
                pEntry++;
            }
            break;
    }

    return retVal;
}

FSTATIC EIT_CURRNEXT_ENTRY_S *EIT_GetFreeEntryCurrNext(SLIST_SERVIDX servIdx)
{
    EIT_CURRNEXT_ENTRY_S *pEntry = NULL;
    int32_t             cnt;
    uint32_t            oldestTime = 0xFFFFFFFF;
    int32_t             oldestEntry = -1;

    /* NOTE: It is assumed that a database entry for requested service
       currently does not exist. */

    /* Attempt to get a free entry first */
    while(eitDat->currNextFirstFree < AV_EIT_CURRNEXT_SERV_MAX)
    {
        if(eitDat->currNext[eitDat->currNextFirstFree].hdr.servIdx < 0)
        {
            pEntry = eitDat->currNext + eitDat->currNextFirstFree;
            eitDat->currNextFirstFree++;
            break;
        }

        eitDat->currNextFirstFree++;
    }

    /* No free entry found: overwrite "oldest" entry */
    if(pEntry == NULL)
    {
        // to do: to be optimised

        for(cnt = 0; cnt < AV_EIT_CURRNEXT_SERV_MAX; cnt++)
        {
            if((eitDat->currNext[cnt].hdr.update == 0) &&
               (eitDat->currNext[cnt].hdr.lastAccessTime <= oldestTime))
            {
                oldestTime = eitDat->currNext[cnt].hdr.lastAccessTime;
                oldestEntry = cnt;
            }
        }

        if(oldestEntry >= 0)
        {
            pEntry = eitDat->currNext + oldestEntry;
        }
    }

    if(pEntry != NULL)
    {
        EIT_CurrNextEntryClear(pEntry);
        EIT_OccupyEntry(&(pEntry->hdr), servIdx);
    }

    return pEntry;
}

/* searches for an entry to match given service index */
FSTATIC EIT_SCHED_ENTRY_S *EIT_GetEntrySched(SLIST_SERVIDX servIdx)
{
    uint32_t            cnt;
    EIT_SCHED_ENTRY_S  *pEntry = eitDat->sched;
    EIT_SCHED_ENTRY_S  *retVal = NULL;

    switch (servIdx)
    {
        case -1:
            /* invalid service */
            break;

        #ifdef APPL_PVR_ENABLED

        case SLIST_SERVIDX_PLAY:
            retVal = &(pEntry[EIT_SCHED_IDX_PLAY]);
            break;

        #endif /* APPL_PVR_ENABLED */

        default:
            for(cnt = 0; cnt < AV_EIT_SCHED_SERV_MAX; cnt++)
            {
                if(pEntry->hdr.servIdx == servIdx)
                {
                    retVal = pEntry;
                    break;
                }
                pEntry++;
            }
            break;
    }

    return retVal;
}

FSTATIC EIT_SCHED_ENTRY_S *EIT_GetFreeEntrySched(SLIST_SERVIDX servIdx)
{
    EIT_SCHED_ENTRY_S  *pEntry = NULL;
    int32_t             cnt;
    uint32_t            oldestTime = 0xFFFFFFFF;
    int32_t             oldestEntry = -1;

    /* NOTE: It is assumed that a database entry for requested service
       currently does not exist. */

    /* Attempt to get a free entry first */
    while(eitDat->schedFirstFree < AV_EIT_SCHED_SERV_MAX)
    {
        if(eitDat->sched[eitDat->schedFirstFree].hdr.servIdx < 0)
        {
            pEntry = eitDat->sched + eitDat->schedFirstFree;
            eitDat->schedFirstFree++;
            break;
        }

        eitDat->schedFirstFree++;
    }

    /* No free entry found: overwrite "oldest" entry */
    if(pEntry == NULL)
    {
        // to do: to be optimised

        for(cnt = 0; cnt < AV_EIT_SCHED_SERV_MAX; cnt++)
        {
            if((eitDat->sched[cnt].hdr.update == 0) &&
               (eitDat->sched[cnt].hdr.lastAccessTime <= oldestTime))
            {
                oldestTime = eitDat->sched[cnt].hdr.lastAccessTime;
                oldestEntry = cnt;
            }
        }

        if(oldestEntry >= 0)
        {
            pEntry = eitDat->sched + oldestEntry;
        }
    }

    if(pEntry != NULL)
    {
        EIT_SchedEntryClear(pEntry);
        EIT_OccupyEntry(&(pEntry->hdr), servIdx);
    }

    return pEntry;
}

FSTATIC EIT_CURRNEXT_ENTRY_S *EIT_GetTargetEntryCurrNext(EIT_DATA_S * inst,
                                                         uint16_t serviceId,
                                                         uint16_t tsId)
{
    EIT_CURRNEXT_ENTRY_S *pCurrNext = NULL;
    uint32_t            cnt;

    #ifdef APPL_PVR_ENABLED

    if(inst->playMode)
    {
        if(inst->currNext[EIT_CURRNEXT_IDX_PLAY].hdr.service_id == serviceId)
        {
            pCurrNext = inst->currNext + EIT_CURRNEXT_IDX_PLAY;
        }
    }
    else
    #endif /* APPL_PVR_ENABLED */
    {
        pCurrNext = inst->currNext;

        for(cnt = 0; cnt < AV_EIT_CURRNEXT_SERV_MAX; cnt++)
        {
            if((pCurrNext->hdr.servIdx >= 0) &&
               (pCurrNext->hdr.update > 0) &&
               (pCurrNext->hdr.service_id == serviceId) &&
               (pCurrNext->hdr.ts_id == tsId))
            {
                break;
            }

            pCurrNext++;
        }

        if(cnt >= AV_EIT_CURRNEXT_SERV_MAX)
        {
            pCurrNext = NULL;
        }

    }

    return pCurrNext;
}

FSTATIC EIT_SCHED_ENTRY_S *EIT_GetTargetEntrySched(EIT_DATA_S * inst,
                                                   uint16_t serviceId,
                                                   uint16_t tsId)
{
    EIT_SCHED_ENTRY_S  *pSched = NULL;
    uint32_t            cnt;

    #ifdef APPL_PVR_ENABLED

    if(inst->playMode)
    {
        if(inst->sched[EIT_SCHED_IDX_PLAY].hdr.service_id == serviceId)
        {
            pSched = inst->sched + EIT_SCHED_IDX_PLAY;
        }
    }
    else
    {

    #endif /* APPL_PVR_ENABLED */

        pSched = inst->sched;

        for(cnt = 0; cnt < AV_EIT_SCHED_SERV_MAX; cnt++)
        {
            if((pSched->hdr.servIdx >= 0) &&
               (pSched->hdr.update > 0) &&
               (pSched->hdr.service_id == serviceId) &&
               (pSched->hdr.ts_id == tsId))
            {
                break;
            }

            pSched++;
        }

        if(cnt >= AV_EIT_SCHED_SERV_MAX)
            pSched = NULL;

    #ifdef APPL_PVR_ENABLED
    }
    #endif /* APPL_PVR_ENABLED */

    return pSched;
}

/* ------------------------------------------------------------------------- */
/*lint -save -e715 */
/* parameter "FAPI_SYS_HandleT handle" not needed in this function */
FSTATIC uint8_t EIT_SectPfFiltered(FAPI_SYS_HandleT handle,
                                   const volatile uint8_t* pSectStart,
                                   const volatile uint8_t* pSectEnd,
                                   const volatile uint8_t* pSect,
                                   uint16_t length, void* pPrivateData)
{
    EIT_DATA_S*         inst = (EIT_DATA_S *) pPrivateData;
    EIT_CURRNEXT_ENTRY_S* pCurrNext;
    EIT_SCHED_ENTRY_S*  pSched;
    uint8_t             tableId;
    uint8_t             version;
    uint8_t             sectNum;
    uint16_t            serviceId;
    uint16_t            tsId;
    uint8_t             verFlag;
    uint8_t*            sectPtr;
    uint32_t            bufSize;

    if ( length < EIT_SECTLEN_MIN ) return 0;

    sectPtr    = (uint8_t*)pSect;
    bufSize    = (uint32_t)(pSectEnd - pSectStart + 1);
    tableId    = *sectPtr;

    sectPtr   += 3;
    if ( sectPtr >= pSectEnd )
    {
        sectPtr -= bufSize;
    }

    serviceId  = LD16(sectPtr);

    sectPtr   += 2;
    if ( sectPtr >= pSectEnd )
    {
        sectPtr -= bufSize;
    }

    version    = ((*sectPtr & 0x3E) >> 1);

    sectPtr   += 1;
    if ( sectPtr >= pSectEnd )
    {
        sectPtr -= bufSize;
    }
    sectNum    = *sectPtr;

    sectPtr   += 2;
    if ( sectPtr >= pSectEnd )
    {
        sectPtr -= bufSize;
    }

    tsId       = LD16(sectPtr);
    //FAPI_SYS_PRINT_DEBUG(4,"[EIT] [PF]  Section: tID: %X tsId %04X servId %04X sec %3i ver %2i\n", tableId, tsId, serviceId, sectNum, version);

    /* According to ETSI guidelines, only section 0 and 1 provide
       present/following information */
    if ( sectNum > 1 )
    {
        return 0;
    }

    /* --------------------------------------------------------------------- */

    if ( tableId == EIT_TABLEID_PF_ACTUALTS ) /* present/following, actual TS */
    {
        verFlag = 0;
    }
    else if ( tableId == EIT_TABLEID_PF_OTHERTS ) /* present/following, other TS */
    {
        verFlag = EIT_VERSION_OTHER;
    }
    else
    {
        /* invalid table ID */
        return 0;
    }

    version |= verFlag;

    /* --------------------------------------------------------------------- */

    pCurrNext = EIT_GetTargetEntryCurrNext (inst, serviceId, tsId);

    if ( pCurrNext != NULL )
    {
        if ( (version != pCurrNext->sectVer.active) ||
             (!PSISI_SectionAvailable(pCurrNext->sectVer.sectFound, sectNum)) )
        {
            return 1;
        }
    }

    /* --------------------------------------------------------------------- */

    pSched = EIT_GetTargetEntrySched (inst, serviceId, tsId);

    if ( pSched != NULL )
    {
        if ( (version != pSched->sectVerPf.active) ||
             (!PSISI_SectionAvailable(pSched->sectVerPf.sectFound, sectNum)) )
        {
            return 1;
        }
    }

    return 0;
}
/*lint -restore */


/*lint -save -e715 */
/* parameter "FAPI_SYS_HandleT handle" not needed in this function */
FSTATIC uint8_t EIT_SectSchedFiltered(FAPI_SYS_HandleT handle,
                                      const volatile uint8_t* pSectStart,
                                      const volatile uint8_t* pSectEnd,
                                      const volatile uint8_t* pSect,
                                      uint16_t length, void* pPrivateData)
{
    EIT_DATA_S*         inst = (EIT_DATA_S *) pPrivateData;
    EIT_SCHED_ENTRY_S*  pSched;
    EIT_TABLEREV_S*     pVersion;
    uint8_t             tableId;
    uint8_t             version;
    uint8_t             sectNum;
    uint16_t            serviceId;
    uint16_t            tsId;
    uint8_t             verFlag;
    uint8_t*            sectPtr;
    uint32_t            bufSize;

    if ( length < EIT_SECTLEN_MIN ) return 0;

    sectPtr    = (uint8_t*)pSect;
    bufSize    = (uint32_t)(pSectEnd - pSectStart + 1);
    tableId    = *sectPtr;

    sectPtr   += 3;
    if ( sectPtr >= pSectEnd )
    {
        sectPtr -= bufSize;
    }

    serviceId  = LD16(sectPtr);

    sectPtr   += 2;
    if ( sectPtr >= pSectEnd )
    {
        sectPtr -= bufSize;
    }

    version    = ((*sectPtr & 0x3E) >> 1);

    sectPtr   += 1;
    if ( sectPtr >= pSectEnd )
    {
        sectPtr -= bufSize;
    }
    sectNum    = *sectPtr;

    sectPtr   += 2;
    if ( sectPtr >= pSectEnd )
    {
        sectPtr -= bufSize;
    }
    tsId       = LD16(sectPtr);

    /* --------------------------------------------------------------------- */

    if ( (tableId & 0xF0) == 0x50 )    /* schedule, actual TS */
    {
        verFlag = 0;
    }
    else if ( (tableId & 0xF0) == 0x60 )    /* schedule, other TS */
    {
        verFlag = EIT_VERSION_OTHER;
    }
    else
    {
        /* invalid table ID */
        return 0;
    }

    version |= verFlag;

    /* --------------------------------------------------------------------- */

    pSched = EIT_GetTargetEntrySched (inst, serviceId, tsId);

    if ( pSched != NULL )
    {
        pVersion = &(pSched->sectVerSched[tableId & 0x0F]);

        if ( (version != pVersion->active) ||
             (!PSISI_SectionAvailable(pVersion->sectFound, sectNum)) )
        {
            return 1;
        }
    }

    return 0;
}
/*lint -restore */


/*lint -save -e715 */
/* parameter "FAPI_SYS_HandleT handle" not needed in this function */
FSTATIC void EIT_SectPfNotify(FAPI_SYS_HandleT handle,
                              SFAL_SECTION_STATUS_E   sectionStatus,
                              const uint8_t * pSect, uint16_t length,
                              void *pPrivateData)
{
    EIT_DATA_S         *inst = (EIT_DATA_S *) pPrivateData;
    EIT_CURRNEXT_ENTRY_S *pCurrNext;
    EIT_CURRNEXT_EVT_S *pEvt = NULL;
    EIT_SCHED_ENTRY_S  *pSched;
    SYS_EVT_EITEVT_S    ctrlEvent;
    uint8_t             tableId;
    uint8_t             version;
    uint8_t             sectNum;
    uint16_t            serviceId;
    uint16_t            tsId;
    uint8_t             verFlag;
    fbool_t             newData = FFALSE;

    if ( sectionStatus != SFAL_SECTION_AVAILABLE )
    {
        FAPI_SYS_PRINT_DEBUG(4,"[EIT] incomplete: status %i \n", sectionStatus);
        return;
    }

    if ( length < EIT_SECTLEN_MIN )
        return;

    tableId    = pSect[0];
    version    = ((pSect[5]& 0x3E) >> 1);
    sectNum    = pSect[6];
    serviceId  = LD16(pSect+3);
    tsId       = LD16(pSect+8);

//    FAPI_SYS_PRINT_DEBUG(1,"[EIT] [PF]  Section: tID: %X tsId %04X servId %04X sec %3i ver %2i\n",
//     tableId, tsId, serviceId, sectNum, version);

    /* According to ETSI guidelines, only section 0 and 1 provide
       present/following information */
    if(sectNum > 1)
    {
        return;
    }

    /* --------------------------------------------------------------------- */

    if(tableId == EIT_TABLEID_PF_ACTUALTS)      /* present/following, actual TS */
    {
        verFlag = 0;
    }
    else if(tableId == EIT_TABLEID_PF_OTHERTS)  /* present/following, other TS */
    {
        verFlag = EIT_VERSION_OTHER;
    }
    else
    {
        /* invalid table ID */
        return;
    }

    version |= verFlag;

    /* --------------------------------------------------------------------- */

    pCurrNext = EIT_GetTargetEntryCurrNext(inst, serviceId, tsId);

    if(pCurrNext != NULL)
    {
        if(version != pCurrNext->sectVer.active)
        {
            /* new table version found: invalidate all existing data */
            (void)FAPEX_SYS_GetSemaphore(inst->evtMutex, FAPEX_SYS_SUSPEND);

            EIT_CurrNextEntryClear(pCurrNext);

            pCurrNext->sectVer.active = version;

            (void)FAPEX_SYS_SetSemaphore(inst->evtMutex, FAPEX_SYS_NO_SUSPEND);
        }

        if(!PSISI_SectionAvailable(pCurrNext->sectVer.sectFound, sectNum))
        {
            (void)FAPEX_SYS_GetSemaphore(inst->evtMutex, FAPEX_SYS_SUSPEND);

            if((sectNum == 0) &&
               ((pCurrNext->curr.availability & EIT_AVAILABLE_BASIC) == 0))
            {
                pEvt = &(pCurrNext->curr);
            }
            else if((sectNum == 1) &&
                    ((pCurrNext->next.availability & EIT_AVAILABLE_BASIC) == 0))
            {
                pEvt = &(pCurrNext->next);
            }

            if(pEvt != NULL)
            {
                newData = EIT_ExtractCurrNext(&(pEvt->event),
                                               pEvt->ext, &(pEvt->extCharSet),
                                               AV_EIT_EXTLEN_CURRNEXT,
                                              &(pEvt->availability),
                                              pSect, (int32_t)length, sectNum);
            }

            (void)FAPEX_SYS_SetSemaphore(inst->evtMutex, FAPEX_SYS_NO_SUSPEND);

            PSISI_SectionMark(pCurrNext->sectVer.sectFound, sectNum);

            if(newData)
            {
                /* generate notification event */
                ctrlEvent.hdr.type   = EVT_EIT_UPDATE;
                ctrlEvent.hdr.length = sizeof(ctrlEvent);
                ctrlEvent.servIdx    = pCurrNext->hdr.servIdx;

                (void)EVT_Send(&ctrlEvent, "EIT update");

                //FAPI_SYS_PRINT_DEBUG(1,"[EIT] [PF] New Data received\n");
            }
        }
    }

    /* --------------------------------------------------------------------- */

    /* Try to insert present/following event also into schedule data. */

    pSched = EIT_GetTargetEntrySched(inst, serviceId, tsId);

    if(pSched != NULL)
    {
        if(version != pSched->sectVerPf.active)
        {
            /* new table version found */
            (void)FAPEX_SYS_GetSemaphore(inst->evtMutex, FAPEX_SYS_SUSPEND);

            memset(&(pSched->sectVerPf), 0, sizeof(EIT_TABLEREV_S));
            pSched->sectVerPf.active = version;

            (void)FAPEX_SYS_SetSemaphore(inst->evtMutex, FAPEX_SYS_NO_SUSPEND);
        }

        if(!PSISI_SectionAvailable(pSched->sectVerPf.sectFound, sectNum))
        {
            newData = EIT_ExtractCurrNext(&(inst->tmpSched.evt),
                                          inst->tmpSched.pExt,
                                           &(inst->tmpSched.extCharSet),
                                           AV_EIT_EXTLEN_SCHED,
                                           NULL,
                                           pSect, (int32_t)length, sectNum);

            if(newData)
            {
                inst->tmpSched.tableId = tableId;

                if(EIT_SchedListInsert(pSched, &(inst->tmpSched)) == FAPI_OK)
                {
                    /* generate notification event */
                    ctrlEvent.hdr.type = EVT_EIT_SCHED_UPDATE;
                    ctrlEvent.hdr.length = sizeof(ctrlEvent);
                    ctrlEvent.servIdx = pSched->hdr.servIdx;

                    (void)EVT_Send(&ctrlEvent, "EIT update");
                }
            }

            PSISI_SectionMark(pSched->sectVerPf.sectFound, sectNum);
        }
    }
}
/*lint -restore */


/*lint -save -e715 */
/* parameter "FAPI_SYS_HandleT handle" not needed in this function */
FSTATIC void EIT_SectSchedNotify(FAPI_SYS_HandleT handle,
                                 SFAL_SECTION_STATUS_E   sectionStatus,
                                 const uint8_t * pSect, uint16_t length,
                                 void *pPrivateData)
{
    EIT_DATA_S         *inst = (EIT_DATA_S *) pPrivateData;
    EIT_SCHED_ENTRY_S  *pSched;
    EIT_TABLEREV_S     *pVersion;
    uint8_t             tableId;
    uint8_t             version;
    uint8_t             sectNum;
    uint16_t            serviceId;
    uint16_t            tsId;
    uint8_t             verFlag;

    if ( sectionStatus != SFAL_SECTION_AVAILABLE )
    {
        FAPI_SYS_PRINT_DEBUG(4,"[EIT] [SCH] incomplete: status %i \n", sectionStatus);
        return;
    }

    if(length < EIT_SECTLEN_MIN)
        return;

    tableId    = pSect[0];
    version    = ((pSect[5]& 0x3E) >> 1);
    sectNum    = pSect[6];
    serviceId  = LD16(pSect+3);
    tsId       = LD16(pSect+8);

//    FAPI_SYS_PRINT_DEBUG(1,"[EIT] [SCH] Section: tID: %X tsId %04X servId %04X sec %3i ver %2i\n",
//       tableId, tsId, serviceId, sectNum, version);

    /* --------------------------------------------------------------------- */

    if((tableId & 0xF0) == 0x50)        /* schedule, actual TS */
    {
        verFlag = 0;
    }
    else if((tableId & 0xF0) == 0x60)   /* schedule, other TS */
    {
        verFlag = EIT_VERSION_OTHER;
    }
    else
    {
        /* invalid table ID */
        return;
    }

    version |= verFlag;

    /* --------------------------------------------------------------------- */

    pSched = EIT_GetTargetEntrySched(inst, serviceId, tsId);

    if(pSched != NULL)
    {
        pVersion = &(pSched->sectVerSched[tableId & 0x0F]);

        if(version != pVersion->active)
        {
            /* FAPI_SYS_PRINT_DEBUG(1,"NEW version: tableId 0x%X new %i old %i\n",
               tableId, version, pVersion->active); */

            (void)FAPEX_SYS_GetSemaphore(inst->evtMutex, FAPEX_SYS_SUSPEND);

            /* remove all sched entries belonging to this table */
            (void)EIT_SchedListRemoveTable(pSched, tableId);

            /* reset section availability variables */
            memset(pVersion, 0, sizeof(EIT_TABLEREV_S));
            pVersion->active = version;

            (void)FAPEX_SYS_SetSemaphore(inst->evtMutex, FAPEX_SYS_NO_SUSPEND);
        }

        if(!PSISI_SectionAvailable(pVersion->sectFound, sectNum))
        {
            if ( EIT_ExtractSched(pSched, &(inst->tmpSched), pSect,
                                 (int32_t)length, sectNum, tableId) )
            {
                PSISI_SectionMark(pVersion->sectFound, sectNum);
            }
        }
    }
}
FSTATIC void EIT_Notify(FAPI_SYS_HandleT handle,
                                 SFAL_SECTION_STATUS_E   sectionStatus,
                                 const uint8_t * pSect, uint16_t length,
                                 void *pPrivateData)
{
    EIT_DATA_S         *inst = (EIT_DATA_S *) pPrivateData;
    EIT_CURRNEXT_ENTRY_S *pCurrNext;
    EIT_CURRNEXT_EVT_S *pEvt = NULL;
    EIT_SCHED_ENTRY_S  *pSched;
    EIT_TABLEREV_S     *pVersion;
    SYS_EVT_EITEVT_S    ctrlEvent;
    uint8_t             tableId;
    uint8_t             version;
    uint8_t             sectNum;
    uint16_t            serviceId;
    uint16_t            tsId;
    uint8_t             verFlag;
    fbool_t             newData = FFALSE;

    if ( sectionStatus != SFAL_SECTION_AVAILABLE )
    {
        FAPI_SYS_PRINT_DEBUG(4,"[EIT] [SCH] incomplete: status %i \n", sectionStatus);
        return;
    }

    if(length < EIT_SECTLEN_MIN)
        return;

    tableId    = pSect[0];
    version    = ((pSect[5]& 0x3E) >> 1);
    sectNum    = pSect[6];
    serviceId  = LD16(pSect+3);
    tsId       = LD16(pSect+8);

//    FAPI_SYS_PRINT_DEBUG(1,"[EIT] [SCH] Section: tID: %X tsId %04X servId %04X sec %3i ver %2i\n",
//       tableId, tsId, serviceId, sectNum, version);
//    FAPI_SYS_PRINT_MSG("[EIT] [test] Section: tID: %X tsId %04X servId %04X sec %3i ver %2i\n",
//       tableId, tsId, serviceId, sectNum, version);

    if((tableId == EIT_TABLEID_PF_ACTUALTS)||(tableId == EIT_TABLEID_PF_OTHERTS))
    {
        /* --------------------------------------------------------------------- */
        if(tableId == EIT_TABLEID_PF_ACTUALTS)      /* present/following, actual TS */
        {
            verFlag = 0;
        }
        else if(tableId == EIT_TABLEID_PF_OTHERTS)  /* present/following, other TS */
        {
            verFlag = EIT_VERSION_OTHER;
        }
        else
        {
            /* invalid table ID */
            return;
        }

        version |= verFlag;

        /* --------------------------------------------------------------------- */

        pCurrNext = EIT_GetTargetEntryCurrNext(inst, serviceId, tsId);

        if(pCurrNext != NULL)
        {
            if(version != pCurrNext->sectVer.active)
            {
                /* new table version found: invalidate all existing data */
                (void)FAPEX_SYS_GetSemaphore(inst->evtMutex, FAPEX_SYS_SUSPEND);

                EIT_CurrNextEntryClear(pCurrNext);

                pCurrNext->sectVer.active = version;

                (void)FAPEX_SYS_SetSemaphore(inst->evtMutex, FAPEX_SYS_NO_SUSPEND);
            }

            if(!PSISI_SectionAvailable(pCurrNext->sectVer.sectFound, sectNum))
            {
                (void)FAPEX_SYS_GetSemaphore(inst->evtMutex, FAPEX_SYS_SUSPEND);

                if((sectNum == 0) &&
                   ((pCurrNext->curr.availability & EIT_AVAILABLE_BASIC) == 0))
                {
                    pEvt = &(pCurrNext->curr);
                }
                else if((sectNum == 1) &&
                        ((pCurrNext->next.availability & EIT_AVAILABLE_BASIC) == 0))
                {
                    pEvt = &(pCurrNext->next);
                }

                if(pEvt != NULL)
                {
                    newData = EIT_ExtractCurrNext(&(pEvt->event),
                                                   pEvt->ext, &(pEvt->extCharSet),
                                                   AV_EIT_EXTLEN_CURRNEXT,
                                                  &(pEvt->availability),
                                                  pSect, (int32_t)length, sectNum);
                }

                (void)FAPEX_SYS_SetSemaphore(inst->evtMutex, FAPEX_SYS_NO_SUSPEND);

                PSISI_SectionMark(pCurrNext->sectVer.sectFound, sectNum);

                if(newData)
                {
                    /* generate notification event */
                    ctrlEvent.hdr.type   = EVT_EIT_UPDATE;
                    ctrlEvent.hdr.length = sizeof(ctrlEvent);
                    ctrlEvent.servIdx    = pCurrNext->hdr.servIdx;

                    (void)EVT_Send(&ctrlEvent, "EIT update");

                    //FAPI_SYS_PRINT_DEBUG(1,"[EIT] [PF] New Data received\n");
                }
            }
        }

        /* --------------------------------------------------------------------- */

        /* Try to insert present/following event also into schedule data. */

        pSched = EIT_GetTargetEntrySched(inst, serviceId, tsId);

        if(pSched != NULL)
        {
            if(version != pSched->sectVerPf.active)
            {
                /* new table version found */
                (void)FAPEX_SYS_GetSemaphore(inst->evtMutex, FAPEX_SYS_SUSPEND);

                memset(&(pSched->sectVerPf), 0, sizeof(EIT_TABLEREV_S));
                pSched->sectVerPf.active = version;

                (void)FAPEX_SYS_SetSemaphore(inst->evtMutex, FAPEX_SYS_NO_SUSPEND);
            }

            if(!PSISI_SectionAvailable(pSched->sectVerPf.sectFound, sectNum))
            {
                newData = EIT_ExtractCurrNext(&(inst->tmpSched.evt),
                                              inst->tmpSched.pExt,
                                               &(inst->tmpSched.extCharSet),
                                               AV_EIT_EXTLEN_SCHED,
                                               NULL,
                                               pSect, (int32_t)length, sectNum);

                if(newData)
                {
                    inst->tmpSched.tableId = tableId;

                    if(EIT_SchedListInsert(pSched, &(inst->tmpSched)) == FAPI_OK)
                    {
                        /* generate notification event */
                        ctrlEvent.hdr.type = EVT_EIT_SCHED_UPDATE;
                        ctrlEvent.hdr.length = sizeof(ctrlEvent);
                        ctrlEvent.servIdx = pSched->hdr.servIdx;

                        (void)EVT_Send(&ctrlEvent, "EIT update");
                    }
                }

                PSISI_SectionMark(pSched->sectVerPf.sectFound, sectNum);
            }
        }
    }
    else if(((tableId & 0xF0) == 0x50)||((tableId & 0xF0) == 0x60))
    {
        /* --------------------------------------------------------------------- */

        if((tableId & 0xF0) == 0x50)        /* schedule, actual TS */
        {
            verFlag = 0;
        }
        else if((tableId & 0xF0) == 0x60)   /* schedule, other TS */
        {
            verFlag = EIT_VERSION_OTHER;
        }
        else
        {
            /* invalid table ID */
            return;
        }

        version |= verFlag;

        /* --------------------------------------------------------------------- */

        pSched = EIT_GetTargetEntrySched(inst, serviceId, tsId);

        if(pSched != NULL)
        {
            pVersion = &(pSched->sectVerSched[tableId & 0x0F]);

            if(version != pVersion->active)
            {
                /* FAPI_SYS_PRINT_DEBUG(1,"NEW version: tableId 0x%X new %i old %i\n",
                   tableId, version, pVersion->active); */

                (void)FAPEX_SYS_GetSemaphore(inst->evtMutex, FAPEX_SYS_SUSPEND);

                /* remove all sched entries belonging to this table */
                (void)EIT_SchedListRemoveTable(pSched, tableId);

                /* reset section availability variables */
                memset(pVersion, 0, sizeof(EIT_TABLEREV_S));
                pVersion->active = version;

                (void)FAPEX_SYS_SetSemaphore(inst->evtMutex, FAPEX_SYS_NO_SUSPEND);
            }

            if(!PSISI_SectionAvailable(pVersion->sectFound, sectNum))
            {
                if ( EIT_ExtractSched(pSched, &(inst->tmpSched), pSect,
                                     (int32_t)length, sectNum, tableId) )
                {
                    PSISI_SectionMark(pVersion->sectFound, sectNum);
                }
            }
        }
    }
    else
        return;

}

FSTATIC fbool_t EIT_ExtractCurrNext(EIT_EVT_S* pEvt,
                                    char_t* pExt, SYS_CHARSET* pExtCharSet,
                                    uint32_t extLen, uint8_t* avl,
                                    const uint8_t* pBuf, int32_t bufLen,
                                    uint8_t sectNum)
{
    uint16_t            sectLen;
    uint16_t            pos = 0;
    uint8_t             runStat;
    uint16_t            loopLen;
    uint16_t            loopPos;
    uint8_t             descTag;
    uint16_t            descLen;
    uint32_t            startTimeUtc;
    uint16_t            startTimeMjd;
    uint32_t            evtDuration;
    uint16_t            eventId;
    int32_t             extBufPos;
    uint32_t            txtPos;
    uint32_t            txtLen;
    uint32_t            ll;
    SYS_LANGCODE        shortLang;      //!< target language for short event descr.
    SYS_LANGCODE        extLang;        //!< target language for ext. event descr.
    SYS_LANGCODE        currLang;
    fbool_t             haveShortEvtDescr;
    SYS_CHARSET         extCharSet;

    if ( bufLen < EIT_SECTLEN_MIN )
    {
        FAPI_SYS_PRINT_DEBUG(2,"[EIT] bufLen %i  < EIT_SECTLEN_MIN %i\n", bufLen,
                 EIT_SECTLEN_MIN);
        return FFALSE;
    }

    sectLen = LD_LEN(pBuf + 1) + 3;

    if ( sectLen > (uint16_t)bufLen )
    {
        FAPI_SYS_PRINT_DEBUG(2,"[EIT] sectLen %i > bufLen %i\n",sectLen,bufLen);
        sectLen = (uint16_t)bufLen;
    }

    /* clear target structures */

    memset(pEvt, 0, sizeof(EIT_EVT_S));

    if ( extLen < 1 )
    {
        pExt = NULL;
    }

    else if ( pExt != NULL )
    {
        pExt[0] = 0;

        if ( extLen > 1 )
        {
            pExt[1] = 0;
        }
    }

    if ( pExtCharSet != NULL )
    {
        *pExtCharSet = SYS_CHARSET_UNDEFINED;
    }

    if(avl != NULL)
        *avl = 0;

    /* EIT header:              14 bytes
       Min. event description:  12 bytes
       --------
       26 bytes
     */

    pos = 14;

    if(sectLen >= (26 + CRC_LEN))
    {
        eventId       = LD16(pBuf + pos + 0);
        startTimeMjd  = LD16(pBuf + pos + 2);
        startTimeUtc  = LD24(pBuf + pos + 4);
        evtDuration   = LD24(pBuf + pos + 7);

        runStat  = pBuf[pos + 10] >> 5;
        loopLen  = LD_LEN(pBuf + pos + 10);
        loopPos  = 0;
        pos     += 12;

        if((uint16_t) (pos + loopLen + CRC_LEN) > sectLen)
        {
            FAPI_SYS_PRINT_DEBUG(1,"Invalid desc. loop length: %i \n", loopLen);
            return FFALSE;
        }

        if(avl != NULL)
            *avl |= EIT_AVAILABLE_BASIC;

        pEvt->runningStatus = runStat;

        if(pEvt->runningStatus == 0)    /* undefined? */
        {
            /* set default values in case of undefined value. */

            if(sectNum == 0)
                pEvt->runningStatus = PSISI_RUNSTAT_RUNNING;
            else if(sectNum == 1)
                pEvt->runningStatus = PSISI_RUNSTAT_NOTRUNNING;
        }

        pEvt->id = eventId;

        TIME_ConvertUTC (startTimeUtc, &(pEvt->startTime));
        TIME_ConvertMJD (startTimeMjd, &(pEvt->startDate));
        TIME_ConvertUTC (evtDuration,  &(pEvt->duration));

        extBufPos         = 0;
        haveShortEvtDescr = FFALSE;

        /* Parse loop for available languages */
        EIT_GetDescrLang(pBuf + pos, loopLen, &shortLang, &extLang);

        while((uint16_t) (loopPos + 2) <= loopLen)      /* parse all descriptors */
        {
            descTag = pBuf[pos + loopPos];
            descLen = pBuf[pos + loopPos + 1] + 2;

            //FAPI_SYS_PRINT_DEBUG(1,"DESC: 0x%X len %4i pos %i\n", descTag, descLen, pos + loopPos);

            switch (descTag)
            {
                case SHORT_EVENT_DESCRIPTOR:

                    if ( shortLang != LANG_NOTSPECIFIED )
                    {
                        currLang  = (SYS_LANGCODE)LD24(((pBuf + pos) + loopPos) + 2);
                        if ( currLang != shortLang )
                        {
                            break;
                        }
                    }

                    if(!haveShortEvtDescr)
                    {
                        EIT_GetShortEventDescr(pBuf + pos + loopPos, descLen,
                                                pEvt);

                        haveShortEvtDescr = FTRUE;
                    }
                    break;

                case EXT_EVENT_DESCRIPTOR:

                    if ( pExt == NULL )
                    {
                        break;
                    }

                    currLang = (SYS_LANGCODE)LD24(pBuf + ((pos + loopPos) + 3));

                    if ( extLang != LANG_NOTSPECIFIED )
                    {
                        if ( currLang != extLang )
                        {
                            break;
                        }
                    }
                    else
                    {
                        /* NOTE: This helps to avoid mixing of
                           languages: string may be constructed out of
                           multiple descriptors. */
                        extLang = currLang;
                    }

                    txtPos = 7 + (pBuf[pos + (loopPos + 6)] + 1);
                    txtLen = pBuf[((pos + loopPos) + txtPos) - 1];

                    if ( (uint32_t)extBufPos < extLen )
                    {
                        ll = UTILS_DBVSIText2String((char_t*)pBuf + pos + loopPos +
                                                    txtPos, txtLen,
                                                    pExt +
                                                    extBufPos,
                                                    (uint16_t)
                                                    (extLen - (uint32_t)extBufPos),
                                                    &extCharSet, FTRUE,
                                                    '\n');

                        if ( pExtCharSet != NULL )
                        {
                            if  ( *pExtCharSet == SYS_CHARSET_UNDEFINED )
                            {
                                *pExtCharSet = extCharSet;
                            }
                            else if ( (*pExtCharSet) != extCharSet )
                            {
                                FAPI_SYS_PRINT_DEBUG(1,"[EIT] Charset mixup: %i %i \n",
                                           *pExtCharSet, extCharSet);
                            }
                        }

                        extBufPos += (int32_t)ll;

                        if ( avl != NULL ) *avl |= EIT_AVAILABLE_EXT;
                    }
                    break;

                case PARENTAL_RATING_DESCRIPTOR:
                    /* extract parental rating */
                    EIT_ProcessParentalDescr(pBuf + pos + loopPos,
                                             descLen, &(pEvt->minAge));
                    break;

                case CONTENT_DESCRIPTOR:
                    EIT_ProcessContentDescr(pBuf + pos + loopPos,
                                             descLen, &(pEvt->content));
                    break;
                default:
                    break;
            }

            loopPos += descLen;
        }
    }

    /* NOTE: special case: empty EIT section: According to ETSI guideline, this
       indicates that no present (section 0) or following (section 1)
       event is available. */
    else
    {
        if(avl != NULL)
            *avl |= EIT_AVAILABLE_NO;
    }

    return FTRUE;
}

/*lint -save -e715 */
/* sectNum is not used here */
FSTATIC fbool_t EIT_ExtractSched (EIT_SCHED_ENTRY_S* pSched, EIT_SCHED_S* pItem,
                                  const uint8_t* pBuf, int32_t bufLen,
                                  uint8_t sectNum, uint8_t tableId)
{
    SYS_EVT_EITEVT_S    ctrlEvent;
    fbool_t             newData = FFALSE;
    uint16_t            sectLen;
    uint16_t            pos = 0;
    uint16_t            loopPos;
    uint16_t            loopLen;
    uint8_t             descTag;
    uint16_t            descLen;
    uint32_t            startTimeUtc;
    uint16_t            startTimeMjd;
    uint32_t            evtDuration;
    SYS_TIME_S          timeCurr;
    SYS_DATE_S          dateCurr;
    uint32_t            txtPos;
    uint32_t            txtLen;
    uint32_t            ll;
    int16_t             utcOffset;
    fbool_t             dstIsActive;
    SYS_LANGCODE        shortLang;      //!< target language for short event descr.
    SYS_LANGCODE        extLang;        //!< target language for ext. event descr.
    SYS_LANGCODE        currLang;
    fbool_t             haveShortEvtDescr;
    SYS_CHARSET         extCharSet;
    fbool_t             hasSynced;

    #if ( AV_SCHED_STORE_EXT != 0 )
    char_t               *pExt;
    int32_t              extBufPos;
    #endif

    if ( bufLen < EIT_SECTLEN_MIN )
    {
        return FFALSE;
    }

    sectLen = (((uint16_t) (pBuf[1] & 0x0F)) << 8) + pBuf[2] + 3;

    if ( sectLen > (uint32_t)bufLen )
    {
        FAPI_SYS_PRINT_DEBUG(1,"???\n");
        sectLen = (uint16_t)bufLen;
    }

    if ( TIME_TimeAndDateGetUtc(&timeCurr, &dateCurr, &hasSynced) != FAPI_OK )
    {
        return FFALSE;
    }
    else if ( !hasSynced )
    {
        return FFALSE;
    }
    else if ( SETTINGS_TimeOffsetGet(&utcOffset, &dstIsActive) != FAPI_OK )
    {
        return FFALSE;
    }

    if ( dstIsActive )
    {
        utcOffset += 60;
    }

    timeCurr.hour   = 0;
    timeCurr.minute = 0;
    timeCurr.second = 0;

    TIME_AddDays(&dateCurr, AV_SCHED_DAYS_MAX);
    TIME_AddSeconds(&timeCurr, &dateCurr, -((int32_t) utcOffset * 60));

    /* --------------------------------------------------------------------- */

    pos = 14;

    while((uint16_t) (pos + 12 + CRC_LEN) <= sectLen)
    {
        /* clear sched data structure */
        memset(pItem, 0, sizeof(EIT_SCHED_S));

        pItem->evt.id  = pBuf[pos + 0] <<  8;
        pItem->evt.id |= pBuf[pos + 1];
        startTimeMjd   = pBuf[pos + 2] <<  8;
        startTimeMjd  |= pBuf[pos + 3];
        startTimeUtc   = pBuf[pos + 4] << 16;
        startTimeUtc  |= pBuf[pos + 5] <<  8;
        startTimeUtc  |= pBuf[pos + 6];
        evtDuration    = pBuf[pos + 7] << 16;
        evtDuration   |= pBuf[pos + 8] <<  8;
        evtDuration   |= pBuf[pos + 9];

        pItem->evt.runningStatus  = pBuf[pos + 10] >> 5;
        pItem->evt.runningStatus &= 0x07;

        loopLen  = pBuf[pos + 10] << 8;
        loopLen |= pBuf[pos + 11];
        loopLen &= 0x0FFF;
        loopPos  = 0;
        pos     += 12;

        if((uint16_t) (pos + loopLen + CRC_LEN) > sectLen)
        {
            FAPI_SYS_PRINT_DEBUG(1,"[EIT] [SCH] Invalid desc. loop length: %i \n", loopLen);
            break;
        }

        pItem->tableId = tableId;

        #if ( AV_SCHED_STORE_EXT != 0 )
        extBufPos = 0;
        pExt = pItem->pExt;
        #endif

        TIME_ConvertUTC(startTimeUtc, &(pItem->evt.startTime));
        TIME_ConvertMJD(startTimeMjd, &(pItem->evt.startDate));
        TIME_ConvertUTC(evtDuration, &(pItem->evt.duration));

        /* test if time is out of bounds */
        if ( TIME_IsLaterThan(&timeCurr, &dateCurr,
                              &(pItem->evt.startTime),
                              &(pItem->evt.startDate)) == FFALSE )
        {
            /* do not parse this loop */
            pos += loopLen;
            continue;
        }

        haveShortEvtDescr = FFALSE;

        /* Parse loop for available languages */
        EIT_GetDescrLang(pBuf + pos, loopLen, &shortLang, &extLang);

        while((uint16_t) (loopPos + 2) <= loopLen)
        {
            descTag = pBuf[pos + loopPos];
            descLen = pBuf[pos + loopPos + 1] + 2;

            //FAPI_SYS_PRINT_DEBUG(1,"[EIT] [SCH] 0x%X len %4i pos %i\n", descTag, descLen, pos + loopPos);

            switch (descTag)
            {
                case SHORT_EVENT_DESCRIPTOR:
                    if(shortLang != LANG_NOTSPECIFIED)
                    {
                        currLang  = (SYS_LANGCODE)LD24(pBuf + pos + loopPos + 2);
                        if(currLang != shortLang)
                            break;
                    }

                    if(!haveShortEvtDescr)
                    {
                        EIT_GetShortEventDescr(pBuf + pos + loopPos, descLen,
                                                &(pItem->evt));
                        haveShortEvtDescr = FTRUE;
                    }
                    break;

                #if ( AV_SCHED_STORE_EXT != 0 )

                case EXT_EVENT_DESCRIPTOR:
                        currLang  = (SYS_LANGCODE)LD24(pBuf + pos + loopPos + 3);

                    if(extLang != LANG_NOTSPECIFIED)
                    {
                        if(currLang != extLang)
                            break;
                    }
                    else
                    {
                        /* NOTE: This helps to avoid mixing of
                           languages: string may be constructed out of
                           multiple descriptors. */
                        extLang = currLang;
                    }

                    txtPos = 7 + (pBuf[pos + loopPos + 6] + 1);
                    txtLen = pBuf[((pos + loopPos) + txtPos) - 1];

                    if ( extBufPos < AV_EIT_EXTLEN_SCHED )
                    {
                        ll = UTILS_DBVSIText2String((char_t*)pBuf + pos +
                                                    loopPos + txtPos, txtLen,
                                                    pExt + extBufPos,
                                                    (uint16_t)
                                                    (AV_EIT_EXTLEN_SCHED -
                                                    extBufPos), &extCharSet,
                                                    FTRUE, '\n');

                        if  ( pItem->extCharSet == SYS_CHARSET_UNDEFINED )
                        {
                            pItem->extCharSet = extCharSet;
                        }
                        else if ( pItem->extCharSet != extCharSet )
                        {
                            FAPI_SYS_PRINT_DEBUG(1,"[EIT] [SCH] Charset mixup: %i %i \n",
                                       pItem->extCharSet, extCharSet);
                        }

                        extBufPos += (int32_t)ll;
                    }
                    break;

                #endif

                case PARENTAL_RATING_DESCRIPTOR:
                    /* extract parental rating */
                    EIT_ProcessParentalDescr(pBuf + pos + loopPos, descLen,
                                             &(pItem->evt.minAge));
                    break;

                case CONTENT_DESCRIPTOR:
                    EIT_ProcessContentDescr(pBuf + pos + loopPos, descLen,
                                            &(pItem->evt.content));
                    break;
                default:
                    break;
            }

            loopPos += descLen;
        }

        pos += loopLen;

        if(EIT_SchedListInsert(pSched, pItem) == FAPI_OK)
        {
            newData = FTRUE;
        }
    }

    if(newData)
    {
        /* generate notification event */
        ctrlEvent.hdr.type = EVT_EIT_SCHED_UPDATE;
        ctrlEvent.hdr.length = sizeof(ctrlEvent);
        ctrlEvent.servIdx = pSched->hdr.servIdx;

        (void)EVT_Send(&ctrlEvent, "EIT sched update");
    }

    return FTRUE;
}
/*lint -restore */

FSTATIC void EIT_GetShortEventDescr(const uint8_t * descData, uint16_t descLen,
                                     EIT_EVT_S* pEvt)
{
    pEvt->name[0]     = 0;
    pEvt->desc[0]     = 0;
    pEvt->nameCharSet = SYS_CHARSET_UNDEFINED;
    pEvt->descCharSet = SYS_CHARSET_UNDEFINED;

    if ( (descLen >= 7) &&
         (descData[0] == SHORT_EVENT_DESCRIPTOR) )
    {
        (void)UTILS_DBVSIText2String ((const char_t*)(descData + 6),
                                      descData[5],
                                      pEvt->name, AV_EIT_NAMELEN_MAX,
                                      &(pEvt->nameCharSet),
                                      FTRUE, '\n');

        (void)UTILS_DBVSIText2String ((const char_t*)
                                      (&(descData[descData[5] + 7])),
                                      descData[descData[5] + 6],
                                      pEvt->desc, AV_EIT_DESCLEN_MAX,
                                      &(pEvt->descCharSet), FTRUE, '\n');
    }
}

FSTATIC void EIT_ProcessParentalDescr(const uint8_t * descData,
                                      uint16_t descLen, uint8_t * minAge)
{
    SYS_CTRYCODE        country;
    uint16_t            pos;
    uint8_t             minAgeFound = 0;
    uint8_t             minAgeCurr;

    #ifdef APPL_PARENTAL_ENABLED
    PCTRL_PARAMS_S      pctrlParams;
    // this definition was moved into the #ifdef clause to avoid MISRA warning
    fbool_t             countryFound = FFALSE;
    #endif /* APPL_PARENTAL_ENABLED */

    if(descLen < 6)
        return;

    #if 0
    {
        pos = 2;

        FAPI_SYS_PRINT_DEBUG(1,"[EIT] PARENTAL DESCRIPTOR FOUND: \n");

        while((uint16_t) (pos + 4) <= descLen)
        {
            FAPI_SYS_PRINT_DEBUG(1,"      --> %c%c%c - 0x%02X \n",
                       descData[pos + 0],
                       descData[pos + 1], descData[pos + 2], descData[pos + 3]);

            pos += 4;
        }
    }
    #endif

    /* ----- Search for preferred country first ---------------------------- */

    #ifdef APPL_PARENTAL_ENABLED

    if(PCTRL_GetParams(&pctrlParams) == FAPI_OK)
    {
        pos = 2;

        while((uint16_t) (pos + 4) <= descLen)
        {
            country = (SYS_CTRYCODE)TRIPLEASCII(descData[pos + 0],
                                  descData[pos + 1], descData[pos + 2]);

            if((country == pctrlParams.country) || (country == CTRY_ETR_ALL))
            {
                minAgeFound = EIT_GetMinAge(country, descData[pos + 3]);
                countryFound = FTRUE;
                break;
            }

            pos += 4;
        }
    }
    #endif /* APPL_PARENTAL_ENABLED */

    // TODO: check for ETSI group-of-country codes (scand./north amerc)

    /* --------------------------------------------------------------------- */

    /* If preferred country has not been found, apply strictest rating
       available in list. */

    // this #ifdef was added here to avoid misra warning when not defined
    #ifdef APPL_PARENTAL_ENABLED
    if(!countryFound)
    #endif /* APPL_PARENTAL_ENABLED */
    {
        pos = 2;

        while((uint16_t) (pos + 4) <= descLen)
        {
            country = (SYS_CTRYCODE)TRIPLEASCII(descData[pos + 0],
                                  descData[pos + 1], descData[pos + 2]);
            minAgeCurr = EIT_GetMinAge(country, descData[pos + 3]);

            if(minAgeCurr > minAgeFound)
            {
                minAgeFound = minAgeCurr;
            }

            pos += 4;
        }
    }

    *minAge = minAgeFound;
}
/*lint -save -e715 */
/* country is not used here see NOTE inside of function */
FSTATIC uint8_t EIT_GetMinAge(SYS_CTRYCODE country, uint8_t rating)
{
    uint8_t             minAge;

    if((rating >= 0x01) && (rating <= 0x0F))
    {
        minAge = rating + 3;
    }
    else
    {
        minAge = 0;
    }

    /* NOTE: country/broadcaster specific handling of the rating field could
       be added here. */

    return minAge;
}
/*lint -restore */

FSTATIC void EIT_ProcessContentDescr(const uint8_t * descData,
                                      uint16_t descLen, uint8_t * content)
{
    uint8_t pos = 2;

    while((uint16_t) (pos + 2) <= descLen)
    {
        *content = descData[pos];
        FAPI_SYS_PRINT_DEBUG(5,"[EIT] content 0x%02x \n",*content);
        pos += 2;
    }
}

FSTATIC void EIT_GetDescrLang(const uint8_t * pLoop, uint16_t loopLen,
                              SYS_LANGCODE * pShort, SYS_LANGCODE * pExt)
{
    uint16_t            loopPos = 0;
    uint8_t             descTag;
    uint16_t            descLen;
    uint8_t             prefCnt;
    SYS_LANGCODE        pref[SETTINGS_UILANGPREF_MAX];
    SYS_LANGCODE        currLang;
    uint8_t             prefShort = SETTINGS_UILANGPREF_MAX;
    uint8_t             prefExt = SETTINGS_UILANGPREF_MAX;

    if(pShort != NULL)
        *pShort = LANG_NOTSPECIFIED;
    if(pExt != NULL)
        *pExt = LANG_NOTSPECIFIED;

    /* get current user interface preferrences */
    for(prefCnt = 0; prefCnt < SETTINGS_UILANGPREF_MAX; prefCnt++)
    {
        if(SETTINGS_UILangPrefGet(prefCnt, pref + prefCnt) != FAPI_OK)
        {
            pref[prefCnt] = LANG_NOTSPECIFIED;
        }
    }

    while((uint16_t) (loopPos + 2) <= loopLen)  /* parse all descriptors */
    {
        descTag = pLoop[loopPos];
        descLen = pLoop[loopPos + 1] + 2;

        //FAPI_SYS_PRINT_DEBUG(1,"DESC: 0x%X len %4i pos %i\n", descTag, descLen, pos + loopPos);

        switch (descTag)
        {
            case SHORT_EVENT_DESCRIPTOR:
                if(pShort != NULL)
                {
                    currLang = (SYS_LANGCODE)(((uint32_t) (pLoop[loopPos + 2]) << 16) |
                        ((uint32_t) (pLoop[loopPos + 3]) << 8) |
                        (uint32_t) (pLoop[loopPos + 4]));

                    for(prefCnt = 0; prefCnt < SETTINGS_UILANGPREF_MAX;
                        prefCnt++)
                    {
                        if((pref[prefCnt] == currLang) && (prefCnt < prefShort))
                        {
                            prefShort = prefCnt;
                            *pShort = currLang;
                            break;
                        }
                    }
                }
                break;

            case EXT_EVENT_DESCRIPTOR:
                if(pExt != NULL)
                {
                    currLang = (SYS_LANGCODE)(((uint32_t)(pLoop[loopPos + 3]) << 16) |
                        ((uint32_t) (pLoop[loopPos + 4]) << 8) |
                        (uint32_t) (pLoop[loopPos + 5]));

                    for(prefCnt = 0; prefCnt < SETTINGS_UILANGPREF_MAX;
                        prefCnt++)
                    {
                        if((pref[prefCnt] == currLang) && (prefCnt < prefExt))
                        {
                            prefExt = prefCnt;
                            *pExt = currLang;
                            break;
                        }
                    }
                }
                break;

            default:
                break;
        }

        loopPos += descLen;

        if((prefShort == 0) && (prefExt == 0))
            break;
    }
}

/*---------------------------------------------------------------------------*/

#if (DEBUG > 0)

void AV_PrintSchedule(void)
{
    uint32_t            cnt = 0;
    char_t              extBuf[AV_EIT_EXTLEN_SCHED];
    SYS_TIME_S          timeCurr;
    SYS_DATE_S          dateCurr;
    EIT_EVT_S           item;
    EIT_EXTEVT_S        extEvt;
    SYS_SERVLOC_S       servDesc;
    EIT_POS_S           refPos;

    extEvt.pBuf   = extBuf;
    extEvt.maxLen = AV_EIT_EXTLEN_SCHED;

    (void)TIME_TimeAndDateGetUtc(&timeCurr, &dateCurr, NULL);
    (void)AV_GetRequestedService(&servDesc);

    FAPI_SYS_PRINT_DEBUG(1,"\n");
    FAPI_SYS_PRINT_DEBUG(1,"------------------------------------------------------------------ \n");
    FAPI_SYS_PRINT_DEBUG(1,"start Time: %04i/%02i/%02i = %02i:%02i \n", dateCurr.year,
               dateCurr.month, dateCurr.day, timeCurr.hour, timeCurr.minute);
    FAPI_SYS_PRINT_DEBUG(1,"------------------------------------------------------------------ \n");

    if(EIT_GetSchedByDate(&servDesc, &item, &extEvt, &refPos,
                            &timeCurr, &dateCurr, FFALSE) < 0 )
    {
        FAPI_SYS_PRINT_DEBUG(1,"   <empty>\n");
    }
    else
    {
        do
        {
            FAPI_SYS_PRINT_DEBUG(1,"%03i %02i/%02i %2i:%02i | %s - %s\n",
                       cnt,
                       item.startDate.month,
                       item.startDate.day,
                       item.startTime.hour,
                       item.startTime.minute, item.name, item.desc);

            FAPI_SYS_PRINT_DEBUG(1,"-> '%s' \n\n", extBuf);

            cnt++;
        }
        while(EIT_GetSchedNext(&item, &extEvt, &refPos) == FAPI_OK);
    }

    FAPI_SYS_PRINT_DEBUG(1,"------------------------------------------------------------------ \n");
}

#endif /* APPL_DEBUG_MODE */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
