/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/event_queue.c
**
**              $Id: event_queue.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Event management module.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save  */
#define DEBUG 1

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#ifndef _LINUX_FAPI
#include <fapi/sys_driver.h>
#include <rtos/rtos.h>
#else
#include <fapex/abst_types.h>
#include <fapex/abst_lib.h>
#endif

#include "config.h"
#include "utils.h"
#include "sys_api.h"
#include "sys_priv.h"
#include "event_queue.h"


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    /* Resources */
    uint8_t            *queue;
#ifndef _LINUX_FAPI
    RTOS_MailboxT       wakeupBox;
#else
    FAPEX_SYS_MailboxT       wakeupBox;
#endif
    /* Parameters */
    int16_t             posWr;
    int16_t             posRd;
    uint16_t            fillCnt;

} EVT_DATA_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC EVT_DATA_S  *evtDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/

int32_t EVT_Init(void)
{
    DBG_Assert(evtDat == NULL);

    evtDat = UTILS_Malloc(sizeof(EVT_DATA_S));
    if(evtDat == NULL)
    {
        FAPI_SYS_PRINT_DEBUG(1,"[%s l=%i] UTILS_Malloc() failed! \n",__FILE__,__LINE__);
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    evtDat->queue = NULL;
    evtDat->wakeupBox = NULL;
    evtDat->posRd = 0;
    evtDat->posWr = 0;
    evtDat->fillCnt = 0;

    evtDat->queue = (uint8_t *) UTILS_Malloc(EVT_EVENTS_MAX * EVT_SIZE_MAX);
    if(evtDat->queue == NULL)
    {
        FAPI_SYS_PRINT_DEBUG(1,"[%s l=%i] UTILS_Malloc() failed! \n",__FILE__,__LINE__);
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

#ifndef _LINUX_FAPI
    evtDat->wakeupBox = RTOS_CreateMailbox(RTOS_MSG_IS_POINTER);
#else
    evtDat->wakeupBox = FAPEX_SYS_CreateMailbox(FAPEX_SYS_MSG_IS_POINTER);
#endif
    if(!evtDat->wakeupBox)
    {
        FAPI_SYS_PRINT_DEBUG(1,"[%s l=%i] RTOS_CreateMailbox() failed!\n",__FILE__,__LINE__);
        return SMARTGO_SYS_ERR_RESOURCES;
    }

    return FAPI_OK;
}

int32_t EVT_Send(const void *pEvt, const char_t *debugString)
{
    SYS_EVTHDR_S       *pEvent = (SYS_EVTHDR_S *) pEvt;
    int32_t             retVal = FAPI_OK;
    uint32_t            status;
    uint32_t            msg;

    if(pEvent == NULL)
    {
        FAPI_SYS_PRINT_DEBUG(1,"[%s l=%i] EVT_Send( pEvt == NULL) \n",__FILE__,__LINE__);
        DBG_Assert(evtDat != NULL);
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    if(pEvent->length > EVT_SIZE_MAX)
    {
        FAPI_SYS_PRINT_DEBUG(1,"[%s l=%i] Event %i: size too large (%i > %i) \n",
                             __FILE__,__LINE__,pEvent->type, pEvent->length, EVT_SIZE_MAX);
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    FAPI_SYS_PRINT_DEBUG(3,"[%s l=%i] Event: ",__FILE__,__LINE__);
    if(debugString != NULL)
    {
        FAPI_SYS_PRINT_DEBUG(3,debugString);
    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(1," unknown (debugString == NULL) ");
        FAPI_SYS_PRINT_DEBUG(1,"\n");
    }
    

#ifndef _LINUX_FAPI
    RTOS_EnterCritical(status);
#else
    FAPEX_SYS_EnterCritical(status);
#endif
    if(evtDat->fillCnt < EVT_EVENTS_MAX)
    {
        /* copy event information to queue */
        (void)memcpy (evtDat->queue + (evtDat->posWr * EVT_SIZE_MAX), pEvent,
                      pEvent->length);

        /* update positioning data */
        evtDat->posWr = (evtDat->posWr + 1) % EVT_EVENTS_MAX;
        FAPI_SYS_PRINT_DEBUG(4,"[%s l=%i][SET] fill cnt %3i %3i ",__FILE__,__LINE__,evtDat->fillCnt,pEvent->type);
        if(debugString != NULL)
        {
            FAPI_SYS_PRINT_DEBUG(4,debugString);
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(4," %i (debugString == NULL) ",pEvent->type);
        }
        FAPI_SYS_PRINT_DEBUG(4,"\n");
        evtDat->fillCnt++;
    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(3,"[%s l=%i] ",__FILE__,__LINE__);
        if(debugString != NULL)
        {
            FAPI_SYS_PRINT_DEBUG(3,"[");
            FAPI_SYS_PRINT_DEBUG(3,debugString);
            FAPI_SYS_PRINT_DEBUG(3,"] ");
        }

        FAPI_SYS_PRINT_DEBUG(1,"Event queue overflow!\n",__FILE__,__LINE__);

        retVal = SMARTGO_SYS_ERR_CAPACITY;
    }

#ifndef _LINUX_FAPI
    RTOS_LeaveCritical(status);
#else
    FAPEX_SYS_LeaveCritical(status);
#endif
    if(retVal == FAPI_OK)
    {
        /* trigger wakeup of pending thread */
#ifndef _LINUX_FAPI
        (void)RTOS_SetMailbox(evtDat->wakeupBox, &msg, RTOS_NO_SUSPEND);
#else
        (void)FAPEX_SYS_SetMailbox(evtDat->wakeupBox, &msg, FAPEX_SYS_NO_SUSPEND);
#endif
    }

    return retVal;
}

int32_t EVT_Get(void *pEvt, fbool_t suspend)
{
    SYS_EVTHDR_S       *pEvent = (SYS_EVTHDR_S *) pEvt;
    int32_t             retVal = FAPI_OK;
    SYS_EVTHDR_S       *pEvtInQ;
    uint32_t            status;
    uint32_t            msg;

#ifndef _LINUX_FAPI
    RTOS_EnterCritical(status);
#else
    FAPEX_SYS_EnterCritical(status);
#endif

    /* if queue is empty, suspend until next event is added to queue */
    while(evtDat->fillCnt < 1)
    {
        if ( suspend == FTRUE )
        {
#ifndef _LINUX_FAPI
            RTOS_LeaveCritical(status);
#else
            FAPEX_SYS_LeaveCritical(status);
#endif

            /* sleep... */
#ifndef _LINUX_FAPI
            if(RTOS_GetMailbox(evtDat->wakeupBox, &msg, RTOS_SUSPEND) == 0)
#else
            if(FAPEX_SYS_GetMailbox(evtDat->wakeupBox, &msg, FAPEX_SYS_SUSPEND) == 0)
#endif
            {
                retVal = SMARTGO_SYS_ERR_RTOS_ERROR;
            }

#ifndef _LINUX_FAPI
            RTOS_EnterCritical(status);
#else
            FAPEX_SYS_EnterCritical(status);
#endif
        }
        else
        {
            retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
            break;
        }
    }

    if(retVal == FAPI_OK)
    {
        pEvtInQ = (SYS_EVTHDR_S *) (void *) (evtDat->queue + (uint32_t) (evtDat->posRd * EVT_SIZE_MAX));
        DBG_Assert(pEvtInQ->length <= EVT_SIZE_MAX);

        /* copy event information from queue */
        (void)memcpy (pEvent, pEvtInQ, pEvtInQ->length);

        /* update positioning data */
        evtDat->posRd = (evtDat->posRd + 1) % EVT_EVENTS_MAX;
        evtDat->fillCnt--;
        FAPI_SYS_PRINT_DEBUG(5,"[%s l=%i][GET] fill cnt %3i %3i\n",__FILE__,__LINE__,evtDat->fillCnt ,pEvent->type);
    }
#ifndef _LINUX_FAPI
    RTOS_LeaveCritical(status);
#else
    FAPEX_SYS_LeaveCritical(status);
#endif

    return retVal;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
