/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/fileop.c
**
**              $Id: fileop.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       File transfer operation handling.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info 715: function argument not referenced
** - Info 774: Boolean always evaluates to true - LINT gives false positives
**
*****************************************************************************
*/
/*lint -save -e715 -e774 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "sys_api.h"
#include "sys_priv.h"
#include "utils.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define FILEOP_CHUNK_SIZE   65536   //!< Size of data copied in one cycle
#define FILEOP_PATH_MAX     256     //!< Max. path length supported
#define FILEOP_PROG_KBYTES  512     //!< Progress event period: every xx KBytes

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef enum
{
    FS_TYPE_NONE = 0,
    FS_TYPE_FILE,
    FS_TYPE_DIR,
    FS_TYPE_MOUNT,
    FS_TYPE_DEV
} FS_TYPE_E;

typedef struct
{
    FS_TYPE_E type;
    char_t    filename[FILEOP_PATH_MAX];
    uint32_t  kbytes;
    uint32_t  bytes;
    F_FIND    fsFind;

} FS_READDIR_S;

typedef int32_t (*FILEOP_ENTRYFOUND_CB)  (char_t* dirPath,
                                          const FS_READDIR_S* pDirInf,
                                          void* arg);
typedef int32_t (*FILEOP_DIRPROCESSED_CB)(char_t* dirPath,
                                          const FS_READDIR_S* pDirInf,
                                          void* arg);
typedef struct
{
    FILEOP_TYPE_E type;
    char_t        srcPath[FILEOP_PATH_MAX];
    char_t        dstPath[FILEOP_PATH_MAX];
    SYS_DEVICE_E  srcDevice;
    SYS_DEVICE_E  dstDevice;

} FILEOP_CMD_S;

typedef struct
{
    RTOS_MailboxT       wakeupBox;
    uint8_t*            transferBuf;

    /* controlled (written) by user thread */
    FILEOP_INFO_S       status;
    fbool_t             abortReq;
    fbool_t             waitForStat;
    FILEOP_CMD_S        cmd;
    FS_TYPE_E           srcType;
    FS_TYPE_E           dstType;

    /* controlled (written) by fileop thread */
    uint16_t            srcPathLen;
    char_t              tmpSrcPath[FILEOP_PATH_MAX];
    char_t              tmpDstPath[FILEOP_PATH_MAX];

} FILEOP_DATA_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC FILEOP_DATA_S*  fileopDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void    FILEOP_ThreadFunction (void* opt);
FSTATIC int32_t FILEOP_CheckStatus    (void);
FSTATIC int32_t FILEOP_CheckPaths     (char_t* src, char_t* dst);
FSTATIC int32_t FILEOP_StartMoveCopyOp(char_t* src, char_t* dst);
FSTATIC int32_t FILEOP_WakeupThread   (void);
FSTATIC void    FILEOP_OperationCopy  (FILEOP_EVT_S* ctrlEvent);
FSTATIC void    FILEOP_OperationMove  (FILEOP_EVT_S* ctrlEvent,
                                       SYS_DEVICE_E srcDevice,
                                       SYS_DEVICE_E dstDevice);
FSTATIC void    FILEOP_OperationDelete(FILEOP_EVT_S* ctrlEvent);
FSTATIC int32_t FILEOP_GetFileInfo    (FILEOP_EVT_S* ctrlEvent,
                                       char_t* filePath);
FSTATIC int32_t FILEOP_GetDirInfoCb   (char_t* dirPath,
                                       const FS_READDIR_S* pDirInf, void* arg);

FSTATIC int32_t FILEOP_CopyFile       (FILEOP_EVT_S* ctrlEvent,
                                       char_t* srcPath, char_t* dstPath);
FSTATIC int32_t FILEOP_MoveFile       (FILEOP_EVT_S* ctrlEvent, char_t* srcPath,
                                       char_t* dstPath);
FSTATIC int32_t FILEOP_CopyDir        (FILEOP_EVT_S* ctrlEvent,
                                       char_t* srcPath, char_t* dstPath);
FSTATIC int32_t FILEOP_RemoveFile     (FILEOP_EVT_S* ctrlEvent, char_t* path);
FSTATIC int32_t FILEOP_RemoveDir      (FILEOP_EVT_S* ctrlEvent, char_t* path);
FSTATIC int32_t FILEOP_CopyDirCb      (char_t* dirPath,
                                       const FS_READDIR_S* pDirInf, void* arg);
FSTATIC int32_t FILEOP_RemoveDirCb    (char_t* dirPath,
                                       const FS_READDIR_S* pDirInf, void* arg);
FSTATIC int32_t FILEOP_RemoveDirDirCb (char_t* dirPath,
                                       const FS_READDIR_S* pDirInf, void* arg);
FSTATIC int32_t FILEOP_ProcessDirRecursive (char_t* dirPath,
                                       char_t* exclPath,
                                       FILEOP_ENTRYFOUND_CB entryFoundCb,
                                       FILEOP_DIRPROCESSED_CB dirProcessedCb,
                                       void* cbArg);
FSTATIC void FILEOP_CollectInfo       (FILEOP_EVT_S* ctrlEvent, char_t* path,
                                       char_t* exclPath);

FSTATIC void FILEOP_SizeAddClusters   (SYS_SIZE_S* size, uint32_t kbytes,
                                       uint32_t bytes);
FSTATIC void FILEOP_ExpandInfo        (FS_READDIR_S *entry);

int32_t FILEOP_Init (void)
{
    int32_t retVal = FAPI_OK;

    DBG_Assert(fileopDat == NULL);

    fileopDat = UTILS_Malloc(sizeof(FILEOP_DATA_S));
    if ( fileopDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(fileopDat, 0, sizeof(FILEOP_DATA_S));

    fileopDat->abortReq      = FFALSE;

    fileopDat->status.active = FFALSE;
    fileopDat->status.type   = FILEOP_TYPE_NONE;
    fileopDat->status.status = FILEOP_STATUS_IDLE;
    fileopDat->status.error  = FAPI_OK;

    fileopDat->waitForStat   = FFALSE;

    do
    {
        fileopDat->wakeupBox = RTOS_CreateMailbox(RTOS_MSG_IS_POINTER);
        if ( fileopDat->wakeupBox == NULL )
        {
            retVal = SMARTGO_SYS_ERR_RESOURCES;
            break;
        }

        fileopDat->transferBuf = UTILS_Malloc(FILEOP_CHUNK_SIZE);
        if ( fileopDat->transferBuf == NULL )
        {
            retVal = APPL_SMARTGO_ERR_OUT_OF_MEMORY;
            break;
        }

        retVal = SYS_ThreadCreate (FILEOP_ThreadFunction, NULL,
                                   THREAD_PRIO_FILEOP,
                                   THREAD_STACKSZ_FILEOP,
                                   THREAD_NAME_FILEOP,
                                   NULL);
        if ( retVal != FAPI_OK ) break;
    }
    while ( 0 );

    return retVal;
}

int32_t FILEOP_GetInfo (FILEOP_INFO_S* pStatus)
{
    if ( fileopDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    *pStatus = fileopDat->status;

    return FAPI_OK;
}

int32_t FILEOP_Copy (char_t* src, char_t* dst)
{
    int32_t retVal;

    if ( fileopDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    retVal = FILEOP_CheckStatus();
    if ( retVal != FAPI_OK ) return retVal;

    retVal = SYS_DeviceGetByPath (src, &(fileopDat->cmd.srcDevice), NULL);
    if ( retVal != FAPI_OK ) return retVal;

    retVal = SYS_DeviceGetByPath (dst, &(fileopDat->cmd.dstDevice), NULL);
    if ( retVal != FAPI_OK ) return retVal;

    (void)SYS_DevicePreAccess (fileopDat->cmd.srcDevice);
    (void)SYS_DevicePreAccess (fileopDat->cmd.dstDevice);

    retVal = FILEOP_CheckPaths(src, dst);
    if ( retVal != FAPI_OK ) return retVal;

    /* store command parameters */
    fileopDat->cmd.type      = FILEOP_TYPE_COPY;

    return FILEOP_StartMoveCopyOp(src, dst);
}

int32_t FILEOP_Move (char_t* src, char_t* dst)
{
    int32_t retVal;

    if ( fileopDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    retVal = FILEOP_CheckStatus();
    if ( retVal != FAPI_OK ) return retVal;

    retVal = SYS_DeviceGetByPath (src, &(fileopDat->cmd.srcDevice), NULL);
    if ( retVal != FAPI_OK ) return retVal;

    retVal = SYS_DeviceGetByPath (dst, &(fileopDat->cmd.dstDevice), NULL);
    if ( retVal != FAPI_OK ) return retVal;

    (void)SYS_DevicePreAccess (fileopDat->cmd.srcDevice);
    (void)SYS_DevicePreAccess (fileopDat->cmd.dstDevice);

    retVal = FILEOP_CheckPaths(src, dst);
    if ( retVal != FAPI_OK ) return retVal;

    /* store command parameters */
    fileopDat->cmd.type      = FILEOP_TYPE_MOVE;

    return FILEOP_StartMoveCopyOp(src, dst);
}

int32_t FILEOP_Delete (char_t* path)
{
    int32_t retVal;

    retVal = FILEOP_CheckStatus();
    if ( retVal != FAPI_OK ) return retVal;

    retVal = SYS_DeviceGetByPath (path, &(fileopDat->cmd.srcDevice), NULL);
    if ( retVal != FAPI_OK ) return retVal;

    (void)SYS_DevicePreAccess (fileopDat->cmd.srcDevice);

    retVal = FILEOP_CheckPaths(path, NULL);
    if ( retVal != FAPI_OK ) return retVal;

    /* store command parameters */
    fileopDat->cmd.type = FILEOP_TYPE_DELETE;

    (void)UTILS_strncpy(fileopDat->cmd.srcPath, path, FILEOP_PATH_MAX);
    fileopDat->cmd.dstPath[0] = 0;
    fileopDat->cmd.dstDevice  = SYS_DEV_NULL;

    return FILEOP_WakeupThread();
}

int32_t FILEOP_GetProperties (char_t* path)
{
    int32_t retVal;

    retVal = FILEOP_CheckStatus();
    if ( retVal != FAPI_OK ) return retVal;

    retVal = SYS_DeviceGetByPath (path, &(fileopDat->cmd.srcDevice), NULL);
    if ( retVal != FAPI_OK ) return retVal;

    (void)SYS_DevicePreAccess (fileopDat->cmd.srcDevice);

    retVal = FILEOP_CheckPaths(path, NULL);
    if ( retVal != FAPI_OK ) return retVal;

    /* store command parameters */
    fileopDat->cmd.type = FILEOP_TYPE_GETPROP;

    (void)UTILS_strncpy(fileopDat->cmd.srcPath, path, FILEOP_PATH_MAX);
    fileopDat->cmd.dstPath[0] = 0;
    fileopDat->cmd.dstDevice  = SYS_DEV_NULL;

    return FILEOP_WakeupThread();
}

int32_t FILEOP_Abort (void)
{
    if ( fileopDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    /* Abort not necessary if no operation is active/about to start */
    if ( FILEOP_CheckStatus() == FAPI_OK ) return FAPI_OK;

    fileopDat->abortReq    = FTRUE;
    fileopDat->waitForStat = FTRUE;

    return FAPI_OK;
}

int32_t FILEOP_Reset (void)
{
    int32_t retVal;

    if ( fileopDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    retVal = FILEOP_CheckStatus();
    if ( retVal != FAPI_OK ) return retVal;

    memset(&(fileopDat->status), 0, sizeof(fileopDat->status));

    fileopDat->status.active = FFALSE;
    fileopDat->status.type   = FILEOP_TYPE_NONE;
    fileopDat->status.status = FILEOP_STATUS_IDLE;
    fileopDat->status.error  = FAPI_OK;

    return FAPI_OK;
}

void FILEOP_HandleEvent (const FILEOP_EVT_S* evt)
{
    if ( fileopDat == NULL ) return;

    fileopDat->status      = evt->data;
    fileopDat->waitForStat = FFALSE;
}

FSTATIC int32_t FILEOP_StartMoveCopyOp (char_t* src, char_t* dst)
{
    int32_t fsRet;

    (void)UTILS_strncpy(fileopDat->cmd.srcPath, src, FILEOP_PATH_MAX);
    (void)UTILS_strncpy(fileopDat->cmd.dstPath, dst, FILEOP_PATH_MAX);

    if ( fileopDat->srcType == FS_TYPE_DIR )
    {
        /* create target directory when it is not existing*/
        if (f_getattr(fileopDat->cmd.dstPath,NULL))
        {
            fsRet = f_mkdir(fileopDat->cmd.dstPath);

            FAPI_SYS_PRINT_DEBUG(4,"[FILEOP] mkdir: '%s' ret %d!\n", fileopDat->cmd.dstPath,
                                                       fsRet);
            if ( fsRet != FAPI_OK )
            {
                return SMARTGO_SYS_ERR_INTERNAL;
            }
        }
    }

    return FILEOP_WakeupThread();
}

FSTATIC int32_t FILEOP_CheckStatus (void)
{
    if ( (fileopDat->waitForStat) ||
         (fileopDat->status.active)  )
    {
        return APPL_SMARTGO_ERR_DEVICE_BUSY;
    }
    else
    {
        return FAPI_OK;
    }
}

FSTATIC int32_t FILEOP_CheckPaths (char_t* src, char_t* dst)
{
    int32_t fsRet;
    F_STAT  fileStats;
    /* --------------------------------------------------------------------- */

    /* get source file type */
    fsRet = f_stat((char_t*)src, &fileStats);

    //FAPI_SYS_PRINT_DEBUG(4,"[FILEOP] Source type '%s': ret %i \n", src, fsRet);

    if ( fsRet < 0 )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    if(fileStats.attr & F_ATTR_VOLUME)
        fileopDat->srcType = FS_TYPE_MOUNT;
    else if(fileStats.attr & F_ATTR_DIR)
        fileopDat->srcType = FS_TYPE_DIR;
    else
        fileopDat->srcType = FS_TYPE_FILE;

    /* --------------------------------------------------------------------- */

    if ( dst != NULL )
    {
        fsRet = f_stat ((char_t*)dst, &fileStats);

        //FAPI_SYS_PRINT_DEBUG(4,"[FILEOP] Dest. type '%s': ret %i \n", dst, fsRet);

        if(fileStats.attr & F_ATTR_VOLUME)
            fileopDat->dstType = FS_TYPE_MOUNT;
        else if(fileStats.attr & F_ATTR_DIR)
            fileopDat->dstType = FS_TYPE_DIR;
        else
            fileopDat->dstType = FS_TYPE_FILE;

        /* if source is folder, destination must not be an existing file. */
        if ( (fileopDat->srcType == FS_TYPE_DIR) &&
             (fileopDat->dstType == FS_TYPE_FILE) )
        {
            return APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
    }
    else
    {
        fileopDat->dstType = FS_TYPE_NONE;
    }

    return FAPI_OK;
}

FSTATIC int32_t FILEOP_WakeupThread (void)
{
    uint32_t msg = 1;
    /* wake up thread */
    (void)RTOS_SetMailbox(fileopDat->wakeupBox, (void*)&msg,RTOS_SUSPEND);

    fileopDat->abortReq    = FFALSE;
    fileopDat->waitForStat = FTRUE;

    return FAPI_OK;
}

FSTATIC int32_t FILEOP_GetFileInfo (FILEOP_EVT_S* ctrlEvent, char_t* filePath)
{
    F_STAT      fileStats;
    uint32_t    kbytes;
    uint32_t    bytes;
    int32_t     retVal = FAPI_OK;

    if ( f_stat((char_t*)filePath, &fileStats) != FAPI_OK )
    {
        retVal = SMARTGO_SYS_ERR_INTERNAL;
    }
    else
    {
        ctrlEvent->data.progress.fileMax++;

        kbytes = fileStats.filesize / 1024;
        bytes  = fileStats.filesize % 1024;

        SYS_SizeAdd         (&(ctrlEvent->data.progress.fileDataMax), kbytes,
                             bytes);
        SYS_SizeAdd         (&(ctrlEvent->data.progress.allDataMax),  kbytes,
                             bytes);
        FILEOP_SizeAddClusters (&(ctrlEvent->data.progress.sizeOnMedium),
                                kbytes, bytes);
    }
    return retVal;
}

FSTATIC int32_t FILEOP_GetDirInfoCb (char_t* dirPath,
                                  const FS_READDIR_S* pDirInf, void* arg)
{
    FILEOP_EVT_S* ctrlEvent = (FILEOP_EVT_S*)arg;

    switch ( pDirInf->type )
    {
        case FS_TYPE_FILE:

            FAPI_SYS_PRINT_DEBUG(4," - '%s': FILE: %s \n", dirPath, pDirInf->filename);

            ctrlEvent->data.progress.fileMax++;

            SYS_SizeAdd (&(ctrlEvent->data.progress.allDataMax),
                         pDirInf->kbytes, pDirInf->bytes);
            FILEOP_SizeAddClusters (&(ctrlEvent->data.progress.sizeOnMedium),
                                    pDirInf->kbytes, pDirInf->bytes);
            break;

        case FS_TYPE_DIR:
        case FS_TYPE_MOUNT:
            FAPI_SYS_PRINT_DEBUG(4," - '%s': DIR:  %s \n", dirPath, pDirInf->filename);
            break;

        case FS_TYPE_DEV:
        case FS_TYPE_NONE:
            FAPI_SYS_PRINT_DEBUG(4," - '%s': type: 0x%X \n", dirPath, pDirInf->type);
            break;
    }

    return FAPI_OK;
}

FSTATIC void FILEOP_ThreadFunction (void* opt)
{
    FILEOP_EVT_S ctrlEvent;
    uint32_t     msg;

    SYS_DevicePrepareThread();

    for(;;)
    {
        FAPI_SYS_PRINT_DEBUG(4,"[FILEOP] Wait for command... \n");

        (void)RTOS_GetMailbox(fileopDat->wakeupBox,&msg,RTOS_SUSPEND);

        FAPI_SYS_PRINT_DEBUG(4,"[FILEOP] Command received: %i \n", fileopDat->cmd.type);

        memset(&ctrlEvent, 0, sizeof(ctrlEvent));

        ctrlEvent.hdr.type    = EVT_FILEOP_STATUS;
        ctrlEvent.hdr.length  = sizeof(ctrlEvent);

        ctrlEvent.data.error  = FAPI_OK;

        /* ----- Mark involved devices as busy ----------------------------- */

        ctrlEvent.data.srcDevice = fileopDat->cmd.srcDevice;
        ctrlEvent.data.dstDevice = fileopDat->cmd.dstDevice;

        if ( ctrlEvent.data.srcDevice != SYS_DEV_NULL )
        {
            (void)SYS_DeviceSetAccessStatus(ctrlEvent.data.srcDevice, FTRUE);
        }

        if ( ctrlEvent.data.dstDevice != SYS_DEV_NULL )
        {
            (void)SYS_DeviceSetAccessStatus(ctrlEvent.data.dstDevice, FTRUE);
        }

        /* ----- Process command ------------------------------------------- */

        switch ( fileopDat->cmd.type )
        {
            case FILEOP_TYPE_COPY:
                ctrlEvent.data.active = FTRUE;
                ctrlEvent.data.type   = FILEOP_TYPE_COPY;
                ctrlEvent.data.status = FILEOP_STATUS_BUSY;

                FILEOP_CollectInfo(&ctrlEvent, fileopDat->cmd.srcPath,
                                               fileopDat->cmd.dstPath);

                /* send initial status event */
                (void)EVT_Send (&ctrlEvent, "FILEOP progress");

                FILEOP_OperationCopy(&ctrlEvent);
                break;

            case FILEOP_TYPE_MOVE:
                ctrlEvent.data.active = FTRUE;
                ctrlEvent.data.type   = FILEOP_TYPE_MOVE;
                ctrlEvent.data.status = FILEOP_STATUS_BUSY;

                FILEOP_CollectInfo(&ctrlEvent, fileopDat->cmd.srcPath,
                                               fileopDat->cmd.dstPath);

                /* send initial status event */
                (void)EVT_Send (&ctrlEvent, "FILEOP progress");

                FILEOP_OperationMove(&ctrlEvent, ctrlEvent.data.srcDevice,
                                     ctrlEvent.data.dstDevice);
                break;

            case FILEOP_TYPE_DELETE:
                ctrlEvent.data.active = FTRUE;
                ctrlEvent.data.type   = FILEOP_TYPE_DELETE;
                ctrlEvent.data.status = FILEOP_STATUS_BUSY;

                FILEOP_CollectInfo(&ctrlEvent, fileopDat->cmd.srcPath,
                                               NULL);

                /* send initial status event */
                (void)EVT_Send (&ctrlEvent, "FILEOP progress");

                FILEOP_OperationDelete(&ctrlEvent);
                break;

            case FILEOP_TYPE_GETPROP:
                ctrlEvent.data.active = FTRUE;
                ctrlEvent.data.type   = FILEOP_TYPE_GETPROP;
                ctrlEvent.data.status = FILEOP_STATUS_BUSY;

                FILEOP_CollectInfo(&ctrlEvent, fileopDat->cmd.srcPath,
                                               NULL);

                ctrlEvent.data.active = FFALSE;
                ctrlEvent.data.status = FILEOP_STATUS_FINISHED;

                /* send status event */
                (void)EVT_Send (&ctrlEvent, "FILEOP progress");
                break;

            case FILEOP_TYPE_NONE:
            default:
                ctrlEvent.data.active = FFALSE;
                ctrlEvent.data.type   = FILEOP_TYPE_NONE;
                ctrlEvent.data.status = FILEOP_STATUS_ERROR;
                ctrlEvent.data.error  = SMARTGO_SYS_ERR_INTERNAL;

                /* send status event */
                (void)EVT_Send (&ctrlEvent, "FILEOP progress");
                break;
        }

        /* ----- Release involved devices ---------------------------------- */

        if ( ctrlEvent.data.srcDevice != SYS_DEV_NULL )
        {
            (void)SYS_DeviceSetAccessStatus(ctrlEvent.data.srcDevice, FFALSE);
        }

        if ( ctrlEvent.data.dstDevice != SYS_DEV_NULL )
        {
            (void)SYS_DeviceSetAccessStatus(ctrlEvent.data.dstDevice, FFALSE);
        }
    }
}

FSTATIC void FILEOP_OperationCopy (FILEOP_EVT_S* ctrlEvent)
{
    int32_t retVal = FAPI_OK;

    switch ( fileopDat->srcType )
    {
        case FS_TYPE_FILE:
            ctrlEvent->data.progress.fileCurr = 0;
            SYS_SizeClear (&(ctrlEvent->data.progress.allDataCurr));

            retVal = FILEOP_CopyFile(ctrlEvent, fileopDat->cmd.srcPath,
                                                fileopDat->cmd.dstPath);
            break;

        case FS_TYPE_DIR:
            retVal = FILEOP_CopyDir(ctrlEvent, fileopDat->cmd.srcPath,
                                               fileopDat->cmd.dstPath);
            break;
        case FS_TYPE_MOUNT:
        case FS_TYPE_DEV:
        case FS_TYPE_NONE:
            break;
    }

    switch ( retVal )
    {
        case FAPI_OK:
            ctrlEvent->data.status = FILEOP_STATUS_FINISHED;
            break;

        case SMARTGO_SYS_ERR_ABORTED:
            ctrlEvent->data.status = FILEOP_STATUS_ABORTED;
            break;

        default:
            ctrlEvent->data.status = FILEOP_STATUS_ERROR;
            ctrlEvent->data.error  = retVal;
            break;
    }

    /* send final event */
    ctrlEvent->data.active = FFALSE;

    (void)EVT_Send (ctrlEvent, "FILEOP progress");
}

FSTATIC void FILEOP_OperationMove (FILEOP_EVT_S* ctrlEvent,
                                   SYS_DEVICE_E srcDevice,
                                   SYS_DEVICE_E dstDevice)
{
    int32_t retVal = FAPI_OK;

    switch ( fileopDat->srcType )
    {
        case FS_TYPE_FILE:

            ctrlEvent->data.progress.fileCurr = 0;
            SYS_SizeClear (&(ctrlEvent->data.progress.allDataCurr));

            /* If source and target file reside on the same partition,
               try to move it by using a dedicated file system operation. */

            if ( srcDevice == dstDevice )
            {
                retVal = FILEOP_MoveFile(ctrlEvent, fileopDat->cmd.srcPath,
                                                    fileopDat->cmd.dstPath);

                /* If moving of files is not supported by host file system,
                   copy/remove operation will serve as fallback solution. */
                if ( retVal != APPL_SMARTGO_ERR_FEATURE_NOT_SUPPORTED ) break;
            }

            retVal = FILEOP_CopyFile(ctrlEvent, fileopDat->cmd.srcPath,
                                                fileopDat->cmd.dstPath);

            if ( retVal == FAPI_OK )
            {
                ctrlEvent->data.progress.fileCurr = 0;
                SYS_SizeClear (&(ctrlEvent->data.progress.allDataCurr));

                retVal = FILEOP_RemoveFile(ctrlEvent, fileopDat->cmd.srcPath);
            }
            break;

        case FS_TYPE_DIR:
            retVal = FILEOP_CopyDir(ctrlEvent, fileopDat->cmd.srcPath,
                                               fileopDat->cmd.dstPath);

            if ( retVal == FAPI_OK )
            {
                if(UTILS_strcmp(fileopDat->cmd.srcPath, fileopDat->cmd.dstPath)!=0)
                {
                    retVal = FILEOP_RemoveDir(ctrlEvent, fileopDat->cmd.srcPath);
                }
            }
            break;
        case FS_TYPE_MOUNT:
        case FS_TYPE_DEV:
        case FS_TYPE_NONE:
            break;
    }

    switch ( retVal )
    {
        case FAPI_OK:
            ctrlEvent->data.status = FILEOP_STATUS_FINISHED;
            break;

        case SMARTGO_SYS_ERR_ABORTED:
            ctrlEvent->data.status = FILEOP_STATUS_ABORTED;
            break;

        default:
            ctrlEvent->data.status = FILEOP_STATUS_ERROR;
            ctrlEvent->data.error  = retVal;
            break;
    }

    /* send final event */
    ctrlEvent->data.active = FFALSE;

    (void)EVT_Send (ctrlEvent, "FILEOP progress");
}

FSTATIC void FILEOP_OperationDelete (FILEOP_EVT_S* ctrlEvent)
{
    int32_t retVal = FAPI_OK;

    switch ( fileopDat->srcType )
    {
        case FS_TYPE_FILE:
            retVal = FILEOP_RemoveFile(ctrlEvent, fileopDat->cmd.srcPath);
            break;

        case FS_TYPE_DIR:
            retVal = FILEOP_RemoveDir(ctrlEvent, fileopDat->cmd.srcPath);
            break;
        case FS_TYPE_MOUNT:
        case FS_TYPE_DEV:
        case FS_TYPE_NONE:
            break;
    }

    switch ( retVal )
    {
        case FAPI_OK:
            ctrlEvent->data.status = FILEOP_STATUS_FINISHED;
            break;

        case SMARTGO_SYS_ERR_ABORTED:
            ctrlEvent->data.status = FILEOP_STATUS_ABORTED;
            break;

        default:
            ctrlEvent->data.status = FILEOP_STATUS_ERROR;
            ctrlEvent->data.error  = retVal;
            break;
    }

    /* send final event */
    ctrlEvent->data.active = FFALSE;

    (void)EVT_Send (ctrlEvent, "FILEOP progress");
}


FSTATIC int32_t FILEOP_CopyFile (FILEOP_EVT_S* ctrlEvent, char_t* srcPath,
                              char_t* dstPath)
{
    F_FILE*     srcFile       = NULL;
    F_FILE*     dstFile       = NULL;
    F_STAT      fileStats;
    int32_t     retVal        = FAPI_OK;
    uint32_t    lastEvtKbytes = 0;
    int32_t     numBytes;
    uint32_t    kbytes;
    uint32_t    bytes;

    FAPI_SYS_PRINT_DEBUG(4,"-> Copy file: '%s' -> '%s'\n", srcPath, dstPath);

    ctrlEvent->data.type = FILEOP_TYPE_COPY;

    // abort when copy to the same file (srcPath = dstPath)
    if (UTILS_strcmp(srcPath, dstPath)== 0) return FAPI_OK;

    do
    {
        /* open source file */
        srcFile = f_open(srcPath, "r");
        if ( srcFile == NULL )
        {
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
            break;
        }

        /* open destination file */
        dstFile = f_open(dstPath, "w");
        if ( dstFile == NULL )
        {
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
            break;
        }

        /* get file size */
        if ( f_stat((char_t*)srcPath, &fileStats) != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(4,"f_stat('%s') failed!\n", srcPath);

            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        kbytes = fileStats.filesize / 1024;
        bytes =  fileStats.filesize % 1024;

        SYS_SizeClear (&(ctrlEvent->data.progress.fileDataCurr));

        ctrlEvent->data.progress.fileDataMax.kbytes = kbytes;
        ctrlEvent->data.progress.fileDataMax.bytes  = bytes;

        while ( SYS_SizeIsSmaller(&(ctrlEvent->data.progress.fileDataCurr),
                                  &(ctrlEvent->data.progress.fileDataMax)) )
        {
            if ( fileopDat->abortReq )
            {
                retVal = SMARTGO_SYS_ERR_ABORTED;
                break;
            }

            numBytes = f_read(fileopDat->transferBuf, 1, FILEOP_CHUNK_SIZE,
                              srcFile);
            if ( numBytes <= 0 )
            {
                retVal = SMARTGO_SYS_ERR_FILEREAD;
                break;
            }

            if ( fileopDat->abortReq )
            {
                retVal = SMARTGO_SYS_ERR_ABORTED;
                break;
            }

            if ( f_write(fileopDat->transferBuf, 1, numBytes, dstFile) !=
                 numBytes )
            {
                retVal = SMARTGO_SYS_ERR_FILEWRITE;
                break;
            }

            SYS_SizeAdd (&(ctrlEvent->data.progress.fileDataCurr), 0,
                         (uint32_t)numBytes);
            SYS_SizeAdd (&(ctrlEvent->data.progress.allDataCurr), 0,
                         (uint32_t)numBytes);

            /* send periodic progress event */
            if ( ctrlEvent->data.progress.fileDataCurr.kbytes >=
                 (lastEvtKbytes + FILEOP_PROG_KBYTES) )
            {
                (void)EVT_Send (ctrlEvent, "FILEOP progress");

                lastEvtKbytes = ctrlEvent->data.progress.fileDataCurr.kbytes;
            }
        }
    }
    while ( 0 );

    /* cleanup */
    if ( dstFile != NULL ) (void)f_close(dstFile);
    if ( srcFile != NULL ) (void)f_close(srcFile);

    /* remove target file on error */
    if ( retVal != FAPI_OK )
    {
        FAPI_SYS_PRINT_DEBUG(4,"-> Copy file: error: %d \n", retVal);

        (void)f_delete(dstPath);
    }
    else
    {
        (void)EVT_Send (ctrlEvent, "FILEOP progress");

        ctrlEvent->data.progress.fileCurr++;
    }

    //FAPI_SYS_PRINT_DEBUG(4,"   Copy file: ret %d\n", retVal);
    return retVal;
}

FSTATIC int32_t FILEOP_MoveFile (FILEOP_EVT_S* ctrlEvent, char_t* srcPath,
                                 char_t* dstPath)
{
    F_STAT      fileStats;
    int32_t     retVal  = SMARTGO_SYS_ERR_INTERNAL;
    uint32_t    kbytes;
    uint32_t    bytes;
    int32_t     fsRet;

    FAPI_SYS_PRINT_DEBUG(4,"-> Move file: '%s' -> '%s'\n", srcPath, dstPath);

    ctrlEvent->data.type = FILEOP_TYPE_MOVE;

    // abort when move to the same file (srcPath = dstPath)
    if (UTILS_strcmp(srcPath, dstPath)== 0) return FAPI_OK;

    do
    {
        /* get file size */
        if ( f_stat(srcPath, &fileStats) != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(4,"f_stat('%s') failed!\n", srcPath);
            break;
        }

        SYS_SizeClear (&(ctrlEvent->data.progress.fileDataCurr));

        kbytes = fileStats.filesize / 1024;
        bytes  = fileStats.filesize % 1024;
        ctrlEvent->data.progress.fileDataMax.kbytes = kbytes;
        ctrlEvent->data.progress.fileDataMax.bytes  = bytes;

        fsRet = f_move(srcPath, dstPath);
        //force to replace a existing file
        if (fsRet == F_ERR_DUPLICATED)
        {
            (void)f_delete(dstPath);
            fsRet = f_move(srcPath, dstPath);
        }

        if ( fsRet == APPL_SMARTGO_ERR_FEATURE_NOT_SUPPORTED )
        {
            retVal = fsRet;
            break;
        }
        else if ( fsRet != FAPI_OK )
        {
            break;
        }


        SYS_SizeAdd (&(ctrlEvent->data.progress.fileDataCurr),
                       ctrlEvent->data.progress.fileDataMax.kbytes,
                       ctrlEvent->data.progress.fileDataMax.bytes);

        SYS_SizeAdd (&(ctrlEvent->data.progress.allDataCurr),
                       ctrlEvent->data.progress.fileDataMax.kbytes,
                       ctrlEvent->data.progress.fileDataMax.bytes);

        retVal = FAPI_OK;
    }
    while ( 0 );

    if ( retVal == FAPI_OK )
    {
        (void)EVT_Send (ctrlEvent, "FILEOP progress");

        ctrlEvent->data.progress.fileCurr++;
    }

    //FAPI_SYS_PRINT_DEBUG(4,"   Move file: ret %d\n", retVal);
    return retVal;
}

FSTATIC int32_t FILEOP_CopyDir (FILEOP_EVT_S* ctrlEvent,
                                char_t* srcPath, char_t* dstPath)
{
    fileopDat->srcPathLen = strlen(srcPath);

    ctrlEvent->data.progress.fileCurr = 0;

    SYS_SizeClear (&(ctrlEvent->data.progress.allDataCurr));

    return FILEOP_ProcessDirRecursive (srcPath, dstPath,
                                       FILEOP_CopyDirCb,
                                       NULL,
                                       ctrlEvent);
}

FSTATIC int32_t FILEOP_CopyDirCb (char_t* dirPath,
                                const FS_READDIR_S* pDirInf, void* arg)
{
    FILEOP_EVT_S*   ctrlEvent = (FILEOP_EVT_S*)arg;
    int32_t         retVal    = FAPI_OK;
    uint32_t        strPos;
    int32_t         fsRet;

    /* ----- Build source path --------------------------------------------- */

    strPos = UTILS_strncpy(fileopDat->tmpSrcPath, dirPath,
                           FILEOP_PATH_MAX);

    if ( (strPos + 1) >= FILEOP_PATH_MAX ) return SMARTGO_SYS_ERR_INTERNAL;

    /* append '/' */
    strPos += UTILS_strncpy(fileopDat->tmpSrcPath + strPos, "/", 2);

    if ( (strPos + 1) >= FILEOP_PATH_MAX ) return SMARTGO_SYS_ERR_INTERNAL;

    /* append filename to target path */
    strPos += UTILS_strncpy(fileopDat->tmpSrcPath + strPos,
                            pDirInf->filename,
                            FILEOP_PATH_MAX - strPos);

    /* ----- Build target path --------------------------------------------- */

    strPos = UTILS_strncpy(fileopDat->tmpDstPath, fileopDat->cmd.dstPath,
                           FILEOP_PATH_MAX);

    if ( (strPos + 1) >= FILEOP_PATH_MAX ) return SMARTGO_SYS_ERR_INTERNAL;

    strPos += UTILS_strncpy(fileopDat->tmpDstPath + strPos,
                            dirPath + fileopDat->srcPathLen,
                            FILEOP_PATH_MAX - strPos);

    if ( (strPos + 1) >= FILEOP_PATH_MAX ) return SMARTGO_SYS_ERR_INTERNAL;

    /* append '/' */
    strPos += UTILS_strncpy(fileopDat->tmpDstPath + strPos, "/", 2);

    if ( (strPos + 1) >= FILEOP_PATH_MAX ) return SMARTGO_SYS_ERR_INTERNAL;

    /* append filename to target path */
    strPos += UTILS_strncpy(fileopDat->tmpDstPath + strPos,
                            pDirInf->filename,
                            FILEOP_PATH_MAX - strPos);

    /* ----- Process file -------------------------------------------------- */

    switch ( pDirInf->type )
    {
        case FS_TYPE_FILE:

            retVal = FILEOP_CopyFile(ctrlEvent,
                                     fileopDat->tmpSrcPath,
                                     fileopDat->tmpDstPath);
            break;

        case FS_TYPE_DIR:
            if (f_getattr(fileopDat->tmpDstPath,NULL))
            {
                fsRet = f_mkdir(fileopDat->tmpDstPath);

                FAPI_SYS_PRINT_DEBUG(4,"[FILEOP] mkdir: '%s' ret %d!\n", fileopDat->tmpDstPath,
                                                           fsRet);
                if ( fsRet != FAPI_OK )
                {
                    return SMARTGO_SYS_ERR_INTERNAL;
                }
            }
            break;

        case FS_TYPE_MOUNT:
        case FS_TYPE_DEV:
        case FS_TYPE_NONE:
            /* do nothing */
            break;
    }

    return retVal;
}

FSTATIC int32_t FILEOP_RemoveFile (FILEOP_EVT_S* ctrlEvent, char_t* path)
{
    F_STAT      fileStats;
    int32_t     retVal  = FAPI_OK;
    uint32_t    kbytes;
    uint32_t    bytes;
    int32_t     fsRet;

    FAPI_SYS_PRINT_DEBUG(4,"-> Remove file: '%s'\n", path);

    ctrlEvent->data.type = FILEOP_TYPE_DELETE;

    do
    {

        /* get file size */
        if ( f_stat(path, &fileStats) != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(4,"f_stat('%s') failed!\n", path);

            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        SYS_SizeClear (&(ctrlEvent->data.progress.fileDataCurr));

        kbytes = fileStats.filesize / 1024;
        bytes  = fileStats.filesize % 1024;
        ctrlEvent->data.progress.fileDataMax.kbytes = kbytes;
        ctrlEvent->data.progress.fileDataMax.bytes  = bytes;

        fsRet = f_delete(path);
        if ( fsRet != FAPI_OK )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        FAPI_SYS_PRINT_DEBUG(4,"[FILEOP] del file: '%s' ret %i!\n", path, fsRet);

        SYS_SizeAdd (&(ctrlEvent->data.progress.allDataCurr), kbytes,
                     bytes);
    }
    while ( 0 );

    if ( retVal == FAPI_OK )
    {
        (void)EVT_Send (ctrlEvent, "FILEOP progress");

        ctrlEvent->data.progress.fileCurr++;
    }

    //FAPI_SYS_PRINT_DEBUG(4,"   Remove file: ret %d\n", retVal);
    return retVal;
}

FSTATIC int32_t FILEOP_RemoveDir (FILEOP_EVT_S* ctrlEvent, char_t* path)
{
    int32_t retVal;
    int32_t  fsRet;

    fileopDat->srcPathLen = strlen(fileopDat->cmd.srcPath);

    ctrlEvent->data.progress.fileCurr = 0;

    SYS_SizeClear (&(ctrlEvent->data.progress.allDataCurr));

    retVal = FILEOP_ProcessDirRecursive (path,
                                         NULL,
                                         FILEOP_RemoveDirCb,
                                         FILEOP_RemoveDirDirCb,
                                         ctrlEvent);

    if ( retVal == FAPI_OK )
    {
        fsRet = f_rmdir(path);
        if ( fsRet != F_NO_ERROR )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
        }
        FAPI_SYS_PRINT_DEBUG(4,"[FILEOP] rmdir: '%s' ret %d!\n", path, fsRet);
    }

    return retVal;
}

FSTATIC int32_t FILEOP_RemoveDirCb (char_t* dirPath,
                                 const FS_READDIR_S* pDirInf, void* arg)
{
    FILEOP_EVT_S* ctrlEvent = (FILEOP_EVT_S*)arg;
    uint32_t      strPos;

    if ( pDirInf->type != FS_TYPE_FILE ) return FAPI_OK;

    /* ----- Build source path --------------------------------------------- */

    strPos = UTILS_strncpy(fileopDat->tmpSrcPath, dirPath,
                           FILEOP_PATH_MAX);

    if ( (strPos + 1) >= FILEOP_PATH_MAX ) return SMARTGO_SYS_ERR_INTERNAL;

    /* append '/' */
    strPos += UTILS_strncpy(fileopDat->tmpSrcPath + strPos, "/", 2);

    if ( (strPos + 1) >= FILEOP_PATH_MAX ) return SMARTGO_SYS_ERR_INTERNAL;

    /* append filename to target path */
    strPos += UTILS_strncpy(fileopDat->tmpSrcPath + strPos,
                            pDirInf->filename,
                            FILEOP_PATH_MAX - strPos);

    if ( fileopDat->abortReq ) return SMARTGO_SYS_ERR_ABORTED;

    return FILEOP_RemoveFile(ctrlEvent, fileopDat->tmpSrcPath);
}

FSTATIC int32_t FILEOP_RemoveDirDirCb (char_t* dirPath,
                                    const FS_READDIR_S* pDirInf, void* arg)
{
    int32_t fsRet = f_rmdir(dirPath);

    FAPI_SYS_PRINT_DEBUG(4,"[FILEOP] rmdir: '%s' ret %d!\n", dirPath, fsRet);

    return fsRet;
}

FSTATIC void FILEOP_ExpandInfo (FS_READDIR_S *entry)
{
    if(entry->fsFind.attr & F_ATTR_VOLUME)
        entry->type = FS_TYPE_MOUNT;
    else if(entry->fsFind.attr & F_ATTR_DIR)
        entry->type = FS_TYPE_DIR;
    else
        entry->type = FS_TYPE_FILE;
    entry->kbytes = entry->fsFind.filesize / 1024;
    entry->bytes  = entry->fsFind.filesize % 1024;

    (void)UTILS_strncpy (entry->filename, entry->fsFind.filename,
                         FILEOP_PATH_MAX);
}

FSTATIC int32_t FILEOP_ProcessDirRecursive(char_t* dirPath,
                                         char_t* exclPath,
                                         FILEOP_ENTRYFOUND_CB entryFoundCb,
                                         FILEOP_DIRPROCESSED_CB dirProcessedCb,
                                         void* cbArg)
{
    FS_READDIR_S   entry;
    char_t         newPath   [FILEOP_PATH_MAX];
    char_t         searchPath[FILEOP_PATH_MAX + 4];
    uint32_t       strPos;
    uint32_t       searchPos;
    int32_t        retVal  = FAPI_OK;
    fbool_t        recurse;
    fbool_t        show;

    strPos    = UTILS_strncpy(newPath,    dirPath, FILEOP_PATH_MAX);
    searchPos = UTILS_strncpy(searchPath, dirPath, FILEOP_PATH_MAX);

    if ( (strPos + 1) >= FILEOP_PATH_MAX )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    /* append '/' to directory path */
    strPos += UTILS_strncpy (newPath + strPos, "/", 2);

    /* append separator and wildcard to the search path */
    (void)UTILS_strncpy (searchPath + searchPos, "/*.*",
                         (FILEOP_PATH_MAX + 4) - searchPos);

    if ( f_findfirst(searchPath, &entry.fsFind) != F_NO_ERROR )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    do
    {
        show    = FTRUE;
        recurse = FFALSE;

        if(entry.fsFind.filename[0] == '.')
            continue;

        FILEOP_ExpandInfo(&entry);

        if( entry.type == FS_TYPE_DIR)
        {
            /* build new directory path */
            if ( (strPos  + 1) >= FILEOP_PATH_MAX )
            {
                return SMARTGO_SYS_ERR_INTERNAL;
            }

            (void)UTILS_strncpy(newPath + strPos, entry.filename,
                          FILEOP_PATH_MAX - strPos);

            /* test for optional directory path to be excluded from
               search */
            if ( (exclPath != NULL) &&
                 (UTILS_strncmpi(newPath, exclPath, -1) == 0) )
            {
                show = FFALSE;
                break;
            }

            recurse = FTRUE;
        }
        else
        {
            /* do nothing */
        }

        if ( show && (entryFoundCb != NULL) )
        {
            retVal = entryFoundCb(dirPath, &entry, cbArg);
            if ( retVal != FAPI_OK )
            {
                break;
            }
        }

        if ( recurse )
        {
            (void)FILEOP_ProcessDirRecursive (newPath, exclPath,
                                        entryFoundCb, dirProcessedCb, cbArg);

            if ( dirProcessedCb != NULL )
            {
                retVal = dirProcessedCb(newPath, &entry, cbArg);
                if ( retVal != FAPI_OK )
                {
                    break;
                }
            }
        }
    } while(f_findnext(&entry.fsFind) == F_NO_ERROR);

    return retVal;
}

FSTATIC void FILEOP_CollectInfo (FILEOP_EVT_S* ctrlEvent, char_t* path,
                                 char_t* exclPath)
{
    if ( fileopDat->srcType == FS_TYPE_FILE )
    {
        (void)FILEOP_GetFileInfo(ctrlEvent, path);
    }
    else
    {
        (void)FILEOP_ProcessDirRecursive (path, exclPath,
                                    FILEOP_GetDirInfoCb, NULL,
                                    ctrlEvent);
    }
}

FSTATIC void FILEOP_SizeAddClusters (SYS_SIZE_S* size, uint32_t kbytes, uint32_t bytes)
{
    uint32_t overhead;

    /* align to kbytes */
    if ( bytes > 0 ) kbytes++;

    overhead = kbytes % FILEOP_CLUSTERSIZE;

    /* align to cluster size */
    if ( overhead > 0 )
    {
        kbytes += FILEOP_CLUSTERSIZE - overhead;
    }

    SYS_SizeAdd(size, kbytes, 0);
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
