/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/flash.c
**
**              $Id: flash.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Sector-based flash access to store user SmartGo-HD's user
**              settings.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save  */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#ifndef _LINUX_FAPI
#include <fapi/sys_driver.h>         /* global type definitions */
#include <fapi/drv_flash.h>
#else
#include <fapex/abst_types.h>
#include <fapex/abst_libvrflash.h>
#endif
#include <flashfs/flashfs.h>

#include "utils.h"
#include "sys_api.h"
#include "sys_priv.h"
#include "fcdata.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    uint32_t                id;
    char_t                  name[FLASHFS_FILE_NAME_MAX_LEN];
    fbool_t                 isFcdata;
    uint32_t                revision;
    uint32_t                minSectors;     //!< Minimum number of sectors.
    uint32_t                minSize;        //!< Minimum reserved space in kbytes.
    uint32_t                chunk_size;     //!< Chunk size in bytes for freq.
                                            //!< changing. data files.

} FLASH_FileT;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*!
******************************************************************************
**
**  \brief File list of the SmartGo-HD FLASH file system.
**
**  /TODO: choose sensible order!
**
**  \sa
**          - FLASH_Init()
******************************************************************************
*/
FSTATIC FLASH_FileT flashFileArray [] =
{
{ FILEID_SATDAT,        FILENAME_SATDAT,        FFALSE, FILEREV_SATDAT,        0, FILESIZE_MIN_SATDAT,        0 },
{ FILEID_TPLIST_SAT,    FILENAME_TPLIST_SAT,    FFALSE, FILEREV_TPLIST_SAT,    0, FILESIZE_MIN_TPLIST_SAT,    0 },
{ FILEID_TPLIST_TER,    FILENAME_TPLIST_TER,    FFALSE, FILEREV_TPLIST_TER,    0, FILESIZE_MIN_TPLIST_TER,    0 },
{ FILEID_TPLIST_CAB,    FILENAME_TPLIST_CAB,    FFALSE, FILEREV_TPLIST_CAB,    0, FILESIZE_MIN_TPLIST_CAB,    0 },
{ FILEID_STARTUPPARAMS, FILENAME_STARTUPPARAMS, FFALSE, FILEREV_STARTUPPARAMS, 0, FILESIZE_MIN_STARTUPPARAMS, 0 },
{ FILEID_FREQCHGDATA,   FILENAME_FREQCHGDATA,   FTRUE,  FILEREV_FREQCHGDATA,   0, FILESIZE_MIN_FREQCHGDATA,   sizeof(FCDATA_ENTRY_S) },
{ FILEID_RECTIMER,      FILENAME_RECTIMER,      FFALSE, FILEREV_RECTIMER,      0, FILESIZE_MIN_RECTIMER,      0 },
{ FILEID_PCTRL,         FILENAME_PCTRL,         FFALSE, FILEREV_PCTRL,         0, FILESIZE_MIN_PCTRL,         0 },
{ FILEID_CADATA,        FILENAME_CADATA,        FFALSE, FILEREV_CADATA,        0, FILESIZE_MIN_CADATA,        0 },
{ FILEID_SERVICEDATA,   FILENAME_SERVICEDATA,   FFALSE, FILEREV_SERVICEDATA,   0, FILESIZE_MIN_SERVICEDATA,   0 },
{ FILEID_PVRDAT,        FILENAME_PVRDAT,        FFALSE, FILEREV_PVRDAT,        0, FILESIZE_MIN_PVRDAT,        0 }
};

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

/*!
********************************************************************************
** \brief   Initialse the FLASH Filesystem of SmartGo-HD.
**
** \param   stackSize   The stack size of the flash filesystem thread.
** \param   prio        The priority  of the flash filesystem thread.
**
** \return  One of the following status codes:
**          - #FAPI_OK in case of success.
**          - #FAPI_ERR_OUT_OF_MEMORY
**          - #SMARTGO_SYS_ERR_CAPACITY
**          - FLASHFS error codes.
**          - FAPI_FLASH error codes.
**
********************************************************************************
*/
int32_t FLASH_Init(uint32_t stackSize, uint32_t prio)
{
#ifndef _LINUX_FAPI
    FAPI_FLASH_OpenParamsT  flashParams;    //!< Flash driver open params.
#else
    FAPEX_VRFLASH_OpenParamsT  flashParams;    //!< Flash driver open params.
#endif
    FAPI_SYS_HandleT        flHdl;
    FLASHFS_InitParamsT     flashfsParams;  //!< FLASH FS init parameters.
    int32_t                 retVal = FAPI_OK;
    uint32_t                cnt;
    uint32_t                sizeCurr;
    int32_t                 numSecBytes;

    flashfsParams.device_type  = sysData->flashDeviceType;
    flashfsParams.first_sector = sysData->flashUsrStartSector;
    flashfsParams.num_sector   = 0;
    flashfsParams.num_file     = sizeof(flashFileArray)
                               / sizeof(flashFileArray[0]);
    flashfsParams.file         = UTILS_Malloc(flashfsParams.num_file
                                                  * sizeof(FLASHFS_FILE_T));
    flashfsParams.stack_size   = stackSize;
    flashfsParams.priority     = prio;
    flashfsParams.flag         = 0;

    if ( flashfsParams.file == NULL )
    {
        return FAPI_ERR_OUT_OF_MEMORY;
    }

#ifndef _LINUX_FAPI
    flashParams.version     = FAPI_FLASH_VERSION;
#else
    flashParams.version     = FAPEX_VRFLASH_VERSION;
#endif
    flashParams.deviceType  = sysData->flashDeviceType;
    flashParams.firstSector = sysData->flashUsrStartSector;
    flashParams.sectorCount = 32;
    flashParams.fsysFlag    = 0;
    flashParams.forceFlush  = 0;

#ifndef _LINUX_FAPI
    flHdl = FAPI_FLASH_Open( &flashParams, &retVal);
#else
    flHdl = FAPEX_VRFLASH_Open( &flashParams, &retVal);
#endif
    if ( flHdl == NULL )
    {
        return retVal;
    }
    for ( cnt = 0; cnt < flashfsParams.num_file; cnt++ )
    {
        flashfsParams.file[cnt].id              = flashFileArray[cnt].id;
        flashfsParams.file[cnt].revision        = flashFileArray[cnt].revision;
        flashfsParams.file[cnt].sector_offset   = flashfsParams.num_sector;
        flashfsParams.file[cnt].num_sector      = 0;

        (void)UTILS_strncpy (flashfsParams.file[cnt].name, flashFileArray[cnt].name,
                       FLASHFS_FILE_NAME_MAX_LEN);

        if ( flashFileArray[cnt].isFcdata )
        {
            flashfsParams.file[cnt].type       = FLASHFS_FILE_FREQ_CHANGING;
            flashfsParams.file[cnt].chunk_size = flashFileArray[cnt].chunk_size;
        }
        else
        {
            flashfsParams.file[cnt].type       = FLASHFS_FILE_SEMI_STATIC;
            flashfsParams.file[cnt].chunk_size = 0; /* unused */
        }

        sizeCurr = 0;

        while ( (sizeCurr < flashFileArray[cnt].minSize)
                || (flashfsParams.file[cnt].num_sector
                    < flashFileArray[cnt].minSectors) )
        {
            if ( (flashfsParams.first_sector + flashfsParams.num_sector)
                 >= (uint32_t)sysData->flashSectors )
            {
                retVal = SMARTGO_SYS_ERR_CAPACITY;
                break;
            }

#ifndef _LINUX_FAPI
            numSecBytes = FAPI_FLASH_GetSectorBytes (flHdl,
                                                flashfsParams.first_sector +
                                                flashfsParams.num_sector);
#else
            numSecBytes = FAPEX_VRFLASH_GetSectorBytes (flHdl,
                                                flashfsParams.first_sector +
                                                flashfsParams.num_sector);
#endif
            if ( numSecBytes < 0 )
            {
                retVal = numSecBytes;
                break;
            }

            sizeCurr += ((uint32_t)numSecBytes) / 1024;

            flashfsParams.file[cnt].num_sector++;

            flashfsParams.num_sector++;
        }
    }
    #if 0
    {
        uint32_t               bytes;
        uint32_t               value;
        const char_t*          string;
        uint32_t               start;
        uint32_t               count;
        uint32_t               index;
        uint32_t               is_locked;
        uint32_t               is_empty;

    // read and report flash device information
        bytes = FAPI_FLASH_GetDeviceSize( flHdl );
        FAPI_SYS_PRINT_DEBUG(1, "device size: %d (0x%08x) bytes\n", bytes, bytes );
        value = FAPI_FLASH_GetDeviceId( flHdl );
        string = FAPI_FLASH_GetDeviceName( flHdl );
        FAPI_SYS_PRINT_DEBUG(1, "device id: 0x%04x (%s)\n", value, string );
        value = FAPI_FLASH_GetManufacturerId( flHdl );
        string = FAPI_FLASH_GetManufacturerName( flHdl );
        FAPI_SYS_PRINT_DEBUG(1, "manufacturer id: 0x%04x (%s)\n", value,string );
        count = FAPI_FLASH_GetSectorCount( flHdl );
        FAPI_SYS_PRINT_DEBUG(1,"sector count: %d\n", count );
        for( index=0; index < count; index++ )
        {
            start = FAPI_FLASH_GetSectorStart( flHdl, index );
            bytes = FAPI_FLASH_GetSectorBytes( flHdl, index );
            is_empty = FAPI_FLASH_IsSectorEmpty( flHdl, index );
            is_locked = FAPI_FLASH_IsSectorProtected( flHdl, index );
            FAPI_SYS_PRINT_DEBUG(1,"sector#%-03d: 0x%08x..0x%08x (%d %s) %c%s %c%s\n",
                                index, start, start+bytes-1,
                                bytes, "bytes",
                                is_empty==1?'=':'!', "empty",
                                is_locked==1?'=':'!', "protected" );
        }
    }
    #endif
#ifndef _LINUX_FAPI
    (void)FAPI_FLASH_Close(flHdl);
#else
    (void)FAPEX_VRFLASH_Close(flHdl);
#endif
    if ( retVal >= 0 )
    {
        retVal = FLASHFS_Init (&flashfsParams);
    }


    FAPI_SYS_FREE(flashfsParams.file);

    return retVal;
}

int32_t FLASH_CheckFileSize (uint32_t fileId, uint32_t requiredSize)
{
    uint32_t    cnt;
    int32_t     retVal;
    uint32_t    reqSizeKb;
    uint32_t    numEntries = sizeof(flashFileArray)
                           / sizeof(flashFileArray[0]);

    for ( cnt = 0; cnt < numEntries; cnt++ )
    {
        if ( flashFileArray[cnt].id == fileId )
        {
            break;
        }
    }

    if ( cnt >= numEntries )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        reqSizeKb = requiredSize / 1024;

        if ( (requiredSize % 1024) > 0 )
        {
            reqSizeKb++;
        }

        if ( reqSizeKb > flashFileArray[cnt].minSize )
        {
            FAPI_SYS_PRINT_DEBUG(2,"[FLASH] File '%s' not large enough: is: %i kbytes, required: %i kbytes!\n",
                     flashFileArray[cnt].name, flashFileArray[cnt].minSize,
                     reqSizeKb);

            retVal = SMARTGO_SYS_ERR_CONFIG_ERROR;
        }
        else
        {
            retVal = FAPI_OK;
        }
    }

    return retVal;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
