/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/jpeg.c
**
**              $Id: jpeg.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       JPEG viewer subsystem implementation.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info 747: Significant prototype coercion (unsigned int to double).
**
*****************************************************************************
*/
/*lint -save -e747 */
#define DEBUG 0

#if (DEBUG > 0)
    #define JPEG_DEBUG
    #define JPEG_Load_DEBUG
    //#define BMP_DEC_DEBUG
    #define JPEG_INTSHOW_DBG
    #define PNG_INTSHOW_DBG
    #define BMP_READTIME_DBG
#endif

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/

#include <stdio.h>
#include "av.h"
#include "utils.h"
#include <fapi/drv_mmu.h>
#include <math.h>
/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define JPEG_PATH_MAX  256
#define BMP24_BYTE_PER_PIXEL 3
#define JPEG_CHUNK_SIZE  (90*1024)
/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef enum
{
    JPEG_CMD_OPEN = 0,
    JPEG_CMD_CLOSE,
    JPEG_CMD_SHOW,
    JPEG_CMD_CLEAR

} JPEG_CMDTYPE_E;

typedef enum
{
    JPEG_FILE_JPG = 0,
    JPEG_FILE_BMP,
    JPEG_FILE_PNG,
    JPEG_FILE_UNKNOWN

} JPEG_FILETYPE_E;
typedef struct
{
    JPEG_CMDTYPE_E              type;
    const JPEG_SCREEN_PARAMS_S* screen;
    const JPEG_PIC_PARAMS_S*    pic;

} JPEG_CMD_S;


typedef struct
{
    ITC_INST_S              itcInst;
    fbool_t                 jpegbusy;
    F_FILE*                 fileP;
    JPEG_FILETYPE_E         fileType;

    /* handled by JPEG thread */
    fbool_t                 haveOsdLayer;
    SYS_OSDLAYER_S          osd;
    OSDHANDLER_LayerParamsT currLayerParams;
    char_t                  filePath[JPEG_PATH_MAX];

} JPEG_DATA_S;


typedef struct
{   uint8_t*            div[10];
    int16_t             Y[1920];
    int16_t             U[1920];
    int16_t             V[1920];
    int32_t             ratioX;
    int32_t             ratioY;
    int32_t             accuY;
    uint16_t            sizeX;
    int32_t             sizeY;
    int16_t             countY;
    int16_t             srcwidth;
    volatile uint32_t*  screenP;
    uint32_t            rowlongs;

} BMP_SCALER_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

static JPEG_DATA_S* jpegDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void    JPEG_ThreadFunction (void* opt);
FSTATIC int16_t JPEG_Load           (int16_t required, void* dataP,
                                     char_t* destP);
FSTATIC void    JPEG_CalcZoom       (const JPEG_PIC_PARAMS_S* pic,
                                     JPEG_t* jpegP);
FSTATIC int32_t JPEG_GetLayer       (JPEG_DATA_S* inst,
                                     const JPEG_SCREEN_PARAMS_S* screen);
FSTATIC void    JPEG_FreeLayer      (JPEG_DATA_S* inst);
FSTATIC void    JPEG_ClearLayer     (JPEG_DATA_S* inst);
FSTATIC void    JPEG_IntShow        (JPEG_DATA_S* inst,
                                     const JPEG_PIC_PARAMS_S* picParams);
FSTATIC JPEG_FILETYPE_E
                JPEG_FileCheck      (const JPEG_PIC_PARAMS_S* picParams);
FSTATIC void    BMP_IntShow         (JPEG_DATA_S* inst,
                                     const JPEG_SCREEN_PARAMS_S* screen);
FSTATIC uint32_t
                BMP_BlockRead       (F_FILE*  srcFile,
                                     uint8_t* destBuffer,
                                     uint32_t rowBytes,
                                     uint32_t fileSize, //intended read size
                                     uint32_t readIdx);
FSTATIC void    BMP_ScaleLine       (uint16_t scanline, uint8_t* rawP,
                                        BMP_SCALER_S* scalerP);
FSTATIC void    PNG_IntShow         (JPEG_DATA_S* inst,
                                     const JPEG_PIC_PARAMS_S* picParams);

int32_t JPEG_Init (const JPEG_INIT_PARAMS_S* params)
{
    ITC_PARAMS_S  itcParams;
    int32_t          retVal;

    jpegDat = (JPEG_DATA_S*)UTILS_Malloc(sizeof(JPEG_DATA_S));
    if ( jpegDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(jpegDat, 0, sizeof(JPEG_DATA_S));

    itcParams.setAbortReq = NULL;

    retVal = ITC_Init(&(jpegDat->itcInst), &itcParams);
    if ( retVal != FAPI_OK ) return retVal;

    jpegDat->jpegbusy                    = FFALSE;

    jpegDat->haveOsdLayer                = FFALSE;

    jpegDat->osd.layerIdx                = params->layer;

    retVal =SYS_ThreadCreate (JPEG_ThreadFunction, jpegDat,
                             THREAD_PRIO_JPEG,
                             THREAD_STACKSZ_JPEG,
                             THREAD_NAME_JPEG,
                             NULL);

    return retVal;

}

int32_t JPEG_Open (const JPEG_SCREEN_PARAMS_S* screen)
{
    JPEG_CMD_S cmd;

    if ( jpegDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if ( screen == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    cmd.type   = JPEG_CMD_OPEN;
    cmd.screen = screen;
    cmd.pic    = NULL;

    return ITC_CmdSend (&(jpegDat->itcInst), (void*)(&cmd), NULL);
}

int32_t JPEG_Close (void)
{
    JPEG_EVT_OSDINFO_S  event;
    JPEG_CMD_S          cmd;

    if ( jpegDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    cmd.type   = JPEG_CMD_CLOSE;
    cmd.screen = NULL;
    cmd.pic    = NULL;

    event.hdr.type =     EVT_JPEG_OSDINFO;
    event.hdr.length =   sizeof(event);
    event.osdIsReady =   FFALSE;
    (void)EVT_Send(&event,"JPEG OSD Info");

    return ITC_CmdSend (&(jpegDat->itcInst), (void*)(&cmd), NULL);
}

int32_t JPEG_Show (const JPEG_PIC_PARAMS_S* pic,
                   const JPEG_SCREEN_PARAMS_S* screen)
{
    JPEG_CMD_S cmd;

    if ( jpegDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if ( pic     == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    cmd.type   = JPEG_CMD_SHOW;
    cmd.screen = screen;
    cmd.pic    = pic;

    jpegDat->fileType = JPEG_FileCheck(pic);
    return ITC_CmdSend (&(jpegDat->itcInst), (void*)(&cmd), NULL);
}

int32_t JPEG_Clear () // KK: never called 23Jan09
{
    JPEG_CMD_S cmd;

    if ( jpegDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    cmd.type   = JPEG_CMD_CLEAR;
    cmd.screen = NULL;
    cmd.pic    = NULL;

    return ITC_CmdSend (&(jpegDat->itcInst), (void*)(&cmd), NULL);
}

fbool_t JPEG_IsBusy (void)
{
    if ( jpegDat == NULL ) return FFALSE;

    return jpegDat->jpegbusy;
}

FSTATIC void JPEG_ThreadFunction (void* opt)
{
    JPEG_DATA_S*            inst = (JPEG_DATA_S*)opt;
    JPEG_EVT_DECODEINFO_S   decodeInfo_event;
    void *                  cmdMsg;
    JPEG_CMD_S              cmd;
    JPEG_PIC_PARAMS_S       picParams;
    SYS_DEVICE_E            device;
    int32_t                 retVal;
    uint32_t                ack = 1;

    memset(&picParams,  0, sizeof(picParams));
    SYS_DevicePrepareThread ();

    for(;;)
    {
        cmdMsg = ITC_CmdWait (&(inst->itcInst));

        if ( cmdMsg == NULL ) continue;

        cmd = *((JPEG_CMD_S*)cmdMsg);

        switch ( cmd.type )
        {
            case JPEG_CMD_OPEN:
                (void)JPEG_GetLayer  (inst, cmd.screen);
                JPEG_ClearLayer(inst);
                break;

            case JPEG_CMD_CLOSE:
                JPEG_FreeLayer (inst);
                break;

            case JPEG_CMD_SHOW:
                if ( cmd.screen != NULL )
                {
                    (void)JPEG_GetLayer(inst, cmd.screen);
                }

                JPEG_ClearLayer(inst);
                picParams = *(cmd.pic);
                (void)UTILS_strncpy(inst->filePath,cmd.pic->path,JPEG_PATH_MAX);
                break;

            case JPEG_CMD_CLEAR:
                JPEG_ClearLayer(inst);
                break;

            default:
                break;
        }

        /* acknowledge command completion */
        (void)ITC_AckSend(&(inst->itcInst), (void *)&ack);

        if ( cmd.type != JPEG_CMD_SHOW ) continue;

        /* ----- Start JPEG decoder ---------------------------------------- */

        if ( !(inst->haveOsdLayer) ) continue;

        if ( inst->itcInst.abortReq ) continue;

        if ( SYS_DeviceGetByPath(inst->filePath, &device, NULL) == FAPI_OK )
        {
            /* mark device as busy */
            (void)SYS_DeviceSetAccessStatus(device, FTRUE);

            (void)SYS_DevicePreAccess (device);
        }
        else
        {
            device = SYS_DEV_NULL;
        }

        inst->jpegbusy = FTRUE;

        inst->fileP = f_open(inst->filePath, "r");

        if (inst->fileP == NULL)
        {
            FAPI_SYS_PRINT_DEBUG(1,"[JPEG] cannot open a file: %s\n", inst->filePath);
        }
        else
        {
            switch(inst->fileType)
            {
                case JPEG_FILE_BMP:
                    BMP_IntShow(inst, cmd.screen);
                    break;
                case JPEG_FILE_JPG:
                    JPEG_IntShow(inst, &picParams);
                    break;
                case JPEG_FILE_PNG:
                    PNG_IntShow(inst, &picParams);
                    break;
                default:
                case JPEG_FILE_UNKNOWN:
                    decodeInfo_event.hdr.type = EVT_JPEG_DECODEINFO;
                    decodeInfo_event.hdr.length = sizeof(decodeInfo_event);
                    decodeInfo_event.infoType  = JPEG_DECODEINFO_UNKNOWN_TYPE;
                    (void)EVT_Send (&decodeInfo_event,"Message Box Info");
                    break;
            }

            retVal = f_close(inst->fileP);
            if (retVal != FAPI_OK)
            {
                FAPI_SYS_PRINT_DEBUG(1,"[JPEG] f_close failed: %s\n", inst->filePath);
            }
            inst->jpegbusy = FFALSE;
        }

        if ( device != SYS_DEV_NULL )
        {
            (void)SYS_DeviceSetAccessStatus(device, FFALSE);
        }
    }
}

FSTATIC int32_t JPEG_GetLayer (JPEG_DATA_S* inst,
                               const JPEG_SCREEN_PARAMS_S* screen)
{
    int32_t             retVal = FAPI_OK;
    uint32_t            errorCode;
    uint16_t            hsize;
    uint16_t            vsize;
    SYS_HDTV_STANDARD   hdSys;
    JPEG_EVT_OSDINFO_S  event;

    /* normalize horizontal and vertical size*/
    hsize = (screen->pos.hsize + 1) & 0xFFFE;
    vsize = (screen->pos.vsize + 1) & 0xFFFE;

    if ( inst->haveOsdLayer )
    {
        if ( (inst->osd.params.xPos        != screen->pos.left)  ||
             (inst->osd.params.yPos        != screen->pos.top)   ||
             (inst->osd.params.inputWidth  != hsize) ||
             (inst->osd.params.inputHeight != vsize) )
        {
            JPEG_FreeLayer(inst); /* free existing layer first */
        }
    }

    if ( !(inst->haveOsdLayer) )
    {
        inst->osd.params.frameBased         = 0;
        retVal = AV_OutputTvStandardCurrGet(&hdSys, NULL);
        if (retVal!=FAPI_OK) return retVal;
        switch ( hdSys )
        {
            case HDSYS_576P:
                inst->osd.params.frameBased =    1;
                inst->osd.params.yPos       = screen->pos.top;
                break;
            case HDSYS_576I:
                inst->osd.params.yPos       = screen->pos.top/2;
                break;
            case HDSYS_720P:
                inst->osd.params.frameBased = 1;
                inst->osd.params.yPos       = screen->pos.top;
                break;
            default:
            case HDSYS_1080I:
            case HDSYS_NUMS:
                inst->osd.params.yPos       = screen->pos.top/2;
                break;
        }
        inst->osd.params.xPos                  = screen->pos.left;
        inst->osd.params.inputWidth            = hsize;
        inst->osd.params.inputHeight           = vsize;
        inst->osd.params.outputWidth           = 0;
        inst->osd.params.outputHeight          = 0;
        inst->osd.params.memoryBasePtr         = NULL;
        inst->osd.colourMode                   = OSDHANDLER_OSD_MODE_16BIT;
        inst->osd.layoutMode                   = OSDHANDLER_OSD_LAYOUT_CbYCrA;
        inst->osd.fillColour                   = 0x8010;
        if ( inst!= NULL )
        {
            retVal = SYS_GetOsdLayer(&(inst->osd), NULL);
            if ( retVal == FAPI_OK )
            {
                inst->haveOsdLayer = FTRUE;
                retVal = OSDHANDLER_GetLayerParameters(inst->osd.layerIdx,
                                                  &(inst->currLayerParams));
                if(retVal != FAPI_OK)
                {
                    FAPI_SYS_PRINT_DEBUG(2,"[JPEG] OSDHANDLER_GetLayerParameters() failed: %ld\n",retVal);
                }
                retVal = OSDHANDLER_ShowLayer(inst->osd.layerIdx,&errorCode);
                if ( retVal != FAPI_OK )
                {
                    FAPI_SYS_PRINT_DEBUG(1,"[JPEG] OSDHANDLER_ShowLayer failed %d\n", retVal);
                }
                JPEG_ClearLayer(inst);
            }
            else
            {
                FAPI_SYS_PRINT_DEBUG(1,"[JPEG] SYS_GetOsdLayer failed %d\n", retVal);
            }
        }
        else
        {
            retVal = APPL_SMARTGO_ERR_OUT_OF_MEMORY;
        }

    }

    event.hdr.type = EVT_JPEG_OSDINFO;
    event.hdr.length =  sizeof(event);
    event.osdIsReady = FTRUE;
    (void)EVT_Send(&event,"JPEG OSD Info");
    return retVal;
}

FSTATIC void JPEG_FreeLayer (JPEG_DATA_S* inst)
{
    if ( (inst != NULL) && ( inst->haveOsdLayer ) )
    {
        (void)SYS_GiveOsdLayer(inst->osd.layerIdx);
        inst->haveOsdLayer = FFALSE;
    }
}

FSTATIC void JPEG_ClearLayer (JPEG_DATA_S* inst)
{
    int32_t retVal;
    retVal =  OSDHANDLER_SetFillColour(inst->osd.layerIdx, 0x8010);
    if (retVal!=FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1,"[JPEG] OSDHANDLER_SetFillColour failed %d\n", retVal);
    }
    retVal = OSDHANDLER_Clear(inst->osd.layerIdx);
    if (retVal!=FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1,"[JPEG] OSDHANDLER_Clear failed %d\n", retVal);
    }

}

FSTATIC int16_t JPEG_Load (int16_t required, void* dataP, char_t* destP)
{
    JPEG_DATA_S*    inst= (JPEG_DATA_S*)dataP;
    int32_t         retVal;

    if ( inst->itcInst.abortReq ) return 0;

    retVal = f_read(destP, 1, required, inst->fileP);

    #ifdef JPEG_Load_DEBUG
    FAPI_SYS_PRINT_DEBUG(1,"[JPEG] read %d (%d) bytes from 0x%x to 0x%x\n",
                             retVal, required, inst->fileP, (uint32_t)destP);
    FAPI_SYS_PRINT_DEBUG(1,"[JPEG] Current r/w position %d \n",f_tell(inst->fileP));
    #endif

    if ( retVal < 0 ) return 0;
    else              return retVal;
}


FSTATIC void JPEG_CalcZoom (const JPEG_PIC_PARAMS_S* pic, JPEG_t* jpegP)
{
    uint32_t     requestedsource= ((uint32_t)jpegP->info.sizeX * 100)/pic->zoom;

    jpegP->image.left  = ((jpegP->info.sizeX - requestedsource)*pic->xpos)/100;
    jpegP->image.right = jpegP->image.left + requestedsource;

    requestedsource= ((uint32_t)jpegP->info.sizeY * 100) / pic->zoom;

    jpegP->image.top    = ((jpegP->info.sizeY - requestedsource)*pic->ypos)/100;
    jpegP->image.bottom = jpegP->image.top + requestedsource;
}

FSTATIC void PNG_IntShow(JPEG_DATA_S* inst,
                         const JPEG_PIC_PARAMS_S* picParams)
{
    JPEG_EVT_INFO_S             event;
    JPEG_EVT_DECODEINFO_S       decodeInfo_event;
    int32_t                     fileSize;
    int32_t                     fileEndPos;
    uint32_t*                   pngPtr = NULL;
    int32_t                     retVal = FAPI_OK;
    int32_t                     bytes = 0;
    uint8_t                     buffer[4];
    uint8_t                     ebuffer[64];
    DRAW_PicItemT*              picItemPtr = NULL;
    DRAW_PicItemT*              convPicItemPtr = NULL;
    uint32_t                    temp = 0;
    uint16_t                    width = 0;
    uint16_t                    height = 0;
    uint16_t                    scale_width = 0;
    uint16_t                    scale_height = 0;
    uint16_t                    max_width = 0;
    uint16_t                    max_height = 0;
    uint16_t                    min_width = 0;
    uint16_t                    min_height = 0;
    FAPI_MMU_HeapSegmentInfoT   segInf;
    uint32_t                    free_uncached_memory = 0;
    fbool_t                     errorMsgShow = FFALSE;
    uint16_t                    i;
    int32_t                     currPos;
    uint32_t                    ratio;

    /* MISRA */
    bytes = bytes;

    do
    {
        /* get the IHDR chunk info from file */
        (void)f_seek(inst->fileP, picParams->filepos, F_SEEK_SET);
        while ( !(f_eof(inst->fileP)) )
        {
            bytes = f_read(buffer, 1, sizeof(buffer), inst->fileP);
            if ((buffer[0] == 0x49) && (buffer[1] == 0x48)
                && (buffer[2] == 0x44) && (buffer[3] == 0x52)) //'IHDR'
            {
                (void)f_read(buffer, 1, sizeof(buffer), inst->fileP);
                width = ((uint32_t)(buffer[0] << 24)) |
                        ((uint32_t)(buffer[1] << 16)) |
                        ((uint32_t)(buffer[2] <<  8)) |
                                    buffer[3];
                (void)f_read(buffer, 1, sizeof(buffer), inst->fileP);
                height = ((uint32_t)(buffer[0] << 24)) |
                         ((uint32_t)(buffer[1] << 16)) |
                         ((uint32_t)(buffer[2] <<  8)) |
                                     buffer[3];
                break;
            }
        }

        #ifdef PNG_INTSHOW_DBG
        if (f_eof(inst->fileP))
        {
            FAPI_SYS_PRINT_DEBUG(1,"\n[PNG] no IHDR info is found in the PNG file\n");
        }
        #endif

        if (( width == 0 ) || ( height == 0 ))
        {
            decodeInfo_event.infoType  = JPEG_DECODEINFO_ERROR;
            errorMsgShow = FTRUE;
            break;
        }

        /* file info for menu display */
        event.hdr.type   = EVT_JPEG_FILEINFO;
        event.hdr.length = sizeof(event);
        event.fileType   = PIC_TYPE_PNG;
        event.sizeX      = width;
        event.sizeY      = height;
        (void)EVT_Send (&event,"PNG Info");

        /* read the png file to the buffer */
        if( picParams->embedded )
        {
            //by searching for EOF flag 'IEND' to get the embeded PNG file size
            fileEndPos = 0;

            (void)f_seek(inst->fileP, picParams->filepos, F_SEEK_SET);
            while (!(f_eof(inst->fileP)))
            {
                currPos = f_tell(inst->fileP);
                if (currPos > picParams->filepos)
                {
                    //be sure the last three bytes in the buffer is also checked
                    (void)f_seek(inst->fileP, -3, F_SEEK_CUR);
                }
                bytes = f_read(ebuffer, 1, sizeof(ebuffer), inst->fileP);
                if (bytes > 4)
                {
                    for (i=0; (i + 3) < sizeof(ebuffer); i++)
                    {
                        if ((ebuffer[i] == 0x49) && (ebuffer[i+1] == 0x45)
                            && (ebuffer[i+2] == 0x4E) && (ebuffer[i+3] == 0x44)) //'IEND'
                        {
                            currPos = f_tell(inst->fileP);
                            fileEndPos = (currPos-bytes) + (int32_t)i + 8;
                            // end of file with 8 bytes:"IEND®B`‚"
                            break;
                        }
                    }
                }
                else
                {
                    break;
                }
                if (fileEndPos != 0)
                    break;
            }

            if ((fileEndPos == 0))
            {
                break; // no valid embedded PNG file is found, return.
            }
            else
            {
                fileSize = fileEndPos - picParams->filepos;
            }
        }
        else
        {
            fileSize = f_filelength(inst->filePath);
        }

        pngPtr = UTILS_Malloc((uint32_t)fileSize);
        if (pngPtr == NULL)
        {
            decodeInfo_event.infoType  = JPEG_DECODEINFO_LOW_MEMORY;
            errorMsgShow = FTRUE;
            break;
        }

        memset(pngPtr, 0, (uint32_t)fileSize);
        (void)f_seek(inst->fileP, picParams->filepos, F_SEEK_SET);
        bytes = f_read(pngPtr, 1, fileSize, inst->fileP);

        #ifdef PNG_INTSHOW_DBG
        FAPI_SYS_PRINT_DEBUG(1,"\n[PNG] read file to memory %d (filesize %d)\n",
                    bytes, fileSize);
        #endif

        /* Read the PNG picture file and
           store it as a DRAW_PicItemT structure (fbm) */
        picItemPtr = UTILS_Malloc(sizeof(DRAW_PicItemT));
        if (picItemPtr == NULL)
        {
            UTILS_Free(pngPtr);
            decodeInfo_event.infoType  = JPEG_DECODEINFO_LOW_MEMORY;
            errorMsgShow = FTRUE;
            break;
        }
        memset(picItemPtr, 0, sizeof(DRAW_PicItemT));

        picItemPtr->clutId = 0;
        picItemPtr->clutPtr = NULL;
        picItemPtr->clutStartIdx = 0;
        picItemPtr->clutEndIdx = 255;

        retVal = DRAW_ReadFile_PNG(pngPtr, picItemPtr);

        if ( retVal )
        {
            UTILS_Free(pngPtr);
            UTILS_Free(picItemPtr);
            if ( retVal == DRAW_ERR_OUT_OF_MEMORY )
            {
                decodeInfo_event.infoType  = JPEG_DECODEINFO_LOW_MEMORY;
            }
            else
            {
                decodeInfo_event.infoType  = JPEG_DECODEINFO_ERROR;
            }
            errorMsgShow = FTRUE;
            break;
        }

        #ifdef PNG_INTSHOW_DBG
        FAPI_SYS_PRINT_DEBUG(1,"\n[PNG] DRAW_ReadFile_PNG %d \n", retVal);
        #endif

        UTILS_Free(pngPtr);
        /* rescale the fbm picture and convert to 32bit RGB if necessary */
        convPicItemPtr = UTILS_Malloc(sizeof(DRAW_PicItemT));
        if (convPicItemPtr == NULL)
        {
            decodeInfo_event.infoType  = JPEG_DECODEINFO_LOW_MEMORY;
            errorMsgShow = FTRUE;
            break;
        }
        memset(convPicItemPtr, 0, sizeof(DRAW_PicItemT));

        convPicItemPtr->clutId = 0;
        convPicItemPtr->clutPtr = NULL;
        convPicItemPtr->clutStartIdx = 0;
        convPicItemPtr->clutEndIdx = 255;
        convPicItemPtr->globalAlpha = 256;

        if(picItemPtr->fbmHeaderPtr->colourMode != DRAW_COLOURMODE_32BIT_RGB)
        {
            // allocate convPicItemPtr->fbmHeaderPtr if needed
            convPicItemPtr->fbmHeaderPtr = (DRAW_FbmHeaderT*)UTILS_Malloc(sizeof(DRAW_FbmHeaderT));
            if (convPicItemPtr->fbmHeaderPtr == NULL)
            {
                DRAW_FreeFbm(picItemPtr);
                UTILS_Free(picItemPtr);
                DRAW_FreeFbm(convPicItemPtr);
                UTILS_Free(convPicItemPtr);
                decodeInfo_event.infoType  = JPEG_DECODEINFO_LOW_MEMORY;
                errorMsgShow = FTRUE;
                break;
            }
            convPicItemPtr->allocInformation |= 0x0001;
            convPicItemPtr->fbmHeaderPtr->colourMode = DRAW_COLOURMODE_32BIT_RGB;
        }

        /* check for the maximum possible size to display pic in OSD based on the memory size */
        if ( FAPI_MMU_GetSegmentInfo(FAPI_MMU_HeapHandleSys0, &segInf) == FAPI_OK )
        {
            free_uncached_memory = segInf.bytesAvailable;
        }

        temp = 0;

        //FIXME: only a workaround. A new API is required to caculate the maximum possible malloc memory.
        free_uncached_memory = (free_uncached_memory * 7)/10;

        if(picItemPtr->fbmHeaderPtr->colourMode != DRAW_COLOURMODE_32BIT_RGB)
        {
            /* additioanal memory needed for non-32bitRGB pic during scaling */
            temp = (uint32_t)((picItemPtr->fbmHeaderPtr->width) *
                    (picItemPtr->fbmHeaderPtr->height) * sizeof(uint32_t));
            if (free_uncached_memory > temp)
            {
                free_uncached_memory -= temp;
            }
            else
            {
                DRAW_FreeFbm(picItemPtr);
                UTILS_Free(picItemPtr);
                DRAW_FreeFbm(convPicItemPtr);
                UTILS_Free(convPicItemPtr);
                decodeInfo_event.infoType  = JPEG_DECODEINFO_LOW_MEMORY;
                errorMsgShow = FTRUE;
                break;
            }
        }

        /* total memory needed for displaying pic in full screen */
        temp = (uint32_t)((inst->osd.params.inputWidth) *
                (inst->osd.params.inputHeight) * sizeof(uint32_t))
                 + sizeof(DRAW_FbmHeaderT);

        /* dynamic set the maximum and minimum boundry of the showing
           pic based on memory */
        if ( free_uncached_memory > temp )
        {
            if ( ((width*1000)/(height)) > ((inst->osd.params.inputWidth*1000)/(inst->osd.params.inputHeight)) )
            {
                max_width = inst->osd.params.inputWidth;
                max_height = (inst->osd.params.inputWidth * height) / width;
            }
            else
            {
                max_height = inst->osd.params.inputHeight;
                max_width = (inst->osd.params.inputHeight * width) / height;
            }
        }
        else
        {
            if ( width > height )
            {
                ratio = (width * 1000) / height;
                max_height = (uint32_t)sqrt((uint32_t)(((free_uncached_memory)*1000)/(ratio * sizeof(uint32_t))));
                max_width = (max_height * ratio)/1000;
            }
            else
            {
                ratio = (height * 1000) / width;
                max_width = (uint32_t)sqrt((uint32_t)(((free_uncached_memory)*1000)/(ratio * sizeof(uint32_t))));
                max_height = (max_width * ratio)/1000;
            }
        }

        min_width = max_width/2;
        min_height = max_height/2;

        /* case 1: display the pic in orignal size if the the orignal size is
                   within the boundary
           case 2: dispaly the pic in upper boundary size if the orignal size is
                   greater than maximum
           case 3: dispaly the pic in lower boundary size if the orignal size is
                   smaller than mimum */
        if( ( width < max_width ) && ( height < max_height ))
        {
            if ( (width > min_width)&&(height > min_height) )
            {
                scale_width = width;
                scale_height = height;
            }
            else
            {
                scale_width = min_width;
                scale_height = min_height;
            }
        }
        else
        {
            scale_width = max_width;
            scale_height = max_height;
        }

        /* rescale the picture according to 32bit RGB */
        retVal = DRAW_ScalePicture(picItemPtr,
                                   convPicItemPtr,
                                   scale_width,
                                   scale_height);

        if ( retVal )
        {
            DRAW_FreeFbm(picItemPtr);
            UTILS_Free(picItemPtr);
            DRAW_FreeFbm(convPicItemPtr);
            UTILS_Free(convPicItemPtr);
            if ( retVal == DRAW_ERR_OUT_OF_MEMORY )
            {
                decodeInfo_event.infoType  = JPEG_DECODEINFO_LOW_MEMORY;
            }
            else
            {
                decodeInfo_event.infoType  = JPEG_DECODEINFO_ERROR;
            }
            errorMsgShow = FTRUE;
            break;
        }

        #ifdef PNG_INTSHOW_DBG
        FAPI_SYS_PRINT_DEBUG(1,"\n[PNG] DRAW_ScalePicture %d \n", retVal);
        #endif

        DRAW_FreeFbm(picItemPtr);
        UTILS_Free(picItemPtr);

        /* draw picture on OSD */
        retVal = DRAW_Fbm(inst->osd.layerIdx,
                 (inst->osd.params.inputWidth - scale_width)/2,
                 (inst->osd.params.inputHeight - scale_height)/2,
                 convPicItemPtr->fbmHeaderPtr,
                 convPicItemPtr->dataPtr);

        if ( retVal )
        {
            DRAW_FreeFbm(convPicItemPtr);
            UTILS_Free(convPicItemPtr);
            if ( retVal == DRAW_ERR_OUT_OF_MEMORY )
            {
                decodeInfo_event.infoType  = JPEG_DECODEINFO_LOW_MEMORY;
            }
            else
            {
                decodeInfo_event.infoType  = JPEG_DECODEINFO_ERROR;
            }
            errorMsgShow = FTRUE;
            break;
        }
        #ifdef PNG_INTSHOW_DBG
        FAPI_SYS_PRINT_DEBUG(1,"\n[PNG] DRAW_Fbm %d \n", retVal);
        #endif

        DRAW_FreeFbm(convPicItemPtr);
        UTILS_Free(convPicItemPtr);
    }
    while(0);

    if (errorMsgShow )
    {
        decodeInfo_event.hdr.type = EVT_JPEG_DECODEINFO;
        decodeInfo_event.hdr.length = sizeof(decodeInfo_event);
        (void)EVT_Send (&decodeInfo_event,"Message Box Info");
    }
}
FSTATIC void JPEG_IntShow (JPEG_DATA_S* inst,
                           const JPEG_PIC_PARAMS_S* picParams)
{
    JPEG_t                  jpeg;
    int32_t                 error;
    JPEG_EVT_INFO_S         event;
    JPEG_EVT_DECODEINFO_S   decodeInfo_event;
    char_t                  again;

    event.hdr.type   = EVT_JPEG_FILEINFO;
    event.hdr.length = sizeof(event);
    event.fileType   = PIC_TYPE_JPEG;

    error = f_seek(inst->fileP, picParams->filepos, F_SEEK_SET);

    jpeg.env.staticP = UTILS_Malloc(32768);
    // KK: refer to JPEG_api.h at least 32 kByte = 32*1024 = 32768 Byte

    if(jpeg.env.staticP)
    {
        jpeg.env.screen    = SCREEN_HD;
        jpeg.env.userDataP = inst;
        jpeg.env.loadFct   = JPEG_Load;
        jpeg.dest.bgcolor  = 0x80108010;// black in YUV 4:2:2 color space

        error = JPEG_ReadHeader(&jpeg);
        if ( !error )
        {
            event.sizeX = jpeg.info.sizeX;
            event.sizeY = jpeg.info.sizeY;

            (void)EVT_Send (&event,"JPEG Info");

            if ( !(inst->itcInst.abortReq) )
            {
                jpeg.dest.right    =inst->osd.params.inputWidth;
                jpeg.dest.bottom   =inst->osd.params.inputHeight;
                jpeg.dest.screenP  =(uint8_t*)inst->currLayerParams.pictureBase;
                jpeg.dest.rowlongs =inst->osd.params.inputWidth/2;
                jpeg.dest.rowlongs =(jpeg.dest.rowlongs + 31) & ~31;
                // rowlongs has to be a multiple of 32 for H60 OSD memory
                // accessing (128 byte alignment)

                jpeg.image.orient  =picParams->orient;
                jpeg.image.fast    =(char_t)FTRUE;
                jpeg.env.temporalP =UTILS_Malloc(jpeg.env.temporalbytes);

                #ifdef JPEG_INTSHOW_DBG
                if (jpeg.env.temporalP==NULL)
                    FAPI_SYS_PRINT_DEBUG(1,
                            "[JPEG] UTILS_Malloc failed, out of memory \n");
                #endif

                if ( jpeg.env.temporalP )
                {
                    if ( picParams->zoom != 100 )
                    {
                        JPEG_CalcZoom (picParams, &jpeg);
                    }

                    error = JPEG_StartDecode(&jpeg);
                    #ifdef JPEG_INTSHOW_DBG
                    if(error!=0)
                        FAPI_SYS_PRINT_DEBUG(1,
                            "[JPEG] JPEG_StartDecode failed, %d\n",error);
                    #endif
                    if ( !error )
                    {
                        do
                        {
                            if ( inst->itcInst.abortReq ) break;

                            again = (char_t)JPEG_Continue(&jpeg);

                        } while ( again );

                        JPEG_FinishDecode(&jpeg);
                    }

                    FAPI_SYS_FREE(jpeg.env.temporalP);
                }
                else
                {
                    decodeInfo_event.hdr.type = EVT_JPEG_DECODEINFO;
                    decodeInfo_event.hdr.length = sizeof(decodeInfo_event);
                    decodeInfo_event.infoType  = JPEG_DECODEINFO_LOW_MEMORY;
                    (void)EVT_Send (&decodeInfo_event,"Message Box Info");
                }
            }
        }

        FAPI_SYS_FREE(jpeg.env.staticP);
    }
    else
    {
        decodeInfo_event.hdr.type = EVT_JPEG_DECODEINFO;
        decodeInfo_event.hdr.length = sizeof(decodeInfo_event);
        decodeInfo_event.infoType  = JPEG_DECODEINFO_LOW_MEMORY;
        (void)EVT_Send (&decodeInfo_event,"Message Box Info");
    }
}

/* ------------------------------------------------------------------------- */

FSTATIC JPEG_FILETYPE_E JPEG_FileCheck (const JPEG_PIC_PARAMS_S* picParams)
{
    JPEG_FILETYPE_E     fileType;
    int32_t             retVal = FAPI_OK;
    uint8_t             buffer[8];
    F_FILE *            fileP = NULL;

    fileP = f_open(picParams->path, "r");

    /* start of image SOI check*/
    (void)f_seek(fileP, picParams->filepos, SEEK_SET);
    retVal = f_read(buffer, 1, sizeof(buffer), fileP);
    if (retVal != (int32_t)sizeof(buffer))
    {
        fileType = JPEG_FILE_UNKNOWN;
    }

    else if((buffer[0] == 0xFF) && (buffer[1] == 0xD8))
    {
        fileType = JPEG_FILE_JPG;
    }
    else if ((buffer[0] == 0x42) && (buffer[1] == 0x4D))
    {
        fileType = JPEG_FILE_BMP;
    }

    else if ((buffer[0] == 0x89) && (buffer[1] == 0x50)
            && (buffer[2] == 0x4E) && (buffer[3] == 0x47)
            && (buffer[4] == 0x0D) && (buffer[5] == 0x0A)
            && (buffer[6] == 0x1A) && (buffer[7] == 0x0A))
    {
        fileType = JPEG_FILE_PNG;
    }
    else
    {
        fileType = JPEG_FILE_UNKNOWN;
    }

    retVal = f_close(fileP);
    if (retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1,"[JPEG] f_close failed: %s\n", picParams->path);
    }

    return fileType;
}

/*!
 ******************************************************************************
 * \brief Internal BMP show function
 *
 * \param  inst     TODO
 * \param  screen   TODO
 *
 *****************************************************************************/

FSTATIC void BMP_IntShow (JPEG_DATA_S* inst, const JPEG_SCREEN_PARAMS_S* screen)
{
    uint8_t                 data[54];
    JPEG_EVT_INFO_S         event;
    JPEG_EVT_DECODEINFO_S   decodeInfo_event;
    uint16_t                bitsperpix;
    int16_t                 picSizeX, picSizeY;
    uint32_t                base;
    uint8_t*                buffer      = NULL;
    uint8_t*                bufferPtr   = NULL;
    uint8_t*                divtabP     = NULL;
    uint8_t*                divP;
    uint16_t                ii, nn, rr;
    BMP_SCALER_S            scalerP;
    int32_t                 retVal      = FAPI_OK;
    uint32_t                rowBytes;
    uint32_t                dataSize  = 0;
    uint32_t                blockSize = 0;
    uint32_t                readRows  = 0;
    uint32_t                rows      = 0;

    uint32_t                chunckIdx = 1;

    int32_t                 startAdd = 0;
    const int8_t            bytePerPixel = 2;

    //! top and left for centralized picture
    int16_t                 centralX,centralY;

    #ifdef BMP_READTIME_DBG
    FSTATIC uint64_t timer_start;
    FSTATIC uint64_t timer_stamp;
    #endif /* BMP_READTIME_DBG */

    do
    {
        retVal = f_read(data, 1, 54, inst->fileP);
        if (retVal != 54) break;

        if((data[0] != 'B') || (data[1] != 'M')) break;

        base = (data[13] << 24) | (data[12] << 16) | (data[11] << 8) | data[10];
        picSizeX = (data[19] << 8) | data[18];
        picSizeY = (data[23] << 8) | data[22];
        if(picSizeY < 0)
        {
            picSizeY= -picSizeY;
        }
        bitsperpix= (data[29] << 8) | data[28];

        retVal = f_seek(inst->fileP, (int32_t) base, F_SEEK_SET);
        dataSize = (data[37] << 24) | (data[36] << 16) | (data[35] << 8) | data[34];

        //send event for displaying picture info
        event.hdr.type   = EVT_JPEG_FILEINFO;
        event.hdr.length = sizeof(event);
        event.sizeX      = (uint16_t)picSizeX;
        event.sizeY      = (uint16_t)picSizeY;
        event.bitsperpix = bitsperpix;
        event.fileType   = PIC_TYPE_BMP;

        (void)EVT_Send (&event, "BMP Info");
        if ((picSizeX >OSDHANDLER_MAX_HRES)||(picSizeY > OSDHANDLER_MAX_VRES))
            break;

        //check bit per pixel
        if(bitsperpix != 24)
        {
            FAPI_SYS_PRINT_DEBUG(1,"[JPEG] The file is not a 24-bit/pixel BMP picture\n");
            return;
        }

        //calculate number of bytes per line
        if(dataSize != 0)
        {
            rowBytes = (dataSize / (uint32_t)picSizeY); //or ((picSizeX*3)+3)& ~3; a multiple of 4 bit
        }
        else
        {
            rowBytes = ((picSizeX * BMP24_BYTE_PER_PIXEL)+3)& ~3;
        }

        //malloc for read buffer
        buffer = (uint8_t*)UTILS_Malloc(JPEG_CHUNK_SIZE);  //for input row data
        if(buffer == NULL)
        {
            FAPI_SYS_PRINT_DEBUG(1,"[JPEG] out of memory\n");
            decodeInfo_event.hdr.type = EVT_JPEG_DECODEINFO;
            decodeInfo_event.hdr.length = sizeof(decodeInfo_event);
            decodeInfo_event.infoType  = JPEG_DECODEINFO_LOW_MEMORY;
            (void)EVT_Send (&decodeInfo_event,"Message Box Info");
            break;
        }


        //malloc for divide table
        divtabP= UTILS_Malloc( 256*45); //memory for a division table
                                        //45 is a factorial of 9 = 9!
        if(divtabP == NULL)
        {
            FAPI_SYS_PRINT_DEBUG(1,"[JPEG] out of memory\n");
            break;
        }

        //initualize divide table and YUV buffer
        base= 0;
        for(ii= 1; ii<10; ii++)
        {   scalerP.div[ii]= &((uint8_t*)divtabP)[base];
            base= base + (ii*256);
        }

        for(ii= 1; ii<10; ii++)
        {   divP= scalerP.div[ii];
            for(nn= 0; nn<256; nn++)
                for(rr= 0; rr < ii; rr++)
                    *divP++= (uint8_t)nn;
        }

        for(ii= 0; ii<1920; ii++)
        {
            scalerP.Y[ii]= 0x00;
            scalerP.U[ii]= 0x00;
            scalerP.V[ii]= 0x00;
        }

        //setup scaler parameters
        scalerP.accuY    = 0;
        scalerP.sizeX    = inst->osd.params.inputWidth;
        scalerP.sizeY    = (int32_t)inst->osd.params.inputHeight;
        //rowlongs is for OSD memory alignment
        scalerP.rowlongs = (inst->osd.params.inputWidth*bytePerPixel)/4;
        scalerP.rowlongs = (scalerP.rowlongs + 31) & ~31; //align to 128 bytes
        scalerP.srcwidth = picSizeX;
        scalerP.countY   = 0;
        scalerP.ratioX   = (int32_t)(((uint32_t)inst->osd.params.inputWidth
                                                 << 16) / (uint32_t)picSizeX);
        scalerP.ratioY   = (int32_t)(((uint32_t)inst->osd.params.inputHeight
                                                << 16) / (uint32_t)picSizeY);

        //centralize picture in full screen view
        if(screen==NULL)
        {
            //calculate top and left position of centralized picture when
            // picture smaller than OSD size
            if(((int16_t)scalerP.sizeX > picSizeX)&&
               ((int16_t)scalerP.sizeY > picSizeY))
            {
                centralX = ((int16_t)scalerP.sizeX - picSizeX)/2;
                centralY = ((int16_t)scalerP.sizeY - picSizeY)/2;

                // startAdd of centralized picture is given in byte
                startAdd = (centralY*(int32_t)scalerP.rowlongs*4)+(centralX*2);
                startAdd = (startAdd + 3) & ~3; //4 bytes alignment
            }
        }

        //give the draw postion in OSD buffer
        scalerP.screenP= (volatile uint32_t*)((uint32_t)startAdd+
                                    inst->currLayerParams.pictureBase);

        ii = 1;
        blockSize   = ((int32_t)JPEG_CHUNK_SIZE / rowBytes)*rowBytes;

        //reads BMP raw data and performs scaling for each line
        for(readRows = 1; readRows > 0; chunckIdx++)
        {
            readRows = BMP_BlockRead(inst->fileP, buffer, blockSize,
                                    dataSize, chunckIdx);
            readRows /= rowBytes;

            for(rows = readRows; rows>0; rows--)
            {
                bufferPtr = buffer+ ((rows-1)*rowBytes);
                BMP_ScaleLine(ii, bufferPtr, &scalerP);
                ii++;
                if(ii > (uint16_t)picSizeY)
                {
                    break;
                }
            }
            if(inst->itcInst.abortReq) break;
        }
    }
    while(0);

    FAPI_SYS_FREE(divtabP);
    FAPI_SYS_FREE(buffer);

    return;
}
/*!
 ******************************************************************************
 * \brief This function read a block size of BMP raw data from the end of
 *        BMP file. It read through a whole file by increasing read index.
 *
 *
 * \param   srcFile         source file
 * \param   destBuffer      destination buffer
 * \param   blockSize       blcok size of read
 * \param   fileSize        maximum size of read
 * \param   readIdx         read index
 *
 * \return  number of read bytes
 *
 *****************************************************************************/
//
FSTATIC uint32_t BMP_BlockRead (F_FILE*  srcFile,
                                uint8_t* destBuffer,
                                uint32_t blockSize,
                                uint32_t fileSize, //total read size
                                uint32_t readIdx) // start from 1
{
    int32_t readByte    = 0;
    int32_t readSize    = 0;
    int32_t readPos     = 0;
    int32_t level       = (int32_t)fileSize-(int32_t)(blockSize*readIdx);
    int32_t retVal      = FAPI_OK;


    for(;;)
    {
        if ((srcFile    == 0)   ||
            (destBuffer == NULL)||
            (blockSize  == 0))
        {
            FAPI_SYS_PRINT_DEBUG(2,"[BMP] BMP_BlockRead failed\n");
            break;
        }
        if(level <=0)
        {
            /* stop condition */
            if ((-level) >= (int32_t)blockSize)
            {
                break;
            }
            readSize = (int32_t)fileSize - (int32_t)(blockSize*(readIdx-1));
            readPos  = (int32_t)fileSize;
        }
        else
        {
            readSize = (int32_t)blockSize;
            readPos  = (int32_t)(blockSize*readIdx);
        }

        //read from the EOF, for the picture is stored upside-down
        retVal = f_seek(srcFile, -readPos, F_SEEK_END);
        if(retVal!= F_NO_ERROR)
        {
            break;
        }

        retVal = f_read(destBuffer, 1, (int32_t)readSize, srcFile);
        if( retVal != readSize )
        {
            break;
        }
        else
        {
            readByte = retVal;
        }

        break; /* end of for(;;)*/
    }

    return (uint32_t)readByte;
}


FSTATIC void BMP_ScaleLine(uint16_t scanline, uint8_t* rawP,
                                                         BMP_SCALER_S* scalerP)
{
    int32_t                         Y= 0;
    int32_t                         U= 0;
    int32_t                         V= 0;
    int32_t                         R, G, B;
    uint32_t                        accuH= 0;
    int32_t                         countH= 0;
    uint32_t                        iVbuf= 0;
    uint32_t                        ii;
    uint8_t *                       divP;
    uint32_t                        rowLimit = 0;

    scanline = scanline;

    if(scalerP->sizeY < 0)
        return;

    ii= 0;
    while(ii < (uint32_t)scalerP->srcwidth)
    {
        B= rawP[(ii*3)+0];
        G= rawP[(ii*3)+1];
        R= rawP[(ii*3)+2];

        Y= Y + (int32_t)((uint32_t)(4096 + (66*R) + (129*G) + (25*B)) >> 8);
        U= U + (int32_t)((uint32_t)(((32768 + (112*B))- (38*R)) - (74*G)) >> 8);
        V= V + (int32_t)((uint32_t)(((32768 + (112*R)) - (94*G)) - (18*B)) >> 8);
        ii++;
        countH++;
        accuH= accuH + (uint32_t)scalerP->ratioX;
        if((accuH > 0x0FFFF) || (ii == (uint32_t)scalerP->srcwidth))
        {
            divP= scalerP->div[countH];

            #ifdef BMP_DEC_DEBUG
            FAPI_SYS_PRINT_DEBUG(1,
            "[1]ii=%d, YUV= %3d, %3d, %3d; countH=%d,divP[YUV]= %3d, %3d, %3d\n",
             ii, Y, U, V, countH,divP[Y],divP[U], divP[V]);
            #endif /* BMP_DEC_DEBUG */

            scalerP->Y[iVbuf]= scalerP->Y[iVbuf] + divP[Y];
            scalerP->U[iVbuf]= scalerP->U[iVbuf] + divP[U];
            scalerP->V[iVbuf]= scalerP->V[iVbuf] + divP[V];

            Y= 0;
            U= 0;
            V= 0;
            iVbuf++;
            accuH= accuH - 0x10000;
            countH= 0;
            if(iVbuf >= scalerP->sizeX)
                break;
        }
        #ifdef BMP_DEC_DEBUG
        else
        {
            FAPI_SYS_PRINT_DEBUG(1,"[BMP] ii=%d, YUV= %3d, %3d, %3d\n", ii, Y, U, V);
        }
        #endif // BMP_DEC_DEBUG end
    }
    scalerP->countY++;
    scalerP->accuY= scalerP->accuY + scalerP->ratioY;
    if((scalerP->accuY > 0x0FFFF) || (scalerP->sizeY == 0))
    {
        divP= scalerP->div[scalerP->countY];
        if ((uint16_t)scalerP->srcwidth >= scalerP->sizeX)
        {
            rowLimit = scalerP->sizeX;
        }
        else
        {
            rowLimit = (uint32_t)scalerP->srcwidth;
        }

        for(ii= 0; ii<(rowLimit>>1); ii++)
        {
            #ifdef BMP_DEC_DEBUG
            FAPI_SYS_PRINT_DEBUG(1,
            "[2]ii=%d,countY=%d; accuY=%5d, divP[UYVY']=%3x, %3x, %3x, %3x; \n",
                     ii, scalerP->countY,
                     scalerP->accuY,divP[scalerP->U[ii*2]],
                     divP[scalerP->Y[ii*2]],divP[scalerP->V[ii*2]],
                     divP[scalerP->Y[(ii*2)+1]]);
            #endif /* BMP_DEC_DEBUG */

            scalerP->screenP[ii]= ((uint32_t)(((uint32_t)((divP[scalerP->U[ii*2]]  << 8) |
                                      divP[scalerP->Y[ii*2]]) << 8) |
                                      divP[scalerP->V[ii*2]]) << 8) |
                                      divP[scalerP->Y[(ii*2)+1]];
            scalerP->U[ii*2]= 0;
            scalerP->V[ii*2]= 0;
            scalerP->Y[ii*2]= 0;
            scalerP->Y[(ii*2)+1]= 0;
        }

        scalerP->screenP= &scalerP->screenP[scalerP->rowlongs];
                                                //move osd memory pointer
        scalerP->countY = 0;
        scalerP->accuY  = scalerP->accuY - 0x10000;
        scalerP->sizeY--;
    }
    #ifdef BMP_DEC_DEBUG
    else
    {
        FAPI_SYS_PRINT_DEBUG(1,"[JPEG] [2]ii=%d,countH=%d; accuY=%5d\n",ii,
                                            scalerP->countY, scalerP->accuY);
    }
    #endif // BMP_DEC_DEBUG end
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
