/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/key_input.c
**
**              $Id: key_input.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Key input module.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info 715: Symbol not referenced
**
*****************************************************************************
*/
/*lint -save -e715 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#ifndef _LINUX_FAPI
#include <fapi/sys_driver.h>
#include <fapi/drv_uart.h>
#include <fapi/drv_ir.h>
#include <fapi/drv_ir_nec.h>
#include <fapi/drv_mmu.h>
#include <rtos/rtos.h>
#else
#include <fapex/abst_types.h>
#include <fapex/abst_lib.h>
#include <fapex/abst_libir.h>
#endif

#include "init.h"
#include "event_queue.h"
#include "key_input.h"
#include "av.h"
#include "settings.h"

#include "utils.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define KEYIN_POLL_INTERVAL          60 // polling interval for console or
                                            // front panel key events

/*---------------------------------------------------------------------------*/
/* external, interim functions                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC uint64_t     lastKeyTime;
FSTATIC fbool_t      fpInitialised = FFALSE;
FSTATIC uint16_t     repIntervalFirst = 0;
FSTATIC uint16_t     repIntervalOthers = 0;
FSTATIC uint16_t     repCount = 0;
FSTATIC FAPI_SYS_HandleT uartHandle;

#ifdef _LINUX_FAPI
static int32_t fd_IR0      = -1;
#endif
/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/


FSTATIC void        KEYIN_ThreadLoop(void *);
FSTATIC uint16_t    getConsoleKey(void);

#ifndef RC_VIA_FP
FSTATIC void        KEYIN_RcuNotifyFunction(FAPI_SYS_HandleT handle, uint32_t* optDataPtr);
#endif /* RC_VIA_FP */

#ifdef TARGET_FUJITSU_STB
FSTATIC uint16_t    getKeypadKey(void);
#endif /* TARGET_FUJITSU_STB */

static KEYIN_SPECIAL_HOTKEY_CALLBACK  SpecialHotKeyCbf;

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/
void KEYIN_RegisterSpecialHotkeyCallback(KEYIN_SPECIAL_HOTKEY_CALLBACK Cbf)
{
    SpecialHotKeyCbf = Cbf;
}

int32_t KEYIN_Init(void)
{
#ifndef _LINUX_FAPI
    FAPI_UART_OpenParamsT uartParams;
#endif
    #ifdef TARGET_FUJITSU_STB
    FD_FP_INIT_PARAMS_S fpInitParams;
    #endif /* TARGET_FUJITSU_STB */

    int32_t             retVal = FAPI_OK;
    #ifndef RC_VIA_FP
#ifndef _LINUX_FAPI
    FAPI_IR_OpenParamsT   open_params;
#else
    FAPEX_IR_OpenParamStrT   open_params;
#endif
    #endif /* RC_VIA_FP */

    lastKeyTime = 0;
    fpInitialised     = FFALSE;
    repIntervalFirst = KEYIN_REP_INTERVAL_FIRST;
    repIntervalOthers = KEYIN_REP_INTERVAL_OTHERS;
    repCount = 0;

    #ifdef TARGET_FUJITSU_STB

    /* STB is equipped with 7-segment display */

    /* init front panel driver (STB) */
    fpInitParams.displayBrightness = 0xFF;
    fpInitParams.displayInvertSignals = FTRUE;

    retVal = FD_FP_Init(&fpInitParams);
    if(retVal != FAPI_OK)
    {
        return retVal;
    }
    else
    {
        /* start with standby-display */
        FD_FP_DisplayStandby();
    }

    fpInitialised = FTRUE;

    #endif /* TARGET_FUJITSU_STB */

    #ifndef RC_VIA_FP
#ifndef _LINUX_FAPI
    open_params.version           = FAPI_IR_VERSION;
    open_params.swProtocol      = FAPI_IR_NEC_Mode; /* NEC protocol callback function  */
    open_params.rcmNotifyFunction = KEYIN_RcuNotifyFunction;
    (void)FAPI_IR_Open(&open_params, &retVal); /* handel is not needed */
    if( retVal != FAPI_OK)
#else
    open_params.mode      = FAPEX_IR_MODE_NEC; /* NEC protocol callback function  */
    fd_IR0 = FAPEX_IR_Open(FAPEXDEV_IR_MINOR_FIRST,O_RDWR, (FAPEXOPENARG3_T)&open_params);
    if ( fd_IR0 < 0)
#endif

    {
        /** TODO error handling */
    }
    #endif  /* RC_VIA_FP */

    /* create polling thread for keyboard and front panel input */
    retVal = SYS_ThreadCreate(KEYIN_ThreadLoop, NULL,
                              THREAD_PRIO_INPOLL,
                              THREAD_STACKSZ_INPOLL, THREAD_NAME_INPOLL, NULL);
    if( retVal != FAPI_OK)
    {
        /** TODO error handling */
    }
#ifndef _LINUX_FAPI
    uartParams.version = FAPI_UART_VERSION;
    uartParams.index   = 0;
    uartHandle = FAPI_UART_Open(&uartParams, &retVal);
    if( retVal != FAPI_OK)
    {
        /** TODO error handling */
    }
#else
    uartHandle = open("/dev/ttyAMA0", O_RDWR);
    if( uartHandle < 0)
    {
        /** TODO error handling */
    }
#endif
    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Set the minimum interval for "repeated key"-events.
**
** All remote key events arriving within the specified minimum time interval
** are discarded.
**
** \param  first      Minimum interval for first "repeated key"-event in ms.
** \param  others     Minimum interval for subsequent "repeated key"-events
**                    in ms.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa KEYIN_GetRepeatInterval
******************************************************************************/
int32_t KEYIN_SetRepeatInterval(uint16_t first, uint16_t others)
{
    repIntervalFirst = first;
    repIntervalOthers = others;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get the current minimum interval for "repeated key"-events.
**
** \param  first      Current minimum interval for first "repeated key"-event
**                    in ms is returned here.
** \param  others     Current minimum interval for subsequent "repeated key"-
**                    events in ms is returned here.
**
** \return -
**
** \sa KEYIN_SetRepeatInterval
******************************************************************************/
void KEYIN_GetRepeatInterval(uint16_t * first, uint16_t * others)
{
    (*first) = repIntervalFirst;
    (*others) = repIntervalOthers;
}



fbool_t KEYIN_FrontPanelIsInitialised(void)
{
    return fpInitialised;
}

/*lint -save -e715 */
/* optional data pointer is not used in this thread */
FSTATIC void KEYIN_ThreadLoop(void *opt)
{
    /* NOTE: Stack is of limited size! */
    uint16_t            key;
    KEYIN_EVT_KEY_S     ctrlEvent;

    #ifdef RC_VIA_FP

    int32_t             retVal;
    FD_GHW_RC_INFO      fpData;

    #endif /* RC_VIA_FP */

    ctrlEvent.hdr.type = EVT_KEY;
    ctrlEvent.hdr.length = sizeof(ctrlEvent);

    for(;;)
    {
#ifndef _LINUX_FAPI
        (void)RTOS_Sleep(KEYIN_POLL_INTERVAL);
#else
        FAPI_SYS_SLEEP(KEYIN_POLL_INTERVAL);
#endif
        #ifdef RC_VIA_FP

        retVal = FD_GHW_ControlRc(0, FD_GHW_RC_POLL_TASK, &fpData);

        if((retVal == FAPI_OK) &&
           (fpData.ActPendingKeyCode != FAPI_IR_KEY_UNDEFINED))
        {
            ctrlEvent.source = KEY_SOURCE_RC;
            ctrlEvent.val = fpData.ActPendingKeyCode;
            ctrlEvent.type = (fpData.FlgRepeatedKey != 0) ?
                KEYIN_KEY_REPEATED : KEYIN_KEY_PRESSED;

            (void)EVT_Send(&ctrlEvent, "RC input");
        }

        #endif /* RC_VIA_FP */

        #ifdef TARGET_FUJITSU_STB

        if ( fpInitialised == FTRUE )
        {
            /* check front panel */
            key = getKeypadKey();
#ifndef _LINUX_FAPI
            if(key != FAPI_IR_KEY_UNDEFINED)
#else
            if(key != FAPEX_IR_KEY_UNDEFINED)
#endif
            {
                ctrlEvent.source = KEY_SOURCE_FRONT;
                ctrlEvent.val = key;
                ctrlEvent.type = KEYIN_KEY_PRESSED;

                (void)EVT_Send(&ctrlEvent, "Frontpad input");
            }
        }

        #endif /* TARGET_FUJITSU_STB */

        /* check serial input (keyboard) */
        key = getConsoleKey();
#ifndef _LINUX_FAPI
        if(key != FAPI_IR_KEY_UNDEFINED)
#else
        if(key != FAPEX_IR_KEY_UNDEFINED)
#endif
        {
            ctrlEvent.source = KEY_SOURCE_CONSOLE;
            ctrlEvent.val = key;
            ctrlEvent.type = KEYIN_KEY_PRESSED;

            (void)EVT_Send(&ctrlEvent, "Console input");
        }
    }
}
/*lint -restore */

/* map received char_t from console to RC key code */
FSTATIC uint16_t getConsoleKey(void)
{
    uint16_t            k;
    uint16_t            rcKey;
#ifdef _LINUX_FAPI
    int32_t   i = 0;
    char_t    args_buff[255];
#endif

#ifndef _LINUX_FAPI
    rcKey = FAPI_IR_KEY_UNDEFINED;
#else
    rcKey = FAPEX_IR_KEY_UNDEFINED;
#endif

#ifndef _LINUX_FAPI
    if(FAPI_UART_CheckRxFifo(uartHandle) == 1)
    {
        k = (uint16_t)FAPI_UART_ReadByte(uartHandle);
#else
    args_buff[i] = '\0';

    /* get command key */
    {
        int32_t len = 4;
        FAPEX_SYS_KEYIN( &args_buff[0], &len );
        k = args_buff[0];
    }
    if( args_buff[0] != '\0' )
    {
#endif
        switch ((char_t) k)
        {
#ifndef _LINUX_FAPI
            case '0':
                rcKey = FAPI_IR_KEY_0;
                break;
            case '1':
                rcKey = FAPI_IR_KEY_1;
                break;
            case '2':
                rcKey = FAPI_IR_KEY_2;
                break;
            case '3':
                rcKey = FAPI_IR_KEY_3;
                break;
            case '4':
                rcKey = FAPI_IR_KEY_4;
                break;
            case '5':
                rcKey = FAPI_IR_KEY_5;
                break;
            case '6':
                rcKey = FAPI_IR_KEY_6;
                break;
            case '7':
                rcKey = FAPI_IR_KEY_7;
                break;
            case '8':
                rcKey = FAPI_IR_KEY_8;
                break;
            case '9':
                rcKey = FAPI_IR_KEY_9;
                break;

            case 'a':
                rcKey = FAPI_IR_KEY_APPL;
                break;
            case 'b':
                rcKey = FAPI_IR_KEY_BACK;
                break;
            case 'h':
                rcKey = FAPI_IR_KEY_HIDE;
                break;
            case 'i':
                rcKey = FAPI_IR_KEY_INFO;
                break;
            case 'm':
                rcKey = FAPI_IR_KEY_MENU;
                break;
            case 'n':
                rcKey = FAPI_IR_KEY_NAVI;
                break;
            case 'e':
                rcKey = FAPI_IR_KEY_EXIT;
                break;
            case 't':
                rcKey = FAPI_IR_KEY_TEXT;
                break;
            case '+':
                rcKey = FAPI_IR_KEY_CHANNEL_UP;
                break;
            case '-':
                rcKey = FAPI_IR_KEY_CHANNEL_DOWN;
                break;
            case '>':
                rcKey = FAPI_IR_KEY_VOL_UP;
                break;
            case '<':
                rcKey = FAPI_IR_KEY_VOL_DOWN;
                break;

            case 'R':
                rcKey = FAPI_IR_KEY_RED;
                break;
            case 'G':
                rcKey = FAPI_IR_KEY_GREEN;
                break;
            case 'Y':
                rcKey = FAPI_IR_KEY_YELLOW;
                break;
            case 'B':
                rcKey = FAPI_IR_KEY_BLUE;
                break;

            case 'p':
                rcKey = FAPI_IR_KEY_PLAY;
                break;
            case 's':
                rcKey = FAPI_IR_KEY_STOP;
                break;
            case 'o':
                rcKey = FAPI_IR_KEY_PAUSE;
                break;
            case 'q':
                rcKey = FAPI_IR_KEY_STANDBY;
                break;
            case 'c':
                rcKey = FAPI_IR_KEY_REC;
                break;

                /* cursor keys (escape sequences) */
            case 0x1B:
                if(FAPI_UART_ReadByte(uartHandle) != 0x5B)
                {
                    break;
                }

                k = (uint16_t)FAPI_UART_ReadByte(uartHandle);

                if(k == 0x41)
                    rcKey = FAPI_IR_KEY_UP;
                else if(k == 0x42)
                    rcKey = FAPI_IR_KEY_DOWN;
                else if(k == 0x43)
                    rcKey = FAPI_IR_KEY_RIGHT;
                else if(k == 0x44)
                    rcKey = FAPI_IR_KEY_LEFT;
                break;

                /* enter key */
            case 0x0D:
                rcKey = FAPI_IR_KEY_OK;
                break;
#else
            case '0':
                rcKey = FAPEX_IR_KEY_0;
                break;
            case '1':
                rcKey = FAPEX_IR_KEY_1;
                break;
            case '2':
                rcKey = FAPEX_IR_KEY_2;
                break;
            case '3':
                rcKey = FAPEX_IR_KEY_3;
                break;
            case '4':
                rcKey = FAPEX_IR_KEY_4;
                break;
            case '5':
                rcKey = FAPEX_IR_KEY_5;
                break;
            case '6':
                rcKey = FAPEX_IR_KEY_6;
                break;
            case '7':
                rcKey = FAPEX_IR_KEY_7;
                break;
            case '8':
                rcKey = FAPEX_IR_KEY_8;
                break;
            case '9':
                rcKey = FAPEX_IR_KEY_9;
                break;

            case 'a':
                rcKey = FAPEX_IR_KEY_APPL;
                break;
            case 'b':
                rcKey = FAPEX_IR_KEY_BACK;
                break;
            case 'h':
                rcKey = FAPEX_IR_KEY_HIDE;
                break;
            case 'i':
                rcKey = FAPEX_IR_KEY_INFO;
                break;
            case 'm':
                rcKey = FAPEX_IR_KEY_MENU;
                break;
            case 'n':
                rcKey = FAPEX_IR_KEY_NAVI;
                break;
            case 'e':
                rcKey = FAPEX_IR_KEY_EXIT;
                break;
            case 't':
                rcKey = FAPEX_IR_KEY_TEXT;
                break;
            case '+':
                rcKey = FAPEX_IR_KEY_CHANNEL_UP;
                break;
            case '-':
                rcKey = FAPEX_IR_KEY_CHANNEL_DOWN;
                break;
            case '>':
                rcKey = FAPEX_IR_KEY_VOL_UP;
                break;
            case '<':
                rcKey = FAPEX_IR_KEY_VOL_DOWN;
                break;

            case 'R':
                rcKey = FAPEX_IR_KEY_RED;
                break;
            case 'G':
                rcKey = FAPEX_IR_KEY_GREEN;
                break;
            case 'Y':
                rcKey = FAPEX_IR_KEY_YELLOW;
                break;
            case 'B':
                rcKey = FAPEX_IR_KEY_BLUE;
                break;

            case 'p':
                rcKey = FAPEX_IR_KEY_PLAY;
                break;
            case 's':
                rcKey = FAPEX_IR_KEY_STOP;
                break;
            case 'o':
                rcKey = FAPEX_IR_KEY_PAUSE;
                break;
            case 'q':
                rcKey = FAPEX_IR_KEY_STANDBY;
                break;
            case 'c':
                rcKey = FAPEX_IR_KEY_REC;
                break;

                /* cursor keys (escape sequences) */
            case 0x1B:
                if(args_buff[1] != 0x5B)
                {
                    break;
                }
                k = args_buff[2];

                if(k == 0x41)
                    rcKey = FAPEX_IR_KEY_UP;
                else if(k == 0x42)
                    rcKey = FAPEX_IR_KEY_DOWN;
                else if(k == 0x43)
                    rcKey = FAPEX_IR_KEY_RIGHT;
                else if(k == 0x44)
                    rcKey = FAPEX_IR_KEY_LEFT;
                break;

                /* enter key */
            case 0x0A:
                rcKey = FAPEX_IR_KEY_OK;
                break;

#endif
            case 0x11:                      // CNTRL-Q
                if (SpecialHotKeyCbf) SpecialHotKeyCbf('Q');
                break;
            case 0x12:                      // CNTRL-R
                if (SpecialHotKeyCbf) SpecialHotKeyCbf('R');
                break;

//            case 0x01:                      // CNTRL-A
//                CISYS_TestEnableTsInput();
//                break;

            #ifdef APPL_DEBUG_MODE

            case 'd':
#ifndef _LINUX_FAPI
                RTOS_ReportThreadTable();
#endif
                break;

            case 'r':
#ifndef _LINUX_FAPI
                {
                    FAPI_MMU_HeapSegmentInfoT   segInf;

                    FAPI_SYS_PRINT_DEBUG(1,"\n");
                    FAPI_SYS_PRINT_DEBUG(1,"Available memory:\n");
                    FAPI_SYS_PRINT_DEBUG(1,"---------------- \n");

                    if ( FAPI_MMU_GetSegmentInfo(FAPI_MMU_HeapHandleSys0, &segInf) == FAPI_OK )
                    {
                        FAPI_SYS_PRINT_DEBUG(1,"System (uncached): %10d bytes\n",
                        segInf.bytesAvailable);
                    }

                    if ( FAPI_MMU_GetSegmentInfo(FAPI_MMU_HeapHandleSys1, &segInf) == FAPI_OK )
                    {
                        FAPI_SYS_PRINT_DEBUG(1,"System (cached):   %10d bytes\n",
                        segInf.bytesAvailable);
                    }

                    if ( FAPI_MMU_GetSegmentInfo(FAPI_MMU_HeapHandleDTcm, &segInf) == FAPI_OK )
                    {
                        FAPI_SYS_PRINT_DEBUG(1,"Int. TCM:          %10d bytes\n",
                        segInf.bytesAvailable);
                    }

                    if ( FAPI_MMU_GetSegmentInfo(FAPI_MMU_HeapHandleSRam, &segInf) == FAPI_OK )
                    {
                        FAPI_SYS_PRINT_DEBUG(1,"SRAM:              %10d bytes\n",
                        segInf.bytesAvailable);
                    }
                }
#endif
                break;

            case 'x':
#ifndef _LINUX_FAPI
                FAPI_MMU_ReportMemory( FAPI_MMU_HeapHandleSys0, 0 );
                FAPI_MMU_ReportMemory( FAPI_MMU_HeapHandleSys1, 0 );
                FAPI_MMU_ReportMemory( FAPI_MMU_HeapHandleDTcm, 0 );
                FAPI_MMU_ReportMemory( FAPI_MMU_HeapHandleSRam, 0 );
#endif
                break;

            #ifdef APPL_USB_ENABLED

            case '*':
                USB_Reconnect();
                break;

            #endif /* APPL_USB_ENABLED */

            #endif /* APPL_DEBUG_MODE */

            default:
                //FAPI_SYS_PRINT_DEBUG(1,"No RC code assigned to this console key %d!\n", k);
                break;
        }
        if( k == 0xff)
        {
            /* test mode
             * send 0xff + FAPI_IR_KEY_XX via UART0
             * e.g. 0xff84 for FAPI_IR_KEY_MENU
             */
#ifndef _LINUX_FAPI
            k = (uint16_t)FAPI_UART_ReadByte(uartHandle);
#else
            {
                int32_t len = 4;
                FAPEX_SYS_KEYIN( &args_buff[0], &len );
                k = args_buff[0];
            }
#endif
            rcKey = k;
        }
    }

    return rcKey;
}

#ifdef TARGET_FUJITSU_STB

/* map received keypad key to RC key code */
FSTATIC uint16_t getKeypadKey(void)
{
    uint16_t            rcKey;

    switch (FD_FP_GetPressedKey())
    {
        case FD_FP_KEY_MENU:
            rcKey = FAPI_IR_KEY_MENU;
            break;
        case FD_FP_KEY_OK:
            rcKey = FAPI_IR_KEY_OK;
            break;
        case FD_FP_KEY_UP:
            rcKey = FAPI_IR_KEY_UP;
            break;
        case FD_FP_KEY_DOWN:
            rcKey = FAPI_IR_KEY_DOWN;
            break;
        case FD_FP_KEY_LEFT:
            rcKey = FAPI_IR_KEY_LEFT;
            break;
        case FD_FP_KEY_RIGHT:
            rcKey = FAPI_IR_KEY_RIGHT;
            break;
        default:
            rcKey = FAPI_IR_KEY_UNDEFINED;
            break;
    }

    return rcKey;
}

#endif /* TARGET_FUJITSU_STB */
/*---------------------------------------------------------------------------*/
/* notify function called by IR driver (IRQ context!) (set in init.c)        */
/*---------------------------------------------------------------------------*/
#ifndef RC_VIA_FP
FSTATIC void KEYIN_RcuNotifyFunction(FAPI_SYS_HandleT handle, uint32_t* optDataPtr)
{
    uint32_t            rcKey;
    KEYIN_KEY_TYPE_E    rcType;
    int32_t             ferr;
    KEYIN_EVT_KEY_S     ctrlEvent;
    uint64_t            timestamp;
    uint64_t            timeInterval;
    uint64_t            minRepInterval;
    uint32_t            repeat_flag  = 0;
#ifndef _LINUX_FAPI
    uint32_t            key_value    = 0;
    int32_t             device_id    = 0;
#endif
    /* Get the key value device id etc. from IR driver */
#ifndef _LINUX_FAPI
    ferr = FAPI_IR_GetKey(handle, &device_id, &key_value, &rcKey, &repeat_flag);

    if(ferr == FAPI_OK)
#else
    FAPEX_IR_KeyParamStrT       keyParams;
    ferr = FAPEX_IR_Ioctl( fd_IR0, FAPEX_IOCCOM_IR_GET_KEY, (FAPEXIOCARG3_T)&keyParams );
    if(ferr >= 0)
#endif
    {
        //FAPI_SYS_PRINT_DEBUG(1,"Key: %i %i \n", rcKey, rcType);

        timestamp = SYS_GET_TIMESTAMP_MS;

        /* Ensure that minimum repeat interval is kept */
        rcType = repeat_flag ? KEYIN_KEY_REPEATED : KEYIN_KEY_PRESSED;
        if(rcType == KEYIN_KEY_REPEATED)
        {
            if(repCount == 0)
                minRepInterval = repIntervalFirst;
            else
                minRepInterval = repIntervalOthers;

            timeInterval = UTILS_GetTimerStampInterval(lastKeyTime, timestamp);

            if(timeInterval < minRepInterval)
            {
                /* discard it.. */
                return;
            }
            else
            {
                repCount++;
            }
        }
        else
        {
            repCount = 0;
        }

        lastKeyTime = timestamp;

        ctrlEvent.hdr.type = EVT_KEY;
        ctrlEvent.hdr.length = sizeof(ctrlEvent);
        ctrlEvent.val = rcKey;
        ctrlEvent.type = rcType;
        ctrlEvent.source = KEY_SOURCE_RC;

        (void)EVT_Send(&ctrlEvent, "RC input");
    }
}
#endif /* RC_VIA_FP */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
