/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/linklst.c
**
**              $Id: linklst.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       General, double-linked list for internal use.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save  */
//#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <fapi/sys_driver.h>         /* global type definitions */

#include "sys_api.h"
#include "linklist.h"


/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

int32_t LINKLST_Init(LINKLST_S * pLst, uint32_t numItems, uint32_t itemSize,
                     void *pItem)
{
    uint32_t            cnt;
    uint8_t            *pTemp = pItem;
    LINKLST_HDR_S      *pCurr;

    if(pLst == NULL)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    pLst->numItems = numItems;

    if(numItems > 0)
    {
        pLst->pFirst = pItem;
        pLst->pLast = (LINKLST_HDR_S *)
            (void *) (pTemp + ((numItems - 1) * itemSize));

        /* set first item */
        pCurr = (LINKLST_HDR_S *) (void *) pTemp;

        pCurr->pPrev = NULL;
        pCurr->pNext = NULL;
        pCurr->listSig = pLst;

        /* set remaining items */
        for(cnt = 1; cnt < numItems; cnt++)
        {
            pTemp += itemSize;

            pCurr = (LINKLST_HDR_S *) (void *) pTemp;

            pCurr->pPrev = (LINKLST_HDR_S *) (void *) (pTemp - itemSize);
            pCurr->pNext = NULL;
            pCurr->pPrev->pNext = pCurr;
            pCurr->listSig = pLst;
        }
    }
    else
    {
        pLst->pFirst = NULL;
        pLst->pLast = NULL;
    }

    return FAPI_OK;
}

int32_t LINKLST_PushFront(LINKLST_S * pLst, LINKLST_HDR_S * pItem)
{
    if(pLst->pFirst == NULL)
    {
        /* list is empty, item is first item */
        pItem->pPrev = NULL;
        pItem->pNext = NULL;
        pLst->pLast = pItem;
    }
    else
    {
        /* insert before first list item */
        pItem->pPrev = NULL;
        pItem->pNext = pLst->pFirst;
        pLst->pFirst->pPrev = pItem;
    }

    pItem->listSig = pLst;
    pLst->pFirst = pItem;
    pLst->numItems++;
    return FAPI_OK;
}

int32_t LINKLST_PushBack(LINKLST_S * pLst, LINKLST_HDR_S * pItem)
{
    if(pLst->pLast == NULL)
    {
        /* list is empty, item is first item */
        pItem->pPrev = NULL;
        pItem->pNext = NULL;
        pLst->pFirst = pItem;
    }
    else
    {
        /* append after list item */
        pItem->pPrev = pLst->pLast;
        pItem->pNext = NULL;
        pLst->pLast->pNext = pItem;
    }

    pItem->listSig = pLst;
    pLst->pLast = pItem;
    pLst->numItems++;
    return FAPI_OK;
}

/* Insert before or after item pRef */
int32_t LINKLST_Insert(LINKLST_S * pLst, LINKLST_HDR_S * pItem,
                       LINKLST_HDR_S * pRef, fbool_t insertBefore)
{
    /* test if reference is part of requested list */
    if(pRef->listSig != pLst)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    if ( insertBefore == FTRUE )
    {
        pItem->pPrev = pRef->pPrev;
        pItem->pNext = pRef;

        if(pRef->pPrev != NULL)
            pRef->pPrev->pNext = pItem;
        pRef->pPrev = pItem;

        /* If pRef had been first item, modify pointer to first item */
        if(pLst->pFirst == pRef)
            pLst->pFirst = pItem;
    }
    else
    {
        pItem->pPrev = pRef;
        pItem->pNext = pRef->pNext;

        if(pRef->pNext != NULL)
            pRef->pNext->pPrev = pItem;
        pRef->pNext = pItem;

        /* If pRef had been last item, modify pointer to last item */
        if(pLst->pLast == pRef)
            pLst->pLast = pItem;
    }

    pItem->listSig = pLst;
    pLst->numItems++;
    return FAPI_OK;
}

int32_t LINKLST_Remove(LINKLST_S * pLst, LINKLST_HDR_S * pItem)
{
    if(pLst->numItems < 1)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    /* test if item is part of requested list */
    if(pItem->listSig != pLst)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    if(pItem->pPrev != NULL)
        pItem->pPrev->pNext = pItem->pNext;
    if(pItem->pNext != NULL)
        pItem->pNext->pPrev = pItem->pPrev;

    if(pLst->pFirst == pItem)
        pLst->pFirst = pItem->pNext;
    if(pLst->pLast == pItem)
        pLst->pLast = pItem->pPrev;

    pItem->listSig = NULL;
    pLst->numItems--;
    return FAPI_OK;
}

LINKLST_HDR_S      *LINKLST_GetFirst(LINKLST_S * pLst)
{
    return pLst->pFirst;
}

LINKLST_HDR_S      *LINKLST_GetLast(LINKLST_S * pLst)
{
    return pLst->pLast;
}

LINKLST_HDR_S      *LINKLST_GetNext(LINKLST_S * pLst, LINKLST_HDR_S * pItem)
{
    /* test if item is part of requested list */
    if(pItem->listSig != pLst)
        return NULL;

    return pItem->pNext;
}

LINKLST_HDR_S      *LINKLST_GetPrevious(LINKLST_S * pLst, LINKLST_HDR_S * pItem)
{
    /* test if item is part of requested list */
    if(pItem->listSig != pLst)
        return NULL;

    return pItem->pPrev;
}

LINKLST_HDR_S      *LINKLST_GetByIndex(LINKLST_S * pLst, uint32_t index)
{
    uint32_t            cnt;
    LINKLST_HDR_S      *pItem;

    if(index >= pLst->numItems)
        return NULL;
    else if(index < (pLst->numItems >> 1))
    {
        /* ascending search */
        cnt = 0;
        pItem = pLst->pFirst;

        while(pItem != NULL)
        {
            if(cnt++ == index)
                break;

            pItem = pItem->pNext;
        }
    }
    else
    {
        /* descending search */
        cnt = pLst->numItems - 1;
        pItem = pLst->pLast;

        while(pItem != NULL)
        {
            if(cnt-- == index)
                break;

            pItem = pItem->pPrev;
        }
    }

    return pItem;
}

uint32_t LINKLST_GetListSize(LINKLST_S * pLst)
{
    return pLst->numItems;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
