/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/mp3.c
**
**              $Id: mp3.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       MP3 player abstraction layer.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/

#include <fapi/sys_services.h>
#include <aal/aal.h>

#include "utils.h"
#include "av.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define MP3_AUDIO_BUFSIZE           32768       //!< Size of MP3 buffer.
//!< the max size should not be more than FAPI_SYS_MMAP_BM_AUDIO_STREAM_BYTES

#define MP3_PATHLEN_MAX             256         //!< Maximum size of MP3 path

#define MP3_ID3_V1_SIZE             128
#define MP3_ID3_V2_HDRSIZE          10          //!< Size of ID3 V2 header.
#define MP3_ID3_V22_FRAMEHDRSIZE    6           //!< Size of ID3 V2.2 frame hdr.
#define MP3_ID3_V23_FRAMEHDRSIZE    10          //!< Size of ID3 V2.3 frame hdr.

#define MP3_ID3_BUFSIZE             256

#define MP3_SAMPLES_PER_FRAME_1152  1152        //!< the number of samples per MP3 audio frame
#define MP3_SAMPLES_PER_FRAME_576   576         //!< the number of samples per MP3 audio frame

#if ( MP3_ID3_BUFSIZE < MP3_ID3_V1_SIZE )
    #error ID3 buffer to small!
#endif

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

/* A */
typedef enum
{
    MP3_CMD_START = 0,
    MP3_CMD_STOP,
    MP3_CMD_PAUSE,
    MP3_CMD_RESUME

} JPEG_CMDTYPE_E;

typedef struct
{
    char_t                  path[MP3_PATHLEN_MAX];
    JPEG_CMDTYPE_E          type;

} MP3_CMD_S;

/*  MP3 information block */
typedef struct
{
    MP3_EVT_STATUS_S      event;        //!< Event data.
    F_FILE*               fileHandle;   //!< File handle;
    char_t                filePath[MP3_PATHLEN_MAX];
                                        //!< File Path;
    FAPI_SYS_HandleT      audioHandle;  //!< Audio decoder/output handle.
    MP3_FILE_INFO_S       fileInfo;     //!< File information from MP3 tag.
    MP3_FRAME_INFO_S      frameInfo;    //!< Frame information.
    fbool_t               ifbufFeed;    //!< if the reading data feeded to
                                        //!< buffer successfully.
} MP3io_S;

/* C */
typedef struct
{
    ITC_INST_S          itcInst;
    MP3_PLAY_STATUS_E   playStatus; //!< written in user thread (API functions).
    fbool_t             pauseReq;   //!< written in user thread (API functions).
    MP3io_S*            MP3ioP;     //!< MP3 information
    uint8_t             buffer[MP3_AUDIO_BUFSIZE];

} MP3_DATA_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

static MP3_DATA_S*      mp3Dat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void        MP3_ThreadFunction     (void* opt);
FSTATIC int32_t     MP3_ThreadOpenPlayback (MP3_DATA_S* inst,
                                            const char_t* filePath);
FSTATIC int32_t     MP3_ThreadPausePlayback(MP3_DATA_S* inst);
FSTATIC void        MP3_ThreadStopPlayback (MP3_DATA_S* inst);
FSTATIC int32_t     MP3_IntGetInfo         (const char_t* filePath,
                                            MP3_FILE_INFO_S* data);
FSTATIC uint32_t    MP3_ID3_ExtractTime    (const char_t* id3Buf,
                                            int32_t id3Size, uint8_t textEnc);
FSTATIC void        MP3_ID3_ExtractString  (char_t* targetStr, int32_t maxLen,
                                            SYS_CHARSET* charSet,
                                            const uint8_t* id3Buf,
                                            int32_t id3Size, uint8_t textEnc);
FSTATIC int32_t    MP3_GetFrameInfo       (MP3_DATA_S* inst,
                                            const char_t* filePath,
                                            MP3_FRAME_INFO_S* data,
                                            int32_t startPos);
FSTATIC fbool_t    MP3_ValidateFormat     (MP3_DATA_S* inst,
                                            const char_t* filePath,
                                            int32_t startPos);

/*!
********************************************************************************
** \brief Initialize the MP3.
**
** This function initializes the MP3.
**
** \return
** - #FAPI_OK if initialization was successful
** - #SMARTGO_SYS_ERR_RESOURCES if creating thread failed
**
********************************************************************************
*/
int32_t MP3_Init (void)
{
    ITC_PARAMS_S    itcParams;
    int32_t         retVal;

    mp3Dat = (MP3_DATA_S*)UTILS_Malloc(sizeof(MP3_DATA_S));
    if ( mp3Dat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(mp3Dat, 0, sizeof(MP3_DATA_S));

    itcParams.setAbortReq = NULL;

    retVal = ITC_Init(&(mp3Dat->itcInst), &itcParams);
    if ( retVal != FAPI_OK ) return retVal;

    mp3Dat->playStatus  = MP3_STATUS_READY;
    mp3Dat->MP3ioP      = NULL;

    return SYS_ThreadCreate (MP3_ThreadFunction, mp3Dat,
                             THREAD_PRIO_MP3,
                             THREAD_STACKSZ_MP3,
                             THREAD_NAME_MP3,
                             NULL);
}

/*!
********************************************************************************
** \brief Extract MP3 information from the file
**
** This function extracts the available information by the given path
** from the file, like music title, artist and album information.
**
** \param filePath  The path of the file that needs to extract information.
** \param data      A pointer to a buffer where to write all extracted
**                      information into.
**
** \return
** - #FAPI_OK if a given parameter was invalid.
** - #APPL_SMARTGO_ERR_BAD_PARAMETER if a given parameter was invalid.
**
********************************************************************************
*/
int32_t MP3_GetInfo (const char_t* filePath, MP3_FILE_INFO_S* data)
{
    int32_t         retVal;
    F_FILE*         fileP;
    SYS_DEVICE_E    device;

    if ( SYS_DeviceGetByPath(filePath, &device, NULL) == FAPI_OK )
    {
        (void)SYS_DevicePreAccess (device);
    }

    fileP = f_open(filePath, "r");

    if ( fileP != NULL )
    {
        (void)f_close(fileP);

        (void)MP3_IntGetInfo(filePath, data);

        retVal = FAPI_OK;
    }
    else
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    FAPI_SYS_PRINT_DEBUG(4,"\n*********************************\n");
    FAPI_SYS_PRINT_DEBUG(4,"File  : %s\n", filePath );
    FAPI_SYS_PRINT_DEBUG(4,"Artist: %s\n", data->artist);
    FAPI_SYS_PRINT_DEBUG(4,"Title : %s\n", data->title);
    FAPI_SYS_PRINT_DEBUG(4,"Album : %s\n", data->album);
    FAPI_SYS_PRINT_DEBUG(4,"Picpos: %d\n", data->picPos);
    FAPI_SYS_PRINT_DEBUG(4,"Length: %d\n", data->length);
    FAPI_SYS_PRINT_DEBUG(4,"*********************************\n");

    return retVal;
}

/*!
*****************************************************************************
** \brief Starts the MP3.
**
** This function first checks the efuse setting, as well as the file format
** if MP3 is capable to start. Then the start commond of MP3 is sent.
**
** \param filePath The path of the file that needs to start.
**
** \return
**          - #FAPI_OK if the commond was successfully sent.
**          - #APPL_SMARTGO_ERR_FEATURE_NOT_SUPPORTED if the MP3 feature is not supported
                                                      or the file can not be played.
**          - #SMARTGO_SYS_ERR_INTERNAL if the command cannot be sent.
** \sa FAPI_AUDEC_Stop
*****************************************************************************
*/
int32_t MP3_Start (const char_t* filePath)
{
    MP3_CMD_S                   cmd;
    int32_t                     retVal;
    FAPI_AUDEC_CapabilitiesT    capabilities;
    MP3_FILE_INFO_S*            fileData;
    int32_t                     startPos;

    if ( mp3Dat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    /* --- MP3 efuse checking --- */
    capabilities = FAPI_AUDEC_GetCapabilities();

    if ( !capabilities.mp3Decode )
        return APPL_SMARTGO_ERR_FEATURE_NOT_SUPPORTED;

    /* --- MP3 format file checking --- */
    fileData = UTILS_Malloc(sizeof(MP3_FILE_INFO_S));
    memset (fileData, 0, sizeof(MP3_FILE_INFO_S));

    startPos = MP3_IntGetInfo(filePath, fileData);
    if ( !(MP3_ValidateFormat(mp3Dat, filePath, startPos)) )
    {
        return APPL_SMARTGO_ERR_FEATURE_NOT_SUPPORTED;
    }

    FAPI_SYS_PRINT_DEBUG(4,"[MP3] MP3 decoder support: %c\n",
                         capabilities.mp3Decode ? 'Y' : 'N' );

    (void)UTILS_strncpy(cmd.path, filePath, MP3_PATHLEN_MAX);
    cmd.type = MP3_CMD_START;

    UTILS_Free(fileData);

    retVal = ITC_CmdSend (&(mp3Dat->itcInst), (void*)(&cmd), NULL);

    if ( retVal == FAPI_OK )
    {
        mp3Dat->pauseReq = FFALSE;
    }

    return retVal;
}

/*!
*****************************************************************************
** \brief Stop the MP3.
**
** This function sends the MP3 stop commond.
**
** \return
**          - #FAPI_OK if the commond was successfully sent.
**          - #APPL_SMARTGO_ERR_FEATURE_NOT_SUPPORTED if the MP3 feature is not supported
                                                      or the file can not be played.
**          - #SMARTGO_SYS_ERR_INTERNAL if the command cannot be sent.
**
** \sa
**          - FAPI_AUDEC_Stop()
*****************************************************************************
*/
int32_t MP3_Stop (void)
{
    MP3_CMD_S   cmd;
    int32_t     retVal;

    FAPI_SYS_PRINT_DEBUG(4,"[MP3]: STOP MP3\n");

    if ( mp3Dat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    cmd.path[0] = 0;
    cmd.type    = MP3_CMD_STOP;

    retVal = ITC_CmdSend (&(mp3Dat->itcInst), (void*)(&cmd), NULL);

    if ( retVal == FAPI_OK )
    {
        mp3Dat->pauseReq = FFALSE;
    }

    return retVal;
}

int32_t MP3_Pause (fbool_t pauseOn)
{
    MP3_CMD_S   cmd;
    int32_t     retVal;

    if (( mp3Dat->MP3ioP == NULL ) || ( mp3Dat->MP3ioP->filePath[0] == 0 ))
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if ( pauseOn == mp3Dat->pauseReq )
    {
        retVal = FAPI_OK;
    }
    else
    {
        mp3Dat->pauseReq = pauseOn;

        if ( pauseOn )
        {
            cmd.type = MP3_CMD_PAUSE;
        }
        else
        {
            cmd.type = MP3_CMD_RESUME;
        }

        (void)UTILS_strncpy(cmd.path, mp3Dat->MP3ioP->filePath, MP3_PATHLEN_MAX);

        retVal = ITC_CmdSend (&(mp3Dat->itcInst), (void*)(&cmd), NULL);
    }

    return retVal;
}

void MP3_EventHandling (const MP3_EVT_STATUS_S* pEvt)
{
    mp3Dat->playStatus  = pEvt->status;
}


FSTATIC void MP3_ThreadFunction (void* opt)
{
    MP3_DATA_S*                 inst = (MP3_DATA_S*)opt;
    void *                      cmdMsg;
    MP3_CMD_S                   cmd;
    SYS_DEVICE_E                device;
    int32_t                     bytes = 0;
    int32_t                     retVal  = FAPI_OK;
    uint32_t                    timeStart = 0;
    uint32_t                    timeCurr = 0;
    uint32_t                    playTime = 0;
    FAPI_AUDEC_StatusEnumT      decoderStatus;
    int32_t                     startPos = 0;
    int32_t                     ii = 0;
    uint32_t                    ack = 1;
    fbool_t                     fileEnd = FFALSE;
    int32_t                     bufFeedStatus = FAPI_OK;
    FAPI_AUOUT_StreamInfoT      streamInfo;

    SYS_DevicePrepareThread ();

    while ( 1 )
    {
        cmdMsg = ITC_CmdWait (&(inst->itcInst));

        if ( cmdMsg == NULL ) continue;

        cmd = *((MP3_CMD_S*)cmdMsg);

        /* acknowledge command reception */
        (void)ITC_AckSend(&(inst->itcInst), (void*)&ack);

        switch ( cmd.type )
        {
            /* ----- stop playback ----------------------------------------- */
            case MP3_CMD_STOP:
                MP3_ThreadStopPlayback (inst);
                break;

            /* ----- pause playback ---------------------------------------- */
            case MP3_CMD_PAUSE:
                (void)MP3_ThreadPausePlayback (inst);
                break;

            /* ----- start/resume playback ---------------------------------------- */
            case MP3_CMD_START:
            case MP3_CMD_RESUME:
                if ( SYS_DeviceGetByPath(cmd.path, &device, NULL) == FAPI_OK )
                {
                    /* mark device as busy */
                    (void)SYS_DeviceSetAccessStatus(device, FTRUE);

                    (void)SYS_DevicePreAccess (device);
                }
                else
                {
                    device = SYS_DEV_NULL;
                }

                if( cmd.type == MP3_CMD_RESUME )
                {
                    retVal = MP3_ThreadPausePlayback (inst);

                    if ( retVal != FAPI_OK )
                    {
                        break;
                    }

                    //reset the start time to resume from the pause time
                    (void)TIME_GetSystemTime(&timeStart);
                    timeStart -= inst->MP3ioP->event.timeCurr * 10;
                }

                do
                {
                    if( cmd.type == MP3_CMD_START )
                    {
                        /* stop in case playback is still active */
                        MP3_ThreadStopPlayback (inst);

                        /* open modules needed for playback */
                        retVal = MP3_ThreadOpenPlayback (inst, cmd.path);
                        if ( retVal != FAPI_OK )
                        {
                            break;
                        }

                        FAPI_SYS_PRINT_DEBUG(4, "[MP3] Play\n");

                        inst->MP3ioP->event.hdr.type   = EVT_MP3_STATUS;
                        inst->MP3ioP->event.hdr.length = sizeof(inst->MP3ioP->event);
                        inst->MP3ioP->event.timeTotal  = 0;
                        inst->MP3ioP->event.timeCurr   = 0;
                        inst->MP3ioP->event.status     = MP3_STATUS_PLAY;

                        /* ----- determine total playback time --------------------- */

                        startPos = MP3_IntGetInfo (cmd.path, &(inst->MP3ioP->fileInfo));

                        /* prefer play time information via ID3 tags if available */
                        if ( inst->MP3ioP->fileInfo.length > 0 )
                        {
                            inst->MP3ioP->event.timeTotal =
                                inst->MP3ioP->fileInfo.length;
                        }
                        else
                        {
                            (void)MP3_GetFrameInfo (inst, cmd.path,
                                                    &(inst->MP3ioP->frameInfo),
                                                    startPos);
                            /* Total play time =
                            (number of frame*samples per frame)/sample frequency */

                            /*lint -save -e571 */
                            inst->MP3ioP->event.timeTotal =
                                (inst->MP3ioP->frameInfo.frameNum*(uint32_t)inst->MP3ioP->frameInfo.samplePerFrame)/
                                ((uint32_t)(inst->MP3ioP->frameInfo.sampleFreq));
                            FAPI_SYS_PRINT_DEBUG(4,"[MP3]frame Sample frequency: %i \n", inst->MP3ioP->frameInfo.sampleFreq);
                            /*lint -restore */
                        }

                        /* send initial playback event */
                        (void)EVT_Send(&(inst->MP3ioP->event), "MP3 Status");

                        /* ----- start file decoding ------------------------------- */

                        fileEnd = FFALSE;

                        (void)AAL_ClearBuffer (inst->MP3ioP->audioHandle);

                        (void)f_seek (inst->MP3ioP->fileHandle, startPos, F_SEEK_SET);

                        for ( ii = 0; (ii < 3) && (!f_eof(inst->MP3ioP->fileHandle));
                              ii++ )
                        {
                            bytes = f_read (inst->buffer, 1, 2048,
                                            inst->MP3ioP->fileHandle);

                            inst->MP3ioP->ifbufFeed = FTRUE;

                            do
                            {
                                if ( inst->itcInst.abortReq )
                                {
                                    break;
                                }

                                bufFeedStatus = AAL_FeedBuffer (inst->MP3ioP->audioHandle,
                                                              inst->buffer, bytes);

                                FAPI_SYS_SLEEP(20);
                            }
                            while ( bufFeedStatus != FAPI_OK );

                            if ( inst->itcInst.abortReq )
                            {
                                break;
                            }
                            else if ( f_eof(inst->MP3ioP->fileHandle) )
                            {
                                break;
                            }

                            inst->MP3ioP->ifbufFeed = FTRUE;

                            if ( AAL_GetStreamInfo(inst->MP3ioP->audioHandle,
                                                   &streamInfo) == FAPI_OK )
                            {
                                FAPI_SYS_PRINT_DEBUG(1, "[_MP3]Sample frequency: %i \n", streamInfo.sampleFreq);
                                FAPI_SYS_PRINT_DEBUG(1, "[_MP3]Channel: %i \n", streamInfo.channels);
                            }
                        }

                        if ( inst->itcInst.abortReq )
                        {
                            break;
                        }

                        (void)AAL_SetSPDIFMode (inst->MP3ioP->audioHandle,
                                                AAL_SPDIFMODE_PCM,
                                                AAL_AUDIOFORMAT_NONE, 0);

                        retVal = AAL_Start (inst->MP3ioP->audioHandle,
                                            AAL_AUDIOFORMAT_MPEG, 0);

                        if ( retVal != FAPI_OK )
                        {
                            FAPI_SYS_PRINT_DEBUG(4,"[MP3] AUDIO start failed\n");
                            break;
                        }

                        (void)TIME_GetSystemTime(&timeStart);

                    }

                    while ( !(inst->itcInst.abortReq) )
                    {
                        if ( !fileEnd )
                        {
                            if ( f_eof(inst->MP3ioP->fileHandle) )
                            {
                                fileEnd = FTRUE;
                                continue;
                            }

                            if ( inst->MP3ioP->ifbufFeed )
                            {
                                bytes = f_read (inst->buffer, 1, MP3_AUDIO_BUFSIZE,
                                                inst->MP3ioP->fileHandle);
                            }

                            if ( bytes == 0 )
                            {
                                fileEnd = FTRUE;
                                continue;
                            }

                            inst->MP3ioP->ifbufFeed = FFALSE;

                            do
                            {
                                /* calculate current output time */
                                if ( TIME_GetSystemTime(&timeCurr) == FAPI_OK )
                                {
                                    playTime = TIME_GetSystemTimeInterval(timeStart, timeCurr) / 10;

                                    if ( playTime != inst->MP3ioP->event.timeCurr )
                                    {
                                        inst->MP3ioP->event.timeCurr = playTime;

                                        (void)EVT_Send(&(inst->MP3ioP->event),
                                                       "MP3 Status");
                                    }
                                }

                                if ( inst->itcInst.abortReq )
                                {
                                    break;
                                }

                                /* try to feed the data to the decoder buffer */
                                bufFeedStatus = AAL_FeedBuffer (inst->MP3ioP->audioHandle,
                                                                inst->buffer, bytes);

                                FAPI_SYS_SLEEP(20);

                            }
                            while ( bufFeedStatus != FAPI_OK );

                            if ( inst->itcInst.abortReq )
                            {
                                break;
                            }

                            inst->MP3ioP->ifbufFeed = FTRUE;

                            (void)AAL_Task(inst->MP3ioP->audioHandle);

                            if ( AAL_GetStreamInfo (inst->MP3ioP->audioHandle,
                                                    &streamInfo) == FAPI_OK )
                            {
                                FAPI_SYS_PRINT_DEBUG(1, "[MP3]Sample frequency: %i \n", streamInfo.sampleFreq);
                                FAPI_SYS_PRINT_DEBUG(1, "[MP3]Channel: %i \n", streamInfo.channels);
                            }
                            bytes = 0;

                        }
                        else
                        {
                            /* update the output time between decoding end and
                               real playback end (remaining samples in the buffer
                               are played out)*/
                            if ( TIME_GetSystemTime(&timeCurr) == FAPI_OK )
                            {
                                playTime = TIME_GetSystemTimeInterval(timeStart, timeCurr) / 10;

                                if ( playTime != inst->MP3ioP->event.timeCurr )
                                {
                                    inst->MP3ioP->event.timeCurr = playTime;

                                    (void)EVT_Send(&(inst->MP3ioP->event),
                                                   "MP3 Status");
                                }
                            }
                            if ( AAL_GetDecStatus(inst->MP3ioP->audioHandle,
                                                  &decoderStatus) == FAPI_OK )
                            {
                                if ( decoderStatus == FAPI_AUDEC_STATUS_WAIT_INPUT )
                                {
                                    /* Playback end reached: send final event */
                                    inst->MP3ioP->event.status = MP3_STATUS_END;
                                    inst->MP3ioP->event.timeCurr =
                                        inst->MP3ioP->event.timeTotal;

                                    (void)EVT_Send(&(inst->MP3ioP->event),
                                                   "MP3 Status");
                                    break;
                                }
                            }
                        }
                    }
                }
                while ( 0 );

                if ( device != SYS_DEV_NULL )
                {
                    (void)SYS_DeviceSetAccessStatus (device, FFALSE);
                }
                break;

            default:
                break;

        }
    }
}

FSTATIC int32_t MP3_ThreadOpenPlayback (MP3_DATA_S* inst,
                                        const char_t* filePath)
{
    AAL_OpenParamsT openParams;
    int32_t         retVal = FAPI_OK;

    if ( inst->MP3ioP != NULL )
    {
        retVal = APPL_SMARTGO_ERR_ALREADY_OPEN;
    }
    else
    {
        do
        {
            inst->MP3ioP = UTILS_Malloc(sizeof(MP3io_S));
            if ( inst->MP3ioP == NULL )
            {
                retVal = APPL_SMARTGO_ERR_OUT_OF_MEMORY;
                break;
            }

            memset(inst->MP3ioP, 0, sizeof(MP3io_S) );

            /* open source file */
            inst->MP3ioP->fileHandle = f_open (filePath, "r");
            if ( inst->MP3ioP->fileHandle == NULL )
            {
                retVal = SMARTGO_SYS_ERR_INTERNAL;
                break;
            }
            (void)UTILS_strncpy(inst->MP3ioP->filePath, filePath,
                                MP3_PATHLEN_MAX);

            /* open audio output */
            memset (&openParams, 0, sizeof(openParams));

            openParams.feedMode                 = AAL_FEEDMODE_BM;
            openParams.speaker[openParams.numSpeakers++]
                                                = FAPI_AUOUT_SPEAKER_LEFT;
            openParams.speaker[openParams.numSpeakers++]
                                                = FAPI_AUOUT_SPEAKER_RIGHT;
            /* NOTE: This will improve playback of short files, because all
                     audio frames are output. However, it would have negative
                     effects in case of errorneous streams. MP3 files are
                     assumed to be (nearly) error-free. */
            openParams.suppressErrorRecognition = 1;

            inst->MP3ioP->audioHandle = AAL_Open (&openParams, &retVal);

            if ( inst->MP3ioP->audioHandle == NULL )
            {
                break;
            }
        }
        while ( 0 );

        if ( retVal != FAPI_OK )
        {
            if ( inst->MP3ioP != NULL )
            {
                if ( inst->MP3ioP->audioHandle != NULL )
                {
                    (void)AAL_Close(inst->MP3ioP->audioHandle);
                }

                if ( inst->MP3ioP->fileHandle != NULL )
                {
                    (void)f_close (inst->MP3ioP->fileHandle);
                }

                UTILS_Free (inst->MP3ioP);

                inst->MP3ioP = NULL;
            }
        }
    }

    return retVal;
}

FSTATIC int32_t MP3_ThreadPausePlayback (MP3_DATA_S* inst)
{
    int32_t         retVal = FAPI_OK;

    if ( inst->MP3ioP != NULL )
    {
        if ( inst->MP3ioP->audioHandle != NULL )
        {
            retVal = AAL_Pause (inst->MP3ioP->audioHandle, (uint8_t)inst->pauseReq);
        }
    }
    else
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    return retVal;
}

FSTATIC void MP3_ThreadStopPlayback (MP3_DATA_S* inst)
{
    if ( inst->MP3ioP != NULL )
    {
        inst->MP3ioP->event.timeTotal = 0;
        inst->MP3ioP->event.timeCurr  = 0;
        inst->MP3ioP->event.status    = MP3_STATUS_READY;

        (void)EVT_Send(&(inst->MP3ioP->event), "MP3 Status");

        if ( inst->MP3ioP->audioHandle != NULL )
        {
            (void)AAL_Stop (inst->MP3ioP->audioHandle);
            (void)AAL_Close(inst->MP3ioP->audioHandle);
        }

        if ( inst->MP3ioP->fileHandle != NULL )
        {
            (void)f_close (inst->MP3ioP->fileHandle);
        }

        UTILS_Free (inst->MP3ioP);

        inst->MP3ioP = NULL;
    }
}

/*!
*****************************************************************************
** \brief Validate the MP3 file.
**
** This function reads the file frame header to check if the MP3 file is validate
** as MPEG Version 1 Layer III or MPEG Version 2 Layer III
**
** \param inst      TODO
** \param filePath  The path of the file that needs to start.
** \param startPos  The position of the file where the 1st frame should start
**
** \return
** - #FTRUE if the file is MP3 formatted.
** - #FFALSE if the file is not MP3 formatted
**
*****************************************************************************
*/
FSTATIC fbool_t MP3_ValidateFormat (MP3_DATA_S* inst, const char_t* filePath,
                                    int32_t startPos)
{
    int32_t       filesize = 0;
    F_FILE*       fileP;
    int32_t       ii = 0;
    uint32_t      frameCount = 0;
    int32_t       bytes;

    filesize = f_filelength(filePath);
    if ( filesize < 0 )
        return FFALSE;

    fileP = f_open(filePath, "r");

    (void)f_seek(fileP, startPos, F_SEEK_SET);

    while ( !(f_eof(fileP)))
    {
        bytes = f_read(inst->buffer, 1, MP3_AUDIO_BUFSIZE, fileP);
        for( ii = 0; (ii + 2) < bytes; ii++ )
        {
            if( inst->buffer[ii] == 0xFF )
            {
                if ( (inst->buffer[ii+1] == 0xFB)||
                     (inst->buffer[ii+1] == 0xFA) ) //MPEG 1
                {
                    frameCount++;
                    ii += 300;
                }
                else if ( (inst->buffer[ii+1] == 0xF3)||
                          (inst->buffer[ii+1] == 0xF2) ) //MPEG 2
                {
                    frameCount++;
                    ii += 300;
                }
            }
        }
        if ( frameCount > 10 ) break;
    }

    (void)f_close( fileP );

    if (frameCount > 0) return FTRUE;
    else                return FFALSE;
}

FSTATIC int32_t MP3_GetFrameInfo (MP3_DATA_S* inst, const char_t* filePath,
                                  MP3_FRAME_INFO_S* data,
                                  int32_t startPos)
{
    int32_t       filesize = 0;
    F_FILE*       fileP;
    int32_t       ii = 0;
    uint32_t      frameCount = 0;
    int32_t       bytes;

    filesize = f_filelength(filePath);
    if ( filesize < 0 )
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    fileP = f_open(filePath, "r");

    (void)f_seek(fileP, startPos, F_SEEK_SET);

    while ( !(f_eof(fileP)) )
    {
        bytes = f_read (inst->buffer, 1, MP3_AUDIO_BUFSIZE, fileP);
        for( ii = 0; (ii + 2) < bytes; ii++ )
        {
            if ( inst->buffer[ii] == 0xFF )
            {
                if ( ((frameCount < 1) || (data->mpegVer == MP3_MPEG_1)) &&
                    ((inst->buffer[ii+1] == 0xFB)||(inst->buffer[ii+1] == 0xFA) ))//MPEG 1
                {
                    /* ----- get the sample Frequency ------ */
                    if( frameCount==0 )
                    {
                        switch ( inst->buffer[ii + 2] & 0x0C )
                        {
                            case 0x0:
                                data->sampleFreq
                                    = MP3_FREQ_44100_HZ;
                                break;
                            case 0x4:
                                data->sampleFreq
                                    = MP3_FREQ_48000_HZ;
                                break;
                            case 0x8:
                                data->sampleFreq
                                    = MP3_FREQ_32000_HZ;
                                break;
                            default:
                                data->sampleFreq
                                    = MP3_FREQ_UNDEF_HZ;
                                break;
                        }
                        data->mpegVer = MP3_MPEG_1;
                    }
                    frameCount++;
                    //FIXME (YC): Step size should be caculated based on expected frame length
                    ii += 300;
                }
                else if ( ((frameCount < 1) || (data->mpegVer == MP3_MPEG_2)) &&
                        ((inst->buffer[ii+1] == 0xF3)||(inst->buffer[ii+1] == 0xF2) ))//MPEG 2
                {
                    /* ----- get the sample Frequency ------ */
                    if( frameCount==0 )
                    {
                        switch ( inst->buffer[ii + 2] & 0x0C )
                        {
                            case 0x0:
                                data->sampleFreq
                                    = MP3_FREQ_22050_HZ;
                                break;
                            case 0x4:
                                data->sampleFreq
                                    = MP3_FREQ_24000_HZ;
                                break;
                            case 0x8:
                                data->sampleFreq
                                    = MP3_FREQ_16000_HZ;
                                break;
                            default:
                                data->sampleFreq
                                    = MP3_FREQ_UNDEF_HZ;
                                break;
                        }
                        data->mpegVer = MP3_MPEG_2;
                    }
                    frameCount++;
                    //FIXME (YC): Step size should be caculated based on expected frame length
                    ii += 130;
                }
                else if ( ((frameCount < 1) || (data->mpegVer == MP3_MPEG_2_5)) &&
                        ((inst->buffer[ii+1] == 0xE3)||(inst->buffer[ii+1] == 0xE2) ))//MPEG 2.5
                {
                    /* ----- get the sample Frequency ------ */
                    if( frameCount==0 )
                    {
                        switch ( inst->buffer[ii + 2] & 0x0C )
                        {
                            case 0x0:
                                data->sampleFreq
                                    = MP3_FREQ_11025_HZ;
                                break;
                            case 0x4:
                                data->sampleFreq
                                    = MP3_FREQ_12000_HZ;
                                break;
                            case 0x8:
                                data->sampleFreq
                                    = MP3_FREQ_8000_HZ;
                                break;
                            default:
                                data->sampleFreq
                                    = MP3_FREQ_UNDEF_HZ;
                                break;
                        }
                        data->mpegVer = MP3_MPEG_2_5;
                    }
                    frameCount++;
                    //FIXME (YC): Step size should be caculated based on expected frame length
                    ii += 100;
                }
            }
        }
    }
    data->frameNum = frameCount;

    if (data->mpegVer == MP3_MPEG_1)
    {
        data->samplePerFrame = MP3_SAMPLES_PER_FRAME_1152;
    }
    else
    {
        data->samplePerFrame = MP3_SAMPLES_PER_FRAME_576;
    }

    (void)f_close( fileP );

    return FAPI_OK;
}

/* Returns byte start position of MP3 audio information in file. */
FSTATIC int32_t MP3_IntGetInfo (const char_t* filePath, MP3_FILE_INFO_S* data)
{
    char_t        pictype[10];
    uint8_t       buffer[MP3_ID3_BUFSIZE];
    int32_t       frameSize;
    int32_t       frameReadSize;
    uint32_t      tag;
    int32_t       pos  = 0;
    fbool_t       done = FFALSE;
    int32_t       filesize = 0;
    int32_t       id3Size = 0;
    F_FILE*       fileP;

    filesize = f_filelength(filePath);
    if ( filesize < 0 )
    {
        return 0;
    }

    fileP = f_open(filePath, "r");

    (void)f_seek(fileP, 0, F_SEEK_SET);
    (void)f_read(buffer, 1, MP3_ID3_V2_HDRSIZE, fileP);

    data->length = 0;

    data->jpegAvl = FFALSE;

    if ( (buffer[0] == 'I') &&
         (buffer[1] == 'D') &&
         (buffer[2] == '3'))  // ID3v2
    {
        id3Size = (int32_t)((((uint32_t)(buffer[6] & 0x7F) << 21) |
                             ((uint32_t)(buffer[7] & 0x7F) << 14) |
                             ((uint32_t)(buffer[8] & 0x7F) <<  7) |
                             ((uint32_t)(buffer[9] & 0x7F)))
                             + MP3_ID3_V2_HDRSIZE);

        if ( id3Size > filesize )
        {
            FAPI_SYS_PRINT_DEBUG(4,"Invalid ID3 size: %i \n", id3Size);
            id3Size = filesize;
        }

        pos = MP3_ID3_V2_HDRSIZE;

        switch ( buffer[3] )
        {
            /* ----- ID3v2.2 ----------------------------------------------- */

            case 2:

                FAPI_SYS_PRINT_DEBUG(4,"[MP3] ID3v2.2 found!\n");

                while ( (!done) &&
                        ((pos + MP3_ID3_V22_FRAMEHDRSIZE) <= id3Size) )
                {
                    (void)f_seek(fileP, pos, F_SEEK_SET);

                    /* read frame header */
                    (void)f_read(buffer, 1, MP3_ID3_V22_FRAMEHDRSIZE, fileP);

                    tag       = ((uint32_t)(buffer[0] << 16)) |
                                ((uint32_t)(buffer[1] <<  8)) |
                                            buffer[2];
                    frameSize = (int32_t)(((uint32_t)(buffer[3] << 16)) |
                                          ((uint32_t)(buffer[4] <<  8)) |
                                                      buffer[5]);

                    if ( (pos + MP3_ID3_V22_FRAMEHDRSIZE + frameSize) > id3Size )
                    {
                        FAPI_SYS_PRINT_DEBUG(4,"[MP3] Invalid ID3 frame size: %i \n", frameSize);
                        frameSize = id3Size - (pos + MP3_ID3_V22_FRAMEHDRSIZE);
                    }

                    if ( frameSize > MP3_ID3_BUFSIZE )
                    {
                        FAPI_SYS_PRINT_DEBUG(4,"[MP3] Insufficient ID3 buffer: %i bytes needed \n", frameSize);
                        frameReadSize = MP3_ID3_BUFSIZE;
                    }
                    else
                    {
                        frameReadSize = frameSize;
                    }

                    switch ( tag )
                    {
                        case 0x00545432: //'TT2':
                            if ( frameReadSize > 1 )
                            {
                                (void)f_read(buffer, 1, frameReadSize, fileP);

                                MP3_ID3_ExtractString(data->title,
                                                      MP3_TAG_STRLEN_MAX,
                                                      &(data->titleCs),
                                                      buffer + 1,
                                                      frameReadSize - 1,
                                                      buffer[0]);
                            }
                            break;

                        case 0x00545031: //'TP1':
                            if ( frameReadSize > 1 )
                            {
                                (void)f_read(buffer, 1, frameReadSize, fileP);;

                                MP3_ID3_ExtractString(data->artist,
                                                      MP3_TAG_STRLEN_MAX,
                                                      &(data->artistCs),
                                                      buffer + 1,
                                                      frameReadSize - 1,
                                                      buffer[0]);
                            }
                            break;

                        case 0x0054414C: //'TAL':
                            if ( frameReadSize > 1 )
                            {
                                (void)f_read(buffer, 1, frameReadSize, fileP);

                                MP3_ID3_ExtractString(data->album,
                                                      MP3_TAG_STRLEN_MAX,
                                                      &(data->albumCs),
                                                      buffer + 1,
                                                      frameReadSize - 1,
                                                      buffer[0]);
                            }
                            break;

                        case 0x00504943: //'PIC':
                            /* NOTE: This is a workaround to help
                                     distinguishing links to external
                                     picture files from embedded picture
                                     data. The MIME type identification
                                     specified by ID3 does not seem
                                     to be reliable. */
                            if ( frameSize > 1000 )
                            {
                                    data->picPos = (uint32_t)(pos
                                                   + MP3_ID3_V22_FRAMEHDRSIZE
                                                   + 1);
                            }
                            break;

                        case 0:
                        case 0x00454E44: //'END':
                            done= FTRUE;
                            break;

                        default:
                            /* do nothing */
                            break;
                    }

                    pos += MP3_ID3_V22_FRAMEHDRSIZE + frameSize;
                };
                break;

            /* ----- ID3v2.3 ----------------------------------------------- */

            case 3:

                FAPI_SYS_PRINT_DEBUG(4,"[MP3] ID3v2.3 found!\n");

                while ( (!done) &&
                        ((pos + MP3_ID3_V23_FRAMEHDRSIZE) <= id3Size) )
                {
                    (void)f_seek(fileP, pos, F_SEEK_SET);

                    /* read frame header */
                    (void)f_read(buffer, 1, MP3_ID3_V23_FRAMEHDRSIZE, fileP);


                    tag       = ((uint32_t)(buffer[0] << 24)) |
                                ((uint32_t)(buffer[1] << 16)) |
                                ((uint32_t)(buffer[2] <<  8)) |
                                            buffer[3];
                    frameSize = (int32_t)(((uint32_t)(buffer[4] << 24)) |
                                          ((uint32_t)(buffer[5] << 16)) |
                                          ((uint32_t)(buffer[6] <<  8)) |
                                                      buffer[7]);

                    if ( (pos + MP3_ID3_V23_FRAMEHDRSIZE + frameSize) > id3Size )
                    {
                        FAPI_SYS_PRINT_DEBUG(4,"[MP3] Invalid ID3 frame size: %i \n", frameSize);
                        frameSize = id3Size - (pos + MP3_ID3_V23_FRAMEHDRSIZE);
                    }

                    if ( frameSize > MP3_ID3_BUFSIZE )
                    {
                        FAPI_SYS_PRINT_DEBUG(4,"[MP3] Insufficient ID3 buffer: %i bytes needed \n", frameSize);

                        frameReadSize = MP3_ID3_BUFSIZE;
                    }
                    else
                    {
                        frameReadSize = frameSize;
                    }

                    switch ( tag )
                    {
                        case 0x54495432: //'TIT2':
                            if ( frameReadSize > 1 )
                            {
                                (void)f_read(buffer, 1, frameReadSize, fileP);

                                MP3_ID3_ExtractString(data->title,
                                                      MP3_TAG_STRLEN_MAX,
                                                      &(data->titleCs),
                                                      buffer + 1,
                                                      frameReadSize - 1,
                                                      buffer[0]);
                            }
                            break;

                        case 0x54504531: //'TPE1'
                            if ( frameReadSize > 1 )
                            {
                                (void)f_read(buffer, 1, frameReadSize, fileP);

                                MP3_ID3_ExtractString(data->artist,
                                                      MP3_TAG_STRLEN_MAX,
                                                      &(data->artistCs),
                                                      buffer + 1,
                                                      frameReadSize - 1,
                                                      buffer[0]);
                            }
                            break;

                        case 0x54414C42: //'TALB'
                            if ( frameReadSize > 1 )
                            {
                                (void)f_read(buffer, 1, frameReadSize, fileP);

                                MP3_ID3_ExtractString(data->album,
                                                      MP3_TAG_STRLEN_MAX,
                                                      &(data->albumCs),
                                                      buffer + 1,
                                                      frameReadSize - 1,
                                                      buffer[0]);
                            }
                            break;

                        case 0x41504943: //'APIC'
                            /* NOTE: This is a workaround to help
                                     distinguishing links to external
                                     picture files from embedded picture
                                     data. The MIME type identification
                                     specified by ID3 does not seem
                                     to be reliable. */
                            if ( frameSize > 1000 )
                            {
                                    data->picPos = (uint32_t)(pos
                                                   + MP3_ID3_V23_FRAMEHDRSIZE
                                                   + 1);
                            }
                            break;

                        case 0x544C454E: //'TLEN'
                            if ( frameReadSize > 1 )
                            {
                                (void)f_read(buffer, 1, frameReadSize, fileP);

                                data->length = MP3_ID3_ExtractTime(
                                                         ((char_t*)buffer) + 1,
                                                         frameReadSize - 1,
                                                         buffer[0]);
                            }
                            break;

                        case 0:
                            done= FTRUE;
                            break;

                        default:
                            /* do nothing */
                            break;
                     }

                     pos += MP3_ID3_V23_FRAMEHDRSIZE + frameSize;
                };
                break;

            /* ----- ID3v2.4 ----------------------------------------------- */

            case 4:
                /* to do: not supported yet */
                break;

            default:
                FAPI_SYS_PRINT_DEBUG(4,"ID3v2: unknown subversion: %i \n", buffer[3]);
                break;
        }
    }

    /* ----- ID3v1 ---------------------------------------------------------- */

    else if ( filesize >= MP3_ID3_V1_SIZE )
    {
        (void)f_seek(fileP, filesize - MP3_ID3_V1_SIZE, F_SEEK_SET);
        (void)f_read(buffer, 1, MP3_ID3_V1_SIZE, fileP);

        if ( (buffer[0] == 'T') && (buffer[1] == 'A') && (buffer[2] == 'G') )
        {
            FAPI_SYS_PRINT_DEBUG(4,"[MP3] ID3v1 found!\n");

            (void)UTILS_strncpy(data->title,  ((char_t*)buffer) + 3,  30);
            (void)UTILS_strncpy(data->artist, ((char_t*)buffer) + 33, 30);
            (void)UTILS_strncpy(data->album,  ((char_t*)buffer) + 63, 30);

            data->titleCs  = SYS_CHARSET_ISO8859_1;
            data->artistCs = SYS_CHARSET_ISO8859_1;
            data->albumCs  = SYS_CHARSET_ISO8859_1;

            // since there is no character encoding identification in ID3v1, the default
            // character set used here is SYS_CHARSET_ISO8859_1. In case any application
            // requires only Unicode, the following commented code can be used instead.

//            data->titleCs  = SYS_CHARSET_ISO_10646;
//            data->artistCs = SYS_CHARSET_ISO_10646;
//            data->albumCs  = SYS_CHARSET_ISO_10646;

            pos = 0;
        }
    }

    if ( data->picPos > 0 )
    {
        (void)f_seek(fileP, (int32_t)(data->picPos), F_SEEK_SET);
        (void)f_read(pictype, 1, 10, fileP);
        if(((pictype[0] == 'i') && (pictype[1] == 'm') && (pictype[2] == 'a') &&
           (pictype[6] == 'j') && (pictype[7] == 'p') && (pictype[8] == 'g')) )
        {
            data->jpegAvl = FTRUE;
            data->picPos = data->picPos +12;  //YC:point to the header of jpeg
        }
        else if(((pictype[0] == 'i') && (pictype[1] == 'm') && (pictype[2] == 'a') &&
           (pictype[6] == 'j') && (pictype[7] == 'p') && (pictype[8] == 'e') && (pictype[9] == 'g')) )
        {
            data->jpegAvl = FTRUE;
            data->picPos = data->picPos + 13;  //YC:point to the header of jpeg
        }
        else if(((pictype[0] == 'P') && (pictype[1] == 'N') && (pictype[2] == 'G') &&
           (pictype[6] == 'P') && (pictype[7] == 'N') && (pictype[8] == 'G')) )
        {
            data->jpegAvl = FTRUE;
            data->picPos = data->picPos + 5;  //YC:point to the header of png
        }
    }

    if ( id3Size > pos ) pos = id3Size;

    (void)f_close(fileP);

    return pos;
}

FSTATIC void MP3_ID3_ExtractString (char_t* targetStr, int32_t maxLen,
                                    SYS_CHARSET* charSet,
                                    const uint8_t* id3Buf,
                                    int32_t id3Size, uint8_t textEnc)
{
    fbool_t  bigEndian = FFALSE;
    int32_t  strBytes  = 0;

    if ( charSet != NULL ) *charSet = SYS_CHARSET_UNDEFINED;

    if ( (id3Size < 1) || (maxLen < 1) ) return;

    if ( textEnc == 0 ) /* ISO-8859-1 character set */
    {
        if ( id3Size < maxLen )
        {
            /* NOTE: One byte is added because some ID3 strings are not
                     terminated correctly. There is no risk because
                     UTILS_strncpy ensures termination within the given
                     boundaries. */
            maxLen = id3Size + 1;
        }

        strBytes = (int32_t)UTILS_strncpy(targetStr, (const char_t*)id3Buf,
                                         (uint32_t)maxLen);

        if ( charSet != NULL ) *charSet = SYS_CHARSET_ISO8859_1;
    }
    else if ( textEnc == 1 ) /* 16-bit unicode 2.0 */
    {
        if ( id3Size < 2 ) return;

        /* Decode Byte-order mark (BOM) */
        if ( (id3Buf[0] == 0xFF) && /* little endian */
             (id3Buf[1] == 0xFE) )
        {
            bigEndian = FFALSE;
        }
        else if ( (id3Buf[0] == 0xFE) && /* big endian*/
                  (id3Buf[1] == 0xFF) )
        {
            bigEndian = FTRUE;
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(4,"[MP3] Unicode string: no BOM found! \n");
            return;
        }

        id3Buf  += 2;
        id3Size -= 2;

        while ( (id3Size >= 2) &&
                ((strBytes + 1) < maxLen) )
        {
            targetStr[strBytes] = bigEndian ? ((char_t)(id3Buf[1]))
                                            : ((char_t)(id3Buf[0]));

            if ( targetStr[strBytes] == 0 ) break;

            strBytes++;
            id3Buf  += 2;
            id3Size -= 2;
        }

        /* add termination */
        targetStr[strBytes] = 0;

        /* NOTE: to do: Currently unicode characters are mapped onto ISO-8859-1
                 characters. In the future this will no be necessary anymore
                 as strings are automatically converted from their original
                 format to the one specified for rendering. */

        if ( charSet != NULL ) *charSet = SYS_CHARSET_ISO8859_1;

    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(4,"[MP3] Unsupported text encoding: %i \n", textEnc);
        return;
    }

    /* Cut trailing whitespaces */
    while ( strBytes > 0 )
    {
        if ( targetStr[strBytes - 1] == ' ' )
        {
            targetStr[strBytes - 1] = 0;
        }
        else
        {
            break;
        }

        strBytes--;
    }
}

/* Returns playback time in seconds. */
FSTATIC uint32_t MP3_ID3_ExtractTime (const char_t* id3Buf, int32_t id3Size,
                                      uint8_t textEnc)
{
    fbool_t    bigEndian = FFALSE;
    int32_t    cnt       = 0;
    uint32_t   length    = 0;
    char_t     chr;

    if ( id3Size < 1 ) return 0;

    if ( textEnc == 0 ) /* ISO-8859-1 character set */
    {
        for ( cnt = 0; cnt < id3Size; cnt++ )
        {
            if ( (id3Buf[cnt] >= '0') &&
                 (id3Buf[cnt] <= '9') )
            {
                length *= 10;
                length += (uint8_t)(id3Buf[cnt] - '0');
            }
        }
    }
    else if ( textEnc == 1 ) /* 16-bit unicode 2.0 */
    {
        if ( id3Size < 2 ) return 0;

        /* Decode Byte-order mark (BOM) */
        if ( (((uint8_t)id3Buf[0]) == 0xFF) && /* little endian */
             (((uint8_t)id3Buf[1]) == 0xFE) )
        {
            bigEndian = FFALSE;
        }
        else if ( (((uint8_t)id3Buf[0]) == 0xFE) && /* big endian*/
                  (((uint8_t)id3Buf[1]) == 0xFF) )
        {
            bigEndian = FTRUE;
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(4,"[MP3] Unicode string: no BOM found! \n");
            return 0;
        }

        id3Buf  += 2;
        id3Size -= 2;

        while ( id3Size >= 2 )
        {
            chr = bigEndian ? id3Buf[1] : id3Buf[0];

            if ( (chr >= '0') &&
                 (chr <= '9') )
            {
                length *= 10;
                length += (uint8_t)(chr - '0');
            }

            id3Buf  += 2;
            id3Size -= 2;
        }
    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(4,"[MP3] Unsupported text encoding: %i \n", textEnc);
        return 0;
    }

    return length / 1000;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
