/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/oad.c
**
**              $Id: oad.c 5947 2010-03-05 17:29:26Z ychen $
**
** \brief       over the air download abstraction layer.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/

#include <fota/fota.h>
#include <fapi/drv_flash.h>

#include "sys_api.h"
#include "av.h"
#include "utils.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define OAD_PARAMS_PID              0x0246
#define OAD_PARAMS_NUM_TABLE        2
#define OAD_PARAMS_PRODUCT_ID       1025

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

/* A */

typedef struct
{
    OAD_STATUS_E        state;

    FOTA_FilterParamsT  filterParams;

    uint32_t            versionId;
    uint32_t            numOfBytes;
    void*               dataPtr;

    uint32_t            progress;
    fbool_t             progressUpdate;

    SYS_NETWORK_S       network; //network information
    TUNE_ITEM_S         tunePars;
    int8_t              feIdx;  // frontend Index
    AV_FEDAT_S*         pFe;
    uint32_t            slotAddr;
    fbool_t             aborted;
    fbool_t             abortReq;

} OAD_DATA_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

static OAD_DATA_S*  oadDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void OAD_Notify ( void* dataPtr, uint32_t numOfBytes,
                          FOTA_FilterStatusT status, void* optDataPtr);

/*!
********************************************************************************
** \brief Initialize the OAD.
**
** This function initializes the OAD.
**
** \return
** - #FAPI_OK if initialization was successful
** - #APPL_SMARTGO_ERR_OUT_OF_MEMORY if there is not enough memory
**
********************************************************************************
*/
int32_t OAD_Init(const OAD_INIT_PARAMS_S* params)
{

    oadDat = (OAD_DATA_S*)UTILS_Malloc(sizeof(OAD_DATA_S));
    if ( oadDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(oadDat, 0, sizeof(OAD_DATA_S));

    oadDat->state = OAD_MODE_SLEEP;
    oadDat->versionId       = params->versionId;

    return FAPI_OK;
}

/*!
*****************************************************************************
** \brief Start the OAD.
**
** This function tune the stream with the given parameter and send the command
** to start OAD.
**
** \param network       the system network to start OAD.
** \param carrierInf    the carrier information to start OAD.
** \param feIdx         the front-end ID to start OAD.
** \param slotAddr      the flash stat address to store the OAD updated image.
**
** \return
**          - #FAPI_OK if the commond was successfully sent.
**
** \sa OAD_Stop
*****************************************************************************
*/
int32_t OAD_Start(const SYS_NETWORK_S* network, TPLIST_ITEM_S* carrierInf,
                  int8_t* feIdx, uint32_t slotAddr)
{
    AV_CMD_S        cmd;
    int32_t         retVal;
    int8_t      selFeIdx;

    oadDat->tunePars.params.sat.frequency = carrierInf->pars.sat.frequency;
    oadDat->tunePars.params.sat.symbol = carrierInf->pars.sat.symbolRate;
    oadDat->tunePars.params.sat.pol = carrierInf->pars.sat.pol;
    oadDat->tunePars.params.sat.modulation = carrierInf->pars.sat.modulation;

    oadDat->network = *(SYS_NETWORK_S*)network;

    oadDat->slotAddr = slotAddr;

    oadDat->aborted  = FFALSE;
    oadDat->abortReq = FFALSE;

    oadDat->filterParams.table_id[0]    = FAPI_TSD_USER_000_TABLE;
    oadDat->filterParams.table_id[1]    = FAPI_TSD_USER_001_TABLE;
    oadDat->filterParams.first_table_id = FAPI_TSD_USER_000_TABLE;
    oadDat->filterParams.product_id     = OAD_PARAMS_PRODUCT_ID;
    oadDat->filterParams.pid            = OAD_PARAMS_PID;
    oadDat->filterParams.numTables      = OAD_PARAMS_NUM_TABLE;

    //tune the stream and send the command to start the OTA
    retVal = AV_TuneModeStart(-1,
                              (const SYS_NETWORK_S*)&(oadDat->network),
                              /*listItem.tpIdx,*/-1,
                              /*NULL,*/(const TUNE_ITEM_S*)&(oadDat->tunePars),
                              &selFeIdx);

    if ( retVal != FAPI_OK ) return retVal;

    oadDat->feIdx = selFeIdx;

    cmd.type              = AV_CMD_START_OAD;
    cmd.arg.oad.feIdx      = selFeIdx;

    if ( feIdx != NULL ) *feIdx = selFeIdx;

    return AV_CmdSend(&cmd);
}

/*!
*****************************************************************************
** \brief Internally start the OAD.
**
** This function sets the AV thread to OAD status and starts the update.
**
** \param pFe     the pointer to the current front-end.
** \param pCmd    the pointer to the AV thread command.
**
** \return
**          - #FAPI_OK
**
** \sa OAD_Stop
*****************************************************************************
*/
int32_t AVINT_OAD_Start(AV_FEDAT_S*    pFe, AV_CMD_S* pCmd)
{
    //MISRA
    pCmd = pCmd;

    if ( (oadDat->state == OAD_MODE_SLEEP) )
    {
        oadDat->pFe     = pFe;
        oadDat->state   = OAD_MODE_START;
        pFe->state      = AV_FESTATE_OAD;
    }

    return FAPI_OK;
}

/*!
*****************************************************************************
** \brief Internally stop the OAD.
**
** This function stops the OAD process and set AV thread back to idle status.
**
** \param pFe     the pointer to the current front-end.
**
** \return
**          - #FAPI_OK
**
** \sa OAD_Stop
*****************************************************************************
*/
int32_t AVINT_OAD_Stop(AV_FEDAT_S* pFe)
{
    SYS_EVT_UPDATE_S    ctrlEvent;
    RSRC_OP             op;

    memset(&ctrlEvent, 0, sizeof(SYS_EVT_UPDATE_S));

    ctrlEvent.hdr.type    = EVT_UPDATE_STATUS;
    ctrlEvent.hdr.length  = sizeof(ctrlEvent);
    ctrlEvent.status      = UPDATE_STAT_ABORTED;
    ctrlEvent.programStat = UPDATE_PROGRAM_NONE;
    ctrlEvent.progCurr    = 0;
    ctrlEvent.progMax     = 1000;
    (void)EVT_Send(&ctrlEvent, "OAD Stop");

    (void)UPDATE_Close();
    (void)FOTA_Stop();
    /* needs calling FOTA_Task() manually to enter FOTA_UPDATE_STATE_ABORT
       state since AV_TaskOad won't be called anymore because av thread state
       has been changed */
    (void)FOTA_Task();
    FOTA_Exit();

    if ( pFe->feIdx == 0 ) op = RSRC_OP_TUNE1;
    else                   op = RSRC_OP_TUNE2;

    (void)PSISI_ReleaseTsdUsage (op, RSRC_TSD_OAD);

    oadDat->state      = OAD_MODE_SLEEP;
    oadDat->pFe->state = AV_FESTATE_IDLE;

    return FAPI_OK;
}

/*!
*****************************************************************************
** \brief Task of OAD updating.
**
** This function processes the on air download which should be called regularly
** by the OAD application, handles the filtering, downloading and all other
** events.
**
** \param pFe     the pointer to the current front-end.
**
** \return
**          - 20  sleep time
**
** \sa OAD_Stop
*****************************************************************************
*/
uint32_t AV_TaskOad (AV_FEDAT_S* pFe)
{
    int32_t             retVal;
    FE_STATUS_E         feStatus;
    SYS_EVT_UPDATE_S    ctrlEvent;

    memset(&ctrlEvent, 0, sizeof(SYS_EVT_UPDATE_S));

    ctrlEvent.hdr.type    = EVT_UPDATE_STATUS;
    ctrlEvent.hdr.length  = sizeof(ctrlEvent);
    ctrlEvent.programStat = UPDATE_PROGRAM_NONE;
    ctrlEvent.progMax     = 1000;

    retVal = FE_GetStatus(pFe->drvHdl, &feStatus);

    if ( ( retVal == FAPI_OK ) && (feStatus != FE_STATUS_LOCKED) )
    {
        retVal = AVINT_Tune(pFe, &(pFe->reqTunePar), FTRUE);
    }

    switch( oadDat->state )
    {
        case OAD_MODE_SLEEP:
            break;

        case OAD_MODE_START:
            ctrlEvent.progCurr    = 0;
            ctrlEvent.status      = UPDATE_STAT_DATACOLLECT;
            (void)EVT_Send(&ctrlEvent, "OAD");

            retVal = FOTA_Init(OAD_Notify,NULL);

            if ( retVal != FAPI_OK )
            {
                oadDat->state = OAD_MODE_ERROR;
                FAPI_SYS_PRINT_DEBUG(1,
                "[OAD] ERROR: FOTA_Init() result(%d)\n",retVal);
            }
            else
            {
                oadDat->state = OAD_MODE_FILTER;
            }
            break;

        case OAD_MODE_FILTER:
            retVal = FOTA_StartFiltering( &(oadDat->filterParams) );

            if ( retVal != FAPI_OK )
            {
                oadDat->state = OAD_MODE_ERROR;
                FAPI_SYS_PRINT_DEBUG(1,
                "[OAD] ERROR: FOTA_StartFiltering() result(%d)\n",retVal);
            }
            else
            {
                oadDat->state = OAD_MODE_RUNNING;
            }
            break;

        case OAD_MODE_RUNNING:
            retVal = FOTA_Task();

            if ( oadDat->progressUpdate )
            {
                ctrlEvent.progCurr    = oadDat->progress;
                ctrlEvent.status      = UPDATE_STAT_DATACOLLECT;
                (void)EVT_Send(&ctrlEvent, "OAD");
                oadDat->progressUpdate = FFALSE;
            }
            break;

        case OAD_MODE_UPDATE:
            oadDat->state = OAD_MODE_RUNNING;
            retVal = FOTA_StartUpdate( &(oadDat->filterParams) );

            if (( retVal != FAPI_OK ) && ( retVal != 1 ))
            {
                oadDat->state = OAD_MODE_ERROR;
                FAPI_SYS_PRINT_DEBUG(1,
                "[OAD] ERROR: FOTA_StartUpdate() result(%d)\n",retVal);
            }
            break;

        case OAD_MODE_PROGRAM:
            retVal = UPDATE_Open();

            if ( retVal != FAPI_OK )
            {
                oadDat->state = OAD_MODE_ERROR;
                FAPI_SYS_PRINT_DEBUG(1,
                "[OAD] ERROR: UPDATE_Open() result(%d)\n",retVal);
            }
            retVal = UPDATE_AddData ( oadDat->dataPtr, oadDat->numOfBytes);

            if ( retVal != FAPI_OK )
            {
                oadDat->state = OAD_MODE_ERROR;
                FAPI_SYS_PRINT_DEBUG(1,
                "[OAD] ERROR: UPDATE_AddData() result(%d)\n",retVal);
            }

            retVal = UPDATE_ProgramImage(oadDat->slotAddr);

            if ( retVal != FAPI_OK )
            {
                oadDat->state = OAD_MODE_ERROR;
                FAPI_SYS_PRINT_DEBUG(1,
                "[OAD] ERROR: UPDATE_ProgramImage() result(%d)\n",retVal);
            }

            (void)UPDATE_Close();
            FOTA_Exit();

            oadDat->state = OAD_MODE_STOP;

            break;

        case OAD_MODE_ERROR:
            ctrlEvent.status      = UPDATE_STAT_ERROR;
            ctrlEvent.progCurr    = 0;
            (void)EVT_Send(&ctrlEvent, "OAD Error");
            break;

        case OAD_MODE_STOP:
            oadDat->state      = OAD_MODE_SLEEP;
            oadDat->pFe->state = AV_FESTATE_IDLE;
            break;

        default:
            break;
    }

    return 20;
}

/*!
*****************************************************************************
** \brief Notification function for section filtering.
**
** This function notifies the current FOTA status.
**
** \param pFe     the pointer to the current front-end.
**
** \sa AV_TaskOad
*****************************************************************************
*/
FSTATIC void OAD_Notify ( void* dataPtr, uint32_t numOfBytes,
                          FOTA_FilterStatusT status, void* optDataPtr)
{
    //MISRA
    optDataPtr = optDataPtr;

    switch ( status.type )
    {
        case FOTA_EVENT_SECTION:
            FAPI_SYS_PRINT_DEBUG(1, "\n FOTA_EVENT_SECTION \n");
            FAPI_SYS_PRINT_DEBUG(1, "  -> Found Version for Update: %d\n",
                                    status.version);
            FAPI_SYS_PRINT_DEBUG(1, "  -> Found Product ID        : %d\n",
                                    status.productId);
            FAPI_SYS_PRINT_DEBUG(1, "\n Starting Download Software...\n\n");

            oadDat->filterParams.data_version = status.version;
            oadDat->state = OAD_MODE_UPDATE;
            break;

        case FOTA_EVENT_UPDATE_FINISHED:
            FAPI_SYS_PRINT_DEBUG(1, "\n FOTA_EVENT_UPDATE_FINISHED \n");

            oadDat->numOfBytes = numOfBytes;
            oadDat->dataPtr    = dataPtr;
            oadDat->state = OAD_MODE_PROGRAM;
            break;

        case FOTA_EVENT_UPDATE_INPROGRESS:
            FAPI_SYS_PRINT_DEBUG(1, "\n FOTA_EVENT_UPDATE_INPROGRESS \n");
            FAPI_SYS_PRINT_DEBUG(1, "  -> status.tableId      : 0x%02X\n",
                                    status.tableId);
            FAPI_SYS_PRINT_DEBUG(1, "  -> status.tableIndex   :   %02d\n",
                                    status.tableIndex);
            FAPI_SYS_PRINT_DEBUG(1, "  -> status.sectionIndex :   %02d\n",
                                    status.sectionIndex);
            FAPI_SYS_PRINT_DEBUG(1, "  -> Section Rec/Max     : %02d/%02d\n",
                                    status.sectReceived, status.sectMax);

            oadDat->progress = ((status.sectReceived*100)/status.sectMax)*10;
            oadDat->progressUpdate = FTRUE;
            break;

        case FOTA_EVENT_ABORT:
            oadDat->state = OAD_MODE_ERROR;
            break;

        default:
            break;
    }
}

fbool_t OAD_AbortReq()
{
    oadDat->aborted = FTRUE;
    return oadDat->abortReq;
}

/*!
*****************************************************************************
** \brief Stop the OAD.
**
** This function stops the tunning of the service and send the command to
** stop OAD.
**
** \return
**          - #FAPI_OK if the commond was successfully sent.
**
** \sa OAD_Start
*****************************************************************************
*/
int32_t OAD_Stop ()
{
    AV_CMD_S        cmd;
    int32_t         retVal;

    oadDat->abortReq = FTRUE;

    while(oadDat->aborted == FTRUE)
    {
        FAPI_SYS_SLEEP(100);
    }

    retVal = AV_TuneModeStop(oadDat->feIdx);

    if ( (retVal != FAPI_OK) &&
         (retVal != SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE) )
    {
         return retVal;
    }

    cmd.type            = AV_CMD_STOP_OAD;
    cmd.arg.abort.feIdx = oadDat->feIdx;

    return AV_CmdSend(&cmd);
}
