/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/parental_control.c
**
**              $Id: parental_control.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Implementation of parental control system.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save  */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <flashfs/flashfs.h>

#include "sys_api.h"
#include "sys_priv.h"
#include "utils.h"
#include "av.h"

#ifdef APPL_PVR_ENABLED
#include "pvr_list.h"
#endif /* APPL_PVR_ENABLED */

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/* According to ETSI guidelines, EIT present/following sections for the
   current TS shall be transmitted at least every 2 seconds. This timeout should
   to be chosen accordingly. */
#define PCTRL_EIT_TIMEOUT  3000  //!< timeout when waiting for EIT event in ms.

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef enum                  /** Internal authentication status. */
{
    AUTH_STAT_IDLE = 0,       //!< Parental control inactive.
    AUTH_STAT_NOTNEEDED,      //!< No authorization needed.
    AUTH_STAT_WAIT_EIT,       //!< Waiting for Parental rating descriptor.
    AUTH_STAT_WAIT_AUTH,      //!< Waiting for authorization.
    AUTH_STAT_OK_SINGLE,      //!< Authorized by normal PIN.
    AUTH_STAT_OK_MASTER       //!< Authorized by master PIN.

} PCTRL_AUTH_STATUS_E;

typedef struct
{
    uint32_t             id;                 // File identification.
    uint32_t             revision;           // File structure revision.
    PCTRL_PARAMS_S       params;
    char_t               pin[PCTRL_PIN_NUMS][PCTRL_PIN_LEN];
    uint32_t             idEnd;              // File identification.

} PCTRL_FILEHDR_S;

typedef struct
{
    RTOS_SemaphoreT      mutexId;             //!< Mutex semaphore.

    PCTRL_FILEHDR_S      cfg;
    fbool_t              dirty;               //!< FTRUE: current data does
                                              //!< not match data stored in Flash.

    /* internal parameters */
    fbool_t              enabled;
    SYS_PRESSTATE_E      presState;

    SYS_SERVLOC_S        authServLoc;
    SLIST_SERVIDX        authServIdx;

    #ifdef APPL_PVR_ENABLED
    PVR_LIST_ITEM_S      pvrItem;
    #endif /* APPL_PVR_ENABLED */

    PCTRL_AUTH_STATUS_E  authStatus;
    uint64_t             authStartTime;
    int32_t              authEvtId;           //!< current event ID: (-1: none)
    PCTRL_EVT_NEEDAUTH_S ctrlEvent;

    fbool_t              outputEnabled;
    fbool_t              newEventAvl;

} PCTRL_DATA_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC PCTRL_DATA_S* pctrlDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void    PCTRL_SetOutputEnable (fbool_t enable);
FSTATIC void    PCTRL_Check           (void);
FSTATIC void    PCTRL_Reset           (void);
FSTATIC fbool_t PCTRL_LoadData        (void);

int32_t PCTRL_Init (void)
{
    int32_t retVal;

    DBG_Assert(pctrlDat == NULL);

    /* Test if maximum flash file size is sufficient. */
    retVal = FLASH_CheckFileSize (FILEID_PCTRL, sizeof(PCTRL_FILEHDR_S));
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    pctrlDat = UTILS_Malloc(sizeof(PCTRL_DATA_S));
    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(pctrlDat, 0, sizeof(PCTRL_DATA_S));

    pctrlDat->mutexId = (RTOS_SemaphoreT)RTOS_CreateSemaphore(1);

    if ( pctrlDat->mutexId == (RTOS_SemaphoreT)NULL )
    {
        return SMARTGO_SYS_ERR_RESOURCES;
    }

    pctrlDat->dirty                  = FTRUE;

    pctrlDat->enabled                = FTRUE;
    pctrlDat->presState              = SYS_PRESSTATE_NONE;
    pctrlDat->authServLoc.listId     = SLIST_NONE;
    pctrlDat->authServLoc.index      = -1;
    pctrlDat->authServIdx            = -1;
    pctrlDat->authStatus             = AUTH_STAT_IDLE;
    pctrlDat->authEvtId              = -1;
    pctrlDat->outputEnabled          = FTRUE;
    pctrlDat->newEventAvl            = FFALSE;

    pctrlDat->ctrlEvent.hdr.type   = EVT_PCTRL_NEEDAUTH;
    pctrlDat->ctrlEvent.hdr.length = sizeof(pctrlDat->ctrlEvent);

    (void)PCTRL_Load();

    return FAPI_OK;
}

int32_t PCTRL_Wakeup (void)
{
    (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

    PCTRL_Reset           ();
    PCTRL_SetOutputEnable (FTRUE);

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);

    return FAPI_OK;
}

int32_t PCTRL_Shutdown (void)
{
    (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

    PCTRL_Reset();

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);

    return FAPI_OK;
}

int32_t PCTRL_SetEnableStatus (fbool_t enabled)
{
    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

    if ( pctrlDat->enabled != enabled )
    {
        pctrlDat->enabled = enabled;

        if ( !(pctrlDat->enabled) )
        {
            PCTRL_SetOutputEnable (FTRUE);
        }
    }

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);

    return FAPI_OK;
}

int32_t PCTRL_GetEnableStatus (fbool_t* enabled)
{
    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if ( enabled  == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    *enabled = pctrlDat->enabled;

    return FAPI_OK;
}

#ifdef APPL_PVR_ENABLED

/*!
*******************************************************************************
**
** \brief  function description
**
** More detailed description
**
** For a refresh of the authentication process it is necessary to reselect
** the current service.
**
** \param   idx     TODO
** \param   minAge  TODO
**
** \return  TODO
**
******************************************************************************/
int32_t PCTRL_SetRecordProtection (PVRLST_IDX idx, uint8_t minAge)
{
    PVR_LIST_ITEM_S     item;
    int32_t             retVal;

    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    retVal = PVR_LIST_GetItemByIndex (idx, &item, NULL);
    if ( retVal != FAPI_OK ) return retVal;

    item.service.minAge = minAge;

    return PVR_LIST_SetItem (&item, idx, FFALSE);
}

/*!
*******************************************************************************
**
** \brief  For a refresh of the authentication process it is necessary to reselect
**         the current service.
**
** \param idx       TODO
** \param minAge    TODO
**
** \return  TODO
******************************************************************************/
int32_t PCTRL_GetRecordProtection (PVRLST_IDX idx, uint8_t* minAge)
{
    PVR_LIST_ITEM_S     item;
    int32_t             retVal;

    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    retVal = PVR_LIST_GetItemByIndex (idx, &item, NULL);
    if ( retVal != FAPI_OK ) return retVal;

    *minAge = item.service.minAge;

    return FAPI_OK;
}

#endif /* APPL_PVR_ENABLED */


/*!
*******************************************************************************
**
** \brief   For a refresh of the authentication process it is necessary to reselect
**          the current service.
**
** \param   pServLoc    TODO
** \param   minAge      TODO
**
** \return  TODO
******************************************************************************/
int32_t PCTRL_SetServiceProtection (const SYS_SERVLOC_S* pServLoc, uint8_t minAge)
{
    SLIST_ITEM_S    item;
    SLIST_SERVIDX   servIdx;
    int32_t         retVal;

    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    servIdx = SLIST_GetServiceIndex (pServLoc);
    if ( servIdx < 0 ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    retVal = SLIST_GetServByIndex (servIdx, &item);
    if ( retVal != FAPI_OK ) return retVal;

    item.minAge = minAge;

    return SLIST_SetServByIndex (servIdx, &item);
}

/*!
*******************************************************************************
**
** \brief  TODO
**
** \param  pServLoc TODO
** \param  minAge   TODO
**
** \return  TODO
******************************************************************************/
int32_t PCTRL_GetServiceProtection (const SYS_SERVLOC_S* pServLoc, uint8_t* minAge)
{
    int32_t         retVal;
    SLIST_ITEM_S    item;

    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    retVal = SLIST_EntryGet (pServLoc, &item);
    if ( retVal == FAPI_OK )
    {
        *minAge = item.minAge;
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  TODO
**
** \return  TODO
******************************************************************************/
int32_t PCTRL_Load (void)
{
    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    FAPI_SYS_PRINT_DEBUG(1,"[SYS] Load parental control settings\n");
    /* read service data list */
    if ( PCTRL_LoadData() == FFALSE )
    {
        FAPI_SYS_PRINT_DEBUG(1,"[SYS] Set parental control defaults\n");
        (void)PCTRL_RestoreDefaults ();
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  TODO
**
** \param   force   TODO
**
** \return Return values
******************************************************************************/
int32_t PCTRL_Save (fbool_t force)
{
    int32_t         retVal;
    FLASHFS_HANDLE  file = NULL;

    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if ( (!force) && (!(pctrlDat->dirty)) ) return FAPI_OK;

    (void)RTOS_GetSemaphore(pctrlDat->mutexId, 1);

    file = FLASHFS_Open (FILEID_PCTRL, FLASHFS_OPEN_WRITE, NULL);
    if ( file == NULL )
    {
        retVal = SMARTGO_SYS_ERR_INTERNAL;
    }
    else
    {
        pctrlDat->cfg.id       = FILEID_PCTRL;
        pctrlDat->cfg.idEnd    = FILEID_PCTRL;
        pctrlDat->cfg.revision = FILEREV_PCTRL;

        /* write data */
        if ( FLASHFS_Write (file, sizeof(PCTRL_FILEHDR_S),
                            &(pctrlDat->cfg), NULL)
             != (int32_t)sizeof(PCTRL_FILEHDR_S) )
        {
            retVal = (int32_t)SMARTGO_SYS_ERR_INTERNAL;
        }
        else
        {
            retVal = FAPI_OK;
        }

        (void)FLASHFS_Close(file);
    }

    if ( retVal == FAPI_OK ) pctrlDat->dirty = FFALSE;
    else                     pctrlDat->dirty = FTRUE;

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);

    return retVal;
}

/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   params  TODO
**
** \return  TODO
******************************************************************************/
int32_t PCTRL_SetParams (const PCTRL_PARAMS_S* params)
{
    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

    pctrlDat->cfg.params = *params;
    pctrlDat->dirty      = FTRUE;

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  TODO
**
** \param   params  TODO
**
** \return  TODO
******************************************************************************/
int32_t PCTRL_GetParams (PCTRL_PARAMS_S* params)
{
    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

    *params = pctrlDat->cfg.params;

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  TODO
**
** \return TODO
******************************************************************************/
int32_t PCTRL_Restart (void)
{
    int32_t retVal;

    if ( pctrlDat == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

        /* restart parental control process if parameters have changed. */
        switch ( pctrlDat->presState )
        {
            case SYS_PRESSTATE_LIVETV:
            case SYS_PRESSTATE_PLAYBACK:
                pctrlDat->authEvtId   = -1;
                pctrlDat->newEventAvl = FFALSE;

                if ( pctrlDat->authStatus != AUTH_STAT_OK_MASTER )
                {
                    pctrlDat->authStatus = AUTH_STAT_IDLE;
                }

                PCTRL_Check();
                break;

            default:
                /* do nothing */
                break;
        }

        (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);

        retVal = FAPI_OK;
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  TODO
**
** \param   pStatus  TODO
**
** \return  TODO
******************************************************************************/
int32_t PCTRL_GetStatus (PCTRL_STATUS_S* pStatus)
{
    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

    memset(pStatus, 0, sizeof(PCTRL_STATUS_S));

    pStatus->outputLocked = (fbool_t)!(pctrlDat->outputEnabled);

    switch ( pctrlDat->authStatus )
    {
        default:
        case AUTH_STAT_IDLE:
        case AUTH_STAT_WAIT_EIT:  pStatus->status = PCTRL_STAT_UNKNOWN;
                                  break;

        case AUTH_STAT_NOTNEEDED: pStatus->status = PCTRL_STAT_NOTNEEDED;
                                  break;

        case AUTH_STAT_WAIT_AUTH: pStatus->status = PCTRL_STAT_LOCKED;
                                  break;

        case AUTH_STAT_OK_SINGLE: pStatus->status = PCTRL_STAT_UNLOCKED_SINGLE;
                                  break;

        case AUTH_STAT_OK_MASTER: pStatus->status = PCTRL_STAT_UNLOCKED_MASTER;
                                  break;
    }

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   type    TODO
** \param   pin     TODO
**
** \return  TODO
**
******************************************************************************/
int32_t PCTRL_SetPin (PCTRL_PIN_TYPE_E type, const char_t* pin)
{
    uint8_t cnt;

    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if ( type >= PCTRL_PIN_NUMS ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

    for ( cnt = 0; cnt < PCTRL_PIN_LEN; cnt++ )
    {
        pctrlDat->cfg.pin[type][cnt] = pin[cnt];
    }

    pctrlDat->dirty  = FTRUE;

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   type    TODO
** \param   pin     TODO
**
** \return  TODO
**
******************************************************************************/
int32_t PCTRL_CheckPin (PCTRL_PIN_TYPE_E type, const char_t* pin)
{
    uint16_t cnt;

    if ( type >= PCTRL_PIN_NUMS ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    for ( cnt = 0; cnt < PCTRL_PIN_LEN; cnt++ )
    {
        if ( pin[cnt] != pctrlDat->cfg.pin[type][cnt] )
        {
            return SMARTGO_SYS_ERR_INVALID_DATA;
        }
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   pServLoc    TODO
** \param   pin         TODO
**
** \return Return values
**      - #FAPI_OK                          Successful authentication.
**      - #SMARTGO_SYS_ERR_INVALID_DATA  No matching PIN found.
**
******************************************************************************/
int32_t PCTRL_AuthenticateService (const SYS_SERVLOC_S* pServLoc, const char_t* pin)
{
    int32_t retVal;

    /* MISRA */
    pServLoc = pServLoc;

    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

    if ( pctrlDat->authStatus != AUTH_STAT_WAIT_AUTH )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        /* test if service matches */
        //if ( (pctrlDat->authServLoc.listId != pServLoc->listId) ||
        //     (pctrlDat->authServLoc.index  != pServLoc->index) )
        //{
        //    retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
        //}
        //else
        //{
            /* Test if PIN is correct, first master, then single PIN. */

            retVal = PCTRL_CheckPin (PCTRL_PIN_MASTER, pin);

            if ( retVal == FAPI_OK )
            {
                pctrlDat->authStatus = AUTH_STAT_OK_MASTER;
            }
            else
            {
                retVal = PCTRL_CheckPin (PCTRL_PIN_SINGLE, pin);

                if ( retVal == FAPI_OK )
                {
                    pctrlDat->authStatus = AUTH_STAT_OK_SINGLE;
                }
            }
        //}
    }

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);

    return retVal;
}

int32_t PCTRL_RestoreDefaults (void)
{
    uint16_t pinCnt;
    uint8_t  pinPos;

    if ( pctrlDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

    memset(&(pctrlDat->cfg.params), 0, sizeof(PCTRL_PARAMS_S));

    pctrlDat->cfg.params.country         = CTRY_NOTSPECIFIED;
    pctrlDat->cfg.params.useStreamRating = FFALSE;
    pctrlDat->cfg.params.viewerAge       = 18;
    pctrlDat->cfg.params.protectionArea  = 0;

    for ( pinCnt = 0; pinCnt < (int32_t)PCTRL_PIN_NUMS; pinCnt++ )
    {
        for ( pinPos = 0; pinPos < PCTRL_PIN_LEN; pinPos++ )
        {
            pctrlDat->cfg.pin[pinCnt][pinPos] = '0';
        }
    }

    pctrlDat->dirty = FTRUE;

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);

    if ( SYS_GetConfig()->setParentalCtrlDefaults != NULL )
    {
        SYS_GetConfig()->setParentalCtrlDefaults();
    }

    return FAPI_OK;
}

void PCTRL_LiveStartHandling (const SYS_SERVLOC_S* pServLoc,
                              const SLIST_ITEM_S* service)
{
    SLIST_SERVIDX newServIdx;

    /* MISRA */
    service = service;

    (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

    newServIdx = SLIST_GetServiceIndex (pServLoc);

    pctrlDat->presState   = SYS_PRESSTATE_LIVETV;
    pctrlDat->authServLoc = *pServLoc;

    if ( newServIdx != pctrlDat->authServIdx )
    {
        pctrlDat->authEvtId   = -1;
        pctrlDat->authServIdx = newServIdx;
        pctrlDat->newEventAvl = FFALSE;

        if ( pctrlDat->authStatus != AUTH_STAT_OK_MASTER )
        {
            pctrlDat->authStatus = AUTH_STAT_IDLE;
        }
    }

    PCTRL_Check();

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);
}

#ifdef APPL_PVR_ENABLED

void PCTRL_PlayStartHandling (const PVR_LIST_ITEM_S* pvrItem,
                              SLIST_SERVIDX servIdx)
{
    (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

    pctrlDat->presState          = SYS_PRESSTATE_PLAYBACK;
    pctrlDat->authServLoc.listId = SLIST_NONE;
    pctrlDat->authServLoc.index  = -1;
    pctrlDat->pvrItem            = *pvrItem;

    if ( servIdx != pctrlDat->authServIdx )
    {
        pctrlDat->authEvtId   = -1;
        pctrlDat->authServIdx = servIdx;
        pctrlDat->newEventAvl = FFALSE;
        if ( pctrlDat->authStatus != AUTH_STAT_OK_MASTER )
        {
            pctrlDat->authStatus = AUTH_STAT_IDLE;
        }
    }

    PCTRL_Check();

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);
}

#endif /* APPL_PVR_ENABLED */

void PCTRL_Stop (void)
{
    PCTRL_SetOutputEnable(FTRUE);
}

void PCTRL_EitEventHandling (SLIST_SERVIDX servIdx)
{
    EIT_EVT_S       currEvt;
    int32_t         retVal;
    fbool_t         newCheck = FFALSE;
    SYS_SERVLOC_S*  pServLoc = NULL;

//FM_Printf("[PCTRL] servIdx=%d, pctrlDat->authServIdx=%d,authStat=%d \n", servIdx,pctrlDat->authServIdx,pctrlDat->authStatus);

    if ( !(pctrlDat->cfg.params.useStreamRating) ) return;

    (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

    switch ( pctrlDat->authStatus )
    {
        case AUTH_STAT_NOTNEEDED:
        case AUTH_STAT_OK_SINGLE:
        case AUTH_STAT_WAIT_AUTH:
        case AUTH_STAT_WAIT_EIT:
            newCheck = FTRUE;
            break;

        default:
        case AUTH_STAT_IDLE:
        case AUTH_STAT_OK_MASTER:
            /* no checks necessary */
            break;
    }
    if ( newCheck && (servIdx == pctrlDat->authServIdx) )
    {
        if ( pctrlDat->presState == SYS_PRESSTATE_LIVETV )
        {
            pServLoc = &(pctrlDat->authServLoc);
        }

        retVal = EIT_GetEvent (pServLoc, EIT_TYPE_CURR, &currEvt);

        switch ( retVal )
        {
            case FAPI_OK:
            case SMARTGO_SYS_ERR_DATA_NOT_PROVIDED:

                /* New 'present' event for current service is either available
                   or not provided. */

                if ( (pctrlDat->authEvtId != (int32_t)currEvt.id) )
                {
                    /* Trigger new authorization check based on new EIT event
                       data. */

                    pctrlDat->newEventAvl = FTRUE;
                }
                break;

            default:
                break;
        }
    }

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);
}

void PCTRL_Task (void)
{
    (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

    if ( pctrlDat->newEventAvl )
    {
        PCTRL_Check();
        pctrlDat->newEventAvl = FFALSE;
    }

    switch ( pctrlDat->authStatus )
    {
        case AUTH_STAT_WAIT_EIT:  /* waiting for EIT event */

            /* test for timeout expiration while waiting for EIT event */
            if ( UTILS_GetTimerStampInterval(pctrlDat->authStartTime,
                                             SYS_GET_TIMESTAMP_MS)
                                             >= PCTRL_EIT_TIMEOUT )
            {
                pctrlDat->authStatus = AUTH_STAT_NOTNEEDED;

                FAPI_SYS_PRINT_DEBUG(4,"[PCTRL] Wait EIT -> TIMEOUT \n");

                PCTRL_SetOutputEnable(FTRUE);
            }
            break;

        case AUTH_STAT_OK_MASTER:
        case AUTH_STAT_OK_SINGLE:
            if ( !(pctrlDat->outputEnabled) )
            {
                PCTRL_SetOutputEnable(FTRUE);
            }
            break;

        default:
            /* do nothing */
            break;
    }

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);
}

FSTATIC void PCTRL_Check (void)
{
    EIT_EVT_S       currEvt;
    int32_t         retVal;
    uint8_t         minAgeManual = 18;
    SYS_SERVLOC_S*  pServLoc = NULL;

    /* do nothing if permanently authorized or parental control is disabled! */
    if ( (!(pctrlDat->enabled)) ||
         (pctrlDat->authStatus == AUTH_STAT_OK_MASTER) ) return;

    switch ( pctrlDat->presState )
    {
        case SYS_PRESSTATE_LIVETV:
            pServLoc = &(pctrlDat->authServLoc);
            retVal   = PCTRL_GetServiceProtection (pServLoc, &minAgeManual);
            break;

        #ifdef APPL_PVR_ENABLED

        case SYS_PRESSTATE_PLAYBACK:
            minAgeManual = pctrlDat->pvrItem.service.minAge;
            retVal       = FAPI_OK;
            break;

        #endif /* APPL_PVR_ENABLED */

        default:
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
    }

    if ( retVal != FAPI_OK )
    {
        return;
    }

    /* ----- handle stream rating ------------------------------------------ */

    if ( pctrlDat->cfg.params.useStreamRating )
    {
        retVal = EIT_GetEvent (pServLoc, EIT_TYPE_CURR, &currEvt);
        switch ( retVal )
        {
            case FAPI_OK: /* event available */

                if ( pctrlDat->authEvtId != (int32_t)currEvt.id )
                {
                    if ( currEvt.minAge > pctrlDat->cfg.params.viewerAge )
                    {
                        pctrlDat->authStatus        = AUTH_STAT_WAIT_AUTH;
                        pctrlDat->ctrlEvent.eventId = currEvt.id;
                        pctrlDat->ctrlEvent.minAge  = currEvt.minAge;
                    }
                    else
                    {
                        pctrlDat->authStatus = AUTH_STAT_NOTNEEDED;
                    }

                    pctrlDat->authEvtId = (int32_t)currEvt.id;
                }
                break;

            case SMARTGO_SYS_ERR_DATA_NOT_PROVIDED: /* event not provided */
                pctrlDat->authStatus = AUTH_STAT_NOTNEEDED;
                break;

            default: /* event not yet available */
                if ( (pctrlDat->authStatus == AUTH_STAT_IDLE) &&
                     (pctrlDat->authEvtId  == -1) )
                {
                    pctrlDat->authStatus    = AUTH_STAT_WAIT_EIT;
                    pctrlDat->authStartTime = SYS_GET_TIMESTAMP_MS;
                }
                break;
        }
    }
    else
    {
        pctrlDat->authStatus = AUTH_STAT_NOTNEEDED;
    }

    /* ----- handle manual lock mechanism ---------------------------------- */

    switch ( pctrlDat->authStatus )
    {
        case AUTH_STAT_IDLE:
        case AUTH_STAT_NOTNEEDED:
        case AUTH_STAT_WAIT_EIT:
            if ( minAgeManual > pctrlDat->cfg.params.viewerAge )
            {
                pctrlDat->authStatus        = AUTH_STAT_WAIT_AUTH;
                pctrlDat->ctrlEvent.eventId = 0; /* not a specific event */
                pctrlDat->ctrlEvent.minAge  = minAgeManual;
            }
            break;

        default:
            /* do nothing */
            break;
    }

    /* ----- handle final status ------------------------------------------- */

    if ( pctrlDat->authStatus == AUTH_STAT_IDLE )
    {
        pctrlDat->authStatus = AUTH_STAT_NOTNEEDED;
    }

    switch ( pctrlDat->authStatus )
    {
        case AUTH_STAT_NOTNEEDED:
            PCTRL_SetOutputEnable(FTRUE);
            break;

        case AUTH_STAT_WAIT_EIT:
            FAPI_SYS_PRINT_DEBUG(4,"[PCTRL] Wait for EIT... \n");
            PCTRL_SetOutputEnable(FFALSE);
            break;

        case AUTH_STAT_WAIT_AUTH:
            PCTRL_SetOutputEnable(FFALSE);

            if ( pServLoc != NULL )
            {
                pctrlDat->ctrlEvent.servLoc = *pServLoc;
            }
            else
            {
                pctrlDat->authServLoc.listId = SLIST_NONE;
                pctrlDat->authServLoc.index  = -1;
            }

            pctrlDat->ctrlEvent.state = pctrlDat->presState;

            (void)EVT_Send(&(pctrlDat->ctrlEvent), "PCTRL Auth");
            break;

        default:
            break;
    }
}

FSTATIC void PCTRL_Reset (void)
{
    pctrlDat->presState          = SYS_PRESSTATE_NONE;
    pctrlDat->authServLoc.listId = SLIST_NONE;
    pctrlDat->authServLoc.index  = -1;
    pctrlDat->authServIdx        = -1;
    pctrlDat->authStatus         = AUTH_STAT_IDLE;
    pctrlDat->authEvtId          = -1;
    pctrlDat->newEventAvl        = FFALSE;
}

FSTATIC void PCTRL_SetOutputEnable (fbool_t enable)
{
    SYS_EVTHDR_S ctrlEvent;

    if ( pctrlDat->outputEnabled != enable )
    {
        pctrlDat->outputEnabled = enable;

        if ( enable )
        {
            (void)AVINT_VideoBlank(FFALSE);
            AVINT_AudioMuteRequestClear (AV_MUTEREQ_PARENTAL);

            ctrlEvent.type   = EVT_PCTRL_NOMOREAUTH;
            ctrlEvent.length = sizeof(ctrlEvent);

            (void)EVT_Send(&ctrlEvent, "PCTRL Noauth");
        }
        else
        {
            // TODO: use API function for this
            if(avData->liveTv.radioScreenVisible == FFALSE)
            {
                (void)AVINT_VideoBlank(FTRUE);
            }

            AVINT_AudioMuteRequestSet (AV_MUTEREQ_PARENTAL);
        }
    }
}

FSTATIC fbool_t PCTRL_LoadData (void)
{
    fbool_t         retVal = FFALSE;
    int32_t         res;
    FLASHFS_HANDLE  file = NULL;

    (void)RTOS_GetSemaphore (pctrlDat->mutexId, 1);

    file = FLASHFS_Open (FILEID_PCTRL, FLASHFS_OPEN_READ, NULL);
    if ( file != NULL )
    {
        for(;;)
        {
            /* ----- read file header -------------------------------------- */
            res = FLASHFS_Read (file, sizeof(PCTRL_FILEHDR_S), &(pctrlDat->cfg), NULL);
            if ( res != (int32_t)sizeof(PCTRL_FILEHDR_S) )
            {
                break; /* file read error, file too small */
            }
            else if ( (pctrlDat->cfg.id       != FILEID_PCTRL)  ||
                      (pctrlDat->cfg.idEnd    != FILEID_PCTRL)  ||
                      (pctrlDat->cfg.revision != FILEREV_PCTRL) )
            {
                break;
            }

            retVal = FTRUE;

            break;//end of for(;;)
        }
        (void)FLASHFS_Close(file);
    }

    if ( retVal ) pctrlDat->dirty = FFALSE;
    else          pctrlDat->dirty = FTRUE;

    (void)RTOS_SetSemaphore (pctrlDat->mutexId, RTOS_NO_SUSPEND);

    return retVal;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
