/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/psi_si.c
**
**              $Id: psi_si.c 5925 2010-03-05 06:43:50Z jmorha $
**
** \brief       SI Table filtering module.
**
** The following tables are supported:
**  - PAT: Program Association Table
**  - PMT: Program Map Table
**  - CAT: Conditional Access
**  - SDT: Service Description
**  - NIT: Network information
**  - TDT: Time Date Table
**  - TOT: Time Offset Table
**
** See here for more detail documentation: \ref app_smartgo_psisi
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info  40: Some defines used in PSISI_Init() placed in makefile.
** - Info 715: Symbol not referenced.
**
******************************************************************************/
/*lint -save -e40 -e715 */
#define DEBUG 4

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <sfal/sfal.h>

#include "av.h"
#include "sys_time.h"
#include "event_queue.h"
#include "utils.h"


/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

//#define PSISI_PAT_DEBUG         /* Program Association Table Debug */
//#define PSISI_PAT_DEBUG_ERR     /* Program Association Table Error Debug */

//#define PSISI_PMT_DEBUG         /* Program Map Table Debug */
//#define PSISI_PMT_DEBUG_ERR     /* Program Map Table Error Debug */
//#define PSISI_PMT_DEBUG_DUMP    /* Program Map Table Print Table Dump */

//#define PSISI_CAT_DEBUG         /* Conditional Access Table Debug */
//#define PSISI_CAT_DEBUG_ERR     /* Conditional Access Table Error Debug */

//#define PSISI_SDT_DEBUG         /* Service Description Table Debug */
//#define PSISI_SDT_DEBUG_ERR     /* Service Description Table Error Debug */
//#define PSISI_SDT_DEBUG_DUMP    /* Service Description Table Print Table Dump */

//#define PSISI_NIT_DEBUG         /* Network information Table Debug */
//#define PSISI_NIT_DEBUG_ERR     /* Network information Table Error Debug */

//#define PSISI_TDT_DEBUG         /* Time Date Table Debug */
//#define PSISI_TDT_DEBUG_ERR     /* Time Date Table Error Debug */

//#define PSISI_TOT_DEBUG         /* Time Offset Table Debug */
//#define PSISI_TOT_DEBUG_ERR     /* Time Offset Table Error Debug*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    uint8_t                 tsdIdx;             //!< TSD index.
    RSRC_OP                 op;                 //!< Resource manager info.

    FAPI_SYS_HandleT        patHdl;             //!< PAT Filter handle
    fbool_t                 patFilterActive;    //!< PAT active flag.
    PSISI_PAT_DATA_S*       pPat;               //!< PAT parsed table content
    #ifdef APPL_MHEG_ENABLED
    PSISI_PAT_BUF_S         pPatBuf;            //!< PAT full table.
    #endif
    FAPI_SYS_HandleT        nitHdl;             //!< NIT Filter handle
    fbool_t                 nitFilterActive;    //!< NIT active flag.
    fbool_t                 nitFilterTimeout;   //!< Timeout for NIT filtering.
    PSISI_NIT_S             nit;                //!< NIT parsed table content.

    FAPI_SYS_HandleT        pmtHdl;             //!< PMT Filter handle
    fbool_t                 pmtFilterActive;    //!< PMT active flag.
    PSISI_FILTER_S          pmtFilter;          //!< PMT Filter Date.
    uint16_t                pmtFilterPid;       //!< Current PMT PID.
    uint16_t                pmtFilterPrgNum;    //!< Current PMT programm number.
    PSISI_PMT_S             pmt;                //!< PMT parsed table content.

    #if (defined APPL_CI_ENABLED) || (defined APPL_CA_ENABLED)

    FAPI_SYS_HandleT        catHdl;             //!< CAT Filter handle.
    fbool_t                 catFilterActive;    //!< CAT active flag.
    fbool_t                 catWantUpdates;     //!< CAT table update was requested.
    PSISI_SECT_NOTIFY_CB    catSectNotify;      //!< CAT Section notification callback.
    PSISI_CAT_S             cat;                //!< CAT parsed table content.
    PSISI_CAT_S             catTmp;             //!< temporary stored CAT.

    #endif /* (defined APPL_CI_ENABLED) || (defined APPL_CA_ENABLED) */

} PSISI_TSD_S;

typedef struct
{
#ifndef _LINUX_FAPI
    RTOS_SemaphoreT     psiMutex;               //!< Mutex semaphore.
#else
    FAPEX_SYS_SemaphoreT     psiMutex;               //!< Mutex semaphore.
#endif
    FAPI_SYS_HandleT    tdttotHdl;              //!< TDT TOT Filter handle.
    fbool_t             tdttotFilterActive;     //!< TDT TOT active flag.
    FAPI_SYS_HandleT    sdtHdl;                 //!< SDT Filter handle.
    fbool_t             sdtFilterActive;        //!< SDT active flag.
    fbool_t             siFiltersOpen;

    PSISI_TSD_S         tsd[PSI_USED_TSDS];     //!< TODO

} PSISI_DATA_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC PSISI_DATA_S* psiDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC int32_t     PSISI_SectionGetNext(const uint32_t * pSectFound,
                                         uint8_t * sectNum);

FSTATIC void        PSISI_FilterClear(PSISI_FILTER_S * pFilter);
FSTATIC void        PSISI_FilterSetVersion(PSISI_FILTER_S * pFilter,
                                           uint8_t version, fbool_t equal);

FSTATIC int32_t     PSISI_PAT_StartFiltering(uint8_t tsdIdx, uint8_t version,
                                             fbool_t equal);
FSTATIC int32_t     PSISI_PAT_StopFiltering(PSISI_TSD_S * pTsd);

FSTATIC void        PSISI_PAT_SectNotify(FAPI_SYS_HandleT handle,
                                         SFAL_SECTION_STATUS_E sectionStatus,
                                         const uint8_t * pSectionBuffer,
                                         uint16_t sectionSize,
                                         void *pPrivateData);
FSTATIC void        PSISI_PMT_SectNotify(FAPI_SYS_HandleT handle,
                                         SFAL_SECTION_STATUS_E sectionStatus,
                                         const uint8_t * pSectionBuffer,
                                         uint16_t sectionSize,
                                         void *pPrivateData);
FSTATIC void        PSISI_TDTTOT_SectNotify(FAPI_SYS_HandleT handle,
                                         SFAL_SECTION_STATUS_E sectionStatus,
                                            const uint8_t * pSectionBuffer,
                                            uint16_t sectionSize,
                                            void *pPrivateData);
FSTATIC void        PSISI_TOT_Process(const uint8_t * pSection,
                                      uint16_t sectionSize);

FSTATIC void        PSISI_SDT_SectNotify(FAPI_SYS_HandleT handle,
                                         SFAL_SECTION_STATUS_E sectionStatus,
                                         const uint8_t * pSectionBuffer,
                                         uint16_t sectionSize,
                                         void *pPrivateData);
#if (defined APPL_CI_ENABLED) || (defined APPL_CA_ENABLED)
FSTATIC void        PSISI_CAT_SectNotify(FAPI_SYS_HandleT handle,
                                         SFAL_SECTION_STATUS_E sectionStatus,
                                         const uint8_t * pSectionBuffer,
                                         uint16_t sectionSize,
                                         void *pPrivateData);
FSTATIC int32_t     PSISI_CAT_NotifyReception(uint8_t tsdIdx);
#endif

FSTATIC void        PSISI_NIT_SectNotify(FAPI_SYS_HandleT handle,
                                         SFAL_SECTION_STATUS_E sectionStatus,
                                         const uint8_t * pSectionBuffer,
                                         uint16_t sectionSize, void *pPrivateData);


/*!
*******************************************************************************
**
** \brief   This funtion initialize the SmartGo-HD PSI-SI module.
**
** This function allocates a memory block for the global psisi structuce.
** Also all SFAL filtersets and filters needed for PSISI filtering are opened
** in this function.
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_OUT_OF_MEMORY
**          - #SMARTGO_SYS_ERR_RESOURCES
**          - SFAL error codes.
**
*******************************************************************************
*/
int32_t PSISI_Init(void)
{
    SFAL_FILTERSET_S    filterSetPars;
    int32_t             retVal = FAPI_OK;
    FAPI_SYS_HandleT    filterSetHdl;
    uint8_t             cnt;
    PSISI_FILTER_S      tdtFilter;
    PSISI_TSD_S        *pTsd;

    DBG_Assert(psiDat == NULL);

    for(;;)
    {
        psiDat = UTILS_Malloc(sizeof(PSISI_DATA_S));
        if(psiDat == NULL)
        {
            retVal = APPL_SMARTGO_ERR_OUT_OF_MEMORY;
            break;
        }

        memset(psiDat, 0, sizeof(PSISI_DATA_S));

#ifndef _LINUX_FAPI
        psiDat->psiMutex = RTOS_CreateSemaphore(1);
        if(psiDat->psiMutex == (RTOS_SemaphoreT) NULL)
#else
        psiDat->psiMutex = FAPEX_SYS_CreateSemaphore(1);
        if(psiDat->psiMutex == (FAPEX_SYS_SemaphoreT) NULL)
#endif
        {
            retVal = SMARTGO_SYS_ERR_RESOURCES;
            break;
        }

        /* ----- Setup one filter for each TSD needed ------------------- */
        for ( cnt = 0; cnt < PSI_USED_TSDS; cnt++ )
        {
            pTsd = psiDat->tsd + cnt;
            pTsd->tsdIdx = cnt;

            /* ----- setup PAT filter --------------------------------------- */
            pTsd->patFilterActive = FFALSE;

#ifndef _LINUX_FAPI
            filterSetPars.tsdInput          = FAPI_TSD0 + cnt;
#else
            filterSetPars.tsdInput          = FAPEX_TSD0 + cnt;
#endif
            filterSetPars.numberOfChannels  = 1;
            filterSetPars.filtersPerChannel = 1;
            filterSetPars.filterDepth       = PAT_FILTER_DEPTH;
            filterSetPars.bufferSize        = INIT_PAT_SIZE;
#ifndef _LINUX_FAPI
            filterSetPars.bufferAddress = INIT_PAT_ADDR + (INIT_PAT_SIZE * cnt);
#endif
            retVal = SFAL_OpenFilterset(&filterSetPars, &filterSetHdl);
            if(retVal != FAPI_OK)
            {
                #ifdef PSISI_PAT_DEBUG_ERR
                FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [PAT] SFAL_OpenFilterset() = %i\n",retVal);
                #endif
                break;
            }

            retVal = SFAL_OpenFilter(filterSetHdl, PAT_FILTER_DEPTH,
                                      &(pTsd->patHdl));
            if(retVal != FAPI_OK)
            {
                #ifdef PSISI_PAT_DEBUG_ERR
                FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [PAT] SFAL_OpenFilter() = %i\n",retVal);
                #endif
                break;
            }

            retVal = SFAL_SetFilterPid(pTsd->patHdl, ISO_PAT_PID);
            if(retVal != FAPI_OK)
            {
                #ifdef PSISI_PAT_DEBUG_ERR
                FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [PAT] SFAL_SetFilterPid() = %i\n",retVal);
                #endif
                break;
            }

            /* ----- setup PMT filter --------------------------------------- */
            pTsd->pmtFilterActive = FFALSE;
            pTsd->pmt.available   = FFALSE;

            filterSetPars.filterDepth   = PMT_FILTER_DEPTH;
            filterSetPars.bufferSize    = INIT_PMT_SIZE;
#ifndef _LINUX_FAPI
            filterSetPars.bufferAddress = INIT_PMT_ADDR + (INIT_PMT_SIZE * cnt);
#endif
            retVal = SFAL_OpenFilterset(&filterSetPars, &filterSetHdl);
            if(retVal != FAPI_OK)
            {
                #ifdef PSISI_PMT_DEBUG_ERR
                FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [PMT] SFAL_OpenFilterset() = %i\n",retVal);
                #endif
                break;
            }

            retVal = SFAL_OpenFilter(filterSetHdl, PMT_FILTER_DEPTH,
                                      &(pTsd->pmtHdl));
            if(retVal != FAPI_OK)
            {
                #ifdef PSISI_PMT_DEBUG_ERR
                FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [PMT] SFAL_OpenFilter() = %i\n",retVal);
                #endif
                break;
            }

            /* ----- setup CAT filter --------------------------------------- */
            #if (defined APPL_CI_ENABLED) || (defined APPL_CA_ENABLED)
            pTsd->catFilterActive = FFALSE;
            pTsd->catWantUpdates = FFALSE;
            pTsd->catHdl = 0;
            pTsd->catSectNotify = NULL;

            filterSetPars.filterDepth   = CAT_FILTER_DEPTH;
            filterSetPars.bufferSize    = INIT_CAT_SIZE;
            filterSetPars.bufferAddress = INIT_CAT_ADDR + (INIT_CAT_SIZE * cnt);
            retVal = SFAL_OpenFilterset(&filterSetPars, &filterSetHdl);
            if(retVal != FAPI_OK)
            {
                #ifdef PSISI_CAT_DEBUG_ERR
                FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [CAT] SFAL_OpenFilterset() = %i\n",retVal);
                #endif
                break;
            }

            retVal = SFAL_OpenFilter(filterSetHdl, CAT_FILTER_DEPTH,
                                      &(pTsd->catHdl));
            if(retVal != FAPI_OK)
            {
                #ifdef PSISI_CAT_DEBUG_ERR
                FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [CAT] SFAL_OpenFilter() = %i\n",retVal);
                #endif
                break;
            }

            retVal = SFAL_SetFilterPid(pTsd->catHdl, ISO_CAT_PID);
            if(retVal != FAPI_OK)
            {
                #ifdef PSISI_CAT_DEBUG_ERR
                FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [CAT] SFAL_SetFilterPid() = %i\n",retVal);
                #endif
                break;
            }

            /* Invalidate CAT structures */
            memset(&(pTsd->cat), 0, sizeof(PSISI_CAT_S));
            memset(&(pTsd->catTmp), 0, sizeof(PSISI_CAT_S));

            #endif /* (defined APPL_CI_ENABLED) || (defined APPL_CA_ENABLED) */

            /* ----- setup NIT filter --------------------------------------- */
            pTsd->nitFilterActive = FFALSE;

            filterSetPars.filterDepth   = NIT_FILTER_DEPTH;
            filterSetPars.bufferSize    = INIT_NIT_SIZE;
#ifndef _LINUX_FAPI
            filterSetPars.bufferAddress = INIT_NIT_ADDR + (INIT_NIT_SIZE * cnt);
#endif
            retVal = SFAL_OpenFilterset(&filterSetPars, &filterSetHdl);
            if(retVal != FAPI_OK)
            {
                #ifdef PSISI_NIT_DEBUG_ERR
                FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [NIT] SFAL_OpenFilterset() = %i\n",retVal);
                #endif
                break;
            }

            retVal = SFAL_OpenFilter(filterSetHdl, NIT_FILTER_DEPTH,
                                      &(pTsd->nitHdl));
            if(retVal != FAPI_OK)
            {
                #ifdef PSISI_NIT_DEBUG_ERR
                FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [NIT] SFAL_OpenFilter() = %i\n",retVal);
                #endif
                break;
            }

            retVal = SFAL_SetFilterPid(pTsd->nitHdl, ISO_NIT_PID);
            if(retVal != FAPI_OK)
            {
                #ifdef PSISI_NIT_DEBUG_ERR
                FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [NIT] SFAL_SetFilterPid() = %i\n",retVal);
                #endif
                break;
            }

            (void)PSISI_NIT_Invalidate();

        }
        if(retVal != FAPI_OK) break;

        /* ----- setup TDT/TOT filters ------------------------------------- */
        filterSetPars.tsdInput          = RSRC_TSD_LIVE;
        filterSetPars.numberOfChannels  = 1;
        filterSetPars.filtersPerChannel = 1;
        filterSetPars.filterDepth       = TDTTOT_FILTER_DEPTH;
#ifndef _LINUX_FAPI
        filterSetPars.bufferAddress     = INIT_TDTTOT_ADDR;
#endif
        filterSetPars.bufferSize        = INIT_TDTTOT_SIZE;

        retVal = SFAL_OpenFilterset(&filterSetPars, &filterSetHdl);
        if(retVal != FAPI_OK)
        {
            #if (defined PSISI_TDT_DEBUG_ERR) || (defined PSISI_TOT_DEBUG_ERR)
            FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [TDT/TOT] SFAL_OpenFilterset() = %i\n",retVal);
            #endif
            break;
        }

        retVal = SFAL_OpenFilter(filterSetHdl, TDTTOT_FILTER_DEPTH,
                                  &(psiDat->tdttotHdl));
        if(retVal != FAPI_OK)
        {
            #if (defined PSISI_TDT_DEBUG_ERR) || (defined PSISI_TOT_DEBUG_ERR)
            FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [TDT/TOT] SFAL_OpenFilter() = %i\n",retVal);
            #endif
            break;
        }

        /* NOTE: TDT and TOT share PID 0x0014! */
        retVal = SFAL_SetFilterPid(psiDat->tdttotHdl, ISO_TDT_PID);
        if(retVal != FAPI_OK)
        {
            #if (defined PSISI_TDT_DEBUG_ERR) || (defined PSISI_TOT_DEBUG_ERR)
            FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [TDT/TOT] SFAL_SetFilterPid() = %i\n",retVal);
            #endif
            break;
        }

        /* NOTE: TDT and TOT are filtered by table_id only. */
        PSISI_FilterClear(&tdtFilter);

        tdtFilter.data [0] = ISO_TDT_TABLE_ID & ISO_TOT_TABLE_ID;
        tdtFilter.equal[0] = (uint8_t) (~(ISO_TDT_TABLE_ID ^ ISO_TOT_TABLE_ID));
        tdtFilter.nonEq[0] = 0x00;

        retVal = SFAL_SetFilterPatterns(psiDat->tdttotHdl,
                                         tdtFilter.data,
                                         tdtFilter.equal, tdtFilter.nonEq);
        if(retVal != FAPI_OK)
        {
            #if (defined PSISI_TDT_DEBUG_ERR) || (defined PSISI_TOT_DEBUG_ERR)
            FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [TDT/TOT] SFAL_SetFilterPatterns() = %i\n",retVal);
            #endif
            break;
        }

        /* ----- setup SDT filter ------------------------------------------ */
        psiDat->sdtFilterActive = FFALSE;

        filterSetPars.tsdInput          = RSRC_TSD_LIVE;
        filterSetPars.numberOfChannels  = 1;
        filterSetPars.filtersPerChannel = 1;
        filterSetPars.filterDepth       = SDT_FILTER_DEPTH;
#ifndef _LINUX_FAPI
        filterSetPars.bufferAddress     = INIT_SDT_ADDR;
#endif
        filterSetPars.bufferSize        = INIT_SDT_SIZE;

        retVal = SFAL_OpenFilterset(&filterSetPars, &filterSetHdl);
        if(retVal != FAPI_OK)
        {
            #ifdef PSISI_SDT_DEBUG_ERR
            FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [SDT] SFAL_OpenFilterset() = %i\n",retVal);
            #endif
            break;
        }

        retVal = SFAL_OpenFilter(filterSetHdl, SDT_FILTER_DEPTH,
                                  &(psiDat->sdtHdl));
        if(retVal != FAPI_OK)
        {
            #ifdef PSISI_SDT_DEBUG_ERR
            FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [SDT] SFAL_OpenFilter() = %i\n",retVal);
            #endif
            break;
        }

        retVal = SFAL_SetFilterPid(psiDat->sdtHdl, ISO_SDT_PID);
        if(retVal != FAPI_OK)
        {
            #ifdef PSISI_SDT_DEBUG_ERR
            FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [SDT] SFAL_SetFilterPid() = %i\n",retVal);
            #endif
            break;
        }

        PSISI_SDT_Invalidate(&(avData->sdt));

        /* ----------------------------------------------------------------- */

        psiDat->siFiltersOpen = FFALSE;

        break; //end of for(;;)
    }

    return retVal;
}


/*!
*******************************************************************************
**
** \brief   TODO
**
*******************************************************************************
*/
void PSISI_InternalHandling(void)
{
    PSISI_TSD_S* pTsd = &(psiDat->tsd[0]);
    uint8_t      tsdIdx;
    uint8_t      currVersion;

    #if (defined APPL_CI_ENABLED) || (defined APPL_CA_ENABLED)
    PSISI_SECT_NOTIFY_CB catSectNotify;
    #endif /* (defined APPL_CI_ENABLED) || (defined APPL_CA_ENABLED) */

    for(tsdIdx = 0; tsdIdx < PSI_USED_TSDS; tsdIdx++, pTsd++)
    {
        /* handle PAT filtering */
        if ( (pTsd->pPat != NULL) &&
             (!(pTsd->patFilterActive)) &&
             (pTsd->pPat->temp.sect.nonEmpty) )
        {
            currVersion = pTsd->pPat->temp.version_number;

            /* filter for next section if PAT is incomplete */
            if(!(pTsd->pPat->temp.sect.allAvailable))
            {
                (void)PSISI_PAT_StartFiltering (tsdIdx, currVersion, FTRUE);
            }
            else
            {
                /* Filter for next version if PAT has been completely
                   received. */

#ifndef _LINUX_FAPI
                (void)RTOS_GetSemaphore(psiDat->psiMutex, RTOS_SUSPEND);
#else
                (void)FAPEX_SYS_GetSemaphore(psiDat->psiMutex, FAPEX_SYS_SUSPEND);
#endif
                pTsd->pPat->curr = pTsd->pPat->temp;    /* store current PAT */

#ifndef _LINUX_FAPI
                (void)RTOS_SetSemaphore(psiDat->psiMutex, RTOS_NO_SUSPEND);
#else
                (void)FAPEX_SYS_SetSemaphore(psiDat->psiMutex, FAPEX_SYS_NO_SUSPEND);
#endif
                memset(&(pTsd->pPat->temp), 0, sizeof(PSISI_PAT_S));

                (void)PSISI_PAT_StartFiltering (tsdIdx, currVersion, FFALSE);
            }
        }

        #if (defined APPL_CI_ENABLED) || (defined APPL_CA_ENABLED)

        /* handle CAT filtering */
        if ( (pTsd->catFilterActive) &&
             (pTsd->catTmp.sect.allAvailable) )
        {
            #ifdef PSISI_CAT_DEBUG_ERR
            FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [CAT] Update TSD %i version %i\n", tsdIdx,
                       pTsd->catTmp.version_number);
            #endif

            catSectNotify = pTsd->catSectNotify;

            (void)PSISI_CAT_StopFiltering(tsdIdx);

            (void)RTOS_GetSemaphore(psiDat->psiMutex, RTOS_SUSPEND);

            pTsd->cat = pTsd->catTmp;   /* store current CAT */

            (void)RTOS_SetSemaphore(psiDat->psiMutex, RTOS_NO_SUSPEND);

            memset(&(pTsd->catTmp), 0, sizeof(PSISI_CAT_S));

            (void)PSISI_CAT_NotifyReception(tsdIdx);

            if(pTsd->catWantUpdates)
            {
                /* Start filtering for next version */
                (void)PSISI_CAT_StartFiltering(tsdIdx, pTsd->cat.version_number,
                                          FFALSE, FTRUE, catSectNotify);
            }
        }

        #endif /* (defined APPL_CI_ENABLED) || (defined APPL_CA_ENABLED) */
    }

    /* handle SDT filtering */
    if( (psiDat->sdtFilterActive) &&
        (avData->sdt.sect.allAvailable))
    {
        (void)PSISI_SDT_StopFiltering();
    }

    /* handle NIT filtering */
    //for(tsdCnt = 0; tsdCnt < PSI_USED_TSDS; tsdCnt++)
    if((psiDat->tsd[0].nitFilterActive) && (psiDat->tsd[0].nit.sect.allAvailable))
    {
        (void)PSISI_NIT_StopFiltering();
    }
}

/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   op      TODO
** \param   tsdIdx  TODO
** \param   pPat    TODO
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_BAD_PARAMETER
**
*******************************************************************************
*/
int32_t PSISI_SetTsdUsage (RSRC_OP op, uint8_t tsdIdx, PSISI_PAT_DATA_S * pPat)
{
    PSISI_TSD_S*    pTsd = psiDat->tsd + tsdIdx;

    DBG_Assert(tsdIdx < PSI_USED_TSDS);
    if(tsdIdx >= PSI_USED_TSDS) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    /* Martin : comment for temp*/
    //DBG_Assert(pTsd->op == 0);

    FAPI_SYS_PRINT_DEBUG(4,"[PSISI] Reserve TSD %i: op 0x%04X PAT 0x%08X\n", tsdIdx, op, pPat);

    if ( pTsd->pPat != pPat )
    {
        (void)PSISI_PAT_StopFiltering(pTsd);

        pTsd->pPat = pPat;
    }

    (void)PSISI_PMT_Invalidate(tsdIdx);
    //(void)PSISI_PAT_Invalidate(pPat);
    (void)PSISI_PAT_StartFiltering (tsdIdx, PSISI_VERSION_ANY, FTRUE);

    FAPI_SYS_PRINT_DEBUG(1, "[psisi][PSISI_SetTsdUsage] PSISI_PAT_StartFiltering called, enter key to continue\n");

    pTsd->op = op;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   op      TODO
** \param   tsdIdx  TODO
**
** \return  One of the following status codes:
**          - always #FAPI_OK.
**
*******************************************************************************
*/
int32_t PSISI_ReleaseTsdUsage (RSRC_OP op, uint8_t tsdIdx)
{
    PSISI_TSD_S*    pTsd = psiDat->tsd + tsdIdx;

    DBG_Assert(tsdIdx < PSI_USED_TSDS);

    FAPI_SYS_PRINT_DEBUG(4,"[PSISI] Release TSD %i: op 0x%04X\n", tsdIdx, op);

    if ( pTsd->pPat != NULL )
    {
        (void)PSISI_PAT_StopFiltering(pTsd);

        pTsd->pPat = NULL;
    }

    (void)PSISI_PMT_StopFiltering(tsdIdx);
    (void)PSISI_PMT_Invalidate(tsdIdx);

    pTsd->op = 0;

    return FAPI_OK;
}


/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   pPat    TODO
**
** \return  One of the following status codes:
**          - Always #FAPI_OK.
**
*******************************************************************************
*/
int32_t PSISI_InvalidateByFe(PSISI_PAT_DATA_S * pPat)
{
    uint8_t             tsdIdx;

    /* Stop any PAT/PMT filtering on given frontend */
    for(tsdIdx = 0; tsdIdx < PSI_USED_TSDS; tsdIdx++)
    {
        if(psiDat->tsd[tsdIdx].pPat == pPat)
        {
            (void)PSISI_PMT_Invalidate(tsdIdx);
        }
    }

    PSISI_PAT_Invalidate(pPat);

    return FAPI_OK;
}


/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   pPat    TODO
**
** \return  One of the following status codes:
**          - always #FAPI_OK.
**
*******************************************************************************
*/
int32_t PSISI_RestartByFe(PSISI_PAT_DATA_S * pPat)
{
    uint8_t             tsdIdx;

    /* Restart PAT filtering */
    for(tsdIdx = 0; tsdIdx < PSI_USED_TSDS; tsdIdx++)
    {
        if(psiDat->tsd[tsdIdx].pPat == pPat)
        {
            (void)PSISI_PAT_StartFiltering (tsdIdx, PSISI_VERSION_ANY, FTRUE);
        }
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   tsdIdx  TODO
** \param   pPid    TODO
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_BAD_PARAMETER
**
*******************************************************************************
*/
int32_t PSISI_PMT_GetStreamPIDs(uint8_t tsdIdx, SYS_STREAM_INFO_S* pPid)
{
    PSISI_TSD_S        *pTsd = psiDat->tsd + tsdIdx;
    PSISI_PMT_STREAM_S  stream;
    uint16_t            pmtPos = 0;
    int32_t             retVal;
    PSISI_DESC_S        desc;
    uint16_t            loopPos;

    DBG_Assert(tsdIdx < PSI_USED_TSDS);
    if(tsdIdx >= PSI_USED_TSDS)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    memset(pPid, 0, sizeof(SYS_STREAM_INFO_S));

    retVal = PSISI_PMT_StreamParseInit(&(pTsd->pmt), &pmtPos);
    if(retVal != FAPI_OK)
    {
        return retVal;
    }

    pPid->pcr = LD_PID(pTsd->pmt.buf + 8);
    pPid->valid = 1;

    /* parse stream info descriptor loop */
    while ( PSISI_PMT_StreamParseGetNext(&(pTsd->pmt), &pmtPos, &stream) ==
            FAPI_OK )
    {
        switch ( stream.type )
        {
            case STRTYPE_VIDEO_MPEG1:
            case STRTYPE_VIDEO_MPEG2:
            case STRTYPE_VIDEO_H264:
            case STRTYPE_VIDEO_VC1:
                if(pPid->video == 0)
                {
                    pPid->video     = stream.pid;
                    pPid->videoType = (uint8_t)stream.type;

                    loopPos = 0;
                    while(PSISI_DescParseGetNext(stream.esInfoStart, &loopPos,
                                 stream.esInfoLen, &desc) == FAPI_OK)
                    {
                        if(desc.tag == STREAM_IDENTIFIER_DESCRIPTOR)
                        {
                            FAPI_SYS_PRINT_DEBUG(2,"set pPid->videoComponentTag %i \n",*(desc.start + 2));
                            pPid->videoComponentTag = *(desc.start + 2);
                        }

                    }
                }
                break;

            case STRTYPE_AUDIO_MPEG1:
            case STRTYPE_AUDIO_MPEG2:
                if(pPid->audio.numStreams >= SYS_AUDIO_STR_MAX) break;
                (void)PSISI_PMT_AddAudioStreams(&stream, SYS_AUDIO_MPEG,
                                          &(pPid->audio));
                break;

            case STRTYPE_AUDIO_AAC_MPEG2:
                if(pPid->audio.numStreams >= SYS_AUDIO_STR_MAX) break;
                (void)PSISI_PMT_AddAudioStreams(&stream, SYS_AUDIO_AAC,
                                          &(pPid->audio));
                break;

            case STRTYPE_AUDIO_AAC_MPEG4:
                if(pPid->audio.numStreams >= SYS_AUDIO_STR_MAX) break;
                (void)PSISI_PMT_AddAudioStreams(&stream, SYS_AUDIO_HEAAC,
                                          &(pPid->audio));
                break;

            case STRTYPE_AUDIO_AC3_ATSC_A52:
                if(pPid->audio.numStreams >= SYS_AUDIO_STR_MAX) break;

                /* Expect AC-3 according to DVB on private PES
                   stream type */
                (void)PSISI_PMT_AddAudioStreams(&stream, SYS_AUDIO_AC3,
                                          &(pPid->audio));
                break;

            case STRTYPE_PRIVDATA:
                if(PSISI_DescFind(stream.esInfoStart, stream.esInfoLen,
                                  TELETEXT_DESCRIPTOR, NULL) == FAPI_OK)
                {
                    pPid->ttx = stream.pid;
                }
                #ifdef APPL_SUBTT_ENABLED
                if(PSISI_DescFind(stream.esInfoStart, stream.esInfoLen,
                                  SUBTITLING_DESCRIPTOR, NULL) == FAPI_OK)
                {
                    /* subtitle found */
                    pPid->subttPresent = 1;
                }
                #endif

                if(PSISI_DescFind(stream.esInfoStart, stream.esInfoLen,
                                  AC_3_DESCRIPTOR, NULL) == FAPI_OK)
                {
                    if(pPid->audio.numStreams >= SYS_AUDIO_STR_MAX) break;
                    (void)PSISI_PMT_AddAudioStreams(&stream, SYS_AUDIO_AC3,
                                              &(pPid->audio));
                }

                if(PSISI_DescFind(stream.esInfoStart, stream.esInfoLen,
                                  DDPDCV_DESCRIPTOR, NULL) == FAPI_OK)
                {
                    if(pPid->audio.numStreams >= SYS_AUDIO_STR_MAX) break;
                    (void)PSISI_PMT_AddAudioStreams(&stream, SYS_AUDIO_DDPDCV,
                                              &(pPid->audio));
                }
                break;

            case 0x0B:
            case 0x0D:
                #ifdef APPL_MHEG_ENABLED
                if(PSISI_DescFind(stream.esInfoStart, stream.esInfoLen,
                                  DATA_BROADCAST_ID_DESCRIPTOR, &desc) == FAPI_OK)
                {
                    if( ( desc.length   >= 4 ) &&
                        ( desc.start[2] == 0x01 ) &&
                        ( desc.start[3] == 0x06 ) )
                    {
                        pPid->mhegPresent = 1;
                    }
                }
                #endif
                break;
            default:
                break;
        }
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   TODO
**
** \note    This function potentially adds multiple entries from a single stream
**          to pAudio as each stereo channel might represent a separate stream.
**
** \param   pPmtStream  TODO
** \param   format      TODO
** \param   pAudio      TODO
**
** \return  Returns number of streams added.
**
*******************************************************************************
*/
uint8_t PSISI_PMT_AddAudioStreams (const PSISI_PMT_STREAM_S* pPmtStream,
                                   uint8_t format,
                                   SYS_STREAMINF_AUDIO_S* pAudio)
{
    PSISI_DESC_S        desc;
    uint16_t            loopPos = 0;
    uint16_t            descPos;
    uint8_t             type;
    uint8_t             langFound = 0;
    uint8_t             entriesAdded = 0;
    fbool_t             descrFound = FFALSE;
    SYS_STR_AUDIO_S*    pStream = NULL;
    uint8_t             cnt;

    while(PSISI_DescParseGetNext(pPmtStream->esInfoStart, &loopPos,
                                 pPmtStream->esInfoLen, &desc) == FAPI_OK)
    {
        if(desc.tag == LANGUAGE_DESCRIPTOR)
        {
            descrFound = FTRUE;
            descPos = 2;

            while((uint16_t) (descPos + 4) <= desc.length)
            {
                if(pAudio->numStreams >= SYS_AUDIO_STR_MAX)
                    break;

                switch (desc.start[descPos + 3])
                {
                    default:
                    case 0:
                        type = SYS_AUDIO_TYPE_NORMAL;
                        break;

                    case 1:
                        type = SYS_AUDIO_TYPE_EFFECTS;
                        break;

                    case 2:
                        type = SYS_AUDIO_TYPE_HEARING_IMPAIRED;
                        break;

                    case 3:
                        type = SYS_AUDIO_TYPE_VISUAL_IMPAIRED;
                        break;
                }

                /* Discard descriptors with unsupported audio type */
                if((SYS_GetConfig()->supportedAudioTypes & type) != 0)
                {
                    pStream = pAudio->stream + pAudio->numStreams;

                    pStream->pid      = pPmtStream->pid;
                    pStream->format   = format;
                    pStream->langCode = PSISI_GetLanguageCode(desc.start +
                                                              descPos);
                    pStream->type     = type;
                    pStream->channel  = (langFound == 0) ? SYS_AUDIOCHAN_LEFT
                                                         : SYS_AUDIOCHAN_RIGHT;

                    pAudio->numStreams++;
                    entriesAdded++;
                }

                descPos += 4;
                langFound++;
            }
        }
    }
    /* If no language descriptor has been found, stream might be added with
       type "unknown". */
    if((!descrFound) &&
       ((SYS_GetConfig()->supportedAudioTypes & SYS_AUDIO_TYPE_UNKNOWN) != 0) &&
        (pAudio->numStreams < SYS_AUDIO_STR_MAX )) // min one free entry
    {
        pStream = pAudio->stream + pAudio->numStreams;

        pStream->pid      = pPmtStream->pid;
        pStream->format   = format;
        pStream->langCode = LANG_NOTSPECIFIED;
        pStream->type     = SYS_AUDIO_TYPE_UNKNOWN;
        pStream->channel  = SYS_AUDIOCHAN_LEFT;

        pAudio->numStreams++;
        entriesAdded++;
    }

    /* add componentag to found languages */
    loopPos = 0;
    while(PSISI_DescParseGetNext(pPmtStream->esInfoStart, &loopPos,
                                 pPmtStream->esInfoLen, &desc) == FAPI_OK)
    {
        if(desc.tag == STREAM_IDENTIFIER_DESCRIPTOR)
        {
            pStream = pAudio->stream;
            for(cnt=0;cnt<SYS_AUDIO_STR_MAX;cnt++)
            {
                if( (pStream->pid    == pPmtStream->pid) &&
                    (pStream->format == format) )
                {
                    FAPI_SYS_PRINT_DEBUG(2,"set pStream->componentTag %i \n",*(desc.start + 2));
                    pStream->componentTag = *(desc.start + 2);
                }
                pStream++;
            }
        }
    }

    return entriesAdded;
}


/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   tsdIdx  TODO
** \param   prgNum  TODO
** \param   pid     TODO
** \param   version Version to be filtered for equal or non-equal, set to
**                  #PSISI_VERSION_ANY for any version.
** \param   equal   Filtering mode: equal or non-equal version, only used if
**                  version is not equal to #PSISI_VERSION_ANY.
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_BAD_PARAMETER
**          - #SMARTGO_SYS_ERR_PROHIBITED
**          - SFAL error codes.
**
*******************************************************************************
*/
int32_t PSISI_PMT_StartFiltering(uint8_t tsdIdx, uint16_t prgNum, uint16_t pid,
                                 uint8_t version, fbool_t equal)
{
    int32_t         retVal;
    PSISI_TSD_S*    pTsd = psiDat->tsd + tsdIdx;

    DBG_Assert(tsdIdx < PSI_USED_TSDS);
    if(tsdIdx >= PSI_USED_TSDS) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    if(pTsd->pmtFilterActive)
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    #ifdef PSISI_PMT_DEBUG
    FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [PMT] Start filtering: TSD %i PRG %i PID %i ver %i %s\n",
               tsdIdx, prgNum, pid, version, equal ? "EQUAL" : "NONEQUAL");
    #endif

    PSISI_FilterClear(&(pTsd->pmtFilter));

    pTsd->pmtFilter.data[0] = ISO_PMT_TABLE_ID;
    pTsd->pmtFilter.data[3] = (prgNum & 0xFF00) >> 8;
    pTsd->pmtFilter.data[4] =  prgNum & 0x00FF;

    pTsd->pmtFilter.equal[0] = 0xFF;
    pTsd->pmtFilter.equal[3] = 0xFF;
    pTsd->pmtFilter.equal[4] = 0xFF;

    PSISI_FilterSetVersion(&(pTsd->pmtFilter), version, equal);

    retVal = SFAL_SetFilterPid(pTsd->pmtHdl, pid);
    if(retVal != FAPI_OK)
    {
        #ifdef PSISI_PMT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [PMT] SFAL_SetFilterPid failed: %i\n", retVal);
        #endif
        return retVal;
    }

    retVal = SFAL_SetFilterPatterns (pTsd->pmtHdl,
                                   pTsd->pmtFilter.data,
                                   pTsd->pmtFilter.equal,
                                   pTsd->pmtFilter.nonEq);
    if(retVal != FAPI_OK)
    {
        #ifdef PSISI_PMT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [PMT] SFAL_SetFilterPatterns failed: %i\n", retVal);
        #endif
        return retVal;
    }

    retVal = SFAL_StartFilter(pTsd->pmtHdl,
                            SFAL_NO_TIMEOUT,
                            SFAL_LOOP_ONE_SHOT,
                            SFAL_CRC_CHECK,  /* CRC check enabled */
                            NULL,
                            PSISI_PMT_SectNotify,
                            pTsd);
    if ( retVal != FAPI_OK )
    {
        #ifdef PSISI_PMT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(2,"[PSISI] SFAL_StartFilter failed: %i\n", retVal);
        #endif
        (void)PSISI_PMT_StopFiltering(tsdIdx);
        return retVal;
    }
    pTsd->pmtFilterActive = FTRUE;
    pTsd->pmtFilterPid    = pid;
    pTsd->pmtFilterPrgNum = prgNum;

    return retVal;
}

/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   tsdIdx  TODO
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #SMARTGO_SYS_ERR_PROHIBITED
**
*******************************************************************************
*/
int32_t PSISI_PMT_StopFiltering(uint8_t tsdIdx)
{
    PSISI_TSD_S* pTsd;

    DBG_Assert(tsdIdx < PSI_USED_TSDS);
    if(tsdIdx >= PSI_USED_TSDS) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    pTsd = psiDat->tsd + tsdIdx;

    if ( pTsd->pmtFilterActive )
    {
        #ifdef PSISI_PMT_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [PMT] Stop filtering: TSD %i\n", tsdIdx);
        #endif

        (void)SFAL_StopFilter(pTsd->pmtHdl);

        pTsd->pmtFilterActive = FFALSE;
    }

    return FAPI_OK;
}


/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   tsdIdx  TODO
** \param   pid     TODO
** \param   prgNum  TODO
**
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_BAD_PARAMETER
**          - #SMARTGO_SYS_ERR_PROHIBITED
**
*******************************************************************************
*/
int32_t PSISI_PMT_GetFilterParams(uint8_t tsdIdx, uint16_t* pid,
                                  uint16_t* prgNum)
{
    PSISI_TSD_S* pTsd = psiDat->tsd + tsdIdx;

    DBG_Assert(tsdIdx < PSI_USED_TSDS);
    if(tsdIdx >= PSI_USED_TSDS) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    if ( !(pTsd->pmtFilterActive) )
    {
        /* filter is inactive */
        #ifdef PSISI_PMT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [PMT] PSISI_PMT_GetFilterParams() Filter no Active\n");
        #endif
        return SMARTGO_SYS_ERR_PROHIBITED;
    }
    else
    {
        *pid    = pTsd->pmtFilterPid;
        *prgNum = pTsd->pmtFilterPrgNum;
        return FAPI_OK;
    }
}

/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   tsdIdx  TODO
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_BAD_PARAMETER
**
*******************************************************************************
*/
int32_t PSISI_PMT_Invalidate(uint8_t tsdIdx)
{
    DBG_Assert(tsdIdx < PSI_USED_TSDS);
    if(tsdIdx >= PSI_USED_TSDS) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    (void)PSISI_PMT_StopFiltering(tsdIdx);

    #ifdef PSISI_PMT_DEBUG
    FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [PMT] Invalidate TSD %i \n", tsdIdx);
    #endif

#ifndef _LINUX_FAPI
    (void)RTOS_GetSemaphore(psiDat->psiMutex, RTOS_SUSPEND);
#else
    (void)FAPEX_SYS_GetSemaphore(psiDat->psiMutex, FAPEX_SYS_SUSPEND);
#endif
    psiDat->tsd[tsdIdx].pmt.available = FFALSE;

#ifndef _LINUX_FAPI
    (void)RTOS_SetSemaphore(psiDat->psiMutex, RTOS_NO_SUSPEND);
#else
    (void)FAPEX_SYS_SetSemaphore(psiDat->psiMutex, FAPEX_SYS_NO_SUSPEND);
#endif
    return FAPI_OK;
}


/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   tsdIdx  TODO
** \param   pPmt    TODO
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #SMARTGO_SYS_ERR_DATA_NOT_AVL
**
*******************************************************************************
*/
int32_t PSISI_PMT_Get(uint8_t tsdIdx, PSISI_PMT_S * pPmt)
{
    PSISI_TSD_S*    pTsd = psiDat->tsd + tsdIdx;
    int32_t         retVal;

#ifndef _LINUX_FAPI
    (void)RTOS_GetSemaphore(psiDat->psiMutex, RTOS_SUSPEND);
#else
    (void)FAPEX_SYS_GetSemaphore(psiDat->psiMutex, FAPEX_SYS_SUSPEND);
#endif
    if ( !(pTsd->pmt.available) ||
          (pTsd->pmt.bufSize < PMT_SECTSIZE_MIN) )
    {
        retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }
    else
    {
        *pPmt = pTsd->pmt;
        retVal = FAPI_OK;
    }

#ifndef _LINUX_FAPI
    (void)RTOS_SetSemaphore(psiDat->psiMutex, RTOS_NO_SUSPEND);
#else
    (void)FAPEX_SYS_SetSemaphore(psiDat->psiMutex, FAPEX_SYS_NO_SUSPEND);
#endif
    return retVal;
}


/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   tsdIdx  TODO
**
** \return  Pointer to the current PMT structure.
**
*******************************************************************************
*/
const PSISI_PMT_S* PSISI_PMT_GetCurrent (uint8_t tsdIdx)
{
    DBG_Assert(tsdIdx < PSI_USED_TSDS);

    if(tsdIdx >= PSI_USED_TSDS) return NULL;

    return &(psiDat->tsd[tsdIdx].pmt);
}


/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   tsdIdx  TODO
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - TODO
**
*******************************************************************************
*/
int32_t PSISI_PMT_NotifyReception (uint8_t tsdIdx)
{
    PSISI_PMT_S*    pPmt = &(psiDat->tsd[tsdIdx].pmt);
    SYS_EVT_PMT_S   ctrlEvent;

    DBG_Assert(tsdIdx < PSI_USED_TSDS);
    if(tsdIdx >= PSI_USED_TSDS) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    if(!(pPmt->available) || (pPmt->bufSize < PMT_SECTSIZE_MIN))
    {
        return SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }

    ctrlEvent.hdr.type       = EVT_PMT_RECEIVED;
    ctrlEvent.hdr.length     = sizeof(ctrlEvent);
    ctrlEvent.tsdIdx         = tsdIdx;
    ctrlEvent.operation      = psiDat->tsd[tsdIdx].op;
    ctrlEvent.program_number = pPmt->program_number;
    ctrlEvent.version_number = pPmt->version_number;

    return EVT_Send(&ctrlEvent, "PMT");
}


/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   handle          TODO
** \param   sectionStatus   TODO
** \param   pSectionBuffer  TODO
** \param   sectionSize     TODO
** \param   pPrivateData    TODO
**
*******************************************************************************
*/
FSTATIC void PSISI_PMT_SectNotify(FAPI_SYS_HandleT handle,
                                  SFAL_SECTION_STATUS_E sectionStatus,
                                  const uint8_t * pSectionBuffer,
                                  uint16_t sectionSize, void *pPrivateData)
{
    PSISI_TSD_S* pTsd = (PSISI_TSD_S*)pPrivateData;

    if ( sectionStatus != SFAL_SECTION_AVAILABLE )
    {
        #ifdef PSISI_PMT_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [PMT] incomplete: status %i \n", sectionStatus);
        #endif
        return;
    }

    pTsd->pmtFilterActive = FFALSE;

    #ifdef PSISI_PMT_DEBUG
    FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [PMT] Section received TSD %i\n", pTsd->tsdIdx);
    #endif

#ifndef _LINUX_FAPI
    (void)RTOS_GetSemaphore(psiDat->psiMutex, RTOS_SUSPEND);
#else
    (void)FAPEX_SYS_GetSemaphore(psiDat->psiMutex, FAPEX_SYS_SUSPEND);
#endif
    /* copy section to internal buffer */
    pTsd->pmt.bufSize = (sectionSize > PMT_SECTSIZE_MAX) ?
        PMT_SECTSIZE_MAX : sectionSize;

    (void)memcpy (pTsd->pmt.buf, (void*) pSectionBuffer, pTsd->pmt.bufSize);

    pTsd->pmt.available      = FTRUE;
    pTsd->pmt.pid            = pTsd->pmtFilterPid;
    pTsd->pmt.program_number = LD16(pTsd->pmt.buf + 3);
    pTsd->pmt.version_number = (pTsd->pmt.buf[5] >> 1) & 0x1F;

#ifndef _LINUX_FAPI
    (void)RTOS_SetSemaphore(psiDat->psiMutex, RTOS_NO_SUSPEND);
#else
    (void)FAPEX_SYS_SetSemaphore(psiDat->psiMutex, FAPEX_SYS_NO_SUSPEND);
#endif
    #ifdef PSISI_PMT_DEBUG
    FAPI_SYS_PRINT_DEBUG(1,"[PSISI] [PMT]: version %i \n", pTsd->pmt.version_number);
    #endif
    #ifdef PSISI_PMT_DEBUG_DUMP
    UTILS_MemDump("[PSISI] [PMT]", pTsd->pmt.buf, pTsd->pmt.bufSize);
    #endif
}

int32_t PSISI_PMT_StreamParseInit(const PSISI_PMT_S * pPmt, uint16_t * pmtPos)
{
    if(!(pPmt->available) || (pPmt->bufSize < PMT_SECTSIZE_MIN))
    {
        return SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }

    *pmtPos = PMT_SECTSIZE_MIN + LD_LEN(pPmt->buf + 10);

    return FAPI_OK;
}

int32_t PSISI_PMT_StreamParseGetNext(const PSISI_PMT_S* pPmt,
                                     uint16_t* pmtPos,
                                     PSISI_PMT_STREAM_S* pInfo)
{
    if((uint16_t) (*pmtPos + 5 + CRC_LEN) > pPmt->bufSize)
    {
        return SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }

    pInfo->type        = pPmt->buf[*pmtPos];
    pInfo->pid         = LD_PID(pPmt->buf + *pmtPos + 1);

    pInfo->esInfoLen   = LD_LEN(pPmt->buf + *pmtPos + 3);
    pInfo->esInfoStart =        pPmt->buf + *pmtPos + 5;

    *pmtPos += 5 + pInfo->esInfoLen;

    return FAPI_OK;
}

int32_t PSISI_DescParseGetNext(const uint8_t * pLoop, uint16_t * loopPos,
                               uint16_t loopLen, PSISI_DESC_S * pInfo)
{
    if((uint16_t) (*loopPos + 2) > loopLen)
        return SMARTGO_SYS_ERR_DATA_NOT_AVL;

    pInfo->tag = pLoop[*loopPos];
    pInfo->start = pLoop + *loopPos;
    pInfo->length = pLoop[*loopPos + 1] + 2;

    *loopPos += pInfo->length;

    if((*loopPos) > loopLen)
        return SMARTGO_SYS_ERR_INVALID_DATA;
    else
        return FAPI_OK;
}

int32_t PSISI_DescFind(const uint8_t * pLoop, uint16_t loopLen,
                       uint8_t tagToFind, PSISI_DESC_S * pInfo)
{
    uint16_t            loopPos = 0;
    uint8_t             tag;
    uint16_t            length;

    while((uint16_t) (loopPos + 1) < loopLen)
    {
        tag = pLoop[loopPos];
        length = (uint16_t) (pLoop[loopPos + 1]) + 2;

        if(tag == tagToFind)
        {
            if(pInfo != NULL)
            {
                pInfo->tag = tag;
                pInfo->start = pLoop + loopPos;
                pInfo->length = length;
            }

            return FAPI_OK;
        }

        loopPos += length;
    }

    return SMARTGO_SYS_ERR_DATA_NOT_AVL;
}

/* ------------------------------------------------------------------------- */

/*
version: Version to be filtered for equal or non-equal,
         set to PSISI_VERSION_ANY for any version
equal:   Filtering mode: equal or non-equal version, only used if
         version != PSISI_VERSION_ANY
*/
FSTATIC int32_t PSISI_PAT_StartFiltering(uint8_t tsdIdx, uint8_t version,
                                         fbool_t equal)
{
    PSISI_TSD_S*    pTsd = psiDat->tsd + tsdIdx;
    PSISI_PAT_S*    pPat = &(pTsd->pPat->temp);
    int32_t         retVal;
    PSISI_FILTER_S  patFilter;
    uint8_t         sectNum = 0;

    DBG_Assert(tsdIdx < PSI_USED_TSDS);
    if(tsdIdx >= PSI_USED_TSDS) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    if ( pPat->sect.allAvailable )
    {
        /* PAT already available */
        return FAPI_OK;
    }
    else if(pTsd->patFilterActive)
    {
        /* not available but filters already set */
        return FAPI_OK;
    }

    retVal = PSISI_SectionGetNext(pPat->sect.avl, &sectNum);
    if(retVal != FAPI_OK)
        return retVal;

    #ifdef PSISI_PAT_DEBUG
    FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [PAT] Start filtering: TSD %i sect %i version %i %s\n",
               tsdIdx, sectNum, version, equal ? "EQUAL" : "NONEQUAL");
    #endif

    PSISI_FilterClear(&patFilter);
    patFilter.data[0] = ISO_PAT_TABLE_ID;    // table id
    patFilter.data[6] = sectNum;             // section number

    patFilter.equal[0] = 0xFF;
    patFilter.equal[6] = 0xFF;

    PSISI_FilterSetVersion(&patFilter, version, equal);

    retVal = SFAL_SetFilterPatterns(pTsd->patHdl,
                                  patFilter.data,
                                  patFilter.equal,
                                  patFilter.nonEq);
    if(retVal != FAPI_OK)
    {
        #ifdef PSISI_PAT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(2,"[PSISI] PAT] SFAL_SetFilterPatterns failed: %i\n",retVal);
        #endif
        return retVal;
    }

    retVal = SFAL_StartFilter(pTsd->patHdl,
                            SFAL_NO_TIMEOUT,
                            SFAL_LOOP_ONE_SHOT,
                            SFAL_CRC_CHECK,       /* CRC check enabled */
                            NULL,
                            PSISI_PAT_SectNotify,
                            pTsd);

    if(retVal != FAPI_OK)
    {
        #ifdef PSISI_PAT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(2,"SFAL_StartFilter failed: %i\n",retVal);
        #endif
        pTsd->patFilterActive = FTRUE;
        (void)PSISI_PAT_StopFiltering(pTsd);
        return retVal;
    }

    pTsd->patFilterActive = FTRUE;

    return FAPI_OK;
}

FSTATIC int32_t PSISI_PAT_StopFiltering(PSISI_TSD_S * pTsd)
{
    if(pTsd->patFilterActive)
    {
        #ifdef PSISI_PAT_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [PAT] Stop filtering: TSD %i \n", pTsd->tsdIdx);
        #endif

        (void)SFAL_StopFilter(pTsd->patHdl);

        pTsd->patFilterActive = FFALSE;
    }

    return FAPI_OK;
}

void PSISI_PAT_Invalidate(PSISI_PAT_DATA_S * pPat)
{
    uint8_t             tsdIdx;

    /* Stop any PAT/PMT filtering on given frontend */
    for(tsdIdx = 0; tsdIdx < PSI_USED_TSDS; tsdIdx++)
    {
        if(psiDat->tsd[tsdIdx].pPat == pPat)
        {
            (void)PSISI_PAT_StopFiltering(psiDat->tsd + tsdIdx);
        }
    }

#ifndef _LINUX_FAPI
    (void)RTOS_GetSemaphore(psiDat->psiMutex, RTOS_SUSPEND);
#else
    (void)FAPEX_SYS_GetSemaphore(psiDat->psiMutex, FAPEX_SYS_SUSPEND);
#endif
    /* Invalidate both curr. and temp. PAT data. */
    #ifdef PSISI_PAT_DEBUG
    FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [PAT] Invalidate\n");
    #endif
    memset(pPat, 0, sizeof(PSISI_PAT_DATA_S));

#ifndef _LINUX_FAPI
    (void)RTOS_SetSemaphore(psiDat->psiMutex, RTOS_NO_SUSPEND);
#else
    (void)FAPEX_SYS_SetSemaphore(psiDat->psiMutex, FAPEX_SYS_NO_SUSPEND);
#endif
}


int32_t PSISI_PAT_GetPmtPid(const PSISI_PAT_S * pPat, uint16_t prgNum,
                            uint16_t * pmtPid)
{
    uint16_t            prgIdx;

    if(pPat->sect.allAvailable)
    {
        for(prgIdx = 0; prgIdx < pPat->numPrograms; prgIdx++)
        {
            if(pPat->program[prgIdx].program_number == prgNum)
            {
                *pmtPid = pPat->program[prgIdx].program_map_pid;
                return FAPI_OK;
            }
        }
    }

    return SMARTGO_SYS_ERR_DATA_NOT_AVL;
}

const PSISI_PAT_S* PSISI_PAT_GetCurrent (uint8_t tsdIdx)
{
    DBG_Assert(tsdIdx < PSI_USED_TSDS);
    if(tsdIdx >= PSI_USED_TSDS) return NULL;

    if ( psiDat->tsd[tsdIdx].pPat != NULL )
    {
        return &(psiDat->tsd[tsdIdx].pPat->curr);
    }
    else
    {
        return NULL;
    }
}


/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   handle          TODO
** \param   sectionStatus   TODO
** \param   pSectionBuffer  TODO
** \param   sectionSize     TODO
** \param   pPrivateData    TODO
**
*******************************************************************************
*/
FSTATIC void PSISI_PAT_SectNotify(FAPI_SYS_HandleT handle,
                                  SFAL_SECTION_STATUS_E sectionStatus,
                                  const uint8_t * pSectionBuffer,
                                  uint16_t sectionSize,
                                  void *pPrivateData)
{
    PSISI_TSD_S        *pTsd = (PSISI_TSD_S *) pPrivateData;
    PSISI_PAT_S        *pPat = &(pTsd->pPat->temp);
    uint8_t             sectNum;
    uint8_t             version;
    uint16_t            sectPos;
    uint16_t            prgNum;
    uint16_t            pmtPid;

    if ( sectionStatus != SFAL_SECTION_AVAILABLE )
    {
        #ifdef PSISI_PAT_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [PAT] incomplete: status %i \n", sectionStatus);
        #endif
        return;
    }

    /* One-shot filtering: is automatically stopped with each section. */
    pTsd->patFilterActive = FFALSE;

    if(pPat->sect.allAvailable)
    {
        /* PAT already available: do not copy data */
        return;
    }
    else
    {
        if(sectionSize < PAT_SECTSIZE_MIN)
        {
            #ifdef PSISI_PAT_DEBUG
            FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [PAT] Section too small: %i bytes \n",
                                                                sectionSize);
            #endif
            return;
        }

        version = (pSectionBuffer[5] & 0x3E) >> 1;
        sectNum = pSectionBuffer[6];

        /* test for version change */
        if(pPat->sect.nonEmpty && (version != pPat->version_number))
        {
            #ifdef PSISI_PAT_DEBUG
            FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [PAT] Version change!\n");
            #endif

            /* Version change occured before all sections had beenreceived:
               reset data structure! */
            memset(pPat, 0, sizeof(PSISI_PAT_S));
        }

        /* section already available? */
        if(PSISI_SectionAvailable(pPat->sect.avl, sectNum))
            return;

        #ifdef PSISI_PAT_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [PAT] Section %i/%i: TSD %i\n",
                   sectNum, pSectionBuffer[7], pTsd->tsdIdx);
        #endif
        if(!(pPat->sect.nonEmpty))      /* first section received */
        {
            pPat->sect.nonEmpty       = FTRUE;
            pPat->transport_stream_id = LD16(pSectionBuffer + 3);
            pPat->version_number = version;
            pPat->network_pid = ISO_NIT_PID; /* default value */
            pPat->numPrograms = 0;
        }

        PSISI_SectionMark(pPat->sect.avl, sectNum);

        if(PSISI_SectionAllAvailable(pPat->sect.avl, pSectionBuffer[7]))
        {
            pPat->sect.allAvailable = FTRUE;
        }

        #ifdef APPL_MHEG_ENABLED

        if(pTsd->pPatBuf.data[pSectionBuffer[7]] != NULL)
        {
            UTILS_Free(pTsd->pPatBuf.data[pSectionBuffer[7]]);
        }
        pTsd->pPatBuf.data[pSectionBuffer[7]] = UTILS_Malloc(sectionSize);
        if(pTsd->pPatBuf.data[pSectionBuffer[7]] != NULL)
        {
            memcpy(pTsd->pPatBuf.data[pSectionBuffer[7]],pSectionBuffer,sectionSize);
            pTsd->pPatBuf.size[pSectionBuffer[7]] = sectionSize;
            pTsd->pPatBuf.numSect = pSectionBuffer[7]+1;
            pTsd->pPatBuf.avl     = pPat->sect.allAvailable;
        }
        #endif

        /* copy section information to internal buffer */
        for(sectPos = 8; (uint16_t) (sectPos + 4 + CRC_LEN) <= sectionSize;
            sectPos += 4)
        {
            prgNum = LD16(pSectionBuffer + sectPos + 0);
            pmtPid = LD_PID(pSectionBuffer + sectPos + 2);

            if(prgNum == 0)
            {
                pPat->network_pid = pmtPid;
            }
            else if(pPat->numPrograms < PSISI_PAT_PRG_MAX)
            {
                pPat->program[pPat->numPrograms].program_number = prgNum;
                pPat->program[pPat->numPrograms].program_map_pid = pmtPid;

                pPat->numPrograms++;
            }
        }
    }
}

#ifdef APPL_MHEG_ENABLED
fbool_t PSISI_PAT_GetCurrentBuf(PSISI_PAT_BUF_S* pPatBuf,uint8_t tsdIdx)
{
    PSISI_PAT_BUF_S* pPat = NULL;
    uint8_t*         fullPtr = NULL;
    uint8_t          cnt;

    if( (psiDat  != NULL) &&
        (pPatBuf != NULL ) &&
        (tsdIdx  <  PSI_USED_TSDS ))
    {
        pPat = &psiDat->tsd[tsdIdx].pPatBuf;
        if (pPat->avl == FFALSE)
        {
            return FFALSE;
        }
        if(pPat->fullData != NULL)
        {
            UTILS_Free(pPat->fullData);
            pPat->fullData = NULL;
        }
        pPat->fullSize = 0;
        for(cnt=0; cnt< pPat->numSect; cnt++)
        {
            pPat->fullSize += pPat->size[cnt];
        }
        pPat->fullData = UTILS_Malloc(pPat->fullSize);
        if(pPat->fullData == NULL)
        {
            return FFALSE;
        }
        fullPtr = pPat->fullData;
        for(cnt=0; cnt< 1/*pPat->numSect*/; cnt++)
        {
            if(pPat->data[cnt] == NULL)
            {
                return FFALSE;
            }
            memcpy(fullPtr,pPat->data[cnt],pPat->size[cnt]);
         //   fullPtr += pPat->size[cnt];
        }
        *pPatBuf = *pPat;
        return FTRUE;
    }

    return FFALSE;
}
#endif
/* ------------------------------------------------------------------------- */

/*
version: Version to be filtered for equal or non-equal,
         set to PSISI_VERSION_ANY for any version
equal:   Filtering mode: equal or non-equal version, only used if
         version != PSISI_VERSION_ANY
*/
int32_t PSISI_SDT_StartFiltering(uint8_t version, fbool_t equal)
{
    PSISI_SDT_S        *pSdt = &(avData->sdt);
    int32_t             retVal;
    PSISI_FILTER_S      sdtFilter;

    if(psiDat->sdtFilterActive)
        return FAPI_OK;

    if(pSdt->sect.allAvailable)
    {
        /* SDT already available */
        return FAPI_OK;
    }

    #ifdef PSISI_SDT_DEBUG
    FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [SDT] Start filtering: version %i %s\n", version,
               equal ? "EQUAL" : "NONEQUAL");
    #endif

    PSISI_FilterClear(&sdtFilter);
    sdtFilter.data[0]  = ISO_SDT_TABLE_ID;    // table id
    sdtFilter.equal[0] = 0xFF;

    PSISI_FilterSetVersion(&sdtFilter, version, equal);

    retVal = SFAL_SetFilterPatterns(psiDat->sdtHdl,
                                  sdtFilter.data,
                                  sdtFilter.equal,
                                  sdtFilter.nonEq);
    if(retVal != FAPI_OK)
    {
        #ifdef PSISI_SDT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [SDT] SFAL_SetFilterPatterns failed: %i\n",retVal);
        #endif
        return retVal;
    }

    retVal = SFAL_StartFilter(psiDat->sdtHdl,
                            SFAL_NO_TIMEOUT,
                            SFAL_LOOP_CONTINUOUS,
                            SFAL_CRC_CHECK,        /* CRC check enabled */
                            NULL,
                            PSISI_SDT_SectNotify,
                            NULL);
    if(retVal != FAPI_OK)
    {
        #ifdef PSISI_SDT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [SDT] SFAL_StartFilter failed: %i\n",retVal);
        #endif
        psiDat->sdtFilterActive = FTRUE;
        (void)PSISI_SDT_StopFiltering();
        return retVal;
    }
    psiDat->sdtFilterActive = FTRUE;

    return FAPI_OK;
}

int32_t PSISI_SDT_StopFiltering(void)
{
    if(psiDat->sdtFilterActive)
    {
        #ifdef PSISI_SDT_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [SDT] Stop filtering.\n");
        #endif

        (void)SFAL_StopFilter(psiDat->sdtHdl);

        psiDat->sdtFilterActive = FFALSE;
    }

    return FAPI_OK;
}

void PSISI_SDT_Invalidate(PSISI_SDT_S * pSdt)
{
    #ifdef PSISI_SDT_DEBUG
    FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [SDT] Invalidate.\n");
    #endif

    memset(pSdt, 0, sizeof(PSISI_SDT_S));
}

int32_t PSISI_SDT_GetNextService(const PSISI_SDT_S * pSdt,
                                 uint16_t * serviceInfoPos,
                                 PSISI_SDT_SERVICE_S * pService)
{
    const uint8_t      *pBuf = pSdt->serviceInfo + (*serviceInfoPos);

    if(((uint16_t) ((*serviceInfoPos) + 5) > pSdt->serviceInfoSize))
    {
        return SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }

    memset(pService, 0, sizeof(PSISI_SDT_SERVICE_S));

    pService->service_id        = *(pBuf) << 8;
    pService->service_id       |= *(pBuf + 1);
    pService->eit_schedule_flag = (pBuf[2] & 0x02) >> 1;
    pService->eit_pf_flag       = (pBuf[2] & 0x01);
    pService->running_status    = (pBuf[3] & 0xE0) >> 5;
    pService->free_ca_mode      = (pBuf[3] & 0x10) >> 4;
    pService->descLoopLength    = *(pBuf + 3) << 8;
    pService->descLoopLength   |= *(pBuf + 4);
    pService->descLoopLength   &= 0x0FFF;
    pService->descLoopStart     = pBuf + 5;

    if(((uint16_t) ((*serviceInfoPos) + 5 + pService->descLoopLength) >
        pSdt->serviceInfoSize))
    {
        #ifdef PSISI_SDT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [SDT] Invalid loop length!\n");
        #endif

        pService->descLoopLength = ((pSdt->serviceInfoSize -
            (*serviceInfoPos) ) - 5);
    }

    (*serviceInfoPos) += pService->descLoopLength + 5;

    return FAPI_OK;
}

int32_t PSISI_SDT_GetServiceById(const PSISI_SDT_S * pSdt, uint16_t service_id,
                                 PSISI_SDT_SERVICE_S * pService)
{
    PSISI_SDT_SERVICE_S sdtService;
    int32_t             retVal;
    uint16_t            serviceInfoPos = 0;

    do
    {
        retVal = PSISI_SDT_GetNextService(pSdt, &serviceInfoPos, &sdtService);

        if((retVal == FAPI_OK) && (sdtService.service_id == service_id))
        {
            *pService = sdtService;
            break;
        }
    }
    while(retVal == FAPI_OK);

    return retVal;
}


/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   handle          TODO
** \param   sectionStatus   TODO
** \param   pSectionBuffer  TODO
** \param   sectionSize     TODO
** \param   pPrivateData    TODO
**
*******************************************************************************
*/
FSTATIC void PSISI_SDT_SectNotify(FAPI_SYS_HandleT handle,
                                  SFAL_SECTION_STATUS_E sectionStatus,
                                  const uint8_t * pSectionBuffer,
                                  uint16_t sectionSize,
                                  void *pPrivateData)
{
    PSISI_SDT_S        *pSdt = &(avData->sdt);
    uint8_t             sectNum;
    uint8_t             version;
    uint16_t            sectPos;
    uint16_t            bytesToCopy;

    if ( sectionStatus != SFAL_SECTION_AVAILABLE )
    {
        #ifdef PSISI_SDT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [SDT] incomplete: status %i \n", sectionStatus);
        #endif
        return;
    }
    if(pSdt->sect.allAvailable)
    {
        /* SDT already available: do not copy data */
        return;
    }
    else
    {
        if(sectionSize < SDT_SECTSIZE_MIN)
        {
            #ifdef PSISI_SDT_DEBUG_ERR
            FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [SDT] section too small: %i bytes \n", sectionSize);
            #endif
            return;
        }

        version = (pSectionBuffer[5] & 0x3E) >> 1;
        sectNum = pSectionBuffer[6];

        /* test for version change */
        if(pSdt->sect.nonEmpty && (version != pSdt->version_number))
        {
            #ifdef PSISI_SDT_DEBUG
            FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [SDT] Version change!\n");
            #endif

            PSISI_SDT_Invalidate(pSdt);
        }

        /* section already available? */
        if(PSISI_SectionAvailable(pSdt->sect.avl, sectNum))
            return;

        if( sectNum > pSectionBuffer[7])
        {
            #ifdef PSISI_SDT_DEBUG_ERR
            FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [SDT] Invalid section num!\n");
            #endif
            return;
        }

        #ifdef PSISI_SDT_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [SDT] Section %i/%i version %i\n", sectNum,
                   pSectionBuffer[7], version);
        #endif
        #ifdef PSISI_SDT_DEBUG_DUMP
        UTILS_MemDump("[PSISI] [SDT]", pSectionBuffer, sectionSize);
        #endif

        if(!(pSdt->sect.nonEmpty))      /* first section received */
        {
            pSdt->sect.nonEmpty       = FTRUE;
            pSdt->transport_stream_id = LD16(pSectionBuffer + 3);
            pSdt->original_network_id = LD16(pSectionBuffer + 8);
            pSdt->version_number      = version;
            pSdt->serviceInfoSize = 0;
        }

        PSISI_SectionMark(pSdt->sect.avl, sectNum);

        if(PSISI_SectionAllAvailable(pSdt->sect.avl, pSectionBuffer[7]))
        {
            pSdt->sect.allAvailable = FTRUE;
        }

        /* copy section information to internal buffer */
        for(sectPos = 11; (uint16_t) (sectPos + 5 + CRC_LEN) <= sectionSize;)
        {
            bytesToCopy = 5 + LD_LEN(pSectionBuffer + sectPos + 3);

            if((uint16_t) (sectPos + bytesToCopy + CRC_LEN) > sectionSize)
            {
                #ifdef PSISI_SDT_DEBUG_ERR
                FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [SDT] Invalid descriptor loop length! pos %d dlen %d crc %d slen %d\n",
                    sectPos, bytesToCopy, CRC_LEN, sectionSize);
                #endif

                bytesToCopy = ((sectionSize - sectPos ) - CRC_LEN);
            }

            /* test buffer limits */
            if((pSdt->serviceInfoSize + bytesToCopy) > PSISI_SDT_BUF_MAX)
            {
                #ifdef PSISI_SDT_DEBUG_ERR
                FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [SDT] Out of service info space!\n");
                #endif
                break;
            }

            (void)memcpy (pSdt->serviceInfo + pSdt->serviceInfoSize,
                          pSectionBuffer + sectPos, bytesToCopy);

            pSdt->serviceInfoSize += bytesToCopy;

            sectPos += bytesToCopy;
        }
    }
}

/* ------------------------------------------------------------------------- */

int32_t PSISI_TDTTOT_StartFiltering(void)
{
    int32_t             retVal;

    if(psiDat->tdttotFilterActive)
        return FAPI_OK;

    #if (defined PSISI_TDT_DEBUG) || (defined PSISI_TOT_DEBUG)
    FAPI_SYS_PRINT_DEBUG(1,"[PSISI] [TDT/TOT] Start filter\n");
    #endif
    retVal = SFAL_StartFilter(psiDat->tdttotHdl,
                            SFAL_NO_TIMEOUT,
                            SFAL_LOOP_CONTINUOUS,
                            SFAL_CRC_CHECK,        /* CRC check enabled */
                            NULL,
                            PSISI_TDTTOT_SectNotify,
                            NULL);
    if(retVal != FAPI_OK)
    {
        #if (defined PSISI_TDT_DEBUG_ERR) || (defined PSISI_TOT_DEBUG_ERR)
        FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [TDT/TOT] SFAL_StartFilter failed: %i\n",retVal);
        #endif
        PSISI_TDTTOT_StopFiltering();
        return retVal;
    }

    psiDat->tdttotFilterActive = FTRUE;

    return FAPI_OK;
}

void PSISI_TDTTOT_StopFiltering(void)
{
    if(psiDat->tdttotFilterActive)
    {
        (void)SFAL_StopFilter(psiDat->tdttotHdl);
        psiDat->tdttotFilterActive = FFALSE;
    }
}


/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   handle          TODO
** \param   sectionStatus   TODO
** \param   pSectionBuffer  TODO
** \param   sectionSize     TODO
** \param   pPrivateData    TODO
**
*******************************************************************************
*/
FSTATIC void PSISI_TDTTOT_SectNotify(FAPI_SYS_HandleT handle,
                                     SFAL_SECTION_STATUS_E sectionStatus,
                                     const uint8_t * pSectionBuffer,
                                     uint16_t sectionSize,
                                     void *pPrivateData)
{
    uint16_t            timeMjd;
    uint32_t            timeUtc;

    /* NOTE: TDT size: 8 bytes, TOT size: 14 bytes*/

    if ( (sectionStatus != SFAL_SECTION_AVAILABLE) ||
         (sectionSize   <  8) )
    {
        #ifdef PSISI_TOT_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [TDTTOT] incomplete: status %i \n", sectionStatus);
        #endif
        return;
    }
    switch (pSectionBuffer[0])
    {
        case ISO_TDT_TABLE_ID:

            #ifdef PSISI_TDT_DEBUG
            FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [TDT] Received!\n");
            #endif
            /* Provide system time with sync if it is requested */
            timeMjd = LD16(pSectionBuffer + 3);
            timeUtc = LD24(pSectionBuffer + 5);

            TIME_Sync(timeUtc, timeMjd);
            break;

        case ISO_TOT_TABLE_ID:

            #ifdef PSISI_TOT_DEBUG
            FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [TOT] Received!\n");
            #endif

            PSISI_TOT_Process(pSectionBuffer, sectionSize);
            break;

        default:
            #if (defined PSISI_TDT_DEBUG_ERR) || (defined PSISI_TOT_DEBUG_ERR)
            FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [TDT/TOT] Unknown table received!\n",
                       pSectionBuffer[0]);
            #endif
            break;
    }
}


FSTATIC void PSISI_TOT_Process(const uint8_t * pSection, uint16_t sectionSize)
{
    PSISI_DESC_S        desc;
    uint16_t            descPos;
    uint16_t            loopPos = 0;
    uint16_t            loopLen;
    fbool_t             autoMode;
    SYS_CTRYCODE        autoCtry;
    SYS_CTRYCODE        ctryCode;
    uint8_t             polarity;
    int16_t             thisTimeOffset;
    int16_t             nextTimeOffset;
    uint16_t            timeMjd;
    uint32_t            timeUtc;
    uint16_t            changeTimeMjd;
    uint32_t            changeTimeUtc;
    fbool_t             matchFound = FFALSE;

    if(sectionSize < (10 + CRC_LEN))
        return;

    if((SETTINGS_AutoTimeOffsetGet(&autoMode, &autoCtry) != FAPI_OK) ||
       (!autoMode))
    {
        return;
    }

    loopLen = LD_LEN(pSection + 8);

    if((uint16_t) (loopLen + 10 + CRC_LEN) > sectionSize)
    {
        #ifdef PSISI_TOT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(2,"[PSISI] TOT: Bad loop length!\n");
        #endif
        loopLen = ((sectionSize - 10) - CRC_LEN);
    }

    timeMjd = LD16(pSection + 3);
    timeUtc = LD24(pSection + 5);

    while(PSISI_DescParseGetNext(pSection + 10, &loopPos, loopLen, &desc)
          == FAPI_OK)
    {
        if(desc.tag != LOCAL_TIME_OFFSET_DESCRIPTOR)
            continue;

        descPos = 2;

        while((uint16_t) (descPos + 13) <= desc.length)
        {
            ctryCode = (SYS_CTRYCODE)LD24(desc.start + descPos);

            /* NOTE: country_region_id is not evaluated! */

            if(ctryCode == autoCtry)
            {
                polarity = desc.start[descPos + 3] & 0x01;

                /* decode time_of_change */
                changeTimeMjd = LD16(desc.start + descPos + 6);
                changeTimeUtc = LD24(desc.start + descPos + 8);

                thisTimeOffset = (int16_t)((UTILS_Bcd2Dec(desc.start[descPos + 4]) * 60)
                    + UTILS_Bcd2Dec(desc.start[descPos + 5]));
                nextTimeOffset = (int16_t)((UTILS_Bcd2Dec(desc.start[descPos + 11]) * 60)
                    + UTILS_Bcd2Dec(desc.start[descPos + 12]));

                if(polarity != 0)
                {
                    thisTimeOffset = -(thisTimeOffset);
                    nextTimeOffset = -(nextTimeOffset);
                }

                TIME_TimeZoneChange(timeUtc, timeMjd,
                                    changeTimeUtc, changeTimeMjd,
                                    thisTimeOffset, nextTimeOffset);
                matchFound = FTRUE;
                break;
            }

            descPos += 13;
        }

        if(matchFound)
            break;
    }
}

/* ------------------------------------------------------------------------- */

#if (defined APPL_CI_ENABLED) || (defined APPL_CA_ENABLED)

/*
version: Version to be filtered for equal or non-equal,
         set to PSISI_VERSION_ANY for any version
equal:   Filtering mode: equal or non-equal version, only used if
         version != PSISI_VERSION_ANY
versionUpdate: FTRUE: filter for table updates until filtering is stopped.
*/
int32_t PSISI_CAT_StartFiltering(uint8_t tsdIdx, uint8_t version, fbool_t equal,
                                 fbool_t versionUpdate,
                                 PSISI_SECT_NOTIFY_CB sectNotify)
{
    PSISI_TSD_S        *pTsd = psiDat->tsd + tsdIdx;
    int32_t             retVal;
    PSISI_FILTER_S      catFilter;

    DBG_Assert(tsdIdx < PSI_USED_TSDS);
    if(tsdIdx >= PSI_USED_TSDS) return APPL_SMARTGO_ERR_BAD_PARAMETER;
    DBG_Assert(pTsd->catHdl != 0);

    if(pTsd->catFilterActive)
        return FAPI_OK;

    memset(&(pTsd->catTmp), 0, sizeof(PSISI_CAT_S));

    #ifdef PSISI_CAT_DEBUG
    FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [CAT] Start CAT filtering: version %i %s\n", version,
               equal ? "EQUAL" : "NONEQUAL");
    #endif

    PSISI_FilterClear(&catFilter);

    catFilter.data[0]  = ISO_CAT_TABLE_ID;    // table id
    catFilter.equal[0] = 0xFF;

    PSISI_FilterSetVersion(&catFilter, version, equal);

    retVal = SFAL_SetFilterPatterns(pTsd->catHdl, catFilter.data,
                                     catFilter.equal, catFilter.nonEq);
    if(retVal != FAPI_OK)
    {
        #ifdef PSISI_CAT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [CAT] SFAL_SetFilterPatterns failed: %i\n",retVal);
        #endif
        return retVal;
    }

    retVal = SFAL_StartFilter(pTsd->catHdl,
                            SFAL_NO_TIMEOUT,
                            SFAL_LOOP_CONTINUOUS,
                            SFAL_CRC_CHECK,        /* CRC check enabled */
                            NULL,
                            PSISI_CAT_SectNotify,
                            pTsd);
    if(retVal != FAPI_OK)
    {
        #ifdef PSISI_CAT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [CAT] SFAL_StartFilter failed: %i\n",retVal);
        #endif
        return retVal;
    }

    pTsd->catSectNotify   = sectNotify;
    pTsd->catFilterActive = FTRUE;
    pTsd->catWantUpdates  = versionUpdate;

    return FAPI_OK;
}

int32_t PSISI_CAT_StopFiltering(uint8_t tsdIdx)
{
    PSISI_TSD_S        *pTsd = psiDat->tsd + tsdIdx;

    DBG_Assert(tsdIdx < PSI_USED_TSDS);
    if(tsdIdx >= PSI_USED_TSDS) return APPL_SMARTGO_ERR_BAD_PARAMETER;
    DBG_Assert(pTsd->catHdl != 0);

    if(pTsd->catFilterActive)
    {
        #ifdef PSISI_CAT_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [CAT] Stop filtering: TSD %i\n", tsdIdx);
        #endif

        (void)SFAL_StopFilter(pTsd->catHdl);

        pTsd->catFilterActive = FFALSE;
    }

    return FAPI_OK;
}

int32_t PSISI_CAT_Invalidate(uint8_t tsdIdx)
{
    DBG_Assert(tsdIdx < PSI_USED_TSDS);
    if(tsdIdx >= PSI_USED_TSDS) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    (void)PSISI_CAT_StopFiltering(tsdIdx);

    #ifdef PSISI_CAT_DEBUG_ERR
    FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [CAT] Invalidate TSD %i \n", tsdIdx);
    #endif

    (void)RTOS_GetSemaphore(psiDat->psiMutex, RTOS_SUSPEND);

    memset(&(psiDat->tsd[tsdIdx].cat), 0, sizeof(PSISI_CAT_S));

    (void)RTOS_SetSemaphore(psiDat->psiMutex, RTOS_NO_SUSPEND);

    return FAPI_OK;
}

const PSISI_CAT_S  *PSISI_CAT_GetCurrent(uint8_t tsdIdx)
{
    DBG_Assert(tsdIdx < PSI_USED_TSDS);
    if(tsdIdx >= PSI_USED_TSDS) return NULL;

    return &(psiDat->tsd[tsdIdx].cat);
}

int32_t PSISI_CAT_Get(uint8_t tsdIdx, PSISI_CAT_S * pCat)
{
    PSISI_CAT_S        *pSrcCat = &(psiDat->tsd[tsdIdx].cat);
    int32_t             retVal;

    (void)RTOS_GetSemaphore(psiDat->psiMutex, RTOS_SUSPEND);

    if(!(pSrcCat->sect.allAvailable))
    {
        retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }
    else
    {
        *pCat = *pSrcCat;
        retVal = FAPI_OK;
    }

    (void)RTOS_SetSemaphore(psiDat->psiMutex, RTOS_NO_SUSPEND);
    return retVal;
}

FSTATIC int32_t PSISI_CAT_NotifyReception(uint8_t tsdIdx)
{
    PSISI_CAT_S        *pCat = &(psiDat->tsd[tsdIdx].cat);
    SYS_EVT_CAT_S       ctrlEvent;

    if(!(pCat->sect.allAvailable))
    {
        return SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }

    ctrlEvent.hdr.type = EVT_CAT_RECEIVED;
    ctrlEvent.hdr.length = sizeof(ctrlEvent);
    ctrlEvent.tsdIdx = tsdIdx;
    ctrlEvent.version_number = pCat->version_number;

    return EVT_Send(&ctrlEvent, "CAT");
}


/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   handle          TODO
** \param   sectionStatus   TODO
** \param   pSectionBuffer  TODO
** \param   sectionSize     TODO
** \param   pPrivateData    TODO
**
*******************************************************************************
*/
FSTATIC void PSISI_CAT_SectNotify(FAPI_SYS_HandleT handle,
                                  SFAL_SECTION_STATUS_E sectionStatus,
                                  const uint8_t * pSectionBuffer,
                                  uint16_t sectionSize,
                                  void *pPrivateData)
{
    PSISI_TSD_S        *pTsd = (PSISI_TSD_S *) pPrivateData;
    PSISI_CAT_S        *pCat = &(pTsd->catTmp);
    uint8_t             sectNum;
    uint8_t             version;
    uint16_t            catInfoLen;
    uint16_t            loopPos;
    PSISI_DESC_S        descr;

    if ( sectionStatus != SFAL_SECTION_AVAILABLE )
    {
        #ifdef PSISI_CAT_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"CAT: incomplete: status %i \n", sectionStatus);
        #endif
        return;
    }
    else if ( pCat->sect.allAvailable )
    {
        /* All CAT sections available: do not copy data */
        return;
    }
    else
    {
        if(sectionSize < CAT_SECTSIZE_MIN)
        {
            #ifdef PSISI_CAT_DEBUG
            FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [CAT] Section too small: %i bytes \n",
                                                                sectionSize);
            #endif
            return;
        }

        version = (pSectionBuffer[5] & 0x3E) >> 1;
        sectNum = pSectionBuffer[6];

        /* test for version change */
        if(pCat->sect.nonEmpty && (version != pCat->version_number))
        {
            #ifdef PSISI_CAT_DEBUG
            FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [CAT] Version change!\n");
            #endif

            memset(pCat, 0, sizeof(PSISI_CAT_S));
        }

        /* section already available? */
        if(PSISI_SectionAvailable(pCat->sect.avl, sectNum))
            return;

        #ifdef PSISI_CAT_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [CAT] Section %i/%i version %i\n", sectNum,
                   pSectionBuffer[7], version);
        #endif

        if(!(pCat->sect.nonEmpty))      /* first section received */
        {
            pCat->sect.nonEmpty         = FTRUE;
            pCat->version_number = version;
            pCat->numStreams = 0;
        }

        PSISI_SectionMark(pCat->sect.avl, sectNum);

        if(PSISI_SectionAllAvailable(pCat->sect.avl, pSectionBuffer[7]))
        {
            pCat->sect.allAvailable = FTRUE;
        }

        /* Copy section information to internal buffer: only non-private
           CA descriptor information is extracted. */

        loopPos = 0;
        catInfoLen = sectionSize - CAT_SECTSIZE_MIN;    /* 4 bytes CRC +
                                                           8 bytes CA section
                                                           header */

        while(PSISI_DescParseGetNext(pSectionBuffer + 8, &loopPos,
                                     catInfoLen, &descr) == FAPI_OK)
        {
            if((descr.tag == CA_DESCRIPTOR) &&
               (pCat->numStreams < PSISI_CAT_STR_MAX))
            {
                pCat->stream[pCat->numStreams].ca_system_id =
                    LD16  (descr.start + 2);
                pCat->stream[pCat->numStreams].pid =
                    LD_PID(descr.start + 4);

                pCat->numStreams++;
            }
        }

        if(pTsd->catSectNotify != NULL)
        {
            pTsd->catSectNotify(pTsd->tsdIdx, pSectionBuffer, sectionSize);
        }
    }
}

#endif /* (defined APPL_CI_ENABLED) || (defined APPL_CA_ENABLED) */

/* ------------------------------------------------------------------------- */


/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   handle          TODO
** \param   sectionStatus   TODO
** \param   pSectionBuffer  TODO
** \param   sectionSize     TODO
** \param   pPrivateData    TODO
**
*******************************************************************************
*/
FSTATIC void PSISI_NIT_SectNotify(FAPI_SYS_HandleT handle,
                                  SFAL_SECTION_STATUS_E sectionStatus,
                                  const uint8_t * pSectionBuffer,
                                  uint16_t sectionSize, void *pPrivateData)
{
    PSISI_TSD_S        *pTsd = (PSISI_TSD_S *) pPrivateData;
    uint8_t             sectNum;
    uint8_t             version;
    uint16_t            sectPos = 0;
    uint16_t            numOfTs;

    if ( pPrivateData == NULL) return;

    if ( sectionStatus == SFAL_SECTION_CRC_ERROR )
    {
        #ifdef PSISI_NIT_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [NIT] incomplete: status CRC ERROR \n");
        #endif
        return;
    }
    if ( sectionStatus == SFAL_SECTION_TIMEOUT_ERROR )
    {
        pTsd->nitFilterTimeout = FTRUE;
        pTsd->nitFilterActive = FFALSE;
        return;
    }
    if(handle == NULL) return;

    if(pTsd->nit.sect.allAvailable)
    {
        /* NIT already available: do not copy data */
        return;
    }
    else
    {
        if(sectionSize < NIT_SECTSIZE_MIN)
        {
            #ifdef PSISI_NIT_DEBUG
            FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [NIT] Section too small: %i bytes \n", sectionSize);
            #endif
            return;
        }

        version = (pSectionBuffer[5] & 0x3E) >> 1;
        sectNum = pSectionBuffer[6];

        /* test for version change */
        if(pTsd->nit.sect.nonEmpty && (version != pTsd->nit.version_number))
        {
            #ifdef PSISI_NIT_DEBUG
            FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [NIT] Version change!\n");
            #endif

            /* Version change occured before all sections had beenreceived:
               reset data structure! */
            for(numOfTs=0;numOfTs<PSISI_NIT_TS_MAX;numOfTs++)
            {
                if(pTsd->nit.ts[numOfTs].descr_loop != NULL)
                {
                    UTILS_Free(pTsd->nit.ts[numOfTs].descr_loop);
                    pTsd->nit.ts[numOfTs].descr_loop = NULL;
                }
            }
            memset(&(pTsd->nit), 0, sizeof(PSISI_NIT_S));
        }

        /* section already available? */
        if(PSISI_SectionAvailable(pTsd->nit.sect.avl, sectNum))
        {
            return;
        }

        #ifdef PSISI_NIT_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [NIT] Section %i/%i: TSD %i\n",
                   sectNum, pSectionBuffer[7], pTsd->tsdIdx);
        #endif

        if(!(pTsd->nit.sect.nonEmpty))      /* first section received */
        {
            pTsd->nit.sect.nonEmpty  = FTRUE;
            pTsd->nit.network_id     = pSectionBuffer[3] <<  8;
            pTsd->nit.network_id    |= pSectionBuffer[4];
            pTsd->nit.version_number = version;
            pTsd->nit.network_pid    = ISO_NIT_PID; /* default value */
            pTsd->nit.numOfTs        = 0;
        }

        PSISI_SectionMark(pTsd->nit.sect.avl, sectNum);

        if(PSISI_SectionAllAvailable(pTsd->nit.sect.avl, pSectionBuffer[7]))
        {
            pTsd->nit.sect.allAvailable = FTRUE;
        }

        sectPos = 12 + LD_LEN(pSectionBuffer + 8);
        numOfTs = pTsd->nit.numOfTs;

        /* copy section information to internal buffer */
        while((sectPos + 4 + CRC_LEN) <= sectionSize)
        {
            pTsd->nit.ts[numOfTs].tsId      = *(pSectionBuffer + sectPos + 0)<<8;
            pTsd->nit.ts[numOfTs].tsId     |= *(pSectionBuffer + sectPos + 1);
            pTsd->nit.ts[numOfTs].onId      = *(pSectionBuffer + sectPos + 2)<<8;
            pTsd->nit.ts[numOfTs].onId     |= *(pSectionBuffer + sectPos + 3);
            pTsd->nit.ts[numOfTs].descrLen  = *(pSectionBuffer + sectPos + 4)<<8;
            pTsd->nit.ts[numOfTs].descrLen |= *(pSectionBuffer + sectPos + 5);
            pTsd->nit.ts[numOfTs].descrLen &= 0x0FFF;

            if(pTsd->nit.ts[numOfTs].descr_loop != NULL)
            {
                UTILS_Free(pTsd->nit.ts[numOfTs].descr_loop);
                pTsd->nit.ts[numOfTs].descr_loop = NULL;
            }
            pTsd->nit.ts[numOfTs].descr_loop = UTILS_Malloc(pTsd->nit.ts[numOfTs].descrLen);
            (void)memcpy (pTsd->nit.ts[numOfTs].descr_loop,
                          pSectionBuffer + sectPos + 6,
                          pTsd->nit.ts[numOfTs].descrLen);


            sectPos += pTsd->nit.ts[numOfTs].descrLen + 6;
            numOfTs++;
        }
        pTsd->nit.numOfTs = numOfTs;
    }
}
/*
version: Version to be filtered for equal or non-equal,
         set to PSISI_VERSION_ANY for any version
equal:   Filtering mode: equal or non-equal version, only used if
         version != PSISI_VERSION_ANY
*/

int32_t PSISI_NIT_StartFiltering(uint8_t  version,
                                 fbool_t  equal,
                                 uint32_t timeout)
{
    PSISI_TSD_S *pTsd = &psiDat->tsd[0];
    int32_t             retVal;
    PSISI_FILTER_S      nitFilter;

    if(pTsd->nit.sect.allAvailable)
    {
        /* NIT already available */
        return FAPI_OK;
    }
    else if(pTsd->nitFilterActive)
    {
        /* not available but filters already set */
        return FAPI_OK;
    }

    (void)PSISI_NIT_Invalidate();

    #ifdef PSISI_NIT_DEBUG
    FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [NIT] Start filtering: TSD 0 version %i %s\n",
                version, equal ? "EQUAL" : "NONEQUAL");
    #endif

    PSISI_FilterClear(&nitFilter);
    nitFilter.data[0]  = ISO_NIT_TABLE_ID; // table id
    nitFilter.equal[0] = 0xFF;

    PSISI_FilterSetVersion(&nitFilter, version, equal);

    retVal = SFAL_SetFilterPatterns(pTsd->nitHdl,
                                     nitFilter.data,
                                     nitFilter.equal, nitFilter.nonEq);
    if(retVal != FAPI_OK)
    {
        #ifdef PSISI_NIT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [NIT] SFAL_SetFilterPatterns failed\n");
        #endif
        return retVal;
    }

    retVal = SFAL_StartFilter(pTsd->nitHdl,
                            timeout,
                            SFAL_LOOP_CONTINUOUS,
                            SFAL_CRC_CHECK,  /* CRC check enabled */
                            NULL,
                            PSISI_NIT_SectNotify,
                            pTsd);
    if(retVal != FAPI_OK)
    {
        #ifdef PSISI_NIT_DEBUG_ERR
        FAPI_SYS_PRINT_DEBUG(2,"[PSISI] [NIT] SFAL_StartFilter failed\n");
        #endif
        pTsd->nitFilterActive = FFALSE;
        (void)PSISI_NIT_StopFiltering();
        return retVal;
    }
    pTsd->nitFilterActive  = FTRUE;
    pTsd->nitFilterTimeout = FFALSE;

    return FAPI_OK;
}

int32_t PSISI_NIT_Invalidate(void)
{
    PSISI_TSD_S *pTsd = &psiDat->tsd[0];
    uint32_t    cnt;

    if(pTsd->nitFilterActive)
    {
        (void)PSISI_NIT_StopFiltering();
    }

    /* Invalidate NIT data. */
    for(cnt=0;cnt<pTsd->nit.numOfTs;cnt++)
    {
        if(pTsd->nit.ts[cnt].descr_loop != NULL)
        {
            UTILS_Free(pTsd->nit.ts[cnt].descr_loop);
        }
    }
    memset(&(pTsd->nit), 0, sizeof(PSISI_NIT_S));

    return FAPI_OK;
}
int32_t PSISI_NIT_StopFiltering(void)
{
    PSISI_TSD_S *pTsd = &psiDat->tsd[0];

    if(pTsd->nitFilterActive)
    {
        #ifdef PSISI_NIT_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] [NIT] Stop filtering: TSD %i \n", pTsd->tsdIdx);
        #endif

        (void)SFAL_StopFilter(pTsd->nitHdl);

        pTsd->nitFilterActive = FFALSE;
    }
    return FAPI_OK;
}

int32_t PSISI_NIT_GetInfo(uint16_t *numOfTs, int32_t *networkId)
{
    PSISI_TSD_S *pTsd = &psiDat->tsd[0];

    if(pTsd->nit.sect.allAvailable)
    {
        if( numOfTs != NULL )
        {
            *numOfTs   = pTsd->nit.numOfTs;
        }
        if( networkId != NULL )
        {
            *networkId = pTsd->nit.network_id;
        }
        return FAPI_OK;
    }
    else if(pTsd->nitFilterTimeout == FTRUE)
    {
        return APPL_SMARTGO_ERR_TIMEOUT;
    }
    return SMARTGO_SYS_ERR_DATA_NOT_AVL;
}

int32_t PSISI_NIT_GetTransportInfo(uint16_t tsIdx, int32_t *tsId, int32_t *onId)
{
    PSISI_TSD_S *pTsd = &psiDat->tsd[0];

    if(pTsd->nit.sect.allAvailable)
    {
        if ( tsId != NULL )
        {
            *tsId = pTsd->nit.ts[tsIdx].tsId;
        }
        if ( onId != NULL )
        {
            *onId = pTsd->nit.ts[tsIdx].onId;
        }
        return FAPI_OK;
    }
    else if(pTsd->nitFilterTimeout == FTRUE)
    {
        return APPL_SMARTGO_ERR_TIMEOUT;
    }
    return SMARTGO_SYS_ERR_DATA_NOT_AVL;
}

int32_t PSISI_NIT_GetTransportDescLoop(uint16_t tsIdx, uint8_t *buf, uint16_t *len)
{
    PSISI_TSD_S *pTsd = &psiDat->tsd[0];

    if(pTsd->nit.sect.allAvailable)
    {
        *len = pTsd->nit.ts[tsIdx].descrLen;
        (void)memcpy (buf, pTsd->nit.ts[tsIdx].descr_loop,
                      pTsd->nit.ts[tsIdx].descrLen);
        return FAPI_OK;
    }
    else if(pTsd->nitFilterTimeout == FTRUE)
    {
        return APPL_SMARTGO_ERR_TIMEOUT;
    }
    return SMARTGO_SYS_ERR_DATA_NOT_AVL;
}

FSTATIC int32_t PSISI_SectionGetNext (const uint32_t* pSectFound,
                                      uint8_t* sectNum)
{
    uint16_t sectIdx = 0;

    while ( sectIdx < PSISI_SECT_MAX )
    {
        if ( ((*pSectFound) & (1UL << (sectIdx & 0x1F))) == 0 )
        {
            *sectNum = (uint8_t)sectIdx;
            return FAPI_OK;
        }

        sectIdx++;

        if ( (sectIdx & 0x1F) == 0 )
        {
            pSectFound++;
        }
    }

    return APPL_SMARTGO_ERR_BAD_PARAMETER;
}

fbool_t PSISI_SectionAllAvailable(const uint32_t * pSectFound,
                                  uint8_t lastSection)
{
    uint16_t numSections = lastSection + 1;

    while(numSections >= 32)
    {
        if((*pSectFound) != 0xFFFFFFFF)
        {
            return FFALSE;
        }
        numSections -= 32;
        pSectFound++;
    }
    if(numSections == 0)
    {
        return FTRUE;
    }
    return (fbool_t)((*pSectFound) == ((1UL << numSections) - 1));
}
FSTATIC void PSISI_FilterClear(PSISI_FILTER_S * pFilter)
{
    memset (pFilter->data,  0, MAX_FILTER_DEPTH);
    memset (pFilter->equal, 0, MAX_FILTER_DEPTH);
    memset (pFilter->nonEq, 0, MAX_FILTER_DEPTH);
}

FSTATIC void PSISI_FilterSetVersion(PSISI_FILTER_S * pFilter, uint8_t version,
                                    fbool_t equal)
{
    if ( version != PSISI_VERSION_ANY )
    {
        pFilter->data[5] = (version & 0x1F) << 1;

        if ( equal )
        {
            pFilter->equal[5] = 0x3E;
        }
        else
        {
            pFilter->nonEq[5] = 0x3E;
        }
    }

    /* set current_next_indicator: always filter for current version */
    pFilter->data[5]  |= 0x01;
    pFilter->equal[5] |= 0x01;
}

#ifdef APPL_FESAT_ENABLED

fbool_t decodeSatDescriptor(uint8_t * pBuf, uint16_t len,
                            FE_TUNE_PARAMS_S * pTunepar)
{
    uint32_t    tempVal;

    if(len < 13)
    {
        return FFALSE;
    }

    /* --------------------------------------------------------------------- */

    tempVal = LD32(pBuf + 2);
    /* convert f to kHz */
    pTunepar->sat.frequency = UTILS_Bcd2Dec(tempVal) * 10;

    /* --------------------------------------------------------------------- */

    switch ((pBuf[8] >> 5) & 3)
    {
        default:
        case 0:
            pTunepar->sat.pol = FE_POL_HOR;
            break;
        case 1:
            pTunepar->sat.pol = FE_POL_VER;
            break;
        case 2:
            pTunepar->sat.pol = FE_POL_LEFT;
            break;
        case 3:
            pTunepar->sat.pol = FE_POL_RIGHT;
            break;
    }

    switch ((pBuf[8] >> 3) & 3)
    {
        default:
        case 0:
            pTunepar->sat.rollOff = FE_ROLL_OFF_0_35;
            break;
        case 1:
            pTunepar->sat.rollOff = FE_ROLL_OFF_0_25;
            break;
        case 2:
            pTunepar->sat.rollOff = FE_ROLL_OFF_0_20;
            break;
    }

    switch (pBuf[8] & 7)
    {
        default:
            pTunepar->sat.modulation = FE_MOD_QPSK;
            break;
        case 4:
            pTunepar->sat.modulation = FE_MOD_AUTO;
            break;
        case 5:
            pTunepar->sat.modulation = FE_MOD_QPSK_S2;
            break;
        case 6:
            pTunepar->sat.modulation = FE_MOD_8PSK;
            break;
        case 7:
            pTunepar->sat.modulation = FE_MOD_QAM_16;
            break;
    }

    /* --------------------------------------------------------------------- */

    tempVal = ((uint32_t)LD32(pBuf + 9)) >> 4;

    /* convert to kSymbols */
    pTunepar->sat.symbolRate = UTILS_Bcd2Dec(tempVal) / 10;

    /* --------------------------------------------------------------------- */

    switch (pBuf[12] & 0x0F)
    {
        default:
        case 0x0F:             /* no FEC used */
        case 0x00:             /* not defined */
            pTunepar->sat.fec = FE_FEC_NONE;
            break;

        case 0x01:
            pTunepar->sat.fec = FE_FEC_1_2;
            break;
        case 0x02:
            pTunepar->sat.fec = FE_FEC_2_3;
            break;
        case 0x03:
            pTunepar->sat.fec = FE_FEC_3_4;
            break;
        case 0x04:
            pTunepar->sat.fec = FE_FEC_5_6;
            break;
        case 0x05:
            pTunepar->sat.fec = FE_FEC_7_8;
            break;
        case 0x06:
            pTunepar->sat.fec = FE_FEC_8_9;
            break;
        case 0x07:
            pTunepar->sat.fec = FE_FEC_3_5;
            break;
        case 0x08:
            pTunepar->sat.fec = FE_FEC_4_5;
            break;
        case 0x09:
            pTunepar->sat.fec = FE_FEC_9_10;
            break;
    }

    /* --------------------------------------------------------------------- */

    pTunepar->sat.iq = FE_IQ_NONE;   // iq is unknown

    /* NOTE: These settings are added later */
    pTunepar->sat.satID           = 0;
    pTunepar->sat.orbitalPosition = 0;
    pTunepar->sat.westEastFlag    = FE_WE_FLAG_WEST;

    return FTRUE;
}

#endif /* APPL_FESAT_ENABLED */

#ifdef APPL_FETERR_ENABLED

fbool_t decodeTerrDescriptor(uint8_t * pBuf, uint16_t len,
                             FE_TUNE_PARAMS_S * pTunepar)
{
    if(len < 13)
    {
        return FFALSE;
    }

    /* store frequency in Hz! */
    pTunepar->terr.frequency = LD32(pBuf + 2) * 10;

    /* --------------------------------------------------------------------- */

    switch (pBuf[6] >> 5)
    {
        case 0:
            pTunepar->terr.bandwidth = FE_BANDWIDTH_8_MHZ;
            break;
        case 1:
            pTunepar->terr.bandwidth = FE_BANDWIDTH_7_MHZ;
            break;
        case 2:
            pTunepar->terr.bandwidth = FE_BANDWIDTH_6_MHZ;
            break;

        case 3:                /* 5 MHz -> not supported */
        default:               /* reserved values */
            /* Bandwidth is needed for tuning -> a valid setting must be
               available */
            return FFALSE;
    }

    /* --------------------------------------------------------------------- */

    switch ((pBuf[7] & 0xC0) >> 6)
    {
        case 0:
            pTunepar->terr.modulation = FE_MOD_QPSK;
            break;
        case 1:
            pTunepar->terr.modulation = FE_MOD_QAM_16;
            break;
        case 2:
            pTunepar->terr.modulation = FE_MOD_QAM_64;
            break;

            /* unknown/invalid */
        case 3:
        default:
            pTunepar->terr.modulation = FE_MOD_NONE;
            break;
    }

    /* --------------------------------------------------------------------- */

    switch (pBuf[7] & 0x07)
    {
        case 0:
            pTunepar->terr.codeRateHP = FE_FEC_1_2;
            break;
        case 1:
            pTunepar->terr.codeRateHP = FE_FEC_2_3;
            break;
        case 2:
            pTunepar->terr.codeRateHP = FE_FEC_3_4;
            break;
        case 3:
            pTunepar->terr.codeRateHP = FE_FEC_5_6;
            break;
        case 4:
            pTunepar->terr.codeRateHP = FE_FEC_7_8;
            break;

            /* unknown/invalid */
        default:
            pTunepar->terr.codeRateHP = FE_FEC_NONE;
            break;
    }

    switch ((pBuf[8] & 0xE0) >> 5)
    {
        case 0:
            pTunepar->terr.codeRateLP = FE_FEC_1_2;
            break;
        case 1:
            pTunepar->terr.codeRateLP = FE_FEC_2_3;
            break;
        case 2:
            pTunepar->terr.codeRateLP = FE_FEC_3_4;
            break;
        case 3:
            pTunepar->terr.codeRateLP = FE_FEC_5_6;
            break;
        case 4:
            pTunepar->terr.codeRateLP = FE_FEC_7_8;
            break;

            /* unknown/invalid */
        default:
            pTunepar->terr.codeRateLP = FE_FEC_NONE;
            break;
    }

    /* --------------------------------------------------------------------- */

    switch ((pBuf[8] & 0x18) >> 3)
    {
        case 0:
            pTunepar->terr.guardInterval = FE_GUARD_INTERVAL_1_32;
            break;
        case 1:
            pTunepar->terr.guardInterval = FE_GUARD_INTERVAL_1_16;
            break;
        case 2:
            pTunepar->terr.guardInterval = FE_GUARD_INTERVAL_1_8;
            break;
        case 3:
            pTunepar->terr.guardInterval = FE_GUARD_INTERVAL_1_4;
            break;

            /* unknown/invalid */
        default:
            pTunepar->terr.guardInterval = FE_GUARD_INTERVAL_NONE;
            break;
    }

    /* --------------------------------------------------------------------- */

    switch ((pBuf[8] & 0x06) >> 1)
    {
        case 0:
            pTunepar->terr.transmission = FE_TRANS_MODE_2K;
            break;
        case 1:
            pTunepar->terr.transmission = FE_TRANS_MODE_8K;
            break;

            /* unknown/invalid */
        case 2:                /* FIXME: 4K mode -> not defined by FAPI */
        case 3:
        default:
            pTunepar->terr.transmission = FE_TRANS_MODE_NONE;
            break;
    }

    /* --------------------------------------------------------------------- */

    pTunepar->terr.hierarchy = FE_HIERARCHY_NONE;
    pTunepar->terr.iq = FE_IQ_NONE;  // iq is unknown

    return FTRUE;
}

#endif /* APPL_FETERR_ENABLED */

#ifdef APPL_FECABLE_ENABLED

fbool_t decodeCableDescriptor(uint8_t * pBuf, uint16_t len,
                              FE_TUNE_PARAMS_S * pTunepar)
{
    uint32_t    tempVal;

    if(len < 13)
    {
        return FFALSE;
    }

    tempVal = LD32(pBuf + 2);

    /* convert f to Hz */
    pTunepar->cable.frequency = UTILS_Bcd2Dec(tempVal) * 100;
    switch ( pBuf[8] )
    {
        case 0:
            pTunepar->cable.modulation = FE_MOD_NONE;
            break;
        case 1:
            pTunepar->cable.modulation = FE_MOD_QAM_16;
            break;
        case 2:
            pTunepar->cable.modulation = FE_MOD_QAM_32;
            break;
        case 3:
            pTunepar->cable.modulation = FE_MOD_QAM_64;
            break;
        case 4:
            pTunepar->cable.modulation = FE_MOD_QAM_128;
            break;
        case 5:
            pTunepar->cable.modulation = FE_MOD_QAM_256;
            break;
        default:
            pTunepar->cable.modulation = FE_MOD_AUTO;
            break;
    }

    /* --------------------------------------------------------------------- */

    tempVal = LD32(pBuf+9);

    /* convert to kSymbols */
    pTunepar->cable.symbolRate = UTILS_Bcd2Dec(tempVal>>4) / 10;

    switch ( tempVal & 0x0F)
    {
        case 0:
            pTunepar->cable.fec = FE_FEC_NONE;
            break;
        case 1:
            pTunepar->cable.fec = FE_FEC_1_2;
            break;
        case 2:
            pTunepar->cable.fec = FE_FEC_2_3;
            break;
        case 3:
            pTunepar->cable.fec = FE_FEC_3_4;
            break;
        case 4:
            pTunepar->cable.fec = FE_FEC_5_6;
            break;
        case 5:
            pTunepar->cable.fec = FE_FEC_7_8;
            break;
        case 6:
            pTunepar->cable.fec = FE_FEC_8_9;
            break;
        case 7:
            pTunepar->cable.fec = FE_FEC_3_5;
            break;
        case 8:
            pTunepar->cable.fec = FE_FEC_4_5;
            break;
        case 9:
            pTunepar->cable.fec = FE_FEC_9_10;
            break;
        default:
            pTunepar->cable.fec = FE_FEC_AUTO;
            break;
    }

    return FTRUE;
}

#endif /* APPL_FECABLE_ENABLED */

uint32_t UTILS_Bcd2Dec(uint32_t bcd)
{
    uint8_t     cnt;
    uint32_t    ret = 0;
    uint32_t    mul = 1;

    for(cnt = 0; cnt < 8; cnt++)
    {
        ret += ((bcd & 0xF) * mul);
        mul *= 10;
        bcd = bcd >> 4;
    }

    return ret;
}

int32_t PSISI_GetServiceDescriptor(const uint8_t * pLoop, uint16_t loopLen,
                                   char_t *servName, uint16_t servNameLen,
                                   SYS_CHARSET* servNameCharSet,
                                   uint8_t * serviceType )
{
    PSISI_DESC_S        desc;
    uint16_t            loopPos = 0;
    uint16_t            textStart;
    uint16_t            textLen;

    if(servNameLen > 0)
    {
        servName[0] = (char_t) 0;
    }

    if ( servNameCharSet != NULL )
    {
        *servNameCharSet = SYS_CHARSET_UNDEFINED;
    }

    while(PSISI_DescParseGetNext(pLoop, &loopPos, loopLen, &desc) == FAPI_OK)
    {
        if(desc.tag == SERVICE_DESCRIPTOR)
        {
            textStart = desc.start[3] + 4;
            textLen = desc.start[textStart];
            textStart += 1;

            *serviceType = desc.start[2];

            if((textLen > 0) && (textStart < desc.length))
            {
                /* descriptor length exceeded? */
                if((uint16_t) (textStart + textLen) > desc.length)
                {
                    textLen = desc.length - textStart;
                }

                if(servNameLen > 0)
                {
                    (void)UTILS_DBVSIText2String((const char_t*)(desc.start + textStart), textLen,
                                           servName, servNameLen,
                                           servNameCharSet, FTRUE, 0);
                }
            }

            return FAPI_OK;
        }
    }

    return SMARTGO_SYS_ERR_DATA_NOT_AVL;
}

#ifdef SLIST_WITH_PROVIDER
int32_t PSISI_GetServiceProviderName(const uint8_t * pLoop, uint16_t loopLen,
                                     char_t *provider, uint16_t providerLen,
                                     SYS_CHARSET* providerCharSet )
{
    PSISI_DESC_S        desc;
    uint16_t            loopPos = 0;
    uint16_t            textStart;
    uint16_t            textLen;

    if(providerLen > 0)
    {
        provider[0] = (char_t) 0;
    }
    if ( providerCharSet != NULL )
    {
        *providerCharSet = SYS_CHARSET_UNDEFINED;
    }
    while(PSISI_DescParseGetNext(pLoop, &loopPos, loopLen, &desc) == FAPI_OK)
    {
        if(desc.tag == SERVICE_DESCRIPTOR)
        {
            textStart = 4;
            textLen = desc.start[3];
            if((textLen > 0) && (textStart < desc.length))
            {
                /* descriptor length exceeded? */
                if((uint16_t) (textStart + textLen) > desc.length)
                {
                    textLen = desc.length - textStart;
                }
                if(providerLen > 0)
                {
                    (void)UTILS_DBVSIText2String((const char_t*)(desc.start + textStart), textLen,
                                                 provider, providerLen,
                                                 providerCharSet, FTRUE, 0);
                }
            }
            return FAPI_OK;
        }
    }
    return SMARTGO_SYS_ERR_DATA_NOT_AVL;
}
#endif /* #ifdef SLIST_WITH_PROVIDER */
SYS_LANGCODE PSISI_GetLanguageCode(const uint8_t * pBuf)
{
    SYS_LANGCODE        code;

    code = (SYS_LANGCODE)LD24(pBuf);

    /* NOTE: This handles some "non-standard-but-neverthelessly-used"
       language codes. */

    if(code == (SYS_LANGCODE)TRIPLEASCII('c', 'e', 's'))
        code = LANG_CZECH;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('c', 'y', 'm'))
        code = LANG_WELSH;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('d', 'e', 'u'))
        code = LANG_GERMAN;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('e', 'l', 'l'))
        code = LANG_GREEK;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('e', 'u', 's'))
        code = LANG_BASQUE;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('f', 'a', 's'))
        code = LANG_PERSIAN;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('f', 'r', 'a'))
        code = LANG_FRENCH;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('g', 'a', 'e'))
        code = LANG_SCOTS_GAELIC;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('h', 'r', 'v'))
        code = LANG_CROATIAN;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('h', 'y', 'e'))
        code = LANG_ARMENIAN;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('i', 's', 'l'))
        code = LANG_ICELANDIC;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('k', 'a', 't'))
        code = LANG_GEORGIAN;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('m', 'r', 'i'))
        code = LANG_MAORI;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('m', 'k', 'd'))
        code = LANG_MACEDONIAN;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('m', 's', 'a'))
        code = LANG_MALAY;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('m', 'y', 'a'))
        code = LANG_BURMESE;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('n', 'l', 'd'))
        code = LANG_DUTCH;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('r', 'o', 'n'))
        code = LANG_ROMANIAN;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('s', 'l', 'k'))
        code = LANG_SLOVAK;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('s', 'q', 'i'))
        code = LANG_ALBANIAN;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('s', 'r', 'p'))
        code = LANG_SERBIAN;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('t', 'i', 'b'))
        code = LANG_TIBETAN;
    else if(code == (SYS_LANGCODE)TRIPLEASCII('z', 'h', 'o'))
        code = LANG_CHINESE;

    //"cse" -> "cze" ??

    return code;
}

/*!
*******************************************************************************
**
** \brief   PSISI_NIT_Get
**
** \note    This function get the pointer of the NIT info
**
** \return  the pointer of NIT info, or NULL
**
*******************************************************************************
*/
const PSISI_NIT_S* PSISI_NIT_Get(void)
{
    if (psiDat->tsd[0].nit.sect.allAvailable)
    {
        return &(psiDat->tsd[0].nit);
    }
    else
    {
        return NULL;
    }
}

/*!
*******************************************************************************
**
** \brief   PSISI_NIT_AddService
**
** \note    This function add service to the structure of NIT service list
**
** \param   pNitTPService: NIT service list
** \param   serviceId: new service ID
**
** \return  Returns the index of the service
**
*******************************************************************************
*/
static uint16_t PSISI_NIT_AddService(PSISI_NIT_TP_INFO_S *pNitTPService, uint16_t serviceId)
{
    uint16_t i = 0;
    uint16_t service_index = 0;

    if(pNitTPService == NULL)
    {
        return CARRIER_SERVICE_MAX;
    }

    for(i=0;i<pNitTPService->serviceCount;i++)
    {
        if (pNitTPService->nitServiceList[i].serviceId == serviceId)
        {
            return i;
        }
    }

    if((i == pNitTPService->serviceCount) && (pNitTPService->serviceCount < CARRIER_SERVICE_MAX))
    {
        service_index = pNitTPService->serviceCount;
        pNitTPService->nitServiceList[service_index].serviceId = serviceId;
        pNitTPService->nitServiceList[service_index].invisible = FFALSE;
        pNitTPService->nitServiceList[service_index].unselectable = FFALSE;
        (pNitTPService->serviceCount)++;
        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] NIT PSISI_NIT_AddService : serviceId 0x%x\n", serviceId);
        return service_index;
    }

    return CARRIER_SERVICE_MAX;
}

/*!
*******************************************************************************
**
** \brief   PSISI_NIT_ParseByTP
**
** \param   feType          the Frontend type.
** \param   pDesBuffer      Descriptors of one carrier.
** \param   len             The length of the carrier.
** \param   pNitTPService   Get the services info of one carrier.
** \param   curtsId   current tsid, if 0 means not check
**
** \return
**          - #FTRUE    If a transponder was found.
**          - #FFALSE   In case of error or no transponder was found.
**
*******************************************************************************
*/
fbool_t PSISI_NIT_ParseByTP(FE_SYSTEM_TYPE_E feType,
                            const uint8_t * pDesBuffer,
                            uint16_t len,
                            PSISI_NIT_TP_INFO_S *pNitTPService,
                            int32_t curtsId)
{
    uint16_t    descPos = 0;
    uint16_t    cnt = 0;
    uint8_t     descLen;
    uint32_t    privDataSpec = 0;
    uint16_t    serviceId;
    uint8_t     service_type;
    uint16_t    service_index = 0;
    uint8_t     *pDesCur = NULL;
    fbool_t     tpFound = FFALSE;
#ifdef APPL_DTG_SIPSI_T2_ENABLED
    uint32_t    hd_lcn_start_pos = 0;
    uint32_t    hd_lcn_len = 0;
    uint16_t    hd_lcn = 0;
    uint16_t    i = 0;
#endif

    if (( pDesBuffer == NULL) || (pNitTPService == NULL))
    {
        return tpFound;
    }

    pNitTPService->desFlag = 0;

    while((uint16_t) (descPos + 2) <= len)
    {
        descLen = pDesBuffer[descPos + 1];

        if((uint16_t) (descPos + descLen + 2) > len)
            break;

        switch (pDesBuffer[descPos])
        {
            case PRIVATE_DATA_SPECIFIER_DESCRIPTOR:
                privDataSpec = LD32(pDesBuffer + descPos + 2);
                break;

            #ifdef APPL_FESAT_ENABLED

            case SAT_DEL_SYSTEM_DESCRIPTOR:        /* Satellite delivery system descriptor */
                if(feType == FE_SYSTEM_TYPE_SATELLITE)
                {
                    tpFound = decodeSatDescriptor((uint8_t *)(pDesBuffer + descPos), (len - descPos), &(pNitTPService->tpItem.pars));
                }
                break;

            #endif /* APPL_FESAT_ENABLED */

            #ifdef APPL_FETERR_ENABLED

            case TER_DEL_SYSTEM_DESCRIPTOR:        /* Terrestrial delivery system descriptor */
                if(feType == FE_SYSTEM_TYPE_TERRESTRIAL)
                {
                    tpFound = decodeTerrDescriptor((uint8_t *)(pDesBuffer + descPos), (len - descPos), &(pNitTPService->tpItem.pars));
                }
                break;

            #endif /* APPL_FETERR_ENABLED */

            #ifdef APPL_FECABLE_ENABLED

            case CAB_DEL_SYSTEM_DESCRIPTOR:        /* Cable delivery system descriptor */
                if ( feType == FE_SYSTEM_TYPE_CABLE)
                {
                    tpFound = decodeCableDescriptor((uint8_t *)(pDesBuffer + descPos), (len - descPos), &(pNitTPService->tpItem.pars));
                }
                break;

            #endif /* APPL_FECABLE_ENABLED */

            case LOGICAL_CHANNEL_DESCRIPTOR:
                /* parse for current TS only */
                if ((curtsId != 0) && ( pNitTPService->tpItem.tsId != curtsId ))
                {
                    break;
                }

                if ( ( (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT) ||
                       (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT_T2) ||
                       (SYS_GetAppProfile() == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW) ) &&
                       (privDataSpec != PRIVATE_DATA_SPECIFIER_UKDTT) )
                {
                    /* NOTE: in UK-DTT profile private descriptors
                           are required to be within a specific
                           private data area. */
                    break;
                }

                FAPI_SYS_PRINT_DEBUG(4,"[PSISI] NIT LCN found: %i bytes\n", descLen);

                /* Parse LCN information*/
                pNitTPService->desFlag |= FLAG_GET_LCN;
                pDesCur = (uint8_t*)(pDesBuffer + descPos + 2);
                for(cnt = 0; (uint16_t) (cnt + 4) <= descLen; cnt += 4)
                {
                    serviceId = LD16(pDesCur + cnt);
                    service_index = PSISI_NIT_AddService(pNitTPService, serviceId);
                    if(service_index == CARRIER_SERVICE_MAX)
                    {
                        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] NIT LCN Exceed max service number\n");
                        break;
                    }
                    pNitTPService->nitServiceList[service_index].lcn = LD16(pDesCur + cnt + 2) & 0x3FF;
                    FAPI_SYS_PRINT_DEBUG(5,"[PSISI] NIT LCN : serviceId 0x%x lcn=%d\n", serviceId, pNitTPService->nitServiceList[service_index].lcn);
                }

                break;

            case SERVICE_ATTRIBUTE_DESCRIPTOR:
                /* parse for current TS only */
                if ((curtsId != 0) && ( pNitTPService->tpItem.tsId != curtsId ))
                {
                    break;
                }

                /* parse for current TS only */
                if ( ( (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT) ||
                       (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT_T2) ||
                       (SYS_GetAppProfile() == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW) )
                  && (privDataSpec != PRIVATE_DATA_SPECIFIER_UKDTT) )
                {
                    /* NOTE: in UK-DTT profile private descriptors
                           are required to be within a specific
                           private data area. */
                    break;
                }

                FAPI_SYS_PRINT_DEBUG(4, "[PSISI] NIT service attr. descr. found: %i bytes\n", descLen);

                /* Parse descriptor information*/
                pNitTPService->desFlag |= FLAG_GET_ATTR;
                pDesCur = (uint8_t*)(pDesBuffer + descPos + 2);
                for(cnt = 0; (uint16_t) (cnt + 3) <= descLen; cnt += 3)
                {
                    serviceId = LD16(pDesCur + cnt);
                    service_index = PSISI_NIT_AddService(pNitTPService, serviceId);
                    if(service_index == CARRIER_SERVICE_MAX)
                    {
                        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] NIT Attr Exceed max service number\n");
                        break;
                    }

                    //set flag
                    if((pDesCur[cnt + 2] & 0x01) != 0)
                    {
                        /* service is visible and selectable */
                        pNitTPService->nitServiceList[service_index].invisible = FFALSE;
                        pNitTPService->nitServiceList[service_index].unselectable = FFALSE;
                    }
                    else
                    {
                        pNitTPService->nitServiceList[service_index].invisible = FTRUE;

                        if ( (pDesCur[cnt + 2] & 0x02) != 0 )
                        {
                            /* service is hidden but selectable */
                            pNitTPService->nitServiceList[service_index].unselectable = FFALSE;
                        }
                        else
                        {
                            /* service is hidden and non-selectable */
                            pNitTPService->nitServiceList[service_index].unselectable = FTRUE;
                        }
                    }
                }

                break;

            case SERVICE_LIST_DESCRIPTOR:
            {
                /* parse for current TS only */
                if ((curtsId != 0) && ( pNitTPService->tpItem.tsId != curtsId ))
                {
                    break;
                }

                /* parse for current TS only */
                FAPI_SYS_PRINT_DEBUG(4,"[PSISI] NIT  Service List found: %i bytes\n", descLen);

                /* Parse Service list information*/
                pDesCur = (uint8_t*)(pDesBuffer + descPos + 2);
                for(cnt = 0; (uint16_t) (cnt + 3) <= descLen; cnt += 3)
                {
                    serviceId = LD16(pDesCur + cnt);
                    service_type = pDesCur[cnt + 2];

                    service_index = PSISI_NIT_AddService(pNitTPService, serviceId);
                    if(service_index == CARRIER_SERVICE_MAX)
                    {
                        FAPI_SYS_PRINT_DEBUG(4,"[PSISI] NIT List Exceed max service number\n");
                        break;
                    }

                    FAPI_SYS_PRINT_DEBUG(5,"[PSISI] NITList : serviceId 0x%x service_type=%d\n", serviceId, service_type);
                    pNitTPService->nitServiceList[service_index].service_type = service_type;
                    switch ( service_type )
                    {
                        case SERVICE_TYPE_DIGITAL_TV:
                        case SERVICE_TYPE_DIGITAL_TV_HD:
                        case SERVICE_TYPE_DIGITAL_TV_ADVANCED:
                        case SERVICE_TYPE_DIGITAL_TV_HD_ADVANCED:
                        case SERVICE_TYPE_DIGITAL_RADIO:
                        case SERVICE_TYPE_DIGITAL_RADIO_ADVANCED:
                        #ifdef APPL_MHEG_ENABLED
                        case SERVICE_TYPE_DATA_BROADCAST:
                        #endif /* APPL_MHEG_ENABLED */
                            pNitTPService->nitServiceList[service_index].service_type = service_type;
                            break;

                        default:
                            if ( (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT) ||
                                  (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT_T2) ||
                                  (SYS_GetAppProfile() == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW)  )
                            {
                                /* NOTE: For UK-DTT profile, unknown service types are not
                                         accepted. */
                            }
                            else
                            {
                                pNitTPService->nitServiceList[service_index].service_type = SERVICE_TYPE_DIGITAL_TV;
                            }
                            break;
                    }
                }
            }
            break;

        #ifdef APPL_DTG_SIPSI_T2_ENABLED
            case HD_SIMULCAST_LOGICAL_CHANNEL_DESCRIPTOR:
            {
                /* parse for current TS only */
                if ((curtsId != 0) && ( pNitTPService->tpItem.tsId != curtsId ))
                {
                    break;
                }

                if ( ( (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT) ||
                        (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT_T2) ||
                        (SYS_GetAppProfile() == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW) ) &&
                        (privDataSpec != PRIVATE_DATA_SPECIFIER_UKDTT) )
                {
                    /* NOTE: in UK-DTT profile private descriptors
                           are required to be within a specific
                           private data area. */
                    break;
                }

                FAPI_SYS_PRINT_DEBUG(4,"[PSISI] NIT HD_SIMULCAST LCN found: %i bytes\n", descLen);

                hd_lcn_len = descLen;
                hd_lcn_start_pos = descPos + 2;
            }
            break;
        #endif/*#ifdef APPL_DTG_SIPSI_T2_ENABLED*/

            default:
                //FAPI_SYS_PRINT_DEBUG(2,"->unknown: 0x%X\n", pDesBuffer[descPos]);
                break;
        }

        descPos += descLen + 2;
    }

#ifdef APPL_DTG_SIPSI_T2_ENABLED
    //we should parse hd_simulcast_lcn after logical_channel_descriptor & SAD
    /* Parse HD_SIMULCAST LCN information*/
    if((hd_lcn_len > 0) && ((curtsId == 0) || ( pNitTPService->tpItem.tsId == curtsId )))
    {
        pDesCur = (uint8_t*)(pDesBuffer + hd_lcn_start_pos);
        for(cnt = 0; (uint16_t) (cnt + 4) <= hd_lcn_len; cnt += 4)
        {
            serviceId = LD16(pDesCur + cnt);
            service_index = PSISI_NIT_AddService(pNitTPService, serviceId);
            hd_lcn = LD16(pDesCur + cnt + 2) & 0x3FF;
            if(service_index == CARRIER_SERVICE_MAX)
            {
                FAPI_SYS_PRINT_DEBUG(4,"[PSISI] NIT HD_SIMULCAST LCN Exceed max service number\n");
                break;
            }

            if(pNitTPService->nitServiceList[service_index].service_type != SERVICE_TYPE_DIGITAL_TV_HD_ADVANCED)
            {
                FAPI_SYS_PRINT_DEBUG(4,"[PSISI] NIT HD_SIMULCAST LCN is not hd service\n");
                continue;
            }

            //check whether this lcn is used in other SD services.
            for(i=0;i<pNitTPService->serviceCount;i++)
            {
                if(pNitTPService->nitServiceList[i].lcn == hd_lcn)
                {
                    if(pNitTPService->nitServiceList[i].serviceId != serviceId)
                    {
                        pNitTPService->nitServiceList[i].lcn = pNitTPService->nitServiceList[service_index].lcn;
                    }
                    break;
                }
            }

            pNitTPService->nitServiceList[service_index].invisible = (fbool_t)!((pDesCur[cnt + 2] & 0x80) >> 7);
            pNitTPService->nitServiceList[service_index].unselectable = FFALSE;
            pNitTPService->nitServiceList[service_index].lcn = hd_lcn;
            FAPI_SYS_PRINT_DEBUG(5,"[PSISI] NIT HD_SIMULCAST LCN : serviceId 0x%x lcn=%d\n", serviceId, pNitTPService->nitServiceList[service_index].lcn);
        }
    }
#endif/*#ifdef APPL_DTG_SIPSI_T2_ENABLED*/

    return tpFound;
}

#ifdef APPL_DTG_SIPSI_T2_ENABLED
/*!
*******************************************************************************
**
** \brief   PSISI_CheckSameLCN
**
** \note    This function checks whether this lcn is also used in hd_simulcast_logical_channel_descriptor
**
** \param   lcn: lcn get from logical_channel_descriptor
** \param   phdlcnBuf: hd_simulcast_logical_channel_descriptor
** \param   hdlcnBuflen: length of hd_simulcast_logical_channel_descriptor
** \param   pserviceId: get the service id of which hd service has the same lcn
**
** \return  FTRUE: there is hd service has this lcn, FFALSE: there is no hd service has this lcn
**
*******************************************************************************
*/
fbool_t PSISI_CheckSameLCN(uint16_t lcn, uint8_t *phdlcnBuf, uint16_t hdlcnBuflen, uint16_t *pserviceId)
{
    uint16_t            cnt;
    uint16_t            serviceId;
    uint16_t            hdlcn;

    if((phdlcnBuf == NULL) || (pserviceId == NULL))
    {
        return FFALSE;
    }

    for(cnt = 0; (uint16_t) (cnt + 4) <= hdlcnBuflen; cnt += 4)
    {
        serviceId = LD16(phdlcnBuf + cnt);
        hdlcn = LD16(phdlcnBuf + cnt + 2) & 0x3FF;
        if(hdlcn == lcn)
        {
            *pserviceId = serviceId;
            return FTRUE;
        }
    }

    return FFALSE;
}

/*!
*******************************************************************************
**
** \brief   PSISI_GetLCNByServiceID
**
** \note    This function get the lcn of one service from logical_channel_descriptor
**
** \param   plcnBuf: logical_channel_descriptor
** \param   lcnBuflen: length of logical_channel_descriptor
** \param   GetserviceId: which service id
**
** \return  lcn of special service
**
*******************************************************************************
*/
uint16_t PSISI_GetLCNByServiceID(uint8_t *plcnBuf, uint16_t lcnBuflen, uint16_t GetserviceId)
{
    uint16_t            cnt;
    uint16_t            serviceId;
    uint16_t            lcn;

    if((plcnBuf == NULL) || (GetserviceId == 0))
    {
        return 0;
    }

    for(cnt = 0; (uint16_t) (cnt + 4) <= lcnBuflen; cnt += 4)
    {
        serviceId = LD16(plcnBuf + cnt);
        lcn = LD16(plcnBuf + cnt + 2) & 0x3FF;

        if(serviceId == GetserviceId)
        {
            return lcn;
        }
    }

    return 0;
}

#endif/*#ifdef APPL_DTG_SIPSI_T2_ENABLED*/

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
