/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/pvr_dec.c
**
**              $Id: pvr_dec.c 5871 2010-03-02 13:42:55Z jmorha $
**
** \brief       Decoder interface implementation for PVR playback engine.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info 715: Symbol 'anySymbol' not referenced
**
*****************************************************************************
*/
/*lint -save -e715 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <fapi/drv_sync.h>

#include <pvr/pvr.h>
#include <val/val.h>
#include <aal/aal.h>

#include "av.h"
#include "pvr_sys.h"

#include "utils.h"

//******************************************************************************
//******************************************************************************
//** Defines and macros
//******************************************************************************
//******************************************************************************

//******************************************************************************
//******************************************************************************
//** Global functions
//******************************************************************************
//******************************************************************************

//******************************************************************************
//******************************************************************************
//** Local data
//******************************************************************************
//******************************************************************************

typedef struct
{
    /* current decoder control state parameters */
    uint8_t                     started;
    uint16_t                    pid;            //!< Video stream PID.
    uint8_t                     trackFormat;    //!< Video stream format,
                                                //!< one of
                                                //!< PVR_TRACK_FORMAT_VIDEO_XXX.

    PVR_DecoderModeT            currMode;
    uint8_t                     currModeIndex;
    uint8_t                     decoderStarted;

} PVRDEC_VideoT;

typedef struct
{
    uint8_t                     trackIndex;     //!< PVR track index.
    uint16_t                    pid;            //!< Audio stream PID.
    uint8_t                     format;         //!< One of SYS_AUDIO_XXX.
    uint8_t                     channel;        //!< One of SYS_AUDIOCHAN_XXX.

    /* current decoder control state parameters */
    uint8_t                     started;
    PVR_DecoderModeT            currMode;
    uint8_t                     currModeIndex;
    uint8_t                     decoderStarted;
    int32_t                     decoderStartStatus;

} PVRDEC_AudioT;

typedef struct
{
    PVRDEC_VideoT  video;
    PVRDEC_AudioT  audio;

} PVRDEC_DataT;

static PVRDEC_DataT* pvrDecPtr = NULL;

//******************************************************************************
//******************************************************************************
//** Local Functions Declaration
//******************************************************************************
//******************************************************************************

static void*    decVideoOpen           (PVR_DecoderTypeT type, void* argPtr);
static int32_t  decVideoClose          (PVR_DecoderTypeT type, void* argPtr,
                                        void* handlePtr);
static int32_t  decVideoStart          (PVR_DecoderTypeT type,
                                        uint8_t trackIndex, uint32_t trackData,
                                        PVR_TrackTypeT trackType,
                                        uint8_t trackFormat, void* argPtr,
                                        uint16_t pid, void* handlePtr);
static int32_t  decVideoStop           (PVR_DecoderTypeT type, void* argPtr,
                                        uint8_t freeze, void* handlePtr);
static int32_t  decVideoSetMode        (PVR_DecoderTypeT type, void* argPtr,
                                        PVR_DecoderModeT mode,
                                        uint8_t modeIndex, void* handlePtr);
static int32_t  decVideoShowIFrame     (PVR_DecoderTypeT type, void* argPtr,
                                        const uint8_t* dataPtr,
                                        uint32_t dataBytes, void* handlePtr);
static int32_t  decVideoBufferReset    (PVR_DecoderTypeT type, void* argPtr,
                                        void* handlePtr);
static uint32_t decVideoBufferGetSize  (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr);
static uint32_t decVideoBufferGetLevel (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr);
static int32_t  decVideoPrepareFrame   (PVR_DecoderTypeT type, void* argPtr,
                                        void* handlePtr);
static uint8_t  decVideoIsFrameDecoded (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr);
static void     decVideoTask           (PVR_DecoderTypeT type, void* argPtr,
                                        void* handlePtr);
static int32_t  decVideoStartDecoder   (PVRDEC_VideoT* vidPtr,
                                        uint16_t pid, uint8_t trackFormat,
                                        VAL_PlayModeEnumT playMode);

static void*    decAudioOpen           (PVR_DecoderTypeT type, void* argPtr);
static int32_t  decAudioClose          (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr);
static int32_t  decAudioStart          (PVR_DecoderTypeT type,
                                        uint8_t trackIndex, uint32_t trackData,
                                        PVR_TrackTypeT trackType,
                                        uint8_t trackFormat, void* argPtr,
                                        uint16_t pid, void* handlePtr);
static int32_t  decAudioStop           (PVR_DecoderTypeT type, void* argPtr,
                                        uint8_t freeze, void* handlePtr);
static int32_t  decAudioSetMode        (PVR_DecoderTypeT type,
                                        void* argPtr, PVR_DecoderModeT mode,
                                        uint8_t modeIndex, void* handlePtr);
static int32_t  decAudioBufferReset    (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr);
static uint32_t decAudioBufferGetSize  (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr);
static uint32_t decAudioBufferGetLevel (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr);
static void     decAudioTask           (PVR_DecoderTypeT type, void* argPtr,
                                        void* handlePtr);
static int32_t  decAudioStartDecoder   (PVRDEC_AudioT* audPtr, uint16_t pid,
                                        uint8_t format, uint8_t channel);

static int32_t  decAvSync              (uint8_t enable, PVR_DecoderTypeT source,
                                        void* argPtr);

/* ------------------------------------------------------------------------- */

//******************************************************************************
//******************************************************************************
//** Local Functions
//******************************************************************************
//******************************************************************************

static void* decVideoOpen (PVR_DecoderTypeT type, void* argPtr)
{
    PVRDEC_DataT*      datPtr   = (PVRDEC_DataT*)argPtr;
    PVRDEC_VideoT*     vidPtr   = &(datPtr->video);

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decVideoOpen\n");

    vidPtr->started        = 0;
    vidPtr->pid            = 0;
    vidPtr->trackFormat    = PVR_TRACK_FORMAT_OTHER;
    vidPtr->currMode       = PVR_DECMODE_NONE;
    vidPtr->currModeIndex  = 0;
    vidPtr->decoderStarted = 0;

    return vidPtr;
}

static int32_t decVideoClose (PVR_DecoderTypeT type, void* argPtr,
                              void* handlePtr)
{
    //PVRDEC_VideoT* vidPtr   = (PVRDEC_VideoT*)handlePtr;
    int32_t             err_code = FAPI_OK;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decVideoClose\n");

    /* currently nothing to do... */

    return err_code;
}

static int32_t decVideoStart (PVR_DecoderTypeT type, uint8_t trackIndex,
                              uint32_t trackData, PVR_TrackTypeT trackType,
                              uint8_t trackFormat, void* argPtr, uint16_t pid,
                              void* handlePtr)
{
    PVRDEC_VideoT*      vidPtr   = (PVRDEC_VideoT*)handlePtr;
    int32_t             retVal   = FAPI_OK;
    VAL_PlayModeEnumT   playMode = VAL_PLAYMODE_NONE;
    PVRSYS_FastModeCfgT fastCfg;
    PVRSYS_SlowModeCfgT slowCfg;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decVideoStart (trackFormat %i pid %i)\n",
                         trackFormat, pid);

    do
    {
        switch ( vidPtr->currMode )
        {
            case PVR_DECMODE_NONE:
            case PVR_DECMODE_PAUSE:
            default:
                playMode = VAL_PLAYMODE_NONE;
                break;

            case PVR_DECMODE_NORMAL:
                playMode = VAL_PLAYMODE_NORMAL;
                break;

            case PVR_DECMODE_FAST:
                retVal = PVRSYS_GetFastModeConfig(vidPtr->currModeIndex,
                                                  &fastCfg);
                if ( retVal == FAPI_OK )
                {
                    playMode = fastCfg.videoMode;
                }
                break;

            case PVR_DECMODE_SLOW:
                retVal = PVRSYS_GetSlowModeConfig(vidPtr->currModeIndex,
                                                  &slowCfg);
                if ( retVal == FAPI_OK )
                {
                    playMode = slowCfg.videoMode;
                }
                break;

            case PVR_DECMODE_FEEDDATA:
                playMode = VAL_PLAYMODE_FAST_LIB_IONLY;
                break;

            case PVR_DECMODE_FEEDCTRL:
                playMode = VAL_PLAYMODE_FAST_CONTROLLEDFEED;
                break;
        }

        if ( playMode != VAL_PLAYMODE_NONE )
        {
            retVal = decVideoStartDecoder (vidPtr, pid, trackFormat, playMode);
        }

    } while ( 0 );

    vidPtr->trackFormat = trackFormat;
    vidPtr->pid         = pid;
    vidPtr->started     = 1;

    return retVal;
}

static int32_t decVideoStop (PVR_DecoderTypeT type, void* argPtr,
                             uint8_t freeze, void* handlePtr)
{
    PVRDEC_VideoT* vidPtr   = (PVRDEC_VideoT*)handlePtr;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decVideoStop\n");

    /* NOTE: Argument freeze is ignored! */

    (void)AVINT_VideoStop(FFALSE);

    vidPtr->started        = 0;
    vidPtr->pid            = 0;
    vidPtr->trackFormat    = PVR_TRACK_FORMAT_OTHER;
    vidPtr->currMode       = PVR_DECMODE_NONE;
    vidPtr->currModeIndex  = 0;
    vidPtr->decoderStarted = 0;

    return FAPI_OK;
}

static int32_t decVideoSetMode (PVR_DecoderTypeT type,
                                void* argPtr, PVR_DecoderModeT mode,
                                uint8_t modeIndex, void* handlePtr)
{
    PVRDEC_VideoT*      vidPtr   = (PVRDEC_VideoT*)handlePtr;
    int32_t             retVal   = FAPI_OK;
    VAL_PlayModeEnumT   playMode = VAL_PLAYMODE_NONE;
    PVRSYS_FastModeCfgT fastCfg;
    PVRSYS_SlowModeCfgT slowCfg;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decVideoSetMode (mode %i)\n", mode);

    if ( vidPtr->started )
    {
        switch ( mode )
        {
            case PVR_DECMODE_NONE:
            default:
                playMode = VAL_PLAYMODE_NONE;
                break;

            case PVR_DECMODE_PAUSE:
                playMode = VAL_PLAYMODE_PAUSE;
                break;

            case PVR_DECMODE_NORMAL:
                playMode = VAL_PLAYMODE_NORMAL;
                break;

            case PVR_DECMODE_FAST:
                retVal = PVRSYS_GetFastModeConfig(modeIndex, &fastCfg);
                if ( retVal == FAPI_OK )
                {
                    playMode = fastCfg.videoMode;
                }
                break;

            case PVR_DECMODE_SLOW:
                retVal = PVRSYS_GetSlowModeConfig(modeIndex, &slowCfg);
                if ( retVal == FAPI_OK )
                {
                    playMode = slowCfg.videoMode;
                }
                break;

            case PVR_DECMODE_FEEDDATA:
                playMode = VAL_PLAYMODE_FAST_LIB_IONLY;
                break;

            case PVR_DECMODE_FEEDCTRL:
                playMode = VAL_PLAYMODE_FAST_CONTROLLEDFEED;
                break;
        }

        if ( playMode != VAL_PLAYMODE_NONE )
        {
            if ( vidPtr->decoderStarted )
            {
                if ( avData->video.valOpenPtr != NULL )
                {
                    retVal = VAL_PlayChangeMode (avData->video.valOpenPtr,
                                                 playMode);
                }
            }
            else
            {
                /* NOTE: Starting decoder in pause mode is not supported
                         by video decoder driver and VAL layer! */
                if ( playMode != VAL_PLAYMODE_PAUSE )
                {
                    retVal = decVideoStartDecoder (vidPtr, vidPtr->pid,
                                                   vidPtr->trackFormat,
                                                   playMode);
                }
            }
        }
    }

    if ( retVal != FAPI_OK )
    {
        FAPI_SYS_PRINT_DEBUG(1,"[PVR] ERROR: decVideoSetMode failure: err %i\n", retVal);
    }

    vidPtr->currMode      = mode;
    vidPtr->currModeIndex = modeIndex;

    return retVal;
}

static int32_t decVideoShowIFrame (PVR_DecoderTypeT type,
                                   void* argPtr, const uint8_t* dataPtr,
                                   uint32_t dataBytes, void* handlePtr)
{
    //PVRDEC_VideoT*  vidPtr = (PVRDEC_VideoT*)handlePtr;
    int32_t         retVal = FAPI_OK;

    /* ensure LW-alignment of dataPtr */
    if ( (((uint32_t)dataPtr) & 0x3) != 0 )
    {
        FAPI_SYS_PRINT_DEBUG(1,"[PVR] NON_ALIGNED I-FRAME!\n");
    }
    else if ( avData->video.valOpenPtr != NULL )
    {
        /*lint -save -e826 */
        retVal = VAL_PlayShowIFrame (avData->video.valOpenPtr,
                                    (const uint32_t*)dataPtr, dataBytes);
        /*lint -restore */
    }

    return retVal;
}

static int32_t decVideoBufferReset (PVR_DecoderTypeT type,
                                    void* argPtr,
                                    void* handlePtr)
{
    //PVRDEC_VideoT* vidPtr = (PVRDEC_VideoT*)handlePtr;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decVideoBufferReset\n");

    // TODO: to be implemented

    return FAPI_OK;
}

static uint32_t decVideoBufferGetSize (PVR_DecoderTypeT type, void* argPtr,
                                       void* handlePtr)
{
    //PVRDEC_VideoT* vidPtr = (PVRDEC_VideoT*)handlePtr;
    uint32_t       size   = 0;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC] decVideoBufferGetSize\n");

    if ( avData->video.valOpenPtr != NULL )
    {
        if ( VAL_GetBufferSize(avData->video.valOpenPtr, &size) != FAPI_OK )
        {
            size  = 0;
        }
    }

    FAPI_SYS_PRINT_DEBUG(2, "[DEC] decVideoBufferGetSize: size %i \n", size);

    return size;
}

static uint32_t decVideoBufferGetLevel (PVR_DecoderTypeT type, void* argPtr,
                                        void* handlePtr)
{
    //PVRDEC_VideoT* vidPtr = (PVRDEC_VideoT*)handlePtr;
    uint32_t       level  = 0;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC] decVideoBufferGetLevel\n");

    if ( avData->video.valOpenPtr != NULL )
    {
        if ( VAL_GetBufferLevel(avData->video.valOpenPtr, &level) != FAPI_OK )
        {
            level  = 0;
        }
    }

    FAPI_SYS_PRINT_DEBUG(2, "[DEC] decVideoBufferGetLevel: level %i \n", level);

    return level;
}

static int32_t decVideoPrepareFrame (PVR_DecoderTypeT type, void* argPtr,
                                    void* handlePtr)
{
    //PVRDEC_VideoT* vidPtr = (PVRDEC_VideoT*)handlePtr;
    int32_t        retVal = FAPI_OK;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC] decVideoPrepareFrame\n");

    if ( avData->video.valOpenPtr != NULL )
    {
        retVal = VAL_PlayPrepareNextFrame(avData->video.valOpenPtr);
    }

    return retVal;
}

static uint8_t decVideoIsFrameDecoded (PVR_DecoderTypeT type,
                                       void* argPtr, void* handlePtr)
{
    //PVRDEC_VideoT* vidPtr       = (PVRDEC_VideoT*)handlePtr;
    uint8_t        frameDecoded = 0;

    if ( avData->video.valOpenPtr != NULL )
    {
        if ( VAL_IsFrameAvailable(avData->video.valOpenPtr, &frameDecoded)
             != FAPI_OK )
        {
            frameDecoded  = 0;
        }
    }

    return frameDecoded;
}

static void decVideoTask (PVR_DecoderTypeT type, void* argPtr, void* handlePtr)
{
    FAPI_SYS_PRINT_DEBUG(3, "[DEC] decVideoTask\n");

    (void)AVINT_VideoTask ();
}

static int32_t decVideoStartDecoder (PVRDEC_VideoT* vidPtr, uint16_t pid,
                                     uint8_t trackFormat,
                                     VAL_PlayModeEnumT playMode)
{
    int32_t         retVal = FAPI_OK;
    uint8_t         streamType;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decVideoStartDecoder (playMode %i)\n", playMode);

    switch ( trackFormat )
    {
        default:
        case PVR_TRACK_FORMAT_VIDEO_MPEG1:
            streamType = STRTYPE_VIDEO_MPEG1;
            break;

        case PVR_TRACK_FORMAT_VIDEO_MPEG2:
            streamType = STRTYPE_VIDEO_MPEG2;
            break;

        case PVR_TRACK_FORMAT_VIDEO_H264:
            streamType = STRTYPE_VIDEO_H264;
            break;
    }

    retVal = AVINT_VideoStart (VAL_ENV_PLAY, pid, streamType, playMode,
                               NULL, NULL);

    if ( retVal == FAPI_OK )
    {
        vidPtr->decoderStarted = 1;
    }

    return retVal;
}

/* ------------------------------------------------------------------------- */

static void* decAudioOpen (PVR_DecoderTypeT type, void* argPtr)
{
    PVRDEC_DataT*  datPtr = (PVRDEC_DataT*)argPtr;
    PVRDEC_AudioT* audPtr = &(datPtr->audio);

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decAudioOpen\n");

    if ( AVINT_AudioOpen() == FAPI_OK )
    {
        audPtr->trackIndex         = 0xFF;
        audPtr->pid                = 0;
        audPtr->format             = SYS_AUDIO_MPEG;

        audPtr->started            = 0;
        audPtr->currMode           = PVR_DECMODE_NONE;
        audPtr->currModeIndex      = 0;
        audPtr->decoderStarted     = 0;
        audPtr->decoderStartStatus = FAPI_OK;
    }
    else
    {
        audPtr = NULL;
    }

    return audPtr;
}

static int32_t decAudioClose (PVR_DecoderTypeT type, void* argPtr,
                              void* handlePtr)
{
    //PVRDEC_AudioT* audPtr = (PVRDEC_AudioT*)handlePtr;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decAudioClose\n");

    (void)AVINT_AudioClose();

    return FAPI_OK;
}

static int32_t decAudioStart (PVR_DecoderTypeT type, uint8_t trackIndex,
                              uint32_t trackData, PVR_TrackTypeT trackType,
                              uint8_t trackFormat, void* argPtr, uint16_t pid,
                              void* handlePtr)
{
    PVRDEC_AudioT*  audPtr    = (PVRDEC_AudioT*)handlePtr;
    int32_t         retVal    = FAPI_OK;
    uint8_t         format;
    uint8_t         channel   = (uint8_t)trackData;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decAudioStart (trackFormat %i pid %i)\n",
                         trackFormat, pid);

    if ( (channel != SYS_AUDIOCHAN_LEFT) && (channel != SYS_AUDIOCHAN_RIGHT) )
    {
        channel = SYS_AUDIOCHAN_LEFT;
    }

    switch ( trackFormat )
    {
        default:
        case PVR_TRACK_FORMAT_AUDIO_MPEG1:
        case PVR_TRACK_FORMAT_AUDIO_MPEG2:
            format = SYS_AUDIO_MPEG;
            break;

        case PVR_TRACK_FORMAT_AUDIO_AC3:
            format = SYS_AUDIO_AC3;
            break;

        case PVR_TRACK_FORMAT_AUDIO_AAC_MPEG2:
            format = SYS_AUDIO_AAC;
            break;

        case PVR_TRACK_FORMAT_AUDIO_AAC_MPEG4:
            format = SYS_AUDIO_HEAAC;
            break;

        case PVR_TRACK_FORMAT_AUDIO_DDPDCV:
            format = SYS_AUDIO_DDPDCV;
            break;
    }

    switch ( audPtr->currMode )
    {
        case PVR_DECMODE_NONE:
        default:
            /* do nothing */
            break;

        case PVR_DECMODE_NORMAL:
        case PVR_DECMODE_PAUSE:
            retVal = decAudioStartDecoder (audPtr, pid, format, channel);
            break;

        case PVR_DECMODE_FAST:
        case PVR_DECMODE_SLOW:
            /* not supported yet */
            break;
    }

    audPtr->trackIndex = trackIndex;
    audPtr->pid        = pid;
    audPtr->format     = format;
    audPtr->channel    = channel;
    audPtr->started    = 1;

    return retVal;
}

static int32_t decAudioStop (PVR_DecoderTypeT type, void* argPtr,
                             uint8_t freeze, void* handlePtr)
{
    PVRDEC_AudioT* audPtr = (PVRDEC_AudioT*)handlePtr;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decAudioStop\n");

    if ( audPtr->decoderStarted )
    {
        AVINT_AudioStop ();

        audPtr->decoderStarted     = 0;
        audPtr->decoderStartStatus = FAPI_OK;
    }

    audPtr->started       = 0;
    audPtr->currMode      = PVR_DECMODE_NONE;
    audPtr->currModeIndex = 0;

    return FAPI_OK;
}

static int32_t decAudioSetMode (PVR_DecoderTypeT type, void* argPtr,
                                PVR_DecoderModeT mode, uint8_t modeIndex,
                                void* handlePtr)
{
    PVRDEC_AudioT*  audPtr = (PVRDEC_AudioT*)handlePtr;
    int32_t         retVal = FAPI_OK;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decAudioSetMode (mode %i)\n", mode);

    if ( audPtr->started )
    {
        switch ( mode )
        {
            case PVR_DECMODE_NORMAL:
                switch ( audPtr->currMode )
                {
                    case PVR_DECMODE_PAUSE:

                        if ( audPtr->decoderStarted )
                        {
                            if ( audPtr->decoderStartStatus == FAPI_OK )
                            {
                                (void)AVINT_AudioPause(FFALSE);
                            }
                        }
                        else
                        {
                            retVal = decAudioStartDecoder (audPtr, audPtr->pid,
                                                           audPtr->format,
                                                           audPtr->channel);
                        }
                        break;

                    case PVR_DECMODE_FAST:
                    case PVR_DECMODE_SLOW:
                        /* not supported yet */
                        break;

                    case PVR_DECMODE_NONE:
                    case PVR_DECMODE_NORMAL:
                    default:
                        /* do nothing */
                        break;
                }
                break;

            case PVR_DECMODE_PAUSE:
                switch ( audPtr->currMode )
                {
                    case PVR_DECMODE_NORMAL:

                        if ( (audPtr->decoderStarted)
                          && (audPtr->decoderStartStatus == FAPI_OK) )
                        {
                            (void)AVINT_AudioPause(FTRUE);
                        }
                        break;

                    case PVR_DECMODE_FAST:
                    case PVR_DECMODE_SLOW:
                        /* not supported yet */
                        break;

                    case PVR_DECMODE_NONE:
                    case PVR_DECMODE_PAUSE:
                    default:
                        /* do nothing */
                        break;
                }
                break;

            default:
                break;
        }
    }

    audPtr->currMode      = mode;
    audPtr->currModeIndex = modeIndex;

    return retVal;
}

static int32_t decAudioBufferReset (PVR_DecoderTypeT type, void* argPtr,
                                    void* handlePtr)
{
    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decAudioBufferReset\n");

    // TODO: to be implemented

    return FAPI_OK;
}

static uint32_t decAudioBufferGetSize (PVR_DecoderTypeT type,
                                       void* argPtr, void* handlePtr)
{
    uint32_t size = 0;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC] decAudioBufferGetSize \n");

    if ( avData->audio.audioHandle != NULL )
    {
        size = AAL_GetBufferSize(avData->audio.audioHandle);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[DEC] decAudioBufferGetSize: size %i \n", size);
    return size;
}

static uint32_t decAudioBufferGetLevel (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr)
{
    uint32_t level = 0;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC] decAudioBufferGetLevel \n");

    if ( avData->audio.audioHandle != NULL )
    {
        level = AAL_GetBufferLevel(avData->audio.audioHandle);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[DEC] decAudioBufferGetLevel: level %i \n", level);
    return level;
}

static void decAudioTask (PVR_DecoderTypeT type, void* argPtr, void* handlePtr)
{
    PVRDEC_AudioT* audPtr = (PVRDEC_AudioT*)handlePtr;

    FAPI_SYS_PRINT_DEBUG(3, "[DEC] decAudioTask\n");

    if ( (audPtr->decoderStarted)
      && (audPtr->decoderStartStatus == FAPI_OK) )
    {
        (void)AVINT_AudioTask ();
    }
}

static int32_t decAudioStartDecoder (PVRDEC_AudioT* audPtr, uint16_t pid,
                                     uint8_t format, uint8_t channel)
{
    int32_t retVal;

    retVal = AVINT_AudioStart (pid, format, channel, NULL);

    audPtr->decoderStarted     = 1;
    audPtr->decoderStartStatus = retVal;

    return retVal;
}

/* ------------------------------------------------------------------------- */

static void* decSpdifOpen (PVR_DecoderTypeT type, void* argPtr)
{
    PVRDEC_DataT*  datPtr = (PVRDEC_DataT*)argPtr;
    PVRDEC_AudioT* audPtr = &(datPtr->audio);

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decSpdifOpen\n");

    /* NOTE: Opening of audio module is done in decAudioOpen! */

    return audPtr;
}

static int32_t decSpdifClose (PVR_DecoderTypeT type, void* argPtr,
                              void* handlePtr)
{
    //PVRDEC_AudioT* audPtr = (PVRDEC_AudioT*)handlePtr;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decSpdifClose\n");

    /* NOTE: Closing of audio module is done in decAudioOpen! */

    return FAPI_OK;
}

static int32_t decSpdifStart (PVR_DecoderTypeT type, uint8_t trackIndex,
                              uint32_t trackData, PVR_TrackTypeT trackType,
                              uint8_t trackFormat, void* argPtr, uint16_t pid,
                              void* handlePtr)
{
    //PVRDEC_AudioT*  audPtr    = (PVRDEC_AudioT*)handlePtr;
    int32_t         retVal    = FAPI_OK;
    uint8_t         format;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decSpdifStart (trackFormat %i pid %i)\n",
                         trackFormat, pid);

    switch ( trackFormat )
    {
        default:
        case PVR_TRACK_FORMAT_AUDIO_MPEG1:
        case PVR_TRACK_FORMAT_AUDIO_MPEG2:
            format = SYS_AUDIO_MPEG;
            break;

        case PVR_TRACK_FORMAT_AUDIO_AC3:
            format = SYS_AUDIO_AC3;
            break;

        case PVR_TRACK_FORMAT_AUDIO_AAC_MPEG2:
            format = SYS_AUDIO_AAC;
            break;

        case PVR_TRACK_FORMAT_AUDIO_AAC_MPEG4:
            format = SYS_AUDIO_HEAAC;
            break;

        case PVR_TRACK_FORMAT_AUDIO_DDPDCV:
            format = SYS_AUDIO_DDPDCV;
            break;
    }

    retVal = AVINT_AudioSPDIFSetStatus (SPDIFSTATUS_PASSTHROUGH, pid, format);

    return retVal;
}

static int32_t decSpdifStop (PVR_DecoderTypeT type, void* argPtr,
                             uint8_t freeze, void* handlePtr)
{
    //PVRDEC_AudioT* audPtr = (PVRDEC_AudioT*)handlePtr;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decSpdifStop\n");

    (void)AVINT_AudioSPDIFSetStatus (SPDIFSTATUS_CURRPCM, 0, SYS_AUDIO_NUMS);

    return FAPI_OK;
}

static uint32_t decSpdifBufferGetSize (PVR_DecoderTypeT type,
                                       void* argPtr, void* handlePtr)
{
    uint32_t size = 0;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC] decSpdifBufferGetSize \n");

    if ( avData->audio.audioHandle != NULL )
    {
        size = AAL_GetSPDIFBufferSize(avData->audio.audioHandle);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[DEC] decSpdifBufferGetSize: size %i \n", size);
    return size;
}

static uint32_t decSpdifBufferGetLevel (PVR_DecoderTypeT type,
                                        void* argPtr, void* handlePtr)
{
    uint32_t level = 0;

    FAPI_SYS_PRINT_DEBUG(2, "[DEC] decSpdifBufferGetLevel \n");

    if ( avData->audio.audioHandle != NULL )
    {
        level = AAL_GetSPDIFBufferLevel(avData->audio.audioHandle);
    }

    FAPI_SYS_PRINT_DEBUG(2, "[DEC] decSpdifBufferGetLevel: level %i \n", level);
    return level;
}

/* ------------------------------------------------------------------------- */

static int32_t decAvSync (uint8_t enable, PVR_DecoderTypeT source, void* argPtr)
{
    //PVRDEC_DataT* datPtr = (PVRDEC_DataT*)argPtr;

    FAPI_SYS_PRINT_DEBUG(1, "[DEC] decAvSync enable %i source %i\n",
                         enable, source);

    // FIXME: needed?
    #if 1

    if ( !enable )
    {
        (void)FAPI_SYNC_SetStcRegulation(0);
    }

    #endif

    return FAPI_OK;
}

/* ------------------------------------------------------------------------- */

int32_t PVRDEC_Init (void)
{
    DBG_Assert(pvrDecPtr == NULL);

    pvrDecPtr = UTILS_Malloc(sizeof(PVRDEC_DataT));
    if ( pvrDecPtr == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset (pvrDecPtr, 0, sizeof(PVRDEC_DataT));

    return FAPI_OK;
}

void PVRDEC_RegisterFunctions (PVR_TargetControlT* ctrlPtr)
{
    PVR_DecoderControlT* decoderPtr;

    DBG_Assert(pvrDecPtr != NULL);

    memset(ctrlPtr, 0, sizeof(PVR_TargetControlT));

    ctrlPtr->argPtr             = pvrDecPtr;
    ctrlPtr->avSync             = decAvSync;

    decoderPtr = ctrlPtr->decoder + ((int32_t)PVR_DECTYPE_VIDEO);

    decoderPtr->open            = decVideoOpen;
    decoderPtr->close           = decVideoClose;
    decoderPtr->start           = decVideoStart;
    decoderPtr->stop            = decVideoStop;
    decoderPtr->setMode         = decVideoSetMode;
    decoderPtr->feedData        = decVideoShowIFrame;
    decoderPtr->resetBuffer     = decVideoBufferReset;
    decoderPtr->getBufferSize   = decVideoBufferGetSize;
    decoderPtr->getBufferLevel  = decVideoBufferGetLevel;
    decoderPtr->prepareData     = decVideoPrepareFrame;
    decoderPtr->isDataDecoded   = decVideoIsFrameDecoded;
    decoderPtr->task            = decVideoTask;

    decoderPtr = ctrlPtr->decoder + ((int32_t)PVR_DECTYPE_AUDIO);

    decoderPtr->open            = decAudioOpen;
    decoderPtr->close           = decAudioClose;
    decoderPtr->start           = decAudioStart;
    decoderPtr->stop            = decAudioStop;
    decoderPtr->setMode         = decAudioSetMode;
    decoderPtr->resetBuffer     = decAudioBufferReset;
    decoderPtr->getBufferSize   = decAudioBufferGetSize;
    decoderPtr->getBufferLevel  = decAudioBufferGetLevel;
    decoderPtr->task            = decAudioTask;

    decoderPtr = ctrlPtr->decoder + ((int32_t)PVR_DECTYPE_AUDIO_2ND);

    decoderPtr->open            = decSpdifOpen;
    decoderPtr->close           = decSpdifClose;
    decoderPtr->start           = decSpdifStart;
    decoderPtr->stop            = decSpdifStop;
    decoderPtr->getBufferSize   = decSpdifBufferGetSize;
    decoderPtr->getBufferLevel  = decSpdifBufferGetLevel;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
