/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/pvr_list.c
**
**              $Id: pvr_list.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       PVR play list of recorded files.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info    754: Local structure member 'reserved' not referenced.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1,
**                dependence placed on C's operator precedence;
**
*****************************************************************************
*/
/*lint -save -e754 -e961 */
#define DEBUG 1

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>

#include "pvr_sys.h"
#include "pvr_list.h"
#include "init.h"
#include "statlist.h"

#include "utils.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define PVR_LIST_DIR_PATH_MAX        32

#define PVR_LIST_RECFILENAME         "rec"
#define PVR_LIST_METAFILENAME        "meta.dat"

#ifdef APPL_EDITMARKER_ENABLED
#define PVR_LIST_MARKERFILENAME      "rec.bm"
#endif /* APPL_EDITMARKER_ENABLED */

#define PVR_LIST_PATHBUF_SIZE        128

#define PVR_LIST_DIRNAME_SIZE        9          //!< Size of directory name
                                                //!< string incl. termination.
#define PVR_LIST_DIRNAME_PREFIX      "REC_"

#define PVR_LIST_REC_PER_DEVICE_MAX  1000

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    uint32_t              id;                 // file identification
    uint32_t              revision;           // file structure revision

} PVR_LIST_FILEHDR_S;

typedef struct
{
    STATLST_S             statLst;                    //!< static list
    PVRLST_IDX            refLst[PVR_LIST_MAX_ITEMS]; //!< continuous list
    int16_t               numItems;                   //!< number of recordings
                                                      //!< available

    char_t                pathBuf[PVR_LIST_PATHBUF_SIZE];
    char_t                tmpBuf [PVR_LIST_PATHBUF_SIZE];

} PVR_LIST_DAT_S;

typedef struct
{
    char_t          dir[PVR_LIST_DIR_PATH_MAX]; //!< Directory name.
    PVR_LIST_ITEM_S data;                       //!< Record item data.
    int8_t          recIdx;                     //!< Current recording status.

} PVR_LIST_ENTRY_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

static PVR_LIST_DAT_S* pvrlstDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC fbool_t PVR_LIST_InsertRefEntry (PVRLST_IDX idx, int16_t* newPos);
FSTATIC int32_t PVR_LIST_RemoveEntry    (int16_t lstPos);
FSTATIC int32_t PVR_LIST_LoadItemData   (const char_t* path,
                                         PVR_LIST_ITEM_S* data);
FSTATIC int32_t PVR_LIST_SaveItemData   (const char_t* path,
                                         const PVR_LIST_ITEM_S* data);
FSTATIC int32_t PVR_LIST_PrepareItemData(const PVR_LIST_ENTRY_S* pEntry);
FSTATIC int32_t PVR_LIST_SaveEntry      (const PVR_LIST_ENTRY_S* pEntry);
FSTATIC int32_t PVR_LIST_DeleteRecord   (PVRLST_IDX idx);

FSTATIC int32_t PVR_LIST_GetMetaFileName(const PVR_LIST_ENTRY_S* pEntry,
                                         char_t* buf, uint32_t bufSize);

FSTATIC fbool_t PVR_LIST_IsRecFolderName (const char_t* dirName);

#ifdef APPL_EDITMARKER_ENABLED

FSTATIC int32_t PVR_LIST_PrepareBlockMarker     (const PVR_LIST_ENTRY_S* pEntry);
FSTATIC int32_t PVR_LIST_GetBlockMarkerFileName (const PVR_LIST_ENTRY_S* pEntry,
                                                 char_t* buf, uint32_t bufSize);
FSTATIC int32_t PVR_LIST_LoadBlockMarkerData    (const char_t* path,
                                                 PVR_BLOCK_MARKER_ITEM_S* pBlockMarker);
FSTATIC int32_t PVR_LIST_WriteBlockMarkerData   (const char_t* path,
                                                 PVR_BLOCK_MARKER_ITEM_S* pBlockMarker);

#endif /* APPL_EDITMARKER_ENABLED */

// NOTE: This function is currently unused
//FSTATIC int32_t PVR_LIST_GetListPos             (PVRLST_IDX idx, int16_t* lstPos);
FSTATIC int32_t PVR_LIST_Import                 (uint32_t revImport,
                                                 uint32_t revCurr,
                                                 F_FILE* file,
                                                 PVR_LIST_ITEM_S* pData);
FSTATIC fbool_t PVR_LIST_FileFind               (const char_t* fileName);
FSTATIC int32_t PVR_LIST_SyncItem               (SYS_DEVICE_E device,
                                                 const char_t* dirName,
                                                 char_t* pathBuf,
                                                 uint32_t bufPos,
                                                 uint32_t bufSize,
                                                 uint32_t dirStartPos);

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/

int32_t PVR_LIST_Init ()
{
    int32_t retVal;

    DBG_Assert(pvrlstDat == NULL);

    pvrlstDat = UTILS_Malloc(sizeof(PVR_LIST_DAT_S));
    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset (pvrlstDat, 0, sizeof(PVR_LIST_DAT_S));

    retVal = STATLST_Init (&(pvrlstDat->statLst), PVR_LIST_MAX_ITEMS,
                           sizeof(PVR_LIST_ENTRY_S));
    if ( retVal != FAPI_OK ) return retVal;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get size of playback list.
**
** \return Number of entries in list.
******************************************************************************/
int32_t PVR_LIST_GetNumItems (int16_t* numItems)
{
    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    *numItems = pvrlstDat->numItems;
    return FAPI_OK;
}

int32_t PVR_LIST_GetIndex (int16_t lstPos, PVRLST_IDX* idx)
{
    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if ( idx       == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    if ( (lstPos < 0) || (lstPos >= pvrlstDat->numItems) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    *idx = pvrlstDat->refLst[lstPos];
    return FAPI_OK;
}

int32_t PVR_LIST_AddRecordItem (const PVR_LIST_ITEM_S* pItem, uint8_t recIdx,
                                const char_t* dirPath, PVRLST_IDX* idx)
{
    int32_t            retVal;
    int32_t            newStatIdx;
    PVR_LIST_ENTRY_S   entry;
    uint16_t           cnt;
    char_t             recDirName[PVR_LIST_DIRNAME_SIZE];
    uint32_t           strPos     = 0;
    uint32_t           dirStartPos;

    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if ( pvrlstDat->numItems >= PVR_LIST_MAX_ITEMS )
    {
        return SMARTGO_SYS_ERR_CAPACITY;
    }

    memset (&entry, 0, sizeof(entry));

    /* ----- create unique directory --------------------------------------- */

    strPos += (uint32_t)SYS_DeviceGetMountPath (pItem->device,
                                                pvrlstDat->pathBuf + strPos,
                                                PVR_LIST_PATHBUF_SIZE - strPos);

    if ( (strPos + 2) >= PVR_LIST_PATHBUF_SIZE )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    strPos += SYS_DeviceAddSeparator (pvrlstDat->pathBuf + strPos,
                                      PVR_LIST_PATHBUF_SIZE - strPos);

    dirStartPos = strPos;

    if ( (dirPath != NULL) &&
         (dirPath[0] != 0) )
    {
        strPos += UTILS_strncpy (pvrlstDat->pathBuf + strPos, dirPath,
                                 PVR_LIST_PATHBUF_SIZE - strPos);

        if ( strPos >= PVR_LIST_PATHBUF_SIZE )
        {
            return SMARTGO_SYS_ERR_INTERNAL;
        }

        strPos += SYS_DeviceAddSeparator (pvrlstDat->pathBuf + strPos,
                                          PVR_LIST_PATHBUF_SIZE - strPos);
    }

    if ( (strPos + sizeof(recDirName) + 1) >= PVR_LIST_PATHBUF_SIZE )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    retVal = SYS_DevicePreAccess (pItem->device);
    if ( retVal != FAPI_OK ) return retVal;

    pvrlstDat->tmpBuf[0] = 0;

    for ( cnt = 0; cnt < PVR_LIST_REC_PER_DEVICE_MAX; cnt++ )
    {
        sprintf(recDirName, "%s%04d", PVR_LIST_DIRNAME_PREFIX, cnt);

        (void)UTILS_strncpy (pvrlstDat->pathBuf + strPos, recDirName,
                             sizeof(recDirName));

        /* check if directory already exists */
        if ( !(PVR_LIST_FileFind(pvrlstDat->pathBuf)) )
        {
            /* attempt to create directory */
            retVal = SYS_DeviceMkdir (pItem->device,
                                      pvrlstDat->pathBuf + dirStartPos);
            if ( retVal == FAPI_OK )
            {
                /* store directory path temporarily */
                (void)UTILS_strncpy (pvrlstDat->tmpBuf, pvrlstDat->pathBuf,
                                     PVR_LIST_PATHBUF_SIZE);
            }

            break;
        }
    }

    if ( cnt >= PVR_LIST_REC_PER_DEVICE_MAX )
    {
        retVal = SMARTGO_SYS_ERR_CAPACITY;
    }

    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    /* ----- Add entry in static list -------------------------------------- */

    (void)UTILS_strncpy (entry.dir, pvrlstDat->pathBuf + dirStartPos,
                         PVR_LIST_DIR_PATH_MAX);

    entry.data   = *pItem;
    entry.recIdx = (int8_t)recIdx;

    /* set a default value into the reserved values */
    memset (entry.data.reserved, 0, sizeof(entry.data.reserved));

    newStatIdx = -1;

    retVal = STATLST_AddEntry (&(pvrlstDat->statLst), &entry,
                               SYS_OWNER_USER, &newStatIdx);
    if ( retVal == FAPI_OK )
    {
        retVal = PVR_LIST_PrepareItemData (&entry);

        if ( retVal == FAPI_OK )
        {
            (void)PVR_LIST_InsertRefEntry (newStatIdx, NULL);

            *idx = newStatIdx;

            #ifdef APPL_EDITMARKER_ENABLED

            (void)PVR_LIST_PrepareBlockMarker (&entry);

            #endif /* APPL_EDITMARKER_ENABLED */
        }
    }

    if ( retVal != FAPI_OK )
    {
        if ( newStatIdx >= 0 )
        {
            (void)STATLST_RemoveEntry(&(pvrlstDat->statLst), newStatIdx);
        }

        // TODO: remove meta data file if available

        if ( pvrlstDat->tmpBuf[0] != 0 )
        {
            (void)SYS_DeviceRmdir (pItem->device,
                                   pvrlstDat->tmpBuf + dirStartPos);
        }
    }

    return retVal;
}

int32_t PVR_LIST_SetItem (const PVR_LIST_ITEM_S* pItem, PVRLST_IDX idx,
                          fbool_t endOfRecording)
{
    PVR_LIST_ENTRY_S* pEntry;

    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    pEntry = STATLST_GetEntry(&(pvrlstDat->statLst), idx);
    if ( pEntry == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    /* Modify entry data */
    pEntry->data   = *pItem;

    if ( endOfRecording )
    {
        pEntry->recIdx = -1;
    }

    (void)PVR_LIST_SaveEntry (pEntry);

    return FAPI_OK;
}

int32_t PVR_LIST_GetFilename (PVRLST_IDX idx, char_t* filename,
                              uint16_t maxLength)
{
    PVR_LIST_ENTRY_S* pEntry;
    uint32_t          strPos;

    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    pEntry = STATLST_GetEntry(&(pvrlstDat->statLst), idx);
    if ( pEntry == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

// FIXME: warum hier + 1 ?
    strPos = UTILS_strncpy (filename, pEntry->dir, PVR_FILE_NAME_LENGTH + 1);

    if ( (strPos + 1) >= maxLength ) return SMARTGO_SYS_ERR_CAPACITY;

    strPos += SYS_DeviceAddSeparator (filename + strPos, 2);

    if ( (strPos + 1) >= maxLength ) return SMARTGO_SYS_ERR_CAPACITY;

    (void)UTILS_strncpy (filename + strPos, PVR_LIST_RECFILENAME,
                         maxLength - strPos);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get a playback list entry.
**
** \param  lstPos  Playback list index.
** \param  pItem   Entry data is returned here (NULL if not needed).
** \param  recIdx  If item is currently being target of a recording, recording
**                 index is returned here (NULL if not needed).
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t PVR_LIST_GetItem (int16_t lstPos, PVR_LIST_ITEM_S* pItem,
                          int8_t* recIdx)
{
    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if ( (lstPos < 0) || (lstPos >= pvrlstDat->numItems) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    return PVR_LIST_GetItemByIndex (pvrlstDat->refLst[lstPos], pItem, recIdx);
}

/*!
*******************************************************************************
**
** \brief  Get list position by recIdx
**
** \param  recIdx  If item is currently being target of a recording, recording
**                 index is returned here (NULL if not needed).
** \param  lstPos  Playback list index.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t PVR_LIST_GetListPosByRecIdx (int8_t recIdx, int16_t* lstPos)
{
    PVR_LIST_ITEM_S     pItem;
    int16_t             numItems, cnt;
    int8_t              currRec;
    int32_t             ferr = FAPI_OK;

    ferr = PVR_LIST_GetNumItems (&numItems);
        if ( ferr != FAPI_OK )
            return ferr;
    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if  ( (recIdx >= RSRC_REC_MAX) || (recIdx < 0) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    for ( cnt = 0 ; cnt < numItems ; cnt++ )
    {
        if ( PVR_LIST_GetItem (cnt, &pItem, &currRec) == FAPI_OK )
        {
            if ( currRec == recIdx )
            {
                *lstPos = cnt;
                break;
            }
        }
    }

    if ( cnt == numItems )
    {
        /* recording Idx was not found */
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        return FAPI_OK;
    }
}


/*!
*******************************************************************************
**
** \brief  Replace playback list entry.
**
** \note If item is not being recorded anymore, replaced data will be
**       instantly stored on target device.
**
** \param  lstPos  Playback list index.
** \param  pItem   Item data to be stored.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t PVR_LIST_ReplaceItem (int16_t lstPos, const PVR_LIST_ITEM_S* pItem)
{
    int32_t    retVal;
    PVRLST_IDX idx;

    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if ( (lstPos < 0) || (lstPos >= pvrlstDat->numItems) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    retVal = PVR_LIST_GetIndex (lstPos, &idx);
    if ( retVal != FAPI_OK ) return retVal;

    return PVR_LIST_SetItem (pItem, idx, FFALSE);
}

int32_t PVR_LIST_GetItemByIndex (PVRLST_IDX idx, PVR_LIST_ITEM_S* pItem,
                                 int8_t* recIdx)
{
    PVR_LIST_ENTRY_S* pEntry;

    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    pEntry = STATLST_GetEntry(&(pvrlstDat->statLst), idx);
    if ( pEntry == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    if ( pItem  != NULL ) *pItem  = pEntry->data;
    if ( recIdx != NULL ) *recIdx = pEntry->recIdx;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get recording information.
**
** This function can be used for getting information on a PVR recording
** as well as simply identifying PVR recording items. For the latter case pItem
** may be set to NULL.
**
** \param  path        Absolute path of assumed recording directory.
** \param  pItem       Optional pointer to structure where recording
**                     information will be copied to.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
******************************************************************************/
int32_t PVR_LIST_GetItemByPath (const char_t* path, PVR_LIST_ITEM_S* pItem)
{
    SYS_DEVICE_E    device;
    int32_t         pathLen;
    uint32_t        strPos;

    /* test if directory name is valid */

    pathLen = (int32_t)strlen(path);

    if ( ((pathLen + 1) < PVR_LIST_DIRNAME_SIZE) ||
         (!(PVR_LIST_IsRecFolderName(path + pathLen - PVR_LIST_DIRNAME_SIZE + 1))) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    strPos = UTILS_strncpy(pvrlstDat->pathBuf, path, PVR_LIST_PATHBUF_SIZE);

    if ( (strPos + 2) >= PVR_LIST_PATHBUF_SIZE ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    strPos += SYS_DeviceAddSeparator(pvrlstDat->pathBuf + strPos, 2);

    strPos += UTILS_strncpy (pvrlstDat->pathBuf + strPos,
                             PVR_LIST_METAFILENAME,
                             PVR_LIST_PATHBUF_SIZE - strPos);

    if ( SYS_DeviceGetByPath (path, &device, NULL) == FAPI_OK )
    {
        (void)SYS_DevicePreAccess (device);
    }

    /* transfer metafile data into given structure */
    return PVR_LIST_LoadItemData (pvrlstDat->pathBuf, pItem);
}

/*!
*******************************************************************************
**
** \brief  Get static index of a recording by its directory path.
**
** \param  path    Full path string on directory of recording to be searched.
** \param  idx     Index is returned here on success (NULL if not needed).
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t PVR_LIST_GetIndexByPath (const char_t* path, PVRLST_IDX* idx)
{
    SYS_DEVICE_E        device;
    int32_t             pathLen;
    uint16_t            pathOffset = 0;
    int32_t             retVal;
    int16_t             cnt;
    PVR_LIST_ENTRY_S*   pEntry;

    pathLen = (int32_t)strlen(path);

    /* test if directory name is valid */
    if ( ((pathLen + 1) < PVR_LIST_DIRNAME_SIZE) ||
         (!(PVR_LIST_IsRecFolderName(path + pathLen - PVR_LIST_DIRNAME_SIZE + 1))) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    retVal = SYS_DeviceGetByPath (path, &device, &pathOffset);

    if ( retVal == FAPI_OK )
    {
        for ( cnt = 0; cnt < pvrlstDat->numItems; cnt++ )
        {
            pEntry = STATLST_GetEntry (&(pvrlstDat->statLst),
                                       pvrlstDat->refLst[cnt]);
            if ( pEntry == NULL ) continue;

            if ( pEntry->data.device == device )
            {
                /* device matches, now search for path match */
                if ( UTILS_strncmpi(pEntry->dir, path + pathOffset,
                                   (pathLen - (int32_t)pathOffset)) == 0 )
                {
                    if ( idx != NULL )
                    {
                        *idx = cnt;
                    }

                    break;
                }
            }
        }

        if ( cnt >= pvrlstDat->numItems )
        {
            /* no matching item found */
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Remove a playback list entry and its corresponding file on target
**         device.
**
** \param  lstPos   Playback list position to be removed.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t PVR_LIST_RemoveItem (int16_t lstPos)
{
    int32_t retVal = FAPI_OK;

    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if ( (lstPos < 0) || (lstPos >= pvrlstDat->numItems) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    retVal = PVR_LIST_DeleteRecord (pvrlstDat->refLst[lstPos]);
    if ( retVal != FAPI_OK ) return retVal;

    return PVR_LIST_RemoveEntry(lstPos);
}

int32_t PVR_LIST_RemoveItemByIndex (PVRLST_IDX idx)
{
    int32_t retVal = FAPI_OK;
    int16_t cnt;

    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    /* Search entry in reference list */
    for ( cnt = 0; cnt < pvrlstDat->numItems; cnt++ )
    {
        if ( pvrlstDat->refLst[cnt] == idx ) break;
    }

    if ( cnt >= pvrlstDat->numItems )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    retVal = PVR_LIST_DeleteRecord (idx);
    if ( retVal != FAPI_OK ) return retVal;

    return PVR_LIST_RemoveEntry(cnt);
}

#ifdef APPL_EDITMARKER_ENABLED

/* Block Marker File Access Functions */
/*!
*******************************************************************************
**
** \brief  Get a playback block marker data entry.
**
** \param  lstPos  Playback list position.
** \param  pItem   Entry data is returned here.
**
** \return One of the following status codes:
**         - #FAPI_OK                           successful
**         - #APPL_SMARTGO_ERR_NOT_INITIALIZED  basic data structure is not initialized
**         - #APPL_SMARTGO_ERR_BAD_PARAMETER    passed parameter is out of scope
**         - #SMARTGO_SYS_ERR_INTERNAL          file access is not possible
******************************************************************************/
int32_t PVR_LIST_GetMarker (int16_t lstPos, PVR_BLOCK_MARKER_ITEM_S* pMarker)
{
    PVR_LIST_ENTRY_S*   pEntry;
    int32_t             retVal = FAPI_OK;

    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if ( (lstPos < 0) || (lstPos >= pvrlstDat->numItems) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    pEntry = STATLST_GetEntry(&(pvrlstDat->statLst), pvrlstDat->refLst[lstPos]);
    if ( pEntry == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    /* assemble complete path to block marker fiel */
    retVal = PVR_LIST_GetBlockMarkerFileName (pEntry, pvrlstDat->pathBuf,
                                              PVR_LIST_PATHBUF_SIZE);
    if ( retVal != FAPI_OK ) return retVal;
    retVal = PVR_LIST_LoadBlockMarkerData (pvrlstDat->pathBuf, pMarker);
    return retVal;
}

int32_t PVR_LIST_GetMarkerByIdx (PVRLST_IDX idx,
                                 PVR_BLOCK_MARKER_ITEM_S* pMarker)
{
    PVR_LIST_ENTRY_S*   pEntry;
    int32_t             retVal = FAPI_OK;

    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;


    pEntry = STATLST_GetEntry(&(pvrlstDat->statLst), idx);
    if ( pEntry == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    /* assemble complete path to block marker fiel */
    retVal = PVR_LIST_GetBlockMarkerFileName (pEntry, pvrlstDat->pathBuf,
                                              PVR_LIST_PATHBUF_SIZE);
    if ( retVal != FAPI_OK ) return retVal;
    retVal = PVR_LIST_LoadBlockMarkerData (pvrlstDat->pathBuf, pMarker);
    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Write back playback block marker data entry.
**
** \param  idx     Playback list index.
** \param  pItem   Entry data is returned here.
**
** \return One of the following status codes:
**         - #FAPI_OK                           successful
**         - #APPL_SMARTGO_ERR_NOT_INITIALIZED  basic data structure is not initialized
**         - #APPL_SMARTGO_ERR_BAD_PARAMETER    passed parameter is out of scope
**         - #SMARTGO_SYS_ERR_INTERNAL          file access is not possible
******************************************************************************/
int32_t PVR_LIST_SetMarker (PVRLST_IDX idx, PVR_BLOCK_MARKER_ITEM_S* pMarker)
{
    PVR_LIST_ENTRY_S*   pEntry;
    int32_t             retVal = FAPI_OK;

    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;


    pEntry = STATLST_GetEntry(&(pvrlstDat->statLst), idx); //pvrlstDat->refLst[lstPos]);
    if ( pEntry == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    retVal = PVR_LIST_GetBlockMarkerFileName (pEntry, pvrlstDat->pathBuf,
                                              PVR_LIST_PATHBUF_SIZE);
    if ( retVal != FAPI_OK ) return retVal;

    retVal = PVR_LIST_WriteBlockMarkerData (pvrlstDat->pathBuf,
                                            pMarker);
    return retVal;
}

#endif /* APPL_EDITMARKER_ENABLED */

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC fbool_t PVR_LIST_InsertRefEntry (PVRLST_IDX idx, int16_t* newPos)
{
    PVR_LIST_ENTRY_S* listEntry;
    PVR_LIST_ENTRY_S* refEntry;
    int16_t           index;
    int16_t           shiftCnt;
    fbool_t           insert = FFALSE;

    DBG_Assert(pvrlstDat->numItems < PVR_LIST_MAX_ITEMS);

    listEntry = STATLST_GetEntry (&(pvrlstDat->statLst), idx);
    if ( listEntry == NULL ) return FFALSE;

    index = pvrlstDat->numItems - 1;

    while ( index >= -1 )
    {
        if ( index == -1 ) insert = FTRUE;
        else
        {
            refEntry = STATLST_GetEntry (&(pvrlstDat->statLst),
                                         pvrlstDat->refLst[index]);
            if ( refEntry != NULL )
            {
                /* test for time and date to keep chronological order */
                insert = TIME_IsLaterThan ( &(listEntry->data.startTime),
                                            &(listEntry->data.startDate),
                                            &(refEntry->data.startTime),
                                            &(refEntry->data.startDate) );
            }
        }

        if ( insert )
        {
            /* shift subsequent entries */
            for ( shiftCnt = pvrlstDat->numItems - 1; shiftCnt > index;
                  shiftCnt-- )
            {
                pvrlstDat->refLst[shiftCnt + 1] = pvrlstDat->refLst[shiftCnt];
            }

            if ( newPos != NULL) *newPos = index + 1;

            pvrlstDat->refLst[index + 1] = idx;
            pvrlstDat->numItems++;
            return FTRUE;
        }

        index--;
    }

    return FFALSE;
}

FSTATIC int32_t PVR_LIST_RemoveEntry (int16_t lstPos)
{
    int32_t retVal;
    int16_t cnt;

    if ( (lstPos < 0) ||
         (lstPos >= pvrlstDat->numItems) ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    retVal = STATLST_RemoveEntry (&(pvrlstDat->statLst),
                                  pvrlstDat->refLst[lstPos]);
    if ( retVal != FAPI_OK ) return retVal;

    /* Remove entry in reference list by shifting subsequent entries. */
    for ( cnt = lstPos; cnt < (int16_t)(pvrlstDat->numItems - 1); cnt++ )
    {
        pvrlstDat->refLst[cnt] = pvrlstDat->refLst[cnt + 1];
    }

    pvrlstDat->numItems--;

    return FAPI_OK;
}

FSTATIC int32_t PVR_LIST_LoadItemData (const char_t* path,
                                       PVR_LIST_ITEM_S* data)
{
    int32_t             retVal = FAPI_OK;
    PVR_LIST_FILEHDR_S  hdr;
    F_FILE*             file;

    file = f_open (path, "r");
    if ( file == NULL )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    do
    {
        /* read header */
        if ( f_read (&hdr, 1, sizeof(hdr), file) != (int32_t)(sizeof(hdr)) )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        if ( hdr.id != FILEID_PVRLIST )
        {
            retVal = SMARTGO_SYS_ERR_INVALID_DATA;
            break;
        }

        if ( hdr.revision != FILEREV_PVRLIST )
        {
            retVal = PVR_LIST_Import (hdr.revision, FILEREV_PVRLIST, file,
                                      data);
        }
        else
        {
            /* read data */
            if ( data != NULL )
            {
                if ( f_read ((void*)data, 1, sizeof(PVR_LIST_ITEM_S), file)
                     != (int32_t)(sizeof(PVR_LIST_ITEM_S)) )
                {
                    retVal = SMARTGO_SYS_ERR_INTERNAL;
                    break;
                }
            }
        }
    }
    while ( 0 );

    (void)f_close (file);

    return retVal;
}

FSTATIC int32_t PVR_LIST_SaveItemData (const char_t* path,
                                       const PVR_LIST_ITEM_S* data)
{
    PVR_LIST_FILEHDR_S hdr;
    int32_t            retVal = FAPI_OK;
    F_FILE*            file;

    file = f_open(path, "a");

    if ( file == NULL )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    hdr.id       = FILEID_PVRLIST;
    hdr.revision = FILEREV_PVRLIST;

    do
    {
        if ( f_seek(file, 0, F_SEEK_SET) != F_NO_ERROR )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        /* write header */
        if ( f_write(&hdr, 1, sizeof(hdr), file) != (int32_t)(sizeof(hdr)) )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        /* write data */
        if ( f_write((void*)data, 1, sizeof(PVR_LIST_ITEM_S), file)
             != (int32_t)(sizeof(PVR_LIST_ITEM_S)) )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }
    }
    while ( 0 );

    (void)f_close (file);

    return retVal;
}

FSTATIC int32_t PVR_LIST_PrepareItemData (const PVR_LIST_ENTRY_S* pEntry)
{
    PVR_LIST_FILEHDR_S hdr;
    int32_t            retVal = FAPI_OK;
    F_FILE*            file;

    retVal = PVR_LIST_GetMetaFileName (pEntry, pvrlstDat->pathBuf,
                                       PVR_LIST_PATHBUF_SIZE);
    if ( retVal != FAPI_OK ) return retVal;

    file = f_open(pvrlstDat->pathBuf, "w");

    if ( file == NULL )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    hdr.id       = FILEID_PVRLIST;
    hdr.revision = FILEREV_PVRLIST;

    do
    {
        /* write header */
        if ( f_write(&hdr, 1, sizeof(hdr), file) != (int32_t)(sizeof(hdr)) )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        /* write data */
        if ( f_write((void*)(&(pEntry->data)), 1, sizeof(pEntry->data), file)
             !=  (int32_t)(sizeof(pEntry->data)) )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }
    }
    while ( 0 );

    (void)f_close (file);

    return retVal;
}

FSTATIC int32_t PVR_LIST_SaveEntry (const PVR_LIST_ENTRY_S* pEntry)
{
    int32_t retVal = FAPI_OK;

    retVal = PVR_LIST_GetMetaFileName (pEntry, pvrlstDat->pathBuf,
                                       PVR_LIST_PATHBUF_SIZE);
    if ( retVal != FAPI_OK ) return retVal;

    retVal = SYS_DevicePreAccess (pEntry->data.device);
    if ( retVal != FAPI_OK ) return retVal;

    return PVR_LIST_SaveItemData (pvrlstDat->pathBuf, &(pEntry->data));
}

FSTATIC int32_t PVR_LIST_DeleteRecord (PVRLST_IDX idx)
{
    uint32_t           strPos = 0;
    uint32_t           tmpPos;
    int32_t            retVal;
    PVR_LIST_ENTRY_S*  pEntry;

    pEntry = STATLST_GetEntry (&(pvrlstDat->statLst), idx);
    if ( pEntry == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    retVal = SYS_DevicePreAccess (pEntry->data.device);
    if ( retVal != FAPI_OK ) return retVal;

    /* ----- Remove TS files ----------------------------------------------- */

    retVal = PVR_LIST_GetFilename (idx, pvrlstDat->pathBuf,
                                   PVR_LIST_PATHBUF_SIZE);
    if ( retVal == FAPI_OK )
    {
        (void)PVRSYS_SetMountPath(pEntry->data.device);

        (void)PVR_DeleteFile(pvrlstDat->pathBuf);
    }

    /* ----- Build directory path ------------------------------------------ */

    strPos += (uint32_t)SYS_DeviceGetMountPath (pEntry->data.device,
                                                pvrlstDat->pathBuf + strPos,
                                                PVR_LIST_PATHBUF_SIZE - strPos);

    strPos += SYS_DeviceAddSeparator (pvrlstDat->pathBuf + strPos, 2);

    if ( (strPos + 1) >= PVR_LIST_PATHBUF_SIZE )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    strPos += UTILS_strncpy (pvrlstDat->pathBuf + strPos, pEntry->dir,
                             PVR_LIST_PATHBUF_SIZE - strPos);

    if ( (strPos + 1) >= PVR_LIST_PATHBUF_SIZE )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    /* ----- Remove meta file ---------------------------------------------- */

    tmpPos = strPos;

    tmpPos += SYS_DeviceAddSeparator (pvrlstDat->pathBuf + tmpPos, 2);

    if ( (tmpPos + 1) >= PVR_LIST_PATHBUF_SIZE )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    tmpPos += UTILS_strncpy(pvrlstDat->pathBuf + tmpPos,
                            PVR_LIST_METAFILENAME,
                            PVR_LIST_PATHBUF_SIZE - tmpPos);

    (void)f_delete(pvrlstDat->pathBuf);

    /* ----- Remove blockmarker file --------------------------------------- */

    #ifdef APPL_EDITMARKER_ENABLED

    tmpPos = strPos;

    tmpPos += SYS_DeviceAddSeparator (pvrlstDat->pathBuf + tmpPos, 2);

    if ( (tmpPos + 1) >= PVR_LIST_PATHBUF_SIZE )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    tmpPos += UTILS_strncpy(pvrlstDat->pathBuf + tmpPos,
                            PVR_LIST_MARKERFILENAME,
                            PVR_LIST_PATHBUF_SIZE - tmpPos);

    (void)f_delete(pvrlstDat->pathBuf);

    #endif /* APPL_EDITMARKER_ENABLED */

    /* ----- Attempt to remove folder -------------------------------------- */

    pvrlstDat->pathBuf[strPos] = 0;

    (void)f_rmdir (pvrlstDat->pathBuf);

    return retVal;
}

/* ----- PVR device management --------------------------------------------- */

/*
    relPath: path to be scanned relative to device's mount path
*/
int32_t PVR_LIST_SyncDevice (SYS_DEVICE_E device, const char_t* relPath)
{
    int32_t            result;
    F_FIND             dirInf;
    uint32_t           strPos = 0;
    int32_t            retVal;
    SYS_DEV_STAT_S     devStat;
    uint32_t           dirStartPos;

    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if ( relPath == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    FAPI_SYS_PRINT_DEBUG(4,"PVR_LIST_SyncDevice: device %i, path '%s' \n", device, relPath);

    if( (device <  (SYS_DEVICE_E)0)
     || (device >= SYS_DEV_NUMS) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    retVal = SYS_DevicePreAccess (device);
    if ( retVal != FAPI_OK ) return retVal;

    /* ----- Remove all entries referencing requested device --------------- */

    /* Remove all associated entries from PVR item list */
    (void)PVR_LIST_DeviceRemoved(device);

    /* ----- Scan device for PVR items ------------------------------------- */

    /* scan only if device is ready */
    retVal = SYS_DeviceGetStatus (device, &devStat);
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }
    else if ( devStat.mountStatus != SYS_DEV_FS_MOUNTED )
    {
        return FAPI_OK;
    }

    strPos += (uint32_t)SYS_DeviceGetMountPath (device,
                                                pvrlstDat->pathBuf + strPos,
                                                PVR_LIST_PATHBUF_SIZE - strPos);

    strPos += SYS_DeviceAddSeparator (pvrlstDat->pathBuf + strPos, 2);

    dirStartPos = strPos;

    /* add path extension if requested */
    if ( relPath[0] != 0 )
    {
        if ( (strPos + 1) >= PVR_LIST_PATHBUF_SIZE )
        {
            return SMARTGO_SYS_ERR_INTERNAL;
        }

        strPos += UTILS_strncpy (pvrlstDat->pathBuf + strPos, relPath,
                                 PVR_LIST_PATHBUF_SIZE - strPos);

        /* Check if there is enough space for path separator + wildcard
           characters which are still to be appended. */
        if ( (strPos + 5) >= PVR_LIST_PATHBUF_SIZE )
        {
            return SMARTGO_SYS_ERR_INTERNAL;
        }

        strPos += SYS_DeviceAddSeparator (pvrlstDat->pathBuf + strPos,
                                          PVR_LIST_PATHBUF_SIZE - strPos);
    }

    /* add wildcard characters but do not update strPos */
    if ( (strPos + 3) >= PVR_LIST_PATHBUF_SIZE )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    (void)UTILS_strncpy (pvrlstDat->pathBuf + strPos, "*.*",
                         PVR_LIST_PATHBUF_SIZE - strPos);

    result = f_findfirst(pvrlstDat->pathBuf, &dirInf);

    while ( result == F_NO_ERROR )
    {
        if ( (dirInf.attr & F_ATTR_DIR) != 0 )
        {
            (void)PVR_LIST_SyncItem (device, dirInf.filename,
                                     pvrlstDat->pathBuf, strPos,
                                     PVR_LIST_PATHBUF_SIZE, dirStartPos);
        }

        result = f_findnext(&dirInf);
    }

    return FAPI_OK;
}

FSTATIC int32_t PVR_LIST_SyncItem (SYS_DEVICE_E device, const char_t* dirName,
                                   char_t* pathBuf, uint32_t bufPos,
                                   uint32_t bufSize, uint32_t dirStartPos)
{
    uint32_t            dirStrLen;
    PVR_LIST_ENTRY_S    newEntry;
    int32_t             newStatIdx;
    int32_t             retVal = FAPI_OK;
    PVR_FileInfoT       fileInfo;

    do
    {
        /* test for folder naming convention */
        if ( !(PVR_LIST_IsRecFolderName(dirName)) )
        {
            retVal = SMARTGO_SYS_ERR_INVALID_DATA;
            break;
        }

        /* add directory name and separator to path */
        if ( (bufPos + PVR_LIST_DIRNAME_SIZE) >= bufSize )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        bufPos += UTILS_strncpy (pathBuf + bufPos, dirName, bufSize - bufPos);

        dirStrLen = bufPos - dirStartPos;

        if ( (dirStrLen + 1) > PVR_LIST_DIR_PATH_MAX )
        {
            FAPI_SYS_PRINT_DEBUG(1,"[PVR] Sync path too long!\n");
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        bufPos += SYS_DeviceAddSeparator (pathBuf + bufPos, 2);

        /* ----- get recording information --------------------------------- */

        if ( (bufPos + strlen(PVR_LIST_RECFILENAME)) >= bufSize )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        (void)UTILS_strncpy (pathBuf + bufPos, PVR_LIST_RECFILENAME,
                             bufSize - bufPos);

        retVal = PVRSYS_SetMountPath (device);
        if ( retVal != FAPI_OK ) break;

        memset (&fileInfo, 0, sizeof(fileInfo));

        retVal = PVR_GetFileInfo(pathBuf + dirStartPos, &fileInfo);
        if ( retVal != FAPI_OK ) break;

        // TODO: remove recording folder if recording is invalid and stored
        //       on internal HDD.

        /* ----- get meta file information --------------------------------- */

        if ( (bufPos + strlen(PVR_LIST_METAFILENAME)) >= bufSize )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
            break;
        }

        bufPos += UTILS_strncpy (pathBuf + bufPos, PVR_LIST_METAFILENAME,
                                 bufSize - bufPos);

        memset (&newEntry, 0, sizeof(newEntry));

        retVal = PVR_LIST_LoadItemData (pathBuf, &(newEntry.data));
        if ( retVal != FAPI_OK ) break;

        // TODO: remove recording folder if meta data is invalid and stored
        //       on internal HDD.

        newEntry.recIdx      = -1;
        newEntry.data.device = device;

        /* test if recording has been properly stopped */
        if ( newEntry.data.size == 0 )
        {
            FAPI_SYS_PRINT_DEBUG(4,"[PVR] Sync: found abandoned recording: '%s'\n",
                       dirName);

            /* add missing information from recording */
            if ( fileInfo.size > 0 )
            {
                /* add size and duration information */
                newEntry.data.size     = fileInfo.size;
                newEntry.data.duration = TIME_ConvToTime (fileInfo.duration
                                                          / 1000);

                /* NOTE: Currently also recordings that would have been
                         discarded if they had been normally stopped
                         (->timeshift recordings) will be found and repaired
                         by this mechanism!
                */

                /* save repaired meta data file */
                (void)PVR_LIST_SaveItemData (pathBuf, &(newEntry.data));

                FAPI_SYS_PRINT_DEBUG(4,"[PVR] Sync: recording repaired!\n");
            }
        }

        FAPI_SYS_PRINT_DEBUG(4,"[PVR] Sync: Add item: '%s'\n", dirName);

        /* add PVR item to internal list */
        (void)UTILS_strncpy (newEntry.dir,
                             pathBuf + dirStartPos, dirStrLen + 1);

        retVal = STATLST_AddEntry (&(pvrlstDat->statLst), &newEntry,
                                   SYS_OWNER_USER, &newStatIdx);
        if ( retVal == FAPI_OK )
        {
            (void)PVR_LIST_InsertRefEntry (newStatIdx, NULL);
        }
    }
    while ( 0 );

    return retVal;
}

/* Remove all recordings referencing given path on given device from list.
   If relPath is NULL, all items are removed.
*/
int32_t PVR_LIST_DeviceRemoved (SYS_DEVICE_E device)
{
    PVR_LIST_ENTRY_S* pEntry;
    int16_t           cnt;

    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    FAPI_SYS_PRINT_DEBUG(4,"PVR_LIST_DeviceRemoved: device %i\n", device);

    if ( pvrlstDat->numItems > 0 )
    {
        for ( cnt = (pvrlstDat->numItems - 1); cnt >= 0; cnt-- )
        {
            pEntry = STATLST_GetEntry (&(pvrlstDat->statLst),
                                       pvrlstDat->refLst[cnt]);
            if ( pEntry == NULL ) continue;

            if ( pEntry->data.device == device )
            {
                (void)PVR_LIST_RemoveEntry(cnt);
            }
        }
    }

    return FAPI_OK;
}

/* lint -save -e737 */
FSTATIC int32_t PVR_LIST_GetMetaFileName (const PVR_LIST_ENTRY_S* pEntry,
                                          char_t* buf, uint32_t bufSize)
{
    uint32_t strPos = 0;

    /* ----- device path --------------------------------------------------- */

    strPos += (uint32_t)SYS_DeviceGetMountPath (pEntry->data.device,
                                                buf + strPos,
                                                bufSize - strPos);

    if ( (strPos + 2) >= bufSize )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    strPos += SYS_DeviceAddSeparator (buf + strPos, 2);

    if ( (strPos + 1) >= bufSize )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    /* ----- directory path ------------------------------------------------ */

    strPos += UTILS_strncpy(buf + strPos, pEntry->dir, bufSize - strPos);

    if ( (strPos + 1) >= bufSize )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    strPos += SYS_DeviceAddSeparator (buf + strPos, 2);

    if ( (strPos + 1) >= bufSize )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    /* ----- file name ----------------------------------------------------- */

    strPos += UTILS_strncpy (buf + strPos, PVR_LIST_METAFILENAME,
                             bufSize - strPos);

    return FAPI_OK;
}
/* lint -restore */


FSTATIC fbool_t PVR_LIST_IsRecFolderName (const char_t* dirName)
{
    char_t*     pPrefix     = PVR_LIST_DIRNAME_PREFIX;
    uint32_t    nameLen     = strlen(dirName);
    uint32_t    prefixLen   = strlen(PVR_LIST_DIRNAME_PREFIX);
    uint32_t    cnt;

    if ( (nameLen + 1) != PVR_LIST_DIRNAME_SIZE ) return FFALSE;

    for ( cnt = 0; cnt < prefixLen; cnt++ )
    {
        if ( dirName[cnt] != pPrefix[cnt] ) return FFALSE;
    }

    return FTRUE;
}

#ifdef APPL_EDITMARKER_ENABLED

FSTATIC int32_t PVR_LIST_PrepareBlockMarker (const PVR_LIST_ENTRY_S* pEntry)
{
    PVR_BLOCK_MARKER_ITEM_S blockMarkerParams;
    int32_t            retVal = FAPI_OK;
    F_FILE*            file;

    memset (&blockMarkerParams, 0, sizeof(PVR_BLOCK_MARKER_ITEM_S));
    retVal = PVR_LIST_GetBlockMarkerFileName (pEntry, pvrlstDat->pathBuf,
                                              PVR_LIST_PATHBUF_SIZE);
    if ( retVal != FAPI_OK ) return retVal;

    file = f_open(pvrlstDat->pathBuf, "w");

    if ( file == NULL )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    /* write data */
    if ( f_write((void*)(&(blockMarkerParams)), 1, sizeof(blockMarkerParams),
                 file) != sizeof(blockMarkerParams) )
    {
        retVal = SMARTGO_SYS_ERR_INTERNAL;
    }

    (void)f_close (file);

    return retVal;
}

FSTATIC int32_t PVR_LIST_GetBlockMarkerFileName (const PVR_LIST_ENTRY_S* pEntry,
                                                 char_t* buf, uint32_t bufSize)
{
    uint32_t  strPos = 0;

    /* ----- device path --------------------------------------------------- */

    strPos += SYS_DeviceGetMountPath (pEntry->data.device, buf + strPos,
                                      bufSize - strPos);

    if ( (strPos + 2) >= bufSize )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    strPos += SYS_DeviceAddSeparator (buf + strPos, 2);

    if ( (strPos + 1) >= bufSize )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    /* ----- directory path ------------------------------------------------ */

    strPos += UTILS_strncpy(buf + strPos, pEntry->dir, bufSize - strPos);

    if ( (strPos + 1) >= bufSize )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    strPos += SYS_DeviceAddSeparator (buf + strPos, 2);

    if ( (strPos + 1) >= bufSize )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    /* ----- file name ----------------------------------------------------- */

    strPos += UTILS_strncpy (buf + strPos, PVR_LIST_MARKERFILENAME,
                             bufSize - strPos);

    return FAPI_OK;
}

FSTATIC int32_t PVR_LIST_LoadBlockMarkerData (const char_t* path,
                                              PVR_BLOCK_MARKER_ITEM_S*
                                              pBlockMarker)
{
    int32_t         retVal = FAPI_OK;
    F_FILE*         file;
    SYS_DEVICE_E    device;

    if ( SYS_DeviceGetByPath (path, &device, NULL) == FAPI_OK )
    {
        (void)SYS_DevicePreAccess (device);
    }

    file = f_open (path, "r");
    if ( file == NULL )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    /* read data */
    if ( pBlockMarker != NULL )
    {
        if ( f_read ((void*)pBlockMarker, 1, sizeof(PVR_BLOCK_MARKER_ITEM_S),
                     file) != sizeof(PVR_BLOCK_MARKER_ITEM_S) )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
        }
    }
    else
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    (void)f_close (file);

    return retVal;
}

FSTATIC int32_t PVR_LIST_WriteBlockMarkerData (const char_t* path,
                                               PVR_BLOCK_MARKER_ITEM_S*
                                               pBlockMarker)
{
    int32_t         retVal = FAPI_OK;
    F_FILE*         file;
    SYS_DEVICE_E    device;

    if ( SYS_DeviceGetByPath (path, &device, NULL) == FAPI_OK )
    {
        (void)SYS_DevicePreAccess (device);
    }

    file = f_open (path, "w");
    if ( file == NULL )
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    /* write data */
    if ( f_write ((void*)(pBlockMarker), 1, sizeof(PVR_BLOCK_MARKER_ITEM_S),
                  file) !=  sizeof(PVR_BLOCK_MARKER_ITEM_S) )
    {
        retVal = SMARTGO_SYS_ERR_INTERNAL;
    }

    (void)f_close (file);

    return retVal;
}

#endif /* APPL_EDITMARKER_ENABLED */

/* NOTE: This function is currently unused */
#if 0
FSTATIC int32_t PVR_LIST_GetListPos (PVRLST_IDX idx, int16_t* lstPos)
{
    int16_t cntPos;

    if ( pvrlstDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if ( lstPos    == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;


    for ( cntPos = 0 ; cntPos < PVR_LIST_MAX_ITEMS ; cntPos++)
    {
        if ( pvrlstDat->refLst[cntPos] == idx )
            break;
    }

    if (cntPos == PVR_LIST_MAX_ITEMS)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    else
    {
        *lstPos = cntPos;
        return FAPI_OK;
    }
}
#endif

/* ------------------------------------------------------------------------- */

typedef struct
{
    SYS_DEVICE_E        device;             //!< Device where rec. resides on.
    SLIST_ITEM_S        service;            //!< Info on recorded service.
    char_t              name [PVR_NAME_LENGTH]; //!< Recording title.
    SYS_TIME_S          startTime;          //!< Record start time (UTC).
    SYS_DATE_S          startDate;          //!< Record start date (UTC).
    uint32_t            size;               //!< File size in PVR data chunks.
    SYS_TIME_S          duration;           //!< Record duration.
    PVRSYS_REC_TYPE_E   type;               //!< Type of recording,
                                            //!< REC_TYPE_NORMAL or
                                            //!< REC_TYPE_BACKGROUND.
    int32_t             lastStop;           //!< Last stop position (-1: not
                                            //!< available).
    SLIST_CA_INFO_S     caInfo;             //!< CA system information.
    uint32_t            flags;              //!< Misc. flags
                                            //!< (see PVR_LIST_FLAG_XXX).
    uint32_t            reserved[3];        //!< Reserved for future usage.
                                            //!< (set to 0)

} PVR_LIST_ITEM_V09_S;

FSTATIC int32_t PVR_LIST_Import (uint32_t revImport, uint32_t revCurr,
                                 F_FILE* file, PVR_LIST_ITEM_S* pData)
{
    int32_t retVal = FAPI_OK;

    if ( (revImport == 9) && (revCurr == 10) )
    {
        PVR_LIST_ITEM_V09_S pvrItem;

        /* seems the only way to get rid of MISRA warning */
        pvrItem.reserved[0] = 0; // MISRA
        pvrItem.reserved[1] = 0; // MISRA
        pvrItem.reserved[2] = 0; // MISRA
        if ( pData != 0 )
        {
            if ( f_read ((void*)(&pvrItem), 1, sizeof(PVR_LIST_ITEM_V09_S),
                         file) != (int32_t)(sizeof(PVR_LIST_ITEM_V09_S)) )
            {
                retVal = SMARTGO_SYS_ERR_INTERNAL;
            }
            else
            {
                memset (pData, 0, sizeof(PVR_LIST_ITEM_S));

                pData->device       = pvrItem.device;
                pData->service      = pvrItem.service;
                pData->nameCharSet  = SYS_CHARSET_ASCII; /* new parameter */

                (void)UTILS_strncpy (pData->name, pvrItem.name,
                                     PVR_NAME_LENGTH);

                pData->startTime    = pvrItem.startTime;
                pData->startDate    = pvrItem.startDate;
                pData->size         = pvrItem.size;
                pData->duration     = pvrItem.duration;
                pData->type         = pvrItem.type;
                pData->lastStop     = pvrItem.lastStop;
                pData->caInfo       = pvrItem.caInfo;
                pData->flags        = pvrItem.flags;
            }
        }
    }
    else
    {
        retVal = SMARTGO_SYS_ERR_INVALID_DATA;
    }

    return retVal;
}

FSTATIC fbool_t PVR_LIST_FileFind (const char_t* fileName)
{
    uint8_t attr;

    return (f_getattr (fileName,&attr) == F_NO_ERROR) ? FTRUE : FFALSE;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
