/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/rectimer.c
**
**              $Id: rectimer.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       Recording timer handling.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "fapex/abst_types.h"

#include <flashfs/flashfs.h>

#include "sys_api.h"
#include "sys_priv.h"
#include "utils.h"

#include "event_queue.h"
#include "statlist.h"

#include "rectimer.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define RECTIMER_CHECK_INTERVAL     1000   // record timer update interval in ms
#define RECTIMER_TIMESYNC_TIMEOUT 180000   // 3 min

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef enum
{
    RECTIMER_UNUSED = 0,
    RECTIMER_NOTSTARTED,
    RECTIMER_STARTED,
    RECTIMER_STOPPED

} RECTIMER_INTSTAT_E;

typedef struct
{
    uint32_t            id;         // file identification
    uint32_t            revision;   // file structure revision

} RECTIMER_FILEHDR_S;

typedef struct
{
    fbool_t             instant;    //!< FTRUE: timer has been allocated by
                                    //!< instant recording.
    RECTIMER_ITEM_S     data;       //!< Record timer parameters.

} RECTIMER_ENTRY_S;

typedef struct
{

    FAPEX_SYS_SemaphoreT     mutexId;

    STATLST_S           statLst;                //!< Static timer list.
    RECTIMER_IDX        refLst[RECTIMER_MAX];   //!< Continuous timer list.
    RECTIMER_INTSTAT_E  status[RECTIMER_MAX];   //!< Activity status of each
                                                //!< timer.
    int16_t             numTimers;              //!< Number of timers in refLst.
    fbool_t             dirty;                  //!< FTRUE: current data does
                                                //!< not match stored data.

    /* sleep timer parameters */
    fbool_t             sleepTimerEnabled;
    SYS_TIME_S          sleepTime;
    SYS_DATE_S          sleepDate;

    uint64_t            timeSyncTimeout;

} RECTIMER_DAT_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC RECTIMER_DAT_S* rtDat       = NULL;
FSTATIC fbool_t         lstRecValid = FTRUE;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void    RECTIMER_Update                 (void);
FSTATIC fbool_t RECTIMER_InsertRefEntry         (RECTIMER_IDX idx,
                                                 RECTIMER_INTSTAT_E reqStatus,
                                                 int16_t* newPos);
FSTATIC void    RECTIMER_ThreadFunction         (void* opt);
FSTATIC int32_t RECTIMER_IntRemove              (RECTIMER_IDX idx);
FSTATIC int32_t RECTIMER_IntReplace             (RECTIMER_IDX idx,
                                                 const RECTIMER_ITEM_S* data);
FSTATIC int32_t RECTIMER_IntHandleExpiredTimer  (RECTIMER_IDX idx);
FSTATIC void    RECTIMER_SendStatusEvent        (RECTIMER_IDX idx,
                                                 RECTIMER_STATUS_E status,
                                                 uint32_t seconds,
                                                 fbool_t removeIt);

void RECTIMER_Invalidate (void)
{
    lstRecValid = FFALSE;
}

int32_t RECTIMER_Init (void)
{
    int32_t  retVal;
    uint32_t maxFileSize;

    DBG_Assert(rtDat == NULL);

    /* Test if maximum flash file size is sufficient. */
    maxFileSize = sizeof(RECTIMER_FILEHDR_S)
                + STATLST_GetMaxDataSize (RECTIMER_MAX,
                                          sizeof(RECTIMER_ENTRY_S));

    retVal = FLASH_CheckFileSize (FILEID_RECTIMER, maxFileSize);

    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    rtDat = UTILS_Malloc(sizeof(RECTIMER_DAT_S));
    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(rtDat, 0, sizeof(RECTIMER_DAT_S));

    rtDat->sleepTimerEnabled  = FFALSE;
    rtDat->dirty              = FTRUE;

    /* create semaphore for mutual exclusion purposes */
    rtDat->mutexId = FAPEX_SYS_CreateSemaphore (1);
    if ( rtDat->mutexId == NULL ) return SMARTGO_SYS_ERR_RESOURCES;

    retVal = STATLST_Init (&(rtDat->statLst), RECTIMER_MAX,
                           sizeof(RECTIMER_ENTRY_S));
    if ( retVal != FAPI_OK ) return retVal;

    (void)RECTIMER_Load();

    /* start timer handling thread */
    return SYS_ThreadCreate (RECTIMER_ThreadFunction, NULL,
                             THREAD_PRIO_TIMER,
                             THREAD_STACKSZ_TIMER,
                             THREAD_NAME_TIMER,
                             NULL);
}

int32_t RECTIMER_Clear (void)
{
    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    (void)STATLST_Clear(&(rtDat->statLst), SYS_OWNER_ALL, NULL, NULL);

    rtDat->numTimers         = 0;
    rtDat->dirty             = FTRUE;
    rtDat->sleepTimerEnabled = FFALSE;

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return FAPI_OK;
}

int32_t RECTIMER_Add (const RECTIMER_ITEM_S* pTimer, int16_t* newPos)
{
    int32_t             retVal = FAPI_OK;
    RECTIMER_ENTRY_S    newEntry;
    RECTIMER_IDX        newStatIdx;

    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    if ( rtDat->numTimers >= RECTIMER_MAX )
    {
        retVal = SMARTGO_SYS_ERR_CAPACITY;
    }
    else
    {
        newEntry.instant = FFALSE;
        newEntry.data    = *pTimer;

        retVal = STATLST_AddEntry (&(rtDat->statLst), &newEntry,
                                   SYS_OWNER_USER, &newStatIdx);
        if ( retVal == FAPI_OK )
        {
            (void)RECTIMER_InsertRefEntry (newStatIdx, RECTIMER_NOTSTARTED,
                                           newPos);
            rtDat->dirty = FTRUE;
        }
    }

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t RECTIMER_AddInstantRec (const RECTIMER_ITEM_S* pTimer,
                                int16_t* newPos,
                                fbool_t keepInstRecTimer)
{
    int32_t          retVal = FAPI_OK;
    RECTIMER_ENTRY_S newEntry;
    RECTIMER_IDX     newStatIdx;

    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    if ( rtDat->numTimers >= RECTIMER_MAX )
    {
        retVal = SMARTGO_SYS_ERR_CAPACITY;
    }
    else
    {
        /*
            keepInstRecTimer defines if after power up again a still existing and
            valid timer will be executed or discarded
        */
        if ( keepInstRecTimer )
        {
            newEntry.instant = FFALSE;
        }
        else
        {
            newEntry.instant = FTRUE;
        }

        newEntry.data    = *pTimer;

        retVal = STATLST_AddEntry (&(rtDat->statLst), &newEntry,
                                   SYS_OWNER_USER, &newStatIdx);
        if ( retVal == FAPI_OK )
        {
            /* Instant recording timers must not be explicitly started. */
            (void)RECTIMER_InsertRefEntry (newStatIdx, RECTIMER_STARTED,
                                           newPos);
            rtDat->dirty = FTRUE;
        }
    }

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}


int32_t RECTIMER_Remove (int16_t lstPos)
{
    int32_t              retVal = FAPI_OK;

    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    if ( (lstPos < 0) || (lstPos >= rtDat->numTimers) )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        retVal = RECTIMER_IntRemove(rtDat->refLst[lstPos]);
    }

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t RECTIMER_Replace (int16_t lstPos, const RECTIMER_ITEM_S* pTimer)
{
    int32_t   retVal = FAPI_OK;

    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    if ( (lstPos < 0) || (lstPos >= rtDat->numTimers) )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        retVal = RECTIMER_IntReplace(rtDat->refLst[lstPos], pTimer);
    }

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t RECTIMER_Get (int16_t lstPos, RECTIMER_ITEM_S* pTimer)
{
    RECTIMER_ENTRY_S* pEntry;
    int32_t              retVal = FAPI_OK;

    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    if ( (lstPos < 0) || (lstPos >= rtDat->numTimers) )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        pEntry = STATLST_GetEntry(&(rtDat->statLst), rtDat->refLst[lstPos]);

        if ( pEntry == NULL )
        {
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
        else
        {
            *pTimer = pEntry->data;
        }
    }

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t RECTIMER_GetIndex (int16_t lstPos, RECTIMER_IDX* idx)
{
    int32_t retVal = FAPI_OK;

    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    if ( (lstPos < 0) || (lstPos >= rtDat->numTimers) )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        *idx = rtDat->refLst[lstPos];
    }

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t RECTIMER_GetIndexByEvtId (SLIST_SERVIDX servIdx, uint16_t eventId,
                               RECTIMER_IDX* idx)
{
    int32_t           retVal = FAPI_OK;
    RECTIMER_ENTRY_S* listEntry;
    int16_t            cnt;

    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    /* search for requested timer */
    for ( cnt = 0; cnt < rtDat->numTimers; cnt++ )
    {
        listEntry = STATLST_GetEntry (&(rtDat->statLst), rtDat->refLst[cnt]);
        if ( listEntry == NULL ) continue;

        if ( (listEntry->data.serviceIndex == servIdx) &&
             (listEntry->data.eventId      == eventId) )
        {
            *idx = rtDat->refLst[cnt];
            break;
        }
    }

    if ( cnt >= rtDat->numTimers ) retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t RECTIMER_GetNumTimers (uint16_t* numTimers)
{
    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    *numTimers = (uint16_t)rtDat->numTimers;
    return FAPI_OK;
}

int32_t RECTIMER_RemoveByIndex (RECTIMER_IDX idx)
{
    int32_t retVal;

    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    retVal = RECTIMER_IntRemove(idx);

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t RECTIMER_ReplaceByIndex (RECTIMER_IDX idx, const RECTIMER_ITEM_S* pTimer)
{
    int32_t retVal;

    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    retVal = RECTIMER_IntReplace(idx, pTimer);

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t RECTIMER_GetByIndex (RECTIMER_IDX idx, RECTIMER_ITEM_S* pTimer)
{
    RECTIMER_ENTRY_S* pEntry;
    int32_t              retVal = FAPI_OK;

    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    pEntry = STATLST_GetEntry(&(rtDat->statLst), idx);

    if ( pEntry == NULL )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        *pTimer = pEntry->data;
    }

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t RECTIMER_Reinsert (RECTIMER_IDX idx)
{
    int32_t              retVal = FAPI_OK;
    RECTIMER_ENTRY_S* listEntry;

    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    /* Get timer data: this is done only to ensure that 'idx' argument is
       valid. */
    listEntry = STATLST_GetEntry (&(rtDat->statLst), idx);
    if ( listEntry == NULL )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        retVal = RECTIMER_IntHandleExpiredTimer(idx);
    }

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t RECTIMER_Save (fbool_t force)
{
    int32_t             retVal = FAPI_OK;
    RECTIMER_FILEHDR_S  hdr;
    FLASHFS_HANDLE      file;

    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if ( (!force) && (!(rtDat->dirty)) ) return FAPI_OK;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    file = FLASHFS_Open (FILEID_RECTIMER, FLASHFS_OPEN_WRITE, &retVal);

    if ( file != NULL )
    {
        hdr.id       = FILEID_RECTIMER;
        hdr.revision = FILEREV_RECTIMER;

        /* write header */
        if ( FLASHFS_Write(file, sizeof(hdr), &hdr, NULL)
             != (int32_t)sizeof(hdr) )
        {
            retVal = SMARTGO_SYS_ERR_INTERNAL;
        }
        else
        {
            /* write list */
            if ( !(STATLST_WriteToFS(&(rtDat->statLst), FFALSE, file)) )
            {
                retVal = SMARTGO_SYS_ERR_INTERNAL;
            }
        }

        (void)FLASHFS_Close(file);
    }

    if ( retVal == FAPI_OK )
    {
        lstRecValid  = FTRUE;
        rtDat->dirty = FFALSE;
    }

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

int32_t RECTIMER_Load (void)
{
    RECTIMER_FILEHDR_S  fileHdr;
    int32_t             retVal = FAPI_OK;
    int32_t             cnt;
    RECTIMER_ENTRY_S*   listEntry;
    int16_t             newPos;
    FLASHFS_HANDLE      file;

    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    FAPI_SYS_PRINT_DEBUG(1,"[SYS] Load record timer list\n");

    /* clear list */
    retVal = RECTIMER_Clear ();
    if ( retVal != FAPI_OK ) return retVal;

    /* If list is marked as invalid refuse loading */
    if ( !lstRecValid ) return retVal;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    file = FLASHFS_Open (FILEID_RECTIMER, FLASHFS_OPEN_READ, NULL);

    if ( file != NULL )
    {
        do
        {
            /* read file header */
            if ( FLASHFS_Read(file, sizeof(fileHdr), &fileHdr, NULL)
                 != (int32_t)sizeof(fileHdr) )
            {
                retVal = SMARTGO_SYS_ERR_INTERNAL;
                break;
            }

            if ( (fileHdr.id       != FILEID_RECTIMER) ||
                 (fileHdr.revision != FILEREV_RECTIMER) )
            {
                // FIXME: set error code here?
                break;
            }

            rtDat->numTimers = 0;

            (void)STATLST_Clear(&(rtDat->statLst), SYS_OWNER_ALL, NULL, NULL);

            /* read item list */
            if ( !(STATLST_LoadFromFS (&(rtDat->statLst), FFALSE, file)) )
            {
                retVal = SMARTGO_SYS_ERR_INTERNAL;
                break;
            }

            /* generate reference list in chronological order */
            for ( cnt = 0; cnt < rtDat->statLst.stat.afterLast; cnt++ )
            {
                listEntry = STATLST_GetEntry(&(rtDat->statLst), cnt);

                /* If entry is non-empty, set reference on it */
                if ( listEntry != NULL )
                {
                    if ( listEntry->instant )
                    {
                        /* instant record timer, remove it from list... */
                        (void)STATLST_RemoveEntry(&(rtDat->statLst), cnt);
                    }
                    else
                    {
                        (void)RECTIMER_InsertRefEntry (cnt, RECTIMER_NOTSTARTED,
                                                       &newPos);
                    }
                }
            }

            if ( rtDat->numTimers != (int16_t)rtDat->statLst.stat.numEntries )
            {
                FAPI_SYS_PRINT_DEBUG(2,"RECTIMER_Load: internal error\n");
                break;
            }

            rtDat->dirty = FFALSE;
        }
        while ( 0 );

        (void)FLASHFS_Close (file);
    }

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

/* NOTE: This function is not thread-safe. It has to be externally protected */
FSTATIC int32_t RECTIMER_IntRemove (RECTIMER_IDX idx)
{
    RECTIMER_ENTRY_S* listEntry;
    int16_t               cnt;

    listEntry = STATLST_GetEntry (&(rtDat->statLst), idx);
    if ( listEntry == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    /* search for timer in refLst */
    for ( cnt = 0; cnt < rtDat->numTimers; cnt++ )
    {
        if ( rtDat->refLst[cnt] == idx ) break;
    }

    if ( rtDat->numTimers > 0 )
    {
        /* Remove entry in reference list by shifting subsequent entries. */
        for ( ; cnt < (int16_t)(rtDat->numTimers - 1); cnt++ )
        {
            rtDat->refLst[cnt] = rtDat->refLst[cnt + 1];
        }

        rtDat->numTimers--;
    }

    (void)STATLST_RemoveEntry(&(rtDat->statLst), idx);

    /* test if removed timer had been started */
    if ( rtDat->status[idx] == RECTIMER_STARTED )
    {
        RECTIMER_SendStatusEvent(idx, RECTIMER_STATUS_REMOVED, 0, FFALSE);
    }

    rtDat->status[idx] = RECTIMER_UNUSED;

    rtDat->dirty = FTRUE;

    return FAPI_OK;
}

/* NOTE: This function is not thread-safe. It has to be externally protected */
FSTATIC int32_t RECTIMER_IntReplace (RECTIMER_IDX idx,
                                     const RECTIMER_ITEM_S* data)
{
    RECTIMER_ENTRY_S*   listEntry;
    RECTIMER_ENTRY_S    newEntry;
    int16_t             cnt;
    int16_t             newPos;
    int32_t             retVal = FAPI_OK;

    listEntry = STATLST_GetEntry (&(rtDat->statLst), idx);
    if ( listEntry == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    /* Replace timer data */
    newEntry.instant = FFALSE;
    newEntry.data    = *data;

    retVal = STATLST_SetEntry (&(rtDat->statLst), idx, &newEntry,
                               SYS_OWNER_USER);
    if ( retVal != FAPI_OK ) return retVal;

    /* Update reference list */
    for ( cnt = 0; cnt < rtDat->numTimers; cnt++ )
    {
        if ( rtDat->refLst[cnt] == idx ) break;
    }

    if ( cnt < rtDat->numTimers ) /* reference found */
    {
        /* Remove entry in reference list by shifting subsequent entries. */
        for ( ; cnt < (int16_t)(rtDat->numTimers - 1); cnt++ )
        {
            rtDat->refLst[cnt] = rtDat->refLst[cnt + 1];
        }

        rtDat->numTimers--;

        /* Re-insert reference in chronological order, keep operation status */
        (void)RECTIMER_InsertRefEntry (idx, rtDat->status[idx], &newPos);
    }

    rtDat->dirty = FTRUE;

    return retVal;
}

/* NOTE: This function is not thread-safe. It has to be externally protected */
FSTATIC int32_t RECTIMER_IntHandleExpiredTimer (RECTIMER_IDX idx)
{
    SYS_EVTHDR_S        ctrlEvent;
    RECTIMER_ENTRY_S*   listEntry;
    RECTIMER_ITEM_S     modItem;
    int32_t             daysToAdd = 0;
    int32_t             retVal = FAPI_OK;

    /* get timer data */
    listEntry = STATLST_GetEntry (&(rtDat->statLst), idx);
    if ( listEntry == NULL )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    switch ( listEntry->data.type )
    {
        default:
        case RECTIMER_TYPE_SINGLE:
            /* timer will be removed */
            break;

        case RECTIMER_TYPE_WEEKDAYS:
            switch ( listEntry->data.startDate.dayOfWeek )
            {
                case SYS_TIME_WEEKDAY_FRIDAY:   daysToAdd = 3; break;
                case SYS_TIME_WEEKDAY_SATURDAY: daysToAdd = 2; break;
                default:                        daysToAdd = 1; break;
            }
            break;

        case RECTIMER_TYPE_WEEKLY:
            daysToAdd = 7;
            break;

        case RECTIMER_TYPE_WEEKENDS:
            switch ( listEntry->data.startDate.dayOfWeek )
            {
                case SYS_TIME_WEEKDAY_SUNDAY:    daysToAdd = 6; break;
                case SYS_TIME_WEEKDAY_MONDAY:    daysToAdd = 5; break;
                case SYS_TIME_WEEKDAY_TUESDAY:   daysToAdd = 4; break;
                case SYS_TIME_WEEKDAY_WEDNESDAY: daysToAdd = 3; break;
                case SYS_TIME_WEEKDAY_THURSDAY:  daysToAdd = 2; break;

                default:
                case SYS_TIME_WEEKDAY_FRIDAY:
                case SYS_TIME_WEEKDAY_SATURDAY:  daysToAdd = 1; break;
            }
            break;

        case RECTIMER_TYPE_DAILY:
            daysToAdd = 1;
            break;
    }

    if ( daysToAdd == 0 )
    {
        /* remove timer */
        retVal = RECTIMER_IntRemove(idx);
    }
    else
    {
        /* update timer and perform re-insertion */
        modItem = listEntry->data;

        TIME_AddDays (&(modItem.startDate), daysToAdd);
        TIME_AddDays (&(modItem.endDate),   daysToAdd);

        /* re-insert timer*/
        retVal = RECTIMER_IntReplace (idx, &modItem);
    }

    if ( retVal == FAPI_OK )
    {
        ctrlEvent.type   = EVT_RECTIMER_LISTCHG;
        ctrlEvent.length = sizeof(ctrlEvent);

        (void)EVT_Send(&ctrlEvent, "Rectimer listchg");
    }

    return retVal;
}

FSTATIC void RECTIMER_SendStatusEvent (RECTIMER_IDX idx,
                                       RECTIMER_STATUS_E status,
                                       uint32_t seconds, fbool_t removeIt)
{
    SYS_EVT_RECTIMER_STATUS_S ctrlEvent;

    /* send "timer stop" event */
    ctrlEvent.hdr.type    = EVT_RECTIMER_STATUS;
    ctrlEvent.hdr.length  = sizeof(ctrlEvent);
    ctrlEvent.idx         = idx;
    ctrlEvent.status      = status;
    ctrlEvent.seconds     = seconds;
    ctrlEvent.removeIt    = removeIt;

    (void)EVT_Send(&ctrlEvent, "Rectimer status");
}

/* --------------------------------------- */

#if 0
void RECTIMER_DefaultsSet (RECTIMER_SETTINGS_S* pData)
{
    uint16_t cnt;

    memset(pData, 0, sizeof(RECTIMER_SETTINGS_S));

    for ( cnt = 0; cnt < RECTIMER_MAX; cnt++ )
    {
        pData->entries[cnt].id = RECTIMER_INVALID;
    }

    (void)SETTINGS_TimerDataChanged();
}
#endif

void RECTIMER_EnterStandby (void)
{
    if ( rtDat != NULL )
    {
        /* disable record/sleep timers */
        (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

        rtDat->sleepTimerEnabled = FFALSE;

        (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);
    }
}

/*lint -save  -e715 */
FSTATIC void RECTIMER_ThreadFunction(void* opt)
{
    while ( 1 )
    {
        RECTIMER_Update();
        FAPI_SYS_SLEEP(RECTIMER_CHECK_INTERVAL);
    }
}
/*lint -restore */

/* NOTE: This function is not thread-safe. It has to be externally protected */
FSTATIC fbool_t RECTIMER_InsertRefEntry (RECTIMER_IDX idx,
                                         RECTIMER_INTSTAT_E reqStatus,
                                         int16_t* newPos)
{
    RECTIMER_ENTRY_S* listEntry;
    RECTIMER_ENTRY_S* refEntry;
    int16_t           index;
    int16_t           shiftCnt;
    fbool_t           insert = FFALSE;

    DBG_Assert(rtDat->numTimers < RECTIMER_MAX);

    listEntry = STATLST_GetEntry (&(rtDat->statLst), idx);
    if ( listEntry == NULL ) return FFALSE;

    index = rtDat->numTimers - 1;

    while ( index >= -1 )
    {
        if ( index == -1 ) insert = FTRUE;
        else
        {
            refEntry = STATLST_GetEntry (&(rtDat->statLst),
                                         rtDat->refLst[index]);
            if ( refEntry != NULL )
            {
                /* test for time and date to keep chronological order */
                insert = TIME_IsLaterThan ( &(listEntry->data.startTime),
                                            &(listEntry->data.startDate),
                                            &(refEntry->data.startTime),
                                            &(refEntry->data.startDate) );
            }
        }

        if ( insert )
        {
            /* shift subsequent entries */
            for ( shiftCnt = rtDat->numTimers - 1; shiftCnt > index;
                  shiftCnt-- )
            {
                rtDat->refLst[shiftCnt + 1] = rtDat->refLst[shiftCnt];
            }

            *newPos = index + 1;

            rtDat->refLst[index + 1] = idx;
            rtDat->numTimers++;
            rtDat->status[idx] = reqStatus;
            return FTRUE;
        }

        index--;
    }

    return FFALSE;
}

FSTATIC void RECTIMER_Update (void)
{
    RECTIMER_ENTRY_S*   listEntry;
    RECTIMER_ITEM_S*    pTimer;
    RECTIMER_IDX        idx;
    SYS_TIME_S          wakeupTime;
    SYS_DATE_S          wakeupDate;
    SYS_EVTHDR_S        sleepTmrEvent;
    SYS_TIME_S          timeCurr;
    SYS_DATE_S          dateCurr;
    fbool_t             hasSynced;
    uint32_t            secsToStart;
    SYS_EVTHDR_S        ctrlEvent;
    fbool_t             removeTimer;

    if ( TIME_TimeAndDateGetUtc (&timeCurr, &dateCurr, &hasSynced) != FAPI_OK )
    {
        return;
    }

    if ( !hasSynced )
    {
        /* system has no time sync yet ask application for time */
        if ( ( SYS_GetOperationMode() == SYS_PWRMODE_UP )   &&
             ( rtDat->numTimers > 0 ) )
        {
            if( rtDat->timeSyncTimeout == 0 )
            {
                rtDat->timeSyncTimeout  = SYS_GET_TIMESTAMP_MS;
                rtDat->timeSyncTimeout += RECTIMER_TIMESYNC_TIMEOUT;
            }

            if (rtDat->timeSyncTimeout < SYS_GET_TIMESTAMP_MS)
            {
                ctrlEvent.type   = EVT_TIMESYNC_TIMEOUT;
                ctrlEvent.length = sizeof(ctrlEvent);

                (void)EVT_Send(&ctrlEvent, "MENU Time");
            }
        }
        return;
    }

    rtDat->timeSyncTimeout = 0;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    /* ----- Test for sleep timer expiration ------------------------------- */

    if ( rtDat->sleepTimerEnabled )
    {
        if ( TIME_IsLaterThan (&timeCurr, &dateCurr,
                               &(rtDat->sleepTime),
                               &(rtDat->sleepDate)) == FTRUE )
        {
            /* send notification */
            sleepTmrEvent.type   = EVT_SLEEPTIMER_EXPIRED;
            sleepTmrEvent.length = sizeof(sleepTmrEvent);

            (void)EVT_Send(&sleepTmrEvent, "Sleep timer exp.");

            rtDat->sleepTimerEnabled = FFALSE;
        }
    }

    /* ----- check all record timers --------------------------------------- */

    for ( idx = 0; idx < RECTIMER_MAX; idx++ )
    {
        listEntry = STATLST_GetEntry(&(rtDat->statLst), idx);
        if ( listEntry == NULL ) continue;

        pTimer = &(listEntry->data);

        /* ----------------------------------------------------------------- */

        switch ( rtDat->status[idx] )
        {
            /* ------------------------------------------------------------- */

            case RECTIMER_NOTSTARTED:
                if ( TIME_IsLaterThan (&timeCurr, &dateCurr,
                                       &(pTimer->endTime),
                                       &(pTimer->endDate)) == FTRUE )
                {
                    FAPI_SYS_PRINT_DEBUG(4,"update: idx %i -> expired\n", idx);

                    /* this timer has already expired */
                    (void)RECTIMER_IntHandleExpiredTimer(idx);
                }
                else if ( pTimer->enabled )
                {
                    /* is timer ready for activation? */
                    if ( TIME_IsLaterThan (&timeCurr, &dateCurr,
                                            &(pTimer->startTime),
                                            &(pTimer->startDate)) == FTRUE )
                    {
                        FAPI_SYS_PRINT_DEBUG(4,"update: idx %i -> start\n", idx);

                        /* mark timer as started */
                        rtDat->status[idx] = RECTIMER_STARTED;

                        /* send "timer start" event */
                        RECTIMER_SendStatusEvent(idx, RECTIMER_STATUS_START, 0,
                                                 FFALSE);
                    }
                    else
                    {
                        /* test for pre-expiration notification time */
                        wakeupTime = pTimer->startTime;
                        wakeupDate = pTimer->startDate;

                        TIME_AddSeconds (&wakeupTime, &wakeupDate,
                                         -RECTIMER_NOTIFY_TIME);

                        if ( TIME_IsLaterThan (&timeCurr, &dateCurr,
                                               &wakeupTime, &wakeupDate) )
                        {
                            secsToStart = TIME_GetLengthInSeconds(&timeCurr,
                                          &(pTimer->startTime));

                            /* send "timer starts soon" event */
                            RECTIMER_SendStatusEvent(idx, RECTIMER_STATUS_START,
                                                     secsToStart, FFALSE);

                            FAPI_SYS_PRINT_DEBUG(4,"update: idx %i -> pre-start: x seconds\n",
                                       idx, secsToStart);
                        }
                    }
                }
                break;

            /* ------------------------------------------------------------- */

            case RECTIMER_STARTED:
                /* is timer ready for deactivation? */
                if ( TIME_IsLaterThan (&timeCurr, &dateCurr,
                                       &(pTimer->endTime),
                                       &(pTimer->endDate)) == FTRUE )
                {
                    FAPI_SYS_PRINT_DEBUG(4,"update: idx %i -> stop\n", idx);

                    rtDat->status[idx] = RECTIMER_STOPPED;

                    if ( pTimer->type == RECTIMER_TYPE_SINGLE )
                    {
                        removeTimer = FTRUE;
                    }
                    else
                    {
                        (void)RECTIMER_IntHandleExpiredTimer(idx);

                        removeTimer = FFALSE;
                    }

                    RECTIMER_SendStatusEvent(idx, RECTIMER_STATUS_STOP, 0,
                                             removeTimer);
                }
                /* Has start time been moved in the future ? */
                else if ( TIME_IsLaterThan (&(pTimer->startTime),
                                            &(pTimer->startDate),
                                            &timeCurr, &dateCurr) )
                {
                    RECTIMER_SendStatusEvent(idx, RECTIMER_STATUS_STOP, 0,
                                             FFALSE);

                    rtDat->status[idx] = RECTIMER_NOTSTARTED;
                }
                else
                {
                    /* repeat start event periodically until timer is
                       deactivated. */
                    RECTIMER_SendStatusEvent(idx, RECTIMER_STATUS_START, 0,
                                             FFALSE);
                }
                break;

            /* ------------------------------------------------------------- */

            default:
                break;
        }
    }

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);
}

/*!
*******************************************************************************
**
** \brief  Set or disable Sleep Timer.
**
** The sleep timer can be used for powering down the system at a specified
** time and date. The system will generate an event of type
** #EVT_SLEEPTIMER_EXPIRED if it is not in standby by the time of expiration.
** The action involved with an expired sleep timer depends on the application.
**
** The sleep timer is cleared after expiration or if the box is put into
** standby mode.
**
** \param  pTime  sleep timer expiration time (NULL if timer is to be disabled)
** \param  pDate  sleep timer expiration date (NULL if timer is to be disabled)
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa TIME_GetSleepTimer
******************************************************************************/
int32_t RECTIMER_SetSleepTimer (const SYS_TIME_S* pTime,
                                const SYS_DATE_S* pDate)
{
    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    if ( (pTime == NULL) || (pDate == NULL) )
    {
        rtDat->sleepTimerEnabled = FFALSE;
    }
    else
    {
        rtDat->sleepTime         = *pTime;
        rtDat->sleepDate         = *pDate;
        rtDat->sleepTimerEnabled = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get the current Sleep Timer status.
**
** \param  enabled  Sleep timer status is returned here.
** \param  pTime    If enabled, expiration time is returned here.
** \param  pDate    If enabled, expiration date is returned here.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa TIME_SetSleepTimer
******************************************************************************/
int32_t RECTIMER_GetSleepTimer (fbool_t* enabled, SYS_TIME_S* pTime,
                                SYS_DATE_S* pDate)
{
    if ( rtDat == NULL ) return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(rtDat->mutexId, FAPEX_SYS_SUSPEND);

    *enabled = rtDat->sleepTimerEnabled;

    if ( rtDat->sleepTimerEnabled )
    {
        *pTime = rtDat->sleepTime;
        *pDate = rtDat->sleepDate;
    }

    (void)FAPEX_SYS_SetSemaphore(rtDat->mutexId, FAPEX_SYS_NO_SUSPEND);

    return FAPI_OK;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
