/*!
*****************************************************************************
**
** \file      ./applications/smartgo-hd/sys/src/screencapture.c
**
**            $Id: screencapture.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief     Functions for converting video and OSD into bitmap file.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
******************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
#define DEBUG 0
/*lint -save  */
#ifdef APPL_SCRNSHOT_ENABLE

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/

#include <smartgo_types.h>
#include <voc/voc.h>

#include "sys_api.h"
#include "utils.h"

#include "sys_priv.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#if (DEBUG > 0)
    #define SCRN_CAPTURE_DEBUG
#endif

#define CAPTURE_FILEPATH_SIZE  128
#define CAPTURE_FILENAME_SIZE  12

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
#ifdef SCRN_CAPTURE_DEBUG
FSTATIC uint64_t timer_start;
FSTATIC uint64_t timer_stamp;
#endif /* end of SCRN_CAPTURE_DEBUG */

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC void bmpWriteNotifyCb(void* arg);
/*!
*******************************************************************************
**
** \brief bmpWriteNotify Call back function .
**
** This function is called after video buffer is copied/captured and before
** writing captured data into BMP files
**
** \param  arg              argument for call back function
**
** \return void no return value
******************************************************************************/
/*lint -save -e715 */
FSTATIC void bmpWriteNotifyCb(void* arg)
{
    //VAL_Freeze (valOpenPtr, FAPI_VIDEC_FREEZE_OFF);
    (void) AV_FreezeVideo(FFALSE);
    (void) AV_Nop();
    #ifdef SCRN_CAPTURE_DEBUG
    timer_stamp = SYS_GET_TIMESTAMP_MS;
    FAPI_SYS_PRINT_DEBUG(1,"[ScrnCapture] video freezed for %lld msec\n",
                                                    (timer_stamp-timer_start));
    #endif /* end of SCRN_CAPTURE_DEBUG */
}
/*lint -restore */
/*!
*******************************************************************************
**
** \brief  Capture current video + OSD and store it as a bitmap file.
**
** By device and path parameters the destination path of the newly created
** file is passed. The file's name will be internally created, based on current
** time and date.
**
** \param  device           Target device for storage.
** \param  path             Target directory relative to device's root directory.
** \param  captureMode      Capture mode selection
** \param  colorSystem      color system selection
** \param  afterCaptureCb   Callback to give control to application after screen
**                          is captured
** \param  arg              argument for call back function
**
** \return One of the following status codes:
**         - #FAPI_OK                 if successful
**         - failure codes returned from #VOC_Open(), #VOC_Close()
******************************************************************************/
/*lint -save -e715 */
int32_t SYS_CaptureScreen (SYS_DEVICE_E device, const char_t* path,
                            SCREENCAPTURE_MODE_E captureMode,
                            SYS_SDTV_STANDARD colorSystem,
                            void (*afterCaptureCb)(void* arg),void* arg)
{
    char_t                          filePath[CAPTURE_FILEPATH_SIZE];
    uint32_t                        strPos = 0;
    uint16_t                        number = 1;
    int32_t                         retVal = FAPI_OK;
    SYS_DEV_STAT_S                  devStatus;
    SYS_DATE_S                      date;
    SYS_TIME_S                      time;
    F_FIND                          entry;
    AV_VIDEO_STREAMINFO_S           videoInfo;
    VOC_VidecInfoT                  videcInfo;
    VAL_VIDEC_DecFrameInfoStrT      frameInfo;
    FAPI_SYS_HandleT                vocInst = NULL;                 //VOC instance
    VOC_OpenParamsT                 vocPars;

    for (;;)
    {
        /* Initialize and open VOC */
        vocPars.functionMode = VOC_MODE_SCREENCAPTURE;
        retVal = VOC_Open(&vocPars, &vocInst);
        if( retVal !=  FAPI_OK) break;

        /*cEnter filesystem for current taskc*/
        (void) f_enterFS();

        retVal = SYS_DevicePreAccess(device);// wake up device
        if(retVal != FAPI_OK) break;

        strPos += (uint32_t)SYS_DeviceGetMountPath (device, filePath + strPos,
                                        CAPTURE_FILEPATH_SIZE - strPos);
        if ( strPos < 1 )
        {
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
            break;
        }

        /* Minimum space needed: filename plus two separators plus termination */
        if ( (strPos + CAPTURE_FILENAME_SIZE + 3) > CAPTURE_FILEPATH_SIZE )
        {
            retVal = SMARTGO_SYS_ERR_CAPACITY;
            break;
        }

        strPos += SYS_DeviceAddSeparator (filePath + strPos,
                                          CAPTURE_FILEPATH_SIZE - strPos);

        if ( (path != NULL) &&
             (path[0] != 0) )
        {
            strPos += UTILS_strncpy (filePath + strPos, path,
                                     CAPTURE_FILEPATH_SIZE - strPos);

            /* Minimum space needed: filename plus one separator plus termination */
            if ( (strPos + CAPTURE_FILENAME_SIZE + 2) > CAPTURE_FILEPATH_SIZE )
            {
                retVal = SMARTGO_SYS_ERR_CAPACITY;
                break;
            }
            /* make directory if path is not existing */
            if (f_getattr(filePath,NULL))
            {
                retVal = f_mkdir(filePath);

                FAPI_SYS_PRINT_DEBUG(4,"[ScrnCapture] mkdir at '%s' ret %d!\n", filePath,
                                                           retVal);
                if ( retVal != FAPI_OK )
                {
                    retVal = SMARTGO_SYS_ERR_INTERNAL;
                    break;
                }
            }

            strPos += SYS_DeviceAddSeparator (filePath + strPos, 2);
        }

        /* Space needed: filename plus termination */
        if ( (strPos + CAPTURE_FILENAME_SIZE + 1) > CAPTURE_FILEPATH_SIZE )
        {
            retVal = SMARTGO_SYS_ERR_CAPACITY;
            break;
        }

        (void)TIME_TimeAndDateGet(&time, &date, NULL);

        retVal = SYS_DeviceGetStatus(device, &devStatus);
        if ( retVal != FAPI_OK )
        {
            break;
        }

        if ( devStatus.mountStatus != SYS_DEV_FS_MOUNTED )
        {
            retVal = SMARTGO_SYS_ERR_PROHIBITED;
            break;
        }

        /* spin up HDD if necessary */
        retVal = SYS_DevicePreAccess (device);
        if ( retVal != FAPI_OK ) break;

        (void) SYS_DeviceSetAccessStatus(device, FTRUE);

        do
        {
            sprintf (filePath + strPos, "%02d%02d%02d%02d.BMP",
                date.year % 100, date.month, date.day, number);

            retVal = f_findfirst(filePath, &entry);

            #ifdef SCRN_CAPTURE_DEBUG
            if(retVal == F_NO_ERROR)
            {
                FAPI_SYS_PRINT_DEBUG(1,"[ScrnCapture] file %d: %s\n",
                                                number, entry.filename);
            }
            #endif /* SCRN_CAPTURE_DEBUG */

            number++;

        } while( (retVal == F_NO_ERROR) && (number < 100) );

        FAPI_SYS_PRINT_DEBUG(1,"[ScrnCapture] filePath=%s\n", filePath);


        /* freeze video */
        (void)AV_FreezeVideo(FTRUE);
        (void)AV_Nop();

        /* get video info for voc lib */
        (void) AV_GetVideoStreamInfo (&videoInfo);
        (void) AV_GetVideoFrameInfo (&frameInfo);

        #ifdef SCRN_CAPTURE_DEBUG
        FAPI_SYS_PRINT_DEBUG(1,
            "FBSizeType=%d, decodedWidth=%d, decodedHeight=%d, frameBank=%d, proggInter=%d\n",
                      videoInfo.decoder.decodeFrameBankSizeType,
                      videoInfo.decoder.decodedWidth,
                      videoInfo.decoder.decodedHeight,
                      frameInfo.frameBank,
                      videoInfo.stream.proggInter);
        #endif /* SCRN_CAPTURE_DEBUG */

        videcInfo.frameBank = frameInfo.frameBank;
        videcInfo.decodedHeight=videoInfo.decoder.decodedHeight;
        videcInfo.decodedWidth=videoInfo.decoder.decodedWidth;
        videcInfo.displayHeight = videoInfo.decoder.displayHeight;
        videcInfo.decodeFrameBankSizeType =
                                videoInfo.decoder.decodeFrameBankSizeType;
        videcInfo.proggInter = videoInfo.stream.proggInter;

        #ifdef SCRN_CAPTURE_DEBUG
        timer_start = SYS_GET_TIMESTAMP_MS;
        #endif /* SCRN_CAPTURE_DEBUG */

        /* call voc subsystem library to capture the screen */
        if(captureMode == SCREENCAPTURE_VIDEO_ONLY)
        {
            retVal = VOC_CaptureScreen(vocInst, filePath, VOC_CAPTURE_VIDEO_ONLY,
                                VOC_SDSYS_PAL,&videcInfo, bmpWriteNotifyCb, arg);
        }
        else if (captureMode == SCREENCAPTURE_VIDEO_OSD)
        {
            switch(colorSystem)
            {
                default:
                case SDSYS_PAL:
                case SDSYS_SECAM:
                case SDSYS_PAL_N:
                case SDSYS_PAL_N_ARG:
                    retVal = VOC_CaptureScreen(vocInst, filePath, VOC_CAPTURE_VIDEO_OSD,
                                VOC_SDSYS_PAL,&videcInfo, bmpWriteNotifyCb, arg);
                    break;

                case SDSYS_PAL_M:
                case SDSYS_NTSC:
                    retVal = VOC_CaptureScreen(vocInst, filePath, VOC_CAPTURE_VIDEO_OSD,
                                VOC_SDSYS_NTSC,&videcInfo, bmpWriteNotifyCb, arg);
                    break;

                case SDSYS_NUMS:
                    break;
            }
        }
        if (retVal != FAPI_OK) break;

        (void) SYS_DeviceSetAccessStatus(device, FFALSE);

        // close and exit VOC
        retVal = VOC_Close(vocInst);
        if( retVal != FAPI_OK) break;
        vocInst = NULL;

        break; /*end of for(;;)*/
    }

    if (retVal != FAPI_OK)
    {
        if(vocInst!=NULL)
        {
            (void)VOC_Close(vocInst);
        }

        (void)AV_FreezeVideo(FFALSE);

        FAPI_SYS_PRINT_DEBUG(1,"[ScrnCapture] error with %d\n", retVal);
    }

    return retVal;
}
/*lint -restore */

#endif /* APPL_SCRNSHOT_ENABLE */
/******************************************************************************/
/* END OF FILE */
