/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/settings.c
**
**              $Id: settings.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       User settings management functions.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save  */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/

#include "fapex/abst_types.h"
#include "fapex/abst_libviout.h"

#include <flashfs/flashfs.h>
#include <val/val.h>

#include "av.h"
#include "settings.h"
#include "sys_time.h"
#include "fcdata.h"
#include "utils.h"
#include "scart.h"

#ifdef APPL_RFMOD_ENABLED
// #include "rf_modulator.h"
#endif /* APPL_RFMOD_ENABLED */

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
typedef struct
{
    uint32_t                      id;               //!< struct identification
    uint32_t                      revision;         //!< struct revision
    SYS_PWRMODE_E                 powerUpMode;      //!< start-up power mode

    SYS_PICFORMAT_E               dispFormat;       //!< format of output display
    SYS_SCALEMETHOD_E             scaleMethod;      //!< method for 16:9 -> 4:3 conversion
    SYS_HDTV_STANDARD             hdTvOutSys;         //!< output TV standard
    SYS_SDTV_STANDARD             sdTvOutSys;         //!< output TV standard
    SYS_VIDEO_OUT                 scartVideoMode;   //!< SCART Video output: FBAS or RGB
    int16_t                       utcOffset;        //!< Signed UTC offset in minutes
    fbool_t                       dstIsActive;      //!< FTRUE: Daylight Saving Time is active
    fbool_t                       autoTime;         //!< FTRUE: Time and Dte is updated
                                                    //!< from stream.
    fbool_t                       autoTimeOffset;   //!< FTRUE: Time offset is updated
                                                    //!< from stream.
    SYS_CTRYCODE                  autoTimeOffsetCtry;   //!< Country setting for
                                                        //!< auto-timeoffset mode.
    uint16_t                      displayTime;      //!< common display time in seconds
    SYS_SPDIF_MODE_E              spdifMode;        //!< SPDIF output mode
    SYS_LANGCODE                  osdLanguage;      //!< OSD Language
    SYS_VIDSTART_MODE_E           servStartMode;    //!< Video start mode
    fbool_t                       serviceSwitchFreeze; //!< blank/freeze on service change
    SYS_LANGCODE                  audLangPref[SETTINGS_AUDIOLANGPREF_MAX];
    SYS_LANGCODE                  uiLangPref[SETTINGS_UILANGPREF_MAX];
    SYS_LANGCODE                  subtLangPref[SETTINGS_SUBTLANGPREF_MAX];
    fbool_t                       antennaPwr[RSRC_FE_MAX]; //!< Antenna power setting for each FE
    fbool_t                       playWelcomeJingle;   //!< Welcome jingle enabling status.
    fbool_t                       usbAutoReconnect;  //!< Regular USB bus checks enabled

    RFMOD_EXCHG_PARAMS_S          rfModPars;         //!< RF modulator settings.
    SCREENCAPTURE_MODE_E          screenShotMode;    //!< Screen shot mode.
    SYS_DEVICE_E                  screenShotTo;      //!< Screen shot to.
    AV_LIPSYNC_MODE_E             lipSyncMode;       //!< Lip Sync mode
    int32_t                       lipSyncDelay;      //!< Lip Sync delay level based on audio

    /* general-purpose user data */
    uint32_t                      userData[SETTINGS_USERDAT_MAX];

    uint32_t                      idEnd;        //!< struct identification

    ETH_IP_S                      ethIp;                //!< Ethernet IP settings

} STARTUP_PARAMS_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC STARTUP_PARAMS_S      *startupDat = NULL;
FSTATIC fbool_t                startupDatDirty = FFALSE;

#ifdef APPL_RFMOD_ENABLED
FSTATIC fbool_t                rfmodValid      = FFALSE;
#endif

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC int32_t               SETTINGS_StartupLoad(STARTUP_PARAMS_S * pars);
FSTATIC int32_t               SETTINGS_StartupSave(STARTUP_PARAMS_S * pars);

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/

int32_t SETTINGS_StartupInit(void)
{
    int32_t retVal;

    DBG_Assert(startupDat == NULL);

    /* Test if maximum flash file size is sufficient. */
    retVal = FLASH_CheckFileSize (FILEID_STARTUPPARAMS,
                                  sizeof(STARTUP_PARAMS_S));

    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    startupDat = UTILS_Malloc(sizeof(STARTUP_PARAMS_S));
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    #ifdef APPL_RFMOD_ENABLED
    rfmodValid = FFALSE;
    #endif
    retVal = SETTINGS_StartupLoad(startupDat);
    if(retVal != FAPI_OK)
    {
        FAPI_SYS_PRINT_DEBUG(1,"[SYS] No startup data available, start reformatting...\n");
        SETTINGS_StartupDefaultsSet();
    }
    else
    {
        #ifdef APPL_RFMOD_ENABLED
        rfmodValid = FTRUE;
        #endif
    }

    return FAPI_OK;
}

void SETTINGS_StartupDefaultsSet(void)
{
    uint32_t                      cnt;

    if(startupDat == NULL)
        return;

    memset(startupDat, 0, sizeof(STARTUP_PARAMS_S));

    #ifdef APPL_RFMOD_ENABLED
    rfmodValid = FFALSE;
    #endif
    startupDat->id             = FILEID_STARTUPPARAMS;
    startupDat->idEnd          = FILEID_STARTUPPARAMS;
    startupDat->revision       = FILEREV_STARTUPPARAMS;
    startupDat->powerUpMode    = PWR_INITIAL_STARTUPMODE;
    startupDat->dispFormat     = PICFORMAT_16_9;
    startupDat->scaleMethod    = SCALEMETHOD_PILLAR_LETTERBOX;
    startupDat->hdTvOutSys     = HDSYS_1080I;
    startupDat->sdTvOutSys     = SDSYS_PAL;
    startupDat->scartVideoMode = VIDEOOUT_RGB;

    startupDat->utcOffset             = 60;
    startupDat->dstIsActive           = FTRUE;
    startupDat->autoTime              = FTRUE;
    startupDat->autoTimeOffset        = FFALSE;
    startupDat->autoTimeOffsetCtry    = CTRY_NOTSPECIFIED;
    startupDat->displayTime           = 5;
    startupDat->spdifMode             = SPDIFMODE_PCMONLY;
    startupDat->lipSyncMode           = AV_LIPSYNC_MODE_AUTO;
    startupDat->lipSyncDelay          = 0;
    startupDat->osdLanguage           = LANG_ENGLISH;
    startupDat->servStartMode         = VIDEOSTART_FAST;
    startupDat->serviceSwitchFreeze   = FFALSE;
    startupDat->playWelcomeJingle     = FTRUE;
    startupDat->usbAutoReconnect      = FTRUE;
    startupDat->screenShotMode        = SCREENCAPTURE_VIDEO_ONLY;
    startupDat->screenShotTo          = SYS_DEV_HDD;
    startupDat->ethIp.useDhcp         = FTRUE;
    startupDat->ethIp.autoDns         = FTRUE;

    for(cnt = 0; cnt < SETTINGS_AUDIOLANGPREF_MAX; cnt++)
    {
        startupDat->audLangPref[cnt] = LANG_NOTSPECIFIED;
    }

    for ( cnt = 0; cnt < SETTINGS_UILANGPREF_MAX; cnt++ )
    {
        startupDat->uiLangPref[cnt] = LANG_NOTSPECIFIED;
    }

    for ( cnt = 0; cnt < SETTINGS_SUBTLANGPREF_MAX; cnt++ )
    {
        startupDat->subtLangPref[cnt] = LANG_NOTSPECIFIED;
    }

    for ( cnt = 0; cnt < RSRC_FE_MAX; cnt++ )
    {
        startupDat->antennaPwr[cnt] = FFALSE;
    }

    for ( cnt = 0; cnt < SETTINGS_USERDAT_MAX; cnt++ )
    {
        startupDat->userData[cnt] = 0;
    }

    if ( SYS_GetConfig()->setStartupDefaults != NULL )
    {
        SYS_GetConfig()->setStartupDefaults();
    }

    startupDatDirty = FTRUE;
}

FSTATIC int32_t SETTINGS_StartupLoad(STARTUP_PARAMS_S * pars)
{
    int32_t             retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
    FLASHFS_HANDLE      file = NULL;
    STARTUP_PARAMS_S    temp;

    do
    {
        file = FLASHFS_Open(FILEID_STARTUPPARAMS,FLASHFS_OPEN_READ,NULL);
        if(file != NULL)
        {
            /* ----- read file header --------------------------------------- */
            if(FLASHFS_Read(file,sizeof(STARTUP_PARAMS_S), &temp, NULL) !=
                (int32_t)sizeof(STARTUP_PARAMS_S))
            {
                break;
            }

            /* Check ID and revision for data verification */
            if((temp.id != FILEID_STARTUPPARAMS) ||
               (temp.idEnd != FILEID_STARTUPPARAMS) ||
               (temp.revision != FILEREV_STARTUPPARAMS))
            {
                retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
                break;
            } else {
                *pars = temp;
            }
            retVal = FAPI_OK;
        }
    }
    while(0);
    (void)FLASHFS_Close(file);

    if(retVal == FAPI_OK)
        startupDatDirty = FFALSE;

    return retVal;
}

FSTATIC int32_t SETTINGS_StartupSave(STARTUP_PARAMS_S * pars)
{
    FLASHFS_HANDLE      file = NULL;
    int32_t             retVal = FAPI_OK;

    file = FLASHFS_Open (FILEID_STARTUPPARAMS, FLASHFS_OPEN_WRITE, &retVal);
    if(file != NULL)
    {
        if(FLASHFS_Write(file, sizeof(STARTUP_PARAMS_S), pars, NULL) !=
            (int32_t)sizeof(STARTUP_PARAMS_S))
        {
            retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
        } else {
            startupDatDirty = FFALSE;
        }
    }
    (void)FLASHFS_Close(file);

    /* for testing */
    (void)SETTINGS_StartupLoad(pars);
    return retVal;
}

/* ------------------------------------------------------------------------- */

int32_t SETTINGS_Init(void)
{
    DBG_Assert(startupDat != NULL);

    /* initialise data */

    /* ---------------------------------------------------------------------- */

    /* do some initialisations according to settings */
    // FIXME: move this to AV_Init: ask for aspect ratio and set accordingly!
    //SETTINGS_AspectRatioSet();

    #ifndef APPL_RFMOD_ENABLED
    /* seems the only way to get rid of MISRA warning */
    startupDat->rfModPars.rfmod_index = 0; // no mudulator in use
    #endif
    return FAPI_OK;
}

fbool_t SETTINGS_SavingIsNecessary(void)
{
    if ( startupDat == NULL ) return FFALSE;

    //return ( (startupDatDirty == FTRUE) ||
    //         (setDatDirty     == FTRUE) );

    return startupDatDirty;
}

/*!
*******************************************************************************
**
** \brief  Store user settings in Flash.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t SETTINGS_Save(void)
{
    int32_t                       retVal = FAPI_OK;

    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    /* test if startup data has to be saved */
    if ( startupDatDirty == FTRUE )
    {
        retVal = SETTINGS_StartupSave(startupDat);
        if(retVal != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(2,"Saving of startup data failed!\n");
        }
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Get application-specific user data.
**
** This data is part of the SETTINGS-structure and is stored during
** SETTINGS_Save.
**
** \param  index  Requested index of user data langword
** \param  data   user data longword is returned here
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t SETTINGS_UserDatGet(uint8_t index, uint32_t * data)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if(index >= SETTINGS_USERDAT_MAX)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    else
    {
        *data = startupDat->userData[index];
        return FAPI_OK;
    }
}

/*!
*******************************************************************************
**
** \brief  Set application-specific user data.
**
** This data is part of the SETTINGS-structure and is stored during
** SETTINGS_Save.
**
** \param  index  Requested index to be set
** \param  data   data to be stored under requested index
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t SETTINGS_UserDatSet(uint8_t index, uint32_t data)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if(index >= SETTINGS_USERDAT_MAX)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if(startupDat->userData[index] != data)
    {
        startupDat->userData[index] = data;
        startupDatDirty             = FTRUE;
    }
    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Enable or disable automatic time mode.
**
** If enabled, time and date will be automatically adjusted as soon as
** the next TOT section is received.
**
** \param enable       FTRUE: enable, FFALSE: disable auto mode.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_AutoTimeGet
******************************************************************************/
int32_t SETTINGS_AutoTimeSet(fbool_t enable)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if(startupDat->autoTime != enable)
    {
        startupDat->autoTime = enable;
        startupDatDirty      = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get current setting of automatic time and date mode.
**
** \param enabled      Current enable status is returned here.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_AutoTimeSet
******************************************************************************/
int32_t SETTINGS_AutoTimeGet(fbool_t* enabled)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if(enabled != NULL)
        (*enabled) = startupDat->autoTime;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Enable or disable automatic time offset mode.
**
** If enabled, time zone setting will be automatically adjusted as soon as
** the next TOT section is received.
**
** \param enable       FTRUE: enable, FFALSE: disable auto mode.
** \param country      Reference country, only needed if time offset is enabled.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_AutoTimeOffsetGet
******************************************************************************/
int32_t SETTINGS_AutoTimeOffsetSet(fbool_t enable, SYS_CTRYCODE country)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if ( (startupDat->autoTimeOffset     != enable) ||
         (startupDat->autoTimeOffsetCtry != country) )
    {
        startupDat->autoTimeOffset     = enable;
        startupDat->autoTimeOffsetCtry = country;
        startupDatDirty                = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get current setting of automatic time offset mode.
**
** \param enabled      Current enable status is returned here.
** \param country      Current reference country is returned here.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_AutoTimeOffsetSet
******************************************************************************/
int32_t SETTINGS_AutoTimeOffsetGet(fbool_t* enabled, SYS_CTRYCODE * country)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if(enabled != NULL)
        (*enabled) = startupDat->autoTimeOffset;
    if(country != NULL)
        (*country) = startupDat->autoTimeOffsetCtry;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get timezone settings.
**
** Returns current setting of UTC offset and status of Daylight Saving Time.
**
** \note If auto-timeoffset mode is enabled, dstIsActive will always be
**       indicated as FFALSE, because TOT information from stream only contains
**       an offset value where current DST setting has been calculated into.
**
** \param utcOffset    current UTC offset (signed) in minutes is returned here.
** \param dstIsActive  DST status: FTRUE: active, else inactive.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_TimeOffsetSet
******************************************************************************/
int32_t SETTINGS_TimeOffsetGet(int16_t * utcOffset, fbool_t * dstIsActive)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if(utcOffset != NULL)
        (*utcOffset) = startupDat->utcOffset;
    if(dstIsActive != NULL)
        (*dstIsActive) = startupDat->dstIsActive;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set UTC offset and status of Daylight Saving Time (DST).
**
** \param utcOffset    Signed UTC offset in minutes.
** \param dstIsActive  DST status: FTRUE: active, else inactive.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_TimeOffsetGet
******************************************************************************/
int32_t SETTINGS_TimeOffsetSet(int16_t utcOffset, fbool_t dstIsActive)
{
    fbool_t valueChanged = FFALSE;

    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if((utcOffset < SETTINGS_UTC_OFFSET_MIN) ||
       (utcOffset > SETTINGS_UTC_OFFSET_MAX))
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if ( (startupDat->utcOffset   != utcOffset) ||
         (startupDat->dstIsActive != dstIsActive) )
    {
        startupDat->utcOffset   = utcOffset;
        startupDat->dstIsActive = dstIsActive;
        startupDatDirty         = FTRUE;
        valueChanged            = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    if(valueChanged)
    {
        /* Force sending of new time event */
        (void)TIME_ForceEvtUpdate();
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get the curent Ddisplay Time setting.
**
** "Display time" means the amount of time a non-permanent OSD item like a
** message or the info box remains on the screen.
**
** \return Current display time setting in seconds.
**
** \sa SETTINGS_DisplayTimeSet
******************************************************************************/
int32_t SETTINGS_DisplayTimeGet(uint16_t * timeInSeconds)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    *timeInSeconds = startupDat->displayTime;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get Display Time.
**
** "Display time" means the amount of time a non-permanent OSD item like a
** message or the info box remains on the screen.
**
** \param  timeInSeconds  Requested display time in seconds.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_DisplayTimeGet
******************************************************************************/
int32_t SETTINGS_DisplayTimeSet(uint16_t timeInSeconds)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if((timeInSeconds < SETTINGS_DISPTIME_MIN) ||
       (timeInSeconds > SETTINGS_DISPTIME_MAX))
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if ( startupDat->displayTime != timeInSeconds )
    {
        startupDat->displayTime = timeInSeconds;
        startupDatDirty         = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get the current display format settings.
**
** \param  format      output picture format (4:3 or anamorph 16:9)
** \param  scaleMethod scaleMethod for 16:9 pictures
**
** \return -
**
** \sa SETTINGS_DispFormatSet
******************************************************************************/
int32_t SETTINGS_DispFormatGet(SYS_PICFORMAT_E*   format,
                               SYS_SCALEMETHOD_E* scaleMethod)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if ( format      != NULL ) (*format)       = startupDat->dispFormat;
    if ( scaleMethod != NULL ) (*scaleMethod ) = startupDat->scaleMethod;

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set display format.
**
** \param  format      output picture format (4:3 or anamorph 16:9)
** \param  scaleMethod scaleMethod for 16:9 pictures
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_DispFormatGet
******************************************************************************/
int32_t SETTINGS_DispFormatSet(SYS_PICFORMAT_E   format,
                               SYS_SCALEMETHOD_E scaleMethod)
{
    fbool_t setFormat = FFALSE;

    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if ( (startupDat->dispFormat  != format) ||
         (startupDat->scaleMethod != scaleMethod) )
    {
        startupDat->dispFormat   = format;
        startupDat->scaleMethod  = scaleMethod;
        startupDatDirty          = FTRUE;

        (void)SETTINGS_AspectRatioSet();
        (void)SETTINGS_ScaleMethodSet();
        setFormat = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    if(setFormat)
    {
        AV_OutputFormatUpdate (FFALSE);
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get the current Lip Sync mode.
**
** \return Current Lip Sync mode.
**
** \sa SETTINGS_LipSyncModeSet
******************************************************************************/
int32_t SETTINGS_LipSyncModeGet(AV_LIPSYNC_MODE_E * lipSyncMode)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    *lipSyncMode = startupDat->lipSyncMode;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set the current Lip Sync mode.
**
** \param  lipSyncMode  Requested lip sync mode
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_LipSyncModeGet
******************************************************************************/
int32_t SETTINGS_LipSyncModeSet(AV_LIPSYNC_MODE_E lipSyncMode)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if ( startupDat->lipSyncMode != lipSyncMode )
    {
        startupDat->lipSyncMode = lipSyncMode;
        startupDatDirty         = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    if ( (startupDatDirty == FTRUE) && (lipSyncMode == AV_LIPSYNC_MODE_AUTO) )
    {
        (void)AV_HDMIAutoSync();
    }
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get the current Lip Sync mode.
**
** \return Current Lip Sync mode.
**
** \sa SETTINGS_LipSyncModeSet
******************************************************************************/
int32_t SETTINGS_LipSyncDelayGet(int32_t * lipSyncDelay)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    *lipSyncDelay = startupDat->lipSyncDelay;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set the current Lip Sync mode.
**
** \param  lipSyncDelay  Requested lip sync mode
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_LipSyncModeGet
******************************************************************************/
int32_t SETTINGS_LipSyncDelaySet(int32_t lipSyncDelay)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if ( startupDat->lipSyncDelay != lipSyncDelay )
    {
        startupDat->lipSyncDelay = lipSyncDelay;
        startupDatDirty            = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    (void)AV_HDMISetLipSyncDelay(lipSyncDelay);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get the current SCART video mode.
**
** \return Current SCART video mode.
**
** \sa SETTINGS_VideoOutputSet
******************************************************************************/
int32_t SETTINGS_VideoOutputGet(SYS_VIDEO_OUT * videoout)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    *videoout = startupDat->scartVideoMode;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set the current SCART video output mode.
**
** \param  videoOut  Requested video output mode
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_VideoOutputGet
******************************************************************************/
int32_t SETTINGS_VideoOutputSet(SYS_VIDEO_OUT videoOut)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if ( startupDat->scartVideoMode != videoOut )
    {
        startupDat->scartVideoMode = videoOut;
        startupDatDirty            = FTRUE;
    }

    /* set requested video output */
    if ( SYS_GetOperationMode() == SYS_PWRMODE_UP )
    {
        AV_SetVideoOutput (startupDat->scartVideoMode);
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    return FAPI_OK;
}


/*!
*******************************************************************************
**
** \brief  Get current output TV standard.
**
** \return Current output TV standard.
**
** \sa SETTINGS_OutputTvStandardSet
******************************************************************************/
int32_t SETTINGS_OutputTvStandardGet(SYS_HDTV_STANDARD *hdSys,
                                     SYS_SDTV_STANDARD *sdSys)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if(hdSys != NULL) *hdSys = startupDat->hdTvOutSys;
    if(sdSys != NULL) *sdSys = startupDat->sdTvOutSys;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set current output TV standard.
**
** \param  hdSys  requested output TV standard for HD path.
** \param  sdSys  requested output TV standard for SD path.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_OutputTvStandardGet
******************************************************************************/
int32_t SETTINGS_OutputTvStandardSet(SYS_HDTV_STANDARD hdSys,
                                     SYS_SDTV_STANDARD sdSys)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if(startupDat->hdTvOutSys != hdSys)
    {
        startupDat->hdTvOutSys = hdSys;
        startupDatDirty = FTRUE;
    }
    if(startupDat->sdTvOutSys != sdSys)
    {
        startupDat->sdTvOutSys = sdSys;
        startupDatDirty = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    // TODO: to be tested!
    (void)AV_SetTvOutputSystem (startupDat->hdTvOutSys,startupDat->sdTvOutSys);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set aspect ratio after SETTINGS_DispFormatSet
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_DispFormatSet
******************************************************************************/
int32_t SETTINGS_AspectRatioSet(void)
{
    int32_t                          retVal;
    FAPEX_VISCALE_AspectRatioEnumT    newHdAspRatio;
    FAPEX_VISCALE_AspectRatioEnumT    newSdAspRatio;

    if ( startupDat->dispFormat == PICFORMAT_16_9 )
    {
        newHdAspRatio = FAPEX_VIOUT_DISPLAY_AR_16_9;
    }
    else /* Display is 4:3 */
    {
        newHdAspRatio = FAPEX_VIOUT_DISPLAY_AR_4_3;
    }

    // SD frame should always cover HD active area (video + OSD) and therefore
    // aspect ratio of SD scaler (confusing, it's more like a kind of active
    // frame info) should set to NONE and follows the HD frame size
    newSdAspRatio = FAPEX_VIOUT_DISPLAY_AR_NONE;

    retVal = AVINT_VideoSetAspectRatio (VAL_VOUT_SD, newSdAspRatio);

    if ( retVal == FAPI_OK )
    {
        retVal = AVINT_VideoSetAspectRatio (VAL_VOUT_HD, newHdAspRatio);
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Set scaling method after SETTINGS_DispFormatSet
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_DispFormatSet
******************************************************************************/
int32_t SETTINGS_ScaleMethodSet(void)
{
    FAPEX_VISCALE_ScalingMethodEnumT  newScaleMethod;

    switch(startupDat->scaleMethod)
    {
        default:
        case SCALEMETHOD_NUMS:
        case SCALEMETHOD_PILLAR_LETTERBOX:
            newScaleMethod = FAPEX_VISCALE_SM_PLBOX;
            break;
        case SCALEMETHOD_FULLSCREEN:
            newScaleMethod = FAPEX_VISCALE_SM_FSZOOM;
            break;
        case SCALEMETHOD_CCO:
            newScaleMethod = FAPEX_VISCALE_SM_FSCCO;
            break;
        case SCALEMETHOD_TV_SCALES:
            newScaleMethod = FAPEX_VISCALE_SM_LTVS;
            break;

        case SCALEMETHOD_NONE:
            newScaleMethod = FAPEX_VISCALE_SM_NONE;
            break;
    }

    return AVINT_VideoSetScaleMethod(newScaleMethod);
}

/*!
*******************************************************************************
**
** \brief  Get current OSD language.
**
** \return Current OSD language setting.
**
** \sa SETTINGS_OsdLangSet
******************************************************************************/
int32_t SETTINGS_OsdLangGet(SYS_LANGCODE * lang)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    *lang = startupDat->osdLanguage;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set OSD language
**
** \attention This modifies only the language setting. The loading of language
**            strings itself is application-specific.
**
** \param  lang  requested language
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_OsdLangGet
******************************************************************************/
int32_t SETTINGS_OsdLangSet(SYS_LANGCODE lang)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if ( startupDat->osdLanguage != lang )
    {
        startupDat->osdLanguage = lang;
        startupDatDirty         = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set SPDIF output mode preference.
**
** There are two possibilities:
**     - Restrict output to PCM data of current audio stream decoded for
**       I2S output.
**     - If a compliant stream is available, send it as coded bitstream in
**       passthrough mode to be decoded by an external device. If not, output
**       PCM samples of current I2S audio stream.
**
** \param  spdifMode  Requested SPIDF output mode
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_DefaultSPDIFModeGet
******************************************************************************/
int32_t SETTINGS_DefaultSPDIFModeSet (SYS_SPDIF_MODE_E spdifMode)
{
    int32_t retVal;

    if ( startupDat == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

        if ( startupDat->spdifMode != spdifMode )
        {
            startupDat->spdifMode = spdifMode;
            startupDatDirty       = FTRUE;

            /* modify current SPDIF output */
            switch ( spdifMode )
            {
                default:
                case SPDIFMODE_PCMONLY:
                    /* ensure that running passthrough stream is stopped */
                    (void)AV_SetSPDIFStatus(SPDIFSTATUS_CURRPCM, -1);
                    break;

                case SPDIFMODE_PREFERPASSTHROUGH:
                    /* if a stream is available for passthrough, this will
                       select the first one */
                    (void)AV_SetSPDIFStatus(SPDIFSTATUS_PASSTHROUGH, -1);
                    break;
            }
        }

        (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

        retVal = FAPI_OK;
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Get SPDIF output mode preference.
**
** \param  spdifMode  Pointer to location where SPDIF mode will be copied to.
**
** \return Current SPDIF output mode setting.
**
** \sa SETTINGS_DefaultSPDIFModeSet
******************************************************************************/
int32_t SETTINGS_DefaultSPDIFModeGet (SYS_SPDIF_MODE_E* spdifMode)
{
    int32_t retVal;

    if ( startupDat == NULL )
    {
        retVal = APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else
    {
        if ( spdifMode != NULL )
        {
            *spdifMode = startupDat->spdifMode;
        }

        retVal = FAPI_OK;
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Set an entry in audio language preference list.
**
** \param  prefIdx  Index in preference list (0: most preferred).
** \param  lang     Language code to be placed in list.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_AudioLangPrefGet
******************************************************************************/
int32_t SETTINGS_AudioLangPrefSet(uint8_t prefIdx, SYS_LANGCODE lang)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if(prefIdx >= SETTINGS_AUDIOLANGPREF_MAX)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    startupDat->audLangPref[prefIdx] = lang;
    startupDatDirty                  = FTRUE;

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get current audio language preference.
**
** \param  prefIdx  Index in preference list (0: most preferred).
** \param  lang     Pointer to location where language code will be copied to.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_AudioLangPrefSet
******************************************************************************/
int32_t SETTINGS_AudioLangPrefGet(uint8_t prefIdx, SYS_LANGCODE * lang)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if(prefIdx >= SETTINGS_AUDIOLANGPREF_MAX)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    *lang = startupDat->audLangPref[prefIdx];
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set an entry in Subtitle language preference list.
**
** \param  prefIdx  Index in preference list (0: most preferred).
** \param  lang     Language code to be placed in list.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_SubtLangPrefGet
******************************************************************************/
int32_t SETTINGS_SubtLangPrefSet(uint8_t prefIdx, SYS_LANGCODE lang)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if(prefIdx >= SETTINGS_SUBTLANGPREF_MAX)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    startupDat->subtLangPref[prefIdx] = lang;
    startupDatDirty                   = FTRUE;

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get current Subtitle language preference.
**
** \param  prefIdx  Index in preference list (0: most preferred).
** \param  lang     Pointer to location where language code will be copied to.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_SubtLangPrefSet
******************************************************************************/
int32_t SETTINGS_SubtLangPrefGet(uint8_t prefIdx, SYS_LANGCODE * lang)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if(prefIdx >= SETTINGS_SUBTLANGPREF_MAX)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    *lang = startupDat->subtLangPref[prefIdx];
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set an entry in user interface language preference list.
**
** \param  prefIdx  Index in preference list (0: most preferred).
** \param  lang     Language code to be placed in list.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_UILangPrefGet
******************************************************************************/
int32_t SETTINGS_UILangPrefSet(uint8_t prefIdx, SYS_LANGCODE lang)
{
    fbool_t valueChanged = FFALSE;
    int32_t retVal;

    if(startupDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(prefIdx >= SETTINGS_UILANGPREF_MAX)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if ( startupDat->uiLangPref[prefIdx] != lang )
    {
        startupDat->uiLangPref[prefIdx] = lang;
        startupDatDirty                 = FTRUE;
        valueChanged                    = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    if(valueChanged)
    {
        /* Ensure database consistency with user interface preference. */
        retVal = EIT_ClearDatabase();
        if(retVal != FAPI_OK)
        {
            /* ignore return value because while initialisation the data base
             * is return APPL_SMARTGO_ERR_NOT_INITIALIZED!
             */
        }
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get current user interface language preference.
**
** \param  prefIdx  Index in preference list (0: most preferred).
** \param  lang     Pointer to location where language code will be copied to.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_UILangPrefSet
******************************************************************************/
int32_t SETTINGS_UILangPrefGet(uint8_t prefIdx, SYS_LANGCODE * lang)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if(prefIdx >= SETTINGS_UILANGPREF_MAX)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    *lang = startupDat->uiLangPref[prefIdx];
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set initial operation mode at power-up.
**
** This specifies if the system starts in standby mode or in active mode after
** powering up. This has no effect on the transition between active and standby
** mode during operation.
**
** \param  reqMode  Requested power-up mode, only SYS_PWRMODE_DOWN and
**                  SYS_PWRMODE_UP is accepted.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_GetPowerUpMode
******************************************************************************/
int32_t SETTINGS_SetPowerUpMode(SYS_PWRMODE_E reqMode)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if(startupDat->powerUpMode != reqMode)
    {
        startupDat->powerUpMode = reqMode;
        startupDatDirty         = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    return FAPI_OK;
}

int32_t SETTINGS_GetPowerUpMode(SYS_PWRMODE_E * powerupMode)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    *powerupMode = startupDat->powerUpMode;
    return FAPI_OK;
}

int32_t SETTINGS_SetServiceStartMode(SYS_VIDSTART_MODE_E reqMode)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if(startupDat->servStartMode != reqMode)
    {
        startupDat->servStartMode = reqMode;
        startupDatDirty           = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    return FAPI_OK;
}

int32_t SETTINGS_GetServiceStartMode(SYS_VIDSTART_MODE_E * startMode)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    *startMode = startupDat->servStartMode;
    return FAPI_OK;
}

int32_t SETTINGS_SetServiceSwitchMode(fbool_t freeze)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if(startupDat->serviceSwitchFreeze != freeze)
    {
        startupDat->serviceSwitchFreeze = freeze;
        startupDatDirty                 = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    return FAPI_OK;
}

int32_t SETTINGS_GetServiceSwitchMode(fbool_t* freeze)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    *freeze = startupDat->serviceSwitchFreeze;
    return FAPI_OK;
}

int32_t SETTINGS_AntennaPowerGet(int8_t feIdx, fbool_t* powerOn)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if((feIdx < 0) || (feIdx >= RSRC_FE_MAX))
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    *powerOn = startupDat->antennaPwr[feIdx];
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set antenna power supply status.
**
** \attention Currently this function does not return an error when the
**  frontend does not support this feature.
**
** \param  feIdx   Requested frontend.
** \param  powerOn Power supply status: FTRUE: On, FFALSE: Off
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_AntennaPowerGet
******************************************************************************/
int32_t SETTINGS_AntennaPowerSet(int8_t feIdx, fbool_t powerOn)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if((feIdx < 0) || (feIdx >= RSRC_FE_MAX))
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if ( startupDat->antennaPwr[feIdx] != powerOn )
    {
        startupDat->antennaPwr[feIdx] = powerOn;
        startupDatDirty               = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    (void)AV_SetAntennaPower(feIdx, powerOn);

    return FAPI_OK;
}

#ifdef APPL_RFMOD_ENABLED

int32_t SETTINGS_SetRfModParams(const RFMOD_EXCHG_PARAMS_S * params)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if(params == NULL)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    startupDat->rfModPars = *params;
    startupDatDirty       = FTRUE;
    rfmodValid            = FTRUE;

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);
    return FAPI_OK;
}

int32_t SETTINGS_GetRfModParams(RFMOD_EXCHG_PARAMS_S * params)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if(params == NULL)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    if(!(rfmodValid))
        return SMARTGO_SYS_ERR_DATA_NOT_AVL;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    *params = startupDat->rfModPars;

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);
    return FAPI_OK;
}

#endif /* APPL_RFMOD_ENABLED */

/*!
*******************************************************************************
**
** \brief  Get play jingle flag status.
**
** Returns current setting of a flag to play a welcome jingle.
**
** \param jingleEnabled    pointer to boolean flag
**
** \return -
**
** \sa SETTINGS_PlayJingleSet
******************************************************************************/
int32_t SETTINGS_PlayJingleGet(fbool_t* jingleEnabled)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    *jingleEnabled = startupDat->playWelcomeJingle;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set play jingle flag status.
**
** set a flag to play a welcome jingle.
**
** \param jingleEnabled    boolean flag
**
** \return -
**
** \sa SETTINGS_PlayJingleGet
******************************************************************************/
int32_t SETTINGS_PlayJingleSet(fbool_t jingleEnabled)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if(startupDat->playWelcomeJingle != jingleEnabled)
    {
        startupDat->playWelcomeJingle = jingleEnabled;
        startupDatDirty         = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get USB auto-reconnect flag.
**
** Returns current setting of the flag whether to regularly check for
** USB bus changes.
**
** \param enabled    pointer to boolean flag
**
** \return -
**
** \sa SETTINGS_UsbAutoConnectSet
******************************************************************************/
int32_t SETTINGS_UsbAutoConnectGet(fbool_t* enabled)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    *enabled = startupDat->usbAutoReconnect;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set USB auto-reconnect flag.
**
** Set the flag whether to regularly check for USB bus changes.
**
** \param enabled    boolean flag
**
** \return -
**
** \sa SETTINGS_UsbAutoConnectGet
******************************************************************************/
int32_t SETTINGS_UsbAutoConnectSet(fbool_t enabled)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if(startupDat->usbAutoReconnect != enabled)
    {
        startupDat->usbAutoReconnect = enabled;
        startupDatDirty              = FTRUE;
    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    return FAPI_OK;
}
/*!
*******************************************************************************
**
** \brief  Save current values of frequently-changing data to flash.
**
** This will store an instance of #FCDATA_ENTRY_S to Flash using the internal
** FCDATA facility. This will not cause a flash sector to be reprogrammed
** before the assigned flash space is completely filled with #FCDATA_ENTRY_S
** entries.
**
** At next startup, the system will try to read the latest FCDATA-values and
** will modify its default settings for the corresponding parameters
** accordingly.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t SETTINGS_StoreFreqChgData(void)
{
    int32_t                       retVal;
    FCDATA_ENTRY_S                entry;
    uint8_t                       lvlMax;
    fbool_t                       isMuted;

    memset (&entry, 0, sizeof(FCDATA_ENTRY_S));

    (void)AV_GetRequestedService(&(entry.servLoc));

    retVal = AV_VolumeGet (&entry.audioLvlLeft,
                           &entry.audioLvlRight, &lvlMax, &isMuted);

    if ( retVal == FAPI_OK )
    {
        retVal = FCDATA_StoreEntry (&entry);
    }

    return retVal;
}

#ifdef APPL_SCRNSHOT_ENABLE
/*!
*******************************************************************************
**
** \brief  Set Screen Shot mode.
**
** \param  mode
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_ScreenShotModeGet
******************************************************************************/
int32_t SETTINGS_ScreenShotModeSet(SCREENCAPTURE_MODE_E mode)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if(mode >= SCREENCAPTURE_NUMS)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    startupDat->screenShotMode = mode;
    startupDatDirty            = FTRUE;

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get Screen Shot mode.
**
** \param  mode
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_ScreenShotModeSet
******************************************************************************/
int32_t SETTINGS_ScreenShotModeGet(SCREENCAPTURE_MODE_E* mode)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    *mode = startupDat->screenShotMode;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set Screen Shot to.
**
** \param  device
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_ScreenShotModeGet
******************************************************************************/
int32_t SETTINGS_ScreenShotToSet(SYS_DEVICE_E device)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    if(device >= SYS_DEV_NUMS)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    startupDat->screenShotTo = device;
    startupDatDirty          = FTRUE;

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Get Screen Shot to.
**
** \param  device
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SETTINGS_ScreenShotModeSet
******************************************************************************/
int32_t SETTINGS_ScreenShotToGet(SYS_DEVICE_E* device)
{
    if(startupDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    *device = startupDat->screenShotTo;
    return FAPI_OK;
}
#endif /* APPL_SCRNSHOT_ENABLE */


#ifdef APPL_ETHERNET_ENABLED
/*!
*******************************************************************************
**
** \brief  Get IP settings.
**
** Returns current setting of the user IP settings.
**  - DHCP flag
**  - IP Address
**  - NetMask
**  - Gateway
**  - DNS mode flag
**  - preverred DNS Server IP
**  - alternate DNS Server IP
**
** \param   ip  pointer to ETH_IP_S
**
** \return  One of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**          - #APPL_SMARTGO_ERR_BAD_PARAMETER
**
** \sa
**      - SETTINGS_EthIpSet
******************************************************************************/
int32_t SETTINGS_EthIpGet(ETH_IP_S* ip)
{
    if(startupDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(ip == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    *ip = startupDat->ethIp;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Set IP settings.
**
** Set the user IP.
**  - DHCP flag
**  - IP Address
**  - NetMask
**  - Gateway
**  - DNS mode flag
**  - preverred DNS Server IP
**  - alternate DNS Server IP
**
** \param   ip  pointer to ETH_IP_S
**
** \return  One of the following status codes:
**          - #FAPI_OK
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**          - #APPL_SMARTGO_ERR_BAD_PARAMETER
**
** \sa
**      - SETTINGS_EthIpGet
******************************************************************************/
int32_t SETTINGS_EthIpSet(ETH_IP_S* ip)
{
    if(startupDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(ip == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    (void)FAPEX_SYS_GetSemaphore(sysData->mutexId, FAPEX_SYS_SUSPEND);

    if(memcmp(&startupDat->ethIp,ip,sizeof(ETH_IP_S)) != 0 )
    {
        startupDat->ethIp = *ip;
        startupDatDirty   = FTRUE;

    }

    (void)FAPEX_SYS_SetSemaphore(sysData->mutexId, FAPEX_SYS_NO_SUSPEND);

    return FAPI_OK;
}

#endif /* APPL_ETHERNET_ENABLED */

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
