/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/slist.c
**
**              $Id: slist.c 5925 2010-03-05 06:43:50Z jmorha $
**
** \brief       Serice list management functions.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Info    715: Symbol not referenced.
**
*****************************************************************************
*/
/*lint -save -e715 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <string.h>

#include <flashfs/flashfs.h>

#include "sys_api.h"
#include "av.h"
#include "eit.h"
#include "utils.h"
#include "statlist.h"

#ifdef APPL_RECTIMER_ENABLED
#include "rectimer.h"
#endif /* APPL_RECTIMER_ENABLED */

#ifdef APPL_PVR_ENABLED
#include "pvr_sys.h"
#endif /* APPL_PVR_ENABLED */

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define SLIST_MAX_ALL        ((SLIST_MAX_TV) + (SLIST_MAX_RADIO) + \
                              (SLIST_MAX_INVISIBLE))

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    SYS_STRBUF_S        nameBuf;
    uint16_t            entriesMax;     //!< max. number of entries
    uint16_t            sizeCurr;       //!< index after last added entry;
    fbool_t             clean;          //!< 1: modified against stored data
    int16_t            *refIdx;         //!< service archive reference index

} SLIST_LIST_S;

#ifdef APPL_LCN_ENABLED

/*
NOTE: Order of areas in list is:
- LCN-services first, then
- non-LCN and LCN services already assigned
*/
typedef struct
{
    int16_t             numEntries;             //!< Num. entries in cont. list.
    SLIST_SERVIDX       servIdx[SLIST_MAX_ALL]; //!< Continuous list sorted by LCN.
    uint16_t            cn[SLIST_MAX_ALL];      //!< Assigned CN for each service
                                                //!< in static list (0: none).
    int16_t             startVariant;           //!< Start of variant area
                                                //!< (none/invalid/duplicate LCN).

} SLIST_LCN_S;

#endif /* APPL_LCN_ENABLED */

typedef struct
{
    FAPEX_SYS_SemaphoreT mutexId;        //!< Mutex semaphore.
    SLIST_PARAMS_S      params;

    uint8_t             numGlobLsts;
    uint8_t             numUsrLsts;

    /* service archive */
    STATLST_S           servLst;
    fbool_t             servClean;

    #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

    STATLST_S           caLst;
    fbool_t             caClean;

    #endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */

    SLIST_LIST_S       *lstGlob;        //!< global lists
    SLIST_LIST_S       *lstUsr;         //!< user lists

    #ifdef APPL_LCN_ENABLED

    SLIST_LCN_S         lstLcn;

    #endif /* APPL_LCN_ENABLED */

} SLIST_DATA_S;

typedef struct   /* service list file header */
{
    uint32_t            id;             //!< file identification
    uint32_t            revision;       //!< file structure revision
    uint16_t            numEntries;     //!< number of entries in file
    SYS_CHARSET         nameCharSet;    //!< Character set of list name.
    char_t              nameStr[SLIST_NAME_LENGTH]; //!< List name string.
    // TODO: add listID?

} SLIST_FILEHDR_S;

typedef struct                  /* service data file header */
{
    uint32_t            id;             //!< file identification
    uint32_t            revision;       //!< file structure revision
    SLIST_PARAMS_S      params;

} SLIST_SERVFILEHDR_S;

#if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

typedef struct                  /* CA data file header */
{
    uint32_t            id;             //!< file identification
    uint32_t            revision;       //!< file structure revision

} SLIST_CAFILEHDR_S;

#endif

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC SLIST_DATA_S *slDat = NULL;
FSTATIC fbool_t      lstValid = FTRUE;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void        SLIST_GlobalEntryRemove(int16_t servIdx);
FSTATIC int32_t     SLIST_AddNewEntry(const SLIST_ITEM_S * pItem,
                                      SLIST_LIST_S * pLst,
                                      SYS_SERVLOC_S * pNewServLoc);
FSTATIC int32_t     SLIST_OverwriteEntry(const SLIST_ITEM_S * pItem,
                                         SLIST_SERVIDX servIdx,
                                         SYS_SERVLOC_S * pServLoc);

FSTATIC int32_t     SLIST_ListAllocate(SLIST_LIST_S * pLst,
                                       uint16_t numElements);
FSTATIC uint32_t    SLIST_ListGetMaxDataSize(uint16_t numElements);
FSTATIC void        SLIST_ClearInternal(SLIST_LIST_S * pLst, fbool_t clearName);
FSTATIC fbool_t     SLIST_ReadList(SLIST_LIST_S * pLst, void *file);
FSTATIC fbool_t     SLIST_WriteList(SLIST_LIST_S * pLst, void *file);

FSTATIC int16_t     SLIST_GlobSearchItem(uint8_t listIdx,
                                         const SLIST_ITEM_S * pItem);

FSTATIC void        SLIST_UserRemoveReference(uint8_t listIdx, int16_t servIdx);

FSTATIC int16_t     SLIST_UserSearchItem(uint8_t listIdx, SLIST_REF_S * ref);

FSTATIC fbool_t     SLIST_ReferenceIsValid(SLIST_REF_S * ref);

FSTATIC SLIST_TYPE  SLIST_GetListId(const SLIST_ITEM_S * pItem);
FSTATIC SLIST_LIST_S *SLIST_GetList(SLIST_TYPE listId, uint8_t * intIdx,
                                    fbool_t* isGlob);
FSTATIC int32_t     SLIST_InsertRefEntry(SLIST_LIST_S* pLst, uint16_t lstIdx,
                                         SLIST_SERVIDX servIdx);
FSTATIC int32_t     SLIST_RemoveRefEntry(SLIST_LIST_S* pLst, uint16_t lstIdx,
                                         fbool_t shiftFollowing);
FSTATIC int16_t     SLIST_FindInList(const SLIST_LIST_S* pLst,
                                     SLIST_SERVIDX servIdx);
FSTATIC int8_t      compByName(SLIST_ITEM_S* comp1, SLIST_ITEM_S* comp2);

FSTATIC fbool_t     SLIST_ServEntriesLoad(void);
FSTATIC fbool_t     SLIST_ServEntriesSave(fbool_t force);

FSTATIC SLIST_SERVIDX SLIST_IntGetServIdx(const SYS_SERVLOC_S* pServLoc);

#ifdef APPL_LCN_ENABLED

FSTATIC int32_t     SLIST_LCN_ClearLists(void);
FSTATIC int32_t     SLIST_LCN_AddEntry(SLIST_SERVIDX idx,
                                       SYS_SERVLOC_S * pServLoc);
FSTATIC int32_t     SLIST_LCN_RemoveEntry(SLIST_SERVIDX idx);
FSTATIC int32_t     SLIST_LCN_ResortEntry(SYS_SERVLOC_S * pServLoc,
                                          SLIST_SERVIDX idx);
FSTATIC int32_t     SLIST_LCN_RestoreNonLcnLists(void);
FSTATIC int32_t     SLIST_LCN_RefEntryAdd(int16_t lcnIdx,
                                          SYS_SERVLOC_S * pServLoc);
FSTATIC int32_t     SLIST_LCN_RefEntryRemove(int16_t lcnIdx);

FSTATIC const SLIST_ITEM_S *SLIST_LCN_GetPreferredService(const SLIST_ITEM_S *
                                                          pServ1,
                                                          const SLIST_ITEM_S *
                                                          pServ2);

#endif /* APPL_LCN_ENABLED */

#if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

FSTATIC fbool_t     SLIST_CaInfoLoad(void);
FSTATIC fbool_t     SLIST_CaInfoSave(fbool_t force);
FSTATIC fbool_t     SLIST_CaInfoAddId(uint16_t ca_system_id,
                                      SLIST_CA_INFO_S * pInfo);

#endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */

/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/
void SLIST_Invalidate(void)
{
    lstValid = FFALSE;

    #ifdef APPL_RECTIMER_ENABLED

    /* Invalidate record timer list -> dependency on service list */
    RECTIMER_Invalidate();

    #endif /* APPL_RECTIMER_ENABLED */
}

int32_t SLIST_Init(void)
{
    int32_t     retVal;
    uint16_t    cnt;
    uint16_t    maxListSize;
    uint32_t    serviceFileSize = 0;
    #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)
    uint32_t    caFileSize      = 0;
    #endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */

    if ( slDat != NULL )
    {
        return APPL_SMARTGO_ERR_ALREADY_INITIALIZED;
    }

    slDat = UTILS_Malloc(sizeof(SLIST_DATA_S));
    if ( slDat == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    memset(slDat, 0, sizeof(SLIST_DATA_S));

    slDat->mutexId = FAPEX_SYS_CreateSemaphore(1);

    if ( slDat->mutexId == NULL )
    {
        return SMARTGO_SYS_ERR_RESOURCES;
    }

    slDat->params.sortByLcn          = FFALSE;
    slDat->params.variantRangeStart  = 1;
    slDat->params.variantRangeLength = 0;
    slDat->params.useLcnVisibility   = FFALSE;

    slDat->numGlobLsts = SLIST_GLOB_NUMLISTS;
    slDat->numUsrLsts  = SLIST_USR_NUMLISTS;
    slDat->lstUsr      = NULL;
    slDat->lstGlob     = NULL;

    retVal = STATLST_Init (&(slDat->servLst), SLIST_MAX_ALL,
                           sizeof(SLIST_ITEM_S));
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    slDat->servClean = FFALSE;

    serviceFileSize += STATLST_GetMaxDataSize (SLIST_MAX_ALL,
                                               sizeof(SLIST_ITEM_S));

    #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

    retVal = STATLST_Init(&(slDat->caLst), SLIST_MAX_ALL,
                          sizeof(SLIST_CA_INFO_S));
    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    slDat->caClean = FFALSE;

    caFileSize += STATLST_GetMaxDataSize (SLIST_MAX_ALL,
                                          sizeof(SLIST_CA_INFO_S));

    #endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */

    /* ----- get memory for global lists ------------------------------------ */

    slDat->lstGlob = UTILS_Malloc((slDat->numGlobLsts * sizeof(SLIST_LIST_S)));

    if ( slDat->lstGlob == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    for ( cnt = 0; cnt < slDat->numGlobLsts; cnt++ )
    {
        switch (cnt + SLIST_OFFSET_GLOB)
        {
            case SLIST_TV:
                maxListSize = SLIST_MAX_TV;
                break;
            case SLIST_RADIO:
                maxListSize = SLIST_MAX_RADIO;
                break;
            case SLIST_INVISIBLE:
                maxListSize = SLIST_MAX_INVISIBLE;
                break;
            default:
                maxListSize = 0;
                break;
        }

        retVal = SLIST_ListAllocate(slDat->lstGlob + cnt, maxListSize);
        if ( retVal != FAPI_OK )
        {
            return retVal;
        }

        serviceFileSize += SLIST_ListGetMaxDataSize (maxListSize);
    }

    /* ----- get memory for user lists -------------------------------------- */

    slDat->lstUsr = UTILS_Malloc((slDat->numUsrLsts * sizeof(SLIST_LIST_S)));

    if ( slDat->lstUsr == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    for ( cnt = 0; cnt < slDat->numUsrLsts; cnt++ )
    {
        retVal = SLIST_ListAllocate(slDat->lstUsr + cnt, SLIST_USR_SIZE);
        if ( retVal != FAPI_OK )
        {
            return retVal;
        }

        serviceFileSize += SLIST_ListGetMaxDataSize (SLIST_USR_SIZE);
    }

    /* ----- Test if maximum flash file size is sufficient ----------------- */

    serviceFileSize += (2 * sizeof(SLIST_SERVFILEHDR_S));

    retVal = FLASH_CheckFileSize (FILEID_SERVICEDATA, serviceFileSize);

    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

    caFileSize += (2 * sizeof(SLIST_CAFILEHDR_S));

    retVal = FLASH_CheckFileSize (FILEID_CADATA, caFileSize);

    if ( retVal != FAPI_OK )
    {
        return retVal;
    }

    #endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */

    /* ----- load service lists --------------------------------------------- */

    (void)SLIST_LoadLists();

    return FAPI_OK;
}

int32_t SLIST_SetParams(const SLIST_PARAMS_S * params)
{
    SLIST_PARAMS_S      currParams;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(params == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    /* Any difference to current parameters? */
    if((slDat->params.sortByLcn != params->sortByLcn) ||
       (slDat->params.variantRangeStart != params->variantRangeStart) ||
       (slDat->params.variantRangeLength != params->variantRangeLength) ||
       (slDat->params.useLcnVisibility != params->useLcnVisibility))
    {
        (void)memcpy( &(currParams), &(slDat->params), sizeof(SLIST_PARAMS_S) );
        //currParams = slDat->params;     /* store current params for
        //                                   comparison */
        (void)memcpy( &(slDat->params),params, sizeof(SLIST_PARAMS_S) );
        //slDat->params = *params;
        slDat->servClean = FFALSE;

    #ifdef APPL_LCN_ENABLED

        if(params->sortByLcn)
        {
            if(currParams.sortByLcn)    /* LCN already active? */
            {
                if((params->variantRangeStart !=
                    currParams.variantRangeStart) ||
                   (params->variantRangeLength !=
                    currParams.variantRangeLength) ||
                   (params->useLcnVisibility != currParams.useLcnVisibility))
                {
                    (void)SLIST_LCN_GenerateLists();
                }
            }
            else
            {
                (void)SLIST_LCN_GenerateLists();
            }
        }
        else
        {
            (void)SLIST_LCN_RestoreNonLcnLists();
        }

    #endif /* APPL_LCN_ENABLED */
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return FAPI_OK;
}

int32_t SLIST_GetParams(SLIST_PARAMS_S * params)
{
    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(params == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    *params = slDat->params;

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return FAPI_OK;
}

fbool_t SLIST_LoadLists(void)
{
    if(slDat == NULL)
    {
        return FTRUE;
    }

    FAPI_SYS_PRINT_DEBUG(1,"[SYS] Load service lists\n");
    /* read service data list */
    if ( SLIST_ServEntriesLoad() == FFALSE )
    {
        #ifdef APPL_RECTIMER_ENABLED

        /* mark lists referring current list as invalid */
        RECTIMER_Invalidate();

        #endif /* APPL_RECTIMER_ENABLED */

        FAPI_SYS_PRINT_DEBUG(1,"[SYS] Set service list defaults\n");
        (void)SLIST_RestoreDefaults();
    }

    return FTRUE;
}

/*!
*******************************************************************************
**
** \brief Set all service lists to default status.
**
** This function clears all lists and calls the application handlers used for
** setting defaults if available.
**
** \return
**      - #FAPI_OK      if successful
**
******************************************************************************/
int32_t SLIST_RestoreDefaults(void)
{
    uint32_t            cnt;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    (void)STATLST_Clear(&(slDat->servLst), SYS_OWNER_ALL, NULL, NULL);
    slDat->servClean = FFALSE;

    #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

    (void)STATLST_Clear(&(slDat->caLst), SYS_OWNER_ALL, NULL, NULL);
    slDat->caClean = FFALSE;

    #endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */

    #ifdef APPL_LCN_ENABLED

    (void)SLIST_LCN_ClearLists();

    #endif /* APPL_LCN_ENABLED */

    slDat->params.sortByLcn          = FFALSE;
    slDat->params.variantRangeStart  = 1;
    slDat->params.variantRangeLength = 0;
    slDat->params.useLcnVisibility   = FFALSE;

    for ( cnt = 0; cnt < slDat->numGlobLsts; cnt++ )
    {
        SLIST_ClearInternal (slDat->lstGlob + cnt, FTRUE);
    }

    for ( cnt = 0; cnt < slDat->numUsrLsts; cnt++ )
    {
        SLIST_ClearInternal (slDat->lstUsr + cnt, FTRUE);
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    for(cnt = 0; cnt < slDat->numGlobLsts; cnt++)
    {
        if(SYS_GetConfig()->setServListDefaults != NULL)
        {
            SYS_GetConfig()->setServListDefaults(SLIST_OFFSET_GLOB + cnt);
        }
    }

    for(cnt = 0; cnt < slDat->numUsrLsts; cnt++)
    {
        if(SYS_GetConfig()->setServListDefaults != NULL)
        {
            SYS_GetConfig()->setServListDefaults(SLIST_OFFSET_USR + cnt);
        }
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief Get service index referenced by specified service list item.
**
** Any item being part of a global service list references its service data at
** a static position in a separate list. If a global list item is moved, its
** corresponding service index does not change. This allows a unique
** identification of services.
**
** \param pServLoc  Service location.
**
** \return
**      - service index, -1 in case of error
**
** \sa SLIST_GetListReference
******************************************************************************/
SLIST_SERVIDX SLIST_GetServiceIndex(const SYS_SERVLOC_S * pServLoc)
{
    SLIST_SERVIDX       servIdx;

    if(slDat == NULL)
    {
        return -1;
    }
    if(pServLoc == NULL)
    {
        return -1;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    servIdx = SLIST_IntGetServIdx(pServLoc);

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return servIdx;
}

/*!
*******************************************************************************
**
** \brief Get global list position referencing requested service index.
**
** Any item being part of a global service list references its service data at
** a static position in a separate list. If a global list item is moved, its
** corresponding service index does not change. This allows a unique
** identification of services.
**
** \param servIdx   service index
**
** \return   service list location (global list only) or -1 if not existing.
**
** \sa SLIST_GetServiceIndex
******************************************************************************/
SYS_SERVLOC_S SLIST_GetListReference(SLIST_SERVIDX servIdx)
{
    SYS_SERVLOC_S       servLoc;
    SLIST_ITEM_S       *pItem;
    SLIST_LIST_S       *pLst;
    uint8_t             cnt;

    memset(&servLoc,0,sizeof(SYS_SERVLOC_S));
    servLoc.listId = SLIST_NONE;
    servLoc.index = -1;

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    pItem = (SLIST_ITEM_S *) STATLST_GetEntry(&(slDat->servLst), servIdx);

    if(pItem != NULL)
    {
        for(cnt = 0; cnt < slDat->numGlobLsts; cnt++)
        {
            pLst = slDat->lstGlob + (cnt & SLIST_MASK_GLOB);

            servLoc.index = SLIST_FindInList(pLst, servIdx);
            if(servLoc.index >= 0)
            {
                servLoc.listId = SLIST_OFFSET_GLOB + cnt;
                break;
            }
        }
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return servLoc;
}


/*!
*******************************************************************************
**
** \brief  Get size of requested service list.
**
** The value returned includes empty entries which may exist at any position
** in the list.
**
** \param  listId  service list identifier
**
** \return Number of entries in requested service list, in case of an invalid
**         list identifier 0 is returned.
******************************************************************************/
uint16_t SLIST_ListSizeGet(SLIST_TYPE listId)
{
    SLIST_LIST_S       *pLst;
    uint16_t            listSize;

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    pLst = SLIST_GetList(listId, NULL, NULL);

    if(pLst == NULL)
    {
        listSize = 0;
    }
    else
    {
        listSize = pLst->sizeCurr;
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return listSize;
}


/*!
*******************************************************************************
**
** \brief Get a service list entry.
**
** \param pServLoc  Service location.
** \param pItem     Pointer to a  SLIST_ITEM_S structure where entry will be
**                  copied into.
**
** \return
**      - #FAPI_OK                            Successful.
**      - #SMARTGO_SYS_ERR_ITEM_EMPTY      Service is empty.
**      - #APPL_SMARTGO_ERR_BAD_PARAMETER             Invalid pointer or invalid entry
**                                          location.
******************************************************************************/
int32_t SLIST_EntryGet(const SYS_SERVLOC_S * pServLoc, SLIST_ITEM_S * pItem)
{
    SLIST_LIST_S       *pLst;
    SLIST_ITEM_S       *pEntry;
    int32_t             retVal;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(pItem == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    pLst = SLIST_GetList(pServLoc->listId, NULL, NULL);
    if(pLst == NULL)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if((pServLoc->index < 0) ||
            ((uint16_t) pServLoc->index >= pLst->sizeCurr))
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        pEntry = STATLST_GetEntry(&(slDat->servLst),
                                  pLst->refIdx[pServLoc->index]);

        if(pEntry == NULL)
        {
            retVal = SMARTGO_SYS_ERR_ITEM_EMPTY;
        }
        else
        {
            *pItem = *pEntry;
            retVal = FAPI_OK;
        }
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Add a service list entry.
**
** \param  pItem        Item data to be added.
** \param  pNewServLoc  If successful, service list location of added entry
**                      is returned here (set to NULL if not needed).
**
** \return One of the following status codes:
** - #FAPI_OK                           - Successfully added to list.
** - #SMARTGO_SYS_ERR_CAPACITY          - No more free list space available.
** - #SMARTGO_SYS_ERR_REPLACED_IN_LIST - Service already in list but data has
**                                        been updated
******************************************************************************/
int32_t SLIST_EntryAdd(const SLIST_ITEM_S * pItem, SYS_SERVLOC_S * pNewServLoc)
{
    int32_t         retVal = FAPI_OK;
    uint8_t         intIdx;
    fbool_t         isGlob;
    SLIST_LIST_S*   pLst;
    SYS_SERVLOC_S   newServLoc;
    SLIST_ITEM_S*   pSrc;
    SLIST_ITEM_S    srcItem;
    SLIST_SERVIDX   srcCnt;
    TPLIST_ITEM_S   carrierNew;
    TPLIST_ITEM_S   carrierRef;
    fbool_t         addService = FTRUE;
    uint8_t         serviceMinAge;

    if ( slDat == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else if ( pItem == NULL )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    newServLoc.listId = SLIST_GetListId(pItem);
    newServLoc.index = -1;

    pLst = SLIST_GetList(newServLoc.listId, &intIdx, &isGlob);

    if ( pLst == NULL )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if ( !isGlob )
    {
        retVal = SMARTGO_SYS_ERR_INTERNAL;
    }
    else if((retVal = TPLIST_GetDataByIndex(&(pItem->network), pItem->tpIdx,
                                            &carrierNew)) != FAPI_OK)
    {
        retVal = SMARTGO_SYS_ERR_INVALID_DATA;
    }
    else
    {
        /* Parse current service list for double entries. */
        for(srcCnt = 0; srcCnt < slDat->servLst.stat.afterLast; srcCnt++)
        {
            pSrc = STATLST_GetEntry(&(slDat->servLst), srcCnt);
            if ( pSrc == NULL )
            {
                continue;
            }

            /* network match? */
            if ( (pSrc->network.type   != pItem->network.type)
              || (pSrc->network.satIdx != pItem->network.satIdx) )
            {
                continue;
            }

            /* service match? */
            if ( pSrc->serviceId != pItem->serviceId )
            {
                continue;
            }

            srcItem = *pSrc;

            /* save existing minAge, will be temporarily overwritten */
            serviceMinAge = srcItem.minAge;

            /* overwrite minAge to be able to perform memcmp on both
               structures */
            srcItem.minAge = pItem->minAge;

            if ( srcItem.tpIdx == pItem->tpIdx )     /* same carrier? */
            {
                if ( srcItem.serviceId == pItem->serviceId )
                {
                    /* Identical services found: Overwrite data if any changes
                       detected. */
                    newServLoc.index = SLIST_FindInList(pLst, srcCnt);

                    /* test if data is different to stored data */
                    if ( srcItem.renamed )
                    {
                        retVal = SMARTGO_SYS_ERR_ALREADY_IN_LIST;
                    }
                    else if(memcmp(pItem, &srcItem, sizeof(SLIST_ITEM_S)) == 0)
                    {
                        retVal = SMARTGO_SYS_ERR_ALREADY_IN_LIST;
                    }
                    else
                    {
                        /* restore minAge */
                        srcItem = *pItem;

                        srcItem.minAge = serviceMinAge;

                        /* overwrite existing entry */
                        retVal = SLIST_OverwriteEntry(&srcItem, srcCnt,
                                                      &newServLoc);
                    }

                    addService = FFALSE;
                    break;
                }
            }
            else
            {
                if(TPLIST_GetDataByIndex(&(srcItem.network), srcItem.tpIdx,
                                         &carrierRef) != FAPI_OK)
                {
                    continue;
                }

                if(carrierRef.onId != carrierNew.onId)
                {
                    continue;
                }

                newServLoc.index = SLIST_FindInList(pLst, srcCnt);

                if(carrierRef.tsId == carrierNew.tsId)
                {
                    if((carrierNew.networkId >= 0) &&
                       (carrierNew.networkId == carrierRef.networkId))
                    {
                        /* Services are identical. Remove the one with weaker
                           signal. */
                        if(carrierRef.sigQuality >= carrierNew.sigQuality)
                        {
                            /* Existing service is stronger, discard new
                               service. */
                            retVal = SMARTGO_SYS_ERR_ALREADY_IN_LIST;
                        }
                        else
                        {
                            /* restore minAge */
                            srcItem = *pItem;

                            srcItem.minAge = serviceMinAge;

                            /* New service is stronger, overwrite existing
                               service. */
                            retVal = SLIST_OverwriteEntry(&srcItem, srcCnt,
                                                          &newServLoc);
                        }

                        addService = FFALSE;
                        break;
                    }
                    else
                    {
                        /* Same transport stream is broadcast on different
                           networks. If LCN is enabled, keep service with
                           stronger signal in LCN area and move other one to
                           non-LCN area. */

                        /* TODO: Implement handling described above! For
                           the moment, remove/overwrite service with weaker
                           signal! */

                        if(carrierRef.sigQuality >= carrierNew.sigQuality)
                        {
                            /* Existing service is stronger, discard new
                               service. */
                            retVal = SMARTGO_SYS_ERR_ALREADY_IN_LIST;
                        }
                        else
                        {
                            /* restore minAge */
                            srcItem = *pItem;

                            srcItem.minAge = serviceMinAge;

                            /* New service is stronger, overwrite existing
                               service. */
                            retVal = SLIST_OverwriteEntry(&srcItem, srcCnt,
                                                          &newServLoc);
                        }

                        addService = FFALSE;
                        break;
                    }
                }
#ifndef APPL_MHEG_ENABLED
                else
                {
                    /* Identical service on different networks: Remove weaker
                       service. */
                    if(carrierRef.sigQuality >= carrierNew.sigQuality)
                    {
                        /* Existing service is stronger, discard new service. */
                        retVal = SMARTGO_SYS_ERR_ALREADY_IN_LIST;
                    }
                    else
                    {
                        srcItem = *pItem;
                        /* restore minAge */
                        srcItem.minAge = serviceMinAge;

                        /* New service is stronger, overwrite existing
                           service. */
                        retVal = SLIST_OverwriteEntry(&srcItem, srcCnt,
                                                      &newServLoc);
                    }
                    addService = FFALSE;
                    break;
                }
#endif
            }
        }

        if(addService)
        {
            // NOTE: newServLoc.listId needs to be valid at this point!
            retVal = SLIST_AddNewEntry(pItem, pLst, &newServLoc);
        }

        if(pNewServLoc != NULL)
        {
            *pNewServLoc = newServLoc;
        }
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}


/*!
*******************************************************************************
**
** \brief  Insert a service list entry.
**
** Inserts an entry into a service list. To append to a list, the position
** after the last valid entry must be specified.
**
** This function has a uniform interface for global and user lists. If a global
** list is referenced, used the pItem argument to pass a complete service list
** data structure. In case of user lists, only reference data is given via the
** ref argument.
**
** For each modified entry in a global list, all references on it in any user
** list are automatically updated.
**
** \param  pServLoc  Service location to insert into.
** \param  pItem     item data (for global lists only, else set to NULL)
** \param  ref       reference data (for user lists only, else set to NULL)
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t SLIST_EntryInsert(const SYS_SERVLOC_S* pServLoc, SLIST_ITEM_S* pItem,
                          SLIST_REF_S* ref)
{
    fbool_t             isGlob;
    SLIST_LIST_S       *pLst;
    int16_t             servIdx;
    int32_t             retVal = FAPI_OK;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    pLst = SLIST_GetList(pServLoc->listId, NULL, &isGlob);

    if(pLst == NULL)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if((pServLoc->index < 0) ||
            ((uint16_t)pServLoc->index > pLst->sizeCurr))
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    /* space left for one more entry? */
    else if(pLst->sizeCurr >= pLst->entriesMax)
    {
        retVal = SMARTGO_SYS_ERR_CAPACITY;
    }
    else if(isGlob)
    {
        /* function is not available for global lists */
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if(!(SLIST_ReferenceIsValid(ref)))
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        servIdx = slDat->lstGlob[ref->listNum].refIdx[ref->index];

        /* insert requested element into target list */
        (void)SLIST_InsertRefEntry(pLst, (uint16_t)(pServLoc->index), servIdx);
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Replace an existing service list entry.
**
** This function has a uniform interface for global and user lists. If a global
** list is referenced, used the pItem argument to pass a complete service list
** data structure. In case of user lists, only reference data is given via the
** ref argument.
**
** For each modified entry in a global list, all references on it in any user
** list are automatically updated.
**
** \param  pServLoc  Service location to be replaced.
** \param  pItem     item data (for global lists only, else set to NULL)
** \param  ref       reference data (for user lists only, set set to NULL)
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t SLIST_EntryReplace(const SYS_SERVLOC_S* pServLoc, SLIST_ITEM_S* pItem,
                           SLIST_REF_S* ref)
{
    SLIST_LIST_S       *pLst;
    fbool_t             isGlob;
    int16_t             servIdx;
    int32_t             retVal = FAPI_OK;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    pLst = SLIST_GetList(pServLoc->listId, NULL, &isGlob);

    if(pLst == NULL)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if((pServLoc->index < 0) ||
            ((uint16_t) pServLoc->index >= pLst->sizeCurr))
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if(isGlob)
    {
        /* function is not available for global lists */
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        servIdx = slDat->lstGlob[ref->listNum].refIdx[ref->index];

        /* copy requested element */
        pLst->refIdx[pServLoc->index] = servIdx;
        pLst->clean                   = FFALSE;
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Remove a service list entry.
**
** For each modified entry in a global list, all references on it in any user
** list are automatically updated.
**
** \param  pServLoc        Service location to be removed.
** \param  shiftFollowing  if 1, shift all subsequent entries down by 1 to
**                         avoid having an empty entry.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t SLIST_EntryRemove(const SYS_SERVLOC_S* pServLoc,
                          fbool_t shiftFollowing)
{
    fbool_t         isGlob;
    SLIST_LIST_S*   pLst;
    SYS_SERVLOC_S   currServLoc;
    int32_t         retVal = FAPI_OK;
    fbool_t         stopService = FFALSE;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    pLst = SLIST_GetList(pServLoc->listId, NULL, &isGlob);
    if(pLst == NULL)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if((pServLoc->index < 0) ||
            ((uint16_t) pServLoc->index >= pLst->sizeCurr))
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        /* If entry to be removed is currently selected live service,
           stop it! */

        (void)AV_GetRequestedService(&currServLoc);

        if(isGlob)
        {
            if(SLIST_IntGetServIdx(&currServLoc) ==
               pLst->refIdx[pServLoc->index])
            {
                stopService = FTRUE;
            }

            SLIST_GlobalEntryRemove(pLst->refIdx[pServLoc->index]);
        }
        else
        {
            if((pServLoc->listId == currServLoc.listId) &&
               (pServLoc->index == currServLoc.index))
            {
                stopService = FTRUE;
            }
        }

        (void)SLIST_RemoveRefEntry (pLst, (uint16_t)(pServLoc->index),
                                    shiftFollowing);
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    if(stopService)
    {
        (void)AV_ResetService();
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Move an entry within a service list.
**
** For each modified entry in a global list, all references on it in any user
** list are automatically updated.
**
** \param  listId          service list identifier
** \param  indexOld        service list index to be moved
** \param  indexNew        target index in service list
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t SLIST_EntryMove(SLIST_TYPE listId, uint16_t indexOld, uint16_t indexNew)
{
    SLIST_LIST_S       *pLst;
    uint16_t            cnt;
    int16_t             savedRef;
    int32_t             retVal;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    pLst = SLIST_GetList(listId, NULL, NULL);

    if(pLst == NULL)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if(indexOld == indexNew)
    {
        retVal = FAPI_OK;
    }
    else if((indexOld >= pLst->sizeCurr) || (indexNew >= pLst->sizeCurr))
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        /* save entry to be moved */
        savedRef = pLst->refIdx[indexOld];

        /* shift other entries */
        if(indexNew < indexOld) /* shift down */
        {
            for(cnt = indexOld; cnt > indexNew; cnt--)
            {
                pLst->refIdx[cnt] = pLst->refIdx[cnt - 1];
            }
        }
        else /* shift up */
        {
            for(cnt = indexOld; cnt < indexNew; cnt++)
            {
                pLst->refIdx[cnt] = pLst->refIdx[cnt + 1];
            }
        }

        /* copy entry to destination position */
        pLst->refIdx[indexNew] = savedRef;
        pLst->clean            = FFALSE;

        retVal = FAPI_OK;
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Test if a service list item is already contained in requested list.
**
** \param  listId    service list identifier
** \param  pItem     item data (for global lists only, else set to NULL)
** \param  ref       reference data (for user lists only, set set to NULL)
**
** \return
**         - #1  Item has been found
**         - #0 Item has not been found
******************************************************************************/
fbool_t SLIST_EntryIsInList(SLIST_TYPE listId, SLIST_ITEM_S * pItem,
                            SLIST_REF_S * ref)
{
    uint8_t             intIdx;
    fbool_t             isGlob;
    fbool_t             retVal;

    if(slDat == NULL)
    {
        return FFALSE;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    if ( SLIST_GetList (listId, &intIdx, &isGlob) == NULL )
    {
        retVal = FFALSE;
    }
    else if ( isGlob )
    {
        retVal = (SLIST_GlobSearchItem (intIdx, pItem) >= 0) ? FTRUE : FFALSE;
    }
    else
    {
        retVal = (SLIST_UserSearchItem (intIdx, ref) >= 0) ? FTRUE : FFALSE;
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}

int32_t SLIST_EntryFindInList(const SYS_SERVLOC_S * pServLoc,
                              SLIST_TYPE reqListId, int16_t * pFoundIndex)
{
    SLIST_SERVIDX       servIdx;
    SLIST_LIST_S       *pLst;
    int16_t             index;
    int32_t             retVal;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    if(pServLoc->listId == reqListId)
    {
        *pFoundIndex = pServLoc->index;
        return FAPI_OK;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    servIdx = SLIST_IntGetServIdx(pServLoc);
    if(servIdx < 0)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        pLst = SLIST_GetList(reqListId, NULL, NULL);
        if(pLst == NULL)
        {
            retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
        else
        {
            /* search entry reference in target list */
            index = SLIST_FindInList(pLst, servIdx);

            if(index >= 0)
            {
                *pFoundIndex = index;
                retVal = FAPI_OK;
            }
            else
            {
                retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
            }
        }
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Store changed service lists.
**
** This stores all available service lists that have been modified since
** system startup or last saving.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t SLIST_Save(fbool_t force)
{
    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (void) SLIST_ServEntriesSave(force);
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Clear a single service list.
**
** \param  listId    service list identifier
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t SLIST_ClearList(SLIST_TYPE listId)
{
    fbool_t             isGlob;
    SLIST_LIST_S       *pLst;
    uint16_t            cnt;
    int16_t             servIdx;
    SYS_SERVLOC_S       currServLoc;
    fbool_t             stopService = FFALSE;
    int32_t             retVal = FAPI_OK;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    pLst = SLIST_GetList(listId, NULL, &isGlob);

    if(pLst == NULL)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        /* If entry to be removed is currently selected live service,
           stop it! */

        (void)AV_GetRequestedService(&currServLoc);

        if(isGlob)  /* clear also service data and user list references */
        {
            if(pLst->refIdx != NULL)
            {
                for(cnt = 0; cnt < pLst->entriesMax; cnt++)
                {
                    servIdx = pLst->refIdx[cnt];
                    if(servIdx < 0)
                    {
                        continue;
                    }

                    SLIST_GlobalEntryRemove(servIdx);

                    if(SLIST_IntGetServIdx(&currServLoc) == servIdx)
                    {
                        stopService = FTRUE;
                    }
                }
            }
        }
        else
        {
            /* If currently selected service is part of list to be removed,
               stop it */
            if(listId == currServLoc.listId)
            {
                stopService = FTRUE;
            }
        }

        SLIST_ClearInternal (pLst, FFALSE);
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    if(stopService)
    {
        (void)AV_ResetService();
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Clear all service lists.
**
** \attention This does \b NOT store the cleared lists.
**
** \return -
******************************************************************************/
int32_t SLIST_ClearAllLists(void)
{
    uint16_t            cnt;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    /* clear all user lists */
    for(cnt = 0; cnt < slDat->numUsrLsts; cnt++)
    {
        (void)SLIST_ClearList(SLIST_OFFSET_USR + cnt);
    }

    for(cnt = 0; cnt < slDat->numGlobLsts; cnt++)
    {
        (void)SLIST_ClearList(SLIST_OFFSET_GLOB + cnt);
    }

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Return status of requested flag in service item.
**
** \param  pItem    pointer to service list item
** \param  flag     flag to be tested (use SLIST_FLAG_XXX constants!)
**
** \return 1: flag is set, 0: flag is cleared
******************************************************************************/
fbool_t SLIST_FlagIsSet(const SLIST_ITEM_S * pItem, uint8_t flag)
{
    return ( ((pItem->flags & flag) != 0) ? FTRUE : FFALSE);
}

void SLIST_SetFlag(SLIST_ITEM_S * pItem, uint8_t flag, fbool_t val)
{
    if ( val == FTRUE )
    {
        pItem->flags |=  flag;
    }
    else
    {
        pItem->flags &= ~flag;
    }
}

/*!
*******************************************************************************
**
** \brief  Get service list name.
**
** \param  listId  service list identifier
**
** \return pointer to name string
**
** \sa SLIST_ListNameSet
******************************************************************************/
SYS_STRING_S SLIST_ListNameGet (SLIST_TYPE listId)
{
    SLIST_LIST_S       *pLst;
    SYS_STRING_S  listName;

    pLst = SLIST_GetList(listId, NULL, NULL);

    if(pLst == NULL)
    {
        STR_Clear (&listName);
    }
    else
    {
        listName = pLst->nameBuf.string;
    }

    return listName;
}

/*!
*******************************************************************************
**
** \brief  Set service list name.
**
** \param  listId  service list identifier
** \param  name    pointer to name string
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa SLIST_ListNameGet
******************************************************************************/
int32_t SLIST_ListNameSet (SLIST_TYPE listId, SYS_STRING_S name)
{
    SLIST_LIST_S       *pLst;
    int32_t             retVal;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    pLst = SLIST_GetList(listId, NULL, NULL);
    if(pLst == NULL)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        (void)STR_BufAppend (&(pLst->nameBuf), 0, name, name.cs);

        retVal = FAPI_OK;
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    if(retVal == FAPI_OK)
    {
        slDat->servClean = FFALSE;
    }
    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Sort a service list.
**
** This function fills a given list with index values referencing the requested
** list following the specified sort order.
**
** \param  listId      service list to be sorted
** \param  mode        sorting mode
** \param  ascending   1: sort in ascending order, else sort in descending
                       order
** \param  pList       pointer to array of list entries
** \param  numEntries  maximum size of resulting list, may be less than the
**                     number of entries in list to be sorted.
**
** \return Number of items in sorted list or -1 in case of error.
**
******************************************************************************/
int16_t SLIST_Sort(SLIST_TYPE listId, SLIST_SORTMODE_E mode, fbool_t ascending,
                   int16_t* pList, uint16_t numEntries)
{
    int8_t(*compFunc) (SLIST_ITEM_S * comp1, SLIST_ITEM_S * comp2) = NULL;
    uint16_t            idxCnt;
    uint16_t            desCnt = 0;
    uint16_t            insCnt;
    uint16_t            sortListSize = 0;       // current size of resulting list
    fbool_t             insert;
    SLIST_ITEM_S       *pSrc;
    SLIST_ITEM_S       *pDes;
    int8_t              compResult;
    SLIST_LIST_S       *pLst;
    int16_t             servIdx;
    int16_t             desIdx;

    if(pList == NULL)
    {
        return -1;
    }
    else if(numEntries == 0)
    {
        return 0;
    }

    pLst = SLIST_GetList(listId, NULL, NULL);
    if(pLst == NULL)
    {
        return -1;
    }

    /* check maximum list size */
    if(numEntries >= pLst->sizeCurr)
    {
        numEntries = pLst->sizeCurr;
    }

    /* assign compare function according to requested sort mode */
    switch (mode)
    {
        case SLIST_SORT_BYINDEX:
            /* sorting not necessary, simply fill the list... */
            if ( ascending == FTRUE )
            {
                for(idxCnt = 0; idxCnt < numEntries; idxCnt++)
                {
                    pList[idxCnt] = (int16_t)idxCnt;
                }
            }
            else
            {
                for(idxCnt = 0; idxCnt < numEntries; idxCnt++)
                {
                    pList[idxCnt] = (int16_t)((pLst->sizeCurr - idxCnt) - 1);
                }
            }

            return (int16_t)numEntries;

        case SLIST_SORT_BYNAME:
            compFunc = compByName;
            break;

        default:
            return -1;
    }

    /* sort all entries */
    for(idxCnt = 0; idxCnt < pLst->sizeCurr; idxCnt++)
    {
        insert = FFALSE;

        /* get item to be inserted */
        servIdx = pLst->refIdx[idxCnt];

        if ( sortListSize == 0 )
        {
            insert = FTRUE;
        }
        else
        {
            pSrc = STATLST_GetEntry(&(slDat->servLst), servIdx);
            if(pSrc == NULL)
            {
                /* special handling for empty list items */
                if ( ascending == FTRUE )
                {
                    desCnt = sortListSize;
                }
                else
                {
                    desCnt = 0;
                    insert = FTRUE;
                }
            }
            else
            {
                for(desCnt = 0; desCnt < sortListSize; desCnt++)
                {
                    /* get item for new item to be compared with */
                    desIdx = pLst->refIdx[pList[desCnt]];

                    if(desIdx < 0)
                    {
                        continue;
                    }
                    pDes = STATLST_GetEntry(&(slDat->servLst), desIdx);
                    if(pDes == NULL)
                    {
                        /* special handling for empty list items */
                        compResult = 1;
                    }
                    else
                    {
                        compResult = compFunc(pSrc, pDes);
                    }
                    if ( ((ascending == FTRUE)  && (compResult <  0)) ||
                         ((ascending == FFALSE) && (compResult >= 0)) )
                    {
                        insert = FTRUE;
                        break;
                    }
                }
            }
            /* append item if possible */
            if ( (insert == FFALSE) &&
                 (sortListSize < numEntries) )
            {
                insert = FTRUE;
            }
        }

        /* insert BEFORE current position */
        if ( insert == FTRUE )
        {
            /* extend target list if possible */
            if ( sortListSize < numEntries )
            {
                sortListSize++;
            }

            for ( insCnt = (sortListSize - 1); insCnt > desCnt; insCnt-- )
            {
                pList[insCnt] = pList[insCnt - 1];
            }

            if ( desCnt < sortListSize )
            {
                pList[desCnt] = (int16_t)idxCnt;
            }
        }
    }

    return (int16_t)sortListSize;
}

int32_t SLIST_SetServByIndex(SLIST_SERVIDX index, const SLIST_ITEM_S * pItem)
{
    SLIST_ITEM_S       *pEntry;
    int32_t             retVal = FAPI_OK;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    pEntry = STATLST_GetEntry(&(slDat->servLst), index);

    /* Verify that entry already exists */
    if(pEntry == NULL)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if(memcmp(pItem, pEntry, sizeof(SLIST_ITEM_S)) != 0)
    {
        retVal = STATLST_SetEntry(&(slDat->servLst), index, pItem,
                                  SYS_OWNER_SYSTEM);

        if(retVal == FAPI_OK)
        {
            slDat->servClean = FFALSE;
        }
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}

int32_t SLIST_SetServNameByIndex(SLIST_SERVIDX index,SYS_STRING_S str)
{
    SLIST_ITEM_S       *pEntry;
    int32_t             retVal = FAPI_OK;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    pEntry = STATLST_GetEntry(&(slDat->servLst), index);

    /* Verify that entry already exists */
    if(pEntry == NULL)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if( strcmp(str.chr, pEntry->name) )
    {
        (void)STR_Copy( str, pEntry->name, &pEntry->nameCharSet,
                  pEntry->nameCharSet,SLIST_NAME_LENGTH);

        pEntry->renamed = 1;

        retVal = STATLST_SetEntry(&(slDat->servLst), index, pEntry,
                                  SYS_OWNER_SYSTEM);

        if(retVal == FAPI_OK)
        {
            slDat->servClean = FFALSE;
        }
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}

int32_t SLIST_GetServByIndex(SLIST_SERVIDX index, SLIST_ITEM_S * pItem)
{
    SLIST_ITEM_S       *pEntry;
    int32_t             retVal;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    pEntry = STATLST_GetEntry(&(slDat->servLst), index);

    if(pEntry == NULL)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        *pItem = *pEntry;
        retVal = FAPI_OK;
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}

int32_t SLIST_SdtUpdatedService(const SLIST_ITEM_S * pItem)
{
    SLIST_ITEM_S       *pSrc;
    SLIST_SERVIDX       srcCnt;
    int32_t             retVal;
    fbool_t             found   = FFALSE;
    fbool_t             changed = FFALSE;
    fbool_t             flag;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    /* search service in list */
    for(srcCnt = 0; srcCnt < slDat->servLst.stat.afterLast; srcCnt++)
    {
        pSrc = STATLST_GetEntry(&(slDat->servLst), srcCnt);

        if(pSrc == NULL)
        {
            continue;
        }

        if((pSrc->serviceId == pItem->serviceId) &&
           (pSrc->tpIdx == pItem->tpIdx) &&
           (pSrc->network.satIdx == pItem->network.satIdx) &&
           (pSrc->network.type == pItem->network.type))
        {
            found = FTRUE;

            if ( ( pSrc->renamed == 0 ) &&
                 ( STR_Cmp (STR_Build (pSrc->nameCharSet, pSrc->name),
                            STR_Build (pItem->nameCharSet, pItem->name))!= 0 )
                 )
            {
                (void)STR_Copy (STR_Build (pItem->nameCharSet, pItem->name),
                          pSrc->name, &(pSrc->nameCharSet), pItem->nameCharSet,
                          SLIST_NAME_LENGTH);

                changed = FTRUE;
            }

            flag = SLIST_FlagIsSet (pItem, SLIST_FLAG_ALLFREE);

            if ( SLIST_FlagIsSet (pSrc, SLIST_FLAG_ALLFREE) != flag )
            {
                SLIST_SetFlag (pSrc, SLIST_FLAG_ALLFREE, flag);
                changed = FTRUE;
            }

            flag = SLIST_FlagIsSet (pItem, SLIST_FLAG_CURRNEXTAVL);

            if ( SLIST_FlagIsSet (pSrc, SLIST_FLAG_CURRNEXTAVL) != flag )
            {
                SLIST_SetFlag (pSrc, SLIST_FLAG_CURRNEXTAVL, flag);
                changed = FTRUE;
            }

            flag = SLIST_FlagIsSet (pItem, SLIST_FLAG_SCHEDAVL);

            if ( SLIST_FlagIsSet (pSrc, SLIST_FLAG_SCHEDAVL) != flag )
            {
                SLIST_SetFlag (pSrc, SLIST_FLAG_SCHEDAVL, flag);
                changed = FTRUE;
            }

            break;
        }
    }

    if ( found )
    {
        if ( changed )
        {
            slDat->servClean = FFALSE;

            retVal = SMARTGO_SYS_ERR_REPLACED_IN_LIST;
        }
        else
        {
            retVal = SMARTGO_SYS_ERR_ALREADY_IN_LIST;
        }
    }
    else
    {
        retVal = FAPI_OK;
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}

/* ------------------------------------------------------------------------- */

FSTATIC SLIST_SERVIDX SLIST_IntGetServIdx(const SYS_SERVLOC_S * pServLoc)
{
    SLIST_SERVIDX       servIdx = -1;
    SLIST_LIST_S       *pLst;

    pLst = SLIST_GetList(pServLoc->listId, NULL, NULL);

    if(pLst != NULL)
    {
        if((pServLoc->index >= 0) &&
           ((uint16_t) pServLoc->index < pLst->sizeCurr))
        {
            servIdx = pLst->refIdx[pServLoc->index];
        }
    }

    return servIdx;
}

FSTATIC int32_t SLIST_AddNewEntry(const SLIST_ITEM_S* pItem,
                                  SLIST_LIST_S* pLst,
                                  SYS_SERVLOC_S* pNewServLoc)
{
    int32_t retVal = FAPI_OK;
    int32_t servIdx;

    #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

    SLIST_CA_INFO_S     caInfo;

    #endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */

    /* space left for one more entry? */
    if(pLst->sizeCurr >= pLst->entriesMax)
    {
        return SMARTGO_SYS_ERR_CAPACITY;
    }

    /* add to service list */
    retVal = STATLST_AddEntry(&(slDat->servLst), pItem,
                              SYS_OWNER_SYSTEM, &servIdx);

    if(retVal != FAPI_OK)
    {
        return retVal;
    }

    slDat->servClean = FFALSE;

    #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

    /* add empty CA info structure */
    memset(&caInfo, 0, sizeof(caInfo));

    (void)STATLST_SetEntry(&(slDat->caLst), servIdx, &caInfo, SYS_OWNER_SYSTEM);

    slDat->caClean = FFALSE;

    #endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */

    #ifdef APPL_LCN_ENABLED

    if(slDat->params.sortByLcn)
    {
        retVal = SLIST_LCN_AddEntry(servIdx, pNewServLoc);

        if(retVal != FAPI_OK)
        {
            /* Cleanup: remove entry from service list and CA info list. */

            (void) STATLST_RemoveEntry(&(slDat->servLst), servIdx);

            #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

            (void) STATLST_RemoveEntry(&(slDat->caLst), servIdx);

            #endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */
        }
    }
    else
    #endif /* APPL_LCN_ENABLED */
    {

        pNewServLoc->index = (int16_t)(pLst->sizeCurr);

        (void)SLIST_InsertRefEntry (pLst, (uint16_t)(pNewServLoc->index),
                                    servIdx);
    }
    return retVal;
}

FSTATIC int32_t SLIST_OverwriteEntry(const SLIST_ITEM_S * pItem,
                                     SLIST_SERVIDX servIdx,
                                     SYS_SERVLOC_S* pServLoc)
{
    int32_t             retVal;

    retVal = STATLST_SetEntry(&(slDat->servLst), servIdx,
                              pItem, SYS_OWNER_SYSTEM);

    if(retVal == FAPI_OK)
    {
        slDat->servClean = FFALSE;
        retVal = SMARTGO_SYS_ERR_REPLACED_IN_LIST;

        #ifdef APPL_LCN_ENABLED

        /* Re-sort entry in case LCN has changed */
        if(slDat->params.sortByLcn)
        {
            (void)SLIST_LCN_ResortEntry(pServLoc, servIdx);
        }

        #endif /* APPL_LCN_ENABLED */
    }

    return retVal;
}

#ifdef APPL_LCN_ENABLED

/*!
*******************************************************************************
**
** \brief Get LCN of a given service.
**
** \param pServLoc  Service location.
** \param pLcn      Destination pointer for LCN to be returned.
**
** \return
**      - #FAPI_OK      if successful
******************************************************************************/
int32_t SLIST_GetLcn(const SYS_SERVLOC_S * pServLoc, uint16_t * pLcn)
{
    SLIST_SERVIDX       servIdx;
    int32_t             retVal;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else if(pServLoc == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if(pLcn == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    servIdx = SLIST_IntGetServIdx(pServLoc);
    if(servIdx < 0)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        *pLcn = slDat->lstLcn.cn[servIdx];
        retVal = FAPI_OK;
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}

/*!
*******************************************************************************
**
** \brief Get service location by a given LCN.
**
** \param lcn       Requested logical channel number.
** \param listId    List to be searched (SLIST_NONE: all lists).
** \param pServLoc  Destination pointer for service location to be returned.
**
** \return
**      - #FAPI_OK      if successful
******************************************************************************/
int32_t SLIST_GetServLocByLcn(uint16_t lcn, SLIST_TYPE listId,
                              SYS_SERVLOC_S * pServLoc)
{
    SLIST_SERVIDX       servIdx = SLIST_SERVIDX_NONE;
    SLIST_LIST_S       *pLst = NULL;
    int32_t             retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
    int16_t             index = -1;
    int16_t             cnt;
    uint8_t             lstCnt;

    if(slDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    else if(pServLoc == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if(lcn < 1)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    if(listId != SLIST_NONE)
    {
        pLst = SLIST_GetList(listId, NULL, NULL);
        if(pLst == NULL)
        {
            return APPL_SMARTGO_ERR_BAD_PARAMETER;
        }
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    /* ---- get service matching given LCN --------------------------------- */

    for(cnt = 0; cnt < slDat->lstLcn.numEntries; cnt++)
    {
        /* Index cannot be greater than passed LCN value */
        if(cnt >= (int16_t) lcn)
        {
            break;
        }

        if(slDat->lstLcn.cn[slDat->lstLcn.servIdx[cnt]] == lcn)
        {
            servIdx = slDat->lstLcn.servIdx[cnt];
            break;
        }
    };

    if(servIdx < 0)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    /* --------------------------------------------------------------------- */

    else if(pLst != NULL)       /* target list has been specified */
    {
        index = SLIST_FindInList(pLst, servIdx);

        if(index >= 0)
        {
            pServLoc->listId = listId;
            pServLoc->index = index;

            retVal = FAPI_OK;
        }
    }
    else                        /* search all global lists */
    {
        for(lstCnt = 0; lstCnt < slDat->numGlobLsts; lstCnt++)
        {
            pLst = slDat->lstGlob + (lstCnt & SLIST_MASK_GLOB);

            index = SLIST_FindInList(pLst, servIdx);
            if(index >= 0)
            {
                pServLoc->listId = SLIST_OFFSET_GLOB + lstCnt;
                pServLoc->index = index;

                retVal = FAPI_OK;
                break;
            }
        }
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}

int32_t SLIST_LCN_GenerateLists(void)
{
    SLIST_ITEM_S       *pSrc;
    SLIST_SERVIDX       srcCnt;
    uint16_t            cnt;

    // TODO: stop live TV?

    /* clear global lists first */
    for(cnt = 0; cnt < slDat->numGlobLsts; cnt++)
    {
        SLIST_ClearInternal(slDat->lstGlob + cnt, FFALSE);
    }

    /* clear LCN-related lists */
    (void) SLIST_LCN_ClearLists();

    /* add each service list entry */
    for(srcCnt = 0; srcCnt < slDat->servLst.stat.afterLast; srcCnt++)
    {
        pSrc = STATLST_GetEntry(&(slDat->servLst), srcCnt);

        if(pSrc != NULL)
        {
            (void) SLIST_LCN_AddEntry(srcCnt, NULL);
        }
    }

    FAPI_SYS_PRINT_DEBUG(4,"[SLST] LCN list generated:\n");
    FAPI_SYS_PRINT_DEBUG(4,"[SLST] numEntries    : %i\n", slDat->lstLcn.numEntries);
    FAPI_SYS_PRINT_DEBUG(4,"[SLST] startVariant  : %i\n", slDat->lstLcn.startVariant);

    return FAPI_OK;
}

FSTATIC int32_t SLIST_LCN_ClearLists(void)
{
    uint16_t            cnt;

    slDat->lstLcn.numEntries = 0;
    slDat->lstLcn.startVariant = 0;

    for(cnt = 0; cnt < SLIST_MAX_ALL; cnt++)
    {
        slDat->lstLcn.cn[cnt] = 0;
    }

    return FAPI_OK;
}

FSTATIC int32_t SLIST_LCN_RestoreNonLcnLists(void)
{
    SLIST_ITEM_S       *pSrc;
    SLIST_SERVIDX       srcCnt;
    SLIST_TYPE          listId;
    SLIST_LIST_S       *pLst;
    uint16_t            cnt;

    for(cnt = 0; cnt < slDat->numGlobLsts; cnt++)
    {
        SLIST_ClearInternal(slDat->lstGlob + cnt, FFALSE);
    }

    /* add each service list entry */
    for(srcCnt = 0; srcCnt < slDat->servLst.stat.afterLast; srcCnt++)
    {
        pSrc = STATLST_GetEntry(&(slDat->servLst), srcCnt);

        if(pSrc == NULL)
        {
            continue;
        }

        listId = SLIST_GetListId(pSrc);

        pLst = SLIST_GetList(listId, NULL, NULL);

        if(pLst != NULL)
        {
            (void)SLIST_InsertRefEntry (pLst, pLst->sizeCurr, srcCnt);
        }
    }

    return FAPI_OK;
}

FSTATIC int32_t SLIST_LCN_AddEntry(SLIST_SERVIDX idx, SYS_SERVLOC_S * pServLoc)
{
    SLIST_LCN_S        *pLcn = &(slDat->lstLcn);
    SLIST_ITEM_S       *pSrc;
    SLIST_ITEM_S       *pRef;
    SLIST_SERVIDX       tmpIdx;
    int16_t             newIndex = -1;
    uint16_t            newCn = 0;
    uint16_t            refCn;
    int16_t             cnt;
    int32_t             retVal;
    int16_t             shiftOffset = 0;
    SYS_SERVLOC_S       newServLoc;

    /* space left for one more entry? */
    if(pLcn->numEntries >= SLIST_MAX_ALL)
    {
        return SMARTGO_SYS_ERR_CAPACITY;
    }

    pSrc = STATLST_GetEntry(&(slDat->servLst), idx);
    if(pSrc == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    FAPI_SYS_PRINT_DEBUG(4,"SLIST_LCN_AddEntry: '%s' ", pSrc->name);

    /*  ----- non-LCN service or invalid LCN -------------------------------- */

    if((pSrc->lcn == 0) ||
       ((uint16_t) pSrc->lcn >= slDat->params.variantRangeStart))
    {
        /* Test for maximum size of variant area */
        if(((uint16_t) (pLcn->numEntries - pLcn->startVariant)) >=
           slDat->params.variantRangeLength)
        {
            return SMARTGO_SYS_ERR_CAPACITY;
        }

        /* add at the end of variant area */
        newIndex = pLcn->numEntries;

        if(newIndex > 0)
        {
            /* Use previous service in list as reference for channel number */
            newCn = pLcn->cn[pLcn->servIdx[newIndex - 1]] + 1;
        }

        /* take non-LCN offset into account */
        if(newCn < slDat->params.variantRangeStart)
        {
            newCn = slDat->params.variantRangeStart;
        }

        /* End of list: no modification of other channel numbers necessary */
        shiftOffset = 0;

        FAPI_SYS_PRINT_DEBUG(4,"NON-LCN: idx %i lcn %i ", newIndex, newCn);
    }

    /* ----- valid LCN service --------------------------------------------- */

    else
    {
        FAPI_SYS_PRINT_DEBUG(4,"LCN (%i) ", pSrc->lcn);

        /* Parse LCN area */
        for(cnt = 0; cnt < pLcn->startVariant; cnt++)
        {
            refCn = pLcn->cn[pLcn->servIdx[cnt]];

            if(refCn > (uint16_t) pSrc->lcn)    /* insertion slot found */
            {
                FAPI_SYS_PRINT_DEBUG(4,"SLOT FOUND (idx %i: %i > %i) ", cnt, refCn,
                           pSrc->lcn);
                break;
            }
            else if(refCn == pSrc->lcn) /* LCN already exists */
            {
                FAPI_SYS_PRINT_DEBUG(4,"DOUBLE ENTRY ");

                /* Compare signal quality of both services and choose
                   strongest for the original LCN position. */
                pRef = STATLST_GetEntry(&(slDat->servLst), pLcn->servIdx[cnt]);
                if(pRef == NULL)
                {
                    return SMARTGO_SYS_ERR_INTERNAL;
                }

                if(SLIST_LCN_GetPreferredService(pSrc, pRef) == pSrc)
                {
                    /* Remove entry to be overwritten from its global list,
                       will be re-inserted later. */
                    (void)SLIST_LCN_RefEntryRemove(cnt);

                    /* service to be added is stronger: Swap indexes. */
                    tmpIdx = pLcn->servIdx[cnt];
                    pLcn->servIdx[cnt] = idx;
                    pLcn->cn[idx] = pSrc->lcn;
                    idx = tmpIdx;
                    pSrc = pRef;

                    /* Add new entry to a global list */
                    (void)SLIST_LCN_RefEntryAdd(cnt, NULL);
                }

                /* move double entry to end of variant area */

                /* Test for maximum size of variant area */
                if(((uint16_t) (pLcn->numEntries - pLcn->startVariant)) >=
                   slDat->params.variantRangeLength)
                {
                    return SMARTGO_SYS_ERR_CAPACITY;
                }

                newIndex = pLcn->numEntries;

                if(newIndex > 0)
                {
                    /* Use previous service in list as reference for channel number */
                    newCn = pLcn->cn[pLcn->servIdx[newIndex - 1]] + 1;
                }

                /* take non-LCN offset into account */
                if(newCn < slDat->params.variantRangeStart)
                {
                    newCn = slDat->params.variantRangeStart;
                }

                shiftOffset = 1;
                break;
            }
        }

        /* no double entries found: insert entry into/at end of LCN area */
        if(newIndex < 0)
        {
            newIndex = cnt;
            newCn = pSrc->lcn;

            FAPI_SYS_PRINT_DEBUG(4,"POS FOUND: idx %i lcn %i ", newIndex, newCn);

            if((newIndex >= pLcn->startVariant) &&
               (newIndex < pLcn->numEntries))
            {
                /* Test if CN of last LCN entry exceeds CN of first non-LCN
                   entry */
                if(newCn >= pLcn->cn[pLcn->servIdx[newIndex]])
                {
                    /* should never happen! */
                    shiftOffset = (int16_t)((newCn - pLcn->cn[pLcn->servIdx[newIndex]]) + 1);
                }
                else
                {
                    shiftOffset = 0;
                }
            }
            else
            {
                /* Inserted into LCN area: no shifting of CNs necessary */
                shiftOffset = 0;
            }

            pLcn->startVariant++;
        }
    }

    FAPI_SYS_PRINT_DEBUG(4,"\n");

    /* --------------------------------------------------------------------- */

    if(newIndex >= 0)
    {
        pLcn->cn[idx] = newCn;  /* store channel number */

        /* shift list entries */
        for(cnt = pLcn->numEntries; cnt > newIndex; cnt--)
        {
            pLcn->servIdx[cnt] = pLcn->servIdx[cnt - 1];
        }

        pLcn->servIdx[newIndex] = idx;
        pLcn->numEntries++;

        /* shift channel numbers */
        if(shiftOffset > 0)
        {
            for(cnt = (newIndex + 1); cnt < pLcn->numEntries; cnt++)
            {
                pLcn->cn[pLcn->servIdx[cnt]] += (uint16_t)shiftOffset;
            }
        }

        /* ----- Sort new entry into global list --------------------------- */

        retVal = SLIST_LCN_RefEntryAdd(newIndex, &newServLoc);

        if((retVal == FAPI_OK) && (pServLoc != NULL))
        {
            *pServLoc = newServLoc;
        }
    }
    else
    {
        /* should never happen */
        retVal = SMARTGO_SYS_ERR_INTERNAL;
    }

    return retVal;
}

FSTATIC int32_t SLIST_LCN_RemoveEntry(SLIST_SERVIDX idx)
{
    SLIST_LCN_S        *pLcn = &(slDat->lstLcn);
    SLIST_ITEM_S       *pDst;
    int16_t             dstIndex;
    int16_t             cnt;

    pDst = STATLST_GetEntry(&(slDat->servLst), idx);
    if(pDst == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    /* Search for entry in LCN-sorted list */
    for(dstIndex = 0; dstIndex < pLcn->numEntries; dstIndex++)
    {
        if(pLcn->servIdx[dstIndex] == idx)
        {
            break;
        }
    }

    if(dstIndex >= pLcn->numEntries)
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    if(dstIndex < pLcn->startVariant)
    {
        pLcn->startVariant--;
    }

    pLcn->cn[idx] = 0;

    /* shift list entries */
    for(cnt = dstIndex; (cnt + 1) < pLcn->numEntries; cnt++)
    {
        pLcn->servIdx[cnt] = pLcn->servIdx[cnt + 1];
    }

    pLcn->numEntries--;

    if(pLcn->startVariant > pLcn->numEntries)
    {
        pLcn->startVariant--;
    }

    return FAPI_OK;
}

FSTATIC int32_t SLIST_LCN_ResortEntry(SYS_SERVLOC_S * pServLoc,
                                      SLIST_SERVIDX idx)
{
    SLIST_ITEM_S       *pItem;
    SLIST_LIST_S       *pLst;
    int32_t             retVal;

    pItem = STATLST_GetEntry(&(slDat->servLst), idx);
    if(pItem == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    pLst = SLIST_GetList(pServLoc->listId, NULL, NULL);
    if(pLst == NULL)
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    /* remove item from LCN list */
    retVal = SLIST_LCN_RemoveEntry(idx);
    if(retVal != FAPI_OK)
    {
        return retVal;
    }

    /* remove item from global list */
    (void)SLIST_RemoveRefEntry(pLst, (uint16_t)pServLoc->index, FTRUE);

    /* re-add to LCN list and global list */
    return SLIST_LCN_AddEntry(idx, pServLoc);
}

/*
lcnIdx:     Index in global LCN list.
pServLoc:   optional: new service location if successful
*/
FSTATIC int32_t SLIST_LCN_RefEntryAdd(int16_t lcnIdx, SYS_SERVLOC_S * pServLoc)
{
    SLIST_LCN_S        *pLcn = &(slDat->lstLcn);
    SLIST_LIST_S       *pLst;
    SLIST_ITEM_S       *pItem;
    SYS_SERVLOC_S       newServLoc;
    int16_t             cnt;
    uint16_t            refLcn;
    int32_t             retVal;

    pItem = STATLST_GetEntry(&(slDat->servLst), pLcn->servIdx[lcnIdx]);
    if(pItem == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    newServLoc.listId = SLIST_GetListId(pItem);
    newServLoc.index = -1;

    pLst = SLIST_GetList(newServLoc.listId, NULL, NULL);
    if(pLst == NULL)
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    refLcn = pLcn->cn[pLcn->servIdx[lcnIdx]];

    /* insert by CN order */
    for(cnt = 0; cnt < (int16_t) pLst->sizeCurr; cnt++)
    {
        if(pLcn->cn[pLst->refIdx[cnt]] >= refLcn)
        {
            newServLoc.index = cnt;
            break;
        }
    }

    if ( newServLoc.index < 0 )    /* append at list end */
    {
        newServLoc.index = (int16_t)pLst->sizeCurr;
    }

    retVal = SLIST_InsertRefEntry (pLst, (uint16_t)(newServLoc.index),
                                   pLcn->servIdx[lcnIdx]);
    if(retVal != FAPI_OK)
    {
        return retVal;
    }

    if(pServLoc != NULL)
    {
        *pServLoc = newServLoc;
    }

    return FAPI_OK;
}

/*
lcnIdx: Index in global LCN list
*/
FSTATIC int32_t SLIST_LCN_RefEntryRemove(int16_t lcnIdx)
{
    SLIST_LCN_S*    pLcn   = &(slDat->lstLcn);
    int32_t         retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
    SLIST_TYPE      listId;
    SLIST_LIST_S*   pLst;
    SLIST_ITEM_S*   pItem;
    uint16_t        cnt;
    SLIST_SERVIDX   refServIdx;

    refServIdx = pLcn->servIdx[lcnIdx];

    pItem = STATLST_GetEntry(&(slDat->servLst), refServIdx);
    if(pItem == NULL)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    listId = SLIST_GetListId(pItem);

    pLst = SLIST_GetList(listId, NULL, NULL);
    if(pLst == NULL)
    {
        return SMARTGO_SYS_ERR_INTERNAL;
    }

    for ( cnt = 0; cnt < pLst->sizeCurr; cnt++ )
    {
        if ( pLst->refIdx[cnt] == refServIdx )
        {
            retVal = SLIST_RemoveRefEntry(pLst, cnt, FTRUE);
            break;
        }
    }

    return retVal;
}

/*
On equality second service list item is declared strongest.
*/
FSTATIC const SLIST_ITEM_S *SLIST_LCN_GetPreferredService(const SLIST_ITEM_S *
                                                          pServ1,
                                                          const SLIST_ITEM_S *
                                                          pServ2)
{
    TPLIST_ITEM_S       tp1;
    TPLIST_ITEM_S       tp2;

    memset(&tp1, 0, sizeof(tp1));
    memset(&tp2, 0, sizeof(tp2));

    (void)TPLIST_GetDataByIndex(&(pServ1->network), pServ1->tpIdx, &tp1);
    (void)TPLIST_GetDataByIndex(&(pServ2->network), pServ2->tpIdx, &tp2);

    if( (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT) ||
        (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT_T2) ||
        (SYS_GetAppProfile() == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW))
    {
        /* Prefer multiplexes originated in the UK (identified by specific
           original_network_id). */

        if((tp1.onId == 0x233A) && (tp2.onId != 0x233A))
        {
            return pServ1;
        }
        else if((tp2.onId == 0x233A) && (tp1.onId != 0x233A))
        {
            return pServ2;
        }
    }

    if(tp1.sigQuality > tp2.sigQuality)
    {
        return pServ1;
    }
    else
    {
        return pServ2;
    }
}

#endif /* APPL_LCN_ENABLED */

/* ------------------------------------------------------------------------- */

FSTATIC int8_t compByName(SLIST_ITEM_S * comp1, SLIST_ITEM_S * comp2)
{
    SYS_STRING_S    str1;
    SYS_STRING_S    str2;
    uint32_t        pos1    = 0;
    uint32_t        pos2    = 0;
    SYS_CHAR        char1;
    SYS_CHAR        char2;
    uint32_t        val1;
    uint32_t        val2;

    memset(&str1,0,sizeof(SYS_STRING_S));
    memset(&str2,0,sizeof(SYS_STRING_S));

    str1.cs  = comp1->nameCharSet;
    str1.chr = comp1->name;

    str2.cs  = comp2->nameCharSet;
    str2.chr = comp2->name;

    while(1)
    {
        pos1 += STR_CharGet (str1, pos1, &char1);
        pos2 += STR_CharGet (str2, pos2, &char2);

        val1 = char1;
        val2 = char2;

        /* test for string end */
        if(val1 == 0)
        {
            if(val2 == 0)
                return 0;       /* strings are equal */
            else
                return -1;      /* string 1 is shorter (->smaller) */
        }
        else if(val2 == 0)
        {
            return 1;           /* string 1 is longer */
        }

        /* compare for equality */
        else if(val1 == val2)
        {
            /* do nothing, wait for next character */
        }

        /* non-equal characters */
        else
        {
            /* Compare a..z/A...Z characters in a special way:

               Their value is normalized to allow case-insensitive comparison
               and also to ensure that they are located "above" any
               other character.
             */

            /* capital letters */
            if ( (val1 > 0x0040) && (val1 < 0x005B) )
            {
                val1 = (val1 + 0x20) | 0x10000;
            }
            /* small letters */
            else if ( (val1 > 0x0060) && (val1 < 0x007B) )
            {
                val1 |= 0x10000;
            }

            /* capital letters */
            if ( (val2 > 0x0040) && (val2 < 0x005B) )
            {
                val2 = (val2 + 0x0020) | 0x10000;
            }
            /* small letters */
            else if ( (val2 > 0x0060) && (val2 < 0x007B) )
            {
                val2 |= 0x10000;
            }

            if(val1 < val2)
            {
                return -1;
            }
            else if(val1 > val2)
            {
                return 1;
            }
            else
            {
                /* do nothing, wait for next character */
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void SLIST_GlobalEntryRemove(int16_t servIdx)
{
    uint8_t cnt;

    #ifdef APPL_RECTIMER_ENABLED

    RECTIMER_ITEM_S     timerItem;
    int16_t             timerCnt;
    uint16_t            numTimers;

    #endif /* APPL_RECTIMER_ENABLED */

    if(servIdx < 0)
    {
        return;
    }

    #ifdef APPL_LCN_ENABLED

    if(slDat->params.sortByLcn)
    {
        (void)SLIST_LCN_RemoveEntry(servIdx);
    }

    #endif /* APPL_LCN_ENABLED */

    /* remove service data */
    (void) STATLST_RemoveEntry(&(slDat->servLst), servIdx);
    slDat->servClean = FFALSE;

    #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

    (void) STATLST_RemoveEntry(&(slDat->caLst), servIdx);
    slDat->caClean = FFALSE;

    #endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */

    /* modify all affected references in user lists  */
    for ( cnt = 0; cnt < slDat->numUsrLsts; cnt++ )
    {
        /* Remove all entries in user lists referencing deleted entry */
        SLIST_UserRemoveReference(cnt, servIdx);
    }

    /* clear EIT database entries referencing removed service */
    (void)EIT_InvalidateService(servIdx);

    #ifdef APPL_PVR_ENABLED

    /* handle all recording operations on service to be removed */
    (void)PVRSYS_InvalidateService(servIdx);

    #endif /* APPL_PVR_ENABLED */

    #ifdef APPL_RECTIMER_ENABLED

    timerCnt = 0;

    /* remove all record timers referencing service to be removed. */
    while((RECTIMER_GetNumTimers(&numTimers) == FAPI_OK) &&
          ((uint16_t) timerCnt < numTimers))
    {
        if((RECTIMER_Get(timerCnt, &timerItem) == FAPI_OK) &&
           (timerItem.serviceIndex == servIdx))
        {
            if(RECTIMER_Remove(timerCnt) != FAPI_OK)
            {
                timerCnt++;
            }
        }
        else
        {
            timerCnt++;
        }
    }

    #endif /* APPL_RECTIMER_ENABLED */
}

FSTATIC SLIST_TYPE SLIST_GetListId(const SLIST_ITEM_S * pItem)
{
    SLIST_TYPE          listId;

    if(SLIST_FlagIsSet(pItem, SLIST_FLAG_HIDDEN_UK) ||
       ((slDat->params.useLcnVisibility) &&
        (!(SLIST_FlagIsSet(pItem, SLIST_FLAG_VISIBLESVC)))))
    {
        listId = SLIST_INVISIBLE;
    }
    else
    {
        switch(pItem->serviceType)
        {
            case SERVICE_TYPE_DIGITAL_TV:               /* FALLTHROUGH */
            case SERVICE_TYPE_DIGITAL_TV_ADVANCED:      /* FALLTHROUGH */
            case SERVICE_TYPE_DIGITAL_TV_HD:            /* FALLTHROUGH */
            case SERVICE_TYPE_DIGITAL_TV_HD_ADVANCED:   /* FALLTHROUGH */
            case SERVICE_TYPE_DATA_BROADCAST:
                listId = SLIST_TV;
                break;
            case SERVICE_TYPE_DIGITAL_RADIO: /* FALLTHROUGH */
            case SERVICE_TYPE_DIGITAL_RADIO_ADVANCED:
                listId = SLIST_RADIO;
                break;
            default:
                listId = SLIST_NONE;
                break;
        }
    }

    return listId;
}

/* NOTE: This function does not need to be protected by mutex, number of lists
         does not change at runtime. */
FSTATIC SLIST_LIST_S *SLIST_GetList(SLIST_TYPE listId, uint8_t * pIntIdx,
                                    fbool_t* pIsGlob)
{
    fbool_t             isGlob;
    uint8_t             intIdx;

    if(listId == SLIST_NONE)
        return NULL;

    isGlob = (listId >= SLIST_OFFSET_GLOB) ? FTRUE : FFALSE;
    intIdx = listId & SLIST_MASK_GLOB;

    if(isGlob)
    {
        if(intIdx >= slDat->numGlobLsts)
            return NULL;
    }
    else if(intIdx >= slDat->numUsrLsts)
        return NULL;

    if(pIntIdx != NULL)
        (*pIntIdx) = intIdx;
    if(pIsGlob != NULL)
        (*pIsGlob) = isGlob;

    return isGlob ? (slDat->lstGlob + intIdx) : (slDat->lstUsr + intIdx);
}

FSTATIC int16_t SLIST_FindInList(const SLIST_LIST_S * pLst,
                                 SLIST_SERVIDX servIdx)
{
    int16_t             index;

    for(index = 0; (uint16_t) index < pLst->sizeCurr; index++)
    {
        if(pLst->refIdx[index] == servIdx)
        {
            return index;
        }
    }

    return -1;
}

FSTATIC int32_t SLIST_InsertRefEntry(SLIST_LIST_S * pLst, uint16_t lstIdx,
                                     SLIST_SERVIDX servIdx)
{
    int16_t cnt;

    if ( pLst->sizeCurr >= pLst->entriesMax )
    {
        return SMARTGO_SYS_ERR_CAPACITY;
    }

    /* shift subsequent entries */
    for(cnt = (int16_t)pLst->sizeCurr - 1; cnt >= (int16_t)lstIdx; cnt--)
    {
        pLst->refIdx[cnt + 1] = pLst->refIdx[cnt];
    }

    pLst->sizeCurr++;

    /* insert requested element */
    pLst->refIdx[lstIdx] = servIdx;
    pLst->clean          = FFALSE;

    return FAPI_OK;
}

FSTATIC int32_t SLIST_RemoveRefEntry(SLIST_LIST_S* pLst, uint16_t lstIdx,
                                     fbool_t shiftFollowing)
{
    uint16_t cnt;

    if ( lstIdx >= pLst->sizeCurr )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    if ( shiftFollowing )
    {
        /* shift subsequent entries */
        for ( cnt = lstIdx; cnt < pLst->sizeCurr; cnt++ )
        {
            pLst->refIdx[cnt] = pLst->refIdx[cnt + 1];
        }

        pLst->sizeCurr--;

        pLst->refIdx[pLst->sizeCurr] = -1;
    }
    else
    {
        pLst->refIdx[lstIdx] = -1;
    }

    pLst->clean = FFALSE;
    return FAPI_OK;
}

/*
Allocates and clears global list
*/
FSTATIC int32_t SLIST_ListAllocate(SLIST_LIST_S * pLst, uint16_t numElements)
{
    int32_t retVal;

    retVal = STR_BufOpen (&(pLst->nameBuf), SLIST_NAME_LENGTH, NULL);
    if ( retVal != FAPI_OK ) return retVal;

    pLst->entriesMax = 0;
    pLst->sizeCurr   = 0;
    pLst->clean      = FFALSE;
    pLst->refIdx     = UTILS_Malloc((numElements * sizeof(int16_t)));

    if ( pLst->refIdx == NULL )
    {
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;
    }

    pLst->entriesMax = numElements;

    SLIST_ClearInternal (pLst, FFALSE);
    return FAPI_OK;
}


/*
clears a list
*/
FSTATIC void SLIST_ClearInternal(SLIST_LIST_S * pLst, fbool_t clearName)
{
    uint16_t cnt;

    pLst->sizeCurr = 0;
    pLst->clean    = FFALSE;

    if(pLst->refIdx != NULL)
    {
        for(cnt = 0; cnt < pLst->entriesMax; cnt++)
        {
            pLst->refIdx[cnt] = -1;
        }
    }

    if ( clearName == FTRUE )
    {
        (void)STR_BufClear (&(pLst->nameBuf));
    }
}

FSTATIC int16_t SLIST_GlobSearchItem(uint8_t listIdx,
                                     const SLIST_ITEM_S * pItem)
{
    uint16_t            cnt;
    SLIST_LIST_S       *pLst = slDat->lstGlob + listIdx;
    int16_t             refIdx;
    SLIST_ITEM_S       *pEntry;

    for(cnt = 0; cnt < pLst->sizeCurr; cnt++)
    {
        refIdx = pLst->refIdx[cnt];

        if(refIdx < 0)
            continue;

        pEntry = STATLST_GetEntry(&(slDat->servLst), refIdx);
        if(pEntry == NULL)
        {
            FAPI_SYS_PRINT_DEBUG(2,"[AV-LIST] Bad reference!\n");
            continue;
        }

        if((pEntry->serviceId == pItem->serviceId) &&
           (pEntry->tpIdx == pItem->tpIdx) &&
           (pEntry->network.satIdx == pItem->network.satIdx) &&
           (pEntry->network.type == pItem->network.type))
        {
            return (int16_t) cnt;
        }
    }

    return -1;
}

/* -------------------------------------------------------------------------- */

FSTATIC int16_t SLIST_UserSearchItem(uint8_t listIdx, SLIST_REF_S * ref)
{
    uint16_t            cnt;
    SLIST_LIST_S       *pLst = slDat->lstUsr + listIdx;
    int16_t             servIdx;

    if(listIdx >= slDat->numUsrLsts)
        return -1;
    if(ref->index < 0)
        return -1;

    servIdx = slDat->lstGlob[ref->listNum].refIdx[ref->index];

    for(cnt = 0; cnt < pLst->sizeCurr; cnt++)
    {
        if(pLst->refIdx[cnt] == servIdx)
            return (int16_t) cnt;
    }

    return -1;
}

FSTATIC void SLIST_UserRemoveReference(uint8_t listIdx, int16_t servIdx)
{
    uint16_t            cnt;
    SLIST_LIST_S       *pLst = slDat->lstUsr + listIdx;

    for(cnt = 0; cnt < pLst->sizeCurr; cnt++)
    {
        if(pLst->refIdx[cnt] == servIdx)
        {
            /* invalidate reference entry */
            pLst->refIdx[cnt] = -1;
            pLst->clean       = FFALSE;
        }
    }
}

/*---------------------------------------------------------------------------*/

FSTATIC fbool_t SLIST_ReferenceIsValid(SLIST_REF_S * ref)
{
    if ( ref->listNum >= slDat->numGlobLsts ) return FFALSE;

    else if((ref->index < 0) ||
            (ref->index >= (int16_t) (slDat->lstGlob[ref->listNum].sizeCurr)))
    {
        return FFALSE;
    }

    else return FTRUE;
}

FSTATIC fbool_t SLIST_ReadList(SLIST_LIST_S * pLst, void *file)
{
    int32_t             res;
    SLIST_FILEHDR_S     fileHdr;

    /* ----- Read header --------------------------------------------------- */

    res = FLASHFS_Read(file, sizeof(SLIST_FILEHDR_S), &fileHdr, NULL);

    if(res != (int32_t)sizeof(SLIST_FILEHDR_S))
    {
        /* file read error, file too small */
        return FFALSE;
    }
    else if((fileHdr.id        != FILEID_SERVICEDATA) ||
            (fileHdr.revision  != FILEREV_SERVICEDATA) ||
            (fileHdr.numEntries > pLst->entriesMax))
    {
        /* invalid identifier */
        return FFALSE;
    }

    /* Read list entries */

    (void)STR_BufAppend (&(pLst->nameBuf), 0, STR_Build(fileHdr.nameCharSet,
                                                  fileHdr.nameStr),
                   fileHdr.nameCharSet);

    pLst->sizeCurr = fileHdr.numEntries;

    if(pLst->sizeCurr > 0)
    {
        res = FLASHFS_Read(file, pLst->sizeCurr * sizeof(int16_t),
                           pLst->refIdx, NULL);

        if(res != (int32_t)(pLst->sizeCurr * sizeof(int16_t)))
        {
            return FFALSE;
        }
    }

    pLst->clean = FTRUE;
    return FTRUE;
}

FSTATIC uint32_t SLIST_ListGetMaxDataSize (uint16_t numElements)
{
    return sizeof(SLIST_FILEHDR_S)
        + (numElements * sizeof(int16_t));
}

/*
force = FTRUE: force saving; FFALSE: save only if modified
*/
FSTATIC fbool_t SLIST_WriteList(SLIST_LIST_S* pLst, void* file)
{
    SLIST_FILEHDR_S     fileHdr;
    int32_t             res;

    /* Write header */
    memset (&fileHdr, 0, sizeof(SLIST_FILEHDR_S));

    fileHdr.id          = FILEID_SERVICEDATA;
    fileHdr.revision    = FILEREV_SERVICEDATA;
    fileHdr.numEntries  = pLst->sizeCurr;
    fileHdr.nameCharSet = SYS_CHARSET_UNDEFINED;
    fileHdr.nameStr[0]  = 0;

    (void)STR_Copy (pLst->nameBuf.string, fileHdr.nameStr, &(fileHdr.nameCharSet),
              pLst->nameBuf.string.cs, SLIST_NAME_LENGTH);

    if ( FLASHFS_Write(file, sizeof(SLIST_FILEHDR_S), &fileHdr, NULL)
         != (int32_t)sizeof(SLIST_FILEHDR_S))
    {
        return FFALSE;
    }

    /* Write list entries */
    if(pLst->sizeCurr > 0)
    {
        res = FLASHFS_Write(file, pLst->sizeCurr * sizeof(int16_t),
                            pLst->refIdx, NULL);

        if ( res != (int32_t)(pLst->sizeCurr * sizeof(int16_t)) )
        {
            return FFALSE;
        }
    }

    pLst->clean = FTRUE;
    return FTRUE;
}

/* -------------------------------------------------------------------------- */

FSTATIC fbool_t SLIST_ServEntriesLoad(void)
{
    fbool_t             retVal = FFALSE;
    int32_t             res;
    SLIST_SERVFILEHDR_S fileHdr;
    uint32_t            cnt;
    fbool_t             abort = FFALSE;

    FLASHFS_HANDLE      file;

    /* If list is marked as invalid, trigger setting of defaults */
    if(!lstValid)
        return FFALSE;

    file = FLASHFS_Open(FILEID_SERVICEDATA, FLASHFS_OPEN_READ, NULL);

    if(!file)
    {
        FAPI_SYS_PRINT_DEBUG(2,"File opening failed!\n");
        return FFALSE;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    for(;;)
    {
        /* ----- read file header ------------------------------------------ */

        res = FLASHFS_Read(file, sizeof(SLIST_SERVFILEHDR_S), &fileHdr,
                           NULL);

        if(res != (int32_t)sizeof(SLIST_SERVFILEHDR_S))
        {
            /* file read error, file too small */
            break;
        }
        else if((fileHdr.id != FILEID_SERVICEDATA) ||
                (fileHdr.revision != FILEREV_SERVICEDATA))
        {
            /* invalid identifier */
            break;
        }

        /* ----- read service list entries --------------------------------- */

        if ( STATLST_LoadFromFS(&(slDat->servLst), FFALSE, file) == FFALSE )
        {
            break;
        }

        /* ----- read global lists ----------------------------------------- */

        for(cnt = 0; cnt < slDat->numGlobLsts; cnt++)
        {
            if ( SLIST_ReadList(slDat->lstGlob + cnt, file) == FFALSE )
            {
                abort = FTRUE;
                break;
            }
        }

        if ( abort == FTRUE ) break;

        /* ----- read user lists ------------------------------------------- */

        for(cnt = 0; cnt < slDat->numUsrLsts; cnt++)
        {
            if ( SLIST_ReadList(slDat->lstUsr + cnt, file) == FFALSE )
            {
                abort = FTRUE;
                break;
            }
        }

        if ( abort == FTRUE ) break;

        /* ----- read file hdr at end of file for verification ------------- */

        res = FLASHFS_Read (file, sizeof(SLIST_SERVFILEHDR_S), &fileHdr,
                            NULL);

        if(res != (int32_t)sizeof(SLIST_SERVFILEHDR_S))
        {
            break;
        }
        else if((fileHdr.id != FILEID_SERVICEDATA) ||
                (fileHdr.revision != FILEREV_SERVICEDATA))
        {
            break;
        }

        /* ----- read CA info ---------------------------------------------- */

        #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

        if(!SLIST_CaInfoLoad())
            break;

        #endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */

        /* ----------------------------------------------------------------- */

        slDat->params = fileHdr.params;

        #ifdef APPL_LCN_ENABLED

        if(slDat->params.sortByLcn)
        {
            (void)SLIST_LCN_GenerateLists();
        }
        else
        {
            (void)SLIST_LCN_ClearLists();
        }

        #endif /* APPL_LCN_ENABLED */

        retVal = FTRUE;
        break; //end of for(;;)
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    (void)FLASHFS_Close(file);

    if ( retVal == FFALSE )
    {

    }
    else
    {
        slDat->servClean = FTRUE;
    }

    return retVal;
}

FSTATIC fbool_t SLIST_ServEntriesSave (fbool_t force)
{
    fbool_t                 retVal = FFALSE;
    SLIST_SERVFILEHDR_S     fileHdr;
    uint32_t                cnt;
    fbool_t                 abort = FFALSE;
    FLASHFS_HANDLE          file  = NULL;

    /* saving necessary? */
    if ( (force            == FFALSE) &&
         (slDat->servClean == FTRUE) )
    {
        /* check if global lists have been modified */
        for(cnt = 0; cnt < slDat->numGlobLsts; cnt++)
        {
            if ( slDat->lstGlob[cnt].clean == FFALSE ) break;
        }

        if(cnt >= slDat->numGlobLsts)
        {
            /* check if user lists have been modified */
            for(cnt = 0; cnt < slDat->numUsrLsts; cnt++)
            {
                if ( slDat->lstUsr[cnt].clean == FFALSE ) break;
            }

            if ( cnt >= slDat->numUsrLsts ) return FTRUE;
        }
    }

    /* Store also lists referred by service lists */
    #ifdef APPL_FESAT_ENABLED
    (void)TPLIST_Save(FE_SYSTEM_TYPE_SATELLITE, FFALSE);
    #endif /* APPL_FESAT_ENABLED */

    #ifdef APPL_FETERR_ENABLED
    (void)TPLIST_Save(FE_SYSTEM_TYPE_TERRESTRIAL, FFALSE);
    #endif /* APPL_FETERR_ENABLED */

    #ifdef APPL_FECABLE_ENABLED
    (void)TPLIST_Save(FE_SYSTEM_TYPE_CABLE, FFALSE);
    #endif /* APPL_FECABLE_ENABLED */

    file = FLASHFS_Open(FILEID_SERVICEDATA, FLASHFS_OPEN_WRITE, NULL);

    if ( file == NULL )
    {
        return FFALSE;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    for(;;)
    {
        /* ----- write file header ----------------------------------------- */

        memset(&fileHdr,0,sizeof(SLIST_SERVFILEHDR_S));

        fileHdr.id = FILEID_SERVICEDATA;
        fileHdr.revision = FILEREV_SERVICEDATA;
        fileHdr.params = slDat->params;

        if ( FLASHFS_Write(file, sizeof(SLIST_SERVFILEHDR_S), &fileHdr,
                           NULL ) != (int32_t)sizeof(SLIST_SERVFILEHDR_S) )
        {
            break;
        }

        /* ----- write service list entries -------------------------------- */

        if ( STATLST_WriteToFS(&(slDat->servLst), FFALSE, file) == FFALSE )
        {
            break;
        }

        /* ----- write global lists ---------------------------------------- */

        for(cnt = 0; cnt < slDat->numGlobLsts; cnt++)
        {
            if ( SLIST_WriteList(slDat->lstGlob + cnt, file) == FFALSE )
            {
                abort = FTRUE;
                break;
            }
        }

        if ( abort == FTRUE ) break;

        /* ----- write user lists ------------------------------------------ */

        for(cnt = 0; cnt < slDat->numUsrLsts; cnt++)
        {
            if ( SLIST_WriteList(slDat->lstUsr + cnt, file) == FFALSE )
            {
                abort = FTRUE;
                break;
            }
        }

        if ( abort == FTRUE ) break;

        /* ----- write file header again at end of file -------------------- */

        if ( FLASHFS_Write(file, sizeof(SLIST_SERVFILEHDR_S), &fileHdr,
                           NULL) != (int32_t)sizeof(SLIST_SERVFILEHDR_S) )
        {
            break;
        }

        /* ----- write CA info --------------------------------------------- */

        #if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

        /* First close service data file for better flash writing
           performance. */
        (void)FLASHFS_Close(file);

        file = NULL;

        if ( !SLIST_CaInfoSave(force) )
        {
            break;
        }

        #endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */

        /* ----------------------------------------------------------------- */

        retVal = FTRUE;

        break;//end of for(;;)
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    /*lint -save -e774 */
    if ( file != NULL )
    {
        (void)FLASHFS_Close(file);
    }
    /*lint -restore */

    if ( retVal == FFALSE )
    {
        (void)FLASHFS_Delete(FILEID_SERVICEDATA);
    }
    else
    {
        lstValid         = FTRUE;
        slDat->servClean = FTRUE;
    }

    return retVal;
}

/* ------------------------------------------------------------------------- */

#if defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED)

int32_t SLIST_CaInfoSet(SLIST_SERVIDX servIdx, const SLIST_CA_INFO_S * pInfo)
{
    int32_t             retVal;

    if(slDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    /* Verify that entry already exists */
    if(STATLST_GetEntry(&(slDat->caLst), servIdx) == NULL)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        retVal = STATLST_SetEntry(&(slDat->caLst), servIdx, pInfo,
                                  SYS_OWNER_SYSTEM);
    }

    if ( retVal == FAPI_OK ) slDat->caClean = FFALSE;

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}

int32_t SLIST_CaInfoGet(SLIST_SERVIDX servIdx, SLIST_CA_INFO_S * pInfo)
{
    SLIST_CA_INFO_S    *pEntry;
    int32_t             retVal;

    if(slDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    pEntry = STATLST_GetEntry(&(slDat->caLst), servIdx);

    if(pEntry == NULL)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        *pInfo = *pEntry;
        retVal = FAPI_OK;
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    return retVal;
}

int32_t SLIST_CaInfoExtract(const PSISI_PMT_S * pPmt,
                            const SYS_STREAM_INFO_S * strPid,
                            SLIST_CA_INFO_S * pInfo)
{
    uint16_t            pid;
    uint16_t            pmtInfoLen;
    uint16_t            pmtPos = 0;
    uint16_t            pmtInfoPos;
    uint8_t             descTag;
    uint16_t            descLen;
    const uint8_t      *pPmtEs;

    /* test for minimum PMT size */
    if(pPmt->bufSize < PMT_SECTSIZE_MIN)
        return SMARTGO_SYS_ERR_INVALID_DATA;

    memset(pInfo, 0, sizeof(SLIST_CA_INFO_S));

    /* ----- read program info --------------------------------------------- */

    pmtInfoLen = LD_LEN(pPmt->buf + 10);
    pmtPos = PMT_SECTSIZE_MIN;
    pmtInfoPos = 0;

    if(pmtInfoLen != 0)
    {
        if((uint16_t) (pmtPos + pmtInfoLen) > pPmt->bufSize)
        {
            FAPI_SYS_PRINT_DEBUG(1,"[CISYS] Invalid PMT data!\n");

            pmtInfoLen = pPmt->bufSize - pmtPos;
        }

        while((uint16_t) (pmtInfoPos + 2) <= pmtInfoLen)
        {
            descTag = pPmt->buf[pmtPos + pmtInfoPos];
            descLen = pPmt->buf[pmtPos + pmtInfoPos + 1] + 2;

            if((uint16_t) (pmtInfoPos + descLen) > pmtInfoLen)
            {
                FAPI_SYS_PRINT_DEBUG(1,"[CISYS] Invalid PMT data (2)!\n");

                descLen = pmtInfoLen - pmtInfoPos;
            }

            if(descTag == CA_DESCRIPTOR)
            {
                if(!SLIST_CaInfoAddId
                   (LD16(pPmt->buf + pmtPos + pmtInfoPos + 2), pInfo))
                {
                    /* No more space in CA system ID list */
                    return FAPI_OK;
                }
            }

            pmtInfoPos += descLen;
        }

        pmtPos += pmtInfoLen;
    }

    /* ----- read ES info -------------------------------------------------- */

    while((uint16_t) (pmtPos + 5) <= pPmt->bufSize)     /* parse each ES loop */
    {
        pPmtEs = pPmt->buf + pmtPos;
        pmtInfoLen = LD_LEN(pPmtEs + 3);
        pmtInfoPos = 0;

        if((uint16_t) (pmtPos + 5 + pmtInfoLen) > pPmt->bufSize)
        {
            FAPI_SYS_PRINT_DEBUG(1,"[CISYS] Invalid PMT data (3)!\n");

            pmtInfoLen = (pPmt->bufSize - pmtPos) - 5;
        }

        pid = LD_PID(pPmtEs + 1);

        if(SLIST_MatchStreamPid(pid, strPid))
        {
            /* parse each descriptor */
            while((uint16_t) (pmtInfoPos + 2) <= pmtInfoLen)
            {
                descTag = pPmtEs[5 + pmtInfoPos];
                descLen = pPmtEs[5 + pmtInfoPos + 1] + 2;

                if((uint16_t) (pmtInfoPos + descLen) > pmtInfoLen)
                {
                    FAPI_SYS_PRINT_DEBUG(1,"[CISYS] Invalid PMT data (4)!\n");

                    descLen = pmtInfoLen - pmtInfoPos;
                }

                if(descTag == CA_DESCRIPTOR)
                {
                    if(!SLIST_CaInfoAddId
                       (LD16(pPmtEs + 5 + pmtInfoPos + 2), pInfo))
                    {
                        /* No more space in CA system ID list */
                        return FAPI_OK;
                    }
                }

                pmtInfoPos += descLen;
            }
        }

        pmtPos += pmtInfoLen + 5;
    }

    return FAPI_OK;
}

FSTATIC fbool_t SLIST_CaInfoAddId(uint16_t ca_system_id,
                                  SLIST_CA_INFO_S * pInfo)
{
    uint8_t             cnt = 0;

    if(pInfo->numIds >= SLIST_CASYSTEMIDS_MAX)
        return FFALSE;

    while(cnt < pInfo->numIds)
    {
        if(pInfo->ca_system_id[cnt++] == ca_system_id)
        {
            /* already in list */
            return FTRUE;
        }
    }

    /* No match found, append to list */
    pInfo->ca_system_id[pInfo->numIds++] = ca_system_id;
    return FTRUE;
}

fbool_t SLIST_MatchStreamPid(uint16_t pid, const SYS_STREAM_INFO_S * strPid)
{
    uint16_t            astrCnt;

    /* compare video PID */
    if ( pid == strPid->video )
    {
        return FTRUE;
    }

    /* compare all audio PIDs */
    for ( astrCnt = 0; astrCnt < strPid->audio.numStreams; astrCnt++ )
    {
        if ( pid == strPid->audio.stream[astrCnt].pid )
        {
            return FTRUE;
        }
    }

    /* compare TTX PID */
    if ( pid == strPid->ttx )
    {
        return FTRUE;
    }

    return FFALSE;
}

FSTATIC fbool_t SLIST_CaInfoLoad (void)
{
    fbool_t             retVal = FFALSE;
    int32_t             res;
    SLIST_CAFILEHDR_S   fileHdr;
    FLASHFS_HANDLE      file;

    file = FLASHFS_Open(FILEID_CADATA, FLASHFS_OPEN_READ, NULL);

    if(!file)
    {
        FAPI_SYS_PRINT_DEBUG(2,"File opening failed!\n");
        return FFALSE;
    }

    do
    {
        /* ----- read file header ------------------------------------------ */

        res = FLASHFS_Read(file, sizeof(SLIST_CAFILEHDR_S), &fileHdr, NULL);

        if ( res != (int32_t)sizeof(SLIST_CAFILEHDR_S) )
        {
            /* file read error, file too small */
            break;
        }
        else if((fileHdr.id       != FILEID_CADATA) ||
                (fileHdr.revision != FILEREV_CADATA))
        {
            /* invalid identifier */
            break;
        }

        /* ----- read CA info entries -------------------------------------- */

        if ( STATLST_LoadFromFS(&(slDat->caLst), FFALSE, file) == FFALSE )
        {
            break;
        }

        /* ----- read file hdr at end of file for verification ------------- */

        res = FLASHFS_Read(file, sizeof(SLIST_CAFILEHDR_S), &fileHdr, NULL);

        if ( res != (int32_t)sizeof(SLIST_CAFILEHDR_S) )
        {
            break;
        }
        else if((fileHdr.id != FILEID_CADATA) ||
                (fileHdr.revision != FILEREV_CADATA))
        {
            break;
        }

        retVal = FTRUE;
    }
    while(0);

    (void)FLASHFS_Close(file);

    if ( retVal == FFALSE )
    {

    }
    else
    {
        slDat->caClean = FTRUE;
    }

    return retVal;
}

FSTATIC fbool_t SLIST_CaInfoSave (fbool_t force)
{
    fbool_t             retVal = FFALSE;
    SLIST_CAFILEHDR_S   fileHdr;
    FLASHFS_HANDLE      file;

    /* saving necessary? */
    if ( (!force) && slDat->caClean ) return FTRUE;

    file = FLASHFS_Open(FILEID_CADATA, FLASHFS_OPEN_WRITE, NULL);

    if ( file == NULL )
    {
        return FFALSE;
    }

    do
    {
        /* ----- write file header ----------------------------------------- */

        memset (&fileHdr,0,sizeof(SLIST_CAFILEHDR_S));

        fileHdr.id       = FILEID_CADATA;
        fileHdr.revision = FILEREV_CADATA;

        if ( FLASHFS_Write(file, sizeof(SLIST_CAFILEHDR_S), &fileHdr, NULL)
             != (int32_t)sizeof(SLIST_CAFILEHDR_S) )
        {
            break;
        }

        /* ----- write service list entries -------------------------------- */

        if ( STATLST_WriteToFS(&(slDat->caLst), FFALSE, file) == FFALSE )
        {
            break;
        }

        /* ----- write file header again at end of file -------------------- */

        if ( FLASHFS_Write(file, sizeof(SLIST_CAFILEHDR_S), &fileHdr, NULL)
             != (int32_t)sizeof(SLIST_CAFILEHDR_S))
        {
            break;
        }

        retVal = FTRUE;
    }
    while ( 0 );

    (void)FLASHFS_Close(file);

    if ( retVal == FFALSE )
    {
        (void)FLASHFS_Delete(FILEID_CADATA);
    }
    else
    {
        slDat->caClean = FTRUE;
    }

    return retVal;
}

#endif /* defined(APPL_CI_ENABLED) || defined(APPL_CA_ENABLED) */

/*!
*******************************************************************************
**
** \brief   SLIST_NitUpdatedService
**
** \note    This function update the lcn and attribute of the service list, or add new services
**
** \param   pnit_service: the service list of one carrier get from NIT
** \param   tp_index: the index of carrier
** \param   pnetwork: network info
**
** \return
**
*******************************************************************************
*/
int32_t SLIST_NitUpdatedService(const PSISI_NIT_SERVICE_INFO_S * pnit_service, TPLIST_IDX tp_index, SYS_NETWORK_S *pnetwork, uint8_t desflag)
{
    SLIST_ITEM_S       *pSrc = NULL;
    SLIST_SERVIDX       srcCnt;
    int32_t             retVal;
    fbool_t             found = FFALSE;
    fbool_t             changed = FFALSE;
    SLIST_ITEM_S        slistItem;
    SYS_SERVLOC_S        targetLoc;

    if ( ( slDat == NULL ) ||
         ( pnit_service == NULL ) )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (void)FAPEX_SYS_GetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    /* search service in list */
    FAPI_SYS_PRINT_DEBUG(5,"NIT target: slDat->servLst.stat.afterLast=%d, tp_index=%d\n", slDat->servLst.stat.afterLast, tp_index);
    for(srcCnt = 0; srcCnt < slDat->servLst.stat.afterLast; srcCnt++)
    {
        pSrc = STATLST_GetEntry(&(slDat->servLst), srcCnt);

        if(pSrc == NULL)
        {
            continue;
        }

        FAPI_SYS_PRINT_DEBUG(5,"NIT Get: pSrc->serviceId=0x%x, pSrc->tpIdx=%d\n", pSrc->serviceId, pSrc->tpIdx);

        if((pSrc->serviceId == pnit_service->serviceId) &&
           (pSrc->tpIdx == tp_index))
        {
            found = FTRUE;

            FAPI_SYS_PRINT_DEBUG(5,"NIT Update: Found Service ID %#x, pSrc->tpIdx =%d\n", pSrc->serviceId, pSrc->tpIdx);
            if(desflag & FLAG_GET_LCN)
            {
                if (pSrc->lcn != pnit_service->lcn)
                {
                    FAPI_SYS_PRINT_DEBUG(3,"NIT Change: Service ID %#x, lcn %d --> %d\n",
                        pSrc->serviceId, pSrc->lcn, pnit_service->lcn);
                       pSrc->lcn = pnit_service->lcn;
                    changed = FTRUE;
                }
            }
            if (pSrc->serviceType != pnit_service->service_type)
            {
                FAPI_SYS_PRINT_DEBUG(3,"NIT Change: Service ID %#x, content %d --> %d\n",
                    pSrc->serviceId, pSrc->serviceType, pnit_service->service_type);
                   pSrc->serviceType = pnit_service->service_type;
                changed = FTRUE;
            }

            if(desflag & FLAG_GET_ATTR)
            {
                if(SLIST_FlagIsSet(pSrc, SLIST_FLAG_HIDDEN_UK) != pnit_service->invisible)
                {
                    SLIST_SetFlag(pSrc, SLIST_FLAG_HIDDEN_UK, (pnit_service->invisible));
                    FAPI_SYS_PRINT_DEBUG(3,"NIT Change: Service ID %#x, Hide flag =%d\n",
                        pSrc->serviceId, (pnit_service->invisible));
                    changed = FTRUE;
                }

                if(SLIST_FlagIsSet(pSrc, SLIST_FLAG_NONSELECT) != pnit_service->unselectable)
                {
                    SLIST_SetFlag(pSrc, SLIST_FLAG_NONSELECT, (pnit_service->unselectable));
                    FAPI_SYS_PRINT_DEBUG(3,"NIT Change: Service ID %#x, select flag =%d\n",
                        pSrc->serviceId, (pnit_service->unselectable));
                    changed = FTRUE;
                }
            }
            break;
        }
    }

    (void)FAPEX_SYS_SetSemaphore(slDat->mutexId, FAPEX_SYS_SUSPEND);

    if ( found )
    {
        if ( changed )
        {
            slDat->servClean = FFALSE;

            retVal = SMARTGO_SYS_ERR_REPLACED_IN_LIST;
        }
        else
        {
            retVal = SMARTGO_SYS_ERR_ALREADY_IN_LIST;
        }
    }
    else
    {
        FAPI_SYS_PRINT_DEBUG(5,"NIT Update: Cannot Found Service ID %#x, service_type=%d, tpIdx =%d, lcn=%d\n", pnit_service->serviceId, pnit_service->service_type, tp_index, pnit_service->lcn);

        //Add it to the service list
        if( (pnit_service->service_type == SERVICE_TYPE_DIGITAL_TV) ||
            (pnit_service->service_type == SERVICE_TYPE_DIGITAL_TV_HD) ||
            (pnit_service->service_type == SERVICE_TYPE_DIGITAL_TV_ADVANCED) ||
            (pnit_service->service_type == SERVICE_TYPE_DIGITAL_TV_HD_ADVANCED) ||
            #ifdef APPL_MHEG_ENABLED
            (pnit_service->service_type == SERVICE_TYPE_DATA_BROADCAST) ||
            #endif
            (pnit_service->service_type == SERVICE_TYPE_DIGITAL_RADIO) ||
            (pnit_service->service_type == SERVICE_TYPE_DIGITAL_RADIO_ADVANCED))
        {
            targetLoc.listId = SLIST_NONE;
            targetLoc.index = -1;

            memset(&slistItem, 0, sizeof(SLIST_ITEM_S));

            slistItem.network      = *pnetwork;
            slistItem.tpIdx        = tp_index;
            slistItem.minAge       = PCTRL_MINAGE_UNLIMITED;
            slistItem.strPid.valid = 0; /* default */

            slistItem.serviceId = pnit_service->serviceId;
            slistItem.lcn = pnit_service->lcn;
            slistItem.serviceType = pnit_service->service_type;
            SLIST_SetFlag(&slistItem, SLIST_FLAG_HIDDEN_UK, (pnit_service->invisible));
            SLIST_SetFlag(&slistItem, SLIST_FLAG_NONSELECT, (pnit_service->unselectable));

            retVal = SLIST_EntryAdd(&slistItem, &targetLoc);
        }
        else
        {
            retVal = FAPI_OK;
        }
    }

    return retVal;
}

/* ------------------------------------------------------------------------- */

#if 0
/* test function */
void SortTest(void)
{
    SLIST_ITEM_S        listEntry;
    int16_t             cnt;
    int16_t             list[500];
    int16_t             listSize;
    SLIST_TYPE          listId = SLIST_TV;
    fbool_t      ascending = FFALSE;
    SYS_SERVLOC_S       servLoc;

    FAPI_SYS_PRINT_DEBUG(1,"\n-------------------------\n");

    listSize = SLIST_Sort(listId, SLIST_SORT_BYNAME, ascending, list,
                          sizeof(list));

    if(listSize < 0)
        FAPI_SYS_PRINT_DEBUG(1,"SLIST_Sort: ERROR \n");
    else
    {
        for(cnt = 0; cnt < listSize; cnt++)
        {
            if(list[cnt] < 0)
                FAPI_SYS_PRINT_DEBUG(1,"%03i - invalid index\n", cnt);
            else
            {
                servLoc.listId = listId;
                servLoc.index = list[cnt];

                if(SLIST_EntryGet(&servLoc, &listEntry) != FAPI_OK)
                {
                    FAPI_SYS_PRINT_DEBUG(1,"%03i - invalid/empty entry \n", cnt);
                }
                else
                {
                    FAPI_SYS_PRINT_DEBUG(1,"%03i - %s \n", cnt, listEntry.name);
                }
            }
        }
    }
    FAPI_SYS_PRINT_DEBUG(1,"-------------------------\n");
}


void SLIST_PrintList(SLIST_TYPE list)
{
    SYS_SERVLOC_S       servLoc;
    SLIST_ITEM_S        item;
    uint16_t            size;
    uint32_t            i;

    size = SLIST_ListSizeGet(list);

    servLoc.listId = list;

    FAPI_SYS_PRINT_DEBUG(1," listId = %3d        Service Name |   Sid |  VPid |TPidx",
               list);
    FAPI_SYS_PRINT_DEBUG(1,"| Flags| Age| LCN |cont|\n");
    for(i = 0; i < size; i++)
    {
        servLoc.index = i;
        SLIST_EntryGet(&servLoc, &item);

        FAPI_SYS_PRINT_DEBUG(1,"%33s | %5d | %5d | %3d |",
                   item.name, item.serviceId, item.strPid.video, item.tpIdx);
        FAPI_SYS_PRINT_DEBUG(1," 0x%02x | %2d | %3d | %2d |\n",
                   item.flags, item.minAge, item.lcn, item.content);

    }
    FAPI_SYS_PRINT_DEBUG(1,"\n");

}
#endif

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
