/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/statlist.c
**
**              $Id: statlist.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e732 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <fapi/sys_driver.h>         /* global type definitions */
#include <flashfs/flashfs.h>

#include "sys_api.h"
#include "sys_priv.h"
#include "utils.h"
#include "statlist.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define ALIGN_2BYTE(a)   (((a) + 1) & 0xFFFFFFFE)

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

int32_t STATLST_Init(STATLST_S * pLst, uint32_t maxEntries, uint32_t entrySize)
{
    memset(pLst, 0, sizeof(STATLST_S));

    pLst->data = (uint8_t *) UTILS_Malloc(maxEntries * entrySize);
    if(pLst->data == NULL)
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    pLst->status =
        (uint8_t *) UTILS_Malloc(sizeof(*(pLst->status)) * maxEntries);
    if(pLst->status == NULL)
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    pLst->stat.entrySize = entrySize;
    pLst->stat.maxEntries = maxEntries;
    pLst->stat.numEntries = 0;
    pLst->stat.afterLast = 0;
    pLst->stat.firstFree = 0;

    memset(pLst->status, 0,
                   sizeof(*(pLst->status)) * pLst->stat.maxEntries);

    pLst->initialized = FTRUE;
    return FAPI_OK;
}

uint32_t STATLST_GetMaxDataSize(uint32_t maxEntries, uint32_t entrySize)
{
    return (sizeof(STATLST_STATUS_S)) +
        ALIGN_2BYTE(sizeof(STATLST_ENTRYSTAT) * maxEntries) +
        ALIGN_2BYTE(entrySize * maxEntries);
}

int32_t STATLST_Clear(STATLST_S * pLst, SYS_OWNER_E owners,
                      STATLST_REMOVE_CB cb, void *cbArg)
{
    int32_t             index;
    uint8_t             refStatus;

    if( (pLst == NULL) ||
        (!pLst->initialized))
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    if(owners == SYS_OWNER_ALL) /* clear all: use fast method */
    {
        if(cb != NULL)
        {
            for(index = 0; index < pLst->stat.afterLast; index++)
            {
                if((pLst->status[index] & STATLST_USED) != 0)
                {
                    cb(cbArg, index);
                }
            }
        }

        memset(pLst->status, 0,
                       sizeof(*(pLst->status)) * pLst->stat.maxEntries);

        pLst->stat.numEntries = 0;
        pLst->stat.afterLast = 0;
        pLst->stat.firstFree = 0;
    }
    else                        /* selective clearing of list */
    {
        refStatus = (((uint8_t) owners) << 4) & STATLST_OWNER;

        for(index = 0; index < pLst->stat.afterLast; index++)
        {
            if(((pLst->status[index] & STATLST_USED) != 0) &&
               ((pLst->status[index] & refStatus) != 0))
            {
                if(cb != NULL)
                    cb(cbArg, index);

                (void)STATLST_RemoveEntry(pLst, index);
            }
        }
    }

    return FAPI_OK;
}

int32_t STATLST_AddEntry(STATLST_S * pLst, const void *pEntry,
                         SYS_OWNER_E owner, int32_t * index)
{
    int32_t             retVal;
    int32_t             slotIdx = -1;

    if( (pLst == NULL) ||
        (!pLst->initialized))
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    *index = -1;

    /* free entries available? */
    if(((uint32_t)pLst->stat.afterLast >= pLst->stat.maxEntries) &&
       ((uint32_t)pLst->stat.firstFree >= pLst->stat.maxEntries))
    {
        return SMARTGO_SYS_ERR_CAPACITY;
    }

    /* search for next free slot */
    while(pLst->stat.firstFree < pLst->stat.afterLast)
    {
        if((pLst->status[pLst->stat.firstFree] & STATLST_USED) == 0)
        {
            slotIdx = pLst->stat.firstFree;
            break;
        }
        else
        {
            pLst->stat.firstFree++;
        }
    }

    /* try to append if no free slots were found */
    if(slotIdx < 0)
    {
        if(((uint32_t)pLst->stat.afterLast < pLst->stat.maxEntries))
        {
            slotIdx = pLst->stat.afterLast;
        }
        else
        {
            return SMARTGO_SYS_ERR_CAPACITY;
        }
    }

    retVal = STATLST_SetEntry(pLst, slotIdx, pEntry, owner);

    if(retVal == FAPI_OK)
    {
        *index = slotIdx;
    }

    return retVal;
}

int32_t STATLST_SetEntry(STATLST_S * pLst, int32_t index, const void *pEntry,
                         SYS_OWNER_E owner)
{

    if( (pLst == NULL) ||
        (!pLst->initialized))
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    if((index < 0) || ((uint32_t)index >= pLst->stat.maxEntries))
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    /* copy entry data */
    memcpy (pLst->data + (index * (pLst->stat.entrySize)),
            (uint8_t *) pEntry, pLst->stat.entrySize);

    if((pLst->status[index] & STATLST_USED) == 0)
    {
        pLst->status[index] |= STATLST_USED;    /* mark entry as used */
        pLst->stat.numEntries++;
    }

    /* set owner */
    pLst->status[index] &= (~STATLST_OWNER);
    pLst->status[index] |= ((((uint8_t) owner) << 4) & STATLST_OWNER);

    if(index >= pLst->stat.afterLast)
    {
        pLst->stat.afterLast = index + 1;
    }

    if(pLst->stat.firstFree == index)
    {
        pLst->stat.firstFree++;
    }

    return FAPI_OK;
}

void               *STATLST_GetEntry(STATLST_S * pLst, int32_t index)
{
    if( (pLst == NULL) ||
        (!pLst->initialized))
    {
        return NULL;
    }

    if((index < 0) || ((uint32_t)index >= pLst->stat.maxEntries))
        return NULL;

    else if((pLst->status[index] & STATLST_USED) == 0)
        return NULL;

    else
        return (void *) (pLst->data + (index * (pLst->stat.entrySize)));
}

int32_t STATLST_RemoveEntry(STATLST_S * pLst, int32_t index)
{
    if( (pLst == NULL) ||
        (!pLst->initialized))
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    if((index < 0) || (index >= pLst->stat.afterLast))
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    /* mark entry as free */
    if((pLst->status[index] & STATLST_USED) != 0)
    {
        if(pLst->stat.numEntries < 1)
        {
            FAPI_SYS_PRINT_DEBUG(2,"[STATLST] Internal error!\n");
        }

        pLst->status[index] = 0;
        pLst->stat.numEntries--;
    }

    if((index + 1) == pLst->stat.afterLast)
    {
        /* entry has been last entry */
        pLst->stat.afterLast--;
    }

    if(pLst->stat.firstFree > index)
    {
        /* entry is new first entry */
        pLst->stat.firstFree = index;
    }

    return FAPI_OK;
}

int32_t STATLST_CopyList(STATLST_S * pDes, STATLST_S * pSrc)
{
    if(!(pDes->initialized) || !(pSrc->initialized))
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if((pDes->stat.maxEntries != pSrc->stat.maxEntries) ||
       (pDes->stat.entrySize != pSrc->stat.entrySize))
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    pDes->stat = pSrc->stat;

    memcpy (pDes->status, pSrc->status,
            sizeof(uint8_t) * pSrc->stat.maxEntries);
    memcpy (pDes->data, pSrc->data,
            pSrc->stat.entrySize * pSrc->stat.maxEntries);

    return FAPI_OK;
}

fbool_t STATLST_LoadFromFS (STATLST_S* pLst, fbool_t targetIsHdd, void* fileHdl)
{
    STATLST_STATUS_S    tempStat;
    fbool_t             retVal = FFALSE;
    int32_t             res;
    int32_t             numBytes;

    if ( !(pLst->initialized) ) return FFALSE;

    if ( targetIsHdd ) return FFALSE; /* not supported yet */

    do
    {
        /* NOTE: It is possible to load lists having a smaller maximum size
           into RAM, but not vice versa. */

        /* ----- Read status structure ------------------------------------- */

        res = FLASHFS_Read(fileHdl, sizeof(tempStat), &tempStat, NULL);

        if ( res != sizeof(tempStat) )
        {
            break;
        }

        // TODO: check version hier?

        if((tempStat.entrySize != pLst->stat.entrySize) ||
           (tempStat.maxEntries > pLst->stat.maxEntries) ||
           (tempStat.numEntries > pLst->stat.maxEntries) ||
           ((uint32_t)tempStat.afterLast > pLst->stat.maxEntries) ||
           ((tempStat.numEntries < tempStat.maxEntries) &&
            ((uint32_t)tempStat.firstFree >= pLst->stat.maxEntries)))
        {
            FAPI_SYS_PRINT_DEBUG(2,"[STATLST] List to load is invalid\n");
            break;
        }

        pLst->stat.numEntries = tempStat.numEntries;
        pLst->stat.afterLast = tempStat.afterLast;
        pLst->stat.firstFree = tempStat.firstFree;

        /* ----- Read entry status array ----------------------------------- */

        memset (pLst->status, 0,
                sizeof(*(pLst->status)) * pLst->stat.maxEntries);

        numBytes = sizeof(*(pLst->status)) * tempStat.maxEntries;

        res = FLASHFS_Read(fileHdl, numBytes, pLst->status, NULL);

        if ( res != numBytes )
        {
            break;
        }

        /* skip alignment padding */
        if ( (numBytes & 1) != 0 )
        {
            res = FLASHFS_Seek(fileHdl, 1, SEEK_CUR);

            if ( res != FAPI_OK )
            {
                break;
            }
        }

        /* ----- Read entry data ------------------------------------------- */

        if ( pLst->stat.afterLast > 0 )    /* FLASHFS_Read() returns error when
                                              reading 0 bytes */
        {
            res = FLASHFS_Read(fileHdl,
                               pLst->stat.afterLast * pLst->stat.entrySize,
                               pLst->data, NULL);

            if ( res != ((uint32_t)pLst->stat.afterLast * pLst->stat.entrySize))
            {
                break;
            }
        }

        retVal = FTRUE;
    }
    while ( 0 );

    if ( retVal == FFALSE )
    {
        (void)STATLST_Clear(pLst, SYS_OWNER_ALL, NULL, NULL);
    }

    return retVal;
}

fbool_t STATLST_WriteToFS (STATLST_S* pLst, fbool_t targetIsHdd, void* fileHdl)
{
    fbool_t retVal = FFALSE;
    int32_t             res;
    int32_t             numBytes;

    if ( !(pLst->initialized) ) return FFALSE;


    if ( targetIsHdd ) return FFALSE; /* not supported yet */

    do
    {
        /* ----- Write status structure ------------------------------------ */
        res = FLASHFS_Write(fileHdl, sizeof(pLst->stat), &(pLst->stat), NULL);

        if ( res != sizeof(pLst->stat) )
        {
            break;
        }

        /* ----- Write entry status array ---------------------------------- */
        numBytes = ALIGN_2BYTE(sizeof(*(pLst->status)) * pLst->stat.maxEntries);

        res = FLASHFS_Write(fileHdl, numBytes, pLst->status, NULL);

        if ( res != numBytes )
        {
            break;
        }

        /* ----- Write entry data ------------------------------------------ */

        if ( pLst->stat.afterLast > 0 )    /* FLASHFS_Write() returns error when
                                              reading 0 bytes */
        {
            res = FLASHFS_Write(fileHdl,
                                pLst->stat.afterLast * pLst->stat.entrySize,
                                pLst->data, NULL);

            if ( res != pLst->stat.afterLast * pLst->stat.entrySize )
            {
                break;
            }
        }

        retVal = FTRUE;
    }
    while ( 0 );

    return retVal;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
