/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/stringop.c
**
**              $Id: stringop.c 5925 2010-03-05 06:43:50Z jmorha $
**
** \brief       Multi-format string operations.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 571: Suspicious cast.
**                (cast of char_t (ASCII) to unit16_t (Unicode) )
** - Warning 650: Constant 'x' out of range for operator y
**                (Lint assumes char_t type to hold 7 bit only,
**                 SW expects 8 bit!)
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    732: Loss of sign (assignment) (int to unsigned short).
** - Info    734: Loss of precision (assignment) (24 bits to 7 bits).
**                (cast of Unicode to char_t (ASCII) )
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Note    946: Relational or subtract operator applied to pointers
**                [MISRA 2004 Rule 17.3].
**                (suppressed for string opperations)
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e571 -e650 -e713 -e715 -e732 -e734 -e737 -e946 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <stdio.h>

#ifndef _LINUX_FAPI
#include <fapi/sys_driver.h>         /* global type definitions */
#include <fapi/sys_services.h>
#else
#include "fapex/abst_types.h"
#endif

#include "sys_api.h"
#include "utils.h"
#include "sys_priv.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/

typedef uint32_t  (*SYS_STR_INTCPY) (SYS_CHARSET srcCs, const char_t* srcBuf,
                                SYS_CHARSET dstCs, char_t* dstBuf,
                                uint32_t maxBytes, uint32_t numBytes);

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

/* DVB/ETSI text character control codes */
#define TEXTCTRL_EMPHASIS_ON        0x86
#define TEXTCTRL_EMPHASIS_OFF       0x87
#define TEXTCTRL_CODE_CRLF          0x8A

#define STR_BYTESPERCHAR_MAX        3

#define STR_UD                      0   /* Indicates "undefined" in internal
                                           character lookup tables. */

#define STR_PRINTABLE_START         0x20    //!< Start of printable characters.

#define STR_8859_UPPERSTART         0xA0    //!< Start of supplementary chars.
#define STR_8859_UPPERBYTES         0x60    //!< Number of supplementary chars.
#define STR_8559_CHARSET_NUMS       (SYS_CHARSET_ISO8859_15 -   \
                                     SYS_CHARSET_ISO8859_1 + 1)
                                            //!< Number of ISO-8859 char sets.
/*
NOTE: If defined, all (16-Bit) unicode strings will internally be stored in
      big endian order (MSByte, LSByte). Otherwise little endian order will be
      used.
*/
#undef STR_UNICODE_BIGENDIAN


#ifdef STR_UNICODE_BIGENDIAN /* big endian format */

    #define STR_UNICODE_RDCHAR(a)    ((((SYS_CHAR)(a)[0]) << 8) | (a)[1])

    #define STR_UNICODE_WRCHAR(a, d) do {                       \
                                         (a)[0] = (d) >> 8;     \
                                         (a)[1] = (d) & 0x00FF; \
                                     } while (0);               \

#else /* little endian format */

    #define STR_UNICODE_RDCHAR(a)    ((a)[0] | (((SYS_CHAR)(a)[1]) << 8))

    #define STR_UNICODE_WRCHAR(a, d) do {                       \
                                    /*lint -save -e572 -e702*/  \
                                         (a)[0] = (d) & 0x00FF; \
                                         (a)[1] = (d) >> 8;     \
                                    /*lint -restore */          \
                                     } while (0);               \

#endif /* STR_UNICODE_BIGENDIAN */

#define STR_UTF8_1BYTE_IS(a)   (((a)[0] & 0x80) == 0x00)

#define STR_UTF8_2BYTE_IS(a)  ((((a)[0] & 0xE0) == 0xC0) && \
                               (((a)[1] & 0xC0) == 0x80))

#define STR_UTF8_3BYTE_IS(a)  ((((a)[0] & 0xF0) == 0xE0) && \
                               (((a)[1] & 0xC0) == 0x80) && \
                               (((a)[2] & 0xC0) == 0x80))


#define STR_UTF8_1BYTE_GET(a) ((a)[0] & 0x7F)

#define STR_UTF8_2BYTE_GET(a) ((((SYS_CHAR)((a)[0] & 0x1F)) << 6) | \
                                           ((a)[1] & 0x3F))

#define STR_UTF8_3BYTE_GET(a) ((((SYS_CHAR)((a)[0] & 0x0F)) << 12) | \
                               (((SYS_CHAR)((a)[1] & 0x3F)) <<  6) | \
                                           ((a)[2] & 0x3F))


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    char_t      followByte;
    SYS_CHAR    unicodeChar;

} STR_TAB0_MAP_S;

typedef struct
{
    uint16_t    follow16Bit;
    SYS_CHAR    unicodeChar;

} STR_TAB0_3BYTE_MAP_S;

typedef struct
{
    /* NOTE: array is addressed as follows: [srcFormat][dstFormat] */
    SYS_STR_INTCPY  conv[SYS_CHARSET_NUMS][SYS_CHARSET_NUMS];

    SYS_STRBUF_S    lineBuf;
    SYS_STRING_S    asciiStr;

    SYS_CHARSET     defaultSiCharSet;

} SYS_STR_DATA_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

static SYS_STR_DATA_S* strDat = NULL;

SYS_CHAR table_8859_unicode[] =
{
/*
  Idx  ------------------------------------ ISO/IEC 8859 ------------------------------------------------------------
       - 01 -  - 02 -  - 03 -  - 04 -  - 05 -  - 06 -  - 07 -  - 08 -  - 09 -  - 10 -  - 11 -  - 13 -  - 14 -  - 15 - */
/*A0*/ 0x00A0, 0x00A0, 0x00A0, 0x00A0, 0x00A0, 0x00A0, 0x00A0, 0x00A0, 0x00A0, 0x00A0, 0x00A0, 0x00A0, 0x00A0, 0x00A0,
/*A1*/ 0x00A1, 0x0104, 0x0126, 0x0104, 0x0401, STR_UD, 0x2018, STR_UD, 0x00A1, 0x0104, 0x0E01, 0x201D, 0x1E02, 0x00A1,
/*A2*/ 0x00A2, 0x02D8, 0x02D8, 0x0138, 0x0402, STR_UD, 0x2019, 0x00A2, 0x00A2, 0x0112, 0x0E02, 0x00A2, 0x1E03, 0x00A2,
/*A3*/ 0x00A3, 0x0141, 0x00A3, 0x0156, 0x0403, STR_UD, 0x00A3, 0x00A3, 0x00A3, 0x0122, 0x0E03, 0x00A3, 0x00A3, 0x00A3,
/*A4*/ 0x00A4, 0x00A4, 0x00A4, 0x00A4, 0x0404, 0x00A4, 0x20AC, 0x00A4, 0x00A4, 0x012A, 0x0E04, 0x00A4, 0x010A, 0x20AC,
/*A5*/ 0x00A5, 0x013D, STR_UD, 0x0128, 0x0405, STR_UD, 0x20AF, 0x00A5, 0x00A5, 0x0128, 0x0E05, 0x201E, 0x010B, 0x00A5,
/*A6*/ 0x00A6, 0x015A, 0x0124, 0x013B, 0x0406, STR_UD, 0x00A6, 0x00A6, 0x00A6, 0x0136, 0x0E06, 0x00A6, 0x1E0A, 0x0160,
/*A7*/ 0x00A7, 0x00A7, 0x00A7, 0x00A7, 0x0407, STR_UD, 0x00A7, 0x00A7, 0x00A7, 0x00A7, 0x0E07, 0x00A7, 0x00A7, 0x00A7,
/*A8*/ 0x00A8, 0x00A8, 0x00A8, 0x00A8, 0x0408, STR_UD, 0x00A8, 0x00A8, 0x00A8, 0x013B, 0x0E08, 0x00D8, 0x1E80, 0x0161,
/*A9*/ 0x00A9, 0x0160, 0x0130, 0x0160, 0x0409, STR_UD, 0x00A9, 0x00A9, 0x00A9, 0x0110, 0x0E09, 0x00A9, 0x00A9, 0x00A9,
/*AA*/ 0x00AA, 0x015E, 0x015E, 0x0112, 0x040A, STR_UD, 0x037A, 0x00D7, 0x00AA, 0x0160, 0x0E0A, 0x0156, 0x1E82, 0x00AA,
/*AB*/ 0x00AB, 0x0164, 0x011E, 0x0122, 0x040B, STR_UD, 0x00AB, 0x00AB, 0x00AB, 0x0166, 0x0E0B, 0x00AB, 0x1E0B, 0x00AB,
/*AC*/ 0x00AC, 0x0179, 0x0134, 0x0166, 0x040C, 0x060C, 0x00AC, 0x00AC, 0x00AC, 0x017D, 0x0E0C, 0x00AC, 0x1EF2, 0x00AC,
/*AD*/ 0x00AD, 0x00AD, 0x00AD, 0x00AD, 0x00AD, 0x00AD, 0x00AD, 0x00AD, 0x00AD, 0x00AD, 0x0E0D, 0x00AD, 0x00AD, 0x00AD,
/*AE*/ 0x00AE, 0x017D, STR_UD, 0x017D, 0x040E, STR_UD, STR_UD, 0x00AE, 0x00AE, 0x016A, 0x0E0E, 0x00AE, 0x00AE, 0x00AE,
/*AF*/ 0x00AF, 0x017B, 0x017B, 0x00AF, 0x040F, STR_UD, 0x2015, 0x00AF, 0x00AF, 0x014A, 0x0E0F, 0x00C6, 0x0178, 0x00AF,
/*B0*/ 0x00B0, 0x00B0, 0x00B0, 0x00B0, 0x0410, STR_UD, 0x00B0, 0x00B0, 0x00B0, 0x00B0, 0x0E10, 0x00B0, 0x1E1E, 0x00B0,
/*B1*/ 0x00B1, 0x0105, 0x0127, 0x0105, 0x0411, STR_UD, 0x00B1, 0x00B1, 0x00B1, 0x0105, 0x0E11, 0x00B1, 0x1E1F, 0x00B1,
/*B2*/ 0x00B2, 0x02DB, 0x00B2, 0x02DB, 0x0412, STR_UD, 0x00B2, 0x00B2, 0x00B2, 0x0113, 0x0E12, 0x00B2, 0x0120, 0x00B2,
/*B3*/ 0x00B3, 0x0142, 0x00B3, 0x0157, 0x0413, STR_UD, 0x00B3, 0x00B3, 0x00B3, 0x0123, 0x0E13, 0x00B3, 0x0121, 0x00B3,
/*B4*/ 0x00B4, 0x00B4, 0x00B4, 0x00B4, 0x0414, STR_UD, 0x0384, 0x00B4, 0x00B4, 0x012B, 0x0E14, 0x201C, 0x1E40, 0x017D,
/*B5*/ 0x00B5, 0x013E, 0x00B5, 0x0129, 0x0415, STR_UD, 0x0385, 0x00B5, 0x00B5, 0x0129, 0x0E15, 0x00B5, 0x1E41, 0x00B5,
/*B6*/ 0x00B6, 0x015B, 0x0125, 0x013C, 0x0416, STR_UD, 0x0386, 0x00B6, 0x00B6, 0x0137, 0x0E16, 0x00B6, 0x00B6, 0x00B6,
/*B7*/ 0x00B7, 0x02C7, 0x00B7, 0x02C7, 0x0417, STR_UD, 0x00B7, 0x00B7, 0x00B7, 0x00B7, 0x0E17, 0x00B7, 0x1E56, 0x00B7,
/*B8*/ 0x00B8, 0x00B8, 0x00B8, 0x00B8, 0x0418, STR_UD, 0x0388, 0x00B8, 0x00B8, 0x013C, 0x0E18, 0x00F8, 0x1E81, 0x017E,
/*B9*/ 0x00B9, 0x0161, 0x0131, 0x0161, 0x0419, STR_UD, 0x0389, 0x00B9, 0x00B9, 0x0111, 0x0E19, 0x00B9, 0x1E57, 0x00B9,
/*BA*/ 0x00BA, 0x015F, 0x015F, 0x0113, 0x041A, STR_UD, 0x038A, 0x00F7, 0x00BA, 0x0161, 0x0E1A, 0x0157, 0x1E83, 0x00BA,
/*BB*/ 0x00BB, 0x0165, 0x011F, 0x0123, 0x041B, 0x061B, 0x00BB, 0x00BB, 0x00BB, 0x0167, 0x0E1B, 0x00BB, 0x1E60, 0x00BB,
/*BC*/ 0x00BC, 0x017A, 0x0135, 0x0167, 0x041C, STR_UD, 0x038C, 0x00BC, 0x00BC, 0x017E, 0x0E1C, 0x00BC, 0x1EF3, 0x0152,
/*BD*/ 0x00BD, 0x02DD, 0x00BD, 0x014A, 0x041D, STR_UD, 0x00BD, 0x00BD, 0x00BD, 0x2015, 0x0E1D, 0x00BD, 0x1E84, 0x0153,
/*BE*/ 0x00BE, 0x017E, STR_UD, 0x017E, 0x041E, STR_UD, 0x038E, 0x00BE, 0x00BE, 0x016B, 0x0E1E, 0x00BE, 0x1E85, 0x0178,
/*BF*/ 0x00BF, 0x017C, 0x017C, 0x014B, 0x041F, 0x061F, 0x038F, STR_UD, 0x00BF, 0x014B, 0x0E1F, 0x00E6, 0x1E61, 0x00BF,
/*C0*/ 0x00C0, 0x0154, 0x00C0, 0x0100, 0x0420, STR_UD, 0x0390, STR_UD, 0x00C0, 0x0100, 0x0E20, 0x0104, 0x00C0, 0x00C0,
/*C1*/ 0x00C1, 0x00C1, 0x00C1, 0x00C1, 0x0421, 0x0621, 0x0391, STR_UD, 0x00C1, 0x00C1, 0x0E21, 0x012E, 0x00C1, 0x00C1,
/*C2*/ 0x00C2, 0x00C2, 0x00C2, 0x00C2, 0x0422, 0x0622, 0x0392, STR_UD, 0x00C2, 0x00C2, 0x0E22, 0x0100, 0x00C2, 0x00C2,
/*C3*/ 0x00C3, 0x0102, STR_UD, 0x00C3, 0x0423, 0x0623, 0x0393, STR_UD, 0x00C3, 0x00C3, 0x0E23, 0x0106, 0x00C3, 0x00C3,
/*C4*/ 0x00C4, 0x00C4, 0x00C4, 0x00C4, 0x0424, 0x0624, 0x0394, STR_UD, 0x00C4, 0x00C4, 0x0E24, 0x00C4, 0x00C4, 0x00C4,
/*C5*/ 0x00C5, 0x0139, 0x010A, 0x00C5, 0x0425, 0x0625, 0x0395, STR_UD, 0x00C5, 0x00C5, 0x0E25, 0x00C5, 0x00C5, 0x00C5,
/*C6*/ 0x00C6, 0x0106, 0x0108, 0x00C6, 0x0426, 0x0626, 0x0396, STR_UD, 0x00C6, 0x00C6, 0x0E26, 0x0118, 0x00C6, 0x00C6,
/*C7*/ 0x00C7, 0x00C7, 0x00C7, 0x012E, 0x0427, 0x0627, 0x0397, STR_UD, 0x00C7, 0x012E, 0x0E27, 0x0112, 0x00C7, 0x00C7,
/*C8*/ 0x00C8, 0x010C, 0x00C8, 0x010C, 0x0428, 0x0628, 0x0398, STR_UD, 0x00C8, 0x010C, 0x0E28, 0x010C, 0x00C8, 0x00C8,
/*C9*/ 0x00C9, 0x00C9, 0x00C9, 0x00C9, 0x0429, 0x0629, 0x0399, STR_UD, 0x00C9, 0x00C9, 0x0E29, 0x00C9, 0x00C9, 0x00C9,
/*CA*/ 0x00CA, 0x0118, 0x00CA, 0x0118, 0x042A, 0x062A, 0x039A, STR_UD, 0x00CA, 0x0118, 0x0E2A, 0x0179, 0x00CA, 0x00CA,
/*CB*/ 0x00CB, 0x00CB, 0x00CB, 0x00CB, 0x042B, 0x062B, 0x039B, STR_UD, 0x00CB, 0x00CB, 0x0E2B, 0x0116, 0x00CB, 0x00CB,
/*CC*/ 0x00CC, 0x011A, 0x00CC, 0x0116, 0x042C, 0x062C, 0x039C, STR_UD, 0x00CC, 0x0116, 0x0E2C, 0x0122, 0x00CC, 0x00CC,
/*CD*/ 0x00CD, 0x00CD, 0x00CD, 0x00CD, 0x042D, 0x062D, 0x039D, STR_UD, 0x00CD, 0x00CD, 0x0E2D, 0x0136, 0x00CD, 0x00CD,
/*CE*/ 0x00CE, 0x00CE, 0x00CE, 0x00CE, 0x042E, 0x062E, 0x039E, STR_UD, 0x00CE, 0x00CE, 0x0E2E, 0x012A, 0x00CE, 0x00CE,
/*CF*/ 0x00CF, 0x010E, 0x00CF, 0x012A, 0x042F, 0x062F, 0x039F, STR_UD, 0x00CF, 0x00CF, 0x0E2F, 0x013B, 0x00CF, 0x00CF,
/*D0*/ 0x00D0, 0x0110, STR_UD, 0x0110, 0x0430, 0x0630, 0x03A0, STR_UD, 0x011E, 0x00D0, 0x0E30, 0x0160, 0x0174, 0x00D0,
/*D1*/ 0x00D1, 0x0143, 0x00D1, 0x0145, 0x0431, 0x0631, 0x03A1, STR_UD, 0x00D1, 0x0145, 0x0E31, 0x0143, 0x00D1, 0x00D1,
/*D2*/ 0x00D2, 0x0147, 0x00D2, 0x014C, 0x0432, 0x0632, STR_UD, STR_UD, 0x00D2, 0x014C, 0x0E32, 0x0145, 0x00D2, 0x00D2,
/*D3*/ 0x00D3, 0x00D3, 0x00D3, 0x0136, 0x0433, 0x0633, 0x03A3, STR_UD, 0x00D3, 0x00D3, 0x0E33, 0x00D3, 0x00D3, 0x00D3,
/*D4*/ 0x00D4, 0x00D4, 0x00D4, 0x00D4, 0x0434, 0x0634, 0x03A4, STR_UD, 0x00D4, 0x00D4, 0x0E34, 0x014C, 0x00D4, 0x00D4,
/*D5*/ 0x00D5, 0x0150, 0x0120, 0x00D5, 0x0435, 0x0635, 0x03A5, STR_UD, 0x00D5, 0x00D5, 0x0E35, 0x00D5, 0x00D5, 0x00D5,
/*D6*/ 0x00D6, 0x00D6, 0x00D6, 0x00D6, 0x0436, 0x0636, 0x03A6, STR_UD, 0x00D6, 0x00D6, 0x0E36, 0x00D6, 0x00D6, 0x00D6,
/*D7*/ 0x00D7, 0x00D7, 0x00D7, 0x00D7, 0x0437, 0x0637, 0x03A7, STR_UD, 0x00D7, 0x0168, 0x0E37, 0x00D7, 0x1E6A, 0x00D7,
/*D8*/ 0x00D8, 0x0158, 0x011C, 0x00D8, 0x0438, 0x0638, 0x03A8, STR_UD, 0x00D8, 0x00D8, 0x0E38, 0x0172, 0x00D8, 0x00D8,
/*D9*/ 0x00D9, 0x016E, 0x00D9, 0x0172, 0x0439, 0x0639, 0x03A9, STR_UD, 0x00D9, 0x0172, 0x0E39, 0x0141, 0x00D9, 0x00D9,
/*DA*/ 0x00DA, 0x00DA, 0x00DA, 0x00DA, 0x043A, 0x063A, 0x03AA, STR_UD, 0x00DA, 0x00DA, 0x0E3A, 0x015A, 0x00DA, 0x00DA,
/*DB*/ 0x00DB, 0x0170, 0x00DB, 0x00DB, 0x043B, STR_UD, 0x03AB, STR_UD, 0x00DB, 0x00DB, STR_UD, 0x016A, 0x00DB, 0x00DB,
/*DC*/ 0x00DC, 0x00DC, 0x00DC, 0x00DC, 0x043C, STR_UD, 0x03AC, STR_UD, 0x00DC, 0x00DC, STR_UD, 0x00DC, 0x00DC, 0x00DC,
/*DD*/ 0x00DD, 0x00DD, 0x016C, 0x0168, 0x043D, STR_UD, 0x03AD, STR_UD, 0x0130, 0x00DD, STR_UD, 0x017B, 0x00DD, 0x00DD,
/*DE*/ 0x00DE, 0x0162, 0x015C, 0x016A, 0x043E, STR_UD, 0x03AE, STR_UD, 0x015E, 0x00DE, STR_UD, 0x017D, 0x0176, 0x00DE,
/*DF*/ 0x00DF, 0x00DF, 0x00DF, 0x00DF, 0x043F, STR_UD, 0x03AF, 0x2017, 0x00DF, 0x00DF, 0x0E3F, 0x00DF, 0x00DF, 0x00DF,
/*E0*/ 0x00E0, 0x0155, 0x00E0, 0x0101, 0x0440, 0x0640, 0x03B0, 0x05D0, 0x00E0, 0x0101, 0x0E40, 0x0105, 0x00E0, 0x00E0,
/*E1*/ 0x00E1, 0x00E1, 0x00E1, 0x00E1, 0x0441, 0x0641, 0x03B1, 0x05D1, 0x00E1, 0x00E1, 0x0E41, 0x012F, 0x00E1, 0x00E1,
/*E2*/ 0x00E2, 0x00E2, 0x00E2, 0x00E2, 0x0442, 0x0642, 0x03B2, 0x05D2, 0x00E2, 0x00E2, 0x0E42, 0x0101, 0x00E2, 0x00E2,
/*E3*/ 0x00E3, 0x0103, STR_UD, 0x00E3, 0x0443, 0x0643, 0x03B3, 0x05D3, 0x00E3, 0x00E3, 0x0E43, 0x0107, 0x00E3, 0x00E3,
/*E4*/ 0x00E4, 0x00E4, 0x00E4, 0x00E4, 0x0444, 0x0644, 0x03B4, 0x05D4, 0x00E4, 0x00E4, 0x0E44, 0x00E4, 0x00E4, 0x00E4,
/*E5*/ 0x00E5, 0x013A, 0x010B, 0x00E5, 0x0445, 0x0645, 0x03B5, 0x05D5, 0x00E5, 0x00E5, 0x0E45, 0x00E5, 0x00E5, 0x00E5,
/*E6*/ 0x00E6, 0x0107, 0x0109, 0x00E6, 0x0446, 0x0646, 0x03B6, 0x05D6, 0x00E6, 0x00E6, 0x0E46, 0x0119, 0x00E6, 0x00E6,
/*E7*/ 0x00E7, 0x00E7, 0x00E7, 0x012F, 0x0447, 0x0647, 0x03B7, 0x05D7, 0x00E7, 0x012F, 0x0E47, 0x0113, 0x00E7, 0x00E7,
/*E8*/ 0x00E8, 0x010D, 0x00E8, 0x010D, 0x0448, 0x0648, 0x03B8, 0x05D8, 0x00E8, 0x010D, 0x0E48, 0x010D, 0x00E8, 0x00E8,
/*E9*/ 0x00E9, 0x00E9, 0x00E9, 0x00E9, 0x0449, 0x0649, 0x03B9, 0x05D9, 0x00E9, 0x00E9, 0x0E49, 0x00E9, 0x00E9, 0x00E9,
/*EA*/ 0x00EA, 0x0119, 0x00EA, 0x0119, 0x044A, 0x064A, 0x03BA, 0x05DA, 0x00EA, 0x0119, 0x0E4A, 0x017A, 0x00EA, 0x00EA,
/*EB*/ 0x00EB, 0x00EB, 0x00EB, 0x00EB, 0x044B, 0x064B, 0x03BB, 0x05DB, 0x00EB, 0x00EB, 0x0E4B, 0x0117, 0x00EB, 0x00EB,
/*EC*/ 0x00EC, 0x011B, 0x00EC, 0x0117, 0x044C, 0x064C, 0x03BC, 0x05DC, 0x00EC, 0x0117, 0x0E4C, 0x0123, 0x00EC, 0x00EC,
/*ED*/ 0x00ED, 0x00ED, 0x00ED, 0x00ED, 0x044D, 0x064D, 0x03BD, 0x05DD, 0x00ED, 0x00ED, 0x0E4D, 0x0137, 0x00ED, 0x00ED,
/*EE*/ 0x00EE, 0x00EE, 0x00EE, 0x00EE, 0x044E, 0x064E, 0x03BE, 0x05DE, 0x00EE, 0x00EE, 0x0E4E, 0x012B, 0x00EE, 0x00EE,
/*EF*/ 0x00EF, 0x010F, 0x00EF, 0x012B, 0x044F, 0x064F, 0x03BF, 0x05DF, 0x00EF, 0x00EF, 0x0E4F, 0x013C, 0x00EF, 0x00EF,
/*F0*/ 0x00F0, 0x0111, STR_UD, 0x0111, 0x2116, 0x0650, 0x03C0, 0x05E0, 0x011F, 0x00F0, 0x0E50, 0x0161, 0x0175, 0x00F0,
/*F1*/ 0x00F1, 0x0144, 0x00F1, 0x0146, 0x0451, 0x0651, 0x03C1, 0x05E1, 0x00F1, 0x0146, 0x0E51, 0x0144, 0x00F1, 0x00F1,
/*F2*/ 0x00F2, 0x0148, 0x00F2, 0x014D, 0x0452, 0x0652, 0x03C2, 0x05E2, 0x00F2, 0x014D, 0x0E52, 0x0146, 0x00F2, 0x00F2,
/*F3*/ 0x00F3, 0x00F3, 0x00F3, 0x0137, 0x0453, STR_UD, 0x03C3, 0x05E3, 0x00F3, 0x00F3, 0x0E53, 0x00F3, 0x00F3, 0x00F3,
/*F4*/ 0x00F4, 0x00F4, 0x00F4, 0x00F4, 0x0454, STR_UD, 0x03C4, 0x05E4, 0x00F4, 0x00F4, 0x0E54, 0x014D, 0x00F4, 0x00F4,
/*F5*/ 0x00F5, 0x0151, 0x0121, 0x00F5, 0x0455, STR_UD, 0x03C5, 0x05E5, 0x00F5, 0x00F5, 0x0E55, 0x00F5, 0x00F5, 0x00F5,
/*F6*/ 0x00F6, 0x00F6, 0x00F6, 0x00F6, 0x0456, STR_UD, 0x03C6, 0x05E6, 0x00F6, 0x00F6, 0x0E56, 0x00F6, 0x00F6, 0x00F6,
/*F7*/ 0x00F7, 0x00F7, 0x00F7, 0x00F7, 0x0457, STR_UD, 0x03C7, 0x05E7, 0x00F7, 0x0169, 0x0E57, 0x00F7, 0x1E6B, 0x00F7,
/*F8*/ 0x00F8, 0x0159, 0x011D, 0x00F8, 0x0458, STR_UD, 0x03C8, 0x05E8, 0x00F8, 0x00F8, 0x0E58, 0x0173, 0x00F8, 0x00F8,
/*F9*/ 0x00F9, 0x016F, 0x00F9, 0x0173, 0x0459, STR_UD, 0x03C9, 0x05E9, 0x00F9, 0x0173, 0x0E59, 0x0142, 0x00F9, 0x00F9,
/*FA*/ 0x00FA, 0x00FA, 0x00FA, 0x00FA, 0x045A, STR_UD, 0x03CA, 0x05EA, 0x00FA, 0x00FA, 0x0E5A, 0x015B, 0x00FA, 0x00FA,
/*FB*/ 0x00FB, 0x0171, 0x00FB, 0x00FB, 0x045B, STR_UD, 0x03CB, STR_UD, 0x00FB, 0x00FB, 0x0E5B, 0x016B, 0x00FB, 0x00FB,
/*FC*/ 0x00FC, 0x00FC, 0x00FC, 0x00FC, 0x045C, STR_UD, 0x03CC, STR_UD, 0x00FC, 0x00FC, STR_UD, 0x00FC, 0x00FC, 0x00FC,
/*FD*/ 0x00FD, 0x00FD, 0x016D, 0x0169, 0x00A7, STR_UD, 0x03CD, 0x200E, 0x0131, 0x00FD, STR_UD, 0x017C, 0x00FD, 0x00FD,
/*FE*/ 0x00FE, 0x0163, 0x015D, 0x016B, 0x045E, STR_UD, 0x03CE, 0x200F, 0x015F, 0x00FE, STR_UD, 0x017E, 0x0177, 0x00FE,
/*FF*/ 0x00FF, 0x02D9, 0x02D9, 0x02D9, 0x045F, STR_UD, STR_UD, STR_UD, 0x00FF, 0x0138, STR_UD, 0x2019, 0x00FF, 0x00FF
};
/*
Highest value of each character set (for estimation of font requirements:
       0x00FF  0x02DD  0x02D9  0x02DB  0x045F  0x0652  0x20AF  0x2017  0x015F  0x2015  0x0E5B  0x201E  0x1EF3  0x20AC
*/

/* ----- Two-byte-characters of ISO 6937 ----------------------------------- */

STR_TAB0_MAP_S table_tab0_c1[] =  { { 0x20, 0x0060 },
                                    { 0x41, 0x00C0 },
                                    { 0x45, 0x00C8 },
                                    { 0x49, 0x00CC },
                                    { 0x4F, 0x00D2 },
                                    { 0x55, 0x00D9 },
                                    { 0x61, 0x00E0 },
                                    { 0x65, 0x00E8 },
                                    { 0x69, 0x00EC },
                                    { 0x6F, 0x00F2 },
                                    { 0x75, 0x00F9 },
                                    { 0x00, 0x0000 } };

STR_TAB0_MAP_S table_tab0_c2[] =  { { 0x20, 0x00B4 },
                                    { 0x41, 0x00C1 },
                                    { 0x43, 0x0106 },
                                    { 0x45, 0x00C9 },
                                    { 0x49, 0x00CD },
                                    { 0x4C, 0x0139 },
                                    { 0x4E, 0x0143 },
                                    { 0x4F, 0x00D3 },
                                    { 0x52, 0x0154 },
                                    { 0x53, 0x015A },
                                    { 0x55, 0x00DA },
                                    { 0x59, 0x00DD },
                                    { 0x5A, 0x0179 },
                                    { 0x61, 0x00E1 },
                                    { 0x63, 0x0107 },
                                    { 0x65, 0x00E9 },
                                    { 0x69, 0x00ED },
                                    { 0x6C, 0x013A },
                                    { 0x6E, 0x0144 },
                                    { 0x6F, 0x00F3 },
                                    { 0x72, 0x0155 },
                                    { 0x73, 0x015B },
                                    { 0x75, 0x00FA },
                                    { 0x79, 0x00FD },
                                    { 0x7A, 0x017A },
                                    { 0x00, 0x0000 } };

STR_TAB0_MAP_S table_tab0_c3[] =  { { 0x20, 0x005E },
                                    { 0x41, 0x00C2 },
                                    { 0x43, 0x0108 },
                                    { 0x45, 0x00CA },
                                    { 0x47, 0x011C },
                                    { 0x48, 0x0124 },
                                    { 0x49, 0x00CE },
                                    { 0x4A, 0x0134 },
                                    { 0x4F, 0x00D4 },
                                    { 0x53, 0x015C },
                                    { 0x55, 0x00DB },
                                    { 0x57, 0x0174 },
                                    { 0x59, 0x0176 },
                                    { 0x61, 0x00E2 },
                                    { 0x63, 0x0109 },
                                    { 0x65, 0x00EA },
                                    { 0x67, 0x011D },
                                    { 0x68, 0x0125 },
                                    { 0x69, 0x00EE },
                                    { 0x6A, 0x0135 },
                                    { 0x6F, 0x00F4 },
                                    { 0x73, 0x015D },
                                    { 0x75, 0x00FB },
                                    { 0x77, 0x0175 },
                                    { 0x79, 0x0177 },
                                    { 0x00, 0x0000 } };

STR_TAB0_MAP_S table_tab0_c4[] =  { { 0x20, 0x02DC },
                                    { 0x41, 0x00C3 },
                                    { 0x49, 0x0128 },
                                    { 0x4E, 0x00D1 },
                                    { 0x4F, 0x00D5 },
                                    { 0x55, 0x0168 },
                                    { 0x61, 0x00E3 },
                                    { 0x69, 0x0129 },
                                    { 0x6E, 0x00F1 },
                                    { 0x6F, 0x00F5 },
                                    { 0x75, 0x0169 },
                                    { 0x00, 0x0000 } };

STR_TAB0_MAP_S table_tab0_c5[] =  { { 0x20, 0x00AF },
                                    { 0x41, 0x0100 },
                                    { 0x45, 0x0112 },
                                    { 0x49, 0x012A },
                                    { 0x4F, 0x014C },
                                    { 0x55, 0x016A },
                                    { 0x61, 0x0101 },
                                    { 0x65, 0x0113 },
                                    { 0x69, 0x012B },
                                    { 0x6F, 0x014D },
                                    { 0x75, 0x016B },
                                    { 0x00, 0x0000 } };

STR_TAB0_MAP_S table_tab0_c6[] =  { { 0x20, 0x02D8 },
                                    { 0x41, 0x0102 },
                                    { 0x47, 0x011E },
                                    { 0x55, 0x016C },
                                    { 0x61, 0x0103 },
                                    { 0x67, 0x011F },
                                    { 0x75, 0x016D },
                                    { 0x00, 0x0000 } };

STR_TAB0_MAP_S table_tab0_c7[] =  { { 0x20, 0x02D9 },
                                    { 0x43, 0x010A },
                                    { 0x45, 0x0116 },
                                    { 0x47, 0x0120 },
                                    { 0x49, 0x0130 },
                                    { 0x5A, 0x017B },
                                    { 0x63, 0x010B },
                                    { 0x65, 0x0117 },
                                    { 0x67, 0x0121 },
                                    { 0x69, 0x0131 },
                                    { 0x7A, 0x017C },
                                    { 0x00, 0x0000 } };

STR_TAB0_MAP_S table_tab0_c8[] =  { { 0x20, 0x00A8 },
                                    { 0x41, 0x00C4 },
                                    { 0x45, 0x00CB },
                                    { 0x49, 0x00CF },
                                    { 0x4F, 0x00D6 },
                                    { 0x55, 0x00DC },
                                    { 0x59, 0x0178 },
                                    { 0x61, 0x00E4 },
                                    { 0x65, 0x00EB },
                                    { 0x69, 0x00EF },
                                    { 0x6F, 0x00F6 },
                                    { 0x75, 0x00FC },
                                    { 0x79, 0x00FF },
                                    { 0x00, 0x0000 } };

STR_TAB0_MAP_S table_tab0_ca[] =  { { 0x20, 0x00B0 },
                                    { 0x41, 0x00C5 },
                                    { 0x55, 0x016E },
                                    { 0x61, 0x00E5 },
                                    { 0x75, 0x016F },
                                    { 0x00, 0x0000 } };

STR_TAB0_MAP_S table_tab0_cb[] =  { { 0x20, 0x00B8 },
                                    { 0x43, 0x00C7 },
                                    { 0x47, 0x0122 },
                                    { 0x4B, 0x0136 },
                                    { 0x4C, 0x013B },
                                    { 0x4E, 0x0145 },
                                    { 0x52, 0x0156 },
                                    { 0x53, 0x015E },
                                    { 0x54, 0x0162 },
                                    { 0x63, 0x00E7 },
                                    { 0x67, 0x0123 },
                                    { 0x6B, 0x0137 },
                                    { 0x6C, 0x013C },
                                    { 0x6E, 0x0146 },
                                    { 0x72, 0x0157 },
                                    { 0x73, 0x015F },
                                    { 0x74, 0x0163 },
                                    { 0x00, 0x0000 } };

STR_TAB0_MAP_S table_tab0_cd[] =  { { 0x20, 0x02DD },
                                    { 0x4F, 0x0150 },
                                    { 0x55, 0x0170 },
                                    { 0x6F, 0x0151 },
                                    { 0x75, 0x0171 },
                                    { 0x00, 0x0000 } };

STR_TAB0_MAP_S table_tab0_ce[] =  { { 0x20, 0x02DB },
                                    { 0x41, 0x0104 },
                                    { 0x45, 0x0118 },
                                    { 0x49, 0x012E },
                                    { 0x55, 0x0172 },
                                    { 0x61, 0x0105 },
                                    { 0x65, 0x0119 },
                                    { 0x69, 0x012F },
                                    { 0x75, 0x0173 },
                                    { 0x00, 0x0000 } };

STR_TAB0_MAP_S table_tab0_cf[] =  { { 0x20, 0x02C7 },
                                    { 0x43, 0x010C },
                                    { 0x44, 0x010E },
                                    { 0x45, 0x011A },
                                    { 0x4C, 0x013D },
                                    { 0x4E, 0x0147 },
                                    { 0x52, 0x0158 },
                                    { 0x53, 0x0160 },
                                    { 0x54, 0x0164 },
                                    { 0x5A, 0x017D },
                                    { 0x63, 0x010D },
                                    { 0x64, 0x010F },
                                    { 0x65, 0x011B },
                                    { 0x6C, 0x013E },
                                    { 0x6E, 0x0148 },
                                    { 0x72, 0x0159 },
                                    { 0x73, 0x0161 },
                                    { 0x74, 0x0165 },
                                    { 0x7A, 0x017E },
                                    { 0x00, 0x0000 } };

STR_TAB0_MAP_S* table_tab0_twobyte[] =    { table_tab0_c1,
                                            table_tab0_c2,
                                            table_tab0_c3,
                                            table_tab0_c4,
                                            table_tab0_c5,
                                            table_tab0_c6,
                                            table_tab0_c7,
                                            table_tab0_c8,
                                            NULL,
                                            table_tab0_ca,
                                            table_tab0_cb,
                                            NULL,
                                            table_tab0_cd,
                                            table_tab0_ce,
                                            table_tab0_cf};

STR_TAB0_3BYTE_MAP_S table_tab0_e1[] =  { { 0xBA80, 0x1E80 },
                                          { 0xBA81, 0x1E81 },
                                          { 0xBA82, 0x1E82 },
                                          { 0xBA83, 0x1E83 },
                                          { 0xBA84, 0x1E84 },
                                          { 0xBA85, 0x1E85 },
                                          { 0xBBB2, 0x1EF2 },
                                          { 0xBBB3, 0x1EF3 },
                                          { 0x0000, 0x0000 } };

STR_TAB0_3BYTE_MAP_S table_tab0_e2[] =  { { 0x8894, 0x2214 },
                                          { 0x8895, 0x2215 },
                                          { 0x99AB, 0x266B },
                                          { 0x9C97, 0x2717 },
                                          { 0x0000, 0x0000 } };


STR_TAB0_3BYTE_MAP_S* table_tab0_threebyte[] = { table_tab0_e1,
                                                 table_tab0_e2 };
/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC uint16_t  DVBSIText2String_8Bit    (const char_t* pSrc, uint16_t srcLen,
                                            char_t* pDst, uint16_t dstLen,
                                            SYS_CHARSET charSet,
                                            fbool_t noEmphasis, char_t crlfChar);

FSTATIC uint16_t  DVBSIText2String_16Bit   (const char_t* pSrc, uint16_t srcLen,
                                            char_t* pDst, uint16_t dstLen,
                                            SYS_CHARSET charSet,
                                            fbool_t noEmphasis, char_t crlfChar);
FSTATIC uint16_t  DVBSIText2String_Utf8    (const char_t* pSrc, uint16_t srcLen,
                                            char_t* pDst, uint16_t dstLen,
                                            SYS_CHARSET charSet,
                                            fbool_t noEmphasis, char_t crlfChar);

FSTATIC uint32_t  STR_InsertChar       (SYS_CHAR chr, SYS_CHARSET charSet, char_t* pStr,
                                        uint32_t bufSize);
FSTATIC uint32_t  STR_Encode           (SYS_CHAR chr, char_t* pBuf,
                                        SYS_CHARSET charSet);
FSTATIC uint8_t   STR_GetTermBytes     (SYS_CHARSET charSet);

FSTATIC uint32_t  STR_INT_8859_Plain   (SYS_CHARSET srcCs, const char_t* srcBuf,
                                        SYS_CHARSET dstCs, char_t* dstBuf,
                                        uint32_t maxBytes, uint32_t numBytes);
FSTATIC uint32_t  STR_INT_8859_8859    (SYS_CHARSET srcCs, const char_t* srcBuf,
                                        SYS_CHARSET dstCs, char_t* dstBuf,
                                        uint32_t maxBytes, uint32_t numBytes);
FSTATIC uint32_t  STR_INT_8859_16Bit   (SYS_CHARSET srcCs, const char_t* srcBuf,
                                        SYS_CHARSET dstCs, char_t* dstBuf,
                                        uint32_t maxBytes, uint32_t numBytes);
FSTATIC uint32_t  STR_INT_8859_Utf8    (SYS_CHARSET srcCs, const char_t* srcBuf,
                                        SYS_CHARSET dstCs, char_t* dstBuf,
                                        uint32_t maxBytes, uint32_t numBytes);
FSTATIC uint32_t  STR_INT_Utf8_Plain   (SYS_CHARSET srcCs, const char_t* srcBuf,
                                  SYS_CHARSET dstCs, char_t* dstBuf,
                                  uint32_t maxBytes, uint32_t numBytes);
FSTATIC uint32_t  STR_INT_Utf8_8859   (SYS_CHARSET srcCs, const char_t* srcBuf,
                                  SYS_CHARSET dstCs, char_t* dstBuf,
                                  uint32_t maxBytes, uint32_t numBytes);
FSTATIC uint32_t  STR_INT_Utf8_16Bit  (SYS_CHARSET srcCs, const char_t* srcBuf,
                                  SYS_CHARSET dstCs, char_t* dstBuf,
                                  uint32_t maxByte, uint32_t numBytess);
FSTATIC uint32_t  STR_INT_16Bit_Plain (SYS_CHARSET srcCs, const char_t* srcBuf,
                                  SYS_CHARSET dstCs, char_t* dstBuf,
                                  uint32_t maxBytes, uint32_t numBytes);
FSTATIC uint32_t  STR_INT_16Bit_8859  (SYS_CHARSET srcCs, const char_t* srcBuf,
                                  SYS_CHARSET dstCs, char_t* dstBuf,
                                  uint32_t maxBytes, uint32_t numBytes);
FSTATIC uint32_t  STR_INT_16Bit_Utf8  (SYS_CHARSET srcCs, const char_t* srcBuf,
                                  SYS_CHARSET dstCs, char_t* dstBuf,
                                  uint32_t maxBytes, uint32_t numBytes);
FSTATIC uint32_t  STR_INT_16Bit_Tab0  (SYS_CHARSET srcCs, const char_t* srcBuf,
                                  SYS_CHARSET dstCs, char_t* dstBuf,
                                  uint32_t maxBytes, uint32_t numBytes);
FSTATIC uint32_t  STR_INT_Tab0_16Bit  (SYS_CHARSET srcCs, const char_t* srcBuf,
                                  SYS_CHARSET dstCs, char_t* dstBuf,
                                  uint32_t maxBytes, uint32_t numBytes);

FSTATIC FINLINE uint32_t STR_CharGetTab0 (const char_t* pStr,
                                     SYS_CHAR* pChr);
FSTATIC fbool_t STR_IsPrintableChar_8Bit(char_t currChar);
FSTATIC fbool_t STR_IsPrintableChar_16Bit(SYS_CHAR currChar);
/* ------------------------------------------------------------------------- */

int32_t STR_Init (const SYS_CONFIGDAT_S* pSysCfg)
{
    uint32_t     cntSrc;
    uint32_t     cntDst;
    int32_t      retVal;

    DBG_Assert(strDat == NULL);

    strDat = (SYS_STR_DATA_S*)UTILS_Malloc((uint32_t)(sizeof(SYS_STR_DATA_S)));
    if ( strDat == NULL )
    {
        return FAPI_ERR_OUT_OF_MEMORY;
    }
    memset(strDat, 0, sizeof(SYS_STR_DATA_S));

    if ( pSysCfg->altDefaultSiCharSet != SYS_CHARSET_UNDEFINED )
    {
        strDat->defaultSiCharSet = pSysCfg->altDefaultSiCharSet;
    }
    else
    {
        /* NOTE: According to ETSI 300 468, the default character set is as
                 set below. However, some broadcasters do not encode text
                 strings according to specification, thus a workaround for
                 the most common problem can be activated by choosing a
                 different default character set. */

        strDat->defaultSiCharSet = SYS_CHARSET_TABLE0;
    }

    /* ----- Initialize general line buffers ------------------------------- */

    retVal = STR_BufOpen (&(strDat->lineBuf), pSysCfg->lineBufSize, NULL);
    if ( retVal != FAPI_OK ) return FAPI_ERR_OUT_OF_MEMORY;

    strDat->asciiStr.chr = UTILS_Malloc((uint32_t)(pSysCfg->asciiBufSize));
    if ( strDat->asciiStr.chr == NULL ) return FAPI_ERR_OUT_OF_MEMORY;

    strDat->asciiStr.cs = SYS_CHARSET_ASCII;

    /* ----- Register conversion functions --------------------------------- */

    for ( cntSrc  = SYS_CHARSET_TABLE0;
          cntSrc <= SYS_CHARSET_ISO8859_15;
          cntSrc++ )
    {
        for ( cntDst  = SYS_CHARSET_TABLE0;
              cntDst <= SYS_CHARSET_ISO8859_15;
              cntDst++ )
        {
            strDat->conv[cntSrc][cntDst] = STR_INT_8859_8859;
        }
    }

    /* ----- no conversion ------------------------------------------------- */

    strDat->conv[SYS_CHARSET_TABLE0    ][SYS_CHARSET_TABLE0    ] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO8859_1 ][SYS_CHARSET_ISO8859_1 ] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO8859_2 ][SYS_CHARSET_ISO8859_2 ] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO8859_3 ][SYS_CHARSET_ISO8859_3 ] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO8859_4 ][SYS_CHARSET_ISO8859_4 ] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO8859_5 ][SYS_CHARSET_ISO8859_5 ] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO8859_6 ][SYS_CHARSET_ISO8859_6 ] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO8859_7 ][SYS_CHARSET_ISO8859_7 ] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO8859_8 ][SYS_CHARSET_ISO8859_8 ] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO8859_9 ][SYS_CHARSET_ISO8859_9 ] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO8859_10][SYS_CHARSET_ISO8859_10] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO8859_11][SYS_CHARSET_ISO8859_11] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO8859_13][SYS_CHARSET_ISO8859_13] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO8859_14][SYS_CHARSET_ISO8859_14] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO8859_15][SYS_CHARSET_ISO8859_15] = STR_INT_8859_Plain;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO_10646 ] = STR_INT_16Bit_Plain;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_UTF8      ] = STR_INT_Utf8_Plain;

    /* ----- conversion from 16 bit unicode -------------------------------- */

    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_TABLE0    ] = STR_INT_16Bit_Tab0;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO8859_1 ] = STR_INT_16Bit_8859;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO8859_2 ] = STR_INT_16Bit_8859;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO8859_3 ] = STR_INT_16Bit_8859;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO8859_4 ] = STR_INT_16Bit_8859;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO8859_5 ] = STR_INT_16Bit_8859;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO8859_6 ] = STR_INT_16Bit_8859;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO8859_7 ] = STR_INT_16Bit_8859;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO8859_8 ] = STR_INT_16Bit_8859;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO8859_9 ] = STR_INT_16Bit_8859;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO8859_10] = STR_INT_16Bit_8859;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO8859_11] = STR_INT_16Bit_8859;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO8859_13] = STR_INT_16Bit_8859;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO8859_14] = STR_INT_16Bit_8859;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_ISO8859_15] = STR_INT_16Bit_8859;
    strDat->conv[SYS_CHARSET_ISO_10646 ][SYS_CHARSET_UTF8      ] = STR_INT_16Bit_Utf8;

    /* ----- conversion from UTF-8 ----------------------------------------- */

    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_TABLE0    ] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO8859_1 ] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO8859_2 ] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO8859_3 ] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO8859_4 ] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO8859_5 ] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO8859_6 ] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO8859_7 ] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO8859_8 ] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO8859_9 ] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO8859_10] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO8859_11] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO8859_13] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO8859_14] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO8859_15] = STR_INT_Utf8_8859;
    strDat->conv[SYS_CHARSET_UTF8      ][SYS_CHARSET_ISO_10646 ] = STR_INT_Utf8_16Bit;

    /* ----- conversion to 16 bit unicode ---------------------------------- */

    strDat->conv[SYS_CHARSET_TABLE0    ][SYS_CHARSET_ISO_10646 ] = STR_INT_Tab0_16Bit;
    strDat->conv[SYS_CHARSET_ISO8859_1 ][SYS_CHARSET_ISO_10646 ] = STR_INT_8859_16Bit;
    strDat->conv[SYS_CHARSET_ISO8859_2 ][SYS_CHARSET_ISO_10646 ] = STR_INT_8859_16Bit;
    strDat->conv[SYS_CHARSET_ISO8859_3 ][SYS_CHARSET_ISO_10646 ] = STR_INT_8859_16Bit;
    strDat->conv[SYS_CHARSET_ISO8859_4 ][SYS_CHARSET_ISO_10646 ] = STR_INT_8859_16Bit;
    strDat->conv[SYS_CHARSET_ISO8859_5 ][SYS_CHARSET_ISO_10646 ] = STR_INT_8859_16Bit;
    strDat->conv[SYS_CHARSET_ISO8859_6 ][SYS_CHARSET_ISO_10646 ] = STR_INT_8859_16Bit;
    strDat->conv[SYS_CHARSET_ISO8859_7 ][SYS_CHARSET_ISO_10646 ] = STR_INT_8859_16Bit;
    strDat->conv[SYS_CHARSET_ISO8859_8 ][SYS_CHARSET_ISO_10646 ] = STR_INT_8859_16Bit;
    strDat->conv[SYS_CHARSET_ISO8859_9 ][SYS_CHARSET_ISO_10646 ] = STR_INT_8859_16Bit;
    strDat->conv[SYS_CHARSET_ISO8859_10][SYS_CHARSET_ISO_10646 ] = STR_INT_8859_16Bit;
    strDat->conv[SYS_CHARSET_ISO8859_11][SYS_CHARSET_ISO_10646 ] = STR_INT_8859_16Bit;
    strDat->conv[SYS_CHARSET_ISO8859_13][SYS_CHARSET_ISO_10646 ] = STR_INT_8859_16Bit;
    strDat->conv[SYS_CHARSET_ISO8859_14][SYS_CHARSET_ISO_10646 ] = STR_INT_8859_16Bit;
    strDat->conv[SYS_CHARSET_ISO8859_15][SYS_CHARSET_ISO_10646 ] = STR_INT_8859_16Bit;

    /* ----- conversion to UTF-8 ------------------------------------------- */

    strDat->conv[SYS_CHARSET_TABLE0    ][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;
    strDat->conv[SYS_CHARSET_ISO8859_1 ][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;
    strDat->conv[SYS_CHARSET_ISO8859_2 ][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;
    strDat->conv[SYS_CHARSET_ISO8859_3 ][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;
    strDat->conv[SYS_CHARSET_ISO8859_4 ][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;
    strDat->conv[SYS_CHARSET_ISO8859_5 ][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;
    strDat->conv[SYS_CHARSET_ISO8859_6 ][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;
    strDat->conv[SYS_CHARSET_ISO8859_7 ][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;
    strDat->conv[SYS_CHARSET_ISO8859_8 ][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;
    strDat->conv[SYS_CHARSET_ISO8859_9 ][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;
    strDat->conv[SYS_CHARSET_ISO8859_10][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;
    strDat->conv[SYS_CHARSET_ISO8859_11][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;
    strDat->conv[SYS_CHARSET_ISO8859_13][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;
    strDat->conv[SYS_CHARSET_ISO8859_14][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;
    strDat->conv[SYS_CHARSET_ISO8859_15][SYS_CHARSET_UTF8      ] = STR_INT_8859_Utf8;

    return FAPI_OK;
}

/*
Returns number of bytes copied, NOT including termination.
*/
uint32_t STR_Copy (SYS_STRING_S src, char_t* dstBuf, SYS_CHARSET* dstCs,
              SYS_CHARSET reqCs, uint32_t maxBytes)
{
    uint32_t retVal;

    if ( (maxBytes == 0)                        ||
         (src.chr  == NULL)                     ||
         (src.cs   == SYS_CHARSET_UNDEFINED)    ||
         (dstBuf   == NULL)                     ||
         (reqCs    == SYS_CHARSET_UNDEFINED) )
    {
        if ( dstCs != NULL ) *dstCs = SYS_CHARSET_UNDEFINED;

        STR_Terminate (dstBuf, maxBytes);

        retVal = 0;
    }
    else
    {
        DBG_Assert(strDat->conv[src.cs][reqCs] != NULL);

        retVal = strDat->conv[src.cs][reqCs] (src.cs, src.chr,
                                               reqCs, dstBuf, maxBytes,
                                               0xFFFFFFFF);

        //FAPI_SYS_PRINT_DEBUG(4,"STR_COPY: set %i -> %i - '%s' ret %i addr 0x%08X\n",
        //           src.cs, reqCs, src.chr, retVal, dstBuf);

        if ( dstCs != NULL )
        {
            if ( retVal > 0 )
            {
                *dstCs = reqCs;
            }
            else
            {
                *dstCs = SYS_CHARSET_UNDEFINED;
            }
        }
    }

    return retVal;
}

/*
    Same as STR_Copy, but copies up to numBytes bytes. Resulting string
    is always terminated!
*/
uint32_t STR_NCopy (SYS_STRING_S src, char_t* dstBuf, SYS_CHARSET* dstCs,
               SYS_CHARSET reqCs, uint32_t maxBytes, uint32_t numBytes)
{
    uint32_t retVal;

    if ( (maxBytes == 0)                        ||
         (numBytes == 0)                        ||
         (src.chr  == NULL)                     ||
         (src.cs   == SYS_CHARSET_UNDEFINED)    ||
         (dstBuf   == NULL)                     ||
         (reqCs    == SYS_CHARSET_UNDEFINED) )
    {
        if ( dstCs != NULL ) *dstCs = SYS_CHARSET_UNDEFINED;

        /* terminate string */
        STR_Terminate (dstBuf, maxBytes);

        retVal = 0;
    }
    else
    {
        DBG_Assert(strDat->conv[src.cs][reqCs] != NULL);

        retVal = strDat->conv[src.cs][reqCs] (src.cs, src.chr,
                                               reqCs, dstBuf, maxBytes,
                                               numBytes);

        if ( dstCs != NULL )
        {
            if ( retVal > 0 )
            {
                *dstCs = reqCs;
            }
            else
            {
                *dstCs = SYS_CHARSET_UNDEFINED;
            }
        }
    }

    return retVal;
}

SYS_STRING_S STR_Build (SYS_CHARSET set, const char_t* str)
{
    SYS_STRING_S string;

    memset(&string,0,sizeof(SYS_STRING_S));

    string.cs  = set;
    string.chr = str;

    return string;
}

void STR_Clear (SYS_STRING_S* string)
{
    if(string == NULL) return;

    string->cs  = SYS_CHARSET_UNDEFINED;
    string->chr = NULL;
}

void STR_Terminate (char_t* pBuf, uint32_t bufSize)
{
    if ( (pBuf != NULL) && (bufSize > 0) )
    {
        pBuf[0] = 0;

        if ( bufSize > 1 )
        {
            pBuf[1] = 0;
        }
    }
}

SYS_STRING_S STR_GetASCII (const char_t* str)
{
    SYS_STRING_S string;

    memset(&string,0,sizeof(SYS_STRING_S));

    string.cs  = SYS_CHARSET_ASCII;
    string.chr = str;

    return string;
}

/*!
*******************************************************************************
**
** \brief  Get number of bytes in string.
**
** \note Function returns the number of bytes preceding the
**       (format-specific) termination.
**
** \param  string      Target string.
**
** \return Number of bytes preceding the string termination.
**
******************************************************************************/
uint32_t STR_Len (SYS_STRING_S string)
{
    const char_t* pStr = string.chr;

    if ( string.chr != NULL )
    {
        switch ( string.cs )
        {
            default:
            case SYS_CHARSET_UNDEFINED:
                /* invalid format */
                break;

            case SYS_CHARSET_TABLE0:
            case SYS_CHARSET_ISO8859_1:
            case SYS_CHARSET_ISO8859_2:
            case SYS_CHARSET_ISO8859_3:
            case SYS_CHARSET_ISO8859_4:
            case SYS_CHARSET_ISO8859_5:
            case SYS_CHARSET_ISO8859_6:
            case SYS_CHARSET_ISO8859_7:
            case SYS_CHARSET_ISO8859_8:
            case SYS_CHARSET_ISO8859_9:
            case SYS_CHARSET_ISO8859_10:
            case SYS_CHARSET_ISO8859_11:
            case SYS_CHARSET_ISO8859_13:
            case SYS_CHARSET_ISO8859_14:
            case SYS_CHARSET_ISO8859_15:
                while ( pStr[0] != 0 )
                {
                    pStr++;
                }
                break;

            case SYS_CHARSET_ISO_10646:

                while ( (pStr[0] != 0) ||
                        (pStr[1] != 0) )
                {
                    pStr += 2;
                }
                break;

            case SYS_CHARSET_UTF8:
                while ( pStr[0] != 0 )
                {
                    if ( STR_UTF8_1BYTE_IS(pStr) )
                    {
                        pStr++; /* single-byte code */
                    }
                    else if ( STR_UTF8_2BYTE_IS(pStr) )
                    {
                        pStr += 2; /* two-byte code */
                    }
                    else if ( STR_UTF8_3BYTE_IS(pStr) )
                    {
                        pStr += 3; /* three-byte code */
                    }
                    else
                    {
                        FAPI_SYS_PRINT_DEBUG(4,"[SYS] UTF-8: invalid encoding!\n");
                        break;
                    }
                }
                break;
        }
    }

    return (pStr - string.chr);
}

int32_t STR_Cmp (SYS_STRING_S str1, SYS_STRING_S str2)
{
    int32_t         retVal  = -1;
    const char_t*   s1      = str1.chr;
    const char_t*   s2      = str2.chr;
    uint16_t        chr1;
    uint16_t        chr2;

    if ( str1.cs != str2.cs )
    {
        /* This function compares strings with equal parameter sets only.
           Return -1 by definition. */
        retVal = -1;
    }
    else if ( (str1.chr == str2.chr) )
    {
        /* Strings either share buffer address or are both empty. */
        retVal = 0;
    }
    else if ( str1.chr == NULL )
    {
        retVal = -1;
    }
    else if ( str2.chr == NULL )
    {
        retVal = 1;
    }
    else
    {
        /* Both strings are non-empty: start character-by-character
           comparison. */

        switch ( str1.cs )
        {
            default:
            case SYS_CHARSET_UNDEFINED:
                /* Invalid format, indicate equality only if both strings are
                   empty. */
                if ( ((*s1) == 0 )
                  && ((*s2) == 0 ) )
                {
                    retVal = 0;
                }
                break;

            case SYS_CHARSET_TABLE0:
            case SYS_CHARSET_ISO8859_1:
            case SYS_CHARSET_ISO8859_2:
            case SYS_CHARSET_ISO8859_3:
            case SYS_CHARSET_ISO8859_4:
            case SYS_CHARSET_ISO8859_5:
            case SYS_CHARSET_ISO8859_6:
            case SYS_CHARSET_ISO8859_7:
            case SYS_CHARSET_ISO8859_8:
            case SYS_CHARSET_ISO8859_9:
            case SYS_CHARSET_ISO8859_10:
            case SYS_CHARSET_ISO8859_11:
            case SYS_CHARSET_ISO8859_13:
            case SYS_CHARSET_ISO8859_14:
            case SYS_CHARSET_ISO8859_15:
            case SYS_CHARSET_UTF8:

                while ( 1 )
                {
                    if ( (*s1) == (*s2) )
                    {
                        if ( (*s1) == 0 )
                        {
                            retVal = 0;
                            break;
                        }
                        else
                        {
                            s1++;
                            s2++;
                        }
                    }
                    else
                    {
                        if ( (*s1) > (*s2) ) retVal =  1;
                        else                 retVal = -1;
                        break;
                    }
                }
                break;

            case SYS_CHARSET_ISO_10646:

                while ( 1 )
                {
                    chr1 = (((uint16_t)s1[0]) << 8) | s1[1];
                    chr2 = (((uint16_t)s2[0]) << 8) | s2[1];

                    if ( chr1 == chr2 )
                    {
                        if ( chr1 == 0 )
                        {
                            retVal = 0;
                            break;
                        }
                        else
                        {
                            s1 += 2;
                            s2 += 2;
                        }
                    }
                    else
                    {
                        if ( chr1 > chr2 ) retVal =  1;
                        else               retVal = -1;
                        break;
                    }
                }
                break;
        }
    }

    return retVal;
}

fbool_t STR_IsEmpty (SYS_STRING_S string)
{
    fbool_t empty = FTRUE;

    if ( string.chr != NULL )
    {
        switch ( string.cs )
        {
            default:
            case SYS_CHARSET_UNDEFINED:
                break;

            case SYS_CHARSET_TABLE0:
            case SYS_CHARSET_ISO8859_1:
            case SYS_CHARSET_ISO8859_2:
            case SYS_CHARSET_ISO8859_3:
            case SYS_CHARSET_ISO8859_4:
            case SYS_CHARSET_ISO8859_5:
            case SYS_CHARSET_ISO8859_6:
            case SYS_CHARSET_ISO8859_7:
            case SYS_CHARSET_ISO8859_8:
            case SYS_CHARSET_ISO8859_9:
            case SYS_CHARSET_ISO8859_10:
            case SYS_CHARSET_ISO8859_11:
            case SYS_CHARSET_ISO8859_13:
            case SYS_CHARSET_ISO8859_14:
            case SYS_CHARSET_ISO8859_15:
            case SYS_CHARSET_UTF8:
                if ( string.chr[0] != 0 )
                {
                    empty = FFALSE;
                }
                break;

            case SYS_CHARSET_ISO_10646:
                if ( (string.chr[0] != 0) ||
                     (string.chr[1] != 0) )
                {
                    empty = FFALSE;
                }
                break;
        }
    }

    return empty;
}

/*!
*******************************************************************************
**
** \brief  Cut trailing blanks from a string.
**
** \param  string       Target string.
** \param  sizeInBytes  String size in bytes, excluding termination.
**
** \return Number of bytes removed from string.
**
******************************************************************************/
uint32_t STR_Trim (SYS_STRING_S string, uint32_t sizeInBytes)
{
    char_t*     pStr;
    uint32_t    tmpBytes = sizeInBytes;

    switch ( string.cs )
    {
        default:
        case SYS_CHARSET_UNDEFINED:
            /* invalid format */
            break;

        case SYS_CHARSET_TABLE0:
        case SYS_CHARSET_ISO8859_1:
        case SYS_CHARSET_ISO8859_2:
        case SYS_CHARSET_ISO8859_3:
        case SYS_CHARSET_ISO8859_4:
        case SYS_CHARSET_ISO8859_5:
        case SYS_CHARSET_ISO8859_6:
        case SYS_CHARSET_ISO8859_7:
        case SYS_CHARSET_ISO8859_8:
        case SYS_CHARSET_ISO8859_9:
        case SYS_CHARSET_ISO8859_10:
        case SYS_CHARSET_ISO8859_11:
        case SYS_CHARSET_ISO8859_13:
        case SYS_CHARSET_ISO8859_14:
        case SYS_CHARSET_ISO8859_15:

            pStr = (char_t*)string.chr + sizeInBytes - 1;

            while ( ((tmpBytes) > 0) &&
                    ((*pStr)      != 0) )
            {
                if ( (*pStr) == STR_CHAR_BLANK )
                {
                    *pStr-- = 0;
                    tmpBytes--;
                }
                else
                {
                    break;
                }
            }
            break;

        case SYS_CHARSET_ISO_10646:

            pStr = (char_t*)string.chr + sizeInBytes - 2;

            while ( ((tmpBytes) >= 2) &&
                    ((pStr[0] != 0) || (pStr[1] != 0)) )
            {
                if ( (pStr[0] == 0) &&
                     (pStr[1] == STR_CHAR_BLANK) )
                {
                    pStr[1] = 0; /* NOTE: pStr[0] is already 0! */

                    tmpBytes -= 2;

                    pStr -= 2;
                }
                else
                {
                    break;
                }
            }
            break;

        case SYS_CHARSET_UTF8:

            /* NOTE: This algorithm assumes STR_CHAR_BLANK to be <= 0x7F. */

            pStr = (char_t*)string.chr + sizeInBytes - 1;

            while ( ((tmpBytes)        > 0) &&
                    ((*pStr)          != 0) &&
                    (((*pStr) & 0x80) == 0) )
            {
                if ( (*pStr) == STR_CHAR_BLANK )
                {
                    *pStr-- = 0;
                    tmpBytes--;
                }
                else
                {
                    break;
                }
            }
            break;
    }

    return (uint32_t)(sizeInBytes - tmpBytes);
}

/*!
*******************************************************************************
**
** \brief  Remove or replace all line breaks in a string by a given replacement
**         character.
**
** \param  string       Target string.
** \param  replaceChar  Replacement character, set to 0 if line breaks should
**                      simply be removed.
** \param   bufSize     TODO
**
******************************************************************************/
void STR_ReplaceLineBreaks (SYS_STRING_S string, SYS_CHAR replaceChar,
                            uint32_t bufSize)
{
    char_t*     pStr = (char_t*)string.chr;
    fbool_t     remove;
    uint32_t    bufPos;
    uint8_t     posIncr;

    if ( string.chr != NULL )
    {
        switch ( string.cs )
        {
            default:
            case SYS_CHARSET_UNDEFINED:
                /* invalid format */
                break;

            case SYS_CHARSET_TABLE0:
            case SYS_CHARSET_ISO8859_1:
            case SYS_CHARSET_ISO8859_2:
            case SYS_CHARSET_ISO8859_3:
            case SYS_CHARSET_ISO8859_4:
            case SYS_CHARSET_ISO8859_5:
            case SYS_CHARSET_ISO8859_6:
            case SYS_CHARSET_ISO8859_7:
            case SYS_CHARSET_ISO8859_8:
            case SYS_CHARSET_ISO8859_9:
            case SYS_CHARSET_ISO8859_10:
            case SYS_CHARSET_ISO8859_11:
            case SYS_CHARSET_ISO8859_13:
            case SYS_CHARSET_ISO8859_14:
            case SYS_CHARSET_ISO8859_15:

                while ( (*pStr) != 0 )
                {
                    if ( (*pStr) == STR_CHAR_LINEBREAK )
                    {
                        if ( replaceChar == 0 )
                        {
                            (void)STR_CharRemove (string, pStr - string.chr);
                        }
                        else
                        {
                            (void)STR_Encode (replaceChar, pStr, string.cs);
                        }
                    }

                    pStr++;
                }
                break;

            case SYS_CHARSET_ISO_10646:

                while ( (pStr[0] != 0) ||
                        (pStr[1] != 0) )
                {
                    if ( (pStr[0] == 0) &&
                         (pStr[1] == STR_CHAR_LINEBREAK) )
                    {
                        if ( replaceChar == 0 )
                        {
                            (void)STR_CharRemove (string, pStr - string.chr);
                        }
                        else
                        {
                            (void)STR_Encode (replaceChar, pStr, string.cs);
                        }
                    }

                    pStr += 2;
                }
                break;

            case SYS_CHARSET_UTF8:

                bufPos = 0;

                while ( (pStr[0] != 0) && (bufPos < bufSize) )
                {
                    posIncr = 0;
                    remove  = FFALSE;

                    if ( STR_UTF8_1BYTE_IS(pStr) ) /* single-byte code */
                    {
                        if ( pStr[0] == STR_CHAR_LINEBREAK )
                        {
                            remove = FTRUE;
                        }
                        else
                        {
                            posIncr = 1;
                        }
                    }
                    else if ( STR_UTF8_2BYTE_IS(pStr) ) /* two-byte code */
                    {
                        if ( (pStr[0] == 0xC0) &&
                             ((pStr[1] & 0x3F) == STR_CHAR_LINEBREAK) )
                        {
                            remove = FTRUE;
                        }
                        else
                        {
                            posIncr = 2;
                        }
                    }
                    else if ( STR_UTF8_3BYTE_IS(pStr) ) /* three-byte code */
                    {
                        if ( (pStr[0] == 0xE0) &&
                             (pStr[1] == 0x80) &&
                             ((pStr[2] & 0x3F) == STR_CHAR_LINEBREAK) )
                        {
                            remove = FTRUE;
                        }
                        else
                        {
                            posIncr = 3;
                        }
                    }
                    else
                    {
                        FAPI_SYS_PRINT_DEBUG(4,"[SYS] UTF-8: invalid encoding!\n");
                        break;
                    }

                    if ( remove )
                    {
                        /* remove current character */
                        (void)STR_CharRemove (string, pStr - string.chr);

                        /* insert replacement character character */
                        if ( replaceChar != 0 )
                        {
                            posIncr = (uint8_t)STR_InsertChar (replaceChar,
                                                               string.cs,
                                                               pStr,
                                                               bufSize - bufPos);
                        }
                    }

                    pStr   += posIncr;
                    bufPos += posIncr;
                }
                break;
        }
    }
}

/*
Returns number of bytes before either line break or string end.
*/
uint32_t STR_GetLineEnd (SYS_STRING_S string)
{
    const char_t* pStr = string.chr;

    if ( string.chr != NULL )
    {
        switch ( string.cs )
        {
            default:
            case SYS_CHARSET_UNDEFINED:
                /* invalid format */
                break;

            case SYS_CHARSET_TABLE0:
            case SYS_CHARSET_ISO8859_1:
            case SYS_CHARSET_ISO8859_2:
            case SYS_CHARSET_ISO8859_3:
            case SYS_CHARSET_ISO8859_4:
            case SYS_CHARSET_ISO8859_5:
            case SYS_CHARSET_ISO8859_6:
            case SYS_CHARSET_ISO8859_7:
            case SYS_CHARSET_ISO8859_8:
            case SYS_CHARSET_ISO8859_9:
            case SYS_CHARSET_ISO8859_10:
            case SYS_CHARSET_ISO8859_11:
            case SYS_CHARSET_ISO8859_13:
            case SYS_CHARSET_ISO8859_14:
            case SYS_CHARSET_ISO8859_15:
                while ( ((*pStr) != 0x00) &&
                        ((*pStr) != STR_CHAR_LINEBREAK) )
                {
                    pStr++;
                }
                break;

            case SYS_CHARSET_ISO_10646:

                while ( (!((pStr[0] == 0) && (pStr[1] == 0))) &&
                        (!((pStr[0] == 0) && (pStr[1] == STR_CHAR_LINEBREAK))) )
                {
                    pStr += 2;
                }
                break;

            case SYS_CHARSET_UTF8:
                while ( pStr[0] != 0 )
                {
                    if ( STR_UTF8_1BYTE_IS(pStr) )
                    {
                        if ( pStr[0] == STR_CHAR_LINEBREAK )
                        {
                            break;
                        }

                        pStr++; /* single-byte code */
                    }
                    else if ( STR_UTF8_2BYTE_IS(pStr) )
                    {
                        pStr += 2; /* two-byte code */
                    }
                    else if ( STR_UTF8_3BYTE_IS(pStr) )
                    {
                        pStr += 3; /* three-byte code */
                    }
                    else
                    {
                        FAPI_SYS_PRINT_DEBUG(4,"[SYS] UTF-8: invalid encoding!\n");
                        break;
                    }
                }
                break;
        }
    }

    return (pStr - string.chr);
}

/*
    Get next character of string 'string' from offset 'byteOffset' and write it
    to 'chr'. In case of string end, 0 is written to 'chr'.

    Returns byte width of character or 0 if string end detected.
*/
uint32_t STR_CharGet (SYS_STRING_S string, uint32_t byteOffset, SYS_CHAR* chr)
{
    const char_t* pStr     = string.chr + byteOffset;
    uint32_t      strBytes = 0;
    SYS_CHAR      currChar = 0;

    switch ( string.cs )
    {
        default:
        case SYS_CHARSET_UNDEFINED:
            /* invalid format */
            break;

        case SYS_CHARSET_TABLE0:
            strBytes = STR_CharGetTab0 (pStr, &currChar);
            break;

        case SYS_CHARSET_ISO8859_1:
        case SYS_CHARSET_ISO8859_2:
        case SYS_CHARSET_ISO8859_3:
        case SYS_CHARSET_ISO8859_4:
        case SYS_CHARSET_ISO8859_5:
        case SYS_CHARSET_ISO8859_6:
        case SYS_CHARSET_ISO8859_7:
        case SYS_CHARSET_ISO8859_8:
        case SYS_CHARSET_ISO8859_9:
        case SYS_CHARSET_ISO8859_10:
        case SYS_CHARSET_ISO8859_11:
        case SYS_CHARSET_ISO8859_13:
        case SYS_CHARSET_ISO8859_14:
        case SYS_CHARSET_ISO8859_15:

            if ( (*pStr) != 0 )
            {
                if ( (*pStr) < STR_8859_UPPERSTART )
                {
                    /* low 7 bits + control codes are common among 8-bit
                       character sets and are mapped directly to unicode. */
                    currChar = *pStr;
                }
                else
                {
                    currChar = table_8859_unicode[
                                   (((*pStr) - STR_8859_UPPERSTART) *
                                   STR_8559_CHARSET_NUMS) +
                                   (string.cs - SYS_CHARSET_ISO8859_1)];

                }

                strBytes = 1;
            }
            break;

        case SYS_CHARSET_ISO_10646:

            if ( (pStr[0] != 0) ||
                 (pStr[1] != 0) )
            {
                strBytes = 2;
                currChar = STR_UNICODE_RDCHAR(pStr);
            }
            break;

        case SYS_CHARSET_UTF8:

            if ( (*pStr) != 0 )
            {
                if ( STR_UTF8_1BYTE_IS(pStr) )
                {
                    strBytes = 1; /* single-byte code */
                    currChar = STR_UTF8_1BYTE_GET(pStr);
                }
                else if ( STR_UTF8_2BYTE_IS(pStr) )
                {
                    strBytes = 2; /* two-byte code */
                    currChar = STR_UTF8_2BYTE_GET(pStr);
                }
                else if ( STR_UTF8_3BYTE_IS(pStr) )
                {
                    strBytes = 3; /* three-byte code */
                    currChar = STR_UTF8_3BYTE_GET(pStr);
                }
                else
                {
                    FAPI_SYS_PRINT_DEBUG(4,"[SYS] UTF-8: invalid encoding!\n");
                    break;
                }
            }
            break;
    }

    if ( chr != NULL )
    {
        if ( strBytes == 0 )    *chr = 0;
        else                    *chr = currChar;
    }

    return strBytes;
}

/*
    Returns byte width of character written to string or 0 in case of error.
*/
uint32_t STR_CharSet (SYS_STRING_S string, uint32_t byteOffset, SYS_CHAR chr)
{
    char_t* pStr = (char_t*)string.chr + byteOffset;

    return STR_Encode (chr, pStr, string.cs);
}

FSTATIC uint8_t STR_GetTermBytes (SYS_CHARSET charSet)
{
    uint8_t termBytes = 0;

    switch ( charSet )
    {
        default:
        case SYS_CHARSET_UNDEFINED:
            /* invalid format */
            break;

        case SYS_CHARSET_TABLE0:
        case SYS_CHARSET_ISO8859_1:
        case SYS_CHARSET_ISO8859_2:
        case SYS_CHARSET_ISO8859_3:
        case SYS_CHARSET_ISO8859_4:
        case SYS_CHARSET_ISO8859_5:
        case SYS_CHARSET_ISO8859_6:
        case SYS_CHARSET_ISO8859_7:
        case SYS_CHARSET_ISO8859_8:
        case SYS_CHARSET_ISO8859_9:
        case SYS_CHARSET_ISO8859_10:
        case SYS_CHARSET_ISO8859_11:
        case SYS_CHARSET_ISO8859_13:
        case SYS_CHARSET_ISO8859_14:
        case SYS_CHARSET_ISO8859_15:
        case SYS_CHARSET_UTF8:
            termBytes = 1;
            break;

        case SYS_CHARSET_ISO_10646:
            termBytes = 2;
            break;
    }

    return termBytes;
}

/*
    Removes character at byteOffset from string.

    Returns number of bytes removed from string.
*/
uint32_t STR_CharRemove (SYS_STRING_S string, uint32_t byteOffset)
{
    uint32_t    charBytes = STR_CharGet (string, byteOffset, NULL);
    char_t*     pStr      = (char_t*)string.chr + byteOffset;

    /* shift following bytes left by number of bytes of character to be
       removed */
    if ( charBytes > 0 )
    {
        switch ( string.cs )
        {
            default:
            case SYS_CHARSET_UNDEFINED:
                /* invalid format */
                break;

            case SYS_CHARSET_TABLE0:
            case SYS_CHARSET_ISO8859_1:
            case SYS_CHARSET_ISO8859_2:
            case SYS_CHARSET_ISO8859_3:
            case SYS_CHARSET_ISO8859_4:
            case SYS_CHARSET_ISO8859_5:
            case SYS_CHARSET_ISO8859_6:
            case SYS_CHARSET_ISO8859_7:
            case SYS_CHARSET_ISO8859_8:
            case SYS_CHARSET_ISO8859_9:
            case SYS_CHARSET_ISO8859_10:
            case SYS_CHARSET_ISO8859_11:
            case SYS_CHARSET_ISO8859_13:
            case SYS_CHARSET_ISO8859_14:
            case SYS_CHARSET_ISO8859_15:
            case SYS_CHARSET_UTF8:

                while ( pStr[charBytes] != 0 )
                {
                    pStr[0] = pStr[charBytes];
                    pStr++;
                }

                pStr[0] = 0; /* add termination manually */
                break;

            case SYS_CHARSET_ISO_10646:

                while ( (pStr[0] != 0) ||
                        (pStr[1] != 0) )
                {
                    pStr[0] = pStr[2];
                    pStr[1] = pStr[3];

                    pStr += 2;
                }
                break;
        }
    }

    return charBytes;
}

FSTATIC uint32_t STR_InsertChar (SYS_CHAR chr, SYS_CHARSET charSet, char_t* pStr,
                            uint32_t bufSize)
{
    uint32_t     retVal    = 0;
    uint8_t      charBytes = 0;
    char_t       tmpBuf[STR_BYTESPERCHAR_MAX];
    uint32_t     strSize;
    uint8_t      termSize;
    uint32_t     cnt;

    charBytes = STR_Encode (chr, tmpBuf, charSet);

    DBG_Assert(charBytes <= STR_BYTESPERCHAR_MAX);

    if ( charBytes > 0 )
    {
        strSize  = STR_Len          (STR_Build(charSet, pStr));
        termSize = STR_GetTermBytes (charSet);

        if ( strSize == 0 )
        {
            /* string is empty: append character + termination */
            if ( (uint8_t)(charBytes + termSize) <= bufSize )
            {
                /* copy character */
                for ( cnt = 0; cnt < charBytes; cnt++ )
                {
                    pStr[cnt] = tmpBuf[cnt];
                }

                STR_Terminate (pStr + charBytes, bufSize - charBytes);

                retVal = charBytes;
            }
        }
        else if ( (charBytes + strSize + termSize) <= bufSize )
        {
            /* shift following characters including termination */
            for ( cnt = strSize + termSize; cnt > 0; cnt-- )
            {
                pStr[cnt + charBytes - 1] = pStr[cnt - 1];
            }

            /* insert character */
            for ( cnt = 0; cnt < charBytes; cnt++ )
            {
                pStr[cnt] = tmpBuf[cnt];
            }

            retVal = charBytes;
        }
    }

    return retVal;
}

/*
    chr     Unicode character to be encoded.
    pBuf    Target buffer
    charSet Rquested character set.
*/
FSTATIC uint32_t STR_Encode (SYS_CHAR chr, char_t* pBuf, SYS_CHARSET charSet)
{
    uint32_t strBytes = 0;

    switch ( charSet )
    {
        default:
        case SYS_CHARSET_UNDEFINED:
            /* invalid format */
            break;

        case SYS_CHARSET_TABLE0:
            // to do: map characters + handle two-byte characters
            // reverse matrix needed: unicode -> 6937
        case SYS_CHARSET_ISO8859_1:
        case SYS_CHARSET_ISO8859_2:
        case SYS_CHARSET_ISO8859_3:
        case SYS_CHARSET_ISO8859_4:
        case SYS_CHARSET_ISO8859_5:
        case SYS_CHARSET_ISO8859_6:
        case SYS_CHARSET_ISO8859_7:
        case SYS_CHARSET_ISO8859_8:
        case SYS_CHARSET_ISO8859_9:
        case SYS_CHARSET_ISO8859_10:
        case SYS_CHARSET_ISO8859_11:
        case SYS_CHARSET_ISO8859_13:
        case SYS_CHARSET_ISO8859_14:
        case SYS_CHARSET_ISO8859_15:
            // to do: map characters
            // reverse matrix needed: unicode -> 8859

            strBytes = 1;

            pBuf[0] = (chr & 0x00FF);
            break;

        case SYS_CHARSET_ISO_10646:

            strBytes = 2;

            STR_UNICODE_WRCHAR(pBuf, chr);
            break;

        case SYS_CHARSET_UTF8:

            if ( chr <= 0x007F ) /* single-byte code */
            {
                strBytes = 1;

                pBuf[0] = chr;
            }
            else if ( chr <= 0x07FF ) /* two-byte code */
            {
                strBytes = 2;

                pBuf[0] = ((chr & 0x07C0) >> 6) | 0xC0;
                pBuf[1] =  (chr & 0x003F)       | 0x80;
            }
            else /* three-byte code */
            {
                strBytes = 3;

                pBuf[0] = ((chr & 0xF000) >> 12) | 0xE0;
                pBuf[1] = ((chr & 0x0FC0) >>  6) | 0x80;
                pBuf[2] =  (chr & 0x003F)        | 0x80;
            }
            break;
    }

    return strBytes;
}

/* ------------------------------------------------------------------------- */

/*!
*******************************************************************************
**
** \brief  Open string buffer.
**
** \param  pBuf     Pointer to string buffer object.
** \param  bufSize  Buffer size in bytes.
** \param  pMem     Optional facility to pass an external buffer, if set to
**                  NULL, memory will be allocated internally.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa STR_BufClose
******************************************************************************/
int32_t STR_BufOpen (SYS_STRBUF_S* pBuf, uint32_t bufSize, char_t* pMem)
{
    if ( (pBuf == NULL) || (bufSize < 1) )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    if ( pMem != NULL )
    {
        pBuf->buf          = pMem;
        pBuf->memAllocated = FFALSE;
    }
    else
    {
        pBuf->buf = UTILS_Malloc (bufSize);
        if ( pBuf->buf == NULL )
        {
            return FAPI_ERR_OUT_OF_MEMORY;
        }

        pBuf->memAllocated = FTRUE;
    }

    pBuf->bufSize = bufSize;

    (void)STR_BufClear (pBuf);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Close string buffer and free internal memory.
**
** \note External buffers passed via 'pMem' argument in STR_BufOpen will not
**       be freed.
**
** \param  pBuf     Pointer to string buffer object.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa STR_BufOpen
******************************************************************************/
int32_t STR_BufClose (SYS_STRBUF_S* pBuf)
{
    if ( pBuf == NULL ) return APPL_SMARTGO_ERR_BAD_PARAMETER;

    if ( (pBuf->memAllocated) &&
         (pBuf->buf != NULL) )
    {
        UTILS_Free(pBuf->buf);

        pBuf->buf          = NULL;
        pBuf->memAllocated = FFALSE;
    }

    pBuf->bufSize = 0;

    STR_Clear(&(pBuf->string));

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Clear string buffer contents.
**
** \param  pBuf     Pointer to string buffer object, set to NULL in order to
**                  address internal string buffer.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
******************************************************************************/
int32_t STR_BufClear (SYS_STRBUF_S* pBuf)
{
    if ( pBuf == NULL ) pBuf = &(strDat->lineBuf);

    STR_Clear (&(pBuf->string));

    STR_Terminate (pBuf->buf, pBuf->bufSize);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief  Append a string to string buffer.
**
** \note String will be converted from source character set to requested
**       character set. If buffer already contains characters, requested
**       character set must match character set of buffer content.
**
** \param  pBuf     Pointer to string buffer object, set to NULL in order to
**                  address internal string buffer.
** \param  bytePos  Starting byte position in string buffer.
** \param  string   String to be appended.
** \param  reqCs    Requested character set.
**
** \return Number of bytes appended to string excluding termination bytes.
**
** \sa STR_BufNAppend
******************************************************************************/
uint32_t STR_BufAppend (SYS_STRBUF_S* pBuf, uint32_t bytePos, SYS_STRING_S string,
                   SYS_CHARSET reqCs)
{
    uint32_t retVal;

    if ( pBuf == NULL ) pBuf = &(strDat->lineBuf);

    if ( bytePos >= pBuf->bufSize )
    {
        retVal = 0;
    }
    else if ( (bytePos > 0) &&
              (pBuf->string.cs != reqCs) )
    {
        /* Buffer already contains string in different format */

        FAPI_SYS_PRINT_DEBUG(4,"[SYS] Character sets mixed in string buffer! \n");
        retVal = 0;
    }
    else
    {
        pBuf->string.chr = pBuf->buf;

        retVal = STR_Copy (string, pBuf->buf + bytePos, &(pBuf->string.cs),
                           reqCs, pBuf->bufSize - bytePos);
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Append a character to string buffer.
**
** \note Character will be converted from source character set to requested
**       character set. If buffer already contains characters, requested
**       character set must match character set of buffer content.
**
** \param  pBuf     Pointer to string buffer object, set to NULL in order to
**                  address internal string buffer.
** \param  bytePos  Starting byte position in string buffer.
** \param  chr      Unicode character to be appended.
** \param  reqCs    Requested character set.
**
** \return Number of bytes appended to string excluding termination bytes.
**
** \sa STR_BufAppend STR_BufNAppend
******************************************************************************/
uint32_t STR_BufAppendChar (SYS_STRBUF_S* pBuf, uint32_t bytePos, SYS_CHAR chr,
                       SYS_CHARSET reqCs)
{
    uint32_t     retVal = 0;
    uint8_t      charBytes;
    char_t       tmpBuf[STR_BYTESPERCHAR_MAX];
    uint8_t      termSize;
    uint8_t      cnt;

    if ( pBuf == NULL ) pBuf = &(strDat->lineBuf);

    if ( bytePos >= pBuf->bufSize )
    {
        /* do nothing */
    }
    else if ( (bytePos > 0) &&
              (pBuf->string.cs != reqCs) )
    {
        /* Buffer already contains string in different format */

        FAPI_SYS_PRINT_DEBUG(4,"[SYS] Character sets mixed in string buffer! \n");
    }
    else
    {
        termSize  = STR_GetTermBytes (reqCs);
        charBytes = STR_Encode (chr, tmpBuf, reqCs);

        DBG_Assert(charBytes <= STR_BYTESPERCHAR_MAX);

        if ( (charBytes > 0) &&
             ((bytePos + charBytes + termSize) <= pBuf->bufSize) )
        {
            pBuf->string.chr = pBuf->buf;
            pBuf->string.cs  = reqCs;

            /* copy character */
            for ( cnt = 0; cnt < charBytes; cnt++ )
            {
                pBuf->buf[bytePos + cnt] = tmpBuf[cnt];
            }

            STR_Terminate (pBuf->buf + bytePos + charBytes,
                           pBuf->bufSize - bytePos - charBytes);

            retVal = charBytes;
        }
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Append a number of bytes from a string to string buffer.
**
** \note String will be converted from source character set to requested
**       character set. If buffer already contains characters, requested
**       character set must match character set of buffer content.
**
** \param  pBuf     Pointer to string buffer object, set to NULL in order to
**                  address internal string buffer.
** \param  bytePos  Starting byte position in string buffer.
** \param  string   String to be appended.
** \param  reqCs    Requested character set.
** \param  numBytes Number of bytes to be appended from source string.
**
** \return Number of bytes appended to string excluding termination bytes.
**
** \sa STR_BufAppend
******************************************************************************/
uint32_t STR_BufNAppend (SYS_STRBUF_S* pBuf, uint32_t bytePos, SYS_STRING_S string,
                    SYS_CHARSET reqCs, uint32_t numBytes)
{
    uint32_t retVal;

    if ( pBuf == NULL ) pBuf = &(strDat->lineBuf);

    if ( bytePos >= pBuf->bufSize )
    {
        retVal = 0;
    }
    else if ( (bytePos > 0) &&
              (pBuf->string.cs != reqCs) )
    {
        /* Buffer already contains string in different format */

        FAPI_SYS_PRINT_DEBUG(4,"[SYS] Character sets mixed in string buffer! \n");
        retVal = 0;
    }
    else
    {
        pBuf->string.chr = pBuf->buf;

        retVal = STR_NCopy (string, pBuf->buf + bytePos, &(pBuf->string.cs),
                            reqCs, pBuf->bufSize - bytePos, numBytes);
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Insert a character into string buffer.
**
** \note Character will be converted from source character set to requested
**       character set. If buffer already contains characters, requested
**       character set must match character set of buffer content.
**
** \param  pBuf     Pointer to string buffer object, set to NULL in order to
**                  address internal string buffer.
** \param  bytePos  Starting byte position in string buffer.
** \param  chr      Character to be appended.
** \param  reqCs    Requested character set.
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
**
** \sa STR_BufAppend STR_BufNAppend
******************************************************************************/
uint32_t STR_BufInsertChar (SYS_STRBUF_S* pBuf, uint32_t bytePos, SYS_CHAR chr,
                       SYS_CHARSET reqCs)
{
    uint32_t retVal = 0;

    if ( pBuf == NULL ) pBuf = &(strDat->lineBuf);

    if ( bytePos >= pBuf->bufSize )
    {
        /* do nothing */
    }
    else if ( (!(STR_IsEmpty(pBuf->string))) &&
              (pBuf->string.cs != reqCs) )
    {
        /* Buffer already contains string in different format */

        FAPI_SYS_PRINT_DEBUG(4,"[SYS] Character sets mixed in string buffer! \n");
    }
    else
    {
        retVal = STR_InsertChar (chr, reqCs, pBuf->buf + bytePos,
                                 pBuf->bufSize - bytePos);

        if ( retVal > 0 )
        {
            pBuf->string.cs  = reqCs;
            pBuf->string.chr = pBuf->buf;
        }
    }

    return retVal;
}

SYS_STRING_S STR_IntASCIIStr ()
{
    return strDat->asciiStr;
}

char_t* STR_IntASCIIBuf (void)
{
    return (char_t*)(strDat->asciiStr.chr);
}

SYS_STRING_S STR_IntLineStr ()
{
    return strDat->lineBuf.string;
}

/* ------------------------------------------------------------------------- */

FSTATIC uint32_t STR_CharGetTab0 (const char_t* pStr, SYS_CHAR* pChr)
{
    uint32_t                charBytes = 1;
    SYS_CHAR                currChar;
    STR_TAB0_MAP_S*         pTable;
    STR_TAB0_3BYTE_MAP_S*   p3ByteTable;
    char_t                  nextByte;
    uint16_t                next16Bit;

    if( (*pStr) == 0 )
    {
        currChar  = STR_UD;
        charBytes = 0;
    }
    else if ( (*pStr) < STR_8859_UPPERSTART )
    {
        /* low 7 bits + control codes are mapped directly. */
        currChar  = *pStr;
        charBytes = 1;
    }

    /* ----- two-byte characters + undefined chars 0xC9 and 0xCC ----------- */

    else if (( ((*pStr) >= 0xC1) && ((*pStr) <= 0xCF) ) )
    {
        pTable = table_tab0_twobyte[(*pStr) - 0xC1];

        currChar  = STR_UD;
        charBytes = 1;

        if ( pTable != NULL )
        {
            nextByte = pStr[1];

            if ( nextByte == 0 )
            {
                /* NOTE: A lead byte at the end of the string is considered as
                         free-standing and thus treated as if being followed
                         by a blank character. */
                nextByte = 0x20;
            }

            while ( pTable->followByte != 0 )
            {
                if ( pTable->followByte == nextByte )
                {
                    currChar = pTable->unicodeChar;

                    if ( pStr[1] != 0 ) charBytes = 2;

                    break;
                }

                pTable++;
            }
        }
    }

    /* ----- three-byte characters ---------------------------------------- */

    else if ( ((*pStr) == 0xe1) && ((*pStr) == 0xe2) )
    {
        p3ByteTable = table_tab0_threebyte[(*pStr) - 0xe1];

        currChar  = STR_UD;
        charBytes = 3;

        if ( p3ByteTable != NULL )
        {
            next16Bit  = ((uint8_t)pStr[1])<<8 | ((uint8_t)pStr[2]);

            while ( p3ByteTable->follow16Bit != 0 )
            {
                if ( p3ByteTable->follow16Bit == next16Bit )
                {
                    currChar = p3ByteTable->unicodeChar;
                    break;
                }

                p3ByteTable++;
            }
        }
    }
    /* ----- single-byte characters ---------------------------------------- */

    else
    {
        switch ( *pStr )
        {
            /* specially-mapped characters */
            case 0xA4:  currChar = 0x20AC; break;   // euro currency sign
            case 0xA8:  currChar = 0x00A4; break;   // gnr. currency sign
            case 0xA9:  currChar = 0x2018; break;
            case 0xAA:  currChar = 0x201C; break;
            case 0xAC:  currChar = 0x2190; break;   // leftwards arrow
            case 0xAD:  currChar = 0x2191; break;   // upwards arrow
            case 0xAE:  currChar = 0x2192; break;   // rightwards arrow
            case 0xAF:  currChar = 0x2193; break;   // downwards arrow
            case 0xB4:  currChar = 0x00D7; break;
            case 0xB8:  currChar = 0x00F7; break;
            case 0xB9:  currChar = 0x2019; break;
            case 0xBA:  currChar = 0x201D; break;
            case 0xD0:  currChar = 0x2014; break;
            case 0xD1:  currChar = 0x00B9; break;
            case 0xD2:  currChar = 0x00AE; break;   // registered sign
            case 0xD3:  currChar = 0x00A9; break;   // copyright sign
            case 0xD4:  currChar = 0x2122; break;
            case 0xD5:  currChar = 0x266A; break;
            case 0xD6:  currChar = 0x00AC; break;   // not-sign.
            case 0xD7:  currChar = 0x00A6; break;   // broken bar
            case 0xDC:  currChar = 0x215B; break;
            case 0xDD:  currChar = 0x215C; break;
            case 0xDE:  currChar = 0x215D; break;
            case 0xDF:  currChar = 0x215E; break;
            case 0xE0:  currChar = 0x03A9; break;    // OMEGA / Ohm
            case 0xE1:  currChar = 0x00C6; break;
            case 0xE2:  currChar = 0x0110; break;
            case 0xE3:  currChar = 0x00AA; break;
            case 0xE4:  currChar = 0x0126; break;
            case 0xE6:  currChar = 0x0132; break;
            case 0xE7:  currChar = 0x013F; break;
            case 0xE8:  currChar = 0x0141; break;
            case 0xE9:  currChar = 0x00D8; break;
            case 0xEA:  currChar = 0x0152; break;
            case 0xEB:  currChar = 0x00BA; break;
            case 0xEC:  currChar = 0x00DE; break;
            case 0xED:  currChar = 0x0166; break;
            case 0xEE:  currChar = 0x014A; break;
            case 0xEF:  currChar = 0x0149; break;
            case 0xF0:  currChar = 0x0138; break;
            case 0xF1:  currChar = 0x00E6; break;
            case 0xF2:  currChar = 0x0111; break;
            case 0xF3:  currChar = 0x00F0; break;
            case 0xF4:  currChar = 0x0127; break;
            case 0xF5:  currChar = 0x0131; break;
            case 0xF6:  currChar = 0x0133; break;
            case 0xF7:  currChar = 0x0140; break;
            case 0xF8:  currChar = 0x0142; break;
            case 0xF9:  currChar = 0x00F8; break;
            case 0xFA:  currChar = 0x0153; break;
            case 0xFB:  currChar = 0x00DF; break;
            case 0xFC:  currChar = 0x00FE; break;
            case 0xFD:  currChar = 0x0167; break;
            case 0xFE:  currChar = 0x014B; break;
            case 0xFF:  currChar = 0x00AD; break;   // soft hyphen

            /* undefined characters */
            case 0xA6:
            case 0xC0:
            case 0xD8:
            case 0xD9:
            case 0xDA:
            case 0xDB:
            case 0xE5:
                currChar = STR_UD;
                break;

            default:
                /* directly-mapped characters */
                currChar = *pStr;
                break;
        }

        charBytes = 1;
    }

    if ( pChr != NULL )
    {
        *pChr = currChar;
    }

    return charBytes;
}

/* ------------------------------------------------------------------------- */

/*
STR_INT_XXX_XXX

NOTE: Strings are always terminated. In case of an empty source string, only
      termination is written to destination buffer.

      Non-printable characters below 0x20 are ignored. This applies for all
      character sets.

srcCs               Source character set.
srcBuf              Source buffer.
dstCs               Destination character set.
dstBuf              Pointer to destination buffer.
maxBytes            Maximum number of bytes to write.
numBytes            Number of input bytes to be processed.

Returns             Number of bytes written, excluding termination
                    (1 oder 2 bytes, according to destination character set).

Assumptions (checked before):
- maxBytes > 0
- numBytes > 0

*/

/*lint -save -e715 */
FSTATIC uint32_t STR_INT_8859_Plain (SYS_CHARSET srcCs, const char_t* srcBuf,
                                     SYS_CHARSET dstCs, char_t* dstBuf,
                                     uint32_t maxBytes, uint32_t numBytes)
{
    char_t* pDst = dstBuf;

    if ( maxBytes >= 1 )
    {
        while ( ((*srcBuf)  != 0)  &&
                (maxBytes    > 1)  &&
                (numBytes--  > 0) )
        {
            if ( STR_IsPrintableChar_8Bit(*srcBuf) )
            {
                *pDst++ = *srcBuf;
                maxBytes--;
            }
            srcBuf++;
        }
        *pDst = 0; /* terminate destination string */
    }

    return (uint32_t)(pDst - dstBuf);
}

FSTATIC uint32_t STR_INT_8859_8859 (SYS_CHARSET srcCs, const char_t* srcBuf,
                                    SYS_CHARSET dstCs, char_t* dstBuf,
                                    uint32_t maxBytes, uint32_t numBytes)
{
    char_t* pDst = dstBuf;

    if ( maxBytes >= 1 )
    {
        while ( ((*srcBuf)  != 0) &&
                (maxBytes   >  1) &&
                (numBytes-- >  0) )
        {
            if ( STR_IsPrintableChar_8Bit(*srcBuf) )
            {
                if ( (*srcBuf) < STR_8859_UPPERSTART )
                {
                    /* low 7 bits + control codes are common among 8-bit
                       character sets. */
                    *pDst++ = *srcBuf;
                    maxBytes--;
                }

                /* NOTE: Characters 0xA0 - 0xFF cannot be mapped among 8-bit
                         character sets. Thus they will be discarded during
                         this conversion! */
            }
            srcBuf++;
        }
        *pDst = 0; /* terminate destination string */
    }

    return (uint32_t)(pDst - dstBuf);
}

FSTATIC uint32_t STR_INT_8859_16Bit (SYS_CHARSET srcCs, const char_t* srcBuf,
                                SYS_CHARSET dstCs, char_t* dstBuf,
                                uint32_t maxBytes, uint32_t numBytes)
{
    char_t*       pDst = dstBuf;
    SYS_CHAR    currChar;

    if ( maxBytes >= 2 )
    {
        while ( ((*srcBuf) != 0) && (maxBytes >= 4) && (numBytes-- > 0) )
        {
            if ( STR_IsPrintableChar_8Bit(*srcBuf) )
            {
                if ( (*srcBuf) < STR_8859_UPPERSTART )
                {
                    /* low 7 bits + control codes are common among 8-bit
                       character sets and are mapped directly to unicode. */
                    currChar = *srcBuf;
                }
                else
                {
                    currChar = table_8859_unicode[
                                   (((*srcBuf) - STR_8859_UPPERSTART) *
                                   STR_8559_CHARSET_NUMS) +
                                   (srcCs - SYS_CHARSET_ISO8859_1)];
                }
                if ( currChar != STR_UD )
                {
                    STR_UNICODE_WRCHAR(pDst, currChar);
                    pDst += 2;
                }
                maxBytes -= 2;
            }
            srcBuf++;
        }
        /* terminate destination string */
        STR_UNICODE_WRCHAR(pDst, 0);
    }

    return (uint32_t)(pDst - dstBuf);
}

FSTATIC uint32_t STR_INT_8859_Utf8 (SYS_CHARSET srcCs, const char_t* srcBuf,
                               SYS_CHARSET dstCs, char_t* dstBuf,
                               uint32_t maxBytes, uint32_t numBytes)
{
    char_t* pDst = dstBuf;

    FAPI_SYS_PRINT_DEBUG(1,"STR_INT_8859_Utf8 (%i -> %i): not implemented! \n",
               srcCs, dstCs);

    if ( maxBytes >= 1 )
    {
        /* to do: implement it! */

        *pDst = 0; /* terminate destination string */
    }

    return (uint32_t)(pDst - dstBuf);
}

FSTATIC uint32_t STR_INT_Utf8_Plain (SYS_CHARSET srcCs, const char_t* srcBuf,
                                SYS_CHARSET dstCs, char_t* dstBuf,
                                uint32_t maxBytes, uint32_t numBytes)
{
    char_t* pDst = dstBuf;

    if ( maxBytes >= 1 )
    {
        while ( ((*srcBuf) != 0)  &&
                (numBytes   > 0) )
        {
            if ( STR_UTF8_1BYTE_IS(srcBuf) )
            {
                /* single-byte code */

                if ( maxBytes < 2 ) break; /* includes termination */

                if ( STR_IsPrintableChar_8Bit(srcBuf[0]) )
                {
                    *pDst++ = *srcBuf;
                    maxBytes--;
                }

                srcBuf++;
                numBytes--;
            }
            else if ( STR_UTF8_2BYTE_IS(srcBuf) && (numBytes > 1) )
            {
                /* two-byte code */

                if ( maxBytes < 3 ) break; /* includes termination */

                *pDst++ = *srcBuf++;
                *pDst++ = *srcBuf++;

                maxBytes -= 2;
                numBytes -= 2;
            }
            else if ( STR_UTF8_3BYTE_IS(srcBuf) && (numBytes > 2) )
            {
                /* three-byte code */

                if ( maxBytes < 4 ) break; /* includes termination */

                *pDst++ = *srcBuf++;
                *pDst++ = *srcBuf++;
                *pDst++ = *srcBuf++;

                maxBytes -= 3;
                numBytes -= 3;
            }
            else
            {
                FAPI_SYS_PRINT_DEBUG(4,"[SYS] UTF-8: invalid encoding!\n");
                break;
            }
        }

        *pDst = 0; /* terminate destination string */
    }

    return (uint32_t)(pDst - dstBuf);
}

FSTATIC uint32_t STR_INT_Utf8_8859 (SYS_CHARSET srcCs, const char_t* srcBuf,
                               SYS_CHARSET dstCs, char_t* dstBuf,
                               uint32_t maxBytes, uint32_t numBytes)
{
    char_t*    pDst = dstBuf;
    SYS_CHAR currChar;

    if ( maxBytes >= 1 )
    {
        while ( ((*srcBuf) != 0)  &&
                (maxBytes  >= 2)  &&
                (numBytes  >  0) )
        {
            if ( STR_UTF8_1BYTE_IS(srcBuf) )
            {
                /* single-byte code */
                currChar = STR_UTF8_1BYTE_GET(srcBuf);

                srcBuf++;
                numBytes--;
            }
            else if ( STR_UTF8_2BYTE_IS(srcBuf) && (numBytes > 1) )
            {
                /* two-byte code */
                currChar = STR_UTF8_2BYTE_GET(srcBuf);

                srcBuf   += 2;
                numBytes -= 2;
            }
            else if ( STR_UTF8_3BYTE_IS(srcBuf) && (numBytes > 2) )
            {
                /* three-byte code */
                currChar = STR_UTF8_3BYTE_GET(srcBuf);

                srcBuf   += 3;
                numBytes -= 3;
            }
            else
            {
                FAPI_SYS_PRINT_DEBUG(4,"[SYS] UTF-8: invalid encoding!\n");
                break;
            }

            if ( STR_IsPrintableChar_16Bit( currChar ) )
            {
                if ( currChar < STR_8859_UPPERSTART )
                {
                    *pDst++ = (currChar & 0xFF);
                    maxBytes--;
                }
                else if ( (currChar <
                          (STR_8859_UPPERSTART + STR_8859_UPPERBYTES)) &&
                          (dstCs == SYS_CHARSET_ISO8859_1) )
                {
                    *pDst++ = (currChar & 0xFF);
                    maxBytes--;
                }

                /* NOTE: Unicode characters >= 0x100 for ISO 8859-1 or
                         >= 0xA0 for other ISO 8859 character sets are
                         discarded in this current implementation. */
                /* to do: map unicode characters onto ISO-8859-x upper bytes! */
            }
        }

        *pDst = 0; /* terminate destination string */
    }

    return (uint32_t)(pDst - dstBuf);
}

FSTATIC uint32_t STR_INT_Utf8_16Bit (SYS_CHARSET srcCs, const char_t* srcBuf,
                                SYS_CHARSET dstCs, char_t* dstBuf,
                                uint32_t maxBytes, uint32_t numBytes)
{
    char_t*    pDst = dstBuf;
    SYS_CHAR currChar;

    if ( maxBytes >= 2 )
    {
        while ( ((*srcBuf) != 0)  &&
                (maxBytes  >= 4)  &&
                (numBytes  >  0) )
        {
            if ( STR_UTF8_1BYTE_IS(srcBuf) )
            {
                /* single-byte code */
                currChar = STR_UTF8_1BYTE_GET(srcBuf);

                srcBuf++;
                numBytes--;
            }
            else if ( STR_UTF8_2BYTE_IS(srcBuf) && (numBytes > 1) )
            {
                /* two-byte code */
                currChar = STR_UTF8_2BYTE_GET(srcBuf);

                if( ( SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT_T2 ) ||
                    ( SYS_GetAppProfile() == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW ) )
                {
                    switch( currChar )
                    {
                        case 0x00A4: currChar = 0x20AC; break; // euro currency sign
                        default: break;
                    }
                }
                srcBuf   += 2;
                numBytes -= 2;
            }
            else if ( STR_UTF8_3BYTE_IS(srcBuf) && (numBytes > 2) )
            {
                /* three-byte code */
                currChar = STR_UTF8_3BYTE_GET(srcBuf);

                srcBuf   += 3;
                numBytes -= 3;
            }
            else
            {
                FAPI_SYS_PRINT_DEBUG(4,"[SYS] UTF-8: invalid encoding!\n");
                break;
            }

            if ( STR_IsPrintableChar_16Bit( currChar ) )
            {
                STR_UNICODE_WRCHAR(pDst, currChar);

                pDst     += 2;
                maxBytes -= 2;
            }
        }

        /* terminate destination string */
        STR_UNICODE_WRCHAR(pDst, 0);
    }

    return (uint32_t)(pDst - dstBuf);
}

FSTATIC uint32_t STR_INT_16Bit_Plain (SYS_CHARSET srcCs, const char_t* srcBuf,
                                 SYS_CHARSET dstCs, char_t* dstBuf,
                                 uint32_t maxBytes, uint32_t numBytes)
{
    char_t*       pDst = dstBuf;
    SYS_CHAR    currChar;

    if ( maxBytes >= 2 )
    {
        while ( (maxBytes >= 4) &&
                (numBytes >= 2) )
        {
            currChar = STR_UNICODE_RDCHAR(srcBuf);

            if ( currChar == 0 )
            {
                /* source string end */
                break;
            }
            else if ( STR_IsPrintableChar_16Bit( currChar ) )
            {
                /* not necessary to use STR_UNICODE_WRCHAR here. No conversion
                   takes place. */
                *pDst++ = srcBuf[0];
                *pDst++ = srcBuf[1];

                maxBytes -= 2;
            }

            srcBuf   += 2;
            numBytes -= 2;
        }

        /* terminate destination string */
        STR_UNICODE_WRCHAR(pDst, 0);
    }

    return (uint32_t)(pDst - dstBuf);
}

FSTATIC uint32_t STR_INT_16Bit_8859 (SYS_CHARSET srcCs, const char_t* srcBuf,
                                SYS_CHARSET dstCs, char_t* dstBuf,
                                uint32_t maxBytes, uint32_t numBytes)
{
    char_t*       pDst = dstBuf;
    SYS_CHAR    currChar;

    if ( maxBytes >= 1 )
    {
        while ( (maxBytes >= 2) &&
                (numBytes >= 2) )
        {
            currChar = STR_UNICODE_RDCHAR(srcBuf);

            if ( currChar == 0 )
            {
                /* source string end */
                break;
            }
            else if ( STR_IsPrintableChar_16Bit( currChar ) )
            {
                /* to do: map each character >= 0xA0 individually for each
                          ISO 8859 table. */

                *pDst++ = currChar & 0x00FF;

                maxBytes--;
            }

            srcBuf   += 2;
            numBytes -= 2;
        }

        *pDst = 0; /* terminate destination string */
    }

    return (uint32_t)(pDst - dstBuf);
}

FSTATIC uint32_t STR_INT_16Bit_Utf8 (SYS_CHARSET srcCs, const char_t* srcBuf,
                                SYS_CHARSET dstCs, char_t* dstBuf,
                                uint32_t maxBytes, uint32_t numBytes)
{
    char_t*       pDst = dstBuf;
    SYS_CHAR    currChar;

    if ( maxBytes >= 1 )
    {
        while ( numBytes >= 2 )
        {
            currChar = STR_UNICODE_RDCHAR(srcBuf);

            if ( currChar == 0 )
            {
                /* source string end */
                break;
            }
            else if ( currChar <= 0x007F ) /* single-byte code */
            {
                if ( maxBytes < 2 ) break; /* includes termination */

                *dstBuf++ = currChar;

                maxBytes--;
            }
            else if ( currChar <= 0x07FF ) /* two-byte code */
            {
                if ( maxBytes < 3 ) break; /* includes termination */

                *dstBuf++ = ((currChar & 0x07C0) >> 6) | 0xC0;
                *dstBuf++ =  (currChar & 0x003F)       | 0x80;

                maxBytes -= 2;
            }
            else /* three-byte code */
            {
                if ( maxBytes < 4 ) break; /* includes termination */

                *dstBuf++ = ((currChar & 0xF000) >> 12) | 0xE0;
                *dstBuf++ = ((currChar & 0x0FC0) >>  6) | 0x80;
                *dstBuf++ =  (currChar & 0x003F)        | 0x80;

                maxBytes -= 3;
            }

            srcBuf   += 2;
            numBytes -= 2;
        }

        *pDst = 0; /* terminate destination string */
    }

    return (uint32_t)(pDst - dstBuf);
}

FSTATIC uint32_t STR_INT_16Bit_Tab0 (SYS_CHARSET srcCs, const char_t* srcBuf,
                                SYS_CHARSET dstCs, char_t* dstBuf,
                                uint32_t maxBytes, uint32_t numBytes)
{
    char_t*       pDst = dstBuf;
    SYS_CHAR    currChar;

    if ( maxBytes >= 1 )
    {
        while ( (maxBytes >= 2) &&
                (numBytes >= 2) )
        {
            currChar = STR_UNICODE_RDCHAR(srcBuf);

            if ( currChar == 0 )
            {
                /* source string end */
                break;
            }
            else if ( STR_IsPrintableChar_16Bit( currChar ) )
            {
                /* to do: map characters for table 0 (0xA0-0xFF) */
                /* to do: handle 2-byte characters of table 0 */
                /* reverse matrix needed: unicode -> 6937 */

                *pDst++ = currChar & 0x00FF;

                maxBytes--;
            }

            srcBuf   += 2;
            numBytes -= 2;
        }

        *pDst = 0; /* terminate destination string */
    }

    return (uint32_t)(pDst - dstBuf);
}

FSTATIC uint32_t  STR_INT_Tab0_16Bit (SYS_CHARSET srcCs, const char_t* srcBuf,
                                 SYS_CHARSET dstCs, char_t* dstBuf,
                                 uint32_t maxBytes, uint32_t numBytes)
{
    char_t*       pDst = dstBuf;
    SYS_CHAR    currChar;
    uint32_t         bytesPerChar;

    if ( maxBytes >= 2 )
    {
        while ( ((*srcBuf) != 0)    &&
                (maxBytes  >= 4)    &&
                (numBytes  >  0) )
        {
            if ( STR_IsPrintableChar_8Bit(*srcBuf) )
            {
                bytesPerChar = STR_CharGetTab0 (srcBuf, &currChar);

                if ( (bytesPerChar == 0) ||
                     (bytesPerChar > numBytes) )
                {
                    break;
                }

                if ( currChar != STR_UD )
                {
                    STR_UNICODE_WRCHAR(pDst, currChar);

                    pDst     += 2;
                    maxBytes -= 2;
                }
            }
            else
            {
                bytesPerChar = 1;
            }

            srcBuf   += bytesPerChar;
            numBytes -= bytesPerChar;
        }

        /* terminate destination string */
        STR_UNICODE_WRCHAR(pDst, 0);
    }

    return (uint32_t)(pDst - dstBuf);
}

/*lint -restore */

/* ------------------------------------------------------------------------- */

/*!
*******************************************************************************

\c UTILS_DBVSIText2String() is a utility function to handle decoding of DVBSI
text characters. It clean up any text array within DVB descriptors. In case of
single byte characters you will get a string which is able to be printed on
OSD or terminal.

The following operations are done (valid for single and double byte characters):
- First it removes control codes for the selection of the character table.
- It also removes control codes for emphasis on/off.
- A carriage return / line feed is converted to '\n' (=0x10 or =0x0010).
- when \c noEmphasis is set to \c FFALSE it will only decode the short form
  of the text(emphased text). If no short form is available the returned length
  is zero. Otherwise the the full text is returned.
- The returned string is terminated with zero.
- The returned length is equal to C-lib function \c strlen().

\note  Usually only service or provider names make use of the emphaised feature.
       If you can't fit the full string try the function with \c noEmphasis
       set to \c FFALSE. If a emphaised version is allowed the returned string
       length is non zero. This should be used since broadcasters don't like
       if the service name is just cut off.

\param pSrc       Pointer to the start of the text character descriptor field.
\param srcLen     Length of the descriptor. In case the destination buffer is
                  limited to a maximum size then set the maximum size for
                  the destination string in case the descriptor length is longer.
\param pDst       Pointer to the destination buffer to be filled.
\param dstLen     Size of destination buffer in bytes.
\param pCharSet   Character set is returned here. Enter NULL if not needed.
\param noEmphasis If set to \c FTRUE then the full text is returned.
                  If set to \c FFALSE then only the emphaised text is returned.
\param crlfChar   0: Ignore CR/LF codes, any other: replace CR/LF by this
                  character.

\return       Length of decoded string in bytes excluding termination.

******************************************************************************/
uint16_t UTILS_DBVSIText2String (const char_t* pSrc, uint16_t srcLen,
                                 char_t *pDst, uint16_t dstLen, SYS_CHARSET* pCharSet,
                                 fbool_t noEmphasis, char_t crlfChar)
{
    uint16_t        dstPos     = 0;
    SYS_CHARSET     charSet    = SYS_CHARSET_UNDEFINED;
    uint16_t        subId;
    #ifdef APPL_STRING_COMPRESSION_ENABLED
    fbool_t         compressedFlag = FFALSE;
    uint8_t         encoding_type_id;
    uint8_t*        pdecompressStr = NULL;
    uint16_t        decompressLen = 512;
    #endif

    /* ----- Check character table indication (defined in ETSI EN 300 468) - */

    if ( (srcLen >= 1) && (dstLen >= 1) )
    {
        if (pSrc[0] >= 0x20) /* table 00 - latin alphabet (default) */
        {
            charSet = strDat->defaultSiCharSet;

            /* NOTE: first byte is valid character -> do not skip! */
        }
        else if ( (pSrc[0] >= 0x01) &&
                  (pSrc[0] <= 0x0F) ) /* alternative latin alphabets */
        {
            switch ( pSrc[0] )
            {
                case 0x01: charSet = SYS_CHARSET_ISO8859_5;     break;
                case 0x02: charSet = SYS_CHARSET_ISO8859_6;     break;
                case 0x03: charSet = SYS_CHARSET_ISO8859_7;     break;
                case 0x04: charSet = SYS_CHARSET_ISO8859_8;     break;
                case 0x05: charSet = SYS_CHARSET_ISO8859_9;     break;
                case 0x06: charSet = SYS_CHARSET_ISO8859_10;    break;
                case 0x07: charSet = SYS_CHARSET_ISO8859_11;    break;
                case 0x09: charSet = SYS_CHARSET_ISO8859_13;    break;
                case 0x0A: charSet = SYS_CHARSET_ISO8859_14;    break;
                case 0x0B: charSet = SYS_CHARSET_ISO8859_15;    break;

                default:
                    /* do nothing */
                    break;
            }

            pSrc++;
            srcLen--;
        }
        else if ( pSrc[0] == 0x10 ) /* ISO 8859 tables */
        {
            if ( srcLen >= 3 )
            {
                subId = (((uint16_t)(pSrc[1])) << 8) | pSrc[2];

                switch ( subId )
                {
                    case 0x0001: charSet = SYS_CHARSET_ISO8859_1;     break;
                    case 0x0002: charSet = SYS_CHARSET_ISO8859_2;     break;
                    case 0x0003: charSet = SYS_CHARSET_ISO8859_3;     break;
                    case 0x0004: charSet = SYS_CHARSET_ISO8859_4;     break;
                    case 0x0005: charSet = SYS_CHARSET_ISO8859_5;     break;
                    case 0x0006: charSet = SYS_CHARSET_ISO8859_6;     break;
                    case 0x0007: charSet = SYS_CHARSET_ISO8859_7;     break;
                    case 0x0008: charSet = SYS_CHARSET_ISO8859_8;     break;
                    case 0x0009: charSet = SYS_CHARSET_ISO8859_9;     break;
                    case 0x000A: charSet = SYS_CHARSET_ISO8859_10;    break;
                    case 0x000B: charSet = SYS_CHARSET_ISO8859_11;    break;
                    case 0x000D: charSet = SYS_CHARSET_ISO8859_13;    break;
                    case 0x000E: charSet = SYS_CHARSET_ISO8859_14;    break;
                    case 0x000F: charSet = SYS_CHARSET_ISO8859_15;    break;

                    default:
                        /* do nothing */
                        break;
                }

                pSrc   += 3;
                srcLen -= 3;
            }
        }
        else if ( pSrc[0] == 0x11 ) /* Basic Multilingual Plane of ISO/IEC 10646-1 */
        {
            charSet = SYS_CHARSET_ISO_10646;

            #if 0
            FAPI_SYS_PRINT_DEBUG(4,"(double byte char table)");
            #endif

            pSrc++;
            srcLen--;
        }
        else if ( pSrc[0] == 0x15 ) /* UTF-8 encoding of ISO/IEC 10646-1 */
        {
            charSet = SYS_CHARSET_UTF8;

            pSrc++;
            srcLen--;
        }
        #ifdef APPL_STRING_COMPRESSION_ENABLED
        else if ( pSrc[0] == 0x1F )
        {
            //it is a compressed string
            encoding_type_id = pSrc[1];
            if((encoding_type_id == 0x01) || (encoding_type_id == 0x02))
            {
                pdecompressStr = UTILS_Malloc(decompressLen);
                if ( pdecompressStr == NULL )
                {
                    return 0;
                }
                pSrc += 2;
                srcLen -= 2;

                STRING_HUFMANN_Decompress(encoding_type_id, (uint8_t *)pSrc,  pdecompressStr, &decompressLen);

                charSet = SYS_CHARSET_UTF8;
                compressedFlag = FTRUE;
            }
        }
        #endif/*#ifdef APPL_STRING_COMPRESSION_ENABLED*/

        /* NOTE: Other character sets are unsupported. */
    }

    if ( charSet == SYS_CHARSET_UNDEFINED )
    {
        if ( pCharSet != NULL )
        {
            *pCharSet = charSet;
        }
        #ifdef APPL_STRING_COMPRESSION_ENABLED
        if ( pdecompressStr != NULL )
        {
            UTILS_Free(pdecompressStr);
        }
        #endif
        return 0;
    }

    /* --------------------------------------------------------------------- */

    switch ( charSet )
    {
        case SYS_CHARSET_TABLE0:
        case SYS_CHARSET_ISO8859_1:
        case SYS_CHARSET_ISO8859_2:
        case SYS_CHARSET_ISO8859_3:
        case SYS_CHARSET_ISO8859_4:
        case SYS_CHARSET_ISO8859_5:
        case SYS_CHARSET_ISO8859_6:
        case SYS_CHARSET_ISO8859_7:
        case SYS_CHARSET_ISO8859_8:
        case SYS_CHARSET_ISO8859_9:
        case SYS_CHARSET_ISO8859_10:
        case SYS_CHARSET_ISO8859_11:
        case SYS_CHARSET_ISO8859_13:
        case SYS_CHARSET_ISO8859_14:
        case SYS_CHARSET_ISO8859_15:
            dstPos += DVBSIText2String_8Bit (pSrc, srcLen, pDst, dstLen,
                                             charSet, noEmphasis, crlfChar);
            break;

        case SYS_CHARSET_ISO_10646:
            dstPos += DVBSIText2String_16Bit (pSrc, srcLen, pDst, dstLen,
                                              charSet, noEmphasis, crlfChar);
            break;

        case SYS_CHARSET_UTF8:
            #ifdef APPL_STRING_COMPRESSION_ENABLED
            if(compressedFlag)
            {
                dstPos += DVBSIText2String_Utf8 ((const char_t*)pdecompressStr, decompressLen, pDst, dstLen,
                                                 charSet, noEmphasis, crlfChar);
            }
            else
            #endif/*#ifdef APPL_STRING_COMPRESSION_ENABLED*/
            {
                dstPos += DVBSIText2String_Utf8 (pSrc, srcLen, pDst, dstLen,
                                                 charSet, noEmphasis, crlfChar);
            }
            break;

        default:
            break;
    }

    /* Add termination: DVBSIText2String_XXX functions take care to leave
       enough space for termination in target buffer! */
    if ( dstPos < dstLen )
    {
        STR_Terminate (pDst + dstPos, dstLen - dstPos);
    }

    if ( pCharSet != NULL )
    {
        *pCharSet = charSet;
    }

    FAPI_SYS_PRINT_DEBUG(4,"UTILS_DBVSIText2String: CHARSET: %i '%s'\n", charSet, pDst);
    #ifdef APPL_STRING_COMPRESSION_ENABLED
    if ( pdecompressStr != NULL )
    {
        UTILS_Free(pdecompressStr);
    }
    #endif
    return dstPos;
}

FSTATIC uint16_t  DVBSIText2String_8Bit (const char_t* pSrc, uint16_t srcLen,
                                    char_t* pDst, uint16_t dstLen,
                                    SYS_CHARSET charSet,
                                    fbool_t noEmphasis, char_t crlfChar)
{
    uint16_t     cnt;
    uint16_t     dstPos  = 0;
    fbool_t   blocked;
    uint8_t      shyChar;

    switch ( charSet )
    {
        case SYS_CHARSET_ISO8859_11:
            /* soft hyphen character is not defined */
            shyChar = 0;
            break;

        case SYS_CHARSET_TABLE0:
            shyChar = 0xFF;
            break;

        case SYS_CHARSET_ISO8859_1:
        case SYS_CHARSET_ISO8859_2:
        case SYS_CHARSET_ISO8859_3:
        case SYS_CHARSET_ISO8859_4:
        case SYS_CHARSET_ISO8859_5:
        case SYS_CHARSET_ISO8859_6:
        case SYS_CHARSET_ISO8859_7:
        case SYS_CHARSET_ISO8859_8:
        case SYS_CHARSET_ISO8859_9:
        case SYS_CHARSET_ISO8859_10:
        case SYS_CHARSET_ISO8859_13:
        case SYS_CHARSET_ISO8859_14:
        case SYS_CHARSET_ISO8859_15:
            shyChar = 0xAD;
            break;

        default:
            /* unsupported character set */
            return 0;
    }

    if ( !noEmphasis )
    {
        /* Only emphasized text requested, block until emphasis is
           activated! */
        blocked = FTRUE;
    }
    else
    {
        blocked = FFALSE;
    }

    for ( cnt = 0; cnt < srcLen; cnt++, pSrc++ )
    {
        /* enough space for one character + termination? */
        if ( (uint16_t)(dstPos + 2) > dstLen ) break;

        /* check for control codes */
        if ( ((*pSrc) >= 0x80) && ((*pSrc) <= 0x9F) )
        {
            switch ( *pSrc )
            {
                case TEXTCTRL_EMPHASIS_ON:
                    if ( !noEmphasis ) blocked = FFALSE;
                    break;

                case TEXTCTRL_EMPHASIS_OFF:
                    if ( !noEmphasis ) blocked = FTRUE;
                    break;

                case TEXTCTRL_CODE_CRLF:
                    if ( crlfChar != 0 )
                    {
                        /* Replace ETSI CR/LF by custom CR/LF character */
                        pDst[dstPos++] = crlfChar;
                    }
                    break;

                default:
                    /* do nothing */
                    break;
            }
        }
        else
        {
            if ( (*pSrc) == '\n' )
            {
                if ( ( SYS_GetAppProfile() != SYS_APP_PROFILE_UKDTT_T2 ) &&
                     ( SYS_GetAppProfile() != SYS_APP_PROFILE_AUSTRALIA_FREEVIEW) )
                {
                    if ( crlfChar != 0 )
                    {
                        /* Replace ASCII CR/LF by custom CR/LF character */
                        pDst[dstPos++] = crlfChar;
                    }
                }
            }
            else if  ( (shyChar != 0) && ((*pSrc) == shyChar) )
            {
                /* In the current implementation, ignore soft hyphen! */
            }
            /* ignore non-printable characters */
            else if ( STR_IsPrintableChar_8Bit(*pSrc) )
            {
                if ( !blocked )
                {
                    pDst[dstPos++] = *pSrc;
                }
            }
        }
    }

    return dstPos;
}

FSTATIC uint16_t  DVBSIText2String_16Bit (const char_t* pSrc, uint16_t srcLen,
                                     char_t* pDst, uint16_t dstLen,
                                     SYS_CHARSET charSet,
                                     fbool_t noEmphasis, char_t crlfChar)
{
    uint16_t     cnt;
    uint16_t     dstPos  = 0;
    fbool_t   blocked;
    uint16_t     shyChar;
    uint16_t     currChar;

    switch ( charSet )
    {
        case SYS_CHARSET_ISO_10646:
            shyChar = 0x00AD;
            break;

        default:
            /* unsupported character set */
            return 0;
    }

    if ( !noEmphasis )
    {
        /* Only emphasized text requested, block until emphasis is
           activated! */
        blocked = FTRUE;
    }
    else
    {
        blocked = FFALSE;
    }

    for ( cnt = 0; cnt < srcLen; cnt += 2, pSrc += 2 )
    {
        /* enough space for one character + termination (2 * 2 bytes) ? */
        if ( (uint16_t)(dstPos + 4) > dstLen ) break;

        /* check for control codes */
        if ( (pSrc[0] == 0xE0) &&
             (pSrc[1] >= 0x80) && (pSrc[1] <= 0x9F) )
        {
            switch ( pSrc[1] )
            {
                case TEXTCTRL_EMPHASIS_ON:
                    if ( !noEmphasis ) blocked = FFALSE;
                    break;

                case TEXTCTRL_EMPHASIS_OFF:
                    if ( !noEmphasis ) blocked = FTRUE;
                    break;

                case TEXTCTRL_CODE_CRLF:
                    if ( crlfChar != 0 )
                    {
                        /* Replace ETSI CR/LF by custom CR/LF character */
                        STR_UNICODE_WRCHAR(pDst + dstPos, crlfChar);

                        dstPos += 2;
                    }
                    break;
                default :
                    /* do nothing */
                    break;
            }
        }
        else
        {
            if ( (pSrc[0] == 0x00) && (pSrc[1] == '\n') )
            {
                if ( crlfChar != 0 )
                {
                    /* Replace ASCII CR/LF by custom CR/LF character */
                    STR_UNICODE_WRCHAR(pDst + dstPos, crlfChar);

                    dstPos += 2;
                }
            }
            else if  ( (shyChar != 0)               &&
                       (pSrc[0] == (shyChar >> 8))  &&
                       (pSrc[1] == (shyChar & 0x00FF)) )
            {
                /* In the current implementation, ignore soft hyphen! */
            }
            else
            {
                if ( !blocked )
                {
                    currChar = (((uint16_t)pSrc[0]) << 8) | pSrc[1];

                    if ( STR_IsPrintableChar_16Bit( currChar ) )
                    {
                        STR_UNICODE_WRCHAR(pDst + dstPos, currChar);
                        dstPos += 2;
                    }
                }
            }
        }
    }

    return dstPos;
}

FSTATIC uint16_t  DVBSIText2String_Utf8 (const char_t* pSrc, uint16_t srcLen,
                                    char_t* pDst, uint16_t dstLen,
                                    SYS_CHARSET charSet,
                                    fbool_t noEmphasis, char_t crlfChar)
{
    char_t*   pDstStart  = pDst;
    fbool_t   blocked;
    //uint8_t      shyChar;      // to do...

    if( (pSrc == NULL) ||
        (pDst == NULL) )
    {
        return 0;
    }
    if ( !noEmphasis )
    {
        /* Only emphasized text requested, block until emphasis is
           activated! */
        blocked = FTRUE;
    }
    else
    {
        blocked = FFALSE;
    }

    while ( (srcLen > 0) && (dstLen > 0) )
    {
        if ( STR_UTF8_1BYTE_IS(pSrc) )             /* single-byte code */
        {
            if ( dstLen < 2 ) break; /* includes termination */

            if ( pSrc[0] == '\n' )
            {
                if ( (crlfChar != 0) &&
                     (dstLen   >= 2) )
                {
                    /* Replace ASCII CR/LF by custom CR/LF character */
                    *pDst++ = crlfChar;
                    dstLen--;
                }
            }
            else if ( STR_IsPrintableChar_8Bit( pSrc[0]) )
            {
                if ( !blocked )
                {
                    *pDst++ = *pSrc;
                    dstLen--;
                }
            }

            pSrc++;
            srcLen--;
        }
        else if ( STR_UTF8_2BYTE_IS(pSrc) && (srcLen > 1) ) /* two-byte code */
        {
            if ( dstLen < 3 ) break; /* includes termination */

            if ( !blocked )
            {
                *pDst++ = pSrc[0];
                *pDst++ = pSrc[1];

                dstLen -= 2;
            }

            pSrc   += 2;
            srcLen -= 2;
        }
        else if ( STR_UTF8_3BYTE_IS(pSrc) && (srcLen > 2) ) /* three-byte code*/
        {
            /* check for control codes */
            if ( (pSrc[0] == 0xEE) &&
                 (pSrc[1] == 0x82) &&
                 (pSrc[2] >= 0x80) &&
                 (pSrc[2] <= 0x9F) )
            {
                switch ( pSrc[2] )
                {
                    case TEXTCTRL_EMPHASIS_ON:
                        if ( !noEmphasis ) blocked = FFALSE;
                        break;

                    case TEXTCTRL_EMPHASIS_OFF:
                        if ( !noEmphasis ) blocked = FTRUE;
                        break;

                    case TEXTCTRL_CODE_CRLF:
                        if ( (crlfChar != 0) &&
                             (dstLen   >= 2) )
                        {
                            /* Replace ETSI CR/LF by custom CR/LF character */
                            *pDst++ = crlfChar;
                            dstLen--;
                        }
                        break;

                    default:
                        /* do nothing */
                        break;
                }
            }
            else
            {
                if ( dstLen < 4 ) break; /* includes termination */

                if ( !blocked )
                {
                    *pDst++ = pSrc[0];
                    *pDst++ = pSrc[1];
                    *pDst++ = pSrc[2];

                    dstLen -= 3;
                }
            }

            pSrc   += 3;
            srcLen -= 3;
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(4,"[SYS] UTF-8: invalid encoding!\n");
            break;
        }
    }

    return (pDst - pDstStart);
}


/*!
*******************************************************************************
**
** \brief   This function checks if the currChar (8-Bit) is a printable character
**
** \param   currChar    Character to by checked.
**
** \return  #FTRUE if the Charater is printable otherwise #FFALSE
**
******************************************************************************/
FSTATIC fbool_t STR_IsPrintableChar_8Bit(char_t currChar)
{
    if ( currChar >= STR_PRINTABLE_START )
    {
        return FTRUE;
    }
    if ( ( SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT_T2 ) ||
         ( SYS_GetAppProfile() == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW) )
    {
        if( currChar == 0x0A )
        {
            return FTRUE;
        }
    }
    return FFALSE;
}

/*!
*******************************************************************************
**
** \brief   This function checks if the currChar (16-Bit) is a printable character
**
** \param   currChar    Character to by checked.
**
** \return  #FTRUE if the Charater is printable otherwise #FFALSE
**
******************************************************************************/
FSTATIC fbool_t STR_IsPrintableChar_16Bit(SYS_CHAR currChar)
{
    if ( currChar >= STR_PRINTABLE_START )
    {
        return FTRUE;
    }

    if ( ( SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT_T2 ) ||
         ( SYS_GetAppProfile() == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW) )
    {
        if( currChar == 0x0A )
        {
            return FTRUE;
        }
    }

    return FFALSE;
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
