/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/time.c
**
**              $Id: time.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       System time and timer management.
**
** See here for more detail documentation: \ref app_smartgo_time
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int
** - Info    713: Loss of precision (assignment) (unsigned int to short).
** - Info    715: Symbol not referenced.
** - Info    737: Loss of sign in promotion from int to unsigned int.
** - Note    961: Violates MISRA 2004 Advisory Rule 12.1, dependence placed on
**                C's operator precedence.
**
*****************************************************************************
*/
/*lint -save -e641 -e713 -e715 -e737 -e961 */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "fapex/abst_types.h"

#include "sys_api.h"
#include "av.h"
#include "event_queue.h"

#include "settings.h"
#include "key_input.h"
#include "sys_time.h"
#include "sys_priv.h"
#include "utils.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define TIME_MJD_TO_JD          2400001L    //!< for MJD to JD conversion */

#define TIME_TICK_INTERVAL      2       //!< time update interval in units of 100 ms
#define TIME_SYNCTOLERANCE      2       //!< In seconds.

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    /* Resources */
    //!< FAPI Timer Handle
#ifndef _LINUX_FAPI
    FAPI_SYS_HandleT    sysTimeHandle;
    //!< Array of SmartGo-HD internal single timer handles
    FAPI_SYS_HandleT    timerHandle[TIME_SINGLETIMERS_MAX];
#else
    FAPEX_SYS_ThreadT   sysTimeHandle;
    FAPEX_SYS_ThreadT   timerHandle[TIME_SINGLETIMERS_MAX];
#endif
    //!< Array of SmartGo-HD internal periodic timer handles
    FAPI_SYS_HandleT    periodTimerHandle[TIME_PERIODTIMERS_MAX];

    /* Parameters */
    fbool_t             forceEvent;     //!<
    fbool_t             syncedOnce;     //!<
    fbool_t             autoTimeMode;   //!< FTRUE -> get time from stream

    uint16_t            milliSeconds;   //!<
    SYS_TIME_S          timeCurr;       //!< current time (UTC)
    SYS_DATE_S          dateCurr;       //!< current date (UTC)
    uint32_t            systemTime;     //!< The SmartGo-HD system time.

    /* timezone change parameters */
    fbool_t             tzChgReq;       //!< FTRUE: Timezone change announced.
    SYS_TIME_S          tzChgTime;      //!< Timezone change time.
    SYS_DATE_S          tzChgDate;      //!< Timezone change date.
    int16_t             tzChgOffset;    //!< Offset after timezone change.

} TIME_DATA_S;

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
FSTATIC TIME_DATA_S *timeDat = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC void         Tick_ThreadFunction(void* opt);
FSTATIC void         Cb1_ThreadFunction(void* opt);
FSTATIC void         Cb2_ThreadFunction(void* opt);

FSTATIC void         TIME_TickCallback(uint64_t val, uint32_t optArg);
FSTATIC FINLINE void TIME_TimerCb(uint64_t val, uint32_t optData);
FSTATIC uint8_t      getDaysPerMonth(uint8_t month, uint8_t year);

FSTATIC void         MjdToYmd(int32_t mjd, uint16_t * yearP, uint16_t * monthP,
                              uint16_t * dayP);
FSTATIC uint16_t     YmdToMjd(uint16_t year, uint16_t month, uint16_t day);
/*---------------------------------------------------------------------------*/
/* public functions                                                          */
/*---------------------------------------------------------------------------*/

/*!
*******************************************************************************
**
** \brief  Initalize the SmartGo-HD time module.
**
** System time Timer is started and the time parameter set to default or if
** available to the user defined settings
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_OUT_OF_MEMORY
**          - #APPL_SMARTGO_ERR_BAD_PARAMETER
**
** \sa
**          - FAPI_TIMER_Open()
**          - FAPI_TIMER_Start()
**          - #FAPI_TIMER_OpenParamsT
**
******************************************************************************/
int32_t TIME_Init(void)
{
    int32_t                 retVal;
    fbool_t                 enabled;
#ifndef _LINUX_FAPI
    FAPI_TIMER_OpenParamsT  timerParams;
#endif
    DBG_Assert(timeDat == NULL);

    timeDat = UTILS_Malloc(sizeof(TIME_DATA_S));
    if(timeDat == NULL)
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    memset(timeDat, 0, sizeof(TIME_DATA_S));

    /* Set default date: Saturday 2000-01-01 00:00:00 */

    timeDat->timeCurr.hour   = 0;
    timeDat->timeCurr.minute = 0;
    timeDat->timeCurr.second = 0;

    timeDat->dateCurr.year      = 2000;
    timeDat->dateCurr.month     = 1;
    timeDat->dateCurr.day       = 1;
    timeDat->dateCurr.dayOfWeek = SYS_TIME_WEEKDAY_SATURDAY;

    timeDat->milliSeconds       = 0;
    timeDat->forceEvent         = FTRUE;
    timeDat->syncedOnce         = FFALSE;

    (void)SETTINGS_AutoTimeGet(&enabled);
    timeDat->autoTimeMode       = enabled;
    timeDat->sysTimeHandle      = NULL;

    timeDat->tzChgReq           = FFALSE;

    /* Setup software timer used for time update */
#ifndef _LINUX_FAPI
    memset(&timerParams, 0, sizeof(timerParams));

    timerParams.version    = FAPI_TIMER_VERSION;
    timerParams.type       = FAPI_TIMER_TYPE_SOFT;
    timerParams.mode       = FAPI_TIMER_MODE_CONTINUOUS;
    timerParams.resolution = FAPI_TIMER_RESOLUTION_100_MSEC;
    timerParams.multiplier = TIME_TICK_INTERVAL;
    timerParams.callback1  = TIME_TickCallback;
    timerParams.exec1InIsr = 0;

    timeDat->sysTimeHandle = FAPI_TIMER_Open(&timerParams, &retVal);
    if(!timeDat->sysTimeHandle)
    {
        /* TODO error handling */
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    (void)FAPI_TIMER_Start(timeDat->sysTimeHandle);

    return FAPI_OK;
#else
    retVal = SYS_ThreadCreate (Tick_ThreadFunction, NULL,
                               THREAD_PRIO_TIMER_CB,
                               THREAD_STACKSZ_TIMER,
                               "Tick",
                               &timeDat->sysTimeHandle);
    if(!timeDat->sysTimeHandle)
    {
        /** /todo error handling */
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    return retVal;
#endif
}

/*!
*******************************************************************************
**
** \brief   Returns current system time in units of 100 ms (deci-seconds).
**
** System time starts from 0 at power-up and counts endlessly. Overflow takes
** place after 2^32 - 1.
**
** \return  Stamp of current system time in deci-seconds or
**          APPL_SMARTGO_ERR_NOT_INITIALIZED.
**
** \sa
**          - TIME_SetSystemTime()
**
******************************************************************************/
int32_t TIME_GetSystemTime(uint32_t * time)
{
    if(timeDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    *time = timeDat->systemTime;
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   Set the system time.
**
** \attention This function use FAPEX_SYS_EnterCritical()!
**
** \param   time    New value for system time.
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
** \sa
**          - TIME_GetSystemTime()
**          - FAPEX_SYS_EnterCritical()
**          - FAPEX_SYS_LeaveCritical()
**
******************************************************************************/
int32_t TIME_SetSystemTime(uint32_t time)
{
    uint32_t            status;

    if(timeDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    FAPEX_SYS_EnterCritical(status);

    timeDat->systemTime = time;

    FAPEX_SYS_LeaveCritical(status);
    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   Calculates the time interval between two timestamps acquired by
**          TIME_GetSystemTime().
**
** This function takes an overflow of the counter value into account. However,
** it is not able to detect multiple overflows.
**
** \param   timeStart   Start time in deci-seconds
** \param   timeEnd     End time in deci-seconds
**
** \return  Time interval in deci-seconds.
**
** \sa
**          - TIME_SetSystemTime()
**          - TIME_GetSystemTime()
**
******************************************************************************/
uint32_t TIME_GetSystemTimeInterval(uint32_t timeStart, uint32_t timeEnd)
{
    if(timeStart <= timeEnd)
    {
        return (timeEnd - timeStart);
    }
    else
    {
        return (0xFFFFFFFF - timeStart + timeEnd + 1);
    }
}

/*!
*******************************************************************************
**
** \brief   Returns current time and date in local time.
**
** Local time means: UTC time + time offset + Daylight Saving Time offset
** if active (specified by SETTINGS_TimeOffsetSet).
**
** \param   pTime       pointer to structure where current time is copied into
** \param   pDate       pointer to structure where current data is copied into
**                      (optional)
** \param   hasSynced   sync status is copied here: FTRUE: if synced at least
**                      once with DVB time (optional)
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
** \sa
**          - TIME_TimeAndDateGetUtc()
**          - TIME_AddUtcOffset()
**
******************************************************************************/
int32_t TIME_TimeAndDateGet(SYS_TIME_S * pTime, SYS_DATE_S * pDate,
                            fbool_t * hasSynced)
{
    if(timeDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    (*pTime) = timeDat->timeCurr;

    if(hasSynced != NULL)
    {
        (*hasSynced) = timeDat->syncedOnce;
    }

    if(pDate != NULL)
    {
        (*pDate) = timeDat->dateCurr;
    }

    /* add uto offset to UTC time and date */
    (void)TIME_AddUtcOffset(pTime, pDate);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   Get current UTC time.
**
** \param   pTime      current time
** \param   pDate      current date
** \param   hasSynced  if 1, system has at least once synced on time
**                     information sent via DVB (TDT). If not, returned time
**                     information is based on a default time that is set at
**                     system startup.
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
** \sa
**          - TIME_TimeAndDateGet()
**
******************************************************************************/
int32_t TIME_TimeAndDateGetUtc(SYS_TIME_S * pTime, SYS_DATE_S * pDate,
                               fbool_t * hasSynced)
{
    if(timeDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    (*pTime) = timeDat->timeCurr;

    if(hasSynced != NULL)
    {
        (*hasSynced) = timeDat->syncedOnce;
    }

    if(pDate != NULL)
        (*pDate) = timeDat->dateCurr;

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   Sets current time and date in local time.
**
** Local time means: UTC time + time offset + Daylight Saving Time offset
** if active (specified by SETTINGS_TimeOffsetSet).
**
** \attention This function use FAPEX_SYS_EnterCritical()!
**
** \param   time    Pointer to structure with current time. (optional)
** \param   date    Pointer to structure with current data. (optional)
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
** \sa
**          - TIME_TimeAndDateGetUtc()
**          - TIME_AddUtcOffset()
**          - FAPEX_SYS_EnterCritical()
**          - FAPEX_SYS_LeaveCritical()
**
******************************************************************************/
int32_t TIME_TimeAndDateSet(const SYS_TIME_S * time, const SYS_DATE_S * date)
{
    uint32_t            status;

    if(timeDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if(timeDat->autoTimeMode == FTRUE)
    {
        return FAPI_OK;
    }

    FAPEX_SYS_EnterCritical(status);
    if(time != NULL)
    {
        timeDat->timeCurr = *time;
    }

    if(date != NULL)
    {
        timeDat->dateCurr = *date;
    }

    if(time != NULL)
    {
        timeDat->forceEvent = FTRUE;
        timeDat->syncedOnce = FTRUE;
    }
    FAPEX_SYS_LeaveCritical(status);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   Converts MJD to YMD representation.
**
** This function converts a given MJD-date (Modified Julian Date) into
** its YMD (Year-Month-Day) representation.
**
** \param   mjd     The MJD to be converted.
** \param   yearP   The address where the year should be stored.
** \param   monthP  The address where the month of the year should be stored.
** \param   dayP    The address where the day of the month should be stored.
**
** \sa
**          - YmdToMjd()
**
*******************************************************************************
*/
FSTATIC void MjdToYmd(int32_t mjd, uint16_t * yearP, uint16_t * monthP,
                     uint16_t * dayP)
{
    int32_t  x, z, m, d, y;
    int32_t  daysPer400Years = 146097L;
    int32_t  fudgedDaysPer4000Years = 1460970L + 31;

    x = mjd + TIME_MJD_TO_JD + 68569L;
    z = 4 * x / daysPer400Years;
    x = x - (daysPer400Years * z + 3) / 4;
    y = 4000 * (x + 1) / fudgedDaysPer4000Years;
    x = x - 1461 * y / 4 + 31;
    m = 80 * x / 2447;
    d = x - 2447 * m / 80;
    x = m / 11;
    m = m + 2 - 12 * x;
    y = 100 * (z - 49) + y + x;

    if(y <= 0)
        y--;

    *yearP = (uint16_t) y;
    *monthP = (uint16_t) m;
    *dayP = (uint16_t) d;

    //if(*yearP <= 0)  /* adjust BC years */
    //    (*yearP)--;
}

/*!
*******************************************************************************
**
** \brief   Converts YMD to MJD representation.
**
** This function converts a given YMD (Year-Month-Day) into its
** MJD (Modified Julian Date) representation.
**
** \attention The given YMD must be between March, 1st 1900 and
**            February, 28th 2100.
**
** \param   year    The year in the range of 1900 to 2100.
** \param   month   The month of the year in the range of 1 (January) to
**                  12 (December).
** \param   day     The day of the month in the range of 1 to 31.
**
** \return  The resultant MJD.
**
** \sa
**          - MjdToYmd()
**
*******************************************************************************
*/
FSTATIC uint16_t YmdToMjd(uint16_t year, uint16_t month, uint16_t day)
{
    uint16_t            l = month > 2 ? 0 : 1;
    uint16_t            k1, k2;

    if(year > 1900)
    {
        year -= 1900;
    }
    /*lint -save -e524 */
    k1 = (uint16_t) (year - l) * 365.25;
    k2 = (uint16_t) (month + 1 + l * 12) * 30.6001;
    /*lint -restore */
    return (14956 + day + k1 + k2);
}

/*!
*******************************************************************************
**
** \brief   Converts SYS_DATE_S to MJD representation and set the dayOfWeek.
**
** \attention The given YMD must be between March, 1st 1900 and
**            February, 28th 2100.
**
** \param   pDate   The date in the range of 1900-03-01 to 2100-02-28.
**
** \return  The current day of the week.
**
*********************************************************************
*/
uint16_t TIME_GetWeekdayByDate(SYS_DATE_S * pDate)
{
    uint16_t            day;

    day = YmdToMjd(pDate->year, pDate->month, pDate->day);

    switch (day % 7)
    {
        case 4:
            pDate->dayOfWeek = SYS_TIME_WEEKDAY_SUNDAY;
            break;
        case 5:
            pDate->dayOfWeek = SYS_TIME_WEEKDAY_MONDAY;
            break;
        case 6:
            pDate->dayOfWeek = SYS_TIME_WEEKDAY_TUESDAY;
            break;
        case 0:
            pDate->dayOfWeek = SYS_TIME_WEEKDAY_WEDNESDAY;
            break;
        case 1:
            pDate->dayOfWeek = SYS_TIME_WEEKDAY_THURSDAY;
            break;
        case 2:
            pDate->dayOfWeek = SYS_TIME_WEEKDAY_FRIDAY;
            break;
        case 3:
            pDate->dayOfWeek = SYS_TIME_WEEKDAY_SATURDAY;
            break;
        default:
            pDate->dayOfWeek = SYS_TIME_WEEKDAY_UNKNOWN;
            break;
    }

    return day;
}


/*!
*******************************************************************************
**
** \brief   This Function forces an event when the next timer tick fires.
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
******************************************************************************/
int32_t TIME_ForceEvtUpdate(void)
{
    if(timeDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    timeDat->forceEvent = FTRUE;

    return FAPI_OK;
}


/*!
*******************************************************************************
**
** \brief   Return the current auto mode.
**
** \return  #FTRUE get time from stream.
**
*******************************************************************************
*/
fbool_t TIME_IsAutoMode(void)
{
    if ( timeDat == NULL ) return FFALSE;

    return timeDat->autoTimeMode;
}

/*!
*******************************************************************************
**
** \brief   Set the current auto mode.
**
** \param   mode    #FTRUE get time from stream.
**
*******************************************************************************
*/
void TIME_SetAutoMode(fbool_t mode)
{
    if(timeDat == NULL)
        return;

    if(timeDat->autoTimeMode != mode)
    {
        timeDat->autoTimeMode = mode;
        timeDat->syncedOnce = (fbool_t)!mode;
        timeDat->forceEvent = mode;
    }
}


/*!
*******************************************************************************
**
** \brief   TODO
**
** \param   utc     TODO
** \param   mjd     TODO
**
*******************************************************************************
*/
void TIME_Sync(uint32_t utc, uint16_t mjd)
{
    SYS_TIME_S          timeSync;
    SYS_DATE_S          dateSync;
    uint32_t            status;
    fbool_t             performSync = FFALSE;
    uint32_t            currSeconds;
    uint32_t            syncSeconds;

    if(timeDat == NULL)
        return;
    if ( timeDat->autoTimeMode == FFALSE) return;

    TIME_ConvertUTC(utc, &timeSync);
    TIME_ConvertMJD(mjd, &dateSync);

    //FAPI_SYS_PRINT_DEBUG(1,"### SYNC TIME/DATE ####\n");
    //FAPI_SYS_PRINT_DEBUG(1,"### INT: %2i:%02i:%02i \n", timeDat->timeCurr.hour,
    //                                       timeDat->timeCurr.minute,
    //                                       timeDat->timeCurr.second);
    //FAPI_SYS_PRINT_DEBUG(1,"### DVB: %2i:%02i:%02i \n", timeSync.hour,
    //                                       timeSync.minute,
    //                                       timeSync.second);
    //FAPI_SYS_PRINT_DEBUG(1,"### Date: %i/%02i/%02i - %i\n", dateSync.year, dateSync.month,
    //                                           dateSync.day,
    //                                           dateSync.dayOfWeek);

    FAPEX_SYS_EnterCritical(status);

    if ( (timeDat->dateCurr.dayOfWeek != dateSync.dayOfWeek) ||
         (timeDat->dateCurr.day       != dateSync.day)       ||
         (timeDat->dateCurr.month     != dateSync.month)     ||
         (timeDat->dateCurr.year      != dateSync.year)      ||
         (timeDat->syncedOnce         != FTRUE ) )
    {
        performSync = FTRUE;
    }
    else
    {
        currSeconds = TIME_ConvToSeconds(&(timeDat->timeCurr));
        syncSeconds = TIME_ConvToSeconds(&(timeSync));

        if ( ((currSeconds + TIME_SYNCTOLERANCE) < syncSeconds) ||
             ((syncSeconds + TIME_SYNCTOLERANCE) < currSeconds) )
        {
            performSync = FTRUE;
        }
    }

    if ( performSync )
    {
        FAPI_SYS_PRINT_DEBUG(4,"[TIME] Sync time! \n");

        timeDat->timeCurr     = timeSync;
        timeDat->dateCurr     = dateSync;
        timeDat->forceEvent   = FTRUE;
        timeDat->syncedOnce   = FTRUE;
    }

    FAPEX_SYS_LeaveCritical(status);
}

/*!
*******************************************************************************
**
** \brief   Return the current sync status.
**
** \return  #FTRUE if time is synced.
**
*******************************************************************************
*/
fbool_t TIME_IsSync (void)
{
    if(timeDat == NULL)
    {
        return FFALSE;
    }
    return timeDat->syncedOnce;
}

/*!
*******************************************************************************
**
** \brief   TODO
**
** \attention This function use FAPEX_SYS_EnterCritical()!
**
** \attention The given YMD must be between March, 1st 1900 and
**            February, 28th 2100.
**
** \param   currUtc         Current time in UTC format.
** \param   currMjd         Current date in MJD format.
** \param   changeUtc       TODO
** \param   changeMjd       TODO
** \param   thisTimeOffset  TODO
** \param   nextTimeOffset  TODO
**
*******************************************************************************
*/
void TIME_TimeZoneChange(uint32_t currUtc, uint16_t currMjd,
                         uint32_t changeUtc, uint16_t changeMjd,
                         int16_t thisTimeOffset, int16_t nextTimeOffset)
{
    SYS_TIME_S          currTime;
    SYS_DATE_S          currDate;
    SYS_TIME_S          changeTime;
    SYS_DATE_S          changeDate;
    uint32_t            status;
    fbool_t             changeInFuture;
    int16_t             currOffset;
    fbool_t             dstActive;

    if(timeDat == NULL)
    {
        return;
    }

    TIME_ConvertUTC(currUtc, &currTime);
    TIME_ConvertMJD(currMjd, &currDate);
    TIME_ConvertUTC(changeUtc, &changeTime);
    TIME_ConvertMJD(changeMjd, &changeDate);

    /* Find out if time zone change is in the future or in the past. */
    changeInFuture = TIME_IsLaterThan(&changeTime, &changeDate,
                                      &currTime, &currDate);

    if(SETTINGS_TimeOffsetGet(&currOffset, &dstActive) != FAPI_OK)
    {
        return;
    }

    if(dstActive)
    {
        currOffset += 60;
    }

    /* Test if change of current timezone is necessary */
    if(changeInFuture)
    {
        if(currOffset != thisTimeOffset)
        {
            changeTime = currTime;
            changeDate = currDate;
            nextTimeOffset = thisTimeOffset;
        }
    }
    else
    {
        if(currOffset != nextTimeOffset)
        {
            changeTime = currTime;
            changeDate = currDate;
        }
        else
        {
            /* nothing to be done! */
            return;
        }
    }

    FAPEX_SYS_EnterCritical(status);

    timeDat->tzChgTime   = changeTime;
    timeDat->tzChgDate   = changeDate;
    timeDat->tzChgOffset = nextTimeOffset;
    timeDat->tzChgReq    = FTRUE;

    FAPEX_SYS_LeaveCritical(status);
}

/*!
*******************************************************************************
**
** \brief   Start a single-shot timer.
**
** \param   timerNum       timer number (0...#TIME_SINGLETIMERS_MAX - 1)
** \param   timeInDeciSecs expiration time in deci-seconds
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
** \sa
**          - TIME_SingleTimerAbort()
**          - FAPI_TIMER_Start()
**          - FAPI_TIMER_Close()
**          - #FAPI_TIMER_OpenParamsT
**
*******************************************************************************
*/
int32_t TIME_SingleTimerLaunch(uint8_t timerNum, uint16_t timeInDeciSecs)
{
#ifndef _LINUX_FAPI
    FAPI_TIMER_OpenParamsT  timerParams;
    int32_t                 retVal;
#endif

    if( timeDat == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }
    if( timerNum >= TIME_SINGLETIMERS_MAX )
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    if(timeDat->timerHandle[timerNum] != NULL)
    {
        (void)TIME_SingleTimerAbort(timerNum);
    }

#ifndef _LINUX_FAPI
    memset(&timerParams, 0, sizeof(timerParams));

    timerParams.version    = FAPI_TIMER_VERSION;
    timerParams.type       = FAPI_TIMER_TYPE_SOFT;
    timerParams.mode       = FAPI_TIMER_MODE_ONESHOT;
    timerParams.resolution = FAPI_TIMER_RESOLUTION_100_MSEC;
    timerParams.multiplier = timeInDeciSecs;
    timerParams.callback1  = TIME_TimerCb;
    timerParams.optData1   = (TIMER_SINGLE << 8) | timerNum;
    timerParams.exec1InIsr = 0;

    timeDat->timerHandle[timerNum] = FAPI_TIMER_Open(&timerParams, &retVal);
    if(timeDat->timerHandle[timerNum] != NULL)
    {
        retVal = FAPI_TIMER_Start(timeDat->timerHandle[timerNum]);
    } else {
        retVal = FAPI_TIMER_Close(timeDat->timerHandle[timerNum]);
    }

    return (retVal);
#else

    return SYS_ThreadCreate (Cb1_ThreadFunction,
                             (void *)((timeInDeciSecs << 16) | (TIMER_SINGLE << 8) | timerNum),
                             THREAD_PRIO_TIMER_CB,
                             THREAD_STACKSZ_TIMER,
                             "Cb1",
                             &timeDat->timerHandle[timerNum]);
#endif
}

/*!
*******************************************************************************
**
** \brief   Abort requested single-shot timer.
**
** \param   timerNum    timer number (0...#TIME_SINGLETIMERS_MAX - 1)
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**          - #APPL_SMARTGO_ERR_BAD_PARAMETER
**
** \sa
**          - TIME_SingleTimerLaunch()
**          - FAPI_TIMER_Close()
**
******************************************************************************/
int32_t TIME_SingleTimerAbort(uint8_t timerNum)
{
    int32_t retVal;

    if(timeDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if(timerNum >= TIME_SINGLETIMERS_MAX)
        return APPL_SMARTGO_ERR_BAD_PARAMETER;

    if(timeDat->timerHandle[timerNum] == NULL)
    {
        return FAPI_OK;
    }
    /* stop timer */
#ifndef _LINUX_FAPI
    retVal = FAPI_TIMER_Close(timeDat->timerHandle[timerNum]);
#else
    retVal = FAPEX_SYS_DestroyThread(timeDat->timerHandle[timerNum]);
#endif
    timeDat->timerHandle[timerNum] = NULL;

    return retVal;
}


/*!
*******************************************************************************
**
** \brief   Close requested single-shot timer.
**
** \param   timerNum    timer number (0...#TIME_SINGLETIMERS_MAX - 1)
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_BAD_PARAMETER
**
** \sa
**          - TIME_SingleTimerLaunch()
**          - FAPI_TIMER_Close()
******************************************************************************/
int32_t TIME_SingleTimerClose(uint8_t timerNum)
{
    int32_t retVal;

    if ( timerNum >= TIME_SINGLETIMERS_MAX )
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
#ifndef _LINUX_FAPI
    retVal = FAPI_TIMER_Close(timeDat->timerHandle[timerNum]);
#else
    retVal = FAPEX_SYS_DestroyThread(timeDat->timerHandle[timerNum]);
#endif

        timeDat->timerHandle[timerNum] = NULL;
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief   Launch a periodic timer.
**
** \param   timerNum        Timer number (0...#TIME_SINGLETIMERS_MAX - 1)
** \param   timeInDeciSecs  Expiration time in deci-seconds.
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**
** \sa
**          - TIME_PeriodicTimerAbort()
**          - FAPI_TIMER_Start()
**          - #FAPI_TIMER_OpenParamsT
**
******************************************************************************/
int32_t TIME_PeriodicTimerLaunch(uint8_t timerNum, uint16_t timeInDeciSecs)
{
#ifndef _LINUX_FAPI
    int32_t                 retVal;
    FAPI_TIMER_OpenParamsT  timerParams;
#endif
    if(timeDat == NULL)
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;

    if(timeDat->periodTimerHandle[timerNum] != NULL)
    {
        (void)TIME_PeriodicTimerAbort(timerNum);
    }
#ifndef _LINUX_FAPI
    memset(&timerParams, 0, sizeof(timerParams));

    timerParams.version    = FAPI_TIMER_VERSION;
    timerParams.type       = FAPI_TIMER_TYPE_SOFT;
    timerParams.mode       = FAPI_TIMER_MODE_CONTINUOUS;
    timerParams.resolution = FAPI_TIMER_RESOLUTION_100_MSEC;
    timerParams.multiplier = timeInDeciSecs;
    timerParams.callback1  = TIME_TimerCb;
    timerParams.optData1   = (TIMER_PERIODIC << 8) | timerNum;
    timerParams.exec1InIsr = 0;

    timeDat->periodTimerHandle[timerNum] = FAPI_TIMER_Open(&timerParams, &retVal);
    if(timeDat->periodTimerHandle[timerNum])
    {
        retVal = FAPI_TIMER_Start(timeDat->periodTimerHandle[timerNum]);
    }

    return (retVal);
#else
    return SYS_ThreadCreate (Cb2_ThreadFunction,
                             (void *)((timeInDeciSecs << 16) | (TIMER_PERIODIC << 8) | timerNum),
                             THREAD_PRIO_TIMER_CB,
                             THREAD_STACKSZ_TIMER,
                             "Cb2",
                             &timeDat->periodTimerHandle[timerNum]);
#endif
}

/*!
*******************************************************************************
**
** \brief   Abort periodic timer.
**
** \param   timerNum    timer number (0...#TIME_SINGLETIMERS_MAX - 1)
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**          - #APPL_SMARTGO_ERR_NOT_INITIALIZED
**          - #APPL_SMARTGO_ERR_BAD_PARAMETER
**
** \sa
**          - TIME_PeriodicTimerLaunch()
**          - FAPI_TIMER_Close()
**
******************************************************************************/
int32_t TIME_PeriodicTimerAbort(uint8_t timerNum)
{
    int32_t retVal;

    if(timeDat == NULL)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    if(timerNum >= TIME_PERIODTIMERS_MAX)
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    /* stop timer */
#ifndef _LINUX_FAPI
    retVal = FAPI_TIMER_Close(timeDat->periodTimerHandle[timerNum]);
#else
    retVal = FAPEX_SYS_DestroyThread(timeDat->periodTimerHandle[timerNum]);
#endif
    timeDat->periodTimerHandle[timerNum] = NULL;

    return retVal;
}

/*!
*******************************************************************************
**
** \brief   Converts the given time from seconds in #SYS_TIME_S format.
**
** \note    The resulting time value may exceed 24 hours!
**
** \param   seconds     The time in seconds to be converted.
**
** \return  The converted time in #SYS_TIME_S format.
**
*******************************************************************************
*/
SYS_TIME_S TIME_ConvToTime(uint32_t seconds)
{
    SYS_TIME_S          time;
    uint32_t            min;

    memset(&time,0,sizeof(SYS_TIME_S));

    min = seconds / 60;

    time.second = seconds % 60;
    time.hour = (uint8_t)(min / 60);
    time.minute = min % 60;

    return time;
}


/*!
*******************************************************************************
**
** \brief   Converts the given time from utc format in to #SYS_TIME_S format.
**
** \param   utc     The time in UTC to be converted.
** \param   pTime   Pointer to #SYS_TIME_S to store the converted time.
**
** \sa
**          - TIME_ConvertMJD()
**
*******************************************************************************
*/
void TIME_ConvertUTC(uint32_t utc, SYS_TIME_S * pTime)
{
    pTime->hour = ((utc >> 20) & 0x03) * 10 + ((utc >> 16) & 0x0F);
    pTime->minute = ((utc >> 12) & 0x07) * 10 + ((utc >> 8) & 0x0F);
    pTime->second = ((utc >> 4) & 0x07) * 10 + (utc & 0x0F);
}

/*!
*******************************************************************************
**
** \brief   Converts the given time from mjd format in to #SYS_TIME_S format.
**
** \param   mjd     The time in MJD to be converted.
** \param   pDate   Pointer to #SYS_TIME_S to store the converted time.
**
** \sa
**          - TIME_ConvertUTC()
**
*******************************************************************************
*/
void TIME_ConvertMJD(uint16_t mjd, SYS_DATE_S * pDate)
{
    uint16_t m, d;

    MjdToYmd(mjd, &pDate->year, &m, &d);
    pDate->month = (uint8_t)m;
    pDate->day = (uint8_t)d;
    pDate->dayOfWeek = (SYS_TIME_WEEKDAY_E) (((mjd + 2) % 7) + 1);
}

/*!
*******************************************************************************
**
** \brief   Add a number of seconds to specified time and date.
**
** \param   pTime     time to add seconds on
** \param   pDate     corresponding date to add seconds on (optional)
** \param   secsToAdd number of seconds to add
**                    (negative values are also supported).
**
******************************************************************************/
void TIME_AddSeconds(SYS_TIME_S * pTime, SYS_DATE_S * pDate, int32_t secsToAdd)
{
    int32_t             offset;
    int32_t             secsCurr;

    while(secsToAdd != 0)
    {
        secsCurr = (int32_t) TIME_ConvToSeconds(pTime);

        /* ----------------------------------------------------------------- */

        if(secsToAdd < -(secsCurr))     /* one day back */
        {
            offset = secsCurr + 1;

            pTime->hour = 23;
            pTime->minute = 59;
            pTime->second = 59;

            secsToAdd += offset;

            /* adjust date */
            if(pDate != NULL)
            {
                TIME_AddDays(pDate, -1);
            }
        }

        /* ----------------------------------------------------------------- */

        else if((secsToAdd + secsCurr) >= TIME_SECSPERDAY)      /* next day */
        {
            offset = TIME_SECSPERDAY - secsCurr;

            pTime->hour = 0;
            pTime->minute = 0;
            pTime->second = 0;

            secsToAdd -= offset;

            /* adjust date */
            if(pDate != NULL)
            {
                TIME_AddDays(pDate, 1);
            }
        }

        /* ----------------------------------------------------------------- */

        else                    /* no change in date necessary */
        {
            secsCurr += secsToAdd;

            pTime->hour = (uint8_t)(secsCurr / TIME_SECSPERHOUR);
            secsCurr -= ((uint32_t) pTime->hour) * TIME_SECSPERHOUR;

            pTime->minute = (uint8_t)(secsCurr / 60);
            secsCurr -= ((uint32_t) pTime->minute) * 60;

            pTime->second = (uint8_t)secsCurr;

            secsToAdd = 0;
        }
    }
}

/*!
*******************************************************************************
**
** \brief   Add a number of hours to specified time and date.
**
** \param   pTime      time to add hours on
** \param   pDate      corresponding date to add hours on (optional)
** \param   hoursToAdd number of hours to add
**                     (negative values are also supported).
**
******************************************************************************/
void TIME_AddHours(SYS_TIME_S * pTime, SYS_DATE_S * pDate, int32_t hoursToAdd)
{
    while(hoursToAdd != 0)
    {
        /* ----------------------------------------------------------------- */

        if(hoursToAdd < -((int32_t) (pTime->hour)))     /* one day back */
        {
            hoursToAdd += pTime->hour + 1;
            pTime->hour = 23;

            /* adjust date */
            if(pDate != NULL)
            {
                TIME_AddDays(pDate, -1);
            }
        }

        /* ----------------------------------------------------------------- */

        else if((int32_t) (hoursToAdd + pTime->hour) >= 24)     /* next day */
        {
            hoursToAdd -= (24 - pTime->hour);
            pTime->hour = 0;

            /* adjust date */
            if(pDate != NULL)
            {
                TIME_AddDays(pDate, 1);
            }
        }

        /* ----------------------------------------------------------------- */

        else                    /* no change in date necessary */
        {
            pTime->hour += (uint8_t)hoursToAdd;
            hoursToAdd = 0;
        }
    }
}

/*!
*******************************************************************************
**
** \brief   Add a number of days to specified date.
**
** \param   pDate      date to add days on
** \param   daysToAdd  number of days to add
**                     (negative values are also supported).
**
******************************************************************************/
void TIME_AddDays(SYS_DATE_S * pDate, int32_t daysToAdd)
{
    uint8_t             daysPerMonth = 0;

    while(daysToAdd != 0)
    {
        if(daysToAdd > 0)
        {
            pDate->dayOfWeek++;
            daysToAdd--;

            /*lint -save -e685 */
            if(pDate->dayOfWeek > SYS_TIME_WEEKDAY_SUNDAY)
            /*lint -restore */
            {
                pDate->dayOfWeek = SYS_TIME_WEEKDAY_MONDAY;
            }

            pDate->day++;

            daysPerMonth = getDaysPerMonth(pDate->month, (uint8_t)pDate->year);

            if(pDate->day > daysPerMonth)
            {
                pDate->day -= daysPerMonth;

                pDate->month++;

                if(pDate->month > 12)
                {
                    pDate->month -= 12;

                    pDate->year++;
                }
            }
        }
        else
        {
            pDate->dayOfWeek--;
            daysToAdd++;

            if(pDate->dayOfWeek < SYS_TIME_WEEKDAY_MONDAY)
            {
                pDate->dayOfWeek = SYS_TIME_WEEKDAY_SUNDAY;
            }

            pDate->day--;

            if(pDate->day < 1)
            {
                pDate->month--;

                if(pDate->month < 1)
                {
                    pDate->month = 12;

                    pDate->year--;
                }

                pDate->day = getDaysPerMonth(pDate->month, (uint8_t)pDate->year);
            }
        }
    }
}

/*!
*******************************************************************************
**
** \brief   Add UTC offset and Daylight Saving Time offset to specified time
**          and date.
**
** \param   pTime     time to add offset on
** \param   pDate     corresponding date to add offset on (optional)
**
** \return  One of the following status codes:
**          - #FAPI_OK if successful
**
** \sa
**          - SETTINGS_TimeOffsetGet()
**          - TIME_AddSeconds()
******************************************************************************/
int32_t TIME_AddUtcOffset(SYS_TIME_S * pTime, SYS_DATE_S * pDate)
{
    int16_t             offset;
    fbool_t dstActive;
    int32_t             retVal;

    retVal = SETTINGS_TimeOffsetGet(&offset, &dstActive);
    if(retVal != FAPI_OK)
    {
        return retVal;
    }

    /* add DST if active */
    if(dstActive)
    {
        offset += 60;
    }

    /* adjust time and date */
    TIME_AddSeconds(pTime, pDate, (int32_t) offset * 60);

    return FAPI_OK;
}

/*!
*******************************************************************************
**
** \brief   Compare two dates.
**
** Function returns FTRUE if time/date 1 is \b later than time/date 2
**
** \param   time1  time 1
** \param   date1  date 1
** \param   time2  time 2
** \param   date2  date 2
**
** \return  Boolean result of comparison.
**
******************************************************************************/
fbool_t TIME_IsLaterThan(const SYS_TIME_S * time1, const SYS_DATE_S * date1,
                         const SYS_TIME_S * time2, const SYS_DATE_S * date2)
{
    if      ( date1->year > date2->year  ) return FTRUE;
    else if ( date1->year == date2->year )
    {
        if      ( date1->month > date2->month  ) return FTRUE;
        else if ( date1->month == date2->month )
        {
            if      ( date1->day > date2->day  ) return FTRUE;
            else if ( date1->day == date2->day )
            {
                if      ( time1->hour > time2->hour  ) return FTRUE;
                else if ( time1->hour == time2->hour )
                {
                    if      ( time1->minute > time2->minute  ) return FTRUE;
                    else if ( time1->minute == time2->minute )
                    {
                        if      ( time1->second > time2->second  ) return FTRUE;
                        else
                        {
                            return FFALSE;
                        }
                    }
                }
            }
        }
    }

    return FFALSE;
}

/*!
*******************************************************************************
**
** \brief   Calculate a time interval in seconds.
**
** \param   pStart  start Time
** \param   pEnd    end Time
**
** \return  Result in seonds.
**
******************************************************************************/
uint32_t TIME_GetLengthInSeconds(const SYS_TIME_S * pStart,
                                 const SYS_TIME_S * pEnd)
{
    uint32_t            result;
    uint32_t            endHour = pEnd->hour;

    /* test for 24-hour-wraparound... */
    if((pStart->hour > endHour) ||
       ((pStart->hour == endHour) && (pStart->minute > pEnd->minute)))
    {
        endHour += 24;
    }

    result = ((uint32_t) (endHour - pStart->hour)) * TIME_SECSPERHOUR;

    if(pStart->minute <= pEnd->minute)
    {
        result += ((pEnd->minute - pStart->minute) * 60);
    }
    else
    {
        result -= ((pStart->minute - pEnd->minute) * 60);
    }

    if(pStart->second <= pEnd->second)
    {
        result += (pEnd->second - pStart->second);
    }
    else
    {
        result -= (pStart->second - pEnd->second);
    }

    return result;
}

/*!
*******************************************************************************
**
** \brief   Get time value in seconds.
**
** \param   pTime   pointer to time data to be converted.
**
** \return  Result of conversion in seconds.
**
******************************************************************************/
uint32_t TIME_ConvToSeconds(const SYS_TIME_S * pTime)
{
    return (((uint32_t) pTime->hour) * TIME_SECSPERHOUR) +
        (((uint32_t) pTime->minute) * 60) + ((uint32_t) pTime->second);
}

/*---------------------------------------------------------------------------*/
/* thread function                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC void Tick_ThreadFunction(void* opt)
{
    while(1) {
        FAPI_SYS_SLEEP(TIME_TICK_INTERVAL * 100);

        TIME_TickCallback(0, 0);
    }
}

/* ------------------------------------------------------------------------- */
/* runs in IRQ context */
/*!
*******************************************************************************
**
** \brief   Callback for Timertick.
**
** \attention This function is called in BSR context!
**
** \param   val     current timer value.
** \param   optArg  optinal argument.
**
** \sa
**          - TIME_Init()
**          - FAPI_TIMER_Open()
**
******************************************************************************/
FSTATIC void TIME_TickCallback(uint64_t val, uint32_t optArg)
{
    SYS_EVT_TIME_S      timeEvent;
    SYS_EVT_TIMEZONE_S  tzEvent;
    uint8_t             daysPerMonth = 0;
    SYS_TIME_S          timeOld;
    SYS_TIME_S          timeNew;
    SYS_DATE_S          dateNew;

    timeOld = timeDat->timeCurr;

    /* ----- update time and date ------------------------------------------ */

    timeDat->systemTime += TIME_TICK_INTERVAL;  /* update system time */

    timeDat->milliSeconds += (TIME_TICK_INTERVAL * 100);

    while(timeDat->milliSeconds >= 1000)
    {
        timeDat->timeCurr.second++;
        timeDat->milliSeconds -= 1000;

        while(timeDat->timeCurr.second >= 60)
        {
            timeDat->timeCurr.minute++;
            timeDat->timeCurr.second -= 60;

            while(timeDat->timeCurr.minute >= 60)
            {
                timeDat->timeCurr.hour++;
                timeDat->timeCurr.minute -= 60;
            }
        }
    }

    /* adjust date if necessary */
    if(timeDat->timeCurr.hour >= 24)
    {
        timeDat->timeCurr.hour -= 24;

        timeDat->dateCurr.dayOfWeek++;
        /*lint -save -e685 */
        if(timeDat->dateCurr.dayOfWeek > SYS_TIME_WEEKDAY_SUNDAY)
        /*lint -restore */
        {
            timeDat->dateCurr.dayOfWeek = SYS_TIME_WEEKDAY_MONDAY;
        }
        timeDat->dateCurr.day++;

        /* update month */
        daysPerMonth = getDaysPerMonth(timeDat->dateCurr.month,
                                        (uint8_t)timeDat->dateCurr.year);

        if(timeDat->dateCurr.day > daysPerMonth)
        {
            timeDat->dateCurr.day -= daysPerMonth;

            timeDat->dateCurr.month++;

            if(timeDat->dateCurr.month > 12)
            {
                timeDat->dateCurr.month -= 12;

                timeDat->dateCurr.year++;
            }
        }
    }

    /* ----- test for timezone changes ------------------------------------- */

    if(timeDat->tzChgReq)
    {
        if(!(TIME_IsLaterThan(&(timeDat->tzChgTime), &(timeDat->tzChgDate),
                              &(timeDat->timeCurr), &(timeDat->dateCurr))))
        {
            tzEvent.hdr.type = EVT_TIMEZONE_CHANGE;
            tzEvent.hdr.length = sizeof(tzEvent);
            tzEvent.utcOffset = timeDat->tzChgOffset;

            (void)EVT_Send(&tzEvent, "Timezone update");

            timeDat->tzChgReq   = FFALSE;
            timeDat->forceEvent = FTRUE;
        }
    }

    /* ----- generate event if necessary ----------------------------------- */

    /* A time event is generated every second or if it is forced */
    if ( (timeDat->forceEvent == FTRUE)               ||
         (timeOld.second != timeDat->timeCurr.second) ||
         (timeOld.minute != timeDat->timeCurr.minute) ||
         (timeOld.hour   != timeDat->timeCurr.hour) )
    {
        timeNew = timeDat->timeCurr;
        dateNew = timeDat->dateCurr;

        (void)TIME_AddUtcOffset(&timeNew, &dateNew);

        timeEvent.hdr.type = EVT_TIME_UPDATE;
        timeEvent.hdr.length = sizeof(timeEvent);
        timeEvent.time = timeNew;
        timeEvent.date = dateNew;

        (void)EVT_Send(&timeEvent, "Time update");

        timeDat->forceEvent = FFALSE;
    }

#if 0
    if(timeOld.second != timeDat->timeCurr.second)
    {
        FM_Printf("SYS: %i:%02i:%02i - %04i-%02i-%02i - %i \n",
                  timeDat->timeCurr.hour,
                  timeDat->timeCurr.minute,
                  timeDat->timeCurr.second,
                  timeDat->dateCurr.year,
                  timeDat->dateCurr.month,
                  timeDat->dateCurr.day,
                  timeDat->dateCurr.dayOfWeek);
    }
#endif
}

/*---------------------------------------------------------------------------*/
/* thread function                                                           */
/*---------------------------------------------------------------------------*/
FSTATIC void Cb1_ThreadFunction(void* opt)
{
    uint32_t  tick, opt_data;

    FAPEX_SYS_ThreadT cur_threadHandle;

    tick     = (uint32_t)opt >> 16;
    opt_data = (uint32_t)opt & 0x0000FFFF;

    cur_threadHandle = FAPEX_SYS_GetThread();
    FAPI_SYS_SLEEP(100 * tick);

    TIME_TimerCb(0, opt_data);

    /* destroy this thread */
    FAPEX_SYS_DestroyThread( cur_threadHandle );
}

FSTATIC void Cb2_ThreadFunction(void* opt)
{
    uint32_t  tick, opt_data;

    tick     = (uint32_t)opt >> 16;
    opt_data = (uint32_t)opt & 0x0000FFFF;

    while(1) {
        FAPI_SYS_SLEEP(100 * tick);

        TIME_TimerCb(0, opt_data);
    }
}
/*!
*******************************************************************************
**
** \brief   Callback for SmartGO-HD internal Timer.
**
** \attention This function is called in BSR context!
**
** \param   val     current timer value.
** \param   optData optinal argument.
**
** \sa
**          - TIME_Init()
**          - FAPI_TIMER_Open()
**
*******************************************************************************
*/
FSTATIC void TIME_TimerCb(uint64_t val, uint32_t optData)
{
    SYS_EVT_TIMER_S     ctrlEvent;

    /* generate event */
    ctrlEvent.hdr.type   = EVT_TIMER_EXPIRED;
    ctrlEvent.hdr.length = sizeof(ctrlEvent);
    ctrlEvent.type       = (SYS_TIMER_TYPE_E)(optData >> 8);
    ctrlEvent.num        = optData & 0xFF;

    (void)EVT_Send(&ctrlEvent, "Timer event");
}


/*!
*******************************************************************************
**
** \brief   Returns the number of days of the given month in the given year.
**
** \param   month   The month.
** \param   year    The year.
**
** \sa
**          - TIME_TickCallback()
**          - TIME_AddDays()
**
*******************************************************************************
*/
FSTATIC uint8_t getDaysPerMonth(uint8_t month, uint8_t year)
{
    switch (month)
    {
        case 1:
        case 3:
        case 5:
        case 7:
        case 8:
        case 10:
        case 12:
        default:
            return 31;

        case 4:
        case 6:
        case 9:
        case 11:
            return 30;

        case 2:
            /* test if currently in a leap year */
            if((year & 0x3) == 0)
                return 29;
            else
                return 28;
    }
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
