/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/ttxsubtitle.c
**
**              $Id: ttxsubtitle.c 5866 2010-03-02 12:09:31Z jmorha $
**
** \brief       TTX and TTX Subtitle control functions.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
*****************************************************************************
*/
/*lint -save  */
#define DEBUG 0

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include <fapi/sys_driver.h>         /* global type definitions */

#include "av.h"

#include "teletext.h"
#include "utils.h"
#include "ttxvbi.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#if (DEBUG > 0)
    #define TTXSUBTT_SETPID_DEBUG
#endif

#define TTXSUBTT_TYPE_INITIAL_PAGE      0x01
#ifdef APPL_TTXT_ENABLED
    #define TTXSUBTT_TYPE_SUBTITLE          0x02
    #define TTXSUBTT_TYPE_HEARING_IMPARED   0x05
#endif

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

typedef struct
{
    FAPEX_SYS_SemaphoreT mutex;
    uint16_t            currPid;        //!< Currently active TTX PID.
    uint16_t            ttxPid;
    int32_t             ttxTsdId;
    int32_t             ttxDecId;
    FAPI_SYS_HandleT    ttxhandle;
    #ifdef APPL_TTXT_ENABLED

    TTXSUBTT_INF_S      stream[TTXSUBTT_STREAMS_MAX];
    uint8_t             numStreams;     //!< number of streams
    int8_t              subStrCurr;     //!< -1: no stream selected
    uint8_t             subStrStarted;
    SYS_SUSPENDSRC      suspendStatus;

    fbool_t             ttxMode;

    #endif                          /* APPL_TTXT_ENABLED */

} TTXSUBTT_PARAMS_S;


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

FSTATIC TTXSUBTT_PARAMS_S *subttPars = NULL;

/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC void         TTXSUBTT_ProcessDescriptor(uint16_t pid,
                                               const PSISI_DESC_S * pDesc);
FSTATIC void         TTXSUBTT_SetPid(uint16_t pid);
FSTATIC void         TTXSUBTT_IntClear(void);

/* ----- Public API -------------------------------------------------------- */

#ifdef APPL_TTXT_ENABLED

int32_t TTXSUBTT_Select(int8_t index)
{
    int32_t             retVal = FAPI_OK;
    TTXSUBTT_INF_S     *pSubtt;

    if(subttPars == NULL) return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    (void)FAPEX_SYS_GetSemaphore(subttPars->mutex, FAPEX_SYS_SUSPEND);

    if(index >= (int16_t) subttPars->numStreams)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else if(index < 0)          /* disable subtitle presentation */
    {
        /* restore CLUT if necessary */
        if(subttPars->subStrStarted)
        {
            (void)TTX_IntHide();      /* disable TTX display */

            subttPars->subStrStarted = 0;
        }

        subttPars->subStrCurr = -1;

        /* reset default TTX PID */
        TTXSUBTT_SetPid(subttPars->ttxPid);
    }
    else
    {
        pSubtt = subttPars->stream + index;

        /* check if stream can be started. If presentation is currently
           suspended, stream will be started as soon as unsuspend is
           received. */

        if((!(subttPars->ttxMode)) && (subttPars->suspendStatus == 0))
        {
            TTXSUBTT_SetPid(pSubtt->pid);

            /* select subtitle page */
            (void)TTX_IntShow(pSubtt->pageNumber);

            subttPars->subStrStarted = 1;

            FAPI_SYS_PRINT_DEBUG(4,"[TTXSUB] Started (page=%d).\n", pSubtt->pageNumber);
        }
        else
        {
            subttPars->subStrStarted = 0;
        }

        subttPars->subStrCurr = index;
    }

    (void)FAPEX_SYS_SetSemaphore(subttPars->mutex, FAPEX_SYS_NO_SUSPEND);

    return retVal;
}

void TTXSUBTT_Hide(void)
{
    TTXSUBTT_Enable(FFALSE, SYS_SUSPENDSRC_USER);
}

void TTXSUBTT_Restart(void)
{
    TTXSUBTT_Enable(FTRUE, SYS_SUSPENDSRC_USER);
}

uint8_t TTXSUBTT_NumStreamsGet(void)
{
    return subttPars->numStreams;
}

int8_t TTXSUBTT_CurrStreamGet(void)
{
    return subttPars->subStrCurr;
}

int32_t TTXSUBTT_StreamInfoGet(uint8_t index, TTXSUBTT_INF_S * pStrInf)
{
    int32_t             retVal = FAPI_OK;

    if(subttPars == NULL) return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    (void)FAPEX_SYS_GetSemaphore(subttPars->mutex, FAPEX_SYS_SUSPEND);

    if(index >= subttPars->numStreams)
    {
        retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
    }
    else
    {
        *pStrInf = subttPars->stream[index];
    }

    (void)FAPEX_SYS_SetSemaphore(subttPars->mutex, FAPEX_SYS_NO_SUSPEND);
    return retVal;
}

#endif /* APPL_TTXT_ENABLED */

/* ----- Private API ------------------------------------------------------- */

int32_t TTXSUBTT_Init(void)
{
    subttPars = UTILS_Malloc(sizeof(TTXSUBTT_PARAMS_S));
    if(subttPars == NULL)
        return APPL_SMARTGO_ERR_OUT_OF_MEMORY;

    subttPars->mutex = FAPEX_SYS_CreateSemaphore(1);
    if(subttPars->mutex == NULL)
    {
        return SMARTGO_SYS_ERR_RESOURCES;
    }

    subttPars->currPid = AV_PID_NONE;
    subttPars->ttxPid = AV_PID_NONE;

    #ifdef APPL_TTXT_ENABLED

    subttPars->numStreams = 0;
    subttPars->subStrCurr = -1;
    subttPars->subStrStarted = 0;
    subttPars->suspendStatus = 0;

    subttPars->ttxMode = FFALSE;

    #endif /* APPL_TTXT_ENABLED */

    return FAPI_OK;
}

void TTXSUBTT_Clear(void)
{
    if (subttPars == NULL) return;
    (void)FAPEX_SYS_GetSemaphore(subttPars->mutex, FAPEX_SYS_SUSPEND);

    TTXSUBTT_IntClear();

    (void)FAPEX_SYS_SetSemaphore(subttPars->mutex, FAPEX_SYS_NO_SUSPEND);
}

int32_t TTXSUBTT_PmtUpdate(const PSISI_PMT_S * pPmt,
                           fbool_t (*pidAvailable) (uint16_t pid))
{
    uint16_t            pmtPos;
    PSISI_PMT_STREAM_S  stream;
    PSISI_DESC_S        desc;
    uint16_t            infoPos;
    uint16_t            firstTtxPid = AV_PID_NONE;

    #ifdef APPL_TTXT_ENABLED

    TTXSUBTT_INF_S      subttCurr;
    int8_t              currIndex;
    int8_t              cnt;

    memset(&subttCurr,0,sizeof(subttCurr));

    #endif /* APPL_TTXT_ENABLED */

    if(subttPars == NULL) return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    (void)FAPEX_SYS_GetSemaphore(subttPars->mutex, FAPEX_SYS_SUSPEND);

    #ifdef APPL_TTXT_ENABLED

    currIndex = subttPars->subStrCurr;

    /* store current selection */
    if(currIndex >= 0)
    {
        subttCurr = subttPars->stream[currIndex];
    }

    #endif /* APPL_TTXT_ENABLED */

    TTXSUBTT_IntClear();        /* clear list first */

    if(PSISI_PMT_StreamParseInit(pPmt, &pmtPos) == FAPI_OK)
    {
        while(PSISI_PMT_StreamParseGetNext(pPmt, &pmtPos, &stream) == FAPI_OK)
        {
            if((pidAvailable != NULL) && (!(pidAvailable(stream.pid))))
            {
                continue;
            }

            if(stream.type == STRTYPE_PRIVDATA)
            {
                infoPos = 0;

                while(PSISI_DescParseGetNext(stream.esInfoStart,
                                             &infoPos, stream.esInfoLen,
                                             &desc) == FAPI_OK)
                {
                    if(desc.tag == TELETEXT_DESCRIPTOR)
                    {
                        if(firstTtxPid == AV_PID_NONE)
                        {
                            firstTtxPid = stream.pid;
                        }

                        TTXSUBTT_ProcessDescriptor(stream.pid, &desc);
                    }
                }
            }
        }
    }

    /* Fallback: Select first TTX PID if none of type "initial teletext page"
       has been found. */
    if(subttPars->ttxPid == AV_PID_NONE)
    {
        subttPars->ttxPid = firstTtxPid;
    }

    /* ----- perform re-selection if necessary ----------------------------- */

    #ifdef APPL_TTXT_ENABLED

    if(!(subttPars->ttxMode) && (currIndex >= 0) && (subttPars->numStreams > 0))
    {
        currIndex = -1;

        /* Search for total match first */
        for(cnt = 0; cnt < (int8_t) subttPars->numStreams; cnt++)
        {
            if((subttPars->stream[cnt].langCode == subttCurr.langCode) &&
               (subttPars->stream[cnt].type == subttCurr.type))
            {
                currIndex = cnt;
                break;
            }
        }

        /* Search for language match next */
        if(currIndex < 0)
        {
            for(cnt = 0; cnt < (int8_t) subttPars->numStreams; cnt++)
            {
                if(subttPars->stream[cnt].langCode == subttCurr.langCode)
                {
                    currIndex = cnt;
                    break;
                }
            }
        }

        /* still no match, set default */
        if(currIndex < 0)
        {
            currIndex = 0;
        }

        (void)FAPEX_SYS_SetSemaphore(subttPars->mutex, FAPEX_SYS_NO_SUSPEND);

        (void)TTXSUBTT_Select(currIndex);
    }
    else
    #endif /* APPL_TTXT_ENABLED */
    {
        /* select default PID */
        TTXSUBTT_SetPid(subttPars->ttxPid);

        (void)FAPEX_SYS_SetSemaphore(subttPars->mutex, FAPEX_SYS_NO_SUSPEND);
    }

    return FAPI_OK;
}

#ifdef APPL_TTXT_ENABLED

void TTXSUBTT_Enable(fbool_t enable, SYS_SUSPENDSRC src)
{
    if(subttPars == NULL) return;
    (void)FAPEX_SYS_GetSemaphore(subttPars->mutex, FAPEX_SYS_SUSPEND);

    if(enable)
    {
        if(subttPars->suspendStatus != 0)
        {
            subttPars->suspendStatus &= ~(src);

            if((!(subttPars->ttxMode)) &&
               (subttPars->suspendStatus == 0) && (subttPars->subStrCurr >= 0))
            {
                FAPI_SYS_PRINT_DEBUG(4,"[TTXSUB] Restart\n");

                TTXSUBTT_SetPid(subttPars->stream[subttPars->subStrCurr].pid);

                (void)TTX_IntShow(subttPars->stream[subttPars->subStrCurr].
                            pageNumber);
                subttPars->subStrStarted = 1;
            }
        }
    }
    else
    {
        if((subttPars->subStrCurr != -1) &&
           (subttPars->subStrStarted == 1) && (subttPars->suspendStatus == 0))
        {
            FAPI_SYS_PRINT_DEBUG(4,"[TTXSUB] Suspend\n");

            (void)TTX_IntHide();      /* suspend presentation */

            subttPars->subStrStarted = 0;

            TTXSUBTT_SetPid(subttPars->ttxPid);
        }

        subttPars->suspendStatus |= src;
    }

    (void)FAPEX_SYS_SetSemaphore(subttPars->mutex, FAPEX_SYS_NO_SUSPEND);
}

int32_t TTXSUBTT_SetTtxMode(fbool_t enable)
{
    int8_t              reselect = -1;

    if(subttPars == NULL) return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    (void)FAPEX_SYS_GetSemaphore(subttPars->mutex, FAPEX_SYS_SUSPEND);

    if(enable != subttPars->ttxMode)
    {
        if(enable)
        {
            /* disable TTX subtitle stream if currently active */
            if(subttPars->subStrStarted)
            {
                (void)TTX_IntHide();  /* disable TTX display */

                subttPars->subStrStarted = 0;
            }

            TTXSUBTT_SetPid(subttPars->ttxPid);
        }
        else
        {
            if(subttPars->subStrCurr >= 0)
            {
                /* Trigger reselection of TTX subtitle stream when leaving
                   TTX mode. */
                reselect = subttPars->subStrCurr;
            }
        }

        subttPars->ttxMode = enable;
    }

    (void)FAPEX_SYS_SetSemaphore(subttPars->mutex, FAPEX_SYS_NO_SUSPEND);

    /* re-select TTX subtitle if necessary */
    if(reselect >= 0)
    {
        (void)TTXSUBTT_Select(subttPars->subStrCurr);
    }

    return FAPI_OK;
}

#endif /* APPL_TTXT_ENABLED */

/* ----- Internal functions ------------------------------------------------ */

FSTATIC void TTXSUBTT_ProcessDescriptor(uint16_t pid, const PSISI_DESC_S * pDesc)
{
    uint16_t            pos = 2;
    uint8_t             teletext_type;

    #ifdef APPL_TTXT_ENABLED
    TTXSUBTT_INF_S     *pInf;
    uint8_t             magazine_number;
    #endif /* APPL_TTXT_ENABLED */

    while((uint16_t) (pos + 5) <= pDesc->length)
    {
        teletext_type = pDesc->start[pos + 3] >> 3;

        switch (teletext_type)
        {
            case TTXSUBTT_TYPE_INITIAL_PAGE:
                if(subttPars->ttxPid == AV_PID_NONE)
                {
                    subttPars->ttxPid = pid;
                }
                break;

            #ifdef APPL_TTXT_ENABLED

            case TTXSUBTT_TYPE_SUBTITLE:
            case TTXSUBTT_TYPE_HEARING_IMPARED:
                if(subttPars->numStreams < TTXSUBTT_STREAMS_MAX)
                {
                    pInf = subttPars->stream + subttPars->numStreams;

                    magazine_number = pDesc->start[pos + 3] & 0x07;

                    pInf->pid = pid;
                    pInf->langCode = PSISI_GetLanguageCode(pDesc->start + pos);
                    pInf->type = (teletext_type == TTXSUBTT_TYPE_SUBTITLE) ?
                                 SYS_SUBTITLE_TYPE_NORMAL : SYS_SUBTITLE_TYPE_HEARINGIMPAIRED;

                    pInf->pageNumber = (((uint16_t)magazine_number&0x00FF)<<8) +
                                       ((uint16_t)pDesc->start[pos + 4]);

                    subttPars->numStreams++;
                }
                break;

            #endif /* APPL_TTXT_ENABLED */

            default:
                break;
        }

        pos += 5;
    }
}


FSTATIC void TTXSUBTT_SetPid(uint16_t pid)
{
    if(subttPars == NULL)
    {
        return;
    }

    if(pid != subttPars->currPid)
    {
        if(subttPars->currPid != AV_PID_NONE)   /* close current TTX */
        {
            (void)TTXVBI_Stop();
            (void)TTXVBI_Close();

            subttPars->currPid   = AV_PID_NONE;
            subttPars->ttxhandle = NULL;
        }

        if(pid != AV_PID_NONE)
        {
            TTXVBI_Open(&subttPars->ttxTsdId, &subttPars->ttxDecId);
            if(subttPars->ttxhandle != NULL)
            {
                (void)TTXVBI_Start(pid);
                subttPars->currPid = pid;
            } else {
               subttPars->currPid   = AV_PID_NONE;
            }
        }
        else
        {
            subttPars->ttxhandle = NULL;
        }

        /* Reset software decoder if necessary */
        #ifdef APPL_TTXT_ENABLED
            #ifdef TTXSUBTT_SETPID_DEBUG
        FAPI_SYS_PRINT_DEBUG(4,"  PID= %d, ttxhandle= %X\n", pid, subttPars->ttxhandle);
            #endif
        TTX_Reset(subttPars->ttxhandle);
        #endif /* APPL_TTXT_ENABLED */
    }
}

FSTATIC void TTXSUBTT_IntClear(void)
{
    #ifdef APPL_TTXT_ENABLED

    /* stop subtitle if currently running */
    if(subttPars->subStrStarted)
    {
        (void)TTX_IntHide();

        subttPars->subStrStarted = 0;
    }

    subttPars->numStreams = 0;
    subttPars->subStrCurr = -1;
    /* NOTE: Do not clear TTX mode: might persist over service/PMT changes. */

    #endif /* APPL_TTXT_ENABLED */

    subttPars->ttxPid = AV_PID_NONE;

    TTXSUBTT_SetPid(AV_PID_NONE);
}

/******************************************************************************/
/*lint -restore */
/* END OF FILE */
