/*!
*****************************************************************************
**
** \file        ./applications/smartgo-hd/sys/src/av_int.c
**
**              $Id: av_int.c 5881 2010-03-03 08:04:49Z jmorha $
**
** \brief       Internal A/V (API) functions.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
**
*****************************************************************************
**
** \note The following LINT statements have been suppressed:
**
** - Warning 641: Converting enum 'anyEnum' to int.
** - Warning 46 : Bit fields in a structure should be typed unsigned or int
**
*****************************************************************************
*/
/*lint -save -e641 -e46 */
#define DEBUG 1

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/
#include "fapex/abst_types.h"
#include "fapex/abst_libtsd.h"
#include "fapex/abst_libaudec.h"
#include "fapex/abst_libauout.h"
#include "fapex/abst_libvidec.h"
#include "fapex/abst_libvienc.h"
#include "fapex/abst_libviout.h"
#include "fapex/sys_config.h"


#include <val/val.h>
#include <aal/aal.h>

#include "scart.h"
#include "event_queue.h"
#include "fcdata.h"

#include "av.h"

#include "utils.h"

/*---------------------------------------------------------------------------*/
/* private data declarations                                                 */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/

#define AV_SCARTOUTCHECKINTERVAL_MIN    500         //!< in ms.

#if (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED)
#define AV_SUBTT_WEIGHT_LANG        0x200
#define AV_SUBTT_WEIGHT_LANG_ORIG   0x100
#define AV_SUBTT_WEIGHT_LANG_UND    0x80
#define AV_SUBTT_WEIGHT_ZERO        0x00
#endif /* (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED) */

#ifdef APPL_SUBTT_ENABLED
#define AV_SUBTT_WEIGHT_ISDVB       0x40
#define AV_SUBTT_WEIGHT_TYPE        0x20
#define AV_SUBTT_WEIGHT_RATIO       0x10
#endif /* APPL_SUBTT_ENABLED */


/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/
#if 0
typedef enum{
    AV_AFD_AS_CODED_FRAME   = 8,
    AV_AFD_4_3,
    AV_AFD_16_9,
    AV_AFD_14_9,
    AV_AFD_RESERVED,
    AV_AFD_4_3_CENTRE_14_9,
    AV_AFD_16_9_CENTRE_14_9,
    AV_AFD_SHOOTPROTECT_4_3

} AV_AFD_E;
#endif

typedef struct {
    FAPEX_VISCALE_AspectRatioEnumT
                        hdmiAR;         // intended aspect ratio for HDMI signaling
    uint8_t             hdmiAFD;        // intended AFD for HDMI
    AV_WSS_E            wss;            // wss signal at PAL 23rd line
                                        // note: LSB of wss is at left
    SCART_MODE_E        scartMode;      // scart pin 8

} AV_AR_SIGNALS_S;

typedef struct{
    // LSB
    uint32_t        codedAR      :1; // coded frame aspect ratio, 0:4_3, 1:16_9
    uint32_t        afd          :4; // coded AFD
    uint32_t        hdmiMode     :1; // HDMI mode 0:576I/P, 1: 720P or 1080I
    uint32_t        dispAR       :1; // display aspect ratio, 0:4_3, 1:16_9
    uint32_t        scaleMethod  :2; // scaling method 0: pillar/letter box,
                                  // 1: fullscreen zoom, 2: CCO, 3: let TV scale
    uint32_t        hdmiAR       :1; // HDMI M bit 0:4_3, 1:16_9
    uint32_t        hdmiAFD      :4; // HDMI R bits
    uint32_t        wss          :4; // WSS signal in PAL
    uint32_t        scartMode    :2; // Scart Pin 8
    uint32_t        hdmiAR_UK    :1; // HDMI M bit for DTG profile
    uint32_t        hdmiAFD_UK   :4; // HDMI R bits for DTG profile
    uint32_t        wss_UK       :4; // WSS signal in PAL for DTG profile
    uint32_t        scartMode_UK :2; // Scart Pin 8 for DTG profile
    //MSB
} AV_AR_TABLE_S;


/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/

/*! Lookup table for aspect ratio handling. The bit order is arranged according
**  to AV_AR_TABLE_S structure. It has 224 cases totally and requires
**  224*4 = 896 Byte memory space.
**  FIXME: Now the HDMI's aspect ratio signaling for 720P or 1080I (always
**         16:9) contradicts a 4:3 display aspect ratio selection and could
**         result in double letter/pillar boxing on a HDMI connected 4:3 TV. So
**         we take the assumption that the 4:3 HDMI equipped TV does not react
**         to aspect ratio signaling.
*/
static AV_AR_TABLE_S aspectTab[224] = {
    {0, 8, 0, 0, 0, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 8, 1, 0, 0, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 8, 0, 1, 0, 1, 8, 7, 2, 0, 8, 8, 1},
    {0, 8, 1, 1, 0, 1, 8, 7, 2, 1, 0, 7, 2},
    {0, 8, 0, 0, 1, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 8, 1, 0, 1, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 8, 0, 1, 1, 1, 8, 7, 2, 0, 8, 8, 1},
    {0, 8, 1, 1, 1, 1, 8, 7, 2, 1, 0, 7, 2},
    {0, 8, 0, 0, 2, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 8, 1, 0, 2, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 8, 0, 1, 2, 1, 8, 7, 2, 0, 8, 8, 1},
    {0, 8, 1, 1, 2, 1, 8, 7, 2, 1, 0, 7, 2},
    {0, 8, 0, 0, 3, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 8, 1, 0, 3, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 8, 0, 1, 3, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 8, 1, 1, 3, 1, 8, 7, 2, 1, 0, 7, 2},
    {1, 8, 0, 0, 0, 0, 8, 8, 1, 0, 8, 11, 1},
    {1, 8, 1, 0, 0, 1, 8, 8, 1, 1, 8, 11, 1},
    {1, 8, 0, 1, 0, 1, 8, 7, 2, 1, 8, 7, 2},
    {1, 8, 1, 1, 0, 1, 8, 7, 2, 1, 8, 7, 2},
    {1, 8, 0, 0, 1, 0, 8, 8, 1, 0, 8, 11, 1},
    {1, 8, 1, 0, 1, 1, 8, 8, 1, 1, 8, 11, 1},
    {1, 8, 0, 1, 1, 1, 8, 7, 2, 1, 8, 7, 2},
    {1, 8, 1, 1, 1, 1, 8, 7, 2, 1, 8, 7, 2},
    {1, 8, 0, 0, 2, 0, 8, 8, 1, 0, 8, 8, 1},
    {1, 8, 1, 0, 2, 1, 8, 8, 1, 1, 8, 8, 1},
    {1, 8, 0, 1, 2, 1, 8, 7, 2, 1, 8, 7, 2},
    {1, 8, 1, 1, 2, 1, 8, 7, 2, 1, 8, 7, 2},
    {1, 8, 0, 0, 3, 1, 8, 7, 2, 0, 8, 11, 1},
    {1, 8, 1, 0, 3, 1, 8, 7, 2, 1, 8, 11, 1},
    {1, 8, 0, 1, 3, 1, 8, 7, 2, 1, 8, 7, 2},
    {1, 8, 1, 1, 3, 1, 8, 7, 2, 1, 8, 7, 2},
    {0, 9, 0, 0, 0, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 9, 1, 0, 0, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 9, 0, 1, 0, 1, 8, 7, 2, 0, 9, 8, 1},
    {0, 9, 1, 1, 0, 1, 8, 7, 2, 1, 9, 7, 2},
    {0, 9, 0, 0, 1, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 9, 1, 0, 1, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 9, 0, 1, 1, 1, 8, 7, 2, 0, 9, 8, 1},
    {0, 9, 1, 1, 1, 1, 8, 7, 2, 1, 9, 7, 2},
    {0, 9, 0, 0, 2, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 9, 1, 0, 2, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 9, 0, 1, 2, 1, 8, 7, 2, 0, 9, 8, 1},
    {0, 9, 1, 1, 2, 1, 8, 7, 2, 1, 9, 7, 2},
    {0, 9, 0, 0, 3, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 9, 1, 0, 3, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 9, 0, 1, 3, 0, 9, 8, 1, 0, 9, 8, 1},
    {0, 9, 1, 1, 3, 1, 9, 7, 2, 1, 9, 7, 2},
    {1, 9, 0, 0, 0, 0, 8, 8, 1, 0, 9, 8, 1},
    {1, 9, 1, 0, 0, 1, 8, 8, 1, 1, 9, 8, 1},
    {1, 9, 0, 1, 0, 1, 8, 7, 2, 0, 9, 8, 1},
    {1, 9, 1, 1, 0, 1, 8, 7, 2, 1, 9, 7, 2},
    {1, 9, 0, 0, 1, 0, 8, 8, 1, 0, 9, 8, 1},
    {1, 9, 1, 0, 1, 1, 8, 8, 1, 1, 9, 8, 1},
    {1, 9, 0, 1, 1, 1, 8, 7, 2, 0, 9, 8, 1},
    {1, 9, 1, 1, 1, 1, 8, 7, 2, 1, 9, 7, 2},
    {1, 9, 0, 0, 2, 0, 8, 8, 1, 0, 9, 8, 1},
    {1, 9, 1, 0, 2, 1, 8, 8, 1, 1, 9, 8, 1},
    {1, 9, 0, 1, 2, 1, 8, 7, 2, 0, 9, 8, 1},
    {1, 9, 1, 1, 2, 1, 8, 7, 2, 1, 9, 7, 2},
    {1, 9, 0, 0, 3, 0, 8, 8, 1, 0, 9, 8, 1},
    {1, 9, 1, 0, 3, 1, 8, 8, 1, 1, 9, 8, 1},
    {1, 9, 0, 1, 3, 0, 9, 8, 1, 0, 9, 8, 1},
    {1, 9, 1, 1, 3, 1, 9, 7, 2, 1, 9, 7, 2},
    {0, 10, 0, 0, 0, 0, 8, 8, 1, 0, 8, 11, 1},
    {0, 10, 1, 0, 0, 1, 8, 8, 1, 1, 8, 11, 1},
    {0, 10, 0, 1, 0, 1, 8, 7, 2, 0, 10, 11, 1},
    {0, 10, 1, 1, 0, 1, 8, 7, 2, 1, 10, 7, 2},
    {0, 10, 0, 0, 1, 0, 8, 8, 1, 0, 8, 11, 1},
    {0, 10, 1, 0, 1, 1, 8, 8, 1, 1, 8, 11, 1},
    {0, 10, 0, 1, 1, 1, 8, 7, 2, 0, 10, 11, 1},
    {0, 10, 1, 1, 1, 1, 8, 7, 2, 1, 10, 7, 2},
    {0, 10, 0, 0, 2, 0, 8, 8, 1, 0, 8, 11, 1},
    {0, 10, 1, 0, 2, 1, 8, 8, 1, 1, 8, 11, 1},
    {0, 10, 0, 1, 2, 1, 8, 7, 2, 0, 10, 11, 1},
    {0, 10, 1, 1, 2, 1, 8, 7, 2, 1, 10, 7, 2},
    {0, 10, 0, 0, 3, 1, 8, 7, 2, 0, 8, 11, 1},
    {0, 10, 1, 0, 3, 1, 8, 7, 2, 1, 8, 11, 1},
    {0, 10, 0, 1, 3, 1, 8, 7, 2, 0, 10, 11, 1},
    {0, 10, 1, 1, 3, 1, 8, 7, 2, 1, 10, 7, 2},
    {1, 10, 0, 0, 0, 0, 8, 8, 1, 0, 10, 11, 1},
    {1, 10, 1, 0, 0, 1, 8, 8, 1, 1, 10, 11, 1},
    {1, 10, 0, 1, 0, 1, 8, 7, 2, 1, 10, 7, 2},
    {1, 10, 1, 1, 0, 1, 8, 7, 2, 1, 10, 7, 2},
    {1, 10, 0, 0, 1, 0, 8, 8, 1, 0, 10, 11, 1},
    {1, 10, 1, 0, 1, 1, 8, 8, 1, 1, 10, 11, 1},
    {1, 10, 0, 1, 1, 1, 8, 7, 2, 1, 10, 7, 2},
    {1, 10, 1, 1, 1, 1, 8, 7, 2, 1, 10, 7, 2},
    {1, 10, 0, 0, 2, 0, 8, 8, 1, 0, 10, 8, 1},
    {1, 10, 1, 0, 2, 1, 8, 8, 1, 1, 10, 8, 1},
    {1, 10, 0, 1, 2, 1, 8, 7, 2, 1, 10, 7, 2},
    {1, 10, 1, 1, 2, 1, 8, 7, 2, 1, 10, 7, 2},
    {1, 10, 0, 0, 3, 1, 8, 7, 2, 0, 10, 11, 1},
    {1, 10, 1, 0, 3, 1, 8, 7, 2, 1, 10, 11, 1},
    {1, 10, 0, 1, 3, 1, 8, 7, 2, 1, 10, 7, 2},
    {1, 10, 1, 1, 3, 1, 8, 7, 2, 1, 10, 7, 2},
    {0, 11, 0, 0, 0, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 11, 1, 0, 0, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 11, 0, 1, 0, 1, 8, 7, 2, 0, 11, 1, 1},
    {0, 11, 1, 1, 0, 1, 8, 7, 2, 1, 11, 7, 2},
    {0, 11, 0, 0, 1, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 11, 1, 0, 1, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 11, 0, 1, 1, 1, 8, 7, 2, 0, 11, 1, 1},
    {0, 11, 1, 1, 1, 1, 8, 7, 2, 1, 11, 7, 2},
    {0, 11, 0, 0, 2, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 11, 1, 0, 2, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 11, 0, 1, 2, 1, 8, 7, 2, 0, 11, 1, 1},
    {0, 11, 1, 1, 2, 1, 8, 7, 2, 1, 11, 7, 2},
    {0, 11, 0, 0, 3, 0, 8, 1, 1, 0, 8, 8, 1},
    {0, 11, 1, 0, 3, 1, 8, 1, 1, 1, 8, 8, 1},
    {0, 11, 0, 1, 3, 0, 11, 1, 1, 0, 11, 1, 1},
    {0, 11, 1, 1, 3, 1, 11, 7, 2, 1, 11, 7, 2},
    {1, 11, 0, 0, 0, 0, 8, 8, 1, 0, 11, 8, 1},
    {1, 11, 1, 0, 0, 1, 8, 8, 1, 1, 11, 8, 1},
    {1, 11, 0, 1, 0, 1, 8, 7, 2, 1, 11, 7, 2},
    {1, 11, 1, 1, 0, 1, 8, 7, 2, 1, 11, 7, 2},
    {1, 11, 0, 0, 1, 0, 8, 8, 1, 0, 11, 8, 1},
    {1, 11, 1, 0, 1, 1, 8, 8, 1, 1, 11, 8, 1},
    {1, 11, 0, 1, 1, 1, 8, 7, 2, 1, 11, 7, 2},
    {1, 11, 1, 1, 1, 1, 8, 7, 2, 1, 11, 7, 2},
    {1, 11, 0, 0, 2, 0, 8, 8, 1, 0, 11, 8, 1},
    {1, 11, 1, 0, 2, 1, 8, 8, 1, 1, 11, 8, 1},
    {1, 11, 0, 1, 2, 1, 8, 7, 2, 1, 11, 7, 2},
    {1, 11, 1, 1, 2, 1, 8, 7, 2, 1, 11, 7, 2},
    {1, 11, 0, 0, 3, 0, 11, 1, 1, 0, 11, 8, 1},
    {1, 11, 1, 0, 3, 0, 11, 1, 1, 1, 11, 8, 1},
    {1, 11, 0, 1, 3, 1, 11, 7, 2, 1, 11, 7, 2},
    {1, 11, 1, 1, 3, 1, 11, 7, 2, 1, 11, 7, 2},
    {0, 13, 0, 0, 0, 0, 8, 8, 1, 0, 8, 14, 1},
    {0, 13, 1, 0, 0, 1, 8, 8, 1, 1, 8, 14, 1},
    {0, 13, 0, 1, 0, 1, 8, 7, 2, 0, 13, 14, 1},
    {0, 13, 1, 1, 0, 1, 8, 7, 2, 1, 8, 7, 2},
    {0, 13, 0, 0, 1, 0, 8, 8, 1, 0, 8, 14, 1},
    {0, 13, 1, 0, 1, 1, 8, 8, 1, 1, 8, 14, 1},
    {0, 13, 0, 1, 1, 1, 8, 7, 2, 0, 13, 14, 1},
    {0, 13, 1, 1, 1, 1, 8, 7, 2, 1, 8, 7, 2},
    {0, 13, 0, 0, 2, 0, 8, 8, 1, 0, 8, 14, 1},
    {0, 13, 1, 0, 2, 1, 8, 8, 1, 1, 8, 14, 1},
    {0, 13, 0, 1, 2, 1, 8, 7, 2, 0, 13, 14, 1},
    {0, 13, 1, 1, 2, 1, 8, 7, 2, 1, 8, 7, 2},
    {0, 13, 0, 0, 3, 0, 8, 14, 1, 0, 8, 14, 1},
    {0, 13, 1, 0, 3, 1, 8, 14, 1, 1, 8, 14, 1},
    {0, 13, 0, 1, 3, 0, 13, 14, 1, 0, 13, 14, 1},
    {0, 13, 1, 1, 3, 1, 8, 7, 2, 1, 8, 7, 2},
    {1, 13, 0, 0, 0, 0, 8, 8, 1, 0, 13, 8, 1},
    {1, 13, 1, 0, 0, 1, 8, 8, 1, 1, 13, 8, 1},
    {1, 13, 0, 1, 0, 1, 8, 7, 2, 1, 13, 14, 2},
    {1, 13, 1, 1, 0, 1, 8, 7, 2, 1, 13, 7, 2},
    {1, 13, 0, 0, 1, 0, 8, 8, 1, 0, 13, 8, 1},
    {1, 13, 1, 0, 1, 1, 8, 8, 1, 1, 13, 8, 1},
    {1, 13, 0, 1, 1, 1, 8, 7, 2, 1, 13, 14, 2},
    {1, 13, 1, 1, 1, 1, 8, 7, 2, 1, 13, 7, 2},
    {1, 13, 0, 0, 2, 0, 8, 8, 1, 0, 13, 14, 1},
    {1, 13, 1, 0, 2, 1, 8, 8, 1, 1, 13, 14, 1},
    {1, 13, 0, 1, 2, 1, 8, 7, 2, 1, 13, 14, 2},
    {1, 13, 1, 1, 2, 1, 8, 7, 2, 1, 13, 7, 2},
    {1, 13, 0, 0, 3, 0, 8, 14, 1, 0, 13, 14, 1},
    {1, 13, 1, 0, 3, 1, 8, 14, 1, 1, 13, 14, 1},
    {1, 13, 0, 1, 3, 0, 8, 14, 1, 1, 13, 14, 2},
    {1, 13, 1, 1, 3, 1, 13, 7, 2, 1, 13, 7, 2},
    {0, 14, 0, 0, 0, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 14, 1, 0, 0, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 14, 0, 1, 0, 1, 8, 7, 2, 0, 14, 11, 1},
    {0, 14, 1, 1, 0, 1, 8, 7, 2, 1, 14, 7, 2},
    {0, 14, 0, 0, 1, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 14, 1, 0, 1, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 14, 0, 1, 1, 1, 8, 7, 2, 0, 14, 11, 1},
    {0, 14, 1, 1, 1, 1, 8, 7, 2, 1, 14, 7, 2},
    {0, 14, 0, 0, 2, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 14, 1, 0, 2, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 14, 0, 1, 2, 1, 8, 7, 2, 0, 14, 11, 1},
    {0, 14, 1, 1, 2, 1, 8, 7, 2, 1, 14, 7, 2},
    {0, 14, 0, 0, 3, 0, 8, 1, 1, 0, 8, 8, 1},
    {0, 14, 1, 0, 3, 1, 8, 1, 1, 1, 8, 8, 1},
    {0, 14, 0, 1, 3, 0, 14, 11, 1, 0, 14, 11, 1},
    {0, 14, 1, 1, 3, 1, 14, 7, 2, 1, 14, 7, 2},
    {1, 14, 0, 0, 0, 0, 8, 11, 1, 0, 14, 11, 1},
    {1, 14, 1, 0, 0, 1, 8, 11, 1, 1, 14, 11, 1},
    {1, 14, 0, 1, 0, 1, 8, 7, 2, 1, 14, 7, 2},
    {1, 14, 1, 1, 0, 1, 8, 7, 2, 1, 14, 7, 2},
    {1, 14, 0, 0, 1, 0, 8, 11, 1, 0, 14, 11, 1},
    {1, 14, 1, 0, 1, 1, 8, 11, 1, 1, 14, 11, 1},
    {1, 14, 0, 1, 1, 1, 8, 7, 2, 1, 14, 7, 2},
    {1, 14, 1, 1, 1, 1, 8, 7, 2, 1, 14, 7, 2},
    {1, 14, 0, 0, 2, 0, 8, 8, 1, 0, 14, 8, 1},
    {1, 14, 1, 0, 2, 1, 8, 8, 1, 1, 14, 8, 1},
    {1, 14, 0, 1, 2, 1, 8, 7, 2, 1, 14, 7, 2},
    {1, 14, 1, 1, 2, 1, 8, 7, 2, 1, 14, 7, 2},
    {1, 14, 0, 0, 3, 0, 8, 1, 1, 0, 14, 1, 1},
    {1, 14, 1, 0, 3, 1, 8, 1, 1, 1, 14, 1, 1},
    {1, 14, 0, 1, 3, 1, 14, 7, 2, 1, 14, 7, 2},
    {1, 14, 1, 1, 3, 1, 14, 7, 2, 1, 14, 7, 2},
    {0, 15, 0, 0, 0, 0, 8, 8, 1, 0, 8, 11, 1},
    {0, 15, 1, 0, 0, 1, 8, 8, 1, 1, 8, 11, 1},
    {0, 15, 0, 1, 0, 1, 8, 7, 2, 0, 15, 11, 1},
    {0, 15, 1, 1, 0, 1, 8, 7, 2, 1, 15, 7, 2},
    {0, 15, 0, 0, 1, 0, 8, 8, 1, 0, 8, 11, 1},
    {0, 15, 1, 0, 1, 1, 8, 8, 1, 1, 8, 11, 1},
    {0, 15, 0, 1, 1, 1, 8, 7, 2, 0, 15, 11, 1},
    {0, 15, 1, 1, 1, 1, 8, 7, 2, 1, 15, 7, 2},
    {0, 15, 0, 0, 2, 0, 8, 8, 1, 0, 8, 8, 1},
    {0, 15, 1, 0, 2, 1, 8, 8, 1, 1, 8, 8, 1},
    {0, 15, 0, 1, 2, 1, 8, 7, 2, 0, 15, 11, 1},
    {0, 15, 1, 1, 2, 1, 8, 7, 2, 1, 15, 7, 2},
    {0, 15, 0, 0, 3, 1, 8, 7, 1, 0, 8, 11, 1},
    {0, 15, 1, 0, 3, 1, 8, 7, 2, 1, 8, 11, 1},
    {0, 15, 0, 1, 3, 1, 8, 7, 2, 0, 15, 11, 1},
    {0, 15, 1, 1, 3, 1, 8, 7, 2, 1, 15, 7, 2},
    {1, 15, 0, 0, 0, 0, 8, 8, 1, 0, 15, 11, 1},
    {1, 15, 1, 0, 0, 1, 8, 8, 1, 1, 15, 11, 1},
    {1, 15, 0, 1, 0, 1, 8, 7, 2, 1, 15, 7, 2},
    {1, 15, 1, 1, 0, 1, 8, 7, 2, 1, 15, 7, 2},
    {1, 15, 0, 0, 1, 0, 8, 8, 1, 0, 15, 11, 1},
    {1, 15, 1, 0, 1, 1, 8, 8, 1, 1, 15, 11, 1},
    {1, 15, 0, 1, 1, 1, 8, 7, 2, 1, 15, 7, 2},
    {1, 15, 1, 1, 1, 1, 8, 7, 2, 1, 15, 7, 2},
    {1, 15, 0, 0, 2, 0, 8, 8, 1, 0, 15, 8, 1},
    {1, 15, 1, 0, 2, 1, 8, 8, 1, 1, 15, 8, 1},
    {1, 15, 0, 1, 2, 1, 8, 7, 2, 1, 15, 7, 2},
    {1, 15, 1, 1, 2, 1, 8, 7, 2, 1, 15, 7, 2},
    {1, 15, 0, 0, 3, 1, 8, 7, 2, 0, 15, 11, 1},
    {1, 15, 1, 0, 3, 1, 8, 7, 2, 1, 15, 11, 1},
    {1, 15, 0, 1, 3, 1, 15, 7, 2, 1, 15, 7, 2},
    {1, 15, 1, 1, 3, 1, 15, 7, 2, 1, 15, 7, 2},
};
/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

FSTATIC int32_t     AVINT_SetFullScreenVideo(SYS_HDTV_STANDARD outHdSys,
                                             SYS_SDTV_STANDARD outSdSys);

FSTATIC void        AVINT_VideoParamChgCb    (VAL_VIDEC_DecFrameInfoStrT*
                                             nextFrameInfoPtr,
                                             VAL_VIDEC_DecFrameInfoStrT*
                                             lastFrameInfoPtr,
                                             void* dataPtr);

FSTATIC void        AVINT_VideoHDMIConnectCb(const VAL_HDMIConnectParamsT*
                                             paramsPtr, void* dataPtr);

#ifdef APPL_HDCP_ENABLED
FSTATIC void        AVINT_VideoHdmiHdcpStateCb(VAL_HdmiHdcpStateT state,
                                             void* dataPtr);
#endif /* APPL_HDCP_ENABLED */

FSTATIC void        AVINT_VideoSystemConvert (SYS_HDTV_STANDARD hdOutput,
                                             SYS_SDTV_STANDARD sdOutput,
                                             VAL_HdEncoderT* hdSys,
                                             VAL_SdEncoderT* sdSys);
FSTATIC void        AVINT_VideoScalingParamChgCb(FAPEX_VISCALE_ParamChangeInfoT*
                                              newParam,
                                              void* paramChangeOptData);
FSTATIC int8_t      AVINT_AudioFindByLang    (const SYS_STREAMINF_AUDIO_S*
                                              pStrAudio, uint8_t format,
                                              SYS_LANGCODE lang,
                                              fbool_t decodableOnly);
FSTATIC int32_t     AVINT_AudioConvertFormat (uint8_t format,
                                              AAL_AudioFormatT* pAalFormat);
FSTATIC int32_t     AVINT_IntHDMISetLipSyncDelay (void);

#ifdef APPL_DTG_SIPSI_ENABLED
FSTATIC void        AVINT_AudioSetStereo(uint8_t reqChannel);
#endif /* APPL_DTG_SIPSI_ENABLED */

#if (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED)

FSTATIC fbool_t     AVINT_SubtChooseStream   (AV_SUBTT_S * directPref,
                                             int8_t* index, fbool_t* isDvb);
//FSTATIC fbool_t     AVINT_SubtFindByLang     (SYS_LANGCODE lang,
//                                             fbool_t dvbFirst, int8_t* index,
//                                             fbool_t* isDvb);
FSTATIC fbool_t     AVINT_SubtCheckLangPrefIsMade  (void);

#endif /* (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED)*/

int32_t AV_Nop(void)
{
    AV_CMD_S            cmd;

    DBG_Assert(avData != NULL);

    cmd.type = AV_CMD_NOP;

    /* wakeup AV thread */
    return AV_CmdSend(&cmd);
}


int32_t AV_Wakeup(void)
{
    AV_CMD_S            cmd;
    SYS_VIDEO_OUT       videoOutput;
    int32_t             retVal;

    DBG_Assert(avData != NULL);

    if(!(avData->sleepMode))
        return FAPI_OK;

    cmd.type = AV_CMD_WAKEUP;

    /* wakeup AV thread */
    retVal = AV_CmdSend(&cmd);

    if(retVal == FAPI_OK)
    {
        avData->sleepMode = FFALSE;

        if(SETTINGS_VideoOutputGet(&videoOutput) == FAPI_OK)
        {
            /* set current video output */
            AV_SetVideoOutput(videoOutput);
        }
    }

    return retVal;
}

int32_t AV_Shutdown(void)
{
    AV_CMD_S    cmd;
    int32_t     retVal;

    DBG_Assert(avData != NULL);

    if(avData->sleepMode)
        return FAPI_OK;

    SCART_SetMode(SCART_MODE_OFF);
    AV_SetVideoOutput(VIDEOOUT_UNKNOWN);

    cmd.type = AV_CMD_GOTOSLEEP;

    /* put AV thread in sleep mode */
    retVal = AV_CmdSend(&cmd);
    if(retVal == FAPI_OK)
    {
        avData->sleepMode = FTRUE;
    }

    return retVal;
}

int32_t AV_SetTvOutputSystem(SYS_HDTV_STANDARD hdSystem,
                             SYS_SDTV_STANDARD sdSystem)
{
    AV_CMD_S cmd;

    if ( avData == NULL )
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    cmd.type                = AV_CMD_SET_OUTPUT_SYSTEM;
    cmd.arg.outsys.hdSystem = hdSystem;
    cmd.arg.outsys.sdSystem = sdSystem;

    return AV_CmdSend(&cmd);
}

void AV_SetVideoOutput(SYS_VIDEO_OUT videoOut)
{
    AV_VIDEO_S*         pVideo = NULL;
    #ifndef _LINUX_FAPI
    FAPI_SYS_HandleT    viencSdHdl;
    #else
    int32_t             viencSdHdl;
    #endif
    int32_t errcode = FAPI_OK;


    if(avData == NULL)
        return;

    pVideo = &(avData->video);

    if (pVideo == NULL)
        return;

    if (pVideo->valDataPtr == NULL)
        return;

    viencSdHdl = VAL_GetIdViencSd(pVideo->valDataPtr);

    if(avData->videoOutCurr != videoOut)
    {
        /* modify DAC settings */
        /* VIDEOOUT_NUMS  not needed here */
        switch (videoOut)
        {
            case VIDEOOUT_RGB:
                #ifndef _LINUX_FAPI
                (void)FAPI_VIENC_SetColourComponent(viencSdHdl,
                                                 FAPI_VIENC_COMPONENT_CVBS_YC);
                #else
                /* Martin added at 20100513
                             *
                             */
                errcode = FAPEX_VIENC_Ioctl(pVideo->valDataPtr->viencSubId[VAL_VOUT_SD],
                                        FAPEX_IOCCOM_VIENC_SET_CCOMPONENT,
                                        (FAPEXIOCARG3_T)FAPEX_VIENC_CCOMPONENT_CVBS_YC); /* replace FAPEX_VIENC_CCOMPONENT_CVBS_RGB with FAPEX_VIENC_CCOMPONENT_CVBS_YC, by Jim.Kang 2010.07.19 */
                if (errcode != FAPI_OK)
                {
                    FAPI_SYS_PRINT_MSG("AV_SetVideoOutput:");
                    FAPI_SYS_PRINT_MSG("set colour component failed:errcode %d\n",
                                       errcode);
                }
                #endif
                break;

            case VIDEOOUT_SVIDEO:
                #ifndef _LINUX_FAPI
                (void)FAPI_VIENC_SetColourComponent(viencSdHdl,
                                                  FAPI_VIENC_COMPONENT_CVBS_YC);
                #else
                 /* Martin added at 20100513
                             *
                             */
                errcode = FAPEX_VIENC_Ioctl(pVideo->valDataPtr->viencSubId[VAL_VOUT_SD],
                                        FAPEX_IOCCOM_VIENC_SET_CCOMPONENT,
                                        (FAPEXIOCARG3_T)FAPEX_VIENC_CCOMPONENT_CVBS_YC);
                if (errcode != FAPI_OK)
                {
                    FAPI_SYS_PRINT_MSG("AV_SetVideoOutput:");
                    FAPI_SYS_PRINT_MSG("set colour component failed:errcode 0x%x\n",
                                       errcode);
                }
                #endif
                break;
            default:
            case VIDEOOUT_UNKNOWN:
                /* do nothing */
                break;
        }

        /* modify SCART FBLK mode */
        SCART_SetVideoOutput(videoOut);

        avData->videoOutCurr = videoOut;
    }
}
/*! Enable or disable output format update (HDMI Infoframe, WSS, Scart Pin8)
*/
void AV_EnableOutputFormatUpdate(fbool_t enable)
{
    if ( avData == NULL )
    {
        return;
    }
    if(avData->outputFormatUpdateOn != enable)
    {
        avData->outputFormatUpdateOn = enable;
    }
}

/*!
** Update aspect ratio signaling (HDMI Infoframe, WSS, Scart Pin8) according to
** coded aspect ratio, coded AFD, display aspect ratio, scaling method, and HDMI
** format
*/
void AV_OutputFormatUpdate(fbool_t force)
{
    AV_AR_TABLE_S                   code;
    AV_AR_TABLE_S*                  pTab     = NULL;
    AV_AR_SIGNALS_S                 arSigs;
    uint32_t                        tabIdx   =0;
    int32_t                         retVal   = FAPI_OK;
    FAPEX_VISCALE_ParamChangeInfoT*  pVsPam   = &avData->video.viscaleParam;
    VAL_DataT*                      pValData = avData->video.valDataPtr;

#ifndef _LINUX_FAPI
    FAPI_SYS_HandleT                handleViscaleHd;
#else
    int32_t                         handleViscaleHd;
#endif
    SYS_HDTV_STANDARD               hdSystem;
    SYS_PICFORMAT_E                 dispAR = PICFORMAT_16_9;
    SYS_SCALEMETHOD_E               scaleMethod = SCALEMETHOD_PILLAR_LETTERBOX;

    #ifdef APPL_MHEG_ENABLED
    AV_WSS_E                        mhegWss;
    uint8_t                         mhegScart;
    #endif

    memset(&code, 0, sizeof(AV_AR_TABLE_S));
    memset(&arSigs, 0, sizeof(AV_AR_SIGNALS_S));

    /* Get aspect ratio parameters  */
    if ( avData == NULL )
    {
        return;
    }
    else if ( SETTINGS_DispFormatGet(&dispAR, &scaleMethod) !=FAPI_OK )
    {
        return;
    }
    else if ( SETTINGS_OutputTvStandardGet(&hdSystem,NULL) !=FAPI_OK )
    {
        return;
    }

    // 1. check coded frame aspect ratio
    switch(pVsPam->codedAspectRatio)
    {
        case FAPEX_VIOUT_DISPLAY_AR_4_3:
            code.codedAR = (uint32_t)PICFORMAT_4_3 ;
            break;
        default:
        case FAPEX_VIOUT_DISPLAY_AR_16_9:
            code.codedAR = (uint32_t)PICFORMAT_16_9;
            break;
    }

    // 2. check coded afd
    if (pVsPam->currAfd ==0)
    {
        code.afd = 8 ;
    }
    else
    {
        code.afd = (uint32_t)pVsPam->currAfd;
    }

    // 3. convert HD system to HDMI format
    if((hdSystem==HDSYS_576P)||(hdSystem==HDSYS_576I))
    {
        code.hdmiMode = ((uint32_t)PICFORMAT_4_3);
    }
    else//((hdmiFormat==HDSYS_720P)||(hdmiFormat==HDSYS_1080I))
    {
        code.hdmiMode = ((uint32_t)PICFORMAT_16_9);
    }

    // 4. check display aspect ratio
    code.dispAR = (uint32_t)(SYS_PICFORMAT_E)dispAR;

    // 5. check scaling method
    code.scaleMethod = (uint32_t)(SYS_SCALEMETHOD_E)scaleMethod;

    /* set default values for aspect ratio signals */
    arSigs.hdmiAR       = FAPEX_VIOUT_DISPLAY_AR_NONE;
    arSigs.hdmiAFD      = 8;
    arSigs.wss          = AV_WSS_DISABLE; // turn off
    arSigs.scartMode    = SCART_MODE_16_9;

    if(avData->outputFormatUpdateOn)
    {
        /* Get aspect ratio signals from look up table */
        for(tabIdx =0; tabIdx<224; tabIdx++)
        {
            pTab = &aspectTab[tabIdx];

            if((pTab->codedAR     == code.codedAR)   &&
               (pTab->afd         == code.afd)       &&
               (pTab->hdmiMode    == code.hdmiMode)  &&
               (pTab->dispAR      == code.dispAR)    &&
               (pTab->scaleMethod == code.scaleMethod) )
            {
                if(pTab->hdmiAR == 0)
                {
                    arSigs.hdmiAR   = FAPEX_VIOUT_DISPLAY_AR_4_3;
                }
                else
                {
                    arSigs.hdmiAR   = FAPEX_VIOUT_DISPLAY_AR_16_9;
                }
                arSigs.hdmiAFD      = (uint8_t)pTab->hdmiAFD;
                arSigs.wss          = (AV_WSS_E)pTab->wss;
                arSigs.scartMode    = (SCART_MODE_E)pTab->scartMode;

                if( (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT) ||
                    (SYS_GetAppProfile() == SYS_APP_PROFILE_UKDTT_T2) ||
                    (SYS_GetAppProfile() == SYS_APP_PROFILE_AUSTRALIA_FREEVIEW))
                {
                    FAPI_SYS_PRINT_DEBUG(2, "[avint] DTG_Profile enabled\n");
                    if(pTab->hdmiAR_UK == 0)
                    {
                        arSigs.hdmiAR   = FAPEX_VIOUT_DISPLAY_AR_4_3;
                    }
                    else
                    {
                        arSigs.hdmiAR   = FAPEX_VIOUT_DISPLAY_AR_16_9;
                    }
                    arSigs.hdmiAFD      = (uint8_t)pTab->hdmiAFD_UK;
                    arSigs.wss          = (AV_WSS_E)pTab->wss_UK;
                    arSigs.scartMode    = (SCART_MODE_E)pTab->scartMode_UK;
                }
                FAPI_SYS_PRINT_DEBUG(2,
                        "[avint] hdmi_R=%d, hdmi_M=%d, wss=%d, pin8=%d\n",
                        arSigs.hdmiAR, arSigs.hdmiAFD,
                        arSigs.wss, arSigs.scartMode);
                break; /* end of for(;;) loop for entry is found */
            }
        }

        #ifdef APPL_MHEG_ENABLED
        if(MHEG_DisplayGetWssScartSignal((FAPEX_VISCALE_WssStateEnumT *) &mhegWss,
                                      &mhegScart))
        {
            arSigs.wss       = mhegWss;
            arSigs.scartMode = (SCART_MODE_E)mhegScart;
        }
        #endif

        /* make HDMI setting*/
        avData->hdmiPictureAr = arSigs.hdmiAR;
        avData->hdmiActiveFormat = arSigs.hdmiAFD;

        retVal = VAL_HdmiUpdateAviInfoFrame(pValData, arSigs.hdmiAR,
                    arSigs.hdmiAFD);
        if (retVal != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(1,"VAL_HdmiUpdateAviInfoFrame failed!\n");
        }

        /* make WSS settings */
        if ( (avData->wssMode != (AV_WSS_E) arSigs.wss) || (force == FTRUE) )
        {
            handleViscaleHd = VAL_GetIdViscaleHd((VAL_DataT*)pValData);
        #ifndef _LINUX_FAPI
            retVal = FAPI_VISCALE_SetWssState(handleViscaleHd,
                                (FAPEX_VISCALE_WssStateEnumT)arSigs.wss);
        #else

            FAPEX_VISCALE_WssStateEnumT wss_state;
            switch (arSigs.wss)
            {
                case AV_WSS_4_3_FULLFORMAT:
                    wss_state = FAPEX_VISCALE_WSS_ENABLE_4_3_FULLFORMAT;
                    break;
                case AV_WSS_14_9_CENTRE:
                    wss_state = FAPEX_VISCALE_WSS_ENABLE_14_9_CENTRE;
                    break;

                case AV_WSS_16_9_CENTRE:
                    wss_state = FAPEX_VISCALE_WSS_ENABLE_16_9_BOX_CENTRE;
                    break;
                case AV_WSS_16_9_BOX_GT_CENTRE:
                    wss_state = FAPEX_VISCALE_WSS_ENABLE_16_9_BOX_GT_CENTRE;
                    break;
                case AV_WSS_14_9_FULLFORMAT:
                    wss_state = FAPEX_VISCALE_WSS_ENABLE_14_9_FULLFORMAT;
                    break;
                case AV_WSS_16_9_ANAMORPHIC:
                    wss_state = FAPEX_VISCALE_WSS_ENABLE_16_9_ANAMORPHIC;
                    break;
                case AV_WSS_DISABLE:
                    wss_state = FAPEX_VISCALE_WSS_DISABLE;
                    break;
                default:
                    FAPI_SYS_PRINT_DEBUG(1, "[av_int] Unknown wss_state, please check\n");
                    wss_state = FAPEX_VISCALE_UNKNOWN_WSS_STATE;
                    break;


            }
            retVal =  FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)handleViscaleHd,
                                  FAPEX_IOCCOM_VIOUT_SCALER_SET_WSSSTATE,
                                  (FAPEXIOCARG3_T)&wss_state);
        #endif
            if (retVal != FAPI_OK )
            {
                FAPI_SYS_PRINT_DEBUG(1,"FAPI_VISCALE_SetWssState failed!\n");
            }
            avData->wssMode = (AV_WSS_E) arSigs.wss;
        }

        /* make Scart settings */
        if ( (SCART_GetMode() != arSigs.scartMode) || (force == FTRUE) )
        {
            SCART_SetMode(arSigs.scartMode);
        }
    }
}


void AVINT_OutputSystemUpdate (SYS_HDTV_STANDARD reqHdSys,
                               SYS_SDTV_STANDARD reqSdSys,
                               fbool_t force)
{
    SYS_EVT_TVSYS_S     ctrlEvent;

    #ifndef _LINUX_FAPI
    FAPI_SYS_HandleT    handleViencSd;
    #else
    int32_t handleViencSd;
    #endif

    int32_t errcode = FAPI_OK;

    if ( avData == NULL )
    {
        return;
    }

    // FIXME: auto-mode handling not needed anymore ?

    if ( (reqHdSys != avData->video.outHdSysCurr) ||
         (reqSdSys != avData->video.outSdSysCurr) ||
         force )
    {
        if ( avData->video.videoFullScreen )
        {
            (void)AVINT_SetFullScreenVideo(reqHdSys,reqSdSys);
        }

        (void)AAL_EnableHdmi(FFALSE);

        if ( AVINT_VideoSetOutputMode(reqHdSys, reqSdSys) == FAPI_OK )
        {
            avData->wssMode = AV_WSS_UNKNOWN;
        #ifndef _LINUX_FAPI
            handleViencSd = VAL_GetHandleViencSd(avData->video.valDataPtr);

            (void)FAPI_VIENC_EnableDisableWss(handleViencSd,FAPI_VIENC_PARAMETER_ON);
        #else
            handleViencSd = VAL_GetIdViencHd(avData->video.valDataPtr);
            errcode = FAPEX_VIENC_Ioctl(handleViencSd,
                                        FAPEX_IOCCOM_VIENC_ENABLE_WSS, (FAPEXIOCARG3_T)NULL);
        #endif
            AV_OutputFormatUpdate(FFALSE); //update aspect ratio signaling

            /* Generate event to notify output TV standard change */
            ctrlEvent.hdr.type   = EVT_OUT_SYS_CHANGE;
            ctrlEvent.hdr.length = sizeof(ctrlEvent);
            ctrlEvent.hdTvSys    = reqHdSys;
            ctrlEvent.sdTvSys    = reqSdSys;

            (void)EVT_Send(&ctrlEvent, "TV Out Sys Change");
        }

        (void)AAL_EnableHdmi(FTRUE);
    }
}

int32_t AVINT_OutputTvStandardGet (VAL_HdEncoderT* hdSys,
                                   VAL_SdEncoderT* sdSys )
{
    AV_VIDEO_S* pVideo = &(avData->video);
    int32_t     retVal = FAPI_OK;

    if ( avData != NULL )
    {
        if ( pVideo->valDataPtr != NULL )
        {
            retVal = VAL_GetOutputMode (pVideo->valDataPtr,hdSys,sdSys);
        }
    }

    return retVal;
}
/* NOTE: Argument useDtgScallingFactors currently unused, left for legacy
         purposes !*/
/*lint -save -e715 */
int32_t AVINT_SetVideoArea (fbool_t fullScreen,
                            fbool_t useDtgScallingFactors,
                            const AV_VO_PICTURE_WINDOW_S * pos)
{
    int32_t             retVal = FAPI_OK;

    if ( fullScreen )
    {
        /* set full screen video according to current TV system. */
        retVal = AVINT_SetFullScreenVideo(avData->video.outHdSysCurr,
                                          avData->video.outSdSysCurr);

        avData->video.videoFullScreen = FTRUE;
    }
    else
    {
        /* Set video area as specified by user */
        if ( avData->video.valDataPtr != NULL )
        {
            retVal = VAL_Resize (avData->video.valDataPtr,
                                 pos->left, pos->top, pos->hsize, pos->vsize);
        }

        avData->video.videoFullScreen = FFALSE;
    }

    return retVal;
}
/*lint -restore */

/* NOTE: Argument useDtgScallingFactors currently unused, left for legacy
         purposes !*/
/*lint -save -e715 */
int32_t AVINT_ZoomVideo (fbool_t full,
                         fbool_t useDtgScallingFactors,
                         const AV_VO_PICTURE_WINDOW_S * pos)
{
    AV_VO_PICTURE_WINDOW_S cutoutPic;
    int32_t                retVal = FAPI_OK;

    if ( full )
    {
        cutoutPic.left  = 0;
        cutoutPic.top   = 0;
        cutoutPic.hsize = 0;
        cutoutPic.vsize = 0;

        pos = &cutoutPic;
    }

    /* Set video area as specified by user */
    if ( avData->video.valDataPtr != NULL )
    {
        retVal = VAL_Crop (avData->video.valDataPtr, pos->left, pos->top,
                           pos->hsize, pos->vsize);
    }

    return retVal;
}
/*lint -restore */

/* set full screen video according to given TV system. */
FSTATIC int32_t AVINT_SetFullScreenVideo(SYS_HDTV_STANDARD outHdSys,
                                         SYS_SDTV_STANDARD outSdSys)
{
    AV_VO_PICTURE_WINDOW_S  area;
    int32_t                 retVal = FAPI_OK;

    memset(&area,0,sizeof(AV_VO_PICTURE_WINDOW_S));

    switch ( outHdSys )
    {
        case HDSYS_576P:
        case HDSYS_576I:
            switch (outSdSys)
            {
                default:
                case SDSYS_PAL:
                case SDSYS_SECAM:
                    area.vsize = VIDEOSZ_Y_PAL;
                    break;

                case SDSYS_NTSC:
                    area.vsize = VIDEOSZ_Y_NTSC;
                    break;
            }
            area.hsize = VIDEOSZ_X_720;
            break;

        case HDSYS_720P:
            area.vsize = VIDEOSZ_Y_720;
            area.hsize = VIDEOSZ_X_1280;
            break;

        default:
        case HDSYS_1080I:
        case HDSYS_1080P:
            area.vsize = VIDEOSZ_Y_1080;
            area.hsize = VIDEOSZ_X_1920;
            break;
    }

    if ( avData->video.valDataPtr != NULL )
    {
        (void)VAL_Crop (avData->video.valDataPtr, 0, 0, 0, 0);

        retVal = VAL_Resize (avData->video.valDataPtr,
                             area.left, area.top, area.hsize, area.vsize);
    }

    return retVal;
}

void AV_ForceTuning(int8_t feIdx)
{
    if(avData == NULL)
        return;

    avData->feDat[feIdx].forceTuning = FTRUE;
}

void AV_ForceLnbSettingsUpdate(int8_t feIdx)
{
    if(avData == NULL)
        return;

    avData->feDat[feIdx].forceLnbSettsUpdate = FTRUE;
}

void AVINT_RouteSignal(const RSRC_CFG_S* cfg)
{
    /* set requested TS input for TSD-x */
    (void)AVINT_SetTsInput(cfg->tsdIdx, cfg->ts);

    #ifdef APPL_CI_ENABLED

    CISYS_SetInput(cfg);

    #endif /* APPL_CI_ENABLED */
}

#ifdef APPL_DISEQC12_ENABLED

int32_t AV_Dis12GotoSatPosition(int8_t feIdx, uint8_t satPos)
{
    int32_t             retVal;
    AV_CMD_S            cmd;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    if(avData->feDat[feIdx].drvHdl == NULL)
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    cmd.type = AV_CMD_DIS12_GOTOPOS;
    cmd.arg.dis12idx.feIdx = feIdx;
    cmd.arg.dis12idx.satPos = satPos;

    return AV_CmdSend(&cmd);
}

int32_t AV_Dis12StoreSatPosition(int8_t feIdx, uint8_t satPos)
{
    int32_t             retVal;
    AV_CMD_S            cmd;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    if(avData->feDat[feIdx].drvHdl == NULL)
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    cmd.type = AV_CMD_DIS12_STOREPOS;
    cmd.arg.dis12idx.feIdx = feIdx;
    cmd.arg.dis12idx.satPos = satPos;

    return AV_CmdSend(&cmd);
}

int32_t AV_Dis12DriveMotor(int8_t feIdx, FE_WE_FLAG_E westEastFlag,
                           uint8_t numSteps)
{
    int32_t             retVal;
    AV_CMD_S            cmd;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    if(avData->feDat[feIdx].drvHdl == NULL)
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    cmd.type = AV_CMD_DIS12_DRIVEMOTOR;
    cmd.arg.dis12move.feIdx = feIdx;
    cmd.arg.dis12move.westEastFlag = westEastFlag;
    cmd.arg.dis12move.numSteps = numSteps;

    return AV_CmdSend(&cmd);
}

int32_t AV_Dis12HaltMotor(int8_t feIdx)
{
    int32_t             retVal;
    AV_CMD_S            cmd;
    SYS_EVT_MOTOR_S  ctrlEvent;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    if(avData->feDat[feIdx].drvHdl == NULL)
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }
    ctrlEvent.hdr.type   = EVT_MOTOR_MOVES;
    ctrlEvent.hdr.length = sizeof(ctrlEvent);
    ctrlEvent.moving     = FFALSE;
    (void)EVT_Send(&ctrlEvent,"Move Motor abort");

    cmd.type = AV_CMD_DIS12_HALTMOTOR;
    cmd.arg.dis12move.feIdx = feIdx;

    return AV_CmdSend(&cmd);
}

int32_t AV_Dis12ReCalcPos(int8_t feIdx)
{
    int32_t             retVal;
    AV_CMD_S            cmd;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    if(avData->feDat[feIdx].drvHdl == NULL)
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    cmd.type = AV_CMD_DIS12_RECALCPOS;
    cmd.arg.dis12move.feIdx = feIdx;

    return AV_CmdSend(&cmd);
}

int32_t AV_Dis12SetLimitWest(int8_t feIdx)
{
    int32_t             retVal;
    AV_CMD_S            cmd;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    if(avData->feDat[feIdx].drvHdl == NULL)
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    cmd.type = AV_CMD_DIS12_SETLIMITWEST;
    cmd.arg.dis12move.feIdx = feIdx;

    return AV_CmdSend(&cmd);
}

int32_t AV_Dis12SetLimitEast(int8_t feIdx)
{
    int32_t             retVal;
    AV_CMD_S            cmd;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    if(avData->feDat[feIdx].drvHdl == NULL)
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    cmd.type = AV_CMD_DIS12_SETLIMITEAST;
    cmd.arg.dis12move.feIdx = feIdx;

    return AV_CmdSend(&cmd);
}

int32_t AV_Dis12ClearLimits(int8_t feIdx)
{
    int32_t             retVal;
    AV_CMD_S            cmd;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    if(avData->feDat[feIdx].drvHdl == NULL)
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    cmd.type = AV_CMD_DIS12_CLEARLIMITS;
    cmd.arg.dis12move.feIdx = feIdx;

    return AV_CmdSend(&cmd);
}

int32_t AV_Dis12GotoZeroPosition(int8_t feIdx)
{
    int32_t             retVal;
    AV_CMD_S            cmd;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    if(avData->feDat[feIdx].drvHdl == NULL)
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    cmd.type = AV_CMD_DIS12_GOTOZEROPOS;
    cmd.arg.dis12move.feIdx = feIdx;

    return AV_CmdSend(&cmd);
}

#endif /* APPL_DISEQC12_ENABLED */

#ifdef APPL_USALS_ENABLED

int32_t AV_UsalsGotoSatPosition(int8_t feIdx, const SYS_ORBPOS_S * orbPos)
{
    int32_t             retVal;
    AV_CMD_S            cmd;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
        return retVal;

    if(avData->feDat[feIdx].usalsHdl == NULL)
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    memset(&cmd, 0, sizeof(AV_CMD_S));
    cmd.type = AV_CMD_USALS_GOTOPOS;
    cmd.arg.usalspos.feIdx = feIdx;
    cmd.arg.usalspos.orbPos = *orbPos;

    return AV_CmdSend(&cmd);
}

int32_t AV_UsalsGotoZero(int8_t feIdx)
{
    int32_t             retVal;
    AV_CMD_S            cmd;

    if((retVal = AV_CheckArgs(feIdx)) != FAPI_OK)
    {
        return retVal;
    }

    if(avData->feDat[feIdx].usalsHdl == NULL)
    {
        return SMARTGO_SYS_ERR_PROHIBITED;
    }

    memset(&cmd, 0, sizeof(AV_CMD_S));
    cmd.type = AV_CMD_USALS_GOTOZERO;
    cmd.arg.usalspos.feIdx  = feIdx;

    return AV_CmdSend(&cmd);
}

#endif /* APPL_USALS_ENABLED */

void AVINT_ScartHandleOutputSource(void)
{
#ifndef _LINUX_FAPI
    uint64_t currTime = SYS_GET_TIMESTAMP_MS;
    FAPI_BOARD_ScartStatusT currScartStatus;

    /* Test for minimum polling interval */
    if ( UTILS_GetTimerStampInterval(avData->scartInputLastCheck, currTime)
         < AV_SCARTOUTCHECKINTERVAL_MIN )
    {
        return;
    }

    avData->scartInputLastCheck = currTime;

    /* satisfy MISRA */
    memset (&currScartStatus, 0, sizeof(currScartStatus));

    if(FAPI_BOARD_CONTROL_SCART (0, FAPI_BOARD_SCART_VCR_GET_STATUS,
                                 &currScartStatus) != FAPI_OK )
    {
        return;
    }

    if ( avData->scartInputStatus != currScartStatus.pin8Info )
    {
        avData->scartInputStatus = currScartStatus.pin8Info;

        FAPI_SYS_PRINT_DEBUG(4,"New VCR-Scart Status: %i\n", avData->scartInputStatus);

        switch ( avData->scartInputStatus )
        {
            case FAPI_BOARD_SCART_SB_OFF:
                SCART_SetOutputSource(FFALSE);
                break;

            default:
                SCART_SetOutputSource(FTRUE);
                break;
        }
    }
#endif
}

/* ------------------------------------------------------------------------- */

int32_t AVINT_VideoInit (void)
{
    AV_VIDEO_S*     pVideo = &(avData->video);
    VAL_InitParamsT initParams;

    memset (pVideo, 0, sizeof(AV_VIDEO_S));

    pVideo->outHdSysCurr     = HDSYS_1080I;
    pVideo->outSdSysCurr     = SDSYS_PAL;
    pVideo->picFormat        = PICFORMAT_NONE;
    pVideo->inSysCurr        = TVSYS_PAL;
    pVideo->videoFullScreen  = FTRUE;
    pVideo->bootScreenActive = FFALSE;
    pVideo->firstFrameWait   = FTRUE;
    pVideo->haveFrameRate    = FFALSE;
    pVideo->hdmiConnected    = FFALSE;

    #ifdef APPL_HDCP_ENABLED
    pVideo->hdmiHdcpState    = VAL_HDMI_HDCPSTATE_OFF;
    #endif

    avData->hdmiLipSyncSet   = FFALSE;

    memset (&initParams, 0, sizeof(initParams));

    initParams.threadPriority = THREAD_PRIO_HDMI;

    return VAL_Init (&initParams);
}

int32_t AVINT_VideoOpen ()
{
    AV_VIDEO_S*         pVideo = &(avData->video);
    VAL_UpParamsT       valParams;
    int32_t             retVal       = FAPI_OK;
    VAL_HdEncoderT      hdSys;
    VAL_SdEncoderT      sdSys;
    SYS_PICFORMAT_E     dispFormat   = PICFORMAT_16_9;
    SYS_SCALEMETHOD_E   scaleMethod  = SCALEMETHOD_PILLAR_LETTERBOX;
    int32_t             lipSyncDelay = 0;
    // TODO: set default values here!
    SYS_HDTV_STANDARD   hdOutput   = HDSYS_1080I;
    SYS_SDTV_STANDARD   sdOutput   = SDSYS_PAL;
#ifndef _LINUX_FAPI
    FAPI_SYS_HandleT    handleViscaleHd;
    FAPI_SYS_HandleT    handleViencSd;
#else
    int32_t           handleViscaleHd;
    int32_t           handleViencSd;
#endif

    if ( pVideo->valDataPtr != NULL )
    {
        return FAPI_ERR_ALREADY_OPEN;
    }

    FAPI_SYS_PRINT_DEBUG(1, "[av_int] enter AVINT_VideoOpen\n");

    (void)SETTINGS_OutputTvStandardGet (&hdOutput, &sdOutput);
    (void)SETTINGS_DispFormatGet       (&dispFormat, &scaleMethod);
    (void)SETTINGS_LipSyncDelayGet     (&lipSyncDelay);

    pVideo->outHdSysCurr  = hdOutput;
    pVideo->outSdSysCurr  = sdOutput;
    pVideo->hdmiConnected = FFALSE;

    avData->avLipSyncDelay = lipSyncDelay;

    /* trigger processing of picture format information at first GOP */
    pVideo->firstFrameWait = FTRUE;
    pVideo->picFormat      = PICFORMAT_NONE;

    AVINT_VideoSystemConvert (hdOutput, sdOutput, &hdSys, &sdSys);

    memset(&valParams, 0, sizeof(valParams));

    valParams.hdSystemType  = hdSys;
    valParams.sdSystemType  = sdSys;

    #ifdef APPL_HDCP_ENABLED
    valParams.enableHDCP    = (uint8_t)FTRUE;
    pVideo->hdmiHdcpEnabled = (fbool_t)valParams.enableHDCP;
    #else
    valParams.enableHDCP    = FFALSE;
    #endif /* APPL_HDCP_ENABLED */

    switch ( dispFormat )
    {
        default:
        case PICFORMAT_4_3:
            valParams.displayAspectRatio[VAL_VOUT_HD] = FAPEX_VIOUT_DISPLAY_AR_4_3;
            break;

        case PICFORMAT_16_9:
            valParams.displayAspectRatio[VAL_VOUT_HD] = FAPEX_VIOUT_DISPLAY_AR_16_9;
            break;
    }

    // SD frame should always cover HD active area (video + OSD) and therefore
    // aspect ratio of SD scaler (confusing, it's more like a kind of active
    // frame info) should set to NONE and follows the HD frame size
    valParams.displayAspectRatio[VAL_VOUT_SD] = FAPEX_VIOUT_DISPLAY_AR_NONE;

    valParams.scalingMethod = FAPEX_VISCALE_SM_PLBOX;

    switch ( scaleMethod )
    {
        default:
        case SCALEMETHOD_PILLAR_LETTERBOX:
            break;
        case SCALEMETHOD_FULLSCREEN:
            valParams.scalingMethod = FAPEX_VISCALE_SM_FSZOOM;
            break;
        case SCALEMETHOD_CCO:
            valParams.scalingMethod = FAPEX_VISCALE_SM_FSCCO;
            break;
        case SCALEMETHOD_TV_SCALES:
            valParams.scalingMethod = FAPEX_VISCALE_SM_LTVS;
            break;

        case SCALEMETHOD_NONE:
            valParams.scalingMethod = FAPEX_VISCALE_SM_NONE;
            break;
    }
    /*  viencSubId & viencId & vioudId will be opned in VAL_GoUp */
    pVideo->valDataPtr = VAL_GoUp(&valParams, &retVal);

    pVideo->valOpenPtr = NULL;
    if (NULL == pVideo->valDataPtr)
    {
        FAPI_SYS_PRINT_DEBUG(1, "[av_int] VAL_GoUp failed\n");
    }
    pVideo->freezeDecoder.system        = FAPEX_VIDEC_CODEC_TYPE_UNKNOWN;
    pVideo->freezeDecoder.frameBankSize = FAPEX_VIDEC_FRAME_BANK_SIZE_UNKNOWN;
    FAPI_SYS_PRINT_DEBUG(1, "[av_int] after VAL_GoUp\n");
    if ( pVideo->valDataPtr != NULL )
    {
        FAPEX_VISCALE_WssStateEnumT wss_state;

        (void)VAL_SetHDMIConnectionCb (pVideo->valDataPtr,
                                       AVINT_VideoHDMIConnectCb, pVideo);
        #ifdef APPL_HDCP_ENABLED
        (void)VAL_SetHdmiHdcpStateCb (pVideo->valDataPtr,
                                        AVINT_VideoHdmiHdcpStateCb, pVideo);
        #else
        (void)VAL_SetHdmiHdcpStateCb (pVideo->valDataPtr, NULL, NULL);
        #endif

        /* Aspect ratio handling initialization */
        handleViscaleHd = VAL_GetIdViscaleHd(pVideo->valDataPtr);

        #ifndef _LINUX_FAPI
        (void)FAPI_VISCALE_SetWssState (handleViscaleHd,
                                         FAPI_VISCALE_WSS_DISABLE);
        #else

        wss_state = FAPEX_VISCALE_WSS_DISABLE;
        retVal =  FAPEX_VIOUT_Ioctl( (FAPEXIOCARG1_T)handleViscaleHd,
                                      FAPEX_IOCCOM_VIOUT_SCALER_SET_WSSSTATE,
                                      (FAPEXIOCARG3_T)&wss_state );
        if (retVal != FAPI_OK )
        {
            FAPI_SYS_PRINT_DEBUG(1,"[av_int]:FAPEX_IOCCOM_VIOUT_SCALER_SET_WSSSTATE failed, errcode = %d\n", retVal);
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(1, "[av_int]:FAPEX_IOCCOM_VIOUT_SCALER_SET_WSSSTATE success, errcode = %d\n", retVal);
        }
        #endif


        handleViencSd = VAL_GetIdViencSd ((VAL_DataT*)pVideo->valDataPtr);

        #ifndef _LINUX_FAPI
        (void)FAPI_VIENC_EnableDisableWss (handleViencSd,
                                           FAPI_VIENC_PARAMETER_ON);
        #else
        retVal = FAPEX_VIENC_Ioctl((FAPEXIOCARG1_T)handleViencSd,
                          (FAPEXIOCARG2_T)FAPEX_IOCCOM_VIENC_ENABLE_WSS, (FAPEXIOCARG3_T)NULL);
        if (retVal != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1,"[AVINT_VideoOpen]: FAPEX_IOCCOM_VIENC_ENABLE_WSS failed, errcode = %d\n", retVal);
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(1, "[av_int]:FAPEX_IOCCOM_VIENC_ENABLE_WSS success, errcode = %d\n", retVal);
        }
        #endif

        retVal = VAL_SetScalingParamChangeCb (pVideo->valDataPtr,
                                              AVINT_VideoScalingParamChgCb,
                                              (void*)NULL);

        AV_EnableOutputFormatUpdate(FTRUE);
        /* end of Aspect ratio handling initializion */
    }
    FAPI_SYS_PRINT_DEBUG(1, "[av_int] quit AVINT_VideoOpen\n");
    return retVal;
}

int32_t AVINT_VideoClose (void)
{
    AV_VIDEO_S* pVideo = &(avData->video);
    int32_t     retVal;

    if ( pVideo->valDataPtr != NULL )
    {
        retVal = VAL_GoDown(pVideo->valDataPtr);

        pVideo->valOpenPtr       = NULL;
        pVideo->valDataPtr       = NULL;
        pVideo->bootScreenActive = FFALSE;
        pVideo->picFormat        = PICFORMAT_NONE;

        avData->hdmiLipSyncSet   = FFALSE;
    }
    else
    {
        retVal = FAPI_ERR_NOT_OPEN;
    }

    return retVal;
}

int32_t AVINT_VideoTask (void)
{
#ifndef _LINUX_FAPI

    AV_VIDEO_S* pVideo = &(avData->video);
    int32_t     retVal;
    FAPI_VIDEC_HealthStateEnumT health;

    if ( pVideo->valOpenPtr != NULL )
    {
        retVal = VAL_Task(pVideo->valOpenPtr);

        if ( (retVal == FAPI_OK) && !(avData->hdmiLipSyncSet) )
        {
            retVal = AVINT_IntHDMISetLipSyncDelay();
        }
        if( VAL_GetHealth (avData->video.valOpenPtr,&health) == FAPI_OK)
        {
            switch(health)
            {
                case FAPI_VIDEC_UNKNOWN_HEALTH:
                    FAPI_SYS_PRINT_DEBUG(3,"[AVINT_VideoTask] ViDec Health state: FAPI_VIDEC_UNKNOWN_HEALTH\n");
                    break;
                case FAPI_VIDEC_GOOD_HEALTH:
                    FAPI_SYS_PRINT_DEBUG(8,"[AVINT_VideoTask] ViDec Health state:FAPI_VIDEC_GOOD_HEALTH\n");
                    break;
                case FAPI_VIDEC_BAD_HEALTH_L0:
                   // FAPI_SYS_PRINT_DEBUG(3,"[AVINT_VideoTask] ViDec Health state:FAPI_VIDEC_BAD_HEALTH_L0\n");
                   // (void) VAL_Recover(avData->video.valOpenPtr);
                    break;
                case FAPI_VIDEC_BAD_HEALTH_L1:
                    FAPI_SYS_PRINT_DEBUG(3,"[AVINT_VideoTask] ViDec Health state:FAPI_VIDEC_BAD_HEALTH_L1\n");
                    (void) VAL_Recover(avData->video.valOpenPtr);
                    break;
                case FAPI_VIDEC_BAD_HEALTH_L2:
                    FAPI_SYS_PRINT_DEBUG(3,"[AVINT_VideoTask] ViDec Health state:FAPI_VIDEC_BAD_HEALTH_L2\n");
                    //(void) VAL_Recover(avData->video.valOpenPtr);
                    (void) VAL_Reboot(avData->video.valOpenPtr,&avData->video.valOpenPtr->decoder);
                    break;
                case FAPI_VIDEC_DEAD:
                    FAPI_SYS_PRINT_DEBUG(3,"[AVINT_VideoTask] ViDec Health state:FAPI_VIDEC_DEAD\n");
                    (void) VAL_Reboot(avData->video.valOpenPtr,&avData->video.valOpenPtr->decoder);
                    break;
            }
        }
    }
    else
    {
        retVal = FAPI_ERR_NOT_OPEN;
    }

    return retVal;
#else

    /* Martin added at 20100513*/

    AV_VIDEO_S* pVideo = &(avData->video);
    int32_t     retVal;
    FAPEX_VIDEC_StatusStrT videc_status;

    if ( pVideo->valOpenPtr != NULL )
    {
        retVal = VAL_Task(pVideo->valOpenPtr);

        if ( (retVal == FAPI_OK) && !(avData->hdmiLipSyncSet) )
        {
            retVal = AVINT_IntHDMISetLipSyncDelay();
        }

        memset(&videc_status, 0, sizeof(FAPEX_VIDEC_StatusStrT));
        retVal = FAPEX_VIDEC_Ioctl(pVideo->valOpenPtr->videcId,
                                   FAPEX_IOCCOM_VIDEC_GET_STATUS,
                                   (FAPEXIOCARG3_T)(&videc_status));
        if (retVal == FAPI_OK)
        {
            switch (videc_status.healthState)
            {
                case FAPEX_VIDEC_HEALTH_UNKNOWN:
                    FAPI_SYS_PRINT_DEBUG(3,"[AVINT_VideoTask] ViDec Health state: FAPEX_VIDEC_HEALTH_UNKNOWN\n");
                    break;

                case FAPEX_VIDEC_HEALTH_GOOD:
                    FAPI_SYS_PRINT_DEBUG(8,"[AVINT_VideoTask] ViDec Health state:FAPEX_VIDEC_HEALTH_GOOD\n");
                    break;

                case FAPEX_VIDEC_HEALTH_BAD_L0:
                    break;

                case FAPEX_VIDEC_HEALTH_BAD_L1:
                    FAPI_SYS_PRINT_DEBUG(3,"[AVINT_VideoTask] ViDec Health state:FAPEX_VIDEC_HEALTH_BAD_L1\n");
                    (void) VAL_Recover(avData->video.valOpenPtr);
                    break;

                case FAPEX_VIDEC_HEALTH_BAD_L2:
                    FAPI_SYS_PRINT_DEBUG(3,"[AVINT_VideoTask] ViDec Health state:FAPEX_VIDEC_HEALTH_BAD_L2\n");
                    //(void) VAL_Recover(avData->video.valOpenPtr);
                    (void) VAL_Reboot(avData->video.valOpenPtr,&avData->video.valOpenPtr->decoder);
                    break;

                case FAPEX_VIDEC_HEALTH_DEAD:
                    FAPI_SYS_PRINT_DEBUG(3,"[AVINT_VideoTask] ViDec Health state:FAPEX_VIDEC_HEALTH_DEAD\n");
                    (void) VAL_Reboot(avData->video.valOpenPtr,&avData->video.valOpenPtr->decoder);
                    break;

                default:
                    FAPI_SYS_PRINT_MSG("Undefined healthState(%d), please check\n", videc_status.healthState);
                    break;

            }
        }
        else
        {
            FAPI_SYS_PRINT_ERR("Error get videc status string, errcode (%d)\n", retVal);
        }
    }
    else
    {
        retVal = FAPI_ERR_NOT_OPEN;
    }

    return retVal;

    /* Martin added end*/

#endif

}

int32_t AVINT_VideoSetOutputMode (SYS_HDTV_STANDARD hdOutput,
                                  SYS_SDTV_STANDARD sdOutput)
{
    AV_VIDEO_S*             pVideo = &(avData->video);
    int32_t                 retVal;
    VAL_HdEncoderT          hdSys;
    VAL_SdEncoderT          sdSys;
    const SYS_CONFIGDAT_S*  sysCfg;

    if ( pVideo->valDataPtr != NULL )
    {
        AVINT_VideoSystemConvert (hdOutput, sdOutput, &hdSys, &sdSys);

        retVal = VAL_SetOutputMode(pVideo->valDataPtr, hdSys, sdSys);

        if ( retVal == FAPI_OK )
        {
            pVideo->outHdSysCurr = hdOutput;
            pVideo->outSdSysCurr = sdOutput;

            /* restart radio screen if it was visible before */
            if ( avData->liveTv.radioScreenVisible)
            {
                sysCfg = SYS_GetConfig();
                if( ( sysCfg != NULL ) &&
                    ( sysCfg->radioScreen != NULL) )
                {
                    (void)AVINT_VideoStop(FFALSE);
                    (void)AVINT_VideoIFrameShow (sysCfg->radioScreen,
                                                 sysCfg->radioScreenSize,
                                                 STRTYPE_VIDEO_MPEG2);
                }
            }
        }
        #ifdef APPL_MHEG_ENABLED
        MHEG_DisplayHasChanged();
        #endif
    }
    else
    {
        retVal = FAPI_ERR_NOT_OPEN;
    }

    return retVal;
}

int32_t AVINT_VideoStart (VAL_EnvironmentT environment, uint16_t pid,
                          uint8_t streamType, VAL_PlayModeEnumT playMode,
                          void (*videoStartCb)(void* optData),
                          void* videoStartCbArg)
{
#ifndef _LINUX_FAPI

    AV_VIDEO_S*         pVideo = &(avData->video);
    int32_t             retVal = FAPI_OK;
    VAL_DecoderTypeT    decoder;
    VAL_OpenParamsT     vidOpenParams;

    if ( pVideo->valOpenPtr == NULL )
    {
        switch ( streamType )
        {
            default:
            case STRTYPE_VIDEO_MPEG1:
            case STRTYPE_VIDEO_MPEG2:
                decoder.frameBankSize = FAPI_VIDEC_FRAME_BANK_SIZE_1920HD;
                decoder.system        = FAPI_VIDEC_MPEG2_MPHL;
                break;

            case STRTYPE_VIDEO_H264:
                /* NOTE: Workaround for timeshift: attempt to start in video format
                         that is already known from previous live presentation. */
                if ( (pVideo->freezeDecoder.system == FAPI_VIDEC_H264_HP41)
                  && (pVideo->freezeDecoder.frameBankSize !=
                      FAPI_VIDEC_UNKNOWN_FRAME_BANK_SIZE) )
                {
                    decoder.frameBankSize = pVideo->freezeDecoder.frameBankSize;
                }
                else
                {
                    /* set default frame bank size if unknown */
                    decoder.frameBankSize = FAPI_VIDEC_FRAME_BANK_SIZE_1920HD;
                }

                decoder.system = FAPI_VIDEC_H264_HP41;
                break;
        }

        memset(&vidOpenParams, 0, sizeof(vidOpenParams));

        vidOpenParams.environment   = environment;
        vidOpenParams.decoder       = decoder;
        #ifdef APPL_MHEG_ENABLED
        vidOpenParams.enableAfd     = MHEG_DisplayGetAfd();
        #else
        vidOpenParams.enableAfd     = FTRUE;
        #endif

        FAPI_SYS_PRINT_DEBUG(5,"vidOpenParams.enableAfd = %d\n",vidOpenParams.enableAfd);
        vidOpenParams.h264AudType   = VAL_H264_AUD_TYPE_AUTO;

        pVideo->valOpenPtr = VAL_Open (pVideo->valDataPtr, &vidOpenParams,
                                       &retVal);

        if ( pVideo->valOpenPtr != NULL )
        {
            retVal = VAL_SetParamChangeCb (pVideo->valOpenPtr,
                                           AVINT_VideoParamChgCb, pVideo);

            if ( retVal == FAPI_OK )
            {
                retVal = VAL_SetFrameAvailableCb (pVideo->valOpenPtr,
                                                  videoStartCb,
                                                  videoStartCbArg);

                if ( retVal == FAPI_OK )
                {
                    retVal = VAL_Start (pVideo->valOpenPtr, pid, playMode);
                }
            }

            if ( retVal != FAPI_OK )
            {
                (void)VAL_Close(pVideo->valOpenPtr);

                pVideo->valOpenPtr = NULL;
            }
            else
            {
                pVideo->currPid = pid;
            }
        }
    }

    return retVal;
#else

    /* Martin added at 20100513*/

    AV_VIDEO_S*         pVideo = &(avData->video);
    int32_t             retVal = FAPI_OK;
    VAL_DecoderTypeT    decoder;
    VAL_OpenParamsT     vidOpenParams;


    FAPI_SYS_PRINT_DEBUG(1, "[av_int][AVINT_VideoStart] enter\n");
    if ( pVideo->valOpenPtr == NULL )
    {
        switch ( streamType )
        {
            default:
            case STRTYPE_VIDEO_MPEG1:
            case STRTYPE_VIDEO_MPEG2:
                FAPI_SYS_PRINT_DEBUG(1, "[av_int][AVINT_VideoStart] streamType = %d\n", streamType);
                decoder.frameBankSize = FAPEX_VIDEC_FRAME_BANK_SIZE_1920HD;
                decoder.system        = FAPEX_VIDEC_CODEC_TYPE_MPEG2;
                break;

            case STRTYPE_VIDEO_H264:
                /* NOTE: Workaround for timeshift: attempt to start in video format
                         that is already known from previous live presentation. */
                if ( (pVideo->freezeDecoder.system == FAPEX_VIDEC_CODEC_TYPE_H264)
                  && (pVideo->freezeDecoder.frameBankSize !=
                      FAPEX_VIDEC_FRAME_BANK_SIZE_UNKNOWN) )
                {
                    decoder.frameBankSize = pVideo->freezeDecoder.frameBankSize;
                }
                else
                {
                    /* set default frame bank size if unknown */
                    decoder.frameBankSize = FAPEX_VIDEC_FRAME_BANK_SIZE_1920HD;
                }

                decoder.system = FAPEX_VIDEC_CODEC_TYPE_H264;
                break;
        }

        memset(&vidOpenParams, 0, sizeof(vidOpenParams));

        vidOpenParams.environment   = environment;
        vidOpenParams.decoder       = decoder;
        #ifdef APPL_MHEG_ENABLED
        vidOpenParams.enableAfd     = MHEG_DisplayGetAfd();
        #else
        vidOpenParams.enableAfd     = FTRUE;
        #endif

        FAPI_SYS_PRINT_DEBUG(1,"vidOpenParams.enableAfd = %d\n",vidOpenParams.enableAfd);
        vidOpenParams.h264AudType   = VAL_H264_AUD_TYPE_AUTO;

        pVideo->valOpenPtr = VAL_Open (pVideo->valDataPtr, &vidOpenParams,
                                       &retVal);

        if ( pVideo->valOpenPtr != NULL )
        {
            retVal = VAL_SetParamChangeCb (pVideo->valOpenPtr,
                                           AVINT_VideoParamChgCb, (void*)pVideo);

            if ( retVal == FAPI_OK )
            {
                retVal = VAL_SetFrameAvailableCb (pVideo->valOpenPtr,
                                                  videoStartCb,
                                                  videoStartCbArg);

                if ( retVal == FAPI_OK )
                {
                    /* Martin commented at 20100513*/
                    /* set the 4th parameter to 0 means no PCR detected */
                    /* The 4th parameter will be upadted later */
                    retVal = VAL_Start (pVideo->valOpenPtr, pid, playMode, 0);
                }
            }

            if ( retVal != FAPI_OK )
            {
                (void)VAL_Close(pVideo->valOpenPtr);

                pVideo->valOpenPtr = NULL;
            }
            else
            {
                pVideo->currPid = pid;
            }
        }
    }

    return retVal;

    /* Martin added end*/

#endif
}

int32_t AVINT_VideoStop (fbool_t freezeVideo)
{

    AV_VIDEO_S* pVideo = &(avData->video);
    int32_t     retVal = FAPI_OK;
    VAL_StatusT status;

    if ( pVideo->valOpenPtr != NULL )
    {
        if ( freezeVideo
          && (VAL_GetStatus (pVideo->valOpenPtr, &status) == FAPI_OK) )
        {
            pVideo->freezeDecoder = status.decoder;
        }
        else
        {

            #ifndef _LINUX_FAPI
            pVideo->freezeDecoder.system        = FAPI_VIDEC_UNKNOWN_DECODER;
            pVideo->freezeDecoder.frameBankSize =
                                             FAPI_VIDEC_UNKNOWN_FRAME_BANK_SIZE;
            #else
            pVideo->freezeDecoder.system        = FAPEX_VIDEC_CODEC_TYPE_UNKNOWN;
            pVideo->freezeDecoder.frameBankSize =
                                 FAPEX_VIDEC_FRAME_BANK_SIZE_UNKNOWN;
            #endif
        }

        (void)VAL_Stop (pVideo->valOpenPtr, freezeVideo);
        (void)VAL_Close(pVideo->valOpenPtr);

        pVideo->valOpenPtr       = NULL;
        pVideo->bootScreenActive = FFALSE;
        pVideo->haveFrameRate    = FFALSE;
        pVideo->currPid = 0;

        avData->hdmiLipSyncSet   = FFALSE;
    }

    return retVal;

}

int32_t AVINT_VideoBlank (fbool_t blank)
{
    AV_VIDEO_S* pVideo = &(avData->video);
    int32_t     retVal = FAPI_OK;

    if ( avData != NULL )
    {
        if ( pVideo->valDataPtr != NULL )
        {
            retVal = VAL_Blank(pVideo->valDataPtr, blank);
        }
    }

    return retVal;
}

int32_t AVINT_VideoFreeze (fbool_t freeze)
{
    AV_VIDEO_S* pVideo = &(avData->video);
    int32_t     retVal = FAPI_OK;

    FAPI_SYS_PRINT_DEBUG(2,"[AV] AVINT_VideoFreeze(%s)\n",(freeze)?"FTRUE":"FFALSE");

    if ( pVideo->valOpenPtr != NULL )
    {
        retVal = VAL_Freeze (pVideo->valOpenPtr, freeze);
    #ifndef _LINUX_FAPI
        if(retVal != FAPI_OK)
        {
            FAPI_SYS_PRINT_DEBUG(1,"[AV] VAL_Freeze() : %s\n",SYS_GetErrorString(retVal));
        }
    #endif
    }

    return retVal;
}


int32_t AVINT_VideoIFrameShow (const uint32_t* data,
                               uint32_t sizeInBytes,
                               uint32_t format)
{

    AV_VIDEO_S*     pVideo = &(avData->video);
    int32_t         retVal = FAPI_OK;
    VAL_OpenParamsT vidOpenParams;

    if ( pVideo->valDataPtr != NULL )
    {
        if ( pVideo->valOpenPtr == NULL )
        {
            memset(&vidOpenParams, 0, sizeof(vidOpenParams));

            vidOpenParams.environment           = VAL_ENV_LIVE;

            /* Martin modified at 20100513 */

            #ifndef _LINUX_FAPI
            if( format == STRTYPE_VIDEO_H264)
            {
                vidOpenParams.decoder.system        = FAPI_VIDEC_H264_HP41;
            }
            else
            {
                vidOpenParams.decoder.system        = FAPI_VIDEC_MPEG2_MPHL;

            }

            vidOpenParams.decoder.frameBankSize = FAPI_VIDEC_FRAME_BANK_SIZE_1920HD;
            #else
            if( format == STRTYPE_VIDEO_H264)
            {
                vidOpenParams.decoder.system        = FAPEX_VIDEC_CODEC_TYPE_H264;
            }
            else
            {
                vidOpenParams.decoder.system        = FAPEX_VIDEC_CODEC_TYPE_MPEG2;
            }

            vidOpenParams.decoder.frameBankSize = FAPEX_VIDEC_FRAME_BANK_SIZE_1920HD;
            #endif

            /* Martin modified end*/

            vidOpenParams.enableAfd             = FFALSE;
            vidOpenParams.h264AudType           = VAL_H264_AUD_TYPE_AUD;

            pVideo->valOpenPtr = VAL_Open (pVideo->valDataPtr, &vidOpenParams,
                                           &retVal);

            if ( pVideo->valOpenPtr != NULL )
            {
                retVal = VAL_SetParamChangeCb (pVideo->valOpenPtr,
                                           AVINT_VideoParamChgCb, (void*)pVideo);

                if ( retVal == FAPI_OK )
                {
                    retVal = VAL_IFrameDecode (pVideo->valOpenPtr, data,
                                           sizeInBytes);
                    if ( retVal == FAPI_OK )
                    {
                        retVal = VAL_IFrameDisplay (pVideo->valOpenPtr);
                    }
                }

                if ( retVal != FAPI_OK )
                {
                    (void)VAL_Close(pVideo->valOpenPtr);

                    pVideo->valOpenPtr = NULL;
                }
            }
        }
        pVideo->currPid = 0;
    }
    else
    {
        retVal = FAPI_ERR_NOT_OPEN;
    }

    return retVal;

}

int32_t AVINT_VideoSetAspectRatio (VAL_VideoOutputT output,
                                   FAPEX_VISCALE_AspectRatioEnumT
                                   aspectRatio)
{
    AV_VIDEO_S* pVideo = &(avData->video);
    int32_t     retVal = FAPI_OK;

    if ( avData != NULL )
    {
        if ( pVideo->valDataPtr != NULL )
        {
            retVal = VAL_SetDisplayAspectRatio (pVideo->valDataPtr,
                                                output, aspectRatio);
        }
    }

    // TODO Check if correct
    #ifdef APPL_MHEG_ENABLED
    if(retVal == FAPI_OK)
    {
        MHEG_DisplayDispAspectChanged();
    }
    #endif

    return retVal;
}

int32_t AVINT_VideoGetAspectRatio (VAL_VideoOutputT output,
                                   FAPEX_VISCALE_AspectRatioEnumT* aspectRatio)
{
    AV_VIDEO_S* pVideo = &(avData->video);
    int32_t     retVal = FAPI_OK;

    if ( avData != NULL )
    {
        if ( pVideo->valDataPtr != NULL )
        {
            retVal = VAL_GetDisplayAspectRatio (pVideo->valDataPtr,
                                                output, (int32_t*)aspectRatio);
        }
    }

    return retVal;
}


int32_t AVINT_VideoSetScaleMethod(FAPEX_VISCALE_ScalingMethodEnumT newScaleMethod)
{
    AV_VIDEO_S* pVideo = &(avData->video);
    int32_t     retVal = FAPI_OK;

    if ( avData != NULL )
    {
        if ( pVideo->valDataPtr != NULL )
        {
            retVal = VAL_SetHdScalingMethod(pVideo->valDataPtr,newScaleMethod);
        }
    }

    return retVal;
}

int32_t AVINT_VideoGetScaleMethod(FAPEX_VISCALE_ScalingMethodEnumT* scaleMethod)
{
    AV_VIDEO_S* pVideo = &(avData->video);
    int32_t     retVal = FAPI_OK;

    if ( avData != NULL )
    {
        if ( pVideo->valDataPtr != NULL )
        {
            retVal = VAL_GetHdScalingMethod(pVideo->valDataPtr,(int32_t*)scaleMethod);
        }
    }
    return retVal;
}
int32_t AVINT_VideoGetStreamInfo (AV_VIDEO_STREAMINFO_S* info)
{
#ifndef _LINUX_FAPI

    AV_VIDEO_S*         pVideo = &(avData->video);
    FAPI_SYS_HandleT    handle;

    if ( info != NULL )
    {
        memset (info, 0, sizeof(AV_VIDEO_STREAMINFO_S));

        if ( pVideo->valOpenPtr != NULL )
        {
            handle = VAL_GetHandleVidec (pVideo->valOpenPtr);

            if ( handle != NULL )
            {
                (void)FAPI_VIDEC_GetVesParams(handle, &(info->stream),
                                              FAPI_SYS_NO_SUSPEND);
                (void)FAPI_VIDEC_GetState(handle, &(info->decoder));
            }
        }
    }

    return FAPI_OK;
#else
    /* Martin added at 20100512 */
    AV_VIDEO_S *pVideo = &(avData->video);
    int32_t err_code = FAPI_OK;

    if (info != NULL)
    {
        memset (info, 0, sizeof(AV_VIDEO_STREAMINFO_S));

        if (pVideo->valOpenPtr != NULL)
        {
            if (pVideo->valOpenPtr->videcId < 0)
            {
                return SMARTGO_SYS_ERR_INTERNAL;
            }
            err_code = FAPEX_VIDEC_Ioctl(pVideo->valOpenPtr->videcId,
                                        FAPEX_IOCCOM_VIDEC_GET_STATUS,
                                        (FAPEXIOCARG3_T)&(info->decoder));
            if (err_code != FAPI_OK)
            {
                FAPI_SYS_PRINT_DEBUG(1, "[AVINT_VideoGetStreamInfo]:Get decoder status failed, errcode(0x%x)\n", err_code);
                return SMARTGO_SYS_ERR_INTERNAL;
            }
        }
    }
    return err_code;

#endif
}

#ifndef _LINUX_FAPI
int32_t AVINT_VideoGetFrameInfo (FAPI_VIDEC_VideoDecInfoStateT* info)
{
    AV_VIDEO_S* pVideo = &(avData->video);
    int32_t     retVal = FAPI_OK;

    if ( info != NULL )
    {
        if ( pVideo->valOpenPtr != NULL )
        {
            retVal = VAL_GetVideoFrameInfo (pVideo->valOpenPtr, info);
        }
        else
        {
            retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
        }
    }

    return retVal;
}
#else
/* Martin added at 20100512 */
int32_t AVINT_VideoGetFrameInfo (VAL_VIDEC_DecFrameInfoStrT *info)
{
    AV_VIDEO_S* pVideo = &(avData->video);
    int32_t     retVal = FAPI_OK;

    if ( info != NULL )
    {
        if ( pVideo->valOpenPtr != NULL )
        {
            retVal = VAL_GetVideoFrameInfo (pVideo->valOpenPtr, info);
        }
        else
        {
            retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
        }
    }

    return retVal;

}
#endif

/*lint -save -e715 */
FSTATIC void AVINT_VideoParamChgCb (VAL_VIDEC_DecFrameInfoStrT*
                                    nextFrameInfoPtr,
                                    VAL_VIDEC_DecFrameInfoStrT*
                                    lastFrameInfoPtr,
                                    void* dataPtr)
{
    AV_VIDEO_S*         pVideo = (AV_VIDEO_S*)dataPtr;
    SYS_PICFORMAT_E     newPicFormat;
    SYS_EVT_PICFORMAT_S ctrlEvent;

    /* decode aspect ratio field */
    switch ( nextFrameInfoPtr->aspectRatio )
    {
        case FAPEX_VIDEC_AR_4_3:
            newPicFormat = PICFORMAT_4_3;
            break;

        default:
        case FAPEX_VIDEC_AR_16_9:
            // TODO: detect also other formats (14:9, 2.21:1, 2.35:1)
            newPicFormat = PICFORMAT_16_9;
            break;

        case FAPEX_VIDEC_AR_UNKNOWN:
            newPicFormat = PICFORMAT_NONE;
            break;
    }

    if ( (newPicFormat != pVideo->picFormat) ||
         (pVideo->firstFrameWait) ||
         (lastFrameInfoPtr->decodedHeight!=nextFrameInfoPtr->decodedHeight)||
         (lastFrameInfoPtr->decodedWidth!=nextFrameInfoPtr->decodedWidth))
    {
        pVideo->picFormat = newPicFormat;

        ctrlEvent.hdr.type   = EVT_PIC_FORMAT_CHANGE;
        ctrlEvent.hdr.length = sizeof(ctrlEvent);
        ctrlEvent.format     = pVideo->picFormat;

        (void)EVT_Send(&ctrlEvent, "Pic format change");
    }

    pVideo->firstFrameWait = FFALSE;

    // TODO: Video format detection to be ported!
    #if 0

    SYS_HDTV_STANDARD          newHdTvSys = HDSYS_1080I;
    SYS_SDTV_STANDARD          newSdTvSys = SDSYS_PAL;
    SYS_EVT_TVSYS_S            tvSysEvent;

    /* ----- Video format detection ---------------------------------------- */

    /* NOTE: detection by video_format field is not 100% reliable, that is
             why it has been replaced by a detection method based on the
             vertical picture size */
    switch ( seqInfo.verSize )
    {
        default:
        case  288:
        case  576:  newSdTvSys = SDSYS_PAL;   break;
        case  240:
        case  480:  newSdTvSys = SDSYS_NTSC;  break;
        case  720:  newHdTvSys = HDSYS_720P;  break;
        case 1080:  newHdTvSys = HDSYS_1080i; break;
    }

    if ( newTvSys != pVideo->inSysCurr )
    {
        pVideo->inSysCurr = newTvSys;

        tvSysEvent.hdr.type   = EVT_IN_SYS_CHANGE;
        tvSysEvent.hdr.length = sizeof(tvSysEvent);
        tvSysEvent.hdSys      = newHdTvSys;
        tvSysEvent.sdSys      = newSdTvSys;

        (void)EVT_Send(&tvSysEvent, "TV In Sys Change");
    }

    #endif
}
/*lint -restore */

FSTATIC void AVINT_VideoScalingParamChgCb (FAPEX_VISCALE_ParamChangeInfoT*
                                           newParam, void* paramChangeOptData)
{
    FAPEX_VISCALE_ParamChangeInfoT*  currParam = &(avData->video.viscaleParam);
    SYS_EVTHDR_S                    ctrlEvent;

    paramChangeOptData = paramChangeOptData;

    if ((currParam->codedAspectRatio != newParam->codedAspectRatio)||
       (currParam->currAfd != newParam->currAfd)||
       (currParam->currWss != newParam->currWss))
    {
        *currParam = *newParam;
        ctrlEvent.type   = EVT_AFD_WSS_CHANGE_NOTIFY;
        ctrlEvent.length = sizeof(ctrlEvent);

         (void)EVT_Send(&ctrlEvent, "AFD Format Change");
    }
}

FSTATIC void AVINT_VideoHDMIConnectCb (const VAL_HDMIConnectParamsT* paramsPtr,
                                       void* dataPtr)
{
    SYS_EVT_HDMICONNECT_S   ctrlEvent;
    AV_VIDEO_S*             pVideo           = (AV_VIDEO_S*)dataPtr;
    fbool_t                 newConnectStatus = (paramsPtr->connected == 0) ?
                                               FFALSE : FTRUE;

    if ( pVideo->hdmiConnected != newConnectStatus )
    {
        if ( newConnectStatus )
        {
            /* HDMI has been connected, trigger restarting of audio. */

            FAPI_SYS_PRINT_DEBUG(1,"[AV] HDMI connected!\n");

            (void)AAL_EnableHdmi(0);
            (void)AAL_EnableHdmi(1);
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(1,"[AV] HDMI disconnected!\n");
        }

        pVideo->hdmiConnected = newConnectStatus;
    }

    ctrlEvent.hdr.type      = EVT_HDMI_CONNECTION;
    ctrlEvent.hdr.length    = sizeof(ctrlEvent);
    ctrlEvent.connected     = (paramsPtr->connected == 0) ? FFALSE : FTRUE;
    ctrlEvent.edidAvailable = (paramsPtr->haveEDID  == 0) ? FFALSE : FTRUE;

    (void)EVT_Send(&ctrlEvent, "TV Out Sys Change");
}

#ifdef APPL_HDCP_ENABLED
FSTATIC void AVINT_VideoHdmiHdcpStateCb(VAL_HdmiHdcpStateT newState,
                                                            void *dataPtr)
{
    SYS_EVT_HDCPSTATE_S     ctrlEvent;
    AV_VIDEO_S*             pVideo           = (AV_VIDEO_S*)dataPtr;

    if( pVideo->hdmiHdcpState != newState)
    {
        pVideo->hdmiHdcpState = newState;
        FAPI_SYS_PRINT_DEBUG(1, "[AVINT] HDCP=%d\n", newState);
    }

    ctrlEvent.hdr.type      = EVT_HDMI_HDCP_STATE;
    ctrlEvent.hdr.length    = sizeof(ctrlEvent);
    ctrlEvent.state         =  newState;

    (void)EVT_Send(&ctrlEvent, "HDMI Auth. State Change");
}
#endif /* APPL_HDCP_ENABLED */

FSTATIC void AVINT_VideoSystemConvert (SYS_HDTV_STANDARD hdOutput,
                                       SYS_SDTV_STANDARD sdOutput,
                                       VAL_HdEncoderT* hdSys,
                                       VAL_SdEncoderT* sdSys)
{
    switch ( hdOutput )
    {
        case HDSYS_576P:

            switch ( sdOutput )
            {
                default:
                //case SDSYS_AUTO_PAL_NTSC:// FIXME
                case SDSYS_PAL:
                case SDSYS_SECAM:
                case SDSYS_PAL_N:
                case SDSYS_PAL_N_ARG:   *hdSys = VAL_720x576x50P;   break;
                case SDSYS_PAL_M:
                case SDSYS_NTSC:        *hdSys = VAL_720x480x5994P; break;
            }
            break;

        case HDSYS_576I:
            switch ( sdOutput )
            {
                default:
                //case SDSYS_AUTO_PAL_NTSC:// FIXME
                case SDSYS_PAL:
                case SDSYS_SECAM:
                case SDSYS_PAL_N:
                case SDSYS_PAL_N_ARG:   *hdSys = VAL_720x576x50I;   break;
                case SDSYS_PAL_M:
                case SDSYS_NTSC:        *hdSys = VAL_720x480x5994I; break;
            }
            break;

        case HDSYS_720P:
            switch ( sdOutput )
            {
                default:
                //case SDSYS_AUTO_PAL_NTSC:// FIXME
                case SDSYS_PAL:
                case SDSYS_SECAM:
                case SDSYS_PAL_N:
                case SDSYS_PAL_N_ARG:   *hdSys = VAL_1280x720x50P;   break;
                case SDSYS_PAL_M:
                case SDSYS_NTSC:        *hdSys = VAL_1280x720x5994P; break;
            }
            break;

        default:
        //case HDSYS_AUTO: // FIXME: it should handled more smartly
        case HDSYS_1080I:
            switch ( sdOutput )
            {
                default:
                //case SDSYS_AUTO_PAL_NTSC:// FIXME
                case SDSYS_PAL:
                case SDSYS_SECAM:
                case SDSYS_PAL_N:
                case SDSYS_PAL_N_ARG:   *hdSys = VAL_1920x1080x50I;   break;
                case SDSYS_PAL_M:
                case SDSYS_NTSC:        *hdSys = VAL_1920x1080x5994I; break;
            }
            break;
        case HDSYS_1080P:
            switch ( sdOutput )
            {
                default:
                //case SDSYS_AUTO_PAL_NTSC:// FIXME
                case SDSYS_PAL:
                case SDSYS_SECAM:
                case SDSYS_PAL_N:
                case SDSYS_PAL_N_ARG:   *hdSys = VAL_1920x1080x50P;   break;
                case SDSYS_PAL_M:
                case SDSYS_NTSC:        *hdSys = VAL_1920x1080x5994P; break;
            }
            break;
    }

    switch ( sdOutput )
    {
        default:
        case SDSYS_PAL:   *sdSys = VAL_SD_PAL;   break;
        case SDSYS_NTSC:  *sdSys = VAL_SD_NTSC;  break;
        case SDSYS_SECAM: *sdSys = VAL_SD_SECAM; break;
        case SDSYS_PAL_M: *sdSys = VAL_SD_PAL_M; break;
        case SDSYS_PAL_N: *sdSys = VAL_SD_PAL_N; break;
        case SDSYS_PAL_N_ARG:       *sdSys = VAL_SD_PAL_N_ARG; break;

        // FIXME: it should handled more intelligent
        //case SDSYS_AUTO_PAL_NTSC:   *sdSys = VAL_SD_PAL; break;
    }
}

/* ------------------------------------------------------------------------- */

int32_t AVINT_AudioInit(void)
{
    AV_AUDIO_S*     pAudio = &(avData->audio);
    AAL_InitParamsT initParams;
    int32_t         retVal;
    FCDATA_ENTRY_S  fcEntry;

    memset (pAudio, 0, sizeof(AV_AUDIO_S));

    pAudio->lvlLeft           = AV_DEFAULTVOLUME_LEFT;
    pAudio->lvlRight          = AV_DEFAULTVOLUME_RIGHT;
    pAudio->balance           = AV_AUDIO_BALANCE_EQUAL;
    pAudio->stereoMode        = AV_STEREO_MODE_STEREO;
    pAudio->reqChannel        = SYS_AUDIOCHAN_LEFT;
    pAudio->startNotification = NULL;
    pAudio->muteStatus        = 0;
    pAudio->muteByUser        = FFALSE;
    pAudio->audioHandle       = NULL;

    /* set SPDIF compliant formats in preferred order */
    pAudio->spdifFormat[pAudio->spdifFormatNums++] = SYS_AUDIO_AC3;
    /* NOTE: MPEG via SPDIF has been removed for the moment, does not make
             so much sense as multichannel MPEG is rarely broadcast. */
    //pAudio->spdifFormat[pAudio->spdifFormatNums++] = SYS_AUDIO_MPEG;

    /* Attempt to get latest "frequently-changed"-settings. */

    if ( FCDATA_GetEntry(&fcEntry) == FAPI_OK )
    {
        pAudio->lvlLeft       = fcEntry.audioLvlLeft;
        pAudio->lvlRight      = fcEntry.audioLvlRight;
    }

#ifndef _LINUX_FAPI
    (void)FAPI_AUDEC_Reboot(0);
#else
    AAL_RebootAudec();
#endif
    memset(&initParams, 0, sizeof(initParams));

    /* NOTE: This relies on live and playback operation using the same TSD/BM
             block. */
    initParams.blockIndex = RSRC_TSD_LIVE; //!< || RSRC_TSD_PLAY

    retVal = AAL_Init(&initParams);

    return retVal;
}

int32_t AVINT_AudioWakeup (void)
{
    AV_AUDIO_S* pAudio  = &(avData->audio);

    /* set initial volume */
    AVINT_AudioSetStereoMode   (pAudio->stereoMode);
    (void)AVINT_AudioVolumeSet (pAudio->lvlLeft, pAudio->lvlRight);

    /* Reset mute status */
    pAudio->muteStatus = 0;
    pAudio->muteByUser = FFALSE;

    (void)AAL_Mute(FFALSE);

    return FAPI_OK;
}

int32_t AVINT_AudioShutdown (void)
{
    (void)AAL_Mute(FTRUE);

    (void)AVINT_AudioClose();

    return FAPI_OK;
}

int32_t AVINT_AudioOpen (void)
{
    AV_AUDIO_S*     pAudio  = &(avData->audio);
    int32_t         retVal  = FAPI_OK;
    AAL_OpenParamsT openParams;

    if ( pAudio->audioHandle == NULL )
    {
        memset(&openParams, 0, sizeof(openParams));

        openParams.feedMode                          = AAL_FEEDMODE_TSD;
        openParams.speaker[openParams.numSpeakers++] = AAL_AUOUT_SPEAKER_LEFT;
        openParams.speaker[openParams.numSpeakers++] = AAL_AUOUT_SPEAKER_RIGHT;
        openParams.suppressErrorRecognition          = 0;

        pAudio->audioHandle = AAL_Open (&openParams, &retVal);
        if (NULL != pAudio->audioHandle)
        {
            FAPI_SYS_PRINT_DEBUG(1, "[av_int][AVINT_AudioOpen] AAL_Open() success\n");
        }
        else
        {
            FAPI_SYS_PRINT_DEBUG(1, "[av_int][AVINT_AudioOpen] AAL_Open() failed\n");
        }
    }
    else
    {
        retVal = FAPI_ERR_ALREADY_OPEN;
    }

    return retVal;
}

int32_t AVINT_AudioClose (void)
{
    int32_t retVal;

    if ( avData->audio.audioHandle != NULL )
    {
        (void)AAL_Close(avData->audio.audioHandle);

        avData->audio.audioHandle = NULL;

        retVal = FAPI_OK;
    }
    else
    {
        retVal = FAPI_ERR_NOT_OPEN;
    }

    return retVal;
}

int32_t AVINT_AudioTask (void)
{
    int32_t retVal;

    if ( avData->audio.audioHandle != NULL )
    {
        retVal = AAL_Task (avData->audio.audioHandle);
    }
    else
    {
        retVal = FAPI_ERR_NOT_OPEN;
    }

    return retVal;
}

void AVINT_AudioMuteRequestSet(uint8_t req)
{
    if ( avData->audio.muteStatus == 0 )
    {
        (void)AAL_Mute(FTRUE);
    }

    avData->audio.muteStatus |= req;
}

void AVINT_AudioMuteRequestClear(uint8_t req)
{
    avData->audio.muteStatus &= (~req);

    if ( avData->audio.muteStatus == 0 )
    {
        (void)AAL_Mute(FFALSE);
    }
}

#ifdef APPL_DTG_SIPSI_ENABLED
/*!
*******************************************************************************
**
** \brief   AVINT_AudioSetStereo
**
** \note    This function get audio info, and set stereo mode.
**
** \param   reqChannel: left or right channel
**
** \return  NA
**
*******************************************************************************
*/
FSTATIC void AVINT_AudioSetStereo(uint8_t reqChannel)
{
    AV_AUDIO_STREAMINFO_S   audioInfo;
    AV_STEREO_MODE_E    mode;

    memset(&audioInfo,  0, sizeof(audioInfo));

    (void)AV_GetAudioStreamInfo(&audioInfo);

    if(audioInfo.stream.channels == AAL_AUOUT_CHANNEL_STEREO)
    {
        mode = AV_STEREO_MODE_STEREO;
    }
    else
    {
        switch(reqChannel)
        {
            default:
            case SYS_AUDIOCHAN_LEFT:
                mode = AV_STEREO_MODE_MONO_LEFT;
                break;

            case SYS_AUDIOCHAN_RIGHT:
                mode = AV_STEREO_MODE_MONO_RIGHT;
                break;
        }
    }

    AVINT_AudioSetStereoMode (mode);
}
#endif/*#ifdef APPL_DTG_SIPSI_ENABLED*/

int32_t AVINT_AudioStart (uint16_t pid, uint8_t format, uint8_t reqChannel,
                          void (*startNotification)(void))
{
    AV_AUDIO_S*         pAudio  = &(avData->audio);
    int32_t             retVal  = FAPI_OK;
    AAL_AudioFormatT    aalFormat = AAL_AUDIOFORMAT_NONE;

    if ( pAudio->audioHandle == NULL )
    {
        retVal = FAPI_ERR_NOT_OPEN;
    }
    else
    {
        pAudio->reqChannel        = reqChannel;
        pAudio->startNotification = startNotification;

        /* NOTE: In SmartGo-SD, audio start notifier was set here:
                 FD_AUD_SetAudioStartNotifyFunction (AVINT_AudioStartNotify);
        */
        retVal = AVINT_AudioConvertFormat (format, &aalFormat);

        if ( retVal == FAPI_OK )
        {
            retVal = AAL_Start (pAudio->audioHandle, aalFormat, pid);
            if( retVal == FAPI_OK)
            {
                pAudio->currPid = pid;
                #ifdef APPL_DTG_SIPSI_ENABLED
                AVINT_AudioSetStereo(reqChannel);
                #endif
            }
        }
    }

    return retVal;
}

void AVINT_AudioStop (void)
{
    AV_AUDIO_S* pAudio  = &(avData->audio);

    if ( pAudio->audioHandle != NULL )
    {
        (void)AAL_Stop (pAudio->audioHandle);
        pAudio->currPid = 0;
    }
    /* NOTE: In SmartGo-SD, audio start notifier was cleared here:
             FD_AUD_SetAudioStartNotifyFunction (NULL);
    */
}

int32_t AVINT_AudioPause (fbool_t enterPause)
{
    AV_AUDIO_S* pAudio = &(avData->audio);
    int32_t     retVal;

    if ( pAudio->audioHandle != NULL )
    {
        retVal = AAL_Pause (pAudio->audioHandle, enterPause);
    }
    else
    {
        retVal = FAPI_OK;
    }

    return retVal;
}

int32_t AVINT_AudioVolumeSet (uint8_t lvlLeft, uint8_t lvlRight)
{
    uint16_t balLvlLeft;
    uint16_t balLvlRight;

    #if (((AAL_AUOUT_MAX_VOL/AV_VOLUME_STEP)+1) == 32)

    /* this is for the standard settings of AAL_AUOUT_MAX_VOL == 0xFE and AV_VOLUME_STEP == 8 */
    #define VOL_CURVE 1             /* the curve can be more linear or fully logarithmic or a mixture */
    /*  flat curve */
    #if VOL_CURVE==2
    const uint8_t       volLevels[32] = { 0, 2, 4, 6, 9, 13, 16, 22,
        26, 31, 35, 40, 46, 51, 57, 64,
        71, 79, 87, 95, 104, 114, 125, 136,
        148, 160, 174, 188, 204, 220, 237, 255,
    };
    #elif VOL_CURVE==1
    /*  medium curve */
    const uint8_t       volLevels[32] = { 0, 3, 5, 7, 9, 11, 14, 17,
        20, 23, 27, 31, 36, 41, 46, 52,
        59, 66, 74, 83, 92, 102, 113, 125,
        138, 152, 167, 183, 200, 218, 238, 255,
    };
    #else
    /*  steep (logarithmic) curve */
    const uint8_t       volLevels[32] = { 0, 1, 2, 3, 4, 5, 6, 7,
        8, 9, 10, 11, 12, 13, 14, 16,
        19, 23, 27, 32, 38, 45, 54, 64,
        76, 91, 108, 128, 152, 181, 215, 255,
    };
    #endif

    #endif /* ((AAL_AUOUT_MAX_VOL/AV_VOLUME_STEP)+1 == 32) */

    if((lvlLeft > FAPI_AUOUT_MAX_VOL) || (lvlRight > FAPI_AUOUT_MAX_VOL))
    {
        return APPL_SMARTGO_ERR_BAD_PARAMETER;
    }

    /* calculate volume levels according to balance settings */
    if(avData->audio.balance == AV_AUDIO_BALANCE_EQUAL)
    {
        balLvlLeft  = lvlLeft;
        balLvlRight = lvlRight;
    }
    else if(avData->audio.balance == AV_AUDIO_BALANCE_LEFT_MAX)
    {
        balLvlLeft  = lvlLeft;
        balLvlRight = 0;
    }
    else if(avData->audio.balance == AV_AUDIO_BALANCE_RIGHT_MAX)
    {
        balLvlLeft  = 0;
        balLvlRight = lvlRight;
    }
    else if ( avData->audio.balance < AV_AUDIO_BALANCE_EQUAL )
    {
        balLvlLeft  = lvlLeft;

        /*lint -save -e571 */
        balLvlRight = (uint16_t)((lvlRight *
                      (avData->audio.balance + AV_AUDIO_BALANCE_RIGHT_MAX))
                      / 100);
        /*lint -restore */
    }
    else
    {
        /*lint -save -e571 */
        balLvlLeft  = (uint16_t)((lvlLeft *
                      (AV_AUDIO_BALANCE_RIGHT_MAX - avData->audio.balance))
                      / 100);
        /*lint -restore */

        balLvlRight = lvlRight;
    }

    #if ( ((AAL_AUOUT_MAX_VOL/AV_VOLUME_STEP)+1) == 32 )

    (void)AAL_SetVolume (volLevels[(uint8_t)balLvlLeft  / AV_VOLUME_STEP],
                         volLevels[(uint8_t)balLvlRight / AV_VOLUME_STEP]);

    #else

    /* this is the default setting in case somebody modifies
       AAL_AUOUT_MAX_VOL or AV_VOLUME_STEP */
    (void)AAL_SetVolume (balLvlLeft, balLvlRight);

    #endif

    avData->audio.lvlLeft  = lvlLeft;
    avData->audio.lvlRight = lvlRight;

    return FAPI_OK;
}

void AVINT_AudioSetStereoMode (AV_STEREO_MODE_E mode)
{
    AAL_StereoModeT aalStereoMode;

    avData->audio.stereoMode = mode;

    switch ( mode )
    {
        default:
        case AV_STEREO_MODE_STEREO:
            aalStereoMode = AAL_STEREO_MODE_STEREO;
            break;

        case AV_STEREO_MODE_MONO_LEFT:
            aalStereoMode = AAL_STEREO_MODE_MONO_LEFT;
            break;

        case AV_STEREO_MODE_MONO_RIGHT:
            aalStereoMode = AAL_STEREO_MODE_MONO_RIGHT;
            break;

        case AV_STEREO_MODE_MONO_LEFT_ONLY:
            aalStereoMode = AAL_STEREO_MODE_MONO_LEFT_ONLY;
            break;

        case AV_STEREO_MODE_MONO_RIGHT_ONLY:
            aalStereoMode = AAL_STEREO_MODE_MONO_RIGHT_ONLY;
            break;
    }

    (void)AAL_SetStereoMode (aalStereoMode);
}

int32_t AVINT_AudioGetStreamInfo (AV_AUDIO_STREAMINFO_S* info)
{
    AV_AUDIO_S* pAudio = &(avData->audio);

    if ( info != NULL )
    {
        memset (info, 0, sizeof(AV_AUDIO_STREAMINFO_S));

        if ( pAudio->audioHandle != NULL )
        {
            (void)AAL_GetStreamInfo(pAudio->audioHandle, &(info->stream));
        }
    }

    return FAPI_OK;
}

int32_t AVINT_AudioSPDIFSetStatus (AV_SPDIF_STATUS_E type, uint16_t pid,
                                   uint8_t format)
{
    AV_AUDIO_S*         pAudio  = &(avData->audio);
    int32_t             retVal  = FAPI_OK;
    AAL_AudioFormatT    aalFormat    = AAL_AUDIOFORMAT_NONE; /* satisfy MISRA */
    AAL_SPDIFModeT      aalSpdifMode = AAL_SPDIFMODE_OFF;    /* satisfy MISRA */

    if ( pAudio->audioHandle == NULL )
    {
        retVal = FAPI_ERR_NOT_OPEN;
    }
    else
    {
        switch ( type )
        {
            case SPDIFSTATUS_DISABLED:
                aalSpdifMode = AAL_SPDIFMODE_OFF;
                aalFormat    = AAL_AUDIOFORMAT_NONE;
                pid          = 0;
                break;

            case SPDIFSTATUS_CURRPCM:
                aalSpdifMode = AAL_SPDIFMODE_PCM;
                aalFormat    = AAL_AUDIOFORMAT_NONE;
                pid          = 0;
                break;

            case SPDIFSTATUS_PASSTHROUGH:
                aalSpdifMode = AAL_SPDIFMODE_PASSTHROUGH;

                retVal = AVINT_AudioConvertFormat (format, &aalFormat);
                break;

            default:
                retVal = APPL_SMARTGO_ERR_BAD_PARAMETER;
                break;
        }

        if ( retVal == FAPI_OK )
        {
            retVal = AAL_SetSPDIFMode (pAudio->audioHandle, aalSpdifMode,
                                       aalFormat, pid);
        }
    }

    return retVal;
}

int8_t AVINT_AudioSPDIFFindFirst (SYS_STREAMINF_AUDIO_S* pStrAudio)
{
    AV_AUDIO_S* pAudio  = &(avData->audio);
    uint8_t     cnt;
    uint8_t     formatCnt;
    int8_t      index = -1;

    /* Search for first available SPDIF compliant stream in order of
       preference. */
    for ( formatCnt = 0; formatCnt < pAudio->spdifFormatNums; formatCnt++ )
    {
        for ( cnt = 0; cnt < pStrAudio->numStreams; cnt++ )
        {
            if ( pStrAudio->stream[cnt].format
                 == pAudio->spdifFormat[formatCnt] )
            {
                index = (int8_t)cnt;
                break;
            }
        }

        if ( cnt < pStrAudio->numStreams )
        {
            break;
        }
    }

    return index;
}

fbool_t AVINT_AudioIsSPDIFCompliant (uint8_t format)
{
    AV_AUDIO_S* pAudio = &(avData->audio);
    fbool_t     retVal = FFALSE;
    uint8_t     formatCnt;

    for ( formatCnt = 0; formatCnt < pAudio->spdifFormatNums; formatCnt++ )
    {
        if ( format == pAudio->spdifFormat[formatCnt] )
        {
            retVal = FTRUE;
            break;
        }
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  Choose an audio stream according to user preferences.
**
** \param  pStrAudio        Pointer to audio stream information structure
                            serving as basis for stream search.
** \param  format           Requested audio format, choose #SYS_AUDIO_ALL to
**                          disable this filter.
** \param  directPref       "Direct" language preference that will override
**                          the general audio preferences made by the user via
**                          SETTINGS_AudioLangPrefSet(). Pass LANG_NOTSPECIFIED
**                          if no direct preference exists.
** \param  preferDecodable  #FTRUE: Only include streams in search that are
**                          decodable by current audio firmware.
**
** \return One of the following status codes:
**         - An audio index if successful or -1.
**
******************************************************************************/
int8_t AVINT_AudioChooseStream (const SYS_STREAMINF_AUDIO_S* pStrAudio,
                                uint8_t format, SYS_LANGCODE directPref,
                                fbool_t preferDecodable)
{
    int8_t                  audioIdx         = -1;
    int8_t                  firstFormatMatch = -1;
    fbool_t                 preferenceMade   = FFALSE;
    uint8_t                 cnt;
    SYS_LANGCODE            prefLang;
    const SYS_STR_AUDIO_S*  pStream;
    fbool_t                 isDecodable;

    /* Test if any stream is available */
    if ( pStrAudio->numStreams < 1 )
    {
        return -1;
    }

    /* language explicitly requested? */
    if ( directPref != LANG_NOTSPECIFIED )
    {
        preferenceMade = FTRUE;

        audioIdx = AVINT_AudioFindByLang (pStrAudio, format, directPref,
                                          preferDecodable);
    }

    /* language preference set? */
    if ( audioIdx < 0 )
    {
        for ( cnt = 0; cnt < SETTINGS_AUDIOLANGPREF_MAX; cnt++ )
        {
            if ( SETTINGS_AudioLangPrefGet(cnt, &prefLang) != FAPI_OK )
            {
                break;
            }

            if ( prefLang != LANG_NOTSPECIFIED )
            {
                preferenceMade = FTRUE;

                audioIdx = AVINT_AudioFindByLang (pStrAudio, format, prefLang,
                                                  preferDecodable);
                if ( audioIdx >= 0 )
                {
                    break;
                }
            }
        }
    }

#ifdef APPL_DTG_SIPSI_T2_ENABLED
    /* Preferred language (direct or by setting) not found: Search for qaa
     * "origianl language". */
    if ( (audioIdx < 0) && preferenceMade )
    {
        audioIdx = AVINT_AudioFindByLang (pStrAudio, format, LANG_ORIGINAL,
                                          preferDecodable);
    }
#endif

    /* Preferred language (direct or by setting) not found: Search for first
       Occurrence of "undefined" language. */
    if ( (audioIdx < 0) && preferenceMade )
    {
        audioIdx = AVINT_AudioFindByLang (pStrAudio, format, LANG_UNDETERMINED,
                                          preferDecodable);
    }

    /* Either no preference passed or no match found:
       Choose first available stream matching format and not being of a
       "special" type e.g. visual impaired commentary. */
    if ( audioIdx < 0 )
    {
        for ( cnt = 0; cnt < pStrAudio->numStreams; cnt++ )
        {
            pStream = pStrAudio->stream + cnt;

            /* filter by format */
            if ( (pStream->format != format)
              && (format          != SYS_AUDIO_ALL) )
            {
                continue;
            }

            if ( firstFormatMatch < 0 )
            {
                firstFormatMatch = (int8_t)cnt;
            }

            if ( preferDecodable )
            {
                if ( AV_AudioFormatIsDecodable (pStream->format,
                                                &isDecodable) == FAPI_OK )
                {
                    if ( !isDecodable )
                    {
                        continue;
                    }
                }
            }

            if ( (pStream->type == SYS_AUDIO_TYPE_NORMAL)  ||
                 (pStream->type == SYS_AUDIO_TYPE_EFFECTS) ||
                 (pStream->type == SYS_AUDIO_TYPE_UNKNOWN) )
            {
                audioIdx = (int8_t)cnt;
                break;
            }
        }
    }

    /* choose first available stream matching format */
    if ( audioIdx < 0 )
    {
        audioIdx = firstFormatMatch;
    }

    return audioIdx;
}

int8_t AVINT_AudioChooseSPDIFStream (const SYS_STREAMINF_AUDIO_S* pStrAudio,
                                     SYS_LANGCODE directPref)
{
    AV_AUDIO_S* pAudio = &(avData->audio);
    uint8_t     formatCnt;
    int8_t      index = -1;

    /* Search for SPDIF compliant stream in order of preference. */
    for ( formatCnt = 0; formatCnt < pAudio->spdifFormatNums; formatCnt++ )
    {
        index = AVINT_AudioChooseStream (pStrAudio,
                                         pAudio->spdifFormat[formatCnt],
                                         directPref, FFALSE);

        if ( index >= 0 )
        {
            break;
        }
    }

    return index;
}

/*
Searches for an audio stream indicating requested language. Audio types other
than "normal" (e.g. visual impaired commentary) are ignored.

returns:    >= 0   index of matching stream
            -1     no matching stream found
*/
FSTATIC int8_t AVINT_AudioFindByLang (const SYS_STREAMINF_AUDIO_S* pStrAudio,
                                      uint8_t format, SYS_LANGCODE lang,
                                      fbool_t decodableOnly)
{
    int8_t                  cnt;
    int8_t                  index = -1;
    const SYS_STR_AUDIO_S*  pStream;
    fbool_t                 isDecodable;

    for ( cnt = 0; cnt < (int8_t)pStrAudio->numStreams; cnt++ )
    {
        pStream = pStrAudio->stream + cnt;

        /* filter by format */
        if ( (pStream->format != format)
          && (format          != SYS_AUDIO_ALL) )
        {
            continue;
        }

        /* filter by language */
        if ( pStream->langCode != lang )
        {
            continue;
        }

        /* filter by decodability if requested */
        if ( decodableOnly )
        {
            if ( AV_AudioFormatIsDecodable (pStream->format, &isDecodable)
                 == FAPI_OK )
            {
                if ( !isDecodable )
                {
                    continue;
                }
            }
        }

        /* filter type of audio stream */
        if ( (pStream->type == SYS_AUDIO_TYPE_NORMAL)
          || (pStream->type == SYS_AUDIO_TYPE_EFFECTS)
          || (pStream->type == SYS_AUDIO_TYPE_UNKNOWN) )
        {
            index = cnt;
            break;
        }
    }

    return index;
}

FSTATIC int32_t AVINT_AudioConvertFormat (uint8_t format,
                                          AAL_AudioFormatT* pAalFormat)
{
    int32_t          retVal    = FAPI_OK;
    AAL_AudioFormatT aalFormat = AAL_AUDIOFORMAT_NONE;

    switch ( format )
    {
        case SYS_AUDIO_MPEG:
            aalFormat = AAL_AUDIOFORMAT_MPEG;
            break;

        case SYS_AUDIO_AC3:
            aalFormat = AAL_AUDIOFORMAT_AC3;
            break;

        case SYS_AUDIO_AAC:
            aalFormat = AAL_AUDIOFORMAT_AAC;
            break;

        case SYS_AUDIO_HEAAC:
            aalFormat = AAL_AUDIOFORMAT_HEAAC;
            break;

        case SYS_AUDIO_DDPDCV:
            aalFormat = AAL_AUDIOFORMAT_DDP;
            break;

        default:
            retVal = FAPI_ERR_BAD_PARAMETER;
            break;
    }

    if ( (retVal == FAPI_OK) && (pAalFormat != NULL) )
    {
        *pAalFormat = aalFormat;
    }

    return retVal;
}

/* Martin modified at 20100513 */
int32_t AVINT_HDMIGetEDIDLipSyncDelay (int32_t* delay)
{

    AV_VIDEO_S*             pVideo = &(avData->video);
    VAL_HDMI_EdidInfoStrT   edid;
    int32_t                 retVal = FAPI_OK;
    int32_t                 video_delay = 0;
    int32_t                 audio_delay = 0;
    int32_t                 edid_delay = 0;
    int32_t                 edid_interlaced_delay = 0;
    AV_VIDEO_STREAMINFO_S   info;

    memset(&edid, 0, sizeof(edid));

    if ( pVideo->valDataPtr != NULL )
    {
        retVal = VAL_GetHDMI_EDID(pVideo->valDataPtr, &edid);

        /* delay information is invalid */
        if(!(edid.videoLatency) || (edid.videoLatency > 251) ||
           !(edid.audioLatency) || (edid.audioLatency > 251))
        {
            *delay = 0xDEAD;
            retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;

            return retVal;
        }

        video_delay = (int32_t)( (edid.videoLatency - 1) * 2);
        audio_delay = (int32_t)( (edid.audioLatency - 1) * 2);
        edid_delay = (video_delay - audio_delay);

        /* either audio or video is not supported */
        if(!(edid.interlacedVideoLatency) || (edid.interlacedVideoLatency > 251) ||
           !(edid.interlacedAudioLatency) || (edid.interlacedAudioLatency > 251)
          )
        {
            /* use same settings as progressive */
            edid_interlaced_delay = (video_delay - audio_delay);
        }
        else
        {
            /* see HDMI 1.3 spec 8.3.2 for further information */
            video_delay = (int32_t)( (edid.interlacedVideoLatency - 1) * 2);
            audio_delay = (int32_t)( (edid.interlacedAudioLatency - 1) * 2);
            edid_interlaced_delay = (video_delay - audio_delay);
        }

        /* determine progressive or interlaced */
        if(AVINT_VideoGetStreamInfo(&info) == FAPI_OK)
        {
            if(info.stream.frameField == 0)
            {
                *delay = edid_interlaced_delay;
            }
            else
            {
                *delay = edid_delay;
            }
        }
        else
        {
            retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
        }
    }
    else
    {
         *delay = 0xDEAD;

         retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }

    return retVal;

}

int32_t AVINT_HDMISetLipSyncDelay (int32_t delay)
{
    int32_t                 retVal = FAPI_OK;

    if (avData != NULL)
    {
        avData->avLipSyncDelay = delay;
        avData->hdmiLipSyncSet = FFALSE;

        retVal = AVINT_IntHDMISetLipSyncDelay();
    }
    else
    {
        retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }

    return retVal;
}

/* Martin modified at 20100513*/
FSTATIC int32_t AVINT_IntHDMISetLipSyncDelay (void)
{

    AV_AUDIO_S*             pAudio  = &(avData->audio);
    AV_VIDEO_S*             pVideo = &(avData->video);
    int32_t                 frame_time = 0;
    AV_VIDEO_STREAMINFO_S   info;
    int32_t                 retVal = FAPI_OK;
    fbool_t                 blocked = FFALSE;
    int32_t                 delay = 0;

    if ( (pVideo->valDataPtr != NULL)
      && (pAudio->audioHandle != NULL)
      && (!(avData->hdmiLipSyncSet)) )
    {
        if( !(pVideo->haveFrameRate) )
        {
            blocked = FTRUE;

            retVal = AVINT_VideoGetStreamInfo(&info);
            if ( (retVal == FAPI_OK) && (info.stream.frameRate!= 0) )
            {
                pVideo->haveFrameRate = FTRUE;
                blocked = FFALSE;
            }
        }

        if ( !blocked )
        {
            delay = avData->avLipSyncDelay;

            if( delay >=0 )
            {
                (void)VAL_SetOutputDelay( pVideo->valDataPtr, 0 );
                retVal = AAL_SetOutputDelay( pAudio->audioHandle, delay );
                if ( retVal == FAPI_OK )
                {
                    avData->hdmiLipSyncSet = FTRUE;
                }
            }
            else
            {
                // in case video has to be delayed, we can divide the delay value
                // by the framrate/100 (vesFrameRate) and then try to speed up audio
                // by the remainder
                (void)AVINT_VideoGetStreamInfo(&info);
                frame_time = (int32_t)( 100000 / info.stream.frameRate);
                retVal = AAL_SetOutputDelay( pAudio->audioHandle,
                                  ( ( delay * 100 ) / frame_time) % frame_time);
                if ( retVal != FAPI_OK )
                {
                    return retVal;
                }
                else
                {
                    retVal = VAL_SetOutputDelay( pVideo->valDataPtr,
                                                 abs(delay));
                }
                if ( retVal == FAPI_OK )
                {
                    avData->hdmiLipSyncSet = FTRUE;
                }
            }
        }
    }
    else
    {
        retVal = SMARTGO_SYS_ERR_DATA_NOT_AVL;
    }

    return retVal;

}

int32_t AVINT_HDMIAutoSync (void)
{
    int32_t delay;
    int32_t retVal;

    retVal = AVINT_HDMIGetEDIDLipSyncDelay(&delay);

    if ( retVal == FAPI_OK )
    {
        avData->hdmiLipSyncSet = FFALSE;
        avData->avLipSyncDelay = delay;
        /* set the lip sync value to the flash */
        (void)SETTINGS_LipSyncDelaySet(delay);
        retVal = AVINT_IntHDMISetLipSyncDelay();
    }

    return retVal;
}

#if (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED)

/*!
*******************************************************************************
**
** \brief  The function selects subtitle stream by specifying type and index
**
** \param  index        subtitle index
** \param  isDvb        type of subtitle, e.g. DVB or teletext subtitle
** \param  appSelected  indicate selection is from application or system
**
** \return  One of the following status codes:
**          - FAPI_OK   select successfully
**          - APPL_SMARTGO_ERR_NOT_INITIALIZED
**          - APPL_SMARTGO_ERR_BAD_PARAMETER
****************************************************************************
**/
int32_t AVINT_SubtSelectStream(int8_t index, fbool_t isDvb, fbool_t appSelected)
{
    int32_t             retVal = FAPI_OK;
    int8_t              dvbSubttIdx = -1;
    int8_t              ttxSubttIdx = -1;

    #ifdef APPL_SUBTT_ENABLED
    SUBTT_STREAMINF_S   dvbSubttInfo;
    #endif /* APPL_SUBTT_ENABLED */

    #ifdef APPL_TTXT_ENABLED
    TTXSUBTT_INF_S      ttxSubttInfo;
    #endif /* APPL_TTXT_ENABLED */

    /* start subtitle with selected stream index*/
    if(NULL == avData)
    {
        return APPL_SMARTGO_ERR_NOT_INITIALIZED;
    }

    if(appSelected)
    {
        avData->subtt.appSelected = appSelected;
    }

    if(index < 0) /* turn off subtitle */
    {
        avData->subtt.langCode    = LANG_NONE;
        avData->subtt.displayType = SUBTT_DISPLAY_NONE;
        avData->subtt.type        = SYS_SUBTITLE_TYPE_NORMAL;
    }
    else /* one of DVB and TTX subtitle is selected */
    {
        /* check if index is valid */
        if(isDvb)
        {
            #ifdef APPL_SUBTT_ENABLED
            if(index < SUBTT_NumStreamsGet())
            {
                dvbSubttIdx = index;
            }
            else
            {
                return APPL_SMARTGO_ERR_BAD_PARAMETER;
            }
            #endif /* APPL_SUBTT_ENABLED */
        }
        else
        {
            #ifdef APPL_TTXT_ENABLED
            if(index < TTXSUBTT_NumStreamsGet())
            {
                ttxSubttIdx = index;
            }
            else
            {
                return APPL_SMARTGO_ERR_BAD_PARAMETER;
            }
            #endif /* APPL_TTXT_ENABLED */
        }

        /* remember stream info of user selection */
        if( (FTRUE == appSelected) &&
            ((-1 != dvbSubttIdx) || (-1 != ttxSubttIdx)))
        {
            if(NULL != avData)
            {
                avData->subtt.isDvb = isDvb;
                if(isDvb)
                {
                    #ifdef APPL_SUBTT_ENABLED
                    (void)SUBTT_StreamInfoGet((uint8_t)index, &dvbSubttInfo);
                    avData->subtt.langCode    = dvbSubttInfo.langCode;
                    avData->subtt.displayType = dvbSubttInfo.displayType;
                    avData->subtt.type        = dvbSubttInfo.type;
                    #endif /* APPL_SUBTT_ENABLED */
                }
                else
                {
                    #ifdef APPL_TTXT_ENABLED
                    (void)TTXSUBTT_StreamInfoGet((uint8_t)index, &ttxSubttInfo);
                    avData->subtt.langCode    = ttxSubttInfo.langCode;
                    avData->subtt.displayType = SUBTT_DISPLAY_NONE;
                    avData->subtt.type        = SYS_SUBTITLE_TYPE_NORMAL;
                    #endif /* APPL_TTXT_ENABLED */
                }
            }
        }
    }

    if(isDvb)
    {
        #ifdef APPL_TTXT_ENABLED
        (void)TTXSUBTT_Select(ttxSubttIdx);
        #endif /* APPL_TTXT_ENABLED */

        #ifdef APPL_SUBTT_ENABLED
        (void)SUBTT_Select(dvbSubttIdx);
        #endif /* APPL_SUBTT_ENABLED */
    }
    else
    {
        #ifdef APPL_SUBTT_ENABLED
        (void)SUBTT_Select(dvbSubttIdx);
        #endif /* APPL_SUBTT_ENABLED */

        #ifdef APPL_TTXT_ENABLED
        (void)TTXSUBTT_Select(ttxSubttIdx);
        #endif /* APPL_TTXT_ENABLED */
    }

    return retVal;
}

/*!
*******************************************************************************
**
** \brief  The function Chooses subtitle stream by weighting stream info
**
** \note   This function chooses a favorable subtitle stream by weighting
**         information of preferred language ( from user select or language
**         setting), type (HEARING_IMPAIRED, VISUAL_IMPAIRED), display type
**         (intended display aspect ratio), transmission type (DVB or Teletext
**         subtitle).
**
** \param  directPref   pointer to direct subtitle preference
** \param  index        pointer to subtitle index which will be filled
** \param  isDvb        pointer to subtitle type flag which will be filled
**
** \return  One of the following status codes:
**          - FTURE     stream found and values of index and isDvb are valid
**          - FFALSE    stream not found and values of index and isDvb are
**                      invalid
****************************************************************************
**/
FSTATIC fbool_t AVINT_SubtChooseStream(AV_SUBTT_S * directPref, int8_t* index, fbool_t* isDvb)
{
    SYS_LANGCODE     prefLang[SETTINGS_SUBTLANGPREF_MAX];
    #ifdef APPL_SUBTT_ENABLED
    uint8_t          prefType       = SYS_SUBTITLE_TYPE_NORMAL;
    SUBTT_DISPLAY_E  prefDispType   = SUBTT_DISPLAY_NONE;
    SYS_PICFORMAT_E  dispFormat     = PICFORMAT_NONE;
    #endif /* APPL_SUBTT_ENABLED */
    fbool_t          prefIsDvb      = FTRUE;
    fbool_t          found          = FFALSE;
    int8_t           cnt            = 0;
    uint8_t          langCnt        = 0;
    uint8_t          langMax        = 0;
    uint8_t          subttMax       = 0;
    uint16_t         matchValue     = 0;
    uint16_t         matchTemp      = 0;
    int8_t           matchIndex     = -1;
    fbool_t          matchIsDvb     = FTRUE;

    #ifdef APPL_SUBTT_ENABLED
    SUBTT_STREAMINF_S    dvbInf;
    #endif /* APPL_SUBTT_ENABLED */

    #ifdef APPL_TTXT_ENABLED
    TTXSUBTT_INF_S       ttxInf;
    #endif /* APPL_TTXT_ENABLED */

    memset(prefLang, LANG_NONE, SETTINGS_SUBTLANGPREF_MAX*sizeof(SYS_LANGCODE));

    /* prepare preferred a language list with greater weight at large index */
    if(directPref != 0)
    {
        prefLang[langMax++] = directPref->langCode;
        #ifdef APPL_SUBTT_ENABLED
        prefDispType        = directPref->displayType;
        prefIsDvb           = directPref->isDvb;
        #endif /* APPL_SUBTT_ENABLED */
    }
    else
    {
        if(AVINT_SubtCheckLangPrefIsMade())
        {
            for(langCnt = SETTINGS_SUBTLANGPREF_MAX; langCnt >0 ; langCnt-- )
            {
                (void)SETTINGS_SubtLangPrefGet((uint8_t)(langCnt-1),
                                        &prefLang[langCnt-1]);
                langMax++;
            }
        }
    }


    #ifdef APPL_SUBTT_ENABLED
    /* match stream in DVB subtitle list */

    /* prepare preferred display aspect ratio type */
    (void)SETTINGS_DispFormatGet(&dispFormat,NULL);
    switch(dispFormat)
    {
        case PICFORMAT_4_3:
            prefDispType = SUBTT_DISPLAY_4_3;
            break;
        case PICFORMAT_16_9:
            prefDispType = SUBTT_DISPLAY_16_9;
            break;
        default:
        case PICFORMAT_NONE:
            break;
    }

    subttMax  = SUBTT_NumStreamsGet();

    if ( subttMax > SUBTT_STREAMS_MAX )
    {
        subttMax = SUBTT_STREAMS_MAX;
    }

    for ( cnt = 0; cnt < subttMax; cnt++ )
    {
        matchTemp = 0;

        if ( SUBTT_StreamInfoGet ((uint8_t)cnt, &dvbInf) == FAPI_OK )
        {
            if(prefIsDvb)/* weight for transmission type */
            {
                matchTemp |= AV_SUBTT_WEIGHT_ISDVB; //is DVB subtitle
            }

            if(dvbInf.type == prefType) /* weight for impairing type */
            {
                matchTemp |= AV_SUBTT_WEIGHT_TYPE;
            }

            if(dvbInf.displayType == prefDispType) /* weight for display type */
            {
                matchTemp |= AV_SUBTT_WEIGHT_RATIO;
            }

            for(langCnt = langMax; langCnt > 0; langCnt--)
            {
                /* nullify weight when preferred language is none */
                if(prefLang[langCnt-1] == LANG_NONE)
                {
                    matchTemp &= AV_SUBTT_WEIGHT_ZERO;
                }
                else
                {
                    /* weight for preferred languages */
                    if(dvbInf.langCode == prefLang[langCnt-1])
                    {
                        matchTemp |= (AV_SUBTT_WEIGHT_LANG + langCnt);
                    }

                    /* weight for "original" language "qaa". If no stream
                     * matches preferred languages, an original language should
                     * be selected. */
                    if(dvbInf.langCode == LANG_ORIGINAL)
                    {
                        matchTemp |= AV_SUBTT_WEIGHT_LANG_ORIG;
                    }

                    /* weight for "undefined" language. If no stream matches
                     * preferred languages and original language, an
                     * undetermined should be selected */
                    if(dvbInf.langCode == LANG_UNDETERMINED)
                    {
                        matchTemp |= AV_SUBTT_WEIGHT_LANG_UND;
                    }
                }
            }

            if(matchTemp > matchValue)
            {
                matchValue = matchTemp;
                matchIndex = cnt;
                matchIsDvb = FTRUE;
            }
        }
    }
    #endif /* APPL_SUBTT_ENABLED */

    /* match stream in teletext subtitle list */

    #ifdef APPL_TTXT_ENABLED
    subttMax  = TTXSUBTT_NumStreamsGet();

    if ( subttMax > TTXSUBTT_STREAMS_MAX )
    {
        subttMax = TTXSUBTT_STREAMS_MAX;
    }

    for ( cnt = 0; cnt < subttMax; cnt++ )
    {
        matchTemp = 0;

        if ( TTXSUBTT_StreamInfoGet((uint8_t)cnt, &ttxInf) == FAPI_OK )
        {
            for(langCnt = langMax; langCnt > 0; langCnt--)
            {
                /* nullify weight when preferred language is none */
                if(prefLang[langCnt-1] == LANG_NONE)
                {
                    matchTemp &= AV_SUBTT_WEIGHT_ZERO;
                }
                else
                {
                    /* weight for preferred languages */
                    if(ttxInf.langCode == prefLang[langCnt-1])
                    {
                        matchTemp |= (AV_SUBTT_WEIGHT_LANG + langCnt);
                    }

                    /* weighting for original language "qaa". If no stream
                     * matches preferred languages, an original language should
                     * be selected. */
                    if(ttxInf.langCode == LANG_ORIGINAL)
                    {
                        matchTemp |= AV_SUBTT_WEIGHT_LANG_ORIG;
                    }

                    /* weight for "undefined" language. If no stream matches
                     * preferred languages and original language, an
                     * undetermined should be selected */
                    if(ttxInf.langCode == LANG_UNDETERMINED)
                    {
                        matchTemp |= AV_SUBTT_WEIGHT_LANG_UND;
                    }
                }
            }

            if(matchTemp > matchValue)
            {
                matchValue = matchTemp;
                matchIndex = cnt;
                matchIsDvb = FFALSE;
            }
        }
    }
    #endif /* APPL_TTXT_ENABLED */

    if (matchValue > 0 )
    {
        found = FTRUE;
        *index = matchIndex;
        *isDvb = matchIsDvb;
    }

    return found;

}

#if 0
/*!
*******************************************************************************
**
** \brief  The function find subtitle stream by specific language
**
** \param  lang         language to be matched
** \param  dvbFirst     indicate DVB subtitle has precedence
** \param  index        pointer to subtitle index which will be filled
** \param  isDvb        pointer to subtitle type flag which will be filled

** \return  One of the following status codes:
**          - FTRUE     specified subtitle stream is found
**          - FFALSE    specified subtitle stream not found
****************************************************************************
**/
FSTATIC fbool_t AVINT_SubtFindByLang(SYS_LANGCODE lang, fbool_t dvbFirst,
                                    int8_t* index, fbool_t* isDvb)
{
    int8_t  cnt;
    uint8_t numStreams = 0;
    fbool_t found = FFALSE;
    int8_t  tmpIndex = -1;
    fbool_t tmpIsDvb = FTRUE;


    #ifdef APPL_SUBTT_ENABLED
    SUBTT_STREAMINF_S    dvbInf;
    #endif /* APPL_SUBTT_ENABLED */

    #ifdef APPL_TTXT_ENABLED
    TTXSUBTT_INF_S       ttxInf;
    #endif /* APPL_TTXT_ENABLED */

    do
    {
        #ifdef APPL_SUBTT_ENABLED
        //find preferred language in subtitle streams
        numStreams  = SUBTT_NumStreamsGet();

        for(cnt = 0; cnt <= numStreams; cnt++)
        {
            if (SUBTT_StreamInfoGet((uint8_t)cnt,&dvbInf) == FAPI_OK)
            {
                if (lang == LANG_NOTSPECIFIED)
                {
                    found = FTRUE;
                    tmpIndex = cnt;
                    tmpIsDvb = FTRUE;
                    break;
                }
                else
                {
                    if(dvbInf.langCode == lang)
                    {
                        found = FTRUE;
                        tmpIndex = cnt;
                        tmpIsDvb = FTRUE;
                        break;
                    }
                }
            }
        }
        if ( found && dvbFirst)
        {
            break;
        }
        #endif /* APPL_SUBTT_ENABLED */

        #ifdef APPL_TTXT_ENABLED
        //find preferred language in teletext subtitle streams
        numStreams  = TTXSUBTT_NumStreamsGet();

        for(cnt = 0; cnt <= numStreams; cnt++)
        {
            if (TTXSUBTT_StreamInfoGet((uint8_t)cnt,&ttxInf) == FAPI_OK)
            {
                if (lang == LANG_NOTSPECIFIED)
                {
                    found = FTRUE;
                    tmpIndex = cnt;
                    tmpIsDvb = FFALSE;
                    break;
                }
                else
                {
                    if(ttxInf.langCode == lang)
                    {
                        found = FTRUE;
                        tmpIndex = cnt;
                        tmpIsDvb = FFALSE;
                        break;
                    }
                }
            }
        }
        #endif /* APPL_TTXT_ENABLED */

    } while(0);

    if(found)
    {
        *index = tmpIndex;
        *isDvb = tmpIsDvb;
    }

    return found;
}
#endif

// check if subtitle language preference is made
FSTATIC fbool_t AVINT_SubtCheckLangPrefIsMade(void)
{
    fbool_t             preferenceMade = FFALSE;
    SYS_LANGCODE        subtLangPref;
    uint8_t             cnt = 0;

    for(;cnt<SETTINGS_SUBTLANGPREF_MAX; cnt++)
    {
        (void)SETTINGS_SubtLangPrefGet(cnt, &subtLangPref);

        if(subtLangPref != LANG_NONE)
        {
            preferenceMade = FTRUE;
            break;
        }
    }

    return preferenceMade;
}

#endif /* #if (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED) */

/* actions to do when PMT update is present */
void AVINT_SubtPmtUpdate(void)
{
    #if (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED)

    int8_t       index;
    fbool_t      isDvb;
    AV_SUBTT_S*  pSubtt = NULL;

    if(NULL != avData)
    {
        if(avData->subtt.appSelected)
        {
            pSubtt = &avData->subtt;
        }
    }

    if(AVINT_SubtChooseStream(pSubtt, &index, &isDvb))
    {
        (void)AVINT_SubtSelectStream(index, isDvb, FFALSE);
    }

    #endif /* #if (defined APPL_SUBTT_ENABLED) || (defined APPL_TTXT_ENABLED) */
}

/*!
*******************************************************************************
**
** \brief  The function resets subtitle structure
**
** \param   void
**
** \return  void
****************************************************************************
**/
void AVINT_SubtResetUserSelect(void)
{
    if(NULL == avData)
    {
        return;
    }

    memset(&avData->subtt, 0, sizeof(AV_SUBTT_S));

    return;
}

#ifdef APPL_HDCP_ENABLED
/*!
*******************************************************************************
**
** \brief  Dynamic ON/OFF HDCP function.
**
** \param  enable  FTRUE: enable, FFALSE: disable
**
** \return One of the following status codes:
**         - #FAPI_OK if successful
******************************************************************************/
int32_t AVINT_DynamicEnableHDCP(fbool_t enable)
{
    int32_t                 retVal = FAPI_OK;
    AV_VIDEO_S*             pVideo = &(avData->video);

    retVal = VAL_HdmiHdcpEnable(pVideo->valDataPtr, (uint32_t)enable);
    if(retVal == FAPI_OK)
    {
        pVideo->hdmiHdcpEnabled = enable;
    }
    FAPI_SYS_PRINT_DEBUG(1, "[AVINT] HDCP_OnOff(%d)\n", enable);

    return retVal;
}
#endif

char_t*  SYS_GetErrorString(int32_t error)
{
    switch(error)
    {
        case 0: return("FAPI_OK");
        case -1 : return("FAPI_ERR_BAD_PARAMETER");
        case -2 : return("FAPI_ERR_OUT_OF_MEMORY");
        case -3 : return("FAPI_ERR_ALREADY_INITIALIZED");
        case -4 : return("FAPI_ERR_NOT_INITIALIZED");
        case -5 : return("FAPI_ERR_FEATURE_NOT_SUPPORTED");
        case -6 : return("FAPI_ERR_TIMEOUT");
        case -7 : return("FAPI_ERR_DEVICE_BUSY");
        case -8 : return("FAPI_ERR_INVALID_HANDLE");
        case -9 : return("FAPI_ERR_SEMAPHORE_CREATE");
        case -10 : return("FAPI_ERR_UNSUPPORTED_VERSION");
        case -11 : return("FAPI_ERR_MAILQUEUE_CREATE");
        case -12 : return("FAPI_ERR_NOT_OPEN");
        case -13 : return("FAPI_ERR_ALREADY_OPEN");
        case -1000 : return("FAPI_ATA_MODULE_BASE");
        case -1001 : return("FAPI_ATA_ERR_BAD_PARAMETER");
        case -1002 : return("FAPI_ATA_ERR_OUT_OF_MEMORY");
        case -1003 : return("FAPI_ATA_ERR_ALREADY_INITIALIZED");
        case -1004 : return("FAPI_ATA_ERR_NOT_INITIALIZED");
        case -1005 : return("FAPI_ATA_ERR_FEATURE_NOT_SUPPORTED");
        case -1006 : return("FAPI_ATA_ERR_TIMEOUT");
        case -1007 : return("FAPI_ATA_ERR_DEVICE_BUSY");
        case -1008 : return("FAPI_ATA_ERR_INVALID_HANDLE");
        case -1009 : return("FAPI_ATA_ERR_SEMAPHORE_CREATE");
        case -1010 : return("FAPI_ATA_ERR_UNSUPPORTED_VERSION");
        case -1101 : return("FAPI_ATA_ERR_DEVICE_NOT_SUPPORTED");
        case -1102 : return("FAPI_ATA_ERR_UPI_CONFIGURATION");
        case -1103 : return("FAPI_ATA_ERR_SET_COMMAND");
        case -1104 : return("FAPI_ATA_ERR_IDENTIFY_DEVICE");
        case -1105 : return("FAPI_ATA_ERR_NOT_ENABLED");
        case -1106 : return("FAPI_ATA_ERR_SECTOR_COUNT");
        case -1107 : return("FAPI_ATA_ERR_LBA_NUMBER");
        case -1108 : return("FAPI_ATA_ERR_COMMAND_FAILED");
        case -1109 : return("FAPI_ATA_ERR_DMA_REQUEST_FAILED");
        case -1110 : return("FAPI_ATA_ERR_DMA_TRANSFER_FAILED");
        case -1111 : return("FAPI_ATA_ERR_PIOMODE_NOT_SUPPORTED");
        case -1112 : return("FAPI_ATA_ERR_DMAMODE_NOT_SUPPORTED");
        case -1113 : return("FAPI_ATA_ERR_DEVICE_IN_STANDBY");
        case -2000 : return("FAPI_AUDEC_MODULE_BASE");
        case -2001 : return("FAPI_AUDEC_ERR_BAD_PARAMETER");
        case -2002 : return("FAPI_AUDEC_ERR_OUT_OF_MEMORY");
        case -2003 : return("FAPI_AUDEC_ERR_ALREADY_INITIALIZED");
        case -2004 : return("FAPI_AUDEC_ERR_NOT_INITIALIZED");
        case -2005 : return("FAPI_AUDEC_ERR_FEATURE_NOT_SUPPORTED");
        case -2006 : return("FAPI_AUDEC_ERR_TIMEOUT");
        case -2007 : return("FAPI_AUDEC_ERR_DEVICE_BUSY");
        case -2008 : return("FAPI_AUDEC_ERR_INVALID_HANDLE");
        case -2009 : return("FAPI_AUDEC_ERR_SEMAPHORE_CREATE");
        case -2010 : return("FAPI_AUDEC_ERR_UNSUPPORTED_VERSION");
        case -2100 : return("FAPI_AUDEC_ERR_AUDIO_PROCESSOR");
        case -2101 : return("FAPI_AUDEC_ERR_HEADER_NOT_FOUND");
        case -2102 : return("FAPI_AUDEC_ERR_ALREADY_RUNNING");
        case -2103 : return("FAPI_AUDEC_ERR_FIRMWARE_NOT_SUPPORTED");
        case -2104 : return("FAPI_AUDEC_ERR_CODEC_NOT_SUPPORTED");
        case -2105 : return("FAPI_AUDEC_ERR_AUDIO_SUSPENDED");
        case -3000 : return("AAL_AUOUT_MODULE_BASE");
        case -3001 : return("AAL_AUOUT_ERR_BAD_PARAMETER");
        case -3002 : return("AAL_AUOUT_ERR_OUT_OF_MEMORY");
        case -3003 : return("AAL_AUOUT_ERR_ALREADY_INITIALIZED");
        case -3004 : return("AAL_AUOUT_ERR_NOT_INITIALIZED");
        case -3005 : return("AAL_AUOUT_ERR_FEATURE_NOT_SUPPORTED");
        case -3006 : return("AAL_AUOUT_ERR_TIMEOUT");
        case -3007 : return("AAL_AUOUT_ERR_DEVICE_BUSY");
        case -3008 : return("AAL_AUOUT_ERR_INVALID_HANDLE");
        case -3009 : return("AAL_AUOUT_ERR_SEMAPHORE_CREATE");
        case -3010 : return("AAL_AUOUT_ERR_UNSUPPORTED_VERSION");
        case -3100 : return("AAL_AUOUT_ERR_HDMI_RUNNING");
        case -3101 : return("AAL_AUOUT_ERR_MULTIPLE_ASSIGNMENT");
        case -3102 : return("AAL_AUOUT_ERR_MAX_HANDLES_REACHED");
        case -3103 : return("AAL_AUOUT_ERR_MAX_FDCLK_REACHED");
        case -3104 : return("AAL_AUOUT_ERR_INVALID_SAMPLE_FREQ");
        case -3105 : return("AAL_AUOUT_WAR_NEED_RESTART");
        case -4000 : return("FAPI_BOOT_MODULE_BASE");
        case -4001 : return("FAPI_BOOT_ERR_BAD_PARAMETER");
        case -4002 : return("FAPI_BOOT_ERR_OUT_OF_MEMORY");
        case -4003 : return("FAPI_BOOT_ERR_ALREADY_INITIALIZED");
        case -4004 : return("FAPI_BOOT_ERR_NOT_INITIALIZED");
        case -4005 : return("FAPI_BOOT_ERR_FEATURE_NOT_SUPPORTED");
        case -4008 : return("FAPI_BOOT_ERR_INVALID_HANDLE");
        case -4009 : return("FAPI_BOOT_ERR_SEMAPHORE_CREATE");
        case -4010 : return("FAPI_BOOT_ERR_UNSUPPORTED_VERSION");
        case -4101 : return("FAPI_BOOT_ERR_DEVICE_UNKNOWN");
        case -4102 : return("FAPI_BOOT_ERR_MODE_UNKNOWN");
        case -5000 : return("FAPI_BM_MODULE_BASE");
        case -5001 : return("FAPI_BM_ERR_BAD_PARAMETER");
        case -5002 : return("FAPI_BM_ERR_OUT_OF_MEMORY");
        case -5003 : return("FAPI_BM_ERR_ALREADY_INITIALIZED");
        case -5004 : return("FAPI_BM_ERR_NOT_INITIALIZED");
        case -5005 : return("FAPI_BM_ERR_FEATURE_NOT_SUPPORTED");
        case -5006 : return("FAPI_BM_ERR_TIMEOUT");
        case -5007 : return("FAPI_BM_ERR_DEVICE_BUSY");
        case -5008 : return("FAPI_BM_ERR_INVALID_HANDLE");
        case -5009 : return("FAPI_BM_ERR_SEMAPHORE_CREATE");
        case -5010 : return("FAPI_BM_ERR_UNSUPPORTED_VERSION");
        case -5011 : return("FAPI_BM_ERR_MAILQUEUE_CREATE");
        case -5100 : return("FAPI_BM_ERR_PTR_OUTSIDE_BUFFER_BOUNDS");
        case -5101 : return("FAPI_BM_ERR_NO_MORE_HWCB_AVAILABLE");
        case -5102 : return("FAPI_BM_ERR_HWCB_USED_BY_OTHER_BM");
        case -5103 : return("FAPI_BM_ERR_NO_HWCB");
        case -5104 : return("FAPI_BM_ERR_AUD_NOT_ASSIGNED");
        case -5105 : return("FAPI_BM_ERR_POINTER_OUT_OF_BOUNDS");
        case -5106 : return("FAPI_BM_ERR_PURPOSE_ALREADY_ASSIGNED");
        case -5107 : return("FAPI_BM_ERR_INVALID_PURPOSE");
        case -5108 : return("FAPI_BM_ERR_PURPOSE_NOT_ASSIGNED");
        case -5109 : return("FAPI_BM_ERR_HIGH_LEVEL_IRQ_NOT_ASSIGNED");
        case -5110 : return("FAPI_BM_ERR_LOW_LEVEL_IRQ_NOT_ASSIGNED");
        case -6000 : return("FAPI_CLKPWR_MODULE_BASE");
        case -6001 : return("FAPI_CLKPWR_ERR_BAD_PARAMETER");
        case -6002 : return("FAPI_CLKPWR_ERR_OUT_OF_MEMORY");
        case -6003 : return("FAPI_CLKPWR_ERR_ALREADY_INITIALIZED");
        case -6004 : return("FAPI_CLKPWR_ERR_NOT_INITIALIZED");
        case -6005 : return("FAPI_CLKPWR_ERR_FEATURE_NOT_SUPPORTED");
        case -6006 : return("FAPI_CLKPWR_ERR_TIMEOUT");
        case -6007 : return("FAPI_CLKPWR_ERR_DEVICE_BUSY");
        case -6008 : return("FAPI_CLKPWR_ERR_INVALID_HANDLE");
        case -6009 : return("FAPI_CLKPWR_ERR_SEMAPHORE_CREATE");
        case -6010 : return("FAPI_CLKPWR_ERR_UNSUPPORTED_VERSION");
        case -6011 : return("FAPI_CLKPWR_ERR_MAILQUEUE_CREATE");
        case -6100 : return("FAPI_CLKPWR_ERR_INVALID_START_VALUE");
        case -7000 : return("FAPI_DESC_MODULE_BASE");
        case -7001 : return("FAPI_DESC_ERR_BAD_PARAMETER");
        case -7002 : return("FAPI_DESC_ERR_OUT_OF_MEMORY");
        case -7003 : return("FAPI_DESC_ERR_ALREADY_INITIALIZED");
        case -7004 : return("FAPI_DESC_ERR_NOT_INITIALIZED");
        case -7005 : return("FAPI_DESC_ERR_FEATURE_NOT_SUPPORTED");
        case -7006 : return("FAPI_DESC_ERR_TIMEOUT");
        case -7007 : return("FAPI_DESC_ERR_DEVICE_BUSY");
        case -7008 : return("FAPI_DESC_ERR_INVALID_HANDLE");
        case -7009 : return("FAPI_DESC_ERR_SEMAPHORE_CREATE");
        case -7010 : return("FAPI_DESC_ERR_UNSUPPORTED_VERSION");
        case -7011 : return("FAPI_DESC_ERR_MAILQUEUE_CREATE");
        case -7100 : return("FAPI_DESC_ERR_INVALID_STATUS");
        case -7101 : return("FAPI_DESC_ERR_INVALID_KEY_LENGTH");
        case -8000 : return("FAPI_DMA_MODULE_BASE");
        case -8001 : return("FAPI_DMA_ERR_BAD_PARAMETER");
        case -8002 : return("FAPI_DMA_ERR_OUT_OF_MEMORY");
        case -8003 : return("FAPI_DMA_ERR_ALREADY_INITIALIZED");
        case -8004 : return("FAPI_DMA_ERR_NOT_INITIALIZED");
        case -8005 : return("FAPI_DMA_ERR_FEATURE_NOT_SUPPORTED");
        case -8006 : return("FAPI_DMA_ERR_TIMEOUT");
        case -8007 : return("FAPI_DMA_ERR_DEVICE_BUSY");
        case -8008 : return("FAPI_DMA_ERR_INVALID_HANDLE");
        case -8009 : return("FAPI_DMA_ERR_SEMAPHORE_CREATE");
        case -8010 : return("FAPI_DMA_ERR_UNSUPPORTED_VERSION");
        case -8101 : return("FAPI_DMA_ERR_INIT_FAILED");
        case -8102 : return("FAPI_DMA_ERR_TOO_MANY_OPEN");
        case -8103 : return("FAPI_DMA_ERR_CONFIG_OVERFLOW");
        case -8104 : return("FAPI_DMA_ERR_FIFO_OVERFLOW");
        case -8105 : return("FAPI_DMA_ERR_BAD_FEATURE");
        case -8106 : return("FAPI_DMA_ERR_BAD_REQUEST");
        case -8107 : return("FAPI_DMA_ERR_STATUS");
        case -8108 : return("FAPI_DMA_ERR_CCIR_OVERFLOW");
        case -8109 : return("FAPI_DMA_ERR_AUDIO_OVERFLOW");
        case -8110 : return("FAPI_DMA_ERR_CANCELED");
        case -9000 : return("FAPI_ETH_MODULE_BASE");
        case -9001 : return("FAPI_ETH_ERR_BAD_PARAMETER");
        case -9002 : return("FAPI_ETH_ERR_OUT_OF_MEMORY");
        case -9003 : return("FAPI_ETH_ERR_ALREADY_INITIALIZED");
        case -9004 : return("FAPI_ETH_ERR_NOT_INITIALIZED");
        case -9005 : return("FAPI_ETH_ERR_FEATURE_NOT_SUPPORTED");
        case -9006 : return("FAPI_ETH_ERR_TIMEOUT");
        case -9007 : return("FAPI_ETH_ERR_DEVICE_BUSY");
        case -9008 : return("FAPI_ETH_ERR_INVALID_HANDLE");
        case -9009 : return("FAPI_ETH_ERR_SEMAPHORE_CREATE");
        case -9010 : return("FAPI_ETH_ERR_UNSUPPORTED_VERSION");
        case -9011 : return("FAPI_ETH_ERR_MAILQUEUE_CREATE");
        case -9100 : return("FAPI_ETH_ERR_NODEV");
        case -9101 : return("FAPI_ETH_ERR_DATACCESS");
        case -9102 : return("FAPI_ETH_ERR_RXSTOP");
        case -9103 : return("FAPI_ETH_ERR_TXSTOP");
        case -9104 : return("FAPI_ETH_ERR_NULL_PTR");
        case -9105 : return("FAPI_ETH_ERR_FATALSTOP");
        case -9106 : return("FAPI_ETH_ERR_TOO_BIG_FRAME");
        case -9107 : return("FAPI_ETH_ERR_TOO_SHORT_FRAME");
        case -9108 : return("FAPI_ETH_ERR_AGAIN");
        case -9109 : return("FAPI_ETH_ERR_LOCK_FAIL");
        case -9110 : return("FAPI_ETH_ERR_INV_ADDR");
        case -9111 : return("FAPI_ETH_ERR_PHY_BUSY");
        case -9112 : return("FAPI_ETH_ERR_UNKNOWN");
        case -9113 : return("FAPI_ETH_ERR_LINKDOWN");
        case -10000 : return("FAPI_FLASH_MODULE_BASE");
        case -10001 : return("FAPI_FLASH_ERR_BAD_PARAMETER");
        case -10002 : return("FAPI_FLASH_ERR_OUT_OF_MEMORY");
        case -10003 : return("FAPI_FLASH_ERR_ALREADY_INITIALIZED");
        case -10004 : return("FAPI_FLASH_ERR_NOT_INITIALIZED");
        case -10005 : return("FAPI_FLASH_ERR_FEATURE_NOT_SUPPORTED");
        case -10006 : return("FAPI_FLASH_ERR_TIMEOUT");
        case -10007 : return("FAPI_FLASH_ERR_DEVICE_BUSY");
        case -10008 : return("FAPI_FLASH_ERR_INVALID_HANDLE");
        case -10009 : return("FAPI_FLASH_ERR_SEMAPHORE_CREATE");
        case -10010 : return("FAPI_FLASH_ERR_UNSUPPORTED_VERSION");
        case -10011 : return("FAPI_FLASH_ERR_MAILQUEUE_CREATE");
        case -10101 : return("FAPI_FLASH_ERR_DEVICE_NOT_SUPPORTED");
        case -10102 : return("FAPI_FLASH_ERR_SECTOR_PROTECTED");
        case -10103 : return("FAPI_FLASH_ERR_NO_MAGIC_ID_FOUND");
        case -10104 : return("FAPI_FLASH_ERR_PROCESS_FAILED");
        case -10105 : return("FAPI_FLASH_ERR_PROTECTED");
        case -10106 : return("FAPI_FLASH_ERR_INVALID_FUNCTION");
        case -10107 : return("FAPI_FLASH_ERR_INVALID_DEVICE");
        case -10108 : return("FAPI_FLASH_ERR_EXT_ADDRESS_FAILED");
        case -10109 : return("FAPI_FLASH_ERR_OVERLAP_DETECTED");
        case -11000 : return("FAPI_FPC_MODULE_BASE");
        case -11001 : return("FAPI_FPC_ERR_BAD_PARAMETER");
        case -11002 : return("FAPI_FPC_ERR_OUT_OF_MEMORY");
        case -11003 : return("FAPI_FPC_ERR_ALREADY_INITIALIZED");
        case -11004 : return("FAPI_FPC_ERR_NOT_INITIALIZED");
        case -11005 : return("FAPI_FPC_ERR_FEATURE_NOT_SUPPORTED");
        case -11006 : return("FAPI_FPC_ERR_TIMEOUT");
        case -11007 : return("FAPI_FPC_ERR_DEVICE_BUSY");
        case -11008 : return("FAPI_FPC_ERR_INVALID_HANDLE");
        case -11009 : return("FAPI_FPC_ERR_SEMAPHORE_CREATE");
        case -11010 : return("FAPI_FPC_ERR_UNSUPPORTED_VERSION");
        case -11100 : return("FAPI_FPC_ERR_MODE_NOT_SUPPORTED");
        case -11101 : return("FAPI_FPC_DIGIT_CARRY");
        case -11102 : return("FAPI_FPC_BOUNDS_REACHED");
        case -11103 : return("FAPI_FPC_MAX_HANDLE_COUNT");
        case -12000 : return("FAPI_GPIO_MODULE_BASE");
        case -12001 : return("FAPI_GPIO_ERR_BAD_PARAMETER");
        case -12002 : return("FAPI_GPIO_ERR_OUT_OF_MEMORY");
        case -12003 : return("FAPI_GPIO_ERR_ALREADY_INITIALIZED");
        case -12004 : return("FAPI_GPIO_ERR_NOT_INITIALIZED");
        case -12005 : return("FAPI_GPIO_ERR_FEATURE_NOT_SUPPORTED");
        case -12006 : return("FAPI_GPIO_ERR_TIMEOUT");
        case -12007 : return("FAPI_GPIO_ERR_DEVICE_BUSY");
        case -12008 : return("FAPI_GPIO_ERR_INVALID_HANDLE");
        case -12009 : return("FAPI_GPIO_ERR_SEMAPHORE_CREATE");
        case -12010 : return("FAPI_GPIO_ERR_UNSUPPORTED_VERSION");
        case -12011 : return("FAPI_GPIO_ERR_MAILQUEUE_CREATE");
        case -12101 : return("FAPI_GPIO_ERR_PIN_NOT_FOUND");
        case -13000 : return("FAPI_GPREG_MODULE_BASE");
        case -13001 : return("FAPI_GPREG_ERR_BAD_PARAMETER");
        case -13002 : return("FAPI_GPREG_ERR_OUT_OF_MEMORY");
        case -13003 : return("FAPI_GPREG_ERR_ALREADY_INITIALIZED");
        case -13004 : return("FAPI_GPREG_ERR_NOT_INITIALIZED");
        case -13005 : return("FAPI_GPREG_ERR_FEATURE_NOT_SUPPORTED");
        case -13006 : return("FAPI_GPREG_ERR_TIMEOUT");
        case -13007 : return("FAPI_GPREG_ERR_DEVICE_BUSY");
        case -13008 : return("FAPI_GPREG_ERR_INVALID_HANDLE");
        case -13009 : return("FAPI_GPREG_ERR_SEMAPHORE_CREATE");
        case -13010 : return("FAPI_GPREG_ERR_UNSUPPORTED_VERSION");
        case -14000 : return("FAPI_HDMI_MODULE_BASE");
        case -14001 : return("FAPI_HDMI_ERR_BAD_PARAMETER");
        case -14002 : return("FAPI_HDMI_ERR_OUT_OF_MEMORY");
        case -14003 : return("FAPI_HDMI_ERR_ALREADY_INITIALIZED");
        case -14004 : return("FAPI_HDMI_ERR_NOT_INITIALIZED");
        case -14005 : return("FAPI_HDMI_ERR_FEATURE_NOT_SUPPORTED");
        case -14006 : return("FAPI_HDMI_ERR_TIMEOUT");
        case -14007 : return("FAPI_HDMI_ERR_DEVICE_BUSY");
        case -14008 : return("FAPI_HDMI_ERR_INVALID_HANDLE");
        case -14009 : return("FAPI_HDMI_ERR_SEMAPHORE_CREATE");
        case -14010 : return("FAPI_HDMI_ERR_UNSUPPORTED_VERSION");
        case -14100 : return("FAPI_HDMI_ERR_OPEN_PARAM");
        case -14101 : return("FAPI_HDMI_ERR_PARAMETER");
        case -14110 : return("FAPI_HDMI_ERR_I2C_OPEN");
        case -14120 : return("FAPI_HDMI_ERR_EDID");
        case -14121 : return("FAPI_HDMI_ERR_DDC_TIMEOUT");
        case -14130 : return("FAPI_HDMI_ERR_HDCP_TIMEOUT");
        case -14131 : return("FAPI_HDMI_ERR_HDCP_MAX_DEVS_EXCEED");
        case -14132 : return("FAPI_HDMI_ERR_HDCP_MAX_CASCADE_EXCEEDED");
        case -14133 : return("FAPI_HDMI_ERR_HDCP_SHA_1_MISMATCH");
        case -14134 : return("FAPI_HDMI_ERR_HDCP_AN_DDC");
        case -14135 : return("FAPI_HDMI_ERR_HDCP_AKSV_DDC");
        case -14136 : return("FAPI_HDMI_ERR_HDCP_BKSV");
        case -14137 : return("FAPI_HDMI_ERR_HDCP_RI_FAILURE");
        case -14138 : return("FAPI_HDMI_ERR_HDCP_RI_NOT_ABAILABLE");
        case -14140 : return("FAPI_HDMI_ERR_HPD_NOT_DETECT");
        case -14141 : return("FAPI_HDMI_ERR_EXIST_IN_REVOCATION_LISTS");
        case -14142 : return("FAPI_HDMI_ERR_HDCP_NOT_SUPPORT");
        case -14143 : return("FAPI_HDMI_ERR_HDCP_AKSV");
        case -14151 : return("FAPI_HDMI_ERR_CEC_BUSY");
        case -14152 : return("FAPI_HDMI_ERR_CEC_SENDING");
        case -14153 : return("FAPI_HDMI_ERR_CEC_GPIO");
        case -14154 : return("FAPI_HDMI_ERR_CEC_MESSAGE_BUFFER_FULL");
        case -14155 : return("FAPI_HDMI_ERR_CEC_MESSAGE_BUFFER_EMPTY");
        case -14156 : return("FAPI_HDMI_ERR_CEC_RECEIVE_PARAMETER");
        case -14157 : return("FAPI_HDMI_ERR_CEC_SEND_PARAMETER");
        case -14158 : return("FAPI_HDMI_ERR_CEC_TIMER");
        case -14159 : return("FAPI_HDMI_ERR_CEC_OPEN");
        case -15000 : return("FAPI_I2C_MODULE_BASE");
        case -15001 : return("FAPI_I2C_ERR_BAD_PARAMETER");
        case -15002 : return("FAPI_I2C_ERR_OUT_OF_MEMORY");
        case -15003 : return("FAPI_I2C_ERR_ALREADY_INITIALIZED");
        case -15004 : return("FAPI_I2C_ERR_NOT_INITIALIZED");
        case -15005 : return("FAPI_I2C_ERR_FEATURE_NOT_SUPPORTED");
        case -15006 : return("FAPI_I2C_ERR_TIMEOUT");
        case -15007 : return("FAPI_I2C_ERR_DEVICE_BUSY");
        case -15008 : return("FAPI_I2C_ERR_INVALID_HANDLE");
        case -15009 : return("FAPI_I2C_ERR_SEMAPHORE_CREATE");
        case -15010 : return("FAPI_I2C_ERR_UNSUPPORTED_VERSION");
        case -15011 : return("FAPI_I2C_ERR_MAILQUEUE_CREATE");
        case -15100 : return("FAPI_I2C_ERR_BUSY");
        case -15101 : return("FAPI_I2C_ERR_BUS_OCCUP");
        case -15102 : return("FAPI_I2C_ERR_SL_STOP_REC");
        case -15103 : return("FAPI_I2C_ERR_RC_NACK");
        case -15104 : return("FAPI_I2C_ERR_BE_STOP");
        case -15105 : return("FAPI_I2C_ERR_BE_START");
        case -15106 : return("FAPI_I2C_ERR_AL_ADRSTA");
        case -15107 : return("FAPI_I2C_ERR_AL_DATA");
        case -15108 : return("FAPI_I2C_ERR_AL_STOP");
        case -15109 : return("FAPI_I2C_ERR_SL_NACK");
        case -15110 : return("FAPI_I2C_ERR_MS_STOP_REC");
        case -15111 : return("FAPI_I2C_ERR_AMDMA");
        case -15112 : return("FAPI_I2C_ERR_UNSUPPORTED");
        case -15113 : return("FAPI_I2C_ERR_GPIO_FAILED");
        case -15114 : return("FAPI_I2C_ERR_TIMER_FAILED");
        case -16000 : return("FAPI_ICC_MODULE_BASE");
        case -16001 : return("FAPI_ICC_ERR_BAD_PARAMETER");
        case -16002 : return("FAPI_ICC_ERR_OUT_OF_MEMORY");
        case -16003 : return("FAPI_ICC_ERR_ALREADY_INITIALIZED");
        case -16004 : return("FAPI_ICC_ERR_NOT_INITIALIZED");
        case -16005 : return("FAPI_ICC_ERR_FEATURE_NOT_SUPPORTED");
        case -16006 : return("FAPI_ICC_ERR_TIMEOUT");
        case -16007 : return("FAPI_ICC_ERR_DEVICE_BUSY");
        case -16008 : return("FAPI_ICC_ERR_INVALID_HANDLE");
        case -16009 : return("FAPI_ICC_ERR_SEMAPHORE_CREATE");
        case -16010 : return("FAPI_ICC_ERR_UNSUPPORTED_VERSION");
        case -16100 : return("FAPI_ICC_ERR_SLOT_EMPTY");
        case -16101 : return("FAPI_ICC_ERR_RUNNING");
        case -16102 : return("FAPI_ICC_ERR_OVERFLOW");
        case -16103 : return("FAPI_ICC_ERR_NO_ANSWER");
        case -16104 : return("FAPI_ICC_ERR_NO_BAUDRATE");
        case -16105 : return("FAPI_ICC_ERR_UNKNOWN");
        case -16106 : return("FAPI_ICC_ERR_NO_PROTOCOL");
        case -16107 : return("FAPI_ICC_ERR_WRITE_FAIL");
        case -16108 : return("FAPI_ICC_ERR_READ_FAIL");
        case -16109 : return("FAPI_ICC_ERR_BUFFER_OVERFLOW");
        case -16110 : return("FAPI_ICC_ERR_OUTOFRANGE");
        case -17000 : return("FAPI_INTR_MODULE_BASE");
        case -17001 : return("FAPI_INTR_ERR_BAD_PARAMETER");
        case -17002 : return("FAPI_INTR_ERR_OUT_OF_MEMORY");
        case -17003 : return("FAPI_INTR_ERR_ALREADY_INITIALIZED");
        case -17004 : return("FAPI_INTR_ERR_NOT_INITIALIZED");
        case -17005 : return("FAPI_INTR_ERR_FEATURE_NOT_SUPPORTED");
        case -17006 : return("FAPI_INTR_ERR_TIMEOUT");
        case -17007 : return("FAPI_INTR_ERR_DEVICE_BUSY");
        case -17008 : return("FAPI_INTR_ERR_INVALID_HANDLE");
        case -17009 : return("FAPI_INTR_ERR_SEMAPHORE_CREATE");
        case -17010 : return("FAPI_INTR_ERR_UNSUPPORTED_VERSION");
        case -18000 : return("FAPI_IR_MODULE_BASE");
        case -18001 : return("FAPI_IR_ERR_BAD_PARAMETER");
        case -18002 : return("FAPI_IR_ERR_OUT_OF_MEMORY");
        case -18003 : return("FAPI_IR_ERR_ALREADY_INITIALIZED");
        case -18004 : return("FAPI_IR_ERR_NOT_INITIALIZED");
        case -18005 : return("FAPI_IR_ERR_FEATURE_NOT_SUPPORTED");
        case -18006 : return("FAPI_IR_ERR_TIMEOUT");
        case -18007 : return("FAPI_IR_ERR_DEVICE_BUSY");
        case -18008 : return("FAPI_IR_ERR_INVALID_HANDLE");
        case -18009 : return("FAPI_IR_ERR_SEMAPHORE_CREATE");
        case -18010 : return("FAPI_IR_ERR_UNSUPPORTED_VERSION");
        case -18100 : return("FAPI_IR_ERR_NO_SEQUENCE");
        case -18101 : return("FAPI_IR_ERR_INVAL_SEQ");
        case -18102 : return("FAPI_IR_ERR_UNSUPPORTED_MODE");
        case -18103 : return("FAPI_IR_ERR_HARDWARE_MODE");
        case -18104 : return("FAPI_IR_ERR_PARAM_OUTOFRANGE");
        case -19000 : return("FAPI_MMU_MODULE_BASE");
        case -19001 : return("FAPI_MMU_ERR_BAD_PARAMETER");
        case -19002 : return("FAPI_MMU_ERR_OUT_OF_MEMORY");
        case -19003 : return("FAPI_MMU_ERR_ALREADY_INITIALIZED");
        case -19004 : return("FAPI_MMU_ERR_NOT_INITIALIZED");
        case -19005 : return("FAPI_MMU_ERR_FEATURE_NOT_SUPPORTED");
        case -19008 : return("FAPI_MMU_ERR_INVALID_HANDLE");
        case -19009 : return("FAPI_MMU_ERR_SEMAPHORE_CREATE");
        case -19010 : return("FAPI_MMU_ERR_UNSUPPORTED_VERSION");
        case -19101 : return("FAPI_MMU_ERR_SEGMENT_CREATE");
        case -20000 : return("FAPI_SSP_MODULE_BASE");
        case -20001 : return("FAPI_SSP_ERR_BAD_PARAMETER");
        case -20002 : return("FAPI_SSP_ERR_OUT_OF_MEMORY");
        case -20003 : return("FAPI_SSP_ERR_ALREADY_INITIALIZED");
        case -20004 : return("FAPI_SSP_ERR_NOT_INITIALIZED");
        case -20005 : return("FAPI_SSP_ERR_FEATURE_NOT_SUPPORTED");
        case -20006 : return("FAPI_SSP_ERR_TIMEOUT");
        case -20007 : return("FAPI_SSP_ERR_DEVICE_BUSY");
        case -20008 : return("FAPI_SSP_ERR_INVALID_HANDLE");
        case -20009 : return("FAPI_SSP_ERR_SEMAPHORE_CREATE");
        case -20010 : return("FAPI_SSP_ERR_UNSUPPORTED_VERSION");
        case -20100 : return("FAPI_SSP_ERR_BUSY");
        case -20101 : return("FAPI_SSP_ERR_NO_DATA");
        case -21000 : return("FAPI_SYNC_MODULE_BASE");
        case -21001 : return("FAPI_SYNC_ERR_BAD_PARAMETER");
        case -21002 : return("FAPI_SYNC_ERR_OUT_OF_MEMORY");
        case -21003 : return("FAPI_SYNC_ERR_ALREADY_INITIALIZED");
        case -21004 : return("FAPI_SYNC_ERR_NOT_INITIALIZED");
        case -21005 : return("FAPI_SYNC_ERR_FEATURE_NOT_SUPPORTED");
        case -21006 : return("FAPI_SYNC_ERR_TIMEOUT");
        case -21007 : return("FAPI_SYNC_ERR_DEVICE_BUSY");
        case -21008 : return("FAPI_SYNC_ERR_INVALID_HANDLE");
        case -21009 : return("FAPI_SYNC_ERR_SEMAPHORE_CREATE");
        case -21010 : return("FAPI_SYNC_ERR_UNSUPPORTED_VERSION");
        case -21011 : return("FAPI_SYNC_ERR_MAILQUEUE_CREATE");
        case -21100 : return("FAPI_SYNC_ERR_PCR_NOT_AVAILABLE");
        case -21101 : return("FAPI_SYNC_ERR_STC_NOT_AVAILABLE");
        case -21102 : return("FAPI_SYNC_ERR_PCR_PID_FAILED");
        case -21103 : return("FAPI_SYNC_ERR_INVALID_STATE");
        case -21104 : return("FAPI_SYNC_ERR_INVALID_DECODER_OFFSET");
        case -22000 : return("FAPI_TIMER_MODULE_BASE");
        case -22001 : return("FAPI_TIMER_ERR_BAD_PARAMETER");
        case -22002 : return("FAPI_TIMER_ERR_OUT_OF_MEMORY");
        case -22003 : return("FAPI_TIMER_ERR_ALREADY_INITIALIZED");
        case -22004 : return("FAPI_TIMER_ERR_NOT_INITIALIZED");
        case -22005 : return("FAPI_TIMER_ERR_FEATURE_NOT_SUPPORTED");
        case -22006 : return("FAPI_TIMER_ERR_TIMEOUT");
        case -22007 : return("FAPI_TIMER_ERR_DEVICE_BUSY");
        case -22008 : return("FAPI_TIMER_ERR_INVALID_HANDLE");
        case -22009 : return("FAPI_TIMER_ERR_SEMAPHORE_CREATE");
        case -22010 : return("FAPI_TIMER_ERR_UNSUPPORTED_VERSION");
        case -22011 : return("FAPI_TIMER_ERR_MAILQUEUE_CREATE");
        case -23000 : return("FAPI_TSD_MODULE_BASE");
        case -23001 : return("FAPI_TSD_ERR_BAD_PARAMETER");
        case -23002 : return("FAPI_TSD_ERR_OUT_OF_MEMORY");
        case -23003 : return("FAPI_TSD_ERR_ALREADY_INITIALIZED");
        case -23004 : return("FAPI_TSD_ERR_NOT_INITIALIZED");
        case -23005 : return("FAPI_TSD_ERR_FEATURE_NOT_SUPPORTED");
        case -23006 : return("FAPI_TSD_ERR_TIMEOUT");
        case -23007 : return("FAPI_TSD_ERR_DEVICE_BUSY");
        case -23008 : return("FAPI_TSD_ERR_INVALID_HANDLE");
        case -23009 : return("FAPI_TSD_ERR_SEMAPHORE_CREATE");
        case -23010 : return("FAPI_TSD_ERR_UNSUPPORTED_VERSION");
        case -23011 : return("FAPI_TSD_ERR_MAILQUEUE_CREATE");
        case -23100 : return("FAPI_TSD_ERR_INVALID_PID");
        case -23101 : return("FAPI_TSD_ERR_NO_PID_ASSIGNED");
        case -23102 : return("FAPI_TSD_ERR_PES_PARSER_COUNT_EXEEDED");
        case -23103 : return("FAPI_TSD_ERR_NO_RECONFIGURE");
        case -23104 : return("FAPI_TSD_ERR_INVALID_FILTER_SETTINGS");
        case -23105 : return("FAPI_TSD_ERR_PID_ALREADY_ASSIGNED");
        case -23106 : return("FAPI_TSD_ERR_NO_MAPPED_BUFFER");
        case -23107 : return("FAPI_TSD_ERR_NOTIFIER_ALREADY_ASSIGNED");
        case -23108 : return("FAPI_TSD_ERR_GPIO_FAILED");
        case -23109 : return("FAPI_TSD_ERR_NO_FILTER_ASSIGNED");
        case -23110 : return("FAPI_TSD_ERR_NO_PES_PARSER_ASSIGNED");
        case -23111 : return("FAPI_TSD_ERR_SYNC_DISABLED");
        case -23112 : return("FAPI_TSD_ERR_STC_SELECT");
        case -23113 : return("FAPI_TSD_ERR_PID_ACTIVE");
        case -23114 : return("FAPI_TSD_ERR_EVALUATOR_ALREADY_ASSIGNED");
        case -24000 : return("FAPI_TTX_MODULE_BASE");
        case -24001 : return("FAPI_TTX_ERR_BAD_PARAMETER");
        case -24002 : return("FAPI_TTX_ERR_OUT_OF_MEMORY");
        case -24003 : return("FAPI_TTX_ERR_ALREADY_INITIALIZED");
        case -24004 : return("FAPI_TTX_ERR_NOT_INITIALIZED");
        case -24005 : return("FAPI_TTX_ERR_FEATURE_NOT_SUPPORTED");
        case -24006 : return("FAPI_TTX_ERR_TIMEOUT");
        case -24007 : return("FAPI_TTX_ERR_DEVICE_BUSY");
        case -24008 : return("FAPI_TTX_ERR_INVALID_HANDLE");
        case -24009 : return("FAPI_TTX_ERR_SEMAPHORE_CREATE");
        case -24010 : return("FAPI_TTX_ERR_UNSUPPORTED_VERSION");
        case -24011 : return("FAPI_TTX_ERR_MAILQUEUE_CREATE");
        case -24100 : return("FAPI_TTX_ERR_NO_PID_ASSIGNED");
        case -24101 : return("FAPI_TTX_ERR_NOT_SYNCHRONISED");
        case -24102 : return("FAPI_TTX_ERR_BM_ERROR");
        case -25000 : return("FAPI_UART_MODULE_BASE");
        case -25001 : return("FAPI_UART_ERR_BAD_PARAMETER");
        case -25002 : return("FAPI_UART_ERR_OUT_OF_MEMORY");
        case -25003 : return("FAPI_UART_ERR_ALREADY_INITIALIZED");
        case -25004 : return("FAPI_UART_ERR_NOT_INITIALIZED");
        case -25005 : return("FAPI_UART_ERR_FEATURE_NOT_SUPPORTED");
        case -25006 : return("FAPI_UART_ERR_TIMEOUT");
        case -25007 : return("FAPI_UART_ERR_DEVICE_BUSY");
        case -25008 : return("FAPI_UART_ERR_INVALID_HANDLE");
        case -25009 : return("FAPI_UART_ERR_SEMAPHORE_CREATE");
        case -25010 : return("FAPI_UART_ERR_UNSUPPORTED_VERSION");
        case -25011 : return("FAPI_UART_ERR_MAILQUEUE_CREATE");
        case -25101 : return("FAPI_UART_ERR_FIFO_OVERFLOW");
        case -26000 : return("FAPI_UPI_MODULE_BASE");
        case -26001 : return("FAPI_UPI_ERR_BAD_PARAMETER");
        case -26002 : return("FAPI_UPI_ERR_OUT_OF_MEMORY");
        case -26003 : return("FAPI_UPI_ERR_ALREADY_INITIALIZED");
        case -26004 : return("FAPI_UPI_ERR_NOT_INITIALIZED");
        case -26005 : return("FAPI_UPI_ERR_FEATURE_NOT_SUPPORTED");
        case -26006 : return("FAPI_UPI_ERR_TIMEOUT");
        case -26007 : return("FAPI_UPI_ERR_DEVICE_BUSY");
        case -26008 : return("FAPI_UPI_ERR_INVALID_HANDLE");
        case -26009 : return("FAPI_UPI_ERR_SEMAPHORE_CREATE");
        case -26010 : return("FAPI_UPI_ERR_UNSUPPORTED_VERSION");
        case -27000 : return("FAPI_USB_MODULE_BASE");
        case -27001 : return("FAPI_USB_ERR_BAD_PARAMETER");
        case -27002 : return("FAPI_USB_ERR_OUT_OF_MEMORY");
        case -27003 : return("FAPI_USB_ERR_ALREADY_INITIALIZED");
        case -27004 : return("FAPI_USB_ERR_NOT_INITIALIZED");
        case -27005 : return("FAPI_USB_ERR_FEATURE_NOT_SUPPORTED");
        case -27006 : return("FAPI_USB_ERR_TIMEOUT");
        case -27007 : return("FAPI_USB_ERR_DEVICE_BUSY");
        case -27008 : return("FAPI_USB_ERR_INVALID_HANDLE");
        case -27009 : return("FAPI_USB_ERR_SEMAPHORE_CREATE");
        case -27010 : return("FAPI_USB_ERR_UNSUPPORTED_VERSION");
        case -27100 : return("FAPI_USB_ERR_WRONG_ID");
        case -27101 : return("FAPI_USB_ERR_ALREADY_OPEN");
        case -27102 : return("FAPI_USB_ERR_TOO_MANY_DRIVERS");
        case -27103 : return("FAPI_USB_ERR_ALREADY_RUNNING");
        case -27104 : return("FAPI_USB_ERR_NOT_RUNNING");
        case -27105 : return("FAPI_USB_ERR_MUSB_START");
        case -28000 : return("FAPI_VIDEC_MODULE_BASE");
        case -28001 : return("FAPI_VIDEC_ERR_BAD_PARAMETER");
        case -28002 : return("FAPI_VIDEC_ERR_OUT_OF_MEMORY");
        case -28003 : return("FAPI_VIDEC_ERR_ALREADY_INITIALIZED");
        case -28004 : return("FAPI_VIDEC_ERR_NOT_INITIALIZED");
        case -28005 : return("FAPI_VIDEC_ERR_FEATURE_NOT_SUPPORTED");
        case -28006 : return("FAPI_VIDEC_ERR_TIMEOUT");
        case -28007 : return("FAPI_VIDEC_ERR_DEVICE_BUSY");
        case -28008 : return("FAPI_VIDEC_ERR_INVALID_HANDLE");
        case -28009 : return("FAPI_VIDEC_ERR_SEMAPHORE_CREATE");
        case -28010 : return("FAPI_VIDEC_ERR_UNSUPPORTED_VERSION");
        case -28101 : return("FAPI_VIDEC_ERR_DRV_VERSION_CHECK_FAILED");
        case -28102 : return("FAPI_VIDEC_ERR_INVALID_OPEN_PARAMS");
        case -28103 : return("FAPI_VIDEC_ERR_INVALID_COMBINATION_OF_OPEN_PARAMS");
        case -28104 : return("FAPI_VIDEC_ERR_EXCEEDED_MAX_HANDLES");
        case -28105 : return("FAPI_VIDEC_ERR_ISR_STATE_MACHINE_ALLOCATION_FAILURE");
        case -28106 : return("FAPI_VIDEC_ERR_UNSUPPORTED_FUNCTIONALITY");
        case -28107 : return("FAPI_VIDEC_ERR_HANDLE_CHECK_FAILED");
        case -28108 : return("FAPI_VIDEC_ERR_INVALID_ARGUEMENT_LIST");
        case -28109 : return("FAPI_VIDEC_ERR_FIRMWARE_FLASH_FAILURE");
        case -28110 : return("FAPI_VIDEC_ERR_VIDEC_FIRMWARE_VERSION_MISMATCH");
        case -28111 : return("FAPI_VIDEC_ERR_VIDEC_FIRMWARE_NOT_FOUND");
        case -28112 : return("FAPI_VIDEC_ERR_BOOT_TIMEOUT");
        case -28113 : return("FAPI_VIDEC_ERR_START_TIMEOUT");
        case -28114 : return("FAPI_VIDEC_ERR_STOP_TIMEOUT");
        case -28115 : return("FAPI_VIDEC_ERR_RESET_TIMEOUT");
        case -28116 : return("FAPI_VIDEC_ERR_ENABLE_DISABLE_TIMEOUT");
        case -28117 : return("FAPI_VIDEC_ERR_UNSUPPORTED_STREAM");
        case -28118 : return("FAPI_VIDEC_ERR_CALLBACK_ALREADY_ASSIGNED");
        case -28119 : return("FAPI_VIDEC_ERR_VIDEO_DECODER_NOT_RUNNING");
        case -28120 : return("FAPI_VIDEC_ERR_WRONG_HANDLE");
        case -28121 : return("FAPI_VIDEC_ERR_INVALID_INITIALIZATION_PARAMS");
        case -28122 : return("FAPI_VIDEC_ERR_DIRECT_IFRAME_EXCEEDS_ES_BUFF");
        case -28123 : return("FAPI_VIDEC_ERR_DIRECT_IFRAME_WITH_WRONG_HEADER");
        case -28124 : return("FAPI_VIDEC_ERR_DIRECT_IFRAME_NO_DECODED_FRAME_AVAILABLE");
        case -28125 : return("FAPI_VIDEC_ERR_ISR_ASSOCIATION_FAILURE");
        case -28126 : return("FAPI_VIDEC_ERR_VIDEO_DECODER_NOT_BOOTED");
        case -28127 : return("FAPI_VIDEC_ERR_VES_BUFFER_OVERFLOW");
        case -28128 : return("FAPI_VIDEC_ERR_INCORRECT_PLAYBACK_ENVIRONMENT");
        case -28129 : return("FAPI_VIDEC_ERR_VES_PARAMS_NOT_AVAILABLE");
        case -28130 : return("FAPI_VIDEC_ERR_VIDEO_MEMORY_SEGMENT_ALLOCATION_FAILURE");
        case -28131 : return("FAPI_VIDEC_ERR_VIDEO_MEMORY_CHUNK_ALLOCATION_FAILURE");
        case -28132 : return("FAPI_VIDEC_ERR_VIDEO_MEMORY_CHUNK_VIRTUAL_ADDRESS_UNAVAILABLE");
        case -28133 : return("FAPI_VIDEC_ERR_VIDEO_MEMORY_CHUNK_PHYSICAL_ADDRESS_UNAVAILABLE");
        case -28134 : return("FAPI_VIDEC_ERR_VIDEO_MEMORY_SEGMENT_STATIC_ADDRESS_INVALID");
        case -28135 : return("FAPI_VIDEC_ERR_VIDEO_DRIVER_PREINITIALIZATION_FAILURE");
        case -28136 : return("FAPI_VIDEC_ERR_VIDEO_WRONG_MODE");
        case -28137 : return("FAPI_VIDEC_ERR_VIDEO_BOARD_MEMORY_NOT_REGISTERED");
        case -28138 : return("FAPI_VIDEC_ERR_VIDEO_BOARD_MEMORY_REGISTRATION_FAILURE");
        case -28139 : return("FAPI_VIDEC_ERR_INCORRECT_PLAYBACK_MODE");
        case -29000 : return("FAPI_VIENC_MODULE_BASE");
        case -29001 : return("FAPI_VIENC_ERR_BAD_PARAMETER");
        case -29002 : return("FAPI_VIENC_ERR_OUT_OF_MEMORY");
        case -29003 : return("FAPI_VIENC_ERR_ALREADY_INITIALIZED");
        case -29004 : return("FAPI_VIENC_ERR_NOT_INITIALIZED");
        case -29005 : return("FAPI_VIENC_ERR_FEATURE_NOT_SUPPORTED");
        case -29006 : return("FAPI_VIENC_ERR_TIMEOUT");
        case -29007 : return("FAPI_VIENC_ERR_DEVICE_BUSY");
        case -29008 : return("FAPI_VIENC_ERR_INVALID_HANDLE");
        case -29009 : return("FAPI_VIENC_ERR_SEMAPHORE_CREATE");
        case -29010 : return("FAPI_VIENC_ERR_UNSUPPORTED_VERSION");
        case -29101 : return("FAPI_VIENC_ERR_DRV_VERSION_CHECK_FAILED");
        case -29102 : return("FAPI_VIENC_ERR_INVALID_OPEN_PARAMS");
        case -29103 : return("FAPI_VIENC_ERR_INVALID_COMBINATION_OF_OPEN_PARAMS");
        case -29104 : return("FAPI_VIENC_ERR_EXCEEDED_MAX_HANDLES");
        case -29105 : return("FAPI_VIENC_ERR_UNSUPPORTED_FUNCTIONALITY");
        case -29106 : return("FAPI_VIENC_ERR_HANDLE_CHECK_FAILED");
        case -29107 : return("FAPI_VIENC_ERR_HANDLE_OPERATION_BLOCKED");
        case -29108 : return("FAPI_VIENC_ERR_UNKNOWN_DEVICE");
        case -29109 : return("FAPI_VIENC_ERR_INVALID_ARGUEMENT_LIST");
        case -29110 : return("FAPI_VIENC_ERR_RGB_GPIOS_ALLOCATION_FAILURE");
        case -29111 : return("FAPI_VIENC_ERR_HSYNC_GPIOS_ALLOCATION_FAILURE");
        case -29112 : return("FAPI_VIENC_ERR_VSYNC_GPIOS_ALLOCATION_FAILURE");
        case -29113 : return("FAPI_VIENC_ERR_CLOCK_GPIOS_ALLOCATION_FAILURE");
        case -29114 : return("FAPI_VIENC_ERR_UNDEFINED_PLATFORM");
        case -30000 : return("FAPI_VISCALE_MODULE_BASE");
        case -30001 : return("FAPI_VISCALE_ERR_BAD_PARAMETER");
        case -30002 : return("FAPI_VISCALE_ERR_OUT_OF_MEMORY");
        case -30003 : return("FAPI_VISCALE_ERR_ALREADY_INITIALIZED");
        case -30004 : return("FAPI_VISCALE_ERR_NOT_INITIALIZED");
        case -30005 : return("FAPI_VISCALE_ERR_FEATURE_NOT_SUPPORTED");
        case -30006 : return("FAPI_VISCALE_ERR_TIMEOUT");
        case -30007 : return("FAPI_VISCALE_ERR_DEVICE_BUSY");
        case -30008 : return("FAPI_VISCALE_ERR_INVALID_HANDLE");
        case -30009 : return("FAPI_VISCALE_ERR_SEMAPHORE_CREATE");
        case -30010 : return("FAPI_VISCALE_ERR_UNSUPPORTED_VERSION");
        case -30101 : return("FAPI_VISCALE_ERR_DRV_VERSION_CHECK_FAILED");
        case -30102 : return("FAPI_VISCALE_ERR_INVALID_INITIALIZATION_PARAMS");
        case -30103 : return("FAPI_VISCALE_ERR_INVALID_OPEN_PARAMS");
        case -30104 : return("FAPI_VISCALE_ERR_INVALID_SCALER_PARAMS");
        case -30105 : return("FAPI_VISCALE_ERR_INVALID_COMBINATION_OF_OPEN_PARAMS");
        case -30106 : return("FAPI_VISCALE_ERR_EXCEEDED_MAX_HANDLES");
        case -30107 : return("FAPI_VISCALE_ERR_UNSUPPORTED_FUNCTIONALITY");
        case -30108 : return("FAPI_VISCALE_ERR_HANDLE_CHECK_FAILED");
        case -30109 : return("FAPI_VISCALE_ERR_HANDLE_OPERATION_BLOCKED");
        case -30110 : return("FAPI_VISCALE_ERR_WRONG_HANDLE");
        case -30111 : return("FAPI_VISCALE_ERR_OPERATION_TIMEOUT");
        case -30112 : return("FAPI_VISCALE_ERR_EXCEEDED_MAX_VALUE");
        case -30113 : return("FAPI_VISCALE_ERR_ISR_STATE_MACHINE_ALLOCATION_FAILURE");
        case -30114 : return("FAPI_VISCALE_ERR_INVALID_SCALING_METHOD");
        case -30115 : return("FAPI_VISCALE_ERR_PARAMETER_RANGE_EXCEEDED");
        case -30116 : return("FAPI_VISCALE_ERR_UNDEFINED_PLATFORM");
        case -30117 : return("FAPI_VISCALE_ERR_INVALID_ARGUEMENT_LIST");
        case -31000 : return("FAPI_BOARD_MODULE_BASE");
        case -31001 : return("FAPI_BOARD_ERR_BAD_PARAMETER");
        case -31002 : return("FAPI_BOARD_ERR_OUT_OF_MEMORY");
        case -31003 : return("FAPI_BOARD_ERR_ALREADY_INITIALIZED");
        case -31004 : return("FAPI_BOARD_ERR_NOT_INITIALIZED");
        case -31005 : return("FAPI_BOARD_ERR_FEATURE_NOT_SUPPORTED");
        case -31006 : return("FAPI_BOARD_ERR_TIMEOUT");
        case -31007 : return("FAPI_BOARD_ERR_DEVICE_BUSY");
        case -31008 : return("FAPI_BOARD_ERR_INVALID_HANDLE");
        case -31009 : return("FAPI_BOARD_ERR_SEMAPHORE_CREATE");
        case -31010 : return("FAPI_BOARD_ERR_UNSUPPORTED_VERSION");
        case -32000 : return("FAPI_BOARDS_MODULE_BASE");
        case -1000000 : return("RTOS_SHARED_MODULE_BASE");
        case -1000101 : return("RTOS_ERR_NOT_INITIALISED");
        case -1000102 : return("RTOS_ERR_ALREADY_INITIALIZED");
        case -1000103 : return("RTOS_ERR_TABLE_OVERFLOW");
        case -1000104 : return("RTOS_ERR_INVALID_HANDLE");
        case -1000105 : return("RTOS_ERR_INVALID_PARAMETER");
        case -1000106 : return("RTOS_ERR_TIMEOUT");
        case -1000107 : return("RTOS_ERR_OUT_OF_MEMORY");
        case -1000108 : return("RTOS_ERR_NOT_ALLOWED");
        case -1000109 : return("RTOS_ERR_HANDLE_DESTROYED");
        case -1100000 : return("RTOS_FAMOS_MODULE_BASE");
        case -1100101 : return("FAMOS_ERR_NOT_INITIALISED");
        case -1100102 : return("FAMOS_ERR_ALREADY_INITIALIZED");
        case -1100103 : return("FAMOS_ERR_TABLE_OVERFLOW");
        case -1100104 : return("FAMOS_ERR_INVALID_HANDLE");
        case -1100105 : return("FAMOS_ERR_INVALID_PARAMETER");
        case -1100106 : return("FAMOS_ERR_TIMEOUT");
        case -1100107 : return("FAMOS_ERR_OUT_OF_MEMORY");
        case -1100108 : return("FAMOS_ERR_NOT_ALLOWED");
        case -1100109 : return("FAMOS_ERR_HANDLE_DESTROYED");
        case -1200000 : return("RTOS_STUB_MODULE_BASE");
        case -1200001 : return("RTOS_STUB_ERR_BAD_PARAMETER");
        case -1200002 : return("RTOS_STUB_ERR_OUT_OF_MEMORY");
        case -1200003 : return("RTOS_STUB_ERR_ALREADY_INITIALIZED");
        case -1200004 : return("RTOS_STUB_ERR_NOT_INITIALIZED");
        case -1200005 : return("RTOS_STUB_ERR_FEATURE_NOT_SUPPORTED");
        case -1200006 : return("RTOS_STUB_ERR_TIMEOUT");
        case -1200007 : return("RTOS_STUB_ERR_DEVICE_BUSY");
        case -1200008 : return("RTOS_STUB_ERR_INVALID_HANDLE");
        case -1200009 : return("RTOS_STUB_ERR_SEMAPHORE_CREATE");
        case -1200010 : return("RTOS_STUB_ERR_UNSUPPORTED_VERSION");
        case -2000000 : return("SUBSYS_FE_MODULE_BASE");
        case -2000001 : return("FE_ERR_BAD_PARAMETER");
        case -2000002 : return("FE_ERR_OUT_OF_MEMORY");
        case -2000003 : return("FE_ERR_ALREADY_INITIALIZED");
        case -2000004 : return("FE_ERR_NOT_INITIALIZED");
        case -2000005 : return("FE_ERR_FEATURE_NOT_SUPPORTED");
        case -2000006 : return("FE_ERR_TIMEOUT");
        case -2000007 : return("FE_ERR_DEVICE_BUSY");
        case -2000008 : return("FE_ERR_INVALID_HANDLE");
        case -2000009 : return("FE_ERR_SEMAPHORE_CREATE");
        case -2000010 : return("FE_ERR_UNSUPPORTED_VERSION");
        case -2000101 : return("FE_ERR_FE_ALL_OPEN");
        case -2000102 : return("FE_ERR_FE_OPEN");
        case -2000103 : return("FE_ERR_FE_TUNE");
        case -2000104 : return("FE_ERR_FE_BUSY");
        case -2000105 : return("FE_ERR_FE_GPIO");
        case -2000106 : return("FE_ERR_FE_I2C");
        case -2000107 : return("FE_ERR_FE_TIMER");
        case -2000108 : return("FE_ERR_FE_NOT_LOCKED");
        case -2000109 : return("FE_ERR_FE_OTHER");
        case -2000110 : return("FE_ERR_FE_UNKNOWN_DEVICE");
        case -2000111 : return("FE_ERR_FE_UPI_CONFIGURATION");
        case -2100000 : return("SUBSYS_FE_CE6353_MODULE_BASE");
        case -2200000 : return("SUBSYS_FE_DSF89000_MODULE_BASE");
        case -2300000 : return("SUBSYS_FE_MB86668_MODULE_BASE");
        case -2400000 : return("SUBSYS_PAINT_MODULE_BASE");
        case -2500000 : return("SUBSYS_PAINT_DRAW_MODULE_BASE");
        case -2500001 : return("DRAW_ERR_BAD_PARAMETER");
        case -2500002 : return("DRAW_ERR_OUT_OF_MEMORY");
        case -2500003 : return("DRAW_ERR_ALREADY_INITIALIZED");
        case -2500004 : return("DRAW_ERR_NOT_INITIALIZED");
        case -2500005 : return("DRAW_ERR_FEATURE_NOT_SUPPORTED");
        case -2500006 : return("DRAW_ERR_TIMEOUT");
        case -2500007 : return("DRAW_ERR_DEVICE_BUSY");
        case -2500008 : return("DRAW_ERR_INVALID_HANDLE");
        case -2500009 : return("DRAW_ERR_SEMAPHORE_CREATE");
        case -2500010 : return("DRAW_ERR_UNSUPPORTED_VERSION");
        case -2500100 : return("DRAW_ERR_OUT_OF_OSD_SIZE");
        case -2500101 : return("DRAW_ERR_WRONG_COLOUR_MODE");
        case -2500102 : return("DRAW_ERR_WRONG_LAYOUT_MODE");
        case -2500103 : return("DRAW_ERR_BITMAP_NOT_SUPPORTED");
        case -2500104 : return("DRAW_ERR_UNKNOWN_FILE_TYPE");
        case -2500105 : return("DRAW_ERR_PNG_NOT_SUPPORTED");
        case -2500106 : return("DRAW_ERR_PNG_INIT");
        case -2500107 : return("DRAW_ERR_PNG_DECODE");
        case -2600000 : return("SUBSYS_PAINT_FONT_MODULE_BASE");
        case -2600001 : return("FONT_ERR_BAD_PARAMETER");
        case -2600002 : return("FONT_ERR_OUT_OF_MEMORY");
        case -2600003 : return("FONT_ERR_ALREADY_INITIALIZED");
        case -2600004 : return("FONT_ERR_NOT_INITIALIZED");
        case -2600005 : return("FONT_ERR_FEATURE_NOT_SUPPORTED");
        case -2600006 : return("FONT_ERR_TIMEOUT");
        case -2600007 : return("FONT_ERR_DEVICE_BUSY");
        case -2600008 : return("FONT_ERR_INVALID_HANDLE");
        case -2600009 : return("FONT_ERR_SEMAPHORE_CREATE");
        case -2600010 : return("FONT_ERR_UNSUPPORTED_VERSION");
        case -2600100 : return("FONT_ERR_OUT_OF_OSD_SIZE");
        case -2600200 : return("FONT_ERR_TTF_CANNOT_INITIAL");
        case -2600201 : return("FONT_ERR_TTF_UNKNOWN_FORMAT");
        case -2600202 : return("FONT_ERR_TTF_CANNOT_LOAD_FONT_FORMAT");
        case -2600203 : return("FONT_ERR_TTF_CANNOT_SET_FONT_SIZE");
        case -2600204 : return("FONT_ERR_FBF_CANNOT_SUPPORT");
        case -2600205 : return("FONT_ERR_FREETYPE_NOT_REGISTERED");
        case -2601000 : return("SUBSYS_PAINT_FONTSERVER_MODULE_BASE");
        case -2601001 : return("FONTSERVER_ERR_BAD_PARAMETER");
        case -2601002 : return("FONTSERVER_ERR_OUT_OF_MEMORY");
        case -2601003 : return("FONTSERVER_ERR_ALREADY_INITIALIZED");
        case -2601004 : return("FONTSERVER_ERR_NOT_INITIALIZED");
        case -2601005 : return("FONTSERVER_ERR_FEATURE_NOT_SUPPORTED");
        case -2601006 : return("FONTSERVER_ERR_TIMEOUT");
        case -2601007 : return("FONTSERVER_ERR_DEVICE_BUSY");
        case -2601008 : return("FONTSERVER_ERR_INVALID_HANDLE");
        case -2601009 : return("FONTSERVER_ERR_SEMAPHORE_CREATE");
        case -2601010 : return("FONTSERVER_ERR_UNSUPPORTED_VERSION");
        case -2601100 : return("FONTSERVER_ERR_BAD_FORMAT");
        case -2601101 : return("FONTSERVER_ERR_OUT_OF_OSD_SIZE");
        case -2601102 : return("FONTSERVER_ERR_NEGATIVE_CHAR_ADVANCE");
        case -2601103 : return("FONTSERVER_ERR_MAX_HANDLE_COUNT");
        case -2601104 : return("FONTSERVER_ERR_BAD_COLOUR");
        case -2601105 : return("FONTSERVER_ERR_MAX_FONT_COUNT");
        case -2601106 : return("FONTSERVER_ERR_CHAR_NOT_SUPPORTED");
        case -2601200 : return("FONTSERVER_ERR_FBF_FONT_INIT");
        case -2601201 : return("FONTSERVER_ERR_FBF_BAD_FBF_FILE");
        case -2601202 : return("FONTSERVER_ERR_FBF_MEMORY_FAULT");
        case -2601203 : return("FONTSERVER_ERR_FBF_KERNING_NOT_SUPPORTED");
        case -2601300 : return("FONTSERVER_ERR_FT_INIT");
        case -2601301 : return("FONTSERVER_ERR_FT_INIT_CACHE_SUBSYSTEM");
        case -2601302 : return("FONTSERVER_ERR_FT_BAD_FORMAT");
        case -2601303 : return("FONTSERVER_ERR_FT_CANNOT_SET_FONT_SIZE");
        case -2601304 : return("FONTSERVER_ERR_FT_LOOKUP_SCALER");
        case -2601305 : return("FONTSERVER_ERR_FT_BAD_FONT_SIZE");
        case -2601306 : return("FONTSERVER_ERR_FT_STYLE_NOT_AVAILABLE");
        case -2601307 : return("FONTSERVER_ERR_FT_FREETYPE_NOT_REGISTERED");
        case -2601308 : return("FONTSERVER_ERR_FT_KERNING_NOT_SUPPORTED");
        case -2700000 : return("SUBSYS_PAINT_OSDHANDLER_MODULE_BASE");
        case -2700001 : return("OSDHANDLER_ERR_BAD_PARAMETER");
        case -2700002 : return("OSDHANDLER_ERR_OUT_OF_MEMORY");
        case -2700003 : return("OSDHANDLER_ERR_ALREADY_INITIALIZED");
        case -2700004 : return("OSDHANDLER_ERR_NOT_INITIALIZED");
        case -2700005 : return("OSDHANDLER_ERR_FEATURE_NOT_SUPPORTED");
        case -2700006 : return("OSDHANDLER_ERR_TIMEOUT");
        case -2700007 : return("OSDHANDLER_ERR_DEVICE_BUSY");
        case -2700008 : return("OSDHANDLER_ERR_INVALID_HANDLE");
        case -2700009 : return("OSDHANDLER_ERR_SEMAPHORE_CREATE");
        case -2700010 : return("OSDHANDLER_ERR_UNSUPPORTED_VERSION");
        case -2700100 : return("OSDHANDLER_ERR_BAD_LAYER_ID");
        case -2700101 : return("OSDHANDLER_ERR_NO_OSD_HANDLE");
        case -2700102 : return("OSDHANDLER_ERR_BAD_COLOUR_MODE");
        case -2700103 : return("OSDHANDLER_ERR_OUT_OF_SIZE");
        case -2700104 : return("OSDHANDLER_ERR_OUT_OF_BOUNDS");
        case -2700105 : return("OSDHANDLER_ERR_LAYER_NOT_STARTED");
        case -2700106 : return("OSDHANDLER_ERR_CP_16KB_EXCEEDED");
        case -2700200 : return("OSDHANDLER_ERR_VL_ALREADY_EXISTS");
        case -2700201 : return("OSDHANDLER_ERR_VL_ALREADY_HIDDEN");
        case -2700202 : return("OSDHANDLER_ERR_VL_NO_FREE_LAYER");
        case -2700203 : return("OSDHANDLER_ERR_VL_LAYER_IS_LOCKED");
        case -2700204 : return("OSDHANDLER_ERR_VL_LAYER_IS_VIRTUAL");
        case -2700205 : return("OSDHANDLER_ERR_VL_BUSY");
        case -2700206 : return("OSDHANDLER_ERR_VL_ALREADY_UPDATED");
        case -2700207 : return("OSDHANDLER_ERR_VL_FIFO_OVERFLOW");
        case -2700300 : return("OSDHANDLER_FB_ERR_BAD_HANDLE");
        case -2700301 : return("OSDHANDLER_FB_ERR_MEMORY_ALIGN");
        case -2700302 : return("OSDHANDLER_FB_ERR_BAD_COLOUR_MODE");
        case -2800000 : return("SUBSYS_STORAGE_MODULE_BASE");
        case -2900000 : return("SUBSYS_STORAGE_MMCSD_MODULE_BASE");
        case -2900001 : return("SUBSYS_STORAGE_MMCSD_ERR_BAD_PARAMETER");
        case -2900002 : return("SUBSYS_STORAGE_MMCSD_ERR_OUT_OF_MEMORY");
        case -2900003 : return("SUBSYS_STORAGE_MMCSD_ERR_ALREADY_INITIALIZED");
        case -2900004 : return("SUBSYS_STORAGE_MMCSD_ERR_NOT_INITIALIZED");
        case -2900005 : return("SUBSYS_STORAGE_MMCSD_ERR_FEATURE_NOT_SUPPORTED");
        case -2900006 : return("SUBSYS_STORAGE_MMCSD_ERR_TIMEOUT");
        case -2900007 : return("SUBSYS_STORAGE_MMCSD_ERR_DEVICE_BUSY");
        case -2900008 : return("SUBSYS_STORAGE_MMCSD_ERR_INVALID_HANDLE");
        case -2900009 : return("SUBSYS_STORAGE_MMCSD_ERR_SEMAPHORE_CREATE");
        case -2900010 : return("SUBSYS_STORAGE_MMCSD_ERR_UNSUPPORTED_VERSION");
        case -2900100 : return("SUBSYS_STORAGE_MMCSD_ERR_UNABLE_CREATE_SSP_HANDLE");
        case -2900101 : return("SUBSYS_STORAGE_MMCSD_ERR_NO_DATA");
        case -2900102 : return("SUBSYS_STORAGE_MMCSD_ERR_NO_RESPONSE");
        case -2900103 : return("SUBSYS_STORAGE_MMCSD_ERR_NO_CARD");
        case -2900104 : return("SUBSYS_STORAGE_MMCSD_ERR_BAD_ADDRESS");
        case -2900105 : return("SUBSYS_STORAGE_MMCSD_ERR_NOT_READY");
        case -3000000 : return("SUBSYS_AUDIO_MIXER_MODULE_BASE");
        case -3000001 : return("SUBSYS_MIXER_ERR_BAD_PARAMETER");
        case -3000002 : return("SUBSYS_MIXER_ERR_OUT_OF_MEMORY");
        case -3000003 : return("SUBSYS_MIXER_ERR_ALREADY_INITIALIZED");
        case -3000004 : return("SUBSYS_MIXER_ERR_NOT_INITIALIZED");
        case -3000005 : return("SUBSYS_MIXER_ERR_FEATURE_NOT_SUPPORTED");
        case -3000006 : return("SUBSYS_MIXER_ERR_TIMEOUT");
        case -3000007 : return("SUBSYS_MIXER_ERR_DEVICE_BUSY");
        case -3000008 : return("SUBSYS_MIXER_ERR_INVALID_HANDLE");
        case -3000009 : return("SUBSYS_MIXER_ERR_SEMAPHORE_CREATE");
        case -3000010 : return("SUBSYS_MIXER_ERR_UNSUPPORTED_VERSION");
        case -3000100 : return("SUBSYS_MIXER_ERR_ALREADY_RUNNING");
        case -3100000 : return("SUBSYS_FS_FLASHFS_MODULE_BASE");
        case -3100001 : return("FLASHFS_ERR_BAD_PARAMETER");
        case -3100002 : return("FLASHFS_ERR_OUT_OF_MEMORY");
        case -3100003 : return("FLASHFS_ERR_ALREADY_INITIALIZED");
        case -3100004 : return("FLASHFS_ERR_NOT_INITIALIZED");
        case -3100005 : return("FLASHFS_ERR_FEATURE_NOT_SUPPORTED");
        case -3100006 : return("FLASHFS_ERR_TIMEOUT");
        case -3100007 : return("FLASHFS_ERR_DEVICE_BUSY");
        case -3100008 : return("FLASHFS_ERR_INVALID_HANDLE");
        case -3100009 : return("FLASHFS_ERR_SEMAPHORE_CREATE");
        case -3100010 : return("FLASHFS_ERR_UNSUPPORTED_VERSION");
        case -3100011 : return("FLASHFS_ERR_MAILQUEUE_CREATE");
        case -3100012 : return("FLASHFS_ERR_NOT_OPEN");
        case -3100013 : return("FLASHFS_ERR_ALREADY_OPEN");
        case -3100100 : return("FLASHFS_ERR_INVALID_DEVICE");
        case -3100101 : return("FLASHFS_ERR_INVALID_ALLOCATION");
        case -3100102 : return("FLASHFS_ERR_DUPLICATED_ID");
        case -3100103 : return("FLASHFS_ERR_FILE_NOT_EXIST");
        case -3100104 : return("FLASHFS_ERR_CHECKSUM");
        case -3100105 : return("FLASHFS_ERR_END_OF_FILE");
        case -3100106 : return("FLASHFS_ERR_PROHIBITED");
        case -3100107 : return("FLASHFS_ERR_INVALID_FILE");
        case -3200000 : return("SUBSYS_FOTA_MODULE_BASE");
        case -3200001 : return("SUBSYS_FOTA_ERR_BAD_PARAMETER");
        case -3200002 : return("SUBSYS_FOTA_ERR_OUT_OF_MEMORY");
        case -3200003 : return("SUBSYS_FOTA_ERR_ALREADY_INITIALIZED");
        case -3200004 : return("SUBSYS_FOTA_ERR_NOT_INITIALIZED");
        case -3200005 : return("SUBSYS_FOTA_ERR_FEATURE_NOT_SUPPORTED");
        case -3200006 : return("SUBSYS_FOTA_ERR_TIMEOUT");
        case -3200007 : return("SUBSYS_FOTA_ERR_DEVICE_BUSY");
        case -3200008 : return("SUBSYS_FOTA_ERR_INVALID_HANDLE");
        case -3200009 : return("SUBSYS_FOTA_ERR_SEMAPHORE_CREATE");
        case -3200010 : return("SUBSYS_FOTA_ERR_UNSUPPORTED_VERSION");
        case -3200100 : return("SUBSYS_FOTA_ERR_UPDATE_ABORT");
        case -3300000 : return("SUBSYS_GPROF_MODULE_BASE");
        case -3300001 : return("SUBSYS_GPROF_ERR_BAD_PARAMETER");
        case -3300002 : return("SUBSYS_GPROF_ERR_OUT_OF_MEMORY");
        case -3300003 : return("SUBSYS_GPROF_ERR_ALREADY_INITIALIZED");
        case -3300004 : return("SUBSYS_GPROF_ERR_NOT_INITIALIZED");
        case -3300005 : return("SUBSYS_GPROF_ERR_FEATURE_NOT_SUPPORTED");
        case -3300006 : return("SUBSYS_GPROF_ERR_TIMEOUT");
        case -3300007 : return("SUBSYS_GPROF_ERR_DEVICE_BUSY");
        case -3300008 : return("SUBSYS_GPROF_ERR_INVALID_HANDLE");
        case -3300009 : return("SUBSYS_GPROF_ERR_SEMAPHORE_CREATE");
        case -3300010 : return("SUBSYS_GPROF_ERR_UNSUPPORTED_VERSION");
        case -3300101 : return("SUBSYS_GPROF_ERR_HDD_INIT");
        case -3300102 : return("SUBSYS_GPROF_ERR_RAM_INIT");
        case -3300103 : return("SUBSYS_GPROF_ERR_CREATE_THREAD");
        case -3400000 : return("SUBSYS_VOC_MODULE_BASE");
        case -3400001 : return("VOC_ERR_BAD_PARAMETER");
        case -3400002 : return("VOC_ERR_OUT_OF_MEMORY");
        case -3400003 : return("VOC_ERR_ALREADY_INITIALIZED");
        case -3400004 : return("VOC_ERR_NOT_INITIALIZED");
        case -3400005 : return("VOC_ERR_FEATURE_NOT_SUPPORTED");
        case -3400006 : return("VOC_ERR_TIMEOUT");
        case -3400007 : return("VOC_ERR_DEVICE_BUSY");
        case -3400008 : return("VOC_ERR_INVALID_HANDLE");
        case -3400009 : return("VOC_ERR_SEMAPHORE_CREATE");
        case -3400010 : return("VOC_ERR_UNSUPPORTED_VERSION");
        case -3400012 : return("VOC_ERR_NOT_OPEN");
        case -3400013 : return("VOC_ERR_ALREADY_OPEN");
        case -3400100 : return("VOC_ERR_OUT_OF_OSD_SIZE");
        case -3400101 : return("VOC_ERR_WRONG_COLOUR_MODE");
        case -3400102 : return("VOC_ERR_UNKNOWN_FRAMEBANK");
        case -3500000 : return("SUBSYS_VIDEO_VAL_MODULE_BASE");
        case -3500001 : return("SUBSYS_VIDEO_VAL_ERR_BAD_PARAMETER");
        case -3500002 : return("SUBSYS_VIDEO_VAL_ERR_OUT_OF_MEMORY");
        case -3500003 : return("SUBSYS_VIDEO_VAL_ERR_ALREADY_INITIALIZED");
        case -3500004 : return("SUBSYS_VIDEO_VAL_ERR_NOT_INITIALIZED");
        case -3500005 : return("SUBSYS_VIDEO_VAL_ERR_FEATURE_NOT_SUPPORTED");
        case -3500006 : return("SUBSYS_VIDEO_VAL_ERR_TIMEOUT");
        case -3500007 : return("SUBSYS_VIDEO_VAL_ERR_DEVICE_BUSY");
        case -3500008 : return("SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE");
        case -3500009 : return("SUBSYS_VIDEO_VAL_ERR_SEMAPHORE_CREATE");
        case -3500010 : return("SUBSYS_VIDEO_VAL_ERR_UNSUPPORTED_VERSION");
        case -3500012 : return("SUBSYS_VIDEO_VAL_ERR_NOT_OPEN");
        case -3500013 : return("SUBSYS_VIDEO_VAL_ERR_ALREADY_OPEN");
        case -3500100 : return("SUBSYS_VIDEO_VAL_DATA_NOT_AVAILABLE");
        case -3500101 : return("SUBSYS_VIDEO_VAL_ERROR_STATE");
        case -3500102 : return("SUBSYS_VIDEO_VAL_ALREADY_STARTED");
        case -3500103 : return("SUBSYS_VIDEO_VAL_BAD_MODE");
        case -3500104 : return("SUBSYS_VIDEO_VAL_INTERNAL");
        case -3500105 : return("SUBSYS_VIDEO_VAL_ERR_THREAD_CREATE");
        case -3600000 : return("SUBSYS_SFAL_MODULE_BASE");
        case -3600001 : return("SUBSYS_SFAL_ERR_BAD_PARAMETER");
        case -3600002 : return("SUBSYS_SFAL_ERR_OUT_OF_MEMORY");
        case -3600003 : return("SUBSYS_SFAL_ERR_ALREADY_INITIALIZED");
        case -3600004 : return("SUBSYS_SFAL_ERR_NOT_INITIALIZED");
        case -3600005 : return("SUBSYS_SFAL_ERR_FEATURE_NOT_SUPPORTED");
        case -3600006 : return("SUBSYS_SFAL_ERR_TIMEOUT");
        case -3600007 : return("SUBSYS_SFAL_ERR_DEVICE_BUSY");
        case -3600008 : return("SUBSYS_SFAL_ERR_INVALID_HANDLE");
        case -3600009 : return("SUBSYS_SFAL_ERR_SEMAPHORE_CREATE");
        case -3600010 : return("SUBSYS_SFAL_ERR_UNSUPPORTED_VERSION");
        case -3600011 : return("SUBSYS_SFAL_ERR_MAILQUEUE_CREATE");
        case -3600100 : return("SUBSYS_SFAL_ERR_INVALID_PID");
        case -3600101 : return("SUBSYS_SFAL_ERR_NO_PID_ASSIGNED");
        case -3600102 : return("SUBSYS_SFAL_ERR_THREAD_CREATE");
        case -3600103 : return("SUBSYS_SFAL_ERR_NO_FREE_CHANNEL");
        case -3600104 : return("SUBSYS_SFAL_ERR_NO_FREE_FILTER");
        case -3700000 : return("SUBSYS_AUDIO_AAL_MODULE_BASE");
        case -3700001 : return("SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER");
        case -3700002 : return("SUBSYS_AUDIO_AAL_ERR_OUT_OF_MEMORY");
        case -3700003 : return("SUBSYS_AUDIO_AAL_ERR_ALREADY_INITIALIZED");
        case -3700004 : return("SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED");
        case -3700005 : return("SUBSYS_AUDIO_AAL_ERR_FEATURE_NOT_SUPPORTED");
        case -3700006 : return("SUBSYS_AUDIO_AAL_ERR_TIMEOUT");
        case -3700007 : return("SUBSYS_AUDIO_AAL_ERR_DEVICE_BUSY");
        case -3700008 : return("SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE");
        case -3700009 : return("SUBSYS_AUDIO_AAL_ERR_SEMAPHORE_CREATE");
        case -3700010 : return("SUBSYS_AUDIO_AAL_ERR_UNSUPPORTED_VERSION");
        case -3700012 : return("SUBSYS_AUDIO_AAL_ERR_NOT_OPEN");
        case -3700013 : return("SUBSYS_AUDIO_AAL_ERR_ALREADY_OPEN");
        case -3700100 : return("SUBSYS_AUDIO_AAL_DATA_NOT_AVAILABLE");
        case -3700101 : return("SUBSYS_AUDIO_AAL_ALREADY_STARTED");
        case -3800000 : return("SUBSYS_CI_MODULE_BASE");
        case -3800001 : return("SUBSYS_CI_ERR_BAD_PARAMETER");
        case -3800005 : return("SUBSYS_CI_ERR_FEATURE_NOT_SUPPORTED");
        case -3800020 : return("SUBSYS_CI_ERR_NO_HW_DRIVER");
        case -3800021 : return("SUBSYS_CI_ERR_BAD_INIT_PARAMETER");
        case -3800022 : return("SUBSYS_CI_ERR_INIT_PHY_LAYER");
        case -3800023 : return("SUBSYS_CI_ERR_INIT_TRA_LAYER");
        case -3800024 : return("SUBSYS_CI_ERR_INIT_SES_LAYER");
        case -3900000 : return("SUBSYS_AVCAP_MODULE_BASE");
        case -3900001 : return("SUBSYS_AVCAP_ERR_BAD_PARAMETER");
        case -3900002 : return("SUBSYS_AVCAP_ERR_OUT_OF_MEMORY");
        case -3900003 : return("SUBSYS_AVCAP_ERR_ALREADY_INITIALIZED");
        case -3900004 : return("SUBSYS_AVCAP_ERR_NOT_INITIALIZED");
        case -3900005 : return("SUBSYS_AVCAP_ERR_FEATURE_NOT_SUPPORTED");
        case -3900006 : return("SUBSYS_AVCAP_ERR_TIMEOUT");
        case -3900007 : return("SUBSYS_AVCAP_ERR_DEVICE_BUSY");
        case -3900008 : return("SUBSYS_AVCAP_ERR_INVALID_HANDLE");
        case -3900009 : return("SUBSYS_AVCAP_ERR_SEMAPHORE_CREATE");
        case -3900010 : return("SUBSYS_AVCAP_ERR_UNSUPPORTED_VERSION");
        case -3900011 : return("SUBSYS_AVCAP_ERR_MAILQUEUE_CREATE");
        case -3900100 : return("SUBSYS_AVCAP_ERR_DMA_FAILED");
        case -3900101 : return("SUBSYS_AVCAP_ERR_ALLREADY_OPEN");
        case -5000000 : return("MIDDLEWARE_PVR_MODULE_BASE");
        case -5000001 : return("MIDDLEWARE_PVR_ERR_BAD_PARAMETER");
        case -5000002 : return("MIDDLEWARE_PVR_ERR_OUT_OF_MEMORY");
        case -5000003 : return("MIDDLEWARE_PVR_ERR_ALREADY_INITIALIZED");
        case -5000004 : return("MIDDLEWARE_PVR_ERR_NOT_INITIALIZED");
        case -5000005 : return("MIDDLEWARE_PVR_ERR_FEATURE_NOT_SUPPORTED");
        case -5000006 : return("MIDDLEWARE_PVR_ERR_TIMEOUT");
        case -5000007 : return("MIDDLEWARE_PVR_ERR_DEVICE_BUSY");
        case -5000008 : return("MIDDLEWARE_PVR_ERR_INVALID_HANDLE");
        case -5000009 : return("MIDDLEWARE_PVR_ERR_SEMAPHORE_CREATE");
        case -5000010 : return("MIDDLEWARE_PVR_ERR_UNSUPPORTED_VERSION");
        case -5000100 : return("MIDDLEWARE_PVR_ERR_FILE_HANDLING");
        case -5000101 : return("MIDDLEWARE_PVR_ERR_BAD_FILE_FORMAT");
        case -5000102 : return("MIDDLEWARE_PVR_ERR_FILE_READ");
        case -5000103 : return("MIDDLEWARE_PVR_ERR_BAD_PACKET_SIZE");
        case -5000104 : return("MIDDLEWARE_PVR_ERR_FILE_EXISTS");
        case -5000105 : return("MIDDLEWARE_PVR_ERR_THREAD_ERROR");
        case -5000106 : return("MIDDLEWARE_PVR_ERR_BAD_MODE");
        case -5000107 : return("MIDDLEWARE_PVR_ERR_DISK_WRITE");
        case -5000108 : return("MIDDLEWARE_PVR_ERR_NO_VIDEO");
        case -5000109 : return("MIDDLEWARE_PVR_ERR_NO_TRACK_ASSIGNED");
        case -5000110 : return("MIDDLEWARE_PVR_ERR_WRONG_MARKER_TYPE");
        case -5000111 : return("MIDDLEWARE_PVR_ERR_END_OF_DATA");
        case -5000112 : return("MIDDLEWARE_PVR_ERR_BEGIN_OF_DATA");
        case -5000113 : return("MIDDLEWARE_PVR_ERR_FILE_DOES_NOT_EXIST");
        case -5000114 : return("MIDDLEWARE_PVR_ERR_TSD");
        case -5000115 : return("MIDDLEWARE_PVR_ERR_BAD_FILE_VERSION");
        case -5000116 : return("MIDDLEWARE_PVR_ERR_NO_BUFFER_SPACE");
        case -5000117 : return("MIDDLEWARE_PVR_ERR_CRYPTO_OP_FAILED");
        case -10000000 : return("APPLICATION_SMARTGO_MODULE_BASE");
        case -10000001 : return("APPL_SMARTGO_ERR_BAD_PARAMETER");
        case -10000002 : return("APPL_SMARTGO_ERR_OUT_OF_MEMORY");
        case -10000003 : return("APPL_SMARTGO_ERR_ALREADY_INITIALIZED");
        case -10000004 : return("APPL_SMARTGO_ERR_NOT_INITIALIZED");
        case -10000005 : return("APPL_SMARTGO_ERR_FEATURE_NOT_SUPPORTED");
        case -10000006 : return("APPL_SMARTGO_ERR_TIMEOUT");
        case -10000007 : return("APPL_SMARTGO_ERR_DEVICE_BUSY");
        case -10000008 : return("APPL_SMARTGO_ERR_INVALID_HANDLE");
        case -10000009 : return("APPL_SMARTGO_ERR_SEMAPHORE_CREATE");
        case -10000010 : return("APPL_SMARTGO_ERR_UNSUPPORTED_VERSION");
        case -10000012 : return("APPL_SMARTGO_ERR_NOT_OPEN");
        case -10000013 : return("APPL_SMARTGO_ERR_ALREADY_OPEN");
        case -10001001 : return("SMARTGO_SYS_ERR_PROHIBITED");
        case -10001002 : return("SMARTGO_SYS_ERR_INTERNAL");
        case -10001003 : return("SMARTGO_SYS_ERR_ABORTED");
        case -10001004 : return("SMARTGO_SYS_ERR_CONFIG_ERROR");
        case -10001005 : return("SMARTGO_SYS_ERR_CAPACITY");
        case -10001006 : return("SMARTGO_SYS_ERR_DATA_NOT_AVL");
        case -10001007 : return("SMARTGO_SYS_ERR_ALREADY_IN_LIST");
        case -10001008 : return("SMARTGO_SYS_ERR_REPLACED_IN_LIST");
        case -10001009 : return("SMARTGO_SYS_ERR_INVALID_DATA");
        case -10001010 : return("SMARTGO_SYS_ERR_RESOURCES");
        case -10001011 : return("SMARTGO_SYS_ERR_ITEM_EMPTY");
        case -10001013 : return("SMARTGO_SYS_ERR_RTOS_ERROR");
        case -10001014 : return("SMARTGO_SYS_ERR_DATA_NOT_PROVIDED");
        case -10001015 : return("SMARTGO_SYS_ERR_RSRC");
        case -10001016 : return("SMARTGO_SYS_ERR_RSRC_LIVE");
        case -10001017 : return("SMARTGO_SYS_ERR_RSRC_PLAY");
        case -10001018 : return("SMARTGO_SYS_ERR_RSRC_RECORD");
        case -10001019 : return("SMARTGO_SYS_ERR_RSRC_SCAN");
        case -10001020 : return("SMARTGO_SYS_ERR_RSRC_CI");
        case -10001021 : return("SMARTGO_SYS_ERR_RSRC_NETWORK");
        case -10001022 : return("SMARTGO_SYS_ERR_RSRC_PIP");
        case -10001023 : return("SMARTGO_SYS_ERR_RSRC_TUNEMODE");
        case -10001024 : return("SMARTGO_SYS_ERR_RSRC_UPDATE");
        case -10001025 : return("SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE");
        case -10001026 : return("SMARTGO_SYS_ERR_UPDATE_CRC");
        case -10001027 : return("SMARTGO_SYS_ERR_UPDATE_CAPACITY");
        case -10001028 : return("SMARTGO_SYS_ERR_FILEREAD");
        case -10001029 : return("SMARTGO_SYS_ERR_FILEWRITE");
        case -10002001 : return("SMARTGO_FGS_ERR_BAD_PARAMETER");
        case -10002002 : return("SMARTGO_FGS_ERR_OUT_OF_MEMORY");
        case -10002003 : return("SMARTGO_FGS_ERR_ALREADY_INITIALIZED");
        case -10002004 : return("SMARTGO_FGS_ERR_NOT_INITIALIZED");
        case -10002005 : return("SMARTGO_FGS_ERR_FEATURE_NOT_SUPPORTED");
        case -10002006 : return("SMARTGO_FGS_ERR_TIMEOUT");
        case -10002007 : return("SMARTGO_FGS_ERR_DEVICE_BUSY");
        case -10002008 : return("SMARTGO_FGS_ERR_INVALID_HANDLE");
        case -10002009 : return("SMARTGO_FGS_ERR_SEMAPHORE_CREATE");
        case -10002010 : return("SMARTGO_FGS_ERR_UNSUPPORTED_VERSION");
        case -10002101 : return("SMARTGO_FGS_ERR_PROHIBITED");
        case -10002102 : return("SMARTGO_FGS_ERR_RESOURCES");
        case -10003001 : return("SMARTGO_APPL_ERR_PROHIBITED");
        case -10003002 : return("SMARTGO_APPL_ERR_INTERNAL");
        case -10003003 : return("SMARTGO_APPL_ERR_CAPACITY");
        case -10003004 : return("SMARTGO_APPL_ERR_ALREADY_OPEN");
        case -10003005 : return("SMARTGO_APPL_ERR_DATA_NOT_AVL");

        default: return("UNKNOWN_ERROR");
    }
}


/******************************************************************************/
/*lint -restore */
/* END OF FILE */
